/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__gl2__uguu
#define __PYX_HAVE_API__renpy__gl2__uguu
#include <stdint.h>
#include <stddef.h>
#include "renpygl.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "stringsource",
  "uguu.pyx",
};

/*--- Type declarations ---*/
struct __pyx_obj_5renpy_3gl2_4uguu_ptr;
struct __pyx_obj_5renpy_3gl2_4uguu_Buffer;
struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer;
struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer;
struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer;
struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer;

/* "uguugl.pxd":324
 *     GLenum GL_RENDERBUFFER_STENCIL_SIZE
 * 
 * ctypedef void (*glActiveTexture_type)(GLenum  texture) nogil             # <<<<<<<<<<<<<<
 * cdef glActiveTexture_type glActiveTexture
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type)(GLenum);

/* "uguugl.pxd":327
 * cdef glActiveTexture_type glActiveTexture
 * 
 * ctypedef void (*glAttachShader_type)(GLuint  program, GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glAttachShader_type glAttachShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type)(GLuint, GLuint);

/* "uguugl.pxd":330
 * cdef glAttachShader_type glAttachShader
 * 
 * ctypedef void (*glBindAttribLocation_type)(GLuint  program, GLuint  index, const GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glBindAttribLocation_type glBindAttribLocation
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type)(GLuint, GLuint, GLchar const *);

/* "uguugl.pxd":333
 * cdef glBindAttribLocation_type glBindAttribLocation
 * 
 * ctypedef void (*glBindBuffer_type)(GLenum  target, GLuint  buffer) nogil             # <<<<<<<<<<<<<<
 * cdef glBindBuffer_type glBindBuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type)(GLenum, GLuint);

/* "uguugl.pxd":336
 * cdef glBindBuffer_type glBindBuffer
 * 
 * ctypedef void (*glBindFramebuffer_type)(GLenum  target, GLuint  framebuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glBindFramebuffer_type glBindFramebuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type)(GLenum, GLuint);

/* "uguugl.pxd":339
 * cdef glBindFramebuffer_type glBindFramebuffer
 * 
 * ctypedef void (*glBindRenderbuffer_type)(GLenum  target, GLuint  renderbuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glBindRenderbuffer_type glBindRenderbuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type)(GLenum, GLuint);

/* "uguugl.pxd":342
 * cdef glBindRenderbuffer_type glBindRenderbuffer
 * 
 * ctypedef void (*glBindTexture_type)(GLenum  target, GLuint  texture) nogil             # <<<<<<<<<<<<<<
 * cdef glBindTexture_type glBindTexture
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type)(GLenum, GLuint);

/* "uguugl.pxd":345
 * cdef glBindTexture_type glBindTexture
 * 
 * ctypedef void (*glBlendColor_type)(GLfloat  red, GLfloat  green, GLfloat  blue, GLfloat  alpha) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendColor_type glBlendColor
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type)(GLfloat, GLfloat, GLfloat, GLfloat);

/* "uguugl.pxd":348
 * cdef glBlendColor_type glBlendColor
 * 
 * ctypedef void (*glBlendEquation_type)(GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendEquation_type glBlendEquation
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type)(GLenum);

/* "uguugl.pxd":351
 * cdef glBlendEquation_type glBlendEquation
 * 
 * ctypedef void (*glBlendEquationSeparate_type)(GLenum  modeRGB, GLenum  modeAlpha) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendEquationSeparate_type glBlendEquationSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type)(GLenum, GLenum);

/* "uguugl.pxd":354
 * cdef glBlendEquationSeparate_type glBlendEquationSeparate
 * 
 * ctypedef void (*glBlendFunc_type)(GLenum  sfactor, GLenum  dfactor) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendFunc_type glBlendFunc
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type)(GLenum, GLenum);

/* "uguugl.pxd":357
 * cdef glBlendFunc_type glBlendFunc
 * 
 * ctypedef void (*glBlendFuncSeparate_type)(GLenum  sfactorRGB, GLenum  dfactorRGB, GLenum  sfactorAlpha, GLenum  dfactorAlpha) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendFuncSeparate_type glBlendFuncSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type)(GLenum, GLenum, GLenum, GLenum);

/* "uguugl.pxd":360
 * cdef glBlendFuncSeparate_type glBlendFuncSeparate
 * 
 * ctypedef void (*glBufferData_type)(GLenum  target, GLsizeiptr  size, const void * data, GLenum  usage) nogil             # <<<<<<<<<<<<<<
 * cdef glBufferData_type glBufferData
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBufferData_type)(GLenum, GLsizeiptr, void const *, GLenum);

/* "uguugl.pxd":363
 * cdef glBufferData_type glBufferData
 * 
 * ctypedef void (*glBufferSubData_type)(GLenum  target, GLintptr  offset, GLsizeiptr  size, const void * data) nogil             # <<<<<<<<<<<<<<
 * cdef glBufferSubData_type glBufferSubData
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type)(GLenum, GLintptr, GLsizeiptr, void const *);

/* "uguugl.pxd":366
 * cdef glBufferSubData_type glBufferSubData
 * 
 * ctypedef GLenum (*glCheckFramebufferStatus_type)(GLenum  target) nogil             # <<<<<<<<<<<<<<
 * cdef glCheckFramebufferStatus_type glCheckFramebufferStatus
 * 
 */
typedef GLenum (*__pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type)(GLenum);

/* "uguugl.pxd":369
 * cdef glCheckFramebufferStatus_type glCheckFramebufferStatus
 * 
 * ctypedef void (*glClear_type)(GLbitfield  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glClear_type glClear
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glClear_type)(GLbitfield);

/* "uguugl.pxd":372
 * cdef glClear_type glClear
 * 
 * ctypedef void (*glClearColor_type)(GLfloat  red, GLfloat  green, GLfloat  blue, GLfloat  alpha) nogil             # <<<<<<<<<<<<<<
 * cdef glClearColor_type glClearColor
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glClearColor_type)(GLfloat, GLfloat, GLfloat, GLfloat);

/* "uguugl.pxd":375
 * cdef glClearColor_type glClearColor
 * 
 * ctypedef void (*glClearStencil_type)(GLint  s) nogil             # <<<<<<<<<<<<<<
 * cdef glClearStencil_type glClearStencil
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type)(GLint);

/* "uguugl.pxd":378
 * cdef glClearStencil_type glClearStencil
 * 
 * ctypedef void (*glColorMask_type)(GLboolean  red, GLboolean  green, GLboolean  blue, GLboolean  alpha) nogil             # <<<<<<<<<<<<<<
 * cdef glColorMask_type glColorMask
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glColorMask_type)(GLboolean, GLboolean, GLboolean, GLboolean);

/* "uguugl.pxd":381
 * cdef glColorMask_type glColorMask
 * 
 * ctypedef void (*glCompileShader_type)(GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glCompileShader_type glCompileShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type)(GLuint);

/* "uguugl.pxd":384
 * cdef glCompileShader_type glCompileShader
 * 
 * ctypedef void (*glCompressedTexImage2D_type)(GLenum  target, GLint  level, GLenum  internalformat, GLsizei  width, GLsizei  height, GLint  border, GLsizei  imageSize, const void * data) nogil             # <<<<<<<<<<<<<<
 * cdef glCompressedTexImage2D_type glCompressedTexImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, void const *);

/* "uguugl.pxd":387
 * cdef glCompressedTexImage2D_type glCompressedTexImage2D
 * 
 * ctypedef void (*glCompressedTexSubImage2D_type)(GLenum  target, GLint  level, GLint  xoffset, GLint  yoffset, GLsizei  width, GLsizei  height, GLenum  format, GLsizei  imageSize, const void * data) nogil             # <<<<<<<<<<<<<<
 * cdef glCompressedTexSubImage2D_type glCompressedTexSubImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, void const *);

/* "uguugl.pxd":390
 * cdef glCompressedTexSubImage2D_type glCompressedTexSubImage2D
 * 
 * ctypedef void (*glCopyTexImage2D_type)(GLenum  target, GLint  level, GLenum  internalformat, GLint  x, GLint  y, GLsizei  width, GLsizei  height, GLint  border) nogil             # <<<<<<<<<<<<<<
 * cdef glCopyTexImage2D_type glCopyTexImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type)(GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint);

/* "uguugl.pxd":393
 * cdef glCopyTexImage2D_type glCopyTexImage2D
 * 
 * ctypedef void (*glCopyTexSubImage2D_type)(GLenum  target, GLint  level, GLint  xoffset, GLint  yoffset, GLint  x, GLint  y, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glCopyTexSubImage2D_type glCopyTexSubImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type)(GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei);

/* "uguugl.pxd":396
 * cdef glCopyTexSubImage2D_type glCopyTexSubImage2D
 * 
 * ctypedef GLuint (*glCreateProgram_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glCreateProgram_type glCreateProgram
 * 
 */
typedef GLuint (*__pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type)(void);

/* "uguugl.pxd":399
 * cdef glCreateProgram_type glCreateProgram
 * 
 * ctypedef GLuint (*glCreateShader_type)(GLenum  type) nogil             # <<<<<<<<<<<<<<
 * cdef glCreateShader_type glCreateShader
 * 
 */
typedef GLuint (*__pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type)(GLenum);

/* "uguugl.pxd":402
 * cdef glCreateShader_type glCreateShader
 * 
 * ctypedef void (*glCullFace_type)(GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glCullFace_type glCullFace
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCullFace_type)(GLenum);

/* "uguugl.pxd":405
 * cdef glCullFace_type glCullFace
 * 
 * ctypedef void (*glDeleteBuffers_type)(GLsizei  n, const GLuint * buffers) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteBuffers_type glDeleteBuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type)(GLsizei, GLuint const *);

/* "uguugl.pxd":408
 * cdef glDeleteBuffers_type glDeleteBuffers
 * 
 * ctypedef void (*glDeleteFramebuffers_type)(GLsizei  n, const GLuint * framebuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteFramebuffers_type glDeleteFramebuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type)(GLsizei, GLuint const *);

/* "uguugl.pxd":411
 * cdef glDeleteFramebuffers_type glDeleteFramebuffers
 * 
 * ctypedef void (*glDeleteProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteProgram_type glDeleteProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type)(GLuint);

/* "uguugl.pxd":414
 * cdef glDeleteProgram_type glDeleteProgram
 * 
 * ctypedef void (*glDeleteRenderbuffers_type)(GLsizei  n, const GLuint * renderbuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteRenderbuffers_type glDeleteRenderbuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type)(GLsizei, GLuint const *);

/* "uguugl.pxd":417
 * cdef glDeleteRenderbuffers_type glDeleteRenderbuffers
 * 
 * ctypedef void (*glDeleteShader_type)(GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteShader_type glDeleteShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type)(GLuint);

/* "uguugl.pxd":420
 * cdef glDeleteShader_type glDeleteShader
 * 
 * ctypedef void (*glDeleteTextures_type)(GLsizei  n, const GLuint * textures) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteTextures_type glDeleteTextures
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type)(GLsizei, GLuint const *);

/* "uguugl.pxd":423
 * cdef glDeleteTextures_type glDeleteTextures
 * 
 * ctypedef void (*glDepthFunc_type)(GLenum  func) nogil             # <<<<<<<<<<<<<<
 * cdef glDepthFunc_type glDepthFunc
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type)(GLenum);

/* "uguugl.pxd":426
 * cdef glDepthFunc_type glDepthFunc
 * 
 * ctypedef void (*glDepthMask_type)(GLboolean  flag) nogil             # <<<<<<<<<<<<<<
 * cdef glDepthMask_type glDepthMask
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type)(GLboolean);

/* "uguugl.pxd":429
 * cdef glDepthMask_type glDepthMask
 * 
 * ctypedef void (*glDetachShader_type)(GLuint  program, GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glDetachShader_type glDetachShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type)(GLuint, GLuint);

/* "uguugl.pxd":432
 * cdef glDetachShader_type glDetachShader
 * 
 * ctypedef void (*glDisable_type)(GLenum  cap) nogil             # <<<<<<<<<<<<<<
 * cdef glDisable_type glDisable
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDisable_type)(GLenum);

/* "uguugl.pxd":435
 * cdef glDisable_type glDisable
 * 
 * ctypedef void (*glDisableVertexAttribArray_type)(GLuint  index) nogil             # <<<<<<<<<<<<<<
 * cdef glDisableVertexAttribArray_type glDisableVertexAttribArray
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type)(GLuint);

/* "uguugl.pxd":438
 * cdef glDisableVertexAttribArray_type glDisableVertexAttribArray
 * 
 * ctypedef void (*glDrawArrays_type)(GLenum  mode, GLint  first, GLsizei  count) nogil             # <<<<<<<<<<<<<<
 * cdef glDrawArrays_type glDrawArrays
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type)(GLenum, GLint, GLsizei);

/* "uguugl.pxd":441
 * cdef glDrawArrays_type glDrawArrays
 * 
 * ctypedef void (*glDrawElements_type)(GLenum  mode, GLsizei  count, GLenum  type, const void * indices) nogil             # <<<<<<<<<<<<<<
 * cdef glDrawElements_type glDrawElements
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type)(GLenum, GLsizei, GLenum, void const *);

/* "uguugl.pxd":444
 * cdef glDrawElements_type glDrawElements
 * 
 * ctypedef void (*glEnable_type)(GLenum  cap) nogil             # <<<<<<<<<<<<<<
 * cdef glEnable_type glEnable
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glEnable_type)(GLenum);

/* "uguugl.pxd":447
 * cdef glEnable_type glEnable
 * 
 * ctypedef void (*glEnableVertexAttribArray_type)(GLuint  index) nogil             # <<<<<<<<<<<<<<
 * cdef glEnableVertexAttribArray_type glEnableVertexAttribArray
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type)(GLuint);

/* "uguugl.pxd":450
 * cdef glEnableVertexAttribArray_type glEnableVertexAttribArray
 * 
 * ctypedef void (*glFinish_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glFinish_type glFinish
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFinish_type)(void);

/* "uguugl.pxd":453
 * cdef glFinish_type glFinish
 * 
 * ctypedef void (*glFlush_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glFlush_type glFlush
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFlush_type)(void);

/* "uguugl.pxd":456
 * cdef glFlush_type glFlush
 * 
 * ctypedef void (*glFramebufferRenderbuffer_type)(GLenum  target, GLenum  attachment, GLenum  renderbuffertarget, GLuint  renderbuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glFramebufferRenderbuffer_type glFramebufferRenderbuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type)(GLenum, GLenum, GLenum, GLuint);

/* "uguugl.pxd":459
 * cdef glFramebufferRenderbuffer_type glFramebufferRenderbuffer
 * 
 * ctypedef void (*glFramebufferTexture2D_type)(GLenum  target, GLenum  attachment, GLenum  textarget, GLuint  texture, GLint  level) nogil             # <<<<<<<<<<<<<<
 * cdef glFramebufferTexture2D_type glFramebufferTexture2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type)(GLenum, GLenum, GLenum, GLuint, GLint);

/* "uguugl.pxd":462
 * cdef glFramebufferTexture2D_type glFramebufferTexture2D
 * 
 * ctypedef void (*glFrontFace_type)(GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glFrontFace_type glFrontFace
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type)(GLenum);

/* "uguugl.pxd":465
 * cdef glFrontFace_type glFrontFace
 * 
 * ctypedef void (*glGenBuffers_type)(GLsizei  n, GLuint * buffers) nogil             # <<<<<<<<<<<<<<
 * cdef glGenBuffers_type glGenBuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type)(GLsizei, GLuint *);

/* "uguugl.pxd":468
 * cdef glGenBuffers_type glGenBuffers
 * 
 * ctypedef void (*glGenFramebuffers_type)(GLsizei  n, GLuint * framebuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glGenFramebuffers_type glGenFramebuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type)(GLsizei, GLuint *);

/* "uguugl.pxd":471
 * cdef glGenFramebuffers_type glGenFramebuffers
 * 
 * ctypedef void (*glGenRenderbuffers_type)(GLsizei  n, GLuint * renderbuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glGenRenderbuffers_type glGenRenderbuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type)(GLsizei, GLuint *);

/* "uguugl.pxd":474
 * cdef glGenRenderbuffers_type glGenRenderbuffers
 * 
 * ctypedef void (*glGenTextures_type)(GLsizei  n, GLuint * textures) nogil             # <<<<<<<<<<<<<<
 * cdef glGenTextures_type glGenTextures
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type)(GLsizei, GLuint *);

/* "uguugl.pxd":477
 * cdef glGenTextures_type glGenTextures
 * 
 * ctypedef void (*glGenerateMipmap_type)(GLenum  target) nogil             # <<<<<<<<<<<<<<
 * cdef glGenerateMipmap_type glGenerateMipmap
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type)(GLenum);

/* "uguugl.pxd":480
 * cdef glGenerateMipmap_type glGenerateMipmap
 * 
 * ctypedef void (*glGetActiveAttrib_type)(GLuint  program, GLuint  index, GLsizei  bufSize, GLsizei * length, GLint * size, GLenum * type, GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetActiveAttrib_type glGetActiveAttrib
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type)(GLuint, GLuint, GLsizei, GLsizei *, GLint *, GLenum *, GLchar *);

/* "uguugl.pxd":483
 * cdef glGetActiveAttrib_type glGetActiveAttrib
 * 
 * ctypedef void (*glGetActiveUniform_type)(GLuint  program, GLuint  index, GLsizei  bufSize, GLsizei * length, GLint * size, GLenum * type, GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetActiveUniform_type glGetActiveUniform
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type)(GLuint, GLuint, GLsizei, GLsizei *, GLint *, GLenum *, GLchar *);

/* "uguugl.pxd":486
 * cdef glGetActiveUniform_type glGetActiveUniform
 * 
 * ctypedef void (*glGetAttachedShaders_type)(GLuint  program, GLsizei  maxCount, GLsizei * count, GLuint * shaders) nogil             # <<<<<<<<<<<<<<
 * cdef glGetAttachedShaders_type glGetAttachedShaders
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type)(GLuint, GLsizei, GLsizei *, GLuint *);

/* "uguugl.pxd":489
 * cdef glGetAttachedShaders_type glGetAttachedShaders
 * 
 * ctypedef GLint (*glGetAttribLocation_type)(GLuint  program, const GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetAttribLocation_type glGetAttribLocation
 * 
 */
typedef GLint (*__pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type)(GLuint, GLchar const *);

/* "uguugl.pxd":492
 * cdef glGetAttribLocation_type glGetAttribLocation
 * 
 * ctypedef void (*glGetBooleanv_type)(GLenum  pname, GLboolean * data) nogil             # <<<<<<<<<<<<<<
 * cdef glGetBooleanv_type glGetBooleanv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type)(GLenum, GLboolean *);

/* "uguugl.pxd":495
 * cdef glGetBooleanv_type glGetBooleanv
 * 
 * ctypedef void (*glGetBufferParameteriv_type)(GLenum  target, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetBufferParameteriv_type glGetBufferParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type)(GLenum, GLenum, GLint *);

/* "uguugl.pxd":498
 * cdef glGetBufferParameteriv_type glGetBufferParameteriv
 * 
 * ctypedef GLenum (*glGetError_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glGetError_type glGetError
 * 
 */
typedef GLenum (*__pyx_t_5renpy_3gl2_6uguugl_glGetError_type)(void);

/* "uguugl.pxd":501
 * cdef glGetError_type glGetError
 * 
 * ctypedef void (*glGetFloatv_type)(GLenum  pname, GLfloat * data) nogil             # <<<<<<<<<<<<<<
 * cdef glGetFloatv_type glGetFloatv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type)(GLenum, GLfloat *);

/* "uguugl.pxd":504
 * cdef glGetFloatv_type glGetFloatv
 * 
 * ctypedef void (*glGetFramebufferAttachmentParameteriv_type)(GLenum  target, GLenum  attachment, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetFramebufferAttachmentParameteriv_type glGetFramebufferAttachmentParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type)(GLenum, GLenum, GLenum, GLint *);

/* "uguugl.pxd":507
 * cdef glGetFramebufferAttachmentParameteriv_type glGetFramebufferAttachmentParameteriv
 * 
 * ctypedef void (*glGetIntegerv_type)(GLenum  pname, GLint * data) nogil             # <<<<<<<<<<<<<<
 * cdef glGetIntegerv_type glGetIntegerv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type)(GLenum, GLint *);

/* "uguugl.pxd":510
 * cdef glGetIntegerv_type glGetIntegerv
 * 
 * ctypedef void (*glGetProgramInfoLog_type)(GLuint  program, GLsizei  bufSize, GLsizei * length, GLchar * infoLog) nogil             # <<<<<<<<<<<<<<
 * cdef glGetProgramInfoLog_type glGetProgramInfoLog
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type)(GLuint, GLsizei, GLsizei *, GLchar *);

/* "uguugl.pxd":513
 * cdef glGetProgramInfoLog_type glGetProgramInfoLog
 * 
 * ctypedef void (*glGetProgramiv_type)(GLuint  program, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetProgramiv_type glGetProgramiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type)(GLuint, GLenum, GLint *);

/* "uguugl.pxd":516
 * cdef glGetProgramiv_type glGetProgramiv
 * 
 * ctypedef void (*glGetRenderbufferParameteriv_type)(GLenum  target, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetRenderbufferParameteriv_type glGetRenderbufferParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type)(GLenum, GLenum, GLint *);

/* "uguugl.pxd":519
 * cdef glGetRenderbufferParameteriv_type glGetRenderbufferParameteriv
 * 
 * ctypedef void (*glGetShaderInfoLog_type)(GLuint  shader, GLsizei  bufSize, GLsizei * length, GLchar * infoLog) nogil             # <<<<<<<<<<<<<<
 * cdef glGetShaderInfoLog_type glGetShaderInfoLog
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type)(GLuint, GLsizei, GLsizei *, GLchar *);

/* "uguugl.pxd":522
 * cdef glGetShaderInfoLog_type glGetShaderInfoLog
 * 
 * ctypedef void (*glGetShaderSource_type)(GLuint  shader, GLsizei  bufSize, GLsizei * length, GLchar * source) nogil             # <<<<<<<<<<<<<<
 * cdef glGetShaderSource_type glGetShaderSource
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type)(GLuint, GLsizei, GLsizei *, GLchar *);

/* "uguugl.pxd":525
 * cdef glGetShaderSource_type glGetShaderSource
 * 
 * ctypedef void (*glGetShaderiv_type)(GLuint  shader, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetShaderiv_type glGetShaderiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type)(GLuint, GLenum, GLint *);

/* "uguugl.pxd":528
 * cdef glGetShaderiv_type glGetShaderiv
 * 
 * ctypedef const GLubyte * (*glGetString_type)(GLenum  name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetString_type glGetString
 * 
 */
typedef GLubyte const *(*__pyx_t_5renpy_3gl2_6uguugl_glGetString_type)(GLenum);

/* "uguugl.pxd":531
 * cdef glGetString_type glGetString
 * 
 * ctypedef void (*glGetTexParameterfv_type)(GLenum  target, GLenum  pname, GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetTexParameterfv_type glGetTexParameterfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type)(GLenum, GLenum, GLfloat *);

/* "uguugl.pxd":534
 * cdef glGetTexParameterfv_type glGetTexParameterfv
 * 
 * ctypedef void (*glGetTexParameteriv_type)(GLenum  target, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetTexParameteriv_type glGetTexParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type)(GLenum, GLenum, GLint *);

/* "uguugl.pxd":537
 * cdef glGetTexParameteriv_type glGetTexParameteriv
 * 
 * ctypedef GLint (*glGetUniformLocation_type)(GLuint  program, const GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetUniformLocation_type glGetUniformLocation
 * 
 */
typedef GLint (*__pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type)(GLuint, GLchar const *);

/* "uguugl.pxd":540
 * cdef glGetUniformLocation_type glGetUniformLocation
 * 
 * ctypedef void (*glGetUniformfv_type)(GLuint  program, GLint  location, GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetUniformfv_type glGetUniformfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type)(GLuint, GLint, GLfloat *);

/* "uguugl.pxd":543
 * cdef glGetUniformfv_type glGetUniformfv
 * 
 * ctypedef void (*glGetUniformiv_type)(GLuint  program, GLint  location, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetUniformiv_type glGetUniformiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type)(GLuint, GLint, GLint *);

/* "uguugl.pxd":546
 * cdef glGetUniformiv_type glGetUniformiv
 * 
 * ctypedef void (*glGetVertexAttribPointerv_type)(GLuint  index, GLenum  pname, void ** pointer) nogil             # <<<<<<<<<<<<<<
 * cdef glGetVertexAttribPointerv_type glGetVertexAttribPointerv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type)(GLuint, GLenum, void **);

/* "uguugl.pxd":549
 * cdef glGetVertexAttribPointerv_type glGetVertexAttribPointerv
 * 
 * ctypedef void (*glGetVertexAttribfv_type)(GLuint  index, GLenum  pname, GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetVertexAttribfv_type glGetVertexAttribfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type)(GLuint, GLenum, GLfloat *);

/* "uguugl.pxd":552
 * cdef glGetVertexAttribfv_type glGetVertexAttribfv
 * 
 * ctypedef void (*glGetVertexAttribiv_type)(GLuint  index, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetVertexAttribiv_type glGetVertexAttribiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type)(GLuint, GLenum, GLint *);

/* "uguugl.pxd":555
 * cdef glGetVertexAttribiv_type glGetVertexAttribiv
 * 
 * ctypedef void (*glHint_type)(GLenum  target, GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glHint_type glHint
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glHint_type)(GLenum, GLenum);

/* "uguugl.pxd":558
 * cdef glHint_type glHint
 * 
 * ctypedef GLboolean (*glIsBuffer_type)(GLuint  buffer) nogil             # <<<<<<<<<<<<<<
 * cdef glIsBuffer_type glIsBuffer
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type)(GLuint);

/* "uguugl.pxd":561
 * cdef glIsBuffer_type glIsBuffer
 * 
 * ctypedef GLboolean (*glIsEnabled_type)(GLenum  cap) nogil             # <<<<<<<<<<<<<<
 * cdef glIsEnabled_type glIsEnabled
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type)(GLenum);

/* "uguugl.pxd":564
 * cdef glIsEnabled_type glIsEnabled
 * 
 * ctypedef GLboolean (*glIsFramebuffer_type)(GLuint  framebuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glIsFramebuffer_type glIsFramebuffer
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type)(GLuint);

/* "uguugl.pxd":567
 * cdef glIsFramebuffer_type glIsFramebuffer
 * 
 * ctypedef GLboolean (*glIsProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glIsProgram_type glIsProgram
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type)(GLuint);

/* "uguugl.pxd":570
 * cdef glIsProgram_type glIsProgram
 * 
 * ctypedef GLboolean (*glIsRenderbuffer_type)(GLuint  renderbuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glIsRenderbuffer_type glIsRenderbuffer
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type)(GLuint);

/* "uguugl.pxd":573
 * cdef glIsRenderbuffer_type glIsRenderbuffer
 * 
 * ctypedef GLboolean (*glIsShader_type)(GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glIsShader_type glIsShader
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsShader_type)(GLuint);

/* "uguugl.pxd":576
 * cdef glIsShader_type glIsShader
 * 
 * ctypedef GLboolean (*glIsTexture_type)(GLuint  texture) nogil             # <<<<<<<<<<<<<<
 * cdef glIsTexture_type glIsTexture
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type)(GLuint);

/* "uguugl.pxd":579
 * cdef glIsTexture_type glIsTexture
 * 
 * ctypedef void (*glLineWidth_type)(GLfloat  width) nogil             # <<<<<<<<<<<<<<
 * cdef glLineWidth_type glLineWidth
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type)(GLfloat);

/* "uguugl.pxd":582
 * cdef glLineWidth_type glLineWidth
 * 
 * ctypedef void (*glLinkProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glLinkProgram_type glLinkProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type)(GLuint);

/* "uguugl.pxd":585
 * cdef glLinkProgram_type glLinkProgram
 * 
 * ctypedef void (*glPixelStorei_type)(GLenum  pname, GLint  param) nogil             # <<<<<<<<<<<<<<
 * cdef glPixelStorei_type glPixelStorei
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type)(GLenum, GLint);

/* "uguugl.pxd":588
 * cdef glPixelStorei_type glPixelStorei
 * 
 * ctypedef void (*glPolygonOffset_type)(GLfloat  factor, GLfloat  units) nogil             # <<<<<<<<<<<<<<
 * cdef glPolygonOffset_type glPolygonOffset
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type)(GLfloat, GLfloat);

/* "uguugl.pxd":591
 * cdef glPolygonOffset_type glPolygonOffset
 * 
 * ctypedef void (*glReadPixels_type)(GLint  x, GLint  y, GLsizei  width, GLsizei  height, GLenum  format, GLenum  type, void * pixels) nogil             # <<<<<<<<<<<<<<
 * cdef glReadPixels_type glReadPixels
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type)(GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, void *);

/* "uguugl.pxd":594
 * cdef glReadPixels_type glReadPixels
 * 
 * ctypedef void (*glRenderbufferStorage_type)(GLenum  target, GLenum  internalformat, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glRenderbufferStorage_type glRenderbufferStorage
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type)(GLenum, GLenum, GLsizei, GLsizei);

/* "uguugl.pxd":597
 * cdef glRenderbufferStorage_type glRenderbufferStorage
 * 
 * ctypedef void (*glSampleCoverage_type)(GLfloat  value, GLboolean  invert) nogil             # <<<<<<<<<<<<<<
 * cdef glSampleCoverage_type glSampleCoverage
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type)(GLfloat, GLboolean);

/* "uguugl.pxd":600
 * cdef glSampleCoverage_type glSampleCoverage
 * 
 * ctypedef void (*glScissor_type)(GLint  x, GLint  y, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glScissor_type glScissor
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glScissor_type)(GLint, GLint, GLsizei, GLsizei);

/* "uguugl.pxd":603
 * cdef glScissor_type glScissor
 * 
 * ctypedef void (*glShaderSource_type)(GLuint  shader, GLsizei  count, const GLchar *const* string, const GLint * length) nogil             # <<<<<<<<<<<<<<
 * cdef glShaderSource_type glShaderSource
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type)(GLuint, GLsizei, GLchar const *const *, GLint const *);

/* "uguugl.pxd":606
 * cdef glShaderSource_type glShaderSource
 * 
 * ctypedef void (*glStencilFunc_type)(GLenum  func, GLint  ref, GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilFunc_type glStencilFunc
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type)(GLenum, GLint, GLuint);

/* "uguugl.pxd":609
 * cdef glStencilFunc_type glStencilFunc
 * 
 * ctypedef void (*glStencilFuncSeparate_type)(GLenum  face, GLenum  func, GLint  ref, GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilFuncSeparate_type glStencilFuncSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type)(GLenum, GLenum, GLint, GLuint);

/* "uguugl.pxd":612
 * cdef glStencilFuncSeparate_type glStencilFuncSeparate
 * 
 * ctypedef void (*glStencilMask_type)(GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilMask_type glStencilMask
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type)(GLuint);

/* "uguugl.pxd":615
 * cdef glStencilMask_type glStencilMask
 * 
 * ctypedef void (*glStencilMaskSeparate_type)(GLenum  face, GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilMaskSeparate_type glStencilMaskSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type)(GLenum, GLuint);

/* "uguugl.pxd":618
 * cdef glStencilMaskSeparate_type glStencilMaskSeparate
 * 
 * ctypedef void (*glStencilOp_type)(GLenum  fail, GLenum  zfail, GLenum  zpass) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilOp_type glStencilOp
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type)(GLenum, GLenum, GLenum);

/* "uguugl.pxd":621
 * cdef glStencilOp_type glStencilOp
 * 
 * ctypedef void (*glStencilOpSeparate_type)(GLenum  face, GLenum  sfail, GLenum  dpfail, GLenum  dppass) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilOpSeparate_type glStencilOpSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type)(GLenum, GLenum, GLenum, GLenum);

/* "uguugl.pxd":624
 * cdef glStencilOpSeparate_type glStencilOpSeparate
 * 
 * ctypedef void (*glTexImage2D_type)(GLenum  target, GLint  level, GLint  internalformat, GLsizei  width, GLsizei  height, GLint  border, GLenum  format, GLenum  type, const void * pixels) nogil             # <<<<<<<<<<<<<<
 * cdef glTexImage2D_type glTexImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type)(GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, void const *);

/* "uguugl.pxd":627
 * cdef glTexImage2D_type glTexImage2D
 * 
 * ctypedef void (*glTexParameterf_type)(GLenum  target, GLenum  pname, GLfloat  param) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameterf_type glTexParameterf
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type)(GLenum, GLenum, GLfloat);

/* "uguugl.pxd":630
 * cdef glTexParameterf_type glTexParameterf
 * 
 * ctypedef void (*glTexParameterfv_type)(GLenum  target, GLenum  pname, const GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameterfv_type glTexParameterfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type)(GLenum, GLenum, GLfloat const *);

/* "uguugl.pxd":633
 * cdef glTexParameterfv_type glTexParameterfv
 * 
 * ctypedef void (*glTexParameteri_type)(GLenum  target, GLenum  pname, GLint  param) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameteri_type glTexParameteri
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type)(GLenum, GLenum, GLint);

/* "uguugl.pxd":636
 * cdef glTexParameteri_type glTexParameteri
 * 
 * ctypedef void (*glTexParameteriv_type)(GLenum  target, GLenum  pname, const GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameteriv_type glTexParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type)(GLenum, GLenum, GLint const *);

/* "uguugl.pxd":639
 * cdef glTexParameteriv_type glTexParameteriv
 * 
 * ctypedef void (*glTexSubImage2D_type)(GLenum  target, GLint  level, GLint  xoffset, GLint  yoffset, GLsizei  width, GLsizei  height, GLenum  format, GLenum  type, const void * pixels) nogil             # <<<<<<<<<<<<<<
 * cdef glTexSubImage2D_type glTexSubImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, void const *);

/* "uguugl.pxd":642
 * cdef glTexSubImage2D_type glTexSubImage2D
 * 
 * ctypedef void (*glUniform1f_type)(GLint  location, GLfloat  v0) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1f_type glUniform1f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type)(GLint, GLfloat);

/* "uguugl.pxd":645
 * cdef glUniform1f_type glUniform1f
 * 
 * ctypedef void (*glUniform1fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1fv_type glUniform1fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type)(GLint, GLsizei, GLfloat const *);

/* "uguugl.pxd":648
 * cdef glUniform1fv_type glUniform1fv
 * 
 * ctypedef void (*glUniform1i_type)(GLint  location, GLint  v0) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1i_type glUniform1i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type)(GLint, GLint);

/* "uguugl.pxd":651
 * cdef glUniform1i_type glUniform1i
 * 
 * ctypedef void (*glUniform1iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1iv_type glUniform1iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type)(GLint, GLsizei, GLint const *);

/* "uguugl.pxd":654
 * cdef glUniform1iv_type glUniform1iv
 * 
 * ctypedef void (*glUniform2f_type)(GLint  location, GLfloat  v0, GLfloat  v1) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2f_type glUniform2f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type)(GLint, GLfloat, GLfloat);

/* "uguugl.pxd":657
 * cdef glUniform2f_type glUniform2f
 * 
 * ctypedef void (*glUniform2fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2fv_type glUniform2fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type)(GLint, GLsizei, GLfloat const *);

/* "uguugl.pxd":660
 * cdef glUniform2fv_type glUniform2fv
 * 
 * ctypedef void (*glUniform2i_type)(GLint  location, GLint  v0, GLint  v1) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2i_type glUniform2i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type)(GLint, GLint, GLint);

/* "uguugl.pxd":663
 * cdef glUniform2i_type glUniform2i
 * 
 * ctypedef void (*glUniform2iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2iv_type glUniform2iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type)(GLint, GLsizei, GLint const *);

/* "uguugl.pxd":666
 * cdef glUniform2iv_type glUniform2iv
 * 
 * ctypedef void (*glUniform3f_type)(GLint  location, GLfloat  v0, GLfloat  v1, GLfloat  v2) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3f_type glUniform3f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type)(GLint, GLfloat, GLfloat, GLfloat);

/* "uguugl.pxd":669
 * cdef glUniform3f_type glUniform3f
 * 
 * ctypedef void (*glUniform3fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3fv_type glUniform3fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type)(GLint, GLsizei, GLfloat const *);

/* "uguugl.pxd":672
 * cdef glUniform3fv_type glUniform3fv
 * 
 * ctypedef void (*glUniform3i_type)(GLint  location, GLint  v0, GLint  v1, GLint  v2) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3i_type glUniform3i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type)(GLint, GLint, GLint, GLint);

/* "uguugl.pxd":675
 * cdef glUniform3i_type glUniform3i
 * 
 * ctypedef void (*glUniform3iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3iv_type glUniform3iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type)(GLint, GLsizei, GLint const *);

/* "uguugl.pxd":678
 * cdef glUniform3iv_type glUniform3iv
 * 
 * ctypedef void (*glUniform4f_type)(GLint  location, GLfloat  v0, GLfloat  v1, GLfloat  v2, GLfloat  v3) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4f_type glUniform4f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type)(GLint, GLfloat, GLfloat, GLfloat, GLfloat);

/* "uguugl.pxd":681
 * cdef glUniform4f_type glUniform4f
 * 
 * ctypedef void (*glUniform4fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4fv_type glUniform4fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type)(GLint, GLsizei, GLfloat const *);

/* "uguugl.pxd":684
 * cdef glUniform4fv_type glUniform4fv
 * 
 * ctypedef void (*glUniform4i_type)(GLint  location, GLint  v0, GLint  v1, GLint  v2, GLint  v3) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4i_type glUniform4i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type)(GLint, GLint, GLint, GLint, GLint);

/* "uguugl.pxd":687
 * cdef glUniform4i_type glUniform4i
 * 
 * ctypedef void (*glUniform4iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4iv_type glUniform4iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type)(GLint, GLsizei, GLint const *);

/* "uguugl.pxd":690
 * cdef glUniform4iv_type glUniform4iv
 * 
 * ctypedef void (*glUniformMatrix2fv_type)(GLint  location, GLsizei  count, GLboolean  transpose, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniformMatrix2fv_type glUniformMatrix2fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type)(GLint, GLsizei, GLboolean, GLfloat const *);

/* "uguugl.pxd":693
 * cdef glUniformMatrix2fv_type glUniformMatrix2fv
 * 
 * ctypedef void (*glUniformMatrix3fv_type)(GLint  location, GLsizei  count, GLboolean  transpose, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniformMatrix3fv_type glUniformMatrix3fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type)(GLint, GLsizei, GLboolean, GLfloat const *);

/* "uguugl.pxd":696
 * cdef glUniformMatrix3fv_type glUniformMatrix3fv
 * 
 * ctypedef void (*glUniformMatrix4fv_type)(GLint  location, GLsizei  count, GLboolean  transpose, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniformMatrix4fv_type glUniformMatrix4fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type)(GLint, GLsizei, GLboolean, GLfloat const *);

/* "uguugl.pxd":699
 * cdef glUniformMatrix4fv_type glUniformMatrix4fv
 * 
 * ctypedef void (*glUseProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glUseProgram_type glUseProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type)(GLuint);

/* "uguugl.pxd":702
 * cdef glUseProgram_type glUseProgram
 * 
 * ctypedef void (*glValidateProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glValidateProgram_type glValidateProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type)(GLuint);

/* "uguugl.pxd":705
 * cdef glValidateProgram_type glValidateProgram
 * 
 * ctypedef void (*glVertexAttrib1f_type)(GLuint  index, GLfloat  x) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib1f_type glVertexAttrib1f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type)(GLuint, GLfloat);

/* "uguugl.pxd":708
 * cdef glVertexAttrib1f_type glVertexAttrib1f
 * 
 * ctypedef void (*glVertexAttrib1fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib1fv_type glVertexAttrib1fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type)(GLuint, GLfloat const *);

/* "uguugl.pxd":711
 * cdef glVertexAttrib1fv_type glVertexAttrib1fv
 * 
 * ctypedef void (*glVertexAttrib2f_type)(GLuint  index, GLfloat  x, GLfloat  y) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib2f_type glVertexAttrib2f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type)(GLuint, GLfloat, GLfloat);

/* "uguugl.pxd":714
 * cdef glVertexAttrib2f_type glVertexAttrib2f
 * 
 * ctypedef void (*glVertexAttrib2fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib2fv_type glVertexAttrib2fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type)(GLuint, GLfloat const *);

/* "uguugl.pxd":717
 * cdef glVertexAttrib2fv_type glVertexAttrib2fv
 * 
 * ctypedef void (*glVertexAttrib3f_type)(GLuint  index, GLfloat  x, GLfloat  y, GLfloat  z) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib3f_type glVertexAttrib3f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type)(GLuint, GLfloat, GLfloat, GLfloat);

/* "uguugl.pxd":720
 * cdef glVertexAttrib3f_type glVertexAttrib3f
 * 
 * ctypedef void (*glVertexAttrib3fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib3fv_type glVertexAttrib3fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type)(GLuint, GLfloat const *);

/* "uguugl.pxd":723
 * cdef glVertexAttrib3fv_type glVertexAttrib3fv
 * 
 * ctypedef void (*glVertexAttrib4f_type)(GLuint  index, GLfloat  x, GLfloat  y, GLfloat  z, GLfloat  w) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib4f_type glVertexAttrib4f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type)(GLuint, GLfloat, GLfloat, GLfloat, GLfloat);

/* "uguugl.pxd":726
 * cdef glVertexAttrib4f_type glVertexAttrib4f
 * 
 * ctypedef void (*glVertexAttrib4fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib4fv_type glVertexAttrib4fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type)(GLuint, GLfloat const *);

/* "uguugl.pxd":729
 * cdef glVertexAttrib4fv_type glVertexAttrib4fv
 * 
 * ctypedef void (*glVertexAttribPointer_type)(GLuint  index, GLint  size, GLenum  type, GLboolean  normalized, GLsizei  stride, const void * pointer) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttribPointer_type glVertexAttribPointer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type)(GLuint, GLint, GLenum, GLboolean, GLsizei, void const *);

/* "uguugl.pxd":732
 * cdef glVertexAttribPointer_type glVertexAttribPointer
 * 
 * ctypedef void (*glViewport_type)(GLint  x, GLint  y, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glViewport_type glViewport
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glViewport_type)(GLint, GLint, GLsizei, GLsizei);

/* "renpy/gl2/uguu.pyx":56
 * 
 * 
 * cdef class ptr:             # <<<<<<<<<<<<<<
 *     """
 *     This is a class that wraps a generic contiguous Python buffer, and
 */
struct __pyx_obj_5renpy_3gl2_4uguu_ptr {
  PyObject_HEAD
  void *ptr;
  Py_buffer view;
};


/* "renpy/gl2/uguu.pyx":89
 *         return ptr(o)
 * 
 * cdef class Buffer:             # <<<<<<<<<<<<<<
 *     """
 *     The base class for all buffers.
 */
struct __pyx_obj_5renpy_3gl2_4uguu_Buffer {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer *__pyx_vtab;
  Py_ssize_t length;
  Py_ssize_t itemsize;
  char const *format;
  void *data;
  int readonly;
};


/* "renpy/gl2/uguu.pyx":143
 *             self.data = NULL
 * 
 * cdef class BytesBuffer(Buffer):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, length):
 */
struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer {
  struct __pyx_obj_5renpy_3gl2_4uguu_Buffer __pyx_base;
};


/* "renpy/gl2/uguu.pyx":152
 *         return bytes(<char *> self.data)
 * 
 * cdef class BytesListBuffer(Buffer):             # <<<<<<<<<<<<<<
 *     cdef object value
 * 
 */
struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer {
  struct __pyx_obj_5renpy_3gl2_4uguu_Buffer __pyx_base;
  PyObject *value;
};


/* "renpy/gl2/uguu.pyx":164
 *             (<const char **> self.data)[i] = <const char *> (<ptr> self.value[i]).ptr
 * 
 * cdef class IntBuffer(Buffer):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, value):
 */
struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer {
  struct __pyx_obj_5renpy_3gl2_4uguu_Buffer __pyx_base;
};


/* "renpy/gl2/uguu.pyx":181
 *         return (<int*> self.data)[index]
 * 
 * cdef class FloatBuffer(Buffer):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, value):
 */
struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer {
  struct __pyx_obj_5renpy_3gl2_4uguu_Buffer __pyx_base;
};



/* "renpy/gl2/uguu.pyx":89
 *         return ptr(o)
 * 
 * cdef class Buffer:             # <<<<<<<<<<<<<<
 *     """
 *     The base class for all buffers.
 */

struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer {
  PyObject *(*setup_buffer)(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *, Py_ssize_t, Py_ssize_t, char const *, int);
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer *__pyx_vtabptr_5renpy_3gl2_4uguu_Buffer;


/* "renpy/gl2/uguu.pyx":143
 *             self.data = NULL
 * 
 * cdef class BytesBuffer(Buffer):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, length):
 */

struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesBuffer {
  struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesBuffer *__pyx_vtabptr_5renpy_3gl2_4uguu_BytesBuffer;


/* "renpy/gl2/uguu.pyx":152
 *         return bytes(<char *> self.data)
 * 
 * cdef class BytesListBuffer(Buffer):             # <<<<<<<<<<<<<<
 *     cdef object value
 * 
 */

struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesListBuffer {
  struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_vtabptr_5renpy_3gl2_4uguu_BytesListBuffer;


/* "renpy/gl2/uguu.pyx":164
 *             (<const char **> self.data)[i] = <const char *> (<ptr> self.value[i]).ptr
 * 
 * cdef class IntBuffer(Buffer):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, value):
 */

struct __pyx_vtabstruct_5renpy_3gl2_4uguu_IntBuffer {
  struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_IntBuffer *__pyx_vtabptr_5renpy_3gl2_4uguu_IntBuffer;


/* "renpy/gl2/uguu.pyx":181
 *         return (<int*> self.data)[index]
 * 
 * cdef class FloatBuffer(Buffer):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, value):
 */

struct __pyx_vtabstruct_5renpy_3gl2_4uguu_FloatBuffer {
  struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_FloatBuffer *__pyx_vtabptr_5renpy_3gl2_4uguu_FloatBuffer;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLenum(GLenum value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLuint(GLuint value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLint(GLint value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLboolean(GLboolean value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLenum __Pyx_PyInt_As_GLenum(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLuint __Pyx_PyInt_As_GLuint(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE ptrdiff_t __Pyx_PyInt_As_ptrdiff_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLbitfield __Pyx_PyInt_As_GLbitfield(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLint __Pyx_PyInt_As_GLint(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLboolean __Pyx_PyInt_As_GLboolean(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLsizei __Pyx_PyInt_As_GLsizei(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_5renpy_3gl2_4uguu_6Buffer_setup_buffer(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, Py_ssize_t __pyx_v_length, Py_ssize_t __pyx_v_itemsize, char const *__pyx_v_format, int __pyx_v_readonly); /* proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'renpy.gl2.uguugl' */
static __pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type *__pyx_vp_5renpy_3gl2_6uguugl_glActiveTexture = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glActiveTexture (*__pyx_vp_5renpy_3gl2_6uguugl_glActiveTexture)
static __pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type *__pyx_vp_5renpy_3gl2_6uguugl_glAttachShader = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glAttachShader (*__pyx_vp_5renpy_3gl2_6uguugl_glAttachShader)
static __pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type *__pyx_vp_5renpy_3gl2_6uguugl_glBindAttribLocation = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBindAttribLocation (*__pyx_vp_5renpy_3gl2_6uguugl_glBindAttribLocation)
static __pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glBindBuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBindBuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glBindBuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glBindFramebuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBindFramebuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glBindFramebuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glBindRenderbuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBindRenderbuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glBindRenderbuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type *__pyx_vp_5renpy_3gl2_6uguugl_glBindTexture = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBindTexture (*__pyx_vp_5renpy_3gl2_6uguugl_glBindTexture)
static __pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type *__pyx_vp_5renpy_3gl2_6uguugl_glBlendColor = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBlendColor (*__pyx_vp_5renpy_3gl2_6uguugl_glBlendColor)
static __pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type *__pyx_vp_5renpy_3gl2_6uguugl_glBlendEquation = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBlendEquation (*__pyx_vp_5renpy_3gl2_6uguugl_glBlendEquation)
static __pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type *__pyx_vp_5renpy_3gl2_6uguugl_glBlendEquationSeparate = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBlendEquationSeparate (*__pyx_vp_5renpy_3gl2_6uguugl_glBlendEquationSeparate)
static __pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type *__pyx_vp_5renpy_3gl2_6uguugl_glBlendFunc = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBlendFunc (*__pyx_vp_5renpy_3gl2_6uguugl_glBlendFunc)
static __pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type *__pyx_vp_5renpy_3gl2_6uguugl_glBlendFuncSeparate = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBlendFuncSeparate (*__pyx_vp_5renpy_3gl2_6uguugl_glBlendFuncSeparate)
static __pyx_t_5renpy_3gl2_6uguugl_glBufferData_type *__pyx_vp_5renpy_3gl2_6uguugl_glBufferData = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBufferData (*__pyx_vp_5renpy_3gl2_6uguugl_glBufferData)
static __pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type *__pyx_vp_5renpy_3gl2_6uguugl_glBufferSubData = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glBufferSubData (*__pyx_vp_5renpy_3gl2_6uguugl_glBufferSubData)
static __pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type *__pyx_vp_5renpy_3gl2_6uguugl_glCheckFramebufferStatus = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCheckFramebufferStatus (*__pyx_vp_5renpy_3gl2_6uguugl_glCheckFramebufferStatus)
static __pyx_t_5renpy_3gl2_6uguugl_glClear_type *__pyx_vp_5renpy_3gl2_6uguugl_glClear = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glClear (*__pyx_vp_5renpy_3gl2_6uguugl_glClear)
static __pyx_t_5renpy_3gl2_6uguugl_glClearColor_type *__pyx_vp_5renpy_3gl2_6uguugl_glClearColor = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glClearColor (*__pyx_vp_5renpy_3gl2_6uguugl_glClearColor)
static __pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type *__pyx_vp_5renpy_3gl2_6uguugl_glClearStencil = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glClearStencil (*__pyx_vp_5renpy_3gl2_6uguugl_glClearStencil)
static __pyx_t_5renpy_3gl2_6uguugl_glColorMask_type *__pyx_vp_5renpy_3gl2_6uguugl_glColorMask = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glColorMask (*__pyx_vp_5renpy_3gl2_6uguugl_glColorMask)
static __pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type *__pyx_vp_5renpy_3gl2_6uguugl_glCompileShader = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCompileShader (*__pyx_vp_5renpy_3gl2_6uguugl_glCompileShader)
static __pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glCompressedTexImage2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexImage2D (*__pyx_vp_5renpy_3gl2_6uguugl_glCompressedTexImage2D)
static __pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D (*__pyx_vp_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D)
static __pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glCopyTexImage2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCopyTexImage2D (*__pyx_vp_5renpy_3gl2_6uguugl_glCopyTexImage2D)
static __pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glCopyTexSubImage2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCopyTexSubImage2D (*__pyx_vp_5renpy_3gl2_6uguugl_glCopyTexSubImage2D)
static __pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type *__pyx_vp_5renpy_3gl2_6uguugl_glCreateProgram = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCreateProgram (*__pyx_vp_5renpy_3gl2_6uguugl_glCreateProgram)
static __pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type *__pyx_vp_5renpy_3gl2_6uguugl_glCreateShader = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCreateShader (*__pyx_vp_5renpy_3gl2_6uguugl_glCreateShader)
static __pyx_t_5renpy_3gl2_6uguugl_glCullFace_type *__pyx_vp_5renpy_3gl2_6uguugl_glCullFace = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glCullFace (*__pyx_vp_5renpy_3gl2_6uguugl_glCullFace)
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type *__pyx_vp_5renpy_3gl2_6uguugl_glDeleteBuffers = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDeleteBuffers (*__pyx_vp_5renpy_3gl2_6uguugl_glDeleteBuffers)
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type *__pyx_vp_5renpy_3gl2_6uguugl_glDeleteFramebuffers = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDeleteFramebuffers (*__pyx_vp_5renpy_3gl2_6uguugl_glDeleteFramebuffers)
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type *__pyx_vp_5renpy_3gl2_6uguugl_glDeleteProgram = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDeleteProgram (*__pyx_vp_5renpy_3gl2_6uguugl_glDeleteProgram)
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type *__pyx_vp_5renpy_3gl2_6uguugl_glDeleteRenderbuffers = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDeleteRenderbuffers (*__pyx_vp_5renpy_3gl2_6uguugl_glDeleteRenderbuffers)
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type *__pyx_vp_5renpy_3gl2_6uguugl_glDeleteShader = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDeleteShader (*__pyx_vp_5renpy_3gl2_6uguugl_glDeleteShader)
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type *__pyx_vp_5renpy_3gl2_6uguugl_glDeleteTextures = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDeleteTextures (*__pyx_vp_5renpy_3gl2_6uguugl_glDeleteTextures)
static __pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type *__pyx_vp_5renpy_3gl2_6uguugl_glDepthFunc = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDepthFunc (*__pyx_vp_5renpy_3gl2_6uguugl_glDepthFunc)
static __pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type *__pyx_vp_5renpy_3gl2_6uguugl_glDepthMask = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDepthMask (*__pyx_vp_5renpy_3gl2_6uguugl_glDepthMask)
static __pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type *__pyx_vp_5renpy_3gl2_6uguugl_glDetachShader = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDetachShader (*__pyx_vp_5renpy_3gl2_6uguugl_glDetachShader)
static __pyx_t_5renpy_3gl2_6uguugl_glDisable_type *__pyx_vp_5renpy_3gl2_6uguugl_glDisable = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDisable (*__pyx_vp_5renpy_3gl2_6uguugl_glDisable)
static __pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type *__pyx_vp_5renpy_3gl2_6uguugl_glDisableVertexAttribArray = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDisableVertexAttribArray (*__pyx_vp_5renpy_3gl2_6uguugl_glDisableVertexAttribArray)
static __pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type *__pyx_vp_5renpy_3gl2_6uguugl_glDrawArrays = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDrawArrays (*__pyx_vp_5renpy_3gl2_6uguugl_glDrawArrays)
static __pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type *__pyx_vp_5renpy_3gl2_6uguugl_glDrawElements = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glDrawElements (*__pyx_vp_5renpy_3gl2_6uguugl_glDrawElements)
static __pyx_t_5renpy_3gl2_6uguugl_glEnable_type *__pyx_vp_5renpy_3gl2_6uguugl_glEnable = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glEnable (*__pyx_vp_5renpy_3gl2_6uguugl_glEnable)
static __pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type *__pyx_vp_5renpy_3gl2_6uguugl_glEnableVertexAttribArray = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glEnableVertexAttribArray (*__pyx_vp_5renpy_3gl2_6uguugl_glEnableVertexAttribArray)
static __pyx_t_5renpy_3gl2_6uguugl_glFinish_type *__pyx_vp_5renpy_3gl2_6uguugl_glFinish = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glFinish (*__pyx_vp_5renpy_3gl2_6uguugl_glFinish)
static __pyx_t_5renpy_3gl2_6uguugl_glFlush_type *__pyx_vp_5renpy_3gl2_6uguugl_glFlush = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glFlush (*__pyx_vp_5renpy_3gl2_6uguugl_glFlush)
static __pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glFramebufferTexture2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glFramebufferTexture2D (*__pyx_vp_5renpy_3gl2_6uguugl_glFramebufferTexture2D)
static __pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type *__pyx_vp_5renpy_3gl2_6uguugl_glFrontFace = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glFrontFace (*__pyx_vp_5renpy_3gl2_6uguugl_glFrontFace)
static __pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type *__pyx_vp_5renpy_3gl2_6uguugl_glGenBuffers = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGenBuffers (*__pyx_vp_5renpy_3gl2_6uguugl_glGenBuffers)
static __pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type *__pyx_vp_5renpy_3gl2_6uguugl_glGenFramebuffers = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGenFramebuffers (*__pyx_vp_5renpy_3gl2_6uguugl_glGenFramebuffers)
static __pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type *__pyx_vp_5renpy_3gl2_6uguugl_glGenRenderbuffers = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGenRenderbuffers (*__pyx_vp_5renpy_3gl2_6uguugl_glGenRenderbuffers)
static __pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type *__pyx_vp_5renpy_3gl2_6uguugl_glGenTextures = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGenTextures (*__pyx_vp_5renpy_3gl2_6uguugl_glGenTextures)
static __pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type *__pyx_vp_5renpy_3gl2_6uguugl_glGenerateMipmap = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGenerateMipmap (*__pyx_vp_5renpy_3gl2_6uguugl_glGenerateMipmap)
static __pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetActiveAttrib = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetActiveAttrib (*__pyx_vp_5renpy_3gl2_6uguugl_glGetActiveAttrib)
static __pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetActiveUniform = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetActiveUniform (*__pyx_vp_5renpy_3gl2_6uguugl_glGetActiveUniform)
static __pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetAttachedShaders = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetAttachedShaders (*__pyx_vp_5renpy_3gl2_6uguugl_glGetAttachedShaders)
static __pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetAttribLocation = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetAttribLocation (*__pyx_vp_5renpy_3gl2_6uguugl_glGetAttribLocation)
static __pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetBooleanv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetBooleanv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetBooleanv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetBufferParameteriv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetBufferParameteriv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetBufferParameteriv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetError_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetError = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetError (*__pyx_vp_5renpy_3gl2_6uguugl_glGetError)
static __pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetFloatv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetFloatv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetFloatv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetIntegerv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetIntegerv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetIntegerv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetProgramInfoLog = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetProgramInfoLog (*__pyx_vp_5renpy_3gl2_6uguugl_glGetProgramInfoLog)
static __pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetProgramiv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetProgramiv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetProgramiv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderInfoLog = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetShaderInfoLog (*__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderInfoLog)
static __pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderSource = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetShaderSource (*__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderSource)
static __pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderiv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetShaderiv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderiv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetString_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetString = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetString (*__pyx_vp_5renpy_3gl2_6uguugl_glGetString)
static __pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetTexParameterfv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameterfv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetTexParameterfv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetTexParameteriv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameteriv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetTexParameteriv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformLocation = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetUniformLocation (*__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformLocation)
static __pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformfv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetUniformfv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformfv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformiv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetUniformiv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformiv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribfv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribfv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribfv)
static __pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type *__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribiv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribiv (*__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribiv)
static __pyx_t_5renpy_3gl2_6uguugl_glHint_type *__pyx_vp_5renpy_3gl2_6uguugl_glHint = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glHint (*__pyx_vp_5renpy_3gl2_6uguugl_glHint)
static __pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsBuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsBuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glIsBuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsEnabled = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsEnabled (*__pyx_vp_5renpy_3gl2_6uguugl_glIsEnabled)
static __pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsFramebuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsFramebuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glIsFramebuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsProgram = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsProgram (*__pyx_vp_5renpy_3gl2_6uguugl_glIsProgram)
static __pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsRenderbuffer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsRenderbuffer (*__pyx_vp_5renpy_3gl2_6uguugl_glIsRenderbuffer)
static __pyx_t_5renpy_3gl2_6uguugl_glIsShader_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsShader = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsShader (*__pyx_vp_5renpy_3gl2_6uguugl_glIsShader)
static __pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type *__pyx_vp_5renpy_3gl2_6uguugl_glIsTexture = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glIsTexture (*__pyx_vp_5renpy_3gl2_6uguugl_glIsTexture)
static __pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type *__pyx_vp_5renpy_3gl2_6uguugl_glLineWidth = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glLineWidth (*__pyx_vp_5renpy_3gl2_6uguugl_glLineWidth)
static __pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type *__pyx_vp_5renpy_3gl2_6uguugl_glLinkProgram = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glLinkProgram (*__pyx_vp_5renpy_3gl2_6uguugl_glLinkProgram)
static __pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type *__pyx_vp_5renpy_3gl2_6uguugl_glPixelStorei = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glPixelStorei (*__pyx_vp_5renpy_3gl2_6uguugl_glPixelStorei)
static __pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type *__pyx_vp_5renpy_3gl2_6uguugl_glPolygonOffset = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glPolygonOffset (*__pyx_vp_5renpy_3gl2_6uguugl_glPolygonOffset)
static __pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type *__pyx_vp_5renpy_3gl2_6uguugl_glReadPixels = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glReadPixels (*__pyx_vp_5renpy_3gl2_6uguugl_glReadPixels)
static __pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type *__pyx_vp_5renpy_3gl2_6uguugl_glRenderbufferStorage = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glRenderbufferStorage (*__pyx_vp_5renpy_3gl2_6uguugl_glRenderbufferStorage)
static __pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type *__pyx_vp_5renpy_3gl2_6uguugl_glSampleCoverage = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glSampleCoverage (*__pyx_vp_5renpy_3gl2_6uguugl_glSampleCoverage)
static __pyx_t_5renpy_3gl2_6uguugl_glScissor_type *__pyx_vp_5renpy_3gl2_6uguugl_glScissor = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glScissor (*__pyx_vp_5renpy_3gl2_6uguugl_glScissor)
static __pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type *__pyx_vp_5renpy_3gl2_6uguugl_glShaderSource = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glShaderSource (*__pyx_vp_5renpy_3gl2_6uguugl_glShaderSource)
static __pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type *__pyx_vp_5renpy_3gl2_6uguugl_glStencilFunc = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glStencilFunc (*__pyx_vp_5renpy_3gl2_6uguugl_glStencilFunc)
static __pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type *__pyx_vp_5renpy_3gl2_6uguugl_glStencilFuncSeparate = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glStencilFuncSeparate (*__pyx_vp_5renpy_3gl2_6uguugl_glStencilFuncSeparate)
static __pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type *__pyx_vp_5renpy_3gl2_6uguugl_glStencilMask = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glStencilMask (*__pyx_vp_5renpy_3gl2_6uguugl_glStencilMask)
static __pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type *__pyx_vp_5renpy_3gl2_6uguugl_glStencilMaskSeparate = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glStencilMaskSeparate (*__pyx_vp_5renpy_3gl2_6uguugl_glStencilMaskSeparate)
static __pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type *__pyx_vp_5renpy_3gl2_6uguugl_glStencilOp = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glStencilOp (*__pyx_vp_5renpy_3gl2_6uguugl_glStencilOp)
static __pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type *__pyx_vp_5renpy_3gl2_6uguugl_glStencilOpSeparate = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glStencilOpSeparate (*__pyx_vp_5renpy_3gl2_6uguugl_glStencilOpSeparate)
static __pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glTexImage2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glTexImage2D (*__pyx_vp_5renpy_3gl2_6uguugl_glTexImage2D)
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type *__pyx_vp_5renpy_3gl2_6uguugl_glTexParameterf = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glTexParameterf (*__pyx_vp_5renpy_3gl2_6uguugl_glTexParameterf)
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type *__pyx_vp_5renpy_3gl2_6uguugl_glTexParameterfv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glTexParameterfv (*__pyx_vp_5renpy_3gl2_6uguugl_glTexParameterfv)
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type *__pyx_vp_5renpy_3gl2_6uguugl_glTexParameteri = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glTexParameteri (*__pyx_vp_5renpy_3gl2_6uguugl_glTexParameteri)
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type *__pyx_vp_5renpy_3gl2_6uguugl_glTexParameteriv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glTexParameteriv (*__pyx_vp_5renpy_3gl2_6uguugl_glTexParameteriv)
static __pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type *__pyx_vp_5renpy_3gl2_6uguugl_glTexSubImage2D = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glTexSubImage2D (*__pyx_vp_5renpy_3gl2_6uguugl_glTexSubImage2D)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform1f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform1f (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform1f)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform1fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform1fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform1fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform1i = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform1i (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform1i)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform1iv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform1iv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform1iv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform2f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform2f (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform2f)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform2fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform2fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform2fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform2i = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform2i (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform2i)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform2iv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform2iv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform2iv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform3f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform3f (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform3f)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform3fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform3fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform3fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform3i = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform3i (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform3i)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform3iv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform3iv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform3iv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform4f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform4f (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform4f)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform4fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform4fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform4fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform4i = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform4i (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform4i)
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniform4iv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniform4iv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniform4iv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix2fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix2fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix2fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix3fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix3fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix3fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix4fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix4fv (*__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix4fv)
static __pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type *__pyx_vp_5renpy_3gl2_6uguugl_glUseProgram = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glUseProgram (*__pyx_vp_5renpy_3gl2_6uguugl_glUseProgram)
static __pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type *__pyx_vp_5renpy_3gl2_6uguugl_glValidateProgram = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glValidateProgram (*__pyx_vp_5renpy_3gl2_6uguugl_glValidateProgram)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib1f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1f (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib1f)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib1fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1fv (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib1fv)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib2f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2f (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib2f)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib2fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2fv (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib2fv)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib3f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3f (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib3f)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib3fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3fv (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib3fv)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib4f = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4f (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib4f)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib4fv = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4fv (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib4fv)
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type *__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttribPointer = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glVertexAttribPointer (*__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttribPointer)
static __pyx_t_5renpy_3gl2_6uguugl_glViewport_type *__pyx_vp_5renpy_3gl2_6uguugl_glViewport = 0;
#define __pyx_v_5renpy_3gl2_6uguugl_glViewport (*__pyx_vp_5renpy_3gl2_6uguugl_glViewport)

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'renpy.gl2.uguu' */
static PyTypeObject *__pyx_ptype_5renpy_3gl2_4uguu_ptr = 0;
static PyTypeObject *__pyx_ptype_5renpy_3gl2_4uguu_Buffer = 0;
static PyTypeObject *__pyx_ptype_5renpy_3gl2_4uguu_BytesBuffer = 0;
static PyTypeObject *__pyx_ptype_5renpy_3gl2_4uguu_BytesListBuffer = 0;
static PyTypeObject *__pyx_ptype_5renpy_3gl2_4uguu_IntBuffer = 0;
static PyTypeObject *__pyx_ptype_5renpy_3gl2_4uguu_FloatBuffer = 0;
static PyObject *__pyx_f_5renpy_3gl2_4uguu_proxy_return_string(GLubyte const *); /*proto*/
static struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_f_5renpy_3gl2_4uguu_get_ptr(PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "renpy.gl2.uguu"
extern int __pyx_module_is_main_renpy__gl2__uguu;
int __pyx_module_is_main_renpy__gl2__uguu = 0;

/* Implementation of 'renpy.gl2.uguu' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_IndexError;
static const char __pyx_k_n[] = "n";
static const char __pyx_k_o[] = "o";
static const char __pyx_k_s[] = "s";
static const char __pyx_k_v[] = "v";
static const char __pyx_k_w[] = "w";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_z[] = "z";
static const char __pyx_k_ro[] = "ro";
static const char __pyx_k_v0[] = "v0";
static const char __pyx_k_v1[] = "v1";
static const char __pyx_k_v2[] = "v2";
static const char __pyx_k_v3[] = "v3";
static const char __pyx_k_cap[] = "cap";
static const char __pyx_k_red[] = "red";
static const char __pyx_k_ref[] = "ref";
static const char __pyx_k_blue[] = "blue";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_face[] = "face";
static const char __pyx_k_fail[] = "fail";
static const char __pyx_k_flag[] = "flag";
static const char __pyx_k_func[] = "func";
static const char __pyx_k_load[] = "load";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mask[] = "mask";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_GL_CW[] = "GL_CW";
static const char __pyx_k_alpha[] = "alpha";
static const char __pyx_k_count[] = "count";
static const char __pyx_k_first[] = "first";
static const char __pyx_k_green[] = "green";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_level[] = "level";
static const char __pyx_k_param[] = "param";
static const char __pyx_k_pname[] = "pname";
static const char __pyx_k_sfail[] = "sfail";
static const char __pyx_k_units[] = "units";
static const char __pyx_k_usage[] = "usage";
static const char __pyx_k_v_ptr[] = "v_ptr";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_width[] = "width";
static const char __pyx_k_zfail[] = "zfail";
static const char __pyx_k_zpass[] = "zpass";
static const char __pyx_k_GL_CCW[] = "GL_CCW";
static const char __pyx_k_GL_INT[] = "GL_INT";
static const char __pyx_k_GL_ONE[] = "GL_ONE";
static const char __pyx_k_GL_RGB[] = "GL_RGB";
static const char __pyx_k_border[] = "border";
static const char __pyx_k_buffer[] = "buffer";
static const char __pyx_k_dpfail[] = "dpfail";
static const char __pyx_k_dppass[] = "dppass";
static const char __pyx_k_factor[] = "factor";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_glHint[] = "glHint";
static const char __pyx_k_height[] = "height";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_invert[] = "invert";
static const char __pyx_k_length[] = "length";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_params[] = "params";
static const char __pyx_k_pixels[] = "pixels";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_shader[] = "shader";
static const char __pyx_k_source[] = "source";
static const char __pyx_k_stride[] = "stride";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_target[] = "target";
static const char __pyx_k_uguugl[] = "uguugl";
static const char __pyx_k_GL_BACK[] = "GL_BACK";
static const char __pyx_k_GL_BOOL[] = "GL_BOOL";
static const char __pyx_k_GL_BYTE[] = "GL_BYTE";
static const char __pyx_k_GL_DECR[] = "GL_DECR";
static const char __pyx_k_GL_INCR[] = "GL_INCR";
static const char __pyx_k_GL_KEEP[] = "GL_KEEP";
static const char __pyx_k_GL_LESS[] = "GL_LESS";
static const char __pyx_k_GL_NONE[] = "GL_NONE";
static const char __pyx_k_GL_RGBA[] = "GL_RGBA";
static const char __pyx_k_GL_TRUE[] = "GL_TRUE";
static const char __pyx_k_GL_ZERO[] = "GL_ZERO";
static const char __pyx_k_bufSize[] = "bufSize";
static const char __pyx_k_buffers[] = "buffers";
static const char __pyx_k_dfactor[] = "dfactor";
static const char __pyx_k_glClear[] = "glClear";
static const char __pyx_k_glFlush[] = "glFlush";
static const char __pyx_k_indices[] = "indices";
static const char __pyx_k_infoLog[] = "infoLog";
static const char __pyx_k_modeRGB[] = "modeRGB";
static const char __pyx_k_pointer[] = "pointer";
static const char __pyx_k_program[] = "program";
static const char __pyx_k_sfactor[] = "sfactor";
static const char __pyx_k_shaders[] = "shaders";
static const char __pyx_k_texture[] = "texture";
static const char __pyx_k_xoffset[] = "xoffset";
static const char __pyx_k_yoffset[] = "yoffset";
static const char __pyx_k_GL_ALPHA[] = "GL_ALPHA";
static const char __pyx_k_GL_BLEND[] = "GL_BLEND";
static const char __pyx_k_GL_EQUAL[] = "GL_EQUAL";
static const char __pyx_k_GL_FALSE[] = "GL_FALSE";
static const char __pyx_k_GL_FLOAT[] = "GL_FLOAT";
static const char __pyx_k_GL_FRONT[] = "GL_FRONT";
static const char __pyx_k_GL_LINES[] = "GL_LINES";
static const char __pyx_k_GL_NEVER[] = "GL_NEVER";
static const char __pyx_k_GL_RGBA4[] = "GL_RGBA4";
static const char __pyx_k_GL_SHORT[] = "GL_SHORT";
static const char __pyx_k_data_ptr[] = "data_ptr";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_glEnable[] = "glEnable";
static const char __pyx_k_glFinish[] = "glFinish";
static const char __pyx_k_location[] = "location";
static const char __pyx_k_maxCount[] = "maxCount";
static const char __pyx_k_name_ptr[] = "name_ptr";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_size_ptr[] = "size_ptr";
static const char __pyx_k_textures[] = "textures";
static const char __pyx_k_type_ptr[] = "type_ptr";
static const char __pyx_k_GL_ALWAYS[] = "GL_ALWAYS";
static const char __pyx_k_GL_DITHER[] = "GL_DITHER";
static const char __pyx_k_GL_GEQUAL[] = "GL_GEQUAL";
static const char __pyx_k_GL_INVERT[] = "GL_INVERT";
static const char __pyx_k_GL_LEQUAL[] = "GL_LEQUAL";
static const char __pyx_k_GL_LINEAR[] = "GL_LINEAR";
static const char __pyx_k_GL_NICEST[] = "GL_NICEST";
static const char __pyx_k_GL_POINTS[] = "GL_POINTS";
static const char __pyx_k_GL_REPEAT[] = "GL_REPEAT";
static const char __pyx_k_GL_VENDOR[] = "GL_VENDOR";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_count_ptr[] = "count_ptr";
static const char __pyx_k_glDisable[] = "glDisable";
static const char __pyx_k_glScissor[] = "glScissor";
static const char __pyx_k_imageSize[] = "imageSize";
static const char __pyx_k_modeAlpha[] = "modeAlpha";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_textarget[] = "textarget";
static const char __pyx_k_transpose[] = "transpose";
static const char __pyx_k_value_ptr[] = "value_ptr";
static const char __pyx_k_GL_FASTEST[] = "GL_FASTEST";
static const char __pyx_k_GL_GREATER[] = "GL_GREATER";
static const char __pyx_k_GL_NEAREST[] = "GL_NEAREST";
static const char __pyx_k_GL_REPLACE[] = "GL_REPLACE";
static const char __pyx_k_GL_RGB5_A1[] = "GL_RGB5_A1";
static const char __pyx_k_GL_SAMPLES[] = "GL_SAMPLES";
static const char __pyx_k_GL_TEXTURE[] = "GL_TEXTURE";
static const char __pyx_k_GL_VERSION[] = "GL_VERSION";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_attachment[] = "attachment";
static const char __pyx_k_dfactorRGB[] = "dfactorRGB";
static const char __pyx_k_glCullFace[] = "glCullFace";
static const char __pyx_k_glGetError[] = "glGetError";
static const char __pyx_k_glIsBuffer[] = "glIsBuffer";
static const char __pyx_k_glIsShader[] = "glIsShader";
static const char __pyx_k_glViewport[] = "glViewport";
static const char __pyx_k_length_ptr[] = "length_ptr";
static const char __pyx_k_normalized[] = "normalized";
static const char __pyx_k_params_ptr[] = "params_ptr";
static const char __pyx_k_pixels_ptr[] = "pixels_ptr";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_sfactorRGB[] = "sfactorRGB";
static const char __pyx_k_source_ptr[] = "source_ptr";
static const char __pyx_k_string_ptr[] = "string_ptr";
static const char __pyx_k_GL_FUNC_ADD[] = "GL_FUNC_ADD";
static const char __pyx_k_GL_INT_VEC2[] = "GL_INT_VEC2";
static const char __pyx_k_GL_INT_VEC3[] = "GL_INT_VEC3";
static const char __pyx_k_GL_INT_VEC4[] = "GL_INT_VEC4";
static const char __pyx_k_GL_NOTEQUAL[] = "GL_NOTEQUAL";
static const char __pyx_k_GL_NO_ERROR[] = "GL_NO_ERROR";
static const char __pyx_k_GL_RED_BITS[] = "GL_RED_BITS";
static const char __pyx_k_GL_RENDERER[] = "GL_RENDERER";
static const char __pyx_k_GL_TEXTURE0[] = "GL_TEXTURE0";
static const char __pyx_k_GL_TEXTURE1[] = "GL_TEXTURE1";
static const char __pyx_k_GL_TEXTURE2[] = "GL_TEXTURE2";
static const char __pyx_k_GL_TEXTURE3[] = "GL_TEXTURE3";
static const char __pyx_k_GL_TEXTURE4[] = "GL_TEXTURE4";
static const char __pyx_k_GL_TEXTURE5[] = "GL_TEXTURE5";
static const char __pyx_k_GL_TEXTURE6[] = "GL_TEXTURE6";
static const char __pyx_k_GL_TEXTURE7[] = "GL_TEXTURE7";
static const char __pyx_k_GL_TEXTURE8[] = "GL_TEXTURE8";
static const char __pyx_k_GL_TEXTURE9[] = "GL_TEXTURE9";
static const char __pyx_k_GL_VIEWPORT[] = "GL_VIEWPORT";
static const char __pyx_k_buffers_ptr[] = "buffers_ptr";
static const char __pyx_k_framebuffer[] = "framebuffer";
static const char __pyx_k_glBlendFunc[] = "glBlendFunc";
static const char __pyx_k_glColorMask[] = "glColorMask";
static const char __pyx_k_glDepthFunc[] = "glDepthFunc";
static const char __pyx_k_glDepthMask[] = "glDepthMask";
static const char __pyx_k_glFrontFace[] = "glFrontFace";
static const char __pyx_k_glGetFloatv[] = "glGetFloatv";
static const char __pyx_k_glGetString[] = "glGetString";
static const char __pyx_k_glIsEnabled[] = "glIsEnabled";
static const char __pyx_k_glIsProgram[] = "glIsProgram";
static const char __pyx_k_glIsTexture[] = "glIsTexture";
static const char __pyx_k_glLineWidth[] = "glLineWidth";
static const char __pyx_k_glStencilOp[] = "glStencilOp";
static const char __pyx_k_glUniform1f[] = "glUniform1f";
static const char __pyx_k_glUniform1i[] = "glUniform1i";
static const char __pyx_k_glUniform2f[] = "glUniform2f";
static const char __pyx_k_glUniform2i[] = "glUniform2i";
static const char __pyx_k_glUniform3f[] = "glUniform3f";
static const char __pyx_k_glUniform3i[] = "glUniform3i";
static const char __pyx_k_glUniform4f[] = "glUniform4f";
static const char __pyx_k_glUniform4i[] = "glUniform4i";
static const char __pyx_k_indices_ptr[] = "indices_ptr";
static const char __pyx_k_infoLog_ptr[] = "infoLog_ptr";
static const char __pyx_k_pointer_ptr[] = "pointer_ptr";
static const char __pyx_k_shaders_ptr[] = "shaders_ptr";
static const char __pyx_k_GL_BLUE_BITS[] = "GL_BLUE_BITS";
static const char __pyx_k_GL_BOOL_VEC2[] = "GL_BOOL_VEC2";
static const char __pyx_k_GL_BOOL_VEC3[] = "GL_BOOL_VEC3";
static const char __pyx_k_GL_BOOL_VEC4[] = "GL_BOOL_VEC4";
static const char __pyx_k_GL_CULL_FACE[] = "GL_CULL_FACE";
static const char __pyx_k_GL_DECR_WRAP[] = "GL_DECR_WRAP";
static const char __pyx_k_GL_DONT_CARE[] = "GL_DONT_CARE";
static const char __pyx_k_GL_DST_ALPHA[] = "GL_DST_ALPHA";
static const char __pyx_k_GL_DST_COLOR[] = "GL_DST_COLOR";
static const char __pyx_k_GL_INCR_WRAP[] = "GL_INCR_WRAP";
static const char __pyx_k_GL_LINE_LOOP[] = "GL_LINE_LOOP";
static const char __pyx_k_GL_LUMINANCE[] = "GL_LUMINANCE";
static const char __pyx_k_GL_SRC_ALPHA[] = "GL_SRC_ALPHA";
static const char __pyx_k_GL_SRC_COLOR[] = "GL_SRC_COLOR";
static const char __pyx_k_GL_TEXTURE10[] = "GL_TEXTURE10";
static const char __pyx_k_GL_TEXTURE11[] = "GL_TEXTURE11";
static const char __pyx_k_GL_TEXTURE12[] = "GL_TEXTURE12";
static const char __pyx_k_GL_TEXTURE13[] = "GL_TEXTURE13";
static const char __pyx_k_GL_TEXTURE14[] = "GL_TEXTURE14";
static const char __pyx_k_GL_TEXTURE15[] = "GL_TEXTURE15";
static const char __pyx_k_GL_TEXTURE16[] = "GL_TEXTURE16";
static const char __pyx_k_GL_TEXTURE17[] = "GL_TEXTURE17";
static const char __pyx_k_GL_TEXTURE18[] = "GL_TEXTURE18";
static const char __pyx_k_GL_TEXTURE19[] = "GL_TEXTURE19";
static const char __pyx_k_GL_TEXTURE20[] = "GL_TEXTURE20";
static const char __pyx_k_GL_TEXTURE21[] = "GL_TEXTURE21";
static const char __pyx_k_GL_TEXTURE22[] = "GL_TEXTURE22";
static const char __pyx_k_GL_TEXTURE23[] = "GL_TEXTURE23";
static const char __pyx_k_GL_TEXTURE24[] = "GL_TEXTURE24";
static const char __pyx_k_GL_TEXTURE25[] = "GL_TEXTURE25";
static const char __pyx_k_GL_TEXTURE26[] = "GL_TEXTURE26";
static const char __pyx_k_GL_TEXTURE27[] = "GL_TEXTURE27";
static const char __pyx_k_GL_TEXTURE28[] = "GL_TEXTURE28";
static const char __pyx_k_GL_TEXTURE29[] = "GL_TEXTURE29";
static const char __pyx_k_GL_TEXTURE30[] = "GL_TEXTURE30";
static const char __pyx_k_GL_TEXTURE31[] = "GL_TEXTURE31";
static const char __pyx_k_GL_TRIANGLES[] = "GL_TRIANGLES";
static const char __pyx_k_dfactorAlpha[] = "dfactorAlpha";
static const char __pyx_k_framebuffers[] = "framebuffers";
static const char __pyx_k_glBindBuffer[] = "glBindBuffer";
static const char __pyx_k_glBlendColor[] = "glBlendColor";
static const char __pyx_k_glBufferData[] = "glBufferData";
static const char __pyx_k_glClearColor[] = "glClearColor";
static const char __pyx_k_glDrawArrays[] = "glDrawArrays";
static const char __pyx_k_glGenBuffers[] = "glGenBuffers";
static const char __pyx_k_glReadPixels[] = "glReadPixels";
static const char __pyx_k_glTexImage2D[] = "glTexImage2D";
static const char __pyx_k_glUniform1fv[] = "glUniform1fv";
static const char __pyx_k_glUniform1iv[] = "glUniform1iv";
static const char __pyx_k_glUniform2fv[] = "glUniform2fv";
static const char __pyx_k_glUniform2iv[] = "glUniform2iv";
static const char __pyx_k_glUniform3fv[] = "glUniform3fv";
static const char __pyx_k_glUniform3iv[] = "glUniform3iv";
static const char __pyx_k_glUniform4fv[] = "glUniform4fv";
static const char __pyx_k_glUniform4iv[] = "glUniform4iv";
static const char __pyx_k_glUseProgram[] = "glUseProgram";
static const char __pyx_k_renderbuffer[] = "renderbuffer";
static const char __pyx_k_sfactorAlpha[] = "sfactorAlpha";
static const char __pyx_k_textures_ptr[] = "textures_ptr";
static const char __pyx_k_GL_ALPHA_BITS[] = "GL_ALPHA_BITS";
static const char __pyx_k_GL_DEPTH_BITS[] = "GL_DEPTH_BITS";
static const char __pyx_k_GL_DEPTH_FUNC[] = "GL_DEPTH_FUNC";
static const char __pyx_k_GL_DEPTH_TEST[] = "GL_DEPTH_TEST";
static const char __pyx_k_GL_EXTENSIONS[] = "GL_EXTENSIONS";
static const char __pyx_k_GL_FLOAT_MAT2[] = "GL_FLOAT_MAT2";
static const char __pyx_k_GL_FLOAT_MAT3[] = "GL_FLOAT_MAT3";
static const char __pyx_k_GL_FLOAT_MAT4[] = "GL_FLOAT_MAT4";
static const char __pyx_k_GL_FLOAT_VEC2[] = "GL_FLOAT_VEC2";
static const char __pyx_k_GL_FLOAT_VEC3[] = "GL_FLOAT_VEC3";
static const char __pyx_k_GL_FLOAT_VEC4[] = "GL_FLOAT_VEC4";
static const char __pyx_k_GL_FRONT_FACE[] = "GL_FRONT_FACE";
static const char __pyx_k_GL_GREEN_BITS[] = "GL_GREEN_BITS";
static const char __pyx_k_GL_LINE_STRIP[] = "GL_LINE_STRIP";
static const char __pyx_k_GL_LINE_WIDTH[] = "GL_LINE_WIDTH";
static const char __pyx_k_GL_SAMPLER_2D[] = "GL_SAMPLER_2D";
static const char __pyx_k_GL_TEXTURE_2D[] = "GL_TEXTURE_2D";
static const char __pyx_k_glBindTexture[] = "glBindTexture";
static const char __pyx_k_glGenTextures[] = "glGenTextures";
static const char __pyx_k_glGetBooleanv[] = "glGetBooleanv";
static const char __pyx_k_glGetIntegerv[] = "glGetIntegerv";
static const char __pyx_k_glGetShaderiv[] = "glGetShaderiv";
static const char __pyx_k_glLinkProgram[] = "glLinkProgram";
static const char __pyx_k_glPixelStorei[] = "glPixelStorei";
static const char __pyx_k_glStencilFunc[] = "glStencilFunc";
static const char __pyx_k_glStencilMask[] = "glStencilMask";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_renderbuffers[] = "renderbuffers";
static const char __pyx_k_GL_BLEND_COLOR[] = "GL_BLEND_COLOR";
static const char __pyx_k_GL_BUFFER_SIZE[] = "GL_BUFFER_SIZE";
static const char __pyx_k_GL_DEPTH_RANGE[] = "GL_DEPTH_RANGE";
static const char __pyx_k_GL_FRAMEBUFFER[] = "GL_FRAMEBUFFER";
static const char __pyx_k_GL_LINK_STATUS[] = "GL_LINK_STATUS";
static const char __pyx_k_GL_SCISSOR_BOX[] = "GL_SCISSOR_BOX";
static const char __pyx_k_GL_SHADER_TYPE[] = "GL_SHADER_TYPE";
static const char __pyx_k_GL_STATIC_DRAW[] = "GL_STATIC_DRAW";
static const char __pyx_k_GL_STENCIL_REF[] = "GL_STENCIL_REF";
static const char __pyx_k_GL_STREAM_DRAW[] = "GL_STREAM_DRAW";
static const char __pyx_k_glAttachShader[] = "glAttachShader";
static const char __pyx_k_glClearStencil[] = "glClearStencil";
static const char __pyx_k_glCreateShader[] = "glCreateShader";
static const char __pyx_k_glDeleteShader[] = "glDeleteShader";
static const char __pyx_k_glDetachShader[] = "glDetachShader";
static const char __pyx_k_glDrawElements[] = "glDrawElements";
static const char __pyx_k_glGetProgramiv[] = "glGetProgramiv";
static const char __pyx_k_glGetUniformfv[] = "glGetUniformfv";
static const char __pyx_k_glGetUniformiv[] = "glGetUniformiv";
static const char __pyx_k_glShaderSource[] = "glShaderSource";
static const char __pyx_k_internalformat[] = "internalformat";
static const char __pyx_k_renpy_gl2_uguu[] = "renpy.gl2.uguu";
static const char __pyx_k_GL_ARRAY_BUFFER[] = "GL_ARRAY_BUFFER";
static const char __pyx_k_GL_BUFFER_USAGE[] = "GL_BUFFER_USAGE";
static const char __pyx_k_GL_DYNAMIC_DRAW[] = "GL_DYNAMIC_DRAW";
static const char __pyx_k_GL_INVALID_ENUM[] = "GL_INVALID_ENUM";
static const char __pyx_k_GL_RENDERBUFFER[] = "GL_RENDERBUFFER";
static const char __pyx_k_GL_SAMPLER_CUBE[] = "GL_SAMPLER_CUBE";
static const char __pyx_k_GL_SCISSOR_TEST[] = "GL_SCISSOR_TEST";
static const char __pyx_k_GL_STENCIL_BITS[] = "GL_STENCIL_BITS";
static const char __pyx_k_GL_STENCIL_FAIL[] = "GL_STENCIL_FAIL";
static const char __pyx_k_GL_STENCIL_FUNC[] = "GL_STENCIL_FUNC";
static const char __pyx_k_GL_STENCIL_TEST[] = "GL_STENCIL_TEST";
static const char __pyx_k_GL_TRIANGLE_FAN[] = "GL_TRIANGLE_FAN";
static const char __pyx_k_GL_UNSIGNED_INT[] = "GL_UNSIGNED_INT";
static const char __pyx_k_glActiveTexture[] = "glActiveTexture";
static const char __pyx_k_glBlendEquation[] = "glBlendEquation";
static const char __pyx_k_glBufferSubData[] = "glBufferSubData";
static const char __pyx_k_glCompileShader[] = "glCompileShader";
static const char __pyx_k_glCreateProgram[] = "glCreateProgram";
static const char __pyx_k_glDeleteBuffers[] = "glDeleteBuffers";
static const char __pyx_k_glDeleteProgram[] = "glDeleteProgram";
static const char __pyx_k_glIsFramebuffer[] = "glIsFramebuffer";
static const char __pyx_k_glPolygonOffset[] = "glPolygonOffset";
static const char __pyx_k_glTexParameterf[] = "glTexParameterf";
static const char __pyx_k_glTexParameteri[] = "glTexParameteri";
static const char __pyx_k_glTexSubImage2D[] = "glTexSubImage2D";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_GL_BLEND_DST_RGB[] = "GL_BLEND_DST_RGB";
static const char __pyx_k_GL_BLEND_SRC_RGB[] = "GL_BLEND_SRC_RGB";
static const char __pyx_k_GL_CLAMP_TO_EDGE[] = "GL_CLAMP_TO_EDGE";
static const char __pyx_k_GL_DELETE_STATUS[] = "GL_DELETE_STATUS";
static const char __pyx_k_GL_FUNC_SUBTRACT[] = "GL_FUNC_SUBTRACT";
static const char __pyx_k_GL_INVALID_VALUE[] = "GL_INVALID_VALUE";
static const char __pyx_k_GL_OUT_OF_MEMORY[] = "GL_OUT_OF_MEMORY";
static const char __pyx_k_GL_SUBPIXEL_BITS[] = "GL_SUBPIXEL_BITS";
static const char __pyx_k_GL_UNSIGNED_BYTE[] = "GL_UNSIGNED_BYTE";
static const char __pyx_k_GL_VERTEX_SHADER[] = "GL_VERTEX_SHADER";
static const char __pyx_k_framebuffers_ptr[] = "framebuffers_ptr";
static const char __pyx_k_glCopyTexImage2D[] = "glCopyTexImage2D";
static const char __pyx_k_glDeleteTextures[] = "glDeleteTextures";
static const char __pyx_k_glGenerateMipmap[] = "glGenerateMipmap";
static const char __pyx_k_glIsRenderbuffer[] = "glIsRenderbuffer";
static const char __pyx_k_glSampleCoverage[] = "glSampleCoverage";
static const char __pyx_k_glTexParameterfv[] = "glTexParameterfv";
static const char __pyx_k_glTexParameteriv[] = "glTexParameteriv";
static const char __pyx_k_glVertexAttrib1f[] = "glVertexAttrib1f";
static const char __pyx_k_glVertexAttrib2f[] = "glVertexAttrib2f";
static const char __pyx_k_glVertexAttrib3f[] = "glVertexAttrib3f";
static const char __pyx_k_glVertexAttrib4f[] = "glVertexAttrib4f";
static const char __pyx_k_GL_ACTIVE_TEXTURE[] = "GL_ACTIVE_TEXTURE";
static const char __pyx_k_GL_BLEND_EQUATION[] = "GL_BLEND_EQUATION";
static const char __pyx_k_GL_COMPILE_STATUS[] = "GL_COMPILE_STATUS";
static const char __pyx_k_GL_CONSTANT_ALPHA[] = "GL_CONSTANT_ALPHA";
static const char __pyx_k_GL_CONSTANT_COLOR[] = "GL_CONSTANT_COLOR";
static const char __pyx_k_GL_CULL_FACE_MODE[] = "GL_CULL_FACE_MODE";
static const char __pyx_k_GL_FRONT_AND_BACK[] = "GL_FRONT_AND_BACK";
static const char __pyx_k_GL_PACK_ALIGNMENT[] = "GL_PACK_ALIGNMENT";
static const char __pyx_k_GL_SAMPLE_BUFFERS[] = "GL_SAMPLE_BUFFERS";
static const char __pyx_k_GL_STENCIL_INDEX8[] = "GL_STENCIL_INDEX8";
static const char __pyx_k_GL_TEXTURE_WRAP_S[] = "GL_TEXTURE_WRAP_S";
static const char __pyx_k_GL_TEXTURE_WRAP_T[] = "GL_TEXTURE_WRAP_T";
static const char __pyx_k_GL_TRIANGLE_STRIP[] = "GL_TRIANGLE_STRIP";
static const char __pyx_k_GL_UNSIGNED_SHORT[] = "GL_UNSIGNED_SHORT";
static const char __pyx_k_glBindFramebuffer[] = "glBindFramebuffer";
static const char __pyx_k_glGenFramebuffers[] = "glGenFramebuffers";
static const char __pyx_k_glGetActiveAttrib[] = "glGetActiveAttrib";
static const char __pyx_k_glGetShaderSource[] = "glGetShaderSource";
static const char __pyx_k_glValidateProgram[] = "glValidateProgram";
static const char __pyx_k_glVertexAttrib1fv[] = "glVertexAttrib1fv";
static const char __pyx_k_glVertexAttrib2fv[] = "glVertexAttrib2fv";
static const char __pyx_k_glVertexAttrib3fv[] = "glVertexAttrib3fv";
static const char __pyx_k_glVertexAttrib4fv[] = "glVertexAttrib4fv";
static const char __pyx_k_renderbuffers_ptr[] = "renderbuffers_ptr";
static const char __pyx_k_GL_ACTIVE_UNIFORMS[] = "GL_ACTIVE_UNIFORMS";
static const char __pyx_k_GL_BLEND_DST_ALPHA[] = "GL_BLEND_DST_ALPHA";
static const char __pyx_k_GL_BLEND_SRC_ALPHA[] = "GL_BLEND_SRC_ALPHA";
static const char __pyx_k_GL_COLOR_WRITEMASK[] = "GL_COLOR_WRITEMASK";
static const char __pyx_k_GL_CURRENT_PROGRAM[] = "GL_CURRENT_PROGRAM";
static const char __pyx_k_GL_DEPTH_COMPONENT[] = "GL_DEPTH_COMPONENT";
static const char __pyx_k_GL_DEPTH_WRITEMASK[] = "GL_DEPTH_WRITEMASK";
static const char __pyx_k_GL_FRAGMENT_SHADER[] = "GL_FRAGMENT_SHADER";
static const char __pyx_k_GL_INFO_LOG_LENGTH[] = "GL_INFO_LOG_LENGTH";
static const char __pyx_k_GL_LUMINANCE_ALPHA[] = "GL_LUMINANCE_ALPHA";
static const char __pyx_k_GL_MIRRORED_REPEAT[] = "GL_MIRRORED_REPEAT";
static const char __pyx_k_GL_SAMPLE_COVERAGE[] = "GL_SAMPLE_COVERAGE";
static const char __pyx_k_GL_VALIDATE_STATUS[] = "GL_VALIDATE_STATUS";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_glBindRenderbuffer[] = "glBindRenderbuffer";
static const char __pyx_k_glGenRenderbuffers[] = "glGenRenderbuffers";
static const char __pyx_k_glGetActiveUniform[] = "glGetActiveUniform";
static const char __pyx_k_glGetShaderInfoLog[] = "glGetShaderInfoLog";
static const char __pyx_k_glUniformMatrix2fv[] = "glUniformMatrix2fv";
static const char __pyx_k_glUniformMatrix3fv[] = "glUniformMatrix3fv";
static const char __pyx_k_glUniformMatrix4fv[] = "glUniformMatrix4fv";
static const char __pyx_k_index_out_of_range[] = "index out of range";
static const char __pyx_k_renderbuffertarget[] = "renderbuffertarget";
static const char __pyx_k_GL_ATTACHED_SHADERS[] = "GL_ATTACHED_SHADERS";
static const char __pyx_k_GL_COLOR_BUFFER_BIT[] = "GL_COLOR_BUFFER_BIT";
static const char __pyx_k_GL_DEPTH_ATTACHMENT[] = "GL_DEPTH_ATTACHMENT";
static const char __pyx_k_GL_DEPTH_BUFFER_BIT[] = "GL_DEPTH_BUFFER_BIT";
static const char __pyx_k_GL_MAX_TEXTURE_SIZE[] = "GL_MAX_TEXTURE_SIZE";
static const char __pyx_k_GL_STENCIL_BACK_REF[] = "GL_STENCIL_BACK_REF";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP[] = "GL_TEXTURE_CUBE_MAP";
static const char __pyx_k_GL_UNPACK_ALIGNMENT[] = "GL_UNPACK_ALIGNMENT";
static const char __pyx_k_glBlendFuncSeparate[] = "glBlendFuncSeparate";
static const char __pyx_k_glCopyTexSubImage2D[] = "glCopyTexSubImage2D";
static const char __pyx_k_glGetAttribLocation[] = "glGetAttribLocation";
static const char __pyx_k_glGetProgramInfoLog[] = "glGetProgramInfoLog";
static const char __pyx_k_glGetTexParameterfv[] = "glGetTexParameterfv";
static const char __pyx_k_glGetTexParameteriv[] = "glGetTexParameteriv";
static const char __pyx_k_glGetVertexAttribfv[] = "glGetVertexAttribfv";
static const char __pyx_k_glGetVertexAttribiv[] = "glGetVertexAttribiv";
static const char __pyx_k_glStencilOpSeparate[] = "glStencilOpSeparate";
static const char __pyx_k_GL_ACTIVE_ATTRIBUTES[] = "GL_ACTIVE_ATTRIBUTES";
static const char __pyx_k_GL_COLOR_ATTACHMENT0[] = "GL_COLOR_ATTACHMENT0";
static const char __pyx_k_GL_COLOR_CLEAR_VALUE[] = "GL_COLOR_CLEAR_VALUE";
static const char __pyx_k_GL_DEPTH_CLEAR_VALUE[] = "GL_DEPTH_CLEAR_VALUE";
static const char __pyx_k_GL_DEPTH_COMPONENT16[] = "GL_DEPTH_COMPONENT16";
static const char __pyx_k_GL_INVALID_OPERATION[] = "GL_INVALID_OPERATION";
static const char __pyx_k_GL_MAX_VIEWPORT_DIMS[] = "GL_MAX_VIEWPORT_DIMS";
static const char __pyx_k_GL_STENCIL_BACK_FAIL[] = "GL_STENCIL_BACK_FAIL";
static const char __pyx_k_GL_STENCIL_BACK_FUNC[] = "GL_STENCIL_BACK_FUNC";
static const char __pyx_k_GL_STENCIL_WRITEMASK[] = "GL_STENCIL_WRITEMASK";
static const char __pyx_k_glBindAttribLocation[] = "glBindAttribLocation";
static const char __pyx_k_glDeleteFramebuffers[] = "glDeleteFramebuffers";
static const char __pyx_k_glGetAttachedShaders[] = "glGetAttachedShaders";
static const char __pyx_k_glGetUniformLocation[] = "glGetUniformLocation";
static const char __pyx_k_GL_BLEND_EQUATION_RGB[] = "GL_BLEND_EQUATION_RGB";
static const char __pyx_k_GL_MAX_VERTEX_ATTRIBS[] = "GL_MAX_VERTEX_ATTRIBS";
static const char __pyx_k_GL_RENDERBUFFER_WIDTH[] = "GL_RENDERBUFFER_WIDTH";
static const char __pyx_k_GL_SRC_ALPHA_SATURATE[] = "GL_SRC_ALPHA_SATURATE";
static const char __pyx_k_GL_STENCIL_ATTACHMENT[] = "GL_STENCIL_ATTACHMENT";
static const char __pyx_k_GL_STENCIL_BUFFER_BIT[] = "GL_STENCIL_BUFFER_BIT";
static const char __pyx_k_GL_STENCIL_VALUE_MASK[] = "GL_STENCIL_VALUE_MASK";
static const char __pyx_k_GL_TEXTURE_BINDING_2D[] = "GL_TEXTURE_BINDING_2D";
static const char __pyx_k_GL_TEXTURE_MAG_FILTER[] = "GL_TEXTURE_MAG_FILTER";
static const char __pyx_k_GL_TEXTURE_MIN_FILTER[] = "GL_TEXTURE_MIN_FILTER";
static const char __pyx_k_glDeleteRenderbuffers[] = "glDeleteRenderbuffers";
static const char __pyx_k_glRenderbufferStorage[] = "glRenderbufferStorage";
static const char __pyx_k_glStencilFuncSeparate[] = "glStencilFuncSeparate";
static const char __pyx_k_glStencilMaskSeparate[] = "glStencilMaskSeparate";
static const char __pyx_k_glVertexAttribPointer[] = "glVertexAttribPointer";
static const char __pyx_k_GL_FRAMEBUFFER_BINDING[] = "GL_FRAMEBUFFER_BINDING";
static const char __pyx_k_GL_ONE_MINUS_DST_ALPHA[] = "GL_ONE_MINUS_DST_ALPHA";
static const char __pyx_k_GL_ONE_MINUS_DST_COLOR[] = "GL_ONE_MINUS_DST_COLOR";
static const char __pyx_k_GL_ONE_MINUS_SRC_ALPHA[] = "GL_ONE_MINUS_SRC_ALPHA";
static const char __pyx_k_GL_ONE_MINUS_SRC_COLOR[] = "GL_ONE_MINUS_SRC_COLOR";
static const char __pyx_k_GL_POLYGON_OFFSET_FILL[] = "GL_POLYGON_OFFSET_FILL";
static const char __pyx_k_GL_RENDERBUFFER_HEIGHT[] = "GL_RENDERBUFFER_HEIGHT";
static const char __pyx_k_GL_STENCIL_CLEAR_VALUE[] = "GL_STENCIL_CLEAR_VALUE";
static const char __pyx_k_glCompressedTexImage2D[] = "glCompressedTexImage2D";
static const char __pyx_k_glFramebufferTexture2D[] = "glFramebufferTexture2D";
static const char __pyx_k_glGetBufferParameteriv[] = "glGetBufferParameteriv";
static const char __pyx_k_GL_ARRAY_BUFFER_BINDING[] = "GL_ARRAY_BUFFER_BINDING";
static const char __pyx_k_GL_BLEND_EQUATION_ALPHA[] = "GL_BLEND_EQUATION_ALPHA";
static const char __pyx_k_GL_ELEMENT_ARRAY_BUFFER[] = "GL_ELEMENT_ARRAY_BUFFER";
static const char __pyx_k_GL_FRAMEBUFFER_COMPLETE[] = "GL_FRAMEBUFFER_COMPLETE";
static const char __pyx_k_GL_GENERATE_MIPMAP_HINT[] = "GL_GENERATE_MIPMAP_HINT";
static const char __pyx_k_GL_LINEAR_MIPMAP_LINEAR[] = "GL_LINEAR_MIPMAP_LINEAR";
static const char __pyx_k_GL_POLYGON_OFFSET_UNITS[] = "GL_POLYGON_OFFSET_UNITS";
static const char __pyx_k_GL_RENDERBUFFER_BINDING[] = "GL_RENDERBUFFER_BINDING";
static const char __pyx_k_GL_SHADER_SOURCE_LENGTH[] = "GL_SHADER_SOURCE_LENGTH";
static const char __pyx_k_GL_UNSIGNED_SHORT_5_6_5[] = "GL_UNSIGNED_SHORT_5_6_5";
static const char __pyx_k_glBlendEquationSeparate[] = "glBlendEquationSeparate";
static const char __pyx_k_GL_CURRENT_VERTEX_ATTRIB[] = "GL_CURRENT_VERTEX_ATTRIB";
static const char __pyx_k_GL_FUNC_REVERSE_SUBTRACT[] = "GL_FUNC_REVERSE_SUBTRACT";
static const char __pyx_k_GL_LINEAR_MIPMAP_NEAREST[] = "GL_LINEAR_MIPMAP_NEAREST";
static const char __pyx_k_GL_MAX_RENDERBUFFER_SIZE[] = "GL_MAX_RENDERBUFFER_SIZE";
static const char __pyx_k_GL_NEAREST_MIPMAP_LINEAR[] = "GL_NEAREST_MIPMAP_LINEAR";
static const char __pyx_k_GL_POLYGON_OFFSET_FACTOR[] = "GL_POLYGON_OFFSET_FACTOR";
static const char __pyx_k_GL_RENDERBUFFER_RED_SIZE[] = "GL_RENDERBUFFER_RED_SIZE";
static const char __pyx_k_GL_SAMPLE_COVERAGE_VALUE[] = "GL_SAMPLE_COVERAGE_VALUE";
static const char __pyx_k_glCheckFramebufferStatus[] = "glCheckFramebufferStatus";
static const char __pyx_k_GL_NEAREST_MIPMAP_NEAREST[] = "GL_NEAREST_MIPMAP_NEAREST";
static const char __pyx_k_GL_RENDERBUFFER_BLUE_SIZE[] = "GL_RENDERBUFFER_BLUE_SIZE";
static const char __pyx_k_GL_SAMPLE_COVERAGE_INVERT[] = "GL_SAMPLE_COVERAGE_INVERT";
static const char __pyx_k_GL_STENCIL_BACK_WRITEMASK[] = "GL_STENCIL_BACK_WRITEMASK";
static const char __pyx_k_GL_UNSIGNED_SHORT_4_4_4_4[] = "GL_UNSIGNED_SHORT_4_4_4_4";
static const char __pyx_k_GL_UNSIGNED_SHORT_5_5_5_1[] = "GL_UNSIGNED_SHORT_5_5_5_1";
static const char __pyx_k_glCompressedTexSubImage2D[] = "glCompressedTexSubImage2D";
static const char __pyx_k_glEnableVertexAttribArray[] = "glEnableVertexAttribArray";
static const char __pyx_k_glFramebufferRenderbuffer[] = "glFramebufferRenderbuffer";
static const char __pyx_k_glGetVertexAttribPointerv[] = "glGetVertexAttribPointerv";
static const char __pyx_k_GL_FRAMEBUFFER_UNSUPPORTED[] = "GL_FRAMEBUFFER_UNSUPPORTED";
static const char __pyx_k_GL_MAX_TEXTURE_IMAGE_UNITS[] = "GL_MAX_TEXTURE_IMAGE_UNITS";
static const char __pyx_k_GL_RENDERBUFFER_ALPHA_SIZE[] = "GL_RENDERBUFFER_ALPHA_SIZE";
static const char __pyx_k_GL_RENDERBUFFER_DEPTH_SIZE[] = "GL_RENDERBUFFER_DEPTH_SIZE";
static const char __pyx_k_GL_RENDERBUFFER_GREEN_SIZE[] = "GL_RENDERBUFFER_GREEN_SIZE";
static const char __pyx_k_GL_STENCIL_BACK_VALUE_MASK[] = "GL_STENCIL_BACK_VALUE_MASK";
static const char __pyx_k_GL_STENCIL_PASS_DEPTH_FAIL[] = "GL_STENCIL_PASS_DEPTH_FAIL";
static const char __pyx_k_GL_STENCIL_PASS_DEPTH_PASS[] = "GL_STENCIL_PASS_DEPTH_PASS";
static const char __pyx_k_glDisableVertexAttribArray[] = "glDisableVertexAttribArray";
static const char __pyx_k_GL_ALIASED_LINE_WIDTH_RANGE[] = "GL_ALIASED_LINE_WIDTH_RANGE";
static const char __pyx_k_GL_ALIASED_POINT_SIZE_RANGE[] = "GL_ALIASED_POINT_SIZE_RANGE";
static const char __pyx_k_GL_ONE_MINUS_CONSTANT_ALPHA[] = "GL_ONE_MINUS_CONSTANT_ALPHA";
static const char __pyx_k_GL_ONE_MINUS_CONSTANT_COLOR[] = "GL_ONE_MINUS_CONSTANT_COLOR";
static const char __pyx_k_GL_SAMPLE_ALPHA_TO_COVERAGE[] = "GL_SAMPLE_ALPHA_TO_COVERAGE";
static const char __pyx_k_GL_SHADING_LANGUAGE_VERSION[] = "GL_SHADING_LANGUAGE_VERSION";
static const char __pyx_k_GL_TEXTURE_BINDING_CUBE_MAP[] = "GL_TEXTURE_BINDING_CUBE_MAP";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_SIZE[] = "GL_VERTEX_ATTRIB_ARRAY_SIZE";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_TYPE[] = "GL_VERTEX_ATTRIB_ARRAY_TYPE";
static const char __pyx_k_GL_ACTIVE_UNIFORM_MAX_LENGTH[] = "GL_ACTIVE_UNIFORM_MAX_LENGTH";
static const char __pyx_k_GL_MAX_CUBE_MAP_TEXTURE_SIZE[] = "GL_MAX_CUBE_MAP_TEXTURE_SIZE";
static const char __pyx_k_GL_RENDERBUFFER_STENCIL_SIZE[] = "GL_RENDERBUFFER_STENCIL_SIZE";
static const char __pyx_k_glGetRenderbufferParameteriv[] = "glGetRenderbufferParameteriv";
static const char __pyx_k_GL_COMPRESSED_TEXTURE_FORMATS[] = "GL_COMPRESSED_TEXTURE_FORMATS";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_STRIDE[] = "GL_VERTEX_ATTRIB_ARRAY_STRIDE";
static const char __pyx_k_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH[] = "GL_ACTIVE_ATTRIBUTE_MAX_LENGTH";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_X[] = "GL_TEXTURE_CUBE_MAP_NEGATIVE_X";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_Y[] = "GL_TEXTURE_CUBE_MAP_NEGATIVE_Y";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_Z[] = "GL_TEXTURE_CUBE_MAP_NEGATIVE_Z";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_X[] = "GL_TEXTURE_CUBE_MAP_POSITIVE_X";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_Y[] = "GL_TEXTURE_CUBE_MAP_POSITIVE_Y";
static const char __pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_Z[] = "GL_TEXTURE_CUBE_MAP_POSITIVE_Z";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_ENABLED[] = "GL_VERTEX_ATTRIB_ARRAY_ENABLED";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_POINTER[] = "GL_VERTEX_ATTRIB_ARRAY_POINTER";
static const char __pyx_k_GL_ELEMENT_ARRAY_BUFFER_BINDING[] = "GL_ELEMENT_ARRAY_BUFFER_BINDING";
static const char __pyx_k_GL_RENDERBUFFER_INTERNAL_FORMAT[] = "GL_RENDERBUFFER_INTERNAL_FORMAT";
static const char __pyx_k_GL_STENCIL_BACK_PASS_DEPTH_FAIL[] = "GL_STENCIL_BACK_PASS_DEPTH_FAIL";
static const char __pyx_k_GL_STENCIL_BACK_PASS_DEPTH_PASS[] = "GL_STENCIL_BACK_PASS_DEPTH_PASS";
static const char __pyx_k_home_tom_ab_renpy_renpy_gl2_ugu[] = "/home/tom/ab/renpy/renpy/gl2/uguu.pyx";
static const char __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_OBJECT[] = "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE";
static const char __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR[] = "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL";
static const char __pyx_k_GL_FRAMEBUFFER_INCOMPLETE_ATTACH[] = "GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT";
static const char __pyx_k_GL_FRAMEBUFFER_INCOMPLETE_MISSIN[] = "GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT";
static const char __pyx_k_GL_INVALID_FRAMEBUFFER_OPERATION[] = "GL_INVALID_FRAMEBUFFER_OPERATION";
static const char __pyx_k_GL_MAX_COMBINED_TEXTURE_IMAGE_UN[] = "GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS";
static const char __pyx_k_GL_MAX_VERTEX_TEXTURE_IMAGE_UNIT[] = "GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS";
static const char __pyx_k_GL_NUM_COMPRESSED_TEXTURE_FORMAT[] = "GL_NUM_COMPRESSED_TEXTURE_FORMATS";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_BUFFER_BI[] = "GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING";
static const char __pyx_k_GL_VERTEX_ATTRIB_ARRAY_NORMALIZE[] = "GL_VERTEX_ATTRIB_ARRAY_NORMALIZED";
static const char __pyx_k_glGetFramebufferAttachmentParame[] = "glGetFramebufferAttachmentParameteriv";
static const char __pyx_k_self_data_cannot_be_converted_to[] = "self.data cannot be converted to a Python object for pickling";
static const char __pyx_k_self_ptr_self_view_cannot_be_con[] = "self.ptr,self.view cannot be converted to a Python object for pickling";
static const char __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_OBJECT_2[] = "GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME";
static const char __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR_2[] = "GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE";
static PyObject *__pyx_n_s_GL_ACTIVE_ATTRIBUTES;
static PyObject *__pyx_n_s_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH;
static PyObject *__pyx_n_s_GL_ACTIVE_TEXTURE;
static PyObject *__pyx_n_s_GL_ACTIVE_UNIFORMS;
static PyObject *__pyx_n_s_GL_ACTIVE_UNIFORM_MAX_LENGTH;
static PyObject *__pyx_n_s_GL_ALIASED_LINE_WIDTH_RANGE;
static PyObject *__pyx_n_s_GL_ALIASED_POINT_SIZE_RANGE;
static PyObject *__pyx_n_s_GL_ALPHA;
static PyObject *__pyx_n_s_GL_ALPHA_BITS;
static PyObject *__pyx_n_s_GL_ALWAYS;
static PyObject *__pyx_n_s_GL_ARRAY_BUFFER;
static PyObject *__pyx_n_s_GL_ARRAY_BUFFER_BINDING;
static PyObject *__pyx_n_s_GL_ATTACHED_SHADERS;
static PyObject *__pyx_n_s_GL_BACK;
static PyObject *__pyx_n_s_GL_BLEND;
static PyObject *__pyx_n_s_GL_BLEND_COLOR;
static PyObject *__pyx_n_s_GL_BLEND_DST_ALPHA;
static PyObject *__pyx_n_s_GL_BLEND_DST_RGB;
static PyObject *__pyx_n_s_GL_BLEND_EQUATION;
static PyObject *__pyx_n_s_GL_BLEND_EQUATION_ALPHA;
static PyObject *__pyx_n_s_GL_BLEND_EQUATION_RGB;
static PyObject *__pyx_n_s_GL_BLEND_SRC_ALPHA;
static PyObject *__pyx_n_s_GL_BLEND_SRC_RGB;
static PyObject *__pyx_n_s_GL_BLUE_BITS;
static PyObject *__pyx_n_s_GL_BOOL;
static PyObject *__pyx_n_s_GL_BOOL_VEC2;
static PyObject *__pyx_n_s_GL_BOOL_VEC3;
static PyObject *__pyx_n_s_GL_BOOL_VEC4;
static PyObject *__pyx_n_s_GL_BUFFER_SIZE;
static PyObject *__pyx_n_s_GL_BUFFER_USAGE;
static PyObject *__pyx_n_s_GL_BYTE;
static PyObject *__pyx_n_s_GL_CCW;
static PyObject *__pyx_n_s_GL_CLAMP_TO_EDGE;
static PyObject *__pyx_n_s_GL_COLOR_ATTACHMENT0;
static PyObject *__pyx_n_s_GL_COLOR_BUFFER_BIT;
static PyObject *__pyx_n_s_GL_COLOR_CLEAR_VALUE;
static PyObject *__pyx_n_s_GL_COLOR_WRITEMASK;
static PyObject *__pyx_n_s_GL_COMPILE_STATUS;
static PyObject *__pyx_n_s_GL_COMPRESSED_TEXTURE_FORMATS;
static PyObject *__pyx_n_s_GL_CONSTANT_ALPHA;
static PyObject *__pyx_n_s_GL_CONSTANT_COLOR;
static PyObject *__pyx_n_s_GL_CULL_FACE;
static PyObject *__pyx_n_s_GL_CULL_FACE_MODE;
static PyObject *__pyx_n_s_GL_CURRENT_PROGRAM;
static PyObject *__pyx_n_s_GL_CURRENT_VERTEX_ATTRIB;
static PyObject *__pyx_n_s_GL_CW;
static PyObject *__pyx_n_s_GL_DECR;
static PyObject *__pyx_n_s_GL_DECR_WRAP;
static PyObject *__pyx_n_s_GL_DELETE_STATUS;
static PyObject *__pyx_n_s_GL_DEPTH_ATTACHMENT;
static PyObject *__pyx_n_s_GL_DEPTH_BITS;
static PyObject *__pyx_n_s_GL_DEPTH_BUFFER_BIT;
static PyObject *__pyx_n_s_GL_DEPTH_CLEAR_VALUE;
static PyObject *__pyx_n_s_GL_DEPTH_COMPONENT;
static PyObject *__pyx_n_s_GL_DEPTH_COMPONENT16;
static PyObject *__pyx_n_s_GL_DEPTH_FUNC;
static PyObject *__pyx_n_s_GL_DEPTH_RANGE;
static PyObject *__pyx_n_s_GL_DEPTH_TEST;
static PyObject *__pyx_n_s_GL_DEPTH_WRITEMASK;
static PyObject *__pyx_n_s_GL_DITHER;
static PyObject *__pyx_n_s_GL_DONT_CARE;
static PyObject *__pyx_n_s_GL_DST_ALPHA;
static PyObject *__pyx_n_s_GL_DST_COLOR;
static PyObject *__pyx_n_s_GL_DYNAMIC_DRAW;
static PyObject *__pyx_n_s_GL_ELEMENT_ARRAY_BUFFER;
static PyObject *__pyx_n_s_GL_ELEMENT_ARRAY_BUFFER_BINDING;
static PyObject *__pyx_n_s_GL_EQUAL;
static PyObject *__pyx_n_s_GL_EXTENSIONS;
static PyObject *__pyx_n_s_GL_FALSE;
static PyObject *__pyx_n_s_GL_FASTEST;
static PyObject *__pyx_n_s_GL_FLOAT;
static PyObject *__pyx_n_s_GL_FLOAT_MAT2;
static PyObject *__pyx_n_s_GL_FLOAT_MAT3;
static PyObject *__pyx_n_s_GL_FLOAT_MAT4;
static PyObject *__pyx_n_s_GL_FLOAT_VEC2;
static PyObject *__pyx_n_s_GL_FLOAT_VEC3;
static PyObject *__pyx_n_s_GL_FLOAT_VEC4;
static PyObject *__pyx_n_s_GL_FRAGMENT_SHADER;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_OBJECT;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_OBJECT_2;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR_2;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_BINDING;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_COMPLETE;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_INCOMPLETE_ATTACH;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_INCOMPLETE_MISSIN;
static PyObject *__pyx_n_s_GL_FRAMEBUFFER_UNSUPPORTED;
static PyObject *__pyx_n_s_GL_FRONT;
static PyObject *__pyx_n_s_GL_FRONT_AND_BACK;
static PyObject *__pyx_n_s_GL_FRONT_FACE;
static PyObject *__pyx_n_s_GL_FUNC_ADD;
static PyObject *__pyx_n_s_GL_FUNC_REVERSE_SUBTRACT;
static PyObject *__pyx_n_s_GL_FUNC_SUBTRACT;
static PyObject *__pyx_n_s_GL_GENERATE_MIPMAP_HINT;
static PyObject *__pyx_n_s_GL_GEQUAL;
static PyObject *__pyx_n_s_GL_GREATER;
static PyObject *__pyx_n_s_GL_GREEN_BITS;
static PyObject *__pyx_n_s_GL_INCR;
static PyObject *__pyx_n_s_GL_INCR_WRAP;
static PyObject *__pyx_n_s_GL_INFO_LOG_LENGTH;
static PyObject *__pyx_n_s_GL_INT;
static PyObject *__pyx_n_s_GL_INT_VEC2;
static PyObject *__pyx_n_s_GL_INT_VEC3;
static PyObject *__pyx_n_s_GL_INT_VEC4;
static PyObject *__pyx_n_s_GL_INVALID_ENUM;
static PyObject *__pyx_n_s_GL_INVALID_FRAMEBUFFER_OPERATION;
static PyObject *__pyx_n_s_GL_INVALID_OPERATION;
static PyObject *__pyx_n_s_GL_INVALID_VALUE;
static PyObject *__pyx_n_s_GL_INVERT;
static PyObject *__pyx_n_s_GL_KEEP;
static PyObject *__pyx_n_s_GL_LEQUAL;
static PyObject *__pyx_n_s_GL_LESS;
static PyObject *__pyx_n_s_GL_LINEAR;
static PyObject *__pyx_n_s_GL_LINEAR_MIPMAP_LINEAR;
static PyObject *__pyx_n_s_GL_LINEAR_MIPMAP_NEAREST;
static PyObject *__pyx_n_s_GL_LINES;
static PyObject *__pyx_n_s_GL_LINE_LOOP;
static PyObject *__pyx_n_s_GL_LINE_STRIP;
static PyObject *__pyx_n_s_GL_LINE_WIDTH;
static PyObject *__pyx_n_s_GL_LINK_STATUS;
static PyObject *__pyx_n_s_GL_LUMINANCE;
static PyObject *__pyx_n_s_GL_LUMINANCE_ALPHA;
static PyObject *__pyx_n_s_GL_MAX_COMBINED_TEXTURE_IMAGE_UN;
static PyObject *__pyx_n_s_GL_MAX_CUBE_MAP_TEXTURE_SIZE;
static PyObject *__pyx_n_s_GL_MAX_RENDERBUFFER_SIZE;
static PyObject *__pyx_n_s_GL_MAX_TEXTURE_IMAGE_UNITS;
static PyObject *__pyx_n_s_GL_MAX_TEXTURE_SIZE;
static PyObject *__pyx_n_s_GL_MAX_VERTEX_ATTRIBS;
static PyObject *__pyx_n_s_GL_MAX_VERTEX_TEXTURE_IMAGE_UNIT;
static PyObject *__pyx_n_s_GL_MAX_VIEWPORT_DIMS;
static PyObject *__pyx_n_s_GL_MIRRORED_REPEAT;
static PyObject *__pyx_n_s_GL_NEAREST;
static PyObject *__pyx_n_s_GL_NEAREST_MIPMAP_LINEAR;
static PyObject *__pyx_n_s_GL_NEAREST_MIPMAP_NEAREST;
static PyObject *__pyx_n_s_GL_NEVER;
static PyObject *__pyx_n_s_GL_NICEST;
static PyObject *__pyx_n_s_GL_NONE;
static PyObject *__pyx_n_s_GL_NOTEQUAL;
static PyObject *__pyx_n_s_GL_NO_ERROR;
static PyObject *__pyx_n_s_GL_NUM_COMPRESSED_TEXTURE_FORMAT;
static PyObject *__pyx_n_s_GL_ONE;
static PyObject *__pyx_n_s_GL_ONE_MINUS_CONSTANT_ALPHA;
static PyObject *__pyx_n_s_GL_ONE_MINUS_CONSTANT_COLOR;
static PyObject *__pyx_n_s_GL_ONE_MINUS_DST_ALPHA;
static PyObject *__pyx_n_s_GL_ONE_MINUS_DST_COLOR;
static PyObject *__pyx_n_s_GL_ONE_MINUS_SRC_ALPHA;
static PyObject *__pyx_n_s_GL_ONE_MINUS_SRC_COLOR;
static PyObject *__pyx_n_s_GL_OUT_OF_MEMORY;
static PyObject *__pyx_n_s_GL_PACK_ALIGNMENT;
static PyObject *__pyx_n_s_GL_POINTS;
static PyObject *__pyx_n_s_GL_POLYGON_OFFSET_FACTOR;
static PyObject *__pyx_n_s_GL_POLYGON_OFFSET_FILL;
static PyObject *__pyx_n_s_GL_POLYGON_OFFSET_UNITS;
static PyObject *__pyx_n_s_GL_RED_BITS;
static PyObject *__pyx_n_s_GL_RENDERBUFFER;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_ALPHA_SIZE;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_BINDING;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_BLUE_SIZE;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_DEPTH_SIZE;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_GREEN_SIZE;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_HEIGHT;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_INTERNAL_FORMAT;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_RED_SIZE;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_STENCIL_SIZE;
static PyObject *__pyx_n_s_GL_RENDERBUFFER_WIDTH;
static PyObject *__pyx_n_s_GL_RENDERER;
static PyObject *__pyx_n_s_GL_REPEAT;
static PyObject *__pyx_n_s_GL_REPLACE;
static PyObject *__pyx_n_s_GL_RGB;
static PyObject *__pyx_n_s_GL_RGB5_A1;
static PyObject *__pyx_n_s_GL_RGBA;
static PyObject *__pyx_n_s_GL_RGBA4;
static PyObject *__pyx_n_s_GL_SAMPLER_2D;
static PyObject *__pyx_n_s_GL_SAMPLER_CUBE;
static PyObject *__pyx_n_s_GL_SAMPLES;
static PyObject *__pyx_n_s_GL_SAMPLE_ALPHA_TO_COVERAGE;
static PyObject *__pyx_n_s_GL_SAMPLE_BUFFERS;
static PyObject *__pyx_n_s_GL_SAMPLE_COVERAGE;
static PyObject *__pyx_n_s_GL_SAMPLE_COVERAGE_INVERT;
static PyObject *__pyx_n_s_GL_SAMPLE_COVERAGE_VALUE;
static PyObject *__pyx_n_s_GL_SCISSOR_BOX;
static PyObject *__pyx_n_s_GL_SCISSOR_TEST;
static PyObject *__pyx_n_s_GL_SHADER_SOURCE_LENGTH;
static PyObject *__pyx_n_s_GL_SHADER_TYPE;
static PyObject *__pyx_n_s_GL_SHADING_LANGUAGE_VERSION;
static PyObject *__pyx_n_s_GL_SHORT;
static PyObject *__pyx_n_s_GL_SRC_ALPHA;
static PyObject *__pyx_n_s_GL_SRC_ALPHA_SATURATE;
static PyObject *__pyx_n_s_GL_SRC_COLOR;
static PyObject *__pyx_n_s_GL_STATIC_DRAW;
static PyObject *__pyx_n_s_GL_STENCIL_ATTACHMENT;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_FAIL;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_FUNC;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_PASS_DEPTH_FAIL;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_PASS_DEPTH_PASS;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_REF;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_VALUE_MASK;
static PyObject *__pyx_n_s_GL_STENCIL_BACK_WRITEMASK;
static PyObject *__pyx_n_s_GL_STENCIL_BITS;
static PyObject *__pyx_n_s_GL_STENCIL_BUFFER_BIT;
static PyObject *__pyx_n_s_GL_STENCIL_CLEAR_VALUE;
static PyObject *__pyx_n_s_GL_STENCIL_FAIL;
static PyObject *__pyx_n_s_GL_STENCIL_FUNC;
static PyObject *__pyx_n_s_GL_STENCIL_INDEX8;
static PyObject *__pyx_n_s_GL_STENCIL_PASS_DEPTH_FAIL;
static PyObject *__pyx_n_s_GL_STENCIL_PASS_DEPTH_PASS;
static PyObject *__pyx_n_s_GL_STENCIL_REF;
static PyObject *__pyx_n_s_GL_STENCIL_TEST;
static PyObject *__pyx_n_s_GL_STENCIL_VALUE_MASK;
static PyObject *__pyx_n_s_GL_STENCIL_WRITEMASK;
static PyObject *__pyx_n_s_GL_STREAM_DRAW;
static PyObject *__pyx_n_s_GL_SUBPIXEL_BITS;
static PyObject *__pyx_n_s_GL_TEXTURE;
static PyObject *__pyx_n_s_GL_TEXTURE0;
static PyObject *__pyx_n_s_GL_TEXTURE1;
static PyObject *__pyx_n_s_GL_TEXTURE10;
static PyObject *__pyx_n_s_GL_TEXTURE11;
static PyObject *__pyx_n_s_GL_TEXTURE12;
static PyObject *__pyx_n_s_GL_TEXTURE13;
static PyObject *__pyx_n_s_GL_TEXTURE14;
static PyObject *__pyx_n_s_GL_TEXTURE15;
static PyObject *__pyx_n_s_GL_TEXTURE16;
static PyObject *__pyx_n_s_GL_TEXTURE17;
static PyObject *__pyx_n_s_GL_TEXTURE18;
static PyObject *__pyx_n_s_GL_TEXTURE19;
static PyObject *__pyx_n_s_GL_TEXTURE2;
static PyObject *__pyx_n_s_GL_TEXTURE20;
static PyObject *__pyx_n_s_GL_TEXTURE21;
static PyObject *__pyx_n_s_GL_TEXTURE22;
static PyObject *__pyx_n_s_GL_TEXTURE23;
static PyObject *__pyx_n_s_GL_TEXTURE24;
static PyObject *__pyx_n_s_GL_TEXTURE25;
static PyObject *__pyx_n_s_GL_TEXTURE26;
static PyObject *__pyx_n_s_GL_TEXTURE27;
static PyObject *__pyx_n_s_GL_TEXTURE28;
static PyObject *__pyx_n_s_GL_TEXTURE29;
static PyObject *__pyx_n_s_GL_TEXTURE3;
static PyObject *__pyx_n_s_GL_TEXTURE30;
static PyObject *__pyx_n_s_GL_TEXTURE31;
static PyObject *__pyx_n_s_GL_TEXTURE4;
static PyObject *__pyx_n_s_GL_TEXTURE5;
static PyObject *__pyx_n_s_GL_TEXTURE6;
static PyObject *__pyx_n_s_GL_TEXTURE7;
static PyObject *__pyx_n_s_GL_TEXTURE8;
static PyObject *__pyx_n_s_GL_TEXTURE9;
static PyObject *__pyx_n_s_GL_TEXTURE_2D;
static PyObject *__pyx_n_s_GL_TEXTURE_BINDING_2D;
static PyObject *__pyx_n_s_GL_TEXTURE_BINDING_CUBE_MAP;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_X;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_Y;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_Z;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_X;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_Y;
static PyObject *__pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_Z;
static PyObject *__pyx_n_s_GL_TEXTURE_MAG_FILTER;
static PyObject *__pyx_n_s_GL_TEXTURE_MIN_FILTER;
static PyObject *__pyx_n_s_GL_TEXTURE_WRAP_S;
static PyObject *__pyx_n_s_GL_TEXTURE_WRAP_T;
static PyObject *__pyx_n_s_GL_TRIANGLES;
static PyObject *__pyx_n_s_GL_TRIANGLE_FAN;
static PyObject *__pyx_n_s_GL_TRIANGLE_STRIP;
static PyObject *__pyx_n_s_GL_TRUE;
static PyObject *__pyx_n_s_GL_UNPACK_ALIGNMENT;
static PyObject *__pyx_n_s_GL_UNSIGNED_BYTE;
static PyObject *__pyx_n_s_GL_UNSIGNED_INT;
static PyObject *__pyx_n_s_GL_UNSIGNED_SHORT;
static PyObject *__pyx_n_s_GL_UNSIGNED_SHORT_4_4_4_4;
static PyObject *__pyx_n_s_GL_UNSIGNED_SHORT_5_5_5_1;
static PyObject *__pyx_n_s_GL_UNSIGNED_SHORT_5_6_5;
static PyObject *__pyx_n_s_GL_VALIDATE_STATUS;
static PyObject *__pyx_n_s_GL_VENDOR;
static PyObject *__pyx_n_s_GL_VERSION;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_BUFFER_BI;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_ENABLED;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_NORMALIZE;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_POINTER;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_SIZE;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_STRIDE;
static PyObject *__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_TYPE;
static PyObject *__pyx_n_s_GL_VERTEX_SHADER;
static PyObject *__pyx_n_s_GL_VIEWPORT;
static PyObject *__pyx_n_s_GL_ZERO;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_alpha;
static PyObject *__pyx_n_s_attachment;
static PyObject *__pyx_n_s_blue;
static PyObject *__pyx_n_s_border;
static PyObject *__pyx_n_s_bufSize;
static PyObject *__pyx_n_s_buffer;
static PyObject *__pyx_n_s_buffers;
static PyObject *__pyx_n_s_buffers_ptr;
static PyObject *__pyx_n_s_cap;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_count;
static PyObject *__pyx_n_s_count_ptr;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_data_ptr;
static PyObject *__pyx_n_s_dfactor;
static PyObject *__pyx_n_s_dfactorAlpha;
static PyObject *__pyx_n_s_dfactorRGB;
static PyObject *__pyx_n_s_dpfail;
static PyObject *__pyx_n_s_dppass;
static PyObject *__pyx_n_s_face;
static PyObject *__pyx_n_s_factor;
static PyObject *__pyx_n_s_fail;
static PyObject *__pyx_n_s_first;
static PyObject *__pyx_n_s_flag;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_framebuffer;
static PyObject *__pyx_n_s_framebuffers;
static PyObject *__pyx_n_s_framebuffers_ptr;
static PyObject *__pyx_n_s_func;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_glActiveTexture;
static PyObject *__pyx_n_s_glAttachShader;
static PyObject *__pyx_n_s_glBindAttribLocation;
static PyObject *__pyx_n_s_glBindBuffer;
static PyObject *__pyx_n_s_glBindFramebuffer;
static PyObject *__pyx_n_s_glBindRenderbuffer;
static PyObject *__pyx_n_s_glBindTexture;
static PyObject *__pyx_n_s_glBlendColor;
static PyObject *__pyx_n_s_glBlendEquation;
static PyObject *__pyx_n_s_glBlendEquationSeparate;
static PyObject *__pyx_n_s_glBlendFunc;
static PyObject *__pyx_n_s_glBlendFuncSeparate;
static PyObject *__pyx_n_s_glBufferData;
static PyObject *__pyx_n_s_glBufferSubData;
static PyObject *__pyx_n_s_glCheckFramebufferStatus;
static PyObject *__pyx_n_s_glClear;
static PyObject *__pyx_n_s_glClearColor;
static PyObject *__pyx_n_s_glClearStencil;
static PyObject *__pyx_n_s_glColorMask;
static PyObject *__pyx_n_s_glCompileShader;
static PyObject *__pyx_n_s_glCompressedTexImage2D;
static PyObject *__pyx_n_s_glCompressedTexSubImage2D;
static PyObject *__pyx_n_s_glCopyTexImage2D;
static PyObject *__pyx_n_s_glCopyTexSubImage2D;
static PyObject *__pyx_n_s_glCreateProgram;
static PyObject *__pyx_n_s_glCreateShader;
static PyObject *__pyx_n_s_glCullFace;
static PyObject *__pyx_n_s_glDeleteBuffers;
static PyObject *__pyx_n_s_glDeleteFramebuffers;
static PyObject *__pyx_n_s_glDeleteProgram;
static PyObject *__pyx_n_s_glDeleteRenderbuffers;
static PyObject *__pyx_n_s_glDeleteShader;
static PyObject *__pyx_n_s_glDeleteTextures;
static PyObject *__pyx_n_s_glDepthFunc;
static PyObject *__pyx_n_s_glDepthMask;
static PyObject *__pyx_n_s_glDetachShader;
static PyObject *__pyx_n_s_glDisable;
static PyObject *__pyx_n_s_glDisableVertexAttribArray;
static PyObject *__pyx_n_s_glDrawArrays;
static PyObject *__pyx_n_s_glDrawElements;
static PyObject *__pyx_n_s_glEnable;
static PyObject *__pyx_n_s_glEnableVertexAttribArray;
static PyObject *__pyx_n_s_glFinish;
static PyObject *__pyx_n_s_glFlush;
static PyObject *__pyx_n_s_glFramebufferRenderbuffer;
static PyObject *__pyx_n_s_glFramebufferTexture2D;
static PyObject *__pyx_n_s_glFrontFace;
static PyObject *__pyx_n_s_glGenBuffers;
static PyObject *__pyx_n_s_glGenFramebuffers;
static PyObject *__pyx_n_s_glGenRenderbuffers;
static PyObject *__pyx_n_s_glGenTextures;
static PyObject *__pyx_n_s_glGenerateMipmap;
static PyObject *__pyx_n_s_glGetActiveAttrib;
static PyObject *__pyx_n_s_glGetActiveUniform;
static PyObject *__pyx_n_s_glGetAttachedShaders;
static PyObject *__pyx_n_s_glGetAttribLocation;
static PyObject *__pyx_n_s_glGetBooleanv;
static PyObject *__pyx_n_s_glGetBufferParameteriv;
static PyObject *__pyx_n_s_glGetError;
static PyObject *__pyx_n_s_glGetFloatv;
static PyObject *__pyx_n_s_glGetFramebufferAttachmentParame;
static PyObject *__pyx_n_s_glGetIntegerv;
static PyObject *__pyx_n_s_glGetProgramInfoLog;
static PyObject *__pyx_n_s_glGetProgramiv;
static PyObject *__pyx_n_s_glGetRenderbufferParameteriv;
static PyObject *__pyx_n_s_glGetShaderInfoLog;
static PyObject *__pyx_n_s_glGetShaderSource;
static PyObject *__pyx_n_s_glGetShaderiv;
static PyObject *__pyx_n_s_glGetString;
static PyObject *__pyx_n_s_glGetTexParameterfv;
static PyObject *__pyx_n_s_glGetTexParameteriv;
static PyObject *__pyx_n_s_glGetUniformLocation;
static PyObject *__pyx_n_s_glGetUniformfv;
static PyObject *__pyx_n_s_glGetUniformiv;
static PyObject *__pyx_n_s_glGetVertexAttribPointerv;
static PyObject *__pyx_n_s_glGetVertexAttribfv;
static PyObject *__pyx_n_s_glGetVertexAttribiv;
static PyObject *__pyx_n_s_glHint;
static PyObject *__pyx_n_s_glIsBuffer;
static PyObject *__pyx_n_s_glIsEnabled;
static PyObject *__pyx_n_s_glIsFramebuffer;
static PyObject *__pyx_n_s_glIsProgram;
static PyObject *__pyx_n_s_glIsRenderbuffer;
static PyObject *__pyx_n_s_glIsShader;
static PyObject *__pyx_n_s_glIsTexture;
static PyObject *__pyx_n_s_glLineWidth;
static PyObject *__pyx_n_s_glLinkProgram;
static PyObject *__pyx_n_s_glPixelStorei;
static PyObject *__pyx_n_s_glPolygonOffset;
static PyObject *__pyx_n_s_glReadPixels;
static PyObject *__pyx_n_s_glRenderbufferStorage;
static PyObject *__pyx_n_s_glSampleCoverage;
static PyObject *__pyx_n_s_glScissor;
static PyObject *__pyx_n_s_glShaderSource;
static PyObject *__pyx_n_s_glStencilFunc;
static PyObject *__pyx_n_s_glStencilFuncSeparate;
static PyObject *__pyx_n_s_glStencilMask;
static PyObject *__pyx_n_s_glStencilMaskSeparate;
static PyObject *__pyx_n_s_glStencilOp;
static PyObject *__pyx_n_s_glStencilOpSeparate;
static PyObject *__pyx_n_s_glTexImage2D;
static PyObject *__pyx_n_s_glTexParameterf;
static PyObject *__pyx_n_s_glTexParameterfv;
static PyObject *__pyx_n_s_glTexParameteri;
static PyObject *__pyx_n_s_glTexParameteriv;
static PyObject *__pyx_n_s_glTexSubImage2D;
static PyObject *__pyx_n_s_glUniform1f;
static PyObject *__pyx_n_s_glUniform1fv;
static PyObject *__pyx_n_s_glUniform1i;
static PyObject *__pyx_n_s_glUniform1iv;
static PyObject *__pyx_n_s_glUniform2f;
static PyObject *__pyx_n_s_glUniform2fv;
static PyObject *__pyx_n_s_glUniform2i;
static PyObject *__pyx_n_s_glUniform2iv;
static PyObject *__pyx_n_s_glUniform3f;
static PyObject *__pyx_n_s_glUniform3fv;
static PyObject *__pyx_n_s_glUniform3i;
static PyObject *__pyx_n_s_glUniform3iv;
static PyObject *__pyx_n_s_glUniform4f;
static PyObject *__pyx_n_s_glUniform4fv;
static PyObject *__pyx_n_s_glUniform4i;
static PyObject *__pyx_n_s_glUniform4iv;
static PyObject *__pyx_n_s_glUniformMatrix2fv;
static PyObject *__pyx_n_s_glUniformMatrix3fv;
static PyObject *__pyx_n_s_glUniformMatrix4fv;
static PyObject *__pyx_n_s_glUseProgram;
static PyObject *__pyx_n_s_glValidateProgram;
static PyObject *__pyx_n_s_glVertexAttrib1f;
static PyObject *__pyx_n_s_glVertexAttrib1fv;
static PyObject *__pyx_n_s_glVertexAttrib2f;
static PyObject *__pyx_n_s_glVertexAttrib2fv;
static PyObject *__pyx_n_s_glVertexAttrib3f;
static PyObject *__pyx_n_s_glVertexAttrib3fv;
static PyObject *__pyx_n_s_glVertexAttrib4f;
static PyObject *__pyx_n_s_glVertexAttrib4fv;
static PyObject *__pyx_n_s_glVertexAttribPointer;
static PyObject *__pyx_n_s_glViewport;
static PyObject *__pyx_n_s_green;
static PyObject *__pyx_n_s_height;
static PyObject *__pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu;
static PyObject *__pyx_n_s_imageSize;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_kp_s_index_out_of_range;
static PyObject *__pyx_n_s_indices;
static PyObject *__pyx_n_s_indices_ptr;
static PyObject *__pyx_n_s_infoLog;
static PyObject *__pyx_n_s_infoLog_ptr;
static PyObject *__pyx_n_s_internalformat;
static PyObject *__pyx_n_s_invert;
static PyObject *__pyx_n_s_length;
static PyObject *__pyx_n_s_length_ptr;
static PyObject *__pyx_n_s_level;
static PyObject *__pyx_n_s_load;
static PyObject *__pyx_n_s_location;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mask;
static PyObject *__pyx_n_s_maxCount;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_modeAlpha;
static PyObject *__pyx_n_s_modeRGB;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_name_ptr;
static PyObject *__pyx_n_s_normalized;
static PyObject *__pyx_n_s_o;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_param;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_s_params_ptr;
static PyObject *__pyx_n_s_pixels;
static PyObject *__pyx_n_s_pixels_ptr;
static PyObject *__pyx_n_s_pname;
static PyObject *__pyx_n_s_pointer;
static PyObject *__pyx_n_s_pointer_ptr;
static PyObject *__pyx_n_s_program;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_red;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_ref;
static PyObject *__pyx_n_s_renderbuffer;
static PyObject *__pyx_n_s_renderbuffers;
static PyObject *__pyx_n_s_renderbuffers_ptr;
static PyObject *__pyx_n_s_renderbuffertarget;
static PyObject *__pyx_n_s_renpy_gl2_uguu;
static PyObject *__pyx_n_s_ro;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_kp_s_self_data_cannot_be_converted_to;
static PyObject *__pyx_kp_s_self_ptr_self_view_cannot_be_con;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_sfactor;
static PyObject *__pyx_n_s_sfactorAlpha;
static PyObject *__pyx_n_s_sfactorRGB;
static PyObject *__pyx_n_s_sfail;
static PyObject *__pyx_n_s_shader;
static PyObject *__pyx_n_s_shaders;
static PyObject *__pyx_n_s_shaders_ptr;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_size_ptr;
static PyObject *__pyx_n_s_source;
static PyObject *__pyx_n_s_source_ptr;
static PyObject *__pyx_n_s_stride;
static PyObject *__pyx_n_s_string;
static PyObject *__pyx_n_s_string_ptr;
static PyObject *__pyx_n_s_target;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_textarget;
static PyObject *__pyx_n_s_texture;
static PyObject *__pyx_n_s_textures;
static PyObject *__pyx_n_s_textures_ptr;
static PyObject *__pyx_n_s_transpose;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_type_ptr;
static PyObject *__pyx_n_s_uguugl;
static PyObject *__pyx_n_s_units;
static PyObject *__pyx_n_s_usage;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_n_s_v0;
static PyObject *__pyx_n_s_v1;
static PyObject *__pyx_n_s_v2;
static PyObject *__pyx_n_s_v3;
static PyObject *__pyx_n_s_v_ptr;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_value_ptr;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_width;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_xoffset;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_yoffset;
static PyObject *__pyx_n_s_z;
static PyObject *__pyx_n_s_zfail;
static PyObject *__pyx_n_s_zpass;
static int __pyx_pf_5renpy_3gl2_4uguu_3ptr___init__(struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self, PyObject *__pyx_v_o, PyObject *__pyx_v_ro); /* proto */
static void __pyx_pf_5renpy_3gl2_4uguu_3ptr_2__dealloc__(struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_3ptr_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_3ptr_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5renpy_3gl2_4uguu_6Buffer___getbuffer__(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, Py_buffer *__pyx_v_buffer, CYTHON_UNUSED int __pyx_v_flags); /* proto */
static void __pyx_pf_5renpy_3gl2_4uguu_6Buffer_2__releasebuffer__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, CYTHON_UNUSED Py_buffer *__pyx_v_buffer); /* proto */
static void __pyx_pf_5renpy_3gl2_4uguu_6Buffer_4__dealloc__(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_6Buffer_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_6Buffer_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self, PyObject *__pyx_v_length); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_2get(struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5renpy_3gl2_4uguu_9IntBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_2__getitem__(struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_2__getitem__(struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_glActiveTexture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_texture); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_2glAttachShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_shader); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_4glBindAttribLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_index, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_6glBindBuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_buffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_8glBindFramebuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_framebuffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_10glBindRenderbuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_renderbuffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_12glBindTexture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_texture); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_14glBlendColor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_red, PyObject *__pyx_v_green, PyObject *__pyx_v_blue, PyObject *__pyx_v_alpha); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_16glBlendEquation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_18glBlendEquationSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_modeRGB, PyObject *__pyx_v_modeAlpha); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_20glBlendFunc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sfactor, PyObject *__pyx_v_dfactor); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_22glBlendFuncSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sfactorRGB, PyObject *__pyx_v_dfactorRGB, PyObject *__pyx_v_sfactorAlpha, PyObject *__pyx_v_dfactorAlpha); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_24glBufferData(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_size, PyObject *__pyx_v_data, PyObject *__pyx_v_usage); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_26glBufferSubData(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_offset, PyObject *__pyx_v_size, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_28glCheckFramebufferStatus(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_30glClear(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mask); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_32glClearColor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_red, PyObject *__pyx_v_green, PyObject *__pyx_v_blue, PyObject *__pyx_v_alpha); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_34glClearStencil(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_36glColorMask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_red, PyObject *__pyx_v_green, PyObject *__pyx_v_blue, PyObject *__pyx_v_alpha); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_38glCompileShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_40glCompressedTexImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_border, PyObject *__pyx_v_imageSize, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_42glCompressedTexSubImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_xoffset, PyObject *__pyx_v_yoffset, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_format, PyObject *__pyx_v_imageSize, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_44glCopyTexImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_border); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_46glCopyTexSubImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_xoffset, PyObject *__pyx_v_yoffset, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_48glCreateProgram(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_50glCreateShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_type); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_52glCullFace(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_54glDeleteBuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_buffers); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_56glDeleteFramebuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_framebuffers); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_58glDeleteProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_60glDeleteRenderbuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_renderbuffers); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_62glDeleteShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_64glDeleteTextures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_textures); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_66glDepthFunc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_func); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_68glDepthMask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_flag); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_70glDetachShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_shader); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_72glDisable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cap); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_74glDisableVertexAttribArray(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_76glDrawArrays(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_first, PyObject *__pyx_v_count); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_78glDrawElements(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_count, PyObject *__pyx_v_type, PyObject *__pyx_v_indices); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_80glEnable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cap); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_82glEnableVertexAttribArray(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_84glFinish(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_86glFlush(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_88glFramebufferRenderbuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_attachment, PyObject *__pyx_v_renderbuffertarget, PyObject *__pyx_v_renderbuffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_90glFramebufferTexture2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_attachment, PyObject *__pyx_v_textarget, PyObject *__pyx_v_texture, PyObject *__pyx_v_level); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_92glFrontFace(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_94glGenBuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_buffers); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_96glGenFramebuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_framebuffers); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_98glGenRenderbuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_renderbuffers); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_100glGenTextures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_textures); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_102glGenerateMipmap(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_104glGetActiveAttrib(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_index, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_size, PyObject *__pyx_v_type, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_106glGetActiveUniform(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_index, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_size, PyObject *__pyx_v_type, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_108glGetAttachedShaders(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_maxCount, PyObject *__pyx_v_count, PyObject *__pyx_v_shaders); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_110glGetAttribLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_112glGetBooleanv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_114glGetBufferParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_116glGetError(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_118glGetFloatv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_120glGetFramebufferAttachmentParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_attachment, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_122glGetIntegerv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_124glGetProgramInfoLog(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_infoLog); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_126glGetProgramiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_128glGetRenderbufferParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_130glGetShaderInfoLog(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_infoLog); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_132glGetShaderSource(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_source); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_134glGetShaderiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_136glGetString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_138glGetTexParameterfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_140glGetTexParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_142glGetUniformLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_144glGetUniformfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_location, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_146glGetUniformiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_location, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_148glGetVertexAttribPointerv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_pname, PyObject *__pyx_v_pointer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_150glGetVertexAttribfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_152glGetVertexAttribiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_154glHint(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_mode); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_156glIsBuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_buffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_158glIsEnabled(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cap); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_160glIsFramebuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_framebuffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_162glIsProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_164glIsRenderbuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_renderbuffer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_166glIsShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_168glIsTexture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_texture); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_170glLineWidth(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_172glLinkProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_174glPixelStorei(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_param); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_176glPolygonOffset(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_factor, PyObject *__pyx_v_units); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_178glReadPixels(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_format, PyObject *__pyx_v_type, PyObject *__pyx_v_pixels); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_180glRenderbufferStorage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_width, PyObject *__pyx_v_height); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_182glSampleCoverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_value, PyObject *__pyx_v_invert); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_184glScissor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_186glShaderSource(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_count, PyObject *__pyx_v_string, PyObject *__pyx_v_length); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_188glStencilFunc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_func, PyObject *__pyx_v_ref, PyObject *__pyx_v_mask); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_190glStencilFuncSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_face, PyObject *__pyx_v_func, PyObject *__pyx_v_ref, PyObject *__pyx_v_mask); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_192glStencilMask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mask); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_194glStencilMaskSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_face, PyObject *__pyx_v_mask); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_196glStencilOp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_fail, PyObject *__pyx_v_zfail, PyObject *__pyx_v_zpass); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_198glStencilOpSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_face, PyObject *__pyx_v_sfail, PyObject *__pyx_v_dpfail, PyObject *__pyx_v_dppass); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_200glTexImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_border, PyObject *__pyx_v_format, PyObject *__pyx_v_type, PyObject *__pyx_v_pixels); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_202glTexParameterf(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_param); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_204glTexParameterfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_206glTexParameteri(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_param); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_208glTexParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_210glTexSubImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_xoffset, PyObject *__pyx_v_yoffset, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_format, PyObject *__pyx_v_type, PyObject *__pyx_v_pixels); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_212glUniform1f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_214glUniform1fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_216glUniform1i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_218glUniform1iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_220glUniform2f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_222glUniform2fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_224glUniform2i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_226glUniform2iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_228glUniform3f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_230glUniform3fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_232glUniform3i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_234glUniform3iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_236glUniform4f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2, PyObject *__pyx_v_v3); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_238glUniform4fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_240glUniform4i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2, PyObject *__pyx_v_v3); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_242glUniform4iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_244glUniformMatrix2fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_transpose, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_246glUniformMatrix3fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_transpose, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_248glUniformMatrix4fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_transpose, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_250glUseProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_252glValidateProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_254glVertexAttrib1f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_256glVertexAttrib1fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_258glVertexAttrib2f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_260glVertexAttrib2fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_262glVertexAttrib3f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_z); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_264glVertexAttrib3fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_266glVertexAttrib4f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_z, PyObject *__pyx_v_w); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_268glVertexAttrib4fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_270glVertexAttribPointer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_size, PyObject *__pyx_v_type, PyObject *__pyx_v_normalized, PyObject *__pyx_v_stride, PyObject *__pyx_v_pointer); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_4uguu_272glViewport(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height); /* proto */
static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_ptr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_Buffer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_BytesBuffer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_BytesListBuffer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_IntBuffer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_FloatBuffer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__100;
static PyObject *__pyx_tuple__102;
static PyObject *__pyx_tuple__104;
static PyObject *__pyx_tuple__106;
static PyObject *__pyx_tuple__108;
static PyObject *__pyx_tuple__110;
static PyObject *__pyx_tuple__112;
static PyObject *__pyx_tuple__114;
static PyObject *__pyx_tuple__116;
static PyObject *__pyx_tuple__118;
static PyObject *__pyx_tuple__120;
static PyObject *__pyx_tuple__122;
static PyObject *__pyx_tuple__124;
static PyObject *__pyx_tuple__126;
static PyObject *__pyx_tuple__129;
static PyObject *__pyx_tuple__131;
static PyObject *__pyx_tuple__133;
static PyObject *__pyx_tuple__135;
static PyObject *__pyx_tuple__137;
static PyObject *__pyx_tuple__139;
static PyObject *__pyx_tuple__141;
static PyObject *__pyx_tuple__143;
static PyObject *__pyx_tuple__145;
static PyObject *__pyx_tuple__147;
static PyObject *__pyx_tuple__149;
static PyObject *__pyx_tuple__151;
static PyObject *__pyx_tuple__153;
static PyObject *__pyx_tuple__155;
static PyObject *__pyx_tuple__157;
static PyObject *__pyx_tuple__159;
static PyObject *__pyx_tuple__161;
static PyObject *__pyx_tuple__163;
static PyObject *__pyx_tuple__165;
static PyObject *__pyx_tuple__167;
static PyObject *__pyx_tuple__169;
static PyObject *__pyx_tuple__171;
static PyObject *__pyx_tuple__173;
static PyObject *__pyx_tuple__175;
static PyObject *__pyx_tuple__177;
static PyObject *__pyx_tuple__179;
static PyObject *__pyx_tuple__181;
static PyObject *__pyx_tuple__183;
static PyObject *__pyx_tuple__185;
static PyObject *__pyx_tuple__187;
static PyObject *__pyx_tuple__189;
static PyObject *__pyx_tuple__191;
static PyObject *__pyx_tuple__193;
static PyObject *__pyx_tuple__195;
static PyObject *__pyx_tuple__197;
static PyObject *__pyx_tuple__199;
static PyObject *__pyx_tuple__201;
static PyObject *__pyx_tuple__203;
static PyObject *__pyx_tuple__205;
static PyObject *__pyx_tuple__207;
static PyObject *__pyx_tuple__209;
static PyObject *__pyx_tuple__211;
static PyObject *__pyx_tuple__213;
static PyObject *__pyx_tuple__215;
static PyObject *__pyx_tuple__217;
static PyObject *__pyx_tuple__219;
static PyObject *__pyx_tuple__221;
static PyObject *__pyx_tuple__223;
static PyObject *__pyx_tuple__225;
static PyObject *__pyx_tuple__227;
static PyObject *__pyx_tuple__229;
static PyObject *__pyx_tuple__231;
static PyObject *__pyx_tuple__233;
static PyObject *__pyx_tuple__235;
static PyObject *__pyx_tuple__237;
static PyObject *__pyx_tuple__239;
static PyObject *__pyx_tuple__241;
static PyObject *__pyx_tuple__243;
static PyObject *__pyx_tuple__245;
static PyObject *__pyx_tuple__247;
static PyObject *__pyx_tuple__249;
static PyObject *__pyx_tuple__251;
static PyObject *__pyx_tuple__253;
static PyObject *__pyx_tuple__255;
static PyObject *__pyx_tuple__257;
static PyObject *__pyx_tuple__259;
static PyObject *__pyx_tuple__261;
static PyObject *__pyx_tuple__263;
static PyObject *__pyx_tuple__265;
static PyObject *__pyx_tuple__267;
static PyObject *__pyx_tuple__269;
static PyObject *__pyx_tuple__271;
static PyObject *__pyx_tuple__273;
static PyObject *__pyx_tuple__275;
static PyObject *__pyx_tuple__277;
static PyObject *__pyx_tuple__279;
static PyObject *__pyx_tuple__281;
static PyObject *__pyx_tuple__283;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
static PyObject *__pyx_codeobj__48;
static PyObject *__pyx_codeobj__50;
static PyObject *__pyx_codeobj__52;
static PyObject *__pyx_codeobj__54;
static PyObject *__pyx_codeobj__56;
static PyObject *__pyx_codeobj__58;
static PyObject *__pyx_codeobj__60;
static PyObject *__pyx_codeobj__62;
static PyObject *__pyx_codeobj__63;
static PyObject *__pyx_codeobj__65;
static PyObject *__pyx_codeobj__67;
static PyObject *__pyx_codeobj__69;
static PyObject *__pyx_codeobj__71;
static PyObject *__pyx_codeobj__73;
static PyObject *__pyx_codeobj__75;
static PyObject *__pyx_codeobj__77;
static PyObject *__pyx_codeobj__79;
static PyObject *__pyx_codeobj__81;
static PyObject *__pyx_codeobj__83;
static PyObject *__pyx_codeobj__85;
static PyObject *__pyx_codeobj__87;
static PyObject *__pyx_codeobj__89;
static PyObject *__pyx_codeobj__91;
static PyObject *__pyx_codeobj__93;
static PyObject *__pyx_codeobj__95;
static PyObject *__pyx_codeobj__97;
static PyObject *__pyx_codeobj__98;
static PyObject *__pyx_codeobj__99;
static PyObject *__pyx_codeobj__101;
static PyObject *__pyx_codeobj__103;
static PyObject *__pyx_codeobj__105;
static PyObject *__pyx_codeobj__107;
static PyObject *__pyx_codeobj__109;
static PyObject *__pyx_codeobj__111;
static PyObject *__pyx_codeobj__113;
static PyObject *__pyx_codeobj__115;
static PyObject *__pyx_codeobj__117;
static PyObject *__pyx_codeobj__119;
static PyObject *__pyx_codeobj__121;
static PyObject *__pyx_codeobj__123;
static PyObject *__pyx_codeobj__125;
static PyObject *__pyx_codeobj__127;
static PyObject *__pyx_codeobj__128;
static PyObject *__pyx_codeobj__130;
static PyObject *__pyx_codeobj__132;
static PyObject *__pyx_codeobj__134;
static PyObject *__pyx_codeobj__136;
static PyObject *__pyx_codeobj__138;
static PyObject *__pyx_codeobj__140;
static PyObject *__pyx_codeobj__142;
static PyObject *__pyx_codeobj__144;
static PyObject *__pyx_codeobj__146;
static PyObject *__pyx_codeobj__148;
static PyObject *__pyx_codeobj__150;
static PyObject *__pyx_codeobj__152;
static PyObject *__pyx_codeobj__154;
static PyObject *__pyx_codeobj__156;
static PyObject *__pyx_codeobj__158;
static PyObject *__pyx_codeobj__160;
static PyObject *__pyx_codeobj__162;
static PyObject *__pyx_codeobj__164;
static PyObject *__pyx_codeobj__166;
static PyObject *__pyx_codeobj__168;
static PyObject *__pyx_codeobj__170;
static PyObject *__pyx_codeobj__172;
static PyObject *__pyx_codeobj__174;
static PyObject *__pyx_codeobj__176;
static PyObject *__pyx_codeobj__178;
static PyObject *__pyx_codeobj__180;
static PyObject *__pyx_codeobj__182;
static PyObject *__pyx_codeobj__184;
static PyObject *__pyx_codeobj__186;
static PyObject *__pyx_codeobj__188;
static PyObject *__pyx_codeobj__190;
static PyObject *__pyx_codeobj__192;
static PyObject *__pyx_codeobj__194;
static PyObject *__pyx_codeobj__196;
static PyObject *__pyx_codeobj__198;
static PyObject *__pyx_codeobj__200;
static PyObject *__pyx_codeobj__202;
static PyObject *__pyx_codeobj__204;
static PyObject *__pyx_codeobj__206;
static PyObject *__pyx_codeobj__208;
static PyObject *__pyx_codeobj__210;
static PyObject *__pyx_codeobj__212;
static PyObject *__pyx_codeobj__214;
static PyObject *__pyx_codeobj__216;
static PyObject *__pyx_codeobj__218;
static PyObject *__pyx_codeobj__220;
static PyObject *__pyx_codeobj__222;
static PyObject *__pyx_codeobj__224;
static PyObject *__pyx_codeobj__226;
static PyObject *__pyx_codeobj__228;
static PyObject *__pyx_codeobj__230;
static PyObject *__pyx_codeobj__232;
static PyObject *__pyx_codeobj__234;
static PyObject *__pyx_codeobj__236;
static PyObject *__pyx_codeobj__238;
static PyObject *__pyx_codeobj__240;
static PyObject *__pyx_codeobj__242;
static PyObject *__pyx_codeobj__244;
static PyObject *__pyx_codeobj__246;
static PyObject *__pyx_codeobj__248;
static PyObject *__pyx_codeobj__250;
static PyObject *__pyx_codeobj__252;
static PyObject *__pyx_codeobj__254;
static PyObject *__pyx_codeobj__256;
static PyObject *__pyx_codeobj__258;
static PyObject *__pyx_codeobj__260;
static PyObject *__pyx_codeobj__262;
static PyObject *__pyx_codeobj__264;
static PyObject *__pyx_codeobj__266;
static PyObject *__pyx_codeobj__268;
static PyObject *__pyx_codeobj__270;
static PyObject *__pyx_codeobj__272;
static PyObject *__pyx_codeobj__274;
static PyObject *__pyx_codeobj__276;
static PyObject *__pyx_codeobj__278;
static PyObject *__pyx_codeobj__280;
static PyObject *__pyx_codeobj__282;
static PyObject *__pyx_codeobj__284;

/* "renpy/gl2/uguu.pyx":43
 * from uguugl import load
 * 
 * cdef object proxy_return_string(const GLubyte *s):             # <<<<<<<<<<<<<<
 *     """
 *     This is used for string return values. It returns the return value as
 */

static PyObject *__pyx_f_5renpy_3gl2_4uguu_proxy_return_string(GLubyte const *__pyx_v_s) {
  char const *__pyx_v_ss;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("proxy_return_string", 0);

  /* "renpy/gl2/uguu.pyx":49
 *     """
 * 
 *     if s == NULL:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_t_1 = ((__pyx_v_s == NULL) != 0);
  if (__pyx_t_1) {

    /* "renpy/gl2/uguu.pyx":50
 * 
 *     if s == NULL:
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     cdef const char *ss = <const char *> s
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "renpy/gl2/uguu.pyx":49
 *     """
 * 
 *     if s == NULL:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  }

  /* "renpy/gl2/uguu.pyx":52
 *         return None
 * 
 *     cdef const char *ss = <const char *> s             # <<<<<<<<<<<<<<
 *     return ss
 * 
 */
  __pyx_v_ss = ((char const *)__pyx_v_s);

  /* "renpy/gl2/uguu.pyx":53
 * 
 *     cdef const char *ss = <const char *> s
 *     return ss             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBytes_FromString(__pyx_v_ss); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":43
 * from uguugl import load
 * 
 * cdef object proxy_return_string(const GLubyte *s):             # <<<<<<<<<<<<<<
 *     """
 *     This is used for string return values. It returns the return value as
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.proxy_return_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":65
 *     cdef Py_buffer view
 * 
 *     def __init__(self, o, ro=True):             # <<<<<<<<<<<<<<
 *         if o is None:
 *             self.ptr = NULL
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_4uguu_3ptr_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_3gl2_4uguu_3ptr_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_o = 0;
  PyObject *__pyx_v_ro = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_o,&__pyx_n_s_ro,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_True);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_o)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ro);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 65, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_o = values[0];
    __pyx_v_ro = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 65, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.ptr.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_3ptr___init__(((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_v_self), __pyx_v_o, __pyx_v_ro);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_4uguu_3ptr___init__(struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self, PyObject *__pyx_v_o, PyObject *__pyx_v_ro) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  void *__pyx_t_5;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/uguu.pyx":66
 * 
 *     def __init__(self, o, ro=True):
 *         if o is None:             # <<<<<<<<<<<<<<
 *             self.ptr = NULL
 *             return
 */
  __pyx_t_1 = (__pyx_v_o == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/uguu.pyx":67
 *     def __init__(self, o, ro=True):
 *         if o is None:
 *             self.ptr = NULL             # <<<<<<<<<<<<<<
 *             return
 * 
 */
    __pyx_v_self->ptr = NULL;

    /* "renpy/gl2/uguu.pyx":68
 *         if o is None:
 *             self.ptr = NULL
 *             return             # <<<<<<<<<<<<<<
 * 
 *         PyObject_GetBuffer(o, &self.view, PyBUF_CONTIG_RO if ro else PyBUF_CONTIG)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "renpy/gl2/uguu.pyx":66
 * 
 *     def __init__(self, o, ro=True):
 *         if o is None:             # <<<<<<<<<<<<<<
 *             self.ptr = NULL
 *             return
 */
  }

  /* "renpy/gl2/uguu.pyx":70
 *             return
 * 
 *         PyObject_GetBuffer(o, &self.view, PyBUF_CONTIG_RO if ro else PyBUF_CONTIG)             # <<<<<<<<<<<<<<
 *         self.ptr = self.view.buf
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_ro); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 70, __pyx_L1_error)
  if (__pyx_t_2) {
    __pyx_t_3 = PyBUF_CONTIG_RO;
  } else {
    __pyx_t_3 = PyBUF_CONTIG;
  }
  __pyx_t_4 = PyObject_GetBuffer(__pyx_v_o, (&__pyx_v_self->view), __pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 70, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":71
 * 
 *         PyObject_GetBuffer(o, &self.view, PyBUF_CONTIG_RO if ro else PyBUF_CONTIG)
 *         self.ptr = self.view.buf             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_5 = __pyx_v_self->view.buf;
  __pyx_v_self->ptr = __pyx_t_5;

  /* "renpy/gl2/uguu.pyx":65
 *     cdef Py_buffer view
 * 
 *     def __init__(self, o, ro=True):             # <<<<<<<<<<<<<<
 *         if o is None:
 *             self.ptr = NULL
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.ptr.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":73
 *         self.ptr = self.view.buf
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.ptr:
 *             PyBuffer_Release(&self.view)
 */

/* Python wrapper */
static void __pyx_pw_5renpy_3gl2_4uguu_3ptr_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5renpy_3gl2_4uguu_3ptr_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5renpy_3gl2_4uguu_3ptr_2__dealloc__(((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5renpy_3gl2_4uguu_3ptr_2__dealloc__(struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "renpy/gl2/uguu.pyx":74
 * 
 *     def __dealloc__(self):
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             PyBuffer_Release(&self.view)
 *             self.ptr = NULL
 */
  __pyx_t_1 = (__pyx_v_self->ptr != 0);
  if (__pyx_t_1) {

    /* "renpy/gl2/uguu.pyx":75
 *     def __dealloc__(self):
 *         if self.ptr:
 *             PyBuffer_Release(&self.view)             # <<<<<<<<<<<<<<
 *             self.ptr = NULL
 * 
 */
    PyBuffer_Release((&__pyx_v_self->view));

    /* "renpy/gl2/uguu.pyx":76
 *         if self.ptr:
 *             PyBuffer_Release(&self.view)
 *             self.ptr = NULL             # <<<<<<<<<<<<<<
 * 
 * cdef ptr get_ptr(o):
 */
    __pyx_v_self->ptr = NULL;

    /* "renpy/gl2/uguu.pyx":74
 * 
 *     def __dealloc__(self):
 *         if self.ptr:             # <<<<<<<<<<<<<<
 *             PyBuffer_Release(&self.view)
 *             self.ptr = NULL
 */
  }

  /* "renpy/gl2/uguu.pyx":73
 *         self.ptr = self.view.buf
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.ptr:
 *             PyBuffer_Release(&self.view)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_3ptr_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_3ptr_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_3ptr_4__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_3ptr_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.ptr.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_3ptr_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_3ptr_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_3ptr_6__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_3ptr_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.ptr.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":78
 *             self.ptr = NULL
 * 
 * cdef ptr get_ptr(o):             # <<<<<<<<<<<<<<
 *     """
 *     If o is a ptr, return it. Otherwise, convert the buffer into a ptr, and
 */

static struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_f_5renpy_3gl2_4uguu_get_ptr(PyObject *__pyx_v_o) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("get_ptr", 0);

  /* "renpy/gl2/uguu.pyx":84
 *     """
 * 
 *     if isinstance(o, ptr):             # <<<<<<<<<<<<<<
 *         return o
 *     else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, __pyx_ptype_5renpy_3gl2_4uguu_ptr); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/uguu.pyx":85
 * 
 *     if isinstance(o, ptr):
 *         return o             # <<<<<<<<<<<<<<
 *     else:
 *         return ptr(o)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    if (!(likely(((__pyx_v_o) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_o, __pyx_ptype_5renpy_3gl2_4uguu_ptr))))) __PYX_ERR(1, 85, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_o);
    __pyx_r = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_v_o);
    goto __pyx_L0;

    /* "renpy/gl2/uguu.pyx":84
 *     """
 * 
 *     if isinstance(o, ptr):             # <<<<<<<<<<<<<<
 *         return o
 *     else:
 */
  }

  /* "renpy/gl2/uguu.pyx":87
 *         return o
 *     else:
 *         return ptr(o)             # <<<<<<<<<<<<<<
 * 
 * cdef class Buffer:
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_o);
    __Pyx_GIVEREF(__pyx_v_o);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_o);
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_4uguu_ptr), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_4);
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "renpy/gl2/uguu.pyx":78
 *             self.ptr = NULL
 * 
 * cdef ptr get_ptr(o):             # <<<<<<<<<<<<<<
 *     """
 *     If o is a ptr, return it. Otherwise, convert the buffer into a ptr, and
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.uguu.get_ptr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":100
 *     cdef int readonly
 * 
 *     cdef setup_buffer(self, Py_ssize_t length, Py_ssize_t itemsize, const char *format, int readonly):             # <<<<<<<<<<<<<<
 *         """
 *         This is called by a specific buffer's init method to set up various fields, and especially
 */

static PyObject *__pyx_f_5renpy_3gl2_4uguu_6Buffer_setup_buffer(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, Py_ssize_t __pyx_v_length, Py_ssize_t __pyx_v_itemsize, char const *__pyx_v_format, int __pyx_v_readonly) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("setup_buffer", 0);

  /* "renpy/gl2/uguu.pyx":115
 *         """
 * 
 *         self.length = length             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 *         self.format = format
 */
  __pyx_v_self->length = __pyx_v_length;

  /* "renpy/gl2/uguu.pyx":116
 * 
 *         self.length = length
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 *         self.format = format
 *         self.readonly = readonly
 */
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "renpy/gl2/uguu.pyx":117
 *         self.length = length
 *         self.itemsize = itemsize
 *         self.format = format             # <<<<<<<<<<<<<<
 *         self.readonly = readonly
 *         self.data = calloc(self.length, self.itemsize)
 */
  __pyx_v_self->format = __pyx_v_format;

  /* "renpy/gl2/uguu.pyx":118
 *         self.itemsize = itemsize
 *         self.format = format
 *         self.readonly = readonly             # <<<<<<<<<<<<<<
 *         self.data = calloc(self.length, self.itemsize)
 * 
 */
  __pyx_v_self->readonly = __pyx_v_readonly;

  /* "renpy/gl2/uguu.pyx":119
 *         self.format = format
 *         self.readonly = readonly
 *         self.data = calloc(self.length, self.itemsize)             # <<<<<<<<<<<<<<
 * 
 *     def __getbuffer__(self, Py_buffer *buffer, int flags):
 */
  __pyx_v_self->data = calloc(__pyx_v_self->length, __pyx_v_self->itemsize);

  /* "renpy/gl2/uguu.pyx":100
 *     cdef int readonly
 * 
 *     cdef setup_buffer(self, Py_ssize_t length, Py_ssize_t itemsize, const char *format, int readonly):             # <<<<<<<<<<<<<<
 *         """
 *         This is called by a specific buffer's init method to set up various fields, and especially
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":121
 *         self.data = calloc(self.length, self.itemsize)
 * 
 *     def __getbuffer__(self, Py_buffer *buffer, int flags):             # <<<<<<<<<<<<<<
 * 
 *         buffer.buf = self.data
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5renpy_3gl2_4uguu_6Buffer_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_buffer, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5renpy_3gl2_4uguu_6Buffer_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_buffer, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_6Buffer___getbuffer__(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), ((Py_buffer *)__pyx_v_buffer), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_4uguu_6Buffer___getbuffer__(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, Py_buffer *__pyx_v_buffer, CYTHON_UNUSED int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  void *__pyx_t_1;
  char const *__pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_buffer != NULL) {
    __pyx_v_buffer->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_buffer->obj);
  }

  /* "renpy/gl2/uguu.pyx":123
 *     def __getbuffer__(self, Py_buffer *buffer, int flags):
 * 
 *         buffer.buf = self.data             # <<<<<<<<<<<<<<
 *         buffer.format = self.format
 *         buffer.internal = NULL
 */
  __pyx_t_1 = __pyx_v_self->data;
  __pyx_v_buffer->buf = __pyx_t_1;

  /* "renpy/gl2/uguu.pyx":124
 * 
 *         buffer.buf = self.data
 *         buffer.format = self.format             # <<<<<<<<<<<<<<
 *         buffer.internal = NULL
 *         buffer.itemsize = self.itemsize
 */
  __pyx_t_2 = __pyx_v_self->format;
  __pyx_v_buffer->format = __pyx_t_2;

  /* "renpy/gl2/uguu.pyx":125
 *         buffer.buf = self.data
 *         buffer.format = self.format
 *         buffer.internal = NULL             # <<<<<<<<<<<<<<
 *         buffer.itemsize = self.itemsize
 *         buffer.len = self.length * self.itemsize
 */
  __pyx_v_buffer->internal = NULL;

  /* "renpy/gl2/uguu.pyx":126
 *         buffer.format = self.format
 *         buffer.internal = NULL
 *         buffer.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         buffer.len = self.length * self.itemsize
 *         buffer.ndim = 1
 */
  __pyx_t_3 = __pyx_v_self->itemsize;
  __pyx_v_buffer->itemsize = __pyx_t_3;

  /* "renpy/gl2/uguu.pyx":127
 *         buffer.internal = NULL
 *         buffer.itemsize = self.itemsize
 *         buffer.len = self.length * self.itemsize             # <<<<<<<<<<<<<<
 *         buffer.ndim = 1
 *         buffer.obj = self
 */
  __pyx_v_buffer->len = (__pyx_v_self->length * __pyx_v_self->itemsize);

  /* "renpy/gl2/uguu.pyx":128
 *         buffer.itemsize = self.itemsize
 *         buffer.len = self.length * self.itemsize
 *         buffer.ndim = 1             # <<<<<<<<<<<<<<
 *         buffer.obj = self
 *         buffer.readonly = self.readonly
 */
  __pyx_v_buffer->ndim = 1;

  /* "renpy/gl2/uguu.pyx":129
 *         buffer.len = self.length * self.itemsize
 *         buffer.ndim = 1
 *         buffer.obj = self             # <<<<<<<<<<<<<<
 *         buffer.readonly = self.readonly
 *         buffer.shape = &self.length
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_buffer->obj);
  __Pyx_DECREF(__pyx_v_buffer->obj);
  __pyx_v_buffer->obj = ((PyObject *)__pyx_v_self);

  /* "renpy/gl2/uguu.pyx":130
 *         buffer.ndim = 1
 *         buffer.obj = self
 *         buffer.readonly = self.readonly             # <<<<<<<<<<<<<<
 *         buffer.shape = &self.length
 *         buffer.strides = &self.itemsize
 */
  __pyx_t_4 = __pyx_v_self->readonly;
  __pyx_v_buffer->readonly = __pyx_t_4;

  /* "renpy/gl2/uguu.pyx":131
 *         buffer.obj = self
 *         buffer.readonly = self.readonly
 *         buffer.shape = &self.length             # <<<<<<<<<<<<<<
 *         buffer.strides = &self.itemsize
 *         buffer.suboffsets = NULL
 */
  __pyx_v_buffer->shape = (&__pyx_v_self->length);

  /* "renpy/gl2/uguu.pyx":132
 *         buffer.readonly = self.readonly
 *         buffer.shape = &self.length
 *         buffer.strides = &self.itemsize             # <<<<<<<<<<<<<<
 *         buffer.suboffsets = NULL
 * 
 */
  __pyx_v_buffer->strides = (&__pyx_v_self->itemsize);

  /* "renpy/gl2/uguu.pyx":133
 *         buffer.shape = &self.length
 *         buffer.strides = &self.itemsize
 *         buffer.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __releasebuffer__(self, Py_buffer *buffer):
 */
  __pyx_v_buffer->suboffsets = NULL;

  /* "renpy/gl2/uguu.pyx":121
 *         self.data = calloc(self.length, self.itemsize)
 * 
 *     def __getbuffer__(self, Py_buffer *buffer, int flags):             # <<<<<<<<<<<<<<
 * 
 *         buffer.buf = self.data
 */

  /* function exit code */
  __pyx_r = 0;
  if (__pyx_v_buffer != NULL && __pyx_v_buffer->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_buffer->obj = NULL;
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":135
 *         buffer.suboffsets = NULL
 * 
 *     def __releasebuffer__(self, Py_buffer *buffer):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5renpy_3gl2_4uguu_6Buffer_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_buffer); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5renpy_3gl2_4uguu_6Buffer_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_buffer) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5renpy_3gl2_4uguu_6Buffer_2__releasebuffer__(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), ((Py_buffer *)__pyx_v_buffer));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5renpy_3gl2_4uguu_6Buffer_2__releasebuffer__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, CYTHON_UNUSED Py_buffer *__pyx_v_buffer) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl2/uguu.pyx":138
 *         pass
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.data:
 *             free(self.data)
 */

/* Python wrapper */
static void __pyx_pw_5renpy_3gl2_4uguu_6Buffer_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5renpy_3gl2_4uguu_6Buffer_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5renpy_3gl2_4uguu_6Buffer_4__dealloc__(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5renpy_3gl2_4uguu_6Buffer_4__dealloc__(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "renpy/gl2/uguu.pyx":139
 * 
 *     def __dealloc__(self):
 *         if self.data:             # <<<<<<<<<<<<<<
 *             free(self.data)
 *             self.data = NULL
 */
  __pyx_t_1 = (__pyx_v_self->data != 0);
  if (__pyx_t_1) {

    /* "renpy/gl2/uguu.pyx":140
 *     def __dealloc__(self):
 *         if self.data:
 *             free(self.data)             # <<<<<<<<<<<<<<
 *             self.data = NULL
 * 
 */
    free(__pyx_v_self->data);

    /* "renpy/gl2/uguu.pyx":141
 *         if self.data:
 *             free(self.data)
 *             self.data = NULL             # <<<<<<<<<<<<<<
 * 
 * cdef class BytesBuffer(Buffer):
 */
    __pyx_v_self->data = NULL;

    /* "renpy/gl2/uguu.pyx":139
 * 
 *     def __dealloc__(self):
 *         if self.data:             # <<<<<<<<<<<<<<
 *             free(self.data)
 *             self.data = NULL
 */
  }

  /* "renpy/gl2/uguu.pyx":138
 *         pass
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         if self.data:
 *             free(self.data)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_6Buffer_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_6Buffer_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_6Buffer_6__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_6Buffer_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.Buffer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_6Buffer_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_6Buffer_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_6Buffer_8__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_6Buffer_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.Buffer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":145
 * cdef class BytesBuffer(Buffer):
 * 
 *     def __init__(self, length):             # <<<<<<<<<<<<<<
 * 
 *         self.setup_buffer(length, 1, "B", 0)
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_length = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_length,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 145, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_length = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 145, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer___init__(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *)__pyx_v_self), __pyx_v_length);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self, PyObject *__pyx_v_length) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/uguu.pyx":147
 *     def __init__(self, length):
 * 
 *         self.setup_buffer(length, 1, "B", 0)             # <<<<<<<<<<<<<<
 * 
 *     def get(self):
 */
  __pyx_t_1 = __Pyx_PyIndex_AsSsize_t(__pyx_v_length); if (unlikely((__pyx_t_1 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 147, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesBuffer *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.setup_buffer(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), __pyx_t_1, 1, ((char const *)"B"), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 147, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl2/uguu.pyx":145
 * cdef class BytesBuffer(Buffer):
 * 
 *     def __init__(self, length):             # <<<<<<<<<<<<<<
 * 
 *         self.setup_buffer(length, 1, "B", 0)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":149
 *         self.setup_buffer(length, 1, "B", 0)
 * 
 *     def get(self):             # <<<<<<<<<<<<<<
 *         return bytes(<char *> self.data)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_3get(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_3get(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_2get(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_2get(struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get", 0);

  /* "renpy/gl2/uguu.pyx":150
 * 
 *     def get(self):
 *         return bytes(<char *> self.data)             # <<<<<<<<<<<<<<
 * 
 * cdef class BytesListBuffer(Buffer):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(((char *)__pyx_v_self->__pyx_base.data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyBytes_Type)), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":149
 *         self.setup_buffer(length, 1, "B", 0)
 * 
 *     def get(self):             # <<<<<<<<<<<<<<
 *         return bytes(<char *> self.data)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesBuffer.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_4__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesBuffer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_6__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11BytesBuffer_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesBuffer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":155
 *     cdef object value
 * 
 *     def __init__(self, value):             # <<<<<<<<<<<<<<
 *         self.value = [ ptr(v) for v in value ]
 *         self.setup_buffer(len(value), sizeof(const char *), "P", 1)
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 155, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_value = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 155, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesListBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer___init__(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_v_i;
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/uguu.pyx":156
 * 
 *     def __init__(self, value):
 *         self.value = [ ptr(v) for v in value ]             # <<<<<<<<<<<<<<
 *         self.setup_buffer(len(value), sizeof(const char *), "P", 1)
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_v_value)) || PyTuple_CheckExact(__pyx_v_value)) {
    __pyx_t_2 = __pyx_v_value; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 156, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(1, 156, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 156, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(1, 156, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 156, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(1, 156, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_v);
    __Pyx_GIVEREF(__pyx_v_v);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_v);
    __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_4uguu_ptr), __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_6))) __PYX_ERR(1, 156, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->value);
  __Pyx_DECREF(__pyx_v_self->value);
  __pyx_v_self->value = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":157
 *     def __init__(self, value):
 *         self.value = [ ptr(v) for v in value ]
 *         self.setup_buffer(len(value), sizeof(const char *), "P", 1)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
 */
  __pyx_t_3 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 157, __pyx_L1_error)
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesListBuffer *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.setup_buffer(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), __pyx_t_3, (sizeof(char const *)), ((char const *)"P"), 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":161
 *         cdef int i
 * 
 *         for 0 <= i < self.length:             # <<<<<<<<<<<<<<
 *             (<const char **> self.data)[i] = <const char *> (<ptr> self.value[i]).ptr
 * 
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.length;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_3; __pyx_v_i++) {

    /* "renpy/gl2/uguu.pyx":162
 * 
 *         for 0 <= i < self.length:
 *             (<const char **> self.data)[i] = <const char *> (<ptr> self.value[i]).ptr             # <<<<<<<<<<<<<<
 * 
 * cdef class IntBuffer(Buffer):
 */
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_self->value, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 162, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    (((char const **)__pyx_v_self->__pyx_base.data)[__pyx_v_i]) = ((char const *)((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1)->ptr);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/uguu.pyx":155
 *     cdef object value
 * 
 *     def __init__(self, value):             # <<<<<<<<<<<<<<
 *         self.value = [ ptr(v) for v in value ]
 *         self.setup_buffer(len(value), sizeof(const char *), "P", 1)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesListBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer_2__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesListBuffer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer_4__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_15BytesListBuffer_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.BytesListBuffer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":166
 * cdef class IntBuffer(Buffer):
 * 
 *     def __init__(self, value):             # <<<<<<<<<<<<<<
 * 
 *         self.setup_buffer(len(value), sizeof(int), "i", 0)
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 166, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_value = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 166, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.IntBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_9IntBuffer___init__(((struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_4uguu_9IntBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/uguu.pyx":168
 *     def __init__(self, value):
 * 
 *         self.setup_buffer(len(value), sizeof(int), "i", 0)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(1, 168, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_5renpy_3gl2_4uguu_IntBuffer *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.setup_buffer(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), __pyx_t_1, (sizeof(int)), ((char const *)"i"), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl2/uguu.pyx":172
 *         cdef int i
 * 
 *         for 0 <= i < self.length:             # <<<<<<<<<<<<<<
 *             (<int*> self.data)[i] = <int> value[i]
 * 
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.length;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "renpy/gl2/uguu.pyx":173
 * 
 *         for 0 <= i < self.length:
 *             (<int*> self.data)[i] = <int> value[i]             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, index):
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_value, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 173, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    (((int *)__pyx_v_self->__pyx_base.data)[__pyx_v_i]) = ((int)__pyx_t_3);
  }

  /* "renpy/gl2/uguu.pyx":166
 * cdef class IntBuffer(Buffer):
 * 
 *     def __init__(self, value):             # <<<<<<<<<<<<<<
 * 
 *         self.setup_buffer(len(value), sizeof(int), "i", 0)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.IntBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":175
 *             (<int*> self.data)[i] = <int> value[i]
 * 
 *     def __getitem__(self, index):             # <<<<<<<<<<<<<<
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_2__getitem__(((struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_2__getitem__(struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "renpy/gl2/uguu.pyx":176
 * 
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError("index out of range")
 * 
 */
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 176, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(1, 176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->__pyx_base.length); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_index, __pyx_t_2, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(1, 176, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "renpy/gl2/uguu.pyx":177
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")             # <<<<<<<<<<<<<<
 * 
 *         return (<int*> self.data)[index]
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_IndexError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 177, __pyx_L1_error)

    /* "renpy/gl2/uguu.pyx":176
 * 
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError("index out of range")
 * 
 */
  }

  /* "renpy/gl2/uguu.pyx":179
 *             raise IndexError("index out of range")
 * 
 *         return (<int*> self.data)[index]             # <<<<<<<<<<<<<<
 * 
 * cdef class FloatBuffer(Buffer):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 179, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_From_int((((int *)__pyx_v_self->__pyx_base.data)[__pyx_t_5])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":175
 *             (<int*> self.data)[i] = <int> value[i]
 * 
 *     def __getitem__(self, index):             # <<<<<<<<<<<<<<
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.uguu.IntBuffer.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_4__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.IntBuffer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_6__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_9IntBuffer_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.IntBuffer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":183
 * cdef class FloatBuffer(Buffer):
 * 
 *     def __init__(self, value):             # <<<<<<<<<<<<<<
 * 
 *         self.setup_buffer(len(value), sizeof(float), "f", 0)
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 183, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_value = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 183, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.FloatBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer___init__(((struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *)__pyx_v_self), __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer___init__(struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  float __pyx_t_3;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/uguu.pyx":185
 *     def __init__(self, value):
 * 
 *         self.setup_buffer(len(value), sizeof(float), "f", 0)             # <<<<<<<<<<<<<<
 * 
 *         cdef int i
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(1, 185, __pyx_L1_error)
  __pyx_t_2 = ((struct __pyx_vtabstruct_5renpy_3gl2_4uguu_FloatBuffer *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.setup_buffer(((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)__pyx_v_self), __pyx_t_1, (sizeof(float)), ((char const *)"f"), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl2/uguu.pyx":189
 *         cdef int i
 * 
 *         for 0 <= i < self.length:             # <<<<<<<<<<<<<<
 *             (<float *> self.data)[i] = <float> value[i]
 * 
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.length;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "renpy/gl2/uguu.pyx":190
 * 
 *         for 0 <= i < self.length:
 *             (<float *> self.data)[i] = <float> value[i]             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, index):
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_value, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 190, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    (((float *)__pyx_v_self->__pyx_base.data)[__pyx_v_i]) = ((float)__pyx_t_3);
  }

  /* "renpy/gl2/uguu.pyx":183
 * cdef class FloatBuffer(Buffer):
 * 
 *     def __init__(self, value):             # <<<<<<<<<<<<<<
 * 
 *         self.setup_buffer(len(value), sizeof(float), "f", 0)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.FloatBuffer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":192
 *             (<float *> self.data)[i] = <float> value[i]
 * 
 *     def __getitem__(self, index):             # <<<<<<<<<<<<<<
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_2__getitem__(((struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_2__getitem__(struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "renpy/gl2/uguu.pyx":193
 * 
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError("index out of range")
 * 
 */
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_index, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 193, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->__pyx_base.length); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_index, __pyx_t_2, Py_GE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "renpy/gl2/uguu.pyx":194
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")             # <<<<<<<<<<<<<<
 * 
 *         return (<float *> self.data)[index]
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_IndexError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 194, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 194, __pyx_L1_error)

    /* "renpy/gl2/uguu.pyx":193
 * 
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:             # <<<<<<<<<<<<<<
 *             raise IndexError("index out of range")
 * 
 */
  }

  /* "renpy/gl2/uguu.pyx":196
 *             raise IndexError("index out of range")
 * 
 *         return (<float *> self.data)[index]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 196, __pyx_L1_error)
  __pyx_t_4 = PyFloat_FromDouble((((float *)__pyx_v_self->__pyx_base.data)[__pyx_t_5])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 196, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":192
 *             (<float *> self.data)[i] = <float> value[i]
 * 
 *     def __getitem__(self, index):             # <<<<<<<<<<<<<<
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.uguu.FloatBuffer.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_4__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_4__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.FloatBuffer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_6__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_11FloatBuffer_6__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.FloatBuffer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":231
 * from uguugl cimport GLuint64EXT
 * 
 * def glActiveTexture(texture):             # <<<<<<<<<<<<<<
 *     uguugl.glActiveTexture(texture)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_1glActiveTexture(PyObject *__pyx_self, PyObject *__pyx_v_texture); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_1glActiveTexture = {"glActiveTexture", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_1glActiveTexture, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_1glActiveTexture(PyObject *__pyx_self, PyObject *__pyx_v_texture) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glActiveTexture (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_glActiveTexture(__pyx_self, ((PyObject *)__pyx_v_texture));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_glActiveTexture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_texture) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glActiveTexture", 0);

  /* "renpy/gl2/uguu.pyx":232
 * 
 * def glActiveTexture(texture):
 *     uguugl.glActiveTexture(texture)             # <<<<<<<<<<<<<<
 * 
 * def glAttachShader(program, shader):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_texture); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 232, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glActiveTexture(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":231
 * from uguugl cimport GLuint64EXT
 * 
 * def glActiveTexture(texture):             # <<<<<<<<<<<<<<
 *     uguugl.glActiveTexture(texture)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glActiveTexture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":234
 *     uguugl.glActiveTexture(texture)
 * 
 * def glAttachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glAttachShader(program, shader)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_3glAttachShader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_3glAttachShader = {"glAttachShader", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_3glAttachShader, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_3glAttachShader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glAttachShader (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_shader,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glAttachShader", 1, 2, 2, 1); __PYX_ERR(1, 234, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glAttachShader") < 0)) __PYX_ERR(1, 234, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_program = values[0];
    __pyx_v_shader = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glAttachShader", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 234, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glAttachShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_2glAttachShader(__pyx_self, __pyx_v_program, __pyx_v_shader);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_2glAttachShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glAttachShader", 0);

  /* "renpy/gl2/uguu.pyx":235
 * 
 * def glAttachShader(program, shader):
 *     uguugl.glAttachShader(program, shader)             # <<<<<<<<<<<<<<
 * 
 * def glBindAttribLocation(program, index, name):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 235, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 235, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glAttachShader(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":234
 *     uguugl.glActiveTexture(texture)
 * 
 * def glAttachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glAttachShader(program, shader)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glAttachShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":237
 *     uguugl.glAttachShader(program, shader)
 * 
 * def glBindAttribLocation(program, index, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_5glBindAttribLocation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_5glBindAttribLocation = {"glBindAttribLocation", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_5glBindAttribLocation, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_5glBindAttribLocation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBindAttribLocation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_index,&__pyx_n_s_name,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBindAttribLocation", 1, 3, 3, 1); __PYX_ERR(1, 237, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBindAttribLocation", 1, 3, 3, 2); __PYX_ERR(1, 237, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBindAttribLocation") < 0)) __PYX_ERR(1, 237, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_program = values[0];
    __pyx_v_index = values[1];
    __pyx_v_name = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBindAttribLocation", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 237, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindAttribLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_4glBindAttribLocation(__pyx_self, __pyx_v_program, __pyx_v_index, __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_4glBindAttribLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_index, PyObject *__pyx_v_name) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_name_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLuint __pyx_t_3;
  __Pyx_RefNannySetupContext("glBindAttribLocation", 0);

  /* "renpy/gl2/uguu.pyx":238
 * 
 * def glBindAttribLocation(program, index, name):
 *     cdef ptr name_ptr = get_ptr(name)             # <<<<<<<<<<<<<<
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_name)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_name_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":239
 * def glBindAttribLocation(program, index, name):
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glBindBuffer(target, buffer):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 239, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_3 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 239, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBindAttribLocation(__pyx_t_2, __pyx_t_3, ((GLchar const *)__pyx_v_name_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":237
 *     uguugl.glAttachShader(program, shader)
 * 
 * def glBindAttribLocation(program, index, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindAttribLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_name_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":241
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 * 
 * def glBindBuffer(target, buffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindBuffer(target, buffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_7glBindBuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_7glBindBuffer = {"glBindBuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_7glBindBuffer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_7glBindBuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_buffer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBindBuffer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_buffer,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_buffer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBindBuffer", 1, 2, 2, 1); __PYX_ERR(1, 241, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBindBuffer") < 0)) __PYX_ERR(1, 241, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_target = values[0];
    __pyx_v_buffer = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBindBuffer", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 241, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindBuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_6glBindBuffer(__pyx_self, __pyx_v_target, __pyx_v_buffer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_6glBindBuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_buffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glBindBuffer", 0);

  /* "renpy/gl2/uguu.pyx":242
 * 
 * def glBindBuffer(target, buffer):
 *     uguugl.glBindBuffer(target, buffer)             # <<<<<<<<<<<<<<
 * 
 * def glBindFramebuffer(target, framebuffer):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 242, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_buffer); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 242, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBindBuffer(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":241
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 * 
 * def glBindBuffer(target, buffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindBuffer(target, buffer)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindBuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":244
 *     uguugl.glBindBuffer(target, buffer)
 * 
 * def glBindFramebuffer(target, framebuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9glBindFramebuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_9glBindFramebuffer = {"glBindFramebuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_9glBindFramebuffer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_9glBindFramebuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_framebuffer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBindFramebuffer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_framebuffer,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_framebuffer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBindFramebuffer", 1, 2, 2, 1); __PYX_ERR(1, 244, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBindFramebuffer") < 0)) __PYX_ERR(1, 244, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_target = values[0];
    __pyx_v_framebuffer = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBindFramebuffer", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 244, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindFramebuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_8glBindFramebuffer(__pyx_self, __pyx_v_target, __pyx_v_framebuffer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_8glBindFramebuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_framebuffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glBindFramebuffer", 0);

  /* "renpy/gl2/uguu.pyx":245
 * 
 * def glBindFramebuffer(target, framebuffer):
 *     uguugl.glBindFramebuffer(target, framebuffer)             # <<<<<<<<<<<<<<
 * 
 * def glBindRenderbuffer(target, renderbuffer):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 245, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_framebuffer); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 245, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBindFramebuffer(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":244
 *     uguugl.glBindBuffer(target, buffer)
 * 
 * def glBindFramebuffer(target, framebuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindFramebuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":247
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 * def glBindRenderbuffer(target, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11glBindRenderbuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_11glBindRenderbuffer = {"glBindRenderbuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_11glBindRenderbuffer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_11glBindRenderbuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_renderbuffer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBindRenderbuffer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_renderbuffer,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_renderbuffer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBindRenderbuffer", 1, 2, 2, 1); __PYX_ERR(1, 247, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBindRenderbuffer") < 0)) __PYX_ERR(1, 247, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_target = values[0];
    __pyx_v_renderbuffer = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBindRenderbuffer", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 247, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindRenderbuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_10glBindRenderbuffer(__pyx_self, __pyx_v_target, __pyx_v_renderbuffer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_10glBindRenderbuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_renderbuffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glBindRenderbuffer", 0);

  /* "renpy/gl2/uguu.pyx":248
 * 
 * def glBindRenderbuffer(target, renderbuffer):
 *     uguugl.glBindRenderbuffer(target, renderbuffer)             # <<<<<<<<<<<<<<
 * 
 * def glBindTexture(target, texture):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 248, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_renderbuffer); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 248, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBindRenderbuffer(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":247
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 * def glBindRenderbuffer(target, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindRenderbuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":250
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 * def glBindTexture(target, texture):             # <<<<<<<<<<<<<<
 *     uguugl.glBindTexture(target, texture)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_13glBindTexture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_13glBindTexture = {"glBindTexture", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_13glBindTexture, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_13glBindTexture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_texture = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBindTexture (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_texture,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_texture)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBindTexture", 1, 2, 2, 1); __PYX_ERR(1, 250, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBindTexture") < 0)) __PYX_ERR(1, 250, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_target = values[0];
    __pyx_v_texture = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBindTexture", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 250, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindTexture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_12glBindTexture(__pyx_self, __pyx_v_target, __pyx_v_texture);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_12glBindTexture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_texture) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glBindTexture", 0);

  /* "renpy/gl2/uguu.pyx":251
 * 
 * def glBindTexture(target, texture):
 *     uguugl.glBindTexture(target, texture)             # <<<<<<<<<<<<<<
 * 
 * def glBlendColor(red, green, blue, alpha):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_texture); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBindTexture(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":250
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 * def glBindTexture(target, texture):             # <<<<<<<<<<<<<<
 *     uguugl.glBindTexture(target, texture)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBindTexture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":253
 *     uguugl.glBindTexture(target, texture)
 * 
 * def glBlendColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_15glBlendColor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_15glBlendColor = {"glBlendColor", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_15glBlendColor, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_15glBlendColor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_red = 0;
  PyObject *__pyx_v_green = 0;
  PyObject *__pyx_v_blue = 0;
  PyObject *__pyx_v_alpha = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBlendColor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_red,&__pyx_n_s_green,&__pyx_n_s_blue,&__pyx_n_s_alpha,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_red)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_green)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendColor", 1, 4, 4, 1); __PYX_ERR(1, 253, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_blue)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendColor", 1, 4, 4, 2); __PYX_ERR(1, 253, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendColor", 1, 4, 4, 3); __PYX_ERR(1, 253, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBlendColor") < 0)) __PYX_ERR(1, 253, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_red = values[0];
    __pyx_v_green = values[1];
    __pyx_v_blue = values[2];
    __pyx_v_alpha = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBlendColor", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 253, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendColor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_14glBlendColor(__pyx_self, __pyx_v_red, __pyx_v_green, __pyx_v_blue, __pyx_v_alpha);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_14glBlendColor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_red, PyObject *__pyx_v_green, PyObject *__pyx_v_blue, PyObject *__pyx_v_alpha) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLfloat __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  GLfloat __pyx_t_4;
  __Pyx_RefNannySetupContext("glBlendColor", 0);

  /* "renpy/gl2/uguu.pyx":254
 * 
 * def glBlendColor(red, green, blue, alpha):
 *     uguugl.glBlendColor(red, green, blue, alpha)             # <<<<<<<<<<<<<<
 * 
 * def glBlendEquation(mode):
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_red); if (unlikely((__pyx_t_1 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 254, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_green); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 254, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_blue); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 254, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_alpha); if (unlikely((__pyx_t_4 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 254, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBlendColor(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":253
 *     uguugl.glBindTexture(target, texture)
 * 
 * def glBlendColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendColor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":256
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 * def glBlendEquation(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquation(mode)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_17glBlendEquation(PyObject *__pyx_self, PyObject *__pyx_v_mode); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_17glBlendEquation = {"glBlendEquation", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_17glBlendEquation, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_17glBlendEquation(PyObject *__pyx_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBlendEquation (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_16glBlendEquation(__pyx_self, ((PyObject *)__pyx_v_mode));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_16glBlendEquation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glBlendEquation", 0);

  /* "renpy/gl2/uguu.pyx":257
 * 
 * def glBlendEquation(mode):
 *     uguugl.glBlendEquation(mode)             # <<<<<<<<<<<<<<
 * 
 * def glBlendEquationSeparate(modeRGB, modeAlpha):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_mode); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 257, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBlendEquation(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":256
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 * def glBlendEquation(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquation(mode)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendEquation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":259
 *     uguugl.glBlendEquation(mode)
 * 
 * def glBlendEquationSeparate(modeRGB, modeAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_19glBlendEquationSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_19glBlendEquationSeparate = {"glBlendEquationSeparate", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_19glBlendEquationSeparate, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_19glBlendEquationSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_modeRGB = 0;
  PyObject *__pyx_v_modeAlpha = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBlendEquationSeparate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_modeRGB,&__pyx_n_s_modeAlpha,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_modeRGB)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_modeAlpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendEquationSeparate", 1, 2, 2, 1); __PYX_ERR(1, 259, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBlendEquationSeparate") < 0)) __PYX_ERR(1, 259, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_modeRGB = values[0];
    __pyx_v_modeAlpha = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBlendEquationSeparate", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 259, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendEquationSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_18glBlendEquationSeparate(__pyx_self, __pyx_v_modeRGB, __pyx_v_modeAlpha);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_18glBlendEquationSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_modeRGB, PyObject *__pyx_v_modeAlpha) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  __Pyx_RefNannySetupContext("glBlendEquationSeparate", 0);

  /* "renpy/gl2/uguu.pyx":260
 * 
 * def glBlendEquationSeparate(modeRGB, modeAlpha):
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)             # <<<<<<<<<<<<<<
 * 
 * def glBlendFunc(sfactor, dfactor):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_modeRGB); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 260, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_modeAlpha); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 260, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBlendEquationSeparate(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":259
 *     uguugl.glBlendEquation(mode)
 * 
 * def glBlendEquationSeparate(modeRGB, modeAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendEquationSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":262
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 * def glBlendFunc(sfactor, dfactor):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_21glBlendFunc(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_21glBlendFunc = {"glBlendFunc", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_21glBlendFunc, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_21glBlendFunc(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sfactor = 0;
  PyObject *__pyx_v_dfactor = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBlendFunc (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sfactor,&__pyx_n_s_dfactor,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sfactor)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dfactor)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendFunc", 1, 2, 2, 1); __PYX_ERR(1, 262, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBlendFunc") < 0)) __PYX_ERR(1, 262, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_sfactor = values[0];
    __pyx_v_dfactor = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBlendFunc", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 262, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendFunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_20glBlendFunc(__pyx_self, __pyx_v_sfactor, __pyx_v_dfactor);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_20glBlendFunc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sfactor, PyObject *__pyx_v_dfactor) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  __Pyx_RefNannySetupContext("glBlendFunc", 0);

  /* "renpy/gl2/uguu.pyx":263
 * 
 * def glBlendFunc(sfactor, dfactor):
 *     uguugl.glBlendFunc(sfactor, dfactor)             # <<<<<<<<<<<<<<
 * 
 * def glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_sfactor); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 263, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_dfactor); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 263, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBlendFunc(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":262
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 * def glBlendFunc(sfactor, dfactor):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendFunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":265
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 * def glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_23glBlendFuncSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_23glBlendFuncSeparate = {"glBlendFuncSeparate", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_23glBlendFuncSeparate, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_23glBlendFuncSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sfactorRGB = 0;
  PyObject *__pyx_v_dfactorRGB = 0;
  PyObject *__pyx_v_sfactorAlpha = 0;
  PyObject *__pyx_v_dfactorAlpha = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBlendFuncSeparate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sfactorRGB,&__pyx_n_s_dfactorRGB,&__pyx_n_s_sfactorAlpha,&__pyx_n_s_dfactorAlpha,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sfactorRGB)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dfactorRGB)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendFuncSeparate", 1, 4, 4, 1); __PYX_ERR(1, 265, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sfactorAlpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendFuncSeparate", 1, 4, 4, 2); __PYX_ERR(1, 265, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dfactorAlpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBlendFuncSeparate", 1, 4, 4, 3); __PYX_ERR(1, 265, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBlendFuncSeparate") < 0)) __PYX_ERR(1, 265, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_sfactorRGB = values[0];
    __pyx_v_dfactorRGB = values[1];
    __pyx_v_sfactorAlpha = values[2];
    __pyx_v_dfactorAlpha = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBlendFuncSeparate", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 265, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendFuncSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_22glBlendFuncSeparate(__pyx_self, __pyx_v_sfactorRGB, __pyx_v_dfactorRGB, __pyx_v_sfactorAlpha, __pyx_v_dfactorAlpha);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_22glBlendFuncSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sfactorRGB, PyObject *__pyx_v_dfactorRGB, PyObject *__pyx_v_sfactorAlpha, PyObject *__pyx_v_dfactorAlpha) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  GLenum __pyx_t_4;
  __Pyx_RefNannySetupContext("glBlendFuncSeparate", 0);

  /* "renpy/gl2/uguu.pyx":266
 * 
 * def glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha):
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)             # <<<<<<<<<<<<<<
 * 
 * def glBufferData(target, size, data, usage):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_sfactorRGB); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 266, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_dfactorRGB); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 266, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_sfactorAlpha); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 266, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_dfactorAlpha); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 266, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBlendFuncSeparate(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":265
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 * def glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBlendFuncSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":268
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 * def glBufferData(target, size, data, usage):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_25glBufferData(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_25glBufferData = {"glBufferData", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_25glBufferData, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_25glBufferData(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_usage = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBufferData (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_size,&__pyx_n_s_data,&__pyx_n_s_usage,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBufferData", 1, 4, 4, 1); __PYX_ERR(1, 268, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBufferData", 1, 4, 4, 2); __PYX_ERR(1, 268, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_usage)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBufferData", 1, 4, 4, 3); __PYX_ERR(1, 268, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBufferData") < 0)) __PYX_ERR(1, 268, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_target = values[0];
    __pyx_v_size = values[1];
    __pyx_v_data = values[2];
    __pyx_v_usage = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBufferData", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 268, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBufferData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_24glBufferData(__pyx_self, __pyx_v_target, __pyx_v_size, __pyx_v_data, __pyx_v_usage);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_24glBufferData(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_size, PyObject *__pyx_v_data, PyObject *__pyx_v_usage) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLsizeiptr __pyx_t_3;
  GLenum __pyx_t_4;
  __Pyx_RefNannySetupContext("glBufferData", 0);

  /* "renpy/gl2/uguu.pyx":269
 * 
 * def glBufferData(target, size, data, usage):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":270
 * def glBufferData(target, size, data, usage):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)             # <<<<<<<<<<<<<<
 * 
 * def glBufferSubData(target, offset, size, data):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 270, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_ptrdiff_t(__pyx_v_size); if (unlikely((__pyx_t_3 == ((GLsizeiptr)-1)) && PyErr_Occurred())) __PYX_ERR(1, 270, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_usage); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 270, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBufferData(__pyx_t_2, __pyx_t_3, ((void const *)__pyx_v_data_ptr->ptr), __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":268
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 * def glBufferData(target, size, data, usage):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glBufferData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":272
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 * 
 * def glBufferSubData(target, offset, size, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_27glBufferSubData(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_27glBufferSubData = {"glBufferSubData", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_27glBufferSubData, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_27glBufferSubData(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_offset = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glBufferSubData (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_offset,&__pyx_n_s_size,&__pyx_n_s_data,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_offset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBufferSubData", 1, 4, 4, 1); __PYX_ERR(1, 272, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBufferSubData", 1, 4, 4, 2); __PYX_ERR(1, 272, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glBufferSubData", 1, 4, 4, 3); __PYX_ERR(1, 272, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glBufferSubData") < 0)) __PYX_ERR(1, 272, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_target = values[0];
    __pyx_v_offset = values[1];
    __pyx_v_size = values[2];
    __pyx_v_data = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glBufferSubData", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 272, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glBufferSubData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_26glBufferSubData(__pyx_self, __pyx_v_target, __pyx_v_offset, __pyx_v_size, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_26glBufferSubData(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_offset, PyObject *__pyx_v_size, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLintptr __pyx_t_3;
  GLsizeiptr __pyx_t_4;
  __Pyx_RefNannySetupContext("glBufferSubData", 0);

  /* "renpy/gl2/uguu.pyx":273
 * 
 * def glBufferSubData(target, offset, size, data):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":274
 * def glBufferSubData(target, offset, size, data):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glCheckFramebufferStatus(target):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 274, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_ptrdiff_t(__pyx_v_offset); if (unlikely((__pyx_t_3 == ((GLintptr)-1)) && PyErr_Occurred())) __PYX_ERR(1, 274, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_ptrdiff_t(__pyx_v_size); if (unlikely((__pyx_t_4 == ((GLsizeiptr)-1)) && PyErr_Occurred())) __PYX_ERR(1, 274, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glBufferSubData(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((void const *)__pyx_v_data_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":272
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 * 
 * def glBufferSubData(target, offset, size, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glBufferSubData", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":276
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 * 
 * def glCheckFramebufferStatus(target):             # <<<<<<<<<<<<<<
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_29glCheckFramebufferStatus(PyObject *__pyx_self, PyObject *__pyx_v_target); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_29glCheckFramebufferStatus = {"glCheckFramebufferStatus", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_29glCheckFramebufferStatus, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_29glCheckFramebufferStatus(PyObject *__pyx_self, PyObject *__pyx_v_target) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCheckFramebufferStatus (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_28glCheckFramebufferStatus(__pyx_self, ((PyObject *)__pyx_v_target));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_28glCheckFramebufferStatus(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glCheckFramebufferStatus", 0);

  /* "renpy/gl2/uguu.pyx":277
 * 
 * def glCheckFramebufferStatus(target):
 *     return uguugl.glCheckFramebufferStatus(target)             # <<<<<<<<<<<<<<
 * 
 * def glClear(mask):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 277, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLenum(__pyx_v_5renpy_3gl2_6uguugl_glCheckFramebufferStatus(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":276
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 * 
 * def glCheckFramebufferStatus(target):             # <<<<<<<<<<<<<<
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glCheckFramebufferStatus", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":279
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 * def glClear(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glClear(mask)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_31glClear(PyObject *__pyx_self, PyObject *__pyx_v_mask); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_31glClear = {"glClear", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_31glClear, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_31glClear(PyObject *__pyx_self, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glClear (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_30glClear(__pyx_self, ((PyObject *)__pyx_v_mask));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_30glClear(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLbitfield __pyx_t_1;
  __Pyx_RefNannySetupContext("glClear", 0);

  /* "renpy/gl2/uguu.pyx":280
 * 
 * def glClear(mask):
 *     uguugl.glClear(mask)             # <<<<<<<<<<<<<<
 * 
 * def glClearColor(red, green, blue, alpha):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLbitfield(__pyx_v_mask); if (unlikely((__pyx_t_1 == ((GLbitfield)-1)) && PyErr_Occurred())) __PYX_ERR(1, 280, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glClear(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":279
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 * def glClear(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glClear(mask)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glClear", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":282
 *     uguugl.glClear(mask)
 * 
 * def glClearColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_33glClearColor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_33glClearColor = {"glClearColor", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_33glClearColor, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_33glClearColor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_red = 0;
  PyObject *__pyx_v_green = 0;
  PyObject *__pyx_v_blue = 0;
  PyObject *__pyx_v_alpha = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glClearColor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_red,&__pyx_n_s_green,&__pyx_n_s_blue,&__pyx_n_s_alpha,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_red)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_green)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glClearColor", 1, 4, 4, 1); __PYX_ERR(1, 282, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_blue)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glClearColor", 1, 4, 4, 2); __PYX_ERR(1, 282, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glClearColor", 1, 4, 4, 3); __PYX_ERR(1, 282, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glClearColor") < 0)) __PYX_ERR(1, 282, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_red = values[0];
    __pyx_v_green = values[1];
    __pyx_v_blue = values[2];
    __pyx_v_alpha = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glClearColor", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 282, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glClearColor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_32glClearColor(__pyx_self, __pyx_v_red, __pyx_v_green, __pyx_v_blue, __pyx_v_alpha);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_32glClearColor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_red, PyObject *__pyx_v_green, PyObject *__pyx_v_blue, PyObject *__pyx_v_alpha) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLfloat __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  GLfloat __pyx_t_4;
  __Pyx_RefNannySetupContext("glClearColor", 0);

  /* "renpy/gl2/uguu.pyx":283
 * 
 * def glClearColor(red, green, blue, alpha):
 *     uguugl.glClearColor(red, green, blue, alpha)             # <<<<<<<<<<<<<<
 * 
 * def glClearStencil(s):
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_red); if (unlikely((__pyx_t_1 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 283, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_green); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 283, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_blue); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 283, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_alpha); if (unlikely((__pyx_t_4 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 283, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glClearColor(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":282
 *     uguugl.glClear(mask)
 * 
 * def glClearColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glClearColor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":285
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 * def glClearStencil(s):             # <<<<<<<<<<<<<<
 *     uguugl.glClearStencil(s)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_35glClearStencil(PyObject *__pyx_self, PyObject *__pyx_v_s); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_35glClearStencil = {"glClearStencil", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_35glClearStencil, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_35glClearStencil(PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glClearStencil (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_34glClearStencil(__pyx_self, ((PyObject *)__pyx_v_s));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_34glClearStencil(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  __Pyx_RefNannySetupContext("glClearStencil", 0);

  /* "renpy/gl2/uguu.pyx":286
 * 
 * def glClearStencil(s):
 *     uguugl.glClearStencil(s)             # <<<<<<<<<<<<<<
 * 
 * def glColorMask(red, green, blue, alpha):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_s); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 286, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glClearStencil(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":285
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 * def glClearStencil(s):             # <<<<<<<<<<<<<<
 *     uguugl.glClearStencil(s)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glClearStencil", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":288
 *     uguugl.glClearStencil(s)
 * 
 * def glColorMask(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_37glColorMask(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_37glColorMask = {"glColorMask", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_37glColorMask, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_37glColorMask(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_red = 0;
  PyObject *__pyx_v_green = 0;
  PyObject *__pyx_v_blue = 0;
  PyObject *__pyx_v_alpha = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glColorMask (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_red,&__pyx_n_s_green,&__pyx_n_s_blue,&__pyx_n_s_alpha,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_red)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_green)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glColorMask", 1, 4, 4, 1); __PYX_ERR(1, 288, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_blue)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glColorMask", 1, 4, 4, 2); __PYX_ERR(1, 288, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glColorMask", 1, 4, 4, 3); __PYX_ERR(1, 288, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glColorMask") < 0)) __PYX_ERR(1, 288, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_red = values[0];
    __pyx_v_green = values[1];
    __pyx_v_blue = values[2];
    __pyx_v_alpha = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glColorMask", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 288, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glColorMask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_36glColorMask(__pyx_self, __pyx_v_red, __pyx_v_green, __pyx_v_blue, __pyx_v_alpha);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_36glColorMask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_red, PyObject *__pyx_v_green, PyObject *__pyx_v_blue, PyObject *__pyx_v_alpha) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLboolean __pyx_t_1;
  GLboolean __pyx_t_2;
  GLboolean __pyx_t_3;
  GLboolean __pyx_t_4;
  __Pyx_RefNannySetupContext("glColorMask", 0);

  /* "renpy/gl2/uguu.pyx":289
 * 
 * def glColorMask(red, green, blue, alpha):
 *     uguugl.glColorMask(red, green, blue, alpha)             # <<<<<<<<<<<<<<
 * 
 * def glCompileShader(shader):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLboolean(__pyx_v_red); if (unlikely((__pyx_t_1 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 289, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLboolean(__pyx_v_green); if (unlikely((__pyx_t_2 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 289, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLboolean(__pyx_v_blue); if (unlikely((__pyx_t_3 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 289, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLboolean(__pyx_v_alpha); if (unlikely((__pyx_t_4 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 289, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glColorMask(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":288
 *     uguugl.glClearStencil(s)
 * 
 * def glColorMask(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glColorMask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":291
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 * def glCompileShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glCompileShader(shader)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_39glCompileShader(PyObject *__pyx_self, PyObject *__pyx_v_shader); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_39glCompileShader = {"glCompileShader", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_39glCompileShader, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_39glCompileShader(PyObject *__pyx_self, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCompileShader (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_38glCompileShader(__pyx_self, ((PyObject *)__pyx_v_shader));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_38glCompileShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glCompileShader", 0);

  /* "renpy/gl2/uguu.pyx":292
 * 
 * def glCompileShader(shader):
 *     uguugl.glCompileShader(shader)             # <<<<<<<<<<<<<<
 * 
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 292, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glCompileShader(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":291
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 * def glCompileShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glCompileShader(shader)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCompileShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":294
 *     uguugl.glCompileShader(shader)
 * 
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_41glCompressedTexImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_41glCompressedTexImage2D = {"glCompressedTexImage2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_41glCompressedTexImage2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_41glCompressedTexImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_v_internalformat = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_border = 0;
  PyObject *__pyx_v_imageSize = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCompressedTexImage2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_level,&__pyx_n_s_internalformat,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_border,&__pyx_n_s_imageSize,&__pyx_n_s_data,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 1); __PYX_ERR(1, 294, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_internalformat)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 2); __PYX_ERR(1, 294, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 3); __PYX_ERR(1, 294, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 4); __PYX_ERR(1, 294, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 5); __PYX_ERR(1, 294, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_imageSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 6); __PYX_ERR(1, 294, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, 7); __PYX_ERR(1, 294, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glCompressedTexImage2D") < 0)) __PYX_ERR(1, 294, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_target = values[0];
    __pyx_v_level = values[1];
    __pyx_v_internalformat = values[2];
    __pyx_v_width = values[3];
    __pyx_v_height = values[4];
    __pyx_v_border = values[5];
    __pyx_v_imageSize = values[6];
    __pyx_v_data = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glCompressedTexImage2D", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 294, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCompressedTexImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_40glCompressedTexImage2D(__pyx_self, __pyx_v_target, __pyx_v_level, __pyx_v_internalformat, __pyx_v_width, __pyx_v_height, __pyx_v_border, __pyx_v_imageSize, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_40glCompressedTexImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_border, PyObject *__pyx_v_imageSize, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLint __pyx_t_3;
  GLenum __pyx_t_4;
  GLsizei __pyx_t_5;
  GLsizei __pyx_t_6;
  GLint __pyx_t_7;
  GLsizei __pyx_t_8;
  __Pyx_RefNannySetupContext("glCompressedTexImage2D", 0);

  /* "renpy/gl2/uguu.pyx":295
 * 
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":296
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_internalformat); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_5 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_6 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLint(__pyx_v_border); if (unlikely((__pyx_t_7 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_GLsizei(__pyx_v_imageSize); if (unlikely((__pyx_t_8 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 296, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexImage2D(__pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, ((void const *)__pyx_v_data_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":294
 *     uguugl.glCompileShader(shader)
 * 
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glCompressedTexImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":298
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_43glCompressedTexSubImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_43glCompressedTexSubImage2D = {"glCompressedTexSubImage2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_43glCompressedTexSubImage2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_43glCompressedTexSubImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_v_xoffset = 0;
  PyObject *__pyx_v_yoffset = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_imageSize = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCompressedTexSubImage2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_level,&__pyx_n_s_xoffset,&__pyx_n_s_yoffset,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_format,&__pyx_n_s_imageSize,&__pyx_n_s_data,0};
    PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 1); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xoffset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 2); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yoffset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 3); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 4); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 5); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 6); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_imageSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 7); __PYX_ERR(1, 298, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, 8); __PYX_ERR(1, 298, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glCompressedTexSubImage2D") < 0)) __PYX_ERR(1, 298, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 9) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
    }
    __pyx_v_target = values[0];
    __pyx_v_level = values[1];
    __pyx_v_xoffset = values[2];
    __pyx_v_yoffset = values[3];
    __pyx_v_width = values[4];
    __pyx_v_height = values[5];
    __pyx_v_format = values[6];
    __pyx_v_imageSize = values[7];
    __pyx_v_data = values[8];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glCompressedTexSubImage2D", 1, 9, 9, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 298, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCompressedTexSubImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_42glCompressedTexSubImage2D(__pyx_self, __pyx_v_target, __pyx_v_level, __pyx_v_xoffset, __pyx_v_yoffset, __pyx_v_width, __pyx_v_height, __pyx_v_format, __pyx_v_imageSize, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_42glCompressedTexSubImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_xoffset, PyObject *__pyx_v_yoffset, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_format, PyObject *__pyx_v_imageSize, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLint __pyx_t_3;
  GLint __pyx_t_4;
  GLint __pyx_t_5;
  GLsizei __pyx_t_6;
  GLsizei __pyx_t_7;
  GLenum __pyx_t_8;
  GLsizei __pyx_t_9;
  __Pyx_RefNannySetupContext("glCompressedTexSubImage2D", 0);

  /* "renpy/gl2/uguu.pyx":299
 * 
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":300
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_xoffset); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLint(__pyx_v_yoffset); if (unlikely((__pyx_t_5 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_6 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_7 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_GLenum(__pyx_v_format); if (unlikely((__pyx_t_8 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyInt_As_GLsizei(__pyx_v_imageSize); if (unlikely((__pyx_t_9 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 300, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D(__pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, ((void const *)__pyx_v_data_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":298
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glCompressedTexSubImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":302
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_45glCopyTexImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_45glCopyTexImage2D = {"glCopyTexImage2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_45glCopyTexImage2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_45glCopyTexImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_v_internalformat = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_border = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCopyTexImage2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_level,&__pyx_n_s_internalformat,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_border,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 1); __PYX_ERR(1, 302, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_internalformat)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 2); __PYX_ERR(1, 302, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 3); __PYX_ERR(1, 302, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 4); __PYX_ERR(1, 302, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 5); __PYX_ERR(1, 302, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 6); __PYX_ERR(1, 302, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, 7); __PYX_ERR(1, 302, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glCopyTexImage2D") < 0)) __PYX_ERR(1, 302, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_target = values[0];
    __pyx_v_level = values[1];
    __pyx_v_internalformat = values[2];
    __pyx_v_x = values[3];
    __pyx_v_y = values[4];
    __pyx_v_width = values[5];
    __pyx_v_height = values[6];
    __pyx_v_border = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glCopyTexImage2D", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 302, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCopyTexImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_44glCopyTexImage2D(__pyx_self, __pyx_v_target, __pyx_v_level, __pyx_v_internalformat, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height, __pyx_v_border);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_44glCopyTexImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_border) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLint __pyx_t_2;
  GLenum __pyx_t_3;
  GLint __pyx_t_4;
  GLint __pyx_t_5;
  GLsizei __pyx_t_6;
  GLsizei __pyx_t_7;
  GLint __pyx_t_8;
  __Pyx_RefNannySetupContext("glCopyTexImage2D", 0);

  /* "renpy/gl2/uguu.pyx":303
 * 
 * def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border):
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)             # <<<<<<<<<<<<<<
 * 
 * def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_internalformat); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_x); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLint(__pyx_v_y); if (unlikely((__pyx_t_5 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_6 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_7 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_GLint(__pyx_v_border); if (unlikely((__pyx_t_8 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glCopyTexImage2D(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8);

  /* "renpy/gl2/uguu.pyx":302
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCopyTexImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":305
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 * def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_47glCopyTexSubImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_47glCopyTexSubImage2D = {"glCopyTexSubImage2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_47glCopyTexSubImage2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_47glCopyTexSubImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_v_xoffset = 0;
  PyObject *__pyx_v_yoffset = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCopyTexSubImage2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_level,&__pyx_n_s_xoffset,&__pyx_n_s_yoffset,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 1); __PYX_ERR(1, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xoffset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 2); __PYX_ERR(1, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yoffset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 3); __PYX_ERR(1, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 4); __PYX_ERR(1, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 5); __PYX_ERR(1, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 6); __PYX_ERR(1, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, 7); __PYX_ERR(1, 305, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glCopyTexSubImage2D") < 0)) __PYX_ERR(1, 305, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_target = values[0];
    __pyx_v_level = values[1];
    __pyx_v_xoffset = values[2];
    __pyx_v_yoffset = values[3];
    __pyx_v_x = values[4];
    __pyx_v_y = values[5];
    __pyx_v_width = values[6];
    __pyx_v_height = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glCopyTexSubImage2D", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 305, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCopyTexSubImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_46glCopyTexSubImage2D(__pyx_self, __pyx_v_target, __pyx_v_level, __pyx_v_xoffset, __pyx_v_yoffset, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_46glCopyTexSubImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_xoffset, PyObject *__pyx_v_yoffset, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLint __pyx_t_2;
  GLint __pyx_t_3;
  GLint __pyx_t_4;
  GLint __pyx_t_5;
  GLint __pyx_t_6;
  GLsizei __pyx_t_7;
  GLsizei __pyx_t_8;
  __Pyx_RefNannySetupContext("glCopyTexSubImage2D", 0);

  /* "renpy/gl2/uguu.pyx":306
 * 
 * def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height):
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)             # <<<<<<<<<<<<<<
 * 
 * def glCreateProgram():
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_xoffset); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_yoffset); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLint(__pyx_v_x); if (unlikely((__pyx_t_5 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLint(__pyx_v_y); if (unlikely((__pyx_t_6 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_7 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_8 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 306, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glCopyTexSubImage2D(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8);

  /* "renpy/gl2/uguu.pyx":305
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 * def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCopyTexSubImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":308
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 * def glCreateProgram():             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateProgram()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_49glCreateProgram(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_49glCreateProgram = {"glCreateProgram", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_49glCreateProgram, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_49glCreateProgram(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCreateProgram (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_48glCreateProgram(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_48glCreateProgram(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("glCreateProgram", 0);

  /* "renpy/gl2/uguu.pyx":309
 * 
 * def glCreateProgram():
 *     return uguugl.glCreateProgram()             # <<<<<<<<<<<<<<
 * 
 * def glCreateShader(type):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_GLuint(__pyx_v_5renpy_3gl2_6uguugl_glCreateProgram()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":308
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 * def glCreateProgram():             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateProgram()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glCreateProgram", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":311
 *     return uguugl.glCreateProgram()
 * 
 * def glCreateShader(type):             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateShader(type)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_51glCreateShader(PyObject *__pyx_self, PyObject *__pyx_v_type); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_51glCreateShader = {"glCreateShader", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_51glCreateShader, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_51glCreateShader(PyObject *__pyx_self, PyObject *__pyx_v_type) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCreateShader (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_50glCreateShader(__pyx_self, ((PyObject *)__pyx_v_type));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_50glCreateShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glCreateShader", 0);

  /* "renpy/gl2/uguu.pyx":312
 * 
 * def glCreateShader(type):
 *     return uguugl.glCreateShader(type)             # <<<<<<<<<<<<<<
 * 
 * def glCullFace(mode):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 312, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLuint(__pyx_v_5renpy_3gl2_6uguugl_glCreateShader(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":311
 *     return uguugl.glCreateProgram()
 * 
 * def glCreateShader(type):             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateShader(type)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glCreateShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":314
 *     return uguugl.glCreateShader(type)
 * 
 * def glCullFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glCullFace(mode)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_53glCullFace(PyObject *__pyx_self, PyObject *__pyx_v_mode); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_53glCullFace = {"glCullFace", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_53glCullFace, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_53glCullFace(PyObject *__pyx_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glCullFace (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_52glCullFace(__pyx_self, ((PyObject *)__pyx_v_mode));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_52glCullFace(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glCullFace", 0);

  /* "renpy/gl2/uguu.pyx":315
 * 
 * def glCullFace(mode):
 *     uguugl.glCullFace(mode)             # <<<<<<<<<<<<<<
 * 
 * def glDeleteBuffers(n, buffers):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_mode); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 315, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glCullFace(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":314
 *     return uguugl.glCreateShader(type)
 * 
 * def glCullFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glCullFace(mode)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glCullFace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":317
 *     uguugl.glCullFace(mode)
 * 
 * def glDeleteBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_55glDeleteBuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_55glDeleteBuffers = {"glDeleteBuffers", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_55glDeleteBuffers, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_55glDeleteBuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_buffers = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDeleteBuffers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_buffers,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_buffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDeleteBuffers", 1, 2, 2, 1); __PYX_ERR(1, 317, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDeleteBuffers") < 0)) __PYX_ERR(1, 317, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_buffers = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDeleteBuffers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 317, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteBuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_54glDeleteBuffers(__pyx_self, __pyx_v_n, __pyx_v_buffers);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_54glDeleteBuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_buffers) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_buffers_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glDeleteBuffers", 0);

  /* "renpy/gl2/uguu.pyx":318
 * 
 * def glDeleteBuffers(n, buffers):
 *     cdef ptr buffers_ptr = get_ptr(buffers)             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_buffers)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_buffers_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":319
 * def glDeleteBuffers(n, buffers):
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glDeleteFramebuffers(n, framebuffers):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 319, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteBuffers(__pyx_t_2, ((GLuint const *)__pyx_v_buffers_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":317
 *     uguugl.glCullFace(mode)
 * 
 * def glDeleteBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteBuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buffers_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":321
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 * 
 * def glDeleteFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_57glDeleteFramebuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_57glDeleteFramebuffers = {"glDeleteFramebuffers", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_57glDeleteFramebuffers, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_57glDeleteFramebuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_framebuffers = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDeleteFramebuffers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_framebuffers,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_framebuffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDeleteFramebuffers", 1, 2, 2, 1); __PYX_ERR(1, 321, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDeleteFramebuffers") < 0)) __PYX_ERR(1, 321, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_framebuffers = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDeleteFramebuffers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 321, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteFramebuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_56glDeleteFramebuffers(__pyx_self, __pyx_v_n, __pyx_v_framebuffers);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_56glDeleteFramebuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_framebuffers) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_framebuffers_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glDeleteFramebuffers", 0);

  /* "renpy/gl2/uguu.pyx":322
 * 
 * def glDeleteFramebuffers(n, framebuffers):
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_framebuffers)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 322, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_framebuffers_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":323
 * def glDeleteFramebuffers(n, framebuffers):
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glDeleteProgram(program):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 323, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteFramebuffers(__pyx_t_2, ((GLuint const *)__pyx_v_framebuffers_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":321
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 * 
 * def glDeleteFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteFramebuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_framebuffers_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":325
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 * 
 * def glDeleteProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteProgram(program)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_59glDeleteProgram(PyObject *__pyx_self, PyObject *__pyx_v_program); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_59glDeleteProgram = {"glDeleteProgram", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_59glDeleteProgram, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_59glDeleteProgram(PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDeleteProgram (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_58glDeleteProgram(__pyx_self, ((PyObject *)__pyx_v_program));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_58glDeleteProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glDeleteProgram", 0);

  /* "renpy/gl2/uguu.pyx":326
 * 
 * def glDeleteProgram(program):
 *     uguugl.glDeleteProgram(program)             # <<<<<<<<<<<<<<
 * 
 * def glDeleteRenderbuffers(n, renderbuffers):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 326, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteProgram(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":325
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 * 
 * def glDeleteProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteProgram(program)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteProgram", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":328
 *     uguugl.glDeleteProgram(program)
 * 
 * def glDeleteRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_61glDeleteRenderbuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_61glDeleteRenderbuffers = {"glDeleteRenderbuffers", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_61glDeleteRenderbuffers, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_61glDeleteRenderbuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_renderbuffers = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDeleteRenderbuffers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_renderbuffers,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_renderbuffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDeleteRenderbuffers", 1, 2, 2, 1); __PYX_ERR(1, 328, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDeleteRenderbuffers") < 0)) __PYX_ERR(1, 328, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_renderbuffers = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDeleteRenderbuffers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 328, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteRenderbuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_60glDeleteRenderbuffers(__pyx_self, __pyx_v_n, __pyx_v_renderbuffers);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_60glDeleteRenderbuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_renderbuffers) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_renderbuffers_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glDeleteRenderbuffers", 0);

  /* "renpy/gl2/uguu.pyx":329
 * 
 * def glDeleteRenderbuffers(n, renderbuffers):
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_renderbuffers)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_renderbuffers_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":330
 * def glDeleteRenderbuffers(n, renderbuffers):
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glDeleteShader(shader):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 330, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteRenderbuffers(__pyx_t_2, ((GLuint const *)__pyx_v_renderbuffers_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":328
 *     uguugl.glDeleteProgram(program)
 * 
 * def glDeleteRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteRenderbuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_renderbuffers_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":332
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glDeleteShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteShader(shader)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_63glDeleteShader(PyObject *__pyx_self, PyObject *__pyx_v_shader); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_63glDeleteShader = {"glDeleteShader", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_63glDeleteShader, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_63glDeleteShader(PyObject *__pyx_self, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDeleteShader (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_62glDeleteShader(__pyx_self, ((PyObject *)__pyx_v_shader));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_62glDeleteShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glDeleteShader", 0);

  /* "renpy/gl2/uguu.pyx":333
 * 
 * def glDeleteShader(shader):
 *     uguugl.glDeleteShader(shader)             # <<<<<<<<<<<<<<
 * 
 * def glDeleteTextures(n, textures):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 333, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteShader(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":332
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glDeleteShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteShader(shader)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":335
 *     uguugl.glDeleteShader(shader)
 * 
 * def glDeleteTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_65glDeleteTextures(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_65glDeleteTextures = {"glDeleteTextures", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_65glDeleteTextures, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_65glDeleteTextures(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_textures = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDeleteTextures (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_textures,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_textures)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDeleteTextures", 1, 2, 2, 1); __PYX_ERR(1, 335, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDeleteTextures") < 0)) __PYX_ERR(1, 335, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_textures = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDeleteTextures", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 335, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteTextures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_64glDeleteTextures(__pyx_self, __pyx_v_n, __pyx_v_textures);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_64glDeleteTextures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_textures) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_textures_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glDeleteTextures", 0);

  /* "renpy/gl2/uguu.pyx":336
 * 
 * def glDeleteTextures(n, textures):
 *     cdef ptr textures_ptr = get_ptr(textures)             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_textures)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_textures_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":337
 * def glDeleteTextures(n, textures):
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glDepthFunc(func):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 337, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteTextures(__pyx_t_2, ((GLuint const *)__pyx_v_textures_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":335
 *     uguugl.glDeleteShader(shader)
 * 
 * def glDeleteTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glDeleteTextures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_textures_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":339
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 * 
 * def glDepthFunc(func):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthFunc(func)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_67glDepthFunc(PyObject *__pyx_self, PyObject *__pyx_v_func); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_67glDepthFunc = {"glDepthFunc", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_67glDepthFunc, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_67glDepthFunc(PyObject *__pyx_self, PyObject *__pyx_v_func) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDepthFunc (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_66glDepthFunc(__pyx_self, ((PyObject *)__pyx_v_func));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_66glDepthFunc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_func) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glDepthFunc", 0);

  /* "renpy/gl2/uguu.pyx":340
 * 
 * def glDepthFunc(func):
 *     uguugl.glDepthFunc(func)             # <<<<<<<<<<<<<<
 * 
 * def glDepthMask(flag):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_func); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 340, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDepthFunc(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":339
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 * 
 * def glDepthFunc(func):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthFunc(func)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDepthFunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":342
 *     uguugl.glDepthFunc(func)
 * 
 * def glDepthMask(flag):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthMask(flag)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_69glDepthMask(PyObject *__pyx_self, PyObject *__pyx_v_flag); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_69glDepthMask = {"glDepthMask", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_69glDepthMask, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_69glDepthMask(PyObject *__pyx_self, PyObject *__pyx_v_flag) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDepthMask (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_68glDepthMask(__pyx_self, ((PyObject *)__pyx_v_flag));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_68glDepthMask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_flag) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLboolean __pyx_t_1;
  __Pyx_RefNannySetupContext("glDepthMask", 0);

  /* "renpy/gl2/uguu.pyx":343
 * 
 * def glDepthMask(flag):
 *     uguugl.glDepthMask(flag)             # <<<<<<<<<<<<<<
 * 
 * def glDetachShader(program, shader):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLboolean(__pyx_v_flag); if (unlikely((__pyx_t_1 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 343, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDepthMask(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":342
 *     uguugl.glDepthFunc(func)
 * 
 * def glDepthMask(flag):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthMask(flag)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDepthMask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":345
 *     uguugl.glDepthMask(flag)
 * 
 * def glDetachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDetachShader(program, shader)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_71glDetachShader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_71glDetachShader = {"glDetachShader", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_71glDetachShader, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_71glDetachShader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDetachShader (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_shader,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDetachShader", 1, 2, 2, 1); __PYX_ERR(1, 345, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDetachShader") < 0)) __PYX_ERR(1, 345, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_program = values[0];
    __pyx_v_shader = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDetachShader", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 345, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDetachShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_70glDetachShader(__pyx_self, __pyx_v_program, __pyx_v_shader);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_70glDetachShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glDetachShader", 0);

  /* "renpy/gl2/uguu.pyx":346
 * 
 * def glDetachShader(program, shader):
 *     uguugl.glDetachShader(program, shader)             # <<<<<<<<<<<<<<
 * 
 * def glDisable(cap):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 346, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 346, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDetachShader(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":345
 *     uguugl.glDepthMask(flag)
 * 
 * def glDetachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDetachShader(program, shader)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDetachShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":348
 *     uguugl.glDetachShader(program, shader)
 * 
 * def glDisable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glDisable(cap)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_73glDisable(PyObject *__pyx_self, PyObject *__pyx_v_cap); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_73glDisable = {"glDisable", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_73glDisable, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_73glDisable(PyObject *__pyx_self, PyObject *__pyx_v_cap) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDisable (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_72glDisable(__pyx_self, ((PyObject *)__pyx_v_cap));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_72glDisable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cap) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glDisable", 0);

  /* "renpy/gl2/uguu.pyx":349
 * 
 * def glDisable(cap):
 *     uguugl.glDisable(cap)             # <<<<<<<<<<<<<<
 * 
 * def glDisableVertexAttribArray(index):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_cap); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 349, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDisable(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":348
 *     uguugl.glDetachShader(program, shader)
 * 
 * def glDisable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glDisable(cap)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDisable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":351
 *     uguugl.glDisable(cap)
 * 
 * def glDisableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_75glDisableVertexAttribArray(PyObject *__pyx_self, PyObject *__pyx_v_index); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_75glDisableVertexAttribArray = {"glDisableVertexAttribArray", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_75glDisableVertexAttribArray, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_75glDisableVertexAttribArray(PyObject *__pyx_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDisableVertexAttribArray (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_74glDisableVertexAttribArray(__pyx_self, ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_74glDisableVertexAttribArray(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glDisableVertexAttribArray", 0);

  /* "renpy/gl2/uguu.pyx":352
 * 
 * def glDisableVertexAttribArray(index):
 *     uguugl.glDisableVertexAttribArray(index)             # <<<<<<<<<<<<<<
 * 
 * def glDrawArrays(mode, first, count):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 352, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDisableVertexAttribArray(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":351
 *     uguugl.glDisable(cap)
 * 
 * def glDisableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDisableVertexAttribArray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":354
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 * def glDrawArrays(mode, first, count):             # <<<<<<<<<<<<<<
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_77glDrawArrays(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_77glDrawArrays = {"glDrawArrays", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_77glDrawArrays, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_77glDrawArrays(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_first = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDrawArrays (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mode,&__pyx_n_s_first,&__pyx_n_s_count,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_first)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDrawArrays", 1, 3, 3, 1); __PYX_ERR(1, 354, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDrawArrays", 1, 3, 3, 2); __PYX_ERR(1, 354, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDrawArrays") < 0)) __PYX_ERR(1, 354, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_mode = values[0];
    __pyx_v_first = values[1];
    __pyx_v_count = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDrawArrays", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 354, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDrawArrays", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_76glDrawArrays(__pyx_self, __pyx_v_mode, __pyx_v_first, __pyx_v_count);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_76glDrawArrays(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_first, PyObject *__pyx_v_count) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glDrawArrays", 0);

  /* "renpy/gl2/uguu.pyx":355
 * 
 * def glDrawArrays(mode, first, count):
 *     uguugl.glDrawArrays(mode, first, count)             # <<<<<<<<<<<<<<
 * 
 * def glDrawElements(mode, count, type, indices):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_mode); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 355, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_first); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 355, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 355, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDrawArrays(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":354
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 * def glDrawArrays(mode, first, count):             # <<<<<<<<<<<<<<
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDrawArrays", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":357
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 * def glDrawElements(mode, count, type, indices):             # <<<<<<<<<<<<<<
 *     cdef ptr indices_ptr = get_ptr(indices)
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_79glDrawElements(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_79glDrawElements = {"glDrawElements", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_79glDrawElements, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_79glDrawElements(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_indices = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glDrawElements (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mode,&__pyx_n_s_count,&__pyx_n_s_type,&__pyx_n_s_indices,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDrawElements", 1, 4, 4, 1); __PYX_ERR(1, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDrawElements", 1, 4, 4, 2); __PYX_ERR(1, 357, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_indices)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glDrawElements", 1, 4, 4, 3); __PYX_ERR(1, 357, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glDrawElements") < 0)) __PYX_ERR(1, 357, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_mode = values[0];
    __pyx_v_count = values[1];
    __pyx_v_type = values[2];
    __pyx_v_indices = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glDrawElements", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 357, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glDrawElements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_78glDrawElements(__pyx_self, __pyx_v_mode, __pyx_v_count, __pyx_v_type, __pyx_v_indices);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_78glDrawElements(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode, PyObject *__pyx_v_count, PyObject *__pyx_v_type, PyObject *__pyx_v_indices) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_indices_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLsizei __pyx_t_3;
  GLenum __pyx_t_4;
  __Pyx_RefNannySetupContext("glDrawElements", 0);

  /* "renpy/gl2/uguu.pyx":358
 * 
 * def glDrawElements(mode, count, type, indices):
 *     cdef ptr indices_ptr = get_ptr(indices)             # <<<<<<<<<<<<<<
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_indices)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_indices_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":359
 * def glDrawElements(mode, count, type, indices):
 *     cdef ptr indices_ptr = get_ptr(indices)
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glEnable(cap):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_mode); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 359, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 359, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 359, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glDrawElements(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((void const *)__pyx_v_indices_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":357
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 * def glDrawElements(mode, count, type, indices):             # <<<<<<<<<<<<<<
 *     cdef ptr indices_ptr = get_ptr(indices)
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glDrawElements", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_indices_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":361
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 * 
 * def glEnable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glEnable(cap)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_81glEnable(PyObject *__pyx_self, PyObject *__pyx_v_cap); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_81glEnable = {"glEnable", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_81glEnable, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_81glEnable(PyObject *__pyx_self, PyObject *__pyx_v_cap) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glEnable (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_80glEnable(__pyx_self, ((PyObject *)__pyx_v_cap));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_80glEnable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cap) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glEnable", 0);

  /* "renpy/gl2/uguu.pyx":362
 * 
 * def glEnable(cap):
 *     uguugl.glEnable(cap)             # <<<<<<<<<<<<<<
 * 
 * def glEnableVertexAttribArray(index):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_cap); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 362, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glEnable(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":361
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 * 
 * def glEnable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glEnable(cap)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glEnable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":364
 *     uguugl.glEnable(cap)
 * 
 * def glEnableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_83glEnableVertexAttribArray(PyObject *__pyx_self, PyObject *__pyx_v_index); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_83glEnableVertexAttribArray = {"glEnableVertexAttribArray", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_83glEnableVertexAttribArray, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_83glEnableVertexAttribArray(PyObject *__pyx_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glEnableVertexAttribArray (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_82glEnableVertexAttribArray(__pyx_self, ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_82glEnableVertexAttribArray(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glEnableVertexAttribArray", 0);

  /* "renpy/gl2/uguu.pyx":365
 * 
 * def glEnableVertexAttribArray(index):
 *     uguugl.glEnableVertexAttribArray(index)             # <<<<<<<<<<<<<<
 * 
 * def glFinish():
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 365, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glEnableVertexAttribArray(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":364
 *     uguugl.glEnable(cap)
 * 
 * def glEnableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glEnableVertexAttribArray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":367
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 * def glFinish():             # <<<<<<<<<<<<<<
 *     uguugl.glFinish()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_85glFinish(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_85glFinish = {"glFinish", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_85glFinish, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_85glFinish(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFinish (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_84glFinish(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_84glFinish(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFinish", 0);

  /* "renpy/gl2/uguu.pyx":368
 * 
 * def glFinish():
 *     uguugl.glFinish()             # <<<<<<<<<<<<<<
 * 
 * def glFlush():
 */
  __pyx_v_5renpy_3gl2_6uguugl_glFinish();

  /* "renpy/gl2/uguu.pyx":367
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 * def glFinish():             # <<<<<<<<<<<<<<
 *     uguugl.glFinish()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":370
 *     uguugl.glFinish()
 * 
 * def glFlush():             # <<<<<<<<<<<<<<
 *     uguugl.glFlush()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_87glFlush(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_87glFlush = {"glFlush", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_87glFlush, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_87glFlush(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFlush (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_86glFlush(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_86glFlush(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFlush", 0);

  /* "renpy/gl2/uguu.pyx":371
 * 
 * def glFlush():
 *     uguugl.glFlush()             # <<<<<<<<<<<<<<
 * 
 * def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer):
 */
  __pyx_v_5renpy_3gl2_6uguugl_glFlush();

  /* "renpy/gl2/uguu.pyx":370
 *     uguugl.glFinish()
 * 
 * def glFlush():             # <<<<<<<<<<<<<<
 *     uguugl.glFlush()
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":373
 *     uguugl.glFlush()
 * 
 * def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_89glFramebufferRenderbuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_89glFramebufferRenderbuffer = {"glFramebufferRenderbuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_89glFramebufferRenderbuffer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_89glFramebufferRenderbuffer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_attachment = 0;
  PyObject *__pyx_v_renderbuffertarget = 0;
  PyObject *__pyx_v_renderbuffer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFramebufferRenderbuffer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_attachment,&__pyx_n_s_renderbuffertarget,&__pyx_n_s_renderbuffer,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_attachment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferRenderbuffer", 1, 4, 4, 1); __PYX_ERR(1, 373, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_renderbuffertarget)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferRenderbuffer", 1, 4, 4, 2); __PYX_ERR(1, 373, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_renderbuffer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferRenderbuffer", 1, 4, 4, 3); __PYX_ERR(1, 373, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glFramebufferRenderbuffer") < 0)) __PYX_ERR(1, 373, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_target = values[0];
    __pyx_v_attachment = values[1];
    __pyx_v_renderbuffertarget = values[2];
    __pyx_v_renderbuffer = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glFramebufferRenderbuffer", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 373, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glFramebufferRenderbuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_88glFramebufferRenderbuffer(__pyx_self, __pyx_v_target, __pyx_v_attachment, __pyx_v_renderbuffertarget, __pyx_v_renderbuffer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_88glFramebufferRenderbuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_attachment, PyObject *__pyx_v_renderbuffertarget, PyObject *__pyx_v_renderbuffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  GLuint __pyx_t_4;
  __Pyx_RefNannySetupContext("glFramebufferRenderbuffer", 0);

  /* "renpy/gl2/uguu.pyx":374
 * 
 * def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer):
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)             # <<<<<<<<<<<<<<
 * 
 * def glFramebufferTexture2D(target, attachment, textarget, texture, level):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 374, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_attachment); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 374, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_renderbuffertarget); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 374, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLuint(__pyx_v_renderbuffer); if (unlikely((__pyx_t_4 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 374, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":373
 *     uguugl.glFlush()
 * 
 * def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glFramebufferRenderbuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":376
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 * def glFramebufferTexture2D(target, attachment, textarget, texture, level):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_91glFramebufferTexture2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_91glFramebufferTexture2D = {"glFramebufferTexture2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_91glFramebufferTexture2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_91glFramebufferTexture2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_attachment = 0;
  PyObject *__pyx_v_textarget = 0;
  PyObject *__pyx_v_texture = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFramebufferTexture2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_attachment,&__pyx_n_s_textarget,&__pyx_n_s_texture,&__pyx_n_s_level,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_attachment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferTexture2D", 1, 5, 5, 1); __PYX_ERR(1, 376, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_textarget)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferTexture2D", 1, 5, 5, 2); __PYX_ERR(1, 376, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_texture)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferTexture2D", 1, 5, 5, 3); __PYX_ERR(1, 376, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glFramebufferTexture2D", 1, 5, 5, 4); __PYX_ERR(1, 376, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glFramebufferTexture2D") < 0)) __PYX_ERR(1, 376, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_target = values[0];
    __pyx_v_attachment = values[1];
    __pyx_v_textarget = values[2];
    __pyx_v_texture = values[3];
    __pyx_v_level = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glFramebufferTexture2D", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 376, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glFramebufferTexture2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_90glFramebufferTexture2D(__pyx_self, __pyx_v_target, __pyx_v_attachment, __pyx_v_textarget, __pyx_v_texture, __pyx_v_level);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_90glFramebufferTexture2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_attachment, PyObject *__pyx_v_textarget, PyObject *__pyx_v_texture, PyObject *__pyx_v_level) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  GLuint __pyx_t_4;
  GLint __pyx_t_5;
  __Pyx_RefNannySetupContext("glFramebufferTexture2D", 0);

  /* "renpy/gl2/uguu.pyx":377
 * 
 * def glFramebufferTexture2D(target, attachment, textarget, texture, level):
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)             # <<<<<<<<<<<<<<
 * 
 * def glFrontFace(mode):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 377, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_attachment); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 377, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_textarget); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 377, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLuint(__pyx_v_texture); if (unlikely((__pyx_t_4 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 377, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_5 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 377, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glFramebufferTexture2D(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

  /* "renpy/gl2/uguu.pyx":376
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 * def glFramebufferTexture2D(target, attachment, textarget, texture, level):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glFramebufferTexture2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":379
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 * def glFrontFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glFrontFace(mode)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_93glFrontFace(PyObject *__pyx_self, PyObject *__pyx_v_mode); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_93glFrontFace = {"glFrontFace", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_93glFrontFace, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_93glFrontFace(PyObject *__pyx_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glFrontFace (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_92glFrontFace(__pyx_self, ((PyObject *)__pyx_v_mode));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_92glFrontFace(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glFrontFace", 0);

  /* "renpy/gl2/uguu.pyx":380
 * 
 * def glFrontFace(mode):
 *     uguugl.glFrontFace(mode)             # <<<<<<<<<<<<<<
 * 
 * def glGenBuffers(n, buffers):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_mode); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 380, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glFrontFace(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":379
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 * def glFrontFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glFrontFace(mode)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glFrontFace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":382
 *     uguugl.glFrontFace(mode)
 * 
 * def glGenBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_95glGenBuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_95glGenBuffers = {"glGenBuffers", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_95glGenBuffers, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_95glGenBuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_buffers = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGenBuffers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_buffers,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_buffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGenBuffers", 1, 2, 2, 1); __PYX_ERR(1, 382, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGenBuffers") < 0)) __PYX_ERR(1, 382, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_buffers = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGenBuffers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 382, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenBuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_94glGenBuffers(__pyx_self, __pyx_v_n, __pyx_v_buffers);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_94glGenBuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_buffers) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_buffers_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glGenBuffers", 0);

  /* "renpy/gl2/uguu.pyx":383
 * 
 * def glGenBuffers(n, buffers):
 *     cdef ptr buffers_ptr = get_ptr(buffers)             # <<<<<<<<<<<<<<
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_buffers)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_buffers_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":384
 * def glGenBuffers(n, buffers):
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGenFramebuffers(n, framebuffers):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 384, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGenBuffers(__pyx_t_2, ((GLuint *)__pyx_v_buffers_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":382
 *     uguugl.glFrontFace(mode)
 * 
 * def glGenBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenBuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_buffers_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":386
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 * 
 * def glGenFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_97glGenFramebuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_97glGenFramebuffers = {"glGenFramebuffers", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_97glGenFramebuffers, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_97glGenFramebuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_framebuffers = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGenFramebuffers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_framebuffers,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_framebuffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGenFramebuffers", 1, 2, 2, 1); __PYX_ERR(1, 386, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGenFramebuffers") < 0)) __PYX_ERR(1, 386, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_framebuffers = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGenFramebuffers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 386, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenFramebuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_96glGenFramebuffers(__pyx_self, __pyx_v_n, __pyx_v_framebuffers);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_96glGenFramebuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_framebuffers) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_framebuffers_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glGenFramebuffers", 0);

  /* "renpy/gl2/uguu.pyx":387
 * 
 * def glGenFramebuffers(n, framebuffers):
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)             # <<<<<<<<<<<<<<
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_framebuffers)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_framebuffers_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":388
 * def glGenFramebuffers(n, framebuffers):
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGenRenderbuffers(n, renderbuffers):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 388, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGenFramebuffers(__pyx_t_2, ((GLuint *)__pyx_v_framebuffers_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":386
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 * 
 * def glGenFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenFramebuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_framebuffers_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":390
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 * 
 * def glGenRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_99glGenRenderbuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_99glGenRenderbuffers = {"glGenRenderbuffers", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_99glGenRenderbuffers, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_99glGenRenderbuffers(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_renderbuffers = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGenRenderbuffers (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_renderbuffers,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_renderbuffers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGenRenderbuffers", 1, 2, 2, 1); __PYX_ERR(1, 390, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGenRenderbuffers") < 0)) __PYX_ERR(1, 390, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_renderbuffers = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGenRenderbuffers", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 390, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenRenderbuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_98glGenRenderbuffers(__pyx_self, __pyx_v_n, __pyx_v_renderbuffers);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_98glGenRenderbuffers(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_renderbuffers) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_renderbuffers_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glGenRenderbuffers", 0);

  /* "renpy/gl2/uguu.pyx":391
 * 
 * def glGenRenderbuffers(n, renderbuffers):
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)             # <<<<<<<<<<<<<<
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_renderbuffers)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_renderbuffers_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":392
 * def glGenRenderbuffers(n, renderbuffers):
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGenTextures(n, textures):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 392, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGenRenderbuffers(__pyx_t_2, ((GLuint *)__pyx_v_renderbuffers_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":390
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 * 
 * def glGenRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenRenderbuffers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_renderbuffers_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":394
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glGenTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_101glGenTextures(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_101glGenTextures = {"glGenTextures", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_101glGenTextures, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_101glGenTextures(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_n = 0;
  PyObject *__pyx_v_textures = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGenTextures (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n,&__pyx_n_s_textures,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_n)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_textures)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGenTextures", 1, 2, 2, 1); __PYX_ERR(1, 394, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGenTextures") < 0)) __PYX_ERR(1, 394, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n = values[0];
    __pyx_v_textures = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGenTextures", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 394, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenTextures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_100glGenTextures(__pyx_self, __pyx_v_n, __pyx_v_textures);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_100glGenTextures(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_n, PyObject *__pyx_v_textures) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_textures_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLsizei __pyx_t_2;
  __Pyx_RefNannySetupContext("glGenTextures", 0);

  /* "renpy/gl2/uguu.pyx":395
 * 
 * def glGenTextures(n, textures):
 *     cdef ptr textures_ptr = get_ptr(textures)             # <<<<<<<<<<<<<<
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_textures)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_textures_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":396
 * def glGenTextures(n, textures):
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGenerateMipmap(target):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLsizei(__pyx_v_n); if (unlikely((__pyx_t_2 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 396, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGenTextures(__pyx_t_2, ((GLuint *)__pyx_v_textures_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":394
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glGenTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenTextures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_textures_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":398
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 * 
 * def glGenerateMipmap(target):             # <<<<<<<<<<<<<<
 *     uguugl.glGenerateMipmap(target)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_103glGenerateMipmap(PyObject *__pyx_self, PyObject *__pyx_v_target); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_103glGenerateMipmap = {"glGenerateMipmap", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_103glGenerateMipmap, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_103glGenerateMipmap(PyObject *__pyx_self, PyObject *__pyx_v_target) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGenerateMipmap (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_102glGenerateMipmap(__pyx_self, ((PyObject *)__pyx_v_target));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_102glGenerateMipmap(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  __Pyx_RefNannySetupContext("glGenerateMipmap", 0);

  /* "renpy/gl2/uguu.pyx":399
 * 
 * def glGenerateMipmap(target):
 *     uguugl.glGenerateMipmap(target)             # <<<<<<<<<<<<<<
 * 
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 399, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGenerateMipmap(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":398
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 * 
 * def glGenerateMipmap(target):             # <<<<<<<<<<<<<<
 *     uguugl.glGenerateMipmap(target)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGenerateMipmap", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":401
 *     uguugl.glGenerateMipmap(target)
 * 
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_105glGetActiveAttrib(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_105glGetActiveAttrib = {"glGetActiveAttrib", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_105glGetActiveAttrib, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_105glGetActiveAttrib(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_bufSize = 0;
  PyObject *__pyx_v_length = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetActiveAttrib (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_index,&__pyx_n_s_bufSize,&__pyx_n_s_length,&__pyx_n_s_size,&__pyx_n_s_type,&__pyx_n_s_name,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, 1); __PYX_ERR(1, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bufSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, 2); __PYX_ERR(1, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, 3); __PYX_ERR(1, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, 4); __PYX_ERR(1, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, 5); __PYX_ERR(1, 401, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, 6); __PYX_ERR(1, 401, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetActiveAttrib") < 0)) __PYX_ERR(1, 401, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
    }
    __pyx_v_program = values[0];
    __pyx_v_index = values[1];
    __pyx_v_bufSize = values[2];
    __pyx_v_length = values[3];
    __pyx_v_size = values[4];
    __pyx_v_type = values[5];
    __pyx_v_name = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetActiveAttrib", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 401, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetActiveAttrib", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_104glGetActiveAttrib(__pyx_self, __pyx_v_program, __pyx_v_index, __pyx_v_bufSize, __pyx_v_length, __pyx_v_size, __pyx_v_type, __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_104glGetActiveAttrib(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_index, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_size, PyObject *__pyx_v_type, PyObject *__pyx_v_name) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_length_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_size_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_type_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_name_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLuint __pyx_t_3;
  GLsizei __pyx_t_4;
  __Pyx_RefNannySetupContext("glGetActiveAttrib", 0);

  /* "renpy/gl2/uguu.pyx":402
 * 
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):
 *     cdef ptr length_ptr = get_ptr(length)             # <<<<<<<<<<<<<<
 *     cdef ptr size_ptr = get_ptr(size)
 *     cdef ptr type_ptr = get_ptr(type)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":403
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)             # <<<<<<<<<<<<<<
 *     cdef ptr type_ptr = get_ptr(type)
 *     cdef ptr name_ptr = get_ptr(name)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_size)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_size_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":404
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 *     cdef ptr type_ptr = get_ptr(type)             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_type)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_type_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":405
 *     cdef ptr size_ptr = get_ptr(size)
 *     cdef ptr type_ptr = get_ptr(type)
 *     cdef ptr name_ptr = get_ptr(name)             # <<<<<<<<<<<<<<
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_name)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_name_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":406
 *     cdef ptr type_ptr = get_ptr(type)
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 406, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_3 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 406, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_bufSize); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 406, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetActiveAttrib(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((GLsizei *)__pyx_v_length_ptr->ptr), ((GLint *)__pyx_v_size_ptr->ptr), ((GLenum *)__pyx_v_type_ptr->ptr), ((GLchar *)__pyx_v_name_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":401
 *     uguugl.glGenerateMipmap(target)
 * 
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetActiveAttrib", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_length_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_size_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_type_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_name_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":408
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_107glGetActiveUniform(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_107glGetActiveUniform = {"glGetActiveUniform", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_107glGetActiveUniform, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_107glGetActiveUniform(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_bufSize = 0;
  PyObject *__pyx_v_length = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetActiveUniform (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_index,&__pyx_n_s_bufSize,&__pyx_n_s_length,&__pyx_n_s_size,&__pyx_n_s_type,&__pyx_n_s_name,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, 1); __PYX_ERR(1, 408, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bufSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, 2); __PYX_ERR(1, 408, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, 3); __PYX_ERR(1, 408, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, 4); __PYX_ERR(1, 408, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, 5); __PYX_ERR(1, 408, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, 6); __PYX_ERR(1, 408, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetActiveUniform") < 0)) __PYX_ERR(1, 408, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
    }
    __pyx_v_program = values[0];
    __pyx_v_index = values[1];
    __pyx_v_bufSize = values[2];
    __pyx_v_length = values[3];
    __pyx_v_size = values[4];
    __pyx_v_type = values[5];
    __pyx_v_name = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetActiveUniform", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 408, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetActiveUniform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_106glGetActiveUniform(__pyx_self, __pyx_v_program, __pyx_v_index, __pyx_v_bufSize, __pyx_v_length, __pyx_v_size, __pyx_v_type, __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_106glGetActiveUniform(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_index, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_size, PyObject *__pyx_v_type, PyObject *__pyx_v_name) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_length_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_size_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_type_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_name_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLuint __pyx_t_3;
  GLsizei __pyx_t_4;
  __Pyx_RefNannySetupContext("glGetActiveUniform", 0);

  /* "renpy/gl2/uguu.pyx":409
 * 
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):
 *     cdef ptr length_ptr = get_ptr(length)             # <<<<<<<<<<<<<<
 *     cdef ptr size_ptr = get_ptr(size)
 *     cdef ptr type_ptr = get_ptr(type)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":410
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)             # <<<<<<<<<<<<<<
 *     cdef ptr type_ptr = get_ptr(type)
 *     cdef ptr name_ptr = get_ptr(name)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_size)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_size_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":411
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 *     cdef ptr type_ptr = get_ptr(type)             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_type)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 411, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_type_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":412
 *     cdef ptr size_ptr = get_ptr(size)
 *     cdef ptr type_ptr = get_ptr(type)
 *     cdef ptr name_ptr = get_ptr(name)             # <<<<<<<<<<<<<<
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_name)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 412, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_name_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":413
 *     cdef ptr type_ptr = get_ptr(type)
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetAttachedShaders(program, maxCount, count, shaders):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 413, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_3 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 413, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_bufSize); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 413, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetActiveUniform(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((GLsizei *)__pyx_v_length_ptr->ptr), ((GLint *)__pyx_v_size_ptr->ptr), ((GLenum *)__pyx_v_type_ptr->ptr), ((GLchar *)__pyx_v_name_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":408
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetActiveUniform", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_length_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_size_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_type_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_name_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":415
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetAttachedShaders(program, maxCount, count, shaders):             # <<<<<<<<<<<<<<
 *     cdef ptr count_ptr = get_ptr(count)
 *     cdef ptr shaders_ptr = get_ptr(shaders)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_109glGetAttachedShaders(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_109glGetAttachedShaders = {"glGetAttachedShaders", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_109glGetAttachedShaders, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_109glGetAttachedShaders(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_maxCount = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_shaders = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetAttachedShaders (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_maxCount,&__pyx_n_s_count,&__pyx_n_s_shaders,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_maxCount)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetAttachedShaders", 1, 4, 4, 1); __PYX_ERR(1, 415, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetAttachedShaders", 1, 4, 4, 2); __PYX_ERR(1, 415, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shaders)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetAttachedShaders", 1, 4, 4, 3); __PYX_ERR(1, 415, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetAttachedShaders") < 0)) __PYX_ERR(1, 415, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_program = values[0];
    __pyx_v_maxCount = values[1];
    __pyx_v_count = values[2];
    __pyx_v_shaders = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetAttachedShaders", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 415, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetAttachedShaders", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_108glGetAttachedShaders(__pyx_self, __pyx_v_program, __pyx_v_maxCount, __pyx_v_count, __pyx_v_shaders);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_108glGetAttachedShaders(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_maxCount, PyObject *__pyx_v_count, PyObject *__pyx_v_shaders) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_count_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_shaders_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetAttachedShaders", 0);

  /* "renpy/gl2/uguu.pyx":416
 * 
 * def glGetAttachedShaders(program, maxCount, count, shaders):
 *     cdef ptr count_ptr = get_ptr(count)             # <<<<<<<<<<<<<<
 *     cdef ptr shaders_ptr = get_ptr(shaders)
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_count)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_count_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":417
 * def glGetAttachedShaders(program, maxCount, count, shaders):
 *     cdef ptr count_ptr = get_ptr(count)
 *     cdef ptr shaders_ptr = get_ptr(shaders)             # <<<<<<<<<<<<<<
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_shaders)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_shaders_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":418
 *     cdef ptr count_ptr = get_ptr(count)
 *     cdef ptr shaders_ptr = get_ptr(shaders)
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetAttribLocation(program, name):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 418, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_maxCount); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 418, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetAttachedShaders(__pyx_t_2, __pyx_t_3, ((GLsizei *)__pyx_v_count_ptr->ptr), ((GLuint *)__pyx_v_shaders_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":415
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetAttachedShaders(program, maxCount, count, shaders):             # <<<<<<<<<<<<<<
 *     cdef ptr count_ptr = get_ptr(count)
 *     cdef ptr shaders_ptr = get_ptr(shaders)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetAttachedShaders", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_count_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_shaders_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":420
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)
 * 
 * def glGetAttribLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_111glGetAttribLocation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_111glGetAttribLocation = {"glGetAttribLocation", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_111glGetAttribLocation, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_111glGetAttribLocation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetAttribLocation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetAttribLocation", 1, 2, 2, 1); __PYX_ERR(1, 420, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetAttribLocation") < 0)) __PYX_ERR(1, 420, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_program = values[0];
    __pyx_v_name = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetAttribLocation", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 420, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetAttribLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_110glGetAttribLocation(__pyx_self, __pyx_v_program, __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_110glGetAttribLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_name) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_name_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glGetAttribLocation", 0);

  /* "renpy/gl2/uguu.pyx":421
 * 
 * def glGetAttribLocation(program, name):
 *     cdef ptr name_ptr = get_ptr(name)             # <<<<<<<<<<<<<<
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_name)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_name_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":422
 * def glGetAttribLocation(program, name):
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetBooleanv(pname, data):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 422, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyInt_From_GLint(__pyx_v_5renpy_3gl2_6uguugl_glGetAttribLocation(__pyx_t_2, ((GLchar const *)__pyx_v_name_ptr->ptr))); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":420
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)
 * 
 * def glGetAttribLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetAttribLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_name_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":424
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetBooleanv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_113glGetBooleanv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_113glGetBooleanv = {"glGetBooleanv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_113glGetBooleanv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_113glGetBooleanv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetBooleanv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pname,&__pyx_n_s_data,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetBooleanv", 1, 2, 2, 1); __PYX_ERR(1, 424, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetBooleanv") < 0)) __PYX_ERR(1, 424, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_pname = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetBooleanv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 424, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetBooleanv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_112glGetBooleanv(__pyx_self, __pyx_v_pname, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_112glGetBooleanv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  __Pyx_RefNannySetupContext("glGetBooleanv", 0);

  /* "renpy/gl2/uguu.pyx":425
 * 
 * def glGetBooleanv(pname, data):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":426
 * def glGetBooleanv(pname, data):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetBufferParameteriv(target, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 426, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetBooleanv(__pyx_t_2, ((GLboolean *)__pyx_v_data_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":424
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetBooleanv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetBooleanv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":428
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 * 
 * def glGetBufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_115glGetBufferParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_115glGetBufferParameteriv = {"glGetBufferParameteriv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_115glGetBufferParameteriv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_115glGetBufferParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetBufferParameteriv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetBufferParameteriv", 1, 3, 3, 1); __PYX_ERR(1, 428, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetBufferParameteriv", 1, 3, 3, 2); __PYX_ERR(1, 428, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetBufferParameteriv") < 0)) __PYX_ERR(1, 428, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetBufferParameteriv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 428, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetBufferParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_114glGetBufferParameteriv(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_114glGetBufferParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetBufferParameteriv", 0);

  /* "renpy/gl2/uguu.pyx":429
 * 
 * def glGetBufferParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":430
 * def glGetBufferParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetError():
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 430, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 430, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetBufferParameteriv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":428
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 * 
 * def glGetBufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetBufferParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":432
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetError():             # <<<<<<<<<<<<<<
 *     return uguugl.glGetError()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_117glGetError(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_117glGetError = {"glGetError", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_117glGetError, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_117glGetError(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetError (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_116glGetError(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_116glGetError(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("glGetError", 0);

  /* "renpy/gl2/uguu.pyx":433
 * 
 * def glGetError():
 *     return uguugl.glGetError()             # <<<<<<<<<<<<<<
 * 
 * def glGetFloatv(pname, data):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_GLenum(__pyx_v_5renpy_3gl2_6uguugl_glGetError()); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":432
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetError():             # <<<<<<<<<<<<<<
 *     return uguugl.glGetError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetError", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":435
 *     return uguugl.glGetError()
 * 
 * def glGetFloatv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_119glGetFloatv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_119glGetFloatv = {"glGetFloatv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_119glGetFloatv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_119glGetFloatv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetFloatv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pname,&__pyx_n_s_data,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetFloatv", 1, 2, 2, 1); __PYX_ERR(1, 435, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetFloatv") < 0)) __PYX_ERR(1, 435, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_pname = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetFloatv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 435, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetFloatv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_118glGetFloatv(__pyx_self, __pyx_v_pname, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_118glGetFloatv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  __Pyx_RefNannySetupContext("glGetFloatv", 0);

  /* "renpy/gl2/uguu.pyx":436
 * 
 * def glGetFloatv(pname, data):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":437
 * def glGetFloatv(pname, data):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 437, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetFloatv(__pyx_t_2, ((GLfloat *)__pyx_v_data_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":435
 *     return uguugl.glGetError()
 * 
 * def glGetFloatv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetFloatv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":439
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 * 
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_121glGetFramebufferAttachmentParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_121glGetFramebufferAttachmentParameteriv = {"glGetFramebufferAttachmentParameteriv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_121glGetFramebufferAttachmentParameteriv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_121glGetFramebufferAttachmentParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_attachment = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetFramebufferAttachmentParameteriv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_attachment,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_attachment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetFramebufferAttachmentParameteriv", 1, 4, 4, 1); __PYX_ERR(1, 439, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetFramebufferAttachmentParameteriv", 1, 4, 4, 2); __PYX_ERR(1, 439, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetFramebufferAttachmentParameteriv", 1, 4, 4, 3); __PYX_ERR(1, 439, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetFramebufferAttachmentParameteriv") < 0)) __PYX_ERR(1, 439, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_target = values[0];
    __pyx_v_attachment = values[1];
    __pyx_v_pname = values[2];
    __pyx_v_params = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetFramebufferAttachmentParameteriv", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 439, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetFramebufferAttachmentParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_120glGetFramebufferAttachmentParameteriv(__pyx_self, __pyx_v_target, __pyx_v_attachment, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_120glGetFramebufferAttachmentParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_attachment, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  GLenum __pyx_t_4;
  __Pyx_RefNannySetupContext("glGetFramebufferAttachmentParameteriv", 0);

  /* "renpy/gl2/uguu.pyx":440
 * 
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":441
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetIntegerv(pname, data):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 441, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_attachment); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 441, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 441, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":439
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 * 
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetFramebufferAttachmentParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":443
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetIntegerv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_123glGetIntegerv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_123glGetIntegerv = {"glGetIntegerv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_123glGetIntegerv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_123glGetIntegerv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetIntegerv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pname,&__pyx_n_s_data,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetIntegerv", 1, 2, 2, 1); __PYX_ERR(1, 443, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetIntegerv") < 0)) __PYX_ERR(1, 443, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_pname = values[0];
    __pyx_v_data = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetIntegerv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 443, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetIntegerv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_122glGetIntegerv(__pyx_self, __pyx_v_pname, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_122glGetIntegerv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_data_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  __Pyx_RefNannySetupContext("glGetIntegerv", 0);

  /* "renpy/gl2/uguu.pyx":444
 * 
 * def glGetIntegerv(pname, data):
 *     cdef ptr data_ptr = get_ptr(data)             # <<<<<<<<<<<<<<
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_data)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_data_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":445
 * def glGetIntegerv(pname, data):
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 445, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetIntegerv(__pyx_t_2, ((GLint *)__pyx_v_data_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":443
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetIntegerv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetIntegerv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_data_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":447
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 * 
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_125glGetProgramInfoLog(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_125glGetProgramInfoLog = {"glGetProgramInfoLog", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_125glGetProgramInfoLog, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_125glGetProgramInfoLog(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_bufSize = 0;
  PyObject *__pyx_v_length = 0;
  PyObject *__pyx_v_infoLog = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetProgramInfoLog (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_bufSize,&__pyx_n_s_length,&__pyx_n_s_infoLog,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bufSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetProgramInfoLog", 1, 4, 4, 1); __PYX_ERR(1, 447, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetProgramInfoLog", 1, 4, 4, 2); __PYX_ERR(1, 447, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_infoLog)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetProgramInfoLog", 1, 4, 4, 3); __PYX_ERR(1, 447, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetProgramInfoLog") < 0)) __PYX_ERR(1, 447, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_program = values[0];
    __pyx_v_bufSize = values[1];
    __pyx_v_length = values[2];
    __pyx_v_infoLog = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetProgramInfoLog", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 447, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetProgramInfoLog", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_124glGetProgramInfoLog(__pyx_self, __pyx_v_program, __pyx_v_bufSize, __pyx_v_length, __pyx_v_infoLog);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_124glGetProgramInfoLog(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_infoLog) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_length_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_infoLog_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetProgramInfoLog", 0);

  /* "renpy/gl2/uguu.pyx":448
 * 
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):
 *     cdef ptr length_ptr = get_ptr(length)             # <<<<<<<<<<<<<<
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":449
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)             # <<<<<<<<<<<<<<
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_infoLog)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_infoLog_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":450
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetProgramiv(program, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 450, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_bufSize); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 450, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetProgramInfoLog(__pyx_t_2, __pyx_t_3, ((GLsizei *)__pyx_v_length_ptr->ptr), ((GLchar *)__pyx_v_infoLog_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":447
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 * 
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetProgramInfoLog", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_length_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_infoLog_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":452
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetProgramiv(program, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_127glGetProgramiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_127glGetProgramiv = {"glGetProgramiv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_127glGetProgramiv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_127glGetProgramiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetProgramiv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetProgramiv", 1, 3, 3, 1); __PYX_ERR(1, 452, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetProgramiv", 1, 3, 3, 2); __PYX_ERR(1, 452, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetProgramiv") < 0)) __PYX_ERR(1, 452, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_program = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetProgramiv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 452, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetProgramiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_126glGetProgramiv(__pyx_self, __pyx_v_program, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_126glGetProgramiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetProgramiv", 0);

  /* "renpy/gl2/uguu.pyx":453
 * 
 * def glGetProgramiv(program, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":454
 * def glGetProgramiv(program, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetRenderbufferParameteriv(target, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 454, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 454, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetProgramiv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":452
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetProgramiv(program, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetProgramiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":456
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetRenderbufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_129glGetRenderbufferParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_129glGetRenderbufferParameteriv = {"glGetRenderbufferParameteriv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_129glGetRenderbufferParameteriv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_129glGetRenderbufferParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetRenderbufferParameteriv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetRenderbufferParameteriv", 1, 3, 3, 1); __PYX_ERR(1, 456, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetRenderbufferParameteriv", 1, 3, 3, 2); __PYX_ERR(1, 456, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetRenderbufferParameteriv") < 0)) __PYX_ERR(1, 456, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetRenderbufferParameteriv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 456, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetRenderbufferParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_128glGetRenderbufferParameteriv(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_128glGetRenderbufferParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetRenderbufferParameteriv", 0);

  /* "renpy/gl2/uguu.pyx":457
 * 
 * def glGetRenderbufferParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":458
 * def glGetRenderbufferParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 458, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 458, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":456
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetRenderbufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetRenderbufferParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":460
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_131glGetShaderInfoLog(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_131glGetShaderInfoLog = {"glGetShaderInfoLog", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_131glGetShaderInfoLog, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_131glGetShaderInfoLog(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_v_bufSize = 0;
  PyObject *__pyx_v_length = 0;
  PyObject *__pyx_v_infoLog = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetShaderInfoLog (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shader,&__pyx_n_s_bufSize,&__pyx_n_s_length,&__pyx_n_s_infoLog,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bufSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderInfoLog", 1, 4, 4, 1); __PYX_ERR(1, 460, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderInfoLog", 1, 4, 4, 2); __PYX_ERR(1, 460, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_infoLog)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderInfoLog", 1, 4, 4, 3); __PYX_ERR(1, 460, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetShaderInfoLog") < 0)) __PYX_ERR(1, 460, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_shader = values[0];
    __pyx_v_bufSize = values[1];
    __pyx_v_length = values[2];
    __pyx_v_infoLog = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetShaderInfoLog", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 460, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetShaderInfoLog", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_130glGetShaderInfoLog(__pyx_self, __pyx_v_shader, __pyx_v_bufSize, __pyx_v_length, __pyx_v_infoLog);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_130glGetShaderInfoLog(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_infoLog) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_length_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_infoLog_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetShaderInfoLog", 0);

  /* "renpy/gl2/uguu.pyx":461
 * 
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):
 *     cdef ptr length_ptr = get_ptr(length)             # <<<<<<<<<<<<<<
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":462
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)             # <<<<<<<<<<<<<<
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_infoLog)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_infoLog_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":463
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetShaderSource(shader, bufSize, length, source):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 463, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_bufSize); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 463, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetShaderInfoLog(__pyx_t_2, __pyx_t_3, ((GLsizei *)__pyx_v_length_ptr->ptr), ((GLchar *)__pyx_v_infoLog_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":460
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetShaderInfoLog", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_length_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_infoLog_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":465
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetShaderSource(shader, bufSize, length, source):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr source_ptr = get_ptr(source)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_133glGetShaderSource(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_133glGetShaderSource = {"glGetShaderSource", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_133glGetShaderSource, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_133glGetShaderSource(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_v_bufSize = 0;
  PyObject *__pyx_v_length = 0;
  PyObject *__pyx_v_source = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetShaderSource (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shader,&__pyx_n_s_bufSize,&__pyx_n_s_length,&__pyx_n_s_source,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bufSize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderSource", 1, 4, 4, 1); __PYX_ERR(1, 465, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderSource", 1, 4, 4, 2); __PYX_ERR(1, 465, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_source)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderSource", 1, 4, 4, 3); __PYX_ERR(1, 465, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetShaderSource") < 0)) __PYX_ERR(1, 465, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_shader = values[0];
    __pyx_v_bufSize = values[1];
    __pyx_v_length = values[2];
    __pyx_v_source = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetShaderSource", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 465, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetShaderSource", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_132glGetShaderSource(__pyx_self, __pyx_v_shader, __pyx_v_bufSize, __pyx_v_length, __pyx_v_source);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_132glGetShaderSource(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_bufSize, PyObject *__pyx_v_length, PyObject *__pyx_v_source) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_length_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_source_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetShaderSource", 0);

  /* "renpy/gl2/uguu.pyx":466
 * 
 * def glGetShaderSource(shader, bufSize, length, source):
 *     cdef ptr length_ptr = get_ptr(length)             # <<<<<<<<<<<<<<
 *     cdef ptr source_ptr = get_ptr(source)
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":467
 * def glGetShaderSource(shader, bufSize, length, source):
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr source_ptr = get_ptr(source)             # <<<<<<<<<<<<<<
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_source)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_source_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":468
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr source_ptr = get_ptr(source)
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetShaderiv(shader, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 468, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_bufSize); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 468, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetShaderSource(__pyx_t_2, __pyx_t_3, ((GLsizei *)__pyx_v_length_ptr->ptr), ((GLchar *)__pyx_v_source_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":465
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetShaderSource(shader, bufSize, length, source):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr source_ptr = get_ptr(source)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetShaderSource", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_length_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_source_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":470
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)
 * 
 * def glGetShaderiv(shader, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_135glGetShaderiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_135glGetShaderiv = {"glGetShaderiv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_135glGetShaderiv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_135glGetShaderiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetShaderiv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shader,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderiv", 1, 3, 3, 1); __PYX_ERR(1, 470, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetShaderiv", 1, 3, 3, 2); __PYX_ERR(1, 470, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetShaderiv") < 0)) __PYX_ERR(1, 470, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_shader = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetShaderiv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 470, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetShaderiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_134glGetShaderiv(__pyx_self, __pyx_v_shader, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_134glGetShaderiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetShaderiv", 0);

  /* "renpy/gl2/uguu.pyx":471
 * 
 * def glGetShaderiv(shader, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":472
 * def glGetShaderiv(shader, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetString(name):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 472, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 472, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetShaderiv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":470
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)
 * 
 * def glGetShaderiv(shader, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetShaderiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":474
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetString(name):             # <<<<<<<<<<<<<<
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_137glGetString(PyObject *__pyx_self, PyObject *__pyx_v_name); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_137glGetString = {"glGetString", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_137glGetString, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_137glGetString(PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetString (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_136glGetString(__pyx_self, ((PyObject *)__pyx_v_name));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_136glGetString(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glGetString", 0);

  /* "renpy/gl2/uguu.pyx":475
 * 
 * def glGetString(name):
 *     return proxy_return_string(uguugl.glGetString(name))             # <<<<<<<<<<<<<<
 * 
 * def glGetTexParameterfv(target, pname, params):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_name); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 475, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_5renpy_3gl2_4uguu_proxy_return_string(__pyx_v_5renpy_3gl2_6uguugl_glGetString(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":474
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetString(name):             # <<<<<<<<<<<<<<
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetString", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":477
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 * def glGetTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_139glGetTexParameterfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_139glGetTexParameterfv = {"glGetTexParameterfv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_139glGetTexParameterfv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_139glGetTexParameterfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetTexParameterfv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetTexParameterfv", 1, 3, 3, 1); __PYX_ERR(1, 477, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetTexParameterfv", 1, 3, 3, 2); __PYX_ERR(1, 477, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetTexParameterfv") < 0)) __PYX_ERR(1, 477, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetTexParameterfv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 477, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetTexParameterfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_138glGetTexParameterfv(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_138glGetTexParameterfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetTexParameterfv", 0);

  /* "renpy/gl2/uguu.pyx":478
 * 
 * def glGetTexParameterfv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":479
 * def glGetTexParameterfv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetTexParameteriv(target, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 479, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 479, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameterfv(__pyx_t_2, __pyx_t_3, ((GLfloat *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":477
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 * def glGetTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetTexParameterfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":481
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_141glGetTexParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_141glGetTexParameteriv = {"glGetTexParameteriv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_141glGetTexParameteriv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_141glGetTexParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetTexParameteriv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetTexParameteriv", 1, 3, 3, 1); __PYX_ERR(1, 481, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetTexParameteriv", 1, 3, 3, 2); __PYX_ERR(1, 481, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetTexParameteriv") < 0)) __PYX_ERR(1, 481, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetTexParameteriv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 481, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetTexParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_140glGetTexParameteriv(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_140glGetTexParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetTexParameteriv", 0);

  /* "renpy/gl2/uguu.pyx":482
 * 
 * def glGetTexParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":483
 * def glGetTexParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetUniformLocation(program, name):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 483, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 483, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameteriv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":481
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetTexParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":485
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetUniformLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_143glGetUniformLocation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_143glGetUniformLocation = {"glGetUniformLocation", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_143glGetUniformLocation, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_143glGetUniformLocation(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetUniformLocation (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_name,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetUniformLocation", 1, 2, 2, 1); __PYX_ERR(1, 485, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetUniformLocation") < 0)) __PYX_ERR(1, 485, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_program = values[0];
    __pyx_v_name = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetUniformLocation", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 485, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetUniformLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_142glGetUniformLocation(__pyx_self, __pyx_v_program, __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_142glGetUniformLocation(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_name) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_name_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glGetUniformLocation", 0);

  /* "renpy/gl2/uguu.pyx":486
 * 
 * def glGetUniformLocation(program, name):
 *     cdef ptr name_ptr = get_ptr(name)             # <<<<<<<<<<<<<<
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_name)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_name_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":487
 * def glGetUniformLocation(program, name):
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetUniformfv(program, location, params):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 487, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyInt_From_GLint(__pyx_v_5renpy_3gl2_6uguugl_glGetUniformLocation(__pyx_t_2, ((GLchar const *)__pyx_v_name_ptr->ptr))); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":485
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetUniformLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetUniformLocation", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_name_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":489
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetUniformfv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_145glGetUniformfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_145glGetUniformfv = {"glGetUniformfv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_145glGetUniformfv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_145glGetUniformfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetUniformfv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_location,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetUniformfv", 1, 3, 3, 1); __PYX_ERR(1, 489, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetUniformfv", 1, 3, 3, 2); __PYX_ERR(1, 489, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetUniformfv") < 0)) __PYX_ERR(1, 489, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_program = values[0];
    __pyx_v_location = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetUniformfv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 489, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetUniformfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_144glGetUniformfv(__pyx_self, __pyx_v_program, __pyx_v_location, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_144glGetUniformfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_location, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLint __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetUniformfv", 0);

  /* "renpy/gl2/uguu.pyx":490
 * 
 * def glGetUniformfv(program, location, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":491
 * def glGetUniformfv(program, location, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetUniformiv(program, location, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 491, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 491, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetUniformfv(__pyx_t_2, __pyx_t_3, ((GLfloat *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":489
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetUniformfv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetUniformfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":493
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetUniformiv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_147glGetUniformiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_147glGetUniformiv = {"glGetUniformiv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_147glGetUniformiv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_147glGetUniformiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_program = 0;
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetUniformiv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_program,&__pyx_n_s_location,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_program)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetUniformiv", 1, 3, 3, 1); __PYX_ERR(1, 493, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetUniformiv", 1, 3, 3, 2); __PYX_ERR(1, 493, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetUniformiv") < 0)) __PYX_ERR(1, 493, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_program = values[0];
    __pyx_v_location = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetUniformiv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 493, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetUniformiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_146glGetUniformiv(__pyx_self, __pyx_v_program, __pyx_v_location, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_146glGetUniformiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program, PyObject *__pyx_v_location, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLint __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetUniformiv", 0);

  /* "renpy/gl2/uguu.pyx":494
 * 
 * def glGetUniformiv(program, location, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":495
 * def glGetUniformiv(program, location, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetVertexAttribPointerv(index, pname, pointer):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 495, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 495, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetUniformiv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":493
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetUniformiv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetUniformiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":497
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 * 
 * def glGetVertexAttribPointerv(index, pname, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_149glGetVertexAttribPointerv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_149glGetVertexAttribPointerv = {"glGetVertexAttribPointerv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_149glGetVertexAttribPointerv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_149glGetVertexAttribPointerv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_pointer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetVertexAttribPointerv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_pname,&__pyx_n_s_pointer,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetVertexAttribPointerv", 1, 3, 3, 1); __PYX_ERR(1, 497, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pointer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetVertexAttribPointerv", 1, 3, 3, 2); __PYX_ERR(1, 497, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetVertexAttribPointerv") < 0)) __PYX_ERR(1, 497, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_index = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_pointer = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetVertexAttribPointerv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 497, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetVertexAttribPointerv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_148glGetVertexAttribPointerv(__pyx_self, __pyx_v_index, __pyx_v_pname, __pyx_v_pointer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_148glGetVertexAttribPointerv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_pname, PyObject *__pyx_v_pointer) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_pointer_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetVertexAttribPointerv", 0);

  /* "renpy/gl2/uguu.pyx":498
 * 
 * def glGetVertexAttribPointerv(index, pname, pointer):
 *     cdef ptr pointer_ptr = get_ptr(pointer)             # <<<<<<<<<<<<<<
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_pointer)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pointer_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":499
 * def glGetVertexAttribPointerv(index, pname, pointer):
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetVertexAttribfv(index, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 499, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 499, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv(__pyx_t_2, __pyx_t_3, ((void **)__pyx_v_pointer_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":497
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 * 
 * def glGetVertexAttribPointerv(index, pname, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetVertexAttribPointerv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pointer_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":501
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 * 
 * def glGetVertexAttribfv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_151glGetVertexAttribfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_151glGetVertexAttribfv = {"glGetVertexAttribfv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_151glGetVertexAttribfv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_151glGetVertexAttribfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetVertexAttribfv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetVertexAttribfv", 1, 3, 3, 1); __PYX_ERR(1, 501, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetVertexAttribfv", 1, 3, 3, 2); __PYX_ERR(1, 501, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetVertexAttribfv") < 0)) __PYX_ERR(1, 501, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_index = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetVertexAttribfv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 501, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetVertexAttribfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_150glGetVertexAttribfv(__pyx_self, __pyx_v_index, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_150glGetVertexAttribfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetVertexAttribfv", 0);

  /* "renpy/gl2/uguu.pyx":502
 * 
 * def glGetVertexAttribfv(index, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":503
 * def glGetVertexAttribfv(index, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glGetVertexAttribiv(index, pname, params):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 503, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 503, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribfv(__pyx_t_2, __pyx_t_3, ((GLfloat *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":501
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 * 
 * def glGetVertexAttribfv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetVertexAttribfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":505
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetVertexAttribiv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_153glGetVertexAttribiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_153glGetVertexAttribiv = {"glGetVertexAttribiv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_153glGetVertexAttribiv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_153glGetVertexAttribiv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glGetVertexAttribiv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetVertexAttribiv", 1, 3, 3, 1); __PYX_ERR(1, 505, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glGetVertexAttribiv", 1, 3, 3, 2); __PYX_ERR(1, 505, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glGetVertexAttribiv") < 0)) __PYX_ERR(1, 505, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_index = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glGetVertexAttribiv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 505, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetVertexAttribiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_152glGetVertexAttribiv(__pyx_self, __pyx_v_index, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_152glGetVertexAttribiv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glGetVertexAttribiv", 0);

  /* "renpy/gl2/uguu.pyx":506
 * 
 * def glGetVertexAttribiv(index, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":507
 * def glGetVertexAttribiv(index, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glHint(target, mode):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 507, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 507, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribiv(__pyx_t_2, __pyx_t_3, ((GLint *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":505
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetVertexAttribiv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glGetVertexAttribiv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":509
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 * 
 * def glHint(target, mode):             # <<<<<<<<<<<<<<
 *     uguugl.glHint(target, mode)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_155glHint(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_155glHint = {"glHint", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_155glHint, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_155glHint(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glHint (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_mode,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glHint", 1, 2, 2, 1); __PYX_ERR(1, 509, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glHint") < 0)) __PYX_ERR(1, 509, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_target = values[0];
    __pyx_v_mode = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glHint", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 509, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glHint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_154glHint(__pyx_self, __pyx_v_target, __pyx_v_mode);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_154glHint(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_mode) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  __Pyx_RefNannySetupContext("glHint", 0);

  /* "renpy/gl2/uguu.pyx":510
 * 
 * def glHint(target, mode):
 *     uguugl.glHint(target, mode)             # <<<<<<<<<<<<<<
 * 
 * def glIsBuffer(buffer):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 510, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_mode); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 510, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glHint(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":509
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 * 
 * def glHint(target, mode):             # <<<<<<<<<<<<<<
 *     uguugl.glHint(target, mode)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glHint", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":512
 *     uguugl.glHint(target, mode)
 * 
 * def glIsBuffer(buffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsBuffer(buffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_157glIsBuffer(PyObject *__pyx_self, PyObject *__pyx_v_buffer); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_157glIsBuffer = {"glIsBuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_157glIsBuffer, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_157glIsBuffer(PyObject *__pyx_self, PyObject *__pyx_v_buffer) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsBuffer (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_156glIsBuffer(__pyx_self, ((PyObject *)__pyx_v_buffer));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_156glIsBuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_buffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsBuffer", 0);

  /* "renpy/gl2/uguu.pyx":513
 * 
 * def glIsBuffer(buffer):
 *     return uguugl.glIsBuffer(buffer)             # <<<<<<<<<<<<<<
 * 
 * def glIsEnabled(cap):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_buffer); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 513, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsBuffer(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":512
 *     uguugl.glHint(target, mode)
 * 
 * def glIsBuffer(buffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsBuffer(buffer)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsBuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":515
 *     return uguugl.glIsBuffer(buffer)
 * 
 * def glIsEnabled(cap):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsEnabled(cap)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_159glIsEnabled(PyObject *__pyx_self, PyObject *__pyx_v_cap); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_159glIsEnabled = {"glIsEnabled", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_159glIsEnabled, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_159glIsEnabled(PyObject *__pyx_self, PyObject *__pyx_v_cap) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsEnabled (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_158glIsEnabled(__pyx_self, ((PyObject *)__pyx_v_cap));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_158glIsEnabled(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_cap) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsEnabled", 0);

  /* "renpy/gl2/uguu.pyx":516
 * 
 * def glIsEnabled(cap):
 *     return uguugl.glIsEnabled(cap)             # <<<<<<<<<<<<<<
 * 
 * def glIsFramebuffer(framebuffer):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_cap); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 516, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsEnabled(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 516, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":515
 *     return uguugl.glIsBuffer(buffer)
 * 
 * def glIsEnabled(cap):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsEnabled(cap)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsEnabled", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":518
 *     return uguugl.glIsEnabled(cap)
 * 
 * def glIsFramebuffer(framebuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_161glIsFramebuffer(PyObject *__pyx_self, PyObject *__pyx_v_framebuffer); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_161glIsFramebuffer = {"glIsFramebuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_161glIsFramebuffer, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_161glIsFramebuffer(PyObject *__pyx_self, PyObject *__pyx_v_framebuffer) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsFramebuffer (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_160glIsFramebuffer(__pyx_self, ((PyObject *)__pyx_v_framebuffer));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_160glIsFramebuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_framebuffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsFramebuffer", 0);

  /* "renpy/gl2/uguu.pyx":519
 * 
 * def glIsFramebuffer(framebuffer):
 *     return uguugl.glIsFramebuffer(framebuffer)             # <<<<<<<<<<<<<<
 * 
 * def glIsProgram(program):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_framebuffer); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 519, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsFramebuffer(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":518
 *     return uguugl.glIsEnabled(cap)
 * 
 * def glIsFramebuffer(framebuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsFramebuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":521
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 * def glIsProgram(program):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsProgram(program)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_163glIsProgram(PyObject *__pyx_self, PyObject *__pyx_v_program); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_163glIsProgram = {"glIsProgram", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_163glIsProgram, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_163glIsProgram(PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsProgram (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_162glIsProgram(__pyx_self, ((PyObject *)__pyx_v_program));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_162glIsProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsProgram", 0);

  /* "renpy/gl2/uguu.pyx":522
 * 
 * def glIsProgram(program):
 *     return uguugl.glIsProgram(program)             # <<<<<<<<<<<<<<
 * 
 * def glIsRenderbuffer(renderbuffer):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 522, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsProgram(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":521
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 * def glIsProgram(program):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsProgram(program)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsProgram", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":524
 *     return uguugl.glIsProgram(program)
 * 
 * def glIsRenderbuffer(renderbuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_165glIsRenderbuffer(PyObject *__pyx_self, PyObject *__pyx_v_renderbuffer); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_165glIsRenderbuffer = {"glIsRenderbuffer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_165glIsRenderbuffer, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_165glIsRenderbuffer(PyObject *__pyx_self, PyObject *__pyx_v_renderbuffer) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsRenderbuffer (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_164glIsRenderbuffer(__pyx_self, ((PyObject *)__pyx_v_renderbuffer));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_164glIsRenderbuffer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_renderbuffer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsRenderbuffer", 0);

  /* "renpy/gl2/uguu.pyx":525
 * 
 * def glIsRenderbuffer(renderbuffer):
 *     return uguugl.glIsRenderbuffer(renderbuffer)             # <<<<<<<<<<<<<<
 * 
 * def glIsShader(shader):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_renderbuffer); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 525, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsRenderbuffer(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":524
 *     return uguugl.glIsProgram(program)
 * 
 * def glIsRenderbuffer(renderbuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsRenderbuffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":527
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 * def glIsShader(shader):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsShader(shader)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_167glIsShader(PyObject *__pyx_self, PyObject *__pyx_v_shader); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_167glIsShader = {"glIsShader", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_167glIsShader, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_167glIsShader(PyObject *__pyx_self, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsShader (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_166glIsShader(__pyx_self, ((PyObject *)__pyx_v_shader));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_166glIsShader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsShader", 0);

  /* "renpy/gl2/uguu.pyx":528
 * 
 * def glIsShader(shader):
 *     return uguugl.glIsShader(shader)             # <<<<<<<<<<<<<<
 * 
 * def glIsTexture(texture):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 528, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsShader(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":527
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 * def glIsShader(shader):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsShader(shader)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsShader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":530
 *     return uguugl.glIsShader(shader)
 * 
 * def glIsTexture(texture):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsTexture(texture)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_169glIsTexture(PyObject *__pyx_self, PyObject *__pyx_v_texture); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_169glIsTexture = {"glIsTexture", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_169glIsTexture, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_169glIsTexture(PyObject *__pyx_self, PyObject *__pyx_v_texture) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glIsTexture (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_168glIsTexture(__pyx_self, ((PyObject *)__pyx_v_texture));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_168glIsTexture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_texture) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("glIsTexture", 0);

  /* "renpy/gl2/uguu.pyx":531
 * 
 * def glIsTexture(texture):
 *     return uguugl.glIsTexture(texture)             # <<<<<<<<<<<<<<
 * 
 * def glLineWidth(width):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_texture); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 531, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_GLboolean(__pyx_v_5renpy_3gl2_6uguugl_glIsTexture(__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/uguu.pyx":530
 *     return uguugl.glIsShader(shader)
 * 
 * def glIsTexture(texture):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsTexture(texture)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.uguu.glIsTexture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":533
 *     return uguugl.glIsTexture(texture)
 * 
 * def glLineWidth(width):             # <<<<<<<<<<<<<<
 *     uguugl.glLineWidth(width)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_171glLineWidth(PyObject *__pyx_self, PyObject *__pyx_v_width); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_171glLineWidth = {"glLineWidth", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_171glLineWidth, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_171glLineWidth(PyObject *__pyx_self, PyObject *__pyx_v_width) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glLineWidth (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_170glLineWidth(__pyx_self, ((PyObject *)__pyx_v_width));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_170glLineWidth(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLfloat __pyx_t_1;
  __Pyx_RefNannySetupContext("glLineWidth", 0);

  /* "renpy/gl2/uguu.pyx":534
 * 
 * def glLineWidth(width):
 *     uguugl.glLineWidth(width)             # <<<<<<<<<<<<<<
 * 
 * def glLinkProgram(program):
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_width); if (unlikely((__pyx_t_1 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 534, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glLineWidth(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":533
 *     return uguugl.glIsTexture(texture)
 * 
 * def glLineWidth(width):             # <<<<<<<<<<<<<<
 *     uguugl.glLineWidth(width)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glLineWidth", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":536
 *     uguugl.glLineWidth(width)
 * 
 * def glLinkProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glLinkProgram(program)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_173glLinkProgram(PyObject *__pyx_self, PyObject *__pyx_v_program); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_173glLinkProgram = {"glLinkProgram", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_173glLinkProgram, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_173glLinkProgram(PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glLinkProgram (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_172glLinkProgram(__pyx_self, ((PyObject *)__pyx_v_program));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_172glLinkProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glLinkProgram", 0);

  /* "renpy/gl2/uguu.pyx":537
 * 
 * def glLinkProgram(program):
 *     uguugl.glLinkProgram(program)             # <<<<<<<<<<<<<<
 * 
 * def glPixelStorei(pname, param):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 537, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glLinkProgram(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":536
 *     uguugl.glLineWidth(width)
 * 
 * def glLinkProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glLinkProgram(program)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glLinkProgram", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":539
 *     uguugl.glLinkProgram(program)
 * 
 * def glPixelStorei(pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glPixelStorei(pname, param)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_175glPixelStorei(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_175glPixelStorei = {"glPixelStorei", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_175glPixelStorei, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_175glPixelStorei(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_param = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glPixelStorei (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pname,&__pyx_n_s_param,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glPixelStorei", 1, 2, 2, 1); __PYX_ERR(1, 539, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glPixelStorei") < 0)) __PYX_ERR(1, 539, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_pname = values[0];
    __pyx_v_param = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glPixelStorei", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 539, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glPixelStorei", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_174glPixelStorei(__pyx_self, __pyx_v_pname, __pyx_v_param);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_174glPixelStorei(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pname, PyObject *__pyx_v_param) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLint __pyx_t_2;
  __Pyx_RefNannySetupContext("glPixelStorei", 0);

  /* "renpy/gl2/uguu.pyx":540
 * 
 * def glPixelStorei(pname, param):
 *     uguugl.glPixelStorei(pname, param)             # <<<<<<<<<<<<<<
 * 
 * def glPolygonOffset(factor, units):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 540, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_param); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 540, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glPixelStorei(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":539
 *     uguugl.glLinkProgram(program)
 * 
 * def glPixelStorei(pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glPixelStorei(pname, param)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glPixelStorei", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":542
 *     uguugl.glPixelStorei(pname, param)
 * 
 * def glPolygonOffset(factor, units):             # <<<<<<<<<<<<<<
 *     uguugl.glPolygonOffset(factor, units)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_177glPolygonOffset(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_177glPolygonOffset = {"glPolygonOffset", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_177glPolygonOffset, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_177glPolygonOffset(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_factor = 0;
  PyObject *__pyx_v_units = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glPolygonOffset (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_factor,&__pyx_n_s_units,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_factor)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_units)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glPolygonOffset", 1, 2, 2, 1); __PYX_ERR(1, 542, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glPolygonOffset") < 0)) __PYX_ERR(1, 542, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_factor = values[0];
    __pyx_v_units = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glPolygonOffset", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 542, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glPolygonOffset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_176glPolygonOffset(__pyx_self, __pyx_v_factor, __pyx_v_units);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_176glPolygonOffset(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_factor, PyObject *__pyx_v_units) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLfloat __pyx_t_1;
  GLfloat __pyx_t_2;
  __Pyx_RefNannySetupContext("glPolygonOffset", 0);

  /* "renpy/gl2/uguu.pyx":543
 * 
 * def glPolygonOffset(factor, units):
 *     uguugl.glPolygonOffset(factor, units)             # <<<<<<<<<<<<<<
 * 
 * def glReadPixels(x, y, width, height, format, type, pixels):
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_factor); if (unlikely((__pyx_t_1 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 543, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_units); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 543, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glPolygonOffset(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":542
 *     uguugl.glPixelStorei(pname, param)
 * 
 * def glPolygonOffset(factor, units):             # <<<<<<<<<<<<<<
 *     uguugl.glPolygonOffset(factor, units)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glPolygonOffset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":545
 *     uguugl.glPolygonOffset(factor, units)
 * 
 * def glReadPixels(x, y, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_179glReadPixels(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_179glReadPixels = {"glReadPixels", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_179glReadPixels, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_179glReadPixels(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_pixels = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glReadPixels (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_format,&__pyx_n_s_type,&__pyx_n_s_pixels,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, 1); __PYX_ERR(1, 545, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, 2); __PYX_ERR(1, 545, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, 3); __PYX_ERR(1, 545, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, 4); __PYX_ERR(1, 545, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, 5); __PYX_ERR(1, 545, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pixels)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, 6); __PYX_ERR(1, 545, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glReadPixels") < 0)) __PYX_ERR(1, 545, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
    __pyx_v_width = values[2];
    __pyx_v_height = values[3];
    __pyx_v_format = values[4];
    __pyx_v_type = values[5];
    __pyx_v_pixels = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glReadPixels", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 545, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glReadPixels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_178glReadPixels(__pyx_self, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height, __pyx_v_format, __pyx_v_type, __pyx_v_pixels);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_178glReadPixels(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_format, PyObject *__pyx_v_type, PyObject *__pyx_v_pixels) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_pixels_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLint __pyx_t_3;
  GLsizei __pyx_t_4;
  GLsizei __pyx_t_5;
  GLenum __pyx_t_6;
  GLenum __pyx_t_7;
  __Pyx_RefNannySetupContext("glReadPixels", 0);

  /* "renpy/gl2/uguu.pyx":546
 * 
 * def glReadPixels(x, y, width, height, format, type, pixels):
 *     cdef ptr pixels_ptr = get_ptr(pixels)             # <<<<<<<<<<<<<<
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_pixels)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pixels_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":547
 * def glReadPixels(x, y, width, height, format, type, pixels):
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glRenderbufferStorage(target, internalformat, width, height):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_x); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 547, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_y); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 547, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 547, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_5 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 547, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLenum(__pyx_v_format); if (unlikely((__pyx_t_6 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 547, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_7 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 547, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glReadPixels(__pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, ((void *)__pyx_v_pixels_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":545
 *     uguugl.glPolygonOffset(factor, units)
 * 
 * def glReadPixels(x, y, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glReadPixels", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pixels_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":549
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 * 
 * def glRenderbufferStorage(target, internalformat, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_181glRenderbufferStorage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_181glRenderbufferStorage = {"glRenderbufferStorage", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_181glRenderbufferStorage, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_181glRenderbufferStorage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_internalformat = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glRenderbufferStorage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_internalformat,&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_internalformat)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glRenderbufferStorage", 1, 4, 4, 1); __PYX_ERR(1, 549, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glRenderbufferStorage", 1, 4, 4, 2); __PYX_ERR(1, 549, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glRenderbufferStorage", 1, 4, 4, 3); __PYX_ERR(1, 549, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glRenderbufferStorage") < 0)) __PYX_ERR(1, 549, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_target = values[0];
    __pyx_v_internalformat = values[1];
    __pyx_v_width = values[2];
    __pyx_v_height = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glRenderbufferStorage", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 549, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glRenderbufferStorage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_180glRenderbufferStorage(__pyx_self, __pyx_v_target, __pyx_v_internalformat, __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_180glRenderbufferStorage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_width, PyObject *__pyx_v_height) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLsizei __pyx_t_3;
  GLsizei __pyx_t_4;
  __Pyx_RefNannySetupContext("glRenderbufferStorage", 0);

  /* "renpy/gl2/uguu.pyx":550
 * 
 * def glRenderbufferStorage(target, internalformat, width, height):
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)             # <<<<<<<<<<<<<<
 * 
 * def glSampleCoverage(value, invert):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 550, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_internalformat); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 550, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 550, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 550, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glRenderbufferStorage(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":549
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 * 
 * def glRenderbufferStorage(target, internalformat, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glRenderbufferStorage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":552
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 * def glSampleCoverage(value, invert):             # <<<<<<<<<<<<<<
 *     uguugl.glSampleCoverage(value, invert)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_183glSampleCoverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_183glSampleCoverage = {"glSampleCoverage", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_183glSampleCoverage, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_183glSampleCoverage(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_invert = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glSampleCoverage (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_value,&__pyx_n_s_invert,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_invert)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glSampleCoverage", 1, 2, 2, 1); __PYX_ERR(1, 552, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glSampleCoverage") < 0)) __PYX_ERR(1, 552, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_value = values[0];
    __pyx_v_invert = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glSampleCoverage", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 552, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glSampleCoverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_182glSampleCoverage(__pyx_self, __pyx_v_value, __pyx_v_invert);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_182glSampleCoverage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_value, PyObject *__pyx_v_invert) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLfloat __pyx_t_1;
  GLboolean __pyx_t_2;
  __Pyx_RefNannySetupContext("glSampleCoverage", 0);

  /* "renpy/gl2/uguu.pyx":553
 * 
 * def glSampleCoverage(value, invert):
 *     uguugl.glSampleCoverage(value, invert)             # <<<<<<<<<<<<<<
 * 
 * def glScissor(x, y, width, height):
 */
  __pyx_t_1 = __pyx_PyFloat_AsFloat(__pyx_v_value); if (unlikely((__pyx_t_1 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 553, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLboolean(__pyx_v_invert); if (unlikely((__pyx_t_2 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 553, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glSampleCoverage(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":552
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 * def glSampleCoverage(value, invert):             # <<<<<<<<<<<<<<
 *     uguugl.glSampleCoverage(value, invert)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glSampleCoverage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":555
 *     uguugl.glSampleCoverage(value, invert)
 * 
 * def glScissor(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glScissor(x, y, width, height)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_185glScissor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_185glScissor = {"glScissor", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_185glScissor, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_185glScissor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glScissor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glScissor", 1, 4, 4, 1); __PYX_ERR(1, 555, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glScissor", 1, 4, 4, 2); __PYX_ERR(1, 555, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glScissor", 1, 4, 4, 3); __PYX_ERR(1, 555, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glScissor") < 0)) __PYX_ERR(1, 555, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
    __pyx_v_width = values[2];
    __pyx_v_height = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glScissor", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 555, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glScissor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_184glScissor(__pyx_self, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_184glScissor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  GLsizei __pyx_t_4;
  __Pyx_RefNannySetupContext("glScissor", 0);

  /* "renpy/gl2/uguu.pyx":556
 * 
 * def glScissor(x, y, width, height):
 *     uguugl.glScissor(x, y, width, height)             # <<<<<<<<<<<<<<
 * 
 * def glShaderSource(shader, count, string, length):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_x); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 556, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_y); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 556, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 556, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 556, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glScissor(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":555
 *     uguugl.glSampleCoverage(value, invert)
 * 
 * def glScissor(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glScissor(x, y, width, height)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glScissor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":558
 *     uguugl.glScissor(x, y, width, height)
 * 
 * def glShaderSource(shader, count, string, length):             # <<<<<<<<<<<<<<
 *     cdef ptr string_ptr = get_ptr(string)
 *     cdef ptr length_ptr = get_ptr(length)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_187glShaderSource(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_187glShaderSource = {"glShaderSource", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_187glShaderSource, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_187glShaderSource(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_string = 0;
  PyObject *__pyx_v_length = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glShaderSource (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shader,&__pyx_n_s_count,&__pyx_n_s_string,&__pyx_n_s_length,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glShaderSource", 1, 4, 4, 1); __PYX_ERR(1, 558, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_string)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glShaderSource", 1, 4, 4, 2); __PYX_ERR(1, 558, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_length)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glShaderSource", 1, 4, 4, 3); __PYX_ERR(1, 558, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glShaderSource") < 0)) __PYX_ERR(1, 558, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_shader = values[0];
    __pyx_v_count = values[1];
    __pyx_v_string = values[2];
    __pyx_v_length = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glShaderSource", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 558, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glShaderSource", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_186glShaderSource(__pyx_self, __pyx_v_shader, __pyx_v_count, __pyx_v_string, __pyx_v_length);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_186glShaderSource(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_count, PyObject *__pyx_v_string, PyObject *__pyx_v_length) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_string_ptr = 0;
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_length_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glShaderSource", 0);

  /* "renpy/gl2/uguu.pyx":559
 * 
 * def glShaderSource(shader, count, string, length):
 *     cdef ptr string_ptr = get_ptr(string)             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_string)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_string_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":560
 * def glShaderSource(shader, count, string, length):
 *     cdef ptr string_ptr = get_ptr(string)
 *     cdef ptr length_ptr = get_ptr(length)             # <<<<<<<<<<<<<<
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_length)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_length_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":561
 *     cdef ptr string_ptr = get_ptr(string)
 *     cdef ptr length_ptr = get_ptr(length)
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glStencilFunc(func, ref, mask):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_shader); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 561, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 561, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glShaderSource(__pyx_t_2, __pyx_t_3, ((GLchar const *const *)__pyx_v_string_ptr->ptr), ((GLint const *)__pyx_v_length_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":558
 *     uguugl.glScissor(x, y, width, height)
 * 
 * def glShaderSource(shader, count, string, length):             # <<<<<<<<<<<<<<
 *     cdef ptr string_ptr = get_ptr(string)
 *     cdef ptr length_ptr = get_ptr(length)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glShaderSource", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_string_ptr);
  __Pyx_XDECREF((PyObject *)__pyx_v_length_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":563
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)
 * 
 * def glStencilFunc(func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_189glStencilFunc(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_189glStencilFunc = {"glStencilFunc", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_189glStencilFunc, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_189glStencilFunc(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_func = 0;
  PyObject *__pyx_v_ref = 0;
  PyObject *__pyx_v_mask = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glStencilFunc (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_func,&__pyx_n_s_ref,&__pyx_n_s_mask,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_func)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilFunc", 1, 3, 3, 1); __PYX_ERR(1, 563, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mask)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilFunc", 1, 3, 3, 2); __PYX_ERR(1, 563, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glStencilFunc") < 0)) __PYX_ERR(1, 563, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_func = values[0];
    __pyx_v_ref = values[1];
    __pyx_v_mask = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glStencilFunc", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 563, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilFunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_188glStencilFunc(__pyx_self, __pyx_v_func, __pyx_v_ref, __pyx_v_mask);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_188glStencilFunc(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_func, PyObject *__pyx_v_ref, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLint __pyx_t_2;
  GLuint __pyx_t_3;
  __Pyx_RefNannySetupContext("glStencilFunc", 0);

  /* "renpy/gl2/uguu.pyx":564
 * 
 * def glStencilFunc(func, ref, mask):
 *     uguugl.glStencilFunc(func, ref, mask)             # <<<<<<<<<<<<<<
 * 
 * def glStencilFuncSeparate(face, func, ref, mask):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_func); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 564, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_ref); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 564, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLuint(__pyx_v_mask); if (unlikely((__pyx_t_3 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 564, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glStencilFunc(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":563
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)
 * 
 * def glStencilFunc(func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilFunc", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":566
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 * def glStencilFuncSeparate(face, func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_191glStencilFuncSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_191glStencilFuncSeparate = {"glStencilFuncSeparate", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_191glStencilFuncSeparate, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_191glStencilFuncSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_face = 0;
  PyObject *__pyx_v_func = 0;
  PyObject *__pyx_v_ref = 0;
  PyObject *__pyx_v_mask = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glStencilFuncSeparate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_face,&__pyx_n_s_func,&__pyx_n_s_ref,&__pyx_n_s_mask,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_face)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_func)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilFuncSeparate", 1, 4, 4, 1); __PYX_ERR(1, 566, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilFuncSeparate", 1, 4, 4, 2); __PYX_ERR(1, 566, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mask)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilFuncSeparate", 1, 4, 4, 3); __PYX_ERR(1, 566, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glStencilFuncSeparate") < 0)) __PYX_ERR(1, 566, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_face = values[0];
    __pyx_v_func = values[1];
    __pyx_v_ref = values[2];
    __pyx_v_mask = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glStencilFuncSeparate", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 566, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilFuncSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_190glStencilFuncSeparate(__pyx_self, __pyx_v_face, __pyx_v_func, __pyx_v_ref, __pyx_v_mask);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_190glStencilFuncSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_face, PyObject *__pyx_v_func, PyObject *__pyx_v_ref, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLint __pyx_t_3;
  GLuint __pyx_t_4;
  __Pyx_RefNannySetupContext("glStencilFuncSeparate", 0);

  /* "renpy/gl2/uguu.pyx":567
 * 
 * def glStencilFuncSeparate(face, func, ref, mask):
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)             # <<<<<<<<<<<<<<
 * 
 * def glStencilMask(mask):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_face); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 567, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_func); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 567, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_ref); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 567, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLuint(__pyx_v_mask); if (unlikely((__pyx_t_4 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 567, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glStencilFuncSeparate(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":566
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 * def glStencilFuncSeparate(face, func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilFuncSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":569
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 * def glStencilMask(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMask(mask)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_193glStencilMask(PyObject *__pyx_self, PyObject *__pyx_v_mask); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_193glStencilMask = {"glStencilMask", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_193glStencilMask, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_193glStencilMask(PyObject *__pyx_self, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glStencilMask (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_192glStencilMask(__pyx_self, ((PyObject *)__pyx_v_mask));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_192glStencilMask(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glStencilMask", 0);

  /* "renpy/gl2/uguu.pyx":570
 * 
 * def glStencilMask(mask):
 *     uguugl.glStencilMask(mask)             # <<<<<<<<<<<<<<
 * 
 * def glStencilMaskSeparate(face, mask):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_mask); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 570, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glStencilMask(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":569
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 * def glStencilMask(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMask(mask)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilMask", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":572
 *     uguugl.glStencilMask(mask)
 * 
 * def glStencilMaskSeparate(face, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_195glStencilMaskSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_195glStencilMaskSeparate = {"glStencilMaskSeparate", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_195glStencilMaskSeparate, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_195glStencilMaskSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_face = 0;
  PyObject *__pyx_v_mask = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glStencilMaskSeparate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_face,&__pyx_n_s_mask,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_face)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mask)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilMaskSeparate", 1, 2, 2, 1); __PYX_ERR(1, 572, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glStencilMaskSeparate") < 0)) __PYX_ERR(1, 572, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_face = values[0];
    __pyx_v_mask = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glStencilMaskSeparate", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 572, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilMaskSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_194glStencilMaskSeparate(__pyx_self, __pyx_v_face, __pyx_v_mask);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_194glStencilMaskSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_face, PyObject *__pyx_v_mask) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glStencilMaskSeparate", 0);

  /* "renpy/gl2/uguu.pyx":573
 * 
 * def glStencilMaskSeparate(face, mask):
 *     uguugl.glStencilMaskSeparate(face, mask)             # <<<<<<<<<<<<<<
 * 
 * def glStencilOp(fail, zfail, zpass):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_face); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 573, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_mask); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 573, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glStencilMaskSeparate(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":572
 *     uguugl.glStencilMask(mask)
 * 
 * def glStencilMaskSeparate(face, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilMaskSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":575
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 * def glStencilOp(fail, zfail, zpass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_197glStencilOp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_197glStencilOp = {"glStencilOp", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_197glStencilOp, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_197glStencilOp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_fail = 0;
  PyObject *__pyx_v_zfail = 0;
  PyObject *__pyx_v_zpass = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glStencilOp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_fail,&__pyx_n_s_zfail,&__pyx_n_s_zpass,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fail)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_zfail)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilOp", 1, 3, 3, 1); __PYX_ERR(1, 575, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_zpass)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilOp", 1, 3, 3, 2); __PYX_ERR(1, 575, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glStencilOp") < 0)) __PYX_ERR(1, 575, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_fail = values[0];
    __pyx_v_zfail = values[1];
    __pyx_v_zpass = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glStencilOp", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 575, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilOp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_196glStencilOp(__pyx_self, __pyx_v_fail, __pyx_v_zfail, __pyx_v_zpass);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_196glStencilOp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_fail, PyObject *__pyx_v_zfail, PyObject *__pyx_v_zpass) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glStencilOp", 0);

  /* "renpy/gl2/uguu.pyx":576
 * 
 * def glStencilOp(fail, zfail, zpass):
 *     uguugl.glStencilOp(fail, zfail, zpass)             # <<<<<<<<<<<<<<
 * 
 * def glStencilOpSeparate(face, sfail, dpfail, dppass):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_fail); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_zfail); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_zpass); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glStencilOp(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":575
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 * def glStencilOp(fail, zfail, zpass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilOp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":578
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 * def glStencilOpSeparate(face, sfail, dpfail, dppass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_199glStencilOpSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_199glStencilOpSeparate = {"glStencilOpSeparate", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_199glStencilOpSeparate, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_199glStencilOpSeparate(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_face = 0;
  PyObject *__pyx_v_sfail = 0;
  PyObject *__pyx_v_dpfail = 0;
  PyObject *__pyx_v_dppass = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glStencilOpSeparate (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_face,&__pyx_n_s_sfail,&__pyx_n_s_dpfail,&__pyx_n_s_dppass,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_face)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sfail)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilOpSeparate", 1, 4, 4, 1); __PYX_ERR(1, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dpfail)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilOpSeparate", 1, 4, 4, 2); __PYX_ERR(1, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dppass)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glStencilOpSeparate", 1, 4, 4, 3); __PYX_ERR(1, 578, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glStencilOpSeparate") < 0)) __PYX_ERR(1, 578, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_face = values[0];
    __pyx_v_sfail = values[1];
    __pyx_v_dpfail = values[2];
    __pyx_v_dppass = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glStencilOpSeparate", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 578, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilOpSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_198glStencilOpSeparate(__pyx_self, __pyx_v_face, __pyx_v_sfail, __pyx_v_dpfail, __pyx_v_dppass);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_198glStencilOpSeparate(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_face, PyObject *__pyx_v_sfail, PyObject *__pyx_v_dpfail, PyObject *__pyx_v_dppass) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  GLenum __pyx_t_4;
  __Pyx_RefNannySetupContext("glStencilOpSeparate", 0);

  /* "renpy/gl2/uguu.pyx":579
 * 
 * def glStencilOpSeparate(face, sfail, dpfail, dppass):
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)             # <<<<<<<<<<<<<<
 * 
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_face); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 579, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_sfail); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 579, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_dpfail); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 579, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_dppass); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 579, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glStencilOpSeparate(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":578
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 * def glStencilOpSeparate(face, sfail, dpfail, dppass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glStencilOpSeparate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":581
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_201glTexImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_201glTexImage2D = {"glTexImage2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_201glTexImage2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_201glTexImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_v_internalformat = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_border = 0;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_pixels = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glTexImage2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_level,&__pyx_n_s_internalformat,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_border,&__pyx_n_s_format,&__pyx_n_s_type,&__pyx_n_s_pixels,0};
    PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 1); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_internalformat)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 2); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 3); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 4); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 5); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 6); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 7); __PYX_ERR(1, 581, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pixels)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, 8); __PYX_ERR(1, 581, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glTexImage2D") < 0)) __PYX_ERR(1, 581, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 9) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
    }
    __pyx_v_target = values[0];
    __pyx_v_level = values[1];
    __pyx_v_internalformat = values[2];
    __pyx_v_width = values[3];
    __pyx_v_height = values[4];
    __pyx_v_border = values[5];
    __pyx_v_format = values[6];
    __pyx_v_type = values[7];
    __pyx_v_pixels = values[8];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glTexImage2D", 1, 9, 9, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 581, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_200glTexImage2D(__pyx_self, __pyx_v_target, __pyx_v_level, __pyx_v_internalformat, __pyx_v_width, __pyx_v_height, __pyx_v_border, __pyx_v_format, __pyx_v_type, __pyx_v_pixels);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_200glTexImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_internalformat, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_border, PyObject *__pyx_v_format, PyObject *__pyx_v_type, PyObject *__pyx_v_pixels) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_pixels_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLint __pyx_t_3;
  GLint __pyx_t_4;
  GLsizei __pyx_t_5;
  GLsizei __pyx_t_6;
  GLint __pyx_t_7;
  GLenum __pyx_t_8;
  GLenum __pyx_t_9;
  __Pyx_RefNannySetupContext("glTexImage2D", 0);

  /* "renpy/gl2/uguu.pyx":582
 * 
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):
 *     cdef ptr pixels_ptr = get_ptr(pixels)             # <<<<<<<<<<<<<<
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_pixels)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pixels_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":583
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glTexParameterf(target, pname, param):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_internalformat); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_5 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_6 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLint(__pyx_v_border); if (unlikely((__pyx_t_7 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_GLenum(__pyx_v_format); if (unlikely((__pyx_t_8 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_9 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glTexImage2D(__pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, ((void const *)__pyx_v_pixels_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":581
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pixels_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":585
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glTexParameterf(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_203glTexParameterf(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_203glTexParameterf = {"glTexParameterf", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_203glTexParameterf, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_203glTexParameterf(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_param = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glTexParameterf (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_param,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameterf", 1, 3, 3, 1); __PYX_ERR(1, 585, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameterf", 1, 3, 3, 2); __PYX_ERR(1, 585, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glTexParameterf") < 0)) __PYX_ERR(1, 585, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_param = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glTexParameterf", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 585, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameterf", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_202glTexParameterf(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_param);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_202glTexParameterf(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_param) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLfloat __pyx_t_3;
  __Pyx_RefNannySetupContext("glTexParameterf", 0);

  /* "renpy/gl2/uguu.pyx":586
 * 
 * def glTexParameterf(target, pname, param):
 *     uguugl.glTexParameterf(target, pname, param)             # <<<<<<<<<<<<<<
 * 
 * def glTexParameterfv(target, pname, params):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 586, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 586, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_param); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 586, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameterf(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":585
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glTexParameterf(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameterf", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":588
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 * def glTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_205glTexParameterfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_205glTexParameterfv = {"glTexParameterfv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_205glTexParameterfv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_205glTexParameterfv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glTexParameterfv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameterfv", 1, 3, 3, 1); __PYX_ERR(1, 588, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameterfv", 1, 3, 3, 2); __PYX_ERR(1, 588, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glTexParameterfv") < 0)) __PYX_ERR(1, 588, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glTexParameterfv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 588, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameterfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_204glTexParameterfv(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_204glTexParameterfv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glTexParameterfv", 0);

  /* "renpy/gl2/uguu.pyx":589
 * 
 * def glTexParameterfv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":590
 * def glTexParameterfv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glTexParameteri(target, pname, param):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 590, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 590, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameterfv(__pyx_t_2, __pyx_t_3, ((GLfloat const *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":588
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 * def glTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameterfv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":592
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 * 
 * def glTexParameteri(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_207glTexParameteri(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_207glTexParameteri = {"glTexParameteri", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_207glTexParameteri, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_207glTexParameteri(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_param = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glTexParameteri (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_param,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameteri", 1, 3, 3, 1); __PYX_ERR(1, 592, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameteri", 1, 3, 3, 2); __PYX_ERR(1, 592, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glTexParameteri") < 0)) __PYX_ERR(1, 592, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_param = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glTexParameteri", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 592, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameteri", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_206glTexParameteri(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_param);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_206glTexParameteri(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_param) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLenum __pyx_t_1;
  GLenum __pyx_t_2;
  GLint __pyx_t_3;
  __Pyx_RefNannySetupContext("glTexParameteri", 0);

  /* "renpy/gl2/uguu.pyx":593
 * 
 * def glTexParameteri(target, pname, param):
 *     uguugl.glTexParameteri(target, pname, param)             # <<<<<<<<<<<<<<
 * 
 * def glTexParameteriv(target, pname, params):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_1 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 593, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 593, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_param); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 593, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameteri(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":592
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 * 
 * def glTexParameteri(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameteri", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":595
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 * def glTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_209glTexParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_209glTexParameteriv = {"glTexParameteriv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_209glTexParameteriv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_209glTexParameteriv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_pname = 0;
  PyObject *__pyx_v_params = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glTexParameteriv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_pname,&__pyx_n_s_params,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pname)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameteriv", 1, 3, 3, 1); __PYX_ERR(1, 595, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_params)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexParameteriv", 1, 3, 3, 2); __PYX_ERR(1, 595, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glTexParameteriv") < 0)) __PYX_ERR(1, 595, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_target = values[0];
    __pyx_v_pname = values[1];
    __pyx_v_params = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glTexParameteriv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 595, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_208glTexParameteriv(__pyx_self, __pyx_v_target, __pyx_v_pname, __pyx_v_params);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_208glTexParameteriv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_pname, PyObject *__pyx_v_params) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_params_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLenum __pyx_t_3;
  __Pyx_RefNannySetupContext("glTexParameteriv", 0);

  /* "renpy/gl2/uguu.pyx":596
 * 
 * def glTexParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_params)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":597
 * def glTexParameteriv(target, pname, params):
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 597, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_pname); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 597, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameteriv(__pyx_t_2, __pyx_t_3, ((GLint const *)__pyx_v_params_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":595
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 * def glTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexParameteriv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_params_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":599
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 * 
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_211glTexSubImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_211glTexSubImage2D = {"glTexSubImage2D", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_211glTexSubImage2D, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_211glTexSubImage2D(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_target = 0;
  PyObject *__pyx_v_level = 0;
  PyObject *__pyx_v_xoffset = 0;
  PyObject *__pyx_v_yoffset = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_pixels = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glTexSubImage2D (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_target,&__pyx_n_s_level,&__pyx_n_s_xoffset,&__pyx_n_s_yoffset,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_format,&__pyx_n_s_type,&__pyx_n_s_pixels,0};
    PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_target)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_level)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 1); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xoffset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 2); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yoffset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 3); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 4); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 5); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 6); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 7); __PYX_ERR(1, 599, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pixels)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, 8); __PYX_ERR(1, 599, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glTexSubImage2D") < 0)) __PYX_ERR(1, 599, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 9) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
    }
    __pyx_v_target = values[0];
    __pyx_v_level = values[1];
    __pyx_v_xoffset = values[2];
    __pyx_v_yoffset = values[3];
    __pyx_v_width = values[4];
    __pyx_v_height = values[5];
    __pyx_v_format = values[6];
    __pyx_v_type = values[7];
    __pyx_v_pixels = values[8];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glTexSubImage2D", 1, 9, 9, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 599, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexSubImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_210glTexSubImage2D(__pyx_self, __pyx_v_target, __pyx_v_level, __pyx_v_xoffset, __pyx_v_yoffset, __pyx_v_width, __pyx_v_height, __pyx_v_format, __pyx_v_type, __pyx_v_pixels);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_210glTexSubImage2D(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_target, PyObject *__pyx_v_level, PyObject *__pyx_v_xoffset, PyObject *__pyx_v_yoffset, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_format, PyObject *__pyx_v_type, PyObject *__pyx_v_pixels) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_pixels_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLenum __pyx_t_2;
  GLint __pyx_t_3;
  GLint __pyx_t_4;
  GLint __pyx_t_5;
  GLsizei __pyx_t_6;
  GLsizei __pyx_t_7;
  GLenum __pyx_t_8;
  GLenum __pyx_t_9;
  __Pyx_RefNannySetupContext("glTexSubImage2D", 0);

  /* "renpy/gl2/uguu.pyx":600
 * 
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):
 *     cdef ptr pixels_ptr = get_ptr(pixels)             # <<<<<<<<<<<<<<
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_pixels)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pixels_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":601
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform1f(location, v0):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLenum(__pyx_v_target); if (unlikely((__pyx_t_2 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_level); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_xoffset); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLint(__pyx_v_yoffset); if (unlikely((__pyx_t_5 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_6 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_7 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_8 = __Pyx_PyInt_As_GLenum(__pyx_v_format); if (unlikely((__pyx_t_8 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_t_9 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_9 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 601, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glTexSubImage2D(__pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9, ((void const *)__pyx_v_pixels_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":599
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 * 
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glTexSubImage2D", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pixels_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":603
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glUniform1f(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1f(location, v0)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_213glUniform1f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_213glUniform1f = {"glUniform1f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_213glUniform1f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_213glUniform1f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform1f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform1f", 1, 2, 2, 1); __PYX_ERR(1, 603, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform1f") < 0)) __PYX_ERR(1, 603, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform1f", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 603, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_212glUniform1f(__pyx_self, __pyx_v_location, __pyx_v_v0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_212glUniform1f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLfloat __pyx_t_2;
  __Pyx_RefNannySetupContext("glUniform1f", 0);

  /* "renpy/gl2/uguu.pyx":604
 * 
 * def glUniform1f(location, v0):
 *     uguugl.glUniform1f(location, v0)             # <<<<<<<<<<<<<<
 * 
 * def glUniform1fv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 604, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 604, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1f(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":603
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glUniform1f(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1f(location, v0)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":606
 *     uguugl.glUniform1f(location, v0)
 * 
 * def glUniform1fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_215glUniform1fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_215glUniform1fv = {"glUniform1fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_215glUniform1fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_215glUniform1fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform1fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform1fv", 1, 3, 3, 1); __PYX_ERR(1, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform1fv", 1, 3, 3, 2); __PYX_ERR(1, 606, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform1fv") < 0)) __PYX_ERR(1, 606, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform1fv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 606, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_214glUniform1fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_214glUniform1fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform1fv", 0);

  /* "renpy/gl2/uguu.pyx":607
 * 
 * def glUniform1fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":608
 * def glUniform1fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform1i(location, v0):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 608, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 608, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1fv(__pyx_t_2, __pyx_t_3, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":606
 *     uguugl.glUniform1f(location, v0)
 * 
 * def glUniform1fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":610
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform1i(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1i(location, v0)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_217glUniform1i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_217glUniform1i = {"glUniform1i", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_217glUniform1i, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_217glUniform1i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform1i (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform1i", 1, 2, 2, 1); __PYX_ERR(1, 610, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform1i") < 0)) __PYX_ERR(1, 610, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform1i", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 610, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_216glUniform1i(__pyx_self, __pyx_v_location, __pyx_v_v0);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_216glUniform1i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLint __pyx_t_2;
  __Pyx_RefNannySetupContext("glUniform1i", 0);

  /* "renpy/gl2/uguu.pyx":611
 * 
 * def glUniform1i(location, v0):
 *     uguugl.glUniform1i(location, v0)             # <<<<<<<<<<<<<<
 * 
 * def glUniform1iv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 611, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 611, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1i(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":610
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform1i(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1i(location, v0)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":613
 *     uguugl.glUniform1i(location, v0)
 * 
 * def glUniform1iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_219glUniform1iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_219glUniform1iv = {"glUniform1iv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_219glUniform1iv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_219glUniform1iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform1iv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform1iv", 1, 3, 3, 1); __PYX_ERR(1, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform1iv", 1, 3, 3, 2); __PYX_ERR(1, 613, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform1iv") < 0)) __PYX_ERR(1, 613, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform1iv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 613, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_218glUniform1iv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_218glUniform1iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform1iv", 0);

  /* "renpy/gl2/uguu.pyx":614
 * 
 * def glUniform1iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":615
 * def glUniform1iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform2f(location, v0, v1):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 615, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 615, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1iv(__pyx_t_2, __pyx_t_3, ((GLint const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":613
 *     uguugl.glUniform1i(location, v0)
 * 
 * def glUniform1iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform1iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":617
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform2f(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_221glUniform2f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_221glUniform2f = {"glUniform2f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_221glUniform2f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_221glUniform2f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_v_v1 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform2f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,&__pyx_n_s_v1,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2f", 1, 3, 3, 1); __PYX_ERR(1, 617, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2f", 1, 3, 3, 2); __PYX_ERR(1, 617, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform2f") < 0)) __PYX_ERR(1, 617, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
    __pyx_v_v1 = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform2f", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 617, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_220glUniform2f(__pyx_self, __pyx_v_location, __pyx_v_v0, __pyx_v_v1);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_220glUniform2f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform2f", 0);

  /* "renpy/gl2/uguu.pyx":618
 * 
 * def glUniform2f(location, v0, v1):
 *     uguugl.glUniform2f(location, v0, v1)             # <<<<<<<<<<<<<<
 * 
 * def glUniform2fv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 618, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 618, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_v1); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 618, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2f(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":617
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform2f(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":620
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 * def glUniform2fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_223glUniform2fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_223glUniform2fv = {"glUniform2fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_223glUniform2fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_223glUniform2fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform2fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2fv", 1, 3, 3, 1); __PYX_ERR(1, 620, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2fv", 1, 3, 3, 2); __PYX_ERR(1, 620, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform2fv") < 0)) __PYX_ERR(1, 620, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform2fv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 620, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_222glUniform2fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_222glUniform2fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform2fv", 0);

  /* "renpy/gl2/uguu.pyx":621
 * 
 * def glUniform2fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":622
 * def glUniform2fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform2i(location, v0, v1):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 622, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 622, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2fv(__pyx_t_2, __pyx_t_3, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":620
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 * def glUniform2fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":624
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform2i(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_225glUniform2i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_225glUniform2i = {"glUniform2i", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_225glUniform2i, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_225glUniform2i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_v_v1 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform2i (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,&__pyx_n_s_v1,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2i", 1, 3, 3, 1); __PYX_ERR(1, 624, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2i", 1, 3, 3, 2); __PYX_ERR(1, 624, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform2i") < 0)) __PYX_ERR(1, 624, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
    __pyx_v_v1 = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform2i", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 624, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_224glUniform2i(__pyx_self, __pyx_v_location, __pyx_v_v0, __pyx_v_v1);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_224glUniform2i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLint __pyx_t_2;
  GLint __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform2i", 0);

  /* "renpy/gl2/uguu.pyx":625
 * 
 * def glUniform2i(location, v0, v1):
 *     uguugl.glUniform2i(location, v0, v1)             # <<<<<<<<<<<<<<
 * 
 * def glUniform2iv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 625, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 625, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_v1); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 625, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2i(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":624
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform2i(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":627
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 * def glUniform2iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_227glUniform2iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_227glUniform2iv = {"glUniform2iv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_227glUniform2iv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_227glUniform2iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform2iv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2iv", 1, 3, 3, 1); __PYX_ERR(1, 627, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform2iv", 1, 3, 3, 2); __PYX_ERR(1, 627, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform2iv") < 0)) __PYX_ERR(1, 627, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform2iv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 627, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_226glUniform2iv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_226glUniform2iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform2iv", 0);

  /* "renpy/gl2/uguu.pyx":628
 * 
 * def glUniform2iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":629
 * def glUniform2iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform3f(location, v0, v1, v2):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 629, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 629, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2iv(__pyx_t_2, __pyx_t_3, ((GLint const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":627
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 * def glUniform2iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform2iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":631
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform3f(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_229glUniform3f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_229glUniform3f = {"glUniform3f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_229glUniform3f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_229glUniform3f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_v_v1 = 0;
  PyObject *__pyx_v_v2 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform3f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,&__pyx_n_s_v1,&__pyx_n_s_v2,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3f", 1, 4, 4, 1); __PYX_ERR(1, 631, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3f", 1, 4, 4, 2); __PYX_ERR(1, 631, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3f", 1, 4, 4, 3); __PYX_ERR(1, 631, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform3f") < 0)) __PYX_ERR(1, 631, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
    __pyx_v_v1 = values[2];
    __pyx_v_v2 = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform3f", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 631, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_228glUniform3f(__pyx_self, __pyx_v_location, __pyx_v_v0, __pyx_v_v1, __pyx_v_v2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_228glUniform3f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  GLfloat __pyx_t_4;
  __Pyx_RefNannySetupContext("glUniform3f", 0);

  /* "renpy/gl2/uguu.pyx":632
 * 
 * def glUniform3f(location, v0, v1, v2):
 *     uguugl.glUniform3f(location, v0, v1, v2)             # <<<<<<<<<<<<<<
 * 
 * def glUniform3fv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 632, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 632, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_v1); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 632, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_v2); if (unlikely((__pyx_t_4 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 632, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3f(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":631
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform3f(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":634
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 * def glUniform3fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_231glUniform3fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_231glUniform3fv = {"glUniform3fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_231glUniform3fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_231glUniform3fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform3fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3fv", 1, 3, 3, 1); __PYX_ERR(1, 634, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3fv", 1, 3, 3, 2); __PYX_ERR(1, 634, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform3fv") < 0)) __PYX_ERR(1, 634, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform3fv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 634, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_230glUniform3fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_230glUniform3fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform3fv", 0);

  /* "renpy/gl2/uguu.pyx":635
 * 
 * def glUniform3fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":636
 * def glUniform3fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform3i(location, v0, v1, v2):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 636, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 636, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3fv(__pyx_t_2, __pyx_t_3, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":634
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 * def glUniform3fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":638
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform3i(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_233glUniform3i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_233glUniform3i = {"glUniform3i", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_233glUniform3i, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_233glUniform3i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_v_v1 = 0;
  PyObject *__pyx_v_v2 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform3i (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,&__pyx_n_s_v1,&__pyx_n_s_v2,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3i", 1, 4, 4, 1); __PYX_ERR(1, 638, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3i", 1, 4, 4, 2); __PYX_ERR(1, 638, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3i", 1, 4, 4, 3); __PYX_ERR(1, 638, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform3i") < 0)) __PYX_ERR(1, 638, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
    __pyx_v_v1 = values[2];
    __pyx_v_v2 = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform3i", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 638, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_232glUniform3i(__pyx_self, __pyx_v_location, __pyx_v_v0, __pyx_v_v1, __pyx_v_v2);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_232glUniform3i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLint __pyx_t_2;
  GLint __pyx_t_3;
  GLint __pyx_t_4;
  __Pyx_RefNannySetupContext("glUniform3i", 0);

  /* "renpy/gl2/uguu.pyx":639
 * 
 * def glUniform3i(location, v0, v1, v2):
 *     uguugl.glUniform3i(location, v0, v1, v2)             # <<<<<<<<<<<<<<
 * 
 * def glUniform3iv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 639, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 639, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_v1); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 639, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_v2); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 639, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3i(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":638
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform3i(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":641
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 * def glUniform3iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_235glUniform3iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_235glUniform3iv = {"glUniform3iv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_235glUniform3iv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_235glUniform3iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform3iv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3iv", 1, 3, 3, 1); __PYX_ERR(1, 641, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform3iv", 1, 3, 3, 2); __PYX_ERR(1, 641, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform3iv") < 0)) __PYX_ERR(1, 641, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform3iv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 641, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_234glUniform3iv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_234glUniform3iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform3iv", 0);

  /* "renpy/gl2/uguu.pyx":642
 * 
 * def glUniform3iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":643
 * def glUniform3iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform4f(location, v0, v1, v2, v3):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 643, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 643, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3iv(__pyx_t_2, __pyx_t_3, ((GLint const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":641
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 * def glUniform3iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform3iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":645
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform4f(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_237glUniform4f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_237glUniform4f = {"glUniform4f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_237glUniform4f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_237glUniform4f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_v_v1 = 0;
  PyObject *__pyx_v_v2 = 0;
  PyObject *__pyx_v_v3 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform4f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,&__pyx_n_s_v1,&__pyx_n_s_v2,&__pyx_n_s_v3,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4f", 1, 5, 5, 1); __PYX_ERR(1, 645, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4f", 1, 5, 5, 2); __PYX_ERR(1, 645, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4f", 1, 5, 5, 3); __PYX_ERR(1, 645, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v3)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4f", 1, 5, 5, 4); __PYX_ERR(1, 645, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform4f") < 0)) __PYX_ERR(1, 645, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
    __pyx_v_v1 = values[2];
    __pyx_v_v2 = values[3];
    __pyx_v_v3 = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform4f", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 645, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_236glUniform4f(__pyx_self, __pyx_v_location, __pyx_v_v0, __pyx_v_v1, __pyx_v_v2, __pyx_v_v3);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_236glUniform4f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2, PyObject *__pyx_v_v3) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  GLfloat __pyx_t_4;
  GLfloat __pyx_t_5;
  __Pyx_RefNannySetupContext("glUniform4f", 0);

  /* "renpy/gl2/uguu.pyx":646
 * 
 * def glUniform4f(location, v0, v1, v2, v3):
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)             # <<<<<<<<<<<<<<
 * 
 * def glUniform4fv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 646, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 646, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_v1); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 646, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_v2); if (unlikely((__pyx_t_4 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 646, __pyx_L1_error)
  __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_v_v3); if (unlikely((__pyx_t_5 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 646, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4f(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

  /* "renpy/gl2/uguu.pyx":645
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform4f(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":648
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 * def glUniform4fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_239glUniform4fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_239glUniform4fv = {"glUniform4fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_239glUniform4fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_239glUniform4fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform4fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4fv", 1, 3, 3, 1); __PYX_ERR(1, 648, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4fv", 1, 3, 3, 2); __PYX_ERR(1, 648, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform4fv") < 0)) __PYX_ERR(1, 648, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform4fv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 648, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_238glUniform4fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_238glUniform4fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform4fv", 0);

  /* "renpy/gl2/uguu.pyx":649
 * 
 * def glUniform4fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":650
 * def glUniform4fv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniform4i(location, v0, v1, v2, v3):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 650, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 650, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4fv(__pyx_t_2, __pyx_t_3, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":648
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 * def glUniform4fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":652
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform4i(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_241glUniform4i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_241glUniform4i = {"glUniform4i", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_241glUniform4i, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_241glUniform4i(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_v0 = 0;
  PyObject *__pyx_v_v1 = 0;
  PyObject *__pyx_v_v2 = 0;
  PyObject *__pyx_v_v3 = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform4i (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_v0,&__pyx_n_s_v1,&__pyx_n_s_v2,&__pyx_n_s_v3,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v0)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4i", 1, 5, 5, 1); __PYX_ERR(1, 652, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4i", 1, 5, 5, 2); __PYX_ERR(1, 652, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4i", 1, 5, 5, 3); __PYX_ERR(1, 652, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v3)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4i", 1, 5, 5, 4); __PYX_ERR(1, 652, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform4i") < 0)) __PYX_ERR(1, 652, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_location = values[0];
    __pyx_v_v0 = values[1];
    __pyx_v_v1 = values[2];
    __pyx_v_v2 = values[3];
    __pyx_v_v3 = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform4i", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 652, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_240glUniform4i(__pyx_self, __pyx_v_location, __pyx_v_v0, __pyx_v_v1, __pyx_v_v2, __pyx_v_v3);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_240glUniform4i(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_v0, PyObject *__pyx_v_v1, PyObject *__pyx_v_v2, PyObject *__pyx_v_v3) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLint __pyx_t_2;
  GLint __pyx_t_3;
  GLint __pyx_t_4;
  GLint __pyx_t_5;
  __Pyx_RefNannySetupContext("glUniform4i", 0);

  /* "renpy/gl2/uguu.pyx":653
 * 
 * def glUniform4i(location, v0, v1, v2, v3):
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)             # <<<<<<<<<<<<<<
 * 
 * def glUniform4iv(location, count, value):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 653, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_v0); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 653, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_v1); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 653, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLint(__pyx_v_v2); if (unlikely((__pyx_t_4 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 653, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLint(__pyx_v_v3); if (unlikely((__pyx_t_5 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 653, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4i(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

  /* "renpy/gl2/uguu.pyx":652
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform4i(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4i", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":655
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 * def glUniform4iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_243glUniform4iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_243glUniform4iv = {"glUniform4iv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_243glUniform4iv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_243glUniform4iv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniform4iv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4iv", 1, 3, 3, 1); __PYX_ERR(1, 655, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniform4iv", 1, 3, 3, 2); __PYX_ERR(1, 655, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniform4iv") < 0)) __PYX_ERR(1, 655, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_value = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniform4iv", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 655, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_242glUniform4iv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_242glUniform4iv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  __Pyx_RefNannySetupContext("glUniform4iv", 0);

  /* "renpy/gl2/uguu.pyx":656
 * 
 * def glUniform4iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":657
 * def glUniform4iv(location, count, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniformMatrix2fv(location, count, transpose, value):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 657, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 657, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4iv(__pyx_t_2, __pyx_t_3, ((GLint const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":655
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 * def glUniform4iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniform4iv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":659
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniformMatrix2fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_245glUniformMatrix2fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_245glUniformMatrix2fv = {"glUniformMatrix2fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_245glUniformMatrix2fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_245glUniformMatrix2fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_transpose = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniformMatrix2fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_transpose,&__pyx_n_s_value,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix2fv", 1, 4, 4, 1); __PYX_ERR(1, 659, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transpose)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix2fv", 1, 4, 4, 2); __PYX_ERR(1, 659, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix2fv", 1, 4, 4, 3); __PYX_ERR(1, 659, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniformMatrix2fv") < 0)) __PYX_ERR(1, 659, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_transpose = values[2];
    __pyx_v_value = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniformMatrix2fv", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 659, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniformMatrix2fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_244glUniformMatrix2fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_transpose, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_244glUniformMatrix2fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_transpose, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  GLboolean __pyx_t_4;
  __Pyx_RefNannySetupContext("glUniformMatrix2fv", 0);

  /* "renpy/gl2/uguu.pyx":660
 * 
 * def glUniformMatrix2fv(location, count, transpose, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":661
 * def glUniformMatrix2fv(location, count, transpose, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniformMatrix3fv(location, count, transpose, value):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 661, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 661, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLboolean(__pyx_v_transpose); if (unlikely((__pyx_t_4 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 661, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix2fv(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":659
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniformMatrix2fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniformMatrix2fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":663
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix3fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_247glUniformMatrix3fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_247glUniformMatrix3fv = {"glUniformMatrix3fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_247glUniformMatrix3fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_247glUniformMatrix3fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_transpose = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniformMatrix3fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_transpose,&__pyx_n_s_value,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix3fv", 1, 4, 4, 1); __PYX_ERR(1, 663, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transpose)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix3fv", 1, 4, 4, 2); __PYX_ERR(1, 663, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix3fv", 1, 4, 4, 3); __PYX_ERR(1, 663, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniformMatrix3fv") < 0)) __PYX_ERR(1, 663, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_transpose = values[2];
    __pyx_v_value = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniformMatrix3fv", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 663, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniformMatrix3fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_246glUniformMatrix3fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_transpose, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_246glUniformMatrix3fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_transpose, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  GLboolean __pyx_t_4;
  __Pyx_RefNannySetupContext("glUniformMatrix3fv", 0);

  /* "renpy/gl2/uguu.pyx":664
 * 
 * def glUniformMatrix3fv(location, count, transpose, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":665
 * def glUniformMatrix3fv(location, count, transpose, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUniformMatrix4fv(location, count, transpose, value):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 665, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 665, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLboolean(__pyx_v_transpose); if (unlikely((__pyx_t_4 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 665, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix3fv(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":663
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix3fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniformMatrix3fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":667
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix4fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_249glUniformMatrix4fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_249glUniformMatrix4fv = {"glUniformMatrix4fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_249glUniformMatrix4fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_249glUniformMatrix4fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_location = 0;
  PyObject *__pyx_v_count = 0;
  PyObject *__pyx_v_transpose = 0;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUniformMatrix4fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_location,&__pyx_n_s_count,&__pyx_n_s_transpose,&__pyx_n_s_value,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_location)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix4fv", 1, 4, 4, 1); __PYX_ERR(1, 667, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transpose)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix4fv", 1, 4, 4, 2); __PYX_ERR(1, 667, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glUniformMatrix4fv", 1, 4, 4, 3); __PYX_ERR(1, 667, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glUniformMatrix4fv") < 0)) __PYX_ERR(1, 667, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_location = values[0];
    __pyx_v_count = values[1];
    __pyx_v_transpose = values[2];
    __pyx_v_value = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glUniformMatrix4fv", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 667, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniformMatrix4fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_248glUniformMatrix4fv(__pyx_self, __pyx_v_location, __pyx_v_count, __pyx_v_transpose, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_248glUniformMatrix4fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_location, PyObject *__pyx_v_count, PyObject *__pyx_v_transpose, PyObject *__pyx_v_value) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_value_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  GLboolean __pyx_t_4;
  __Pyx_RefNannySetupContext("glUniformMatrix4fv", 0);

  /* "renpy/gl2/uguu.pyx":668
 * 
 * def glUniformMatrix4fv(location, count, transpose, value):
 *     cdef ptr value_ptr = get_ptr(value)             # <<<<<<<<<<<<<<
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_value)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_value_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":669
 * def glUniformMatrix4fv(location, count, transpose, value):
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glUseProgram(program):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_location); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 669, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_count); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 669, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLboolean(__pyx_v_transpose); if (unlikely((__pyx_t_4 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 669, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix4fv(__pyx_t_2, __pyx_t_3, __pyx_t_4, ((GLfloat const *)__pyx_v_value_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":667
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix4fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glUniformMatrix4fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":671
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUseProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glUseProgram(program)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_251glUseProgram(PyObject *__pyx_self, PyObject *__pyx_v_program); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_251glUseProgram = {"glUseProgram", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_251glUseProgram, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_251glUseProgram(PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glUseProgram (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_250glUseProgram(__pyx_self, ((PyObject *)__pyx_v_program));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_250glUseProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glUseProgram", 0);

  /* "renpy/gl2/uguu.pyx":672
 * 
 * def glUseProgram(program):
 *     uguugl.glUseProgram(program)             # <<<<<<<<<<<<<<
 * 
 * def glValidateProgram(program):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 672, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glUseProgram(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":671
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUseProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glUseProgram(program)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glUseProgram", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":674
 *     uguugl.glUseProgram(program)
 * 
 * def glValidateProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glValidateProgram(program)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_253glValidateProgram(PyObject *__pyx_self, PyObject *__pyx_v_program); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_253glValidateProgram = {"glValidateProgram", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_253glValidateProgram, METH_O, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_253glValidateProgram(PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glValidateProgram (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_252glValidateProgram(__pyx_self, ((PyObject *)__pyx_v_program));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_252glValidateProgram(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  __Pyx_RefNannySetupContext("glValidateProgram", 0);

  /* "renpy/gl2/uguu.pyx":675
 * 
 * def glValidateProgram(program):
 *     uguugl.glValidateProgram(program)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib1f(index, x):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_program); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 675, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glValidateProgram(__pyx_t_1);

  /* "renpy/gl2/uguu.pyx":674
 *     uguugl.glUseProgram(program)
 * 
 * def glValidateProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glValidateProgram(program)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glValidateProgram", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":677
 *     uguugl.glValidateProgram(program)
 * 
 * def glVertexAttrib1f(index, x):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_255glVertexAttrib1f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_255glVertexAttrib1f = {"glVertexAttrib1f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_255glVertexAttrib1f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_255glVertexAttrib1f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib1f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_x,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib1f", 1, 2, 2, 1); __PYX_ERR(1, 677, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib1f") < 0)) __PYX_ERR(1, 677, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_x = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib1f", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 677, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib1f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_254glVertexAttrib1f(__pyx_self, __pyx_v_index, __pyx_v_x);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_254glVertexAttrib1f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  GLfloat __pyx_t_2;
  __Pyx_RefNannySetupContext("glVertexAttrib1f", 0);

  /* "renpy/gl2/uguu.pyx":678
 * 
 * def glVertexAttrib1f(index, x):
 *     uguugl.glVertexAttrib1f(index, x)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib1fv(index, v):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 678, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_x); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 678, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1f(__pyx_t_1, __pyx_t_2);

  /* "renpy/gl2/uguu.pyx":677
 *     uguugl.glValidateProgram(program)
 * 
 * def glVertexAttrib1f(index, x):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib1f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":680
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 * def glVertexAttrib1fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_257glVertexAttrib1fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_257glVertexAttrib1fv = {"glVertexAttrib1fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_257glVertexAttrib1fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_257glVertexAttrib1fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib1fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_v,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib1fv", 1, 2, 2, 1); __PYX_ERR(1, 680, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib1fv") < 0)) __PYX_ERR(1, 680, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_v = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib1fv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 680, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib1fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_256glVertexAttrib1fv(__pyx_self, __pyx_v_index, __pyx_v_v);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_256glVertexAttrib1fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_v_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glVertexAttrib1fv", 0);

  /* "renpy/gl2/uguu.pyx":681
 * 
 * def glVertexAttrib1fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":682
 * def glVertexAttrib1fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib2f(index, x, y):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 682, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1fv(__pyx_t_2, ((GLfloat const *)__pyx_v_v_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":680
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 * def glVertexAttrib1fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib1fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":684
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib2f(index, x, y):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_259glVertexAttrib2f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_259glVertexAttrib2f = {"glVertexAttrib2f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_259glVertexAttrib2f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_259glVertexAttrib2f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib2f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib2f", 1, 3, 3, 1); __PYX_ERR(1, 684, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib2f", 1, 3, 3, 2); __PYX_ERR(1, 684, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib2f") < 0)) __PYX_ERR(1, 684, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_index = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib2f", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 684, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib2f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_258glVertexAttrib2f(__pyx_self, __pyx_v_index, __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_258glVertexAttrib2f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x, PyObject *__pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  __Pyx_RefNannySetupContext("glVertexAttrib2f", 0);

  /* "renpy/gl2/uguu.pyx":685
 * 
 * def glVertexAttrib2f(index, x, y):
 *     uguugl.glVertexAttrib2f(index, x, y)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib2fv(index, v):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 685, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_x); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 685, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_y); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 685, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2f(__pyx_t_1, __pyx_t_2, __pyx_t_3);

  /* "renpy/gl2/uguu.pyx":684
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib2f(index, x, y):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib2f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":687
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 * def glVertexAttrib2fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_261glVertexAttrib2fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_261glVertexAttrib2fv = {"glVertexAttrib2fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_261glVertexAttrib2fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_261glVertexAttrib2fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib2fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_v,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib2fv", 1, 2, 2, 1); __PYX_ERR(1, 687, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib2fv") < 0)) __PYX_ERR(1, 687, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_v = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib2fv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 687, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib2fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_260glVertexAttrib2fv(__pyx_self, __pyx_v_index, __pyx_v_v);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_260glVertexAttrib2fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_v_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glVertexAttrib2fv", 0);

  /* "renpy/gl2/uguu.pyx":688
 * 
 * def glVertexAttrib2fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":689
 * def glVertexAttrib2fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib3f(index, x, y, z):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 689, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2fv(__pyx_t_2, ((GLfloat const *)__pyx_v_v_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":687
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 * def glVertexAttrib2fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib2fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":691
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib3f(index, x, y, z):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_263glVertexAttrib3f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_263glVertexAttrib3f = {"glVertexAttrib3f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_263glVertexAttrib3f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_263glVertexAttrib3f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_z = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib3f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_z,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib3f", 1, 4, 4, 1); __PYX_ERR(1, 691, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib3f", 1, 4, 4, 2); __PYX_ERR(1, 691, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_z)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib3f", 1, 4, 4, 3); __PYX_ERR(1, 691, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib3f") < 0)) __PYX_ERR(1, 691, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_index = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
    __pyx_v_z = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib3f", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 691, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib3f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_262glVertexAttrib3f(__pyx_self, __pyx_v_index, __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_262glVertexAttrib3f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_z) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  GLfloat __pyx_t_4;
  __Pyx_RefNannySetupContext("glVertexAttrib3f", 0);

  /* "renpy/gl2/uguu.pyx":692
 * 
 * def glVertexAttrib3f(index, x, y, z):
 *     uguugl.glVertexAttrib3f(index, x, y, z)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib3fv(index, v):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 692, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_x); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 692, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_y); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 692, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_z); if (unlikely((__pyx_t_4 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 692, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3f(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":691
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib3f(index, x, y, z):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib3f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":694
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 * def glVertexAttrib3fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_265glVertexAttrib3fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_265glVertexAttrib3fv = {"glVertexAttrib3fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_265glVertexAttrib3fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_265glVertexAttrib3fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib3fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_v,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib3fv", 1, 2, 2, 1); __PYX_ERR(1, 694, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib3fv") < 0)) __PYX_ERR(1, 694, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_v = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib3fv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 694, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib3fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_264glVertexAttrib3fv(__pyx_self, __pyx_v_index, __pyx_v_v);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_264glVertexAttrib3fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_v_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glVertexAttrib3fv", 0);

  /* "renpy/gl2/uguu.pyx":695
 * 
 * def glVertexAttrib3fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":696
 * def glVertexAttrib3fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib4f(index, x, y, z, w):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 696, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3fv(__pyx_t_2, ((GLfloat const *)__pyx_v_v_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":694
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 * def glVertexAttrib3fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib3fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":698
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib4f(index, x, y, z, w):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_267glVertexAttrib4f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_267glVertexAttrib4f = {"glVertexAttrib4f", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_267glVertexAttrib4f, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_267glVertexAttrib4f(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_z = 0;
  PyObject *__pyx_v_w = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib4f (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_z,&__pyx_n_s_w,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib4f", 1, 5, 5, 1); __PYX_ERR(1, 698, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib4f", 1, 5, 5, 2); __PYX_ERR(1, 698, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_z)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib4f", 1, 5, 5, 3); __PYX_ERR(1, 698, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib4f", 1, 5, 5, 4); __PYX_ERR(1, 698, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib4f") < 0)) __PYX_ERR(1, 698, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_index = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
    __pyx_v_z = values[3];
    __pyx_v_w = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib4f", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 698, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib4f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_266glVertexAttrib4f(__pyx_self, __pyx_v_index, __pyx_v_x, __pyx_v_y, __pyx_v_z, __pyx_v_w);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_266glVertexAttrib4f(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_z, PyObject *__pyx_v_w) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLuint __pyx_t_1;
  GLfloat __pyx_t_2;
  GLfloat __pyx_t_3;
  GLfloat __pyx_t_4;
  GLfloat __pyx_t_5;
  __Pyx_RefNannySetupContext("glVertexAttrib4f", 0);

  /* "renpy/gl2/uguu.pyx":699
 * 
 * def glVertexAttrib4f(index, x, y, z, w):
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttrib4fv(index, v):
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_1 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 699, __pyx_L1_error)
  __pyx_t_2 = __pyx_PyFloat_AsFloat(__pyx_v_x); if (unlikely((__pyx_t_2 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 699, __pyx_L1_error)
  __pyx_t_3 = __pyx_PyFloat_AsFloat(__pyx_v_y); if (unlikely((__pyx_t_3 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 699, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_z); if (unlikely((__pyx_t_4 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 699, __pyx_L1_error)
  __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_v_w); if (unlikely((__pyx_t_5 == ((GLfloat)-1)) && PyErr_Occurred())) __PYX_ERR(1, 699, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4f(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5);

  /* "renpy/gl2/uguu.pyx":698
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib4f(index, x, y, z, w):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib4f", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":701
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 * def glVertexAttrib4fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_269glVertexAttrib4fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_269glVertexAttrib4fv = {"glVertexAttrib4fv", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_269glVertexAttrib4fv, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_269glVertexAttrib4fv(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_v = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttrib4fv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_v,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_v)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttrib4fv", 1, 2, 2, 1); __PYX_ERR(1, 701, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttrib4fv") < 0)) __PYX_ERR(1, 701, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_index = values[0];
    __pyx_v_v = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttrib4fv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 701, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib4fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_268glVertexAttrib4fv(__pyx_self, __pyx_v_index, __pyx_v_v);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_268glVertexAttrib4fv(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_v) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_v_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  __Pyx_RefNannySetupContext("glVertexAttrib4fv", 0);

  /* "renpy/gl2/uguu.pyx":702
 * 
 * def glVertexAttrib4fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_v)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 702, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":703
 * def glVertexAttrib4fv(index, v):
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 703, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4fv(__pyx_t_2, ((GLfloat const *)__pyx_v_v_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":701
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 * def glVertexAttrib4fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttrib4fv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_v_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":705
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_271glVertexAttribPointer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_271glVertexAttribPointer = {"glVertexAttribPointer", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_271glVertexAttribPointer, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_271glVertexAttribPointer(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_v_normalized = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_pointer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glVertexAttribPointer (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_index,&__pyx_n_s_size,&__pyx_n_s_type,&__pyx_n_s_normalized,&__pyx_n_s_stride,&__pyx_n_s_pointer,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttribPointer", 1, 6, 6, 1); __PYX_ERR(1, 705, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttribPointer", 1, 6, 6, 2); __PYX_ERR(1, 705, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_normalized)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttribPointer", 1, 6, 6, 3); __PYX_ERR(1, 705, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttribPointer", 1, 6, 6, 4); __PYX_ERR(1, 705, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pointer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glVertexAttribPointer", 1, 6, 6, 5); __PYX_ERR(1, 705, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glVertexAttribPointer") < 0)) __PYX_ERR(1, 705, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_index = values[0];
    __pyx_v_size = values[1];
    __pyx_v_type = values[2];
    __pyx_v_normalized = values[3];
    __pyx_v_stride = values[4];
    __pyx_v_pointer = values[5];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glVertexAttribPointer", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 705, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttribPointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_270glVertexAttribPointer(__pyx_self, __pyx_v_index, __pyx_v_size, __pyx_v_type, __pyx_v_normalized, __pyx_v_stride, __pyx_v_pointer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_270glVertexAttribPointer(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_index, PyObject *__pyx_v_size, PyObject *__pyx_v_type, PyObject *__pyx_v_normalized, PyObject *__pyx_v_stride, PyObject *__pyx_v_pointer) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *__pyx_v_pointer_ptr = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLuint __pyx_t_2;
  GLint __pyx_t_3;
  GLenum __pyx_t_4;
  GLboolean __pyx_t_5;
  GLsizei __pyx_t_6;
  __Pyx_RefNannySetupContext("glVertexAttribPointer", 0);

  /* "renpy/gl2/uguu.pyx":706
 * 
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):
 *     cdef ptr pointer_ptr = get_ptr(pointer)             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_4uguu_get_ptr(__pyx_v_pointer)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 706, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pointer_ptr = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/uguu.pyx":707
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)             # <<<<<<<<<<<<<<
 * 
 * def glViewport(x, y, width, height):
 */
  __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_index); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 707, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLint(__pyx_v_size); if (unlikely((__pyx_t_3 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 707, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_4 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(1, 707, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_GLboolean(__pyx_v_normalized); if (unlikely((__pyx_t_5 == ((GLboolean)-1)) && PyErr_Occurred())) __PYX_ERR(1, 707, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_GLsizei(__pyx_v_stride); if (unlikely((__pyx_t_6 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 707, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttribPointer(__pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, ((void const *)__pyx_v_pointer_ptr->ptr));

  /* "renpy/gl2/uguu.pyx":705
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguu.glVertexAttribPointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_pointer_ptr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguu.pyx":709
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 * 
 * def glViewport(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glViewport(x, y, width, height)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_273glViewport(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_4uguu_273glViewport = {"glViewport", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_273glViewport, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_4uguu_273glViewport(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glViewport (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glViewport", 1, 4, 4, 1); __PYX_ERR(1, 709, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glViewport", 1, 4, 4, 2); __PYX_ERR(1, 709, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("glViewport", 1, 4, 4, 3); __PYX_ERR(1, 709, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "glViewport") < 0)) __PYX_ERR(1, 709, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
    __pyx_v_width = values[2];
    __pyx_v_height = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("glViewport", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 709, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glViewport", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_4uguu_272glViewport(__pyx_self, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_4uguu_272glViewport(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_width, PyObject *__pyx_v_height) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  GLint __pyx_t_1;
  GLint __pyx_t_2;
  GLsizei __pyx_t_3;
  GLsizei __pyx_t_4;
  __Pyx_RefNannySetupContext("glViewport", 0);

  /* "renpy/gl2/uguu.pyx":710
 * 
 * def glViewport(x, y, width, height):
 *     uguugl.glViewport(x, y, width, height)             # <<<<<<<<<<<<<<
 * 
 * GL_FALSE = uguugl.GL_FALSE
 */
  __pyx_t_1 = __Pyx_PyInt_As_GLint(__pyx_v_x); if (unlikely((__pyx_t_1 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 710, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_v_y); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 710, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 710, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(1, 710, __pyx_L1_error)
  __pyx_v_5renpy_3gl2_6uguugl_glViewport(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4);

  /* "renpy/gl2/uguu.pyx":709
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 * 
 * def glViewport(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glViewport(x, y, width, height)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.uguu.glViewport", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_ptr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_4uguu_ptr *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_4uguu_ptr *)o);
  p->view.obj = NULL;
  return o;
}

static void __pyx_tp_dealloc_5renpy_3gl2_4uguu_ptr(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5renpy_3gl2_4uguu_3ptr_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5renpy_3gl2_4uguu_ptr[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_3ptr_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_3ptr_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_3gl2_4uguu_ptr = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.uguu.ptr", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_4uguu_ptr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_ptr, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    This is a class that wraps a generic contiguous Python buffer, and\n    allows the retrieval of a pointer to that buffer.\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_4uguu_ptr, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_4uguu_3ptr_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_4uguu_ptr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer __pyx_vtable_5renpy_3gl2_4uguu_Buffer;

static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_Buffer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *)o);
  p->__pyx_vtab = __pyx_vtabptr_5renpy_3gl2_4uguu_Buffer;
  return o;
}

static void __pyx_tp_dealloc_5renpy_3gl2_4uguu_Buffer(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5renpy_3gl2_4uguu_6Buffer_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5renpy_3gl2_4uguu_Buffer[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_6Buffer_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_6Buffer_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyBufferProcs __pyx_tp_as_buffer_Buffer = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_pw_5renpy_3gl2_4uguu_6Buffer_1__getbuffer__, /*bf_getbuffer*/
  __pyx_pw_5renpy_3gl2_4uguu_6Buffer_3__releasebuffer__, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type_5renpy_3gl2_4uguu_Buffer = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.uguu.Buffer", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_Buffer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_Buffer, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    The base class for all buffers.\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_4uguu_Buffer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_4uguu_Buffer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesBuffer __pyx_vtable_5renpy_3gl2_4uguu_BytesBuffer;

static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_BytesBuffer(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *p;
  PyObject *o = __pyx_tp_new_5renpy_3gl2_4uguu_Buffer(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer*)__pyx_vtabptr_5renpy_3gl2_4uguu_BytesBuffer;
  return o;
}

static PyMethodDef __pyx_methods_5renpy_3gl2_4uguu_BytesBuffer[] = {
  {"get", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_3get, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_3gl2_4uguu_BytesBuffer = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.uguu.BytesBuffer", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_4uguu_BytesBuffer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_Buffer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_4uguu_BytesBuffer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_4uguu_11BytesBuffer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_4uguu_BytesBuffer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_BytesListBuffer __pyx_vtable_5renpy_3gl2_4uguu_BytesListBuffer;

static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_BytesListBuffer(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *p;
  PyObject *o = __pyx_tp_new_5renpy_3gl2_4uguu_Buffer(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer*)__pyx_vtabptr_5renpy_3gl2_4uguu_BytesListBuffer;
  p->value = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5renpy_3gl2_4uguu_BytesListBuffer(PyObject *o) {
  struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *p = (struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->value);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_Buffer(o);
}

static int __pyx_tp_traverse_5renpy_3gl2_4uguu_BytesListBuffer(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *p = (struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)o;
  e = ((likely(__pyx_ptype_5renpy_3gl2_4uguu_Buffer)) ? ((__pyx_ptype_5renpy_3gl2_4uguu_Buffer->tp_traverse) ? __pyx_ptype_5renpy_3gl2_4uguu_Buffer->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5renpy_3gl2_4uguu_BytesListBuffer)); if (e) return e;
  if (p->value) {
    e = (*v)(p->value, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_3gl2_4uguu_BytesListBuffer(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *p = (struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer *)o;
  if (likely(__pyx_ptype_5renpy_3gl2_4uguu_Buffer)) { if (__pyx_ptype_5renpy_3gl2_4uguu_Buffer->tp_clear) __pyx_ptype_5renpy_3gl2_4uguu_Buffer->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5renpy_3gl2_4uguu_BytesListBuffer);
  tmp = ((PyObject*)p->value);
  p->value = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5renpy_3gl2_4uguu_BytesListBuffer[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_3gl2_4uguu_BytesListBuffer = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.uguu.BytesListBuffer", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_4uguu_BytesListBuffer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_BytesListBuffer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5renpy_3gl2_4uguu_BytesListBuffer, /*tp_traverse*/
  __pyx_tp_clear_5renpy_3gl2_4uguu_BytesListBuffer, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_4uguu_BytesListBuffer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_4uguu_15BytesListBuffer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_4uguu_BytesListBuffer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_IntBuffer __pyx_vtable_5renpy_3gl2_4uguu_IntBuffer;

static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_IntBuffer(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *p;
  PyObject *o = __pyx_tp_new_5renpy_3gl2_4uguu_Buffer(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer*)__pyx_vtabptr_5renpy_3gl2_4uguu_IntBuffer;
  return o;
}
static PyObject *__pyx_sq_item_5renpy_3gl2_4uguu_IntBuffer(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyMethodDef __pyx_methods_5renpy_3gl2_4uguu_IntBuffer[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_IntBuffer = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_5renpy_3gl2_4uguu_IntBuffer, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_IntBuffer = {
  0, /*mp_length*/
  __pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_3__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_5renpy_3gl2_4uguu_IntBuffer = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.uguu.IntBuffer", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_4uguu_IntBuffer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_Buffer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_IntBuffer, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_IntBuffer, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_4uguu_IntBuffer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_4uguu_9IntBuffer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_4uguu_IntBuffer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_3gl2_4uguu_FloatBuffer __pyx_vtable_5renpy_3gl2_4uguu_FloatBuffer;

static PyObject *__pyx_tp_new_5renpy_3gl2_4uguu_FloatBuffer(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *p;
  PyObject *o = __pyx_tp_new_5renpy_3gl2_4uguu_Buffer(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5renpy_3gl2_4uguu_Buffer*)__pyx_vtabptr_5renpy_3gl2_4uguu_FloatBuffer;
  return o;
}
static PyObject *__pyx_sq_item_5renpy_3gl2_4uguu_FloatBuffer(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyMethodDef __pyx_methods_5renpy_3gl2_4uguu_FloatBuffer[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_FloatBuffer = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_5renpy_3gl2_4uguu_FloatBuffer, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_FloatBuffer = {
  0, /*mp_length*/
  __pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_3__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_5renpy_3gl2_4uguu_FloatBuffer = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.uguu.FloatBuffer", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_4uguu_FloatBuffer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_4uguu_Buffer, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_FloatBuffer, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_FloatBuffer, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_4uguu_FloatBuffer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_4uguu_11FloatBuffer_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_4uguu_FloatBuffer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_uguu(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_uguu},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "uguu",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_GL_ACTIVE_ATTRIBUTES, __pyx_k_GL_ACTIVE_ATTRIBUTES, sizeof(__pyx_k_GL_ACTIVE_ATTRIBUTES), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH, __pyx_k_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH, sizeof(__pyx_k_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACTIVE_TEXTURE, __pyx_k_GL_ACTIVE_TEXTURE, sizeof(__pyx_k_GL_ACTIVE_TEXTURE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACTIVE_UNIFORMS, __pyx_k_GL_ACTIVE_UNIFORMS, sizeof(__pyx_k_GL_ACTIVE_UNIFORMS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ACTIVE_UNIFORM_MAX_LENGTH, __pyx_k_GL_ACTIVE_UNIFORM_MAX_LENGTH, sizeof(__pyx_k_GL_ACTIVE_UNIFORM_MAX_LENGTH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ALIASED_LINE_WIDTH_RANGE, __pyx_k_GL_ALIASED_LINE_WIDTH_RANGE, sizeof(__pyx_k_GL_ALIASED_LINE_WIDTH_RANGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ALIASED_POINT_SIZE_RANGE, __pyx_k_GL_ALIASED_POINT_SIZE_RANGE, sizeof(__pyx_k_GL_ALIASED_POINT_SIZE_RANGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ALPHA, __pyx_k_GL_ALPHA, sizeof(__pyx_k_GL_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ALPHA_BITS, __pyx_k_GL_ALPHA_BITS, sizeof(__pyx_k_GL_ALPHA_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ALWAYS, __pyx_k_GL_ALWAYS, sizeof(__pyx_k_GL_ALWAYS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ARRAY_BUFFER, __pyx_k_GL_ARRAY_BUFFER, sizeof(__pyx_k_GL_ARRAY_BUFFER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ARRAY_BUFFER_BINDING, __pyx_k_GL_ARRAY_BUFFER_BINDING, sizeof(__pyx_k_GL_ARRAY_BUFFER_BINDING), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ATTACHED_SHADERS, __pyx_k_GL_ATTACHED_SHADERS, sizeof(__pyx_k_GL_ATTACHED_SHADERS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BACK, __pyx_k_GL_BACK, sizeof(__pyx_k_GL_BACK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND, __pyx_k_GL_BLEND, sizeof(__pyx_k_GL_BLEND), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_COLOR, __pyx_k_GL_BLEND_COLOR, sizeof(__pyx_k_GL_BLEND_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_DST_ALPHA, __pyx_k_GL_BLEND_DST_ALPHA, sizeof(__pyx_k_GL_BLEND_DST_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_DST_RGB, __pyx_k_GL_BLEND_DST_RGB, sizeof(__pyx_k_GL_BLEND_DST_RGB), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_EQUATION, __pyx_k_GL_BLEND_EQUATION, sizeof(__pyx_k_GL_BLEND_EQUATION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_EQUATION_ALPHA, __pyx_k_GL_BLEND_EQUATION_ALPHA, sizeof(__pyx_k_GL_BLEND_EQUATION_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_EQUATION_RGB, __pyx_k_GL_BLEND_EQUATION_RGB, sizeof(__pyx_k_GL_BLEND_EQUATION_RGB), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_SRC_ALPHA, __pyx_k_GL_BLEND_SRC_ALPHA, sizeof(__pyx_k_GL_BLEND_SRC_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLEND_SRC_RGB, __pyx_k_GL_BLEND_SRC_RGB, sizeof(__pyx_k_GL_BLEND_SRC_RGB), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BLUE_BITS, __pyx_k_GL_BLUE_BITS, sizeof(__pyx_k_GL_BLUE_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BOOL, __pyx_k_GL_BOOL, sizeof(__pyx_k_GL_BOOL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BOOL_VEC2, __pyx_k_GL_BOOL_VEC2, sizeof(__pyx_k_GL_BOOL_VEC2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BOOL_VEC3, __pyx_k_GL_BOOL_VEC3, sizeof(__pyx_k_GL_BOOL_VEC3), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BOOL_VEC4, __pyx_k_GL_BOOL_VEC4, sizeof(__pyx_k_GL_BOOL_VEC4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BUFFER_SIZE, __pyx_k_GL_BUFFER_SIZE, sizeof(__pyx_k_GL_BUFFER_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BUFFER_USAGE, __pyx_k_GL_BUFFER_USAGE, sizeof(__pyx_k_GL_BUFFER_USAGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_BYTE, __pyx_k_GL_BYTE, sizeof(__pyx_k_GL_BYTE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CCW, __pyx_k_GL_CCW, sizeof(__pyx_k_GL_CCW), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CLAMP_TO_EDGE, __pyx_k_GL_CLAMP_TO_EDGE, sizeof(__pyx_k_GL_CLAMP_TO_EDGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_COLOR_ATTACHMENT0, __pyx_k_GL_COLOR_ATTACHMENT0, sizeof(__pyx_k_GL_COLOR_ATTACHMENT0), 0, 0, 1, 1},
  {&__pyx_n_s_GL_COLOR_BUFFER_BIT, __pyx_k_GL_COLOR_BUFFER_BIT, sizeof(__pyx_k_GL_COLOR_BUFFER_BIT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_COLOR_CLEAR_VALUE, __pyx_k_GL_COLOR_CLEAR_VALUE, sizeof(__pyx_k_GL_COLOR_CLEAR_VALUE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_COLOR_WRITEMASK, __pyx_k_GL_COLOR_WRITEMASK, sizeof(__pyx_k_GL_COLOR_WRITEMASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_COMPILE_STATUS, __pyx_k_GL_COMPILE_STATUS, sizeof(__pyx_k_GL_COMPILE_STATUS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_COMPRESSED_TEXTURE_FORMATS, __pyx_k_GL_COMPRESSED_TEXTURE_FORMATS, sizeof(__pyx_k_GL_COMPRESSED_TEXTURE_FORMATS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONSTANT_ALPHA, __pyx_k_GL_CONSTANT_ALPHA, sizeof(__pyx_k_GL_CONSTANT_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CONSTANT_COLOR, __pyx_k_GL_CONSTANT_COLOR, sizeof(__pyx_k_GL_CONSTANT_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CULL_FACE, __pyx_k_GL_CULL_FACE, sizeof(__pyx_k_GL_CULL_FACE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CULL_FACE_MODE, __pyx_k_GL_CULL_FACE_MODE, sizeof(__pyx_k_GL_CULL_FACE_MODE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CURRENT_PROGRAM, __pyx_k_GL_CURRENT_PROGRAM, sizeof(__pyx_k_GL_CURRENT_PROGRAM), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CURRENT_VERTEX_ATTRIB, __pyx_k_GL_CURRENT_VERTEX_ATTRIB, sizeof(__pyx_k_GL_CURRENT_VERTEX_ATTRIB), 0, 0, 1, 1},
  {&__pyx_n_s_GL_CW, __pyx_k_GL_CW, sizeof(__pyx_k_GL_CW), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DECR, __pyx_k_GL_DECR, sizeof(__pyx_k_GL_DECR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DECR_WRAP, __pyx_k_GL_DECR_WRAP, sizeof(__pyx_k_GL_DECR_WRAP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DELETE_STATUS, __pyx_k_GL_DELETE_STATUS, sizeof(__pyx_k_GL_DELETE_STATUS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_ATTACHMENT, __pyx_k_GL_DEPTH_ATTACHMENT, sizeof(__pyx_k_GL_DEPTH_ATTACHMENT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_BITS, __pyx_k_GL_DEPTH_BITS, sizeof(__pyx_k_GL_DEPTH_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_BUFFER_BIT, __pyx_k_GL_DEPTH_BUFFER_BIT, sizeof(__pyx_k_GL_DEPTH_BUFFER_BIT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_CLEAR_VALUE, __pyx_k_GL_DEPTH_CLEAR_VALUE, sizeof(__pyx_k_GL_DEPTH_CLEAR_VALUE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_COMPONENT, __pyx_k_GL_DEPTH_COMPONENT, sizeof(__pyx_k_GL_DEPTH_COMPONENT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_COMPONENT16, __pyx_k_GL_DEPTH_COMPONENT16, sizeof(__pyx_k_GL_DEPTH_COMPONENT16), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_FUNC, __pyx_k_GL_DEPTH_FUNC, sizeof(__pyx_k_GL_DEPTH_FUNC), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_RANGE, __pyx_k_GL_DEPTH_RANGE, sizeof(__pyx_k_GL_DEPTH_RANGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_TEST, __pyx_k_GL_DEPTH_TEST, sizeof(__pyx_k_GL_DEPTH_TEST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DEPTH_WRITEMASK, __pyx_k_GL_DEPTH_WRITEMASK, sizeof(__pyx_k_GL_DEPTH_WRITEMASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DITHER, __pyx_k_GL_DITHER, sizeof(__pyx_k_GL_DITHER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DONT_CARE, __pyx_k_GL_DONT_CARE, sizeof(__pyx_k_GL_DONT_CARE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DST_ALPHA, __pyx_k_GL_DST_ALPHA, sizeof(__pyx_k_GL_DST_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DST_COLOR, __pyx_k_GL_DST_COLOR, sizeof(__pyx_k_GL_DST_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_DYNAMIC_DRAW, __pyx_k_GL_DYNAMIC_DRAW, sizeof(__pyx_k_GL_DYNAMIC_DRAW), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ELEMENT_ARRAY_BUFFER, __pyx_k_GL_ELEMENT_ARRAY_BUFFER, sizeof(__pyx_k_GL_ELEMENT_ARRAY_BUFFER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ELEMENT_ARRAY_BUFFER_BINDING, __pyx_k_GL_ELEMENT_ARRAY_BUFFER_BINDING, sizeof(__pyx_k_GL_ELEMENT_ARRAY_BUFFER_BINDING), 0, 0, 1, 1},
  {&__pyx_n_s_GL_EQUAL, __pyx_k_GL_EQUAL, sizeof(__pyx_k_GL_EQUAL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_EXTENSIONS, __pyx_k_GL_EXTENSIONS, sizeof(__pyx_k_GL_EXTENSIONS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FALSE, __pyx_k_GL_FALSE, sizeof(__pyx_k_GL_FALSE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FASTEST, __pyx_k_GL_FASTEST, sizeof(__pyx_k_GL_FASTEST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT, __pyx_k_GL_FLOAT, sizeof(__pyx_k_GL_FLOAT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT_MAT2, __pyx_k_GL_FLOAT_MAT2, sizeof(__pyx_k_GL_FLOAT_MAT2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT_MAT3, __pyx_k_GL_FLOAT_MAT3, sizeof(__pyx_k_GL_FLOAT_MAT3), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT_MAT4, __pyx_k_GL_FLOAT_MAT4, sizeof(__pyx_k_GL_FLOAT_MAT4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT_VEC2, __pyx_k_GL_FLOAT_VEC2, sizeof(__pyx_k_GL_FLOAT_VEC2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT_VEC3, __pyx_k_GL_FLOAT_VEC3, sizeof(__pyx_k_GL_FLOAT_VEC3), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FLOAT_VEC4, __pyx_k_GL_FLOAT_VEC4, sizeof(__pyx_k_GL_FLOAT_VEC4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAGMENT_SHADER, __pyx_k_GL_FRAGMENT_SHADER, sizeof(__pyx_k_GL_FRAGMENT_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER, __pyx_k_GL_FRAMEBUFFER, sizeof(__pyx_k_GL_FRAMEBUFFER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_OBJECT, __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_OBJECT, sizeof(__pyx_k_GL_FRAMEBUFFER_ATTACHMENT_OBJECT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_OBJECT_2, __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_OBJECT_2, sizeof(__pyx_k_GL_FRAMEBUFFER_ATTACHMENT_OBJECT_2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR, __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR, sizeof(__pyx_k_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR_2, __pyx_k_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR_2, sizeof(__pyx_k_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR_2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_BINDING, __pyx_k_GL_FRAMEBUFFER_BINDING, sizeof(__pyx_k_GL_FRAMEBUFFER_BINDING), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_COMPLETE, __pyx_k_GL_FRAMEBUFFER_COMPLETE, sizeof(__pyx_k_GL_FRAMEBUFFER_COMPLETE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_INCOMPLETE_ATTACH, __pyx_k_GL_FRAMEBUFFER_INCOMPLETE_ATTACH, sizeof(__pyx_k_GL_FRAMEBUFFER_INCOMPLETE_ATTACH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_INCOMPLETE_MISSIN, __pyx_k_GL_FRAMEBUFFER_INCOMPLETE_MISSIN, sizeof(__pyx_k_GL_FRAMEBUFFER_INCOMPLETE_MISSIN), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRAMEBUFFER_UNSUPPORTED, __pyx_k_GL_FRAMEBUFFER_UNSUPPORTED, sizeof(__pyx_k_GL_FRAMEBUFFER_UNSUPPORTED), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRONT, __pyx_k_GL_FRONT, sizeof(__pyx_k_GL_FRONT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRONT_AND_BACK, __pyx_k_GL_FRONT_AND_BACK, sizeof(__pyx_k_GL_FRONT_AND_BACK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FRONT_FACE, __pyx_k_GL_FRONT_FACE, sizeof(__pyx_k_GL_FRONT_FACE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FUNC_ADD, __pyx_k_GL_FUNC_ADD, sizeof(__pyx_k_GL_FUNC_ADD), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FUNC_REVERSE_SUBTRACT, __pyx_k_GL_FUNC_REVERSE_SUBTRACT, sizeof(__pyx_k_GL_FUNC_REVERSE_SUBTRACT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_FUNC_SUBTRACT, __pyx_k_GL_FUNC_SUBTRACT, sizeof(__pyx_k_GL_FUNC_SUBTRACT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_GENERATE_MIPMAP_HINT, __pyx_k_GL_GENERATE_MIPMAP_HINT, sizeof(__pyx_k_GL_GENERATE_MIPMAP_HINT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_GEQUAL, __pyx_k_GL_GEQUAL, sizeof(__pyx_k_GL_GEQUAL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_GREATER, __pyx_k_GL_GREATER, sizeof(__pyx_k_GL_GREATER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_GREEN_BITS, __pyx_k_GL_GREEN_BITS, sizeof(__pyx_k_GL_GREEN_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INCR, __pyx_k_GL_INCR, sizeof(__pyx_k_GL_INCR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INCR_WRAP, __pyx_k_GL_INCR_WRAP, sizeof(__pyx_k_GL_INCR_WRAP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INFO_LOG_LENGTH, __pyx_k_GL_INFO_LOG_LENGTH, sizeof(__pyx_k_GL_INFO_LOG_LENGTH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INT, __pyx_k_GL_INT, sizeof(__pyx_k_GL_INT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INT_VEC2, __pyx_k_GL_INT_VEC2, sizeof(__pyx_k_GL_INT_VEC2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INT_VEC3, __pyx_k_GL_INT_VEC3, sizeof(__pyx_k_GL_INT_VEC3), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INT_VEC4, __pyx_k_GL_INT_VEC4, sizeof(__pyx_k_GL_INT_VEC4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INVALID_ENUM, __pyx_k_GL_INVALID_ENUM, sizeof(__pyx_k_GL_INVALID_ENUM), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INVALID_FRAMEBUFFER_OPERATION, __pyx_k_GL_INVALID_FRAMEBUFFER_OPERATION, sizeof(__pyx_k_GL_INVALID_FRAMEBUFFER_OPERATION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INVALID_OPERATION, __pyx_k_GL_INVALID_OPERATION, sizeof(__pyx_k_GL_INVALID_OPERATION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INVALID_VALUE, __pyx_k_GL_INVALID_VALUE, sizeof(__pyx_k_GL_INVALID_VALUE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_INVERT, __pyx_k_GL_INVERT, sizeof(__pyx_k_GL_INVERT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_KEEP, __pyx_k_GL_KEEP, sizeof(__pyx_k_GL_KEEP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LEQUAL, __pyx_k_GL_LEQUAL, sizeof(__pyx_k_GL_LEQUAL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LESS, __pyx_k_GL_LESS, sizeof(__pyx_k_GL_LESS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINEAR, __pyx_k_GL_LINEAR, sizeof(__pyx_k_GL_LINEAR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINEAR_MIPMAP_LINEAR, __pyx_k_GL_LINEAR_MIPMAP_LINEAR, sizeof(__pyx_k_GL_LINEAR_MIPMAP_LINEAR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINEAR_MIPMAP_NEAREST, __pyx_k_GL_LINEAR_MIPMAP_NEAREST, sizeof(__pyx_k_GL_LINEAR_MIPMAP_NEAREST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINES, __pyx_k_GL_LINES, sizeof(__pyx_k_GL_LINES), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINE_LOOP, __pyx_k_GL_LINE_LOOP, sizeof(__pyx_k_GL_LINE_LOOP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINE_STRIP, __pyx_k_GL_LINE_STRIP, sizeof(__pyx_k_GL_LINE_STRIP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINE_WIDTH, __pyx_k_GL_LINE_WIDTH, sizeof(__pyx_k_GL_LINE_WIDTH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LINK_STATUS, __pyx_k_GL_LINK_STATUS, sizeof(__pyx_k_GL_LINK_STATUS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LUMINANCE, __pyx_k_GL_LUMINANCE, sizeof(__pyx_k_GL_LUMINANCE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_LUMINANCE_ALPHA, __pyx_k_GL_LUMINANCE_ALPHA, sizeof(__pyx_k_GL_LUMINANCE_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_COMBINED_TEXTURE_IMAGE_UN, __pyx_k_GL_MAX_COMBINED_TEXTURE_IMAGE_UN, sizeof(__pyx_k_GL_MAX_COMBINED_TEXTURE_IMAGE_UN), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_CUBE_MAP_TEXTURE_SIZE, __pyx_k_GL_MAX_CUBE_MAP_TEXTURE_SIZE, sizeof(__pyx_k_GL_MAX_CUBE_MAP_TEXTURE_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_RENDERBUFFER_SIZE, __pyx_k_GL_MAX_RENDERBUFFER_SIZE, sizeof(__pyx_k_GL_MAX_RENDERBUFFER_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_TEXTURE_IMAGE_UNITS, __pyx_k_GL_MAX_TEXTURE_IMAGE_UNITS, sizeof(__pyx_k_GL_MAX_TEXTURE_IMAGE_UNITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_TEXTURE_SIZE, __pyx_k_GL_MAX_TEXTURE_SIZE, sizeof(__pyx_k_GL_MAX_TEXTURE_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_VERTEX_ATTRIBS, __pyx_k_GL_MAX_VERTEX_ATTRIBS, sizeof(__pyx_k_GL_MAX_VERTEX_ATTRIBS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_VERTEX_TEXTURE_IMAGE_UNIT, __pyx_k_GL_MAX_VERTEX_TEXTURE_IMAGE_UNIT, sizeof(__pyx_k_GL_MAX_VERTEX_TEXTURE_IMAGE_UNIT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MAX_VIEWPORT_DIMS, __pyx_k_GL_MAX_VIEWPORT_DIMS, sizeof(__pyx_k_GL_MAX_VIEWPORT_DIMS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_MIRRORED_REPEAT, __pyx_k_GL_MIRRORED_REPEAT, sizeof(__pyx_k_GL_MIRRORED_REPEAT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NEAREST, __pyx_k_GL_NEAREST, sizeof(__pyx_k_GL_NEAREST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NEAREST_MIPMAP_LINEAR, __pyx_k_GL_NEAREST_MIPMAP_LINEAR, sizeof(__pyx_k_GL_NEAREST_MIPMAP_LINEAR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NEAREST_MIPMAP_NEAREST, __pyx_k_GL_NEAREST_MIPMAP_NEAREST, sizeof(__pyx_k_GL_NEAREST_MIPMAP_NEAREST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NEVER, __pyx_k_GL_NEVER, sizeof(__pyx_k_GL_NEVER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NICEST, __pyx_k_GL_NICEST, sizeof(__pyx_k_GL_NICEST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NONE, __pyx_k_GL_NONE, sizeof(__pyx_k_GL_NONE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NOTEQUAL, __pyx_k_GL_NOTEQUAL, sizeof(__pyx_k_GL_NOTEQUAL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NO_ERROR, __pyx_k_GL_NO_ERROR, sizeof(__pyx_k_GL_NO_ERROR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_NUM_COMPRESSED_TEXTURE_FORMAT, __pyx_k_GL_NUM_COMPRESSED_TEXTURE_FORMAT, sizeof(__pyx_k_GL_NUM_COMPRESSED_TEXTURE_FORMAT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE, __pyx_k_GL_ONE, sizeof(__pyx_k_GL_ONE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE_MINUS_CONSTANT_ALPHA, __pyx_k_GL_ONE_MINUS_CONSTANT_ALPHA, sizeof(__pyx_k_GL_ONE_MINUS_CONSTANT_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE_MINUS_CONSTANT_COLOR, __pyx_k_GL_ONE_MINUS_CONSTANT_COLOR, sizeof(__pyx_k_GL_ONE_MINUS_CONSTANT_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE_MINUS_DST_ALPHA, __pyx_k_GL_ONE_MINUS_DST_ALPHA, sizeof(__pyx_k_GL_ONE_MINUS_DST_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE_MINUS_DST_COLOR, __pyx_k_GL_ONE_MINUS_DST_COLOR, sizeof(__pyx_k_GL_ONE_MINUS_DST_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE_MINUS_SRC_ALPHA, __pyx_k_GL_ONE_MINUS_SRC_ALPHA, sizeof(__pyx_k_GL_ONE_MINUS_SRC_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ONE_MINUS_SRC_COLOR, __pyx_k_GL_ONE_MINUS_SRC_COLOR, sizeof(__pyx_k_GL_ONE_MINUS_SRC_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_OUT_OF_MEMORY, __pyx_k_GL_OUT_OF_MEMORY, sizeof(__pyx_k_GL_OUT_OF_MEMORY), 0, 0, 1, 1},
  {&__pyx_n_s_GL_PACK_ALIGNMENT, __pyx_k_GL_PACK_ALIGNMENT, sizeof(__pyx_k_GL_PACK_ALIGNMENT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_POINTS, __pyx_k_GL_POINTS, sizeof(__pyx_k_GL_POINTS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_POLYGON_OFFSET_FACTOR, __pyx_k_GL_POLYGON_OFFSET_FACTOR, sizeof(__pyx_k_GL_POLYGON_OFFSET_FACTOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_POLYGON_OFFSET_FILL, __pyx_k_GL_POLYGON_OFFSET_FILL, sizeof(__pyx_k_GL_POLYGON_OFFSET_FILL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_POLYGON_OFFSET_UNITS, __pyx_k_GL_POLYGON_OFFSET_UNITS, sizeof(__pyx_k_GL_POLYGON_OFFSET_UNITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RED_BITS, __pyx_k_GL_RED_BITS, sizeof(__pyx_k_GL_RED_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER, __pyx_k_GL_RENDERBUFFER, sizeof(__pyx_k_GL_RENDERBUFFER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_ALPHA_SIZE, __pyx_k_GL_RENDERBUFFER_ALPHA_SIZE, sizeof(__pyx_k_GL_RENDERBUFFER_ALPHA_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_BINDING, __pyx_k_GL_RENDERBUFFER_BINDING, sizeof(__pyx_k_GL_RENDERBUFFER_BINDING), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_BLUE_SIZE, __pyx_k_GL_RENDERBUFFER_BLUE_SIZE, sizeof(__pyx_k_GL_RENDERBUFFER_BLUE_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_DEPTH_SIZE, __pyx_k_GL_RENDERBUFFER_DEPTH_SIZE, sizeof(__pyx_k_GL_RENDERBUFFER_DEPTH_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_GREEN_SIZE, __pyx_k_GL_RENDERBUFFER_GREEN_SIZE, sizeof(__pyx_k_GL_RENDERBUFFER_GREEN_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_HEIGHT, __pyx_k_GL_RENDERBUFFER_HEIGHT, sizeof(__pyx_k_GL_RENDERBUFFER_HEIGHT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_INTERNAL_FORMAT, __pyx_k_GL_RENDERBUFFER_INTERNAL_FORMAT, sizeof(__pyx_k_GL_RENDERBUFFER_INTERNAL_FORMAT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_RED_SIZE, __pyx_k_GL_RENDERBUFFER_RED_SIZE, sizeof(__pyx_k_GL_RENDERBUFFER_RED_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_STENCIL_SIZE, __pyx_k_GL_RENDERBUFFER_STENCIL_SIZE, sizeof(__pyx_k_GL_RENDERBUFFER_STENCIL_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERBUFFER_WIDTH, __pyx_k_GL_RENDERBUFFER_WIDTH, sizeof(__pyx_k_GL_RENDERBUFFER_WIDTH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RENDERER, __pyx_k_GL_RENDERER, sizeof(__pyx_k_GL_RENDERER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_REPEAT, __pyx_k_GL_REPEAT, sizeof(__pyx_k_GL_REPEAT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_REPLACE, __pyx_k_GL_REPLACE, sizeof(__pyx_k_GL_REPLACE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RGB, __pyx_k_GL_RGB, sizeof(__pyx_k_GL_RGB), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RGB5_A1, __pyx_k_GL_RGB5_A1, sizeof(__pyx_k_GL_RGB5_A1), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RGBA, __pyx_k_GL_RGBA, sizeof(__pyx_k_GL_RGBA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_RGBA4, __pyx_k_GL_RGBA4, sizeof(__pyx_k_GL_RGBA4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLER_2D, __pyx_k_GL_SAMPLER_2D, sizeof(__pyx_k_GL_SAMPLER_2D), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLER_CUBE, __pyx_k_GL_SAMPLER_CUBE, sizeof(__pyx_k_GL_SAMPLER_CUBE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLES, __pyx_k_GL_SAMPLES, sizeof(__pyx_k_GL_SAMPLES), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLE_ALPHA_TO_COVERAGE, __pyx_k_GL_SAMPLE_ALPHA_TO_COVERAGE, sizeof(__pyx_k_GL_SAMPLE_ALPHA_TO_COVERAGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLE_BUFFERS, __pyx_k_GL_SAMPLE_BUFFERS, sizeof(__pyx_k_GL_SAMPLE_BUFFERS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLE_COVERAGE, __pyx_k_GL_SAMPLE_COVERAGE, sizeof(__pyx_k_GL_SAMPLE_COVERAGE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLE_COVERAGE_INVERT, __pyx_k_GL_SAMPLE_COVERAGE_INVERT, sizeof(__pyx_k_GL_SAMPLE_COVERAGE_INVERT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SAMPLE_COVERAGE_VALUE, __pyx_k_GL_SAMPLE_COVERAGE_VALUE, sizeof(__pyx_k_GL_SAMPLE_COVERAGE_VALUE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SCISSOR_BOX, __pyx_k_GL_SCISSOR_BOX, sizeof(__pyx_k_GL_SCISSOR_BOX), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SCISSOR_TEST, __pyx_k_GL_SCISSOR_TEST, sizeof(__pyx_k_GL_SCISSOR_TEST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SHADER_SOURCE_LENGTH, __pyx_k_GL_SHADER_SOURCE_LENGTH, sizeof(__pyx_k_GL_SHADER_SOURCE_LENGTH), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SHADER_TYPE, __pyx_k_GL_SHADER_TYPE, sizeof(__pyx_k_GL_SHADER_TYPE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SHADING_LANGUAGE_VERSION, __pyx_k_GL_SHADING_LANGUAGE_VERSION, sizeof(__pyx_k_GL_SHADING_LANGUAGE_VERSION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SHORT, __pyx_k_GL_SHORT, sizeof(__pyx_k_GL_SHORT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SRC_ALPHA, __pyx_k_GL_SRC_ALPHA, sizeof(__pyx_k_GL_SRC_ALPHA), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SRC_ALPHA_SATURATE, __pyx_k_GL_SRC_ALPHA_SATURATE, sizeof(__pyx_k_GL_SRC_ALPHA_SATURATE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SRC_COLOR, __pyx_k_GL_SRC_COLOR, sizeof(__pyx_k_GL_SRC_COLOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STATIC_DRAW, __pyx_k_GL_STATIC_DRAW, sizeof(__pyx_k_GL_STATIC_DRAW), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_ATTACHMENT, __pyx_k_GL_STENCIL_ATTACHMENT, sizeof(__pyx_k_GL_STENCIL_ATTACHMENT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_FAIL, __pyx_k_GL_STENCIL_BACK_FAIL, sizeof(__pyx_k_GL_STENCIL_BACK_FAIL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_FUNC, __pyx_k_GL_STENCIL_BACK_FUNC, sizeof(__pyx_k_GL_STENCIL_BACK_FUNC), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_PASS_DEPTH_FAIL, __pyx_k_GL_STENCIL_BACK_PASS_DEPTH_FAIL, sizeof(__pyx_k_GL_STENCIL_BACK_PASS_DEPTH_FAIL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_PASS_DEPTH_PASS, __pyx_k_GL_STENCIL_BACK_PASS_DEPTH_PASS, sizeof(__pyx_k_GL_STENCIL_BACK_PASS_DEPTH_PASS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_REF, __pyx_k_GL_STENCIL_BACK_REF, sizeof(__pyx_k_GL_STENCIL_BACK_REF), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_VALUE_MASK, __pyx_k_GL_STENCIL_BACK_VALUE_MASK, sizeof(__pyx_k_GL_STENCIL_BACK_VALUE_MASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BACK_WRITEMASK, __pyx_k_GL_STENCIL_BACK_WRITEMASK, sizeof(__pyx_k_GL_STENCIL_BACK_WRITEMASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BITS, __pyx_k_GL_STENCIL_BITS, sizeof(__pyx_k_GL_STENCIL_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_BUFFER_BIT, __pyx_k_GL_STENCIL_BUFFER_BIT, sizeof(__pyx_k_GL_STENCIL_BUFFER_BIT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_CLEAR_VALUE, __pyx_k_GL_STENCIL_CLEAR_VALUE, sizeof(__pyx_k_GL_STENCIL_CLEAR_VALUE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_FAIL, __pyx_k_GL_STENCIL_FAIL, sizeof(__pyx_k_GL_STENCIL_FAIL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_FUNC, __pyx_k_GL_STENCIL_FUNC, sizeof(__pyx_k_GL_STENCIL_FUNC), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_INDEX8, __pyx_k_GL_STENCIL_INDEX8, sizeof(__pyx_k_GL_STENCIL_INDEX8), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_PASS_DEPTH_FAIL, __pyx_k_GL_STENCIL_PASS_DEPTH_FAIL, sizeof(__pyx_k_GL_STENCIL_PASS_DEPTH_FAIL), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_PASS_DEPTH_PASS, __pyx_k_GL_STENCIL_PASS_DEPTH_PASS, sizeof(__pyx_k_GL_STENCIL_PASS_DEPTH_PASS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_REF, __pyx_k_GL_STENCIL_REF, sizeof(__pyx_k_GL_STENCIL_REF), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_TEST, __pyx_k_GL_STENCIL_TEST, sizeof(__pyx_k_GL_STENCIL_TEST), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_VALUE_MASK, __pyx_k_GL_STENCIL_VALUE_MASK, sizeof(__pyx_k_GL_STENCIL_VALUE_MASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STENCIL_WRITEMASK, __pyx_k_GL_STENCIL_WRITEMASK, sizeof(__pyx_k_GL_STENCIL_WRITEMASK), 0, 0, 1, 1},
  {&__pyx_n_s_GL_STREAM_DRAW, __pyx_k_GL_STREAM_DRAW, sizeof(__pyx_k_GL_STREAM_DRAW), 0, 0, 1, 1},
  {&__pyx_n_s_GL_SUBPIXEL_BITS, __pyx_k_GL_SUBPIXEL_BITS, sizeof(__pyx_k_GL_SUBPIXEL_BITS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE, __pyx_k_GL_TEXTURE, sizeof(__pyx_k_GL_TEXTURE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE0, __pyx_k_GL_TEXTURE0, sizeof(__pyx_k_GL_TEXTURE0), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE1, __pyx_k_GL_TEXTURE1, sizeof(__pyx_k_GL_TEXTURE1), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE10, __pyx_k_GL_TEXTURE10, sizeof(__pyx_k_GL_TEXTURE10), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE11, __pyx_k_GL_TEXTURE11, sizeof(__pyx_k_GL_TEXTURE11), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE12, __pyx_k_GL_TEXTURE12, sizeof(__pyx_k_GL_TEXTURE12), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE13, __pyx_k_GL_TEXTURE13, sizeof(__pyx_k_GL_TEXTURE13), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE14, __pyx_k_GL_TEXTURE14, sizeof(__pyx_k_GL_TEXTURE14), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE15, __pyx_k_GL_TEXTURE15, sizeof(__pyx_k_GL_TEXTURE15), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE16, __pyx_k_GL_TEXTURE16, sizeof(__pyx_k_GL_TEXTURE16), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE17, __pyx_k_GL_TEXTURE17, sizeof(__pyx_k_GL_TEXTURE17), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE18, __pyx_k_GL_TEXTURE18, sizeof(__pyx_k_GL_TEXTURE18), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE19, __pyx_k_GL_TEXTURE19, sizeof(__pyx_k_GL_TEXTURE19), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE2, __pyx_k_GL_TEXTURE2, sizeof(__pyx_k_GL_TEXTURE2), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE20, __pyx_k_GL_TEXTURE20, sizeof(__pyx_k_GL_TEXTURE20), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE21, __pyx_k_GL_TEXTURE21, sizeof(__pyx_k_GL_TEXTURE21), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE22, __pyx_k_GL_TEXTURE22, sizeof(__pyx_k_GL_TEXTURE22), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE23, __pyx_k_GL_TEXTURE23, sizeof(__pyx_k_GL_TEXTURE23), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE24, __pyx_k_GL_TEXTURE24, sizeof(__pyx_k_GL_TEXTURE24), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE25, __pyx_k_GL_TEXTURE25, sizeof(__pyx_k_GL_TEXTURE25), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE26, __pyx_k_GL_TEXTURE26, sizeof(__pyx_k_GL_TEXTURE26), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE27, __pyx_k_GL_TEXTURE27, sizeof(__pyx_k_GL_TEXTURE27), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE28, __pyx_k_GL_TEXTURE28, sizeof(__pyx_k_GL_TEXTURE28), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE29, __pyx_k_GL_TEXTURE29, sizeof(__pyx_k_GL_TEXTURE29), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE3, __pyx_k_GL_TEXTURE3, sizeof(__pyx_k_GL_TEXTURE3), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE30, __pyx_k_GL_TEXTURE30, sizeof(__pyx_k_GL_TEXTURE30), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE31, __pyx_k_GL_TEXTURE31, sizeof(__pyx_k_GL_TEXTURE31), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE4, __pyx_k_GL_TEXTURE4, sizeof(__pyx_k_GL_TEXTURE4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE5, __pyx_k_GL_TEXTURE5, sizeof(__pyx_k_GL_TEXTURE5), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE6, __pyx_k_GL_TEXTURE6, sizeof(__pyx_k_GL_TEXTURE6), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE7, __pyx_k_GL_TEXTURE7, sizeof(__pyx_k_GL_TEXTURE7), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE8, __pyx_k_GL_TEXTURE8, sizeof(__pyx_k_GL_TEXTURE8), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE9, __pyx_k_GL_TEXTURE9, sizeof(__pyx_k_GL_TEXTURE9), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_2D, __pyx_k_GL_TEXTURE_2D, sizeof(__pyx_k_GL_TEXTURE_2D), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_BINDING_2D, __pyx_k_GL_TEXTURE_BINDING_2D, sizeof(__pyx_k_GL_TEXTURE_BINDING_2D), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_BINDING_CUBE_MAP, __pyx_k_GL_TEXTURE_BINDING_CUBE_MAP, sizeof(__pyx_k_GL_TEXTURE_BINDING_CUBE_MAP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP, __pyx_k_GL_TEXTURE_CUBE_MAP, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_X, __pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_X, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_X), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, __pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_Y), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, __pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP_NEGATIVE_Z), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_X, __pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_X, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_X), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_Y, __pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_Y, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_Y), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_Z, __pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_Z, sizeof(__pyx_k_GL_TEXTURE_CUBE_MAP_POSITIVE_Z), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_MAG_FILTER, __pyx_k_GL_TEXTURE_MAG_FILTER, sizeof(__pyx_k_GL_TEXTURE_MAG_FILTER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_MIN_FILTER, __pyx_k_GL_TEXTURE_MIN_FILTER, sizeof(__pyx_k_GL_TEXTURE_MIN_FILTER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_WRAP_S, __pyx_k_GL_TEXTURE_WRAP_S, sizeof(__pyx_k_GL_TEXTURE_WRAP_S), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TEXTURE_WRAP_T, __pyx_k_GL_TEXTURE_WRAP_T, sizeof(__pyx_k_GL_TEXTURE_WRAP_T), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TRIANGLES, __pyx_k_GL_TRIANGLES, sizeof(__pyx_k_GL_TRIANGLES), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TRIANGLE_FAN, __pyx_k_GL_TRIANGLE_FAN, sizeof(__pyx_k_GL_TRIANGLE_FAN), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TRIANGLE_STRIP, __pyx_k_GL_TRIANGLE_STRIP, sizeof(__pyx_k_GL_TRIANGLE_STRIP), 0, 0, 1, 1},
  {&__pyx_n_s_GL_TRUE, __pyx_k_GL_TRUE, sizeof(__pyx_k_GL_TRUE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNPACK_ALIGNMENT, __pyx_k_GL_UNPACK_ALIGNMENT, sizeof(__pyx_k_GL_UNPACK_ALIGNMENT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNSIGNED_BYTE, __pyx_k_GL_UNSIGNED_BYTE, sizeof(__pyx_k_GL_UNSIGNED_BYTE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNSIGNED_INT, __pyx_k_GL_UNSIGNED_INT, sizeof(__pyx_k_GL_UNSIGNED_INT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNSIGNED_SHORT, __pyx_k_GL_UNSIGNED_SHORT, sizeof(__pyx_k_GL_UNSIGNED_SHORT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNSIGNED_SHORT_4_4_4_4, __pyx_k_GL_UNSIGNED_SHORT_4_4_4_4, sizeof(__pyx_k_GL_UNSIGNED_SHORT_4_4_4_4), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNSIGNED_SHORT_5_5_5_1, __pyx_k_GL_UNSIGNED_SHORT_5_5_5_1, sizeof(__pyx_k_GL_UNSIGNED_SHORT_5_5_5_1), 0, 0, 1, 1},
  {&__pyx_n_s_GL_UNSIGNED_SHORT_5_6_5, __pyx_k_GL_UNSIGNED_SHORT_5_6_5, sizeof(__pyx_k_GL_UNSIGNED_SHORT_5_6_5), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VALIDATE_STATUS, __pyx_k_GL_VALIDATE_STATUS, sizeof(__pyx_k_GL_VALIDATE_STATUS), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VENDOR, __pyx_k_GL_VENDOR, sizeof(__pyx_k_GL_VENDOR), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERSION, __pyx_k_GL_VERSION, sizeof(__pyx_k_GL_VERSION), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_BUFFER_BI, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_BUFFER_BI, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_BUFFER_BI), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_ENABLED, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_ENABLED, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_ENABLED), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_NORMALIZE, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_NORMALIZE, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_NORMALIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_POINTER, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_POINTER, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_POINTER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_SIZE, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_SIZE, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_STRIDE, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_STRIDE, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_STRIDE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_TYPE, __pyx_k_GL_VERTEX_ATTRIB_ARRAY_TYPE, sizeof(__pyx_k_GL_VERTEX_ATTRIB_ARRAY_TYPE), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VERTEX_SHADER, __pyx_k_GL_VERTEX_SHADER, sizeof(__pyx_k_GL_VERTEX_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_GL_VIEWPORT, __pyx_k_GL_VIEWPORT, sizeof(__pyx_k_GL_VIEWPORT), 0, 0, 1, 1},
  {&__pyx_n_s_GL_ZERO, __pyx_k_GL_ZERO, sizeof(__pyx_k_GL_ZERO), 0, 0, 1, 1},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_alpha, __pyx_k_alpha, sizeof(__pyx_k_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_attachment, __pyx_k_attachment, sizeof(__pyx_k_attachment), 0, 0, 1, 1},
  {&__pyx_n_s_blue, __pyx_k_blue, sizeof(__pyx_k_blue), 0, 0, 1, 1},
  {&__pyx_n_s_border, __pyx_k_border, sizeof(__pyx_k_border), 0, 0, 1, 1},
  {&__pyx_n_s_bufSize, __pyx_k_bufSize, sizeof(__pyx_k_bufSize), 0, 0, 1, 1},
  {&__pyx_n_s_buffer, __pyx_k_buffer, sizeof(__pyx_k_buffer), 0, 0, 1, 1},
  {&__pyx_n_s_buffers, __pyx_k_buffers, sizeof(__pyx_k_buffers), 0, 0, 1, 1},
  {&__pyx_n_s_buffers_ptr, __pyx_k_buffers_ptr, sizeof(__pyx_k_buffers_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_cap, __pyx_k_cap, sizeof(__pyx_k_cap), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_count, __pyx_k_count, sizeof(__pyx_k_count), 0, 0, 1, 1},
  {&__pyx_n_s_count_ptr, __pyx_k_count_ptr, sizeof(__pyx_k_count_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_data_ptr, __pyx_k_data_ptr, sizeof(__pyx_k_data_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_dfactor, __pyx_k_dfactor, sizeof(__pyx_k_dfactor), 0, 0, 1, 1},
  {&__pyx_n_s_dfactorAlpha, __pyx_k_dfactorAlpha, sizeof(__pyx_k_dfactorAlpha), 0, 0, 1, 1},
  {&__pyx_n_s_dfactorRGB, __pyx_k_dfactorRGB, sizeof(__pyx_k_dfactorRGB), 0, 0, 1, 1},
  {&__pyx_n_s_dpfail, __pyx_k_dpfail, sizeof(__pyx_k_dpfail), 0, 0, 1, 1},
  {&__pyx_n_s_dppass, __pyx_k_dppass, sizeof(__pyx_k_dppass), 0, 0, 1, 1},
  {&__pyx_n_s_face, __pyx_k_face, sizeof(__pyx_k_face), 0, 0, 1, 1},
  {&__pyx_n_s_factor, __pyx_k_factor, sizeof(__pyx_k_factor), 0, 0, 1, 1},
  {&__pyx_n_s_fail, __pyx_k_fail, sizeof(__pyx_k_fail), 0, 0, 1, 1},
  {&__pyx_n_s_first, __pyx_k_first, sizeof(__pyx_k_first), 0, 0, 1, 1},
  {&__pyx_n_s_flag, __pyx_k_flag, sizeof(__pyx_k_flag), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_framebuffer, __pyx_k_framebuffer, sizeof(__pyx_k_framebuffer), 0, 0, 1, 1},
  {&__pyx_n_s_framebuffers, __pyx_k_framebuffers, sizeof(__pyx_k_framebuffers), 0, 0, 1, 1},
  {&__pyx_n_s_framebuffers_ptr, __pyx_k_framebuffers_ptr, sizeof(__pyx_k_framebuffers_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_func, __pyx_k_func, sizeof(__pyx_k_func), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_glActiveTexture, __pyx_k_glActiveTexture, sizeof(__pyx_k_glActiveTexture), 0, 0, 1, 1},
  {&__pyx_n_s_glAttachShader, __pyx_k_glAttachShader, sizeof(__pyx_k_glAttachShader), 0, 0, 1, 1},
  {&__pyx_n_s_glBindAttribLocation, __pyx_k_glBindAttribLocation, sizeof(__pyx_k_glBindAttribLocation), 0, 0, 1, 1},
  {&__pyx_n_s_glBindBuffer, __pyx_k_glBindBuffer, sizeof(__pyx_k_glBindBuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glBindFramebuffer, __pyx_k_glBindFramebuffer, sizeof(__pyx_k_glBindFramebuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glBindRenderbuffer, __pyx_k_glBindRenderbuffer, sizeof(__pyx_k_glBindRenderbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glBindTexture, __pyx_k_glBindTexture, sizeof(__pyx_k_glBindTexture), 0, 0, 1, 1},
  {&__pyx_n_s_glBlendColor, __pyx_k_glBlendColor, sizeof(__pyx_k_glBlendColor), 0, 0, 1, 1},
  {&__pyx_n_s_glBlendEquation, __pyx_k_glBlendEquation, sizeof(__pyx_k_glBlendEquation), 0, 0, 1, 1},
  {&__pyx_n_s_glBlendEquationSeparate, __pyx_k_glBlendEquationSeparate, sizeof(__pyx_k_glBlendEquationSeparate), 0, 0, 1, 1},
  {&__pyx_n_s_glBlendFunc, __pyx_k_glBlendFunc, sizeof(__pyx_k_glBlendFunc), 0, 0, 1, 1},
  {&__pyx_n_s_glBlendFuncSeparate, __pyx_k_glBlendFuncSeparate, sizeof(__pyx_k_glBlendFuncSeparate), 0, 0, 1, 1},
  {&__pyx_n_s_glBufferData, __pyx_k_glBufferData, sizeof(__pyx_k_glBufferData), 0, 0, 1, 1},
  {&__pyx_n_s_glBufferSubData, __pyx_k_glBufferSubData, sizeof(__pyx_k_glBufferSubData), 0, 0, 1, 1},
  {&__pyx_n_s_glCheckFramebufferStatus, __pyx_k_glCheckFramebufferStatus, sizeof(__pyx_k_glCheckFramebufferStatus), 0, 0, 1, 1},
  {&__pyx_n_s_glClear, __pyx_k_glClear, sizeof(__pyx_k_glClear), 0, 0, 1, 1},
  {&__pyx_n_s_glClearColor, __pyx_k_glClearColor, sizeof(__pyx_k_glClearColor), 0, 0, 1, 1},
  {&__pyx_n_s_glClearStencil, __pyx_k_glClearStencil, sizeof(__pyx_k_glClearStencil), 0, 0, 1, 1},
  {&__pyx_n_s_glColorMask, __pyx_k_glColorMask, sizeof(__pyx_k_glColorMask), 0, 0, 1, 1},
  {&__pyx_n_s_glCompileShader, __pyx_k_glCompileShader, sizeof(__pyx_k_glCompileShader), 0, 0, 1, 1},
  {&__pyx_n_s_glCompressedTexImage2D, __pyx_k_glCompressedTexImage2D, sizeof(__pyx_k_glCompressedTexImage2D), 0, 0, 1, 1},
  {&__pyx_n_s_glCompressedTexSubImage2D, __pyx_k_glCompressedTexSubImage2D, sizeof(__pyx_k_glCompressedTexSubImage2D), 0, 0, 1, 1},
  {&__pyx_n_s_glCopyTexImage2D, __pyx_k_glCopyTexImage2D, sizeof(__pyx_k_glCopyTexImage2D), 0, 0, 1, 1},
  {&__pyx_n_s_glCopyTexSubImage2D, __pyx_k_glCopyTexSubImage2D, sizeof(__pyx_k_glCopyTexSubImage2D), 0, 0, 1, 1},
  {&__pyx_n_s_glCreateProgram, __pyx_k_glCreateProgram, sizeof(__pyx_k_glCreateProgram), 0, 0, 1, 1},
  {&__pyx_n_s_glCreateShader, __pyx_k_glCreateShader, sizeof(__pyx_k_glCreateShader), 0, 0, 1, 1},
  {&__pyx_n_s_glCullFace, __pyx_k_glCullFace, sizeof(__pyx_k_glCullFace), 0, 0, 1, 1},
  {&__pyx_n_s_glDeleteBuffers, __pyx_k_glDeleteBuffers, sizeof(__pyx_k_glDeleteBuffers), 0, 0, 1, 1},
  {&__pyx_n_s_glDeleteFramebuffers, __pyx_k_glDeleteFramebuffers, sizeof(__pyx_k_glDeleteFramebuffers), 0, 0, 1, 1},
  {&__pyx_n_s_glDeleteProgram, __pyx_k_glDeleteProgram, sizeof(__pyx_k_glDeleteProgram), 0, 0, 1, 1},
  {&__pyx_n_s_glDeleteRenderbuffers, __pyx_k_glDeleteRenderbuffers, sizeof(__pyx_k_glDeleteRenderbuffers), 0, 0, 1, 1},
  {&__pyx_n_s_glDeleteShader, __pyx_k_glDeleteShader, sizeof(__pyx_k_glDeleteShader), 0, 0, 1, 1},
  {&__pyx_n_s_glDeleteTextures, __pyx_k_glDeleteTextures, sizeof(__pyx_k_glDeleteTextures), 0, 0, 1, 1},
  {&__pyx_n_s_glDepthFunc, __pyx_k_glDepthFunc, sizeof(__pyx_k_glDepthFunc), 0, 0, 1, 1},
  {&__pyx_n_s_glDepthMask, __pyx_k_glDepthMask, sizeof(__pyx_k_glDepthMask), 0, 0, 1, 1},
  {&__pyx_n_s_glDetachShader, __pyx_k_glDetachShader, sizeof(__pyx_k_glDetachShader), 0, 0, 1, 1},
  {&__pyx_n_s_glDisable, __pyx_k_glDisable, sizeof(__pyx_k_glDisable), 0, 0, 1, 1},
  {&__pyx_n_s_glDisableVertexAttribArray, __pyx_k_glDisableVertexAttribArray, sizeof(__pyx_k_glDisableVertexAttribArray), 0, 0, 1, 1},
  {&__pyx_n_s_glDrawArrays, __pyx_k_glDrawArrays, sizeof(__pyx_k_glDrawArrays), 0, 0, 1, 1},
  {&__pyx_n_s_glDrawElements, __pyx_k_glDrawElements, sizeof(__pyx_k_glDrawElements), 0, 0, 1, 1},
  {&__pyx_n_s_glEnable, __pyx_k_glEnable, sizeof(__pyx_k_glEnable), 0, 0, 1, 1},
  {&__pyx_n_s_glEnableVertexAttribArray, __pyx_k_glEnableVertexAttribArray, sizeof(__pyx_k_glEnableVertexAttribArray), 0, 0, 1, 1},
  {&__pyx_n_s_glFinish, __pyx_k_glFinish, sizeof(__pyx_k_glFinish), 0, 0, 1, 1},
  {&__pyx_n_s_glFlush, __pyx_k_glFlush, sizeof(__pyx_k_glFlush), 0, 0, 1, 1},
  {&__pyx_n_s_glFramebufferRenderbuffer, __pyx_k_glFramebufferRenderbuffer, sizeof(__pyx_k_glFramebufferRenderbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glFramebufferTexture2D, __pyx_k_glFramebufferTexture2D, sizeof(__pyx_k_glFramebufferTexture2D), 0, 0, 1, 1},
  {&__pyx_n_s_glFrontFace, __pyx_k_glFrontFace, sizeof(__pyx_k_glFrontFace), 0, 0, 1, 1},
  {&__pyx_n_s_glGenBuffers, __pyx_k_glGenBuffers, sizeof(__pyx_k_glGenBuffers), 0, 0, 1, 1},
  {&__pyx_n_s_glGenFramebuffers, __pyx_k_glGenFramebuffers, sizeof(__pyx_k_glGenFramebuffers), 0, 0, 1, 1},
  {&__pyx_n_s_glGenRenderbuffers, __pyx_k_glGenRenderbuffers, sizeof(__pyx_k_glGenRenderbuffers), 0, 0, 1, 1},
  {&__pyx_n_s_glGenTextures, __pyx_k_glGenTextures, sizeof(__pyx_k_glGenTextures), 0, 0, 1, 1},
  {&__pyx_n_s_glGenerateMipmap, __pyx_k_glGenerateMipmap, sizeof(__pyx_k_glGenerateMipmap), 0, 0, 1, 1},
  {&__pyx_n_s_glGetActiveAttrib, __pyx_k_glGetActiveAttrib, sizeof(__pyx_k_glGetActiveAttrib), 0, 0, 1, 1},
  {&__pyx_n_s_glGetActiveUniform, __pyx_k_glGetActiveUniform, sizeof(__pyx_k_glGetActiveUniform), 0, 0, 1, 1},
  {&__pyx_n_s_glGetAttachedShaders, __pyx_k_glGetAttachedShaders, sizeof(__pyx_k_glGetAttachedShaders), 0, 0, 1, 1},
  {&__pyx_n_s_glGetAttribLocation, __pyx_k_glGetAttribLocation, sizeof(__pyx_k_glGetAttribLocation), 0, 0, 1, 1},
  {&__pyx_n_s_glGetBooleanv, __pyx_k_glGetBooleanv, sizeof(__pyx_k_glGetBooleanv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetBufferParameteriv, __pyx_k_glGetBufferParameteriv, sizeof(__pyx_k_glGetBufferParameteriv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetError, __pyx_k_glGetError, sizeof(__pyx_k_glGetError), 0, 0, 1, 1},
  {&__pyx_n_s_glGetFloatv, __pyx_k_glGetFloatv, sizeof(__pyx_k_glGetFloatv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetFramebufferAttachmentParame, __pyx_k_glGetFramebufferAttachmentParame, sizeof(__pyx_k_glGetFramebufferAttachmentParame), 0, 0, 1, 1},
  {&__pyx_n_s_glGetIntegerv, __pyx_k_glGetIntegerv, sizeof(__pyx_k_glGetIntegerv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetProgramInfoLog, __pyx_k_glGetProgramInfoLog, sizeof(__pyx_k_glGetProgramInfoLog), 0, 0, 1, 1},
  {&__pyx_n_s_glGetProgramiv, __pyx_k_glGetProgramiv, sizeof(__pyx_k_glGetProgramiv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetRenderbufferParameteriv, __pyx_k_glGetRenderbufferParameteriv, sizeof(__pyx_k_glGetRenderbufferParameteriv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetShaderInfoLog, __pyx_k_glGetShaderInfoLog, sizeof(__pyx_k_glGetShaderInfoLog), 0, 0, 1, 1},
  {&__pyx_n_s_glGetShaderSource, __pyx_k_glGetShaderSource, sizeof(__pyx_k_glGetShaderSource), 0, 0, 1, 1},
  {&__pyx_n_s_glGetShaderiv, __pyx_k_glGetShaderiv, sizeof(__pyx_k_glGetShaderiv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetString, __pyx_k_glGetString, sizeof(__pyx_k_glGetString), 0, 0, 1, 1},
  {&__pyx_n_s_glGetTexParameterfv, __pyx_k_glGetTexParameterfv, sizeof(__pyx_k_glGetTexParameterfv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetTexParameteriv, __pyx_k_glGetTexParameteriv, sizeof(__pyx_k_glGetTexParameteriv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetUniformLocation, __pyx_k_glGetUniformLocation, sizeof(__pyx_k_glGetUniformLocation), 0, 0, 1, 1},
  {&__pyx_n_s_glGetUniformfv, __pyx_k_glGetUniformfv, sizeof(__pyx_k_glGetUniformfv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetUniformiv, __pyx_k_glGetUniformiv, sizeof(__pyx_k_glGetUniformiv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetVertexAttribPointerv, __pyx_k_glGetVertexAttribPointerv, sizeof(__pyx_k_glGetVertexAttribPointerv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetVertexAttribfv, __pyx_k_glGetVertexAttribfv, sizeof(__pyx_k_glGetVertexAttribfv), 0, 0, 1, 1},
  {&__pyx_n_s_glGetVertexAttribiv, __pyx_k_glGetVertexAttribiv, sizeof(__pyx_k_glGetVertexAttribiv), 0, 0, 1, 1},
  {&__pyx_n_s_glHint, __pyx_k_glHint, sizeof(__pyx_k_glHint), 0, 0, 1, 1},
  {&__pyx_n_s_glIsBuffer, __pyx_k_glIsBuffer, sizeof(__pyx_k_glIsBuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glIsEnabled, __pyx_k_glIsEnabled, sizeof(__pyx_k_glIsEnabled), 0, 0, 1, 1},
  {&__pyx_n_s_glIsFramebuffer, __pyx_k_glIsFramebuffer, sizeof(__pyx_k_glIsFramebuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glIsProgram, __pyx_k_glIsProgram, sizeof(__pyx_k_glIsProgram), 0, 0, 1, 1},
  {&__pyx_n_s_glIsRenderbuffer, __pyx_k_glIsRenderbuffer, sizeof(__pyx_k_glIsRenderbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_glIsShader, __pyx_k_glIsShader, sizeof(__pyx_k_glIsShader), 0, 0, 1, 1},
  {&__pyx_n_s_glIsTexture, __pyx_k_glIsTexture, sizeof(__pyx_k_glIsTexture), 0, 0, 1, 1},
  {&__pyx_n_s_glLineWidth, __pyx_k_glLineWidth, sizeof(__pyx_k_glLineWidth), 0, 0, 1, 1},
  {&__pyx_n_s_glLinkProgram, __pyx_k_glLinkProgram, sizeof(__pyx_k_glLinkProgram), 0, 0, 1, 1},
  {&__pyx_n_s_glPixelStorei, __pyx_k_glPixelStorei, sizeof(__pyx_k_glPixelStorei), 0, 0, 1, 1},
  {&__pyx_n_s_glPolygonOffset, __pyx_k_glPolygonOffset, sizeof(__pyx_k_glPolygonOffset), 0, 0, 1, 1},
  {&__pyx_n_s_glReadPixels, __pyx_k_glReadPixels, sizeof(__pyx_k_glReadPixels), 0, 0, 1, 1},
  {&__pyx_n_s_glRenderbufferStorage, __pyx_k_glRenderbufferStorage, sizeof(__pyx_k_glRenderbufferStorage), 0, 0, 1, 1},
  {&__pyx_n_s_glSampleCoverage, __pyx_k_glSampleCoverage, sizeof(__pyx_k_glSampleCoverage), 0, 0, 1, 1},
  {&__pyx_n_s_glScissor, __pyx_k_glScissor, sizeof(__pyx_k_glScissor), 0, 0, 1, 1},
  {&__pyx_n_s_glShaderSource, __pyx_k_glShaderSource, sizeof(__pyx_k_glShaderSource), 0, 0, 1, 1},
  {&__pyx_n_s_glStencilFunc, __pyx_k_glStencilFunc, sizeof(__pyx_k_glStencilFunc), 0, 0, 1, 1},
  {&__pyx_n_s_glStencilFuncSeparate, __pyx_k_glStencilFuncSeparate, sizeof(__pyx_k_glStencilFuncSeparate), 0, 0, 1, 1},
  {&__pyx_n_s_glStencilMask, __pyx_k_glStencilMask, sizeof(__pyx_k_glStencilMask), 0, 0, 1, 1},
  {&__pyx_n_s_glStencilMaskSeparate, __pyx_k_glStencilMaskSeparate, sizeof(__pyx_k_glStencilMaskSeparate), 0, 0, 1, 1},
  {&__pyx_n_s_glStencilOp, __pyx_k_glStencilOp, sizeof(__pyx_k_glStencilOp), 0, 0, 1, 1},
  {&__pyx_n_s_glStencilOpSeparate, __pyx_k_glStencilOpSeparate, sizeof(__pyx_k_glStencilOpSeparate), 0, 0, 1, 1},
  {&__pyx_n_s_glTexImage2D, __pyx_k_glTexImage2D, sizeof(__pyx_k_glTexImage2D), 0, 0, 1, 1},
  {&__pyx_n_s_glTexParameterf, __pyx_k_glTexParameterf, sizeof(__pyx_k_glTexParameterf), 0, 0, 1, 1},
  {&__pyx_n_s_glTexParameterfv, __pyx_k_glTexParameterfv, sizeof(__pyx_k_glTexParameterfv), 0, 0, 1, 1},
  {&__pyx_n_s_glTexParameteri, __pyx_k_glTexParameteri, sizeof(__pyx_k_glTexParameteri), 0, 0, 1, 1},
  {&__pyx_n_s_glTexParameteriv, __pyx_k_glTexParameteriv, sizeof(__pyx_k_glTexParameteriv), 0, 0, 1, 1},
  {&__pyx_n_s_glTexSubImage2D, __pyx_k_glTexSubImage2D, sizeof(__pyx_k_glTexSubImage2D), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform1f, __pyx_k_glUniform1f, sizeof(__pyx_k_glUniform1f), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform1fv, __pyx_k_glUniform1fv, sizeof(__pyx_k_glUniform1fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform1i, __pyx_k_glUniform1i, sizeof(__pyx_k_glUniform1i), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform1iv, __pyx_k_glUniform1iv, sizeof(__pyx_k_glUniform1iv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform2f, __pyx_k_glUniform2f, sizeof(__pyx_k_glUniform2f), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform2fv, __pyx_k_glUniform2fv, sizeof(__pyx_k_glUniform2fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform2i, __pyx_k_glUniform2i, sizeof(__pyx_k_glUniform2i), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform2iv, __pyx_k_glUniform2iv, sizeof(__pyx_k_glUniform2iv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform3f, __pyx_k_glUniform3f, sizeof(__pyx_k_glUniform3f), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform3fv, __pyx_k_glUniform3fv, sizeof(__pyx_k_glUniform3fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform3i, __pyx_k_glUniform3i, sizeof(__pyx_k_glUniform3i), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform3iv, __pyx_k_glUniform3iv, sizeof(__pyx_k_glUniform3iv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform4f, __pyx_k_glUniform4f, sizeof(__pyx_k_glUniform4f), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform4fv, __pyx_k_glUniform4fv, sizeof(__pyx_k_glUniform4fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform4i, __pyx_k_glUniform4i, sizeof(__pyx_k_glUniform4i), 0, 0, 1, 1},
  {&__pyx_n_s_glUniform4iv, __pyx_k_glUniform4iv, sizeof(__pyx_k_glUniform4iv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniformMatrix2fv, __pyx_k_glUniformMatrix2fv, sizeof(__pyx_k_glUniformMatrix2fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniformMatrix3fv, __pyx_k_glUniformMatrix3fv, sizeof(__pyx_k_glUniformMatrix3fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUniformMatrix4fv, __pyx_k_glUniformMatrix4fv, sizeof(__pyx_k_glUniformMatrix4fv), 0, 0, 1, 1},
  {&__pyx_n_s_glUseProgram, __pyx_k_glUseProgram, sizeof(__pyx_k_glUseProgram), 0, 0, 1, 1},
  {&__pyx_n_s_glValidateProgram, __pyx_k_glValidateProgram, sizeof(__pyx_k_glValidateProgram), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib1f, __pyx_k_glVertexAttrib1f, sizeof(__pyx_k_glVertexAttrib1f), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib1fv, __pyx_k_glVertexAttrib1fv, sizeof(__pyx_k_glVertexAttrib1fv), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib2f, __pyx_k_glVertexAttrib2f, sizeof(__pyx_k_glVertexAttrib2f), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib2fv, __pyx_k_glVertexAttrib2fv, sizeof(__pyx_k_glVertexAttrib2fv), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib3f, __pyx_k_glVertexAttrib3f, sizeof(__pyx_k_glVertexAttrib3f), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib3fv, __pyx_k_glVertexAttrib3fv, sizeof(__pyx_k_glVertexAttrib3fv), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib4f, __pyx_k_glVertexAttrib4f, sizeof(__pyx_k_glVertexAttrib4f), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttrib4fv, __pyx_k_glVertexAttrib4fv, sizeof(__pyx_k_glVertexAttrib4fv), 0, 0, 1, 1},
  {&__pyx_n_s_glVertexAttribPointer, __pyx_k_glVertexAttribPointer, sizeof(__pyx_k_glVertexAttribPointer), 0, 0, 1, 1},
  {&__pyx_n_s_glViewport, __pyx_k_glViewport, sizeof(__pyx_k_glViewport), 0, 0, 1, 1},
  {&__pyx_n_s_green, __pyx_k_green, sizeof(__pyx_k_green), 0, 0, 1, 1},
  {&__pyx_n_s_height, __pyx_k_height, sizeof(__pyx_k_height), 0, 0, 1, 1},
  {&__pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_k_home_tom_ab_renpy_renpy_gl2_ugu, sizeof(__pyx_k_home_tom_ab_renpy_renpy_gl2_ugu), 0, 0, 1, 0},
  {&__pyx_n_s_imageSize, __pyx_k_imageSize, sizeof(__pyx_k_imageSize), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_kp_s_index_out_of_range, __pyx_k_index_out_of_range, sizeof(__pyx_k_index_out_of_range), 0, 0, 1, 0},
  {&__pyx_n_s_indices, __pyx_k_indices, sizeof(__pyx_k_indices), 0, 0, 1, 1},
  {&__pyx_n_s_indices_ptr, __pyx_k_indices_ptr, sizeof(__pyx_k_indices_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_infoLog, __pyx_k_infoLog, sizeof(__pyx_k_infoLog), 0, 0, 1, 1},
  {&__pyx_n_s_infoLog_ptr, __pyx_k_infoLog_ptr, sizeof(__pyx_k_infoLog_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_internalformat, __pyx_k_internalformat, sizeof(__pyx_k_internalformat), 0, 0, 1, 1},
  {&__pyx_n_s_invert, __pyx_k_invert, sizeof(__pyx_k_invert), 0, 0, 1, 1},
  {&__pyx_n_s_length, __pyx_k_length, sizeof(__pyx_k_length), 0, 0, 1, 1},
  {&__pyx_n_s_length_ptr, __pyx_k_length_ptr, sizeof(__pyx_k_length_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_level, __pyx_k_level, sizeof(__pyx_k_level), 0, 0, 1, 1},
  {&__pyx_n_s_load, __pyx_k_load, sizeof(__pyx_k_load), 0, 0, 1, 1},
  {&__pyx_n_s_location, __pyx_k_location, sizeof(__pyx_k_location), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mask, __pyx_k_mask, sizeof(__pyx_k_mask), 0, 0, 1, 1},
  {&__pyx_n_s_maxCount, __pyx_k_maxCount, sizeof(__pyx_k_maxCount), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_modeAlpha, __pyx_k_modeAlpha, sizeof(__pyx_k_modeAlpha), 0, 0, 1, 1},
  {&__pyx_n_s_modeRGB, __pyx_k_modeRGB, sizeof(__pyx_k_modeRGB), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_name_ptr, __pyx_k_name_ptr, sizeof(__pyx_k_name_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_normalized, __pyx_k_normalized, sizeof(__pyx_k_normalized), 0, 0, 1, 1},
  {&__pyx_n_s_o, __pyx_k_o, sizeof(__pyx_k_o), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_param, __pyx_k_param, sizeof(__pyx_k_param), 0, 0, 1, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_s_params_ptr, __pyx_k_params_ptr, sizeof(__pyx_k_params_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_pixels, __pyx_k_pixels, sizeof(__pyx_k_pixels), 0, 0, 1, 1},
  {&__pyx_n_s_pixels_ptr, __pyx_k_pixels_ptr, sizeof(__pyx_k_pixels_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_pname, __pyx_k_pname, sizeof(__pyx_k_pname), 0, 0, 1, 1},
  {&__pyx_n_s_pointer, __pyx_k_pointer, sizeof(__pyx_k_pointer), 0, 0, 1, 1},
  {&__pyx_n_s_pointer_ptr, __pyx_k_pointer_ptr, sizeof(__pyx_k_pointer_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_program, __pyx_k_program, sizeof(__pyx_k_program), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_red, __pyx_k_red, sizeof(__pyx_k_red), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_ref, __pyx_k_ref, sizeof(__pyx_k_ref), 0, 0, 1, 1},
  {&__pyx_n_s_renderbuffer, __pyx_k_renderbuffer, sizeof(__pyx_k_renderbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_renderbuffers, __pyx_k_renderbuffers, sizeof(__pyx_k_renderbuffers), 0, 0, 1, 1},
  {&__pyx_n_s_renderbuffers_ptr, __pyx_k_renderbuffers_ptr, sizeof(__pyx_k_renderbuffers_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_renderbuffertarget, __pyx_k_renderbuffertarget, sizeof(__pyx_k_renderbuffertarget), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_gl2_uguu, __pyx_k_renpy_gl2_uguu, sizeof(__pyx_k_renpy_gl2_uguu), 0, 0, 1, 1},
  {&__pyx_n_s_ro, __pyx_k_ro, sizeof(__pyx_k_ro), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_kp_s_self_data_cannot_be_converted_to, __pyx_k_self_data_cannot_be_converted_to, sizeof(__pyx_k_self_data_cannot_be_converted_to), 0, 0, 1, 0},
  {&__pyx_kp_s_self_ptr_self_view_cannot_be_con, __pyx_k_self_ptr_self_view_cannot_be_con, sizeof(__pyx_k_self_ptr_self_view_cannot_be_con), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_sfactor, __pyx_k_sfactor, sizeof(__pyx_k_sfactor), 0, 0, 1, 1},
  {&__pyx_n_s_sfactorAlpha, __pyx_k_sfactorAlpha, sizeof(__pyx_k_sfactorAlpha), 0, 0, 1, 1},
  {&__pyx_n_s_sfactorRGB, __pyx_k_sfactorRGB, sizeof(__pyx_k_sfactorRGB), 0, 0, 1, 1},
  {&__pyx_n_s_sfail, __pyx_k_sfail, sizeof(__pyx_k_sfail), 0, 0, 1, 1},
  {&__pyx_n_s_shader, __pyx_k_shader, sizeof(__pyx_k_shader), 0, 0, 1, 1},
  {&__pyx_n_s_shaders, __pyx_k_shaders, sizeof(__pyx_k_shaders), 0, 0, 1, 1},
  {&__pyx_n_s_shaders_ptr, __pyx_k_shaders_ptr, sizeof(__pyx_k_shaders_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_size_ptr, __pyx_k_size_ptr, sizeof(__pyx_k_size_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_source, __pyx_k_source, sizeof(__pyx_k_source), 0, 0, 1, 1},
  {&__pyx_n_s_source_ptr, __pyx_k_source_ptr, sizeof(__pyx_k_source_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_stride, __pyx_k_stride, sizeof(__pyx_k_stride), 0, 0, 1, 1},
  {&__pyx_n_s_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 0, 1, 1},
  {&__pyx_n_s_string_ptr, __pyx_k_string_ptr, sizeof(__pyx_k_string_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_target, __pyx_k_target, sizeof(__pyx_k_target), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_textarget, __pyx_k_textarget, sizeof(__pyx_k_textarget), 0, 0, 1, 1},
  {&__pyx_n_s_texture, __pyx_k_texture, sizeof(__pyx_k_texture), 0, 0, 1, 1},
  {&__pyx_n_s_textures, __pyx_k_textures, sizeof(__pyx_k_textures), 0, 0, 1, 1},
  {&__pyx_n_s_textures_ptr, __pyx_k_textures_ptr, sizeof(__pyx_k_textures_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_transpose, __pyx_k_transpose, sizeof(__pyx_k_transpose), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_type_ptr, __pyx_k_type_ptr, sizeof(__pyx_k_type_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_uguugl, __pyx_k_uguugl, sizeof(__pyx_k_uguugl), 0, 0, 1, 1},
  {&__pyx_n_s_units, __pyx_k_units, sizeof(__pyx_k_units), 0, 0, 1, 1},
  {&__pyx_n_s_usage, __pyx_k_usage, sizeof(__pyx_k_usage), 0, 0, 1, 1},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {&__pyx_n_s_v0, __pyx_k_v0, sizeof(__pyx_k_v0), 0, 0, 1, 1},
  {&__pyx_n_s_v1, __pyx_k_v1, sizeof(__pyx_k_v1), 0, 0, 1, 1},
  {&__pyx_n_s_v2, __pyx_k_v2, sizeof(__pyx_k_v2), 0, 0, 1, 1},
  {&__pyx_n_s_v3, __pyx_k_v3, sizeof(__pyx_k_v3), 0, 0, 1, 1},
  {&__pyx_n_s_v_ptr, __pyx_k_v_ptr, sizeof(__pyx_k_v_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_value_ptr, __pyx_k_value_ptr, sizeof(__pyx_k_value_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_width, __pyx_k_width, sizeof(__pyx_k_width), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_xoffset, __pyx_k_xoffset, sizeof(__pyx_k_xoffset), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_yoffset, __pyx_k_yoffset, sizeof(__pyx_k_yoffset), 0, 0, 1, 1},
  {&__pyx_n_s_z, __pyx_k_z, sizeof(__pyx_k_z), 0, 0, 1, 1},
  {&__pyx_n_s_zfail, __pyx_k_zfail, sizeof(__pyx_k_zfail), 0, 0, 1, 1},
  {&__pyx_n_s_zpass, __pyx_k_zpass, sizeof(__pyx_k_zpass), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 2, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(1, 177, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_self_ptr_self_view_cannot_be_con); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "(tree fragment)":4
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.ptr,self.view cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_self_ptr_self_view_cannot_be_con); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "renpy/gl2/uguu.pyx":177
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")             # <<<<<<<<<<<<<<
 * 
 *         return (<int*> self.data)[index]
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_index_out_of_range); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "renpy/gl2/uguu.pyx":194
 *     def __getitem__(self, index):
 *         if index < 0 or index >= self.length:
 *             raise IndexError("index out of range")             # <<<<<<<<<<<<<<
 * 
 *         return (<float *> self.data)[index]
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_index_out_of_range); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "renpy/gl2/uguu.pyx":231
 * from uguugl cimport GLuint64EXT
 * 
 * def glActiveTexture(texture):             # <<<<<<<<<<<<<<
 *     uguugl.glActiveTexture(texture)
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_texture); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glActiveTexture, 231, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(1, 231, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":234
 *     uguugl.glActiveTexture(texture)
 * 
 * def glAttachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glAttachShader(program, shader)
 * 
 */
  __pyx_tuple__17 = PyTuple_Pack(2, __pyx_n_s_program, __pyx_n_s_shader); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glAttachShader, 234, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(1, 234, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":237
 *     uguugl.glAttachShader(program, shader)
 * 
 * def glBindAttribLocation(program, index, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 */
  __pyx_tuple__19 = PyTuple_Pack(4, __pyx_n_s_program, __pyx_n_s_index, __pyx_n_s_name, __pyx_n_s_name_ptr); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBindAttribLocation, 237, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(1, 237, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":241
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 * 
 * def glBindBuffer(target, buffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindBuffer(target, buffer)
 * 
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_n_s_target, __pyx_n_s_buffer); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBindBuffer, 241, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(1, 241, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":244
 *     uguugl.glBindBuffer(target, buffer)
 * 
 * def glBindFramebuffer(target, framebuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 */
  __pyx_tuple__23 = PyTuple_Pack(2, __pyx_n_s_target, __pyx_n_s_framebuffer); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBindFramebuffer, 244, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(1, 244, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":247
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 * def glBindRenderbuffer(target, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 */
  __pyx_tuple__25 = PyTuple_Pack(2, __pyx_n_s_target, __pyx_n_s_renderbuffer); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBindRenderbuffer, 247, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(1, 247, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":250
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 * def glBindTexture(target, texture):             # <<<<<<<<<<<<<<
 *     uguugl.glBindTexture(target, texture)
 * 
 */
  __pyx_tuple__27 = PyTuple_Pack(2, __pyx_n_s_target, __pyx_n_s_texture); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBindTexture, 250, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(1, 250, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":253
 *     uguugl.glBindTexture(target, texture)
 * 
 * def glBlendColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 */
  __pyx_tuple__29 = PyTuple_Pack(4, __pyx_n_s_red, __pyx_n_s_green, __pyx_n_s_blue, __pyx_n_s_alpha); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBlendColor, 253, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(1, 253, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":256
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 * def glBlendEquation(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquation(mode)
 * 
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_mode); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBlendEquation, 256, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(1, 256, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":259
 *     uguugl.glBlendEquation(mode)
 * 
 * def glBlendEquationSeparate(modeRGB, modeAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 */
  __pyx_tuple__33 = PyTuple_Pack(2, __pyx_n_s_modeRGB, __pyx_n_s_modeAlpha); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBlendEquationSeparate, 259, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(1, 259, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":262
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 * def glBlendFunc(sfactor, dfactor):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 */
  __pyx_tuple__35 = PyTuple_Pack(2, __pyx_n_s_sfactor, __pyx_n_s_dfactor); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBlendFunc, 262, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(1, 262, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":265
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 * def glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 */
  __pyx_tuple__37 = PyTuple_Pack(4, __pyx_n_s_sfactorRGB, __pyx_n_s_dfactorRGB, __pyx_n_s_sfactorAlpha, __pyx_n_s_dfactorAlpha); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBlendFuncSeparate, 265, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(1, 265, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":268
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 * def glBufferData(target, size, data, usage):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 */
  __pyx_tuple__39 = PyTuple_Pack(5, __pyx_n_s_target, __pyx_n_s_size, __pyx_n_s_data, __pyx_n_s_usage, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(1, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBufferData, 268, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(1, 268, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":272
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 * 
 * def glBufferSubData(target, offset, size, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 */
  __pyx_tuple__41 = PyTuple_Pack(5, __pyx_n_s_target, __pyx_n_s_offset, __pyx_n_s_size, __pyx_n_s_data, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(1, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glBufferSubData, 272, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(1, 272, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":276
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 * 
 * def glCheckFramebufferStatus(target):             # <<<<<<<<<<<<<<
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 */
  __pyx_tuple__43 = PyTuple_Pack(1, __pyx_n_s_target); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(1, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCheckFramebufferStatus, 276, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(1, 276, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":279
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 * def glClear(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glClear(mask)
 * 
 */
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_n_s_mask); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(1, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glClear, 279, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(1, 279, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":282
 *     uguugl.glClear(mask)
 * 
 * def glClearColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 */
  __pyx_tuple__47 = PyTuple_Pack(4, __pyx_n_s_red, __pyx_n_s_green, __pyx_n_s_blue, __pyx_n_s_alpha); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(1, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __pyx_codeobj__48 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__47, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glClearColor, 282, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__48)) __PYX_ERR(1, 282, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":285
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 * def glClearStencil(s):             # <<<<<<<<<<<<<<
 *     uguugl.glClearStencil(s)
 * 
 */
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_n_s_s); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);
  __pyx_codeobj__50 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__49, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glClearStencil, 285, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__50)) __PYX_ERR(1, 285, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":288
 *     uguugl.glClearStencil(s)
 * 
 * def glColorMask(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 */
  __pyx_tuple__51 = PyTuple_Pack(4, __pyx_n_s_red, __pyx_n_s_green, __pyx_n_s_blue, __pyx_n_s_alpha); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(1, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);
  __pyx_codeobj__52 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__51, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glColorMask, 288, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__52)) __PYX_ERR(1, 288, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":291
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 * def glCompileShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glCompileShader(shader)
 * 
 */
  __pyx_tuple__53 = PyTuple_Pack(1, __pyx_n_s_shader); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(1, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);
  __pyx_codeobj__54 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__53, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCompileShader, 291, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__54)) __PYX_ERR(1, 291, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":294
 *     uguugl.glCompileShader(shader)
 * 
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 */
  __pyx_tuple__55 = PyTuple_Pack(9, __pyx_n_s_target, __pyx_n_s_level, __pyx_n_s_internalformat, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_border, __pyx_n_s_imageSize, __pyx_n_s_data, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(1, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);
  __pyx_codeobj__56 = (PyObject*)__Pyx_PyCode_New(8, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__55, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCompressedTexImage2D, 294, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__56)) __PYX_ERR(1, 294, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":298
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 */
  __pyx_tuple__57 = PyTuple_Pack(10, __pyx_n_s_target, __pyx_n_s_level, __pyx_n_s_xoffset, __pyx_n_s_yoffset, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_format, __pyx_n_s_imageSize, __pyx_n_s_data, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(1, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  __pyx_codeobj__58 = (PyObject*)__Pyx_PyCode_New(9, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__57, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCompressedTexSubImage2D, 298, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__58)) __PYX_ERR(1, 298, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":302
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 */
  __pyx_tuple__59 = PyTuple_Pack(8, __pyx_n_s_target, __pyx_n_s_level, __pyx_n_s_internalformat, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_border); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(1, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);
  __pyx_codeobj__60 = (PyObject*)__Pyx_PyCode_New(8, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__59, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCopyTexImage2D, 302, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__60)) __PYX_ERR(1, 302, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":305
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 * def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 */
  __pyx_tuple__61 = PyTuple_Pack(8, __pyx_n_s_target, __pyx_n_s_level, __pyx_n_s_xoffset, __pyx_n_s_yoffset, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_width, __pyx_n_s_height); if (unlikely(!__pyx_tuple__61)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);
  __pyx_codeobj__62 = (PyObject*)__Pyx_PyCode_New(8, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__61, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCopyTexSubImage2D, 305, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__62)) __PYX_ERR(1, 305, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":308
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 * def glCreateProgram():             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateProgram()
 * 
 */
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCreateProgram, 308, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(1, 308, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":311
 *     return uguugl.glCreateProgram()
 * 
 * def glCreateShader(type):             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateShader(type)
 * 
 */
  __pyx_tuple__64 = PyTuple_Pack(1, __pyx_n_s_type); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(1, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  __pyx_codeobj__65 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__64, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCreateShader, 311, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__65)) __PYX_ERR(1, 311, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":314
 *     return uguugl.glCreateShader(type)
 * 
 * def glCullFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glCullFace(mode)
 * 
 */
  __pyx_tuple__66 = PyTuple_Pack(1, __pyx_n_s_mode); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(1, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);
  __pyx_codeobj__67 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__66, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glCullFace, 314, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__67)) __PYX_ERR(1, 314, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":317
 *     uguugl.glCullFace(mode)
 * 
 * def glDeleteBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 */
  __pyx_tuple__68 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_buffers, __pyx_n_s_buffers_ptr); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(1, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_codeobj__69 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__68, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDeleteBuffers, 317, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__69)) __PYX_ERR(1, 317, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":321
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 * 
 * def glDeleteFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 */
  __pyx_tuple__70 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_framebuffers, __pyx_n_s_framebuffers_ptr); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  __pyx_codeobj__71 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__70, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDeleteFramebuffers, 321, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__71)) __PYX_ERR(1, 321, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":325
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 * 
 * def glDeleteProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteProgram(program)
 * 
 */
  __pyx_tuple__72 = PyTuple_Pack(1, __pyx_n_s_program); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(1, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);
  __pyx_codeobj__73 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__72, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDeleteProgram, 325, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__73)) __PYX_ERR(1, 325, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":328
 *     uguugl.glDeleteProgram(program)
 * 
 * def glDeleteRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 */
  __pyx_tuple__74 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_renderbuffers, __pyx_n_s_renderbuffers_ptr); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(1, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);
  __pyx_codeobj__75 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__74, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDeleteRenderbuffers, 328, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__75)) __PYX_ERR(1, 328, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":332
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glDeleteShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteShader(shader)
 * 
 */
  __pyx_tuple__76 = PyTuple_Pack(1, __pyx_n_s_shader); if (unlikely(!__pyx_tuple__76)) __PYX_ERR(1, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);
  __pyx_codeobj__77 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__76, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDeleteShader, 332, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__77)) __PYX_ERR(1, 332, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":335
 *     uguugl.glDeleteShader(shader)
 * 
 * def glDeleteTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 */
  __pyx_tuple__78 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_textures, __pyx_n_s_textures_ptr); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);
  __pyx_codeobj__79 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__78, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDeleteTextures, 335, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__79)) __PYX_ERR(1, 335, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":339
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 * 
 * def glDepthFunc(func):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthFunc(func)
 * 
 */
  __pyx_tuple__80 = PyTuple_Pack(1, __pyx_n_s_func); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(1, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);
  __pyx_codeobj__81 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__80, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDepthFunc, 339, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__81)) __PYX_ERR(1, 339, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":342
 *     uguugl.glDepthFunc(func)
 * 
 * def glDepthMask(flag):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthMask(flag)
 * 
 */
  __pyx_tuple__82 = PyTuple_Pack(1, __pyx_n_s_flag); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(1, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);
  __pyx_codeobj__83 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__82, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDepthMask, 342, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__83)) __PYX_ERR(1, 342, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":345
 *     uguugl.glDepthMask(flag)
 * 
 * def glDetachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDetachShader(program, shader)
 * 
 */
  __pyx_tuple__84 = PyTuple_Pack(2, __pyx_n_s_program, __pyx_n_s_shader); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(1, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);
  __pyx_codeobj__85 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__84, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDetachShader, 345, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__85)) __PYX_ERR(1, 345, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":348
 *     uguugl.glDetachShader(program, shader)
 * 
 * def glDisable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glDisable(cap)
 * 
 */
  __pyx_tuple__86 = PyTuple_Pack(1, __pyx_n_s_cap); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(1, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);
  __pyx_codeobj__87 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__86, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDisable, 348, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__87)) __PYX_ERR(1, 348, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":351
 *     uguugl.glDisable(cap)
 * 
 * def glDisableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 */
  __pyx_tuple__88 = PyTuple_Pack(1, __pyx_n_s_index); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(1, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);
  __pyx_codeobj__89 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__88, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDisableVertexAttribArray, 351, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__89)) __PYX_ERR(1, 351, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":354
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 * def glDrawArrays(mode, first, count):             # <<<<<<<<<<<<<<
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 */
  __pyx_tuple__90 = PyTuple_Pack(3, __pyx_n_s_mode, __pyx_n_s_first, __pyx_n_s_count); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(1, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);
  __pyx_codeobj__91 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__90, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDrawArrays, 354, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__91)) __PYX_ERR(1, 354, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":357
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 * def glDrawElements(mode, count, type, indices):             # <<<<<<<<<<<<<<
 *     cdef ptr indices_ptr = get_ptr(indices)
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 */
  __pyx_tuple__92 = PyTuple_Pack(5, __pyx_n_s_mode, __pyx_n_s_count, __pyx_n_s_type, __pyx_n_s_indices, __pyx_n_s_indices_ptr); if (unlikely(!__pyx_tuple__92)) __PYX_ERR(1, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);
  __pyx_codeobj__93 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__92, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glDrawElements, 357, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__93)) __PYX_ERR(1, 357, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":361
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 * 
 * def glEnable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glEnable(cap)
 * 
 */
  __pyx_tuple__94 = PyTuple_Pack(1, __pyx_n_s_cap); if (unlikely(!__pyx_tuple__94)) __PYX_ERR(1, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);
  __pyx_codeobj__95 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__94, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glEnable, 361, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__95)) __PYX_ERR(1, 361, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":364
 *     uguugl.glEnable(cap)
 * 
 * def glEnableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 */
  __pyx_tuple__96 = PyTuple_Pack(1, __pyx_n_s_index); if (unlikely(!__pyx_tuple__96)) __PYX_ERR(1, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);
  __pyx_codeobj__97 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__96, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glEnableVertexAttribArray, 364, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__97)) __PYX_ERR(1, 364, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":367
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 * def glFinish():             # <<<<<<<<<<<<<<
 *     uguugl.glFinish()
 * 
 */
  __pyx_codeobj__98 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glFinish, 367, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__98)) __PYX_ERR(1, 367, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":370
 *     uguugl.glFinish()
 * 
 * def glFlush():             # <<<<<<<<<<<<<<
 *     uguugl.glFlush()
 * 
 */
  __pyx_codeobj__99 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glFlush, 370, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__99)) __PYX_ERR(1, 370, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":373
 *     uguugl.glFlush()
 * 
 * def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 */
  __pyx_tuple__100 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_attachment, __pyx_n_s_renderbuffertarget, __pyx_n_s_renderbuffer); if (unlikely(!__pyx_tuple__100)) __PYX_ERR(1, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__100);
  __Pyx_GIVEREF(__pyx_tuple__100);
  __pyx_codeobj__101 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__100, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glFramebufferRenderbuffer, 373, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__101)) __PYX_ERR(1, 373, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":376
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 * def glFramebufferTexture2D(target, attachment, textarget, texture, level):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 */
  __pyx_tuple__102 = PyTuple_Pack(5, __pyx_n_s_target, __pyx_n_s_attachment, __pyx_n_s_textarget, __pyx_n_s_texture, __pyx_n_s_level); if (unlikely(!__pyx_tuple__102)) __PYX_ERR(1, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);
  __pyx_codeobj__103 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__102, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glFramebufferTexture2D, 376, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__103)) __PYX_ERR(1, 376, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":379
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 * def glFrontFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glFrontFace(mode)
 * 
 */
  __pyx_tuple__104 = PyTuple_Pack(1, __pyx_n_s_mode); if (unlikely(!__pyx_tuple__104)) __PYX_ERR(1, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);
  __pyx_codeobj__105 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__104, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glFrontFace, 379, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__105)) __PYX_ERR(1, 379, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":382
 *     uguugl.glFrontFace(mode)
 * 
 * def glGenBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 */
  __pyx_tuple__106 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_buffers, __pyx_n_s_buffers_ptr); if (unlikely(!__pyx_tuple__106)) __PYX_ERR(1, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);
  __pyx_codeobj__107 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__106, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGenBuffers, 382, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__107)) __PYX_ERR(1, 382, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":386
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 * 
 * def glGenFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 */
  __pyx_tuple__108 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_framebuffers, __pyx_n_s_framebuffers_ptr); if (unlikely(!__pyx_tuple__108)) __PYX_ERR(1, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);
  __pyx_codeobj__109 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__108, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGenFramebuffers, 386, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__109)) __PYX_ERR(1, 386, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":390
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 * 
 * def glGenRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 */
  __pyx_tuple__110 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_renderbuffers, __pyx_n_s_renderbuffers_ptr); if (unlikely(!__pyx_tuple__110)) __PYX_ERR(1, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);
  __pyx_codeobj__111 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__110, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGenRenderbuffers, 390, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__111)) __PYX_ERR(1, 390, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":394
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glGenTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 */
  __pyx_tuple__112 = PyTuple_Pack(3, __pyx_n_s_n, __pyx_n_s_textures, __pyx_n_s_textures_ptr); if (unlikely(!__pyx_tuple__112)) __PYX_ERR(1, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);
  __pyx_codeobj__113 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__112, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGenTextures, 394, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__113)) __PYX_ERR(1, 394, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":398
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 * 
 * def glGenerateMipmap(target):             # <<<<<<<<<<<<<<
 *     uguugl.glGenerateMipmap(target)
 * 
 */
  __pyx_tuple__114 = PyTuple_Pack(1, __pyx_n_s_target); if (unlikely(!__pyx_tuple__114)) __PYX_ERR(1, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);
  __pyx_codeobj__115 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__114, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGenerateMipmap, 398, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__115)) __PYX_ERR(1, 398, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":401
 *     uguugl.glGenerateMipmap(target)
 * 
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */
  __pyx_tuple__116 = PyTuple_Pack(11, __pyx_n_s_program, __pyx_n_s_index, __pyx_n_s_bufSize, __pyx_n_s_length, __pyx_n_s_size, __pyx_n_s_type, __pyx_n_s_name, __pyx_n_s_length_ptr, __pyx_n_s_size_ptr, __pyx_n_s_type_ptr, __pyx_n_s_name_ptr); if (unlikely(!__pyx_tuple__116)) __PYX_ERR(1, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);
  __pyx_codeobj__117 = (PyObject*)__Pyx_PyCode_New(7, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__116, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetActiveAttrib, 401, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__117)) __PYX_ERR(1, 401, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":408
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */
  __pyx_tuple__118 = PyTuple_Pack(11, __pyx_n_s_program, __pyx_n_s_index, __pyx_n_s_bufSize, __pyx_n_s_length, __pyx_n_s_size, __pyx_n_s_type, __pyx_n_s_name, __pyx_n_s_length_ptr, __pyx_n_s_size_ptr, __pyx_n_s_type_ptr, __pyx_n_s_name_ptr); if (unlikely(!__pyx_tuple__118)) __PYX_ERR(1, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);
  __pyx_codeobj__119 = (PyObject*)__Pyx_PyCode_New(7, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__118, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetActiveUniform, 408, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__119)) __PYX_ERR(1, 408, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":415
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetAttachedShaders(program, maxCount, count, shaders):             # <<<<<<<<<<<<<<
 *     cdef ptr count_ptr = get_ptr(count)
 *     cdef ptr shaders_ptr = get_ptr(shaders)
 */
  __pyx_tuple__120 = PyTuple_Pack(6, __pyx_n_s_program, __pyx_n_s_maxCount, __pyx_n_s_count, __pyx_n_s_shaders, __pyx_n_s_count_ptr, __pyx_n_s_shaders_ptr); if (unlikely(!__pyx_tuple__120)) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);
  __pyx_codeobj__121 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__120, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetAttachedShaders, 415, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__121)) __PYX_ERR(1, 415, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":420
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)
 * 
 * def glGetAttribLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 */
  __pyx_tuple__122 = PyTuple_Pack(3, __pyx_n_s_program, __pyx_n_s_name, __pyx_n_s_name_ptr); if (unlikely(!__pyx_tuple__122)) __PYX_ERR(1, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);
  __pyx_codeobj__123 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__122, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetAttribLocation, 420, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__123)) __PYX_ERR(1, 420, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":424
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetBooleanv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 */
  __pyx_tuple__124 = PyTuple_Pack(3, __pyx_n_s_pname, __pyx_n_s_data, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__124)) __PYX_ERR(1, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__124);
  __Pyx_GIVEREF(__pyx_tuple__124);
  __pyx_codeobj__125 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__124, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetBooleanv, 424, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__125)) __PYX_ERR(1, 424, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":428
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 * 
 * def glGetBufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__126 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__126)) __PYX_ERR(1, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);
  __pyx_codeobj__127 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__126, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetBufferParameteriv, 428, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__127)) __PYX_ERR(1, 428, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":432
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetError():             # <<<<<<<<<<<<<<
 *     return uguugl.glGetError()
 * 
 */
  __pyx_codeobj__128 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetError, 432, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__128)) __PYX_ERR(1, 432, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":435
 *     return uguugl.glGetError()
 * 
 * def glGetFloatv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 */
  __pyx_tuple__129 = PyTuple_Pack(3, __pyx_n_s_pname, __pyx_n_s_data, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__129)) __PYX_ERR(1, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);
  __pyx_codeobj__130 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__129, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetFloatv, 435, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__130)) __PYX_ERR(1, 435, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":439
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 * 
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__131 = PyTuple_Pack(5, __pyx_n_s_target, __pyx_n_s_attachment, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__131)) __PYX_ERR(1, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__131);
  __Pyx_GIVEREF(__pyx_tuple__131);
  __pyx_codeobj__132 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__131, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetFramebufferAttachmentParame, 439, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__132)) __PYX_ERR(1, 439, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":443
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetIntegerv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 */
  __pyx_tuple__133 = PyTuple_Pack(3, __pyx_n_s_pname, __pyx_n_s_data, __pyx_n_s_data_ptr); if (unlikely(!__pyx_tuple__133)) __PYX_ERR(1, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);
  __pyx_codeobj__134 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__133, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetIntegerv, 443, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__134)) __PYX_ERR(1, 443, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":447
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 * 
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */
  __pyx_tuple__135 = PyTuple_Pack(6, __pyx_n_s_program, __pyx_n_s_bufSize, __pyx_n_s_length, __pyx_n_s_infoLog, __pyx_n_s_length_ptr, __pyx_n_s_infoLog_ptr); if (unlikely(!__pyx_tuple__135)) __PYX_ERR(1, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);
  __pyx_codeobj__136 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__135, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetProgramInfoLog, 447, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__136)) __PYX_ERR(1, 447, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":452
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetProgramiv(program, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__137 = PyTuple_Pack(4, __pyx_n_s_program, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__137)) __PYX_ERR(1, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__137);
  __Pyx_GIVEREF(__pyx_tuple__137);
  __pyx_codeobj__138 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__137, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetProgramiv, 452, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__138)) __PYX_ERR(1, 452, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":456
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetRenderbufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__139 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__139)) __PYX_ERR(1, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__139);
  __Pyx_GIVEREF(__pyx_tuple__139);
  __pyx_codeobj__140 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__139, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetRenderbufferParameteriv, 456, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__140)) __PYX_ERR(1, 456, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":460
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */
  __pyx_tuple__141 = PyTuple_Pack(6, __pyx_n_s_shader, __pyx_n_s_bufSize, __pyx_n_s_length, __pyx_n_s_infoLog, __pyx_n_s_length_ptr, __pyx_n_s_infoLog_ptr); if (unlikely(!__pyx_tuple__141)) __PYX_ERR(1, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__141);
  __Pyx_GIVEREF(__pyx_tuple__141);
  __pyx_codeobj__142 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__141, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetShaderInfoLog, 460, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__142)) __PYX_ERR(1, 460, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":465
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetShaderSource(shader, bufSize, length, source):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr source_ptr = get_ptr(source)
 */
  __pyx_tuple__143 = PyTuple_Pack(6, __pyx_n_s_shader, __pyx_n_s_bufSize, __pyx_n_s_length, __pyx_n_s_source, __pyx_n_s_length_ptr, __pyx_n_s_source_ptr); if (unlikely(!__pyx_tuple__143)) __PYX_ERR(1, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__143);
  __Pyx_GIVEREF(__pyx_tuple__143);
  __pyx_codeobj__144 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__143, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetShaderSource, 465, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__144)) __PYX_ERR(1, 465, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":470
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)
 * 
 * def glGetShaderiv(shader, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__145 = PyTuple_Pack(4, __pyx_n_s_shader, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__145)) __PYX_ERR(1, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__145);
  __Pyx_GIVEREF(__pyx_tuple__145);
  __pyx_codeobj__146 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__145, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetShaderiv, 470, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__146)) __PYX_ERR(1, 470, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":474
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetString(name):             # <<<<<<<<<<<<<<
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 */
  __pyx_tuple__147 = PyTuple_Pack(1, __pyx_n_s_name); if (unlikely(!__pyx_tuple__147)) __PYX_ERR(1, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__147);
  __Pyx_GIVEREF(__pyx_tuple__147);
  __pyx_codeobj__148 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__147, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetString, 474, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__148)) __PYX_ERR(1, 474, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":477
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 * def glGetTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 */
  __pyx_tuple__149 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__149)) __PYX_ERR(1, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__149);
  __Pyx_GIVEREF(__pyx_tuple__149);
  __pyx_codeobj__150 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__149, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetTexParameterfv, 477, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__150)) __PYX_ERR(1, 477, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":481
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__151 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__151)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__151);
  __Pyx_GIVEREF(__pyx_tuple__151);
  __pyx_codeobj__152 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__151, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetTexParameteriv, 481, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__152)) __PYX_ERR(1, 481, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":485
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetUniformLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 */
  __pyx_tuple__153 = PyTuple_Pack(3, __pyx_n_s_program, __pyx_n_s_name, __pyx_n_s_name_ptr); if (unlikely(!__pyx_tuple__153)) __PYX_ERR(1, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__153);
  __Pyx_GIVEREF(__pyx_tuple__153);
  __pyx_codeobj__154 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__153, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetUniformLocation, 485, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__154)) __PYX_ERR(1, 485, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":489
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetUniformfv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 */
  __pyx_tuple__155 = PyTuple_Pack(4, __pyx_n_s_program, __pyx_n_s_location, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__155)) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__155);
  __Pyx_GIVEREF(__pyx_tuple__155);
  __pyx_codeobj__156 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__155, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetUniformfv, 489, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__156)) __PYX_ERR(1, 489, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":493
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetUniformiv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__157 = PyTuple_Pack(4, __pyx_n_s_program, __pyx_n_s_location, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__157)) __PYX_ERR(1, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__157);
  __Pyx_GIVEREF(__pyx_tuple__157);
  __pyx_codeobj__158 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__157, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetUniformiv, 493, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__158)) __PYX_ERR(1, 493, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":497
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 * 
 * def glGetVertexAttribPointerv(index, pname, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 */
  __pyx_tuple__159 = PyTuple_Pack(4, __pyx_n_s_index, __pyx_n_s_pname, __pyx_n_s_pointer, __pyx_n_s_pointer_ptr); if (unlikely(!__pyx_tuple__159)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__159);
  __Pyx_GIVEREF(__pyx_tuple__159);
  __pyx_codeobj__160 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__159, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetVertexAttribPointerv, 497, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__160)) __PYX_ERR(1, 497, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":501
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 * 
 * def glGetVertexAttribfv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 */
  __pyx_tuple__161 = PyTuple_Pack(4, __pyx_n_s_index, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__161)) __PYX_ERR(1, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__161);
  __Pyx_GIVEREF(__pyx_tuple__161);
  __pyx_codeobj__162 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__161, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetVertexAttribfv, 501, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__162)) __PYX_ERR(1, 501, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":505
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetVertexAttribiv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_tuple__163 = PyTuple_Pack(4, __pyx_n_s_index, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__163)) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__163);
  __Pyx_GIVEREF(__pyx_tuple__163);
  __pyx_codeobj__164 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__163, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glGetVertexAttribiv, 505, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__164)) __PYX_ERR(1, 505, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":509
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 * 
 * def glHint(target, mode):             # <<<<<<<<<<<<<<
 *     uguugl.glHint(target, mode)
 * 
 */
  __pyx_tuple__165 = PyTuple_Pack(2, __pyx_n_s_target, __pyx_n_s_mode); if (unlikely(!__pyx_tuple__165)) __PYX_ERR(1, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__165);
  __Pyx_GIVEREF(__pyx_tuple__165);
  __pyx_codeobj__166 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__165, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glHint, 509, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__166)) __PYX_ERR(1, 509, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":512
 *     uguugl.glHint(target, mode)
 * 
 * def glIsBuffer(buffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsBuffer(buffer)
 * 
 */
  __pyx_tuple__167 = PyTuple_Pack(1, __pyx_n_s_buffer); if (unlikely(!__pyx_tuple__167)) __PYX_ERR(1, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__167);
  __Pyx_GIVEREF(__pyx_tuple__167);
  __pyx_codeobj__168 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__167, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsBuffer, 512, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__168)) __PYX_ERR(1, 512, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":515
 *     return uguugl.glIsBuffer(buffer)
 * 
 * def glIsEnabled(cap):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsEnabled(cap)
 * 
 */
  __pyx_tuple__169 = PyTuple_Pack(1, __pyx_n_s_cap); if (unlikely(!__pyx_tuple__169)) __PYX_ERR(1, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__169);
  __Pyx_GIVEREF(__pyx_tuple__169);
  __pyx_codeobj__170 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__169, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsEnabled, 515, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__170)) __PYX_ERR(1, 515, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":518
 *     return uguugl.glIsEnabled(cap)
 * 
 * def glIsFramebuffer(framebuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 */
  __pyx_tuple__171 = PyTuple_Pack(1, __pyx_n_s_framebuffer); if (unlikely(!__pyx_tuple__171)) __PYX_ERR(1, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__171);
  __Pyx_GIVEREF(__pyx_tuple__171);
  __pyx_codeobj__172 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__171, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsFramebuffer, 518, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__172)) __PYX_ERR(1, 518, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":521
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 * def glIsProgram(program):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsProgram(program)
 * 
 */
  __pyx_tuple__173 = PyTuple_Pack(1, __pyx_n_s_program); if (unlikely(!__pyx_tuple__173)) __PYX_ERR(1, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__173);
  __Pyx_GIVEREF(__pyx_tuple__173);
  __pyx_codeobj__174 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__173, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsProgram, 521, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__174)) __PYX_ERR(1, 521, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":524
 *     return uguugl.glIsProgram(program)
 * 
 * def glIsRenderbuffer(renderbuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 */
  __pyx_tuple__175 = PyTuple_Pack(1, __pyx_n_s_renderbuffer); if (unlikely(!__pyx_tuple__175)) __PYX_ERR(1, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__175);
  __Pyx_GIVEREF(__pyx_tuple__175);
  __pyx_codeobj__176 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__175, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsRenderbuffer, 524, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__176)) __PYX_ERR(1, 524, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":527
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 * def glIsShader(shader):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsShader(shader)
 * 
 */
  __pyx_tuple__177 = PyTuple_Pack(1, __pyx_n_s_shader); if (unlikely(!__pyx_tuple__177)) __PYX_ERR(1, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__177);
  __Pyx_GIVEREF(__pyx_tuple__177);
  __pyx_codeobj__178 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__177, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsShader, 527, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__178)) __PYX_ERR(1, 527, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":530
 *     return uguugl.glIsShader(shader)
 * 
 * def glIsTexture(texture):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsTexture(texture)
 * 
 */
  __pyx_tuple__179 = PyTuple_Pack(1, __pyx_n_s_texture); if (unlikely(!__pyx_tuple__179)) __PYX_ERR(1, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__179);
  __Pyx_GIVEREF(__pyx_tuple__179);
  __pyx_codeobj__180 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__179, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glIsTexture, 530, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__180)) __PYX_ERR(1, 530, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":533
 *     return uguugl.glIsTexture(texture)
 * 
 * def glLineWidth(width):             # <<<<<<<<<<<<<<
 *     uguugl.glLineWidth(width)
 * 
 */
  __pyx_tuple__181 = PyTuple_Pack(1, __pyx_n_s_width); if (unlikely(!__pyx_tuple__181)) __PYX_ERR(1, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__181);
  __Pyx_GIVEREF(__pyx_tuple__181);
  __pyx_codeobj__182 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__181, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glLineWidth, 533, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__182)) __PYX_ERR(1, 533, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":536
 *     uguugl.glLineWidth(width)
 * 
 * def glLinkProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glLinkProgram(program)
 * 
 */
  __pyx_tuple__183 = PyTuple_Pack(1, __pyx_n_s_program); if (unlikely(!__pyx_tuple__183)) __PYX_ERR(1, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__183);
  __Pyx_GIVEREF(__pyx_tuple__183);
  __pyx_codeobj__184 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__183, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glLinkProgram, 536, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__184)) __PYX_ERR(1, 536, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":539
 *     uguugl.glLinkProgram(program)
 * 
 * def glPixelStorei(pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glPixelStorei(pname, param)
 * 
 */
  __pyx_tuple__185 = PyTuple_Pack(2, __pyx_n_s_pname, __pyx_n_s_param); if (unlikely(!__pyx_tuple__185)) __PYX_ERR(1, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__185);
  __Pyx_GIVEREF(__pyx_tuple__185);
  __pyx_codeobj__186 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__185, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glPixelStorei, 539, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__186)) __PYX_ERR(1, 539, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":542
 *     uguugl.glPixelStorei(pname, param)
 * 
 * def glPolygonOffset(factor, units):             # <<<<<<<<<<<<<<
 *     uguugl.glPolygonOffset(factor, units)
 * 
 */
  __pyx_tuple__187 = PyTuple_Pack(2, __pyx_n_s_factor, __pyx_n_s_units); if (unlikely(!__pyx_tuple__187)) __PYX_ERR(1, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__187);
  __Pyx_GIVEREF(__pyx_tuple__187);
  __pyx_codeobj__188 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__187, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glPolygonOffset, 542, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__188)) __PYX_ERR(1, 542, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":545
 *     uguugl.glPolygonOffset(factor, units)
 * 
 * def glReadPixels(x, y, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 */
  __pyx_tuple__189 = PyTuple_Pack(8, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_format, __pyx_n_s_type, __pyx_n_s_pixels, __pyx_n_s_pixels_ptr); if (unlikely(!__pyx_tuple__189)) __PYX_ERR(1, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__189);
  __Pyx_GIVEREF(__pyx_tuple__189);
  __pyx_codeobj__190 = (PyObject*)__Pyx_PyCode_New(7, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__189, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glReadPixels, 545, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__190)) __PYX_ERR(1, 545, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":549
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 * 
 * def glRenderbufferStorage(target, internalformat, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 */
  __pyx_tuple__191 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_internalformat, __pyx_n_s_width, __pyx_n_s_height); if (unlikely(!__pyx_tuple__191)) __PYX_ERR(1, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__191);
  __Pyx_GIVEREF(__pyx_tuple__191);
  __pyx_codeobj__192 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__191, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glRenderbufferStorage, 549, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__192)) __PYX_ERR(1, 549, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":552
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 * def glSampleCoverage(value, invert):             # <<<<<<<<<<<<<<
 *     uguugl.glSampleCoverage(value, invert)
 * 
 */
  __pyx_tuple__193 = PyTuple_Pack(2, __pyx_n_s_value, __pyx_n_s_invert); if (unlikely(!__pyx_tuple__193)) __PYX_ERR(1, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__193);
  __Pyx_GIVEREF(__pyx_tuple__193);
  __pyx_codeobj__194 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__193, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glSampleCoverage, 552, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__194)) __PYX_ERR(1, 552, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":555
 *     uguugl.glSampleCoverage(value, invert)
 * 
 * def glScissor(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glScissor(x, y, width, height)
 * 
 */
  __pyx_tuple__195 = PyTuple_Pack(4, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_width, __pyx_n_s_height); if (unlikely(!__pyx_tuple__195)) __PYX_ERR(1, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__195);
  __Pyx_GIVEREF(__pyx_tuple__195);
  __pyx_codeobj__196 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__195, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glScissor, 555, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__196)) __PYX_ERR(1, 555, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":558
 *     uguugl.glScissor(x, y, width, height)
 * 
 * def glShaderSource(shader, count, string, length):             # <<<<<<<<<<<<<<
 *     cdef ptr string_ptr = get_ptr(string)
 *     cdef ptr length_ptr = get_ptr(length)
 */
  __pyx_tuple__197 = PyTuple_Pack(6, __pyx_n_s_shader, __pyx_n_s_count, __pyx_n_s_string, __pyx_n_s_length, __pyx_n_s_string_ptr, __pyx_n_s_length_ptr); if (unlikely(!__pyx_tuple__197)) __PYX_ERR(1, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__197);
  __Pyx_GIVEREF(__pyx_tuple__197);
  __pyx_codeobj__198 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__197, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glShaderSource, 558, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__198)) __PYX_ERR(1, 558, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":563
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)
 * 
 * def glStencilFunc(func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 */
  __pyx_tuple__199 = PyTuple_Pack(3, __pyx_n_s_func, __pyx_n_s_ref, __pyx_n_s_mask); if (unlikely(!__pyx_tuple__199)) __PYX_ERR(1, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__199);
  __Pyx_GIVEREF(__pyx_tuple__199);
  __pyx_codeobj__200 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__199, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glStencilFunc, 563, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__200)) __PYX_ERR(1, 563, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":566
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 * def glStencilFuncSeparate(face, func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 */
  __pyx_tuple__201 = PyTuple_Pack(4, __pyx_n_s_face, __pyx_n_s_func, __pyx_n_s_ref, __pyx_n_s_mask); if (unlikely(!__pyx_tuple__201)) __PYX_ERR(1, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__201);
  __Pyx_GIVEREF(__pyx_tuple__201);
  __pyx_codeobj__202 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__201, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glStencilFuncSeparate, 566, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__202)) __PYX_ERR(1, 566, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":569
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 * def glStencilMask(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMask(mask)
 * 
 */
  __pyx_tuple__203 = PyTuple_Pack(1, __pyx_n_s_mask); if (unlikely(!__pyx_tuple__203)) __PYX_ERR(1, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__203);
  __Pyx_GIVEREF(__pyx_tuple__203);
  __pyx_codeobj__204 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__203, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glStencilMask, 569, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__204)) __PYX_ERR(1, 569, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":572
 *     uguugl.glStencilMask(mask)
 * 
 * def glStencilMaskSeparate(face, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 */
  __pyx_tuple__205 = PyTuple_Pack(2, __pyx_n_s_face, __pyx_n_s_mask); if (unlikely(!__pyx_tuple__205)) __PYX_ERR(1, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__205);
  __Pyx_GIVEREF(__pyx_tuple__205);
  __pyx_codeobj__206 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__205, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glStencilMaskSeparate, 572, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__206)) __PYX_ERR(1, 572, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":575
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 * def glStencilOp(fail, zfail, zpass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 */
  __pyx_tuple__207 = PyTuple_Pack(3, __pyx_n_s_fail, __pyx_n_s_zfail, __pyx_n_s_zpass); if (unlikely(!__pyx_tuple__207)) __PYX_ERR(1, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__207);
  __Pyx_GIVEREF(__pyx_tuple__207);
  __pyx_codeobj__208 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__207, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glStencilOp, 575, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__208)) __PYX_ERR(1, 575, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":578
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 * def glStencilOpSeparate(face, sfail, dpfail, dppass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 */
  __pyx_tuple__209 = PyTuple_Pack(4, __pyx_n_s_face, __pyx_n_s_sfail, __pyx_n_s_dpfail, __pyx_n_s_dppass); if (unlikely(!__pyx_tuple__209)) __PYX_ERR(1, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__209);
  __Pyx_GIVEREF(__pyx_tuple__209);
  __pyx_codeobj__210 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__209, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glStencilOpSeparate, 578, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__210)) __PYX_ERR(1, 578, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":581
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 */
  __pyx_tuple__211 = PyTuple_Pack(10, __pyx_n_s_target, __pyx_n_s_level, __pyx_n_s_internalformat, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_border, __pyx_n_s_format, __pyx_n_s_type, __pyx_n_s_pixels, __pyx_n_s_pixels_ptr); if (unlikely(!__pyx_tuple__211)) __PYX_ERR(1, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__211);
  __Pyx_GIVEREF(__pyx_tuple__211);
  __pyx_codeobj__212 = (PyObject*)__Pyx_PyCode_New(9, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__211, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glTexImage2D, 581, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__212)) __PYX_ERR(1, 581, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":585
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glTexParameterf(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 */
  __pyx_tuple__213 = PyTuple_Pack(3, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_param); if (unlikely(!__pyx_tuple__213)) __PYX_ERR(1, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__213);
  __Pyx_GIVEREF(__pyx_tuple__213);
  __pyx_codeobj__214 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__213, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glTexParameterf, 585, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__214)) __PYX_ERR(1, 585, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":588
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 * def glTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 */
  __pyx_tuple__215 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__215)) __PYX_ERR(1, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__215);
  __Pyx_GIVEREF(__pyx_tuple__215);
  __pyx_codeobj__216 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__215, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glTexParameterfv, 588, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__216)) __PYX_ERR(1, 588, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":592
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 * 
 * def glTexParameteri(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 */
  __pyx_tuple__217 = PyTuple_Pack(3, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_param); if (unlikely(!__pyx_tuple__217)) __PYX_ERR(1, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__217);
  __Pyx_GIVEREF(__pyx_tuple__217);
  __pyx_codeobj__218 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__217, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glTexParameteri, 592, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__218)) __PYX_ERR(1, 592, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":595
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 * def glTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 */
  __pyx_tuple__219 = PyTuple_Pack(4, __pyx_n_s_target, __pyx_n_s_pname, __pyx_n_s_params, __pyx_n_s_params_ptr); if (unlikely(!__pyx_tuple__219)) __PYX_ERR(1, 595, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__219);
  __Pyx_GIVEREF(__pyx_tuple__219);
  __pyx_codeobj__220 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__219, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glTexParameteriv, 595, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__220)) __PYX_ERR(1, 595, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":599
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 * 
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 */
  __pyx_tuple__221 = PyTuple_Pack(10, __pyx_n_s_target, __pyx_n_s_level, __pyx_n_s_xoffset, __pyx_n_s_yoffset, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_format, __pyx_n_s_type, __pyx_n_s_pixels, __pyx_n_s_pixels_ptr); if (unlikely(!__pyx_tuple__221)) __PYX_ERR(1, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__221);
  __Pyx_GIVEREF(__pyx_tuple__221);
  __pyx_codeobj__222 = (PyObject*)__Pyx_PyCode_New(9, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__221, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glTexSubImage2D, 599, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__222)) __PYX_ERR(1, 599, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":603
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glUniform1f(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1f(location, v0)
 * 
 */
  __pyx_tuple__223 = PyTuple_Pack(2, __pyx_n_s_location, __pyx_n_s_v0); if (unlikely(!__pyx_tuple__223)) __PYX_ERR(1, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__223);
  __Pyx_GIVEREF(__pyx_tuple__223);
  __pyx_codeobj__224 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__223, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform1f, 603, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__224)) __PYX_ERR(1, 603, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":606
 *     uguugl.glUniform1f(location, v0)
 * 
 * def glUniform1fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__225 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__225)) __PYX_ERR(1, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__225);
  __Pyx_GIVEREF(__pyx_tuple__225);
  __pyx_codeobj__226 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__225, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform1fv, 606, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__226)) __PYX_ERR(1, 606, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":610
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform1i(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1i(location, v0)
 * 
 */
  __pyx_tuple__227 = PyTuple_Pack(2, __pyx_n_s_location, __pyx_n_s_v0); if (unlikely(!__pyx_tuple__227)) __PYX_ERR(1, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__227);
  __Pyx_GIVEREF(__pyx_tuple__227);
  __pyx_codeobj__228 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__227, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform1i, 610, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__228)) __PYX_ERR(1, 610, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":613
 *     uguugl.glUniform1i(location, v0)
 * 
 * def glUniform1iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_tuple__229 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__229)) __PYX_ERR(1, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__229);
  __Pyx_GIVEREF(__pyx_tuple__229);
  __pyx_codeobj__230 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__229, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform1iv, 613, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__230)) __PYX_ERR(1, 613, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":617
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform2f(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 */
  __pyx_tuple__231 = PyTuple_Pack(3, __pyx_n_s_location, __pyx_n_s_v0, __pyx_n_s_v1); if (unlikely(!__pyx_tuple__231)) __PYX_ERR(1, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__231);
  __Pyx_GIVEREF(__pyx_tuple__231);
  __pyx_codeobj__232 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__231, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform2f, 617, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__232)) __PYX_ERR(1, 617, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":620
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 * def glUniform2fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__233 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__233)) __PYX_ERR(1, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__233);
  __Pyx_GIVEREF(__pyx_tuple__233);
  __pyx_codeobj__234 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__233, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform2fv, 620, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__234)) __PYX_ERR(1, 620, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":624
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform2i(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 */
  __pyx_tuple__235 = PyTuple_Pack(3, __pyx_n_s_location, __pyx_n_s_v0, __pyx_n_s_v1); if (unlikely(!__pyx_tuple__235)) __PYX_ERR(1, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__235);
  __Pyx_GIVEREF(__pyx_tuple__235);
  __pyx_codeobj__236 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__235, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform2i, 624, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__236)) __PYX_ERR(1, 624, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":627
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 * def glUniform2iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_tuple__237 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__237)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__237);
  __Pyx_GIVEREF(__pyx_tuple__237);
  __pyx_codeobj__238 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__237, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform2iv, 627, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__238)) __PYX_ERR(1, 627, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":631
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform3f(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 */
  __pyx_tuple__239 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_v0, __pyx_n_s_v1, __pyx_n_s_v2); if (unlikely(!__pyx_tuple__239)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__239);
  __Pyx_GIVEREF(__pyx_tuple__239);
  __pyx_codeobj__240 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__239, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform3f, 631, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__240)) __PYX_ERR(1, 631, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":634
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 * def glUniform3fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__241 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__241)) __PYX_ERR(1, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__241);
  __Pyx_GIVEREF(__pyx_tuple__241);
  __pyx_codeobj__242 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__241, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform3fv, 634, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__242)) __PYX_ERR(1, 634, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":638
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform3i(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 */
  __pyx_tuple__243 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_v0, __pyx_n_s_v1, __pyx_n_s_v2); if (unlikely(!__pyx_tuple__243)) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__243);
  __Pyx_GIVEREF(__pyx_tuple__243);
  __pyx_codeobj__244 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__243, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform3i, 638, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__244)) __PYX_ERR(1, 638, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":641
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 * def glUniform3iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_tuple__245 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__245)) __PYX_ERR(1, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__245);
  __Pyx_GIVEREF(__pyx_tuple__245);
  __pyx_codeobj__246 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__245, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform3iv, 641, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__246)) __PYX_ERR(1, 641, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":645
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform4f(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 */
  __pyx_tuple__247 = PyTuple_Pack(5, __pyx_n_s_location, __pyx_n_s_v0, __pyx_n_s_v1, __pyx_n_s_v2, __pyx_n_s_v3); if (unlikely(!__pyx_tuple__247)) __PYX_ERR(1, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__247);
  __Pyx_GIVEREF(__pyx_tuple__247);
  __pyx_codeobj__248 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__247, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform4f, 645, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__248)) __PYX_ERR(1, 645, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":648
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 * def glUniform4fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__249 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__249)) __PYX_ERR(1, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__249);
  __Pyx_GIVEREF(__pyx_tuple__249);
  __pyx_codeobj__250 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__249, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform4fv, 648, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__250)) __PYX_ERR(1, 648, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":652
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform4i(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 */
  __pyx_tuple__251 = PyTuple_Pack(5, __pyx_n_s_location, __pyx_n_s_v0, __pyx_n_s_v1, __pyx_n_s_v2, __pyx_n_s_v3); if (unlikely(!__pyx_tuple__251)) __PYX_ERR(1, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__251);
  __Pyx_GIVEREF(__pyx_tuple__251);
  __pyx_codeobj__252 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__251, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform4i, 652, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__252)) __PYX_ERR(1, 652, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":655
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 * def glUniform4iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_tuple__253 = PyTuple_Pack(4, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__253)) __PYX_ERR(1, 655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__253);
  __Pyx_GIVEREF(__pyx_tuple__253);
  __pyx_codeobj__254 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__253, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniform4iv, 655, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__254)) __PYX_ERR(1, 655, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":659
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniformMatrix2fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__255 = PyTuple_Pack(5, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_transpose, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__255)) __PYX_ERR(1, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__255);
  __Pyx_GIVEREF(__pyx_tuple__255);
  __pyx_codeobj__256 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__255, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniformMatrix2fv, 659, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__256)) __PYX_ERR(1, 659, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":663
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix3fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__257 = PyTuple_Pack(5, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_transpose, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__257)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__257);
  __Pyx_GIVEREF(__pyx_tuple__257);
  __pyx_codeobj__258 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__257, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniformMatrix3fv, 663, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__258)) __PYX_ERR(1, 663, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":667
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix4fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_tuple__259 = PyTuple_Pack(5, __pyx_n_s_location, __pyx_n_s_count, __pyx_n_s_transpose, __pyx_n_s_value, __pyx_n_s_value_ptr); if (unlikely(!__pyx_tuple__259)) __PYX_ERR(1, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__259);
  __Pyx_GIVEREF(__pyx_tuple__259);
  __pyx_codeobj__260 = (PyObject*)__Pyx_PyCode_New(4, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__259, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUniformMatrix4fv, 667, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__260)) __PYX_ERR(1, 667, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":671
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUseProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glUseProgram(program)
 * 
 */
  __pyx_tuple__261 = PyTuple_Pack(1, __pyx_n_s_program); if (unlikely(!__pyx_tuple__261)) __PYX_ERR(1, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__261);
  __Pyx_GIVEREF(__pyx_tuple__261);
  __pyx_codeobj__262 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__261, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glUseProgram, 671, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__262)) __PYX_ERR(1, 671, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":674
 *     uguugl.glUseProgram(program)
 * 
 * def glValidateProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glValidateProgram(program)
 * 
 */
  __pyx_tuple__263 = PyTuple_Pack(1, __pyx_n_s_program); if (unlikely(!__pyx_tuple__263)) __PYX_ERR(1, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__263);
  __Pyx_GIVEREF(__pyx_tuple__263);
  __pyx_codeobj__264 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__263, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glValidateProgram, 674, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__264)) __PYX_ERR(1, 674, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":677
 *     uguugl.glValidateProgram(program)
 * 
 * def glVertexAttrib1f(index, x):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 */
  __pyx_tuple__265 = PyTuple_Pack(2, __pyx_n_s_index, __pyx_n_s_x); if (unlikely(!__pyx_tuple__265)) __PYX_ERR(1, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__265);
  __Pyx_GIVEREF(__pyx_tuple__265);
  __pyx_codeobj__266 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__265, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib1f, 677, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__266)) __PYX_ERR(1, 677, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":680
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 * def glVertexAttrib1fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_tuple__267 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_v, __pyx_n_s_v_ptr); if (unlikely(!__pyx_tuple__267)) __PYX_ERR(1, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__267);
  __Pyx_GIVEREF(__pyx_tuple__267);
  __pyx_codeobj__268 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__267, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib1fv, 680, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__268)) __PYX_ERR(1, 680, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":684
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib2f(index, x, y):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 */
  __pyx_tuple__269 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_x, __pyx_n_s_y); if (unlikely(!__pyx_tuple__269)) __PYX_ERR(1, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__269);
  __Pyx_GIVEREF(__pyx_tuple__269);
  __pyx_codeobj__270 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__269, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib2f, 684, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__270)) __PYX_ERR(1, 684, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":687
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 * def glVertexAttrib2fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_tuple__271 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_v, __pyx_n_s_v_ptr); if (unlikely(!__pyx_tuple__271)) __PYX_ERR(1, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__271);
  __Pyx_GIVEREF(__pyx_tuple__271);
  __pyx_codeobj__272 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__271, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib2fv, 687, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__272)) __PYX_ERR(1, 687, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":691
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib3f(index, x, y, z):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 */
  __pyx_tuple__273 = PyTuple_Pack(4, __pyx_n_s_index, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_z); if (unlikely(!__pyx_tuple__273)) __PYX_ERR(1, 691, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__273);
  __Pyx_GIVEREF(__pyx_tuple__273);
  __pyx_codeobj__274 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__273, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib3f, 691, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__274)) __PYX_ERR(1, 691, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":694
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 * def glVertexAttrib3fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_tuple__275 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_v, __pyx_n_s_v_ptr); if (unlikely(!__pyx_tuple__275)) __PYX_ERR(1, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__275);
  __Pyx_GIVEREF(__pyx_tuple__275);
  __pyx_codeobj__276 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__275, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib3fv, 694, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__276)) __PYX_ERR(1, 694, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":698
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib4f(index, x, y, z, w):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 */
  __pyx_tuple__277 = PyTuple_Pack(5, __pyx_n_s_index, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_z, __pyx_n_s_w); if (unlikely(!__pyx_tuple__277)) __PYX_ERR(1, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__277);
  __Pyx_GIVEREF(__pyx_tuple__277);
  __pyx_codeobj__278 = (PyObject*)__Pyx_PyCode_New(5, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__277, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib4f, 698, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__278)) __PYX_ERR(1, 698, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":701
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 * def glVertexAttrib4fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_tuple__279 = PyTuple_Pack(3, __pyx_n_s_index, __pyx_n_s_v, __pyx_n_s_v_ptr); if (unlikely(!__pyx_tuple__279)) __PYX_ERR(1, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__279);
  __Pyx_GIVEREF(__pyx_tuple__279);
  __pyx_codeobj__280 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__279, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttrib4fv, 701, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__280)) __PYX_ERR(1, 701, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":705
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 */
  __pyx_tuple__281 = PyTuple_Pack(7, __pyx_n_s_index, __pyx_n_s_size, __pyx_n_s_type, __pyx_n_s_normalized, __pyx_n_s_stride, __pyx_n_s_pointer, __pyx_n_s_pointer_ptr); if (unlikely(!__pyx_tuple__281)) __PYX_ERR(1, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__281);
  __Pyx_GIVEREF(__pyx_tuple__281);
  __pyx_codeobj__282 = (PyObject*)__Pyx_PyCode_New(6, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__281, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glVertexAttribPointer, 705, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__282)) __PYX_ERR(1, 705, __pyx_L1_error)

  /* "renpy/gl2/uguu.pyx":709
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 * 
 * def glViewport(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glViewport(x, y, width, height)
 * 
 */
  __pyx_tuple__283 = PyTuple_Pack(4, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_width, __pyx_n_s_height); if (unlikely(!__pyx_tuple__283)) __PYX_ERR(1, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__283);
  __Pyx_GIVEREF(__pyx_tuple__283);
  __pyx_codeobj__284 = (PyObject*)__Pyx_PyCode_New(4, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__283, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_glViewport, 709, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__284)) __PYX_ERR(1, 709, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(1, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC inituguu(void); /*proto*/
PyMODINIT_FUNC inituguu(void)
#else
PyMODINIT_FUNC PyInit_uguu(void); /*proto*/
PyMODINIT_FUNC PyInit_uguu(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_uguu(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_uguu(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("uguu", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__gl2__uguu) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.gl2.uguu")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.gl2.uguu", __pyx_m) < 0)) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_4uguu_ptr) < 0) __PYX_ERR(1, 56, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_4uguu_ptr.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "ptr", (PyObject *)&__pyx_type_5renpy_3gl2_4uguu_ptr) < 0) __PYX_ERR(1, 56, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_4uguu_ptr) < 0) __PYX_ERR(1, 56, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_4uguu_ptr = &__pyx_type_5renpy_3gl2_4uguu_ptr;
  __pyx_vtabptr_5renpy_3gl2_4uguu_Buffer = &__pyx_vtable_5renpy_3gl2_4uguu_Buffer;
  __pyx_vtable_5renpy_3gl2_4uguu_Buffer.setup_buffer = (PyObject *(*)(struct __pyx_obj_5renpy_3gl2_4uguu_Buffer *, Py_ssize_t, Py_ssize_t, char const *, int))__pyx_f_5renpy_3gl2_4uguu_6Buffer_setup_buffer;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_4uguu_Buffer) < 0) __PYX_ERR(1, 89, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_4uguu_Buffer.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_4uguu_Buffer.tp_dict, __pyx_vtabptr_5renpy_3gl2_4uguu_Buffer) < 0) __PYX_ERR(1, 89, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Buffer", (PyObject *)&__pyx_type_5renpy_3gl2_4uguu_Buffer) < 0) __PYX_ERR(1, 89, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_4uguu_Buffer) < 0) __PYX_ERR(1, 89, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_4uguu_Buffer = &__pyx_type_5renpy_3gl2_4uguu_Buffer;
  __pyx_vtabptr_5renpy_3gl2_4uguu_BytesBuffer = &__pyx_vtable_5renpy_3gl2_4uguu_BytesBuffer;
  __pyx_vtable_5renpy_3gl2_4uguu_BytesBuffer.__pyx_base = *__pyx_vtabptr_5renpy_3gl2_4uguu_Buffer;
  __pyx_type_5renpy_3gl2_4uguu_BytesBuffer.tp_base = __pyx_ptype_5renpy_3gl2_4uguu_Buffer;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_4uguu_BytesBuffer) < 0) __PYX_ERR(1, 143, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_4uguu_BytesBuffer.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_4uguu_BytesBuffer.tp_dict, __pyx_vtabptr_5renpy_3gl2_4uguu_BytesBuffer) < 0) __PYX_ERR(1, 143, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "BytesBuffer", (PyObject *)&__pyx_type_5renpy_3gl2_4uguu_BytesBuffer) < 0) __PYX_ERR(1, 143, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_4uguu_BytesBuffer) < 0) __PYX_ERR(1, 143, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_4uguu_BytesBuffer = &__pyx_type_5renpy_3gl2_4uguu_BytesBuffer;
  __pyx_vtabptr_5renpy_3gl2_4uguu_BytesListBuffer = &__pyx_vtable_5renpy_3gl2_4uguu_BytesListBuffer;
  __pyx_vtable_5renpy_3gl2_4uguu_BytesListBuffer.__pyx_base = *__pyx_vtabptr_5renpy_3gl2_4uguu_Buffer;
  __pyx_type_5renpy_3gl2_4uguu_BytesListBuffer.tp_base = __pyx_ptype_5renpy_3gl2_4uguu_Buffer;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_4uguu_BytesListBuffer) < 0) __PYX_ERR(1, 152, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_4uguu_BytesListBuffer.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_4uguu_BytesListBuffer.tp_dict, __pyx_vtabptr_5renpy_3gl2_4uguu_BytesListBuffer) < 0) __PYX_ERR(1, 152, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "BytesListBuffer", (PyObject *)&__pyx_type_5renpy_3gl2_4uguu_BytesListBuffer) < 0) __PYX_ERR(1, 152, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_4uguu_BytesListBuffer) < 0) __PYX_ERR(1, 152, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_4uguu_BytesListBuffer = &__pyx_type_5renpy_3gl2_4uguu_BytesListBuffer;
  __pyx_vtabptr_5renpy_3gl2_4uguu_IntBuffer = &__pyx_vtable_5renpy_3gl2_4uguu_IntBuffer;
  __pyx_vtable_5renpy_3gl2_4uguu_IntBuffer.__pyx_base = *__pyx_vtabptr_5renpy_3gl2_4uguu_Buffer;
  __pyx_type_5renpy_3gl2_4uguu_IntBuffer.tp_base = __pyx_ptype_5renpy_3gl2_4uguu_Buffer;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_4uguu_IntBuffer) < 0) __PYX_ERR(1, 164, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_4uguu_IntBuffer.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_4uguu_IntBuffer.tp_dict, __pyx_vtabptr_5renpy_3gl2_4uguu_IntBuffer) < 0) __PYX_ERR(1, 164, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "IntBuffer", (PyObject *)&__pyx_type_5renpy_3gl2_4uguu_IntBuffer) < 0) __PYX_ERR(1, 164, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_4uguu_IntBuffer) < 0) __PYX_ERR(1, 164, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_4uguu_IntBuffer = &__pyx_type_5renpy_3gl2_4uguu_IntBuffer;
  __pyx_vtabptr_5renpy_3gl2_4uguu_FloatBuffer = &__pyx_vtable_5renpy_3gl2_4uguu_FloatBuffer;
  __pyx_vtable_5renpy_3gl2_4uguu_FloatBuffer.__pyx_base = *__pyx_vtabptr_5renpy_3gl2_4uguu_Buffer;
  __pyx_type_5renpy_3gl2_4uguu_FloatBuffer.tp_base = __pyx_ptype_5renpy_3gl2_4uguu_Buffer;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_4uguu_FloatBuffer) < 0) __PYX_ERR(1, 181, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_4uguu_FloatBuffer.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_4uguu_FloatBuffer.tp_dict, __pyx_vtabptr_5renpy_3gl2_4uguu_FloatBuffer) < 0) __PYX_ERR(1, 181, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "FloatBuffer", (PyObject *)&__pyx_type_5renpy_3gl2_4uguu_FloatBuffer) < 0) __PYX_ERR(1, 181, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_4uguu_FloatBuffer) < 0) __PYX_ERR(1, 181, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_4uguu_FloatBuffer = &__pyx_type_5renpy_3gl2_4uguu_FloatBuffer;
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("renpy.gl2.uguugl"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glActiveTexture", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glActiveTexture, "__pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glAttachShader", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glAttachShader, "__pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBindAttribLocation", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBindAttribLocation, "__pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBindBuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBindBuffer, "__pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBindFramebuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBindFramebuffer, "__pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBindRenderbuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBindRenderbuffer, "__pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBindTexture", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBindTexture, "__pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBlendColor", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBlendColor, "__pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBlendEquation", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBlendEquation, "__pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBlendEquationSeparate", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBlendEquationSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBlendFunc", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBlendFunc, "__pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBlendFuncSeparate", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBlendFuncSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBufferData", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBufferData, "__pyx_t_5renpy_3gl2_6uguugl_glBufferData_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glBufferSubData", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glBufferSubData, "__pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCheckFramebufferStatus", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCheckFramebufferStatus, "__pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glClear", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glClear, "__pyx_t_5renpy_3gl2_6uguugl_glClear_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glClearColor", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glClearColor, "__pyx_t_5renpy_3gl2_6uguugl_glClearColor_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glClearStencil", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glClearStencil, "__pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glColorMask", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glColorMask, "__pyx_t_5renpy_3gl2_6uguugl_glColorMask_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCompileShader", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCompileShader, "__pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCompressedTexImage2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCompressedTexImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCompressedTexSubImage2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCopyTexImage2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCopyTexImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCopyTexSubImage2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCopyTexSubImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCreateProgram", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCreateProgram, "__pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCreateShader", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCreateShader, "__pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glCullFace", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glCullFace, "__pyx_t_5renpy_3gl2_6uguugl_glCullFace_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDeleteBuffers", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDeleteBuffers, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDeleteFramebuffers", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDeleteFramebuffers, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDeleteProgram", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDeleteProgram, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDeleteRenderbuffers", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDeleteRenderbuffers, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDeleteShader", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDeleteShader, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDeleteTextures", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDeleteTextures, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDepthFunc", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDepthFunc, "__pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDepthMask", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDepthMask, "__pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDetachShader", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDetachShader, "__pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDisable", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDisable, "__pyx_t_5renpy_3gl2_6uguugl_glDisable_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDisableVertexAttribArray", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDisableVertexAttribArray, "__pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDrawArrays", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDrawArrays, "__pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glDrawElements", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glDrawElements, "__pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glEnable", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glEnable, "__pyx_t_5renpy_3gl2_6uguugl_glEnable_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glEnableVertexAttribArray", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glEnableVertexAttribArray, "__pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glFinish", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glFinish, "__pyx_t_5renpy_3gl2_6uguugl_glFinish_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glFlush", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glFlush, "__pyx_t_5renpy_3gl2_6uguugl_glFlush_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glFramebufferRenderbuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer, "__pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glFramebufferTexture2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glFramebufferTexture2D, "__pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glFrontFace", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glFrontFace, "__pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGenBuffers", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGenBuffers, "__pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGenFramebuffers", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGenFramebuffers, "__pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGenRenderbuffers", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGenRenderbuffers, "__pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGenTextures", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGenTextures, "__pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGenerateMipmap", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGenerateMipmap, "__pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetActiveAttrib", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetActiveAttrib, "__pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetActiveUniform", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetActiveUniform, "__pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetAttachedShaders", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetAttachedShaders, "__pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetAttribLocation", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetAttribLocation, "__pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetBooleanv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetBooleanv, "__pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetBufferParameteriv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetBufferParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetError", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetError, "__pyx_t_5renpy_3gl2_6uguugl_glGetError_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetFloatv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetFloatv, "__pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetFramebufferAttachmentParameteriv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetIntegerv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetIntegerv, "__pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetProgramInfoLog", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetProgramInfoLog, "__pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetProgramiv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetProgramiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetRenderbufferParameteriv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetShaderInfoLog", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderInfoLog, "__pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetShaderSource", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderSource, "__pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetShaderiv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetShaderiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetString", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetString, "__pyx_t_5renpy_3gl2_6uguugl_glGetString_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetTexParameterfv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetTexParameterfv, "__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetTexParameteriv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetTexParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetUniformLocation", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformLocation, "__pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetUniformfv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformfv, "__pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetUniformiv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetUniformiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetVertexAttribPointerv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv, "__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetVertexAttribfv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribfv, "__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glGetVertexAttribiv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glGetVertexAttribiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glHint", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glHint, "__pyx_t_5renpy_3gl2_6uguugl_glHint_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsBuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsBuffer, "__pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsEnabled", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsEnabled, "__pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsFramebuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsFramebuffer, "__pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsProgram", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsProgram, "__pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsRenderbuffer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsRenderbuffer, "__pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsShader", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsShader, "__pyx_t_5renpy_3gl2_6uguugl_glIsShader_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glIsTexture", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glIsTexture, "__pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glLineWidth", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glLineWidth, "__pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glLinkProgram", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glLinkProgram, "__pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glPixelStorei", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glPixelStorei, "__pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glPolygonOffset", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glPolygonOffset, "__pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glReadPixels", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glReadPixels, "__pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glRenderbufferStorage", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glRenderbufferStorage, "__pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glSampleCoverage", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glSampleCoverage, "__pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glScissor", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glScissor, "__pyx_t_5renpy_3gl2_6uguugl_glScissor_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glShaderSource", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glShaderSource, "__pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glStencilFunc", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glStencilFunc, "__pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glStencilFuncSeparate", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glStencilFuncSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glStencilMask", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glStencilMask, "__pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glStencilMaskSeparate", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glStencilMaskSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glStencilOp", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glStencilOp, "__pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glStencilOpSeparate", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glStencilOpSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glTexImage2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glTexImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glTexParameterf", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glTexParameterf, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glTexParameterfv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glTexParameterfv, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glTexParameteri", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glTexParameteri, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glTexParameteriv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glTexParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glTexSubImage2D", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glTexSubImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform1f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform1f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform1fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform1fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform1i", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform1i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform1iv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform1iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform2f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform2f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform2fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform2fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform2i", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform2i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform2iv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform2iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform3f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform3f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform3fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform3fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform3i", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform3i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform3iv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform3iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform4f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform4f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform4fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform4fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform4i", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform4i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniform4iv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniform4iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniformMatrix2fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix2fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniformMatrix3fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix3fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUniformMatrix4fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUniformMatrix4fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glUseProgram", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glUseProgram, "__pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glValidateProgram", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glValidateProgram, "__pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib1f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib1f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib1fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib1fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib2f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib2f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib2fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib2fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib3f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib3f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib3fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib3fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib4f", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib4f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttrib4fv", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttrib4fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glVertexAttribPointer", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glVertexAttribPointer, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glViewport", (void **)&__pyx_vp_5renpy_3gl2_6uguugl_glViewport, "__pyx_t_5renpy_3gl2_6uguugl_glViewport_type") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "renpy/gl2/uguu.pyx":41
 * 
 * cimport uguugl
 * from uguugl import load             # <<<<<<<<<<<<<<
 * 
 * cdef object proxy_return_string(const GLubyte *s):
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_load);
  __Pyx_GIVEREF(__pyx_n_s_load);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_load);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_uguugl, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_load); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_load, __pyx_t_2) < 0) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":231
 * from uguugl cimport GLuint64EXT
 * 
 * def glActiveTexture(texture):             # <<<<<<<<<<<<<<
 *     uguugl.glActiveTexture(texture)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_1glActiveTexture, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glActiveTexture, __pyx_t_3) < 0) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":234
 *     uguugl.glActiveTexture(texture)
 * 
 * def glAttachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glAttachShader(program, shader)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_3glAttachShader, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glAttachShader, __pyx_t_3) < 0) __PYX_ERR(1, 234, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":237
 *     uguugl.glAttachShader(program, shader)
 * 
 * def glBindAttribLocation(program, index, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_5glBindAttribLocation, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBindAttribLocation, __pyx_t_3) < 0) __PYX_ERR(1, 237, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":241
 *     uguugl.glBindAttribLocation(program, index, <const GLchar *> name_ptr.ptr)
 * 
 * def glBindBuffer(target, buffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindBuffer(target, buffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_7glBindBuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBindBuffer, __pyx_t_3) < 0) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":244
 *     uguugl.glBindBuffer(target, buffer)
 * 
 * def glBindFramebuffer(target, framebuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_9glBindFramebuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBindFramebuffer, __pyx_t_3) < 0) __PYX_ERR(1, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":247
 *     uguugl.glBindFramebuffer(target, framebuffer)
 * 
 * def glBindRenderbuffer(target, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_11glBindRenderbuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBindRenderbuffer, __pyx_t_3) < 0) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":250
 *     uguugl.glBindRenderbuffer(target, renderbuffer)
 * 
 * def glBindTexture(target, texture):             # <<<<<<<<<<<<<<
 *     uguugl.glBindTexture(target, texture)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_13glBindTexture, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBindTexture, __pyx_t_3) < 0) __PYX_ERR(1, 250, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":253
 *     uguugl.glBindTexture(target, texture)
 * 
 * def glBlendColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_15glBlendColor, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBlendColor, __pyx_t_3) < 0) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":256
 *     uguugl.glBlendColor(red, green, blue, alpha)
 * 
 * def glBlendEquation(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquation(mode)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_17glBlendEquation, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBlendEquation, __pyx_t_3) < 0) __PYX_ERR(1, 256, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":259
 *     uguugl.glBlendEquation(mode)
 * 
 * def glBlendEquationSeparate(modeRGB, modeAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_19glBlendEquationSeparate, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBlendEquationSeparate, __pyx_t_3) < 0) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":262
 *     uguugl.glBlendEquationSeparate(modeRGB, modeAlpha)
 * 
 * def glBlendFunc(sfactor, dfactor):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_21glBlendFunc, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBlendFunc, __pyx_t_3) < 0) __PYX_ERR(1, 262, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":265
 *     uguugl.glBlendFunc(sfactor, dfactor)
 * 
 * def glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha):             # <<<<<<<<<<<<<<
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_23glBlendFuncSeparate, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBlendFuncSeparate, __pyx_t_3) < 0) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":268
 *     uguugl.glBlendFuncSeparate(sfactorRGB, dfactorRGB, sfactorAlpha, dfactorAlpha)
 * 
 * def glBufferData(target, size, data, usage):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_25glBufferData, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 268, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBufferData, __pyx_t_3) < 0) __PYX_ERR(1, 268, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":272
 *     uguugl.glBufferData(target, size, <const void *> data_ptr.ptr, usage)
 * 
 * def glBufferSubData(target, offset, size, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_27glBufferSubData, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glBufferSubData, __pyx_t_3) < 0) __PYX_ERR(1, 272, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":276
 *     uguugl.glBufferSubData(target, offset, size, <const void *> data_ptr.ptr)
 * 
 * def glCheckFramebufferStatus(target):             # <<<<<<<<<<<<<<
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_29glCheckFramebufferStatus, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCheckFramebufferStatus, __pyx_t_3) < 0) __PYX_ERR(1, 276, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":279
 *     return uguugl.glCheckFramebufferStatus(target)
 * 
 * def glClear(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glClear(mask)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_31glClear, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glClear, __pyx_t_3) < 0) __PYX_ERR(1, 279, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":282
 *     uguugl.glClear(mask)
 * 
 * def glClearColor(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_33glClearColor, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glClearColor, __pyx_t_3) < 0) __PYX_ERR(1, 282, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":285
 *     uguugl.glClearColor(red, green, blue, alpha)
 * 
 * def glClearStencil(s):             # <<<<<<<<<<<<<<
 *     uguugl.glClearStencil(s)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_35glClearStencil, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glClearStencil, __pyx_t_3) < 0) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":288
 *     uguugl.glClearStencil(s)
 * 
 * def glColorMask(red, green, blue, alpha):             # <<<<<<<<<<<<<<
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_37glColorMask, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glColorMask, __pyx_t_3) < 0) __PYX_ERR(1, 288, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":291
 *     uguugl.glColorMask(red, green, blue, alpha)
 * 
 * def glCompileShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glCompileShader(shader)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_39glCompileShader, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 291, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCompileShader, __pyx_t_3) < 0) __PYX_ERR(1, 291, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":294
 *     uguugl.glCompileShader(shader)
 * 
 * def glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_41glCompressedTexImage2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCompressedTexImage2D, __pyx_t_3) < 0) __PYX_ERR(1, 294, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":298
 *     uguugl.glCompressedTexImage2D(target, level, internalformat, width, height, border, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_43glCompressedTexSubImage2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCompressedTexSubImage2D, __pyx_t_3) < 0) __PYX_ERR(1, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":302
 *     uguugl.glCompressedTexSubImage2D(target, level, xoffset, yoffset, width, height, format, imageSize, <const void *> data_ptr.ptr)
 * 
 * def glCopyTexImage2D(target, level, internalformat, x, y, width, height, border):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_45glCopyTexImage2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCopyTexImage2D, __pyx_t_3) < 0) __PYX_ERR(1, 302, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":305
 *     uguugl.glCopyTexImage2D(target, level, internalformat, x, y, width, height, border)
 * 
 * def glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_47glCopyTexSubImage2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCopyTexSubImage2D, __pyx_t_3) < 0) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":308
 *     uguugl.glCopyTexSubImage2D(target, level, xoffset, yoffset, x, y, width, height)
 * 
 * def glCreateProgram():             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateProgram()
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_49glCreateProgram, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCreateProgram, __pyx_t_3) < 0) __PYX_ERR(1, 308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":311
 *     return uguugl.glCreateProgram()
 * 
 * def glCreateShader(type):             # <<<<<<<<<<<<<<
 *     return uguugl.glCreateShader(type)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_51glCreateShader, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCreateShader, __pyx_t_3) < 0) __PYX_ERR(1, 311, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":314
 *     return uguugl.glCreateShader(type)
 * 
 * def glCullFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glCullFace(mode)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_53glCullFace, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glCullFace, __pyx_t_3) < 0) __PYX_ERR(1, 314, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":317
 *     uguugl.glCullFace(mode)
 * 
 * def glDeleteBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_55glDeleteBuffers, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDeleteBuffers, __pyx_t_3) < 0) __PYX_ERR(1, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":321
 *     uguugl.glDeleteBuffers(n, <const GLuint *> buffers_ptr.ptr)
 * 
 * def glDeleteFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_57glDeleteFramebuffers, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDeleteFramebuffers, __pyx_t_3) < 0) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":325
 *     uguugl.glDeleteFramebuffers(n, <const GLuint *> framebuffers_ptr.ptr)
 * 
 * def glDeleteProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteProgram(program)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_59glDeleteProgram, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDeleteProgram, __pyx_t_3) < 0) __PYX_ERR(1, 325, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":328
 *     uguugl.glDeleteProgram(program)
 * 
 * def glDeleteRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_61glDeleteRenderbuffers, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDeleteRenderbuffers, __pyx_t_3) < 0) __PYX_ERR(1, 328, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":332
 *     uguugl.glDeleteRenderbuffers(n, <const GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glDeleteShader(shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDeleteShader(shader)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_63glDeleteShader, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDeleteShader, __pyx_t_3) < 0) __PYX_ERR(1, 332, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":335
 *     uguugl.glDeleteShader(shader)
 * 
 * def glDeleteTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_65glDeleteTextures, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDeleteTextures, __pyx_t_3) < 0) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":339
 *     uguugl.glDeleteTextures(n, <const GLuint *> textures_ptr.ptr)
 * 
 * def glDepthFunc(func):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthFunc(func)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_67glDepthFunc, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDepthFunc, __pyx_t_3) < 0) __PYX_ERR(1, 339, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":342
 *     uguugl.glDepthFunc(func)
 * 
 * def glDepthMask(flag):             # <<<<<<<<<<<<<<
 *     uguugl.glDepthMask(flag)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_69glDepthMask, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 342, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDepthMask, __pyx_t_3) < 0) __PYX_ERR(1, 342, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":345
 *     uguugl.glDepthMask(flag)
 * 
 * def glDetachShader(program, shader):             # <<<<<<<<<<<<<<
 *     uguugl.glDetachShader(program, shader)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_71glDetachShader, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDetachShader, __pyx_t_3) < 0) __PYX_ERR(1, 345, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":348
 *     uguugl.glDetachShader(program, shader)
 * 
 * def glDisable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glDisable(cap)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_73glDisable, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDisable, __pyx_t_3) < 0) __PYX_ERR(1, 348, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":351
 *     uguugl.glDisable(cap)
 * 
 * def glDisableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_75glDisableVertexAttribArray, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDisableVertexAttribArray, __pyx_t_3) < 0) __PYX_ERR(1, 351, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":354
 *     uguugl.glDisableVertexAttribArray(index)
 * 
 * def glDrawArrays(mode, first, count):             # <<<<<<<<<<<<<<
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_77glDrawArrays, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 354, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDrawArrays, __pyx_t_3) < 0) __PYX_ERR(1, 354, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":357
 *     uguugl.glDrawArrays(mode, first, count)
 * 
 * def glDrawElements(mode, count, type, indices):             # <<<<<<<<<<<<<<
 *     cdef ptr indices_ptr = get_ptr(indices)
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_79glDrawElements, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glDrawElements, __pyx_t_3) < 0) __PYX_ERR(1, 357, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":361
 *     uguugl.glDrawElements(mode, count, type, <const void *> indices_ptr.ptr)
 * 
 * def glEnable(cap):             # <<<<<<<<<<<<<<
 *     uguugl.glEnable(cap)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_81glEnable, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glEnable, __pyx_t_3) < 0) __PYX_ERR(1, 361, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":364
 *     uguugl.glEnable(cap)
 * 
 * def glEnableVertexAttribArray(index):             # <<<<<<<<<<<<<<
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_83glEnableVertexAttribArray, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glEnableVertexAttribArray, __pyx_t_3) < 0) __PYX_ERR(1, 364, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":367
 *     uguugl.glEnableVertexAttribArray(index)
 * 
 * def glFinish():             # <<<<<<<<<<<<<<
 *     uguugl.glFinish()
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_85glFinish, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glFinish, __pyx_t_3) < 0) __PYX_ERR(1, 367, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":370
 *     uguugl.glFinish()
 * 
 * def glFlush():             # <<<<<<<<<<<<<<
 *     uguugl.glFlush()
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_87glFlush, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glFlush, __pyx_t_3) < 0) __PYX_ERR(1, 370, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":373
 *     uguugl.glFlush()
 * 
 * def glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_89glFramebufferRenderbuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glFramebufferRenderbuffer, __pyx_t_3) < 0) __PYX_ERR(1, 373, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":376
 *     uguugl.glFramebufferRenderbuffer(target, attachment, renderbuffertarget, renderbuffer)
 * 
 * def glFramebufferTexture2D(target, attachment, textarget, texture, level):             # <<<<<<<<<<<<<<
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_91glFramebufferTexture2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glFramebufferTexture2D, __pyx_t_3) < 0) __PYX_ERR(1, 376, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":379
 *     uguugl.glFramebufferTexture2D(target, attachment, textarget, texture, level)
 * 
 * def glFrontFace(mode):             # <<<<<<<<<<<<<<
 *     uguugl.glFrontFace(mode)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_93glFrontFace, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glFrontFace, __pyx_t_3) < 0) __PYX_ERR(1, 379, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":382
 *     uguugl.glFrontFace(mode)
 * 
 * def glGenBuffers(n, buffers):             # <<<<<<<<<<<<<<
 *     cdef ptr buffers_ptr = get_ptr(buffers)
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_95glGenBuffers, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGenBuffers, __pyx_t_3) < 0) __PYX_ERR(1, 382, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":386
 *     uguugl.glGenBuffers(n, <GLuint *> buffers_ptr.ptr)
 * 
 * def glGenFramebuffers(n, framebuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr framebuffers_ptr = get_ptr(framebuffers)
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_97glGenFramebuffers, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGenFramebuffers, __pyx_t_3) < 0) __PYX_ERR(1, 386, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":390
 *     uguugl.glGenFramebuffers(n, <GLuint *> framebuffers_ptr.ptr)
 * 
 * def glGenRenderbuffers(n, renderbuffers):             # <<<<<<<<<<<<<<
 *     cdef ptr renderbuffers_ptr = get_ptr(renderbuffers)
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_99glGenRenderbuffers, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGenRenderbuffers, __pyx_t_3) < 0) __PYX_ERR(1, 390, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":394
 *     uguugl.glGenRenderbuffers(n, <GLuint *> renderbuffers_ptr.ptr)
 * 
 * def glGenTextures(n, textures):             # <<<<<<<<<<<<<<
 *     cdef ptr textures_ptr = get_ptr(textures)
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_101glGenTextures, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGenTextures, __pyx_t_3) < 0) __PYX_ERR(1, 394, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":398
 *     uguugl.glGenTextures(n, <GLuint *> textures_ptr.ptr)
 * 
 * def glGenerateMipmap(target):             # <<<<<<<<<<<<<<
 *     uguugl.glGenerateMipmap(target)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_103glGenerateMipmap, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGenerateMipmap, __pyx_t_3) < 0) __PYX_ERR(1, 398, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":401
 *     uguugl.glGenerateMipmap(target)
 * 
 * def glGetActiveAttrib(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_105glGetActiveAttrib, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetActiveAttrib, __pyx_t_3) < 0) __PYX_ERR(1, 401, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":408
 *     uguugl.glGetActiveAttrib(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetActiveUniform(program, index, bufSize, length, size, type, name):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr size_ptr = get_ptr(size)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_107glGetActiveUniform, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 408, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetActiveUniform, __pyx_t_3) < 0) __PYX_ERR(1, 408, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":415
 *     uguugl.glGetActiveUniform(program, index, bufSize, <GLsizei *> length_ptr.ptr, <GLint *> size_ptr.ptr, <GLenum *> type_ptr.ptr, <GLchar *> name_ptr.ptr)
 * 
 * def glGetAttachedShaders(program, maxCount, count, shaders):             # <<<<<<<<<<<<<<
 *     cdef ptr count_ptr = get_ptr(count)
 *     cdef ptr shaders_ptr = get_ptr(shaders)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_109glGetAttachedShaders, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetAttachedShaders, __pyx_t_3) < 0) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":420
 *     uguugl.glGetAttachedShaders(program, maxCount, <GLsizei *> count_ptr.ptr, <GLuint *> shaders_ptr.ptr)
 * 
 * def glGetAttribLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_111glGetAttribLocation, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 420, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetAttribLocation, __pyx_t_3) < 0) __PYX_ERR(1, 420, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":424
 *     return uguugl.glGetAttribLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetBooleanv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_113glGetBooleanv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetBooleanv, __pyx_t_3) < 0) __PYX_ERR(1, 424, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":428
 *     uguugl.glGetBooleanv(pname, <GLboolean *> data_ptr.ptr)
 * 
 * def glGetBufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_115glGetBufferParameteriv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetBufferParameteriv, __pyx_t_3) < 0) __PYX_ERR(1, 428, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":432
 *     uguugl.glGetBufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetError():             # <<<<<<<<<<<<<<
 *     return uguugl.glGetError()
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_117glGetError, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetError, __pyx_t_3) < 0) __PYX_ERR(1, 432, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":435
 *     return uguugl.glGetError()
 * 
 * def glGetFloatv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_119glGetFloatv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetFloatv, __pyx_t_3) < 0) __PYX_ERR(1, 435, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":439
 *     uguugl.glGetFloatv(pname, <GLfloat *> data_ptr.ptr)
 * 
 * def glGetFramebufferAttachmentParameteriv(target, attachment, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_121glGetFramebufferAttachmentParameteriv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetFramebufferAttachmentParame, __pyx_t_3) < 0) __PYX_ERR(1, 439, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":443
 *     uguugl.glGetFramebufferAttachmentParameteriv(target, attachment, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetIntegerv(pname, data):             # <<<<<<<<<<<<<<
 *     cdef ptr data_ptr = get_ptr(data)
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_123glGetIntegerv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetIntegerv, __pyx_t_3) < 0) __PYX_ERR(1, 443, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":447
 *     uguugl.glGetIntegerv(pname, <GLint *> data_ptr.ptr)
 * 
 * def glGetProgramInfoLog(program, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_125glGetProgramInfoLog, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetProgramInfoLog, __pyx_t_3) < 0) __PYX_ERR(1, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":452
 *     uguugl.glGetProgramInfoLog(program, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetProgramiv(program, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_127glGetProgramiv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetProgramiv, __pyx_t_3) < 0) __PYX_ERR(1, 452, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":456
 *     uguugl.glGetProgramiv(program, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetRenderbufferParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_129glGetRenderbufferParameteriv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetRenderbufferParameteriv, __pyx_t_3) < 0) __PYX_ERR(1, 456, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":460
 *     uguugl.glGetRenderbufferParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetShaderInfoLog(shader, bufSize, length, infoLog):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr infoLog_ptr = get_ptr(infoLog)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_131glGetShaderInfoLog, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetShaderInfoLog, __pyx_t_3) < 0) __PYX_ERR(1, 460, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":465
 *     uguugl.glGetShaderInfoLog(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> infoLog_ptr.ptr)
 * 
 * def glGetShaderSource(shader, bufSize, length, source):             # <<<<<<<<<<<<<<
 *     cdef ptr length_ptr = get_ptr(length)
 *     cdef ptr source_ptr = get_ptr(source)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_133glGetShaderSource, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetShaderSource, __pyx_t_3) < 0) __PYX_ERR(1, 465, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":470
 *     uguugl.glGetShaderSource(shader, bufSize, <GLsizei *> length_ptr.ptr, <GLchar *> source_ptr.ptr)
 * 
 * def glGetShaderiv(shader, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_135glGetShaderiv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetShaderiv, __pyx_t_3) < 0) __PYX_ERR(1, 470, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":474
 *     uguugl.glGetShaderiv(shader, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetString(name):             # <<<<<<<<<<<<<<
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_137glGetString, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetString, __pyx_t_3) < 0) __PYX_ERR(1, 474, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":477
 *     return proxy_return_string(uguugl.glGetString(name))
 * 
 * def glGetTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_139glGetTexParameterfv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetTexParameterfv, __pyx_t_3) < 0) __PYX_ERR(1, 477, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":481
 *     uguugl.glGetTexParameterfv(target, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_141glGetTexParameteriv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetTexParameteriv, __pyx_t_3) < 0) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":485
 *     uguugl.glGetTexParameteriv(target, pname, <GLint *> params_ptr.ptr)
 * 
 * def glGetUniformLocation(program, name):             # <<<<<<<<<<<<<<
 *     cdef ptr name_ptr = get_ptr(name)
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_143glGetUniformLocation, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetUniformLocation, __pyx_t_3) < 0) __PYX_ERR(1, 485, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":489
 *     return uguugl.glGetUniformLocation(program, <const GLchar *> name_ptr.ptr)
 * 
 * def glGetUniformfv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_145glGetUniformfv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetUniformfv, __pyx_t_3) < 0) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":493
 *     uguugl.glGetUniformfv(program, location, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetUniformiv(program, location, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_147glGetUniformiv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetUniformiv, __pyx_t_3) < 0) __PYX_ERR(1, 493, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":497
 *     uguugl.glGetUniformiv(program, location, <GLint *> params_ptr.ptr)
 * 
 * def glGetVertexAttribPointerv(index, pname, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_149glGetVertexAttribPointerv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetVertexAttribPointerv, __pyx_t_3) < 0) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":501
 *     uguugl.glGetVertexAttribPointerv(index, pname, <void **> pointer_ptr.ptr)
 * 
 * def glGetVertexAttribfv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_151glGetVertexAttribfv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetVertexAttribfv, __pyx_t_3) < 0) __PYX_ERR(1, 501, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":505
 *     uguugl.glGetVertexAttribfv(index, pname, <GLfloat *> params_ptr.ptr)
 * 
 * def glGetVertexAttribiv(index, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_153glGetVertexAttribiv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glGetVertexAttribiv, __pyx_t_3) < 0) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":509
 *     uguugl.glGetVertexAttribiv(index, pname, <GLint *> params_ptr.ptr)
 * 
 * def glHint(target, mode):             # <<<<<<<<<<<<<<
 *     uguugl.glHint(target, mode)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_155glHint, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glHint, __pyx_t_3) < 0) __PYX_ERR(1, 509, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":512
 *     uguugl.glHint(target, mode)
 * 
 * def glIsBuffer(buffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsBuffer(buffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_157glIsBuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsBuffer, __pyx_t_3) < 0) __PYX_ERR(1, 512, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":515
 *     return uguugl.glIsBuffer(buffer)
 * 
 * def glIsEnabled(cap):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsEnabled(cap)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_159glIsEnabled, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsEnabled, __pyx_t_3) < 0) __PYX_ERR(1, 515, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":518
 *     return uguugl.glIsEnabled(cap)
 * 
 * def glIsFramebuffer(framebuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_161glIsFramebuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsFramebuffer, __pyx_t_3) < 0) __PYX_ERR(1, 518, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":521
 *     return uguugl.glIsFramebuffer(framebuffer)
 * 
 * def glIsProgram(program):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsProgram(program)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_163glIsProgram, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsProgram, __pyx_t_3) < 0) __PYX_ERR(1, 521, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":524
 *     return uguugl.glIsProgram(program)
 * 
 * def glIsRenderbuffer(renderbuffer):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_165glIsRenderbuffer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsRenderbuffer, __pyx_t_3) < 0) __PYX_ERR(1, 524, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":527
 *     return uguugl.glIsRenderbuffer(renderbuffer)
 * 
 * def glIsShader(shader):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsShader(shader)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_167glIsShader, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsShader, __pyx_t_3) < 0) __PYX_ERR(1, 527, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":530
 *     return uguugl.glIsShader(shader)
 * 
 * def glIsTexture(texture):             # <<<<<<<<<<<<<<
 *     return uguugl.glIsTexture(texture)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_169glIsTexture, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glIsTexture, __pyx_t_3) < 0) __PYX_ERR(1, 530, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":533
 *     return uguugl.glIsTexture(texture)
 * 
 * def glLineWidth(width):             # <<<<<<<<<<<<<<
 *     uguugl.glLineWidth(width)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_171glLineWidth, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glLineWidth, __pyx_t_3) < 0) __PYX_ERR(1, 533, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":536
 *     uguugl.glLineWidth(width)
 * 
 * def glLinkProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glLinkProgram(program)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_173glLinkProgram, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glLinkProgram, __pyx_t_3) < 0) __PYX_ERR(1, 536, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":539
 *     uguugl.glLinkProgram(program)
 * 
 * def glPixelStorei(pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glPixelStorei(pname, param)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_175glPixelStorei, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glPixelStorei, __pyx_t_3) < 0) __PYX_ERR(1, 539, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":542
 *     uguugl.glPixelStorei(pname, param)
 * 
 * def glPolygonOffset(factor, units):             # <<<<<<<<<<<<<<
 *     uguugl.glPolygonOffset(factor, units)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_177glPolygonOffset, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glPolygonOffset, __pyx_t_3) < 0) __PYX_ERR(1, 542, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":545
 *     uguugl.glPolygonOffset(factor, units)
 * 
 * def glReadPixels(x, y, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_179glReadPixels, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glReadPixels, __pyx_t_3) < 0) __PYX_ERR(1, 545, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":549
 *     uguugl.glReadPixels(x, y, width, height, format, type, <void *> pixels_ptr.ptr)
 * 
 * def glRenderbufferStorage(target, internalformat, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_181glRenderbufferStorage, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glRenderbufferStorage, __pyx_t_3) < 0) __PYX_ERR(1, 549, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":552
 *     uguugl.glRenderbufferStorage(target, internalformat, width, height)
 * 
 * def glSampleCoverage(value, invert):             # <<<<<<<<<<<<<<
 *     uguugl.glSampleCoverage(value, invert)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_183glSampleCoverage, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glSampleCoverage, __pyx_t_3) < 0) __PYX_ERR(1, 552, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":555
 *     uguugl.glSampleCoverage(value, invert)
 * 
 * def glScissor(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glScissor(x, y, width, height)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_185glScissor, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glScissor, __pyx_t_3) < 0) __PYX_ERR(1, 555, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":558
 *     uguugl.glScissor(x, y, width, height)
 * 
 * def glShaderSource(shader, count, string, length):             # <<<<<<<<<<<<<<
 *     cdef ptr string_ptr = get_ptr(string)
 *     cdef ptr length_ptr = get_ptr(length)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_187glShaderSource, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glShaderSource, __pyx_t_3) < 0) __PYX_ERR(1, 558, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":563
 *     uguugl.glShaderSource(shader, count, <const GLchar *const*> string_ptr.ptr, <const GLint *> length_ptr.ptr)
 * 
 * def glStencilFunc(func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_189glStencilFunc, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glStencilFunc, __pyx_t_3) < 0) __PYX_ERR(1, 563, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":566
 *     uguugl.glStencilFunc(func, ref, mask)
 * 
 * def glStencilFuncSeparate(face, func, ref, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_191glStencilFuncSeparate, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glStencilFuncSeparate, __pyx_t_3) < 0) __PYX_ERR(1, 566, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":569
 *     uguugl.glStencilFuncSeparate(face, func, ref, mask)
 * 
 * def glStencilMask(mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMask(mask)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_193glStencilMask, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glStencilMask, __pyx_t_3) < 0) __PYX_ERR(1, 569, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":572
 *     uguugl.glStencilMask(mask)
 * 
 * def glStencilMaskSeparate(face, mask):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_195glStencilMaskSeparate, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glStencilMaskSeparate, __pyx_t_3) < 0) __PYX_ERR(1, 572, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":575
 *     uguugl.glStencilMaskSeparate(face, mask)
 * 
 * def glStencilOp(fail, zfail, zpass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_197glStencilOp, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glStencilOp, __pyx_t_3) < 0) __PYX_ERR(1, 575, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":578
 *     uguugl.glStencilOp(fail, zfail, zpass)
 * 
 * def glStencilOpSeparate(face, sfail, dpfail, dppass):             # <<<<<<<<<<<<<<
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_199glStencilOpSeparate, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glStencilOpSeparate, __pyx_t_3) < 0) __PYX_ERR(1, 578, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":581
 *     uguugl.glStencilOpSeparate(face, sfail, dpfail, dppass)
 * 
 * def glTexImage2D(target, level, internalformat, width, height, border, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_201glTexImage2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glTexImage2D, __pyx_t_3) < 0) __PYX_ERR(1, 581, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":585
 *     uguugl.glTexImage2D(target, level, internalformat, width, height, border, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glTexParameterf(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_203glTexParameterf, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glTexParameterf, __pyx_t_3) < 0) __PYX_ERR(1, 585, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":588
 *     uguugl.glTexParameterf(target, pname, param)
 * 
 * def glTexParameterfv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_205glTexParameterfv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glTexParameterfv, __pyx_t_3) < 0) __PYX_ERR(1, 588, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":592
 *     uguugl.glTexParameterfv(target, pname, <const GLfloat *> params_ptr.ptr)
 * 
 * def glTexParameteri(target, pname, param):             # <<<<<<<<<<<<<<
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_207glTexParameteri, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glTexParameteri, __pyx_t_3) < 0) __PYX_ERR(1, 592, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":595
 *     uguugl.glTexParameteri(target, pname, param)
 * 
 * def glTexParameteriv(target, pname, params):             # <<<<<<<<<<<<<<
 *     cdef ptr params_ptr = get_ptr(params)
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_209glTexParameteriv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 595, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glTexParameteriv, __pyx_t_3) < 0) __PYX_ERR(1, 595, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":599
 *     uguugl.glTexParameteriv(target, pname, <const GLint *> params_ptr.ptr)
 * 
 * def glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, pixels):             # <<<<<<<<<<<<<<
 *     cdef ptr pixels_ptr = get_ptr(pixels)
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_211glTexSubImage2D, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glTexSubImage2D, __pyx_t_3) < 0) __PYX_ERR(1, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":603
 *     uguugl.glTexSubImage2D(target, level, xoffset, yoffset, width, height, format, type, <const void *> pixels_ptr.ptr)
 * 
 * def glUniform1f(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1f(location, v0)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_213glUniform1f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform1f, __pyx_t_3) < 0) __PYX_ERR(1, 603, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":606
 *     uguugl.glUniform1f(location, v0)
 * 
 * def glUniform1fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_215glUniform1fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform1fv, __pyx_t_3) < 0) __PYX_ERR(1, 606, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":610
 *     uguugl.glUniform1fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform1i(location, v0):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform1i(location, v0)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_217glUniform1i, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform1i, __pyx_t_3) < 0) __PYX_ERR(1, 610, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":613
 *     uguugl.glUniform1i(location, v0)
 * 
 * def glUniform1iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_219glUniform1iv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform1iv, __pyx_t_3) < 0) __PYX_ERR(1, 613, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":617
 *     uguugl.glUniform1iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform2f(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_221glUniform2f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform2f, __pyx_t_3) < 0) __PYX_ERR(1, 617, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":620
 *     uguugl.glUniform2f(location, v0, v1)
 * 
 * def glUniform2fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_223glUniform2fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform2fv, __pyx_t_3) < 0) __PYX_ERR(1, 620, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":624
 *     uguugl.glUniform2fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform2i(location, v0, v1):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_225glUniform2i, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform2i, __pyx_t_3) < 0) __PYX_ERR(1, 624, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":627
 *     uguugl.glUniform2i(location, v0, v1)
 * 
 * def glUniform2iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_227glUniform2iv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform2iv, __pyx_t_3) < 0) __PYX_ERR(1, 627, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":631
 *     uguugl.glUniform2iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform3f(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_229glUniform3f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform3f, __pyx_t_3) < 0) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":634
 *     uguugl.glUniform3f(location, v0, v1, v2)
 * 
 * def glUniform3fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_231glUniform3fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform3fv, __pyx_t_3) < 0) __PYX_ERR(1, 634, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":638
 *     uguugl.glUniform3fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform3i(location, v0, v1, v2):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_233glUniform3i, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform3i, __pyx_t_3) < 0) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":641
 *     uguugl.glUniform3i(location, v0, v1, v2)
 * 
 * def glUniform3iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_235glUniform3iv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform3iv, __pyx_t_3) < 0) __PYX_ERR(1, 641, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":645
 *     uguugl.glUniform3iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniform4f(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_237glUniform4f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform4f, __pyx_t_3) < 0) __PYX_ERR(1, 645, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":648
 *     uguugl.glUniform4f(location, v0, v1, v2, v3)
 * 
 * def glUniform4fv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_239glUniform4fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform4fv, __pyx_t_3) < 0) __PYX_ERR(1, 648, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":652
 *     uguugl.glUniform4fv(location, count, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniform4i(location, v0, v1, v2, v3):             # <<<<<<<<<<<<<<
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_241glUniform4i, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform4i, __pyx_t_3) < 0) __PYX_ERR(1, 652, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":655
 *     uguugl.glUniform4i(location, v0, v1, v2, v3)
 * 
 * def glUniform4iv(location, count, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_243glUniform4iv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniform4iv, __pyx_t_3) < 0) __PYX_ERR(1, 655, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":659
 *     uguugl.glUniform4iv(location, count, <const GLint *> value_ptr.ptr)
 * 
 * def glUniformMatrix2fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_245glUniformMatrix2fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniformMatrix2fv, __pyx_t_3) < 0) __PYX_ERR(1, 659, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":663
 *     uguugl.glUniformMatrix2fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix3fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_247glUniformMatrix3fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniformMatrix3fv, __pyx_t_3) < 0) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":667
 *     uguugl.glUniformMatrix3fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUniformMatrix4fv(location, count, transpose, value):             # <<<<<<<<<<<<<<
 *     cdef ptr value_ptr = get_ptr(value)
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_249glUniformMatrix4fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUniformMatrix4fv, __pyx_t_3) < 0) __PYX_ERR(1, 667, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":671
 *     uguugl.glUniformMatrix4fv(location, count, transpose, <const GLfloat *> value_ptr.ptr)
 * 
 * def glUseProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glUseProgram(program)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_251glUseProgram, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glUseProgram, __pyx_t_3) < 0) __PYX_ERR(1, 671, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":674
 *     uguugl.glUseProgram(program)
 * 
 * def glValidateProgram(program):             # <<<<<<<<<<<<<<
 *     uguugl.glValidateProgram(program)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_253glValidateProgram, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glValidateProgram, __pyx_t_3) < 0) __PYX_ERR(1, 674, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":677
 *     uguugl.glValidateProgram(program)
 * 
 * def glVertexAttrib1f(index, x):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_255glVertexAttrib1f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib1f, __pyx_t_3) < 0) __PYX_ERR(1, 677, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":680
 *     uguugl.glVertexAttrib1f(index, x)
 * 
 * def glVertexAttrib1fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_257glVertexAttrib1fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib1fv, __pyx_t_3) < 0) __PYX_ERR(1, 680, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":684
 *     uguugl.glVertexAttrib1fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib2f(index, x, y):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_259glVertexAttrib2f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 684, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib2f, __pyx_t_3) < 0) __PYX_ERR(1, 684, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":687
 *     uguugl.glVertexAttrib2f(index, x, y)
 * 
 * def glVertexAttrib2fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_261glVertexAttrib2fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib2fv, __pyx_t_3) < 0) __PYX_ERR(1, 687, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":691
 *     uguugl.glVertexAttrib2fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib3f(index, x, y, z):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_263glVertexAttrib3f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 691, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib3f, __pyx_t_3) < 0) __PYX_ERR(1, 691, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":694
 *     uguugl.glVertexAttrib3f(index, x, y, z)
 * 
 * def glVertexAttrib3fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_265glVertexAttrib3fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib3fv, __pyx_t_3) < 0) __PYX_ERR(1, 694, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":698
 *     uguugl.glVertexAttrib3fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttrib4f(index, x, y, z, w):             # <<<<<<<<<<<<<<
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_267glVertexAttrib4f, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib4f, __pyx_t_3) < 0) __PYX_ERR(1, 698, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":701
 *     uguugl.glVertexAttrib4f(index, x, y, z, w)
 * 
 * def glVertexAttrib4fv(index, v):             # <<<<<<<<<<<<<<
 *     cdef ptr v_ptr = get_ptr(v)
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_269glVertexAttrib4fv, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttrib4fv, __pyx_t_3) < 0) __PYX_ERR(1, 701, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":705
 *     uguugl.glVertexAttrib4fv(index, <const GLfloat *> v_ptr.ptr)
 * 
 * def glVertexAttribPointer(index, size, type, normalized, stride, pointer):             # <<<<<<<<<<<<<<
 *     cdef ptr pointer_ptr = get_ptr(pointer)
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_271glVertexAttribPointer, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glVertexAttribPointer, __pyx_t_3) < 0) __PYX_ERR(1, 705, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":709
 *     uguugl.glVertexAttribPointer(index, size, type, normalized, stride, <const void *> pointer_ptr.ptr)
 * 
 * def glViewport(x, y, width, height):             # <<<<<<<<<<<<<<
 *     uguugl.glViewport(x, y, width, height)
 * 
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_4uguu_273glViewport, NULL, __pyx_n_s_renpy_gl2_uguu); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_glViewport, __pyx_t_3) < 0) __PYX_ERR(1, 709, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":712
 *     uguugl.glViewport(x, y, width, height)
 * 
 * GL_FALSE = uguugl.GL_FALSE             # <<<<<<<<<<<<<<
 * GL_NONE = uguugl.GL_NONE
 * GL_NO_ERROR = uguugl.GL_NO_ERROR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FALSE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FALSE, __pyx_t_3) < 0) __PYX_ERR(1, 712, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":713
 * 
 * GL_FALSE = uguugl.GL_FALSE
 * GL_NONE = uguugl.GL_NONE             # <<<<<<<<<<<<<<
 * GL_NO_ERROR = uguugl.GL_NO_ERROR
 * GL_POINTS = uguugl.GL_POINTS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NONE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NONE, __pyx_t_3) < 0) __PYX_ERR(1, 713, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":714
 * GL_FALSE = uguugl.GL_FALSE
 * GL_NONE = uguugl.GL_NONE
 * GL_NO_ERROR = uguugl.GL_NO_ERROR             # <<<<<<<<<<<<<<
 * GL_POINTS = uguugl.GL_POINTS
 * GL_ZERO = uguugl.GL_ZERO
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NO_ERROR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NO_ERROR, __pyx_t_3) < 0) __PYX_ERR(1, 714, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":715
 * GL_NONE = uguugl.GL_NONE
 * GL_NO_ERROR = uguugl.GL_NO_ERROR
 * GL_POINTS = uguugl.GL_POINTS             # <<<<<<<<<<<<<<
 * GL_ZERO = uguugl.GL_ZERO
 * GL_LINES = uguugl.GL_LINES
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_POINTS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_POINTS, __pyx_t_3) < 0) __PYX_ERR(1, 715, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":716
 * GL_NO_ERROR = uguugl.GL_NO_ERROR
 * GL_POINTS = uguugl.GL_POINTS
 * GL_ZERO = uguugl.GL_ZERO             # <<<<<<<<<<<<<<
 * GL_LINES = uguugl.GL_LINES
 * GL_ONE = uguugl.GL_ONE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ZERO); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ZERO, __pyx_t_3) < 0) __PYX_ERR(1, 716, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":717
 * GL_POINTS = uguugl.GL_POINTS
 * GL_ZERO = uguugl.GL_ZERO
 * GL_LINES = uguugl.GL_LINES             # <<<<<<<<<<<<<<
 * GL_ONE = uguugl.GL_ONE
 * GL_TRUE = uguugl.GL_TRUE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINES); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINES, __pyx_t_3) < 0) __PYX_ERR(1, 717, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":718
 * GL_ZERO = uguugl.GL_ZERO
 * GL_LINES = uguugl.GL_LINES
 * GL_ONE = uguugl.GL_ONE             # <<<<<<<<<<<<<<
 * GL_TRUE = uguugl.GL_TRUE
 * GL_LINE_LOOP = uguugl.GL_LINE_LOOP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE, __pyx_t_3) < 0) __PYX_ERR(1, 718, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":719
 * GL_LINES = uguugl.GL_LINES
 * GL_ONE = uguugl.GL_ONE
 * GL_TRUE = uguugl.GL_TRUE             # <<<<<<<<<<<<<<
 * GL_LINE_LOOP = uguugl.GL_LINE_LOOP
 * GL_LINE_STRIP = uguugl.GL_LINE_STRIP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TRUE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TRUE, __pyx_t_3) < 0) __PYX_ERR(1, 719, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":720
 * GL_ONE = uguugl.GL_ONE
 * GL_TRUE = uguugl.GL_TRUE
 * GL_LINE_LOOP = uguugl.GL_LINE_LOOP             # <<<<<<<<<<<<<<
 * GL_LINE_STRIP = uguugl.GL_LINE_STRIP
 * GL_TRIANGLES = uguugl.GL_TRIANGLES
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINE_LOOP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINE_LOOP, __pyx_t_3) < 0) __PYX_ERR(1, 720, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":721
 * GL_TRUE = uguugl.GL_TRUE
 * GL_LINE_LOOP = uguugl.GL_LINE_LOOP
 * GL_LINE_STRIP = uguugl.GL_LINE_STRIP             # <<<<<<<<<<<<<<
 * GL_TRIANGLES = uguugl.GL_TRIANGLES
 * GL_TRIANGLE_STRIP = uguugl.GL_TRIANGLE_STRIP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINE_STRIP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINE_STRIP, __pyx_t_3) < 0) __PYX_ERR(1, 721, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":722
 * GL_LINE_LOOP = uguugl.GL_LINE_LOOP
 * GL_LINE_STRIP = uguugl.GL_LINE_STRIP
 * GL_TRIANGLES = uguugl.GL_TRIANGLES             # <<<<<<<<<<<<<<
 * GL_TRIANGLE_STRIP = uguugl.GL_TRIANGLE_STRIP
 * GL_TRIANGLE_FAN = uguugl.GL_TRIANGLE_FAN
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TRIANGLES); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TRIANGLES, __pyx_t_3) < 0) __PYX_ERR(1, 722, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":723
 * GL_LINE_STRIP = uguugl.GL_LINE_STRIP
 * GL_TRIANGLES = uguugl.GL_TRIANGLES
 * GL_TRIANGLE_STRIP = uguugl.GL_TRIANGLE_STRIP             # <<<<<<<<<<<<<<
 * GL_TRIANGLE_FAN = uguugl.GL_TRIANGLE_FAN
 * GL_DEPTH_BUFFER_BIT = uguugl.GL_DEPTH_BUFFER_BIT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TRIANGLE_STRIP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 723, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TRIANGLE_STRIP, __pyx_t_3) < 0) __PYX_ERR(1, 723, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":724
 * GL_TRIANGLES = uguugl.GL_TRIANGLES
 * GL_TRIANGLE_STRIP = uguugl.GL_TRIANGLE_STRIP
 * GL_TRIANGLE_FAN = uguugl.GL_TRIANGLE_FAN             # <<<<<<<<<<<<<<
 * GL_DEPTH_BUFFER_BIT = uguugl.GL_DEPTH_BUFFER_BIT
 * GL_NEVER = uguugl.GL_NEVER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TRIANGLE_FAN); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 724, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TRIANGLE_FAN, __pyx_t_3) < 0) __PYX_ERR(1, 724, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":725
 * GL_TRIANGLE_STRIP = uguugl.GL_TRIANGLE_STRIP
 * GL_TRIANGLE_FAN = uguugl.GL_TRIANGLE_FAN
 * GL_DEPTH_BUFFER_BIT = uguugl.GL_DEPTH_BUFFER_BIT             # <<<<<<<<<<<<<<
 * GL_NEVER = uguugl.GL_NEVER
 * GL_LESS = uguugl.GL_LESS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_BUFFER_BIT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_BUFFER_BIT, __pyx_t_3) < 0) __PYX_ERR(1, 725, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":726
 * GL_TRIANGLE_FAN = uguugl.GL_TRIANGLE_FAN
 * GL_DEPTH_BUFFER_BIT = uguugl.GL_DEPTH_BUFFER_BIT
 * GL_NEVER = uguugl.GL_NEVER             # <<<<<<<<<<<<<<
 * GL_LESS = uguugl.GL_LESS
 * GL_EQUAL = uguugl.GL_EQUAL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NEVER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NEVER, __pyx_t_3) < 0) __PYX_ERR(1, 726, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":727
 * GL_DEPTH_BUFFER_BIT = uguugl.GL_DEPTH_BUFFER_BIT
 * GL_NEVER = uguugl.GL_NEVER
 * GL_LESS = uguugl.GL_LESS             # <<<<<<<<<<<<<<
 * GL_EQUAL = uguugl.GL_EQUAL
 * GL_LEQUAL = uguugl.GL_LEQUAL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LESS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LESS, __pyx_t_3) < 0) __PYX_ERR(1, 727, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":728
 * GL_NEVER = uguugl.GL_NEVER
 * GL_LESS = uguugl.GL_LESS
 * GL_EQUAL = uguugl.GL_EQUAL             # <<<<<<<<<<<<<<
 * GL_LEQUAL = uguugl.GL_LEQUAL
 * GL_GREATER = uguugl.GL_GREATER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_EQUAL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_EQUAL, __pyx_t_3) < 0) __PYX_ERR(1, 728, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":729
 * GL_LESS = uguugl.GL_LESS
 * GL_EQUAL = uguugl.GL_EQUAL
 * GL_LEQUAL = uguugl.GL_LEQUAL             # <<<<<<<<<<<<<<
 * GL_GREATER = uguugl.GL_GREATER
 * GL_NOTEQUAL = uguugl.GL_NOTEQUAL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LEQUAL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LEQUAL, __pyx_t_3) < 0) __PYX_ERR(1, 729, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":730
 * GL_EQUAL = uguugl.GL_EQUAL
 * GL_LEQUAL = uguugl.GL_LEQUAL
 * GL_GREATER = uguugl.GL_GREATER             # <<<<<<<<<<<<<<
 * GL_NOTEQUAL = uguugl.GL_NOTEQUAL
 * GL_GEQUAL = uguugl.GL_GEQUAL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_GREATER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_GREATER, __pyx_t_3) < 0) __PYX_ERR(1, 730, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":731
 * GL_LEQUAL = uguugl.GL_LEQUAL
 * GL_GREATER = uguugl.GL_GREATER
 * GL_NOTEQUAL = uguugl.GL_NOTEQUAL             # <<<<<<<<<<<<<<
 * GL_GEQUAL = uguugl.GL_GEQUAL
 * GL_ALWAYS = uguugl.GL_ALWAYS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NOTEQUAL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NOTEQUAL, __pyx_t_3) < 0) __PYX_ERR(1, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":732
 * GL_GREATER = uguugl.GL_GREATER
 * GL_NOTEQUAL = uguugl.GL_NOTEQUAL
 * GL_GEQUAL = uguugl.GL_GEQUAL             # <<<<<<<<<<<<<<
 * GL_ALWAYS = uguugl.GL_ALWAYS
 * GL_SRC_COLOR = uguugl.GL_SRC_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_GEQUAL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_GEQUAL, __pyx_t_3) < 0) __PYX_ERR(1, 732, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":733
 * GL_NOTEQUAL = uguugl.GL_NOTEQUAL
 * GL_GEQUAL = uguugl.GL_GEQUAL
 * GL_ALWAYS = uguugl.GL_ALWAYS             # <<<<<<<<<<<<<<
 * GL_SRC_COLOR = uguugl.GL_SRC_COLOR
 * GL_ONE_MINUS_SRC_COLOR = uguugl.GL_ONE_MINUS_SRC_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ALWAYS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ALWAYS, __pyx_t_3) < 0) __PYX_ERR(1, 733, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":734
 * GL_GEQUAL = uguugl.GL_GEQUAL
 * GL_ALWAYS = uguugl.GL_ALWAYS
 * GL_SRC_COLOR = uguugl.GL_SRC_COLOR             # <<<<<<<<<<<<<<
 * GL_ONE_MINUS_SRC_COLOR = uguugl.GL_ONE_MINUS_SRC_COLOR
 * GL_SRC_ALPHA = uguugl.GL_SRC_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SRC_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SRC_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 734, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":735
 * GL_ALWAYS = uguugl.GL_ALWAYS
 * GL_SRC_COLOR = uguugl.GL_SRC_COLOR
 * GL_ONE_MINUS_SRC_COLOR = uguugl.GL_ONE_MINUS_SRC_COLOR             # <<<<<<<<<<<<<<
 * GL_SRC_ALPHA = uguugl.GL_SRC_ALPHA
 * GL_ONE_MINUS_SRC_ALPHA = uguugl.GL_ONE_MINUS_SRC_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE_MINUS_SRC_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE_MINUS_SRC_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 735, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":736
 * GL_SRC_COLOR = uguugl.GL_SRC_COLOR
 * GL_ONE_MINUS_SRC_COLOR = uguugl.GL_ONE_MINUS_SRC_COLOR
 * GL_SRC_ALPHA = uguugl.GL_SRC_ALPHA             # <<<<<<<<<<<<<<
 * GL_ONE_MINUS_SRC_ALPHA = uguugl.GL_ONE_MINUS_SRC_ALPHA
 * GL_DST_ALPHA = uguugl.GL_DST_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SRC_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 736, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SRC_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":737
 * GL_ONE_MINUS_SRC_COLOR = uguugl.GL_ONE_MINUS_SRC_COLOR
 * GL_SRC_ALPHA = uguugl.GL_SRC_ALPHA
 * GL_ONE_MINUS_SRC_ALPHA = uguugl.GL_ONE_MINUS_SRC_ALPHA             # <<<<<<<<<<<<<<
 * GL_DST_ALPHA = uguugl.GL_DST_ALPHA
 * GL_ONE_MINUS_DST_ALPHA = uguugl.GL_ONE_MINUS_DST_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE_MINUS_SRC_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE_MINUS_SRC_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 737, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":738
 * GL_SRC_ALPHA = uguugl.GL_SRC_ALPHA
 * GL_ONE_MINUS_SRC_ALPHA = uguugl.GL_ONE_MINUS_SRC_ALPHA
 * GL_DST_ALPHA = uguugl.GL_DST_ALPHA             # <<<<<<<<<<<<<<
 * GL_ONE_MINUS_DST_ALPHA = uguugl.GL_ONE_MINUS_DST_ALPHA
 * GL_DST_COLOR = uguugl.GL_DST_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DST_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 738, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DST_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 738, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":739
 * GL_ONE_MINUS_SRC_ALPHA = uguugl.GL_ONE_MINUS_SRC_ALPHA
 * GL_DST_ALPHA = uguugl.GL_DST_ALPHA
 * GL_ONE_MINUS_DST_ALPHA = uguugl.GL_ONE_MINUS_DST_ALPHA             # <<<<<<<<<<<<<<
 * GL_DST_COLOR = uguugl.GL_DST_COLOR
 * GL_ONE_MINUS_DST_COLOR = uguugl.GL_ONE_MINUS_DST_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE_MINUS_DST_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 739, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE_MINUS_DST_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 739, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":740
 * GL_DST_ALPHA = uguugl.GL_DST_ALPHA
 * GL_ONE_MINUS_DST_ALPHA = uguugl.GL_ONE_MINUS_DST_ALPHA
 * GL_DST_COLOR = uguugl.GL_DST_COLOR             # <<<<<<<<<<<<<<
 * GL_ONE_MINUS_DST_COLOR = uguugl.GL_ONE_MINUS_DST_COLOR
 * GL_SRC_ALPHA_SATURATE = uguugl.GL_SRC_ALPHA_SATURATE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DST_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 740, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DST_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 740, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":741
 * GL_ONE_MINUS_DST_ALPHA = uguugl.GL_ONE_MINUS_DST_ALPHA
 * GL_DST_COLOR = uguugl.GL_DST_COLOR
 * GL_ONE_MINUS_DST_COLOR = uguugl.GL_ONE_MINUS_DST_COLOR             # <<<<<<<<<<<<<<
 * GL_SRC_ALPHA_SATURATE = uguugl.GL_SRC_ALPHA_SATURATE
 * GL_STENCIL_BUFFER_BIT = uguugl.GL_STENCIL_BUFFER_BIT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE_MINUS_DST_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE_MINUS_DST_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 741, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":742
 * GL_DST_COLOR = uguugl.GL_DST_COLOR
 * GL_ONE_MINUS_DST_COLOR = uguugl.GL_ONE_MINUS_DST_COLOR
 * GL_SRC_ALPHA_SATURATE = uguugl.GL_SRC_ALPHA_SATURATE             # <<<<<<<<<<<<<<
 * GL_STENCIL_BUFFER_BIT = uguugl.GL_STENCIL_BUFFER_BIT
 * GL_FRONT = uguugl.GL_FRONT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SRC_ALPHA_SATURATE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 742, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SRC_ALPHA_SATURATE, __pyx_t_3) < 0) __PYX_ERR(1, 742, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":743
 * GL_ONE_MINUS_DST_COLOR = uguugl.GL_ONE_MINUS_DST_COLOR
 * GL_SRC_ALPHA_SATURATE = uguugl.GL_SRC_ALPHA_SATURATE
 * GL_STENCIL_BUFFER_BIT = uguugl.GL_STENCIL_BUFFER_BIT             # <<<<<<<<<<<<<<
 * GL_FRONT = uguugl.GL_FRONT
 * GL_BACK = uguugl.GL_BACK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BUFFER_BIT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BUFFER_BIT, __pyx_t_3) < 0) __PYX_ERR(1, 743, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":744
 * GL_SRC_ALPHA_SATURATE = uguugl.GL_SRC_ALPHA_SATURATE
 * GL_STENCIL_BUFFER_BIT = uguugl.GL_STENCIL_BUFFER_BIT
 * GL_FRONT = uguugl.GL_FRONT             # <<<<<<<<<<<<<<
 * GL_BACK = uguugl.GL_BACK
 * GL_FRONT_AND_BACK = uguugl.GL_FRONT_AND_BACK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRONT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRONT, __pyx_t_3) < 0) __PYX_ERR(1, 744, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":745
 * GL_STENCIL_BUFFER_BIT = uguugl.GL_STENCIL_BUFFER_BIT
 * GL_FRONT = uguugl.GL_FRONT
 * GL_BACK = uguugl.GL_BACK             # <<<<<<<<<<<<<<
 * GL_FRONT_AND_BACK = uguugl.GL_FRONT_AND_BACK
 * GL_INVALID_ENUM = uguugl.GL_INVALID_ENUM
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BACK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BACK, __pyx_t_3) < 0) __PYX_ERR(1, 745, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":746
 * GL_FRONT = uguugl.GL_FRONT
 * GL_BACK = uguugl.GL_BACK
 * GL_FRONT_AND_BACK = uguugl.GL_FRONT_AND_BACK             # <<<<<<<<<<<<<<
 * GL_INVALID_ENUM = uguugl.GL_INVALID_ENUM
 * GL_INVALID_VALUE = uguugl.GL_INVALID_VALUE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRONT_AND_BACK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRONT_AND_BACK, __pyx_t_3) < 0) __PYX_ERR(1, 746, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":747
 * GL_BACK = uguugl.GL_BACK
 * GL_FRONT_AND_BACK = uguugl.GL_FRONT_AND_BACK
 * GL_INVALID_ENUM = uguugl.GL_INVALID_ENUM             # <<<<<<<<<<<<<<
 * GL_INVALID_VALUE = uguugl.GL_INVALID_VALUE
 * GL_INVALID_OPERATION = uguugl.GL_INVALID_OPERATION
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INVALID_ENUM); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INVALID_ENUM, __pyx_t_3) < 0) __PYX_ERR(1, 747, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":748
 * GL_FRONT_AND_BACK = uguugl.GL_FRONT_AND_BACK
 * GL_INVALID_ENUM = uguugl.GL_INVALID_ENUM
 * GL_INVALID_VALUE = uguugl.GL_INVALID_VALUE             # <<<<<<<<<<<<<<
 * GL_INVALID_OPERATION = uguugl.GL_INVALID_OPERATION
 * GL_OUT_OF_MEMORY = uguugl.GL_OUT_OF_MEMORY
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INVALID_VALUE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INVALID_VALUE, __pyx_t_3) < 0) __PYX_ERR(1, 748, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":749
 * GL_INVALID_ENUM = uguugl.GL_INVALID_ENUM
 * GL_INVALID_VALUE = uguugl.GL_INVALID_VALUE
 * GL_INVALID_OPERATION = uguugl.GL_INVALID_OPERATION             # <<<<<<<<<<<<<<
 * GL_OUT_OF_MEMORY = uguugl.GL_OUT_OF_MEMORY
 * GL_INVALID_FRAMEBUFFER_OPERATION = uguugl.GL_INVALID_FRAMEBUFFER_OPERATION
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INVALID_OPERATION); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 749, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INVALID_OPERATION, __pyx_t_3) < 0) __PYX_ERR(1, 749, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":750
 * GL_INVALID_VALUE = uguugl.GL_INVALID_VALUE
 * GL_INVALID_OPERATION = uguugl.GL_INVALID_OPERATION
 * GL_OUT_OF_MEMORY = uguugl.GL_OUT_OF_MEMORY             # <<<<<<<<<<<<<<
 * GL_INVALID_FRAMEBUFFER_OPERATION = uguugl.GL_INVALID_FRAMEBUFFER_OPERATION
 * GL_CW = uguugl.GL_CW
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_OUT_OF_MEMORY); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 750, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_OUT_OF_MEMORY, __pyx_t_3) < 0) __PYX_ERR(1, 750, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":751
 * GL_INVALID_OPERATION = uguugl.GL_INVALID_OPERATION
 * GL_OUT_OF_MEMORY = uguugl.GL_OUT_OF_MEMORY
 * GL_INVALID_FRAMEBUFFER_OPERATION = uguugl.GL_INVALID_FRAMEBUFFER_OPERATION             # <<<<<<<<<<<<<<
 * GL_CW = uguugl.GL_CW
 * GL_CCW = uguugl.GL_CCW
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INVALID_FRAMEBUFFER_OPERATION); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 751, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INVALID_FRAMEBUFFER_OPERATION, __pyx_t_3) < 0) __PYX_ERR(1, 751, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":752
 * GL_OUT_OF_MEMORY = uguugl.GL_OUT_OF_MEMORY
 * GL_INVALID_FRAMEBUFFER_OPERATION = uguugl.GL_INVALID_FRAMEBUFFER_OPERATION
 * GL_CW = uguugl.GL_CW             # <<<<<<<<<<<<<<
 * GL_CCW = uguugl.GL_CCW
 * GL_LINE_WIDTH = uguugl.GL_LINE_WIDTH
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CW); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CW, __pyx_t_3) < 0) __PYX_ERR(1, 752, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":753
 * GL_INVALID_FRAMEBUFFER_OPERATION = uguugl.GL_INVALID_FRAMEBUFFER_OPERATION
 * GL_CW = uguugl.GL_CW
 * GL_CCW = uguugl.GL_CCW             # <<<<<<<<<<<<<<
 * GL_LINE_WIDTH = uguugl.GL_LINE_WIDTH
 * GL_CULL_FACE = uguugl.GL_CULL_FACE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CCW); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CCW, __pyx_t_3) < 0) __PYX_ERR(1, 753, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":754
 * GL_CW = uguugl.GL_CW
 * GL_CCW = uguugl.GL_CCW
 * GL_LINE_WIDTH = uguugl.GL_LINE_WIDTH             # <<<<<<<<<<<<<<
 * GL_CULL_FACE = uguugl.GL_CULL_FACE
 * GL_CULL_FACE_MODE = uguugl.GL_CULL_FACE_MODE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINE_WIDTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 754, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINE_WIDTH, __pyx_t_3) < 0) __PYX_ERR(1, 754, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":755
 * GL_CCW = uguugl.GL_CCW
 * GL_LINE_WIDTH = uguugl.GL_LINE_WIDTH
 * GL_CULL_FACE = uguugl.GL_CULL_FACE             # <<<<<<<<<<<<<<
 * GL_CULL_FACE_MODE = uguugl.GL_CULL_FACE_MODE
 * GL_FRONT_FACE = uguugl.GL_FRONT_FACE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CULL_FACE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 755, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CULL_FACE, __pyx_t_3) < 0) __PYX_ERR(1, 755, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":756
 * GL_LINE_WIDTH = uguugl.GL_LINE_WIDTH
 * GL_CULL_FACE = uguugl.GL_CULL_FACE
 * GL_CULL_FACE_MODE = uguugl.GL_CULL_FACE_MODE             # <<<<<<<<<<<<<<
 * GL_FRONT_FACE = uguugl.GL_FRONT_FACE
 * GL_DEPTH_RANGE = uguugl.GL_DEPTH_RANGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CULL_FACE_MODE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 756, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CULL_FACE_MODE, __pyx_t_3) < 0) __PYX_ERR(1, 756, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":757
 * GL_CULL_FACE = uguugl.GL_CULL_FACE
 * GL_CULL_FACE_MODE = uguugl.GL_CULL_FACE_MODE
 * GL_FRONT_FACE = uguugl.GL_FRONT_FACE             # <<<<<<<<<<<<<<
 * GL_DEPTH_RANGE = uguugl.GL_DEPTH_RANGE
 * GL_DEPTH_TEST = uguugl.GL_DEPTH_TEST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRONT_FACE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRONT_FACE, __pyx_t_3) < 0) __PYX_ERR(1, 757, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":758
 * GL_CULL_FACE_MODE = uguugl.GL_CULL_FACE_MODE
 * GL_FRONT_FACE = uguugl.GL_FRONT_FACE
 * GL_DEPTH_RANGE = uguugl.GL_DEPTH_RANGE             # <<<<<<<<<<<<<<
 * GL_DEPTH_TEST = uguugl.GL_DEPTH_TEST
 * GL_DEPTH_WRITEMASK = uguugl.GL_DEPTH_WRITEMASK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_RANGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_RANGE, __pyx_t_3) < 0) __PYX_ERR(1, 758, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":759
 * GL_FRONT_FACE = uguugl.GL_FRONT_FACE
 * GL_DEPTH_RANGE = uguugl.GL_DEPTH_RANGE
 * GL_DEPTH_TEST = uguugl.GL_DEPTH_TEST             # <<<<<<<<<<<<<<
 * GL_DEPTH_WRITEMASK = uguugl.GL_DEPTH_WRITEMASK
 * GL_DEPTH_CLEAR_VALUE = uguugl.GL_DEPTH_CLEAR_VALUE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_TEST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_TEST, __pyx_t_3) < 0) __PYX_ERR(1, 759, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":760
 * GL_DEPTH_RANGE = uguugl.GL_DEPTH_RANGE
 * GL_DEPTH_TEST = uguugl.GL_DEPTH_TEST
 * GL_DEPTH_WRITEMASK = uguugl.GL_DEPTH_WRITEMASK             # <<<<<<<<<<<<<<
 * GL_DEPTH_CLEAR_VALUE = uguugl.GL_DEPTH_CLEAR_VALUE
 * GL_DEPTH_FUNC = uguugl.GL_DEPTH_FUNC
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_WRITEMASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 760, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_WRITEMASK, __pyx_t_3) < 0) __PYX_ERR(1, 760, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":761
 * GL_DEPTH_TEST = uguugl.GL_DEPTH_TEST
 * GL_DEPTH_WRITEMASK = uguugl.GL_DEPTH_WRITEMASK
 * GL_DEPTH_CLEAR_VALUE = uguugl.GL_DEPTH_CLEAR_VALUE             # <<<<<<<<<<<<<<
 * GL_DEPTH_FUNC = uguugl.GL_DEPTH_FUNC
 * GL_STENCIL_TEST = uguugl.GL_STENCIL_TEST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_CLEAR_VALUE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_CLEAR_VALUE, __pyx_t_3) < 0) __PYX_ERR(1, 761, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":762
 * GL_DEPTH_WRITEMASK = uguugl.GL_DEPTH_WRITEMASK
 * GL_DEPTH_CLEAR_VALUE = uguugl.GL_DEPTH_CLEAR_VALUE
 * GL_DEPTH_FUNC = uguugl.GL_DEPTH_FUNC             # <<<<<<<<<<<<<<
 * GL_STENCIL_TEST = uguugl.GL_STENCIL_TEST
 * GL_STENCIL_CLEAR_VALUE = uguugl.GL_STENCIL_CLEAR_VALUE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_FUNC); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_FUNC, __pyx_t_3) < 0) __PYX_ERR(1, 762, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":763
 * GL_DEPTH_CLEAR_VALUE = uguugl.GL_DEPTH_CLEAR_VALUE
 * GL_DEPTH_FUNC = uguugl.GL_DEPTH_FUNC
 * GL_STENCIL_TEST = uguugl.GL_STENCIL_TEST             # <<<<<<<<<<<<<<
 * GL_STENCIL_CLEAR_VALUE = uguugl.GL_STENCIL_CLEAR_VALUE
 * GL_STENCIL_FUNC = uguugl.GL_STENCIL_FUNC
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_TEST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 763, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_TEST, __pyx_t_3) < 0) __PYX_ERR(1, 763, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":764
 * GL_DEPTH_FUNC = uguugl.GL_DEPTH_FUNC
 * GL_STENCIL_TEST = uguugl.GL_STENCIL_TEST
 * GL_STENCIL_CLEAR_VALUE = uguugl.GL_STENCIL_CLEAR_VALUE             # <<<<<<<<<<<<<<
 * GL_STENCIL_FUNC = uguugl.GL_STENCIL_FUNC
 * GL_STENCIL_VALUE_MASK = uguugl.GL_STENCIL_VALUE_MASK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_CLEAR_VALUE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_CLEAR_VALUE, __pyx_t_3) < 0) __PYX_ERR(1, 764, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":765
 * GL_STENCIL_TEST = uguugl.GL_STENCIL_TEST
 * GL_STENCIL_CLEAR_VALUE = uguugl.GL_STENCIL_CLEAR_VALUE
 * GL_STENCIL_FUNC = uguugl.GL_STENCIL_FUNC             # <<<<<<<<<<<<<<
 * GL_STENCIL_VALUE_MASK = uguugl.GL_STENCIL_VALUE_MASK
 * GL_STENCIL_FAIL = uguugl.GL_STENCIL_FAIL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_FUNC); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 765, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_FUNC, __pyx_t_3) < 0) __PYX_ERR(1, 765, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":766
 * GL_STENCIL_CLEAR_VALUE = uguugl.GL_STENCIL_CLEAR_VALUE
 * GL_STENCIL_FUNC = uguugl.GL_STENCIL_FUNC
 * GL_STENCIL_VALUE_MASK = uguugl.GL_STENCIL_VALUE_MASK             # <<<<<<<<<<<<<<
 * GL_STENCIL_FAIL = uguugl.GL_STENCIL_FAIL
 * GL_STENCIL_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_PASS_DEPTH_FAIL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_VALUE_MASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 766, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_VALUE_MASK, __pyx_t_3) < 0) __PYX_ERR(1, 766, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":767
 * GL_STENCIL_FUNC = uguugl.GL_STENCIL_FUNC
 * GL_STENCIL_VALUE_MASK = uguugl.GL_STENCIL_VALUE_MASK
 * GL_STENCIL_FAIL = uguugl.GL_STENCIL_FAIL             # <<<<<<<<<<<<<<
 * GL_STENCIL_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_PASS_DEPTH_FAIL
 * GL_STENCIL_PASS_DEPTH_PASS = uguugl.GL_STENCIL_PASS_DEPTH_PASS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_FAIL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 767, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_FAIL, __pyx_t_3) < 0) __PYX_ERR(1, 767, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":768
 * GL_STENCIL_VALUE_MASK = uguugl.GL_STENCIL_VALUE_MASK
 * GL_STENCIL_FAIL = uguugl.GL_STENCIL_FAIL
 * GL_STENCIL_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_PASS_DEPTH_FAIL             # <<<<<<<<<<<<<<
 * GL_STENCIL_PASS_DEPTH_PASS = uguugl.GL_STENCIL_PASS_DEPTH_PASS
 * GL_STENCIL_REF = uguugl.GL_STENCIL_REF
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_PASS_DEPTH_FAIL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 768, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_PASS_DEPTH_FAIL, __pyx_t_3) < 0) __PYX_ERR(1, 768, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":769
 * GL_STENCIL_FAIL = uguugl.GL_STENCIL_FAIL
 * GL_STENCIL_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_PASS_DEPTH_FAIL
 * GL_STENCIL_PASS_DEPTH_PASS = uguugl.GL_STENCIL_PASS_DEPTH_PASS             # <<<<<<<<<<<<<<
 * GL_STENCIL_REF = uguugl.GL_STENCIL_REF
 * GL_STENCIL_WRITEMASK = uguugl.GL_STENCIL_WRITEMASK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_PASS_DEPTH_PASS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_PASS_DEPTH_PASS, __pyx_t_3) < 0) __PYX_ERR(1, 769, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":770
 * GL_STENCIL_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_PASS_DEPTH_FAIL
 * GL_STENCIL_PASS_DEPTH_PASS = uguugl.GL_STENCIL_PASS_DEPTH_PASS
 * GL_STENCIL_REF = uguugl.GL_STENCIL_REF             # <<<<<<<<<<<<<<
 * GL_STENCIL_WRITEMASK = uguugl.GL_STENCIL_WRITEMASK
 * GL_VIEWPORT = uguugl.GL_VIEWPORT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_REF); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 770, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_REF, __pyx_t_3) < 0) __PYX_ERR(1, 770, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":771
 * GL_STENCIL_PASS_DEPTH_PASS = uguugl.GL_STENCIL_PASS_DEPTH_PASS
 * GL_STENCIL_REF = uguugl.GL_STENCIL_REF
 * GL_STENCIL_WRITEMASK = uguugl.GL_STENCIL_WRITEMASK             # <<<<<<<<<<<<<<
 * GL_VIEWPORT = uguugl.GL_VIEWPORT
 * GL_DITHER = uguugl.GL_DITHER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_WRITEMASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 771, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_WRITEMASK, __pyx_t_3) < 0) __PYX_ERR(1, 771, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":772
 * GL_STENCIL_REF = uguugl.GL_STENCIL_REF
 * GL_STENCIL_WRITEMASK = uguugl.GL_STENCIL_WRITEMASK
 * GL_VIEWPORT = uguugl.GL_VIEWPORT             # <<<<<<<<<<<<<<
 * GL_DITHER = uguugl.GL_DITHER
 * GL_BLEND = uguugl.GL_BLEND
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VIEWPORT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VIEWPORT, __pyx_t_3) < 0) __PYX_ERR(1, 772, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":773
 * GL_STENCIL_WRITEMASK = uguugl.GL_STENCIL_WRITEMASK
 * GL_VIEWPORT = uguugl.GL_VIEWPORT
 * GL_DITHER = uguugl.GL_DITHER             # <<<<<<<<<<<<<<
 * GL_BLEND = uguugl.GL_BLEND
 * GL_SCISSOR_BOX = uguugl.GL_SCISSOR_BOX
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DITHER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 773, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DITHER, __pyx_t_3) < 0) __PYX_ERR(1, 773, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":774
 * GL_VIEWPORT = uguugl.GL_VIEWPORT
 * GL_DITHER = uguugl.GL_DITHER
 * GL_BLEND = uguugl.GL_BLEND             # <<<<<<<<<<<<<<
 * GL_SCISSOR_BOX = uguugl.GL_SCISSOR_BOX
 * GL_SCISSOR_TEST = uguugl.GL_SCISSOR_TEST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND, __pyx_t_3) < 0) __PYX_ERR(1, 774, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":775
 * GL_DITHER = uguugl.GL_DITHER
 * GL_BLEND = uguugl.GL_BLEND
 * GL_SCISSOR_BOX = uguugl.GL_SCISSOR_BOX             # <<<<<<<<<<<<<<
 * GL_SCISSOR_TEST = uguugl.GL_SCISSOR_TEST
 * GL_COLOR_CLEAR_VALUE = uguugl.GL_COLOR_CLEAR_VALUE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SCISSOR_BOX); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SCISSOR_BOX, __pyx_t_3) < 0) __PYX_ERR(1, 775, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":776
 * GL_BLEND = uguugl.GL_BLEND
 * GL_SCISSOR_BOX = uguugl.GL_SCISSOR_BOX
 * GL_SCISSOR_TEST = uguugl.GL_SCISSOR_TEST             # <<<<<<<<<<<<<<
 * GL_COLOR_CLEAR_VALUE = uguugl.GL_COLOR_CLEAR_VALUE
 * GL_COLOR_WRITEMASK = uguugl.GL_COLOR_WRITEMASK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SCISSOR_TEST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SCISSOR_TEST, __pyx_t_3) < 0) __PYX_ERR(1, 776, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":777
 * GL_SCISSOR_BOX = uguugl.GL_SCISSOR_BOX
 * GL_SCISSOR_TEST = uguugl.GL_SCISSOR_TEST
 * GL_COLOR_CLEAR_VALUE = uguugl.GL_COLOR_CLEAR_VALUE             # <<<<<<<<<<<<<<
 * GL_COLOR_WRITEMASK = uguugl.GL_COLOR_WRITEMASK
 * GL_UNPACK_ALIGNMENT = uguugl.GL_UNPACK_ALIGNMENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_COLOR_CLEAR_VALUE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_COLOR_CLEAR_VALUE, __pyx_t_3) < 0) __PYX_ERR(1, 777, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":778
 * GL_SCISSOR_TEST = uguugl.GL_SCISSOR_TEST
 * GL_COLOR_CLEAR_VALUE = uguugl.GL_COLOR_CLEAR_VALUE
 * GL_COLOR_WRITEMASK = uguugl.GL_COLOR_WRITEMASK             # <<<<<<<<<<<<<<
 * GL_UNPACK_ALIGNMENT = uguugl.GL_UNPACK_ALIGNMENT
 * GL_PACK_ALIGNMENT = uguugl.GL_PACK_ALIGNMENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_COLOR_WRITEMASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_COLOR_WRITEMASK, __pyx_t_3) < 0) __PYX_ERR(1, 778, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":779
 * GL_COLOR_CLEAR_VALUE = uguugl.GL_COLOR_CLEAR_VALUE
 * GL_COLOR_WRITEMASK = uguugl.GL_COLOR_WRITEMASK
 * GL_UNPACK_ALIGNMENT = uguugl.GL_UNPACK_ALIGNMENT             # <<<<<<<<<<<<<<
 * GL_PACK_ALIGNMENT = uguugl.GL_PACK_ALIGNMENT
 * GL_MAX_TEXTURE_SIZE = uguugl.GL_MAX_TEXTURE_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNPACK_ALIGNMENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 779, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNPACK_ALIGNMENT, __pyx_t_3) < 0) __PYX_ERR(1, 779, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":780
 * GL_COLOR_WRITEMASK = uguugl.GL_COLOR_WRITEMASK
 * GL_UNPACK_ALIGNMENT = uguugl.GL_UNPACK_ALIGNMENT
 * GL_PACK_ALIGNMENT = uguugl.GL_PACK_ALIGNMENT             # <<<<<<<<<<<<<<
 * GL_MAX_TEXTURE_SIZE = uguugl.GL_MAX_TEXTURE_SIZE
 * GL_MAX_VIEWPORT_DIMS = uguugl.GL_MAX_VIEWPORT_DIMS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_PACK_ALIGNMENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_PACK_ALIGNMENT, __pyx_t_3) < 0) __PYX_ERR(1, 780, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":781
 * GL_UNPACK_ALIGNMENT = uguugl.GL_UNPACK_ALIGNMENT
 * GL_PACK_ALIGNMENT = uguugl.GL_PACK_ALIGNMENT
 * GL_MAX_TEXTURE_SIZE = uguugl.GL_MAX_TEXTURE_SIZE             # <<<<<<<<<<<<<<
 * GL_MAX_VIEWPORT_DIMS = uguugl.GL_MAX_VIEWPORT_DIMS
 * GL_SUBPIXEL_BITS = uguugl.GL_SUBPIXEL_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_TEXTURE_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 781, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_TEXTURE_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 781, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":782
 * GL_PACK_ALIGNMENT = uguugl.GL_PACK_ALIGNMENT
 * GL_MAX_TEXTURE_SIZE = uguugl.GL_MAX_TEXTURE_SIZE
 * GL_MAX_VIEWPORT_DIMS = uguugl.GL_MAX_VIEWPORT_DIMS             # <<<<<<<<<<<<<<
 * GL_SUBPIXEL_BITS = uguugl.GL_SUBPIXEL_BITS
 * GL_RED_BITS = uguugl.GL_RED_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_VIEWPORT_DIMS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_VIEWPORT_DIMS, __pyx_t_3) < 0) __PYX_ERR(1, 782, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":783
 * GL_MAX_TEXTURE_SIZE = uguugl.GL_MAX_TEXTURE_SIZE
 * GL_MAX_VIEWPORT_DIMS = uguugl.GL_MAX_VIEWPORT_DIMS
 * GL_SUBPIXEL_BITS = uguugl.GL_SUBPIXEL_BITS             # <<<<<<<<<<<<<<
 * GL_RED_BITS = uguugl.GL_RED_BITS
 * GL_GREEN_BITS = uguugl.GL_GREEN_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SUBPIXEL_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SUBPIXEL_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 783, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":784
 * GL_MAX_VIEWPORT_DIMS = uguugl.GL_MAX_VIEWPORT_DIMS
 * GL_SUBPIXEL_BITS = uguugl.GL_SUBPIXEL_BITS
 * GL_RED_BITS = uguugl.GL_RED_BITS             # <<<<<<<<<<<<<<
 * GL_GREEN_BITS = uguugl.GL_GREEN_BITS
 * GL_BLUE_BITS = uguugl.GL_BLUE_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RED_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 784, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RED_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 784, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":785
 * GL_SUBPIXEL_BITS = uguugl.GL_SUBPIXEL_BITS
 * GL_RED_BITS = uguugl.GL_RED_BITS
 * GL_GREEN_BITS = uguugl.GL_GREEN_BITS             # <<<<<<<<<<<<<<
 * GL_BLUE_BITS = uguugl.GL_BLUE_BITS
 * GL_ALPHA_BITS = uguugl.GL_ALPHA_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_GREEN_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 785, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_GREEN_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 785, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":786
 * GL_RED_BITS = uguugl.GL_RED_BITS
 * GL_GREEN_BITS = uguugl.GL_GREEN_BITS
 * GL_BLUE_BITS = uguugl.GL_BLUE_BITS             # <<<<<<<<<<<<<<
 * GL_ALPHA_BITS = uguugl.GL_ALPHA_BITS
 * GL_DEPTH_BITS = uguugl.GL_DEPTH_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLUE_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLUE_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 786, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":787
 * GL_GREEN_BITS = uguugl.GL_GREEN_BITS
 * GL_BLUE_BITS = uguugl.GL_BLUE_BITS
 * GL_ALPHA_BITS = uguugl.GL_ALPHA_BITS             # <<<<<<<<<<<<<<
 * GL_DEPTH_BITS = uguugl.GL_DEPTH_BITS
 * GL_STENCIL_BITS = uguugl.GL_STENCIL_BITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ALPHA_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ALPHA_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 787, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":788
 * GL_BLUE_BITS = uguugl.GL_BLUE_BITS
 * GL_ALPHA_BITS = uguugl.GL_ALPHA_BITS
 * GL_DEPTH_BITS = uguugl.GL_DEPTH_BITS             # <<<<<<<<<<<<<<
 * GL_STENCIL_BITS = uguugl.GL_STENCIL_BITS
 * GL_TEXTURE_2D = uguugl.GL_TEXTURE_2D
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 788, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":789
 * GL_ALPHA_BITS = uguugl.GL_ALPHA_BITS
 * GL_DEPTH_BITS = uguugl.GL_DEPTH_BITS
 * GL_STENCIL_BITS = uguugl.GL_STENCIL_BITS             # <<<<<<<<<<<<<<
 * GL_TEXTURE_2D = uguugl.GL_TEXTURE_2D
 * GL_DONT_CARE = uguugl.GL_DONT_CARE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 789, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BITS, __pyx_t_3) < 0) __PYX_ERR(1, 789, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":790
 * GL_DEPTH_BITS = uguugl.GL_DEPTH_BITS
 * GL_STENCIL_BITS = uguugl.GL_STENCIL_BITS
 * GL_TEXTURE_2D = uguugl.GL_TEXTURE_2D             # <<<<<<<<<<<<<<
 * GL_DONT_CARE = uguugl.GL_DONT_CARE
 * GL_FASTEST = uguugl.GL_FASTEST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_2D); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_2D, __pyx_t_3) < 0) __PYX_ERR(1, 790, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":791
 * GL_STENCIL_BITS = uguugl.GL_STENCIL_BITS
 * GL_TEXTURE_2D = uguugl.GL_TEXTURE_2D
 * GL_DONT_CARE = uguugl.GL_DONT_CARE             # <<<<<<<<<<<<<<
 * GL_FASTEST = uguugl.GL_FASTEST
 * GL_NICEST = uguugl.GL_NICEST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DONT_CARE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 791, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DONT_CARE, __pyx_t_3) < 0) __PYX_ERR(1, 791, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":792
 * GL_TEXTURE_2D = uguugl.GL_TEXTURE_2D
 * GL_DONT_CARE = uguugl.GL_DONT_CARE
 * GL_FASTEST = uguugl.GL_FASTEST             # <<<<<<<<<<<<<<
 * GL_NICEST = uguugl.GL_NICEST
 * GL_BYTE = uguugl.GL_BYTE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FASTEST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FASTEST, __pyx_t_3) < 0) __PYX_ERR(1, 792, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":793
 * GL_DONT_CARE = uguugl.GL_DONT_CARE
 * GL_FASTEST = uguugl.GL_FASTEST
 * GL_NICEST = uguugl.GL_NICEST             # <<<<<<<<<<<<<<
 * GL_BYTE = uguugl.GL_BYTE
 * GL_UNSIGNED_BYTE = uguugl.GL_UNSIGNED_BYTE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NICEST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NICEST, __pyx_t_3) < 0) __PYX_ERR(1, 793, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":794
 * GL_FASTEST = uguugl.GL_FASTEST
 * GL_NICEST = uguugl.GL_NICEST
 * GL_BYTE = uguugl.GL_BYTE             # <<<<<<<<<<<<<<
 * GL_UNSIGNED_BYTE = uguugl.GL_UNSIGNED_BYTE
 * GL_SHORT = uguugl.GL_SHORT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BYTE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BYTE, __pyx_t_3) < 0) __PYX_ERR(1, 794, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":795
 * GL_NICEST = uguugl.GL_NICEST
 * GL_BYTE = uguugl.GL_BYTE
 * GL_UNSIGNED_BYTE = uguugl.GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * GL_SHORT = uguugl.GL_SHORT
 * GL_UNSIGNED_SHORT = uguugl.GL_UNSIGNED_SHORT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNSIGNED_BYTE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNSIGNED_BYTE, __pyx_t_3) < 0) __PYX_ERR(1, 795, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":796
 * GL_BYTE = uguugl.GL_BYTE
 * GL_UNSIGNED_BYTE = uguugl.GL_UNSIGNED_BYTE
 * GL_SHORT = uguugl.GL_SHORT             # <<<<<<<<<<<<<<
 * GL_UNSIGNED_SHORT = uguugl.GL_UNSIGNED_SHORT
 * GL_INT = uguugl.GL_INT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SHORT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SHORT, __pyx_t_3) < 0) __PYX_ERR(1, 796, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":797
 * GL_UNSIGNED_BYTE = uguugl.GL_UNSIGNED_BYTE
 * GL_SHORT = uguugl.GL_SHORT
 * GL_UNSIGNED_SHORT = uguugl.GL_UNSIGNED_SHORT             # <<<<<<<<<<<<<<
 * GL_INT = uguugl.GL_INT
 * GL_UNSIGNED_INT = uguugl.GL_UNSIGNED_INT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNSIGNED_SHORT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNSIGNED_SHORT, __pyx_t_3) < 0) __PYX_ERR(1, 797, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":798
 * GL_SHORT = uguugl.GL_SHORT
 * GL_UNSIGNED_SHORT = uguugl.GL_UNSIGNED_SHORT
 * GL_INT = uguugl.GL_INT             # <<<<<<<<<<<<<<
 * GL_UNSIGNED_INT = uguugl.GL_UNSIGNED_INT
 * GL_FLOAT = uguugl.GL_FLOAT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INT, __pyx_t_3) < 0) __PYX_ERR(1, 798, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":799
 * GL_UNSIGNED_SHORT = uguugl.GL_UNSIGNED_SHORT
 * GL_INT = uguugl.GL_INT
 * GL_UNSIGNED_INT = uguugl.GL_UNSIGNED_INT             # <<<<<<<<<<<<<<
 * GL_FLOAT = uguugl.GL_FLOAT
 * GL_INVERT = uguugl.GL_INVERT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNSIGNED_INT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNSIGNED_INT, __pyx_t_3) < 0) __PYX_ERR(1, 799, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":800
 * GL_INT = uguugl.GL_INT
 * GL_UNSIGNED_INT = uguugl.GL_UNSIGNED_INT
 * GL_FLOAT = uguugl.GL_FLOAT             # <<<<<<<<<<<<<<
 * GL_INVERT = uguugl.GL_INVERT
 * GL_TEXTURE = uguugl.GL_TEXTURE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT, __pyx_t_3) < 0) __PYX_ERR(1, 800, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":801
 * GL_UNSIGNED_INT = uguugl.GL_UNSIGNED_INT
 * GL_FLOAT = uguugl.GL_FLOAT
 * GL_INVERT = uguugl.GL_INVERT             # <<<<<<<<<<<<<<
 * GL_TEXTURE = uguugl.GL_TEXTURE
 * GL_DEPTH_COMPONENT = uguugl.GL_DEPTH_COMPONENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INVERT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INVERT, __pyx_t_3) < 0) __PYX_ERR(1, 801, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":802
 * GL_FLOAT = uguugl.GL_FLOAT
 * GL_INVERT = uguugl.GL_INVERT
 * GL_TEXTURE = uguugl.GL_TEXTURE             # <<<<<<<<<<<<<<
 * GL_DEPTH_COMPONENT = uguugl.GL_DEPTH_COMPONENT
 * GL_ALPHA = uguugl.GL_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 802, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE, __pyx_t_3) < 0) __PYX_ERR(1, 802, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":803
 * GL_INVERT = uguugl.GL_INVERT
 * GL_TEXTURE = uguugl.GL_TEXTURE
 * GL_DEPTH_COMPONENT = uguugl.GL_DEPTH_COMPONENT             # <<<<<<<<<<<<<<
 * GL_ALPHA = uguugl.GL_ALPHA
 * GL_RGB = uguugl.GL_RGB
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_COMPONENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_COMPONENT, __pyx_t_3) < 0) __PYX_ERR(1, 803, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":804
 * GL_TEXTURE = uguugl.GL_TEXTURE
 * GL_DEPTH_COMPONENT = uguugl.GL_DEPTH_COMPONENT
 * GL_ALPHA = uguugl.GL_ALPHA             # <<<<<<<<<<<<<<
 * GL_RGB = uguugl.GL_RGB
 * GL_RGBA = uguugl.GL_RGBA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 804, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":805
 * GL_DEPTH_COMPONENT = uguugl.GL_DEPTH_COMPONENT
 * GL_ALPHA = uguugl.GL_ALPHA
 * GL_RGB = uguugl.GL_RGB             # <<<<<<<<<<<<<<
 * GL_RGBA = uguugl.GL_RGBA
 * GL_LUMINANCE = uguugl.GL_LUMINANCE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RGB); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 805, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RGB, __pyx_t_3) < 0) __PYX_ERR(1, 805, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":806
 * GL_ALPHA = uguugl.GL_ALPHA
 * GL_RGB = uguugl.GL_RGB
 * GL_RGBA = uguugl.GL_RGBA             # <<<<<<<<<<<<<<
 * GL_LUMINANCE = uguugl.GL_LUMINANCE
 * GL_LUMINANCE_ALPHA = uguugl.GL_LUMINANCE_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RGBA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RGBA, __pyx_t_3) < 0) __PYX_ERR(1, 806, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":807
 * GL_RGB = uguugl.GL_RGB
 * GL_RGBA = uguugl.GL_RGBA
 * GL_LUMINANCE = uguugl.GL_LUMINANCE             # <<<<<<<<<<<<<<
 * GL_LUMINANCE_ALPHA = uguugl.GL_LUMINANCE_ALPHA
 * GL_KEEP = uguugl.GL_KEEP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LUMINANCE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LUMINANCE, __pyx_t_3) < 0) __PYX_ERR(1, 807, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":808
 * GL_RGBA = uguugl.GL_RGBA
 * GL_LUMINANCE = uguugl.GL_LUMINANCE
 * GL_LUMINANCE_ALPHA = uguugl.GL_LUMINANCE_ALPHA             # <<<<<<<<<<<<<<
 * GL_KEEP = uguugl.GL_KEEP
 * GL_REPLACE = uguugl.GL_REPLACE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LUMINANCE_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 808, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LUMINANCE_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 808, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":809
 * GL_LUMINANCE = uguugl.GL_LUMINANCE
 * GL_LUMINANCE_ALPHA = uguugl.GL_LUMINANCE_ALPHA
 * GL_KEEP = uguugl.GL_KEEP             # <<<<<<<<<<<<<<
 * GL_REPLACE = uguugl.GL_REPLACE
 * GL_INCR = uguugl.GL_INCR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_KEEP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_KEEP, __pyx_t_3) < 0) __PYX_ERR(1, 809, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":810
 * GL_LUMINANCE_ALPHA = uguugl.GL_LUMINANCE_ALPHA
 * GL_KEEP = uguugl.GL_KEEP
 * GL_REPLACE = uguugl.GL_REPLACE             # <<<<<<<<<<<<<<
 * GL_INCR = uguugl.GL_INCR
 * GL_DECR = uguugl.GL_DECR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_REPLACE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 810, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_REPLACE, __pyx_t_3) < 0) __PYX_ERR(1, 810, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":811
 * GL_KEEP = uguugl.GL_KEEP
 * GL_REPLACE = uguugl.GL_REPLACE
 * GL_INCR = uguugl.GL_INCR             # <<<<<<<<<<<<<<
 * GL_DECR = uguugl.GL_DECR
 * GL_VENDOR = uguugl.GL_VENDOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INCR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INCR, __pyx_t_3) < 0) __PYX_ERR(1, 811, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":812
 * GL_REPLACE = uguugl.GL_REPLACE
 * GL_INCR = uguugl.GL_INCR
 * GL_DECR = uguugl.GL_DECR             # <<<<<<<<<<<<<<
 * GL_VENDOR = uguugl.GL_VENDOR
 * GL_RENDERER = uguugl.GL_RENDERER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DECR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 812, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DECR, __pyx_t_3) < 0) __PYX_ERR(1, 812, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":813
 * GL_INCR = uguugl.GL_INCR
 * GL_DECR = uguugl.GL_DECR
 * GL_VENDOR = uguugl.GL_VENDOR             # <<<<<<<<<<<<<<
 * GL_RENDERER = uguugl.GL_RENDERER
 * GL_VERSION = uguugl.GL_VERSION
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VENDOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VENDOR, __pyx_t_3) < 0) __PYX_ERR(1, 813, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":814
 * GL_DECR = uguugl.GL_DECR
 * GL_VENDOR = uguugl.GL_VENDOR
 * GL_RENDERER = uguugl.GL_RENDERER             # <<<<<<<<<<<<<<
 * GL_VERSION = uguugl.GL_VERSION
 * GL_EXTENSIONS = uguugl.GL_EXTENSIONS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERER, __pyx_t_3) < 0) __PYX_ERR(1, 814, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":815
 * GL_VENDOR = uguugl.GL_VENDOR
 * GL_RENDERER = uguugl.GL_RENDERER
 * GL_VERSION = uguugl.GL_VERSION             # <<<<<<<<<<<<<<
 * GL_EXTENSIONS = uguugl.GL_EXTENSIONS
 * GL_NEAREST = uguugl.GL_NEAREST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERSION); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERSION, __pyx_t_3) < 0) __PYX_ERR(1, 815, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":816
 * GL_RENDERER = uguugl.GL_RENDERER
 * GL_VERSION = uguugl.GL_VERSION
 * GL_EXTENSIONS = uguugl.GL_EXTENSIONS             # <<<<<<<<<<<<<<
 * GL_NEAREST = uguugl.GL_NEAREST
 * GL_LINEAR = uguugl.GL_LINEAR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_EXTENSIONS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_EXTENSIONS, __pyx_t_3) < 0) __PYX_ERR(1, 816, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":817
 * GL_VERSION = uguugl.GL_VERSION
 * GL_EXTENSIONS = uguugl.GL_EXTENSIONS
 * GL_NEAREST = uguugl.GL_NEAREST             # <<<<<<<<<<<<<<
 * GL_LINEAR = uguugl.GL_LINEAR
 * GL_NEAREST_MIPMAP_NEAREST = uguugl.GL_NEAREST_MIPMAP_NEAREST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NEAREST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 817, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NEAREST, __pyx_t_3) < 0) __PYX_ERR(1, 817, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":818
 * GL_EXTENSIONS = uguugl.GL_EXTENSIONS
 * GL_NEAREST = uguugl.GL_NEAREST
 * GL_LINEAR = uguugl.GL_LINEAR             # <<<<<<<<<<<<<<
 * GL_NEAREST_MIPMAP_NEAREST = uguugl.GL_NEAREST_MIPMAP_NEAREST
 * GL_LINEAR_MIPMAP_NEAREST = uguugl.GL_LINEAR_MIPMAP_NEAREST
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINEAR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINEAR, __pyx_t_3) < 0) __PYX_ERR(1, 818, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":819
 * GL_NEAREST = uguugl.GL_NEAREST
 * GL_LINEAR = uguugl.GL_LINEAR
 * GL_NEAREST_MIPMAP_NEAREST = uguugl.GL_NEAREST_MIPMAP_NEAREST             # <<<<<<<<<<<<<<
 * GL_LINEAR_MIPMAP_NEAREST = uguugl.GL_LINEAR_MIPMAP_NEAREST
 * GL_NEAREST_MIPMAP_LINEAR = uguugl.GL_NEAREST_MIPMAP_LINEAR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NEAREST_MIPMAP_NEAREST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 819, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NEAREST_MIPMAP_NEAREST, __pyx_t_3) < 0) __PYX_ERR(1, 819, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":820
 * GL_LINEAR = uguugl.GL_LINEAR
 * GL_NEAREST_MIPMAP_NEAREST = uguugl.GL_NEAREST_MIPMAP_NEAREST
 * GL_LINEAR_MIPMAP_NEAREST = uguugl.GL_LINEAR_MIPMAP_NEAREST             # <<<<<<<<<<<<<<
 * GL_NEAREST_MIPMAP_LINEAR = uguugl.GL_NEAREST_MIPMAP_LINEAR
 * GL_LINEAR_MIPMAP_LINEAR = uguugl.GL_LINEAR_MIPMAP_LINEAR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINEAR_MIPMAP_NEAREST); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINEAR_MIPMAP_NEAREST, __pyx_t_3) < 0) __PYX_ERR(1, 820, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":821
 * GL_NEAREST_MIPMAP_NEAREST = uguugl.GL_NEAREST_MIPMAP_NEAREST
 * GL_LINEAR_MIPMAP_NEAREST = uguugl.GL_LINEAR_MIPMAP_NEAREST
 * GL_NEAREST_MIPMAP_LINEAR = uguugl.GL_NEAREST_MIPMAP_LINEAR             # <<<<<<<<<<<<<<
 * GL_LINEAR_MIPMAP_LINEAR = uguugl.GL_LINEAR_MIPMAP_LINEAR
 * GL_TEXTURE_MAG_FILTER = uguugl.GL_TEXTURE_MAG_FILTER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NEAREST_MIPMAP_LINEAR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NEAREST_MIPMAP_LINEAR, __pyx_t_3) < 0) __PYX_ERR(1, 821, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":822
 * GL_LINEAR_MIPMAP_NEAREST = uguugl.GL_LINEAR_MIPMAP_NEAREST
 * GL_NEAREST_MIPMAP_LINEAR = uguugl.GL_NEAREST_MIPMAP_LINEAR
 * GL_LINEAR_MIPMAP_LINEAR = uguugl.GL_LINEAR_MIPMAP_LINEAR             # <<<<<<<<<<<<<<
 * GL_TEXTURE_MAG_FILTER = uguugl.GL_TEXTURE_MAG_FILTER
 * GL_TEXTURE_MIN_FILTER = uguugl.GL_TEXTURE_MIN_FILTER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINEAR_MIPMAP_LINEAR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINEAR_MIPMAP_LINEAR, __pyx_t_3) < 0) __PYX_ERR(1, 822, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":823
 * GL_NEAREST_MIPMAP_LINEAR = uguugl.GL_NEAREST_MIPMAP_LINEAR
 * GL_LINEAR_MIPMAP_LINEAR = uguugl.GL_LINEAR_MIPMAP_LINEAR
 * GL_TEXTURE_MAG_FILTER = uguugl.GL_TEXTURE_MAG_FILTER             # <<<<<<<<<<<<<<
 * GL_TEXTURE_MIN_FILTER = uguugl.GL_TEXTURE_MIN_FILTER
 * GL_TEXTURE_WRAP_S = uguugl.GL_TEXTURE_WRAP_S
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_MAG_FILTER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 823, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_MAG_FILTER, __pyx_t_3) < 0) __PYX_ERR(1, 823, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":824
 * GL_LINEAR_MIPMAP_LINEAR = uguugl.GL_LINEAR_MIPMAP_LINEAR
 * GL_TEXTURE_MAG_FILTER = uguugl.GL_TEXTURE_MAG_FILTER
 * GL_TEXTURE_MIN_FILTER = uguugl.GL_TEXTURE_MIN_FILTER             # <<<<<<<<<<<<<<
 * GL_TEXTURE_WRAP_S = uguugl.GL_TEXTURE_WRAP_S
 * GL_TEXTURE_WRAP_T = uguugl.GL_TEXTURE_WRAP_T
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_MIN_FILTER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 824, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_MIN_FILTER, __pyx_t_3) < 0) __PYX_ERR(1, 824, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":825
 * GL_TEXTURE_MAG_FILTER = uguugl.GL_TEXTURE_MAG_FILTER
 * GL_TEXTURE_MIN_FILTER = uguugl.GL_TEXTURE_MIN_FILTER
 * GL_TEXTURE_WRAP_S = uguugl.GL_TEXTURE_WRAP_S             # <<<<<<<<<<<<<<
 * GL_TEXTURE_WRAP_T = uguugl.GL_TEXTURE_WRAP_T
 * GL_REPEAT = uguugl.GL_REPEAT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_WRAP_S); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 825, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_WRAP_S, __pyx_t_3) < 0) __PYX_ERR(1, 825, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":826
 * GL_TEXTURE_MIN_FILTER = uguugl.GL_TEXTURE_MIN_FILTER
 * GL_TEXTURE_WRAP_S = uguugl.GL_TEXTURE_WRAP_S
 * GL_TEXTURE_WRAP_T = uguugl.GL_TEXTURE_WRAP_T             # <<<<<<<<<<<<<<
 * GL_REPEAT = uguugl.GL_REPEAT
 * GL_POLYGON_OFFSET_UNITS = uguugl.GL_POLYGON_OFFSET_UNITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_WRAP_T); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 826, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_WRAP_T, __pyx_t_3) < 0) __PYX_ERR(1, 826, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":827
 * GL_TEXTURE_WRAP_S = uguugl.GL_TEXTURE_WRAP_S
 * GL_TEXTURE_WRAP_T = uguugl.GL_TEXTURE_WRAP_T
 * GL_REPEAT = uguugl.GL_REPEAT             # <<<<<<<<<<<<<<
 * GL_POLYGON_OFFSET_UNITS = uguugl.GL_POLYGON_OFFSET_UNITS
 * GL_COLOR_BUFFER_BIT = uguugl.GL_COLOR_BUFFER_BIT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_REPEAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_REPEAT, __pyx_t_3) < 0) __PYX_ERR(1, 827, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":828
 * GL_TEXTURE_WRAP_T = uguugl.GL_TEXTURE_WRAP_T
 * GL_REPEAT = uguugl.GL_REPEAT
 * GL_POLYGON_OFFSET_UNITS = uguugl.GL_POLYGON_OFFSET_UNITS             # <<<<<<<<<<<<<<
 * GL_COLOR_BUFFER_BIT = uguugl.GL_COLOR_BUFFER_BIT
 * GL_CONSTANT_COLOR = uguugl.GL_CONSTANT_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_POLYGON_OFFSET_UNITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 828, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_POLYGON_OFFSET_UNITS, __pyx_t_3) < 0) __PYX_ERR(1, 828, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":829
 * GL_REPEAT = uguugl.GL_REPEAT
 * GL_POLYGON_OFFSET_UNITS = uguugl.GL_POLYGON_OFFSET_UNITS
 * GL_COLOR_BUFFER_BIT = uguugl.GL_COLOR_BUFFER_BIT             # <<<<<<<<<<<<<<
 * GL_CONSTANT_COLOR = uguugl.GL_CONSTANT_COLOR
 * GL_ONE_MINUS_CONSTANT_COLOR = uguugl.GL_ONE_MINUS_CONSTANT_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_COLOR_BUFFER_BIT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 829, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_COLOR_BUFFER_BIT, __pyx_t_3) < 0) __PYX_ERR(1, 829, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":830
 * GL_POLYGON_OFFSET_UNITS = uguugl.GL_POLYGON_OFFSET_UNITS
 * GL_COLOR_BUFFER_BIT = uguugl.GL_COLOR_BUFFER_BIT
 * GL_CONSTANT_COLOR = uguugl.GL_CONSTANT_COLOR             # <<<<<<<<<<<<<<
 * GL_ONE_MINUS_CONSTANT_COLOR = uguugl.GL_ONE_MINUS_CONSTANT_COLOR
 * GL_CONSTANT_ALPHA = uguugl.GL_CONSTANT_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CONSTANT_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 830, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONSTANT_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 830, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":831
 * GL_COLOR_BUFFER_BIT = uguugl.GL_COLOR_BUFFER_BIT
 * GL_CONSTANT_COLOR = uguugl.GL_CONSTANT_COLOR
 * GL_ONE_MINUS_CONSTANT_COLOR = uguugl.GL_ONE_MINUS_CONSTANT_COLOR             # <<<<<<<<<<<<<<
 * GL_CONSTANT_ALPHA = uguugl.GL_CONSTANT_ALPHA
 * GL_ONE_MINUS_CONSTANT_ALPHA = uguugl.GL_ONE_MINUS_CONSTANT_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE_MINUS_CONSTANT_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE_MINUS_CONSTANT_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 831, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":832
 * GL_CONSTANT_COLOR = uguugl.GL_CONSTANT_COLOR
 * GL_ONE_MINUS_CONSTANT_COLOR = uguugl.GL_ONE_MINUS_CONSTANT_COLOR
 * GL_CONSTANT_ALPHA = uguugl.GL_CONSTANT_ALPHA             # <<<<<<<<<<<<<<
 * GL_ONE_MINUS_CONSTANT_ALPHA = uguugl.GL_ONE_MINUS_CONSTANT_ALPHA
 * GL_BLEND_COLOR = uguugl.GL_BLEND_COLOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CONSTANT_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 832, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CONSTANT_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 832, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":833
 * GL_ONE_MINUS_CONSTANT_COLOR = uguugl.GL_ONE_MINUS_CONSTANT_COLOR
 * GL_CONSTANT_ALPHA = uguugl.GL_CONSTANT_ALPHA
 * GL_ONE_MINUS_CONSTANT_ALPHA = uguugl.GL_ONE_MINUS_CONSTANT_ALPHA             # <<<<<<<<<<<<<<
 * GL_BLEND_COLOR = uguugl.GL_BLEND_COLOR
 * GL_FUNC_ADD = uguugl.GL_FUNC_ADD
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ONE_MINUS_CONSTANT_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ONE_MINUS_CONSTANT_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 833, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":834
 * GL_CONSTANT_ALPHA = uguugl.GL_CONSTANT_ALPHA
 * GL_ONE_MINUS_CONSTANT_ALPHA = uguugl.GL_ONE_MINUS_CONSTANT_ALPHA
 * GL_BLEND_COLOR = uguugl.GL_BLEND_COLOR             # <<<<<<<<<<<<<<
 * GL_FUNC_ADD = uguugl.GL_FUNC_ADD
 * GL_BLEND_EQUATION = uguugl.GL_BLEND_EQUATION
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_COLOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_COLOR, __pyx_t_3) < 0) __PYX_ERR(1, 834, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":835
 * GL_ONE_MINUS_CONSTANT_ALPHA = uguugl.GL_ONE_MINUS_CONSTANT_ALPHA
 * GL_BLEND_COLOR = uguugl.GL_BLEND_COLOR
 * GL_FUNC_ADD = uguugl.GL_FUNC_ADD             # <<<<<<<<<<<<<<
 * GL_BLEND_EQUATION = uguugl.GL_BLEND_EQUATION
 * GL_BLEND_EQUATION_RGB = uguugl.GL_BLEND_EQUATION_RGB
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FUNC_ADD); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FUNC_ADD, __pyx_t_3) < 0) __PYX_ERR(1, 835, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":836
 * GL_BLEND_COLOR = uguugl.GL_BLEND_COLOR
 * GL_FUNC_ADD = uguugl.GL_FUNC_ADD
 * GL_BLEND_EQUATION = uguugl.GL_BLEND_EQUATION             # <<<<<<<<<<<<<<
 * GL_BLEND_EQUATION_RGB = uguugl.GL_BLEND_EQUATION_RGB
 * GL_FUNC_SUBTRACT = uguugl.GL_FUNC_SUBTRACT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_EQUATION); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_EQUATION, __pyx_t_3) < 0) __PYX_ERR(1, 836, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":837
 * GL_FUNC_ADD = uguugl.GL_FUNC_ADD
 * GL_BLEND_EQUATION = uguugl.GL_BLEND_EQUATION
 * GL_BLEND_EQUATION_RGB = uguugl.GL_BLEND_EQUATION_RGB             # <<<<<<<<<<<<<<
 * GL_FUNC_SUBTRACT = uguugl.GL_FUNC_SUBTRACT
 * GL_FUNC_REVERSE_SUBTRACT = uguugl.GL_FUNC_REVERSE_SUBTRACT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_EQUATION_RGB); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 837, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_EQUATION_RGB, __pyx_t_3) < 0) __PYX_ERR(1, 837, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":838
 * GL_BLEND_EQUATION = uguugl.GL_BLEND_EQUATION
 * GL_BLEND_EQUATION_RGB = uguugl.GL_BLEND_EQUATION_RGB
 * GL_FUNC_SUBTRACT = uguugl.GL_FUNC_SUBTRACT             # <<<<<<<<<<<<<<
 * GL_FUNC_REVERSE_SUBTRACT = uguugl.GL_FUNC_REVERSE_SUBTRACT
 * GL_UNSIGNED_SHORT_4_4_4_4 = uguugl.GL_UNSIGNED_SHORT_4_4_4_4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FUNC_SUBTRACT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 838, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FUNC_SUBTRACT, __pyx_t_3) < 0) __PYX_ERR(1, 838, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":839
 * GL_BLEND_EQUATION_RGB = uguugl.GL_BLEND_EQUATION_RGB
 * GL_FUNC_SUBTRACT = uguugl.GL_FUNC_SUBTRACT
 * GL_FUNC_REVERSE_SUBTRACT = uguugl.GL_FUNC_REVERSE_SUBTRACT             # <<<<<<<<<<<<<<
 * GL_UNSIGNED_SHORT_4_4_4_4 = uguugl.GL_UNSIGNED_SHORT_4_4_4_4
 * GL_UNSIGNED_SHORT_5_5_5_1 = uguugl.GL_UNSIGNED_SHORT_5_5_5_1
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FUNC_REVERSE_SUBTRACT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FUNC_REVERSE_SUBTRACT, __pyx_t_3) < 0) __PYX_ERR(1, 839, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":840
 * GL_FUNC_SUBTRACT = uguugl.GL_FUNC_SUBTRACT
 * GL_FUNC_REVERSE_SUBTRACT = uguugl.GL_FUNC_REVERSE_SUBTRACT
 * GL_UNSIGNED_SHORT_4_4_4_4 = uguugl.GL_UNSIGNED_SHORT_4_4_4_4             # <<<<<<<<<<<<<<
 * GL_UNSIGNED_SHORT_5_5_5_1 = uguugl.GL_UNSIGNED_SHORT_5_5_5_1
 * GL_POLYGON_OFFSET_FILL = uguugl.GL_POLYGON_OFFSET_FILL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNSIGNED_SHORT_4_4_4_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 840, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNSIGNED_SHORT_4_4_4_4, __pyx_t_3) < 0) __PYX_ERR(1, 840, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":841
 * GL_FUNC_REVERSE_SUBTRACT = uguugl.GL_FUNC_REVERSE_SUBTRACT
 * GL_UNSIGNED_SHORT_4_4_4_4 = uguugl.GL_UNSIGNED_SHORT_4_4_4_4
 * GL_UNSIGNED_SHORT_5_5_5_1 = uguugl.GL_UNSIGNED_SHORT_5_5_5_1             # <<<<<<<<<<<<<<
 * GL_POLYGON_OFFSET_FILL = uguugl.GL_POLYGON_OFFSET_FILL
 * GL_POLYGON_OFFSET_FACTOR = uguugl.GL_POLYGON_OFFSET_FACTOR
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNSIGNED_SHORT_5_5_5_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 841, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNSIGNED_SHORT_5_5_5_1, __pyx_t_3) < 0) __PYX_ERR(1, 841, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":842
 * GL_UNSIGNED_SHORT_4_4_4_4 = uguugl.GL_UNSIGNED_SHORT_4_4_4_4
 * GL_UNSIGNED_SHORT_5_5_5_1 = uguugl.GL_UNSIGNED_SHORT_5_5_5_1
 * GL_POLYGON_OFFSET_FILL = uguugl.GL_POLYGON_OFFSET_FILL             # <<<<<<<<<<<<<<
 * GL_POLYGON_OFFSET_FACTOR = uguugl.GL_POLYGON_OFFSET_FACTOR
 * GL_RGBA4 = uguugl.GL_RGBA4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_POLYGON_OFFSET_FILL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_POLYGON_OFFSET_FILL, __pyx_t_3) < 0) __PYX_ERR(1, 842, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":843
 * GL_UNSIGNED_SHORT_5_5_5_1 = uguugl.GL_UNSIGNED_SHORT_5_5_5_1
 * GL_POLYGON_OFFSET_FILL = uguugl.GL_POLYGON_OFFSET_FILL
 * GL_POLYGON_OFFSET_FACTOR = uguugl.GL_POLYGON_OFFSET_FACTOR             # <<<<<<<<<<<<<<
 * GL_RGBA4 = uguugl.GL_RGBA4
 * GL_RGB5_A1 = uguugl.GL_RGB5_A1
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_POLYGON_OFFSET_FACTOR); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 843, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_POLYGON_OFFSET_FACTOR, __pyx_t_3) < 0) __PYX_ERR(1, 843, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":844
 * GL_POLYGON_OFFSET_FILL = uguugl.GL_POLYGON_OFFSET_FILL
 * GL_POLYGON_OFFSET_FACTOR = uguugl.GL_POLYGON_OFFSET_FACTOR
 * GL_RGBA4 = uguugl.GL_RGBA4             # <<<<<<<<<<<<<<
 * GL_RGB5_A1 = uguugl.GL_RGB5_A1
 * GL_TEXTURE_BINDING_2D = uguugl.GL_TEXTURE_BINDING_2D
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RGBA4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RGBA4, __pyx_t_3) < 0) __PYX_ERR(1, 844, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":845
 * GL_POLYGON_OFFSET_FACTOR = uguugl.GL_POLYGON_OFFSET_FACTOR
 * GL_RGBA4 = uguugl.GL_RGBA4
 * GL_RGB5_A1 = uguugl.GL_RGB5_A1             # <<<<<<<<<<<<<<
 * GL_TEXTURE_BINDING_2D = uguugl.GL_TEXTURE_BINDING_2D
 * GL_SAMPLE_ALPHA_TO_COVERAGE = uguugl.GL_SAMPLE_ALPHA_TO_COVERAGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RGB5_A1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RGB5_A1, __pyx_t_3) < 0) __PYX_ERR(1, 845, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":846
 * GL_RGBA4 = uguugl.GL_RGBA4
 * GL_RGB5_A1 = uguugl.GL_RGB5_A1
 * GL_TEXTURE_BINDING_2D = uguugl.GL_TEXTURE_BINDING_2D             # <<<<<<<<<<<<<<
 * GL_SAMPLE_ALPHA_TO_COVERAGE = uguugl.GL_SAMPLE_ALPHA_TO_COVERAGE
 * GL_SAMPLE_COVERAGE = uguugl.GL_SAMPLE_COVERAGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_BINDING_2D); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_BINDING_2D, __pyx_t_3) < 0) __PYX_ERR(1, 846, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":847
 * GL_RGB5_A1 = uguugl.GL_RGB5_A1
 * GL_TEXTURE_BINDING_2D = uguugl.GL_TEXTURE_BINDING_2D
 * GL_SAMPLE_ALPHA_TO_COVERAGE = uguugl.GL_SAMPLE_ALPHA_TO_COVERAGE             # <<<<<<<<<<<<<<
 * GL_SAMPLE_COVERAGE = uguugl.GL_SAMPLE_COVERAGE
 * GL_SAMPLE_BUFFERS = uguugl.GL_SAMPLE_BUFFERS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLE_ALPHA_TO_COVERAGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLE_ALPHA_TO_COVERAGE, __pyx_t_3) < 0) __PYX_ERR(1, 847, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":848
 * GL_TEXTURE_BINDING_2D = uguugl.GL_TEXTURE_BINDING_2D
 * GL_SAMPLE_ALPHA_TO_COVERAGE = uguugl.GL_SAMPLE_ALPHA_TO_COVERAGE
 * GL_SAMPLE_COVERAGE = uguugl.GL_SAMPLE_COVERAGE             # <<<<<<<<<<<<<<
 * GL_SAMPLE_BUFFERS = uguugl.GL_SAMPLE_BUFFERS
 * GL_SAMPLES = uguugl.GL_SAMPLES
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLE_COVERAGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 848, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLE_COVERAGE, __pyx_t_3) < 0) __PYX_ERR(1, 848, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":849
 * GL_SAMPLE_ALPHA_TO_COVERAGE = uguugl.GL_SAMPLE_ALPHA_TO_COVERAGE
 * GL_SAMPLE_COVERAGE = uguugl.GL_SAMPLE_COVERAGE
 * GL_SAMPLE_BUFFERS = uguugl.GL_SAMPLE_BUFFERS             # <<<<<<<<<<<<<<
 * GL_SAMPLES = uguugl.GL_SAMPLES
 * GL_SAMPLE_COVERAGE_VALUE = uguugl.GL_SAMPLE_COVERAGE_VALUE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLE_BUFFERS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 849, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLE_BUFFERS, __pyx_t_3) < 0) __PYX_ERR(1, 849, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":850
 * GL_SAMPLE_COVERAGE = uguugl.GL_SAMPLE_COVERAGE
 * GL_SAMPLE_BUFFERS = uguugl.GL_SAMPLE_BUFFERS
 * GL_SAMPLES = uguugl.GL_SAMPLES             # <<<<<<<<<<<<<<
 * GL_SAMPLE_COVERAGE_VALUE = uguugl.GL_SAMPLE_COVERAGE_VALUE
 * GL_SAMPLE_COVERAGE_INVERT = uguugl.GL_SAMPLE_COVERAGE_INVERT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLES); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 850, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLES, __pyx_t_3) < 0) __PYX_ERR(1, 850, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":851
 * GL_SAMPLE_BUFFERS = uguugl.GL_SAMPLE_BUFFERS
 * GL_SAMPLES = uguugl.GL_SAMPLES
 * GL_SAMPLE_COVERAGE_VALUE = uguugl.GL_SAMPLE_COVERAGE_VALUE             # <<<<<<<<<<<<<<
 * GL_SAMPLE_COVERAGE_INVERT = uguugl.GL_SAMPLE_COVERAGE_INVERT
 * GL_BLEND_DST_RGB = uguugl.GL_BLEND_DST_RGB
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLE_COVERAGE_VALUE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLE_COVERAGE_VALUE, __pyx_t_3) < 0) __PYX_ERR(1, 851, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":852
 * GL_SAMPLES = uguugl.GL_SAMPLES
 * GL_SAMPLE_COVERAGE_VALUE = uguugl.GL_SAMPLE_COVERAGE_VALUE
 * GL_SAMPLE_COVERAGE_INVERT = uguugl.GL_SAMPLE_COVERAGE_INVERT             # <<<<<<<<<<<<<<
 * GL_BLEND_DST_RGB = uguugl.GL_BLEND_DST_RGB
 * GL_BLEND_SRC_RGB = uguugl.GL_BLEND_SRC_RGB
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLE_COVERAGE_INVERT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLE_COVERAGE_INVERT, __pyx_t_3) < 0) __PYX_ERR(1, 852, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":853
 * GL_SAMPLE_COVERAGE_VALUE = uguugl.GL_SAMPLE_COVERAGE_VALUE
 * GL_SAMPLE_COVERAGE_INVERT = uguugl.GL_SAMPLE_COVERAGE_INVERT
 * GL_BLEND_DST_RGB = uguugl.GL_BLEND_DST_RGB             # <<<<<<<<<<<<<<
 * GL_BLEND_SRC_RGB = uguugl.GL_BLEND_SRC_RGB
 * GL_BLEND_DST_ALPHA = uguugl.GL_BLEND_DST_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_DST_RGB); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_DST_RGB, __pyx_t_3) < 0) __PYX_ERR(1, 853, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":854
 * GL_SAMPLE_COVERAGE_INVERT = uguugl.GL_SAMPLE_COVERAGE_INVERT
 * GL_BLEND_DST_RGB = uguugl.GL_BLEND_DST_RGB
 * GL_BLEND_SRC_RGB = uguugl.GL_BLEND_SRC_RGB             # <<<<<<<<<<<<<<
 * GL_BLEND_DST_ALPHA = uguugl.GL_BLEND_DST_ALPHA
 * GL_BLEND_SRC_ALPHA = uguugl.GL_BLEND_SRC_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_SRC_RGB); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 854, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_SRC_RGB, __pyx_t_3) < 0) __PYX_ERR(1, 854, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":855
 * GL_BLEND_DST_RGB = uguugl.GL_BLEND_DST_RGB
 * GL_BLEND_SRC_RGB = uguugl.GL_BLEND_SRC_RGB
 * GL_BLEND_DST_ALPHA = uguugl.GL_BLEND_DST_ALPHA             # <<<<<<<<<<<<<<
 * GL_BLEND_SRC_ALPHA = uguugl.GL_BLEND_SRC_ALPHA
 * GL_CLAMP_TO_EDGE = uguugl.GL_CLAMP_TO_EDGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_DST_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_DST_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 855, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":856
 * GL_BLEND_SRC_RGB = uguugl.GL_BLEND_SRC_RGB
 * GL_BLEND_DST_ALPHA = uguugl.GL_BLEND_DST_ALPHA
 * GL_BLEND_SRC_ALPHA = uguugl.GL_BLEND_SRC_ALPHA             # <<<<<<<<<<<<<<
 * GL_CLAMP_TO_EDGE = uguugl.GL_CLAMP_TO_EDGE
 * GL_GENERATE_MIPMAP_HINT = uguugl.GL_GENERATE_MIPMAP_HINT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_SRC_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 856, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_SRC_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 856, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":857
 * GL_BLEND_DST_ALPHA = uguugl.GL_BLEND_DST_ALPHA
 * GL_BLEND_SRC_ALPHA = uguugl.GL_BLEND_SRC_ALPHA
 * GL_CLAMP_TO_EDGE = uguugl.GL_CLAMP_TO_EDGE             # <<<<<<<<<<<<<<
 * GL_GENERATE_MIPMAP_HINT = uguugl.GL_GENERATE_MIPMAP_HINT
 * GL_DEPTH_COMPONENT16 = uguugl.GL_DEPTH_COMPONENT16
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CLAMP_TO_EDGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CLAMP_TO_EDGE, __pyx_t_3) < 0) __PYX_ERR(1, 857, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":858
 * GL_BLEND_SRC_ALPHA = uguugl.GL_BLEND_SRC_ALPHA
 * GL_CLAMP_TO_EDGE = uguugl.GL_CLAMP_TO_EDGE
 * GL_GENERATE_MIPMAP_HINT = uguugl.GL_GENERATE_MIPMAP_HINT             # <<<<<<<<<<<<<<
 * GL_DEPTH_COMPONENT16 = uguugl.GL_DEPTH_COMPONENT16
 * GL_UNSIGNED_SHORT_5_6_5 = uguugl.GL_UNSIGNED_SHORT_5_6_5
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_GENERATE_MIPMAP_HINT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 858, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_GENERATE_MIPMAP_HINT, __pyx_t_3) < 0) __PYX_ERR(1, 858, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":859
 * GL_CLAMP_TO_EDGE = uguugl.GL_CLAMP_TO_EDGE
 * GL_GENERATE_MIPMAP_HINT = uguugl.GL_GENERATE_MIPMAP_HINT
 * GL_DEPTH_COMPONENT16 = uguugl.GL_DEPTH_COMPONENT16             # <<<<<<<<<<<<<<
 * GL_UNSIGNED_SHORT_5_6_5 = uguugl.GL_UNSIGNED_SHORT_5_6_5
 * GL_MIRRORED_REPEAT = uguugl.GL_MIRRORED_REPEAT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_COMPONENT16); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 859, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_COMPONENT16, __pyx_t_3) < 0) __PYX_ERR(1, 859, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":860
 * GL_GENERATE_MIPMAP_HINT = uguugl.GL_GENERATE_MIPMAP_HINT
 * GL_DEPTH_COMPONENT16 = uguugl.GL_DEPTH_COMPONENT16
 * GL_UNSIGNED_SHORT_5_6_5 = uguugl.GL_UNSIGNED_SHORT_5_6_5             # <<<<<<<<<<<<<<
 * GL_MIRRORED_REPEAT = uguugl.GL_MIRRORED_REPEAT
 * GL_ALIASED_POINT_SIZE_RANGE = uguugl.GL_ALIASED_POINT_SIZE_RANGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_UNSIGNED_SHORT_5_6_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 860, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_UNSIGNED_SHORT_5_6_5, __pyx_t_3) < 0) __PYX_ERR(1, 860, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":861
 * GL_DEPTH_COMPONENT16 = uguugl.GL_DEPTH_COMPONENT16
 * GL_UNSIGNED_SHORT_5_6_5 = uguugl.GL_UNSIGNED_SHORT_5_6_5
 * GL_MIRRORED_REPEAT = uguugl.GL_MIRRORED_REPEAT             # <<<<<<<<<<<<<<
 * GL_ALIASED_POINT_SIZE_RANGE = uguugl.GL_ALIASED_POINT_SIZE_RANGE
 * GL_ALIASED_LINE_WIDTH_RANGE = uguugl.GL_ALIASED_LINE_WIDTH_RANGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MIRRORED_REPEAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 861, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MIRRORED_REPEAT, __pyx_t_3) < 0) __PYX_ERR(1, 861, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":862
 * GL_UNSIGNED_SHORT_5_6_5 = uguugl.GL_UNSIGNED_SHORT_5_6_5
 * GL_MIRRORED_REPEAT = uguugl.GL_MIRRORED_REPEAT
 * GL_ALIASED_POINT_SIZE_RANGE = uguugl.GL_ALIASED_POINT_SIZE_RANGE             # <<<<<<<<<<<<<<
 * GL_ALIASED_LINE_WIDTH_RANGE = uguugl.GL_ALIASED_LINE_WIDTH_RANGE
 * GL_TEXTURE0 = uguugl.GL_TEXTURE0
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ALIASED_POINT_SIZE_RANGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 862, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ALIASED_POINT_SIZE_RANGE, __pyx_t_3) < 0) __PYX_ERR(1, 862, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":863
 * GL_MIRRORED_REPEAT = uguugl.GL_MIRRORED_REPEAT
 * GL_ALIASED_POINT_SIZE_RANGE = uguugl.GL_ALIASED_POINT_SIZE_RANGE
 * GL_ALIASED_LINE_WIDTH_RANGE = uguugl.GL_ALIASED_LINE_WIDTH_RANGE             # <<<<<<<<<<<<<<
 * GL_TEXTURE0 = uguugl.GL_TEXTURE0
 * GL_TEXTURE1 = uguugl.GL_TEXTURE1
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ALIASED_LINE_WIDTH_RANGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ALIASED_LINE_WIDTH_RANGE, __pyx_t_3) < 0) __PYX_ERR(1, 863, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":864
 * GL_ALIASED_POINT_SIZE_RANGE = uguugl.GL_ALIASED_POINT_SIZE_RANGE
 * GL_ALIASED_LINE_WIDTH_RANGE = uguugl.GL_ALIASED_LINE_WIDTH_RANGE
 * GL_TEXTURE0 = uguugl.GL_TEXTURE0             # <<<<<<<<<<<<<<
 * GL_TEXTURE1 = uguugl.GL_TEXTURE1
 * GL_TEXTURE2 = uguugl.GL_TEXTURE2
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 864, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE0, __pyx_t_3) < 0) __PYX_ERR(1, 864, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":865
 * GL_ALIASED_LINE_WIDTH_RANGE = uguugl.GL_ALIASED_LINE_WIDTH_RANGE
 * GL_TEXTURE0 = uguugl.GL_TEXTURE0
 * GL_TEXTURE1 = uguugl.GL_TEXTURE1             # <<<<<<<<<<<<<<
 * GL_TEXTURE2 = uguugl.GL_TEXTURE2
 * GL_TEXTURE3 = uguugl.GL_TEXTURE3
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE1, __pyx_t_3) < 0) __PYX_ERR(1, 865, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":866
 * GL_TEXTURE0 = uguugl.GL_TEXTURE0
 * GL_TEXTURE1 = uguugl.GL_TEXTURE1
 * GL_TEXTURE2 = uguugl.GL_TEXTURE2             # <<<<<<<<<<<<<<
 * GL_TEXTURE3 = uguugl.GL_TEXTURE3
 * GL_TEXTURE4 = uguugl.GL_TEXTURE4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 866, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE2, __pyx_t_3) < 0) __PYX_ERR(1, 866, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":867
 * GL_TEXTURE1 = uguugl.GL_TEXTURE1
 * GL_TEXTURE2 = uguugl.GL_TEXTURE2
 * GL_TEXTURE3 = uguugl.GL_TEXTURE3             # <<<<<<<<<<<<<<
 * GL_TEXTURE4 = uguugl.GL_TEXTURE4
 * GL_TEXTURE5 = uguugl.GL_TEXTURE5
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE3, __pyx_t_3) < 0) __PYX_ERR(1, 867, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":868
 * GL_TEXTURE2 = uguugl.GL_TEXTURE2
 * GL_TEXTURE3 = uguugl.GL_TEXTURE3
 * GL_TEXTURE4 = uguugl.GL_TEXTURE4             # <<<<<<<<<<<<<<
 * GL_TEXTURE5 = uguugl.GL_TEXTURE5
 * GL_TEXTURE6 = uguugl.GL_TEXTURE6
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE4, __pyx_t_3) < 0) __PYX_ERR(1, 868, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":869
 * GL_TEXTURE3 = uguugl.GL_TEXTURE3
 * GL_TEXTURE4 = uguugl.GL_TEXTURE4
 * GL_TEXTURE5 = uguugl.GL_TEXTURE5             # <<<<<<<<<<<<<<
 * GL_TEXTURE6 = uguugl.GL_TEXTURE6
 * GL_TEXTURE7 = uguugl.GL_TEXTURE7
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 869, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE5, __pyx_t_3) < 0) __PYX_ERR(1, 869, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":870
 * GL_TEXTURE4 = uguugl.GL_TEXTURE4
 * GL_TEXTURE5 = uguugl.GL_TEXTURE5
 * GL_TEXTURE6 = uguugl.GL_TEXTURE6             # <<<<<<<<<<<<<<
 * GL_TEXTURE7 = uguugl.GL_TEXTURE7
 * GL_TEXTURE8 = uguugl.GL_TEXTURE8
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE6); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE6, __pyx_t_3) < 0) __PYX_ERR(1, 870, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":871
 * GL_TEXTURE5 = uguugl.GL_TEXTURE5
 * GL_TEXTURE6 = uguugl.GL_TEXTURE6
 * GL_TEXTURE7 = uguugl.GL_TEXTURE7             # <<<<<<<<<<<<<<
 * GL_TEXTURE8 = uguugl.GL_TEXTURE8
 * GL_TEXTURE9 = uguugl.GL_TEXTURE9
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE7); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 871, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE7, __pyx_t_3) < 0) __PYX_ERR(1, 871, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":872
 * GL_TEXTURE6 = uguugl.GL_TEXTURE6
 * GL_TEXTURE7 = uguugl.GL_TEXTURE7
 * GL_TEXTURE8 = uguugl.GL_TEXTURE8             # <<<<<<<<<<<<<<
 * GL_TEXTURE9 = uguugl.GL_TEXTURE9
 * GL_TEXTURE10 = uguugl.GL_TEXTURE10
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 872, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE8, __pyx_t_3) < 0) __PYX_ERR(1, 872, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":873
 * GL_TEXTURE7 = uguugl.GL_TEXTURE7
 * GL_TEXTURE8 = uguugl.GL_TEXTURE8
 * GL_TEXTURE9 = uguugl.GL_TEXTURE9             # <<<<<<<<<<<<<<
 * GL_TEXTURE10 = uguugl.GL_TEXTURE10
 * GL_TEXTURE11 = uguugl.GL_TEXTURE11
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE9); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 873, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE9, __pyx_t_3) < 0) __PYX_ERR(1, 873, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":874
 * GL_TEXTURE8 = uguugl.GL_TEXTURE8
 * GL_TEXTURE9 = uguugl.GL_TEXTURE9
 * GL_TEXTURE10 = uguugl.GL_TEXTURE10             # <<<<<<<<<<<<<<
 * GL_TEXTURE11 = uguugl.GL_TEXTURE11
 * GL_TEXTURE12 = uguugl.GL_TEXTURE12
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE10); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 874, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE10, __pyx_t_3) < 0) __PYX_ERR(1, 874, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":875
 * GL_TEXTURE9 = uguugl.GL_TEXTURE9
 * GL_TEXTURE10 = uguugl.GL_TEXTURE10
 * GL_TEXTURE11 = uguugl.GL_TEXTURE11             # <<<<<<<<<<<<<<
 * GL_TEXTURE12 = uguugl.GL_TEXTURE12
 * GL_TEXTURE13 = uguugl.GL_TEXTURE13
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE11); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 875, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE11, __pyx_t_3) < 0) __PYX_ERR(1, 875, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":876
 * GL_TEXTURE10 = uguugl.GL_TEXTURE10
 * GL_TEXTURE11 = uguugl.GL_TEXTURE11
 * GL_TEXTURE12 = uguugl.GL_TEXTURE12             # <<<<<<<<<<<<<<
 * GL_TEXTURE13 = uguugl.GL_TEXTURE13
 * GL_TEXTURE14 = uguugl.GL_TEXTURE14
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE12); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 876, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE12, __pyx_t_3) < 0) __PYX_ERR(1, 876, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":877
 * GL_TEXTURE11 = uguugl.GL_TEXTURE11
 * GL_TEXTURE12 = uguugl.GL_TEXTURE12
 * GL_TEXTURE13 = uguugl.GL_TEXTURE13             # <<<<<<<<<<<<<<
 * GL_TEXTURE14 = uguugl.GL_TEXTURE14
 * GL_TEXTURE15 = uguugl.GL_TEXTURE15
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE13); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 877, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE13, __pyx_t_3) < 0) __PYX_ERR(1, 877, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":878
 * GL_TEXTURE12 = uguugl.GL_TEXTURE12
 * GL_TEXTURE13 = uguugl.GL_TEXTURE13
 * GL_TEXTURE14 = uguugl.GL_TEXTURE14             # <<<<<<<<<<<<<<
 * GL_TEXTURE15 = uguugl.GL_TEXTURE15
 * GL_TEXTURE16 = uguugl.GL_TEXTURE16
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE14); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 878, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE14, __pyx_t_3) < 0) __PYX_ERR(1, 878, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":879
 * GL_TEXTURE13 = uguugl.GL_TEXTURE13
 * GL_TEXTURE14 = uguugl.GL_TEXTURE14
 * GL_TEXTURE15 = uguugl.GL_TEXTURE15             # <<<<<<<<<<<<<<
 * GL_TEXTURE16 = uguugl.GL_TEXTURE16
 * GL_TEXTURE17 = uguugl.GL_TEXTURE17
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE15); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 879, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE15, __pyx_t_3) < 0) __PYX_ERR(1, 879, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":880
 * GL_TEXTURE14 = uguugl.GL_TEXTURE14
 * GL_TEXTURE15 = uguugl.GL_TEXTURE15
 * GL_TEXTURE16 = uguugl.GL_TEXTURE16             # <<<<<<<<<<<<<<
 * GL_TEXTURE17 = uguugl.GL_TEXTURE17
 * GL_TEXTURE18 = uguugl.GL_TEXTURE18
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE16); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 880, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE16, __pyx_t_3) < 0) __PYX_ERR(1, 880, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":881
 * GL_TEXTURE15 = uguugl.GL_TEXTURE15
 * GL_TEXTURE16 = uguugl.GL_TEXTURE16
 * GL_TEXTURE17 = uguugl.GL_TEXTURE17             # <<<<<<<<<<<<<<
 * GL_TEXTURE18 = uguugl.GL_TEXTURE18
 * GL_TEXTURE19 = uguugl.GL_TEXTURE19
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE17); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 881, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE17, __pyx_t_3) < 0) __PYX_ERR(1, 881, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":882
 * GL_TEXTURE16 = uguugl.GL_TEXTURE16
 * GL_TEXTURE17 = uguugl.GL_TEXTURE17
 * GL_TEXTURE18 = uguugl.GL_TEXTURE18             # <<<<<<<<<<<<<<
 * GL_TEXTURE19 = uguugl.GL_TEXTURE19
 * GL_TEXTURE20 = uguugl.GL_TEXTURE20
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE18); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 882, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE18, __pyx_t_3) < 0) __PYX_ERR(1, 882, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":883
 * GL_TEXTURE17 = uguugl.GL_TEXTURE17
 * GL_TEXTURE18 = uguugl.GL_TEXTURE18
 * GL_TEXTURE19 = uguugl.GL_TEXTURE19             # <<<<<<<<<<<<<<
 * GL_TEXTURE20 = uguugl.GL_TEXTURE20
 * GL_TEXTURE21 = uguugl.GL_TEXTURE21
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE19); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 883, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE19, __pyx_t_3) < 0) __PYX_ERR(1, 883, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":884
 * GL_TEXTURE18 = uguugl.GL_TEXTURE18
 * GL_TEXTURE19 = uguugl.GL_TEXTURE19
 * GL_TEXTURE20 = uguugl.GL_TEXTURE20             # <<<<<<<<<<<<<<
 * GL_TEXTURE21 = uguugl.GL_TEXTURE21
 * GL_TEXTURE22 = uguugl.GL_TEXTURE22
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE20); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 884, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE20, __pyx_t_3) < 0) __PYX_ERR(1, 884, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":885
 * GL_TEXTURE19 = uguugl.GL_TEXTURE19
 * GL_TEXTURE20 = uguugl.GL_TEXTURE20
 * GL_TEXTURE21 = uguugl.GL_TEXTURE21             # <<<<<<<<<<<<<<
 * GL_TEXTURE22 = uguugl.GL_TEXTURE22
 * GL_TEXTURE23 = uguugl.GL_TEXTURE23
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE21); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 885, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE21, __pyx_t_3) < 0) __PYX_ERR(1, 885, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":886
 * GL_TEXTURE20 = uguugl.GL_TEXTURE20
 * GL_TEXTURE21 = uguugl.GL_TEXTURE21
 * GL_TEXTURE22 = uguugl.GL_TEXTURE22             # <<<<<<<<<<<<<<
 * GL_TEXTURE23 = uguugl.GL_TEXTURE23
 * GL_TEXTURE24 = uguugl.GL_TEXTURE24
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE22); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 886, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE22, __pyx_t_3) < 0) __PYX_ERR(1, 886, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":887
 * GL_TEXTURE21 = uguugl.GL_TEXTURE21
 * GL_TEXTURE22 = uguugl.GL_TEXTURE22
 * GL_TEXTURE23 = uguugl.GL_TEXTURE23             # <<<<<<<<<<<<<<
 * GL_TEXTURE24 = uguugl.GL_TEXTURE24
 * GL_TEXTURE25 = uguugl.GL_TEXTURE25
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE23); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 887, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE23, __pyx_t_3) < 0) __PYX_ERR(1, 887, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":888
 * GL_TEXTURE22 = uguugl.GL_TEXTURE22
 * GL_TEXTURE23 = uguugl.GL_TEXTURE23
 * GL_TEXTURE24 = uguugl.GL_TEXTURE24             # <<<<<<<<<<<<<<
 * GL_TEXTURE25 = uguugl.GL_TEXTURE25
 * GL_TEXTURE26 = uguugl.GL_TEXTURE26
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE24); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 888, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE24, __pyx_t_3) < 0) __PYX_ERR(1, 888, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":889
 * GL_TEXTURE23 = uguugl.GL_TEXTURE23
 * GL_TEXTURE24 = uguugl.GL_TEXTURE24
 * GL_TEXTURE25 = uguugl.GL_TEXTURE25             # <<<<<<<<<<<<<<
 * GL_TEXTURE26 = uguugl.GL_TEXTURE26
 * GL_TEXTURE27 = uguugl.GL_TEXTURE27
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE25); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 889, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE25, __pyx_t_3) < 0) __PYX_ERR(1, 889, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":890
 * GL_TEXTURE24 = uguugl.GL_TEXTURE24
 * GL_TEXTURE25 = uguugl.GL_TEXTURE25
 * GL_TEXTURE26 = uguugl.GL_TEXTURE26             # <<<<<<<<<<<<<<
 * GL_TEXTURE27 = uguugl.GL_TEXTURE27
 * GL_TEXTURE28 = uguugl.GL_TEXTURE28
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE26); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE26, __pyx_t_3) < 0) __PYX_ERR(1, 890, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":891
 * GL_TEXTURE25 = uguugl.GL_TEXTURE25
 * GL_TEXTURE26 = uguugl.GL_TEXTURE26
 * GL_TEXTURE27 = uguugl.GL_TEXTURE27             # <<<<<<<<<<<<<<
 * GL_TEXTURE28 = uguugl.GL_TEXTURE28
 * GL_TEXTURE29 = uguugl.GL_TEXTURE29
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE27); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 891, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE27, __pyx_t_3) < 0) __PYX_ERR(1, 891, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":892
 * GL_TEXTURE26 = uguugl.GL_TEXTURE26
 * GL_TEXTURE27 = uguugl.GL_TEXTURE27
 * GL_TEXTURE28 = uguugl.GL_TEXTURE28             # <<<<<<<<<<<<<<
 * GL_TEXTURE29 = uguugl.GL_TEXTURE29
 * GL_TEXTURE30 = uguugl.GL_TEXTURE30
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE28); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 892, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE28, __pyx_t_3) < 0) __PYX_ERR(1, 892, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":893
 * GL_TEXTURE27 = uguugl.GL_TEXTURE27
 * GL_TEXTURE28 = uguugl.GL_TEXTURE28
 * GL_TEXTURE29 = uguugl.GL_TEXTURE29             # <<<<<<<<<<<<<<
 * GL_TEXTURE30 = uguugl.GL_TEXTURE30
 * GL_TEXTURE31 = uguugl.GL_TEXTURE31
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE29); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE29, __pyx_t_3) < 0) __PYX_ERR(1, 893, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":894
 * GL_TEXTURE28 = uguugl.GL_TEXTURE28
 * GL_TEXTURE29 = uguugl.GL_TEXTURE29
 * GL_TEXTURE30 = uguugl.GL_TEXTURE30             # <<<<<<<<<<<<<<
 * GL_TEXTURE31 = uguugl.GL_TEXTURE31
 * GL_ACTIVE_TEXTURE = uguugl.GL_ACTIVE_TEXTURE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE30); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE30, __pyx_t_3) < 0) __PYX_ERR(1, 894, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":895
 * GL_TEXTURE29 = uguugl.GL_TEXTURE29
 * GL_TEXTURE30 = uguugl.GL_TEXTURE30
 * GL_TEXTURE31 = uguugl.GL_TEXTURE31             # <<<<<<<<<<<<<<
 * GL_ACTIVE_TEXTURE = uguugl.GL_ACTIVE_TEXTURE
 * GL_MAX_RENDERBUFFER_SIZE = uguugl.GL_MAX_RENDERBUFFER_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE31); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE31, __pyx_t_3) < 0) __PYX_ERR(1, 895, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":896
 * GL_TEXTURE30 = uguugl.GL_TEXTURE30
 * GL_TEXTURE31 = uguugl.GL_TEXTURE31
 * GL_ACTIVE_TEXTURE = uguugl.GL_ACTIVE_TEXTURE             # <<<<<<<<<<<<<<
 * GL_MAX_RENDERBUFFER_SIZE = uguugl.GL_MAX_RENDERBUFFER_SIZE
 * GL_INCR_WRAP = uguugl.GL_INCR_WRAP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ACTIVE_TEXTURE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 896, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACTIVE_TEXTURE, __pyx_t_3) < 0) __PYX_ERR(1, 896, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":897
 * GL_TEXTURE31 = uguugl.GL_TEXTURE31
 * GL_ACTIVE_TEXTURE = uguugl.GL_ACTIVE_TEXTURE
 * GL_MAX_RENDERBUFFER_SIZE = uguugl.GL_MAX_RENDERBUFFER_SIZE             # <<<<<<<<<<<<<<
 * GL_INCR_WRAP = uguugl.GL_INCR_WRAP
 * GL_DECR_WRAP = uguugl.GL_DECR_WRAP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_RENDERBUFFER_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 897, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_RENDERBUFFER_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 897, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":898
 * GL_ACTIVE_TEXTURE = uguugl.GL_ACTIVE_TEXTURE
 * GL_MAX_RENDERBUFFER_SIZE = uguugl.GL_MAX_RENDERBUFFER_SIZE
 * GL_INCR_WRAP = uguugl.GL_INCR_WRAP             # <<<<<<<<<<<<<<
 * GL_DECR_WRAP = uguugl.GL_DECR_WRAP
 * GL_TEXTURE_CUBE_MAP = uguugl.GL_TEXTURE_CUBE_MAP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INCR_WRAP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 898, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INCR_WRAP, __pyx_t_3) < 0) __PYX_ERR(1, 898, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":899
 * GL_MAX_RENDERBUFFER_SIZE = uguugl.GL_MAX_RENDERBUFFER_SIZE
 * GL_INCR_WRAP = uguugl.GL_INCR_WRAP
 * GL_DECR_WRAP = uguugl.GL_DECR_WRAP             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP = uguugl.GL_TEXTURE_CUBE_MAP
 * GL_TEXTURE_BINDING_CUBE_MAP = uguugl.GL_TEXTURE_BINDING_CUBE_MAP
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DECR_WRAP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DECR_WRAP, __pyx_t_3) < 0) __PYX_ERR(1, 899, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":900
 * GL_INCR_WRAP = uguugl.GL_INCR_WRAP
 * GL_DECR_WRAP = uguugl.GL_DECR_WRAP
 * GL_TEXTURE_CUBE_MAP = uguugl.GL_TEXTURE_CUBE_MAP             # <<<<<<<<<<<<<<
 * GL_TEXTURE_BINDING_CUBE_MAP = uguugl.GL_TEXTURE_BINDING_CUBE_MAP
 * GL_TEXTURE_CUBE_MAP_POSITIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_X
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP, __pyx_t_3) < 0) __PYX_ERR(1, 900, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":901
 * GL_DECR_WRAP = uguugl.GL_DECR_WRAP
 * GL_TEXTURE_CUBE_MAP = uguugl.GL_TEXTURE_CUBE_MAP
 * GL_TEXTURE_BINDING_CUBE_MAP = uguugl.GL_TEXTURE_BINDING_CUBE_MAP             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP_POSITIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_X
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_X
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_BINDING_CUBE_MAP); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_BINDING_CUBE_MAP, __pyx_t_3) < 0) __PYX_ERR(1, 901, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":902
 * GL_TEXTURE_CUBE_MAP = uguugl.GL_TEXTURE_CUBE_MAP
 * GL_TEXTURE_BINDING_CUBE_MAP = uguugl.GL_TEXTURE_BINDING_CUBE_MAP
 * GL_TEXTURE_CUBE_MAP_POSITIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_X             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_X
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Y
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP_POSITIVE_X); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_X, __pyx_t_3) < 0) __PYX_ERR(1, 902, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":903
 * GL_TEXTURE_BINDING_CUBE_MAP = uguugl.GL_TEXTURE_BINDING_CUBE_MAP
 * GL_TEXTURE_CUBE_MAP_POSITIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_X
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_X             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Y
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP_NEGATIVE_X); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_X, __pyx_t_3) < 0) __PYX_ERR(1, 903, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":904
 * GL_TEXTURE_CUBE_MAP_POSITIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_X
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_X
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Y             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Z
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP_POSITIVE_Y); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_Y, __pyx_t_3) < 0) __PYX_ERR(1, 904, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":905
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_X = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_X
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Y
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Z
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP_NEGATIVE_Y); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_Y, __pyx_t_3) < 0) __PYX_ERR(1, 905, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":906
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Y
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Z             # <<<<<<<<<<<<<<
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
 * GL_MAX_CUBE_MAP_TEXTURE_SIZE = uguugl.GL_MAX_CUBE_MAP_TEXTURE_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP_POSITIVE_Z); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 906, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP_POSITIVE_Z, __pyx_t_3) < 0) __PYX_ERR(1, 906, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":907
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Y = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Y
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Z
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z             # <<<<<<<<<<<<<<
 * GL_MAX_CUBE_MAP_TEXTURE_SIZE = uguugl.GL_MAX_CUBE_MAP_TEXTURE_SIZE
 * GL_VERTEX_ATTRIB_ARRAY_ENABLED = uguugl.GL_VERTEX_ATTRIB_ARRAY_ENABLED
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_TEXTURE_CUBE_MAP_NEGATIVE_Z); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_TEXTURE_CUBE_MAP_NEGATIVE_Z, __pyx_t_3) < 0) __PYX_ERR(1, 907, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":908
 * GL_TEXTURE_CUBE_MAP_POSITIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_POSITIVE_Z
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
 * GL_MAX_CUBE_MAP_TEXTURE_SIZE = uguugl.GL_MAX_CUBE_MAP_TEXTURE_SIZE             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_ENABLED = uguugl.GL_VERTEX_ATTRIB_ARRAY_ENABLED
 * GL_VERTEX_ATTRIB_ARRAY_SIZE = uguugl.GL_VERTEX_ATTRIB_ARRAY_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_CUBE_MAP_TEXTURE_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 908, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_CUBE_MAP_TEXTURE_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 908, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":909
 * GL_TEXTURE_CUBE_MAP_NEGATIVE_Z = uguugl.GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
 * GL_MAX_CUBE_MAP_TEXTURE_SIZE = uguugl.GL_MAX_CUBE_MAP_TEXTURE_SIZE
 * GL_VERTEX_ATTRIB_ARRAY_ENABLED = uguugl.GL_VERTEX_ATTRIB_ARRAY_ENABLED             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_SIZE = uguugl.GL_VERTEX_ATTRIB_ARRAY_SIZE
 * GL_VERTEX_ATTRIB_ARRAY_STRIDE = uguugl.GL_VERTEX_ATTRIB_ARRAY_STRIDE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_ENABLED); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_ENABLED, __pyx_t_3) < 0) __PYX_ERR(1, 909, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":910
 * GL_MAX_CUBE_MAP_TEXTURE_SIZE = uguugl.GL_MAX_CUBE_MAP_TEXTURE_SIZE
 * GL_VERTEX_ATTRIB_ARRAY_ENABLED = uguugl.GL_VERTEX_ATTRIB_ARRAY_ENABLED
 * GL_VERTEX_ATTRIB_ARRAY_SIZE = uguugl.GL_VERTEX_ATTRIB_ARRAY_SIZE             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_STRIDE = uguugl.GL_VERTEX_ATTRIB_ARRAY_STRIDE
 * GL_VERTEX_ATTRIB_ARRAY_TYPE = uguugl.GL_VERTEX_ATTRIB_ARRAY_TYPE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 910, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":911
 * GL_VERTEX_ATTRIB_ARRAY_ENABLED = uguugl.GL_VERTEX_ATTRIB_ARRAY_ENABLED
 * GL_VERTEX_ATTRIB_ARRAY_SIZE = uguugl.GL_VERTEX_ATTRIB_ARRAY_SIZE
 * GL_VERTEX_ATTRIB_ARRAY_STRIDE = uguugl.GL_VERTEX_ATTRIB_ARRAY_STRIDE             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_TYPE = uguugl.GL_VERTEX_ATTRIB_ARRAY_TYPE
 * GL_CURRENT_VERTEX_ATTRIB = uguugl.GL_CURRENT_VERTEX_ATTRIB
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_STRIDE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 911, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_STRIDE, __pyx_t_3) < 0) __PYX_ERR(1, 911, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":912
 * GL_VERTEX_ATTRIB_ARRAY_SIZE = uguugl.GL_VERTEX_ATTRIB_ARRAY_SIZE
 * GL_VERTEX_ATTRIB_ARRAY_STRIDE = uguugl.GL_VERTEX_ATTRIB_ARRAY_STRIDE
 * GL_VERTEX_ATTRIB_ARRAY_TYPE = uguugl.GL_VERTEX_ATTRIB_ARRAY_TYPE             # <<<<<<<<<<<<<<
 * GL_CURRENT_VERTEX_ATTRIB = uguugl.GL_CURRENT_VERTEX_ATTRIB
 * GL_VERTEX_ATTRIB_ARRAY_POINTER = uguugl.GL_VERTEX_ATTRIB_ARRAY_POINTER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_TYPE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_TYPE, __pyx_t_3) < 0) __PYX_ERR(1, 912, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":913
 * GL_VERTEX_ATTRIB_ARRAY_STRIDE = uguugl.GL_VERTEX_ATTRIB_ARRAY_STRIDE
 * GL_VERTEX_ATTRIB_ARRAY_TYPE = uguugl.GL_VERTEX_ATTRIB_ARRAY_TYPE
 * GL_CURRENT_VERTEX_ATTRIB = uguugl.GL_CURRENT_VERTEX_ATTRIB             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_POINTER = uguugl.GL_VERTEX_ATTRIB_ARRAY_POINTER
 * GL_NUM_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_NUM_COMPRESSED_TEXTURE_FORMATS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CURRENT_VERTEX_ATTRIB); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 913, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CURRENT_VERTEX_ATTRIB, __pyx_t_3) < 0) __PYX_ERR(1, 913, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":914
 * GL_VERTEX_ATTRIB_ARRAY_TYPE = uguugl.GL_VERTEX_ATTRIB_ARRAY_TYPE
 * GL_CURRENT_VERTEX_ATTRIB = uguugl.GL_CURRENT_VERTEX_ATTRIB
 * GL_VERTEX_ATTRIB_ARRAY_POINTER = uguugl.GL_VERTEX_ATTRIB_ARRAY_POINTER             # <<<<<<<<<<<<<<
 * GL_NUM_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_NUM_COMPRESSED_TEXTURE_FORMATS
 * GL_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_COMPRESSED_TEXTURE_FORMATS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_POINTER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 914, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_POINTER, __pyx_t_3) < 0) __PYX_ERR(1, 914, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":915
 * GL_CURRENT_VERTEX_ATTRIB = uguugl.GL_CURRENT_VERTEX_ATTRIB
 * GL_VERTEX_ATTRIB_ARRAY_POINTER = uguugl.GL_VERTEX_ATTRIB_ARRAY_POINTER
 * GL_NUM_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_NUM_COMPRESSED_TEXTURE_FORMATS             # <<<<<<<<<<<<<<
 * GL_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_COMPRESSED_TEXTURE_FORMATS
 * GL_BUFFER_SIZE = uguugl.GL_BUFFER_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_NUM_COMPRESSED_TEXTURE_FORMATS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_NUM_COMPRESSED_TEXTURE_FORMAT, __pyx_t_3) < 0) __PYX_ERR(1, 915, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":916
 * GL_VERTEX_ATTRIB_ARRAY_POINTER = uguugl.GL_VERTEX_ATTRIB_ARRAY_POINTER
 * GL_NUM_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_NUM_COMPRESSED_TEXTURE_FORMATS
 * GL_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_COMPRESSED_TEXTURE_FORMATS             # <<<<<<<<<<<<<<
 * GL_BUFFER_SIZE = uguugl.GL_BUFFER_SIZE
 * GL_BUFFER_USAGE = uguugl.GL_BUFFER_USAGE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_COMPRESSED_TEXTURE_FORMATS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 916, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_COMPRESSED_TEXTURE_FORMATS, __pyx_t_3) < 0) __PYX_ERR(1, 916, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":917
 * GL_NUM_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_NUM_COMPRESSED_TEXTURE_FORMATS
 * GL_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_COMPRESSED_TEXTURE_FORMATS
 * GL_BUFFER_SIZE = uguugl.GL_BUFFER_SIZE             # <<<<<<<<<<<<<<
 * GL_BUFFER_USAGE = uguugl.GL_BUFFER_USAGE
 * GL_STENCIL_BACK_FUNC = uguugl.GL_STENCIL_BACK_FUNC
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BUFFER_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BUFFER_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 917, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":918
 * GL_COMPRESSED_TEXTURE_FORMATS = uguugl.GL_COMPRESSED_TEXTURE_FORMATS
 * GL_BUFFER_SIZE = uguugl.GL_BUFFER_SIZE
 * GL_BUFFER_USAGE = uguugl.GL_BUFFER_USAGE             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_FUNC = uguugl.GL_STENCIL_BACK_FUNC
 * GL_STENCIL_BACK_FAIL = uguugl.GL_STENCIL_BACK_FAIL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BUFFER_USAGE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 918, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BUFFER_USAGE, __pyx_t_3) < 0) __PYX_ERR(1, 918, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":919
 * GL_BUFFER_SIZE = uguugl.GL_BUFFER_SIZE
 * GL_BUFFER_USAGE = uguugl.GL_BUFFER_USAGE
 * GL_STENCIL_BACK_FUNC = uguugl.GL_STENCIL_BACK_FUNC             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_FAIL = uguugl.GL_STENCIL_BACK_FAIL
 * GL_STENCIL_BACK_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_BACK_PASS_DEPTH_FAIL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_FUNC); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 919, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_FUNC, __pyx_t_3) < 0) __PYX_ERR(1, 919, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":920
 * GL_BUFFER_USAGE = uguugl.GL_BUFFER_USAGE
 * GL_STENCIL_BACK_FUNC = uguugl.GL_STENCIL_BACK_FUNC
 * GL_STENCIL_BACK_FAIL = uguugl.GL_STENCIL_BACK_FAIL             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_BACK_PASS_DEPTH_FAIL
 * GL_STENCIL_BACK_PASS_DEPTH_PASS = uguugl.GL_STENCIL_BACK_PASS_DEPTH_PASS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_FAIL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 920, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_FAIL, __pyx_t_3) < 0) __PYX_ERR(1, 920, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":921
 * GL_STENCIL_BACK_FUNC = uguugl.GL_STENCIL_BACK_FUNC
 * GL_STENCIL_BACK_FAIL = uguugl.GL_STENCIL_BACK_FAIL
 * GL_STENCIL_BACK_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_BACK_PASS_DEPTH_FAIL             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_PASS_DEPTH_PASS = uguugl.GL_STENCIL_BACK_PASS_DEPTH_PASS
 * GL_BLEND_EQUATION_ALPHA = uguugl.GL_BLEND_EQUATION_ALPHA
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_PASS_DEPTH_FAIL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 921, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_PASS_DEPTH_FAIL, __pyx_t_3) < 0) __PYX_ERR(1, 921, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":922
 * GL_STENCIL_BACK_FAIL = uguugl.GL_STENCIL_BACK_FAIL
 * GL_STENCIL_BACK_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_BACK_PASS_DEPTH_FAIL
 * GL_STENCIL_BACK_PASS_DEPTH_PASS = uguugl.GL_STENCIL_BACK_PASS_DEPTH_PASS             # <<<<<<<<<<<<<<
 * GL_BLEND_EQUATION_ALPHA = uguugl.GL_BLEND_EQUATION_ALPHA
 * GL_MAX_VERTEX_ATTRIBS = uguugl.GL_MAX_VERTEX_ATTRIBS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_PASS_DEPTH_PASS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_PASS_DEPTH_PASS, __pyx_t_3) < 0) __PYX_ERR(1, 922, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":923
 * GL_STENCIL_BACK_PASS_DEPTH_FAIL = uguugl.GL_STENCIL_BACK_PASS_DEPTH_FAIL
 * GL_STENCIL_BACK_PASS_DEPTH_PASS = uguugl.GL_STENCIL_BACK_PASS_DEPTH_PASS
 * GL_BLEND_EQUATION_ALPHA = uguugl.GL_BLEND_EQUATION_ALPHA             # <<<<<<<<<<<<<<
 * GL_MAX_VERTEX_ATTRIBS = uguugl.GL_MAX_VERTEX_ATTRIBS
 * GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = uguugl.GL_VERTEX_ATTRIB_ARRAY_NORMALIZED
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BLEND_EQUATION_ALPHA); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 923, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BLEND_EQUATION_ALPHA, __pyx_t_3) < 0) __PYX_ERR(1, 923, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":924
 * GL_STENCIL_BACK_PASS_DEPTH_PASS = uguugl.GL_STENCIL_BACK_PASS_DEPTH_PASS
 * GL_BLEND_EQUATION_ALPHA = uguugl.GL_BLEND_EQUATION_ALPHA
 * GL_MAX_VERTEX_ATTRIBS = uguugl.GL_MAX_VERTEX_ATTRIBS             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = uguugl.GL_VERTEX_ATTRIB_ARRAY_NORMALIZED
 * GL_MAX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_TEXTURE_IMAGE_UNITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_VERTEX_ATTRIBS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 924, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_VERTEX_ATTRIBS, __pyx_t_3) < 0) __PYX_ERR(1, 924, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":925
 * GL_BLEND_EQUATION_ALPHA = uguugl.GL_BLEND_EQUATION_ALPHA
 * GL_MAX_VERTEX_ATTRIBS = uguugl.GL_MAX_VERTEX_ATTRIBS
 * GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = uguugl.GL_VERTEX_ATTRIB_ARRAY_NORMALIZED             # <<<<<<<<<<<<<<
 * GL_MAX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_TEXTURE_IMAGE_UNITS
 * GL_ARRAY_BUFFER = uguugl.GL_ARRAY_BUFFER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_NORMALIZED); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 925, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_NORMALIZE, __pyx_t_3) < 0) __PYX_ERR(1, 925, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":926
 * GL_MAX_VERTEX_ATTRIBS = uguugl.GL_MAX_VERTEX_ATTRIBS
 * GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = uguugl.GL_VERTEX_ATTRIB_ARRAY_NORMALIZED
 * GL_MAX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_TEXTURE_IMAGE_UNITS             # <<<<<<<<<<<<<<
 * GL_ARRAY_BUFFER = uguugl.GL_ARRAY_BUFFER
 * GL_ELEMENT_ARRAY_BUFFER = uguugl.GL_ELEMENT_ARRAY_BUFFER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_TEXTURE_IMAGE_UNITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 926, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_TEXTURE_IMAGE_UNITS, __pyx_t_3) < 0) __PYX_ERR(1, 926, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":927
 * GL_VERTEX_ATTRIB_ARRAY_NORMALIZED = uguugl.GL_VERTEX_ATTRIB_ARRAY_NORMALIZED
 * GL_MAX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_TEXTURE_IMAGE_UNITS
 * GL_ARRAY_BUFFER = uguugl.GL_ARRAY_BUFFER             # <<<<<<<<<<<<<<
 * GL_ELEMENT_ARRAY_BUFFER = uguugl.GL_ELEMENT_ARRAY_BUFFER
 * GL_ARRAY_BUFFER_BINDING = uguugl.GL_ARRAY_BUFFER_BINDING
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ARRAY_BUFFER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 927, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ARRAY_BUFFER, __pyx_t_3) < 0) __PYX_ERR(1, 927, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":928
 * GL_MAX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_TEXTURE_IMAGE_UNITS
 * GL_ARRAY_BUFFER = uguugl.GL_ARRAY_BUFFER
 * GL_ELEMENT_ARRAY_BUFFER = uguugl.GL_ELEMENT_ARRAY_BUFFER             # <<<<<<<<<<<<<<
 * GL_ARRAY_BUFFER_BINDING = uguugl.GL_ARRAY_BUFFER_BINDING
 * GL_ELEMENT_ARRAY_BUFFER_BINDING = uguugl.GL_ELEMENT_ARRAY_BUFFER_BINDING
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ELEMENT_ARRAY_BUFFER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 928, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ELEMENT_ARRAY_BUFFER, __pyx_t_3) < 0) __PYX_ERR(1, 928, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":929
 * GL_ARRAY_BUFFER = uguugl.GL_ARRAY_BUFFER
 * GL_ELEMENT_ARRAY_BUFFER = uguugl.GL_ELEMENT_ARRAY_BUFFER
 * GL_ARRAY_BUFFER_BINDING = uguugl.GL_ARRAY_BUFFER_BINDING             # <<<<<<<<<<<<<<
 * GL_ELEMENT_ARRAY_BUFFER_BINDING = uguugl.GL_ELEMENT_ARRAY_BUFFER_BINDING
 * GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = uguugl.GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ARRAY_BUFFER_BINDING); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 929, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ARRAY_BUFFER_BINDING, __pyx_t_3) < 0) __PYX_ERR(1, 929, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":930
 * GL_ELEMENT_ARRAY_BUFFER = uguugl.GL_ELEMENT_ARRAY_BUFFER
 * GL_ARRAY_BUFFER_BINDING = uguugl.GL_ARRAY_BUFFER_BINDING
 * GL_ELEMENT_ARRAY_BUFFER_BINDING = uguugl.GL_ELEMENT_ARRAY_BUFFER_BINDING             # <<<<<<<<<<<<<<
 * GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = uguugl.GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING
 * GL_STREAM_DRAW = uguugl.GL_STREAM_DRAW
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ELEMENT_ARRAY_BUFFER_BINDING); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 930, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ELEMENT_ARRAY_BUFFER_BINDING, __pyx_t_3) < 0) __PYX_ERR(1, 930, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":931
 * GL_ARRAY_BUFFER_BINDING = uguugl.GL_ARRAY_BUFFER_BINDING
 * GL_ELEMENT_ARRAY_BUFFER_BINDING = uguugl.GL_ELEMENT_ARRAY_BUFFER_BINDING
 * GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = uguugl.GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING             # <<<<<<<<<<<<<<
 * GL_STREAM_DRAW = uguugl.GL_STREAM_DRAW
 * GL_STATIC_DRAW = uguugl.GL_STATIC_DRAW
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 931, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_ATTRIB_ARRAY_BUFFER_BI, __pyx_t_3) < 0) __PYX_ERR(1, 931, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":932
 * GL_ELEMENT_ARRAY_BUFFER_BINDING = uguugl.GL_ELEMENT_ARRAY_BUFFER_BINDING
 * GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = uguugl.GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING
 * GL_STREAM_DRAW = uguugl.GL_STREAM_DRAW             # <<<<<<<<<<<<<<
 * GL_STATIC_DRAW = uguugl.GL_STATIC_DRAW
 * GL_DYNAMIC_DRAW = uguugl.GL_DYNAMIC_DRAW
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STREAM_DRAW); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 932, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STREAM_DRAW, __pyx_t_3) < 0) __PYX_ERR(1, 932, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":933
 * GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING = uguugl.GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING
 * GL_STREAM_DRAW = uguugl.GL_STREAM_DRAW
 * GL_STATIC_DRAW = uguugl.GL_STATIC_DRAW             # <<<<<<<<<<<<<<
 * GL_DYNAMIC_DRAW = uguugl.GL_DYNAMIC_DRAW
 * GL_FRAGMENT_SHADER = uguugl.GL_FRAGMENT_SHADER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STATIC_DRAW); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 933, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STATIC_DRAW, __pyx_t_3) < 0) __PYX_ERR(1, 933, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":934
 * GL_STREAM_DRAW = uguugl.GL_STREAM_DRAW
 * GL_STATIC_DRAW = uguugl.GL_STATIC_DRAW
 * GL_DYNAMIC_DRAW = uguugl.GL_DYNAMIC_DRAW             # <<<<<<<<<<<<<<
 * GL_FRAGMENT_SHADER = uguugl.GL_FRAGMENT_SHADER
 * GL_VERTEX_SHADER = uguugl.GL_VERTEX_SHADER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DYNAMIC_DRAW); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DYNAMIC_DRAW, __pyx_t_3) < 0) __PYX_ERR(1, 934, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":935
 * GL_STATIC_DRAW = uguugl.GL_STATIC_DRAW
 * GL_DYNAMIC_DRAW = uguugl.GL_DYNAMIC_DRAW
 * GL_FRAGMENT_SHADER = uguugl.GL_FRAGMENT_SHADER             # <<<<<<<<<<<<<<
 * GL_VERTEX_SHADER = uguugl.GL_VERTEX_SHADER
 * GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAGMENT_SHADER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 935, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAGMENT_SHADER, __pyx_t_3) < 0) __PYX_ERR(1, 935, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":936
 * GL_DYNAMIC_DRAW = uguugl.GL_DYNAMIC_DRAW
 * GL_FRAGMENT_SHADER = uguugl.GL_FRAGMENT_SHADER
 * GL_VERTEX_SHADER = uguugl.GL_VERTEX_SHADER             # <<<<<<<<<<<<<<
 * GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS
 * GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VERTEX_SHADER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 936, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VERTEX_SHADER, __pyx_t_3) < 0) __PYX_ERR(1, 936, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":937
 * GL_FRAGMENT_SHADER = uguugl.GL_FRAGMENT_SHADER
 * GL_VERTEX_SHADER = uguugl.GL_VERTEX_SHADER
 * GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS             # <<<<<<<<<<<<<<
 * GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
 * GL_SHADER_TYPE = uguugl.GL_SHADER_TYPE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 937, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_VERTEX_TEXTURE_IMAGE_UNIT, __pyx_t_3) < 0) __PYX_ERR(1, 937, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":938
 * GL_VERTEX_SHADER = uguugl.GL_VERTEX_SHADER
 * GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS
 * GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS             # <<<<<<<<<<<<<<
 * GL_SHADER_TYPE = uguugl.GL_SHADER_TYPE
 * GL_FLOAT_VEC2 = uguugl.GL_FLOAT_VEC2
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 938, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_MAX_COMBINED_TEXTURE_IMAGE_UN, __pyx_t_3) < 0) __PYX_ERR(1, 938, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":939
 * GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_VERTEX_TEXTURE_IMAGE_UNITS
 * GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
 * GL_SHADER_TYPE = uguugl.GL_SHADER_TYPE             # <<<<<<<<<<<<<<
 * GL_FLOAT_VEC2 = uguugl.GL_FLOAT_VEC2
 * GL_FLOAT_VEC3 = uguugl.GL_FLOAT_VEC3
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SHADER_TYPE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 939, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SHADER_TYPE, __pyx_t_3) < 0) __PYX_ERR(1, 939, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":940
 * GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS = uguugl.GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS
 * GL_SHADER_TYPE = uguugl.GL_SHADER_TYPE
 * GL_FLOAT_VEC2 = uguugl.GL_FLOAT_VEC2             # <<<<<<<<<<<<<<
 * GL_FLOAT_VEC3 = uguugl.GL_FLOAT_VEC3
 * GL_FLOAT_VEC4 = uguugl.GL_FLOAT_VEC4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT_VEC2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 940, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT_VEC2, __pyx_t_3) < 0) __PYX_ERR(1, 940, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":941
 * GL_SHADER_TYPE = uguugl.GL_SHADER_TYPE
 * GL_FLOAT_VEC2 = uguugl.GL_FLOAT_VEC2
 * GL_FLOAT_VEC3 = uguugl.GL_FLOAT_VEC3             # <<<<<<<<<<<<<<
 * GL_FLOAT_VEC4 = uguugl.GL_FLOAT_VEC4
 * GL_INT_VEC2 = uguugl.GL_INT_VEC2
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT_VEC3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 941, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT_VEC3, __pyx_t_3) < 0) __PYX_ERR(1, 941, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":942
 * GL_FLOAT_VEC2 = uguugl.GL_FLOAT_VEC2
 * GL_FLOAT_VEC3 = uguugl.GL_FLOAT_VEC3
 * GL_FLOAT_VEC4 = uguugl.GL_FLOAT_VEC4             # <<<<<<<<<<<<<<
 * GL_INT_VEC2 = uguugl.GL_INT_VEC2
 * GL_INT_VEC3 = uguugl.GL_INT_VEC3
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT_VEC4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT_VEC4, __pyx_t_3) < 0) __PYX_ERR(1, 942, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":943
 * GL_FLOAT_VEC3 = uguugl.GL_FLOAT_VEC3
 * GL_FLOAT_VEC4 = uguugl.GL_FLOAT_VEC4
 * GL_INT_VEC2 = uguugl.GL_INT_VEC2             # <<<<<<<<<<<<<<
 * GL_INT_VEC3 = uguugl.GL_INT_VEC3
 * GL_INT_VEC4 = uguugl.GL_INT_VEC4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INT_VEC2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 943, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INT_VEC2, __pyx_t_3) < 0) __PYX_ERR(1, 943, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":944
 * GL_FLOAT_VEC4 = uguugl.GL_FLOAT_VEC4
 * GL_INT_VEC2 = uguugl.GL_INT_VEC2
 * GL_INT_VEC3 = uguugl.GL_INT_VEC3             # <<<<<<<<<<<<<<
 * GL_INT_VEC4 = uguugl.GL_INT_VEC4
 * GL_BOOL = uguugl.GL_BOOL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INT_VEC3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 944, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INT_VEC3, __pyx_t_3) < 0) __PYX_ERR(1, 944, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":945
 * GL_INT_VEC2 = uguugl.GL_INT_VEC2
 * GL_INT_VEC3 = uguugl.GL_INT_VEC3
 * GL_INT_VEC4 = uguugl.GL_INT_VEC4             # <<<<<<<<<<<<<<
 * GL_BOOL = uguugl.GL_BOOL
 * GL_BOOL_VEC2 = uguugl.GL_BOOL_VEC2
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INT_VEC4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INT_VEC4, __pyx_t_3) < 0) __PYX_ERR(1, 945, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":946
 * GL_INT_VEC3 = uguugl.GL_INT_VEC3
 * GL_INT_VEC4 = uguugl.GL_INT_VEC4
 * GL_BOOL = uguugl.GL_BOOL             # <<<<<<<<<<<<<<
 * GL_BOOL_VEC2 = uguugl.GL_BOOL_VEC2
 * GL_BOOL_VEC3 = uguugl.GL_BOOL_VEC3
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BOOL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 946, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BOOL, __pyx_t_3) < 0) __PYX_ERR(1, 946, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":947
 * GL_INT_VEC4 = uguugl.GL_INT_VEC4
 * GL_BOOL = uguugl.GL_BOOL
 * GL_BOOL_VEC2 = uguugl.GL_BOOL_VEC2             # <<<<<<<<<<<<<<
 * GL_BOOL_VEC3 = uguugl.GL_BOOL_VEC3
 * GL_BOOL_VEC4 = uguugl.GL_BOOL_VEC4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BOOL_VEC2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 947, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BOOL_VEC2, __pyx_t_3) < 0) __PYX_ERR(1, 947, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":948
 * GL_BOOL = uguugl.GL_BOOL
 * GL_BOOL_VEC2 = uguugl.GL_BOOL_VEC2
 * GL_BOOL_VEC3 = uguugl.GL_BOOL_VEC3             # <<<<<<<<<<<<<<
 * GL_BOOL_VEC4 = uguugl.GL_BOOL_VEC4
 * GL_FLOAT_MAT2 = uguugl.GL_FLOAT_MAT2
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BOOL_VEC3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 948, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BOOL_VEC3, __pyx_t_3) < 0) __PYX_ERR(1, 948, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":949
 * GL_BOOL_VEC2 = uguugl.GL_BOOL_VEC2
 * GL_BOOL_VEC3 = uguugl.GL_BOOL_VEC3
 * GL_BOOL_VEC4 = uguugl.GL_BOOL_VEC4             # <<<<<<<<<<<<<<
 * GL_FLOAT_MAT2 = uguugl.GL_FLOAT_MAT2
 * GL_FLOAT_MAT3 = uguugl.GL_FLOAT_MAT3
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_BOOL_VEC4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 949, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_BOOL_VEC4, __pyx_t_3) < 0) __PYX_ERR(1, 949, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":950
 * GL_BOOL_VEC3 = uguugl.GL_BOOL_VEC3
 * GL_BOOL_VEC4 = uguugl.GL_BOOL_VEC4
 * GL_FLOAT_MAT2 = uguugl.GL_FLOAT_MAT2             # <<<<<<<<<<<<<<
 * GL_FLOAT_MAT3 = uguugl.GL_FLOAT_MAT3
 * GL_FLOAT_MAT4 = uguugl.GL_FLOAT_MAT4
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT_MAT2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT_MAT2, __pyx_t_3) < 0) __PYX_ERR(1, 950, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":951
 * GL_BOOL_VEC4 = uguugl.GL_BOOL_VEC4
 * GL_FLOAT_MAT2 = uguugl.GL_FLOAT_MAT2
 * GL_FLOAT_MAT3 = uguugl.GL_FLOAT_MAT3             # <<<<<<<<<<<<<<
 * GL_FLOAT_MAT4 = uguugl.GL_FLOAT_MAT4
 * GL_SAMPLER_2D = uguugl.GL_SAMPLER_2D
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT_MAT3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 951, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT_MAT3, __pyx_t_3) < 0) __PYX_ERR(1, 951, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":952
 * GL_FLOAT_MAT2 = uguugl.GL_FLOAT_MAT2
 * GL_FLOAT_MAT3 = uguugl.GL_FLOAT_MAT3
 * GL_FLOAT_MAT4 = uguugl.GL_FLOAT_MAT4             # <<<<<<<<<<<<<<
 * GL_SAMPLER_2D = uguugl.GL_SAMPLER_2D
 * GL_SAMPLER_CUBE = uguugl.GL_SAMPLER_CUBE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FLOAT_MAT4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 952, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FLOAT_MAT4, __pyx_t_3) < 0) __PYX_ERR(1, 952, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":953
 * GL_FLOAT_MAT3 = uguugl.GL_FLOAT_MAT3
 * GL_FLOAT_MAT4 = uguugl.GL_FLOAT_MAT4
 * GL_SAMPLER_2D = uguugl.GL_SAMPLER_2D             # <<<<<<<<<<<<<<
 * GL_SAMPLER_CUBE = uguugl.GL_SAMPLER_CUBE
 * GL_DELETE_STATUS = uguugl.GL_DELETE_STATUS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLER_2D); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 953, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLER_2D, __pyx_t_3) < 0) __PYX_ERR(1, 953, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":954
 * GL_FLOAT_MAT4 = uguugl.GL_FLOAT_MAT4
 * GL_SAMPLER_2D = uguugl.GL_SAMPLER_2D
 * GL_SAMPLER_CUBE = uguugl.GL_SAMPLER_CUBE             # <<<<<<<<<<<<<<
 * GL_DELETE_STATUS = uguugl.GL_DELETE_STATUS
 * GL_COMPILE_STATUS = uguugl.GL_COMPILE_STATUS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SAMPLER_CUBE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 954, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SAMPLER_CUBE, __pyx_t_3) < 0) __PYX_ERR(1, 954, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":955
 * GL_SAMPLER_2D = uguugl.GL_SAMPLER_2D
 * GL_SAMPLER_CUBE = uguugl.GL_SAMPLER_CUBE
 * GL_DELETE_STATUS = uguugl.GL_DELETE_STATUS             # <<<<<<<<<<<<<<
 * GL_COMPILE_STATUS = uguugl.GL_COMPILE_STATUS
 * GL_LINK_STATUS = uguugl.GL_LINK_STATUS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DELETE_STATUS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 955, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DELETE_STATUS, __pyx_t_3) < 0) __PYX_ERR(1, 955, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":956
 * GL_SAMPLER_CUBE = uguugl.GL_SAMPLER_CUBE
 * GL_DELETE_STATUS = uguugl.GL_DELETE_STATUS
 * GL_COMPILE_STATUS = uguugl.GL_COMPILE_STATUS             # <<<<<<<<<<<<<<
 * GL_LINK_STATUS = uguugl.GL_LINK_STATUS
 * GL_VALIDATE_STATUS = uguugl.GL_VALIDATE_STATUS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_COMPILE_STATUS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 956, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_COMPILE_STATUS, __pyx_t_3) < 0) __PYX_ERR(1, 956, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":957
 * GL_DELETE_STATUS = uguugl.GL_DELETE_STATUS
 * GL_COMPILE_STATUS = uguugl.GL_COMPILE_STATUS
 * GL_LINK_STATUS = uguugl.GL_LINK_STATUS             # <<<<<<<<<<<<<<
 * GL_VALIDATE_STATUS = uguugl.GL_VALIDATE_STATUS
 * GL_INFO_LOG_LENGTH = uguugl.GL_INFO_LOG_LENGTH
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_LINK_STATUS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 957, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_LINK_STATUS, __pyx_t_3) < 0) __PYX_ERR(1, 957, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":958
 * GL_COMPILE_STATUS = uguugl.GL_COMPILE_STATUS
 * GL_LINK_STATUS = uguugl.GL_LINK_STATUS
 * GL_VALIDATE_STATUS = uguugl.GL_VALIDATE_STATUS             # <<<<<<<<<<<<<<
 * GL_INFO_LOG_LENGTH = uguugl.GL_INFO_LOG_LENGTH
 * GL_ATTACHED_SHADERS = uguugl.GL_ATTACHED_SHADERS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_VALIDATE_STATUS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 958, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_VALIDATE_STATUS, __pyx_t_3) < 0) __PYX_ERR(1, 958, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":959
 * GL_LINK_STATUS = uguugl.GL_LINK_STATUS
 * GL_VALIDATE_STATUS = uguugl.GL_VALIDATE_STATUS
 * GL_INFO_LOG_LENGTH = uguugl.GL_INFO_LOG_LENGTH             # <<<<<<<<<<<<<<
 * GL_ATTACHED_SHADERS = uguugl.GL_ATTACHED_SHADERS
 * GL_ACTIVE_UNIFORMS = uguugl.GL_ACTIVE_UNIFORMS
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_INFO_LOG_LENGTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 959, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_INFO_LOG_LENGTH, __pyx_t_3) < 0) __PYX_ERR(1, 959, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":960
 * GL_VALIDATE_STATUS = uguugl.GL_VALIDATE_STATUS
 * GL_INFO_LOG_LENGTH = uguugl.GL_INFO_LOG_LENGTH
 * GL_ATTACHED_SHADERS = uguugl.GL_ATTACHED_SHADERS             # <<<<<<<<<<<<<<
 * GL_ACTIVE_UNIFORMS = uguugl.GL_ACTIVE_UNIFORMS
 * GL_ACTIVE_UNIFORM_MAX_LENGTH = uguugl.GL_ACTIVE_UNIFORM_MAX_LENGTH
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ATTACHED_SHADERS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 960, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ATTACHED_SHADERS, __pyx_t_3) < 0) __PYX_ERR(1, 960, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":961
 * GL_INFO_LOG_LENGTH = uguugl.GL_INFO_LOG_LENGTH
 * GL_ATTACHED_SHADERS = uguugl.GL_ATTACHED_SHADERS
 * GL_ACTIVE_UNIFORMS = uguugl.GL_ACTIVE_UNIFORMS             # <<<<<<<<<<<<<<
 * GL_ACTIVE_UNIFORM_MAX_LENGTH = uguugl.GL_ACTIVE_UNIFORM_MAX_LENGTH
 * GL_SHADER_SOURCE_LENGTH = uguugl.GL_SHADER_SOURCE_LENGTH
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ACTIVE_UNIFORMS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 961, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACTIVE_UNIFORMS, __pyx_t_3) < 0) __PYX_ERR(1, 961, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":962
 * GL_ATTACHED_SHADERS = uguugl.GL_ATTACHED_SHADERS
 * GL_ACTIVE_UNIFORMS = uguugl.GL_ACTIVE_UNIFORMS
 * GL_ACTIVE_UNIFORM_MAX_LENGTH = uguugl.GL_ACTIVE_UNIFORM_MAX_LENGTH             # <<<<<<<<<<<<<<
 * GL_SHADER_SOURCE_LENGTH = uguugl.GL_SHADER_SOURCE_LENGTH
 * GL_ACTIVE_ATTRIBUTES = uguugl.GL_ACTIVE_ATTRIBUTES
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ACTIVE_UNIFORM_MAX_LENGTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACTIVE_UNIFORM_MAX_LENGTH, __pyx_t_3) < 0) __PYX_ERR(1, 962, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":963
 * GL_ACTIVE_UNIFORMS = uguugl.GL_ACTIVE_UNIFORMS
 * GL_ACTIVE_UNIFORM_MAX_LENGTH = uguugl.GL_ACTIVE_UNIFORM_MAX_LENGTH
 * GL_SHADER_SOURCE_LENGTH = uguugl.GL_SHADER_SOURCE_LENGTH             # <<<<<<<<<<<<<<
 * GL_ACTIVE_ATTRIBUTES = uguugl.GL_ACTIVE_ATTRIBUTES
 * GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = uguugl.GL_ACTIVE_ATTRIBUTE_MAX_LENGTH
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SHADER_SOURCE_LENGTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 963, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SHADER_SOURCE_LENGTH, __pyx_t_3) < 0) __PYX_ERR(1, 963, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":964
 * GL_ACTIVE_UNIFORM_MAX_LENGTH = uguugl.GL_ACTIVE_UNIFORM_MAX_LENGTH
 * GL_SHADER_SOURCE_LENGTH = uguugl.GL_SHADER_SOURCE_LENGTH
 * GL_ACTIVE_ATTRIBUTES = uguugl.GL_ACTIVE_ATTRIBUTES             # <<<<<<<<<<<<<<
 * GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = uguugl.GL_ACTIVE_ATTRIBUTE_MAX_LENGTH
 * GL_SHADING_LANGUAGE_VERSION = uguugl.GL_SHADING_LANGUAGE_VERSION
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ACTIVE_ATTRIBUTES); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 964, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACTIVE_ATTRIBUTES, __pyx_t_3) < 0) __PYX_ERR(1, 964, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":965
 * GL_SHADER_SOURCE_LENGTH = uguugl.GL_SHADER_SOURCE_LENGTH
 * GL_ACTIVE_ATTRIBUTES = uguugl.GL_ACTIVE_ATTRIBUTES
 * GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = uguugl.GL_ACTIVE_ATTRIBUTE_MAX_LENGTH             # <<<<<<<<<<<<<<
 * GL_SHADING_LANGUAGE_VERSION = uguugl.GL_SHADING_LANGUAGE_VERSION
 * GL_CURRENT_PROGRAM = uguugl.GL_CURRENT_PROGRAM
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_ACTIVE_ATTRIBUTE_MAX_LENGTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 965, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_ACTIVE_ATTRIBUTE_MAX_LENGTH, __pyx_t_3) < 0) __PYX_ERR(1, 965, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":966
 * GL_ACTIVE_ATTRIBUTES = uguugl.GL_ACTIVE_ATTRIBUTES
 * GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = uguugl.GL_ACTIVE_ATTRIBUTE_MAX_LENGTH
 * GL_SHADING_LANGUAGE_VERSION = uguugl.GL_SHADING_LANGUAGE_VERSION             # <<<<<<<<<<<<<<
 * GL_CURRENT_PROGRAM = uguugl.GL_CURRENT_PROGRAM
 * GL_STENCIL_BACK_REF = uguugl.GL_STENCIL_BACK_REF
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_SHADING_LANGUAGE_VERSION); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 966, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_SHADING_LANGUAGE_VERSION, __pyx_t_3) < 0) __PYX_ERR(1, 966, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":967
 * GL_ACTIVE_ATTRIBUTE_MAX_LENGTH = uguugl.GL_ACTIVE_ATTRIBUTE_MAX_LENGTH
 * GL_SHADING_LANGUAGE_VERSION = uguugl.GL_SHADING_LANGUAGE_VERSION
 * GL_CURRENT_PROGRAM = uguugl.GL_CURRENT_PROGRAM             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_REF = uguugl.GL_STENCIL_BACK_REF
 * GL_STENCIL_BACK_VALUE_MASK = uguugl.GL_STENCIL_BACK_VALUE_MASK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_CURRENT_PROGRAM); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 967, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_CURRENT_PROGRAM, __pyx_t_3) < 0) __PYX_ERR(1, 967, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":968
 * GL_SHADING_LANGUAGE_VERSION = uguugl.GL_SHADING_LANGUAGE_VERSION
 * GL_CURRENT_PROGRAM = uguugl.GL_CURRENT_PROGRAM
 * GL_STENCIL_BACK_REF = uguugl.GL_STENCIL_BACK_REF             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_VALUE_MASK = uguugl.GL_STENCIL_BACK_VALUE_MASK
 * GL_STENCIL_BACK_WRITEMASK = uguugl.GL_STENCIL_BACK_WRITEMASK
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_REF); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_REF, __pyx_t_3) < 0) __PYX_ERR(1, 968, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":969
 * GL_CURRENT_PROGRAM = uguugl.GL_CURRENT_PROGRAM
 * GL_STENCIL_BACK_REF = uguugl.GL_STENCIL_BACK_REF
 * GL_STENCIL_BACK_VALUE_MASK = uguugl.GL_STENCIL_BACK_VALUE_MASK             # <<<<<<<<<<<<<<
 * GL_STENCIL_BACK_WRITEMASK = uguugl.GL_STENCIL_BACK_WRITEMASK
 * GL_FRAMEBUFFER_BINDING = uguugl.GL_FRAMEBUFFER_BINDING
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_VALUE_MASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 969, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_VALUE_MASK, __pyx_t_3) < 0) __PYX_ERR(1, 969, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":970
 * GL_STENCIL_BACK_REF = uguugl.GL_STENCIL_BACK_REF
 * GL_STENCIL_BACK_VALUE_MASK = uguugl.GL_STENCIL_BACK_VALUE_MASK
 * GL_STENCIL_BACK_WRITEMASK = uguugl.GL_STENCIL_BACK_WRITEMASK             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_BINDING = uguugl.GL_FRAMEBUFFER_BINDING
 * GL_RENDERBUFFER_BINDING = uguugl.GL_RENDERBUFFER_BINDING
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_BACK_WRITEMASK); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 970, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_BACK_WRITEMASK, __pyx_t_3) < 0) __PYX_ERR(1, 970, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":971
 * GL_STENCIL_BACK_VALUE_MASK = uguugl.GL_STENCIL_BACK_VALUE_MASK
 * GL_STENCIL_BACK_WRITEMASK = uguugl.GL_STENCIL_BACK_WRITEMASK
 * GL_FRAMEBUFFER_BINDING = uguugl.GL_FRAMEBUFFER_BINDING             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_BINDING = uguugl.GL_RENDERBUFFER_BINDING
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_BINDING); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 971, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_BINDING, __pyx_t_3) < 0) __PYX_ERR(1, 971, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":972
 * GL_STENCIL_BACK_WRITEMASK = uguugl.GL_STENCIL_BACK_WRITEMASK
 * GL_FRAMEBUFFER_BINDING = uguugl.GL_FRAMEBUFFER_BINDING
 * GL_RENDERBUFFER_BINDING = uguugl.GL_RENDERBUFFER_BINDING             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_BINDING); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_BINDING, __pyx_t_3) < 0) __PYX_ERR(1, 972, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":973
 * GL_FRAMEBUFFER_BINDING = uguugl.GL_FRAMEBUFFER_BINDING
 * GL_RENDERBUFFER_BINDING = uguugl.GL_RENDERBUFFER_BINDING
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 973, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_OBJECT, __pyx_t_3) < 0) __PYX_ERR(1, 973, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":974
 * GL_RENDERBUFFER_BINDING = uguugl.GL_RENDERBUFFER_BINDING
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 974, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_OBJECT_2, __pyx_t_3) < 0) __PYX_ERR(1, 974, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":975
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE
 * GL_FRAMEBUFFER_COMPLETE = uguugl.GL_FRAMEBUFFER_COMPLETE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 975, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR, __pyx_t_3) < 0) __PYX_ERR(1, 975, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":976
 * GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME = uguugl.GL_FRAMEBUFFER_ATTACHMENT_OBJECT_NAME
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_COMPLETE = uguugl.GL_FRAMEBUFFER_COMPLETE
 * GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 976, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_ATTACHMENT_TEXTUR_2, __pyx_t_3) < 0) __PYX_ERR(1, 976, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":977
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE
 * GL_FRAMEBUFFER_COMPLETE = uguugl.GL_FRAMEBUFFER_COMPLETE             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT
 * GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_COMPLETE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 977, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_COMPLETE, __pyx_t_3) < 0) __PYX_ERR(1, 977, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":978
 * GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE = uguugl.GL_FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE
 * GL_FRAMEBUFFER_COMPLETE = uguugl.GL_FRAMEBUFFER_COMPLETE
 * GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT
 * GL_FRAMEBUFFER_UNSUPPORTED = uguugl.GL_FRAMEBUFFER_UNSUPPORTED
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 978, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_INCOMPLETE_ATTACH, __pyx_t_3) < 0) __PYX_ERR(1, 978, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":979
 * GL_FRAMEBUFFER_COMPLETE = uguugl.GL_FRAMEBUFFER_COMPLETE
 * GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT
 * GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER_UNSUPPORTED = uguugl.GL_FRAMEBUFFER_UNSUPPORTED
 * GL_COLOR_ATTACHMENT0 = uguugl.GL_COLOR_ATTACHMENT0
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_INCOMPLETE_MISSIN, __pyx_t_3) < 0) __PYX_ERR(1, 979, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":980
 * GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT
 * GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT
 * GL_FRAMEBUFFER_UNSUPPORTED = uguugl.GL_FRAMEBUFFER_UNSUPPORTED             # <<<<<<<<<<<<<<
 * GL_COLOR_ATTACHMENT0 = uguugl.GL_COLOR_ATTACHMENT0
 * GL_DEPTH_ATTACHMENT = uguugl.GL_DEPTH_ATTACHMENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER_UNSUPPORTED); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 980, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER_UNSUPPORTED, __pyx_t_3) < 0) __PYX_ERR(1, 980, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":981
 * GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT = uguugl.GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT
 * GL_FRAMEBUFFER_UNSUPPORTED = uguugl.GL_FRAMEBUFFER_UNSUPPORTED
 * GL_COLOR_ATTACHMENT0 = uguugl.GL_COLOR_ATTACHMENT0             # <<<<<<<<<<<<<<
 * GL_DEPTH_ATTACHMENT = uguugl.GL_DEPTH_ATTACHMENT
 * GL_STENCIL_ATTACHMENT = uguugl.GL_STENCIL_ATTACHMENT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_COLOR_ATTACHMENT0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 981, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_COLOR_ATTACHMENT0, __pyx_t_3) < 0) __PYX_ERR(1, 981, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":982
 * GL_FRAMEBUFFER_UNSUPPORTED = uguugl.GL_FRAMEBUFFER_UNSUPPORTED
 * GL_COLOR_ATTACHMENT0 = uguugl.GL_COLOR_ATTACHMENT0
 * GL_DEPTH_ATTACHMENT = uguugl.GL_DEPTH_ATTACHMENT             # <<<<<<<<<<<<<<
 * GL_STENCIL_ATTACHMENT = uguugl.GL_STENCIL_ATTACHMENT
 * GL_FRAMEBUFFER = uguugl.GL_FRAMEBUFFER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_DEPTH_ATTACHMENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_DEPTH_ATTACHMENT, __pyx_t_3) < 0) __PYX_ERR(1, 982, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":983
 * GL_COLOR_ATTACHMENT0 = uguugl.GL_COLOR_ATTACHMENT0
 * GL_DEPTH_ATTACHMENT = uguugl.GL_DEPTH_ATTACHMENT
 * GL_STENCIL_ATTACHMENT = uguugl.GL_STENCIL_ATTACHMENT             # <<<<<<<<<<<<<<
 * GL_FRAMEBUFFER = uguugl.GL_FRAMEBUFFER
 * GL_RENDERBUFFER = uguugl.GL_RENDERBUFFER
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_ATTACHMENT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 983, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_ATTACHMENT, __pyx_t_3) < 0) __PYX_ERR(1, 983, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":984
 * GL_DEPTH_ATTACHMENT = uguugl.GL_DEPTH_ATTACHMENT
 * GL_STENCIL_ATTACHMENT = uguugl.GL_STENCIL_ATTACHMENT
 * GL_FRAMEBUFFER = uguugl.GL_FRAMEBUFFER             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER = uguugl.GL_RENDERBUFFER
 * GL_RENDERBUFFER_WIDTH = uguugl.GL_RENDERBUFFER_WIDTH
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_FRAMEBUFFER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_FRAMEBUFFER, __pyx_t_3) < 0) __PYX_ERR(1, 984, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":985
 * GL_STENCIL_ATTACHMENT = uguugl.GL_STENCIL_ATTACHMENT
 * GL_FRAMEBUFFER = uguugl.GL_FRAMEBUFFER
 * GL_RENDERBUFFER = uguugl.GL_RENDERBUFFER             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_WIDTH = uguugl.GL_RENDERBUFFER_WIDTH
 * GL_RENDERBUFFER_HEIGHT = uguugl.GL_RENDERBUFFER_HEIGHT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 985, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER, __pyx_t_3) < 0) __PYX_ERR(1, 985, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":986
 * GL_FRAMEBUFFER = uguugl.GL_FRAMEBUFFER
 * GL_RENDERBUFFER = uguugl.GL_RENDERBUFFER
 * GL_RENDERBUFFER_WIDTH = uguugl.GL_RENDERBUFFER_WIDTH             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_HEIGHT = uguugl.GL_RENDERBUFFER_HEIGHT
 * GL_RENDERBUFFER_INTERNAL_FORMAT = uguugl.GL_RENDERBUFFER_INTERNAL_FORMAT
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_WIDTH); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_WIDTH, __pyx_t_3) < 0) __PYX_ERR(1, 986, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":987
 * GL_RENDERBUFFER = uguugl.GL_RENDERBUFFER
 * GL_RENDERBUFFER_WIDTH = uguugl.GL_RENDERBUFFER_WIDTH
 * GL_RENDERBUFFER_HEIGHT = uguugl.GL_RENDERBUFFER_HEIGHT             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_INTERNAL_FORMAT = uguugl.GL_RENDERBUFFER_INTERNAL_FORMAT
 * GL_STENCIL_INDEX8 = uguugl.GL_STENCIL_INDEX8
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_HEIGHT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_HEIGHT, __pyx_t_3) < 0) __PYX_ERR(1, 987, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":988
 * GL_RENDERBUFFER_WIDTH = uguugl.GL_RENDERBUFFER_WIDTH
 * GL_RENDERBUFFER_HEIGHT = uguugl.GL_RENDERBUFFER_HEIGHT
 * GL_RENDERBUFFER_INTERNAL_FORMAT = uguugl.GL_RENDERBUFFER_INTERNAL_FORMAT             # <<<<<<<<<<<<<<
 * GL_STENCIL_INDEX8 = uguugl.GL_STENCIL_INDEX8
 * GL_RENDERBUFFER_RED_SIZE = uguugl.GL_RENDERBUFFER_RED_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_INTERNAL_FORMAT); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 988, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_INTERNAL_FORMAT, __pyx_t_3) < 0) __PYX_ERR(1, 988, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":989
 * GL_RENDERBUFFER_HEIGHT = uguugl.GL_RENDERBUFFER_HEIGHT
 * GL_RENDERBUFFER_INTERNAL_FORMAT = uguugl.GL_RENDERBUFFER_INTERNAL_FORMAT
 * GL_STENCIL_INDEX8 = uguugl.GL_STENCIL_INDEX8             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_RED_SIZE = uguugl.GL_RENDERBUFFER_RED_SIZE
 * GL_RENDERBUFFER_GREEN_SIZE = uguugl.GL_RENDERBUFFER_GREEN_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_STENCIL_INDEX8); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 989, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_STENCIL_INDEX8, __pyx_t_3) < 0) __PYX_ERR(1, 989, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":990
 * GL_RENDERBUFFER_INTERNAL_FORMAT = uguugl.GL_RENDERBUFFER_INTERNAL_FORMAT
 * GL_STENCIL_INDEX8 = uguugl.GL_STENCIL_INDEX8
 * GL_RENDERBUFFER_RED_SIZE = uguugl.GL_RENDERBUFFER_RED_SIZE             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_GREEN_SIZE = uguugl.GL_RENDERBUFFER_GREEN_SIZE
 * GL_RENDERBUFFER_BLUE_SIZE = uguugl.GL_RENDERBUFFER_BLUE_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_RED_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 990, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_RED_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 990, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":991
 * GL_STENCIL_INDEX8 = uguugl.GL_STENCIL_INDEX8
 * GL_RENDERBUFFER_RED_SIZE = uguugl.GL_RENDERBUFFER_RED_SIZE
 * GL_RENDERBUFFER_GREEN_SIZE = uguugl.GL_RENDERBUFFER_GREEN_SIZE             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_BLUE_SIZE = uguugl.GL_RENDERBUFFER_BLUE_SIZE
 * GL_RENDERBUFFER_ALPHA_SIZE = uguugl.GL_RENDERBUFFER_ALPHA_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_GREEN_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 991, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_GREEN_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 991, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":992
 * GL_RENDERBUFFER_RED_SIZE = uguugl.GL_RENDERBUFFER_RED_SIZE
 * GL_RENDERBUFFER_GREEN_SIZE = uguugl.GL_RENDERBUFFER_GREEN_SIZE
 * GL_RENDERBUFFER_BLUE_SIZE = uguugl.GL_RENDERBUFFER_BLUE_SIZE             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_ALPHA_SIZE = uguugl.GL_RENDERBUFFER_ALPHA_SIZE
 * GL_RENDERBUFFER_DEPTH_SIZE = uguugl.GL_RENDERBUFFER_DEPTH_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_BLUE_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 992, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_BLUE_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 992, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":993
 * GL_RENDERBUFFER_GREEN_SIZE = uguugl.GL_RENDERBUFFER_GREEN_SIZE
 * GL_RENDERBUFFER_BLUE_SIZE = uguugl.GL_RENDERBUFFER_BLUE_SIZE
 * GL_RENDERBUFFER_ALPHA_SIZE = uguugl.GL_RENDERBUFFER_ALPHA_SIZE             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_DEPTH_SIZE = uguugl.GL_RENDERBUFFER_DEPTH_SIZE
 * GL_RENDERBUFFER_STENCIL_SIZE = uguugl.GL_RENDERBUFFER_STENCIL_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_ALPHA_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_ALPHA_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 993, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":994
 * GL_RENDERBUFFER_BLUE_SIZE = uguugl.GL_RENDERBUFFER_BLUE_SIZE
 * GL_RENDERBUFFER_ALPHA_SIZE = uguugl.GL_RENDERBUFFER_ALPHA_SIZE
 * GL_RENDERBUFFER_DEPTH_SIZE = uguugl.GL_RENDERBUFFER_DEPTH_SIZE             # <<<<<<<<<<<<<<
 * GL_RENDERBUFFER_STENCIL_SIZE = uguugl.GL_RENDERBUFFER_STENCIL_SIZE
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_DEPTH_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 994, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_DEPTH_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 994, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":995
 * GL_RENDERBUFFER_ALPHA_SIZE = uguugl.GL_RENDERBUFFER_ALPHA_SIZE
 * GL_RENDERBUFFER_DEPTH_SIZE = uguugl.GL_RENDERBUFFER_DEPTH_SIZE
 * GL_RENDERBUFFER_STENCIL_SIZE = uguugl.GL_RENDERBUFFER_STENCIL_SIZE             # <<<<<<<<<<<<<<
 */
  __pyx_t_3 = __Pyx_PyInt_From_GLenum(GL_RENDERBUFFER_STENCIL_SIZE); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 995, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GL_RENDERBUFFER_STENCIL_SIZE, __pyx_t_3) < 0) __PYX_ERR(1, 995, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl2/uguu.pyx":1
 * from uguugl cimport GLenum             # <<<<<<<<<<<<<<
 * from uguugl cimport GLboolean
 * from uguugl cimport GLbitfield
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.gl2.uguu", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.gl2.uguu");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLenum(GLenum value) {
    const GLenum neg_one = (GLenum) -1, const_zero = (GLenum) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLenum) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLenum) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLenum) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLenum) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLenum) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLenum),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLuint(GLuint value) {
    const GLuint neg_one = (GLuint) -1, const_zero = (GLuint) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLuint) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLuint) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLuint) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLuint) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLuint) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLuint),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLint(GLint value) {
    const GLint neg_one = (GLint) -1, const_zero = (GLint) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLint) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLint) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLint) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLint) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLint) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLint),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLboolean(GLboolean value) {
    const GLboolean neg_one = (GLboolean) -1, const_zero = (GLboolean) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLboolean) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLboolean) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLboolean) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLboolean) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLboolean) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLboolean),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
static CYTHON_INLINE GLenum __Pyx_PyInt_As_GLenum(PyObject *x) {
    const GLenum neg_one = (GLenum) -1, const_zero = (GLenum) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLenum) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLenum, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLenum) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLenum) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLenum, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLenum) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) >= 2 * PyLong_SHIFT) {
                            return (GLenum) (((((GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLenum) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) >= 3 * PyLong_SHIFT) {
                            return (GLenum) (((((((GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLenum) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) >= 4 * PyLong_SHIFT) {
                            return (GLenum) (((((((((GLenum)digits[3]) << PyLong_SHIFT) | (GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLenum) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLenum) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLenum) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLenum) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLenum, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLenum,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLenum) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 2 * PyLong_SHIFT) {
                            return (GLenum) (((GLenum)-1)*(((((GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLenum) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 2 * PyLong_SHIFT) {
                            return (GLenum) ((((((GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLenum) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 3 * PyLong_SHIFT) {
                            return (GLenum) (((GLenum)-1)*(((((((GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLenum) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 3 * PyLong_SHIFT) {
                            return (GLenum) ((((((((GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLenum) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 4 * PyLong_SHIFT) {
                            return (GLenum) (((GLenum)-1)*(((((((((GLenum)digits[3]) << PyLong_SHIFT) | (GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLenum) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 4 * PyLong_SHIFT) {
                            return (GLenum) ((((((((((GLenum)digits[3]) << PyLong_SHIFT) | (GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLenum) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLenum) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLenum val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLenum) -1;
        }
    } else {
        GLenum val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLenum) -1;
        val = __Pyx_PyInt_As_GLenum(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLenum");
    return (GLenum) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLenum");
    return (GLenum) -1;
}

/* CIntFromPy */
static CYTHON_INLINE GLuint __Pyx_PyInt_As_GLuint(PyObject *x) {
    const GLuint neg_one = (GLuint) -1, const_zero = (GLuint) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLuint) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLuint, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLuint) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLuint) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLuint, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLuint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 2 * PyLong_SHIFT) {
                            return (GLuint) (((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLuint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 3 * PyLong_SHIFT) {
                            return (GLuint) (((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLuint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 4 * PyLong_SHIFT) {
                            return (GLuint) (((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLuint) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLuint) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLuint) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLuint) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLuint, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLuint,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLuint) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLuint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLuint) ((((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLuint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLuint) ((((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLuint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLuint) ((((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLuint) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLuint) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLuint val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLuint) -1;
        }
    } else {
        GLuint val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLuint) -1;
        val = __Pyx_PyInt_As_GLuint(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLuint");
    return (GLuint) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLuint");
    return (GLuint) -1;
}

/* CIntFromPy */
static CYTHON_INLINE ptrdiff_t __Pyx_PyInt_As_ptrdiff_t(PyObject *x) {
    const ptrdiff_t neg_one = (ptrdiff_t) -1, const_zero = (ptrdiff_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(ptrdiff_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (ptrdiff_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (ptrdiff_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(ptrdiff_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(ptrdiff_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) >= 2 * PyLong_SHIFT) {
                            return (ptrdiff_t) (((((ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(ptrdiff_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) >= 3 * PyLong_SHIFT) {
                            return (ptrdiff_t) (((((((ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(ptrdiff_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) >= 4 * PyLong_SHIFT) {
                            return (ptrdiff_t) (((((((((ptrdiff_t)digits[3]) << PyLong_SHIFT) | (ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (ptrdiff_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(ptrdiff_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(ptrdiff_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (ptrdiff_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(ptrdiff_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(ptrdiff_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(ptrdiff_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) - 1 > 2 * PyLong_SHIFT) {
                            return (ptrdiff_t) (((ptrdiff_t)-1)*(((((ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(ptrdiff_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) - 1 > 2 * PyLong_SHIFT) {
                            return (ptrdiff_t) ((((((ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(ptrdiff_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) - 1 > 3 * PyLong_SHIFT) {
                            return (ptrdiff_t) (((ptrdiff_t)-1)*(((((((ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(ptrdiff_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) - 1 > 3 * PyLong_SHIFT) {
                            return (ptrdiff_t) ((((((((ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(ptrdiff_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) - 1 > 4 * PyLong_SHIFT) {
                            return (ptrdiff_t) (((ptrdiff_t)-1)*(((((((((ptrdiff_t)digits[3]) << PyLong_SHIFT) | (ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(ptrdiff_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(ptrdiff_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(ptrdiff_t) - 1 > 4 * PyLong_SHIFT) {
                            return (ptrdiff_t) ((((((((((ptrdiff_t)digits[3]) << PyLong_SHIFT) | (ptrdiff_t)digits[2]) << PyLong_SHIFT) | (ptrdiff_t)digits[1]) << PyLong_SHIFT) | (ptrdiff_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(ptrdiff_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(ptrdiff_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(ptrdiff_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            ptrdiff_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (ptrdiff_t) -1;
        }
    } else {
        ptrdiff_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (ptrdiff_t) -1;
        val = __Pyx_PyInt_As_ptrdiff_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to ptrdiff_t");
    return (ptrdiff_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to ptrdiff_t");
    return (ptrdiff_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE GLbitfield __Pyx_PyInt_As_GLbitfield(PyObject *x) {
    const GLbitfield neg_one = (GLbitfield) -1, const_zero = (GLbitfield) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLbitfield) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLbitfield, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLbitfield) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLbitfield) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLbitfield, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLbitfield) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) >= 2 * PyLong_SHIFT) {
                            return (GLbitfield) (((((GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLbitfield) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) >= 3 * PyLong_SHIFT) {
                            return (GLbitfield) (((((((GLbitfield)digits[2]) << PyLong_SHIFT) | (GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLbitfield) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) >= 4 * PyLong_SHIFT) {
                            return (GLbitfield) (((((((((GLbitfield)digits[3]) << PyLong_SHIFT) | (GLbitfield)digits[2]) << PyLong_SHIFT) | (GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLbitfield) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLbitfield) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLbitfield, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLbitfield) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLbitfield, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLbitfield) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLbitfield, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLbitfield,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLbitfield) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) - 1 > 2 * PyLong_SHIFT) {
                            return (GLbitfield) (((GLbitfield)-1)*(((((GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLbitfield) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) - 1 > 2 * PyLong_SHIFT) {
                            return (GLbitfield) ((((((GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLbitfield) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) - 1 > 3 * PyLong_SHIFT) {
                            return (GLbitfield) (((GLbitfield)-1)*(((((((GLbitfield)digits[2]) << PyLong_SHIFT) | (GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLbitfield) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) - 1 > 3 * PyLong_SHIFT) {
                            return (GLbitfield) ((((((((GLbitfield)digits[2]) << PyLong_SHIFT) | (GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLbitfield) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) - 1 > 4 * PyLong_SHIFT) {
                            return (GLbitfield) (((GLbitfield)-1)*(((((((((GLbitfield)digits[3]) << PyLong_SHIFT) | (GLbitfield)digits[2]) << PyLong_SHIFT) | (GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLbitfield) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLbitfield, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLbitfield) - 1 > 4 * PyLong_SHIFT) {
                            return (GLbitfield) ((((((((((GLbitfield)digits[3]) << PyLong_SHIFT) | (GLbitfield)digits[2]) << PyLong_SHIFT) | (GLbitfield)digits[1]) << PyLong_SHIFT) | (GLbitfield)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLbitfield) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLbitfield, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLbitfield) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLbitfield, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLbitfield val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLbitfield) -1;
        }
    } else {
        GLbitfield val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLbitfield) -1;
        val = __Pyx_PyInt_As_GLbitfield(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLbitfield");
    return (GLbitfield) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLbitfield");
    return (GLbitfield) -1;
}

/* CIntFromPy */
static CYTHON_INLINE GLint __Pyx_PyInt_As_GLint(PyObject *x) {
    const GLint neg_one = (GLint) -1, const_zero = (GLint) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLint) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLint, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLint) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLint) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLint, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) >= 2 * PyLong_SHIFT) {
                            return (GLint) (((((GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) >= 3 * PyLong_SHIFT) {
                            return (GLint) (((((((GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) >= 4 * PyLong_SHIFT) {
                            return (GLint) (((((((((GLint)digits[3]) << PyLong_SHIFT) | (GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLint) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLint) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLint) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLint) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLint, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLint,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLint) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLint) (((GLint)-1)*(((((GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLint) ((((((GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLint) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLint) (((GLint)-1)*(((((((GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLint) ((((((((GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLint) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLint) (((GLint)-1)*(((((((((GLint)digits[3]) << PyLong_SHIFT) | (GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLint) ((((((((((GLint)digits[3]) << PyLong_SHIFT) | (GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLint) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLint) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLint val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLint) -1;
        }
    } else {
        GLint val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLint) -1;
        val = __Pyx_PyInt_As_GLint(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLint");
    return (GLint) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLint");
    return (GLint) -1;
}

/* CIntFromPy */
static CYTHON_INLINE GLboolean __Pyx_PyInt_As_GLboolean(PyObject *x) {
    const GLboolean neg_one = (GLboolean) -1, const_zero = (GLboolean) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLboolean) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLboolean, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLboolean) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLboolean) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLboolean, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLboolean) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) >= 2 * PyLong_SHIFT) {
                            return (GLboolean) (((((GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLboolean) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) >= 3 * PyLong_SHIFT) {
                            return (GLboolean) (((((((GLboolean)digits[2]) << PyLong_SHIFT) | (GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLboolean) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) >= 4 * PyLong_SHIFT) {
                            return (GLboolean) (((((((((GLboolean)digits[3]) << PyLong_SHIFT) | (GLboolean)digits[2]) << PyLong_SHIFT) | (GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLboolean) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLboolean) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLboolean, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLboolean) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLboolean, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLboolean) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLboolean, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLboolean,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLboolean) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) - 1 > 2 * PyLong_SHIFT) {
                            return (GLboolean) (((GLboolean)-1)*(((((GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLboolean) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) - 1 > 2 * PyLong_SHIFT) {
                            return (GLboolean) ((((((GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLboolean) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) - 1 > 3 * PyLong_SHIFT) {
                            return (GLboolean) (((GLboolean)-1)*(((((((GLboolean)digits[2]) << PyLong_SHIFT) | (GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLboolean) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) - 1 > 3 * PyLong_SHIFT) {
                            return (GLboolean) ((((((((GLboolean)digits[2]) << PyLong_SHIFT) | (GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLboolean) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) - 1 > 4 * PyLong_SHIFT) {
                            return (GLboolean) (((GLboolean)-1)*(((((((((GLboolean)digits[3]) << PyLong_SHIFT) | (GLboolean)digits[2]) << PyLong_SHIFT) | (GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLboolean) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLboolean, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLboolean) - 1 > 4 * PyLong_SHIFT) {
                            return (GLboolean) ((((((((((GLboolean)digits[3]) << PyLong_SHIFT) | (GLboolean)digits[2]) << PyLong_SHIFT) | (GLboolean)digits[1]) << PyLong_SHIFT) | (GLboolean)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLboolean) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLboolean, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLboolean) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLboolean, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLboolean val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLboolean) -1;
        }
    } else {
        GLboolean val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLboolean) -1;
        val = __Pyx_PyInt_As_GLboolean(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLboolean");
    return (GLboolean) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLboolean");
    return (GLboolean) -1;
}

/* CIntFromPy */
static CYTHON_INLINE GLsizei __Pyx_PyInt_As_GLsizei(PyObject *x) {
    const GLsizei neg_one = (GLsizei) -1, const_zero = (GLsizei) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLsizei) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLsizei, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLsizei) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLsizei) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLsizei, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLsizei) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) >= 2 * PyLong_SHIFT) {
                            return (GLsizei) (((((GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLsizei) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) >= 3 * PyLong_SHIFT) {
                            return (GLsizei) (((((((GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLsizei) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) >= 4 * PyLong_SHIFT) {
                            return (GLsizei) (((((((((GLsizei)digits[3]) << PyLong_SHIFT) | (GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLsizei) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLsizei) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLsizei) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLsizei) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLsizei, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLsizei,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLsizei) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 2 * PyLong_SHIFT) {
                            return (GLsizei) (((GLsizei)-1)*(((((GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLsizei) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 2 * PyLong_SHIFT) {
                            return (GLsizei) ((((((GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLsizei) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 3 * PyLong_SHIFT) {
                            return (GLsizei) (((GLsizei)-1)*(((((((GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLsizei) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 3 * PyLong_SHIFT) {
                            return (GLsizei) ((((((((GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLsizei) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 4 * PyLong_SHIFT) {
                            return (GLsizei) (((GLsizei)-1)*(((((((((GLsizei)digits[3]) << PyLong_SHIFT) | (GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLsizei) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 4 * PyLong_SHIFT) {
                            return (GLsizei) ((((((((((GLsizei)digits[3]) << PyLong_SHIFT) | (GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLsizei) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLsizei) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLsizei val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLsizei) -1;
        }
    } else {
        GLsizei val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLsizei) -1;
        val = __Pyx_PyInt_As_GLsizei(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLsizei");
    return (GLsizei) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLsizei");
    return (GLsizei) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
