/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__style_selected_activate_functions
#define __PYX_HAVE_API__style_selected_activate_functions
#include <string.h>
#include <stdio.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "include/style_common.pxi",
  "gen/style_selected_activate_functions.pyx",
  "../renpy/style.pxd",
  "type.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5renpy_5style_StyleCore;

/* "renpy/style.pxd":39
 * # value
 * #     The value of the style property.
 * ctypedef int (*property_function)(PyObject **cache, int *cache_priorities, int priority, object value) except -1             # <<<<<<<<<<<<<<
 * 
 * cdef inline void assign(int index, PyObject **cache, int *cache_priorities, int priority, PyObject *value):
 */
typedef int (*__pyx_t_5renpy_5style_property_function)(PyObject **, int *, int, PyObject *);

/* "renpy/style.pxd":84
 * cdef void register_property_function(name, property_function function)
 * 
 * cdef class StyleCore:             # <<<<<<<<<<<<<<
 * 
 *     ############################################################## Public Fields
 */
struct __pyx_obj_5renpy_5style_StyleCore {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_5style_StyleCore *__pyx_vtab;
  PyObject *name;
  PyObject *parent;
  PyObject *properties;
  PyObject *prefix;
  PyObject *help;
  int built;
  int building;
  struct __pyx_obj_5renpy_5style_StyleCore *down_parent;
  struct __pyx_obj_5renpy_5style_StyleCore *left_parent;
  PyObject **cache;
  int prefix_offset;
};



struct __pyx_vtabstruct_5renpy_5style_StyleCore {
  PyObject *(*_get)(struct __pyx_obj_5renpy_5style_StyleCore *, int, int __pyx_skip_dispatch);
  PyObject *(*_get_unoffset)(struct __pyx_obj_5renpy_5style_StyleCore *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_5style_StyleCore *__pyx_vtabptr_5renpy_5style_StyleCore;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'renpy.style' */
static PyTypeObject *__pyx_ptype_5renpy_5style_StyleCore = 0;
static CYTHON_INLINE void __pyx_f_5renpy_5style_assign(int, PyObject **, int *, int, PyObject *); /*proto*/
static CYTHON_INLINE void __pyx_f_5renpy_5style_assign_prefixed(int, PyObject **, int *, int, PyObject *, PyObject *); /*proto*/
static void (*__pyx_f_5renpy_5style_register_property_function)(PyObject *, __pyx_t_5renpy_5style_property_function); /*proto*/

/* Module declarations from 'style_selected_activate_functions' */
static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_0(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_1(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_2(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_3(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_2_or_0(PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_3_or_1(PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_black_color_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_margin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_line_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bar_resizing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_left_margin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yminimum_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_alt_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_font_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_underline_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_layout_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_thumb_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_box_reverse_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_outline_scaling_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_text_y_fudge_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_aft_gutter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_right_margin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_rest_indent_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_background_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_outlines_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ymaximum_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yspacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_size_group_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_debug_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_clipping_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_first_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_subtitle_width_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_subpixel_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_strikethrough_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_hyperlink_functions_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xminimum_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_min_width_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_focus_mask_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yfit_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_drop_shadow_color_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_fit_first_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_top_padding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_newline_indent_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_text_align_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_fore_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_aft_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_foreground_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bold_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_vertical_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_drop_shadow_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ypos_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_activate_sound_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_left_padding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xoffset_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_language_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_time_policy_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xmaximum_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_hover_sound_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yanchor_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_fore_gutter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_adjust_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xfill_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_color_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_key_events_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xspacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_size_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_caret_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_padding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_box_wrap_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_line_overlap_split_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_box_wrap_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_kerning_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_italic_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ruby_style_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_box_layout_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xpos_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xanchor_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_top_margin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yoffset_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_child_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_thumb_offset_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xfit_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_hinting_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_altruby_style_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_unscrollable_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_order_reverse_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_mouse_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_keyboard_focus_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_first_indent_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bar_vertical_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_right_padding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_focus_rect_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_antialias_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_cps_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bar_invert_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_line_leading_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_cps_multiplier_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_thumb_shadow_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yfill_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_abortable_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_justify_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_left_gutter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_gutter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_offset_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xcenter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_pos_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_minwidth_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_minimum_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_top_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_base_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xysize_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_speed_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_area_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_right_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_enable_hover_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_textalign_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_box_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_padding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_box_first_spacing_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_right_gutter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xsize_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xpadding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_top_gutter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xalign_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_xmargin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_yalign_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ypadding_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_align_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ysize_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_maximum_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ycenter_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_left_bar_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_ymargin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_margin_property(PyObject **, int *, int, PyObject *); /*proto*/
static int __pyx_f_33style_selected_activate_functions_selected_activate_anchor_property(PyObject **, int *, int, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "style_selected_activate_functions"
extern int __pyx_module_is_main_style_selected_activate_functions;
int __pyx_module_is_main_style_selected_activate_functions = 0;

/* Implementation of 'style_selected_activate_functions' */
static const char __pyx_k_args[] = "_args";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_easy[] = "easy";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_color[] = "color";
static const char __pyx_k_renpy[] = "renpy";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_prefix[] = "prefix";
static const char __pyx_k_unique[] = "_unique";
static const char __pyx_k_duplicate[] = "_duplicate";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_duplicatable[] = "_duplicatable";
static const char __pyx_k_none_is_null[] = "none_is_null";
static const char __pyx_k_expand_anchor[] = "expand_anchor";
static const char __pyx_k_expand_outlines[] = "expand_outlines";
static const char __pyx_k_expand_focus_mask[] = "expand_focus_mask";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_displayable_or_none[] = "displayable_or_none";
static const char __pyx_k_selected_activate_alt[] = "selected_activate_alt";
static const char __pyx_k_selected_activate_pos[] = "selected_activate_pos";
static const char __pyx_k_selected_activate_area[] = "selected_activate_area";
static const char __pyx_k_selected_activate_bold[] = "selected_activate_bold";
static const char __pyx_k_selected_activate_font[] = "selected_activate_font";
static const char __pyx_k_selected_activate_size[] = "selected_activate_size";
static const char __pyx_k_selected_activate_xfit[] = "selected_activate_xfit";
static const char __pyx_k_selected_activate_xpos[] = "selected_activate_xpos";
static const char __pyx_k_selected_activate_yfit[] = "selected_activate_yfit";
static const char __pyx_k_selected_activate_ypos[] = "selected_activate_ypos";
static const char __pyx_k_selected_activate_align[] = "selected_activate_align";
static const char __pyx_k_selected_activate_caret[] = "selected_activate_caret";
static const char __pyx_k_selected_activate_child[] = "selected_activate_child";
static const char __pyx_k_selected_activate_color[] = "selected_activate_color";
static const char __pyx_k_selected_activate_debug[] = "selected_activate_debug";
static const char __pyx_k_selected_activate_mouse[] = "selected_activate_mouse";
static const char __pyx_k_selected_activate_thumb[] = "selected_activate_thumb";
static const char __pyx_k_selected_activate_xfill[] = "selected_activate_xfill";
static const char __pyx_k_selected_activate_xsize[] = "selected_activate_xsize";
static const char __pyx_k_selected_activate_yfill[] = "selected_activate_yfill";
static const char __pyx_k_selected_activate_ysize[] = "selected_activate_ysize";
static const char __pyx_k_selected_activate_anchor[] = "selected_activate_anchor";
static const char __pyx_k_selected_activate_italic[] = "selected_activate_italic";
static const char __pyx_k_selected_activate_layout[] = "selected_activate_layout";
static const char __pyx_k_selected_activate_margin[] = "selected_activate_margin";
static const char __pyx_k_selected_activate_offset[] = "selected_activate_offset";
static const char __pyx_k_selected_activate_xalign[] = "selected_activate_xalign";
static const char __pyx_k_selected_activate_xysize[] = "selected_activate_xysize";
static const char __pyx_k_selected_activate_yalign[] = "selected_activate_yalign";
static const char __pyx_k_renpy_styledata_styleutil[] = "renpy.styledata.styleutil";
static const char __pyx_k_selected_activate_aft_bar[] = "selected_activate_aft_bar";
static const char __pyx_k_selected_activate_hinting[] = "selected_activate_hinting";
static const char __pyx_k_selected_activate_justify[] = "selected_activate_justify";
static const char __pyx_k_selected_activate_kerning[] = "selected_activate_kerning";
static const char __pyx_k_selected_activate_maximum[] = "selected_activate_maximum";
static const char __pyx_k_selected_activate_minimum[] = "selected_activate_minimum";
static const char __pyx_k_selected_activate_padding[] = "selected_activate_padding";
static const char __pyx_k_selected_activate_spacing[] = "selected_activate_spacing";
static const char __pyx_k_selected_activate_top_bar[] = "selected_activate_top_bar";
static const char __pyx_k_selected_activate_xanchor[] = "selected_activate_xanchor";
static const char __pyx_k_selected_activate_xcenter[] = "selected_activate_xcenter";
static const char __pyx_k_selected_activate_xmargin[] = "selected_activate_xmargin";
static const char __pyx_k_selected_activate_xoffset[] = "selected_activate_xoffset";
static const char __pyx_k_selected_activate_yanchor[] = "selected_activate_yanchor";
static const char __pyx_k_selected_activate_ycenter[] = "selected_activate_ycenter";
static const char __pyx_k_selected_activate_ymargin[] = "selected_activate_ymargin";
static const char __pyx_k_selected_activate_yoffset[] = "selected_activate_yoffset";
static const char __pyx_k_selected_activate_base_bar[] = "selected_activate_base_bar";
static const char __pyx_k_selected_activate_box_wrap[] = "selected_activate_box_wrap";
static const char __pyx_k_selected_activate_clipping[] = "selected_activate_clipping";
static const char __pyx_k_selected_activate_fore_bar[] = "selected_activate_fore_bar";
static const char __pyx_k_selected_activate_language[] = "selected_activate_language";
static const char __pyx_k_selected_activate_left_bar[] = "selected_activate_left_bar";
static const char __pyx_k_selected_activate_minwidth[] = "selected_activate_minwidth";
static const char __pyx_k_selected_activate_outlines[] = "selected_activate_outlines";
static const char __pyx_k_selected_activate_slow_cps[] = "selected_activate_slow_cps";
static const char __pyx_k_selected_activate_subpixel[] = "selected_activate_subpixel";
static const char __pyx_k_selected_activate_vertical[] = "selected_activate_vertical";
static const char __pyx_k_selected_activate_xmaximum[] = "selected_activate_xmaximum";
static const char __pyx_k_selected_activate_xminimum[] = "selected_activate_xminimum";
static const char __pyx_k_selected_activate_xpadding[] = "selected_activate_xpadding";
static const char __pyx_k_selected_activate_xspacing[] = "selected_activate_xspacing";
static const char __pyx_k_selected_activate_ymaximum[] = "selected_activate_ymaximum";
static const char __pyx_k_selected_activate_yminimum[] = "selected_activate_yminimum";
static const char __pyx_k_selected_activate_ypadding[] = "selected_activate_ypadding";
static const char __pyx_k_selected_activate_yspacing[] = "selected_activate_yspacing";
static const char __pyx_k_selected_activate_antialias[] = "selected_activate_antialias";
static const char __pyx_k_selected_activate_fit_first[] = "selected_activate_fit_first";
static const char __pyx_k_selected_activate_min_width[] = "selected_activate_min_width";
static const char __pyx_k_selected_activate_right_bar[] = "selected_activate_right_bar";
static const char __pyx_k_selected_activate_textalign[] = "selected_activate_textalign";
static const char __pyx_k_selected_activate_underline[] = "selected_activate_underline";
static const char __pyx_k_selected_activate_aft_gutter[] = "selected_activate_aft_gutter";
static const char __pyx_k_selected_activate_background[] = "selected_activate_background";
static const char __pyx_k_selected_activate_bar_invert[] = "selected_activate_bar_invert";
static const char __pyx_k_selected_activate_bottom_bar[] = "selected_activate_bottom_bar";
static const char __pyx_k_selected_activate_box_layout[] = "selected_activate_box_layout";
static const char __pyx_k_selected_activate_focus_mask[] = "selected_activate_focus_mask";
static const char __pyx_k_selected_activate_focus_rect[] = "selected_activate_focus_rect";
static const char __pyx_k_selected_activate_foreground[] = "selected_activate_foreground";
static const char __pyx_k_selected_activate_key_events[] = "selected_activate_key_events";
static const char __pyx_k_selected_activate_ruby_style[] = "selected_activate_ruby_style";
static const char __pyx_k_selected_activate_size_group[] = "selected_activate_size_group";
static const char __pyx_k_selected_activate_slow_speed[] = "selected_activate_slow_speed";
static const char __pyx_k_selected_activate_text_align[] = "selected_activate_text_align";
static const char __pyx_k_selected_activate_top_gutter[] = "selected_activate_top_gutter";
static const char __pyx_k_selected_activate_top_margin[] = "selected_activate_top_margin";
static const char __pyx_k_selected_activate_black_color[] = "selected_activate_black_color";
static const char __pyx_k_selected_activate_box_reverse[] = "selected_activate_box_reverse";
static const char __pyx_k_selected_activate_box_spacing[] = "selected_activate_box_spacing";
static const char __pyx_k_selected_activate_drop_shadow[] = "selected_activate_drop_shadow";
static const char __pyx_k_selected_activate_fore_gutter[] = "selected_activate_fore_gutter";
static const char __pyx_k_selected_activate_hover_sound[] = "selected_activate_hover_sound";
static const char __pyx_k_selected_activate_left_gutter[] = "selected_activate_left_gutter";
static const char __pyx_k_selected_activate_left_margin[] = "selected_activate_left_margin";
static const char __pyx_k_selected_activate_rest_indent[] = "selected_activate_rest_indent";
static const char __pyx_k_selected_activate_time_policy[] = "selected_activate_time_policy";
static const char __pyx_k_selected_activate_top_padding[] = "selected_activate_top_padding";
static const char __pyx_k_selected_activate_bar_resizing[] = "selected_activate_bar_resizing";
static const char __pyx_k_selected_activate_bar_vertical[] = "selected_activate_bar_vertical";
static const char __pyx_k_selected_activate_enable_hover[] = "selected_activate_enable_hover";
static const char __pyx_k_selected_activate_first_indent[] = "selected_activate_first_indent";
static const char __pyx_k_selected_activate_left_padding[] = "selected_activate_left_padding";
static const char __pyx_k_selected_activate_line_leading[] = "selected_activate_line_leading";
static const char __pyx_k_selected_activate_line_spacing[] = "selected_activate_line_spacing";
static const char __pyx_k_selected_activate_right_gutter[] = "selected_activate_right_gutter";
static const char __pyx_k_selected_activate_right_margin[] = "selected_activate_right_margin";
static const char __pyx_k_selected_activate_text_y_fudge[] = "selected_activate_text_y_fudge";
static const char __pyx_k_selected_activate_thumb_offset[] = "selected_activate_thumb_offset";
static const char __pyx_k_selected_activate_thumb_shadow[] = "selected_activate_thumb_shadow";
static const char __pyx_k_selected_activate_unscrollable[] = "selected_activate_unscrollable";
static const char __pyx_k_selected_activate_altruby_style[] = "selected_activate_altruby_style";
static const char __pyx_k_selected_activate_bottom_gutter[] = "selected_activate_bottom_gutter";
static const char __pyx_k_selected_activate_bottom_margin[] = "selected_activate_bottom_margin";
static const char __pyx_k_selected_activate_first_spacing[] = "selected_activate_first_spacing";
static const char __pyx_k_selected_activate_order_reverse[] = "selected_activate_order_reverse";
static const char __pyx_k_selected_activate_right_padding[] = "selected_activate_right_padding";
static const char __pyx_k_selected_activate_strikethrough[] = "selected_activate_strikethrough";
static const char __pyx_k_selected_activate_activate_sound[] = "selected_activate_activate_sound";
static const char __pyx_k_selected_activate_adjust_spacing[] = "selected_activate_adjust_spacing";
static const char __pyx_k_selected_activate_bottom_padding[] = "selected_activate_bottom_padding";
static const char __pyx_k_selected_activate_box_first_spac[] = "selected_activate_box_first_spacing";
static const char __pyx_k_selected_activate_box_wrap_spaci[] = "selected_activate_box_wrap_spacing";
static const char __pyx_k_selected_activate_drop_shadow_co[] = "selected_activate_drop_shadow_color";
static const char __pyx_k_selected_activate_hyperlink_func[] = "selected_activate_hyperlink_functions";
static const char __pyx_k_selected_activate_keyboard_focus[] = "selected_activate_keyboard_focus";
static const char __pyx_k_selected_activate_line_overlap_s[] = "selected_activate_line_overlap_split";
static const char __pyx_k_selected_activate_newline_indent[] = "selected_activate_newline_indent";
static const char __pyx_k_selected_activate_outline_scalin[] = "selected_activate_outline_scaling";
static const char __pyx_k_selected_activate_slow_abortable[] = "selected_activate_slow_abortable";
static const char __pyx_k_selected_activate_slow_cps_multi[] = "selected_activate_slow_cps_multiplier";
static const char __pyx_k_selected_activate_subtitle_width[] = "selected_activate_subtitle_width";
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_color;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_displayable_or_none;
static PyObject *__pyx_n_s_duplicatable;
static PyObject *__pyx_n_s_duplicate;
static PyObject *__pyx_n_s_easy;
static PyObject *__pyx_n_s_expand_anchor;
static PyObject *__pyx_n_s_expand_focus_mask;
static PyObject *__pyx_n_s_expand_outlines;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_none_is_null;
static PyObject *__pyx_n_s_prefix;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_renpy;
static PyObject *__pyx_n_s_renpy_styledata_styleutil;
static PyObject *__pyx_n_s_selected_activate_activate_sound;
static PyObject *__pyx_n_s_selected_activate_adjust_spacing;
static PyObject *__pyx_n_s_selected_activate_aft_bar;
static PyObject *__pyx_n_s_selected_activate_aft_gutter;
static PyObject *__pyx_n_s_selected_activate_align;
static PyObject *__pyx_n_s_selected_activate_alt;
static PyObject *__pyx_n_s_selected_activate_altruby_style;
static PyObject *__pyx_n_s_selected_activate_anchor;
static PyObject *__pyx_n_s_selected_activate_antialias;
static PyObject *__pyx_n_s_selected_activate_area;
static PyObject *__pyx_n_s_selected_activate_background;
static PyObject *__pyx_n_s_selected_activate_bar_invert;
static PyObject *__pyx_n_s_selected_activate_bar_resizing;
static PyObject *__pyx_n_s_selected_activate_bar_vertical;
static PyObject *__pyx_n_s_selected_activate_base_bar;
static PyObject *__pyx_n_s_selected_activate_black_color;
static PyObject *__pyx_n_s_selected_activate_bold;
static PyObject *__pyx_n_s_selected_activate_bottom_bar;
static PyObject *__pyx_n_s_selected_activate_bottom_gutter;
static PyObject *__pyx_n_s_selected_activate_bottom_margin;
static PyObject *__pyx_n_s_selected_activate_bottom_padding;
static PyObject *__pyx_n_s_selected_activate_box_first_spac;
static PyObject *__pyx_n_s_selected_activate_box_layout;
static PyObject *__pyx_n_s_selected_activate_box_reverse;
static PyObject *__pyx_n_s_selected_activate_box_spacing;
static PyObject *__pyx_n_s_selected_activate_box_wrap;
static PyObject *__pyx_n_s_selected_activate_box_wrap_spaci;
static PyObject *__pyx_n_s_selected_activate_caret;
static PyObject *__pyx_n_s_selected_activate_child;
static PyObject *__pyx_n_s_selected_activate_clipping;
static PyObject *__pyx_n_s_selected_activate_color;
static PyObject *__pyx_n_s_selected_activate_debug;
static PyObject *__pyx_n_s_selected_activate_drop_shadow;
static PyObject *__pyx_n_s_selected_activate_drop_shadow_co;
static PyObject *__pyx_n_s_selected_activate_enable_hover;
static PyObject *__pyx_n_s_selected_activate_first_indent;
static PyObject *__pyx_n_s_selected_activate_first_spacing;
static PyObject *__pyx_n_s_selected_activate_fit_first;
static PyObject *__pyx_n_s_selected_activate_focus_mask;
static PyObject *__pyx_n_s_selected_activate_focus_rect;
static PyObject *__pyx_n_s_selected_activate_font;
static PyObject *__pyx_n_s_selected_activate_fore_bar;
static PyObject *__pyx_n_s_selected_activate_fore_gutter;
static PyObject *__pyx_n_s_selected_activate_foreground;
static PyObject *__pyx_n_s_selected_activate_hinting;
static PyObject *__pyx_n_s_selected_activate_hover_sound;
static PyObject *__pyx_n_s_selected_activate_hyperlink_func;
static PyObject *__pyx_n_s_selected_activate_italic;
static PyObject *__pyx_n_s_selected_activate_justify;
static PyObject *__pyx_n_s_selected_activate_kerning;
static PyObject *__pyx_n_s_selected_activate_key_events;
static PyObject *__pyx_n_s_selected_activate_keyboard_focus;
static PyObject *__pyx_n_s_selected_activate_language;
static PyObject *__pyx_n_s_selected_activate_layout;
static PyObject *__pyx_n_s_selected_activate_left_bar;
static PyObject *__pyx_n_s_selected_activate_left_gutter;
static PyObject *__pyx_n_s_selected_activate_left_margin;
static PyObject *__pyx_n_s_selected_activate_left_padding;
static PyObject *__pyx_n_s_selected_activate_line_leading;
static PyObject *__pyx_n_s_selected_activate_line_overlap_s;
static PyObject *__pyx_n_s_selected_activate_line_spacing;
static PyObject *__pyx_n_s_selected_activate_margin;
static PyObject *__pyx_n_s_selected_activate_maximum;
static PyObject *__pyx_n_s_selected_activate_min_width;
static PyObject *__pyx_n_s_selected_activate_minimum;
static PyObject *__pyx_n_s_selected_activate_minwidth;
static PyObject *__pyx_n_s_selected_activate_mouse;
static PyObject *__pyx_n_s_selected_activate_newline_indent;
static PyObject *__pyx_n_s_selected_activate_offset;
static PyObject *__pyx_n_s_selected_activate_order_reverse;
static PyObject *__pyx_n_s_selected_activate_outline_scalin;
static PyObject *__pyx_n_s_selected_activate_outlines;
static PyObject *__pyx_n_s_selected_activate_padding;
static PyObject *__pyx_n_s_selected_activate_pos;
static PyObject *__pyx_n_s_selected_activate_rest_indent;
static PyObject *__pyx_n_s_selected_activate_right_bar;
static PyObject *__pyx_n_s_selected_activate_right_gutter;
static PyObject *__pyx_n_s_selected_activate_right_margin;
static PyObject *__pyx_n_s_selected_activate_right_padding;
static PyObject *__pyx_n_s_selected_activate_ruby_style;
static PyObject *__pyx_n_s_selected_activate_size;
static PyObject *__pyx_n_s_selected_activate_size_group;
static PyObject *__pyx_n_s_selected_activate_slow_abortable;
static PyObject *__pyx_n_s_selected_activate_slow_cps;
static PyObject *__pyx_n_s_selected_activate_slow_cps_multi;
static PyObject *__pyx_n_s_selected_activate_slow_speed;
static PyObject *__pyx_n_s_selected_activate_spacing;
static PyObject *__pyx_n_s_selected_activate_strikethrough;
static PyObject *__pyx_n_s_selected_activate_subpixel;
static PyObject *__pyx_n_s_selected_activate_subtitle_width;
static PyObject *__pyx_n_s_selected_activate_text_align;
static PyObject *__pyx_n_s_selected_activate_text_y_fudge;
static PyObject *__pyx_n_s_selected_activate_textalign;
static PyObject *__pyx_n_s_selected_activate_thumb;
static PyObject *__pyx_n_s_selected_activate_thumb_offset;
static PyObject *__pyx_n_s_selected_activate_thumb_shadow;
static PyObject *__pyx_n_s_selected_activate_time_policy;
static PyObject *__pyx_n_s_selected_activate_top_bar;
static PyObject *__pyx_n_s_selected_activate_top_gutter;
static PyObject *__pyx_n_s_selected_activate_top_margin;
static PyObject *__pyx_n_s_selected_activate_top_padding;
static PyObject *__pyx_n_s_selected_activate_underline;
static PyObject *__pyx_n_s_selected_activate_unscrollable;
static PyObject *__pyx_n_s_selected_activate_vertical;
static PyObject *__pyx_n_s_selected_activate_xalign;
static PyObject *__pyx_n_s_selected_activate_xanchor;
static PyObject *__pyx_n_s_selected_activate_xcenter;
static PyObject *__pyx_n_s_selected_activate_xfill;
static PyObject *__pyx_n_s_selected_activate_xfit;
static PyObject *__pyx_n_s_selected_activate_xmargin;
static PyObject *__pyx_n_s_selected_activate_xmaximum;
static PyObject *__pyx_n_s_selected_activate_xminimum;
static PyObject *__pyx_n_s_selected_activate_xoffset;
static PyObject *__pyx_n_s_selected_activate_xpadding;
static PyObject *__pyx_n_s_selected_activate_xpos;
static PyObject *__pyx_n_s_selected_activate_xsize;
static PyObject *__pyx_n_s_selected_activate_xspacing;
static PyObject *__pyx_n_s_selected_activate_xysize;
static PyObject *__pyx_n_s_selected_activate_yalign;
static PyObject *__pyx_n_s_selected_activate_yanchor;
static PyObject *__pyx_n_s_selected_activate_ycenter;
static PyObject *__pyx_n_s_selected_activate_yfill;
static PyObject *__pyx_n_s_selected_activate_yfit;
static PyObject *__pyx_n_s_selected_activate_ymargin;
static PyObject *__pyx_n_s_selected_activate_ymaximum;
static PyObject *__pyx_n_s_selected_activate_yminimum;
static PyObject *__pyx_n_s_selected_activate_yoffset;
static PyObject *__pyx_n_s_selected_activate_ypadding;
static PyObject *__pyx_n_s_selected_activate_ypos;
static PyObject *__pyx_n_s_selected_activate_ysize;
static PyObject *__pyx_n_s_selected_activate_yspacing;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_unique;
static PyObject *__pyx_float_0_5;
static PyObject *__pyx_int_0;

/* "include/style_common.pxi":29
 * from renpy.styledata.styleutil import none_is_null, expand_focus_mask, expand_outlines, expand_anchor
 * 
 * cdef inline object index_0(object v):             # <<<<<<<<<<<<<<
 *     return v[0]
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_0(PyObject *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("index_0", 0);

  /* "include/style_common.pxi":30
 * 
 * cdef inline object index_0(object v):
 *     return v[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline object index_1(object v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_v, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "include/style_common.pxi":29
 * from renpy.styledata.styleutil import none_is_null, expand_focus_mask, expand_outlines, expand_anchor
 * 
 * cdef inline object index_0(object v):             # <<<<<<<<<<<<<<
 *     return v[0]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.index_0", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "include/style_common.pxi":32
 *     return v[0]
 * 
 * cdef inline object index_1(object v):             # <<<<<<<<<<<<<<
 *     return v[1]
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_1(PyObject *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("index_1", 0);

  /* "include/style_common.pxi":33
 * 
 * cdef inline object index_1(object v):
 *     return v[1]             # <<<<<<<<<<<<<<
 * 
 * cdef inline object index_2(object v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_v, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "include/style_common.pxi":32
 *     return v[0]
 * 
 * cdef inline object index_1(object v):             # <<<<<<<<<<<<<<
 *     return v[1]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.index_1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "include/style_common.pxi":35
 *     return v[1]
 * 
 * cdef inline object index_2(object v):             # <<<<<<<<<<<<<<
 *     return v[2]
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_2(PyObject *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("index_2", 0);

  /* "include/style_common.pxi":36
 * 
 * cdef inline object index_2(object v):
 *     return v[2]             # <<<<<<<<<<<<<<
 * 
 * cdef inline object index_3(object v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_v, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "include/style_common.pxi":35
 *     return v[1]
 * 
 * cdef inline object index_2(object v):             # <<<<<<<<<<<<<<
 *     return v[2]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.index_2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "include/style_common.pxi":38
 *     return v[2]
 * 
 * cdef inline object index_3(object v):             # <<<<<<<<<<<<<<
 *     return v[3]
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_3(PyObject *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("index_3", 0);

  /* "include/style_common.pxi":39
 * 
 * cdef inline object index_3(object v):
 *     return v[3]             # <<<<<<<<<<<<<<
 * 
 * cdef inline object index_2_or_0(object v):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_v, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "include/style_common.pxi":38
 *     return v[2]
 * 
 * cdef inline object index_3(object v):             # <<<<<<<<<<<<<<
 *     return v[3]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.index_3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "include/style_common.pxi":41
 *     return v[3]
 * 
 * cdef inline object index_2_or_0(object v):             # <<<<<<<<<<<<<<
 *     if len(v) >= 3:
 *         return v[2]
 */

static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_2_or_0(PyObject *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("index_2_or_0", 0);

  /* "include/style_common.pxi":42
 * 
 * cdef inline object index_2_or_0(object v):
 *     if len(v) >= 3:             # <<<<<<<<<<<<<<
 *         return v[2]
 *     else:
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_v); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 42, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 >= 3) != 0);
  if (__pyx_t_2) {

    /* "include/style_common.pxi":43
 * cdef inline object index_2_or_0(object v):
 *     if len(v) >= 3:
 *         return v[2]             # <<<<<<<<<<<<<<
 *     else:
 *         return v[0]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_v, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 43, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "include/style_common.pxi":42
 * 
 * cdef inline object index_2_or_0(object v):
 *     if len(v) >= 3:             # <<<<<<<<<<<<<<
 *         return v[2]
 *     else:
 */
  }

  /* "include/style_common.pxi":45
 *         return v[2]
 *     else:
 *         return v[0]             # <<<<<<<<<<<<<<
 * 
 * cdef inline object index_3_or_1(object v):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_v, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "include/style_common.pxi":41
 *     return v[3]
 * 
 * cdef inline object index_2_or_0(object v):             # <<<<<<<<<<<<<<
 *     if len(v) >= 3:
 *         return v[2]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("style_selected_activate_functions.index_2_or_0", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "include/style_common.pxi":47
 *         return v[0]
 * 
 * cdef inline object index_3_or_1(object v):             # <<<<<<<<<<<<<<
 *     if len(v) >= 4:
 *         return v[3]
 */

static CYTHON_INLINE PyObject *__pyx_f_33style_selected_activate_functions_index_3_or_1(PyObject *__pyx_v_v) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("index_3_or_1", 0);

  /* "include/style_common.pxi":48
 * 
 * cdef inline object index_3_or_1(object v):
 *     if len(v) >= 4:             # <<<<<<<<<<<<<<
 *         return v[3]
 *     else:
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_v); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_t_1 >= 4) != 0);
  if (__pyx_t_2) {

    /* "include/style_common.pxi":49
 * cdef inline object index_3_or_1(object v):
 *     if len(v) >= 4:
 *         return v[3]             # <<<<<<<<<<<<<<
 *     else:
 *         return v[1]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_v, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 49, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "include/style_common.pxi":48
 * 
 * cdef inline object index_3_or_1(object v):
 *     if len(v) >= 4:             # <<<<<<<<<<<<<<
 *         return v[3]
 *     else:
 */
  }

  /* "include/style_common.pxi":51
 *         return v[3]
 *     else:
 *         return v[1]             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetItemInt(__pyx_v_v, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 51, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "include/style_common.pxi":47
 *         return v[0]
 * 
 * cdef inline object index_3_or_1(object v):             # <<<<<<<<<<<<<<
 *     if len(v) >= 4:
 *         return v[3]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("style_selected_activate_functions.index_3_or_1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":3
 * include "style_common.pxi"
 * 
 * cdef int selected_activate_black_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_black_color_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_black_color_property", 0);

  /* "style_selected_activate_functions.pyx":4
 * 
 * cdef int selected_activate_black_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.color(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":6
 *     priority += 0
 * 
 *     v = renpy.easy.color(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_color); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":7
 * 
 *     v = renpy.easy.color(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_black_color", selected_activate_black_color_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":3
 * include "style_common.pxi"
 * 
 * cdef int selected_activate_black_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_black_color_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":11
 * register_property_function("selected_activate_black_color", selected_activate_black_color_property)
 * 
 * cdef int selected_activate_bottom_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_margin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bottom_margin_property", 0);

  /* "style_selected_activate_functions.pyx":12
 * 
 * cdef int selected_activate_bottom_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":14
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bottom_margin", selected_activate_bottom_margin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":11
 * register_property_function("selected_activate_black_color", selected_activate_black_color_property)
 * 
 * cdef int selected_activate_bottom_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":18
 * register_property_function("selected_activate_bottom_margin", selected_activate_bottom_margin_property)
 * 
 * cdef int selected_activate_line_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_line_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_line_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":19
 * 
 * cdef int selected_activate_line_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":21
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_line_spacing", selected_activate_line_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":18
 * register_property_function("selected_activate_bottom_margin", selected_activate_bottom_margin_property)
 * 
 * cdef int selected_activate_line_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":25
 * register_property_function("selected_activate_line_spacing", selected_activate_line_spacing_property)
 * 
 * cdef int selected_activate_bar_resizing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bar_resizing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bar_resizing_property", 0);

  /* "style_selected_activate_functions.pyx":26
 * 
 * cdef int selected_activate_bar_resizing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":28
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bar_resizing", selected_activate_bar_resizing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":25
 * register_property_function("selected_activate_line_spacing", selected_activate_line_spacing_property)
 * 
 * cdef int selected_activate_bar_resizing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":32
 * register_property_function("selected_activate_bar_resizing", selected_activate_bar_resizing_property)
 * 
 * cdef int selected_activate_left_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_left_margin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_left_margin_property", 0);

  /* "style_selected_activate_functions.pyx":33
 * 
 * cdef int selected_activate_left_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":35
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_left_margin", selected_activate_left_margin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":32
 * register_property_function("selected_activate_bar_resizing", selected_activate_bar_resizing_property)
 * 
 * cdef int selected_activate_left_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":39
 * register_property_function("selected_activate_left_margin", selected_activate_left_margin_property)
 * 
 * cdef int selected_activate_yminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yminimum_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_yminimum_property", 0);

  /* "style_selected_activate_functions.pyx":40
 * 
 * cdef int selected_activate_yminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":42
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yminimum", selected_activate_yminimum_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":39
 * register_property_function("selected_activate_left_margin", selected_activate_left_margin_property)
 * 
 * cdef int selected_activate_yminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":46
 * register_property_function("selected_activate_yminimum", selected_activate_yminimum_property)
 * 
 * cdef int selected_activate_alt_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_alt_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_alt_property", 0);

  /* "style_selected_activate_functions.pyx":47
 * 
 * cdef int selected_activate_alt_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":49
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_alt", selected_activate_alt_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":46
 * register_property_function("selected_activate_yminimum", selected_activate_yminimum_property)
 * 
 * cdef int selected_activate_alt_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":53
 * register_property_function("selected_activate_alt", selected_activate_alt_property)
 * 
 * cdef int selected_activate_font_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_font_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_font_property", 0);

  /* "style_selected_activate_functions.pyx":54
 * 
 * cdef int selected_activate_font_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":56
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_font", selected_activate_font_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":53
 * register_property_function("selected_activate_alt", selected_activate_alt_property)
 * 
 * cdef int selected_activate_font_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":60
 * register_property_function("selected_activate_font", selected_activate_font_property)
 * 
 * cdef int selected_activate_underline_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_underline_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_underline_property", 0);

  /* "style_selected_activate_functions.pyx":61
 * 
 * cdef int selected_activate_underline_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":63
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_underline", selected_activate_underline_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":60
 * register_property_function("selected_activate_font", selected_activate_font_property)
 * 
 * cdef int selected_activate_underline_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":67
 * register_property_function("selected_activate_underline", selected_activate_underline_property)
 * 
 * cdef int selected_activate_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_layout_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_layout_property", 0);

  /* "style_selected_activate_functions.pyx":68
 * 
 * cdef int selected_activate_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":70
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_layout", selected_activate_layout_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":67
 * register_property_function("selected_activate_underline", selected_activate_underline_property)
 * 
 * cdef int selected_activate_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":74
 * register_property_function("selected_activate_layout", selected_activate_layout_property)
 * 
 * cdef int selected_activate_thumb_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_thumb_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_thumb_property", 0);

  /* "style_selected_activate_functions.pyx":75
 * 
 * cdef int selected_activate_thumb_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":77
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 77, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 77, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 77, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":78
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_thumb", selected_activate_thumb_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":74
 * register_property_function("selected_activate_layout", selected_activate_layout_property)
 * 
 * cdef int selected_activate_thumb_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_thumb_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":82
 * register_property_function("selected_activate_thumb", selected_activate_thumb_property)
 * 
 * cdef int selected_activate_box_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_box_reverse_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_box_reverse_property", 0);

  /* "style_selected_activate_functions.pyx":83
 * 
 * cdef int selected_activate_box_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":85
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_box_reverse", selected_activate_box_reverse_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":82
 * register_property_function("selected_activate_thumb", selected_activate_thumb_property)
 * 
 * cdef int selected_activate_box_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":89
 * register_property_function("selected_activate_box_reverse", selected_activate_box_reverse_property)
 * 
 * cdef int selected_activate_outline_scaling_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_outline_scaling_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_outline_scaling_property", 0);

  /* "style_selected_activate_functions.pyx":90
 * 
 * cdef int selected_activate_outline_scaling_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":92
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_outline_scaling", selected_activate_outline_scaling_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":89
 * register_property_function("selected_activate_box_reverse", selected_activate_box_reverse_property)
 * 
 * cdef int selected_activate_outline_scaling_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":96
 * register_property_function("selected_activate_outline_scaling", selected_activate_outline_scaling_property)
 * 
 * cdef int selected_activate_text_y_fudge_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_text_y_fudge_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_text_y_fudge_property", 0);

  /* "style_selected_activate_functions.pyx":97
 * 
 * cdef int selected_activate_text_y_fudge_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":99
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_text_y_fudge", selected_activate_text_y_fudge_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":96
 * register_property_function("selected_activate_outline_scaling", selected_activate_outline_scaling_property)
 * 
 * cdef int selected_activate_text_y_fudge_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":103
 * register_property_function("selected_activate_text_y_fudge", selected_activate_text_y_fudge_property)
 * 
 * cdef int selected_activate_aft_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_aft_gutter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_aft_gutter_property", 0);

  /* "style_selected_activate_functions.pyx":104
 * 
 * cdef int selected_activate_aft_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":106
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_aft_gutter", selected_activate_aft_gutter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":103
 * register_property_function("selected_activate_text_y_fudge", selected_activate_text_y_fudge_property)
 * 
 * cdef int selected_activate_aft_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":110
 * register_property_function("selected_activate_aft_gutter", selected_activate_aft_gutter_property)
 * 
 * cdef int selected_activate_right_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_right_margin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_right_margin_property", 0);

  /* "style_selected_activate_functions.pyx":111
 * 
 * cdef int selected_activate_right_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":113
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_right_margin", selected_activate_right_margin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":110
 * register_property_function("selected_activate_aft_gutter", selected_activate_aft_gutter_property)
 * 
 * cdef int selected_activate_right_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":117
 * register_property_function("selected_activate_right_margin", selected_activate_right_margin_property)
 * 
 * cdef int selected_activate_rest_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_rest_indent_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_rest_indent_property", 0);

  /* "style_selected_activate_functions.pyx":118
 * 
 * cdef int selected_activate_rest_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":120
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_rest_indent", selected_activate_rest_indent_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":117
 * register_property_function("selected_activate_right_margin", selected_activate_right_margin_property)
 * 
 * cdef int selected_activate_rest_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":124
 * register_property_function("selected_activate_rest_indent", selected_activate_rest_indent_property)
 * 
 * cdef int selected_activate_background_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_background_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_background_property", 0);

  /* "style_selected_activate_functions.pyx":125
 * 
 * cdef int selected_activate_background_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.displayable_or_none(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":127
 *     priority += 0
 * 
 *     v = renpy.easy.displayable_or_none(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_displayable_or_none); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":128
 * 
 *     v = renpy.easy.displayable_or_none(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_background", selected_activate_background_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":124
 * register_property_function("selected_activate_rest_indent", selected_activate_rest_indent_property)
 * 
 * cdef int selected_activate_background_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_background_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":132
 * register_property_function("selected_activate_background", selected_activate_background_property)
 * 
 * cdef int selected_activate_outlines_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_outlines_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_outlines_property", 0);

  /* "style_selected_activate_functions.pyx":133
 * 
 * cdef int selected_activate_outlines_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = expand_outlines(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":135
 *     priority += 0
 * 
 *     v = expand_outlines(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_outlines); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 135, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":136
 * 
 *     v = expand_outlines(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_outlines", selected_activate_outlines_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":132
 * register_property_function("selected_activate_background", selected_activate_background_property)
 * 
 * cdef int selected_activate_outlines_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_outlines_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":140
 * register_property_function("selected_activate_outlines", selected_activate_outlines_property)
 * 
 * cdef int selected_activate_ymaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ymaximum_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_ymaximum_property", 0);

  /* "style_selected_activate_functions.pyx":141
 * 
 * cdef int selected_activate_ymaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":143
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ymaximum", selected_activate_ymaximum_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":140
 * register_property_function("selected_activate_outlines", selected_activate_outlines_property)
 * 
 * cdef int selected_activate_ymaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":147
 * register_property_function("selected_activate_ymaximum", selected_activate_ymaximum_property)
 * 
 * cdef int selected_activate_yspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yspacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_yspacing_property", 0);

  /* "style_selected_activate_functions.pyx":148
 * 
 * cdef int selected_activate_yspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":150
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yspacing", selected_activate_yspacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":147
 * register_property_function("selected_activate_ymaximum", selected_activate_ymaximum_property)
 * 
 * cdef int selected_activate_yspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":154
 * register_property_function("selected_activate_yspacing", selected_activate_yspacing_property)
 * 
 * cdef int selected_activate_size_group_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_size_group_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_size_group_property", 0);

  /* "style_selected_activate_functions.pyx":155
 * 
 * cdef int selected_activate_size_group_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":157
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_size_group", selected_activate_size_group_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":154
 * register_property_function("selected_activate_yspacing", selected_activate_yspacing_property)
 * 
 * cdef int selected_activate_size_group_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":161
 * register_property_function("selected_activate_size_group", selected_activate_size_group_property)
 * 
 * cdef int selected_activate_debug_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_debug_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_debug_property", 0);

  /* "style_selected_activate_functions.pyx":162
 * 
 * cdef int selected_activate_debug_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":164
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_debug", selected_activate_debug_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":161
 * register_property_function("selected_activate_size_group", selected_activate_size_group_property)
 * 
 * cdef int selected_activate_debug_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":168
 * register_property_function("selected_activate_debug", selected_activate_debug_property)
 * 
 * cdef int selected_activate_clipping_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_clipping_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_clipping_property", 0);

  /* "style_selected_activate_functions.pyx":169
 * 
 * cdef int selected_activate_clipping_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":171
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_clipping", selected_activate_clipping_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":168
 * register_property_function("selected_activate_debug", selected_activate_debug_property)
 * 
 * cdef int selected_activate_clipping_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":175
 * register_property_function("selected_activate_clipping", selected_activate_clipping_property)
 * 
 * cdef int selected_activate_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_first_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_first_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":176
 * 
 * cdef int selected_activate_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":178
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_first_spacing", selected_activate_first_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":175
 * register_property_function("selected_activate_clipping", selected_activate_clipping_property)
 * 
 * cdef int selected_activate_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":182
 * register_property_function("selected_activate_first_spacing", selected_activate_first_spacing_property)
 * 
 * cdef int selected_activate_subtitle_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_subtitle_width_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_subtitle_width_property", 0);

  /* "style_selected_activate_functions.pyx":183
 * 
 * cdef int selected_activate_subtitle_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":185
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_subtitle_width", selected_activate_subtitle_width_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":182
 * register_property_function("selected_activate_first_spacing", selected_activate_first_spacing_property)
 * 
 * cdef int selected_activate_subtitle_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":189
 * register_property_function("selected_activate_subtitle_width", selected_activate_subtitle_width_property)
 * 
 * cdef int selected_activate_subpixel_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_subpixel_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_subpixel_property", 0);

  /* "style_selected_activate_functions.pyx":190
 * 
 * cdef int selected_activate_subpixel_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":192
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_subpixel", selected_activate_subpixel_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":189
 * register_property_function("selected_activate_subtitle_width", selected_activate_subtitle_width_property)
 * 
 * cdef int selected_activate_subpixel_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":196
 * register_property_function("selected_activate_subpixel", selected_activate_subpixel_property)
 * 
 * cdef int selected_activate_strikethrough_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_strikethrough_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_strikethrough_property", 0);

  /* "style_selected_activate_functions.pyx":197
 * 
 * cdef int selected_activate_strikethrough_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":199
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_strikethrough", selected_activate_strikethrough_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":196
 * register_property_function("selected_activate_subpixel", selected_activate_subpixel_property)
 * 
 * cdef int selected_activate_strikethrough_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":203
 * register_property_function("selected_activate_strikethrough", selected_activate_strikethrough_property)
 * 
 * cdef int selected_activate_hyperlink_functions_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_hyperlink_functions_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_hyperlink_functions_property", 0);

  /* "style_selected_activate_functions.pyx":204
 * 
 * cdef int selected_activate_hyperlink_functions_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":206
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_hyperlink_functions", selected_activate_hyperlink_functions_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":203
 * register_property_function("selected_activate_strikethrough", selected_activate_strikethrough_property)
 * 
 * cdef int selected_activate_hyperlink_functions_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":210
 * register_property_function("selected_activate_hyperlink_functions", selected_activate_hyperlink_functions_property)
 * 
 * cdef int selected_activate_xminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xminimum_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xminimum_property", 0);

  /* "style_selected_activate_functions.pyx":211
 * 
 * cdef int selected_activate_xminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":213
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xminimum", selected_activate_xminimum_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":210
 * register_property_function("selected_activate_hyperlink_functions", selected_activate_hyperlink_functions_property)
 * 
 * cdef int selected_activate_xminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":217
 * register_property_function("selected_activate_xminimum", selected_activate_xminimum_property)
 * 
 * cdef int selected_activate_min_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_min_width_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_min_width_property", 0);

  /* "style_selected_activate_functions.pyx":218
 * 
 * cdef int selected_activate_min_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":220
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_min_width", selected_activate_min_width_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":217
 * register_property_function("selected_activate_xminimum", selected_activate_xminimum_property)
 * 
 * cdef int selected_activate_min_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":224
 * register_property_function("selected_activate_min_width", selected_activate_min_width_property)
 * 
 * cdef int selected_activate_focus_mask_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_focus_mask_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_focus_mask_property", 0);

  /* "style_selected_activate_functions.pyx":225
 * 
 * cdef int selected_activate_focus_mask_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = expand_focus_mask(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":227
 *     priority += 0
 * 
 *     v = expand_focus_mask(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_focus_mask); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 227, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 227, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 227, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":228
 * 
 *     v = expand_focus_mask(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_focus_mask", selected_activate_focus_mask_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":224
 * register_property_function("selected_activate_min_width", selected_activate_min_width_property)
 * 
 * cdef int selected_activate_focus_mask_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_focus_mask_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":232
 * register_property_function("selected_activate_focus_mask", selected_activate_focus_mask_property)
 * 
 * cdef int selected_activate_yfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yfit_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_yfit_property", 0);

  /* "style_selected_activate_functions.pyx":233
 * 
 * cdef int selected_activate_yfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":235
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yfit", selected_activate_yfit_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":232
 * register_property_function("selected_activate_focus_mask", selected_activate_focus_mask_property)
 * 
 * cdef int selected_activate_yfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":239
 * register_property_function("selected_activate_yfit", selected_activate_yfit_property)
 * 
 * cdef int selected_activate_drop_shadow_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_drop_shadow_color_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_drop_shadow_color_property", 0);

  /* "style_selected_activate_functions.pyx":240
 * 
 * cdef int selected_activate_drop_shadow_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.color(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":242
 *     priority += 0
 * 
 *     v = renpy.easy.color(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_color); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 242, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 242, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 242, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 242, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 242, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 242, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":243
 * 
 *     v = renpy.easy.color(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_drop_shadow_color", selected_activate_drop_shadow_color_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":239
 * register_property_function("selected_activate_yfit", selected_activate_yfit_property)
 * 
 * cdef int selected_activate_drop_shadow_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_drop_shadow_color_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":247
 * register_property_function("selected_activate_drop_shadow_color", selected_activate_drop_shadow_color_property)
 * 
 * cdef int selected_activate_fit_first_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_fit_first_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_fit_first_property", 0);

  /* "style_selected_activate_functions.pyx":248
 * 
 * cdef int selected_activate_fit_first_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":250
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_fit_first", selected_activate_fit_first_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":247
 * register_property_function("selected_activate_drop_shadow_color", selected_activate_drop_shadow_color_property)
 * 
 * cdef int selected_activate_fit_first_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":254
 * register_property_function("selected_activate_fit_first", selected_activate_fit_first_property)
 * 
 * cdef int selected_activate_top_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_top_padding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_top_padding_property", 0);

  /* "style_selected_activate_functions.pyx":255
 * 
 * cdef int selected_activate_top_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":257
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_top_padding", selected_activate_top_padding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":254
 * register_property_function("selected_activate_fit_first", selected_activate_fit_first_property)
 * 
 * cdef int selected_activate_top_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":261
 * register_property_function("selected_activate_top_padding", selected_activate_top_padding_property)
 * 
 * cdef int selected_activate_newline_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_newline_indent_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_newline_indent_property", 0);

  /* "style_selected_activate_functions.pyx":262
 * 
 * cdef int selected_activate_newline_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":264
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_newline_indent", selected_activate_newline_indent_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":261
 * register_property_function("selected_activate_top_padding", selected_activate_top_padding_property)
 * 
 * cdef int selected_activate_newline_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":268
 * register_property_function("selected_activate_newline_indent", selected_activate_newline_indent_property)
 * 
 * cdef int selected_activate_text_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_text_align_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_text_align_property", 0);

  /* "style_selected_activate_functions.pyx":269
 * 
 * cdef int selected_activate_text_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":271
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_text_align", selected_activate_text_align_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":268
 * register_property_function("selected_activate_newline_indent", selected_activate_newline_indent_property)
 * 
 * cdef int selected_activate_text_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":275
 * register_property_function("selected_activate_text_align", selected_activate_text_align_property)
 * 
 * cdef int selected_activate_fore_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_fore_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_fore_bar_property", 0);

  /* "style_selected_activate_functions.pyx":276
 * 
 * cdef int selected_activate_fore_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":278
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 278, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 278, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 278, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 278, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 278, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":279
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_fore_bar", selected_activate_fore_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":275
 * register_property_function("selected_activate_text_align", selected_activate_text_align_property)
 * 
 * cdef int selected_activate_fore_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_fore_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":283
 * register_property_function("selected_activate_fore_bar", selected_activate_fore_bar_property)
 * 
 * cdef int selected_activate_aft_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_aft_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_aft_bar_property", 0);

  /* "style_selected_activate_functions.pyx":284
 * 
 * cdef int selected_activate_aft_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":286
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 286, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 286, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 286, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 286, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":287
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_aft_bar", selected_activate_aft_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":283
 * register_property_function("selected_activate_fore_bar", selected_activate_fore_bar_property)
 * 
 * cdef int selected_activate_aft_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_aft_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":291
 * register_property_function("selected_activate_aft_bar", selected_activate_aft_bar_property)
 * 
 * cdef int selected_activate_foreground_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_foreground_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_foreground_property", 0);

  /* "style_selected_activate_functions.pyx":292
 * 
 * cdef int selected_activate_foreground_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.displayable_or_none(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":294
 *     priority += 0
 * 
 *     v = renpy.easy.displayable_or_none(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_displayable_or_none); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 294, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 294, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 294, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":295
 * 
 *     v = renpy.easy.displayable_or_none(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_foreground", selected_activate_foreground_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":291
 * register_property_function("selected_activate_aft_bar", selected_activate_aft_bar_property)
 * 
 * cdef int selected_activate_foreground_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_foreground_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":299
 * register_property_function("selected_activate_foreground", selected_activate_foreground_property)
 * 
 * cdef int selected_activate_bold_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bold_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bold_property", 0);

  /* "style_selected_activate_functions.pyx":300
 * 
 * cdef int selected_activate_bold_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":302
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bold", selected_activate_bold_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":299
 * register_property_function("selected_activate_foreground", selected_activate_foreground_property)
 * 
 * cdef int selected_activate_bold_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":306
 * register_property_function("selected_activate_bold", selected_activate_bold_property)
 * 
 * cdef int selected_activate_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_vertical_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_vertical_property", 0);

  /* "style_selected_activate_functions.pyx":307
 * 
 * cdef int selected_activate_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":309
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_vertical", selected_activate_vertical_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":306
 * register_property_function("selected_activate_bold", selected_activate_bold_property)
 * 
 * cdef int selected_activate_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":313
 * register_property_function("selected_activate_vertical", selected_activate_vertical_property)
 * 
 * cdef int selected_activate_drop_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_drop_shadow_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_drop_shadow_property", 0);

  /* "style_selected_activate_functions.pyx":314
 * 
 * cdef int selected_activate_drop_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":316
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_drop_shadow", selected_activate_drop_shadow_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":313
 * register_property_function("selected_activate_vertical", selected_activate_vertical_property)
 * 
 * cdef int selected_activate_drop_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":320
 * register_property_function("selected_activate_drop_shadow", selected_activate_drop_shadow_property)
 * 
 * cdef int selected_activate_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":321
 * 
 * cdef int selected_activate_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":323
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_spacing", selected_activate_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":320
 * register_property_function("selected_activate_drop_shadow", selected_activate_drop_shadow_property)
 * 
 * cdef int selected_activate_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":327
 * register_property_function("selected_activate_spacing", selected_activate_spacing_property)
 * 
 * cdef int selected_activate_ypos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ypos_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_ypos_property", 0);

  /* "style_selected_activate_functions.pyx":328
 * 
 * cdef int selected_activate_ypos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":330
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ypos", selected_activate_ypos_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":327
 * register_property_function("selected_activate_spacing", selected_activate_spacing_property)
 * 
 * cdef int selected_activate_ypos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":334
 * register_property_function("selected_activate_ypos", selected_activate_ypos_property)
 * 
 * cdef int selected_activate_activate_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_activate_sound_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_activate_sound_property", 0);

  /* "style_selected_activate_functions.pyx":335
 * 
 * cdef int selected_activate_activate_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":337
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_activate_sound", selected_activate_activate_sound_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":334
 * register_property_function("selected_activate_ypos", selected_activate_ypos_property)
 * 
 * cdef int selected_activate_activate_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":341
 * register_property_function("selected_activate_activate_sound", selected_activate_activate_sound_property)
 * 
 * cdef int selected_activate_left_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_left_padding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_left_padding_property", 0);

  /* "style_selected_activate_functions.pyx":342
 * 
 * cdef int selected_activate_left_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":344
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_left_padding", selected_activate_left_padding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":341
 * register_property_function("selected_activate_activate_sound", selected_activate_activate_sound_property)
 * 
 * cdef int selected_activate_left_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":348
 * register_property_function("selected_activate_left_padding", selected_activate_left_padding_property)
 * 
 * cdef int selected_activate_xoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xoffset_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xoffset_property", 0);

  /* "style_selected_activate_functions.pyx":349
 * 
 * cdef int selected_activate_xoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":351
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xoffset", selected_activate_xoffset_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":348
 * register_property_function("selected_activate_left_padding", selected_activate_left_padding_property)
 * 
 * cdef int selected_activate_xoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":355
 * register_property_function("selected_activate_xoffset", selected_activate_xoffset_property)
 * 
 * cdef int selected_activate_language_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_language_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_language_property", 0);

  /* "style_selected_activate_functions.pyx":356
 * 
 * cdef int selected_activate_language_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":358
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_language", selected_activate_language_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":355
 * register_property_function("selected_activate_xoffset", selected_activate_xoffset_property)
 * 
 * cdef int selected_activate_language_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":362
 * register_property_function("selected_activate_language", selected_activate_language_property)
 * 
 * cdef int selected_activate_time_policy_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_time_policy_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_time_policy_property", 0);

  /* "style_selected_activate_functions.pyx":363
 * 
 * cdef int selected_activate_time_policy_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":365
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_time_policy", selected_activate_time_policy_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":362
 * register_property_function("selected_activate_language", selected_activate_language_property)
 * 
 * cdef int selected_activate_time_policy_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":369
 * register_property_function("selected_activate_time_policy", selected_activate_time_policy_property)
 * 
 * cdef int selected_activate_xmaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xmaximum_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xmaximum_property", 0);

  /* "style_selected_activate_functions.pyx":370
 * 
 * cdef int selected_activate_xmaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":372
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xmaximum", selected_activate_xmaximum_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":369
 * register_property_function("selected_activate_time_policy", selected_activate_time_policy_property)
 * 
 * cdef int selected_activate_xmaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":376
 * register_property_function("selected_activate_xmaximum", selected_activate_xmaximum_property)
 * 
 * cdef int selected_activate_hover_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_hover_sound_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_hover_sound_property", 0);

  /* "style_selected_activate_functions.pyx":377
 * 
 * cdef int selected_activate_hover_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":379
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_hover_sound", selected_activate_hover_sound_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":376
 * register_property_function("selected_activate_xmaximum", selected_activate_xmaximum_property)
 * 
 * cdef int selected_activate_hover_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":383
 * register_property_function("selected_activate_hover_sound", selected_activate_hover_sound_property)
 * 
 * cdef int selected_activate_yanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yanchor_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_yanchor_property", 0);

  /* "style_selected_activate_functions.pyx":384
 * 
 * cdef int selected_activate_yanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     v = expand_anchor(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":386
 *     priority += 1
 * 
 *     v = expand_anchor(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 386, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 386, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 386, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":387
 * 
 *     v = expand_anchor(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yanchor", selected_activate_yanchor_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":383
 * register_property_function("selected_activate_hover_sound", selected_activate_hover_sound_property)
 * 
 * cdef int selected_activate_yanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_yanchor_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":391
 * register_property_function("selected_activate_yanchor", selected_activate_yanchor_property)
 * 
 * cdef int selected_activate_fore_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_fore_gutter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_fore_gutter_property", 0);

  /* "style_selected_activate_functions.pyx":392
 * 
 * cdef int selected_activate_fore_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":394
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_fore_gutter", selected_activate_fore_gutter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":391
 * register_property_function("selected_activate_yanchor", selected_activate_yanchor_property)
 * 
 * cdef int selected_activate_fore_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":398
 * register_property_function("selected_activate_fore_gutter", selected_activate_fore_gutter_property)
 * 
 * cdef int selected_activate_adjust_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_adjust_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_adjust_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":399
 * 
 * cdef int selected_activate_adjust_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":401
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_adjust_spacing", selected_activate_adjust_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":398
 * register_property_function("selected_activate_fore_gutter", selected_activate_fore_gutter_property)
 * 
 * cdef int selected_activate_adjust_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":405
 * register_property_function("selected_activate_adjust_spacing", selected_activate_adjust_spacing_property)
 * 
 * cdef int selected_activate_xfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xfill_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xfill_property", 0);

  /* "style_selected_activate_functions.pyx":406
 * 
 * cdef int selected_activate_xfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":408
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xfill", selected_activate_xfill_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":405
 * register_property_function("selected_activate_adjust_spacing", selected_activate_adjust_spacing_property)
 * 
 * cdef int selected_activate_xfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":412
 * register_property_function("selected_activate_xfill", selected_activate_xfill_property)
 * 
 * cdef int selected_activate_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_color_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_color_property", 0);

  /* "style_selected_activate_functions.pyx":413
 * 
 * cdef int selected_activate_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.color(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":415
 *     priority += 0
 * 
 *     v = renpy.easy.color(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_color); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 415, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 415, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 415, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 415, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 415, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":416
 * 
 *     v = renpy.easy.color(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_color", selected_activate_color_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":412
 * register_property_function("selected_activate_xfill", selected_activate_xfill_property)
 * 
 * cdef int selected_activate_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_color_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":420
 * register_property_function("selected_activate_color", selected_activate_color_property)
 * 
 * cdef int selected_activate_key_events_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_key_events_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_key_events_property", 0);

  /* "style_selected_activate_functions.pyx":421
 * 
 * cdef int selected_activate_key_events_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":423
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_key_events", selected_activate_key_events_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":420
 * register_property_function("selected_activate_color", selected_activate_color_property)
 * 
 * cdef int selected_activate_key_events_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":427
 * register_property_function("selected_activate_key_events", selected_activate_key_events_property)
 * 
 * cdef int selected_activate_xspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xspacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xspacing_property", 0);

  /* "style_selected_activate_functions.pyx":428
 * 
 * cdef int selected_activate_xspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":430
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xspacing", selected_activate_xspacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":427
 * register_property_function("selected_activate_key_events", selected_activate_key_events_property)
 * 
 * cdef int selected_activate_xspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":434
 * register_property_function("selected_activate_xspacing", selected_activate_xspacing_property)
 * 
 * cdef int selected_activate_size_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_size_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_size_property", 0);

  /* "style_selected_activate_functions.pyx":435
 * 
 * cdef int selected_activate_size_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":437
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_size", selected_activate_size_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":434
 * register_property_function("selected_activate_xspacing", selected_activate_xspacing_property)
 * 
 * cdef int selected_activate_size_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":441
 * register_property_function("selected_activate_size", selected_activate_size_property)
 * 
 * cdef int selected_activate_caret_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_caret_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_caret_property", 0);

  /* "style_selected_activate_functions.pyx":442
 * 
 * cdef int selected_activate_caret_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.displayable_or_none(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":444
 *     priority += 0
 * 
 *     v = renpy.easy.displayable_or_none(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_displayable_or_none); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 444, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 444, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 444, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 444, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":445
 * 
 *     v = renpy.easy.displayable_or_none(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_caret", selected_activate_caret_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":441
 * register_property_function("selected_activate_size", selected_activate_size_property)
 * 
 * cdef int selected_activate_caret_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_caret_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":449
 * register_property_function("selected_activate_caret", selected_activate_caret_property)
 * 
 * cdef int selected_activate_bottom_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_padding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bottom_padding_property", 0);

  /* "style_selected_activate_functions.pyx":450
 * 
 * cdef int selected_activate_bottom_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":452
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bottom_padding", selected_activate_bottom_padding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":449
 * register_property_function("selected_activate_caret", selected_activate_caret_property)
 * 
 * cdef int selected_activate_bottom_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":456
 * register_property_function("selected_activate_bottom_padding", selected_activate_bottom_padding_property)
 * 
 * cdef int selected_activate_box_wrap_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_box_wrap_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_box_wrap_property", 0);

  /* "style_selected_activate_functions.pyx":457
 * 
 * cdef int selected_activate_box_wrap_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":459
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_box_wrap", selected_activate_box_wrap_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":456
 * register_property_function("selected_activate_bottom_padding", selected_activate_bottom_padding_property)
 * 
 * cdef int selected_activate_box_wrap_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":463
 * register_property_function("selected_activate_box_wrap", selected_activate_box_wrap_property)
 * 
 * cdef int selected_activate_line_overlap_split_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_line_overlap_split_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_line_overlap_split_property", 0);

  /* "style_selected_activate_functions.pyx":464
 * 
 * cdef int selected_activate_line_overlap_split_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":466
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_line_overlap_split", selected_activate_line_overlap_split_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":463
 * register_property_function("selected_activate_box_wrap", selected_activate_box_wrap_property)
 * 
 * cdef int selected_activate_line_overlap_split_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":470
 * register_property_function("selected_activate_line_overlap_split", selected_activate_line_overlap_split_property)
 * 
 * cdef int selected_activate_box_wrap_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_box_wrap_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_box_wrap_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":471
 * 
 * cdef int selected_activate_box_wrap_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":473
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_box_wrap_spacing", selected_activate_box_wrap_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":470
 * register_property_function("selected_activate_line_overlap_split", selected_activate_line_overlap_split_property)
 * 
 * cdef int selected_activate_box_wrap_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":477
 * register_property_function("selected_activate_box_wrap_spacing", selected_activate_box_wrap_spacing_property)
 * 
 * cdef int selected_activate_kerning_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_kerning_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_kerning_property", 0);

  /* "style_selected_activate_functions.pyx":478
 * 
 * cdef int selected_activate_kerning_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":480
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_kerning", selected_activate_kerning_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":477
 * register_property_function("selected_activate_box_wrap_spacing", selected_activate_box_wrap_spacing_property)
 * 
 * cdef int selected_activate_kerning_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":484
 * register_property_function("selected_activate_kerning", selected_activate_kerning_property)
 * 
 * cdef int selected_activate_italic_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_italic_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_italic_property", 0);

  /* "style_selected_activate_functions.pyx":485
 * 
 * cdef int selected_activate_italic_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":487
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_italic", selected_activate_italic_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":484
 * register_property_function("selected_activate_kerning", selected_activate_kerning_property)
 * 
 * cdef int selected_activate_italic_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":491
 * register_property_function("selected_activate_italic", selected_activate_italic_property)
 * 
 * cdef int selected_activate_ruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ruby_style_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_ruby_style_property", 0);

  /* "style_selected_activate_functions.pyx":492
 * 
 * cdef int selected_activate_ruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":494
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ruby_style", selected_activate_ruby_style_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":491
 * register_property_function("selected_activate_italic", selected_activate_italic_property)
 * 
 * cdef int selected_activate_ruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":498
 * register_property_function("selected_activate_ruby_style", selected_activate_ruby_style_property)
 * 
 * cdef int selected_activate_box_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_box_layout_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_box_layout_property", 0);

  /* "style_selected_activate_functions.pyx":499
 * 
 * cdef int selected_activate_box_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":501
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_box_layout", selected_activate_box_layout_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":498
 * register_property_function("selected_activate_ruby_style", selected_activate_ruby_style_property)
 * 
 * cdef int selected_activate_box_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":505
 * register_property_function("selected_activate_box_layout", selected_activate_box_layout_property)
 * 
 * cdef int selected_activate_xpos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xpos_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xpos_property", 0);

  /* "style_selected_activate_functions.pyx":506
 * 
 * cdef int selected_activate_xpos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":508
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xpos", selected_activate_xpos_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":505
 * register_property_function("selected_activate_box_layout", selected_activate_box_layout_property)
 * 
 * cdef int selected_activate_xpos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":512
 * register_property_function("selected_activate_xpos", selected_activate_xpos_property)
 * 
 * cdef int selected_activate_xanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xanchor_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_xanchor_property", 0);

  /* "style_selected_activate_functions.pyx":513
 * 
 * cdef int selected_activate_xanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     v = expand_anchor(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":515
 *     priority += 1
 * 
 *     v = expand_anchor(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 515, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 515, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 515, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 515, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 515, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":516
 * 
 *     v = expand_anchor(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xanchor", selected_activate_xanchor_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":512
 * register_property_function("selected_activate_xpos", selected_activate_xpos_property)
 * 
 * cdef int selected_activate_xanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_xanchor_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":520
 * register_property_function("selected_activate_xanchor", selected_activate_xanchor_property)
 * 
 * cdef int selected_activate_top_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_top_margin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_top_margin_property", 0);

  /* "style_selected_activate_functions.pyx":521
 * 
 * cdef int selected_activate_top_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":523
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_top_margin", selected_activate_top_margin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":520
 * register_property_function("selected_activate_xanchor", selected_activate_xanchor_property)
 * 
 * cdef int selected_activate_top_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":527
 * register_property_function("selected_activate_top_margin", selected_activate_top_margin_property)
 * 
 * cdef int selected_activate_yoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yoffset_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_yoffset_property", 0);

  /* "style_selected_activate_functions.pyx":528
 * 
 * cdef int selected_activate_yoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":530
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yoffset", selected_activate_yoffset_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":527
 * register_property_function("selected_activate_top_margin", selected_activate_top_margin_property)
 * 
 * cdef int selected_activate_yoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":534
 * register_property_function("selected_activate_yoffset", selected_activate_yoffset_property)
 * 
 * cdef int selected_activate_child_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_child_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_child_property", 0);

  /* "style_selected_activate_functions.pyx":535
 * 
 * cdef int selected_activate_child_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = renpy.easy.displayable_or_none(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":537
 *     priority += 0
 * 
 *     v = renpy.easy.displayable_or_none(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_easy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_displayable_or_none); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 537, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 537, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 537, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 537, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 537, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":538
 * 
 *     v = renpy.easy.displayable_or_none(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_child", selected_activate_child_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":534
 * register_property_function("selected_activate_yoffset", selected_activate_yoffset_property)
 * 
 * cdef int selected_activate_child_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_child_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":542
 * register_property_function("selected_activate_child", selected_activate_child_property)
 * 
 * cdef int selected_activate_thumb_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_thumb_offset_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_thumb_offset_property", 0);

  /* "style_selected_activate_functions.pyx":543
 * 
 * cdef int selected_activate_thumb_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":545
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_thumb_offset", selected_activate_thumb_offset_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":542
 * register_property_function("selected_activate_child", selected_activate_child_property)
 * 
 * cdef int selected_activate_thumb_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":549
 * register_property_function("selected_activate_thumb_offset", selected_activate_thumb_offset_property)
 * 
 * cdef int selected_activate_xfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xfit_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xfit_property", 0);

  /* "style_selected_activate_functions.pyx":550
 * 
 * cdef int selected_activate_xfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":552
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xfit", selected_activate_xfit_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":549
 * register_property_function("selected_activate_thumb_offset", selected_activate_thumb_offset_property)
 * 
 * cdef int selected_activate_xfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":556
 * register_property_function("selected_activate_xfit", selected_activate_xfit_property)
 * 
 * cdef int selected_activate_hinting_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_hinting_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_hinting_property", 0);

  /* "style_selected_activate_functions.pyx":557
 * 
 * cdef int selected_activate_hinting_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":559
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_hinting", selected_activate_hinting_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":556
 * register_property_function("selected_activate_xfit", selected_activate_xfit_property)
 * 
 * cdef int selected_activate_hinting_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":563
 * register_property_function("selected_activate_hinting", selected_activate_hinting_property)
 * 
 * cdef int selected_activate_altruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_altruby_style_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_altruby_style_property", 0);

  /* "style_selected_activate_functions.pyx":564
 * 
 * cdef int selected_activate_altruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":566
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_altruby_style", selected_activate_altruby_style_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":563
 * register_property_function("selected_activate_hinting", selected_activate_hinting_property)
 * 
 * cdef int selected_activate_altruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":570
 * register_property_function("selected_activate_altruby_style", selected_activate_altruby_style_property)
 * 
 * cdef int selected_activate_unscrollable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_unscrollable_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_unscrollable_property", 0);

  /* "style_selected_activate_functions.pyx":571
 * 
 * cdef int selected_activate_unscrollable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":573
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_unscrollable", selected_activate_unscrollable_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":570
 * register_property_function("selected_activate_altruby_style", selected_activate_altruby_style_property)
 * 
 * cdef int selected_activate_unscrollable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":577
 * register_property_function("selected_activate_unscrollable", selected_activate_unscrollable_property)
 * 
 * cdef int selected_activate_order_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_order_reverse_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_order_reverse_property", 0);

  /* "style_selected_activate_functions.pyx":578
 * 
 * cdef int selected_activate_order_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":580
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_order_reverse", selected_activate_order_reverse_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":577
 * register_property_function("selected_activate_unscrollable", selected_activate_unscrollable_property)
 * 
 * cdef int selected_activate_order_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":584
 * register_property_function("selected_activate_order_reverse", selected_activate_order_reverse_property)
 * 
 * cdef int selected_activate_mouse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_mouse_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_mouse_property", 0);

  /* "style_selected_activate_functions.pyx":585
 * 
 * cdef int selected_activate_mouse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":587
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_mouse", selected_activate_mouse_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":584
 * register_property_function("selected_activate_order_reverse", selected_activate_order_reverse_property)
 * 
 * cdef int selected_activate_mouse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":591
 * register_property_function("selected_activate_mouse", selected_activate_mouse_property)
 * 
 * cdef int selected_activate_keyboard_focus_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_keyboard_focus_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_keyboard_focus_property", 0);

  /* "style_selected_activate_functions.pyx":592
 * 
 * cdef int selected_activate_keyboard_focus_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":594
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_keyboard_focus", selected_activate_keyboard_focus_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":591
 * register_property_function("selected_activate_mouse", selected_activate_mouse_property)
 * 
 * cdef int selected_activate_keyboard_focus_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":598
 * register_property_function("selected_activate_keyboard_focus", selected_activate_keyboard_focus_property)
 * 
 * cdef int selected_activate_first_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_first_indent_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_first_indent_property", 0);

  /* "style_selected_activate_functions.pyx":599
 * 
 * cdef int selected_activate_first_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":601
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_first_indent", selected_activate_first_indent_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":598
 * register_property_function("selected_activate_keyboard_focus", selected_activate_keyboard_focus_property)
 * 
 * cdef int selected_activate_first_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":605
 * register_property_function("selected_activate_first_indent", selected_activate_first_indent_property)
 * 
 * cdef int selected_activate_bar_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bar_vertical_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bar_vertical_property", 0);

  /* "style_selected_activate_functions.pyx":606
 * 
 * cdef int selected_activate_bar_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":608
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bar_vertical", selected_activate_bar_vertical_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":605
 * register_property_function("selected_activate_first_indent", selected_activate_first_indent_property)
 * 
 * cdef int selected_activate_bar_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":612
 * register_property_function("selected_activate_bar_vertical", selected_activate_bar_vertical_property)
 * 
 * cdef int selected_activate_right_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_right_padding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_right_padding_property", 0);

  /* "style_selected_activate_functions.pyx":613
 * 
 * cdef int selected_activate_right_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":615
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_right_padding", selected_activate_right_padding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":612
 * register_property_function("selected_activate_bar_vertical", selected_activate_bar_vertical_property)
 * 
 * cdef int selected_activate_right_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":619
 * register_property_function("selected_activate_right_padding", selected_activate_right_padding_property)
 * 
 * cdef int selected_activate_focus_rect_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_focus_rect_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_focus_rect_property", 0);

  /* "style_selected_activate_functions.pyx":620
 * 
 * cdef int selected_activate_focus_rect_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":622
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_focus_rect", selected_activate_focus_rect_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":619
 * register_property_function("selected_activate_right_padding", selected_activate_right_padding_property)
 * 
 * cdef int selected_activate_focus_rect_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":626
 * register_property_function("selected_activate_focus_rect", selected_activate_focus_rect_property)
 * 
 * cdef int selected_activate_antialias_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_antialias_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_antialias_property", 0);

  /* "style_selected_activate_functions.pyx":627
 * 
 * cdef int selected_activate_antialias_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":629
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_antialias", selected_activate_antialias_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":626
 * register_property_function("selected_activate_focus_rect", selected_activate_focus_rect_property)
 * 
 * cdef int selected_activate_antialias_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":633
 * register_property_function("selected_activate_antialias", selected_activate_antialias_property)
 * 
 * cdef int selected_activate_slow_cps_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_cps_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_slow_cps_property", 0);

  /* "style_selected_activate_functions.pyx":634
 * 
 * cdef int selected_activate_slow_cps_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":636
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_slow_cps", selected_activate_slow_cps_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":633
 * register_property_function("selected_activate_antialias", selected_activate_antialias_property)
 * 
 * cdef int selected_activate_slow_cps_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":640
 * register_property_function("selected_activate_slow_cps", selected_activate_slow_cps_property)
 * 
 * cdef int selected_activate_bar_invert_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bar_invert_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bar_invert_property", 0);

  /* "style_selected_activate_functions.pyx":641
 * 
 * cdef int selected_activate_bar_invert_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":643
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bar_invert", selected_activate_bar_invert_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":640
 * register_property_function("selected_activate_slow_cps", selected_activate_slow_cps_property)
 * 
 * cdef int selected_activate_bar_invert_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":647
 * register_property_function("selected_activate_bar_invert", selected_activate_bar_invert_property)
 * 
 * cdef int selected_activate_line_leading_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_line_leading_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_line_leading_property", 0);

  /* "style_selected_activate_functions.pyx":648
 * 
 * cdef int selected_activate_line_leading_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":650
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_line_leading", selected_activate_line_leading_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":647
 * register_property_function("selected_activate_bar_invert", selected_activate_bar_invert_property)
 * 
 * cdef int selected_activate_line_leading_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":654
 * register_property_function("selected_activate_line_leading", selected_activate_line_leading_property)
 * 
 * cdef int selected_activate_slow_cps_multiplier_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_cps_multiplier_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_slow_cps_multiplier_property", 0);

  /* "style_selected_activate_functions.pyx":655
 * 
 * cdef int selected_activate_slow_cps_multiplier_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":657
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_slow_cps_multiplier", selected_activate_slow_cps_multiplier_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":654
 * register_property_function("selected_activate_line_leading", selected_activate_line_leading_property)
 * 
 * cdef int selected_activate_slow_cps_multiplier_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":661
 * register_property_function("selected_activate_slow_cps_multiplier", selected_activate_slow_cps_multiplier_property)
 * 
 * cdef int selected_activate_thumb_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_thumb_shadow_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_thumb_shadow_property", 0);

  /* "style_selected_activate_functions.pyx":662
 * 
 * cdef int selected_activate_thumb_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":664
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 664, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 664, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 664, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":665
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_thumb_shadow", selected_activate_thumb_shadow_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":661
 * register_property_function("selected_activate_slow_cps_multiplier", selected_activate_slow_cps_multiplier_property)
 * 
 * cdef int selected_activate_thumb_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_thumb_shadow_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":669
 * register_property_function("selected_activate_thumb_shadow", selected_activate_thumb_shadow_property)
 * 
 * cdef int selected_activate_yfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yfill_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_yfill_property", 0);

  /* "style_selected_activate_functions.pyx":670
 * 
 * cdef int selected_activate_yfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 1);

  /* "style_selected_activate_functions.pyx":672
 *     priority += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yfill", selected_activate_yfill_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":669
 * register_property_function("selected_activate_thumb_shadow", selected_activate_thumb_shadow_property)
 * 
 * cdef int selected_activate_yfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 1
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":676
 * register_property_function("selected_activate_yfill", selected_activate_yfill_property)
 * 
 * cdef int selected_activate_slow_abortable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_abortable_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_slow_abortable_property", 0);

  /* "style_selected_activate_functions.pyx":677
 * 
 * cdef int selected_activate_slow_abortable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":679
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_slow_abortable", selected_activate_slow_abortable_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":676
 * register_property_function("selected_activate_yfill", selected_activate_yfill_property)
 * 
 * cdef int selected_activate_slow_abortable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":683
 * register_property_function("selected_activate_slow_abortable", selected_activate_slow_abortable_property)
 * 
 * cdef int selected_activate_justify_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_justify_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_justify_property", 0);

  /* "style_selected_activate_functions.pyx":684
 * 
 * cdef int selected_activate_justify_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":686
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_justify", selected_activate_justify_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":683
 * register_property_function("selected_activate_slow_abortable", selected_activate_slow_abortable_property)
 * 
 * cdef int selected_activate_justify_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":690
 * register_property_function("selected_activate_justify", selected_activate_justify_property)
 * 
 * cdef int selected_activate_left_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_left_gutter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_left_gutter_property", 0);

  /* "style_selected_activate_functions.pyx":691
 * 
 * cdef int selected_activate_left_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":693
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_left_gutter", selected_activate_left_gutter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":690
 * register_property_function("selected_activate_justify", selected_activate_justify_property)
 * 
 * cdef int selected_activate_left_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":697
 * register_property_function("selected_activate_left_gutter", selected_activate_left_gutter_property)
 * 
 * cdef int selected_activate_bottom_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_gutter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_bottom_gutter_property", 0);

  /* "style_selected_activate_functions.pyx":698
 * 
 * cdef int selected_activate_bottom_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":700
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bottom_gutter", selected_activate_bottom_gutter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":697
 * register_property_function("selected_activate_left_gutter", selected_activate_left_gutter_property)
 * 
 * cdef int selected_activate_bottom_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":704
 * register_property_function("selected_activate_bottom_gutter", selected_activate_bottom_gutter_property)
 * 
 * cdef int selected_activate_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_offset_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_offset_property", 0);

  /* "style_selected_activate_functions.pyx":705
 * 
 * cdef int selected_activate_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":707
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":709
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":710
 * 
 *     v = index_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_offset", selected_activate_offset_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":704
 * register_property_function("selected_activate_bottom_gutter", selected_activate_bottom_gutter_property)
 * 
 * cdef int selected_activate_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_offset_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":714
 * register_property_function("selected_activate_offset", selected_activate_offset_property)
 * 
 * cdef int selected_activate_xcenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xcenter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_xcenter_property", 0);

  /* "style_selected_activate_functions.pyx":715
 * 
 * cdef int selected_activate_xcenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":718
 * 
 * 
 *     v = 0.5             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 *     return 0
 */
  __Pyx_INCREF(__pyx_float_0_5);
  __pyx_v_v = __pyx_float_0_5;

  /* "style_selected_activate_functions.pyx":719
 * 
 *     v = 0.5
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 719, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 719, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 719, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 719, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 719, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":720
 *     v = 0.5
 *     v = expand_anchor(v)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xcenter", selected_activate_xcenter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":714
 * register_property_function("selected_activate_offset", selected_activate_offset_property)
 * 
 * cdef int selected_activate_xcenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_xcenter_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":724
 * register_property_function("selected_activate_xcenter", selected_activate_xcenter_property)
 * 
 * cdef int selected_activate_pos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_pos_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_pos_property", 0);

  /* "style_selected_activate_functions.pyx":725
 * 
 * cdef int selected_activate_pos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":727
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":729
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":730
 * 
 *     v = index_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_pos", selected_activate_pos_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":724
 * register_property_function("selected_activate_xcenter", selected_activate_xcenter_property)
 * 
 * cdef int selected_activate_pos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_pos_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":734
 * register_property_function("selected_activate_pos", selected_activate_pos_property)
 * 
 * cdef int selected_activate_minwidth_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_minwidth_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_minwidth_property", 0);

  /* "style_selected_activate_functions.pyx":735
 * 
 * cdef int selected_activate_minwidth_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":737
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_minwidth", selected_activate_minwidth_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":734
 * register_property_function("selected_activate_pos", selected_activate_pos_property)
 * 
 * cdef int selected_activate_minwidth_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":741
 * register_property_function("selected_activate_minwidth", selected_activate_minwidth_property)
 * 
 * cdef int selected_activate_minimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_minimum_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_minimum_property", 0);

  /* "style_selected_activate_functions.pyx":742
 * 
 * cdef int selected_activate_minimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":744
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 744, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":746
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":747
 * 
 *     v = index_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_minimum", selected_activate_minimum_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":741
 * register_property_function("selected_activate_minwidth", selected_activate_minwidth_property)
 * 
 * cdef int selected_activate_minimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_minimum_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":751
 * register_property_function("selected_activate_minimum", selected_activate_minimum_property)
 * 
 * cdef int selected_activate_top_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_top_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_top_bar_property", 0);

  /* "style_selected_activate_functions.pyx":752
 * 
 * cdef int selected_activate_top_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":754
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 754, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 754, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 754, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 754, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 754, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 754, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":755
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_top_bar", selected_activate_top_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":751
 * register_property_function("selected_activate_minimum", selected_activate_minimum_property)
 * 
 * cdef int selected_activate_top_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_top_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":759
 * register_property_function("selected_activate_top_bar", selected_activate_top_bar_property)
 * 
 * cdef int selected_activate_base_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_base_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_base_bar_property", 0);

  /* "style_selected_activate_functions.pyx":760
 * 
 * cdef int selected_activate_base_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":762
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 762, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 762, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 762, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 762, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 762, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 762, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":764
 *     v = none_is_null(value)
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 764, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 764, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 764, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 764, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 764, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":765
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_base_bar", selected_activate_base_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":759
 * register_property_function("selected_activate_top_bar", selected_activate_top_bar_property)
 * 
 * cdef int selected_activate_base_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_base_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":769
 * register_property_function("selected_activate_base_bar", selected_activate_base_bar_property)
 * 
 * cdef int selected_activate_xysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xysize_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_xysize_property", 0);

  /* "style_selected_activate_functions.pyx":770
 * 
 * cdef int selected_activate_xysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":772
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 772, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":774
 *     v = index_0(value)
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":776
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":778
 *     v = index_1(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 778, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":779
 * 
 *     v = index_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xysize", selected_activate_xysize_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":769
 * register_property_function("selected_activate_base_bar", selected_activate_base_bar_property)
 * 
 * cdef int selected_activate_xysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_xysize_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":783
 * register_property_function("selected_activate_xysize", selected_activate_xysize_property)
 * 
 * cdef int selected_activate_slow_speed_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_slow_speed_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_slow_speed_property", 0);

  /* "style_selected_activate_functions.pyx":784
 * 
 * cdef int selected_activate_slow_speed_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":786
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_slow_speed", selected_activate_slow_speed_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":783
 * register_property_function("selected_activate_xysize", selected_activate_xysize_property)
 * 
 * cdef int selected_activate_slow_speed_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":790
 * register_property_function("selected_activate_slow_speed", selected_activate_slow_speed_property)
 * 
 * cdef int selected_activate_area_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_area_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_area_property", 0);

  /* "style_selected_activate_functions.pyx":791
 * 
 * cdef int selected_activate_area_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":793
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 793, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":795
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 * 
 *     v = 0
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 795, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":797
 *     v = index_1(value)
 * 
 *     v = 0             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 * 
 */
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_int_0);

  /* "style_selected_activate_functions.pyx":798
 * 
 *     v = 0
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 * 
 *     v = 0
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 798, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 798, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 798, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 798, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 798, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":800
 *     v = expand_anchor(v)
 * 
 *     v = 0             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 * 
 */
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_int_0);

  /* "style_selected_activate_functions.pyx":801
 * 
 *     v = 0
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 * 
 *     v = True
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 801, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 801, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 801, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 801, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 801, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":803
 *     v = expand_anchor(v)
 * 
 *     v = True             # <<<<<<<<<<<<<<
 * 
 *     v = True
 */
  __Pyx_INCREF(Py_True);
  __Pyx_DECREF_SET(__pyx_v_v, Py_True);

  /* "style_selected_activate_functions.pyx":805
 *     v = True
 * 
 *     v = True             # <<<<<<<<<<<<<<
 * 
 *     v = index_2(value)
 */
  __Pyx_INCREF(Py_True);
  __Pyx_DECREF_SET(__pyx_v_v, Py_True);

  /* "style_selected_activate_functions.pyx":807
 *     v = True
 * 
 *     v = index_2(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_3(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_2(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":809
 *     v = index_2(value)
 * 
 *     v = index_3(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_2(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_3(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":811
 *     v = index_3(value)
 * 
 *     v = index_2(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_3(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_2(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 811, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":813
 *     v = index_2(value)
 * 
 *     v = index_3(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_3(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 813, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":814
 * 
 *     v = index_3(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_area", selected_activate_area_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":790
 * register_property_function("selected_activate_slow_speed", selected_activate_slow_speed_property)
 * 
 * cdef int selected_activate_area_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_area_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":818
 * register_property_function("selected_activate_area", selected_activate_area_property)
 * 
 * cdef int selected_activate_right_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_right_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_right_bar_property", 0);

  /* "style_selected_activate_functions.pyx":819
 * 
 * cdef int selected_activate_right_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":821
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 821, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 821, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 821, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 821, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 821, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":822
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_right_bar", selected_activate_right_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":818
 * register_property_function("selected_activate_area", selected_activate_area_property)
 * 
 * cdef int selected_activate_right_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_right_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":826
 * register_property_function("selected_activate_right_bar", selected_activate_right_bar_property)
 * 
 * cdef int selected_activate_enable_hover_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 *     return 0
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_enable_hover_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_enable_hover_property", 0);

  /* "style_selected_activate_functions.pyx":827
 * 
 * cdef int selected_activate_enable_hover_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":828
 * cdef int selected_activate_enable_hover_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_enable_hover", selected_activate_enable_hover_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":826
 * register_property_function("selected_activate_right_bar", selected_activate_right_bar_property)
 * 
 * cdef int selected_activate_enable_hover_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 *     return 0
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":832
 * register_property_function("selected_activate_enable_hover", selected_activate_enable_hover_property)
 * 
 * cdef int selected_activate_textalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_textalign_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_textalign_property", 0);

  /* "style_selected_activate_functions.pyx":833
 * 
 * cdef int selected_activate_textalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":835
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_textalign", selected_activate_textalign_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":832
 * register_property_function("selected_activate_enable_hover", selected_activate_enable_hover_property)
 * 
 * cdef int selected_activate_textalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":839
 * register_property_function("selected_activate_textalign", selected_activate_textalign_property)
 * 
 * cdef int selected_activate_box_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_box_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_box_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":840
 * 
 * cdef int selected_activate_box_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":842
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_box_spacing", selected_activate_box_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":839
 * register_property_function("selected_activate_textalign", selected_activate_textalign_property)
 * 
 * cdef int selected_activate_box_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":846
 * register_property_function("selected_activate_box_spacing", selected_activate_box_spacing_property)
 * 
 * cdef int selected_activate_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_padding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_padding_property", 0);

  /* "style_selected_activate_functions.pyx":847
 * 
 * cdef int selected_activate_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":849
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 849, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":851
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_2_or_0(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":853
 *     v = index_1(value)
 * 
 *     v = index_2_or_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_3_or_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_2_or_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 853, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":855
 *     v = index_2_or_0(value)
 * 
 *     v = index_3_or_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_3_or_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 855, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":856
 * 
 *     v = index_3_or_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_padding", selected_activate_padding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":846
 * register_property_function("selected_activate_box_spacing", selected_activate_box_spacing_property)
 * 
 * cdef int selected_activate_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_padding_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":860
 * register_property_function("selected_activate_padding", selected_activate_padding_property)
 * 
 * cdef int selected_activate_box_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_box_first_spacing_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_box_first_spacing_property", 0);

  /* "style_selected_activate_functions.pyx":861
 * 
 * cdef int selected_activate_box_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":863
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_box_first_spacing", selected_activate_box_first_spacing_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":860
 * register_property_function("selected_activate_padding", selected_activate_padding_property)
 * 
 * cdef int selected_activate_box_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":867
 * register_property_function("selected_activate_box_first_spacing", selected_activate_box_first_spacing_property)
 * 
 * cdef int selected_activate_right_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_right_gutter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_right_gutter_property", 0);

  /* "style_selected_activate_functions.pyx":868
 * 
 * cdef int selected_activate_right_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":870
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_right_gutter", selected_activate_right_gutter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":867
 * register_property_function("selected_activate_box_first_spacing", selected_activate_box_first_spacing_property)
 * 
 * cdef int selected_activate_right_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":874
 * register_property_function("selected_activate_right_gutter", selected_activate_right_gutter_property)
 * 
 * cdef int selected_activate_bottom_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_bottom_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_bottom_bar_property", 0);

  /* "style_selected_activate_functions.pyx":875
 * 
 * cdef int selected_activate_bottom_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":877
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 877, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 877, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 877, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 877, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 877, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 877, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":878
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_bottom_bar", selected_activate_bottom_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":874
 * register_property_function("selected_activate_right_gutter", selected_activate_right_gutter_property)
 * 
 * cdef int selected_activate_bottom_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_bottom_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":882
 * register_property_function("selected_activate_bottom_bar", selected_activate_bottom_bar_property)
 * 
 * cdef int selected_activate_xsize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xsize_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xsize_property", 0);

  /* "style_selected_activate_functions.pyx":883
 * 
 * cdef int selected_activate_xsize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":886
 * 
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xsize", selected_activate_xsize_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":882
 * register_property_function("selected_activate_bottom_bar", selected_activate_bottom_bar_property)
 * 
 * cdef int selected_activate_xsize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":890
 * register_property_function("selected_activate_xsize", selected_activate_xsize_property)
 * 
 * cdef int selected_activate_xpadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xpadding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xpadding_property", 0);

  /* "style_selected_activate_functions.pyx":891
 * 
 * cdef int selected_activate_xpadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":894
 * 
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xpadding", selected_activate_xpadding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":890
 * register_property_function("selected_activate_xsize", selected_activate_xsize_property)
 * 
 * cdef int selected_activate_xpadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":898
 * register_property_function("selected_activate_xpadding", selected_activate_xpadding_property)
 * 
 * cdef int selected_activate_top_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_top_gutter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_top_gutter_property", 0);

  /* "style_selected_activate_functions.pyx":899
 * 
 * cdef int selected_activate_top_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":901
 *     priority += 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_top_gutter", selected_activate_top_gutter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":898
 * register_property_function("selected_activate_xpadding", selected_activate_xpadding_property)
 * 
 * cdef int selected_activate_top_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":905
 * register_property_function("selected_activate_top_gutter", selected_activate_top_gutter_property)
 * 
 * cdef int selected_activate_xalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xalign_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_xalign_property", 0);

  /* "style_selected_activate_functions.pyx":906
 * 
 * cdef int selected_activate_xalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":909
 * 
 * 
 *     v = expand_anchor(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 909, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 909, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 909, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 909, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 909, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":910
 * 
 *     v = expand_anchor(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xalign", selected_activate_xalign_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":905
 * register_property_function("selected_activate_top_gutter", selected_activate_top_gutter_property)
 * 
 * cdef int selected_activate_xalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_xalign_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":914
 * register_property_function("selected_activate_xalign", selected_activate_xalign_property)
 * 
 * cdef int selected_activate_xmargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_xmargin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_xmargin_property", 0);

  /* "style_selected_activate_functions.pyx":915
 * 
 * cdef int selected_activate_xmargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":918
 * 
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_xmargin", selected_activate_xmargin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":914
 * register_property_function("selected_activate_xalign", selected_activate_xalign_property)
 * 
 * cdef int selected_activate_xmargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":922
 * register_property_function("selected_activate_xmargin", selected_activate_xmargin_property)
 * 
 * cdef int selected_activate_yalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_yalign_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_yalign_property", 0);

  /* "style_selected_activate_functions.pyx":923
 * 
 * cdef int selected_activate_yalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":926
 * 
 * 
 *     v = expand_anchor(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 926, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 926, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 926, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 926, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 926, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 926, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":927
 * 
 *     v = expand_anchor(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_yalign", selected_activate_yalign_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":922
 * register_property_function("selected_activate_xmargin", selected_activate_xmargin_property)
 * 
 * cdef int selected_activate_yalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_yalign_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":931
 * register_property_function("selected_activate_yalign", selected_activate_yalign_property)
 * 
 * cdef int selected_activate_ypadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ypadding_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_ypadding_property", 0);

  /* "style_selected_activate_functions.pyx":932
 * 
 * cdef int selected_activate_ypadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":935
 * 
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ypadding", selected_activate_ypadding_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":931
 * register_property_function("selected_activate_yalign", selected_activate_yalign_property)
 * 
 * cdef int selected_activate_ypadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":939
 * register_property_function("selected_activate_ypadding", selected_activate_ypadding_property)
 * 
 * cdef int selected_activate_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_align_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_align_property", 0);

  /* "style_selected_activate_functions.pyx":940
 * 
 * cdef int selected_activate_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":942
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":944
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 944, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":946
 *     v = index_1(value)
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 946, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":947
 * 
 *     v = index_0(value)
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 947, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 947, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 947, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 947, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":949
 *     v = expand_anchor(v)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 *     return 0
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 949, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":950
 * 
 *     v = index_1(value)
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 950, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 950, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 950, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 950, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 950, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 950, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":951
 *     v = index_1(value)
 *     v = expand_anchor(v)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_align", selected_activate_align_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":939
 * register_property_function("selected_activate_ypadding", selected_activate_ypadding_property)
 * 
 * cdef int selected_activate_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_align_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":955
 * register_property_function("selected_activate_align", selected_activate_align_property)
 * 
 * cdef int selected_activate_ysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ysize_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_ysize_property", 0);

  /* "style_selected_activate_functions.pyx":956
 * 
 * cdef int selected_activate_ysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":959
 * 
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ysize", selected_activate_ysize_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":955
 * register_property_function("selected_activate_align", selected_activate_align_property)
 * 
 * cdef int selected_activate_ysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":963
 * register_property_function("selected_activate_ysize", selected_activate_ysize_property)
 * 
 * cdef int selected_activate_maximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_maximum_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_maximum_property", 0);

  /* "style_selected_activate_functions.pyx":964
 * 
 * cdef int selected_activate_maximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":966
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 966, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":968
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":969
 * 
 *     v = index_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_maximum", selected_activate_maximum_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":963
 * register_property_function("selected_activate_ysize", selected_activate_ysize_property)
 * 
 * cdef int selected_activate_maximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_maximum_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":973
 * register_property_function("selected_activate_maximum", selected_activate_maximum_property)
 * 
 * cdef int selected_activate_ycenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ycenter_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_ycenter_property", 0);

  /* "style_selected_activate_functions.pyx":974
 * 
 * cdef int selected_activate_ycenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":977
 * 
 * 
 *     v = 0.5             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 *     return 0
 */
  __Pyx_INCREF(__pyx_float_0_5);
  __pyx_v_v = __pyx_float_0_5;

  /* "style_selected_activate_functions.pyx":978
 * 
 *     v = 0.5
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 978, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 978, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 978, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 978, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 978, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 978, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":979
 *     v = 0.5
 *     v = expand_anchor(v)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ycenter", selected_activate_ycenter_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":973
 * register_property_function("selected_activate_maximum", selected_activate_maximum_property)
 * 
 * cdef int selected_activate_ycenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_ycenter_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":983
 * register_property_function("selected_activate_ycenter", selected_activate_ycenter_property)
 * 
 * cdef int selected_activate_left_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_left_bar_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_left_bar_property", 0);

  /* "style_selected_activate_functions.pyx":984
 * 
 * cdef int selected_activate_left_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = none_is_null(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":986
 *     priority += 0
 * 
 *     v = none_is_null(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 986, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 986, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 986, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_value};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 986, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 986, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_value);
      __Pyx_GIVEREF(__pyx_v_value);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_value);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 986, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":987
 * 
 *     v = none_is_null(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_left_bar", selected_activate_left_bar_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":983
 * register_property_function("selected_activate_ycenter", selected_activate_ycenter_property)
 * 
 * cdef int selected_activate_left_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_left_bar_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":991
 * register_property_function("selected_activate_left_bar", selected_activate_left_bar_property)
 * 
 * cdef int selected_activate_ymargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_ymargin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, CYTHON_UNUSED PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selected_activate_ymargin_property", 0);

  /* "style_selected_activate_functions.pyx":992
 * 
 * cdef int selected_activate_ymargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":995
 * 
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_ymargin", selected_activate_ymargin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":991
 * register_property_function("selected_activate_left_bar", selected_activate_left_bar_property)
 * 
 * cdef int selected_activate_ymargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":999
 * register_property_function("selected_activate_ymargin", selected_activate_ymargin_property)
 * 
 * cdef int selected_activate_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_margin_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_margin_property", 0);

  /* "style_selected_activate_functions.pyx":1000
 * 
 * cdef int selected_activate_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":1002
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1002, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1004
 *     v = index_0(value)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_2_or_0(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1004, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1006
 *     v = index_1(value)
 * 
 *     v = index_2_or_0(value)             # <<<<<<<<<<<<<<
 * 
 *     v = index_3_or_1(value)
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_2_or_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1006, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1008
 *     v = index_2_or_0(value)
 * 
 *     v = index_3_or_1(value)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_3_or_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1008, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1009
 * 
 *     v = index_3_or_1(value)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_margin", selected_activate_margin_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":999
 * register_property_function("selected_activate_ymargin", selected_activate_ymargin_property)
 * 
 * cdef int selected_activate_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_margin_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "style_selected_activate_functions.pyx":1013
 * register_property_function("selected_activate_margin", selected_activate_margin_property)
 * 
 * cdef int selected_activate_anchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

static int __pyx_f_33style_selected_activate_functions_selected_activate_anchor_property(CYTHON_UNUSED PyObject **__pyx_v_cache, CYTHON_UNUSED int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("selected_activate_anchor_property", 0);

  /* "style_selected_activate_functions.pyx":1014
 * 
 * cdef int selected_activate_anchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 *     priority += 0             # <<<<<<<<<<<<<<
 * 
 *     v = index_0(value)
 */
  __pyx_v_priority = (__pyx_v_priority + 0);

  /* "style_selected_activate_functions.pyx":1016
 *     priority += 0
 * 
 *     v = index_0(value)             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 * 
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_0(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1016, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_v = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1017
 * 
 *     v = index_0(value)
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 * 
 *     v = index_1(value)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1017, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1017, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1017, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1017, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1017, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1017, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1019
 *     v = expand_anchor(v)
 * 
 *     v = index_1(value)             # <<<<<<<<<<<<<<
 *     v = expand_anchor(v)
 *     return 0
 */
  __pyx_t_1 = __pyx_f_33style_selected_activate_functions_index_1(__pyx_v_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1019, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1020
 * 
 *     v = index_1(value)
 *     v = expand_anchor(v)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1020, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_v); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1020, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1020, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_v};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1020, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1020, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_v);
      __Pyx_GIVEREF(__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_v);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1020, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_v, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "style_selected_activate_functions.pyx":1021
 *     v = index_1(value)
 *     v = expand_anchor(v)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * register_property_function("selected_activate_anchor", selected_activate_anchor_property)
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "style_selected_activate_functions.pyx":1013
 * register_property_function("selected_activate_margin", selected_activate_margin_property)
 * 
 * cdef int selected_activate_anchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:             # <<<<<<<<<<<<<<
 *     priority += 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("style_selected_activate_functions.selected_activate_anchor_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/style.pxd":41
 * ctypedef int (*property_function)(PyObject **cache, int *cache_priorities, int priority, object value) except -1
 * 
 * cdef inline void assign(int index, PyObject **cache, int *cache_priorities, int priority, PyObject *value):             # <<<<<<<<<<<<<<
 *     """
 *     Assigns `value` to `index` in `cache`, if it has a higher (or equal) priority than the value
 */

static CYTHON_INLINE void __pyx_f_5renpy_5style_assign(int __pyx_v_index, PyObject **__pyx_v_cache, int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("assign", 0);

  /* "renpy/style.pxd":47
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_1 = ((__pyx_v_priority < (__pyx_v_cache_priorities[__pyx_v_index])) != 0);
  if (__pyx_t_1) {

    /* "renpy/style.pxd":48
 * 
 *     if priority < cache_priorities[index]:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     Py_XDECREF(cache[index])
 */
    goto __pyx_L0;

    /* "renpy/style.pxd":47
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "renpy/style.pxd":50
 *         return
 * 
 *     Py_XDECREF(cache[index])             # <<<<<<<<<<<<<<
 *     Py_XINCREF(value)
 * 
 */
  Py_XDECREF((__pyx_v_cache[__pyx_v_index]));

  /* "renpy/style.pxd":51
 * 
 *     Py_XDECREF(cache[index])
 *     Py_XINCREF(value)             # <<<<<<<<<<<<<<
 * 
 *     cache[index] = value
 */
  Py_XINCREF(__pyx_v_value);

  /* "renpy/style.pxd":53
 *     Py_XINCREF(value)
 * 
 *     cache[index] = value             # <<<<<<<<<<<<<<
 *     cache_priorities[index] = priority
 * 
 */
  (__pyx_v_cache[__pyx_v_index]) = __pyx_v_value;

  /* "renpy/style.pxd":54
 * 
 *     cache[index] = value
 *     cache_priorities[index] = priority             # <<<<<<<<<<<<<<
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):
 */
  (__pyx_v_cache_priorities[__pyx_v_index]) = __pyx_v_priority;

  /* "renpy/style.pxd":41
 * ctypedef int (*property_function)(PyObject **cache, int *cache_priorities, int priority, object value) except -1
 * 
 * cdef inline void assign(int index, PyObject **cache, int *cache_priorities, int priority, PyObject *value):             # <<<<<<<<<<<<<<
 *     """
 *     Assigns `value` to `index` in `cache`, if it has a higher (or equal) priority than the value
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/style.pxd":56
 *     cache_priorities[index] = priority
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):             # <<<<<<<<<<<<<<
 *     """
 *     Like assign, but if the value can be duplicated, duplicates it and assigns
 */

static CYTHON_INLINE void __pyx_f_5renpy_5style_assign_prefixed(int __pyx_v_index, PyObject **__pyx_v_cache, int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_d, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_v_args = NULL;
  PyObject *__pyx_v_dd = NULL;
  PyObject *__pyx_v_value;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("assign_prefixed", 0);

  /* "renpy/style.pxd":62
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_1 = ((__pyx_v_priority < (__pyx_v_cache_priorities[__pyx_v_index])) != 0);
  if (__pyx_t_1) {

    /* "renpy/style.pxd":63
 * 
 *     if priority < cache_priorities[index]:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     if (d is not None) and d._duplicatable:
 */
    goto __pyx_L0;

    /* "renpy/style.pxd":62
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "renpy/style.pxd":65
 *         return
 * 
 *     if (d is not None) and d._duplicatable:             # <<<<<<<<<<<<<<
 * 
 *         args = d._args.copy(prefix=prefix)
 */
  __pyx_t_2 = (__pyx_v_d != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_duplicatable); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "renpy/style.pxd":67
 *     if (d is not None) and d._duplicatable:
 * 
 *         args = d._args.copy(prefix=prefix)             # <<<<<<<<<<<<<<
 *         dd = d._duplicate(args)
 *         dd._unique()
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_args); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_copy); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_prefix, __pyx_v_prefix) < 0) __PYX_ERR(2, 67, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_args = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "renpy/style.pxd":68
 * 
 *         args = d._args.copy(prefix=prefix)
 *         dd = d._duplicate(args)             # <<<<<<<<<<<<<<
 *         dd._unique()
 *     else:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_duplicate); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_args); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_args};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_6);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_args};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_6);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_v_args);
        __Pyx_GIVEREF(__pyx_v_args);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_args);
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_dd = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "renpy/style.pxd":69
 *         args = d._args.copy(prefix=prefix)
 *         dd = d._duplicate(args)
 *         dd._unique()             # <<<<<<<<<<<<<<
 *     else:
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dd, __pyx_n_s_unique); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_7) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 69, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 69, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "renpy/style.pxd":65
 *         return
 * 
 *     if (d is not None) and d._duplicatable:             # <<<<<<<<<<<<<<
 * 
 *         args = d._args.copy(prefix=prefix)
 */
    goto __pyx_L4;
  }

  /* "renpy/style.pxd":72
 *     else:
 * 
 *         dd = d             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject *value = <PyObject *> dd
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_d);
    __pyx_v_dd = __pyx_v_d;
  }
  __pyx_L4:;

  /* "renpy/style.pxd":74
 *         dd = d
 * 
 *     cdef PyObject *value = <PyObject *> dd             # <<<<<<<<<<<<<<
 * 
 *     Py_XDECREF(cache[index])
 */
  __pyx_v_value = ((PyObject *)__pyx_v_dd);

  /* "renpy/style.pxd":76
 *     cdef PyObject *value = <PyObject *> dd
 * 
 *     Py_XDECREF(cache[index])             # <<<<<<<<<<<<<<
 *     Py_XINCREF(value)
 * 
 */
  Py_XDECREF((__pyx_v_cache[__pyx_v_index]));

  /* "renpy/style.pxd":77
 * 
 *     Py_XDECREF(cache[index])
 *     Py_XINCREF(value)             # <<<<<<<<<<<<<<
 * 
 *     cache[index] = value
 */
  Py_XINCREF(__pyx_v_value);

  /* "renpy/style.pxd":79
 *     Py_XINCREF(value)
 * 
 *     cache[index] = value             # <<<<<<<<<<<<<<
 *     cache_priorities[index] = priority
 * 
 */
  (__pyx_v_cache[__pyx_v_index]) = __pyx_v_value;

  /* "renpy/style.pxd":80
 * 
 *     cache[index] = value
 *     cache_priorities[index] = priority             # <<<<<<<<<<<<<<
 * 
 * cdef void register_property_function(name, property_function function)
 */
  (__pyx_v_cache_priorities[__pyx_v_index]) = __pyx_v_priority;

  /* "renpy/style.pxd":56
 *     cache_priorities[index] = priority
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):             # <<<<<<<<<<<<<<
 *     """
 *     Like assign, but if the value can be duplicated, duplicates it and assigns
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_WriteUnraisable("renpy.style.assign_prefixed", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_dd);
  __Pyx_RefNannyFinishContext();
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_style_selected_activate_functions(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_style_selected_activate_functions},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "style_selected_activate_functions",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_color, __pyx_k_color, sizeof(__pyx_k_color), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_displayable_or_none, __pyx_k_displayable_or_none, sizeof(__pyx_k_displayable_or_none), 0, 0, 1, 1},
  {&__pyx_n_s_duplicatable, __pyx_k_duplicatable, sizeof(__pyx_k_duplicatable), 0, 0, 1, 1},
  {&__pyx_n_s_duplicate, __pyx_k_duplicate, sizeof(__pyx_k_duplicate), 0, 0, 1, 1},
  {&__pyx_n_s_easy, __pyx_k_easy, sizeof(__pyx_k_easy), 0, 0, 1, 1},
  {&__pyx_n_s_expand_anchor, __pyx_k_expand_anchor, sizeof(__pyx_k_expand_anchor), 0, 0, 1, 1},
  {&__pyx_n_s_expand_focus_mask, __pyx_k_expand_focus_mask, sizeof(__pyx_k_expand_focus_mask), 0, 0, 1, 1},
  {&__pyx_n_s_expand_outlines, __pyx_k_expand_outlines, sizeof(__pyx_k_expand_outlines), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_none_is_null, __pyx_k_none_is_null, sizeof(__pyx_k_none_is_null), 0, 0, 1, 1},
  {&__pyx_n_s_prefix, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_renpy, __pyx_k_renpy, sizeof(__pyx_k_renpy), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_styledata_styleutil, __pyx_k_renpy_styledata_styleutil, sizeof(__pyx_k_renpy_styledata_styleutil), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_activate_sound, __pyx_k_selected_activate_activate_sound, sizeof(__pyx_k_selected_activate_activate_sound), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_adjust_spacing, __pyx_k_selected_activate_adjust_spacing, sizeof(__pyx_k_selected_activate_adjust_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_aft_bar, __pyx_k_selected_activate_aft_bar, sizeof(__pyx_k_selected_activate_aft_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_aft_gutter, __pyx_k_selected_activate_aft_gutter, sizeof(__pyx_k_selected_activate_aft_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_align, __pyx_k_selected_activate_align, sizeof(__pyx_k_selected_activate_align), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_alt, __pyx_k_selected_activate_alt, sizeof(__pyx_k_selected_activate_alt), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_altruby_style, __pyx_k_selected_activate_altruby_style, sizeof(__pyx_k_selected_activate_altruby_style), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_anchor, __pyx_k_selected_activate_anchor, sizeof(__pyx_k_selected_activate_anchor), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_antialias, __pyx_k_selected_activate_antialias, sizeof(__pyx_k_selected_activate_antialias), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_area, __pyx_k_selected_activate_area, sizeof(__pyx_k_selected_activate_area), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_background, __pyx_k_selected_activate_background, sizeof(__pyx_k_selected_activate_background), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bar_invert, __pyx_k_selected_activate_bar_invert, sizeof(__pyx_k_selected_activate_bar_invert), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bar_resizing, __pyx_k_selected_activate_bar_resizing, sizeof(__pyx_k_selected_activate_bar_resizing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bar_vertical, __pyx_k_selected_activate_bar_vertical, sizeof(__pyx_k_selected_activate_bar_vertical), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_base_bar, __pyx_k_selected_activate_base_bar, sizeof(__pyx_k_selected_activate_base_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_black_color, __pyx_k_selected_activate_black_color, sizeof(__pyx_k_selected_activate_black_color), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bold, __pyx_k_selected_activate_bold, sizeof(__pyx_k_selected_activate_bold), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bottom_bar, __pyx_k_selected_activate_bottom_bar, sizeof(__pyx_k_selected_activate_bottom_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bottom_gutter, __pyx_k_selected_activate_bottom_gutter, sizeof(__pyx_k_selected_activate_bottom_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bottom_margin, __pyx_k_selected_activate_bottom_margin, sizeof(__pyx_k_selected_activate_bottom_margin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_bottom_padding, __pyx_k_selected_activate_bottom_padding, sizeof(__pyx_k_selected_activate_bottom_padding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_box_first_spac, __pyx_k_selected_activate_box_first_spac, sizeof(__pyx_k_selected_activate_box_first_spac), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_box_layout, __pyx_k_selected_activate_box_layout, sizeof(__pyx_k_selected_activate_box_layout), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_box_reverse, __pyx_k_selected_activate_box_reverse, sizeof(__pyx_k_selected_activate_box_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_box_spacing, __pyx_k_selected_activate_box_spacing, sizeof(__pyx_k_selected_activate_box_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_box_wrap, __pyx_k_selected_activate_box_wrap, sizeof(__pyx_k_selected_activate_box_wrap), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_box_wrap_spaci, __pyx_k_selected_activate_box_wrap_spaci, sizeof(__pyx_k_selected_activate_box_wrap_spaci), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_caret, __pyx_k_selected_activate_caret, sizeof(__pyx_k_selected_activate_caret), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_child, __pyx_k_selected_activate_child, sizeof(__pyx_k_selected_activate_child), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_clipping, __pyx_k_selected_activate_clipping, sizeof(__pyx_k_selected_activate_clipping), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_color, __pyx_k_selected_activate_color, sizeof(__pyx_k_selected_activate_color), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_debug, __pyx_k_selected_activate_debug, sizeof(__pyx_k_selected_activate_debug), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_drop_shadow, __pyx_k_selected_activate_drop_shadow, sizeof(__pyx_k_selected_activate_drop_shadow), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_drop_shadow_co, __pyx_k_selected_activate_drop_shadow_co, sizeof(__pyx_k_selected_activate_drop_shadow_co), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_enable_hover, __pyx_k_selected_activate_enable_hover, sizeof(__pyx_k_selected_activate_enable_hover), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_first_indent, __pyx_k_selected_activate_first_indent, sizeof(__pyx_k_selected_activate_first_indent), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_first_spacing, __pyx_k_selected_activate_first_spacing, sizeof(__pyx_k_selected_activate_first_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_fit_first, __pyx_k_selected_activate_fit_first, sizeof(__pyx_k_selected_activate_fit_first), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_focus_mask, __pyx_k_selected_activate_focus_mask, sizeof(__pyx_k_selected_activate_focus_mask), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_focus_rect, __pyx_k_selected_activate_focus_rect, sizeof(__pyx_k_selected_activate_focus_rect), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_font, __pyx_k_selected_activate_font, sizeof(__pyx_k_selected_activate_font), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_fore_bar, __pyx_k_selected_activate_fore_bar, sizeof(__pyx_k_selected_activate_fore_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_fore_gutter, __pyx_k_selected_activate_fore_gutter, sizeof(__pyx_k_selected_activate_fore_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_foreground, __pyx_k_selected_activate_foreground, sizeof(__pyx_k_selected_activate_foreground), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_hinting, __pyx_k_selected_activate_hinting, sizeof(__pyx_k_selected_activate_hinting), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_hover_sound, __pyx_k_selected_activate_hover_sound, sizeof(__pyx_k_selected_activate_hover_sound), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_hyperlink_func, __pyx_k_selected_activate_hyperlink_func, sizeof(__pyx_k_selected_activate_hyperlink_func), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_italic, __pyx_k_selected_activate_italic, sizeof(__pyx_k_selected_activate_italic), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_justify, __pyx_k_selected_activate_justify, sizeof(__pyx_k_selected_activate_justify), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_kerning, __pyx_k_selected_activate_kerning, sizeof(__pyx_k_selected_activate_kerning), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_key_events, __pyx_k_selected_activate_key_events, sizeof(__pyx_k_selected_activate_key_events), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_keyboard_focus, __pyx_k_selected_activate_keyboard_focus, sizeof(__pyx_k_selected_activate_keyboard_focus), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_language, __pyx_k_selected_activate_language, sizeof(__pyx_k_selected_activate_language), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_layout, __pyx_k_selected_activate_layout, sizeof(__pyx_k_selected_activate_layout), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_left_bar, __pyx_k_selected_activate_left_bar, sizeof(__pyx_k_selected_activate_left_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_left_gutter, __pyx_k_selected_activate_left_gutter, sizeof(__pyx_k_selected_activate_left_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_left_margin, __pyx_k_selected_activate_left_margin, sizeof(__pyx_k_selected_activate_left_margin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_left_padding, __pyx_k_selected_activate_left_padding, sizeof(__pyx_k_selected_activate_left_padding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_line_leading, __pyx_k_selected_activate_line_leading, sizeof(__pyx_k_selected_activate_line_leading), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_line_overlap_s, __pyx_k_selected_activate_line_overlap_s, sizeof(__pyx_k_selected_activate_line_overlap_s), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_line_spacing, __pyx_k_selected_activate_line_spacing, sizeof(__pyx_k_selected_activate_line_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_margin, __pyx_k_selected_activate_margin, sizeof(__pyx_k_selected_activate_margin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_maximum, __pyx_k_selected_activate_maximum, sizeof(__pyx_k_selected_activate_maximum), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_min_width, __pyx_k_selected_activate_min_width, sizeof(__pyx_k_selected_activate_min_width), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_minimum, __pyx_k_selected_activate_minimum, sizeof(__pyx_k_selected_activate_minimum), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_minwidth, __pyx_k_selected_activate_minwidth, sizeof(__pyx_k_selected_activate_minwidth), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_mouse, __pyx_k_selected_activate_mouse, sizeof(__pyx_k_selected_activate_mouse), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_newline_indent, __pyx_k_selected_activate_newline_indent, sizeof(__pyx_k_selected_activate_newline_indent), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_offset, __pyx_k_selected_activate_offset, sizeof(__pyx_k_selected_activate_offset), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_order_reverse, __pyx_k_selected_activate_order_reverse, sizeof(__pyx_k_selected_activate_order_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_outline_scalin, __pyx_k_selected_activate_outline_scalin, sizeof(__pyx_k_selected_activate_outline_scalin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_outlines, __pyx_k_selected_activate_outlines, sizeof(__pyx_k_selected_activate_outlines), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_padding, __pyx_k_selected_activate_padding, sizeof(__pyx_k_selected_activate_padding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_pos, __pyx_k_selected_activate_pos, sizeof(__pyx_k_selected_activate_pos), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_rest_indent, __pyx_k_selected_activate_rest_indent, sizeof(__pyx_k_selected_activate_rest_indent), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_right_bar, __pyx_k_selected_activate_right_bar, sizeof(__pyx_k_selected_activate_right_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_right_gutter, __pyx_k_selected_activate_right_gutter, sizeof(__pyx_k_selected_activate_right_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_right_margin, __pyx_k_selected_activate_right_margin, sizeof(__pyx_k_selected_activate_right_margin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_right_padding, __pyx_k_selected_activate_right_padding, sizeof(__pyx_k_selected_activate_right_padding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ruby_style, __pyx_k_selected_activate_ruby_style, sizeof(__pyx_k_selected_activate_ruby_style), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_size, __pyx_k_selected_activate_size, sizeof(__pyx_k_selected_activate_size), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_size_group, __pyx_k_selected_activate_size_group, sizeof(__pyx_k_selected_activate_size_group), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_slow_abortable, __pyx_k_selected_activate_slow_abortable, sizeof(__pyx_k_selected_activate_slow_abortable), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_slow_cps, __pyx_k_selected_activate_slow_cps, sizeof(__pyx_k_selected_activate_slow_cps), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_slow_cps_multi, __pyx_k_selected_activate_slow_cps_multi, sizeof(__pyx_k_selected_activate_slow_cps_multi), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_slow_speed, __pyx_k_selected_activate_slow_speed, sizeof(__pyx_k_selected_activate_slow_speed), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_spacing, __pyx_k_selected_activate_spacing, sizeof(__pyx_k_selected_activate_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_strikethrough, __pyx_k_selected_activate_strikethrough, sizeof(__pyx_k_selected_activate_strikethrough), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_subpixel, __pyx_k_selected_activate_subpixel, sizeof(__pyx_k_selected_activate_subpixel), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_subtitle_width, __pyx_k_selected_activate_subtitle_width, sizeof(__pyx_k_selected_activate_subtitle_width), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_text_align, __pyx_k_selected_activate_text_align, sizeof(__pyx_k_selected_activate_text_align), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_text_y_fudge, __pyx_k_selected_activate_text_y_fudge, sizeof(__pyx_k_selected_activate_text_y_fudge), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_textalign, __pyx_k_selected_activate_textalign, sizeof(__pyx_k_selected_activate_textalign), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_thumb, __pyx_k_selected_activate_thumb, sizeof(__pyx_k_selected_activate_thumb), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_thumb_offset, __pyx_k_selected_activate_thumb_offset, sizeof(__pyx_k_selected_activate_thumb_offset), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_thumb_shadow, __pyx_k_selected_activate_thumb_shadow, sizeof(__pyx_k_selected_activate_thumb_shadow), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_time_policy, __pyx_k_selected_activate_time_policy, sizeof(__pyx_k_selected_activate_time_policy), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_top_bar, __pyx_k_selected_activate_top_bar, sizeof(__pyx_k_selected_activate_top_bar), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_top_gutter, __pyx_k_selected_activate_top_gutter, sizeof(__pyx_k_selected_activate_top_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_top_margin, __pyx_k_selected_activate_top_margin, sizeof(__pyx_k_selected_activate_top_margin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_top_padding, __pyx_k_selected_activate_top_padding, sizeof(__pyx_k_selected_activate_top_padding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_underline, __pyx_k_selected_activate_underline, sizeof(__pyx_k_selected_activate_underline), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_unscrollable, __pyx_k_selected_activate_unscrollable, sizeof(__pyx_k_selected_activate_unscrollable), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_vertical, __pyx_k_selected_activate_vertical, sizeof(__pyx_k_selected_activate_vertical), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xalign, __pyx_k_selected_activate_xalign, sizeof(__pyx_k_selected_activate_xalign), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xanchor, __pyx_k_selected_activate_xanchor, sizeof(__pyx_k_selected_activate_xanchor), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xcenter, __pyx_k_selected_activate_xcenter, sizeof(__pyx_k_selected_activate_xcenter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xfill, __pyx_k_selected_activate_xfill, sizeof(__pyx_k_selected_activate_xfill), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xfit, __pyx_k_selected_activate_xfit, sizeof(__pyx_k_selected_activate_xfit), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xmargin, __pyx_k_selected_activate_xmargin, sizeof(__pyx_k_selected_activate_xmargin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xmaximum, __pyx_k_selected_activate_xmaximum, sizeof(__pyx_k_selected_activate_xmaximum), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xminimum, __pyx_k_selected_activate_xminimum, sizeof(__pyx_k_selected_activate_xminimum), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xoffset, __pyx_k_selected_activate_xoffset, sizeof(__pyx_k_selected_activate_xoffset), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xpadding, __pyx_k_selected_activate_xpadding, sizeof(__pyx_k_selected_activate_xpadding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xpos, __pyx_k_selected_activate_xpos, sizeof(__pyx_k_selected_activate_xpos), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xsize, __pyx_k_selected_activate_xsize, sizeof(__pyx_k_selected_activate_xsize), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xspacing, __pyx_k_selected_activate_xspacing, sizeof(__pyx_k_selected_activate_xspacing), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_xysize, __pyx_k_selected_activate_xysize, sizeof(__pyx_k_selected_activate_xysize), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yalign, __pyx_k_selected_activate_yalign, sizeof(__pyx_k_selected_activate_yalign), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yanchor, __pyx_k_selected_activate_yanchor, sizeof(__pyx_k_selected_activate_yanchor), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ycenter, __pyx_k_selected_activate_ycenter, sizeof(__pyx_k_selected_activate_ycenter), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yfill, __pyx_k_selected_activate_yfill, sizeof(__pyx_k_selected_activate_yfill), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yfit, __pyx_k_selected_activate_yfit, sizeof(__pyx_k_selected_activate_yfit), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ymargin, __pyx_k_selected_activate_ymargin, sizeof(__pyx_k_selected_activate_ymargin), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ymaximum, __pyx_k_selected_activate_ymaximum, sizeof(__pyx_k_selected_activate_ymaximum), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yminimum, __pyx_k_selected_activate_yminimum, sizeof(__pyx_k_selected_activate_yminimum), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yoffset, __pyx_k_selected_activate_yoffset, sizeof(__pyx_k_selected_activate_yoffset), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ypadding, __pyx_k_selected_activate_ypadding, sizeof(__pyx_k_selected_activate_ypadding), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ypos, __pyx_k_selected_activate_ypos, sizeof(__pyx_k_selected_activate_ypos), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_ysize, __pyx_k_selected_activate_ysize, sizeof(__pyx_k_selected_activate_ysize), 0, 0, 1, 1},
  {&__pyx_n_s_selected_activate_yspacing, __pyx_k_selected_activate_yspacing, sizeof(__pyx_k_selected_activate_yspacing), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_unique, __pyx_k_unique, sizeof(__pyx_k_unique), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  return 0;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  __pyx_float_0_5 = PyFloat_FromDouble(0.5); if (unlikely(!__pyx_float_0_5)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(1, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initstyle_selected_activate_functions(void); /*proto*/
PyMODINIT_FUNC initstyle_selected_activate_functions(void)
#else
PyMODINIT_FUNC PyInit_style_selected_activate_functions(void); /*proto*/
PyMODINIT_FUNC PyInit_style_selected_activate_functions(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_style_selected_activate_functions(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_style_selected_activate_functions(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("style_selected_activate_functions", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_style_selected_activate_functions) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "style_selected_activate_functions")) {
      if (unlikely(PyDict_SetItemString(modules, "style_selected_activate_functions", __pyx_m) < 0)) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(3, 9, __pyx_L1_error)
  __pyx_ptype_5renpy_5style_StyleCore = __Pyx_ImportType("renpy.style", "StyleCore", sizeof(struct __pyx_obj_5renpy_5style_StyleCore), 1); if (unlikely(!__pyx_ptype_5renpy_5style_StyleCore)) __PYX_ERR(2, 84, __pyx_L1_error)
  __pyx_vtabptr_5renpy_5style_StyleCore = (struct __pyx_vtabstruct_5renpy_5style_StyleCore*)__Pyx_GetVtable(__pyx_ptype_5renpy_5style_StyleCore->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_5style_StyleCore)) __PYX_ERR(2, 84, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("renpy.style"); if (!__pyx_t_1) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "register_property_function", (void (**)(void))&__pyx_f_5renpy_5style_register_property_function, "void (PyObject *, __pyx_t_5renpy_5style_property_function)") < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "include/style_common.pxi":22
 * # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * import renpy             # <<<<<<<<<<<<<<
 * 
 * from renpy.style cimport register_property_function, assign, assign_prefixed
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_renpy, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_renpy, __pyx_t_2) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "include/style_common.pxi":27
 * from cpython.ref cimport PyObject
 * 
 * from renpy.styledata.styleutil import none_is_null, expand_focus_mask, expand_outlines, expand_anchor             # <<<<<<<<<<<<<<
 * 
 * cdef inline object index_0(object v):
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_none_is_null);
  __Pyx_GIVEREF(__pyx_n_s_none_is_null);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_none_is_null);
  __Pyx_INCREF(__pyx_n_s_expand_focus_mask);
  __Pyx_GIVEREF(__pyx_n_s_expand_focus_mask);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_expand_focus_mask);
  __Pyx_INCREF(__pyx_n_s_expand_outlines);
  __Pyx_GIVEREF(__pyx_n_s_expand_outlines);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_expand_outlines);
  __Pyx_INCREF(__pyx_n_s_expand_anchor);
  __Pyx_GIVEREF(__pyx_n_s_expand_anchor);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_n_s_expand_anchor);
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_renpy_styledata_styleutil, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_none_is_null); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_none_is_null, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_expand_focus_mask); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expand_focus_mask, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_expand_outlines); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expand_outlines, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_expand_anchor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_expand_anchor, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "style_selected_activate_functions.pyx":9
 *     return 0
 * 
 * register_property_function("selected_activate_black_color", selected_activate_black_color_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bottom_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_black_color, __pyx_f_33style_selected_activate_functions_selected_activate_black_color_property);

  /* "style_selected_activate_functions.pyx":16
 *     return 0
 * 
 * register_property_function("selected_activate_bottom_margin", selected_activate_bottom_margin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_line_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bottom_margin, __pyx_f_33style_selected_activate_functions_selected_activate_bottom_margin_property);

  /* "style_selected_activate_functions.pyx":23
 *     return 0
 * 
 * register_property_function("selected_activate_line_spacing", selected_activate_line_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bar_resizing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_line_spacing, __pyx_f_33style_selected_activate_functions_selected_activate_line_spacing_property);

  /* "style_selected_activate_functions.pyx":30
 *     return 0
 * 
 * register_property_function("selected_activate_bar_resizing", selected_activate_bar_resizing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_left_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bar_resizing, __pyx_f_33style_selected_activate_functions_selected_activate_bar_resizing_property);

  /* "style_selected_activate_functions.pyx":37
 *     return 0
 * 
 * register_property_function("selected_activate_left_margin", selected_activate_left_margin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_left_margin, __pyx_f_33style_selected_activate_functions_selected_activate_left_margin_property);

  /* "style_selected_activate_functions.pyx":44
 *     return 0
 * 
 * register_property_function("selected_activate_yminimum", selected_activate_yminimum_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_alt_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yminimum, __pyx_f_33style_selected_activate_functions_selected_activate_yminimum_property);

  /* "style_selected_activate_functions.pyx":51
 *     return 0
 * 
 * register_property_function("selected_activate_alt", selected_activate_alt_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_font_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_alt, __pyx_f_33style_selected_activate_functions_selected_activate_alt_property);

  /* "style_selected_activate_functions.pyx":58
 *     return 0
 * 
 * register_property_function("selected_activate_font", selected_activate_font_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_underline_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_font, __pyx_f_33style_selected_activate_functions_selected_activate_font_property);

  /* "style_selected_activate_functions.pyx":65
 *     return 0
 * 
 * register_property_function("selected_activate_underline", selected_activate_underline_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_underline, __pyx_f_33style_selected_activate_functions_selected_activate_underline_property);

  /* "style_selected_activate_functions.pyx":72
 *     return 0
 * 
 * register_property_function("selected_activate_layout", selected_activate_layout_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_thumb_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_layout, __pyx_f_33style_selected_activate_functions_selected_activate_layout_property);

  /* "style_selected_activate_functions.pyx":80
 *     return 0
 * 
 * register_property_function("selected_activate_thumb", selected_activate_thumb_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_box_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_thumb, __pyx_f_33style_selected_activate_functions_selected_activate_thumb_property);

  /* "style_selected_activate_functions.pyx":87
 *     return 0
 * 
 * register_property_function("selected_activate_box_reverse", selected_activate_box_reverse_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_outline_scaling_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_box_reverse, __pyx_f_33style_selected_activate_functions_selected_activate_box_reverse_property);

  /* "style_selected_activate_functions.pyx":94
 *     return 0
 * 
 * register_property_function("selected_activate_outline_scaling", selected_activate_outline_scaling_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_text_y_fudge_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_outline_scalin, __pyx_f_33style_selected_activate_functions_selected_activate_outline_scaling_property);

  /* "style_selected_activate_functions.pyx":101
 *     return 0
 * 
 * register_property_function("selected_activate_text_y_fudge", selected_activate_text_y_fudge_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_aft_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_text_y_fudge, __pyx_f_33style_selected_activate_functions_selected_activate_text_y_fudge_property);

  /* "style_selected_activate_functions.pyx":108
 *     return 0
 * 
 * register_property_function("selected_activate_aft_gutter", selected_activate_aft_gutter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_right_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_aft_gutter, __pyx_f_33style_selected_activate_functions_selected_activate_aft_gutter_property);

  /* "style_selected_activate_functions.pyx":115
 *     return 0
 * 
 * register_property_function("selected_activate_right_margin", selected_activate_right_margin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_rest_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_right_margin, __pyx_f_33style_selected_activate_functions_selected_activate_right_margin_property);

  /* "style_selected_activate_functions.pyx":122
 *     return 0
 * 
 * register_property_function("selected_activate_rest_indent", selected_activate_rest_indent_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_background_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_rest_indent, __pyx_f_33style_selected_activate_functions_selected_activate_rest_indent_property);

  /* "style_selected_activate_functions.pyx":130
 *     return 0
 * 
 * register_property_function("selected_activate_background", selected_activate_background_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_outlines_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_background, __pyx_f_33style_selected_activate_functions_selected_activate_background_property);

  /* "style_selected_activate_functions.pyx":138
 *     return 0
 * 
 * register_property_function("selected_activate_outlines", selected_activate_outlines_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ymaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_outlines, __pyx_f_33style_selected_activate_functions_selected_activate_outlines_property);

  /* "style_selected_activate_functions.pyx":145
 *     return 0
 * 
 * register_property_function("selected_activate_ymaximum", selected_activate_ymaximum_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ymaximum, __pyx_f_33style_selected_activate_functions_selected_activate_ymaximum_property);

  /* "style_selected_activate_functions.pyx":152
 *     return 0
 * 
 * register_property_function("selected_activate_yspacing", selected_activate_yspacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_size_group_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yspacing, __pyx_f_33style_selected_activate_functions_selected_activate_yspacing_property);

  /* "style_selected_activate_functions.pyx":159
 *     return 0
 * 
 * register_property_function("selected_activate_size_group", selected_activate_size_group_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_debug_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_size_group, __pyx_f_33style_selected_activate_functions_selected_activate_size_group_property);

  /* "style_selected_activate_functions.pyx":166
 *     return 0
 * 
 * register_property_function("selected_activate_debug", selected_activate_debug_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_clipping_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_debug, __pyx_f_33style_selected_activate_functions_selected_activate_debug_property);

  /* "style_selected_activate_functions.pyx":173
 *     return 0
 * 
 * register_property_function("selected_activate_clipping", selected_activate_clipping_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_clipping, __pyx_f_33style_selected_activate_functions_selected_activate_clipping_property);

  /* "style_selected_activate_functions.pyx":180
 *     return 0
 * 
 * register_property_function("selected_activate_first_spacing", selected_activate_first_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_subtitle_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_first_spacing, __pyx_f_33style_selected_activate_functions_selected_activate_first_spacing_property);

  /* "style_selected_activate_functions.pyx":187
 *     return 0
 * 
 * register_property_function("selected_activate_subtitle_width", selected_activate_subtitle_width_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_subpixel_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_subtitle_width, __pyx_f_33style_selected_activate_functions_selected_activate_subtitle_width_property);

  /* "style_selected_activate_functions.pyx":194
 *     return 0
 * 
 * register_property_function("selected_activate_subpixel", selected_activate_subpixel_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_strikethrough_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_subpixel, __pyx_f_33style_selected_activate_functions_selected_activate_subpixel_property);

  /* "style_selected_activate_functions.pyx":201
 *     return 0
 * 
 * register_property_function("selected_activate_strikethrough", selected_activate_strikethrough_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_hyperlink_functions_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_strikethrough, __pyx_f_33style_selected_activate_functions_selected_activate_strikethrough_property);

  /* "style_selected_activate_functions.pyx":208
 *     return 0
 * 
 * register_property_function("selected_activate_hyperlink_functions", selected_activate_hyperlink_functions_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xminimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_hyperlink_func, __pyx_f_33style_selected_activate_functions_selected_activate_hyperlink_functions_property);

  /* "style_selected_activate_functions.pyx":215
 *     return 0
 * 
 * register_property_function("selected_activate_xminimum", selected_activate_xminimum_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_min_width_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xminimum, __pyx_f_33style_selected_activate_functions_selected_activate_xminimum_property);

  /* "style_selected_activate_functions.pyx":222
 *     return 0
 * 
 * register_property_function("selected_activate_min_width", selected_activate_min_width_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_focus_mask_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_min_width, __pyx_f_33style_selected_activate_functions_selected_activate_min_width_property);

  /* "style_selected_activate_functions.pyx":230
 *     return 0
 * 
 * register_property_function("selected_activate_focus_mask", selected_activate_focus_mask_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_focus_mask, __pyx_f_33style_selected_activate_functions_selected_activate_focus_mask_property);

  /* "style_selected_activate_functions.pyx":237
 *     return 0
 * 
 * register_property_function("selected_activate_yfit", selected_activate_yfit_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_drop_shadow_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yfit, __pyx_f_33style_selected_activate_functions_selected_activate_yfit_property);

  /* "style_selected_activate_functions.pyx":245
 *     return 0
 * 
 * register_property_function("selected_activate_drop_shadow_color", selected_activate_drop_shadow_color_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_fit_first_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_drop_shadow_co, __pyx_f_33style_selected_activate_functions_selected_activate_drop_shadow_color_property);

  /* "style_selected_activate_functions.pyx":252
 *     return 0
 * 
 * register_property_function("selected_activate_fit_first", selected_activate_fit_first_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_top_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_fit_first, __pyx_f_33style_selected_activate_functions_selected_activate_fit_first_property);

  /* "style_selected_activate_functions.pyx":259
 *     return 0
 * 
 * register_property_function("selected_activate_top_padding", selected_activate_top_padding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_newline_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_top_padding, __pyx_f_33style_selected_activate_functions_selected_activate_top_padding_property);

  /* "style_selected_activate_functions.pyx":266
 *     return 0
 * 
 * register_property_function("selected_activate_newline_indent", selected_activate_newline_indent_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_text_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_newline_indent, __pyx_f_33style_selected_activate_functions_selected_activate_newline_indent_property);

  /* "style_selected_activate_functions.pyx":273
 *     return 0
 * 
 * register_property_function("selected_activate_text_align", selected_activate_text_align_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_fore_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_text_align, __pyx_f_33style_selected_activate_functions_selected_activate_text_align_property);

  /* "style_selected_activate_functions.pyx":281
 *     return 0
 * 
 * register_property_function("selected_activate_fore_bar", selected_activate_fore_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_aft_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_fore_bar, __pyx_f_33style_selected_activate_functions_selected_activate_fore_bar_property);

  /* "style_selected_activate_functions.pyx":289
 *     return 0
 * 
 * register_property_function("selected_activate_aft_bar", selected_activate_aft_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_foreground_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_aft_bar, __pyx_f_33style_selected_activate_functions_selected_activate_aft_bar_property);

  /* "style_selected_activate_functions.pyx":297
 *     return 0
 * 
 * register_property_function("selected_activate_foreground", selected_activate_foreground_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bold_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_foreground, __pyx_f_33style_selected_activate_functions_selected_activate_foreground_property);

  /* "style_selected_activate_functions.pyx":304
 *     return 0
 * 
 * register_property_function("selected_activate_bold", selected_activate_bold_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bold, __pyx_f_33style_selected_activate_functions_selected_activate_bold_property);

  /* "style_selected_activate_functions.pyx":311
 *     return 0
 * 
 * register_property_function("selected_activate_vertical", selected_activate_vertical_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_drop_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_vertical, __pyx_f_33style_selected_activate_functions_selected_activate_vertical_property);

  /* "style_selected_activate_functions.pyx":318
 *     return 0
 * 
 * register_property_function("selected_activate_drop_shadow", selected_activate_drop_shadow_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_drop_shadow, __pyx_f_33style_selected_activate_functions_selected_activate_drop_shadow_property);

  /* "style_selected_activate_functions.pyx":325
 *     return 0
 * 
 * register_property_function("selected_activate_spacing", selected_activate_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ypos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_spacing, __pyx_f_33style_selected_activate_functions_selected_activate_spacing_property);

  /* "style_selected_activate_functions.pyx":332
 *     return 0
 * 
 * register_property_function("selected_activate_ypos", selected_activate_ypos_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_activate_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ypos, __pyx_f_33style_selected_activate_functions_selected_activate_ypos_property);

  /* "style_selected_activate_functions.pyx":339
 *     return 0
 * 
 * register_property_function("selected_activate_activate_sound", selected_activate_activate_sound_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_left_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_activate_sound, __pyx_f_33style_selected_activate_functions_selected_activate_activate_sound_property);

  /* "style_selected_activate_functions.pyx":346
 *     return 0
 * 
 * register_property_function("selected_activate_left_padding", selected_activate_left_padding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_left_padding, __pyx_f_33style_selected_activate_functions_selected_activate_left_padding_property);

  /* "style_selected_activate_functions.pyx":353
 *     return 0
 * 
 * register_property_function("selected_activate_xoffset", selected_activate_xoffset_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_language_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xoffset, __pyx_f_33style_selected_activate_functions_selected_activate_xoffset_property);

  /* "style_selected_activate_functions.pyx":360
 *     return 0
 * 
 * register_property_function("selected_activate_language", selected_activate_language_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_time_policy_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_language, __pyx_f_33style_selected_activate_functions_selected_activate_language_property);

  /* "style_selected_activate_functions.pyx":367
 *     return 0
 * 
 * register_property_function("selected_activate_time_policy", selected_activate_time_policy_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xmaximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_time_policy, __pyx_f_33style_selected_activate_functions_selected_activate_time_policy_property);

  /* "style_selected_activate_functions.pyx":374
 *     return 0
 * 
 * register_property_function("selected_activate_xmaximum", selected_activate_xmaximum_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_hover_sound_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xmaximum, __pyx_f_33style_selected_activate_functions_selected_activate_xmaximum_property);

  /* "style_selected_activate_functions.pyx":381
 *     return 0
 * 
 * register_property_function("selected_activate_hover_sound", selected_activate_hover_sound_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_hover_sound, __pyx_f_33style_selected_activate_functions_selected_activate_hover_sound_property);

  /* "style_selected_activate_functions.pyx":389
 *     return 0
 * 
 * register_property_function("selected_activate_yanchor", selected_activate_yanchor_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_fore_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yanchor, __pyx_f_33style_selected_activate_functions_selected_activate_yanchor_property);

  /* "style_selected_activate_functions.pyx":396
 *     return 0
 * 
 * register_property_function("selected_activate_fore_gutter", selected_activate_fore_gutter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_adjust_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_fore_gutter, __pyx_f_33style_selected_activate_functions_selected_activate_fore_gutter_property);

  /* "style_selected_activate_functions.pyx":403
 *     return 0
 * 
 * register_property_function("selected_activate_adjust_spacing", selected_activate_adjust_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_adjust_spacing, __pyx_f_33style_selected_activate_functions_selected_activate_adjust_spacing_property);

  /* "style_selected_activate_functions.pyx":410
 *     return 0
 * 
 * register_property_function("selected_activate_xfill", selected_activate_xfill_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xfill, __pyx_f_33style_selected_activate_functions_selected_activate_xfill_property);

  /* "style_selected_activate_functions.pyx":418
 *     return 0
 * 
 * register_property_function("selected_activate_color", selected_activate_color_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_key_events_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_color, __pyx_f_33style_selected_activate_functions_selected_activate_color_property);

  /* "style_selected_activate_functions.pyx":425
 *     return 0
 * 
 * register_property_function("selected_activate_key_events", selected_activate_key_events_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xspacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_key_events, __pyx_f_33style_selected_activate_functions_selected_activate_key_events_property);

  /* "style_selected_activate_functions.pyx":432
 *     return 0
 * 
 * register_property_function("selected_activate_xspacing", selected_activate_xspacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_size_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xspacing, __pyx_f_33style_selected_activate_functions_selected_activate_xspacing_property);

  /* "style_selected_activate_functions.pyx":439
 *     return 0
 * 
 * register_property_function("selected_activate_size", selected_activate_size_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_caret_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_size, __pyx_f_33style_selected_activate_functions_selected_activate_size_property);

  /* "style_selected_activate_functions.pyx":447
 *     return 0
 * 
 * register_property_function("selected_activate_caret", selected_activate_caret_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bottom_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_caret, __pyx_f_33style_selected_activate_functions_selected_activate_caret_property);

  /* "style_selected_activate_functions.pyx":454
 *     return 0
 * 
 * register_property_function("selected_activate_bottom_padding", selected_activate_bottom_padding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_box_wrap_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bottom_padding, __pyx_f_33style_selected_activate_functions_selected_activate_bottom_padding_property);

  /* "style_selected_activate_functions.pyx":461
 *     return 0
 * 
 * register_property_function("selected_activate_box_wrap", selected_activate_box_wrap_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_line_overlap_split_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_box_wrap, __pyx_f_33style_selected_activate_functions_selected_activate_box_wrap_property);

  /* "style_selected_activate_functions.pyx":468
 *     return 0
 * 
 * register_property_function("selected_activate_line_overlap_split", selected_activate_line_overlap_split_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_box_wrap_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_line_overlap_s, __pyx_f_33style_selected_activate_functions_selected_activate_line_overlap_split_property);

  /* "style_selected_activate_functions.pyx":475
 *     return 0
 * 
 * register_property_function("selected_activate_box_wrap_spacing", selected_activate_box_wrap_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_kerning_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_box_wrap_spaci, __pyx_f_33style_selected_activate_functions_selected_activate_box_wrap_spacing_property);

  /* "style_selected_activate_functions.pyx":482
 *     return 0
 * 
 * register_property_function("selected_activate_kerning", selected_activate_kerning_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_italic_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_kerning, __pyx_f_33style_selected_activate_functions_selected_activate_kerning_property);

  /* "style_selected_activate_functions.pyx":489
 *     return 0
 * 
 * register_property_function("selected_activate_italic", selected_activate_italic_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_italic, __pyx_f_33style_selected_activate_functions_selected_activate_italic_property);

  /* "style_selected_activate_functions.pyx":496
 *     return 0
 * 
 * register_property_function("selected_activate_ruby_style", selected_activate_ruby_style_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_box_layout_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ruby_style, __pyx_f_33style_selected_activate_functions_selected_activate_ruby_style_property);

  /* "style_selected_activate_functions.pyx":503
 *     return 0
 * 
 * register_property_function("selected_activate_box_layout", selected_activate_box_layout_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xpos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_box_layout, __pyx_f_33style_selected_activate_functions_selected_activate_box_layout_property);

  /* "style_selected_activate_functions.pyx":510
 *     return 0
 * 
 * register_property_function("selected_activate_xpos", selected_activate_xpos_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xanchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xpos, __pyx_f_33style_selected_activate_functions_selected_activate_xpos_property);

  /* "style_selected_activate_functions.pyx":518
 *     return 0
 * 
 * register_property_function("selected_activate_xanchor", selected_activate_xanchor_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_top_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xanchor, __pyx_f_33style_selected_activate_functions_selected_activate_xanchor_property);

  /* "style_selected_activate_functions.pyx":525
 *     return 0
 * 
 * register_property_function("selected_activate_top_margin", selected_activate_top_margin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yoffset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_top_margin, __pyx_f_33style_selected_activate_functions_selected_activate_top_margin_property);

  /* "style_selected_activate_functions.pyx":532
 *     return 0
 * 
 * register_property_function("selected_activate_yoffset", selected_activate_yoffset_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_child_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yoffset, __pyx_f_33style_selected_activate_functions_selected_activate_yoffset_property);

  /* "style_selected_activate_functions.pyx":540
 *     return 0
 * 
 * register_property_function("selected_activate_child", selected_activate_child_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_thumb_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_child, __pyx_f_33style_selected_activate_functions_selected_activate_child_property);

  /* "style_selected_activate_functions.pyx":547
 *     return 0
 * 
 * register_property_function("selected_activate_thumb_offset", selected_activate_thumb_offset_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xfit_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_thumb_offset, __pyx_f_33style_selected_activate_functions_selected_activate_thumb_offset_property);

  /* "style_selected_activate_functions.pyx":554
 *     return 0
 * 
 * register_property_function("selected_activate_xfit", selected_activate_xfit_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_hinting_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xfit, __pyx_f_33style_selected_activate_functions_selected_activate_xfit_property);

  /* "style_selected_activate_functions.pyx":561
 *     return 0
 * 
 * register_property_function("selected_activate_hinting", selected_activate_hinting_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_altruby_style_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_hinting, __pyx_f_33style_selected_activate_functions_selected_activate_hinting_property);

  /* "style_selected_activate_functions.pyx":568
 *     return 0
 * 
 * register_property_function("selected_activate_altruby_style", selected_activate_altruby_style_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_unscrollable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_altruby_style, __pyx_f_33style_selected_activate_functions_selected_activate_altruby_style_property);

  /* "style_selected_activate_functions.pyx":575
 *     return 0
 * 
 * register_property_function("selected_activate_unscrollable", selected_activate_unscrollable_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_order_reverse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_unscrollable, __pyx_f_33style_selected_activate_functions_selected_activate_unscrollable_property);

  /* "style_selected_activate_functions.pyx":582
 *     return 0
 * 
 * register_property_function("selected_activate_order_reverse", selected_activate_order_reverse_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_mouse_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_order_reverse, __pyx_f_33style_selected_activate_functions_selected_activate_order_reverse_property);

  /* "style_selected_activate_functions.pyx":589
 *     return 0
 * 
 * register_property_function("selected_activate_mouse", selected_activate_mouse_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_keyboard_focus_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_mouse, __pyx_f_33style_selected_activate_functions_selected_activate_mouse_property);

  /* "style_selected_activate_functions.pyx":596
 *     return 0
 * 
 * register_property_function("selected_activate_keyboard_focus", selected_activate_keyboard_focus_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_first_indent_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_keyboard_focus, __pyx_f_33style_selected_activate_functions_selected_activate_keyboard_focus_property);

  /* "style_selected_activate_functions.pyx":603
 *     return 0
 * 
 * register_property_function("selected_activate_first_indent", selected_activate_first_indent_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bar_vertical_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_first_indent, __pyx_f_33style_selected_activate_functions_selected_activate_first_indent_property);

  /* "style_selected_activate_functions.pyx":610
 *     return 0
 * 
 * register_property_function("selected_activate_bar_vertical", selected_activate_bar_vertical_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_right_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bar_vertical, __pyx_f_33style_selected_activate_functions_selected_activate_bar_vertical_property);

  /* "style_selected_activate_functions.pyx":617
 *     return 0
 * 
 * register_property_function("selected_activate_right_padding", selected_activate_right_padding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_focus_rect_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_right_padding, __pyx_f_33style_selected_activate_functions_selected_activate_right_padding_property);

  /* "style_selected_activate_functions.pyx":624
 *     return 0
 * 
 * register_property_function("selected_activate_focus_rect", selected_activate_focus_rect_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_antialias_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_focus_rect, __pyx_f_33style_selected_activate_functions_selected_activate_focus_rect_property);

  /* "style_selected_activate_functions.pyx":631
 *     return 0
 * 
 * register_property_function("selected_activate_antialias", selected_activate_antialias_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_slow_cps_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_antialias, __pyx_f_33style_selected_activate_functions_selected_activate_antialias_property);

  /* "style_selected_activate_functions.pyx":638
 *     return 0
 * 
 * register_property_function("selected_activate_slow_cps", selected_activate_slow_cps_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bar_invert_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_slow_cps, __pyx_f_33style_selected_activate_functions_selected_activate_slow_cps_property);

  /* "style_selected_activate_functions.pyx":645
 *     return 0
 * 
 * register_property_function("selected_activate_bar_invert", selected_activate_bar_invert_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_line_leading_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bar_invert, __pyx_f_33style_selected_activate_functions_selected_activate_bar_invert_property);

  /* "style_selected_activate_functions.pyx":652
 *     return 0
 * 
 * register_property_function("selected_activate_line_leading", selected_activate_line_leading_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_slow_cps_multiplier_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_line_leading, __pyx_f_33style_selected_activate_functions_selected_activate_line_leading_property);

  /* "style_selected_activate_functions.pyx":659
 *     return 0
 * 
 * register_property_function("selected_activate_slow_cps_multiplier", selected_activate_slow_cps_multiplier_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_thumb_shadow_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_slow_cps_multi, __pyx_f_33style_selected_activate_functions_selected_activate_slow_cps_multiplier_property);

  /* "style_selected_activate_functions.pyx":667
 *     return 0
 * 
 * register_property_function("selected_activate_thumb_shadow", selected_activate_thumb_shadow_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yfill_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_thumb_shadow, __pyx_f_33style_selected_activate_functions_selected_activate_thumb_shadow_property);

  /* "style_selected_activate_functions.pyx":674
 *     return 0
 * 
 * register_property_function("selected_activate_yfill", selected_activate_yfill_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_slow_abortable_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yfill, __pyx_f_33style_selected_activate_functions_selected_activate_yfill_property);

  /* "style_selected_activate_functions.pyx":681
 *     return 0
 * 
 * register_property_function("selected_activate_slow_abortable", selected_activate_slow_abortable_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_justify_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_slow_abortable, __pyx_f_33style_selected_activate_functions_selected_activate_slow_abortable_property);

  /* "style_selected_activate_functions.pyx":688
 *     return 0
 * 
 * register_property_function("selected_activate_justify", selected_activate_justify_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_left_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_justify, __pyx_f_33style_selected_activate_functions_selected_activate_justify_property);

  /* "style_selected_activate_functions.pyx":695
 *     return 0
 * 
 * register_property_function("selected_activate_left_gutter", selected_activate_left_gutter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bottom_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_left_gutter, __pyx_f_33style_selected_activate_functions_selected_activate_left_gutter_property);

  /* "style_selected_activate_functions.pyx":702
 *     return 0
 * 
 * register_property_function("selected_activate_bottom_gutter", selected_activate_bottom_gutter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_offset_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bottom_gutter, __pyx_f_33style_selected_activate_functions_selected_activate_bottom_gutter_property);

  /* "style_selected_activate_functions.pyx":712
 *     return 0
 * 
 * register_property_function("selected_activate_offset", selected_activate_offset_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xcenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_offset, __pyx_f_33style_selected_activate_functions_selected_activate_offset_property);

  /* "style_selected_activate_functions.pyx":722
 *     return 0
 * 
 * register_property_function("selected_activate_xcenter", selected_activate_xcenter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_pos_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xcenter, __pyx_f_33style_selected_activate_functions_selected_activate_xcenter_property);

  /* "style_selected_activate_functions.pyx":732
 *     return 0
 * 
 * register_property_function("selected_activate_pos", selected_activate_pos_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_minwidth_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_pos, __pyx_f_33style_selected_activate_functions_selected_activate_pos_property);

  /* "style_selected_activate_functions.pyx":739
 *     return 0
 * 
 * register_property_function("selected_activate_minwidth", selected_activate_minwidth_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_minimum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_minwidth, __pyx_f_33style_selected_activate_functions_selected_activate_minwidth_property);

  /* "style_selected_activate_functions.pyx":749
 *     return 0
 * 
 * register_property_function("selected_activate_minimum", selected_activate_minimum_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_top_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_minimum, __pyx_f_33style_selected_activate_functions_selected_activate_minimum_property);

  /* "style_selected_activate_functions.pyx":757
 *     return 0
 * 
 * register_property_function("selected_activate_top_bar", selected_activate_top_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_base_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_top_bar, __pyx_f_33style_selected_activate_functions_selected_activate_top_bar_property);

  /* "style_selected_activate_functions.pyx":767
 *     return 0
 * 
 * register_property_function("selected_activate_base_bar", selected_activate_base_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_base_bar, __pyx_f_33style_selected_activate_functions_selected_activate_base_bar_property);

  /* "style_selected_activate_functions.pyx":781
 *     return 0
 * 
 * register_property_function("selected_activate_xysize", selected_activate_xysize_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_slow_speed_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xysize, __pyx_f_33style_selected_activate_functions_selected_activate_xysize_property);

  /* "style_selected_activate_functions.pyx":788
 *     return 0
 * 
 * register_property_function("selected_activate_slow_speed", selected_activate_slow_speed_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_area_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_slow_speed, __pyx_f_33style_selected_activate_functions_selected_activate_slow_speed_property);

  /* "style_selected_activate_functions.pyx":816
 *     return 0
 * 
 * register_property_function("selected_activate_area", selected_activate_area_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_right_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_area, __pyx_f_33style_selected_activate_functions_selected_activate_area_property);

  /* "style_selected_activate_functions.pyx":824
 *     return 0
 * 
 * register_property_function("selected_activate_right_bar", selected_activate_right_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_enable_hover_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_right_bar, __pyx_f_33style_selected_activate_functions_selected_activate_right_bar_property);

  /* "style_selected_activate_functions.pyx":830
 *     return 0
 * 
 * register_property_function("selected_activate_enable_hover", selected_activate_enable_hover_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_textalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_enable_hover, __pyx_f_33style_selected_activate_functions_selected_activate_enable_hover_property);

  /* "style_selected_activate_functions.pyx":837
 *     return 0
 * 
 * register_property_function("selected_activate_textalign", selected_activate_textalign_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_box_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_textalign, __pyx_f_33style_selected_activate_functions_selected_activate_textalign_property);

  /* "style_selected_activate_functions.pyx":844
 *     return 0
 * 
 * register_property_function("selected_activate_box_spacing", selected_activate_box_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_padding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_box_spacing, __pyx_f_33style_selected_activate_functions_selected_activate_box_spacing_property);

  /* "style_selected_activate_functions.pyx":858
 *     return 0
 * 
 * register_property_function("selected_activate_padding", selected_activate_padding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_box_first_spacing_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_padding, __pyx_f_33style_selected_activate_functions_selected_activate_padding_property);

  /* "style_selected_activate_functions.pyx":865
 *     return 0
 * 
 * register_property_function("selected_activate_box_first_spacing", selected_activate_box_first_spacing_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_right_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_box_first_spac, __pyx_f_33style_selected_activate_functions_selected_activate_box_first_spacing_property);

  /* "style_selected_activate_functions.pyx":872
 *     return 0
 * 
 * register_property_function("selected_activate_right_gutter", selected_activate_right_gutter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_bottom_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_right_gutter, __pyx_f_33style_selected_activate_functions_selected_activate_right_gutter_property);

  /* "style_selected_activate_functions.pyx":880
 *     return 0
 * 
 * register_property_function("selected_activate_bottom_bar", selected_activate_bottom_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xsize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_bottom_bar, __pyx_f_33style_selected_activate_functions_selected_activate_bottom_bar_property);

  /* "style_selected_activate_functions.pyx":888
 *     return 0
 * 
 * register_property_function("selected_activate_xsize", selected_activate_xsize_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xpadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xsize, __pyx_f_33style_selected_activate_functions_selected_activate_xsize_property);

  /* "style_selected_activate_functions.pyx":896
 *     return 0
 * 
 * register_property_function("selected_activate_xpadding", selected_activate_xpadding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_top_gutter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xpadding, __pyx_f_33style_selected_activate_functions_selected_activate_xpadding_property);

  /* "style_selected_activate_functions.pyx":903
 *     return 0
 * 
 * register_property_function("selected_activate_top_gutter", selected_activate_top_gutter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_top_gutter, __pyx_f_33style_selected_activate_functions_selected_activate_top_gutter_property);

  /* "style_selected_activate_functions.pyx":912
 *     return 0
 * 
 * register_property_function("selected_activate_xalign", selected_activate_xalign_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_xmargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xalign, __pyx_f_33style_selected_activate_functions_selected_activate_xalign_property);

  /* "style_selected_activate_functions.pyx":920
 *     return 0
 * 
 * register_property_function("selected_activate_xmargin", selected_activate_xmargin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_yalign_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_xmargin, __pyx_f_33style_selected_activate_functions_selected_activate_xmargin_property);

  /* "style_selected_activate_functions.pyx":929
 *     return 0
 * 
 * register_property_function("selected_activate_yalign", selected_activate_yalign_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ypadding_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_yalign, __pyx_f_33style_selected_activate_functions_selected_activate_yalign_property);

  /* "style_selected_activate_functions.pyx":937
 *     return 0
 * 
 * register_property_function("selected_activate_ypadding", selected_activate_ypadding_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_align_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ypadding, __pyx_f_33style_selected_activate_functions_selected_activate_ypadding_property);

  /* "style_selected_activate_functions.pyx":953
 *     return 0
 * 
 * register_property_function("selected_activate_align", selected_activate_align_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ysize_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_align, __pyx_f_33style_selected_activate_functions_selected_activate_align_property);

  /* "style_selected_activate_functions.pyx":961
 *     return 0
 * 
 * register_property_function("selected_activate_ysize", selected_activate_ysize_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_maximum_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ysize, __pyx_f_33style_selected_activate_functions_selected_activate_ysize_property);

  /* "style_selected_activate_functions.pyx":971
 *     return 0
 * 
 * register_property_function("selected_activate_maximum", selected_activate_maximum_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ycenter_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_maximum, __pyx_f_33style_selected_activate_functions_selected_activate_maximum_property);

  /* "style_selected_activate_functions.pyx":981
 *     return 0
 * 
 * register_property_function("selected_activate_ycenter", selected_activate_ycenter_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_left_bar_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ycenter, __pyx_f_33style_selected_activate_functions_selected_activate_ycenter_property);

  /* "style_selected_activate_functions.pyx":989
 *     return 0
 * 
 * register_property_function("selected_activate_left_bar", selected_activate_left_bar_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_ymargin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_left_bar, __pyx_f_33style_selected_activate_functions_selected_activate_left_bar_property);

  /* "style_selected_activate_functions.pyx":997
 *     return 0
 * 
 * register_property_function("selected_activate_ymargin", selected_activate_ymargin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_margin_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_ymargin, __pyx_f_33style_selected_activate_functions_selected_activate_ymargin_property);

  /* "style_selected_activate_functions.pyx":1011
 *     return 0
 * 
 * register_property_function("selected_activate_margin", selected_activate_margin_property)             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_anchor_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_margin, __pyx_f_33style_selected_activate_functions_selected_activate_margin_property);

  /* "style_selected_activate_functions.pyx":1023
 *     return 0
 * 
 * register_property_function("selected_activate_anchor", selected_activate_anchor_property)             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_f_5renpy_5style_register_property_function(__pyx_n_s_selected_activate_anchor, __pyx_f_33style_selected_activate_functions_selected_activate_anchor_property);

  /* "style_selected_activate_functions.pyx":1
 * include "style_common.pxi"             # <<<<<<<<<<<<<<
 * 
 * cdef int selected_activate_black_color_property(PyObject **cache, int *cache_priorities, int priority, object value) except -1:
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/style.pxd":56
 *     cache_priorities[index] = priority
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):             # <<<<<<<<<<<<<<
 *     """
 *     Like assign, but if the value can be duplicated, duplicates it and assigns
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init style_selected_activate_functions", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init style_selected_activate_functions");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyErrFetchRestore */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
    static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* GetVTable */
    static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
    #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* FunctionImport */
    #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
