/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__gl2__uguugl
#define __PYX_HAVE_API__renpy__gl2__uguugl
#include <stdint.h>
#include <stddef.h>
#include "renpygl.h"
#include <string.h>
#include <stdio.h>
#include "SDL.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "uguugl.pyx",
};

/*--- Type declarations ---*/

/* "renpy/gl2/uguugl.pxd":324
 *     GLenum GL_RENDERBUFFER_STENCIL_SIZE
 * 
 * ctypedef void (*glActiveTexture_type)(GLenum  texture) nogil             # <<<<<<<<<<<<<<
 * cdef glActiveTexture_type glActiveTexture
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":327
 * cdef glActiveTexture_type glActiveTexture
 * 
 * ctypedef void (*glAttachShader_type)(GLuint  program, GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glAttachShader_type glAttachShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type)(GLuint, GLuint);

/* "renpy/gl2/uguugl.pxd":330
 * cdef glAttachShader_type glAttachShader
 * 
 * ctypedef void (*glBindAttribLocation_type)(GLuint  program, GLuint  index, const GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glBindAttribLocation_type glBindAttribLocation
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type)(GLuint, GLuint, GLchar const *);

/* "renpy/gl2/uguugl.pxd":333
 * cdef glBindAttribLocation_type glBindAttribLocation
 * 
 * ctypedef void (*glBindBuffer_type)(GLenum  target, GLuint  buffer) nogil             # <<<<<<<<<<<<<<
 * cdef glBindBuffer_type glBindBuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type)(GLenum, GLuint);

/* "renpy/gl2/uguugl.pxd":336
 * cdef glBindBuffer_type glBindBuffer
 * 
 * ctypedef void (*glBindFramebuffer_type)(GLenum  target, GLuint  framebuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glBindFramebuffer_type glBindFramebuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type)(GLenum, GLuint);

/* "renpy/gl2/uguugl.pxd":339
 * cdef glBindFramebuffer_type glBindFramebuffer
 * 
 * ctypedef void (*glBindRenderbuffer_type)(GLenum  target, GLuint  renderbuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glBindRenderbuffer_type glBindRenderbuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type)(GLenum, GLuint);

/* "renpy/gl2/uguugl.pxd":342
 * cdef glBindRenderbuffer_type glBindRenderbuffer
 * 
 * ctypedef void (*glBindTexture_type)(GLenum  target, GLuint  texture) nogil             # <<<<<<<<<<<<<<
 * cdef glBindTexture_type glBindTexture
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type)(GLenum, GLuint);

/* "renpy/gl2/uguugl.pxd":345
 * cdef glBindTexture_type glBindTexture
 * 
 * ctypedef void (*glBlendColor_type)(GLfloat  red, GLfloat  green, GLfloat  blue, GLfloat  alpha) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendColor_type glBlendColor
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type)(GLfloat, GLfloat, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":348
 * cdef glBlendColor_type glBlendColor
 * 
 * ctypedef void (*glBlendEquation_type)(GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendEquation_type glBlendEquation
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":351
 * cdef glBlendEquation_type glBlendEquation
 * 
 * ctypedef void (*glBlendEquationSeparate_type)(GLenum  modeRGB, GLenum  modeAlpha) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendEquationSeparate_type glBlendEquationSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type)(GLenum, GLenum);

/* "renpy/gl2/uguugl.pxd":354
 * cdef glBlendEquationSeparate_type glBlendEquationSeparate
 * 
 * ctypedef void (*glBlendFunc_type)(GLenum  sfactor, GLenum  dfactor) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendFunc_type glBlendFunc
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type)(GLenum, GLenum);

/* "renpy/gl2/uguugl.pxd":357
 * cdef glBlendFunc_type glBlendFunc
 * 
 * ctypedef void (*glBlendFuncSeparate_type)(GLenum  sfactorRGB, GLenum  dfactorRGB, GLenum  sfactorAlpha, GLenum  dfactorAlpha) nogil             # <<<<<<<<<<<<<<
 * cdef glBlendFuncSeparate_type glBlendFuncSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type)(GLenum, GLenum, GLenum, GLenum);

/* "renpy/gl2/uguugl.pxd":360
 * cdef glBlendFuncSeparate_type glBlendFuncSeparate
 * 
 * ctypedef void (*glBufferData_type)(GLenum  target, GLsizeiptr  size, const void * data, GLenum  usage) nogil             # <<<<<<<<<<<<<<
 * cdef glBufferData_type glBufferData
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBufferData_type)(GLenum, GLsizeiptr, void const *, GLenum);

/* "renpy/gl2/uguugl.pxd":363
 * cdef glBufferData_type glBufferData
 * 
 * ctypedef void (*glBufferSubData_type)(GLenum  target, GLintptr  offset, GLsizeiptr  size, const void * data) nogil             # <<<<<<<<<<<<<<
 * cdef glBufferSubData_type glBufferSubData
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type)(GLenum, GLintptr, GLsizeiptr, void const *);

/* "renpy/gl2/uguugl.pxd":366
 * cdef glBufferSubData_type glBufferSubData
 * 
 * ctypedef GLenum (*glCheckFramebufferStatus_type)(GLenum  target) nogil             # <<<<<<<<<<<<<<
 * cdef glCheckFramebufferStatus_type glCheckFramebufferStatus
 * 
 */
typedef GLenum (*__pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":369
 * cdef glCheckFramebufferStatus_type glCheckFramebufferStatus
 * 
 * ctypedef void (*glClear_type)(GLbitfield  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glClear_type glClear
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glClear_type)(GLbitfield);

/* "renpy/gl2/uguugl.pxd":372
 * cdef glClear_type glClear
 * 
 * ctypedef void (*glClearColor_type)(GLfloat  red, GLfloat  green, GLfloat  blue, GLfloat  alpha) nogil             # <<<<<<<<<<<<<<
 * cdef glClearColor_type glClearColor
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glClearColor_type)(GLfloat, GLfloat, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":375
 * cdef glClearColor_type glClearColor
 * 
 * ctypedef void (*glClearStencil_type)(GLint  s) nogil             # <<<<<<<<<<<<<<
 * cdef glClearStencil_type glClearStencil
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type)(GLint);

/* "renpy/gl2/uguugl.pxd":378
 * cdef glClearStencil_type glClearStencil
 * 
 * ctypedef void (*glColorMask_type)(GLboolean  red, GLboolean  green, GLboolean  blue, GLboolean  alpha) nogil             # <<<<<<<<<<<<<<
 * cdef glColorMask_type glColorMask
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glColorMask_type)(GLboolean, GLboolean, GLboolean, GLboolean);

/* "renpy/gl2/uguugl.pxd":381
 * cdef glColorMask_type glColorMask
 * 
 * ctypedef void (*glCompileShader_type)(GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glCompileShader_type glCompileShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":384
 * cdef glCompileShader_type glCompileShader
 * 
 * ctypedef void (*glCompressedTexImage2D_type)(GLenum  target, GLint  level, GLenum  internalformat, GLsizei  width, GLsizei  height, GLint  border, GLsizei  imageSize, const void * data) nogil             # <<<<<<<<<<<<<<
 * cdef glCompressedTexImage2D_type glCompressedTexImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type)(GLenum, GLint, GLenum, GLsizei, GLsizei, GLint, GLsizei, void const *);

/* "renpy/gl2/uguugl.pxd":387
 * cdef glCompressedTexImage2D_type glCompressedTexImage2D
 * 
 * ctypedef void (*glCompressedTexSubImage2D_type)(GLenum  target, GLint  level, GLint  xoffset, GLint  yoffset, GLsizei  width, GLsizei  height, GLenum  format, GLsizei  imageSize, const void * data) nogil             # <<<<<<<<<<<<<<
 * cdef glCompressedTexSubImage2D_type glCompressedTexSubImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLsizei, void const *);

/* "renpy/gl2/uguugl.pxd":390
 * cdef glCompressedTexSubImage2D_type glCompressedTexSubImage2D
 * 
 * ctypedef void (*glCopyTexImage2D_type)(GLenum  target, GLint  level, GLenum  internalformat, GLint  x, GLint  y, GLsizei  width, GLsizei  height, GLint  border) nogil             # <<<<<<<<<<<<<<
 * cdef glCopyTexImage2D_type glCopyTexImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type)(GLenum, GLint, GLenum, GLint, GLint, GLsizei, GLsizei, GLint);

/* "renpy/gl2/uguugl.pxd":393
 * cdef glCopyTexImage2D_type glCopyTexImage2D
 * 
 * ctypedef void (*glCopyTexSubImage2D_type)(GLenum  target, GLint  level, GLint  xoffset, GLint  yoffset, GLint  x, GLint  y, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glCopyTexSubImage2D_type glCopyTexSubImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type)(GLenum, GLint, GLint, GLint, GLint, GLint, GLsizei, GLsizei);

/* "renpy/gl2/uguugl.pxd":396
 * cdef glCopyTexSubImage2D_type glCopyTexSubImage2D
 * 
 * ctypedef GLuint (*glCreateProgram_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glCreateProgram_type glCreateProgram
 * 
 */
typedef GLuint (*__pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type)(void);

/* "renpy/gl2/uguugl.pxd":399
 * cdef glCreateProgram_type glCreateProgram
 * 
 * ctypedef GLuint (*glCreateShader_type)(GLenum  type) nogil             # <<<<<<<<<<<<<<
 * cdef glCreateShader_type glCreateShader
 * 
 */
typedef GLuint (*__pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":402
 * cdef glCreateShader_type glCreateShader
 * 
 * ctypedef void (*glCullFace_type)(GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glCullFace_type glCullFace
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glCullFace_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":405
 * cdef glCullFace_type glCullFace
 * 
 * ctypedef void (*glDeleteBuffers_type)(GLsizei  n, const GLuint * buffers) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteBuffers_type glDeleteBuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type)(GLsizei, GLuint const *);

/* "renpy/gl2/uguugl.pxd":408
 * cdef glDeleteBuffers_type glDeleteBuffers
 * 
 * ctypedef void (*glDeleteFramebuffers_type)(GLsizei  n, const GLuint * framebuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteFramebuffers_type glDeleteFramebuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type)(GLsizei, GLuint const *);

/* "renpy/gl2/uguugl.pxd":411
 * cdef glDeleteFramebuffers_type glDeleteFramebuffers
 * 
 * ctypedef void (*glDeleteProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteProgram_type glDeleteProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":414
 * cdef glDeleteProgram_type glDeleteProgram
 * 
 * ctypedef void (*glDeleteRenderbuffers_type)(GLsizei  n, const GLuint * renderbuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteRenderbuffers_type glDeleteRenderbuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type)(GLsizei, GLuint const *);

/* "renpy/gl2/uguugl.pxd":417
 * cdef glDeleteRenderbuffers_type glDeleteRenderbuffers
 * 
 * ctypedef void (*glDeleteShader_type)(GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteShader_type glDeleteShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":420
 * cdef glDeleteShader_type glDeleteShader
 * 
 * ctypedef void (*glDeleteTextures_type)(GLsizei  n, const GLuint * textures) nogil             # <<<<<<<<<<<<<<
 * cdef glDeleteTextures_type glDeleteTextures
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type)(GLsizei, GLuint const *);

/* "renpy/gl2/uguugl.pxd":423
 * cdef glDeleteTextures_type glDeleteTextures
 * 
 * ctypedef void (*glDepthFunc_type)(GLenum  func) nogil             # <<<<<<<<<<<<<<
 * cdef glDepthFunc_type glDepthFunc
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":426
 * cdef glDepthFunc_type glDepthFunc
 * 
 * ctypedef void (*glDepthMask_type)(GLboolean  flag) nogil             # <<<<<<<<<<<<<<
 * cdef glDepthMask_type glDepthMask
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type)(GLboolean);

/* "renpy/gl2/uguugl.pxd":429
 * cdef glDepthMask_type glDepthMask
 * 
 * ctypedef void (*glDetachShader_type)(GLuint  program, GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glDetachShader_type glDetachShader
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type)(GLuint, GLuint);

/* "renpy/gl2/uguugl.pxd":432
 * cdef glDetachShader_type glDetachShader
 * 
 * ctypedef void (*glDisable_type)(GLenum  cap) nogil             # <<<<<<<<<<<<<<
 * cdef glDisable_type glDisable
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDisable_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":435
 * cdef glDisable_type glDisable
 * 
 * ctypedef void (*glDisableVertexAttribArray_type)(GLuint  index) nogil             # <<<<<<<<<<<<<<
 * cdef glDisableVertexAttribArray_type glDisableVertexAttribArray
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":438
 * cdef glDisableVertexAttribArray_type glDisableVertexAttribArray
 * 
 * ctypedef void (*glDrawArrays_type)(GLenum  mode, GLint  first, GLsizei  count) nogil             # <<<<<<<<<<<<<<
 * cdef glDrawArrays_type glDrawArrays
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type)(GLenum, GLint, GLsizei);

/* "renpy/gl2/uguugl.pxd":441
 * cdef glDrawArrays_type glDrawArrays
 * 
 * ctypedef void (*glDrawElements_type)(GLenum  mode, GLsizei  count, GLenum  type, const void * indices) nogil             # <<<<<<<<<<<<<<
 * cdef glDrawElements_type glDrawElements
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type)(GLenum, GLsizei, GLenum, void const *);

/* "renpy/gl2/uguugl.pxd":444
 * cdef glDrawElements_type glDrawElements
 * 
 * ctypedef void (*glEnable_type)(GLenum  cap) nogil             # <<<<<<<<<<<<<<
 * cdef glEnable_type glEnable
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glEnable_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":447
 * cdef glEnable_type glEnable
 * 
 * ctypedef void (*glEnableVertexAttribArray_type)(GLuint  index) nogil             # <<<<<<<<<<<<<<
 * cdef glEnableVertexAttribArray_type glEnableVertexAttribArray
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":450
 * cdef glEnableVertexAttribArray_type glEnableVertexAttribArray
 * 
 * ctypedef void (*glFinish_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glFinish_type glFinish
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFinish_type)(void);

/* "renpy/gl2/uguugl.pxd":453
 * cdef glFinish_type glFinish
 * 
 * ctypedef void (*glFlush_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glFlush_type glFlush
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFlush_type)(void);

/* "renpy/gl2/uguugl.pxd":456
 * cdef glFlush_type glFlush
 * 
 * ctypedef void (*glFramebufferRenderbuffer_type)(GLenum  target, GLenum  attachment, GLenum  renderbuffertarget, GLuint  renderbuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glFramebufferRenderbuffer_type glFramebufferRenderbuffer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type)(GLenum, GLenum, GLenum, GLuint);

/* "renpy/gl2/uguugl.pxd":459
 * cdef glFramebufferRenderbuffer_type glFramebufferRenderbuffer
 * 
 * ctypedef void (*glFramebufferTexture2D_type)(GLenum  target, GLenum  attachment, GLenum  textarget, GLuint  texture, GLint  level) nogil             # <<<<<<<<<<<<<<
 * cdef glFramebufferTexture2D_type glFramebufferTexture2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type)(GLenum, GLenum, GLenum, GLuint, GLint);

/* "renpy/gl2/uguugl.pxd":462
 * cdef glFramebufferTexture2D_type glFramebufferTexture2D
 * 
 * ctypedef void (*glFrontFace_type)(GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glFrontFace_type glFrontFace
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":465
 * cdef glFrontFace_type glFrontFace
 * 
 * ctypedef void (*glGenBuffers_type)(GLsizei  n, GLuint * buffers) nogil             # <<<<<<<<<<<<<<
 * cdef glGenBuffers_type glGenBuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type)(GLsizei, GLuint *);

/* "renpy/gl2/uguugl.pxd":468
 * cdef glGenBuffers_type glGenBuffers
 * 
 * ctypedef void (*glGenFramebuffers_type)(GLsizei  n, GLuint * framebuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glGenFramebuffers_type glGenFramebuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type)(GLsizei, GLuint *);

/* "renpy/gl2/uguugl.pxd":471
 * cdef glGenFramebuffers_type glGenFramebuffers
 * 
 * ctypedef void (*glGenRenderbuffers_type)(GLsizei  n, GLuint * renderbuffers) nogil             # <<<<<<<<<<<<<<
 * cdef glGenRenderbuffers_type glGenRenderbuffers
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type)(GLsizei, GLuint *);

/* "renpy/gl2/uguugl.pxd":474
 * cdef glGenRenderbuffers_type glGenRenderbuffers
 * 
 * ctypedef void (*glGenTextures_type)(GLsizei  n, GLuint * textures) nogil             # <<<<<<<<<<<<<<
 * cdef glGenTextures_type glGenTextures
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type)(GLsizei, GLuint *);

/* "renpy/gl2/uguugl.pxd":477
 * cdef glGenTextures_type glGenTextures
 * 
 * ctypedef void (*glGenerateMipmap_type)(GLenum  target) nogil             # <<<<<<<<<<<<<<
 * cdef glGenerateMipmap_type glGenerateMipmap
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":480
 * cdef glGenerateMipmap_type glGenerateMipmap
 * 
 * ctypedef void (*glGetActiveAttrib_type)(GLuint  program, GLuint  index, GLsizei  bufSize, GLsizei * length, GLint * size, GLenum * type, GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetActiveAttrib_type glGetActiveAttrib
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type)(GLuint, GLuint, GLsizei, GLsizei *, GLint *, GLenum *, GLchar *);

/* "renpy/gl2/uguugl.pxd":483
 * cdef glGetActiveAttrib_type glGetActiveAttrib
 * 
 * ctypedef void (*glGetActiveUniform_type)(GLuint  program, GLuint  index, GLsizei  bufSize, GLsizei * length, GLint * size, GLenum * type, GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetActiveUniform_type glGetActiveUniform
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type)(GLuint, GLuint, GLsizei, GLsizei *, GLint *, GLenum *, GLchar *);

/* "renpy/gl2/uguugl.pxd":486
 * cdef glGetActiveUniform_type glGetActiveUniform
 * 
 * ctypedef void (*glGetAttachedShaders_type)(GLuint  program, GLsizei  maxCount, GLsizei * count, GLuint * shaders) nogil             # <<<<<<<<<<<<<<
 * cdef glGetAttachedShaders_type glGetAttachedShaders
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type)(GLuint, GLsizei, GLsizei *, GLuint *);

/* "renpy/gl2/uguugl.pxd":489
 * cdef glGetAttachedShaders_type glGetAttachedShaders
 * 
 * ctypedef GLint (*glGetAttribLocation_type)(GLuint  program, const GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetAttribLocation_type glGetAttribLocation
 * 
 */
typedef GLint (*__pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type)(GLuint, GLchar const *);

/* "renpy/gl2/uguugl.pxd":492
 * cdef glGetAttribLocation_type glGetAttribLocation
 * 
 * ctypedef void (*glGetBooleanv_type)(GLenum  pname, GLboolean * data) nogil             # <<<<<<<<<<<<<<
 * cdef glGetBooleanv_type glGetBooleanv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type)(GLenum, GLboolean *);

/* "renpy/gl2/uguugl.pxd":495
 * cdef glGetBooleanv_type glGetBooleanv
 * 
 * ctypedef void (*glGetBufferParameteriv_type)(GLenum  target, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetBufferParameteriv_type glGetBufferParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type)(GLenum, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":498
 * cdef glGetBufferParameteriv_type glGetBufferParameteriv
 * 
 * ctypedef GLenum (*glGetError_type)() nogil             # <<<<<<<<<<<<<<
 * cdef glGetError_type glGetError
 * 
 */
typedef GLenum (*__pyx_t_5renpy_3gl2_6uguugl_glGetError_type)(void);

/* "renpy/gl2/uguugl.pxd":501
 * cdef glGetError_type glGetError
 * 
 * ctypedef void (*glGetFloatv_type)(GLenum  pname, GLfloat * data) nogil             # <<<<<<<<<<<<<<
 * cdef glGetFloatv_type glGetFloatv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type)(GLenum, GLfloat *);

/* "renpy/gl2/uguugl.pxd":504
 * cdef glGetFloatv_type glGetFloatv
 * 
 * ctypedef void (*glGetFramebufferAttachmentParameteriv_type)(GLenum  target, GLenum  attachment, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetFramebufferAttachmentParameteriv_type glGetFramebufferAttachmentParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type)(GLenum, GLenum, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":507
 * cdef glGetFramebufferAttachmentParameteriv_type glGetFramebufferAttachmentParameteriv
 * 
 * ctypedef void (*glGetIntegerv_type)(GLenum  pname, GLint * data) nogil             # <<<<<<<<<<<<<<
 * cdef glGetIntegerv_type glGetIntegerv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type)(GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":510
 * cdef glGetIntegerv_type glGetIntegerv
 * 
 * ctypedef void (*glGetProgramInfoLog_type)(GLuint  program, GLsizei  bufSize, GLsizei * length, GLchar * infoLog) nogil             # <<<<<<<<<<<<<<
 * cdef glGetProgramInfoLog_type glGetProgramInfoLog
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type)(GLuint, GLsizei, GLsizei *, GLchar *);

/* "renpy/gl2/uguugl.pxd":513
 * cdef glGetProgramInfoLog_type glGetProgramInfoLog
 * 
 * ctypedef void (*glGetProgramiv_type)(GLuint  program, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetProgramiv_type glGetProgramiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type)(GLuint, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":516
 * cdef glGetProgramiv_type glGetProgramiv
 * 
 * ctypedef void (*glGetRenderbufferParameteriv_type)(GLenum  target, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetRenderbufferParameteriv_type glGetRenderbufferParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type)(GLenum, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":519
 * cdef glGetRenderbufferParameteriv_type glGetRenderbufferParameteriv
 * 
 * ctypedef void (*glGetShaderInfoLog_type)(GLuint  shader, GLsizei  bufSize, GLsizei * length, GLchar * infoLog) nogil             # <<<<<<<<<<<<<<
 * cdef glGetShaderInfoLog_type glGetShaderInfoLog
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type)(GLuint, GLsizei, GLsizei *, GLchar *);

/* "renpy/gl2/uguugl.pxd":522
 * cdef glGetShaderInfoLog_type glGetShaderInfoLog
 * 
 * ctypedef void (*glGetShaderSource_type)(GLuint  shader, GLsizei  bufSize, GLsizei * length, GLchar * source) nogil             # <<<<<<<<<<<<<<
 * cdef glGetShaderSource_type glGetShaderSource
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type)(GLuint, GLsizei, GLsizei *, GLchar *);

/* "renpy/gl2/uguugl.pxd":525
 * cdef glGetShaderSource_type glGetShaderSource
 * 
 * ctypedef void (*glGetShaderiv_type)(GLuint  shader, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetShaderiv_type glGetShaderiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type)(GLuint, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":528
 * cdef glGetShaderiv_type glGetShaderiv
 * 
 * ctypedef const GLubyte * (*glGetString_type)(GLenum  name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetString_type glGetString
 * 
 */
typedef GLubyte const *(*__pyx_t_5renpy_3gl2_6uguugl_glGetString_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":531
 * cdef glGetString_type glGetString
 * 
 * ctypedef void (*glGetTexParameterfv_type)(GLenum  target, GLenum  pname, GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetTexParameterfv_type glGetTexParameterfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type)(GLenum, GLenum, GLfloat *);

/* "renpy/gl2/uguugl.pxd":534
 * cdef glGetTexParameterfv_type glGetTexParameterfv
 * 
 * ctypedef void (*glGetTexParameteriv_type)(GLenum  target, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetTexParameteriv_type glGetTexParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type)(GLenum, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":537
 * cdef glGetTexParameteriv_type glGetTexParameteriv
 * 
 * ctypedef GLint (*glGetUniformLocation_type)(GLuint  program, const GLchar * name) nogil             # <<<<<<<<<<<<<<
 * cdef glGetUniformLocation_type glGetUniformLocation
 * 
 */
typedef GLint (*__pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type)(GLuint, GLchar const *);

/* "renpy/gl2/uguugl.pxd":540
 * cdef glGetUniformLocation_type glGetUniformLocation
 * 
 * ctypedef void (*glGetUniformfv_type)(GLuint  program, GLint  location, GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetUniformfv_type glGetUniformfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type)(GLuint, GLint, GLfloat *);

/* "renpy/gl2/uguugl.pxd":543
 * cdef glGetUniformfv_type glGetUniformfv
 * 
 * ctypedef void (*glGetUniformiv_type)(GLuint  program, GLint  location, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetUniformiv_type glGetUniformiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type)(GLuint, GLint, GLint *);

/* "renpy/gl2/uguugl.pxd":546
 * cdef glGetUniformiv_type glGetUniformiv
 * 
 * ctypedef void (*glGetVertexAttribPointerv_type)(GLuint  index, GLenum  pname, void ** pointer) nogil             # <<<<<<<<<<<<<<
 * cdef glGetVertexAttribPointerv_type glGetVertexAttribPointerv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type)(GLuint, GLenum, void **);

/* "renpy/gl2/uguugl.pxd":549
 * cdef glGetVertexAttribPointerv_type glGetVertexAttribPointerv
 * 
 * ctypedef void (*glGetVertexAttribfv_type)(GLuint  index, GLenum  pname, GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetVertexAttribfv_type glGetVertexAttribfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type)(GLuint, GLenum, GLfloat *);

/* "renpy/gl2/uguugl.pxd":552
 * cdef glGetVertexAttribfv_type glGetVertexAttribfv
 * 
 * ctypedef void (*glGetVertexAttribiv_type)(GLuint  index, GLenum  pname, GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glGetVertexAttribiv_type glGetVertexAttribiv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type)(GLuint, GLenum, GLint *);

/* "renpy/gl2/uguugl.pxd":555
 * cdef glGetVertexAttribiv_type glGetVertexAttribiv
 * 
 * ctypedef void (*glHint_type)(GLenum  target, GLenum  mode) nogil             # <<<<<<<<<<<<<<
 * cdef glHint_type glHint
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glHint_type)(GLenum, GLenum);

/* "renpy/gl2/uguugl.pxd":558
 * cdef glHint_type glHint
 * 
 * ctypedef GLboolean (*glIsBuffer_type)(GLuint  buffer) nogil             # <<<<<<<<<<<<<<
 * cdef glIsBuffer_type glIsBuffer
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":561
 * cdef glIsBuffer_type glIsBuffer
 * 
 * ctypedef GLboolean (*glIsEnabled_type)(GLenum  cap) nogil             # <<<<<<<<<<<<<<
 * cdef glIsEnabled_type glIsEnabled
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type)(GLenum);

/* "renpy/gl2/uguugl.pxd":564
 * cdef glIsEnabled_type glIsEnabled
 * 
 * ctypedef GLboolean (*glIsFramebuffer_type)(GLuint  framebuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glIsFramebuffer_type glIsFramebuffer
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":567
 * cdef glIsFramebuffer_type glIsFramebuffer
 * 
 * ctypedef GLboolean (*glIsProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glIsProgram_type glIsProgram
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":570
 * cdef glIsProgram_type glIsProgram
 * 
 * ctypedef GLboolean (*glIsRenderbuffer_type)(GLuint  renderbuffer) nogil             # <<<<<<<<<<<<<<
 * cdef glIsRenderbuffer_type glIsRenderbuffer
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":573
 * cdef glIsRenderbuffer_type glIsRenderbuffer
 * 
 * ctypedef GLboolean (*glIsShader_type)(GLuint  shader) nogil             # <<<<<<<<<<<<<<
 * cdef glIsShader_type glIsShader
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsShader_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":576
 * cdef glIsShader_type glIsShader
 * 
 * ctypedef GLboolean (*glIsTexture_type)(GLuint  texture) nogil             # <<<<<<<<<<<<<<
 * cdef glIsTexture_type glIsTexture
 * 
 */
typedef GLboolean (*__pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":579
 * cdef glIsTexture_type glIsTexture
 * 
 * ctypedef void (*glLineWidth_type)(GLfloat  width) nogil             # <<<<<<<<<<<<<<
 * cdef glLineWidth_type glLineWidth
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type)(GLfloat);

/* "renpy/gl2/uguugl.pxd":582
 * cdef glLineWidth_type glLineWidth
 * 
 * ctypedef void (*glLinkProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glLinkProgram_type glLinkProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":585
 * cdef glLinkProgram_type glLinkProgram
 * 
 * ctypedef void (*glPixelStorei_type)(GLenum  pname, GLint  param) nogil             # <<<<<<<<<<<<<<
 * cdef glPixelStorei_type glPixelStorei
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type)(GLenum, GLint);

/* "renpy/gl2/uguugl.pxd":588
 * cdef glPixelStorei_type glPixelStorei
 * 
 * ctypedef void (*glPolygonOffset_type)(GLfloat  factor, GLfloat  units) nogil             # <<<<<<<<<<<<<<
 * cdef glPolygonOffset_type glPolygonOffset
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type)(GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":591
 * cdef glPolygonOffset_type glPolygonOffset
 * 
 * ctypedef void (*glReadPixels_type)(GLint  x, GLint  y, GLsizei  width, GLsizei  height, GLenum  format, GLenum  type, void * pixels) nogil             # <<<<<<<<<<<<<<
 * cdef glReadPixels_type glReadPixels
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type)(GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, void *);

/* "renpy/gl2/uguugl.pxd":594
 * cdef glReadPixels_type glReadPixels
 * 
 * ctypedef void (*glRenderbufferStorage_type)(GLenum  target, GLenum  internalformat, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glRenderbufferStorage_type glRenderbufferStorage
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type)(GLenum, GLenum, GLsizei, GLsizei);

/* "renpy/gl2/uguugl.pxd":597
 * cdef glRenderbufferStorage_type glRenderbufferStorage
 * 
 * ctypedef void (*glSampleCoverage_type)(GLfloat  value, GLboolean  invert) nogil             # <<<<<<<<<<<<<<
 * cdef glSampleCoverage_type glSampleCoverage
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type)(GLfloat, GLboolean);

/* "renpy/gl2/uguugl.pxd":600
 * cdef glSampleCoverage_type glSampleCoverage
 * 
 * ctypedef void (*glScissor_type)(GLint  x, GLint  y, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glScissor_type glScissor
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glScissor_type)(GLint, GLint, GLsizei, GLsizei);

/* "renpy/gl2/uguugl.pxd":603
 * cdef glScissor_type glScissor
 * 
 * ctypedef void (*glShaderSource_type)(GLuint  shader, GLsizei  count, const GLchar *const* string, const GLint * length) nogil             # <<<<<<<<<<<<<<
 * cdef glShaderSource_type glShaderSource
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type)(GLuint, GLsizei, GLchar const *const *, GLint const *);

/* "renpy/gl2/uguugl.pxd":606
 * cdef glShaderSource_type glShaderSource
 * 
 * ctypedef void (*glStencilFunc_type)(GLenum  func, GLint  ref, GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilFunc_type glStencilFunc
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type)(GLenum, GLint, GLuint);

/* "renpy/gl2/uguugl.pxd":609
 * cdef glStencilFunc_type glStencilFunc
 * 
 * ctypedef void (*glStencilFuncSeparate_type)(GLenum  face, GLenum  func, GLint  ref, GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilFuncSeparate_type glStencilFuncSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type)(GLenum, GLenum, GLint, GLuint);

/* "renpy/gl2/uguugl.pxd":612
 * cdef glStencilFuncSeparate_type glStencilFuncSeparate
 * 
 * ctypedef void (*glStencilMask_type)(GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilMask_type glStencilMask
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":615
 * cdef glStencilMask_type glStencilMask
 * 
 * ctypedef void (*glStencilMaskSeparate_type)(GLenum  face, GLuint  mask) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilMaskSeparate_type glStencilMaskSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type)(GLenum, GLuint);

/* "renpy/gl2/uguugl.pxd":618
 * cdef glStencilMaskSeparate_type glStencilMaskSeparate
 * 
 * ctypedef void (*glStencilOp_type)(GLenum  fail, GLenum  zfail, GLenum  zpass) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilOp_type glStencilOp
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type)(GLenum, GLenum, GLenum);

/* "renpy/gl2/uguugl.pxd":621
 * cdef glStencilOp_type glStencilOp
 * 
 * ctypedef void (*glStencilOpSeparate_type)(GLenum  face, GLenum  sfail, GLenum  dpfail, GLenum  dppass) nogil             # <<<<<<<<<<<<<<
 * cdef glStencilOpSeparate_type glStencilOpSeparate
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type)(GLenum, GLenum, GLenum, GLenum);

/* "renpy/gl2/uguugl.pxd":624
 * cdef glStencilOpSeparate_type glStencilOpSeparate
 * 
 * ctypedef void (*glTexImage2D_type)(GLenum  target, GLint  level, GLint  internalformat, GLsizei  width, GLsizei  height, GLint  border, GLenum  format, GLenum  type, const void * pixels) nogil             # <<<<<<<<<<<<<<
 * cdef glTexImage2D_type glTexImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type)(GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, void const *);

/* "renpy/gl2/uguugl.pxd":627
 * cdef glTexImage2D_type glTexImage2D
 * 
 * ctypedef void (*glTexParameterf_type)(GLenum  target, GLenum  pname, GLfloat  param) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameterf_type glTexParameterf
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type)(GLenum, GLenum, GLfloat);

/* "renpy/gl2/uguugl.pxd":630
 * cdef glTexParameterf_type glTexParameterf
 * 
 * ctypedef void (*glTexParameterfv_type)(GLenum  target, GLenum  pname, const GLfloat * params) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameterfv_type glTexParameterfv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type)(GLenum, GLenum, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":633
 * cdef glTexParameterfv_type glTexParameterfv
 * 
 * ctypedef void (*glTexParameteri_type)(GLenum  target, GLenum  pname, GLint  param) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameteri_type glTexParameteri
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type)(GLenum, GLenum, GLint);

/* "renpy/gl2/uguugl.pxd":636
 * cdef glTexParameteri_type glTexParameteri
 * 
 * ctypedef void (*glTexParameteriv_type)(GLenum  target, GLenum  pname, const GLint * params) nogil             # <<<<<<<<<<<<<<
 * cdef glTexParameteriv_type glTexParameteriv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type)(GLenum, GLenum, GLint const *);

/* "renpy/gl2/uguugl.pxd":639
 * cdef glTexParameteriv_type glTexParameteriv
 * 
 * ctypedef void (*glTexSubImage2D_type)(GLenum  target, GLint  level, GLint  xoffset, GLint  yoffset, GLsizei  width, GLsizei  height, GLenum  format, GLenum  type, const void * pixels) nogil             # <<<<<<<<<<<<<<
 * cdef glTexSubImage2D_type glTexSubImage2D
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, void const *);

/* "renpy/gl2/uguugl.pxd":642
 * cdef glTexSubImage2D_type glTexSubImage2D
 * 
 * ctypedef void (*glUniform1f_type)(GLint  location, GLfloat  v0) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1f_type glUniform1f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type)(GLint, GLfloat);

/* "renpy/gl2/uguugl.pxd":645
 * cdef glUniform1f_type glUniform1f
 * 
 * ctypedef void (*glUniform1fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1fv_type glUniform1fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type)(GLint, GLsizei, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":648
 * cdef glUniform1fv_type glUniform1fv
 * 
 * ctypedef void (*glUniform1i_type)(GLint  location, GLint  v0) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1i_type glUniform1i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type)(GLint, GLint);

/* "renpy/gl2/uguugl.pxd":651
 * cdef glUniform1i_type glUniform1i
 * 
 * ctypedef void (*glUniform1iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform1iv_type glUniform1iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type)(GLint, GLsizei, GLint const *);

/* "renpy/gl2/uguugl.pxd":654
 * cdef glUniform1iv_type glUniform1iv
 * 
 * ctypedef void (*glUniform2f_type)(GLint  location, GLfloat  v0, GLfloat  v1) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2f_type glUniform2f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type)(GLint, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":657
 * cdef glUniform2f_type glUniform2f
 * 
 * ctypedef void (*glUniform2fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2fv_type glUniform2fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type)(GLint, GLsizei, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":660
 * cdef glUniform2fv_type glUniform2fv
 * 
 * ctypedef void (*glUniform2i_type)(GLint  location, GLint  v0, GLint  v1) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2i_type glUniform2i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type)(GLint, GLint, GLint);

/* "renpy/gl2/uguugl.pxd":663
 * cdef glUniform2i_type glUniform2i
 * 
 * ctypedef void (*glUniform2iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform2iv_type glUniform2iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type)(GLint, GLsizei, GLint const *);

/* "renpy/gl2/uguugl.pxd":666
 * cdef glUniform2iv_type glUniform2iv
 * 
 * ctypedef void (*glUniform3f_type)(GLint  location, GLfloat  v0, GLfloat  v1, GLfloat  v2) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3f_type glUniform3f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type)(GLint, GLfloat, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":669
 * cdef glUniform3f_type glUniform3f
 * 
 * ctypedef void (*glUniform3fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3fv_type glUniform3fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type)(GLint, GLsizei, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":672
 * cdef glUniform3fv_type glUniform3fv
 * 
 * ctypedef void (*glUniform3i_type)(GLint  location, GLint  v0, GLint  v1, GLint  v2) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3i_type glUniform3i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type)(GLint, GLint, GLint, GLint);

/* "renpy/gl2/uguugl.pxd":675
 * cdef glUniform3i_type glUniform3i
 * 
 * ctypedef void (*glUniform3iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform3iv_type glUniform3iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type)(GLint, GLsizei, GLint const *);

/* "renpy/gl2/uguugl.pxd":678
 * cdef glUniform3iv_type glUniform3iv
 * 
 * ctypedef void (*glUniform4f_type)(GLint  location, GLfloat  v0, GLfloat  v1, GLfloat  v2, GLfloat  v3) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4f_type glUniform4f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type)(GLint, GLfloat, GLfloat, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":681
 * cdef glUniform4f_type glUniform4f
 * 
 * ctypedef void (*glUniform4fv_type)(GLint  location, GLsizei  count, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4fv_type glUniform4fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type)(GLint, GLsizei, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":684
 * cdef glUniform4fv_type glUniform4fv
 * 
 * ctypedef void (*glUniform4i_type)(GLint  location, GLint  v0, GLint  v1, GLint  v2, GLint  v3) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4i_type glUniform4i
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type)(GLint, GLint, GLint, GLint, GLint);

/* "renpy/gl2/uguugl.pxd":687
 * cdef glUniform4i_type glUniform4i
 * 
 * ctypedef void (*glUniform4iv_type)(GLint  location, GLsizei  count, const GLint * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniform4iv_type glUniform4iv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type)(GLint, GLsizei, GLint const *);

/* "renpy/gl2/uguugl.pxd":690
 * cdef glUniform4iv_type glUniform4iv
 * 
 * ctypedef void (*glUniformMatrix2fv_type)(GLint  location, GLsizei  count, GLboolean  transpose, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniformMatrix2fv_type glUniformMatrix2fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type)(GLint, GLsizei, GLboolean, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":693
 * cdef glUniformMatrix2fv_type glUniformMatrix2fv
 * 
 * ctypedef void (*glUniformMatrix3fv_type)(GLint  location, GLsizei  count, GLboolean  transpose, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniformMatrix3fv_type glUniformMatrix3fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type)(GLint, GLsizei, GLboolean, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":696
 * cdef glUniformMatrix3fv_type glUniformMatrix3fv
 * 
 * ctypedef void (*glUniformMatrix4fv_type)(GLint  location, GLsizei  count, GLboolean  transpose, const GLfloat * value) nogil             # <<<<<<<<<<<<<<
 * cdef glUniformMatrix4fv_type glUniformMatrix4fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type)(GLint, GLsizei, GLboolean, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":699
 * cdef glUniformMatrix4fv_type glUniformMatrix4fv
 * 
 * ctypedef void (*glUseProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glUseProgram_type glUseProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":702
 * cdef glUseProgram_type glUseProgram
 * 
 * ctypedef void (*glValidateProgram_type)(GLuint  program) nogil             # <<<<<<<<<<<<<<
 * cdef glValidateProgram_type glValidateProgram
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type)(GLuint);

/* "renpy/gl2/uguugl.pxd":705
 * cdef glValidateProgram_type glValidateProgram
 * 
 * ctypedef void (*glVertexAttrib1f_type)(GLuint  index, GLfloat  x) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib1f_type glVertexAttrib1f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type)(GLuint, GLfloat);

/* "renpy/gl2/uguugl.pxd":708
 * cdef glVertexAttrib1f_type glVertexAttrib1f
 * 
 * ctypedef void (*glVertexAttrib1fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib1fv_type glVertexAttrib1fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type)(GLuint, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":711
 * cdef glVertexAttrib1fv_type glVertexAttrib1fv
 * 
 * ctypedef void (*glVertexAttrib2f_type)(GLuint  index, GLfloat  x, GLfloat  y) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib2f_type glVertexAttrib2f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type)(GLuint, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":714
 * cdef glVertexAttrib2f_type glVertexAttrib2f
 * 
 * ctypedef void (*glVertexAttrib2fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib2fv_type glVertexAttrib2fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type)(GLuint, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":717
 * cdef glVertexAttrib2fv_type glVertexAttrib2fv
 * 
 * ctypedef void (*glVertexAttrib3f_type)(GLuint  index, GLfloat  x, GLfloat  y, GLfloat  z) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib3f_type glVertexAttrib3f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type)(GLuint, GLfloat, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":720
 * cdef glVertexAttrib3f_type glVertexAttrib3f
 * 
 * ctypedef void (*glVertexAttrib3fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib3fv_type glVertexAttrib3fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type)(GLuint, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":723
 * cdef glVertexAttrib3fv_type glVertexAttrib3fv
 * 
 * ctypedef void (*glVertexAttrib4f_type)(GLuint  index, GLfloat  x, GLfloat  y, GLfloat  z, GLfloat  w) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib4f_type glVertexAttrib4f
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type)(GLuint, GLfloat, GLfloat, GLfloat, GLfloat);

/* "renpy/gl2/uguugl.pxd":726
 * cdef glVertexAttrib4f_type glVertexAttrib4f
 * 
 * ctypedef void (*glVertexAttrib4fv_type)(GLuint  index, const GLfloat * v) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttrib4fv_type glVertexAttrib4fv
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type)(GLuint, GLfloat const *);

/* "renpy/gl2/uguugl.pxd":729
 * cdef glVertexAttrib4fv_type glVertexAttrib4fv
 * 
 * ctypedef void (*glVertexAttribPointer_type)(GLuint  index, GLint  size, GLenum  type, GLboolean  normalized, GLsizei  stride, const void * pointer) nogil             # <<<<<<<<<<<<<<
 * cdef glVertexAttribPointer_type glVertexAttribPointer
 * 
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type)(GLuint, GLint, GLenum, GLboolean, GLsizei, void const *);

/* "renpy/gl2/uguugl.pxd":732
 * cdef glVertexAttribPointer_type glVertexAttribPointer
 * 
 * ctypedef void (*glViewport_type)(GLint  x, GLint  y, GLsizei  width, GLsizei  height) nogil             # <<<<<<<<<<<<<<
 * cdef glViewport_type glViewport
 */
typedef void (*__pyx_t_5renpy_3gl2_6uguugl_glViewport_type)(GLint, GLint, GLsizei, GLsizei);

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'sdl2' */

/* Module declarations from 'renpy.gl2.uguugl' */
static __pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type __pyx_v_5renpy_3gl2_6uguugl_glActiveTexture;
static __pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type __pyx_v_5renpy_3gl2_6uguugl_glAttachShader;
static __pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type __pyx_v_5renpy_3gl2_6uguugl_glBindAttribLocation;
static __pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type __pyx_v_5renpy_3gl2_6uguugl_glBindBuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type __pyx_v_5renpy_3gl2_6uguugl_glBindFramebuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type __pyx_v_5renpy_3gl2_6uguugl_glBindRenderbuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type __pyx_v_5renpy_3gl2_6uguugl_glBindTexture;
static __pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type __pyx_v_5renpy_3gl2_6uguugl_glBlendColor;
static __pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type __pyx_v_5renpy_3gl2_6uguugl_glBlendEquation;
static __pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type __pyx_v_5renpy_3gl2_6uguugl_glBlendEquationSeparate;
static __pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type __pyx_v_5renpy_3gl2_6uguugl_glBlendFunc;
static __pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type __pyx_v_5renpy_3gl2_6uguugl_glBlendFuncSeparate;
static __pyx_t_5renpy_3gl2_6uguugl_glBufferData_type __pyx_v_5renpy_3gl2_6uguugl_glBufferData;
static __pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type __pyx_v_5renpy_3gl2_6uguugl_glBufferSubData;
static __pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type __pyx_v_5renpy_3gl2_6uguugl_glCheckFramebufferStatus;
static __pyx_t_5renpy_3gl2_6uguugl_glClear_type __pyx_v_5renpy_3gl2_6uguugl_glClear;
static __pyx_t_5renpy_3gl2_6uguugl_glClearColor_type __pyx_v_5renpy_3gl2_6uguugl_glClearColor;
static __pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type __pyx_v_5renpy_3gl2_6uguugl_glClearStencil;
static __pyx_t_5renpy_3gl2_6uguugl_glColorMask_type __pyx_v_5renpy_3gl2_6uguugl_glColorMask;
static __pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type __pyx_v_5renpy_3gl2_6uguugl_glCompileShader;
static __pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexImage2D;
static __pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D;
static __pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type __pyx_v_5renpy_3gl2_6uguugl_glCopyTexImage2D;
static __pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type __pyx_v_5renpy_3gl2_6uguugl_glCopyTexSubImage2D;
static __pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type __pyx_v_5renpy_3gl2_6uguugl_glCreateProgram;
static __pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type __pyx_v_5renpy_3gl2_6uguugl_glCreateShader;
static __pyx_t_5renpy_3gl2_6uguugl_glCullFace_type __pyx_v_5renpy_3gl2_6uguugl_glCullFace;
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type __pyx_v_5renpy_3gl2_6uguugl_glDeleteBuffers;
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type __pyx_v_5renpy_3gl2_6uguugl_glDeleteFramebuffers;
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type __pyx_v_5renpy_3gl2_6uguugl_glDeleteProgram;
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type __pyx_v_5renpy_3gl2_6uguugl_glDeleteRenderbuffers;
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type __pyx_v_5renpy_3gl2_6uguugl_glDeleteShader;
static __pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type __pyx_v_5renpy_3gl2_6uguugl_glDeleteTextures;
static __pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type __pyx_v_5renpy_3gl2_6uguugl_glDepthFunc;
static __pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type __pyx_v_5renpy_3gl2_6uguugl_glDepthMask;
static __pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type __pyx_v_5renpy_3gl2_6uguugl_glDetachShader;
static __pyx_t_5renpy_3gl2_6uguugl_glDisable_type __pyx_v_5renpy_3gl2_6uguugl_glDisable;
static __pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type __pyx_v_5renpy_3gl2_6uguugl_glDisableVertexAttribArray;
static __pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type __pyx_v_5renpy_3gl2_6uguugl_glDrawArrays;
static __pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type __pyx_v_5renpy_3gl2_6uguugl_glDrawElements;
static __pyx_t_5renpy_3gl2_6uguugl_glEnable_type __pyx_v_5renpy_3gl2_6uguugl_glEnable;
static __pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type __pyx_v_5renpy_3gl2_6uguugl_glEnableVertexAttribArray;
static __pyx_t_5renpy_3gl2_6uguugl_glFinish_type __pyx_v_5renpy_3gl2_6uguugl_glFinish;
static __pyx_t_5renpy_3gl2_6uguugl_glFlush_type __pyx_v_5renpy_3gl2_6uguugl_glFlush;
static __pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type __pyx_v_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type __pyx_v_5renpy_3gl2_6uguugl_glFramebufferTexture2D;
static __pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type __pyx_v_5renpy_3gl2_6uguugl_glFrontFace;
static __pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type __pyx_v_5renpy_3gl2_6uguugl_glGenBuffers;
static __pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type __pyx_v_5renpy_3gl2_6uguugl_glGenFramebuffers;
static __pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type __pyx_v_5renpy_3gl2_6uguugl_glGenRenderbuffers;
static __pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type __pyx_v_5renpy_3gl2_6uguugl_glGenTextures;
static __pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type __pyx_v_5renpy_3gl2_6uguugl_glGenerateMipmap;
static __pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type __pyx_v_5renpy_3gl2_6uguugl_glGetActiveAttrib;
static __pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type __pyx_v_5renpy_3gl2_6uguugl_glGetActiveUniform;
static __pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type __pyx_v_5renpy_3gl2_6uguugl_glGetAttachedShaders;
static __pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type __pyx_v_5renpy_3gl2_6uguugl_glGetAttribLocation;
static __pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type __pyx_v_5renpy_3gl2_6uguugl_glGetBooleanv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type __pyx_v_5renpy_3gl2_6uguugl_glGetBufferParameteriv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetError_type __pyx_v_5renpy_3gl2_6uguugl_glGetError;
static __pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type __pyx_v_5renpy_3gl2_6uguugl_glGetFloatv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type __pyx_v_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type __pyx_v_5renpy_3gl2_6uguugl_glGetIntegerv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type __pyx_v_5renpy_3gl2_6uguugl_glGetProgramInfoLog;
static __pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type __pyx_v_5renpy_3gl2_6uguugl_glGetProgramiv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type __pyx_v_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type __pyx_v_5renpy_3gl2_6uguugl_glGetShaderInfoLog;
static __pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type __pyx_v_5renpy_3gl2_6uguugl_glGetShaderSource;
static __pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type __pyx_v_5renpy_3gl2_6uguugl_glGetShaderiv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetString_type __pyx_v_5renpy_3gl2_6uguugl_glGetString;
static __pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameterfv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameteriv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type __pyx_v_5renpy_3gl2_6uguugl_glGetUniformLocation;
static __pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type __pyx_v_5renpy_3gl2_6uguugl_glGetUniformfv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type __pyx_v_5renpy_3gl2_6uguugl_glGetUniformiv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribfv;
static __pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribiv;
static __pyx_t_5renpy_3gl2_6uguugl_glHint_type __pyx_v_5renpy_3gl2_6uguugl_glHint;
static __pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type __pyx_v_5renpy_3gl2_6uguugl_glIsBuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type __pyx_v_5renpy_3gl2_6uguugl_glIsEnabled;
static __pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type __pyx_v_5renpy_3gl2_6uguugl_glIsFramebuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type __pyx_v_5renpy_3gl2_6uguugl_glIsProgram;
static __pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type __pyx_v_5renpy_3gl2_6uguugl_glIsRenderbuffer;
static __pyx_t_5renpy_3gl2_6uguugl_glIsShader_type __pyx_v_5renpy_3gl2_6uguugl_glIsShader;
static __pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type __pyx_v_5renpy_3gl2_6uguugl_glIsTexture;
static __pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type __pyx_v_5renpy_3gl2_6uguugl_glLineWidth;
static __pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type __pyx_v_5renpy_3gl2_6uguugl_glLinkProgram;
static __pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type __pyx_v_5renpy_3gl2_6uguugl_glPixelStorei;
static __pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type __pyx_v_5renpy_3gl2_6uguugl_glPolygonOffset;
static __pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type __pyx_v_5renpy_3gl2_6uguugl_glReadPixels;
static __pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type __pyx_v_5renpy_3gl2_6uguugl_glRenderbufferStorage;
static __pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type __pyx_v_5renpy_3gl2_6uguugl_glSampleCoverage;
static __pyx_t_5renpy_3gl2_6uguugl_glScissor_type __pyx_v_5renpy_3gl2_6uguugl_glScissor;
static __pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type __pyx_v_5renpy_3gl2_6uguugl_glShaderSource;
static __pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type __pyx_v_5renpy_3gl2_6uguugl_glStencilFunc;
static __pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type __pyx_v_5renpy_3gl2_6uguugl_glStencilFuncSeparate;
static __pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type __pyx_v_5renpy_3gl2_6uguugl_glStencilMask;
static __pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type __pyx_v_5renpy_3gl2_6uguugl_glStencilMaskSeparate;
static __pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type __pyx_v_5renpy_3gl2_6uguugl_glStencilOp;
static __pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type __pyx_v_5renpy_3gl2_6uguugl_glStencilOpSeparate;
static __pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type __pyx_v_5renpy_3gl2_6uguugl_glTexImage2D;
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type __pyx_v_5renpy_3gl2_6uguugl_glTexParameterf;
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type __pyx_v_5renpy_3gl2_6uguugl_glTexParameterfv;
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type __pyx_v_5renpy_3gl2_6uguugl_glTexParameteri;
static __pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type __pyx_v_5renpy_3gl2_6uguugl_glTexParameteriv;
static __pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type __pyx_v_5renpy_3gl2_6uguugl_glTexSubImage2D;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type __pyx_v_5renpy_3gl2_6uguugl_glUniform1f;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform1fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type __pyx_v_5renpy_3gl2_6uguugl_glUniform1i;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform1iv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type __pyx_v_5renpy_3gl2_6uguugl_glUniform2f;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform2fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type __pyx_v_5renpy_3gl2_6uguugl_glUniform2i;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform2iv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type __pyx_v_5renpy_3gl2_6uguugl_glUniform3f;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform3fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type __pyx_v_5renpy_3gl2_6uguugl_glUniform3i;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform3iv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type __pyx_v_5renpy_3gl2_6uguugl_glUniform4f;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform4fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type __pyx_v_5renpy_3gl2_6uguugl_glUniform4i;
static __pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type __pyx_v_5renpy_3gl2_6uguugl_glUniform4iv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix2fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix3fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix4fv;
static __pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type __pyx_v_5renpy_3gl2_6uguugl_glUseProgram;
static __pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type __pyx_v_5renpy_3gl2_6uguugl_glValidateProgram;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1f;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1fv;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2f;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2fv;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3f;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3fv;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4f;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4fv;
static __pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type __pyx_v_5renpy_3gl2_6uguugl_glVertexAttribPointer;
static __pyx_t_5renpy_3gl2_6uguugl_glViewport_type __pyx_v_5renpy_3gl2_6uguugl_glViewport;
static void *__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "renpy.gl2.uguugl"
extern int __pyx_module_is_main_renpy__gl2__uguugl;
int __pyx_module_is_main_renpy__gl2__uguugl = 0;

/* Implementation of 'renpy.gl2.uguugl' */
static const char __pyx_k_load[] = "load";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_glHint[] = "glHint";
static const char __pyx_k_glClear[] = "glClear";
static const char __pyx_k_glFlush[] = "glFlush";
static const char __pyx_k_glEnable[] = "glEnable";
static const char __pyx_k_glFinish[] = "glFinish";
static const char __pyx_k_pyx_capi[] = "__pyx_capi__";
static const char __pyx_k_glDisable[] = "glDisable";
static const char __pyx_k_glScissor[] = "glScissor";
static const char __pyx_k_not_found[] = "{} not found.";
static const char __pyx_k_glCullFace[] = "glCullFace";
static const char __pyx_k_glGetError[] = "glGetError";
static const char __pyx_k_glIsBuffer[] = "glIsBuffer";
static const char __pyx_k_glIsShader[] = "glIsShader";
static const char __pyx_k_glViewport[] = "glViewport";
static const char __pyx_k_glBlendFunc[] = "glBlendFunc";
static const char __pyx_k_glColorMask[] = "glColorMask";
static const char __pyx_k_glDepthFunc[] = "glDepthFunc";
static const char __pyx_k_glDepthMask[] = "glDepthMask";
static const char __pyx_k_glFrontFace[] = "glFrontFace";
static const char __pyx_k_glGetFloatv[] = "glGetFloatv";
static const char __pyx_k_glGetString[] = "glGetString";
static const char __pyx_k_glIsEnabled[] = "glIsEnabled";
static const char __pyx_k_glIsProgram[] = "glIsProgram";
static const char __pyx_k_glIsTexture[] = "glIsTexture";
static const char __pyx_k_glLineWidth[] = "glLineWidth";
static const char __pyx_k_glStencilOp[] = "glStencilOp";
static const char __pyx_k_glUniform1f[] = "glUniform1f";
static const char __pyx_k_glUniform1i[] = "glUniform1i";
static const char __pyx_k_glUniform2f[] = "glUniform2f";
static const char __pyx_k_glUniform2i[] = "glUniform2i";
static const char __pyx_k_glUniform3f[] = "glUniform3f";
static const char __pyx_k_glUniform3i[] = "glUniform3i";
static const char __pyx_k_glUniform4f[] = "glUniform4f";
static const char __pyx_k_glUniform4i[] = "glUniform4i";
static const char __pyx_k_glBindBuffer[] = "glBindBuffer";
static const char __pyx_k_glBlendColor[] = "glBlendColor";
static const char __pyx_k_glBufferData[] = "glBufferData";
static const char __pyx_k_glClearColor[] = "glClearColor";
static const char __pyx_k_glDrawArrays[] = "glDrawArrays";
static const char __pyx_k_glGenBuffers[] = "glGenBuffers";
static const char __pyx_k_glReadPixels[] = "glReadPixels";
static const char __pyx_k_glTexImage2D[] = "glTexImage2D";
static const char __pyx_k_glUniform1fv[] = "glUniform1fv";
static const char __pyx_k_glUniform1iv[] = "glUniform1iv";
static const char __pyx_k_glUniform2fv[] = "glUniform2fv";
static const char __pyx_k_glUniform2iv[] = "glUniform2iv";
static const char __pyx_k_glUniform3fv[] = "glUniform3fv";
static const char __pyx_k_glUniform3iv[] = "glUniform3iv";
static const char __pyx_k_glUniform4fv[] = "glUniform4fv";
static const char __pyx_k_glUniform4iv[] = "glUniform4iv";
static const char __pyx_k_glUseProgram[] = "glUseProgram";
static const char __pyx_k_glBindTexture[] = "glBindTexture";
static const char __pyx_k_glGenTextures[] = "glGenTextures";
static const char __pyx_k_glGetBooleanv[] = "glGetBooleanv";
static const char __pyx_k_glGetIntegerv[] = "glGetIntegerv";
static const char __pyx_k_glGetShaderiv[] = "glGetShaderiv";
static const char __pyx_k_glIsBufferARB[] = "glIsBufferARB";
static const char __pyx_k_glLinkProgram[] = "glLinkProgram";
static const char __pyx_k_glPixelStorei[] = "glPixelStorei";
static const char __pyx_k_glStencilFunc[] = "glStencilFunc";
static const char __pyx_k_glStencilMask[] = "glStencilMask";
static const char __pyx_k_glAttachShader[] = "glAttachShader";
static const char __pyx_k_glClearStencil[] = "glClearStencil";
static const char __pyx_k_glCreateShader[] = "glCreateShader";
static const char __pyx_k_glDeleteShader[] = "glDeleteShader";
static const char __pyx_k_glDetachShader[] = "glDetachShader";
static const char __pyx_k_glDrawElements[] = "glDrawElements";
static const char __pyx_k_glGetProgramiv[] = "glGetProgramiv";
static const char __pyx_k_glGetUniformfv[] = "glGetUniformfv";
static const char __pyx_k_glGetUniformiv[] = "glGetUniformiv";
static const char __pyx_k_glShaderSource[] = "glShaderSource";
static const char __pyx_k_glUniform1fARB[] = "glUniform1fARB";
static const char __pyx_k_glUniform1iARB[] = "glUniform1iARB";
static const char __pyx_k_glUniform2fARB[] = "glUniform2fARB";
static const char __pyx_k_glUniform2iARB[] = "glUniform2iARB";
static const char __pyx_k_glUniform3fARB[] = "glUniform3fARB";
static const char __pyx_k_glUniform3iARB[] = "glUniform3iARB";
static const char __pyx_k_glUniform4fARB[] = "glUniform4fARB";
static const char __pyx_k_glUniform4iARB[] = "glUniform4iARB";
static const char __pyx_k_glActiveTexture[] = "glActiveTexture";
static const char __pyx_k_glBindBufferARB[] = "glBindBufferARB";
static const char __pyx_k_glBlendColorEXT[] = "glBlendColorEXT";
static const char __pyx_k_glBlendEquation[] = "glBlendEquation";
static const char __pyx_k_glBufferDataARB[] = "glBufferDataARB";
static const char __pyx_k_glBufferSubData[] = "glBufferSubData";
static const char __pyx_k_glCompileShader[] = "glCompileShader";
static const char __pyx_k_glCreateProgram[] = "glCreateProgram";
static const char __pyx_k_glDeleteBuffers[] = "glDeleteBuffers";
static const char __pyx_k_glDeleteProgram[] = "glDeleteProgram";
static const char __pyx_k_glDrawArraysEXT[] = "glDrawArraysEXT";
static const char __pyx_k_glGenBuffersARB[] = "glGenBuffersARB";
static const char __pyx_k_glIsFramebuffer[] = "glIsFramebuffer";
static const char __pyx_k_glPolygonOffset[] = "glPolygonOffset";
static const char __pyx_k_glTexParameterf[] = "glTexParameterf";
static const char __pyx_k_glTexParameteri[] = "glTexParameteri";
static const char __pyx_k_glTexSubImage2D[] = "glTexSubImage2D";
static const char __pyx_k_glUniform1fvARB[] = "glUniform1fvARB";
static const char __pyx_k_glUniform1ivARB[] = "glUniform1ivARB";
static const char __pyx_k_glUniform2fvARB[] = "glUniform2fvARB";
static const char __pyx_k_glUniform2ivARB[] = "glUniform2ivARB";
static const char __pyx_k_glUniform3fvARB[] = "glUniform3fvARB";
static const char __pyx_k_glUniform3ivARB[] = "glUniform3ivARB";
static const char __pyx_k_glUniform4fvARB[] = "glUniform4fvARB";
static const char __pyx_k_glUniform4ivARB[] = "glUniform4ivARB";
static const char __pyx_k_glBindTextureEXT[] = "glBindTextureEXT";
static const char __pyx_k_glCopyTexImage2D[] = "glCopyTexImage2D";
static const char __pyx_k_glDeleteTextures[] = "glDeleteTextures";
static const char __pyx_k_glGenerateMipmap[] = "glGenerateMipmap";
static const char __pyx_k_glIsRenderbuffer[] = "glIsRenderbuffer";
static const char __pyx_k_glLinkProgramARB[] = "glLinkProgramARB";
static const char __pyx_k_glSampleCoverage[] = "glSampleCoverage";
static const char __pyx_k_glTexParameterfv[] = "glTexParameterfv";
static const char __pyx_k_glTexParameteriv[] = "glTexParameteriv";
static const char __pyx_k_glVertexAttrib1f[] = "glVertexAttrib1f";
static const char __pyx_k_glVertexAttrib2f[] = "glVertexAttrib2f";
static const char __pyx_k_glVertexAttrib3f[] = "glVertexAttrib3f";
static const char __pyx_k_glVertexAttrib4f[] = "glVertexAttrib4f";
static const char __pyx_k_renpy_gl2_uguugl[] = "renpy.gl2.uguugl";
static const char __pyx_k_glBindFramebuffer[] = "glBindFramebuffer";
static const char __pyx_k_glGenFramebuffers[] = "glGenFramebuffers";
static const char __pyx_k_glGetActiveAttrib[] = "glGetActiveAttrib";
static const char __pyx_k_glGetShaderSource[] = "glGetShaderSource";
static const char __pyx_k_glGetUniformfvARB[] = "glGetUniformfvARB";
static const char __pyx_k_glGetUniformivARB[] = "glGetUniformivARB";
static const char __pyx_k_glShaderSourceARB[] = "glShaderSourceARB";
static const char __pyx_k_glValidateProgram[] = "glValidateProgram";
static const char __pyx_k_glVertexAttrib1fv[] = "glVertexAttrib1fv";
static const char __pyx_k_glVertexAttrib2fv[] = "glVertexAttrib2fv";
static const char __pyx_k_glVertexAttrib3fv[] = "glVertexAttrib3fv";
static const char __pyx_k_glVertexAttrib4fv[] = "glVertexAttrib4fv";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_glActiveTextureARB[] = "glActiveTextureARB";
static const char __pyx_k_glBindRenderbuffer[] = "glBindRenderbuffer";
static const char __pyx_k_glBlendEquationEXT[] = "glBlendEquationEXT";
static const char __pyx_k_glBufferSubDataARB[] = "glBufferSubDataARB";
static const char __pyx_k_glCompileShaderARB[] = "glCompileShaderARB";
static const char __pyx_k_glDeleteBuffersARB[] = "glDeleteBuffersARB";
static const char __pyx_k_glGenRenderbuffers[] = "glGenRenderbuffers";
static const char __pyx_k_glGetActiveUniform[] = "glGetActiveUniform";
static const char __pyx_k_glGetShaderInfoLog[] = "glGetShaderInfoLog";
static const char __pyx_k_glIsFramebufferEXT[] = "glIsFramebufferEXT";
static const char __pyx_k_glTexSubImage2DEXT[] = "glTexSubImage2DEXT";
static const char __pyx_k_glUniformMatrix2fv[] = "glUniformMatrix2fv";
static const char __pyx_k_glUniformMatrix3fv[] = "glUniformMatrix3fv";
static const char __pyx_k_glUniformMatrix4fv[] = "glUniformMatrix4fv";
static const char __pyx_k_glVertexAttrib1fNV[] = "glVertexAttrib1fNV";
static const char __pyx_k_glVertexAttrib2fNV[] = "glVertexAttrib2fNV";
static const char __pyx_k_glVertexAttrib3fNV[] = "glVertexAttrib3fNV";
static const char __pyx_k_glVertexAttrib4fNV[] = "glVertexAttrib4fNV";
static const char __pyx_k_glBlendFuncSeparate[] = "glBlendFuncSeparate";
static const char __pyx_k_glCopyTexImage2DEXT[] = "glCopyTexImage2DEXT";
static const char __pyx_k_glCopyTexSubImage2D[] = "glCopyTexSubImage2D";
static const char __pyx_k_glGenerateMipmapEXT[] = "glGenerateMipmapEXT";
static const char __pyx_k_glGetAttribLocation[] = "glGetAttribLocation";
static const char __pyx_k_glGetProgramInfoLog[] = "glGetProgramInfoLog";
static const char __pyx_k_glGetTexParameterfv[] = "glGetTexParameterfv";
static const char __pyx_k_glGetTexParameteriv[] = "glGetTexParameteriv";
static const char __pyx_k_glGetVertexAttribfv[] = "glGetVertexAttribfv";
static const char __pyx_k_glGetVertexAttribiv[] = "glGetVertexAttribiv";
static const char __pyx_k_glIsRenderbufferEXT[] = "glIsRenderbufferEXT";
static const char __pyx_k_glSampleCoverageARB[] = "glSampleCoverageARB";
static const char __pyx_k_glStencilOpSeparate[] = "glStencilOpSeparate";
static const char __pyx_k_glVertexAttrib1fARB[] = "glVertexAttrib1fARB";
static const char __pyx_k_glVertexAttrib1fvNV[] = "glVertexAttrib1fvNV";
static const char __pyx_k_glVertexAttrib2fARB[] = "glVertexAttrib2fARB";
static const char __pyx_k_glVertexAttrib2fvNV[] = "glVertexAttrib2fvNV";
static const char __pyx_k_glVertexAttrib3fARB[] = "glVertexAttrib3fARB";
static const char __pyx_k_glVertexAttrib3fvNV[] = "glVertexAttrib3fvNV";
static const char __pyx_k_glVertexAttrib4fARB[] = "glVertexAttrib4fARB";
static const char __pyx_k_glVertexAttrib4fvNV[] = "glVertexAttrib4fvNV";
static const char __pyx_k_glBindAttribLocation[] = "glBindAttribLocation";
static const char __pyx_k_glDeleteFramebuffers[] = "glDeleteFramebuffers";
static const char __pyx_k_glGenFramebuffersEXT[] = "glGenFramebuffersEXT";
static const char __pyx_k_glGetActiveAttribARB[] = "glGetActiveAttribARB";
static const char __pyx_k_glGetAttachedShaders[] = "glGetAttachedShaders";
static const char __pyx_k_glGetShaderSourceARB[] = "glGetShaderSourceARB";
static const char __pyx_k_glGetUniformLocation[] = "glGetUniformLocation";
static const char __pyx_k_glValidateProgramARB[] = "glValidateProgramARB";
static const char __pyx_k_glVertexAttrib1fvARB[] = "glVertexAttrib1fvARB";
static const char __pyx_k_glVertexAttrib2fvARB[] = "glVertexAttrib2fvARB";
static const char __pyx_k_glVertexAttrib3fvARB[] = "glVertexAttrib3fvARB";
static const char __pyx_k_glVertexAttrib4fvARB[] = "glVertexAttrib4fvARB";
static const char __pyx_k_glDeleteRenderbuffers[] = "glDeleteRenderbuffers";
static const char __pyx_k_glGenRenderbuffersEXT[] = "glGenRenderbuffersEXT";
static const char __pyx_k_glGetActiveUniformARB[] = "glGetActiveUniformARB";
static const char __pyx_k_glGetVertexAttribfvNV[] = "glGetVertexAttribfvNV";
static const char __pyx_k_glGetVertexAttribivNV[] = "glGetVertexAttribivNV";
static const char __pyx_k_glRenderbufferStorage[] = "glRenderbufferStorage";
static const char __pyx_k_glStencilFuncSeparate[] = "glStencilFuncSeparate";
static const char __pyx_k_glStencilMaskSeparate[] = "glStencilMaskSeparate";
static const char __pyx_k_glUniformMatrix2fvARB[] = "glUniformMatrix2fvARB";
static const char __pyx_k_glUniformMatrix3fvARB[] = "glUniformMatrix3fvARB";
static const char __pyx_k_glUniformMatrix4fvARB[] = "glUniformMatrix4fvARB";
static const char __pyx_k_glUseProgramObjectARB[] = "glUseProgramObjectARB";
static const char __pyx_k_glVertexAttribPointer[] = "glVertexAttribPointer";
static const char __pyx_k_glBlendFuncSeparateEXT[] = "glBlendFuncSeparateEXT";
static const char __pyx_k_glCompressedTexImage2D[] = "glCompressedTexImage2D";
static const char __pyx_k_glCopyTexSubImage2DEXT[] = "glCopyTexSubImage2DEXT";
static const char __pyx_k_glFramebufferTexture2D[] = "glFramebufferTexture2D";
static const char __pyx_k_glGetAttribLocationARB[] = "glGetAttribLocationARB";
static const char __pyx_k_glGetBufferParameteriv[] = "glGetBufferParameteriv";
static const char __pyx_k_glGetVertexAttribfvARB[] = "glGetVertexAttribfvARB";
static const char __pyx_k_glGetVertexAttribivARB[] = "glGetVertexAttribivARB";
static const char __pyx_k_glStencilOpSeparateATI[] = "glStencilOpSeparateATI";
static const char __pyx_k_glBindAttribLocationARB[] = "glBindAttribLocationARB";
static const char __pyx_k_glBlendEquationSeparate[] = "glBlendEquationSeparate";
static const char __pyx_k_glBlendFuncSeparateINGR[] = "glBlendFuncSeparateINGR";
static const char __pyx_k_glCreateShaderObjectARB[] = "glCreateShaderObjectARB";
static const char __pyx_k_glDeleteFramebuffersEXT[] = "glDeleteFramebuffersEXT";
static const char __pyx_k_glGetUniformLocationARB[] = "glGetUniformLocationARB";
static const char __pyx_k_glCheckFramebufferStatus[] = "glCheckFramebufferStatus";
static const char __pyx_k_glCreateProgramObjectARB[] = "glCreateProgramObjectARB";
static const char __pyx_k_glDeleteRenderbuffersEXT[] = "glDeleteRenderbuffersEXT";
static const char __pyx_k_glRenderbufferStorageEXT[] = "glRenderbufferStorageEXT";
static const char __pyx_k_glVertexAttribPointerARB[] = "glVertexAttribPointerARB";
static const char __pyx_k_glCompressedTexImage2DARB[] = "glCompressedTexImage2DARB";
static const char __pyx_k_glCompressedTexSubImage2D[] = "glCompressedTexSubImage2D";
static const char __pyx_k_glEnableVertexAttribArray[] = "glEnableVertexAttribArray";
static const char __pyx_k_glFramebufferRenderbuffer[] = "glFramebufferRenderbuffer";
static const char __pyx_k_glFramebufferTexture2DEXT[] = "glFramebufferTexture2DEXT";
static const char __pyx_k_glGetBufferParameterivARB[] = "glGetBufferParameterivARB";
static const char __pyx_k_glGetVertexAttribPointerv[] = "glGetVertexAttribPointerv";
static const char __pyx_k_glBlendEquationSeparateEXT[] = "glBlendEquationSeparateEXT";
static const char __pyx_k_glDisableVertexAttribArray[] = "glDisableVertexAttribArray";
static const char __pyx_k_glCheckFramebufferStatusEXT[] = "glCheckFramebufferStatusEXT";
static const char __pyx_k_glGetVertexAttribPointervNV[] = "glGetVertexAttribPointervNV";
static const char __pyx_k_glCompressedTexSubImage2DARB[] = "glCompressedTexSubImage2DARB";
static const char __pyx_k_glEnableVertexAttribArrayARB[] = "glEnableVertexAttribArrayARB";
static const char __pyx_k_glFramebufferRenderbufferEXT[] = "glFramebufferRenderbufferEXT";
static const char __pyx_k_glGetRenderbufferParameteriv[] = "glGetRenderbufferParameteriv";
static const char __pyx_k_glGetVertexAttribPointervARB[] = "glGetVertexAttribPointervARB";
static const char __pyx_k_glDisableVertexAttribArrayARB[] = "glDisableVertexAttribArrayARB";
static const char __pyx_k_glGetRenderbufferParameterivEXT[] = "glGetRenderbufferParameterivEXT";
static const char __pyx_k_home_tom_ab_renpy_renpy_gl2_ugu[] = "/home/tom/ab/renpy/renpy/gl2/uguugl.pyx";
static const char __pyx_k_glGetFramebufferAttachmentParame[] = "glGetFramebufferAttachmentParameteriv";
static const char __pyx_k_glGetFramebufferAttachmentParame_2[] = "glGetFramebufferAttachmentParameterivEXT";
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_b_glActiveTexture;
static PyObject *__pyx_n_s_glActiveTexture;
static PyObject *__pyx_n_b_glActiveTextureARB;
static PyObject *__pyx_n_b_glAttachShader;
static PyObject *__pyx_n_s_glAttachShader;
static PyObject *__pyx_n_b_glBindAttribLocation;
static PyObject *__pyx_n_s_glBindAttribLocation;
static PyObject *__pyx_n_b_glBindAttribLocationARB;
static PyObject *__pyx_n_b_glBindBuffer;
static PyObject *__pyx_n_s_glBindBuffer;
static PyObject *__pyx_n_b_glBindBufferARB;
static PyObject *__pyx_n_b_glBindFramebuffer;
static PyObject *__pyx_n_s_glBindFramebuffer;
static PyObject *__pyx_n_b_glBindRenderbuffer;
static PyObject *__pyx_n_s_glBindRenderbuffer;
static PyObject *__pyx_n_b_glBindTexture;
static PyObject *__pyx_n_s_glBindTexture;
static PyObject *__pyx_n_b_glBindTextureEXT;
static PyObject *__pyx_n_b_glBlendColor;
static PyObject *__pyx_n_s_glBlendColor;
static PyObject *__pyx_n_b_glBlendColorEXT;
static PyObject *__pyx_n_b_glBlendEquation;
static PyObject *__pyx_n_s_glBlendEquation;
static PyObject *__pyx_n_b_glBlendEquationEXT;
static PyObject *__pyx_n_b_glBlendEquationSeparate;
static PyObject *__pyx_n_s_glBlendEquationSeparate;
static PyObject *__pyx_n_b_glBlendEquationSeparateEXT;
static PyObject *__pyx_n_b_glBlendFunc;
static PyObject *__pyx_n_s_glBlendFunc;
static PyObject *__pyx_n_b_glBlendFuncSeparate;
static PyObject *__pyx_n_s_glBlendFuncSeparate;
static PyObject *__pyx_n_b_glBlendFuncSeparateEXT;
static PyObject *__pyx_n_b_glBlendFuncSeparateINGR;
static PyObject *__pyx_n_b_glBufferData;
static PyObject *__pyx_n_s_glBufferData;
static PyObject *__pyx_n_b_glBufferDataARB;
static PyObject *__pyx_n_b_glBufferSubData;
static PyObject *__pyx_n_s_glBufferSubData;
static PyObject *__pyx_n_b_glBufferSubDataARB;
static PyObject *__pyx_n_b_glCheckFramebufferStatus;
static PyObject *__pyx_n_s_glCheckFramebufferStatus;
static PyObject *__pyx_n_b_glCheckFramebufferStatusEXT;
static PyObject *__pyx_n_b_glClear;
static PyObject *__pyx_n_s_glClear;
static PyObject *__pyx_n_b_glClearColor;
static PyObject *__pyx_n_s_glClearColor;
static PyObject *__pyx_n_b_glClearStencil;
static PyObject *__pyx_n_s_glClearStencil;
static PyObject *__pyx_n_b_glColorMask;
static PyObject *__pyx_n_s_glColorMask;
static PyObject *__pyx_n_b_glCompileShader;
static PyObject *__pyx_n_s_glCompileShader;
static PyObject *__pyx_n_b_glCompileShaderARB;
static PyObject *__pyx_n_b_glCompressedTexImage2D;
static PyObject *__pyx_n_s_glCompressedTexImage2D;
static PyObject *__pyx_n_b_glCompressedTexImage2DARB;
static PyObject *__pyx_n_b_glCompressedTexSubImage2D;
static PyObject *__pyx_n_s_glCompressedTexSubImage2D;
static PyObject *__pyx_n_b_glCompressedTexSubImage2DARB;
static PyObject *__pyx_n_b_glCopyTexImage2D;
static PyObject *__pyx_n_s_glCopyTexImage2D;
static PyObject *__pyx_n_b_glCopyTexImage2DEXT;
static PyObject *__pyx_n_b_glCopyTexSubImage2D;
static PyObject *__pyx_n_s_glCopyTexSubImage2D;
static PyObject *__pyx_n_b_glCopyTexSubImage2DEXT;
static PyObject *__pyx_n_b_glCreateProgram;
static PyObject *__pyx_n_s_glCreateProgram;
static PyObject *__pyx_n_b_glCreateProgramObjectARB;
static PyObject *__pyx_n_b_glCreateShader;
static PyObject *__pyx_n_s_glCreateShader;
static PyObject *__pyx_n_b_glCreateShaderObjectARB;
static PyObject *__pyx_n_b_glCullFace;
static PyObject *__pyx_n_s_glCullFace;
static PyObject *__pyx_n_b_glDeleteBuffers;
static PyObject *__pyx_n_s_glDeleteBuffers;
static PyObject *__pyx_n_b_glDeleteBuffersARB;
static PyObject *__pyx_n_b_glDeleteFramebuffers;
static PyObject *__pyx_n_s_glDeleteFramebuffers;
static PyObject *__pyx_n_b_glDeleteFramebuffersEXT;
static PyObject *__pyx_n_b_glDeleteProgram;
static PyObject *__pyx_n_s_glDeleteProgram;
static PyObject *__pyx_n_b_glDeleteRenderbuffers;
static PyObject *__pyx_n_s_glDeleteRenderbuffers;
static PyObject *__pyx_n_b_glDeleteRenderbuffersEXT;
static PyObject *__pyx_n_b_glDeleteShader;
static PyObject *__pyx_n_s_glDeleteShader;
static PyObject *__pyx_n_b_glDeleteTextures;
static PyObject *__pyx_n_s_glDeleteTextures;
static PyObject *__pyx_n_b_glDepthFunc;
static PyObject *__pyx_n_s_glDepthFunc;
static PyObject *__pyx_n_b_glDepthMask;
static PyObject *__pyx_n_s_glDepthMask;
static PyObject *__pyx_n_b_glDetachShader;
static PyObject *__pyx_n_s_glDetachShader;
static PyObject *__pyx_n_b_glDisable;
static PyObject *__pyx_n_s_glDisable;
static PyObject *__pyx_n_b_glDisableVertexAttribArray;
static PyObject *__pyx_n_s_glDisableVertexAttribArray;
static PyObject *__pyx_n_b_glDisableVertexAttribArrayARB;
static PyObject *__pyx_n_b_glDrawArrays;
static PyObject *__pyx_n_s_glDrawArrays;
static PyObject *__pyx_n_b_glDrawArraysEXT;
static PyObject *__pyx_n_b_glDrawElements;
static PyObject *__pyx_n_s_glDrawElements;
static PyObject *__pyx_n_b_glEnable;
static PyObject *__pyx_n_s_glEnable;
static PyObject *__pyx_n_b_glEnableVertexAttribArray;
static PyObject *__pyx_n_s_glEnableVertexAttribArray;
static PyObject *__pyx_n_b_glEnableVertexAttribArrayARB;
static PyObject *__pyx_n_b_glFinish;
static PyObject *__pyx_n_s_glFinish;
static PyObject *__pyx_n_b_glFlush;
static PyObject *__pyx_n_s_glFlush;
static PyObject *__pyx_n_b_glFramebufferRenderbuffer;
static PyObject *__pyx_n_s_glFramebufferRenderbuffer;
static PyObject *__pyx_n_b_glFramebufferRenderbufferEXT;
static PyObject *__pyx_n_b_glFramebufferTexture2D;
static PyObject *__pyx_n_s_glFramebufferTexture2D;
static PyObject *__pyx_n_b_glFramebufferTexture2DEXT;
static PyObject *__pyx_n_b_glFrontFace;
static PyObject *__pyx_n_s_glFrontFace;
static PyObject *__pyx_n_b_glGenBuffers;
static PyObject *__pyx_n_s_glGenBuffers;
static PyObject *__pyx_n_b_glGenBuffersARB;
static PyObject *__pyx_n_b_glGenFramebuffers;
static PyObject *__pyx_n_s_glGenFramebuffers;
static PyObject *__pyx_n_b_glGenFramebuffersEXT;
static PyObject *__pyx_n_b_glGenRenderbuffers;
static PyObject *__pyx_n_s_glGenRenderbuffers;
static PyObject *__pyx_n_b_glGenRenderbuffersEXT;
static PyObject *__pyx_n_b_glGenTextures;
static PyObject *__pyx_n_s_glGenTextures;
static PyObject *__pyx_n_b_glGenerateMipmap;
static PyObject *__pyx_n_s_glGenerateMipmap;
static PyObject *__pyx_n_b_glGenerateMipmapEXT;
static PyObject *__pyx_n_b_glGetActiveAttrib;
static PyObject *__pyx_n_s_glGetActiveAttrib;
static PyObject *__pyx_n_b_glGetActiveAttribARB;
static PyObject *__pyx_n_b_glGetActiveUniform;
static PyObject *__pyx_n_s_glGetActiveUniform;
static PyObject *__pyx_n_b_glGetActiveUniformARB;
static PyObject *__pyx_n_b_glGetAttachedShaders;
static PyObject *__pyx_n_s_glGetAttachedShaders;
static PyObject *__pyx_n_b_glGetAttribLocation;
static PyObject *__pyx_n_s_glGetAttribLocation;
static PyObject *__pyx_n_b_glGetAttribLocationARB;
static PyObject *__pyx_n_b_glGetBooleanv;
static PyObject *__pyx_n_s_glGetBooleanv;
static PyObject *__pyx_n_b_glGetBufferParameteriv;
static PyObject *__pyx_n_s_glGetBufferParameteriv;
static PyObject *__pyx_n_b_glGetBufferParameterivARB;
static PyObject *__pyx_n_b_glGetError;
static PyObject *__pyx_n_s_glGetError;
static PyObject *__pyx_n_b_glGetFloatv;
static PyObject *__pyx_n_s_glGetFloatv;
static PyObject *__pyx_n_b_glGetFramebufferAttachmentParame;
static PyObject *__pyx_n_s_glGetFramebufferAttachmentParame;
static PyObject *__pyx_n_b_glGetFramebufferAttachmentParame_2;
static PyObject *__pyx_n_b_glGetIntegerv;
static PyObject *__pyx_n_s_glGetIntegerv;
static PyObject *__pyx_n_b_glGetProgramInfoLog;
static PyObject *__pyx_n_s_glGetProgramInfoLog;
static PyObject *__pyx_n_b_glGetProgramiv;
static PyObject *__pyx_n_s_glGetProgramiv;
static PyObject *__pyx_n_b_glGetRenderbufferParameteriv;
static PyObject *__pyx_n_s_glGetRenderbufferParameteriv;
static PyObject *__pyx_n_b_glGetRenderbufferParameterivEXT;
static PyObject *__pyx_n_b_glGetShaderInfoLog;
static PyObject *__pyx_n_s_glGetShaderInfoLog;
static PyObject *__pyx_n_b_glGetShaderSource;
static PyObject *__pyx_n_s_glGetShaderSource;
static PyObject *__pyx_n_b_glGetShaderSourceARB;
static PyObject *__pyx_n_b_glGetShaderiv;
static PyObject *__pyx_n_s_glGetShaderiv;
static PyObject *__pyx_n_b_glGetString;
static PyObject *__pyx_n_s_glGetString;
static PyObject *__pyx_n_b_glGetTexParameterfv;
static PyObject *__pyx_n_s_glGetTexParameterfv;
static PyObject *__pyx_n_b_glGetTexParameteriv;
static PyObject *__pyx_n_s_glGetTexParameteriv;
static PyObject *__pyx_n_b_glGetUniformLocation;
static PyObject *__pyx_n_s_glGetUniformLocation;
static PyObject *__pyx_n_b_glGetUniformLocationARB;
static PyObject *__pyx_n_b_glGetUniformfv;
static PyObject *__pyx_n_s_glGetUniformfv;
static PyObject *__pyx_n_b_glGetUniformfvARB;
static PyObject *__pyx_n_b_glGetUniformiv;
static PyObject *__pyx_n_s_glGetUniformiv;
static PyObject *__pyx_n_b_glGetUniformivARB;
static PyObject *__pyx_n_b_glGetVertexAttribPointerv;
static PyObject *__pyx_n_s_glGetVertexAttribPointerv;
static PyObject *__pyx_n_b_glGetVertexAttribPointervARB;
static PyObject *__pyx_n_b_glGetVertexAttribPointervNV;
static PyObject *__pyx_n_b_glGetVertexAttribfv;
static PyObject *__pyx_n_s_glGetVertexAttribfv;
static PyObject *__pyx_n_b_glGetVertexAttribfvARB;
static PyObject *__pyx_n_b_glGetVertexAttribfvNV;
static PyObject *__pyx_n_b_glGetVertexAttribiv;
static PyObject *__pyx_n_s_glGetVertexAttribiv;
static PyObject *__pyx_n_b_glGetVertexAttribivARB;
static PyObject *__pyx_n_b_glGetVertexAttribivNV;
static PyObject *__pyx_n_b_glHint;
static PyObject *__pyx_n_s_glHint;
static PyObject *__pyx_n_b_glIsBuffer;
static PyObject *__pyx_n_s_glIsBuffer;
static PyObject *__pyx_n_b_glIsBufferARB;
static PyObject *__pyx_n_b_glIsEnabled;
static PyObject *__pyx_n_s_glIsEnabled;
static PyObject *__pyx_n_b_glIsFramebuffer;
static PyObject *__pyx_n_s_glIsFramebuffer;
static PyObject *__pyx_n_b_glIsFramebufferEXT;
static PyObject *__pyx_n_b_glIsProgram;
static PyObject *__pyx_n_s_glIsProgram;
static PyObject *__pyx_n_b_glIsRenderbuffer;
static PyObject *__pyx_n_s_glIsRenderbuffer;
static PyObject *__pyx_n_b_glIsRenderbufferEXT;
static PyObject *__pyx_n_b_glIsShader;
static PyObject *__pyx_n_s_glIsShader;
static PyObject *__pyx_n_b_glIsTexture;
static PyObject *__pyx_n_s_glIsTexture;
static PyObject *__pyx_n_b_glLineWidth;
static PyObject *__pyx_n_s_glLineWidth;
static PyObject *__pyx_n_b_glLinkProgram;
static PyObject *__pyx_n_s_glLinkProgram;
static PyObject *__pyx_n_b_glLinkProgramARB;
static PyObject *__pyx_n_b_glPixelStorei;
static PyObject *__pyx_n_s_glPixelStorei;
static PyObject *__pyx_n_b_glPolygonOffset;
static PyObject *__pyx_n_s_glPolygonOffset;
static PyObject *__pyx_n_b_glReadPixels;
static PyObject *__pyx_n_s_glReadPixels;
static PyObject *__pyx_n_b_glRenderbufferStorage;
static PyObject *__pyx_n_s_glRenderbufferStorage;
static PyObject *__pyx_n_b_glRenderbufferStorageEXT;
static PyObject *__pyx_n_b_glSampleCoverage;
static PyObject *__pyx_n_s_glSampleCoverage;
static PyObject *__pyx_n_b_glSampleCoverageARB;
static PyObject *__pyx_n_b_glScissor;
static PyObject *__pyx_n_s_glScissor;
static PyObject *__pyx_n_b_glShaderSource;
static PyObject *__pyx_n_s_glShaderSource;
static PyObject *__pyx_n_b_glShaderSourceARB;
static PyObject *__pyx_n_b_glStencilFunc;
static PyObject *__pyx_n_s_glStencilFunc;
static PyObject *__pyx_n_b_glStencilFuncSeparate;
static PyObject *__pyx_n_s_glStencilFuncSeparate;
static PyObject *__pyx_n_b_glStencilMask;
static PyObject *__pyx_n_s_glStencilMask;
static PyObject *__pyx_n_b_glStencilMaskSeparate;
static PyObject *__pyx_n_s_glStencilMaskSeparate;
static PyObject *__pyx_n_b_glStencilOp;
static PyObject *__pyx_n_s_glStencilOp;
static PyObject *__pyx_n_b_glStencilOpSeparate;
static PyObject *__pyx_n_s_glStencilOpSeparate;
static PyObject *__pyx_n_b_glStencilOpSeparateATI;
static PyObject *__pyx_n_b_glTexImage2D;
static PyObject *__pyx_n_s_glTexImage2D;
static PyObject *__pyx_n_b_glTexParameterf;
static PyObject *__pyx_n_s_glTexParameterf;
static PyObject *__pyx_n_b_glTexParameterfv;
static PyObject *__pyx_n_s_glTexParameterfv;
static PyObject *__pyx_n_b_glTexParameteri;
static PyObject *__pyx_n_s_glTexParameteri;
static PyObject *__pyx_n_b_glTexParameteriv;
static PyObject *__pyx_n_s_glTexParameteriv;
static PyObject *__pyx_n_b_glTexSubImage2D;
static PyObject *__pyx_n_s_glTexSubImage2D;
static PyObject *__pyx_n_b_glTexSubImage2DEXT;
static PyObject *__pyx_n_b_glUniform1f;
static PyObject *__pyx_n_s_glUniform1f;
static PyObject *__pyx_n_b_glUniform1fARB;
static PyObject *__pyx_n_b_glUniform1fv;
static PyObject *__pyx_n_s_glUniform1fv;
static PyObject *__pyx_n_b_glUniform1fvARB;
static PyObject *__pyx_n_b_glUniform1i;
static PyObject *__pyx_n_s_glUniform1i;
static PyObject *__pyx_n_b_glUniform1iARB;
static PyObject *__pyx_n_b_glUniform1iv;
static PyObject *__pyx_n_s_glUniform1iv;
static PyObject *__pyx_n_b_glUniform1ivARB;
static PyObject *__pyx_n_b_glUniform2f;
static PyObject *__pyx_n_s_glUniform2f;
static PyObject *__pyx_n_b_glUniform2fARB;
static PyObject *__pyx_n_b_glUniform2fv;
static PyObject *__pyx_n_s_glUniform2fv;
static PyObject *__pyx_n_b_glUniform2fvARB;
static PyObject *__pyx_n_b_glUniform2i;
static PyObject *__pyx_n_s_glUniform2i;
static PyObject *__pyx_n_b_glUniform2iARB;
static PyObject *__pyx_n_b_glUniform2iv;
static PyObject *__pyx_n_s_glUniform2iv;
static PyObject *__pyx_n_b_glUniform2ivARB;
static PyObject *__pyx_n_b_glUniform3f;
static PyObject *__pyx_n_s_glUniform3f;
static PyObject *__pyx_n_b_glUniform3fARB;
static PyObject *__pyx_n_b_glUniform3fv;
static PyObject *__pyx_n_s_glUniform3fv;
static PyObject *__pyx_n_b_glUniform3fvARB;
static PyObject *__pyx_n_b_glUniform3i;
static PyObject *__pyx_n_s_glUniform3i;
static PyObject *__pyx_n_b_glUniform3iARB;
static PyObject *__pyx_n_b_glUniform3iv;
static PyObject *__pyx_n_s_glUniform3iv;
static PyObject *__pyx_n_b_glUniform3ivARB;
static PyObject *__pyx_n_b_glUniform4f;
static PyObject *__pyx_n_s_glUniform4f;
static PyObject *__pyx_n_b_glUniform4fARB;
static PyObject *__pyx_n_b_glUniform4fv;
static PyObject *__pyx_n_s_glUniform4fv;
static PyObject *__pyx_n_b_glUniform4fvARB;
static PyObject *__pyx_n_b_glUniform4i;
static PyObject *__pyx_n_s_glUniform4i;
static PyObject *__pyx_n_b_glUniform4iARB;
static PyObject *__pyx_n_b_glUniform4iv;
static PyObject *__pyx_n_s_glUniform4iv;
static PyObject *__pyx_n_b_glUniform4ivARB;
static PyObject *__pyx_n_b_glUniformMatrix2fv;
static PyObject *__pyx_n_s_glUniformMatrix2fv;
static PyObject *__pyx_n_b_glUniformMatrix2fvARB;
static PyObject *__pyx_n_b_glUniformMatrix3fv;
static PyObject *__pyx_n_s_glUniformMatrix3fv;
static PyObject *__pyx_n_b_glUniformMatrix3fvARB;
static PyObject *__pyx_n_b_glUniformMatrix4fv;
static PyObject *__pyx_n_s_glUniformMatrix4fv;
static PyObject *__pyx_n_b_glUniformMatrix4fvARB;
static PyObject *__pyx_n_b_glUseProgram;
static PyObject *__pyx_n_s_glUseProgram;
static PyObject *__pyx_n_b_glUseProgramObjectARB;
static PyObject *__pyx_n_b_glValidateProgram;
static PyObject *__pyx_n_s_glValidateProgram;
static PyObject *__pyx_n_b_glValidateProgramARB;
static PyObject *__pyx_n_b_glVertexAttrib1f;
static PyObject *__pyx_n_s_glVertexAttrib1f;
static PyObject *__pyx_n_b_glVertexAttrib1fARB;
static PyObject *__pyx_n_b_glVertexAttrib1fNV;
static PyObject *__pyx_n_b_glVertexAttrib1fv;
static PyObject *__pyx_n_s_glVertexAttrib1fv;
static PyObject *__pyx_n_b_glVertexAttrib1fvARB;
static PyObject *__pyx_n_b_glVertexAttrib1fvNV;
static PyObject *__pyx_n_b_glVertexAttrib2f;
static PyObject *__pyx_n_s_glVertexAttrib2f;
static PyObject *__pyx_n_b_glVertexAttrib2fARB;
static PyObject *__pyx_n_b_glVertexAttrib2fNV;
static PyObject *__pyx_n_b_glVertexAttrib2fv;
static PyObject *__pyx_n_s_glVertexAttrib2fv;
static PyObject *__pyx_n_b_glVertexAttrib2fvARB;
static PyObject *__pyx_n_b_glVertexAttrib2fvNV;
static PyObject *__pyx_n_b_glVertexAttrib3f;
static PyObject *__pyx_n_s_glVertexAttrib3f;
static PyObject *__pyx_n_b_glVertexAttrib3fARB;
static PyObject *__pyx_n_b_glVertexAttrib3fNV;
static PyObject *__pyx_n_b_glVertexAttrib3fv;
static PyObject *__pyx_n_s_glVertexAttrib3fv;
static PyObject *__pyx_n_b_glVertexAttrib3fvARB;
static PyObject *__pyx_n_b_glVertexAttrib3fvNV;
static PyObject *__pyx_n_b_glVertexAttrib4f;
static PyObject *__pyx_n_s_glVertexAttrib4f;
static PyObject *__pyx_n_b_glVertexAttrib4fARB;
static PyObject *__pyx_n_b_glVertexAttrib4fNV;
static PyObject *__pyx_n_b_glVertexAttrib4fv;
static PyObject *__pyx_n_s_glVertexAttrib4fv;
static PyObject *__pyx_n_b_glVertexAttrib4fvARB;
static PyObject *__pyx_n_b_glVertexAttrib4fvNV;
static PyObject *__pyx_n_b_glVertexAttribPointer;
static PyObject *__pyx_n_s_glVertexAttribPointer;
static PyObject *__pyx_n_b_glVertexAttribPointerARB;
static PyObject *__pyx_n_b_glViewport;
static PyObject *__pyx_n_s_glViewport;
static PyObject *__pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu;
static PyObject *__pyx_n_s_load;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_kp_s_not_found;
static PyObject *__pyx_n_s_pyx_capi;
static PyObject *__pyx_n_s_renpy_gl2_uguugl;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_pf_5renpy_3gl2_6uguugl_load(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_codeobj_;

/* "renpy/gl2/uguugl.pyx":3
 * from sdl2 cimport SDL_GL_GetProcAddress
 * 
 * cdef void *find_gl_command(names):             # <<<<<<<<<<<<<<
 * 
 *     cdef void *rv = NULL
 */

static void *__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(PyObject *__pyx_v_names) {
  void *__pyx_v_rv;
  PyObject *__pyx_v_i = NULL;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  char const *__pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("find_gl_command", 0);

  /* "renpy/gl2/uguugl.pyx":5
 * cdef void *find_gl_command(names):
 * 
 *     cdef void *rv = NULL             # <<<<<<<<<<<<<<
 * 
 *     for i in names:
 */
  __pyx_v_rv = NULL;

  /* "renpy/gl2/uguugl.pyx":7
 *     cdef void *rv = NULL
 * 
 *     for i in names:             # <<<<<<<<<<<<<<
 *         rv = SDL_GL_GetProcAddress(i)
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_names)) || PyTuple_CheckExact(__pyx_v_names)) {
    __pyx_t_1 = __pyx_v_names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_names); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 7, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 7, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 7, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 7, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 7, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 7, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "renpy/gl2/uguugl.pyx":8
 * 
 *     for i in names:
 *         rv = SDL_GL_GetProcAddress(i)             # <<<<<<<<<<<<<<
 * 
 *         if rv != NULL:
 */
    __pyx_t_5 = __Pyx_PyObject_AsString(__pyx_v_i); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 8, __pyx_L1_error)
    __pyx_v_rv = SDL_GL_GetProcAddress(__pyx_t_5);

    /* "renpy/gl2/uguugl.pyx":10
 *         rv = SDL_GL_GetProcAddress(i)
 * 
 *         if rv != NULL:             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
    __pyx_t_6 = ((__pyx_v_rv != NULL) != 0);
    if (__pyx_t_6) {

      /* "renpy/gl2/uguugl.pyx":11
 * 
 *         if rv != NULL:
 *             return rv             # <<<<<<<<<<<<<<
 * 
 *     raise Exception("{} not found.".format(names[0]))
 */
      __pyx_r = __pyx_v_rv;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "renpy/gl2/uguugl.pyx":10
 *         rv = SDL_GL_GetProcAddress(i)
 * 
 *         if rv != NULL:             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
    }

    /* "renpy/gl2/uguugl.pyx":7
 *     cdef void *rv = NULL
 * 
 *     for i in names:             # <<<<<<<<<<<<<<
 *         rv = SDL_GL_GetProcAddress(i)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":13
 *             return rv
 * 
 *     raise Exception("{} not found.".format(names[0]))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_not_found, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_names, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_7};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    {
      __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 13, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_7);
      __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 13, __pyx_L1_error)

  /* "renpy/gl2/uguugl.pyx":3
 * from sdl2 cimport SDL_GL_GetProcAddress
 * 
 * cdef void *find_gl_command(names):             # <<<<<<<<<<<<<<
 * 
 *     cdef void *rv = NULL
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_WriteUnraisable("renpy.gl2.uguugl.find_gl_command", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/uguugl.pyx":428
 * 
 * 
 * def load():             # <<<<<<<<<<<<<<
 * 
 *     global glActiveTexture
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_6uguugl_1load(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_3gl2_6uguugl_1load = {"load", (PyCFunction)__pyx_pw_5renpy_3gl2_6uguugl_1load, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_3gl2_6uguugl_1load(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("load (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_6uguugl_load(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_6uguugl_load(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("load", 0);

  /* "renpy/gl2/uguugl.pyx":431
 * 
 *     global glActiveTexture
 *     glActiveTexture = <glActiveTexture_type> find_gl_command([b'glActiveTexture', b'glActiveTextureARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glAttachShader
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glActiveTexture);
  __Pyx_GIVEREF(__pyx_n_b_glActiveTexture);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glActiveTexture);
  __Pyx_INCREF(__pyx_n_b_glActiveTextureARB);
  __Pyx_GIVEREF(__pyx_n_b_glActiveTextureARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glActiveTextureARB);
  __pyx_v_5renpy_3gl2_6uguugl_glActiveTexture = ((__pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":434
 * 
 *     global glAttachShader
 *     glAttachShader = <glAttachShader_type> find_gl_command([b'glAttachShader'])             # <<<<<<<<<<<<<<
 * 
 *     global glBindAttribLocation
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glAttachShader);
  __Pyx_GIVEREF(__pyx_n_b_glAttachShader);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glAttachShader);
  __pyx_v_5renpy_3gl2_6uguugl_glAttachShader = ((__pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":437
 * 
 *     global glBindAttribLocation
 *     glBindAttribLocation = <glBindAttribLocation_type> find_gl_command([b'glBindAttribLocation', b'glBindAttribLocationARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glBindBuffer
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBindAttribLocation);
  __Pyx_GIVEREF(__pyx_n_b_glBindAttribLocation);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBindAttribLocation);
  __Pyx_INCREF(__pyx_n_b_glBindAttribLocationARB);
  __Pyx_GIVEREF(__pyx_n_b_glBindAttribLocationARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBindAttribLocationARB);
  __pyx_v_5renpy_3gl2_6uguugl_glBindAttribLocation = ((__pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":440
 * 
 *     global glBindBuffer
 *     glBindBuffer = <glBindBuffer_type> find_gl_command([b'glBindBuffer', b'glBindBufferARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glBindFramebuffer
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBindBuffer);
  __Pyx_GIVEREF(__pyx_n_b_glBindBuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBindBuffer);
  __Pyx_INCREF(__pyx_n_b_glBindBufferARB);
  __Pyx_GIVEREF(__pyx_n_b_glBindBufferARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBindBufferARB);
  __pyx_v_5renpy_3gl2_6uguugl_glBindBuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":443
 * 
 *     global glBindFramebuffer
 *     glBindFramebuffer = <glBindFramebuffer_type> find_gl_command([b'glBindFramebuffer'])             # <<<<<<<<<<<<<<
 * 
 *     global glBindRenderbuffer
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBindFramebuffer);
  __Pyx_GIVEREF(__pyx_n_b_glBindFramebuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBindFramebuffer);
  __pyx_v_5renpy_3gl2_6uguugl_glBindFramebuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":446
 * 
 *     global glBindRenderbuffer
 *     glBindRenderbuffer = <glBindRenderbuffer_type> find_gl_command([b'glBindRenderbuffer'])             # <<<<<<<<<<<<<<
 * 
 *     global glBindTexture
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBindRenderbuffer);
  __Pyx_GIVEREF(__pyx_n_b_glBindRenderbuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBindRenderbuffer);
  __pyx_v_5renpy_3gl2_6uguugl_glBindRenderbuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":449
 * 
 *     global glBindTexture
 *     glBindTexture = <glBindTexture_type> find_gl_command([b'glBindTexture', b'glBindTextureEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glBlendColor
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBindTexture);
  __Pyx_GIVEREF(__pyx_n_b_glBindTexture);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBindTexture);
  __Pyx_INCREF(__pyx_n_b_glBindTextureEXT);
  __Pyx_GIVEREF(__pyx_n_b_glBindTextureEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBindTextureEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glBindTexture = ((__pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":452
 * 
 *     global glBlendColor
 *     glBlendColor = <glBlendColor_type> find_gl_command([b'glBlendColor', b'glBlendColorEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glBlendEquation
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBlendColor);
  __Pyx_GIVEREF(__pyx_n_b_glBlendColor);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBlendColor);
  __Pyx_INCREF(__pyx_n_b_glBlendColorEXT);
  __Pyx_GIVEREF(__pyx_n_b_glBlendColorEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBlendColorEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glBlendColor = ((__pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":455
 * 
 *     global glBlendEquation
 *     glBlendEquation = <glBlendEquation_type> find_gl_command([b'glBlendEquation', b'glBlendEquationEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glBlendEquationSeparate
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBlendEquation);
  __Pyx_GIVEREF(__pyx_n_b_glBlendEquation);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBlendEquation);
  __Pyx_INCREF(__pyx_n_b_glBlendEquationEXT);
  __Pyx_GIVEREF(__pyx_n_b_glBlendEquationEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBlendEquationEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glBlendEquation = ((__pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":458
 * 
 *     global glBlendEquationSeparate
 *     glBlendEquationSeparate = <glBlendEquationSeparate_type> find_gl_command([b'glBlendEquationSeparate', b'glBlendEquationSeparateEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glBlendFunc
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBlendEquationSeparate);
  __Pyx_GIVEREF(__pyx_n_b_glBlendEquationSeparate);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBlendEquationSeparate);
  __Pyx_INCREF(__pyx_n_b_glBlendEquationSeparateEXT);
  __Pyx_GIVEREF(__pyx_n_b_glBlendEquationSeparateEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBlendEquationSeparateEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glBlendEquationSeparate = ((__pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":461
 * 
 *     global glBlendFunc
 *     glBlendFunc = <glBlendFunc_type> find_gl_command([b'glBlendFunc'])             # <<<<<<<<<<<<<<
 * 
 *     global glBlendFuncSeparate
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBlendFunc);
  __Pyx_GIVEREF(__pyx_n_b_glBlendFunc);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBlendFunc);
  __pyx_v_5renpy_3gl2_6uguugl_glBlendFunc = ((__pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":464
 * 
 *     global glBlendFuncSeparate
 *     glBlendFuncSeparate = <glBlendFuncSeparate_type> find_gl_command([b'glBlendFuncSeparate', b'glBlendFuncSeparateEXT', b'glBlendFuncSeparateINGR'])             # <<<<<<<<<<<<<<
 * 
 *     global glBufferData
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBlendFuncSeparate);
  __Pyx_GIVEREF(__pyx_n_b_glBlendFuncSeparate);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBlendFuncSeparate);
  __Pyx_INCREF(__pyx_n_b_glBlendFuncSeparateEXT);
  __Pyx_GIVEREF(__pyx_n_b_glBlendFuncSeparateEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBlendFuncSeparateEXT);
  __Pyx_INCREF(__pyx_n_b_glBlendFuncSeparateINGR);
  __Pyx_GIVEREF(__pyx_n_b_glBlendFuncSeparateINGR);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glBlendFuncSeparateINGR);
  __pyx_v_5renpy_3gl2_6uguugl_glBlendFuncSeparate = ((__pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":467
 * 
 *     global glBufferData
 *     glBufferData = <glBufferData_type> find_gl_command([b'glBufferData', b'glBufferDataARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glBufferSubData
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBufferData);
  __Pyx_GIVEREF(__pyx_n_b_glBufferData);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBufferData);
  __Pyx_INCREF(__pyx_n_b_glBufferDataARB);
  __Pyx_GIVEREF(__pyx_n_b_glBufferDataARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBufferDataARB);
  __pyx_v_5renpy_3gl2_6uguugl_glBufferData = ((__pyx_t_5renpy_3gl2_6uguugl_glBufferData_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":470
 * 
 *     global glBufferSubData
 *     glBufferSubData = <glBufferSubData_type> find_gl_command([b'glBufferSubData', b'glBufferSubDataARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glCheckFramebufferStatus
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glBufferSubData);
  __Pyx_GIVEREF(__pyx_n_b_glBufferSubData);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glBufferSubData);
  __Pyx_INCREF(__pyx_n_b_glBufferSubDataARB);
  __Pyx_GIVEREF(__pyx_n_b_glBufferSubDataARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glBufferSubDataARB);
  __pyx_v_5renpy_3gl2_6uguugl_glBufferSubData = ((__pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":473
 * 
 *     global glCheckFramebufferStatus
 *     glCheckFramebufferStatus = <glCheckFramebufferStatus_type> find_gl_command([b'glCheckFramebufferStatus', b'glCheckFramebufferStatusEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glClear
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCheckFramebufferStatus);
  __Pyx_GIVEREF(__pyx_n_b_glCheckFramebufferStatus);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCheckFramebufferStatus);
  __Pyx_INCREF(__pyx_n_b_glCheckFramebufferStatusEXT);
  __Pyx_GIVEREF(__pyx_n_b_glCheckFramebufferStatusEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCheckFramebufferStatusEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glCheckFramebufferStatus = ((__pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":476
 * 
 *     global glClear
 *     glClear = <glClear_type> find_gl_command([b'glClear'])             # <<<<<<<<<<<<<<
 * 
 *     global glClearColor
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glClear);
  __Pyx_GIVEREF(__pyx_n_b_glClear);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glClear);
  __pyx_v_5renpy_3gl2_6uguugl_glClear = ((__pyx_t_5renpy_3gl2_6uguugl_glClear_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":479
 * 
 *     global glClearColor
 *     glClearColor = <glClearColor_type> find_gl_command([b'glClearColor'])             # <<<<<<<<<<<<<<
 * 
 *     global glClearStencil
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glClearColor);
  __Pyx_GIVEREF(__pyx_n_b_glClearColor);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glClearColor);
  __pyx_v_5renpy_3gl2_6uguugl_glClearColor = ((__pyx_t_5renpy_3gl2_6uguugl_glClearColor_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":482
 * 
 *     global glClearStencil
 *     glClearStencil = <glClearStencil_type> find_gl_command([b'glClearStencil'])             # <<<<<<<<<<<<<<
 * 
 *     global glColorMask
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glClearStencil);
  __Pyx_GIVEREF(__pyx_n_b_glClearStencil);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glClearStencil);
  __pyx_v_5renpy_3gl2_6uguugl_glClearStencil = ((__pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":485
 * 
 *     global glColorMask
 *     glColorMask = <glColorMask_type> find_gl_command([b'glColorMask'])             # <<<<<<<<<<<<<<
 * 
 *     global glCompileShader
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glColorMask);
  __Pyx_GIVEREF(__pyx_n_b_glColorMask);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glColorMask);
  __pyx_v_5renpy_3gl2_6uguugl_glColorMask = ((__pyx_t_5renpy_3gl2_6uguugl_glColorMask_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":488
 * 
 *     global glCompileShader
 *     glCompileShader = <glCompileShader_type> find_gl_command([b'glCompileShader', b'glCompileShaderARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glCompressedTexImage2D
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCompileShader);
  __Pyx_GIVEREF(__pyx_n_b_glCompileShader);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCompileShader);
  __Pyx_INCREF(__pyx_n_b_glCompileShaderARB);
  __Pyx_GIVEREF(__pyx_n_b_glCompileShaderARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCompileShaderARB);
  __pyx_v_5renpy_3gl2_6uguugl_glCompileShader = ((__pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":491
 * 
 *     global glCompressedTexImage2D
 *     glCompressedTexImage2D = <glCompressedTexImage2D_type> find_gl_command([b'glCompressedTexImage2D', b'glCompressedTexImage2DARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glCompressedTexSubImage2D
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCompressedTexImage2D);
  __Pyx_GIVEREF(__pyx_n_b_glCompressedTexImage2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCompressedTexImage2D);
  __Pyx_INCREF(__pyx_n_b_glCompressedTexImage2DARB);
  __Pyx_GIVEREF(__pyx_n_b_glCompressedTexImage2DARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCompressedTexImage2DARB);
  __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexImage2D = ((__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":494
 * 
 *     global glCompressedTexSubImage2D
 *     glCompressedTexSubImage2D = <glCompressedTexSubImage2D_type> find_gl_command([b'glCompressedTexSubImage2D', b'glCompressedTexSubImage2DARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glCopyTexImage2D
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCompressedTexSubImage2D);
  __Pyx_GIVEREF(__pyx_n_b_glCompressedTexSubImage2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCompressedTexSubImage2D);
  __Pyx_INCREF(__pyx_n_b_glCompressedTexSubImage2DARB);
  __Pyx_GIVEREF(__pyx_n_b_glCompressedTexSubImage2DARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCompressedTexSubImage2DARB);
  __pyx_v_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D = ((__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":497
 * 
 *     global glCopyTexImage2D
 *     glCopyTexImage2D = <glCopyTexImage2D_type> find_gl_command([b'glCopyTexImage2D', b'glCopyTexImage2DEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glCopyTexSubImage2D
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCopyTexImage2D);
  __Pyx_GIVEREF(__pyx_n_b_glCopyTexImage2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCopyTexImage2D);
  __Pyx_INCREF(__pyx_n_b_glCopyTexImage2DEXT);
  __Pyx_GIVEREF(__pyx_n_b_glCopyTexImage2DEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCopyTexImage2DEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glCopyTexImage2D = ((__pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":500
 * 
 *     global glCopyTexSubImage2D
 *     glCopyTexSubImage2D = <glCopyTexSubImage2D_type> find_gl_command([b'glCopyTexSubImage2D', b'glCopyTexSubImage2DEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glCreateProgram
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCopyTexSubImage2D);
  __Pyx_GIVEREF(__pyx_n_b_glCopyTexSubImage2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCopyTexSubImage2D);
  __Pyx_INCREF(__pyx_n_b_glCopyTexSubImage2DEXT);
  __Pyx_GIVEREF(__pyx_n_b_glCopyTexSubImage2DEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCopyTexSubImage2DEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glCopyTexSubImage2D = ((__pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":503
 * 
 *     global glCreateProgram
 *     glCreateProgram = <glCreateProgram_type> find_gl_command([b'glCreateProgram', b'glCreateProgramObjectARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glCreateShader
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCreateProgram);
  __Pyx_GIVEREF(__pyx_n_b_glCreateProgram);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCreateProgram);
  __Pyx_INCREF(__pyx_n_b_glCreateProgramObjectARB);
  __Pyx_GIVEREF(__pyx_n_b_glCreateProgramObjectARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCreateProgramObjectARB);
  __pyx_v_5renpy_3gl2_6uguugl_glCreateProgram = ((__pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":506
 * 
 *     global glCreateShader
 *     glCreateShader = <glCreateShader_type> find_gl_command([b'glCreateShader', b'glCreateShaderObjectARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glCullFace
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCreateShader);
  __Pyx_GIVEREF(__pyx_n_b_glCreateShader);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCreateShader);
  __Pyx_INCREF(__pyx_n_b_glCreateShaderObjectARB);
  __Pyx_GIVEREF(__pyx_n_b_glCreateShaderObjectARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glCreateShaderObjectARB);
  __pyx_v_5renpy_3gl2_6uguugl_glCreateShader = ((__pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":509
 * 
 *     global glCullFace
 *     glCullFace = <glCullFace_type> find_gl_command([b'glCullFace'])             # <<<<<<<<<<<<<<
 * 
 *     global glDeleteBuffers
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glCullFace);
  __Pyx_GIVEREF(__pyx_n_b_glCullFace);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glCullFace);
  __pyx_v_5renpy_3gl2_6uguugl_glCullFace = ((__pyx_t_5renpy_3gl2_6uguugl_glCullFace_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":512
 * 
 *     global glDeleteBuffers
 *     glDeleteBuffers = <glDeleteBuffers_type> find_gl_command([b'glDeleteBuffers', b'glDeleteBuffersARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glDeleteFramebuffers
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDeleteBuffers);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteBuffers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDeleteBuffers);
  __Pyx_INCREF(__pyx_n_b_glDeleteBuffersARB);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteBuffersARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glDeleteBuffersARB);
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteBuffers = ((__pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":515
 * 
 *     global glDeleteFramebuffers
 *     glDeleteFramebuffers = <glDeleteFramebuffers_type> find_gl_command([b'glDeleteFramebuffers', b'glDeleteFramebuffersEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glDeleteProgram
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDeleteFramebuffers);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteFramebuffers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDeleteFramebuffers);
  __Pyx_INCREF(__pyx_n_b_glDeleteFramebuffersEXT);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteFramebuffersEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glDeleteFramebuffersEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteFramebuffers = ((__pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":518
 * 
 *     global glDeleteProgram
 *     glDeleteProgram = <glDeleteProgram_type> find_gl_command([b'glDeleteProgram'])             # <<<<<<<<<<<<<<
 * 
 *     global glDeleteRenderbuffers
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDeleteProgram);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteProgram);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDeleteProgram);
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteProgram = ((__pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":521
 * 
 *     global glDeleteRenderbuffers
 *     glDeleteRenderbuffers = <glDeleteRenderbuffers_type> find_gl_command([b'glDeleteRenderbuffers', b'glDeleteRenderbuffersEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glDeleteShader
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDeleteRenderbuffers);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteRenderbuffers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDeleteRenderbuffers);
  __Pyx_INCREF(__pyx_n_b_glDeleteRenderbuffersEXT);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteRenderbuffersEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glDeleteRenderbuffersEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteRenderbuffers = ((__pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":524
 * 
 *     global glDeleteShader
 *     glDeleteShader = <glDeleteShader_type> find_gl_command([b'glDeleteShader'])             # <<<<<<<<<<<<<<
 * 
 *     global glDeleteTextures
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDeleteShader);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteShader);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDeleteShader);
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteShader = ((__pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":527
 * 
 *     global glDeleteTextures
 *     glDeleteTextures = <glDeleteTextures_type> find_gl_command([b'glDeleteTextures'])             # <<<<<<<<<<<<<<
 * 
 *     global glDepthFunc
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDeleteTextures);
  __Pyx_GIVEREF(__pyx_n_b_glDeleteTextures);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDeleteTextures);
  __pyx_v_5renpy_3gl2_6uguugl_glDeleteTextures = ((__pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":530
 * 
 *     global glDepthFunc
 *     glDepthFunc = <glDepthFunc_type> find_gl_command([b'glDepthFunc'])             # <<<<<<<<<<<<<<
 * 
 *     global glDepthMask
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDepthFunc);
  __Pyx_GIVEREF(__pyx_n_b_glDepthFunc);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDepthFunc);
  __pyx_v_5renpy_3gl2_6uguugl_glDepthFunc = ((__pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":533
 * 
 *     global glDepthMask
 *     glDepthMask = <glDepthMask_type> find_gl_command([b'glDepthMask'])             # <<<<<<<<<<<<<<
 * 
 *     global glDetachShader
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDepthMask);
  __Pyx_GIVEREF(__pyx_n_b_glDepthMask);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDepthMask);
  __pyx_v_5renpy_3gl2_6uguugl_glDepthMask = ((__pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":536
 * 
 *     global glDetachShader
 *     glDetachShader = <glDetachShader_type> find_gl_command([b'glDetachShader'])             # <<<<<<<<<<<<<<
 * 
 *     global glDisable
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDetachShader);
  __Pyx_GIVEREF(__pyx_n_b_glDetachShader);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDetachShader);
  __pyx_v_5renpy_3gl2_6uguugl_glDetachShader = ((__pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":539
 * 
 *     global glDisable
 *     glDisable = <glDisable_type> find_gl_command([b'glDisable'])             # <<<<<<<<<<<<<<
 * 
 *     global glDisableVertexAttribArray
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDisable);
  __Pyx_GIVEREF(__pyx_n_b_glDisable);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDisable);
  __pyx_v_5renpy_3gl2_6uguugl_glDisable = ((__pyx_t_5renpy_3gl2_6uguugl_glDisable_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":542
 * 
 *     global glDisableVertexAttribArray
 *     glDisableVertexAttribArray = <glDisableVertexAttribArray_type> find_gl_command([b'glDisableVertexAttribArray', b'glDisableVertexAttribArrayARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glDrawArrays
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDisableVertexAttribArray);
  __Pyx_GIVEREF(__pyx_n_b_glDisableVertexAttribArray);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDisableVertexAttribArray);
  __Pyx_INCREF(__pyx_n_b_glDisableVertexAttribArrayARB);
  __Pyx_GIVEREF(__pyx_n_b_glDisableVertexAttribArrayARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glDisableVertexAttribArrayARB);
  __pyx_v_5renpy_3gl2_6uguugl_glDisableVertexAttribArray = ((__pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":545
 * 
 *     global glDrawArrays
 *     glDrawArrays = <glDrawArrays_type> find_gl_command([b'glDrawArrays', b'glDrawArraysEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glDrawElements
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDrawArrays);
  __Pyx_GIVEREF(__pyx_n_b_glDrawArrays);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDrawArrays);
  __Pyx_INCREF(__pyx_n_b_glDrawArraysEXT);
  __Pyx_GIVEREF(__pyx_n_b_glDrawArraysEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glDrawArraysEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glDrawArrays = ((__pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":548
 * 
 *     global glDrawElements
 *     glDrawElements = <glDrawElements_type> find_gl_command([b'glDrawElements'])             # <<<<<<<<<<<<<<
 * 
 *     global glEnable
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 548, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glDrawElements);
  __Pyx_GIVEREF(__pyx_n_b_glDrawElements);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glDrawElements);
  __pyx_v_5renpy_3gl2_6uguugl_glDrawElements = ((__pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":551
 * 
 *     global glEnable
 *     glEnable = <glEnable_type> find_gl_command([b'glEnable'])             # <<<<<<<<<<<<<<
 * 
 *     global glEnableVertexAttribArray
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glEnable);
  __Pyx_GIVEREF(__pyx_n_b_glEnable);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glEnable);
  __pyx_v_5renpy_3gl2_6uguugl_glEnable = ((__pyx_t_5renpy_3gl2_6uguugl_glEnable_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":554
 * 
 *     global glEnableVertexAttribArray
 *     glEnableVertexAttribArray = <glEnableVertexAttribArray_type> find_gl_command([b'glEnableVertexAttribArray', b'glEnableVertexAttribArrayARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glFinish
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glEnableVertexAttribArray);
  __Pyx_GIVEREF(__pyx_n_b_glEnableVertexAttribArray);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glEnableVertexAttribArray);
  __Pyx_INCREF(__pyx_n_b_glEnableVertexAttribArrayARB);
  __Pyx_GIVEREF(__pyx_n_b_glEnableVertexAttribArrayARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glEnableVertexAttribArrayARB);
  __pyx_v_5renpy_3gl2_6uguugl_glEnableVertexAttribArray = ((__pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":557
 * 
 *     global glFinish
 *     glFinish = <glFinish_type> find_gl_command([b'glFinish'])             # <<<<<<<<<<<<<<
 * 
 *     global glFlush
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glFinish);
  __Pyx_GIVEREF(__pyx_n_b_glFinish);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glFinish);
  __pyx_v_5renpy_3gl2_6uguugl_glFinish = ((__pyx_t_5renpy_3gl2_6uguugl_glFinish_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":560
 * 
 *     global glFlush
 *     glFlush = <glFlush_type> find_gl_command([b'glFlush'])             # <<<<<<<<<<<<<<
 * 
 *     global glFramebufferRenderbuffer
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glFlush);
  __Pyx_GIVEREF(__pyx_n_b_glFlush);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glFlush);
  __pyx_v_5renpy_3gl2_6uguugl_glFlush = ((__pyx_t_5renpy_3gl2_6uguugl_glFlush_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":563
 * 
 *     global glFramebufferRenderbuffer
 *     glFramebufferRenderbuffer = <glFramebufferRenderbuffer_type> find_gl_command([b'glFramebufferRenderbuffer', b'glFramebufferRenderbufferEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glFramebufferTexture2D
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glFramebufferRenderbuffer);
  __Pyx_GIVEREF(__pyx_n_b_glFramebufferRenderbuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glFramebufferRenderbuffer);
  __Pyx_INCREF(__pyx_n_b_glFramebufferRenderbufferEXT);
  __Pyx_GIVEREF(__pyx_n_b_glFramebufferRenderbufferEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glFramebufferRenderbufferEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":566
 * 
 *     global glFramebufferTexture2D
 *     glFramebufferTexture2D = <glFramebufferTexture2D_type> find_gl_command([b'glFramebufferTexture2D', b'glFramebufferTexture2DEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glFrontFace
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glFramebufferTexture2D);
  __Pyx_GIVEREF(__pyx_n_b_glFramebufferTexture2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glFramebufferTexture2D);
  __Pyx_INCREF(__pyx_n_b_glFramebufferTexture2DEXT);
  __Pyx_GIVEREF(__pyx_n_b_glFramebufferTexture2DEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glFramebufferTexture2DEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glFramebufferTexture2D = ((__pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":569
 * 
 *     global glFrontFace
 *     glFrontFace = <glFrontFace_type> find_gl_command([b'glFrontFace'])             # <<<<<<<<<<<<<<
 * 
 *     global glGenBuffers
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glFrontFace);
  __Pyx_GIVEREF(__pyx_n_b_glFrontFace);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glFrontFace);
  __pyx_v_5renpy_3gl2_6uguugl_glFrontFace = ((__pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":572
 * 
 *     global glGenBuffers
 *     glGenBuffers = <glGenBuffers_type> find_gl_command([b'glGenBuffers', b'glGenBuffersARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGenFramebuffers
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 572, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGenBuffers);
  __Pyx_GIVEREF(__pyx_n_b_glGenBuffers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGenBuffers);
  __Pyx_INCREF(__pyx_n_b_glGenBuffersARB);
  __Pyx_GIVEREF(__pyx_n_b_glGenBuffersARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGenBuffersARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGenBuffers = ((__pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":575
 * 
 *     global glGenFramebuffers
 *     glGenFramebuffers = <glGenFramebuffers_type> find_gl_command([b'glGenFramebuffers', b'glGenFramebuffersEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glGenRenderbuffers
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGenFramebuffers);
  __Pyx_GIVEREF(__pyx_n_b_glGenFramebuffers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGenFramebuffers);
  __Pyx_INCREF(__pyx_n_b_glGenFramebuffersEXT);
  __Pyx_GIVEREF(__pyx_n_b_glGenFramebuffersEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGenFramebuffersEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glGenFramebuffers = ((__pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":578
 * 
 *     global glGenRenderbuffers
 *     glGenRenderbuffers = <glGenRenderbuffers_type> find_gl_command([b'glGenRenderbuffers', b'glGenRenderbuffersEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glGenTextures
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGenRenderbuffers);
  __Pyx_GIVEREF(__pyx_n_b_glGenRenderbuffers);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGenRenderbuffers);
  __Pyx_INCREF(__pyx_n_b_glGenRenderbuffersEXT);
  __Pyx_GIVEREF(__pyx_n_b_glGenRenderbuffersEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGenRenderbuffersEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glGenRenderbuffers = ((__pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":581
 * 
 *     global glGenTextures
 *     glGenTextures = <glGenTextures_type> find_gl_command([b'glGenTextures'])             # <<<<<<<<<<<<<<
 * 
 *     global glGenerateMipmap
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGenTextures);
  __Pyx_GIVEREF(__pyx_n_b_glGenTextures);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGenTextures);
  __pyx_v_5renpy_3gl2_6uguugl_glGenTextures = ((__pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":584
 * 
 *     global glGenerateMipmap
 *     glGenerateMipmap = <glGenerateMipmap_type> find_gl_command([b'glGenerateMipmap', b'glGenerateMipmapEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetActiveAttrib
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGenerateMipmap);
  __Pyx_GIVEREF(__pyx_n_b_glGenerateMipmap);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGenerateMipmap);
  __Pyx_INCREF(__pyx_n_b_glGenerateMipmapEXT);
  __Pyx_GIVEREF(__pyx_n_b_glGenerateMipmapEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGenerateMipmapEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glGenerateMipmap = ((__pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":587
 * 
 *     global glGetActiveAttrib
 *     glGetActiveAttrib = <glGetActiveAttrib_type> find_gl_command([b'glGetActiveAttrib', b'glGetActiveAttribARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetActiveUniform
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetActiveAttrib);
  __Pyx_GIVEREF(__pyx_n_b_glGetActiveAttrib);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetActiveAttrib);
  __Pyx_INCREF(__pyx_n_b_glGetActiveAttribARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetActiveAttribARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetActiveAttribARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetActiveAttrib = ((__pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":590
 * 
 *     global glGetActiveUniform
 *     glGetActiveUniform = <glGetActiveUniform_type> find_gl_command([b'glGetActiveUniform', b'glGetActiveUniformARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetAttachedShaders
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetActiveUniform);
  __Pyx_GIVEREF(__pyx_n_b_glGetActiveUniform);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetActiveUniform);
  __Pyx_INCREF(__pyx_n_b_glGetActiveUniformARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetActiveUniformARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetActiveUniformARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetActiveUniform = ((__pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":593
 * 
 *     global glGetAttachedShaders
 *     glGetAttachedShaders = <glGetAttachedShaders_type> find_gl_command([b'glGetAttachedShaders'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetAttribLocation
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetAttachedShaders);
  __Pyx_GIVEREF(__pyx_n_b_glGetAttachedShaders);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetAttachedShaders);
  __pyx_v_5renpy_3gl2_6uguugl_glGetAttachedShaders = ((__pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":596
 * 
 *     global glGetAttribLocation
 *     glGetAttribLocation = <glGetAttribLocation_type> find_gl_command([b'glGetAttribLocation', b'glGetAttribLocationARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetBooleanv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetAttribLocation);
  __Pyx_GIVEREF(__pyx_n_b_glGetAttribLocation);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetAttribLocation);
  __Pyx_INCREF(__pyx_n_b_glGetAttribLocationARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetAttribLocationARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetAttribLocationARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetAttribLocation = ((__pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":599
 * 
 *     global glGetBooleanv
 *     glGetBooleanv = <glGetBooleanv_type> find_gl_command([b'glGetBooleanv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetBufferParameteriv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetBooleanv);
  __Pyx_GIVEREF(__pyx_n_b_glGetBooleanv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetBooleanv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetBooleanv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":602
 * 
 *     global glGetBufferParameteriv
 *     glGetBufferParameteriv = <glGetBufferParameteriv_type> find_gl_command([b'glGetBufferParameteriv', b'glGetBufferParameterivARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetError
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetBufferParameteriv);
  __Pyx_GIVEREF(__pyx_n_b_glGetBufferParameteriv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetBufferParameteriv);
  __Pyx_INCREF(__pyx_n_b_glGetBufferParameterivARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetBufferParameterivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetBufferParameterivARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetBufferParameteriv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":605
 * 
 *     global glGetError
 *     glGetError = <glGetError_type> find_gl_command([b'glGetError'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetFloatv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetError);
  __Pyx_GIVEREF(__pyx_n_b_glGetError);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetError);
  __pyx_v_5renpy_3gl2_6uguugl_glGetError = ((__pyx_t_5renpy_3gl2_6uguugl_glGetError_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":608
 * 
 *     global glGetFloatv
 *     glGetFloatv = <glGetFloatv_type> find_gl_command([b'glGetFloatv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetFramebufferAttachmentParameteriv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetFloatv);
  __Pyx_GIVEREF(__pyx_n_b_glGetFloatv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetFloatv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetFloatv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":611
 * 
 *     global glGetFramebufferAttachmentParameteriv
 *     glGetFramebufferAttachmentParameteriv = <glGetFramebufferAttachmentParameteriv_type> find_gl_command([b'glGetFramebufferAttachmentParameteriv', b'glGetFramebufferAttachmentParameterivEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetIntegerv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetFramebufferAttachmentParame);
  __Pyx_GIVEREF(__pyx_n_b_glGetFramebufferAttachmentParame);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetFramebufferAttachmentParame);
  __Pyx_INCREF(__pyx_n_b_glGetFramebufferAttachmentParame_2);
  __Pyx_GIVEREF(__pyx_n_b_glGetFramebufferAttachmentParame_2);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetFramebufferAttachmentParame_2);
  __pyx_v_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":614
 * 
 *     global glGetIntegerv
 *     glGetIntegerv = <glGetIntegerv_type> find_gl_command([b'glGetIntegerv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetProgramInfoLog
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetIntegerv);
  __Pyx_GIVEREF(__pyx_n_b_glGetIntegerv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetIntegerv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetIntegerv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":617
 * 
 *     global glGetProgramInfoLog
 *     glGetProgramInfoLog = <glGetProgramInfoLog_type> find_gl_command([b'glGetProgramInfoLog'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetProgramiv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetProgramInfoLog);
  __Pyx_GIVEREF(__pyx_n_b_glGetProgramInfoLog);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetProgramInfoLog);
  __pyx_v_5renpy_3gl2_6uguugl_glGetProgramInfoLog = ((__pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":620
 * 
 *     global glGetProgramiv
 *     glGetProgramiv = <glGetProgramiv_type> find_gl_command([b'glGetProgramiv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetRenderbufferParameteriv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetProgramiv);
  __Pyx_GIVEREF(__pyx_n_b_glGetProgramiv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetProgramiv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetProgramiv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":623
 * 
 *     global glGetRenderbufferParameteriv
 *     glGetRenderbufferParameteriv = <glGetRenderbufferParameteriv_type> find_gl_command([b'glGetRenderbufferParameteriv', b'glGetRenderbufferParameterivEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetShaderInfoLog
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetRenderbufferParameteriv);
  __Pyx_GIVEREF(__pyx_n_b_glGetRenderbufferParameteriv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetRenderbufferParameteriv);
  __Pyx_INCREF(__pyx_n_b_glGetRenderbufferParameterivEXT);
  __Pyx_GIVEREF(__pyx_n_b_glGetRenderbufferParameterivEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetRenderbufferParameterivEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":626
 * 
 *     global glGetShaderInfoLog
 *     glGetShaderInfoLog = <glGetShaderInfoLog_type> find_gl_command([b'glGetShaderInfoLog'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetShaderSource
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetShaderInfoLog);
  __Pyx_GIVEREF(__pyx_n_b_glGetShaderInfoLog);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetShaderInfoLog);
  __pyx_v_5renpy_3gl2_6uguugl_glGetShaderInfoLog = ((__pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":629
 * 
 *     global glGetShaderSource
 *     glGetShaderSource = <glGetShaderSource_type> find_gl_command([b'glGetShaderSource', b'glGetShaderSourceARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetShaderiv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetShaderSource);
  __Pyx_GIVEREF(__pyx_n_b_glGetShaderSource);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetShaderSource);
  __Pyx_INCREF(__pyx_n_b_glGetShaderSourceARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetShaderSourceARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetShaderSourceARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetShaderSource = ((__pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":632
 * 
 *     global glGetShaderiv
 *     glGetShaderiv = <glGetShaderiv_type> find_gl_command([b'glGetShaderiv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetString
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 632, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetShaderiv);
  __Pyx_GIVEREF(__pyx_n_b_glGetShaderiv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetShaderiv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetShaderiv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":635
 * 
 *     global glGetString
 *     glGetString = <glGetString_type> find_gl_command([b'glGetString'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetTexParameterfv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetString);
  __Pyx_GIVEREF(__pyx_n_b_glGetString);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetString);
  __pyx_v_5renpy_3gl2_6uguugl_glGetString = ((__pyx_t_5renpy_3gl2_6uguugl_glGetString_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":638
 * 
 *     global glGetTexParameterfv
 *     glGetTexParameterfv = <glGetTexParameterfv_type> find_gl_command([b'glGetTexParameterfv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetTexParameteriv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetTexParameterfv);
  __Pyx_GIVEREF(__pyx_n_b_glGetTexParameterfv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetTexParameterfv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameterfv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":641
 * 
 *     global glGetTexParameteriv
 *     glGetTexParameteriv = <glGetTexParameteriv_type> find_gl_command([b'glGetTexParameteriv'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetUniformLocation
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetTexParameteriv);
  __Pyx_GIVEREF(__pyx_n_b_glGetTexParameteriv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetTexParameteriv);
  __pyx_v_5renpy_3gl2_6uguugl_glGetTexParameteriv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":644
 * 
 *     global glGetUniformLocation
 *     glGetUniformLocation = <glGetUniformLocation_type> find_gl_command([b'glGetUniformLocation', b'glGetUniformLocationARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetUniformfv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetUniformLocation);
  __Pyx_GIVEREF(__pyx_n_b_glGetUniformLocation);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetUniformLocation);
  __Pyx_INCREF(__pyx_n_b_glGetUniformLocationARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetUniformLocationARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetUniformLocationARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetUniformLocation = ((__pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":647
 * 
 *     global glGetUniformfv
 *     glGetUniformfv = <glGetUniformfv_type> find_gl_command([b'glGetUniformfv', b'glGetUniformfvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetUniformiv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetUniformfv);
  __Pyx_GIVEREF(__pyx_n_b_glGetUniformfv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetUniformfv);
  __Pyx_INCREF(__pyx_n_b_glGetUniformfvARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetUniformfvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetUniformfvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetUniformfv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":650
 * 
 *     global glGetUniformiv
 *     glGetUniformiv = <glGetUniformiv_type> find_gl_command([b'glGetUniformiv', b'glGetUniformivARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetVertexAttribPointerv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 650, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetUniformiv);
  __Pyx_GIVEREF(__pyx_n_b_glGetUniformiv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetUniformiv);
  __Pyx_INCREF(__pyx_n_b_glGetUniformivARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetUniformivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetUniformivARB);
  __pyx_v_5renpy_3gl2_6uguugl_glGetUniformiv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":653
 * 
 *     global glGetVertexAttribPointerv
 *     glGetVertexAttribPointerv = <glGetVertexAttribPointerv_type> find_gl_command([b'glGetVertexAttribPointerv', b'glGetVertexAttribPointervARB', b'glGetVertexAttribPointervNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetVertexAttribfv
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribPointerv);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribPointerv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetVertexAttribPointerv);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribPointervARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribPointervARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetVertexAttribPointervARB);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribPointervNV);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribPointervNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glGetVertexAttribPointervNV);
  __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":656
 * 
 *     global glGetVertexAttribfv
 *     glGetVertexAttribfv = <glGetVertexAttribfv_type> find_gl_command([b'glGetVertexAttribfv', b'glGetVertexAttribfvARB', b'glGetVertexAttribfvNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glGetVertexAttribiv
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribfv);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribfv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetVertexAttribfv);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribfvARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribfvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetVertexAttribfvARB);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribfvNV);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribfvNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glGetVertexAttribfvNV);
  __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribfv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":659
 * 
 *     global glGetVertexAttribiv
 *     glGetVertexAttribiv = <glGetVertexAttribiv_type> find_gl_command([b'glGetVertexAttribiv', b'glGetVertexAttribivARB', b'glGetVertexAttribivNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glHint
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribiv);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribiv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glGetVertexAttribiv);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribivARB);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glGetVertexAttribivARB);
  __Pyx_INCREF(__pyx_n_b_glGetVertexAttribivNV);
  __Pyx_GIVEREF(__pyx_n_b_glGetVertexAttribivNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glGetVertexAttribivNV);
  __pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribiv = ((__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":662
 * 
 *     global glHint
 *     glHint = <glHint_type> find_gl_command([b'glHint'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsBuffer
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 662, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glHint);
  __Pyx_GIVEREF(__pyx_n_b_glHint);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glHint);
  __pyx_v_5renpy_3gl2_6uguugl_glHint = ((__pyx_t_5renpy_3gl2_6uguugl_glHint_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":665
 * 
 *     global glIsBuffer
 *     glIsBuffer = <glIsBuffer_type> find_gl_command([b'glIsBuffer', b'glIsBufferARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsEnabled
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsBuffer);
  __Pyx_GIVEREF(__pyx_n_b_glIsBuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsBuffer);
  __Pyx_INCREF(__pyx_n_b_glIsBufferARB);
  __Pyx_GIVEREF(__pyx_n_b_glIsBufferARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glIsBufferARB);
  __pyx_v_5renpy_3gl2_6uguugl_glIsBuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":668
 * 
 *     global glIsEnabled
 *     glIsEnabled = <glIsEnabled_type> find_gl_command([b'glIsEnabled'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsFramebuffer
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsEnabled);
  __Pyx_GIVEREF(__pyx_n_b_glIsEnabled);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsEnabled);
  __pyx_v_5renpy_3gl2_6uguugl_glIsEnabled = ((__pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":671
 * 
 *     global glIsFramebuffer
 *     glIsFramebuffer = <glIsFramebuffer_type> find_gl_command([b'glIsFramebuffer', b'glIsFramebufferEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsProgram
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsFramebuffer);
  __Pyx_GIVEREF(__pyx_n_b_glIsFramebuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsFramebuffer);
  __Pyx_INCREF(__pyx_n_b_glIsFramebufferEXT);
  __Pyx_GIVEREF(__pyx_n_b_glIsFramebufferEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glIsFramebufferEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glIsFramebuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":674
 * 
 *     global glIsProgram
 *     glIsProgram = <glIsProgram_type> find_gl_command([b'glIsProgram'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsRenderbuffer
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsProgram);
  __Pyx_GIVEREF(__pyx_n_b_glIsProgram);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsProgram);
  __pyx_v_5renpy_3gl2_6uguugl_glIsProgram = ((__pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":677
 * 
 *     global glIsRenderbuffer
 *     glIsRenderbuffer = <glIsRenderbuffer_type> find_gl_command([b'glIsRenderbuffer', b'glIsRenderbufferEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsShader
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsRenderbuffer);
  __Pyx_GIVEREF(__pyx_n_b_glIsRenderbuffer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsRenderbuffer);
  __Pyx_INCREF(__pyx_n_b_glIsRenderbufferEXT);
  __Pyx_GIVEREF(__pyx_n_b_glIsRenderbufferEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glIsRenderbufferEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glIsRenderbuffer = ((__pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":680
 * 
 *     global glIsShader
 *     glIsShader = <glIsShader_type> find_gl_command([b'glIsShader'])             # <<<<<<<<<<<<<<
 * 
 *     global glIsTexture
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsShader);
  __Pyx_GIVEREF(__pyx_n_b_glIsShader);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsShader);
  __pyx_v_5renpy_3gl2_6uguugl_glIsShader = ((__pyx_t_5renpy_3gl2_6uguugl_glIsShader_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":683
 * 
 *     global glIsTexture
 *     glIsTexture = <glIsTexture_type> find_gl_command([b'glIsTexture'])             # <<<<<<<<<<<<<<
 * 
 *     global glLineWidth
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 683, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glIsTexture);
  __Pyx_GIVEREF(__pyx_n_b_glIsTexture);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glIsTexture);
  __pyx_v_5renpy_3gl2_6uguugl_glIsTexture = ((__pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":686
 * 
 *     global glLineWidth
 *     glLineWidth = <glLineWidth_type> find_gl_command([b'glLineWidth'])             # <<<<<<<<<<<<<<
 * 
 *     global glLinkProgram
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glLineWidth);
  __Pyx_GIVEREF(__pyx_n_b_glLineWidth);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glLineWidth);
  __pyx_v_5renpy_3gl2_6uguugl_glLineWidth = ((__pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":689
 * 
 *     global glLinkProgram
 *     glLinkProgram = <glLinkProgram_type> find_gl_command([b'glLinkProgram', b'glLinkProgramARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glPixelStorei
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glLinkProgram);
  __Pyx_GIVEREF(__pyx_n_b_glLinkProgram);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glLinkProgram);
  __Pyx_INCREF(__pyx_n_b_glLinkProgramARB);
  __Pyx_GIVEREF(__pyx_n_b_glLinkProgramARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glLinkProgramARB);
  __pyx_v_5renpy_3gl2_6uguugl_glLinkProgram = ((__pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":692
 * 
 *     global glPixelStorei
 *     glPixelStorei = <glPixelStorei_type> find_gl_command([b'glPixelStorei'])             # <<<<<<<<<<<<<<
 * 
 *     global glPolygonOffset
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glPixelStorei);
  __Pyx_GIVEREF(__pyx_n_b_glPixelStorei);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glPixelStorei);
  __pyx_v_5renpy_3gl2_6uguugl_glPixelStorei = ((__pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":695
 * 
 *     global glPolygonOffset
 *     glPolygonOffset = <glPolygonOffset_type> find_gl_command([b'glPolygonOffset'])             # <<<<<<<<<<<<<<
 * 
 *     global glReadPixels
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glPolygonOffset);
  __Pyx_GIVEREF(__pyx_n_b_glPolygonOffset);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glPolygonOffset);
  __pyx_v_5renpy_3gl2_6uguugl_glPolygonOffset = ((__pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":698
 * 
 *     global glReadPixels
 *     glReadPixels = <glReadPixels_type> find_gl_command([b'glReadPixels'])             # <<<<<<<<<<<<<<
 * 
 *     global glRenderbufferStorage
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 698, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glReadPixels);
  __Pyx_GIVEREF(__pyx_n_b_glReadPixels);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glReadPixels);
  __pyx_v_5renpy_3gl2_6uguugl_glReadPixels = ((__pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":701
 * 
 *     global glRenderbufferStorage
 *     glRenderbufferStorage = <glRenderbufferStorage_type> find_gl_command([b'glRenderbufferStorage', b'glRenderbufferStorageEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glSampleCoverage
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glRenderbufferStorage);
  __Pyx_GIVEREF(__pyx_n_b_glRenderbufferStorage);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glRenderbufferStorage);
  __Pyx_INCREF(__pyx_n_b_glRenderbufferStorageEXT);
  __Pyx_GIVEREF(__pyx_n_b_glRenderbufferStorageEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glRenderbufferStorageEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glRenderbufferStorage = ((__pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":704
 * 
 *     global glSampleCoverage
 *     glSampleCoverage = <glSampleCoverage_type> find_gl_command([b'glSampleCoverage', b'glSampleCoverageARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glScissor
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 704, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glSampleCoverage);
  __Pyx_GIVEREF(__pyx_n_b_glSampleCoverage);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glSampleCoverage);
  __Pyx_INCREF(__pyx_n_b_glSampleCoverageARB);
  __Pyx_GIVEREF(__pyx_n_b_glSampleCoverageARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glSampleCoverageARB);
  __pyx_v_5renpy_3gl2_6uguugl_glSampleCoverage = ((__pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":707
 * 
 *     global glScissor
 *     glScissor = <glScissor_type> find_gl_command([b'glScissor'])             # <<<<<<<<<<<<<<
 * 
 *     global glShaderSource
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glScissor);
  __Pyx_GIVEREF(__pyx_n_b_glScissor);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glScissor);
  __pyx_v_5renpy_3gl2_6uguugl_glScissor = ((__pyx_t_5renpy_3gl2_6uguugl_glScissor_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":710
 * 
 *     global glShaderSource
 *     glShaderSource = <glShaderSource_type> find_gl_command([b'glShaderSource', b'glShaderSourceARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glStencilFunc
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glShaderSource);
  __Pyx_GIVEREF(__pyx_n_b_glShaderSource);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glShaderSource);
  __Pyx_INCREF(__pyx_n_b_glShaderSourceARB);
  __Pyx_GIVEREF(__pyx_n_b_glShaderSourceARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glShaderSourceARB);
  __pyx_v_5renpy_3gl2_6uguugl_glShaderSource = ((__pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":713
 * 
 *     global glStencilFunc
 *     glStencilFunc = <glStencilFunc_type> find_gl_command([b'glStencilFunc'])             # <<<<<<<<<<<<<<
 * 
 *     global glStencilFuncSeparate
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glStencilFunc);
  __Pyx_GIVEREF(__pyx_n_b_glStencilFunc);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glStencilFunc);
  __pyx_v_5renpy_3gl2_6uguugl_glStencilFunc = ((__pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":716
 * 
 *     global glStencilFuncSeparate
 *     glStencilFuncSeparate = <glStencilFuncSeparate_type> find_gl_command([b'glStencilFuncSeparate'])             # <<<<<<<<<<<<<<
 * 
 *     global glStencilMask
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glStencilFuncSeparate);
  __Pyx_GIVEREF(__pyx_n_b_glStencilFuncSeparate);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glStencilFuncSeparate);
  __pyx_v_5renpy_3gl2_6uguugl_glStencilFuncSeparate = ((__pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":719
 * 
 *     global glStencilMask
 *     glStencilMask = <glStencilMask_type> find_gl_command([b'glStencilMask'])             # <<<<<<<<<<<<<<
 * 
 *     global glStencilMaskSeparate
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glStencilMask);
  __Pyx_GIVEREF(__pyx_n_b_glStencilMask);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glStencilMask);
  __pyx_v_5renpy_3gl2_6uguugl_glStencilMask = ((__pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":722
 * 
 *     global glStencilMaskSeparate
 *     glStencilMaskSeparate = <glStencilMaskSeparate_type> find_gl_command([b'glStencilMaskSeparate'])             # <<<<<<<<<<<<<<
 * 
 *     global glStencilOp
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glStencilMaskSeparate);
  __Pyx_GIVEREF(__pyx_n_b_glStencilMaskSeparate);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glStencilMaskSeparate);
  __pyx_v_5renpy_3gl2_6uguugl_glStencilMaskSeparate = ((__pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":725
 * 
 *     global glStencilOp
 *     glStencilOp = <glStencilOp_type> find_gl_command([b'glStencilOp'])             # <<<<<<<<<<<<<<
 * 
 *     global glStencilOpSeparate
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glStencilOp);
  __Pyx_GIVEREF(__pyx_n_b_glStencilOp);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glStencilOp);
  __pyx_v_5renpy_3gl2_6uguugl_glStencilOp = ((__pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":728
 * 
 *     global glStencilOpSeparate
 *     glStencilOpSeparate = <glStencilOpSeparate_type> find_gl_command([b'glStencilOpSeparate', b'glStencilOpSeparateATI'])             # <<<<<<<<<<<<<<
 * 
 *     global glTexImage2D
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 728, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glStencilOpSeparate);
  __Pyx_GIVEREF(__pyx_n_b_glStencilOpSeparate);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glStencilOpSeparate);
  __Pyx_INCREF(__pyx_n_b_glStencilOpSeparateATI);
  __Pyx_GIVEREF(__pyx_n_b_glStencilOpSeparateATI);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glStencilOpSeparateATI);
  __pyx_v_5renpy_3gl2_6uguugl_glStencilOpSeparate = ((__pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":731
 * 
 *     global glTexImage2D
 *     glTexImage2D = <glTexImage2D_type> find_gl_command([b'glTexImage2D'])             # <<<<<<<<<<<<<<
 * 
 *     global glTexParameterf
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glTexImage2D);
  __Pyx_GIVEREF(__pyx_n_b_glTexImage2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glTexImage2D);
  __pyx_v_5renpy_3gl2_6uguugl_glTexImage2D = ((__pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":734
 * 
 *     global glTexParameterf
 *     glTexParameterf = <glTexParameterf_type> find_gl_command([b'glTexParameterf'])             # <<<<<<<<<<<<<<
 * 
 *     global glTexParameterfv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 734, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glTexParameterf);
  __Pyx_GIVEREF(__pyx_n_b_glTexParameterf);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glTexParameterf);
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameterf = ((__pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":737
 * 
 *     global glTexParameterfv
 *     glTexParameterfv = <glTexParameterfv_type> find_gl_command([b'glTexParameterfv'])             # <<<<<<<<<<<<<<
 * 
 *     global glTexParameteri
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glTexParameterfv);
  __Pyx_GIVEREF(__pyx_n_b_glTexParameterfv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glTexParameterfv);
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameterfv = ((__pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":740
 * 
 *     global glTexParameteri
 *     glTexParameteri = <glTexParameteri_type> find_gl_command([b'glTexParameteri'])             # <<<<<<<<<<<<<<
 * 
 *     global glTexParameteriv
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 740, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glTexParameteri);
  __Pyx_GIVEREF(__pyx_n_b_glTexParameteri);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glTexParameteri);
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameteri = ((__pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":743
 * 
 *     global glTexParameteriv
 *     glTexParameteriv = <glTexParameteriv_type> find_gl_command([b'glTexParameteriv'])             # <<<<<<<<<<<<<<
 * 
 *     global glTexSubImage2D
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glTexParameteriv);
  __Pyx_GIVEREF(__pyx_n_b_glTexParameteriv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glTexParameteriv);
  __pyx_v_5renpy_3gl2_6uguugl_glTexParameteriv = ((__pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":746
 * 
 *     global glTexSubImage2D
 *     glTexSubImage2D = <glTexSubImage2D_type> find_gl_command([b'glTexSubImage2D', b'glTexSubImage2DEXT'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform1f
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 746, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glTexSubImage2D);
  __Pyx_GIVEREF(__pyx_n_b_glTexSubImage2D);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glTexSubImage2D);
  __Pyx_INCREF(__pyx_n_b_glTexSubImage2DEXT);
  __Pyx_GIVEREF(__pyx_n_b_glTexSubImage2DEXT);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glTexSubImage2DEXT);
  __pyx_v_5renpy_3gl2_6uguugl_glTexSubImage2D = ((__pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":749
 * 
 *     global glUniform1f
 *     glUniform1f = <glUniform1f_type> find_gl_command([b'glUniform1f', b'glUniform1fARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform1fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 749, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform1f);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform1f);
  __Pyx_INCREF(__pyx_n_b_glUniform1fARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform1fARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1f = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":752
 * 
 *     global glUniform1fv
 *     glUniform1fv = <glUniform1fv_type> find_gl_command([b'glUniform1fv', b'glUniform1fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform1i
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform1fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform1fv);
  __Pyx_INCREF(__pyx_n_b_glUniform1fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform1fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":755
 * 
 *     global glUniform1i
 *     glUniform1i = <glUniform1i_type> find_gl_command([b'glUniform1i', b'glUniform1iARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform1iv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 755, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform1i);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1i);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform1i);
  __Pyx_INCREF(__pyx_n_b_glUniform1iARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1iARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform1iARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1i = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":758
 * 
 *     global glUniform1iv
 *     glUniform1iv = <glUniform1iv_type> find_gl_command([b'glUniform1iv', b'glUniform1ivARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform2f
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform1iv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1iv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform1iv);
  __Pyx_INCREF(__pyx_n_b_glUniform1ivARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform1ivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform1ivARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform1iv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":761
 * 
 *     global glUniform2f
 *     glUniform2f = <glUniform2f_type> find_gl_command([b'glUniform2f', b'glUniform2fARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform2fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform2f);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform2f);
  __Pyx_INCREF(__pyx_n_b_glUniform2fARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform2fARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2f = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":764
 * 
 *     global glUniform2fv
 *     glUniform2fv = <glUniform2fv_type> find_gl_command([b'glUniform2fv', b'glUniform2fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform2i
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform2fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform2fv);
  __Pyx_INCREF(__pyx_n_b_glUniform2fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform2fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":767
 * 
 *     global glUniform2i
 *     glUniform2i = <glUniform2i_type> find_gl_command([b'glUniform2i', b'glUniform2iARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform2iv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 767, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform2i);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2i);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform2i);
  __Pyx_INCREF(__pyx_n_b_glUniform2iARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2iARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform2iARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2i = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":770
 * 
 *     global glUniform2iv
 *     glUniform2iv = <glUniform2iv_type> find_gl_command([b'glUniform2iv', b'glUniform2ivARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform3f
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 770, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform2iv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2iv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform2iv);
  __Pyx_INCREF(__pyx_n_b_glUniform2ivARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform2ivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform2ivARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform2iv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":773
 * 
 *     global glUniform3f
 *     glUniform3f = <glUniform3f_type> find_gl_command([b'glUniform3f', b'glUniform3fARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform3fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 773, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform3f);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform3f);
  __Pyx_INCREF(__pyx_n_b_glUniform3fARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform3fARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3f = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":776
 * 
 *     global glUniform3fv
 *     glUniform3fv = <glUniform3fv_type> find_gl_command([b'glUniform3fv', b'glUniform3fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform3i
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 776, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform3fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform3fv);
  __Pyx_INCREF(__pyx_n_b_glUniform3fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform3fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":779
 * 
 *     global glUniform3i
 *     glUniform3i = <glUniform3i_type> find_gl_command([b'glUniform3i', b'glUniform3iARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform3iv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 779, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform3i);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3i);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform3i);
  __Pyx_INCREF(__pyx_n_b_glUniform3iARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3iARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform3iARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3i = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":782
 * 
 *     global glUniform3iv
 *     glUniform3iv = <glUniform3iv_type> find_gl_command([b'glUniform3iv', b'glUniform3ivARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform4f
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 782, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform3iv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3iv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform3iv);
  __Pyx_INCREF(__pyx_n_b_glUniform3ivARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform3ivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform3ivARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform3iv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":785
 * 
 *     global glUniform4f
 *     glUniform4f = <glUniform4f_type> find_gl_command([b'glUniform4f', b'glUniform4fARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform4fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 785, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform4f);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform4f);
  __Pyx_INCREF(__pyx_n_b_glUniform4fARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform4fARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4f = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":788
 * 
 *     global glUniform4fv
 *     glUniform4fv = <glUniform4fv_type> find_gl_command([b'glUniform4fv', b'glUniform4fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform4i
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform4fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform4fv);
  __Pyx_INCREF(__pyx_n_b_glUniform4fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform4fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":791
 * 
 *     global glUniform4i
 *     glUniform4i = <glUniform4i_type> find_gl_command([b'glUniform4i', b'glUniform4iARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniform4iv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 791, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform4i);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4i);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform4i);
  __Pyx_INCREF(__pyx_n_b_glUniform4iARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4iARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform4iARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4i = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":794
 * 
 *     global glUniform4iv
 *     glUniform4iv = <glUniform4iv_type> find_gl_command([b'glUniform4iv', b'glUniform4ivARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniformMatrix2fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 794, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniform4iv);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4iv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniform4iv);
  __Pyx_INCREF(__pyx_n_b_glUniform4ivARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniform4ivARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniform4ivARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniform4iv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":797
 * 
 *     global glUniformMatrix2fv
 *     glUniformMatrix2fv = <glUniformMatrix2fv_type> find_gl_command([b'glUniformMatrix2fv', b'glUniformMatrix2fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniformMatrix3fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 797, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniformMatrix2fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniformMatrix2fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniformMatrix2fv);
  __Pyx_INCREF(__pyx_n_b_glUniformMatrix2fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniformMatrix2fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniformMatrix2fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix2fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":800
 * 
 *     global glUniformMatrix3fv
 *     glUniformMatrix3fv = <glUniformMatrix3fv_type> find_gl_command([b'glUniformMatrix3fv', b'glUniformMatrix3fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUniformMatrix4fv
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 800, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniformMatrix3fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniformMatrix3fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniformMatrix3fv);
  __Pyx_INCREF(__pyx_n_b_glUniformMatrix3fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniformMatrix3fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniformMatrix3fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix3fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":803
 * 
 *     global glUniformMatrix4fv
 *     glUniformMatrix4fv = <glUniformMatrix4fv_type> find_gl_command([b'glUniformMatrix4fv', b'glUniformMatrix4fvARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glUseProgram
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUniformMatrix4fv);
  __Pyx_GIVEREF(__pyx_n_b_glUniformMatrix4fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUniformMatrix4fv);
  __Pyx_INCREF(__pyx_n_b_glUniformMatrix4fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glUniformMatrix4fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUniformMatrix4fvARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix4fv = ((__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":806
 * 
 *     global glUseProgram
 *     glUseProgram = <glUseProgram_type> find_gl_command([b'glUseProgram', b'glUseProgramObjectARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glValidateProgram
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glUseProgram);
  __Pyx_GIVEREF(__pyx_n_b_glUseProgram);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glUseProgram);
  __Pyx_INCREF(__pyx_n_b_glUseProgramObjectARB);
  __Pyx_GIVEREF(__pyx_n_b_glUseProgramObjectARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glUseProgramObjectARB);
  __pyx_v_5renpy_3gl2_6uguugl_glUseProgram = ((__pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":809
 * 
 *     global glValidateProgram
 *     glValidateProgram = <glValidateProgram_type> find_gl_command([b'glValidateProgram', b'glValidateProgramARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib1f
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glValidateProgram);
  __Pyx_GIVEREF(__pyx_n_b_glValidateProgram);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glValidateProgram);
  __Pyx_INCREF(__pyx_n_b_glValidateProgramARB);
  __Pyx_GIVEREF(__pyx_n_b_glValidateProgramARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glValidateProgramARB);
  __pyx_v_5renpy_3gl2_6uguugl_glValidateProgram = ((__pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":812
 * 
 *     global glVertexAttrib1f
 *     glVertexAttrib1f = <glVertexAttrib1f_type> find_gl_command([b'glVertexAttrib1f', b'glVertexAttrib1fARB', b'glVertexAttrib1fNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib1fv
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib1f);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib1f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib1f);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib1fARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib1fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib1fARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib1fNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib1fNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib1fNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1f = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":815
 * 
 *     global glVertexAttrib1fv
 *     glVertexAttrib1fv = <glVertexAttrib1fv_type> find_gl_command([b'glVertexAttrib1fv', b'glVertexAttrib1fvARB', b'glVertexAttrib1fvNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib2f
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 815, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib1fv);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib1fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib1fv);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib1fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib1fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib1fvARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib1fvNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib1fvNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib1fvNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1fv = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":818
 * 
 *     global glVertexAttrib2f
 *     glVertexAttrib2f = <glVertexAttrib2f_type> find_gl_command([b'glVertexAttrib2f', b'glVertexAttrib2fARB', b'glVertexAttrib2fNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib2fv
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 818, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib2f);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib2f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib2f);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib2fARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib2fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib2fARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib2fNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib2fNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib2fNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2f = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":821
 * 
 *     global glVertexAttrib2fv
 *     glVertexAttrib2fv = <glVertexAttrib2fv_type> find_gl_command([b'glVertexAttrib2fv', b'glVertexAttrib2fvARB', b'glVertexAttrib2fvNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib3f
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib2fv);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib2fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib2fv);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib2fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib2fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib2fvARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib2fvNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib2fvNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib2fvNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2fv = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":824
 * 
 *     global glVertexAttrib3f
 *     glVertexAttrib3f = <glVertexAttrib3f_type> find_gl_command([b'glVertexAttrib3f', b'glVertexAttrib3fARB', b'glVertexAttrib3fNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib3fv
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib3f);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib3f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib3f);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib3fARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib3fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib3fARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib3fNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib3fNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib3fNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3f = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":827
 * 
 *     global glVertexAttrib3fv
 *     glVertexAttrib3fv = <glVertexAttrib3fv_type> find_gl_command([b'glVertexAttrib3fv', b'glVertexAttrib3fvARB', b'glVertexAttrib3fvNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib4f
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 827, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib3fv);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib3fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib3fv);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib3fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib3fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib3fvARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib3fvNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib3fvNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib3fvNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3fv = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":830
 * 
 *     global glVertexAttrib4f
 *     glVertexAttrib4f = <glVertexAttrib4f_type> find_gl_command([b'glVertexAttrib4f', b'glVertexAttrib4fARB', b'glVertexAttrib4fNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttrib4fv
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 830, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib4f);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib4f);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib4f);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib4fARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib4fARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib4fARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib4fNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib4fNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib4fNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4f = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":833
 * 
 *     global glVertexAttrib4fv
 *     glVertexAttrib4fv = <glVertexAttrib4fv_type> find_gl_command([b'glVertexAttrib4fv', b'glVertexAttrib4fvARB', b'glVertexAttrib4fvNV'])             # <<<<<<<<<<<<<<
 * 
 *     global glVertexAttribPointer
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib4fv);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib4fv);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttrib4fv);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib4fvARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib4fvARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttrib4fvARB);
  __Pyx_INCREF(__pyx_n_b_glVertexAttrib4fvNV);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttrib4fvNV);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_b_glVertexAttrib4fvNV);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4fv = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":836
 * 
 *     global glVertexAttribPointer
 *     glVertexAttribPointer = <glVertexAttribPointer_type> find_gl_command([b'glVertexAttribPointer', b'glVertexAttribPointerARB'])             # <<<<<<<<<<<<<<
 * 
 *     global glViewport
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glVertexAttribPointer);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttribPointer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glVertexAttribPointer);
  __Pyx_INCREF(__pyx_n_b_glVertexAttribPointerARB);
  __Pyx_GIVEREF(__pyx_n_b_glVertexAttribPointerARB);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_b_glVertexAttribPointerARB);
  __pyx_v_5renpy_3gl2_6uguugl_glVertexAttribPointer = ((__pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":839
 * 
 *     global glViewport
 *     glViewport = <glViewport_type> find_gl_command([b'glViewport'])             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 839, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_b_glViewport);
  __Pyx_GIVEREF(__pyx_n_b_glViewport);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_b_glViewport);
  __pyx_v_5renpy_3gl2_6uguugl_glViewport = ((__pyx_t_5renpy_3gl2_6uguugl_glViewport_type)__pyx_f_5renpy_3gl2_6uguugl_find_gl_command(__pyx_t_1));
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":428
 * 
 * 
 * def load():             # <<<<<<<<<<<<<<
 * 
 *     global glActiveTexture
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.uguugl.load", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_uguugl(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_uguugl},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "uguugl",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_b_glActiveTexture, __pyx_k_glActiveTexture, sizeof(__pyx_k_glActiveTexture), 0, 0, 0, 1},
  {&__pyx_n_s_glActiveTexture, __pyx_k_glActiveTexture, sizeof(__pyx_k_glActiveTexture), 0, 0, 1, 1},
  {&__pyx_n_b_glActiveTextureARB, __pyx_k_glActiveTextureARB, sizeof(__pyx_k_glActiveTextureARB), 0, 0, 0, 1},
  {&__pyx_n_b_glAttachShader, __pyx_k_glAttachShader, sizeof(__pyx_k_glAttachShader), 0, 0, 0, 1},
  {&__pyx_n_s_glAttachShader, __pyx_k_glAttachShader, sizeof(__pyx_k_glAttachShader), 0, 0, 1, 1},
  {&__pyx_n_b_glBindAttribLocation, __pyx_k_glBindAttribLocation, sizeof(__pyx_k_glBindAttribLocation), 0, 0, 0, 1},
  {&__pyx_n_s_glBindAttribLocation, __pyx_k_glBindAttribLocation, sizeof(__pyx_k_glBindAttribLocation), 0, 0, 1, 1},
  {&__pyx_n_b_glBindAttribLocationARB, __pyx_k_glBindAttribLocationARB, sizeof(__pyx_k_glBindAttribLocationARB), 0, 0, 0, 1},
  {&__pyx_n_b_glBindBuffer, __pyx_k_glBindBuffer, sizeof(__pyx_k_glBindBuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glBindBuffer, __pyx_k_glBindBuffer, sizeof(__pyx_k_glBindBuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glBindBufferARB, __pyx_k_glBindBufferARB, sizeof(__pyx_k_glBindBufferARB), 0, 0, 0, 1},
  {&__pyx_n_b_glBindFramebuffer, __pyx_k_glBindFramebuffer, sizeof(__pyx_k_glBindFramebuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glBindFramebuffer, __pyx_k_glBindFramebuffer, sizeof(__pyx_k_glBindFramebuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glBindRenderbuffer, __pyx_k_glBindRenderbuffer, sizeof(__pyx_k_glBindRenderbuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glBindRenderbuffer, __pyx_k_glBindRenderbuffer, sizeof(__pyx_k_glBindRenderbuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glBindTexture, __pyx_k_glBindTexture, sizeof(__pyx_k_glBindTexture), 0, 0, 0, 1},
  {&__pyx_n_s_glBindTexture, __pyx_k_glBindTexture, sizeof(__pyx_k_glBindTexture), 0, 0, 1, 1},
  {&__pyx_n_b_glBindTextureEXT, __pyx_k_glBindTextureEXT, sizeof(__pyx_k_glBindTextureEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glBlendColor, __pyx_k_glBlendColor, sizeof(__pyx_k_glBlendColor), 0, 0, 0, 1},
  {&__pyx_n_s_glBlendColor, __pyx_k_glBlendColor, sizeof(__pyx_k_glBlendColor), 0, 0, 1, 1},
  {&__pyx_n_b_glBlendColorEXT, __pyx_k_glBlendColorEXT, sizeof(__pyx_k_glBlendColorEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glBlendEquation, __pyx_k_glBlendEquation, sizeof(__pyx_k_glBlendEquation), 0, 0, 0, 1},
  {&__pyx_n_s_glBlendEquation, __pyx_k_glBlendEquation, sizeof(__pyx_k_glBlendEquation), 0, 0, 1, 1},
  {&__pyx_n_b_glBlendEquationEXT, __pyx_k_glBlendEquationEXT, sizeof(__pyx_k_glBlendEquationEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glBlendEquationSeparate, __pyx_k_glBlendEquationSeparate, sizeof(__pyx_k_glBlendEquationSeparate), 0, 0, 0, 1},
  {&__pyx_n_s_glBlendEquationSeparate, __pyx_k_glBlendEquationSeparate, sizeof(__pyx_k_glBlendEquationSeparate), 0, 0, 1, 1},
  {&__pyx_n_b_glBlendEquationSeparateEXT, __pyx_k_glBlendEquationSeparateEXT, sizeof(__pyx_k_glBlendEquationSeparateEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glBlendFunc, __pyx_k_glBlendFunc, sizeof(__pyx_k_glBlendFunc), 0, 0, 0, 1},
  {&__pyx_n_s_glBlendFunc, __pyx_k_glBlendFunc, sizeof(__pyx_k_glBlendFunc), 0, 0, 1, 1},
  {&__pyx_n_b_glBlendFuncSeparate, __pyx_k_glBlendFuncSeparate, sizeof(__pyx_k_glBlendFuncSeparate), 0, 0, 0, 1},
  {&__pyx_n_s_glBlendFuncSeparate, __pyx_k_glBlendFuncSeparate, sizeof(__pyx_k_glBlendFuncSeparate), 0, 0, 1, 1},
  {&__pyx_n_b_glBlendFuncSeparateEXT, __pyx_k_glBlendFuncSeparateEXT, sizeof(__pyx_k_glBlendFuncSeparateEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glBlendFuncSeparateINGR, __pyx_k_glBlendFuncSeparateINGR, sizeof(__pyx_k_glBlendFuncSeparateINGR), 0, 0, 0, 1},
  {&__pyx_n_b_glBufferData, __pyx_k_glBufferData, sizeof(__pyx_k_glBufferData), 0, 0, 0, 1},
  {&__pyx_n_s_glBufferData, __pyx_k_glBufferData, sizeof(__pyx_k_glBufferData), 0, 0, 1, 1},
  {&__pyx_n_b_glBufferDataARB, __pyx_k_glBufferDataARB, sizeof(__pyx_k_glBufferDataARB), 0, 0, 0, 1},
  {&__pyx_n_b_glBufferSubData, __pyx_k_glBufferSubData, sizeof(__pyx_k_glBufferSubData), 0, 0, 0, 1},
  {&__pyx_n_s_glBufferSubData, __pyx_k_glBufferSubData, sizeof(__pyx_k_glBufferSubData), 0, 0, 1, 1},
  {&__pyx_n_b_glBufferSubDataARB, __pyx_k_glBufferSubDataARB, sizeof(__pyx_k_glBufferSubDataARB), 0, 0, 0, 1},
  {&__pyx_n_b_glCheckFramebufferStatus, __pyx_k_glCheckFramebufferStatus, sizeof(__pyx_k_glCheckFramebufferStatus), 0, 0, 0, 1},
  {&__pyx_n_s_glCheckFramebufferStatus, __pyx_k_glCheckFramebufferStatus, sizeof(__pyx_k_glCheckFramebufferStatus), 0, 0, 1, 1},
  {&__pyx_n_b_glCheckFramebufferStatusEXT, __pyx_k_glCheckFramebufferStatusEXT, sizeof(__pyx_k_glCheckFramebufferStatusEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glClear, __pyx_k_glClear, sizeof(__pyx_k_glClear), 0, 0, 0, 1},
  {&__pyx_n_s_glClear, __pyx_k_glClear, sizeof(__pyx_k_glClear), 0, 0, 1, 1},
  {&__pyx_n_b_glClearColor, __pyx_k_glClearColor, sizeof(__pyx_k_glClearColor), 0, 0, 0, 1},
  {&__pyx_n_s_glClearColor, __pyx_k_glClearColor, sizeof(__pyx_k_glClearColor), 0, 0, 1, 1},
  {&__pyx_n_b_glClearStencil, __pyx_k_glClearStencil, sizeof(__pyx_k_glClearStencil), 0, 0, 0, 1},
  {&__pyx_n_s_glClearStencil, __pyx_k_glClearStencil, sizeof(__pyx_k_glClearStencil), 0, 0, 1, 1},
  {&__pyx_n_b_glColorMask, __pyx_k_glColorMask, sizeof(__pyx_k_glColorMask), 0, 0, 0, 1},
  {&__pyx_n_s_glColorMask, __pyx_k_glColorMask, sizeof(__pyx_k_glColorMask), 0, 0, 1, 1},
  {&__pyx_n_b_glCompileShader, __pyx_k_glCompileShader, sizeof(__pyx_k_glCompileShader), 0, 0, 0, 1},
  {&__pyx_n_s_glCompileShader, __pyx_k_glCompileShader, sizeof(__pyx_k_glCompileShader), 0, 0, 1, 1},
  {&__pyx_n_b_glCompileShaderARB, __pyx_k_glCompileShaderARB, sizeof(__pyx_k_glCompileShaderARB), 0, 0, 0, 1},
  {&__pyx_n_b_glCompressedTexImage2D, __pyx_k_glCompressedTexImage2D, sizeof(__pyx_k_glCompressedTexImage2D), 0, 0, 0, 1},
  {&__pyx_n_s_glCompressedTexImage2D, __pyx_k_glCompressedTexImage2D, sizeof(__pyx_k_glCompressedTexImage2D), 0, 0, 1, 1},
  {&__pyx_n_b_glCompressedTexImage2DARB, __pyx_k_glCompressedTexImage2DARB, sizeof(__pyx_k_glCompressedTexImage2DARB), 0, 0, 0, 1},
  {&__pyx_n_b_glCompressedTexSubImage2D, __pyx_k_glCompressedTexSubImage2D, sizeof(__pyx_k_glCompressedTexSubImage2D), 0, 0, 0, 1},
  {&__pyx_n_s_glCompressedTexSubImage2D, __pyx_k_glCompressedTexSubImage2D, sizeof(__pyx_k_glCompressedTexSubImage2D), 0, 0, 1, 1},
  {&__pyx_n_b_glCompressedTexSubImage2DARB, __pyx_k_glCompressedTexSubImage2DARB, sizeof(__pyx_k_glCompressedTexSubImage2DARB), 0, 0, 0, 1},
  {&__pyx_n_b_glCopyTexImage2D, __pyx_k_glCopyTexImage2D, sizeof(__pyx_k_glCopyTexImage2D), 0, 0, 0, 1},
  {&__pyx_n_s_glCopyTexImage2D, __pyx_k_glCopyTexImage2D, sizeof(__pyx_k_glCopyTexImage2D), 0, 0, 1, 1},
  {&__pyx_n_b_glCopyTexImage2DEXT, __pyx_k_glCopyTexImage2DEXT, sizeof(__pyx_k_glCopyTexImage2DEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glCopyTexSubImage2D, __pyx_k_glCopyTexSubImage2D, sizeof(__pyx_k_glCopyTexSubImage2D), 0, 0, 0, 1},
  {&__pyx_n_s_glCopyTexSubImage2D, __pyx_k_glCopyTexSubImage2D, sizeof(__pyx_k_glCopyTexSubImage2D), 0, 0, 1, 1},
  {&__pyx_n_b_glCopyTexSubImage2DEXT, __pyx_k_glCopyTexSubImage2DEXT, sizeof(__pyx_k_glCopyTexSubImage2DEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glCreateProgram, __pyx_k_glCreateProgram, sizeof(__pyx_k_glCreateProgram), 0, 0, 0, 1},
  {&__pyx_n_s_glCreateProgram, __pyx_k_glCreateProgram, sizeof(__pyx_k_glCreateProgram), 0, 0, 1, 1},
  {&__pyx_n_b_glCreateProgramObjectARB, __pyx_k_glCreateProgramObjectARB, sizeof(__pyx_k_glCreateProgramObjectARB), 0, 0, 0, 1},
  {&__pyx_n_b_glCreateShader, __pyx_k_glCreateShader, sizeof(__pyx_k_glCreateShader), 0, 0, 0, 1},
  {&__pyx_n_s_glCreateShader, __pyx_k_glCreateShader, sizeof(__pyx_k_glCreateShader), 0, 0, 1, 1},
  {&__pyx_n_b_glCreateShaderObjectARB, __pyx_k_glCreateShaderObjectARB, sizeof(__pyx_k_glCreateShaderObjectARB), 0, 0, 0, 1},
  {&__pyx_n_b_glCullFace, __pyx_k_glCullFace, sizeof(__pyx_k_glCullFace), 0, 0, 0, 1},
  {&__pyx_n_s_glCullFace, __pyx_k_glCullFace, sizeof(__pyx_k_glCullFace), 0, 0, 1, 1},
  {&__pyx_n_b_glDeleteBuffers, __pyx_k_glDeleteBuffers, sizeof(__pyx_k_glDeleteBuffers), 0, 0, 0, 1},
  {&__pyx_n_s_glDeleteBuffers, __pyx_k_glDeleteBuffers, sizeof(__pyx_k_glDeleteBuffers), 0, 0, 1, 1},
  {&__pyx_n_b_glDeleteBuffersARB, __pyx_k_glDeleteBuffersARB, sizeof(__pyx_k_glDeleteBuffersARB), 0, 0, 0, 1},
  {&__pyx_n_b_glDeleteFramebuffers, __pyx_k_glDeleteFramebuffers, sizeof(__pyx_k_glDeleteFramebuffers), 0, 0, 0, 1},
  {&__pyx_n_s_glDeleteFramebuffers, __pyx_k_glDeleteFramebuffers, sizeof(__pyx_k_glDeleteFramebuffers), 0, 0, 1, 1},
  {&__pyx_n_b_glDeleteFramebuffersEXT, __pyx_k_glDeleteFramebuffersEXT, sizeof(__pyx_k_glDeleteFramebuffersEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glDeleteProgram, __pyx_k_glDeleteProgram, sizeof(__pyx_k_glDeleteProgram), 0, 0, 0, 1},
  {&__pyx_n_s_glDeleteProgram, __pyx_k_glDeleteProgram, sizeof(__pyx_k_glDeleteProgram), 0, 0, 1, 1},
  {&__pyx_n_b_glDeleteRenderbuffers, __pyx_k_glDeleteRenderbuffers, sizeof(__pyx_k_glDeleteRenderbuffers), 0, 0, 0, 1},
  {&__pyx_n_s_glDeleteRenderbuffers, __pyx_k_glDeleteRenderbuffers, sizeof(__pyx_k_glDeleteRenderbuffers), 0, 0, 1, 1},
  {&__pyx_n_b_glDeleteRenderbuffersEXT, __pyx_k_glDeleteRenderbuffersEXT, sizeof(__pyx_k_glDeleteRenderbuffersEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glDeleteShader, __pyx_k_glDeleteShader, sizeof(__pyx_k_glDeleteShader), 0, 0, 0, 1},
  {&__pyx_n_s_glDeleteShader, __pyx_k_glDeleteShader, sizeof(__pyx_k_glDeleteShader), 0, 0, 1, 1},
  {&__pyx_n_b_glDeleteTextures, __pyx_k_glDeleteTextures, sizeof(__pyx_k_glDeleteTextures), 0, 0, 0, 1},
  {&__pyx_n_s_glDeleteTextures, __pyx_k_glDeleteTextures, sizeof(__pyx_k_glDeleteTextures), 0, 0, 1, 1},
  {&__pyx_n_b_glDepthFunc, __pyx_k_glDepthFunc, sizeof(__pyx_k_glDepthFunc), 0, 0, 0, 1},
  {&__pyx_n_s_glDepthFunc, __pyx_k_glDepthFunc, sizeof(__pyx_k_glDepthFunc), 0, 0, 1, 1},
  {&__pyx_n_b_glDepthMask, __pyx_k_glDepthMask, sizeof(__pyx_k_glDepthMask), 0, 0, 0, 1},
  {&__pyx_n_s_glDepthMask, __pyx_k_glDepthMask, sizeof(__pyx_k_glDepthMask), 0, 0, 1, 1},
  {&__pyx_n_b_glDetachShader, __pyx_k_glDetachShader, sizeof(__pyx_k_glDetachShader), 0, 0, 0, 1},
  {&__pyx_n_s_glDetachShader, __pyx_k_glDetachShader, sizeof(__pyx_k_glDetachShader), 0, 0, 1, 1},
  {&__pyx_n_b_glDisable, __pyx_k_glDisable, sizeof(__pyx_k_glDisable), 0, 0, 0, 1},
  {&__pyx_n_s_glDisable, __pyx_k_glDisable, sizeof(__pyx_k_glDisable), 0, 0, 1, 1},
  {&__pyx_n_b_glDisableVertexAttribArray, __pyx_k_glDisableVertexAttribArray, sizeof(__pyx_k_glDisableVertexAttribArray), 0, 0, 0, 1},
  {&__pyx_n_s_glDisableVertexAttribArray, __pyx_k_glDisableVertexAttribArray, sizeof(__pyx_k_glDisableVertexAttribArray), 0, 0, 1, 1},
  {&__pyx_n_b_glDisableVertexAttribArrayARB, __pyx_k_glDisableVertexAttribArrayARB, sizeof(__pyx_k_glDisableVertexAttribArrayARB), 0, 0, 0, 1},
  {&__pyx_n_b_glDrawArrays, __pyx_k_glDrawArrays, sizeof(__pyx_k_glDrawArrays), 0, 0, 0, 1},
  {&__pyx_n_s_glDrawArrays, __pyx_k_glDrawArrays, sizeof(__pyx_k_glDrawArrays), 0, 0, 1, 1},
  {&__pyx_n_b_glDrawArraysEXT, __pyx_k_glDrawArraysEXT, sizeof(__pyx_k_glDrawArraysEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glDrawElements, __pyx_k_glDrawElements, sizeof(__pyx_k_glDrawElements), 0, 0, 0, 1},
  {&__pyx_n_s_glDrawElements, __pyx_k_glDrawElements, sizeof(__pyx_k_glDrawElements), 0, 0, 1, 1},
  {&__pyx_n_b_glEnable, __pyx_k_glEnable, sizeof(__pyx_k_glEnable), 0, 0, 0, 1},
  {&__pyx_n_s_glEnable, __pyx_k_glEnable, sizeof(__pyx_k_glEnable), 0, 0, 1, 1},
  {&__pyx_n_b_glEnableVertexAttribArray, __pyx_k_glEnableVertexAttribArray, sizeof(__pyx_k_glEnableVertexAttribArray), 0, 0, 0, 1},
  {&__pyx_n_s_glEnableVertexAttribArray, __pyx_k_glEnableVertexAttribArray, sizeof(__pyx_k_glEnableVertexAttribArray), 0, 0, 1, 1},
  {&__pyx_n_b_glEnableVertexAttribArrayARB, __pyx_k_glEnableVertexAttribArrayARB, sizeof(__pyx_k_glEnableVertexAttribArrayARB), 0, 0, 0, 1},
  {&__pyx_n_b_glFinish, __pyx_k_glFinish, sizeof(__pyx_k_glFinish), 0, 0, 0, 1},
  {&__pyx_n_s_glFinish, __pyx_k_glFinish, sizeof(__pyx_k_glFinish), 0, 0, 1, 1},
  {&__pyx_n_b_glFlush, __pyx_k_glFlush, sizeof(__pyx_k_glFlush), 0, 0, 0, 1},
  {&__pyx_n_s_glFlush, __pyx_k_glFlush, sizeof(__pyx_k_glFlush), 0, 0, 1, 1},
  {&__pyx_n_b_glFramebufferRenderbuffer, __pyx_k_glFramebufferRenderbuffer, sizeof(__pyx_k_glFramebufferRenderbuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glFramebufferRenderbuffer, __pyx_k_glFramebufferRenderbuffer, sizeof(__pyx_k_glFramebufferRenderbuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glFramebufferRenderbufferEXT, __pyx_k_glFramebufferRenderbufferEXT, sizeof(__pyx_k_glFramebufferRenderbufferEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glFramebufferTexture2D, __pyx_k_glFramebufferTexture2D, sizeof(__pyx_k_glFramebufferTexture2D), 0, 0, 0, 1},
  {&__pyx_n_s_glFramebufferTexture2D, __pyx_k_glFramebufferTexture2D, sizeof(__pyx_k_glFramebufferTexture2D), 0, 0, 1, 1},
  {&__pyx_n_b_glFramebufferTexture2DEXT, __pyx_k_glFramebufferTexture2DEXT, sizeof(__pyx_k_glFramebufferTexture2DEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glFrontFace, __pyx_k_glFrontFace, sizeof(__pyx_k_glFrontFace), 0, 0, 0, 1},
  {&__pyx_n_s_glFrontFace, __pyx_k_glFrontFace, sizeof(__pyx_k_glFrontFace), 0, 0, 1, 1},
  {&__pyx_n_b_glGenBuffers, __pyx_k_glGenBuffers, sizeof(__pyx_k_glGenBuffers), 0, 0, 0, 1},
  {&__pyx_n_s_glGenBuffers, __pyx_k_glGenBuffers, sizeof(__pyx_k_glGenBuffers), 0, 0, 1, 1},
  {&__pyx_n_b_glGenBuffersARB, __pyx_k_glGenBuffersARB, sizeof(__pyx_k_glGenBuffersARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGenFramebuffers, __pyx_k_glGenFramebuffers, sizeof(__pyx_k_glGenFramebuffers), 0, 0, 0, 1},
  {&__pyx_n_s_glGenFramebuffers, __pyx_k_glGenFramebuffers, sizeof(__pyx_k_glGenFramebuffers), 0, 0, 1, 1},
  {&__pyx_n_b_glGenFramebuffersEXT, __pyx_k_glGenFramebuffersEXT, sizeof(__pyx_k_glGenFramebuffersEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glGenRenderbuffers, __pyx_k_glGenRenderbuffers, sizeof(__pyx_k_glGenRenderbuffers), 0, 0, 0, 1},
  {&__pyx_n_s_glGenRenderbuffers, __pyx_k_glGenRenderbuffers, sizeof(__pyx_k_glGenRenderbuffers), 0, 0, 1, 1},
  {&__pyx_n_b_glGenRenderbuffersEXT, __pyx_k_glGenRenderbuffersEXT, sizeof(__pyx_k_glGenRenderbuffersEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glGenTextures, __pyx_k_glGenTextures, sizeof(__pyx_k_glGenTextures), 0, 0, 0, 1},
  {&__pyx_n_s_glGenTextures, __pyx_k_glGenTextures, sizeof(__pyx_k_glGenTextures), 0, 0, 1, 1},
  {&__pyx_n_b_glGenerateMipmap, __pyx_k_glGenerateMipmap, sizeof(__pyx_k_glGenerateMipmap), 0, 0, 0, 1},
  {&__pyx_n_s_glGenerateMipmap, __pyx_k_glGenerateMipmap, sizeof(__pyx_k_glGenerateMipmap), 0, 0, 1, 1},
  {&__pyx_n_b_glGenerateMipmapEXT, __pyx_k_glGenerateMipmapEXT, sizeof(__pyx_k_glGenerateMipmapEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glGetActiveAttrib, __pyx_k_glGetActiveAttrib, sizeof(__pyx_k_glGetActiveAttrib), 0, 0, 0, 1},
  {&__pyx_n_s_glGetActiveAttrib, __pyx_k_glGetActiveAttrib, sizeof(__pyx_k_glGetActiveAttrib), 0, 0, 1, 1},
  {&__pyx_n_b_glGetActiveAttribARB, __pyx_k_glGetActiveAttribARB, sizeof(__pyx_k_glGetActiveAttribARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetActiveUniform, __pyx_k_glGetActiveUniform, sizeof(__pyx_k_glGetActiveUniform), 0, 0, 0, 1},
  {&__pyx_n_s_glGetActiveUniform, __pyx_k_glGetActiveUniform, sizeof(__pyx_k_glGetActiveUniform), 0, 0, 1, 1},
  {&__pyx_n_b_glGetActiveUniformARB, __pyx_k_glGetActiveUniformARB, sizeof(__pyx_k_glGetActiveUniformARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetAttachedShaders, __pyx_k_glGetAttachedShaders, sizeof(__pyx_k_glGetAttachedShaders), 0, 0, 0, 1},
  {&__pyx_n_s_glGetAttachedShaders, __pyx_k_glGetAttachedShaders, sizeof(__pyx_k_glGetAttachedShaders), 0, 0, 1, 1},
  {&__pyx_n_b_glGetAttribLocation, __pyx_k_glGetAttribLocation, sizeof(__pyx_k_glGetAttribLocation), 0, 0, 0, 1},
  {&__pyx_n_s_glGetAttribLocation, __pyx_k_glGetAttribLocation, sizeof(__pyx_k_glGetAttribLocation), 0, 0, 1, 1},
  {&__pyx_n_b_glGetAttribLocationARB, __pyx_k_glGetAttribLocationARB, sizeof(__pyx_k_glGetAttribLocationARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetBooleanv, __pyx_k_glGetBooleanv, sizeof(__pyx_k_glGetBooleanv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetBooleanv, __pyx_k_glGetBooleanv, sizeof(__pyx_k_glGetBooleanv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetBufferParameteriv, __pyx_k_glGetBufferParameteriv, sizeof(__pyx_k_glGetBufferParameteriv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetBufferParameteriv, __pyx_k_glGetBufferParameteriv, sizeof(__pyx_k_glGetBufferParameteriv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetBufferParameterivARB, __pyx_k_glGetBufferParameterivARB, sizeof(__pyx_k_glGetBufferParameterivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetError, __pyx_k_glGetError, sizeof(__pyx_k_glGetError), 0, 0, 0, 1},
  {&__pyx_n_s_glGetError, __pyx_k_glGetError, sizeof(__pyx_k_glGetError), 0, 0, 1, 1},
  {&__pyx_n_b_glGetFloatv, __pyx_k_glGetFloatv, sizeof(__pyx_k_glGetFloatv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetFloatv, __pyx_k_glGetFloatv, sizeof(__pyx_k_glGetFloatv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetFramebufferAttachmentParame, __pyx_k_glGetFramebufferAttachmentParame, sizeof(__pyx_k_glGetFramebufferAttachmentParame), 0, 0, 0, 1},
  {&__pyx_n_s_glGetFramebufferAttachmentParame, __pyx_k_glGetFramebufferAttachmentParame, sizeof(__pyx_k_glGetFramebufferAttachmentParame), 0, 0, 1, 1},
  {&__pyx_n_b_glGetFramebufferAttachmentParame_2, __pyx_k_glGetFramebufferAttachmentParame_2, sizeof(__pyx_k_glGetFramebufferAttachmentParame_2), 0, 0, 0, 1},
  {&__pyx_n_b_glGetIntegerv, __pyx_k_glGetIntegerv, sizeof(__pyx_k_glGetIntegerv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetIntegerv, __pyx_k_glGetIntegerv, sizeof(__pyx_k_glGetIntegerv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetProgramInfoLog, __pyx_k_glGetProgramInfoLog, sizeof(__pyx_k_glGetProgramInfoLog), 0, 0, 0, 1},
  {&__pyx_n_s_glGetProgramInfoLog, __pyx_k_glGetProgramInfoLog, sizeof(__pyx_k_glGetProgramInfoLog), 0, 0, 1, 1},
  {&__pyx_n_b_glGetProgramiv, __pyx_k_glGetProgramiv, sizeof(__pyx_k_glGetProgramiv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetProgramiv, __pyx_k_glGetProgramiv, sizeof(__pyx_k_glGetProgramiv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetRenderbufferParameteriv, __pyx_k_glGetRenderbufferParameteriv, sizeof(__pyx_k_glGetRenderbufferParameteriv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetRenderbufferParameteriv, __pyx_k_glGetRenderbufferParameteriv, sizeof(__pyx_k_glGetRenderbufferParameteriv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetRenderbufferParameterivEXT, __pyx_k_glGetRenderbufferParameterivEXT, sizeof(__pyx_k_glGetRenderbufferParameterivEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glGetShaderInfoLog, __pyx_k_glGetShaderInfoLog, sizeof(__pyx_k_glGetShaderInfoLog), 0, 0, 0, 1},
  {&__pyx_n_s_glGetShaderInfoLog, __pyx_k_glGetShaderInfoLog, sizeof(__pyx_k_glGetShaderInfoLog), 0, 0, 1, 1},
  {&__pyx_n_b_glGetShaderSource, __pyx_k_glGetShaderSource, sizeof(__pyx_k_glGetShaderSource), 0, 0, 0, 1},
  {&__pyx_n_s_glGetShaderSource, __pyx_k_glGetShaderSource, sizeof(__pyx_k_glGetShaderSource), 0, 0, 1, 1},
  {&__pyx_n_b_glGetShaderSourceARB, __pyx_k_glGetShaderSourceARB, sizeof(__pyx_k_glGetShaderSourceARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetShaderiv, __pyx_k_glGetShaderiv, sizeof(__pyx_k_glGetShaderiv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetShaderiv, __pyx_k_glGetShaderiv, sizeof(__pyx_k_glGetShaderiv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetString, __pyx_k_glGetString, sizeof(__pyx_k_glGetString), 0, 0, 0, 1},
  {&__pyx_n_s_glGetString, __pyx_k_glGetString, sizeof(__pyx_k_glGetString), 0, 0, 1, 1},
  {&__pyx_n_b_glGetTexParameterfv, __pyx_k_glGetTexParameterfv, sizeof(__pyx_k_glGetTexParameterfv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetTexParameterfv, __pyx_k_glGetTexParameterfv, sizeof(__pyx_k_glGetTexParameterfv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetTexParameteriv, __pyx_k_glGetTexParameteriv, sizeof(__pyx_k_glGetTexParameteriv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetTexParameteriv, __pyx_k_glGetTexParameteriv, sizeof(__pyx_k_glGetTexParameteriv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetUniformLocation, __pyx_k_glGetUniformLocation, sizeof(__pyx_k_glGetUniformLocation), 0, 0, 0, 1},
  {&__pyx_n_s_glGetUniformLocation, __pyx_k_glGetUniformLocation, sizeof(__pyx_k_glGetUniformLocation), 0, 0, 1, 1},
  {&__pyx_n_b_glGetUniformLocationARB, __pyx_k_glGetUniformLocationARB, sizeof(__pyx_k_glGetUniformLocationARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetUniformfv, __pyx_k_glGetUniformfv, sizeof(__pyx_k_glGetUniformfv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetUniformfv, __pyx_k_glGetUniformfv, sizeof(__pyx_k_glGetUniformfv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetUniformfvARB, __pyx_k_glGetUniformfvARB, sizeof(__pyx_k_glGetUniformfvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetUniformiv, __pyx_k_glGetUniformiv, sizeof(__pyx_k_glGetUniformiv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetUniformiv, __pyx_k_glGetUniformiv, sizeof(__pyx_k_glGetUniformiv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetUniformivARB, __pyx_k_glGetUniformivARB, sizeof(__pyx_k_glGetUniformivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetVertexAttribPointerv, __pyx_k_glGetVertexAttribPointerv, sizeof(__pyx_k_glGetVertexAttribPointerv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetVertexAttribPointerv, __pyx_k_glGetVertexAttribPointerv, sizeof(__pyx_k_glGetVertexAttribPointerv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetVertexAttribPointervARB, __pyx_k_glGetVertexAttribPointervARB, sizeof(__pyx_k_glGetVertexAttribPointervARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetVertexAttribPointervNV, __pyx_k_glGetVertexAttribPointervNV, sizeof(__pyx_k_glGetVertexAttribPointervNV), 0, 0, 0, 1},
  {&__pyx_n_b_glGetVertexAttribfv, __pyx_k_glGetVertexAttribfv, sizeof(__pyx_k_glGetVertexAttribfv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetVertexAttribfv, __pyx_k_glGetVertexAttribfv, sizeof(__pyx_k_glGetVertexAttribfv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetVertexAttribfvARB, __pyx_k_glGetVertexAttribfvARB, sizeof(__pyx_k_glGetVertexAttribfvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetVertexAttribfvNV, __pyx_k_glGetVertexAttribfvNV, sizeof(__pyx_k_glGetVertexAttribfvNV), 0, 0, 0, 1},
  {&__pyx_n_b_glGetVertexAttribiv, __pyx_k_glGetVertexAttribiv, sizeof(__pyx_k_glGetVertexAttribiv), 0, 0, 0, 1},
  {&__pyx_n_s_glGetVertexAttribiv, __pyx_k_glGetVertexAttribiv, sizeof(__pyx_k_glGetVertexAttribiv), 0, 0, 1, 1},
  {&__pyx_n_b_glGetVertexAttribivARB, __pyx_k_glGetVertexAttribivARB, sizeof(__pyx_k_glGetVertexAttribivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glGetVertexAttribivNV, __pyx_k_glGetVertexAttribivNV, sizeof(__pyx_k_glGetVertexAttribivNV), 0, 0, 0, 1},
  {&__pyx_n_b_glHint, __pyx_k_glHint, sizeof(__pyx_k_glHint), 0, 0, 0, 1},
  {&__pyx_n_s_glHint, __pyx_k_glHint, sizeof(__pyx_k_glHint), 0, 0, 1, 1},
  {&__pyx_n_b_glIsBuffer, __pyx_k_glIsBuffer, sizeof(__pyx_k_glIsBuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glIsBuffer, __pyx_k_glIsBuffer, sizeof(__pyx_k_glIsBuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glIsBufferARB, __pyx_k_glIsBufferARB, sizeof(__pyx_k_glIsBufferARB), 0, 0, 0, 1},
  {&__pyx_n_b_glIsEnabled, __pyx_k_glIsEnabled, sizeof(__pyx_k_glIsEnabled), 0, 0, 0, 1},
  {&__pyx_n_s_glIsEnabled, __pyx_k_glIsEnabled, sizeof(__pyx_k_glIsEnabled), 0, 0, 1, 1},
  {&__pyx_n_b_glIsFramebuffer, __pyx_k_glIsFramebuffer, sizeof(__pyx_k_glIsFramebuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glIsFramebuffer, __pyx_k_glIsFramebuffer, sizeof(__pyx_k_glIsFramebuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glIsFramebufferEXT, __pyx_k_glIsFramebufferEXT, sizeof(__pyx_k_glIsFramebufferEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glIsProgram, __pyx_k_glIsProgram, sizeof(__pyx_k_glIsProgram), 0, 0, 0, 1},
  {&__pyx_n_s_glIsProgram, __pyx_k_glIsProgram, sizeof(__pyx_k_glIsProgram), 0, 0, 1, 1},
  {&__pyx_n_b_glIsRenderbuffer, __pyx_k_glIsRenderbuffer, sizeof(__pyx_k_glIsRenderbuffer), 0, 0, 0, 1},
  {&__pyx_n_s_glIsRenderbuffer, __pyx_k_glIsRenderbuffer, sizeof(__pyx_k_glIsRenderbuffer), 0, 0, 1, 1},
  {&__pyx_n_b_glIsRenderbufferEXT, __pyx_k_glIsRenderbufferEXT, sizeof(__pyx_k_glIsRenderbufferEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glIsShader, __pyx_k_glIsShader, sizeof(__pyx_k_glIsShader), 0, 0, 0, 1},
  {&__pyx_n_s_glIsShader, __pyx_k_glIsShader, sizeof(__pyx_k_glIsShader), 0, 0, 1, 1},
  {&__pyx_n_b_glIsTexture, __pyx_k_glIsTexture, sizeof(__pyx_k_glIsTexture), 0, 0, 0, 1},
  {&__pyx_n_s_glIsTexture, __pyx_k_glIsTexture, sizeof(__pyx_k_glIsTexture), 0, 0, 1, 1},
  {&__pyx_n_b_glLineWidth, __pyx_k_glLineWidth, sizeof(__pyx_k_glLineWidth), 0, 0, 0, 1},
  {&__pyx_n_s_glLineWidth, __pyx_k_glLineWidth, sizeof(__pyx_k_glLineWidth), 0, 0, 1, 1},
  {&__pyx_n_b_glLinkProgram, __pyx_k_glLinkProgram, sizeof(__pyx_k_glLinkProgram), 0, 0, 0, 1},
  {&__pyx_n_s_glLinkProgram, __pyx_k_glLinkProgram, sizeof(__pyx_k_glLinkProgram), 0, 0, 1, 1},
  {&__pyx_n_b_glLinkProgramARB, __pyx_k_glLinkProgramARB, sizeof(__pyx_k_glLinkProgramARB), 0, 0, 0, 1},
  {&__pyx_n_b_glPixelStorei, __pyx_k_glPixelStorei, sizeof(__pyx_k_glPixelStorei), 0, 0, 0, 1},
  {&__pyx_n_s_glPixelStorei, __pyx_k_glPixelStorei, sizeof(__pyx_k_glPixelStorei), 0, 0, 1, 1},
  {&__pyx_n_b_glPolygonOffset, __pyx_k_glPolygonOffset, sizeof(__pyx_k_glPolygonOffset), 0, 0, 0, 1},
  {&__pyx_n_s_glPolygonOffset, __pyx_k_glPolygonOffset, sizeof(__pyx_k_glPolygonOffset), 0, 0, 1, 1},
  {&__pyx_n_b_glReadPixels, __pyx_k_glReadPixels, sizeof(__pyx_k_glReadPixels), 0, 0, 0, 1},
  {&__pyx_n_s_glReadPixels, __pyx_k_glReadPixels, sizeof(__pyx_k_glReadPixels), 0, 0, 1, 1},
  {&__pyx_n_b_glRenderbufferStorage, __pyx_k_glRenderbufferStorage, sizeof(__pyx_k_glRenderbufferStorage), 0, 0, 0, 1},
  {&__pyx_n_s_glRenderbufferStorage, __pyx_k_glRenderbufferStorage, sizeof(__pyx_k_glRenderbufferStorage), 0, 0, 1, 1},
  {&__pyx_n_b_glRenderbufferStorageEXT, __pyx_k_glRenderbufferStorageEXT, sizeof(__pyx_k_glRenderbufferStorageEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glSampleCoverage, __pyx_k_glSampleCoverage, sizeof(__pyx_k_glSampleCoverage), 0, 0, 0, 1},
  {&__pyx_n_s_glSampleCoverage, __pyx_k_glSampleCoverage, sizeof(__pyx_k_glSampleCoverage), 0, 0, 1, 1},
  {&__pyx_n_b_glSampleCoverageARB, __pyx_k_glSampleCoverageARB, sizeof(__pyx_k_glSampleCoverageARB), 0, 0, 0, 1},
  {&__pyx_n_b_glScissor, __pyx_k_glScissor, sizeof(__pyx_k_glScissor), 0, 0, 0, 1},
  {&__pyx_n_s_glScissor, __pyx_k_glScissor, sizeof(__pyx_k_glScissor), 0, 0, 1, 1},
  {&__pyx_n_b_glShaderSource, __pyx_k_glShaderSource, sizeof(__pyx_k_glShaderSource), 0, 0, 0, 1},
  {&__pyx_n_s_glShaderSource, __pyx_k_glShaderSource, sizeof(__pyx_k_glShaderSource), 0, 0, 1, 1},
  {&__pyx_n_b_glShaderSourceARB, __pyx_k_glShaderSourceARB, sizeof(__pyx_k_glShaderSourceARB), 0, 0, 0, 1},
  {&__pyx_n_b_glStencilFunc, __pyx_k_glStencilFunc, sizeof(__pyx_k_glStencilFunc), 0, 0, 0, 1},
  {&__pyx_n_s_glStencilFunc, __pyx_k_glStencilFunc, sizeof(__pyx_k_glStencilFunc), 0, 0, 1, 1},
  {&__pyx_n_b_glStencilFuncSeparate, __pyx_k_glStencilFuncSeparate, sizeof(__pyx_k_glStencilFuncSeparate), 0, 0, 0, 1},
  {&__pyx_n_s_glStencilFuncSeparate, __pyx_k_glStencilFuncSeparate, sizeof(__pyx_k_glStencilFuncSeparate), 0, 0, 1, 1},
  {&__pyx_n_b_glStencilMask, __pyx_k_glStencilMask, sizeof(__pyx_k_glStencilMask), 0, 0, 0, 1},
  {&__pyx_n_s_glStencilMask, __pyx_k_glStencilMask, sizeof(__pyx_k_glStencilMask), 0, 0, 1, 1},
  {&__pyx_n_b_glStencilMaskSeparate, __pyx_k_glStencilMaskSeparate, sizeof(__pyx_k_glStencilMaskSeparate), 0, 0, 0, 1},
  {&__pyx_n_s_glStencilMaskSeparate, __pyx_k_glStencilMaskSeparate, sizeof(__pyx_k_glStencilMaskSeparate), 0, 0, 1, 1},
  {&__pyx_n_b_glStencilOp, __pyx_k_glStencilOp, sizeof(__pyx_k_glStencilOp), 0, 0, 0, 1},
  {&__pyx_n_s_glStencilOp, __pyx_k_glStencilOp, sizeof(__pyx_k_glStencilOp), 0, 0, 1, 1},
  {&__pyx_n_b_glStencilOpSeparate, __pyx_k_glStencilOpSeparate, sizeof(__pyx_k_glStencilOpSeparate), 0, 0, 0, 1},
  {&__pyx_n_s_glStencilOpSeparate, __pyx_k_glStencilOpSeparate, sizeof(__pyx_k_glStencilOpSeparate), 0, 0, 1, 1},
  {&__pyx_n_b_glStencilOpSeparateATI, __pyx_k_glStencilOpSeparateATI, sizeof(__pyx_k_glStencilOpSeparateATI), 0, 0, 0, 1},
  {&__pyx_n_b_glTexImage2D, __pyx_k_glTexImage2D, sizeof(__pyx_k_glTexImage2D), 0, 0, 0, 1},
  {&__pyx_n_s_glTexImage2D, __pyx_k_glTexImage2D, sizeof(__pyx_k_glTexImage2D), 0, 0, 1, 1},
  {&__pyx_n_b_glTexParameterf, __pyx_k_glTexParameterf, sizeof(__pyx_k_glTexParameterf), 0, 0, 0, 1},
  {&__pyx_n_s_glTexParameterf, __pyx_k_glTexParameterf, sizeof(__pyx_k_glTexParameterf), 0, 0, 1, 1},
  {&__pyx_n_b_glTexParameterfv, __pyx_k_glTexParameterfv, sizeof(__pyx_k_glTexParameterfv), 0, 0, 0, 1},
  {&__pyx_n_s_glTexParameterfv, __pyx_k_glTexParameterfv, sizeof(__pyx_k_glTexParameterfv), 0, 0, 1, 1},
  {&__pyx_n_b_glTexParameteri, __pyx_k_glTexParameteri, sizeof(__pyx_k_glTexParameteri), 0, 0, 0, 1},
  {&__pyx_n_s_glTexParameteri, __pyx_k_glTexParameteri, sizeof(__pyx_k_glTexParameteri), 0, 0, 1, 1},
  {&__pyx_n_b_glTexParameteriv, __pyx_k_glTexParameteriv, sizeof(__pyx_k_glTexParameteriv), 0, 0, 0, 1},
  {&__pyx_n_s_glTexParameteriv, __pyx_k_glTexParameteriv, sizeof(__pyx_k_glTexParameteriv), 0, 0, 1, 1},
  {&__pyx_n_b_glTexSubImage2D, __pyx_k_glTexSubImage2D, sizeof(__pyx_k_glTexSubImage2D), 0, 0, 0, 1},
  {&__pyx_n_s_glTexSubImage2D, __pyx_k_glTexSubImage2D, sizeof(__pyx_k_glTexSubImage2D), 0, 0, 1, 1},
  {&__pyx_n_b_glTexSubImage2DEXT, __pyx_k_glTexSubImage2DEXT, sizeof(__pyx_k_glTexSubImage2DEXT), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform1f, __pyx_k_glUniform1f, sizeof(__pyx_k_glUniform1f), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform1f, __pyx_k_glUniform1f, sizeof(__pyx_k_glUniform1f), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform1fARB, __pyx_k_glUniform1fARB, sizeof(__pyx_k_glUniform1fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform1fv, __pyx_k_glUniform1fv, sizeof(__pyx_k_glUniform1fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform1fv, __pyx_k_glUniform1fv, sizeof(__pyx_k_glUniform1fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform1fvARB, __pyx_k_glUniform1fvARB, sizeof(__pyx_k_glUniform1fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform1i, __pyx_k_glUniform1i, sizeof(__pyx_k_glUniform1i), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform1i, __pyx_k_glUniform1i, sizeof(__pyx_k_glUniform1i), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform1iARB, __pyx_k_glUniform1iARB, sizeof(__pyx_k_glUniform1iARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform1iv, __pyx_k_glUniform1iv, sizeof(__pyx_k_glUniform1iv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform1iv, __pyx_k_glUniform1iv, sizeof(__pyx_k_glUniform1iv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform1ivARB, __pyx_k_glUniform1ivARB, sizeof(__pyx_k_glUniform1ivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform2f, __pyx_k_glUniform2f, sizeof(__pyx_k_glUniform2f), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform2f, __pyx_k_glUniform2f, sizeof(__pyx_k_glUniform2f), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform2fARB, __pyx_k_glUniform2fARB, sizeof(__pyx_k_glUniform2fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform2fv, __pyx_k_glUniform2fv, sizeof(__pyx_k_glUniform2fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform2fv, __pyx_k_glUniform2fv, sizeof(__pyx_k_glUniform2fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform2fvARB, __pyx_k_glUniform2fvARB, sizeof(__pyx_k_glUniform2fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform2i, __pyx_k_glUniform2i, sizeof(__pyx_k_glUniform2i), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform2i, __pyx_k_glUniform2i, sizeof(__pyx_k_glUniform2i), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform2iARB, __pyx_k_glUniform2iARB, sizeof(__pyx_k_glUniform2iARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform2iv, __pyx_k_glUniform2iv, sizeof(__pyx_k_glUniform2iv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform2iv, __pyx_k_glUniform2iv, sizeof(__pyx_k_glUniform2iv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform2ivARB, __pyx_k_glUniform2ivARB, sizeof(__pyx_k_glUniform2ivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform3f, __pyx_k_glUniform3f, sizeof(__pyx_k_glUniform3f), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform3f, __pyx_k_glUniform3f, sizeof(__pyx_k_glUniform3f), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform3fARB, __pyx_k_glUniform3fARB, sizeof(__pyx_k_glUniform3fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform3fv, __pyx_k_glUniform3fv, sizeof(__pyx_k_glUniform3fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform3fv, __pyx_k_glUniform3fv, sizeof(__pyx_k_glUniform3fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform3fvARB, __pyx_k_glUniform3fvARB, sizeof(__pyx_k_glUniform3fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform3i, __pyx_k_glUniform3i, sizeof(__pyx_k_glUniform3i), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform3i, __pyx_k_glUniform3i, sizeof(__pyx_k_glUniform3i), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform3iARB, __pyx_k_glUniform3iARB, sizeof(__pyx_k_glUniform3iARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform3iv, __pyx_k_glUniform3iv, sizeof(__pyx_k_glUniform3iv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform3iv, __pyx_k_glUniform3iv, sizeof(__pyx_k_glUniform3iv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform3ivARB, __pyx_k_glUniform3ivARB, sizeof(__pyx_k_glUniform3ivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform4f, __pyx_k_glUniform4f, sizeof(__pyx_k_glUniform4f), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform4f, __pyx_k_glUniform4f, sizeof(__pyx_k_glUniform4f), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform4fARB, __pyx_k_glUniform4fARB, sizeof(__pyx_k_glUniform4fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform4fv, __pyx_k_glUniform4fv, sizeof(__pyx_k_glUniform4fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform4fv, __pyx_k_glUniform4fv, sizeof(__pyx_k_glUniform4fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform4fvARB, __pyx_k_glUniform4fvARB, sizeof(__pyx_k_glUniform4fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform4i, __pyx_k_glUniform4i, sizeof(__pyx_k_glUniform4i), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform4i, __pyx_k_glUniform4i, sizeof(__pyx_k_glUniform4i), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform4iARB, __pyx_k_glUniform4iARB, sizeof(__pyx_k_glUniform4iARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniform4iv, __pyx_k_glUniform4iv, sizeof(__pyx_k_glUniform4iv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniform4iv, __pyx_k_glUniform4iv, sizeof(__pyx_k_glUniform4iv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniform4ivARB, __pyx_k_glUniform4ivARB, sizeof(__pyx_k_glUniform4ivARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniformMatrix2fv, __pyx_k_glUniformMatrix2fv, sizeof(__pyx_k_glUniformMatrix2fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniformMatrix2fv, __pyx_k_glUniformMatrix2fv, sizeof(__pyx_k_glUniformMatrix2fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniformMatrix2fvARB, __pyx_k_glUniformMatrix2fvARB, sizeof(__pyx_k_glUniformMatrix2fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniformMatrix3fv, __pyx_k_glUniformMatrix3fv, sizeof(__pyx_k_glUniformMatrix3fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniformMatrix3fv, __pyx_k_glUniformMatrix3fv, sizeof(__pyx_k_glUniformMatrix3fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniformMatrix3fvARB, __pyx_k_glUniformMatrix3fvARB, sizeof(__pyx_k_glUniformMatrix3fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUniformMatrix4fv, __pyx_k_glUniformMatrix4fv, sizeof(__pyx_k_glUniformMatrix4fv), 0, 0, 0, 1},
  {&__pyx_n_s_glUniformMatrix4fv, __pyx_k_glUniformMatrix4fv, sizeof(__pyx_k_glUniformMatrix4fv), 0, 0, 1, 1},
  {&__pyx_n_b_glUniformMatrix4fvARB, __pyx_k_glUniformMatrix4fvARB, sizeof(__pyx_k_glUniformMatrix4fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glUseProgram, __pyx_k_glUseProgram, sizeof(__pyx_k_glUseProgram), 0, 0, 0, 1},
  {&__pyx_n_s_glUseProgram, __pyx_k_glUseProgram, sizeof(__pyx_k_glUseProgram), 0, 0, 1, 1},
  {&__pyx_n_b_glUseProgramObjectARB, __pyx_k_glUseProgramObjectARB, sizeof(__pyx_k_glUseProgramObjectARB), 0, 0, 0, 1},
  {&__pyx_n_b_glValidateProgram, __pyx_k_glValidateProgram, sizeof(__pyx_k_glValidateProgram), 0, 0, 0, 1},
  {&__pyx_n_s_glValidateProgram, __pyx_k_glValidateProgram, sizeof(__pyx_k_glValidateProgram), 0, 0, 1, 1},
  {&__pyx_n_b_glValidateProgramARB, __pyx_k_glValidateProgramARB, sizeof(__pyx_k_glValidateProgramARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib1f, __pyx_k_glVertexAttrib1f, sizeof(__pyx_k_glVertexAttrib1f), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib1f, __pyx_k_glVertexAttrib1f, sizeof(__pyx_k_glVertexAttrib1f), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib1fARB, __pyx_k_glVertexAttrib1fARB, sizeof(__pyx_k_glVertexAttrib1fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib1fNV, __pyx_k_glVertexAttrib1fNV, sizeof(__pyx_k_glVertexAttrib1fNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib1fv, __pyx_k_glVertexAttrib1fv, sizeof(__pyx_k_glVertexAttrib1fv), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib1fv, __pyx_k_glVertexAttrib1fv, sizeof(__pyx_k_glVertexAttrib1fv), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib1fvARB, __pyx_k_glVertexAttrib1fvARB, sizeof(__pyx_k_glVertexAttrib1fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib1fvNV, __pyx_k_glVertexAttrib1fvNV, sizeof(__pyx_k_glVertexAttrib1fvNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib2f, __pyx_k_glVertexAttrib2f, sizeof(__pyx_k_glVertexAttrib2f), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib2f, __pyx_k_glVertexAttrib2f, sizeof(__pyx_k_glVertexAttrib2f), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib2fARB, __pyx_k_glVertexAttrib2fARB, sizeof(__pyx_k_glVertexAttrib2fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib2fNV, __pyx_k_glVertexAttrib2fNV, sizeof(__pyx_k_glVertexAttrib2fNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib2fv, __pyx_k_glVertexAttrib2fv, sizeof(__pyx_k_glVertexAttrib2fv), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib2fv, __pyx_k_glVertexAttrib2fv, sizeof(__pyx_k_glVertexAttrib2fv), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib2fvARB, __pyx_k_glVertexAttrib2fvARB, sizeof(__pyx_k_glVertexAttrib2fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib2fvNV, __pyx_k_glVertexAttrib2fvNV, sizeof(__pyx_k_glVertexAttrib2fvNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib3f, __pyx_k_glVertexAttrib3f, sizeof(__pyx_k_glVertexAttrib3f), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib3f, __pyx_k_glVertexAttrib3f, sizeof(__pyx_k_glVertexAttrib3f), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib3fARB, __pyx_k_glVertexAttrib3fARB, sizeof(__pyx_k_glVertexAttrib3fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib3fNV, __pyx_k_glVertexAttrib3fNV, sizeof(__pyx_k_glVertexAttrib3fNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib3fv, __pyx_k_glVertexAttrib3fv, sizeof(__pyx_k_glVertexAttrib3fv), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib3fv, __pyx_k_glVertexAttrib3fv, sizeof(__pyx_k_glVertexAttrib3fv), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib3fvARB, __pyx_k_glVertexAttrib3fvARB, sizeof(__pyx_k_glVertexAttrib3fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib3fvNV, __pyx_k_glVertexAttrib3fvNV, sizeof(__pyx_k_glVertexAttrib3fvNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib4f, __pyx_k_glVertexAttrib4f, sizeof(__pyx_k_glVertexAttrib4f), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib4f, __pyx_k_glVertexAttrib4f, sizeof(__pyx_k_glVertexAttrib4f), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib4fARB, __pyx_k_glVertexAttrib4fARB, sizeof(__pyx_k_glVertexAttrib4fARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib4fNV, __pyx_k_glVertexAttrib4fNV, sizeof(__pyx_k_glVertexAttrib4fNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib4fv, __pyx_k_glVertexAttrib4fv, sizeof(__pyx_k_glVertexAttrib4fv), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttrib4fv, __pyx_k_glVertexAttrib4fv, sizeof(__pyx_k_glVertexAttrib4fv), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttrib4fvARB, __pyx_k_glVertexAttrib4fvARB, sizeof(__pyx_k_glVertexAttrib4fvARB), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttrib4fvNV, __pyx_k_glVertexAttrib4fvNV, sizeof(__pyx_k_glVertexAttrib4fvNV), 0, 0, 0, 1},
  {&__pyx_n_b_glVertexAttribPointer, __pyx_k_glVertexAttribPointer, sizeof(__pyx_k_glVertexAttribPointer), 0, 0, 0, 1},
  {&__pyx_n_s_glVertexAttribPointer, __pyx_k_glVertexAttribPointer, sizeof(__pyx_k_glVertexAttribPointer), 0, 0, 1, 1},
  {&__pyx_n_b_glVertexAttribPointerARB, __pyx_k_glVertexAttribPointerARB, sizeof(__pyx_k_glVertexAttribPointerARB), 0, 0, 0, 1},
  {&__pyx_n_b_glViewport, __pyx_k_glViewport, sizeof(__pyx_k_glViewport), 0, 0, 0, 1},
  {&__pyx_n_s_glViewport, __pyx_k_glViewport, sizeof(__pyx_k_glViewport), 0, 0, 1, 1},
  {&__pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_k_home_tom_ab_renpy_renpy_gl2_ugu, sizeof(__pyx_k_home_tom_ab_renpy_renpy_gl2_ugu), 0, 0, 1, 0},
  {&__pyx_n_s_load, __pyx_k_load, sizeof(__pyx_k_load), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_kp_s_not_found, __pyx_k_not_found, sizeof(__pyx_k_not_found), 0, 0, 1, 0},
  {&__pyx_n_s_pyx_capi, __pyx_k_pyx_capi, sizeof(__pyx_k_pyx_capi), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_gl2_uguugl, __pyx_k_renpy_gl2_uguugl, sizeof(__pyx_k_renpy_gl2_uguugl), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  return 0;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "renpy/gl2/uguugl.pyx":428
 * 
 * 
 * def load():             # <<<<<<<<<<<<<<
 * 
 *     global glActiveTexture
 */
  __pyx_codeobj_ = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_ugu, __pyx_n_s_load, 428, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj_)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC inituguugl(void); /*proto*/
PyMODINIT_FUNC inituguugl(void)
#else
PyMODINIT_FUNC PyInit_uguugl(void); /*proto*/
PyMODINIT_FUNC PyInit_uguugl(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_uguugl(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_uguugl(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("uguugl", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__gl2__uguugl) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.gl2.uguugl")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.gl2.uguugl", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glActiveTexture, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glActiveTexture, "__pyx_t_5renpy_3gl2_6uguugl_glActiveTexture_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glAttachShader, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glAttachShader, "__pyx_t_5renpy_3gl2_6uguugl_glAttachShader_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBindAttribLocation, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBindAttribLocation, "__pyx_t_5renpy_3gl2_6uguugl_glBindAttribLocation_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBindBuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBindBuffer, "__pyx_t_5renpy_3gl2_6uguugl_glBindBuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBindFramebuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBindFramebuffer, "__pyx_t_5renpy_3gl2_6uguugl_glBindFramebuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBindRenderbuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBindRenderbuffer, "__pyx_t_5renpy_3gl2_6uguugl_glBindRenderbuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBindTexture, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBindTexture, "__pyx_t_5renpy_3gl2_6uguugl_glBindTexture_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBlendColor, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBlendColor, "__pyx_t_5renpy_3gl2_6uguugl_glBlendColor_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBlendEquation, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBlendEquation, "__pyx_t_5renpy_3gl2_6uguugl_glBlendEquation_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBlendEquationSeparate, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBlendEquationSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glBlendEquationSeparate_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBlendFunc, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBlendFunc, "__pyx_t_5renpy_3gl2_6uguugl_glBlendFunc_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBlendFuncSeparate, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBlendFuncSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glBlendFuncSeparate_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBufferData, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBufferData, "__pyx_t_5renpy_3gl2_6uguugl_glBufferData_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glBufferSubData, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glBufferSubData, "__pyx_t_5renpy_3gl2_6uguugl_glBufferSubData_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCheckFramebufferStatus, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCheckFramebufferStatus, "__pyx_t_5renpy_3gl2_6uguugl_glCheckFramebufferStatus_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glClear, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glClear, "__pyx_t_5renpy_3gl2_6uguugl_glClear_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glClearColor, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glClearColor, "__pyx_t_5renpy_3gl2_6uguugl_glClearColor_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glClearStencil, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glClearStencil, "__pyx_t_5renpy_3gl2_6uguugl_glClearStencil_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glColorMask, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glColorMask, "__pyx_t_5renpy_3gl2_6uguugl_glColorMask_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCompileShader, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCompileShader, "__pyx_t_5renpy_3gl2_6uguugl_glCompileShader_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCompressedTexImage2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCompressedTexImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexImage2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCompressedTexSubImage2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCompressedTexSubImage2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCopyTexImage2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCopyTexImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCopyTexImage2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCopyTexSubImage2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCopyTexSubImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glCopyTexSubImage2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCreateProgram, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCreateProgram, "__pyx_t_5renpy_3gl2_6uguugl_glCreateProgram_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCreateShader, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCreateShader, "__pyx_t_5renpy_3gl2_6uguugl_glCreateShader_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glCullFace, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glCullFace, "__pyx_t_5renpy_3gl2_6uguugl_glCullFace_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDeleteBuffers, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDeleteBuffers, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteBuffers_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDeleteFramebuffers, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDeleteFramebuffers, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteFramebuffers_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDeleteProgram, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDeleteProgram, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteProgram_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDeleteRenderbuffers, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDeleteRenderbuffers, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteRenderbuffers_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDeleteShader, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDeleteShader, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteShader_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDeleteTextures, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDeleteTextures, "__pyx_t_5renpy_3gl2_6uguugl_glDeleteTextures_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDepthFunc, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDepthFunc, "__pyx_t_5renpy_3gl2_6uguugl_glDepthFunc_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDepthMask, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDepthMask, "__pyx_t_5renpy_3gl2_6uguugl_glDepthMask_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDetachShader, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDetachShader, "__pyx_t_5renpy_3gl2_6uguugl_glDetachShader_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDisable, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDisable, "__pyx_t_5renpy_3gl2_6uguugl_glDisable_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDisableVertexAttribArray, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDisableVertexAttribArray, "__pyx_t_5renpy_3gl2_6uguugl_glDisableVertexAttribArray_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDrawArrays, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDrawArrays, "__pyx_t_5renpy_3gl2_6uguugl_glDrawArrays_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glDrawElements, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glDrawElements, "__pyx_t_5renpy_3gl2_6uguugl_glDrawElements_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glEnable, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glEnable, "__pyx_t_5renpy_3gl2_6uguugl_glEnable_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glEnableVertexAttribArray, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glEnableVertexAttribArray, "__pyx_t_5renpy_3gl2_6uguugl_glEnableVertexAttribArray_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glFinish, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glFinish, "__pyx_t_5renpy_3gl2_6uguugl_glFinish_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glFlush, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glFlush, "__pyx_t_5renpy_3gl2_6uguugl_glFlush_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glFramebufferRenderbuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer, "__pyx_t_5renpy_3gl2_6uguugl_glFramebufferRenderbuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glFramebufferTexture2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glFramebufferTexture2D, "__pyx_t_5renpy_3gl2_6uguugl_glFramebufferTexture2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glFrontFace, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glFrontFace, "__pyx_t_5renpy_3gl2_6uguugl_glFrontFace_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGenBuffers, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGenBuffers, "__pyx_t_5renpy_3gl2_6uguugl_glGenBuffers_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGenFramebuffers, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGenFramebuffers, "__pyx_t_5renpy_3gl2_6uguugl_glGenFramebuffers_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGenRenderbuffers, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGenRenderbuffers, "__pyx_t_5renpy_3gl2_6uguugl_glGenRenderbuffers_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGenTextures, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGenTextures, "__pyx_t_5renpy_3gl2_6uguugl_glGenTextures_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGenerateMipmap, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGenerateMipmap, "__pyx_t_5renpy_3gl2_6uguugl_glGenerateMipmap_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetActiveAttrib, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetActiveAttrib, "__pyx_t_5renpy_3gl2_6uguugl_glGetActiveAttrib_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetActiveUniform, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetActiveUniform, "__pyx_t_5renpy_3gl2_6uguugl_glGetActiveUniform_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetAttachedShaders, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetAttachedShaders, "__pyx_t_5renpy_3gl2_6uguugl_glGetAttachedShaders_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetAttribLocation, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetAttribLocation, "__pyx_t_5renpy_3gl2_6uguugl_glGetAttribLocation_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetBooleanv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetBooleanv, "__pyx_t_5renpy_3gl2_6uguugl_glGetBooleanv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetBufferParameteriv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetBufferParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetBufferParameteriv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetError, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetError, "__pyx_t_5renpy_3gl2_6uguugl_glGetError_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetFloatv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetFloatv, "__pyx_t_5renpy_3gl2_6uguugl_glGetFloatv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetFramebufferAttachmentParame, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetFramebufferAttachmentParameteriv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetIntegerv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetIntegerv, "__pyx_t_5renpy_3gl2_6uguugl_glGetIntegerv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetProgramInfoLog, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetProgramInfoLog, "__pyx_t_5renpy_3gl2_6uguugl_glGetProgramInfoLog_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetProgramiv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetProgramiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetProgramiv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetRenderbufferParameteriv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetRenderbufferParameteriv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetShaderInfoLog, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetShaderInfoLog, "__pyx_t_5renpy_3gl2_6uguugl_glGetShaderInfoLog_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetShaderSource, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetShaderSource, "__pyx_t_5renpy_3gl2_6uguugl_glGetShaderSource_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetShaderiv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetShaderiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetShaderiv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetString, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetString, "__pyx_t_5renpy_3gl2_6uguugl_glGetString_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetTexParameterfv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetTexParameterfv, "__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameterfv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetTexParameteriv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetTexParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glGetTexParameteriv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetUniformLocation, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetUniformLocation, "__pyx_t_5renpy_3gl2_6uguugl_glGetUniformLocation_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetUniformfv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetUniformfv, "__pyx_t_5renpy_3gl2_6uguugl_glGetUniformfv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetUniformiv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetUniformiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetUniformiv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetVertexAttribPointerv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv, "__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribPointerv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetVertexAttribfv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribfv, "__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribfv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glGetVertexAttribiv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glGetVertexAttribiv, "__pyx_t_5renpy_3gl2_6uguugl_glGetVertexAttribiv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glHint, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glHint, "__pyx_t_5renpy_3gl2_6uguugl_glHint_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsBuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsBuffer, "__pyx_t_5renpy_3gl2_6uguugl_glIsBuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsEnabled, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsEnabled, "__pyx_t_5renpy_3gl2_6uguugl_glIsEnabled_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsFramebuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsFramebuffer, "__pyx_t_5renpy_3gl2_6uguugl_glIsFramebuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsProgram, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsProgram, "__pyx_t_5renpy_3gl2_6uguugl_glIsProgram_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsRenderbuffer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsRenderbuffer, "__pyx_t_5renpy_3gl2_6uguugl_glIsRenderbuffer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsShader, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsShader, "__pyx_t_5renpy_3gl2_6uguugl_glIsShader_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glIsTexture, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glIsTexture, "__pyx_t_5renpy_3gl2_6uguugl_glIsTexture_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glLineWidth, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glLineWidth, "__pyx_t_5renpy_3gl2_6uguugl_glLineWidth_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glLinkProgram, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glLinkProgram, "__pyx_t_5renpy_3gl2_6uguugl_glLinkProgram_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glPixelStorei, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glPixelStorei, "__pyx_t_5renpy_3gl2_6uguugl_glPixelStorei_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glPolygonOffset, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glPolygonOffset, "__pyx_t_5renpy_3gl2_6uguugl_glPolygonOffset_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glReadPixels, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glReadPixels, "__pyx_t_5renpy_3gl2_6uguugl_glReadPixels_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glRenderbufferStorage, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glRenderbufferStorage, "__pyx_t_5renpy_3gl2_6uguugl_glRenderbufferStorage_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glSampleCoverage, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glSampleCoverage, "__pyx_t_5renpy_3gl2_6uguugl_glSampleCoverage_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glScissor, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glScissor, "__pyx_t_5renpy_3gl2_6uguugl_glScissor_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glShaderSource, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glShaderSource, "__pyx_t_5renpy_3gl2_6uguugl_glShaderSource_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glStencilFunc, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glStencilFunc, "__pyx_t_5renpy_3gl2_6uguugl_glStencilFunc_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glStencilFuncSeparate, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glStencilFuncSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glStencilFuncSeparate_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glStencilMask, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glStencilMask, "__pyx_t_5renpy_3gl2_6uguugl_glStencilMask_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glStencilMaskSeparate, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glStencilMaskSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glStencilMaskSeparate_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glStencilOp, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glStencilOp, "__pyx_t_5renpy_3gl2_6uguugl_glStencilOp_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glStencilOpSeparate, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glStencilOpSeparate, "__pyx_t_5renpy_3gl2_6uguugl_glStencilOpSeparate_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glTexImage2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glTexImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glTexImage2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glTexParameterf, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glTexParameterf, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameterf_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glTexParameterfv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glTexParameterfv, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameterfv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glTexParameteri, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glTexParameteri, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameteri_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glTexParameteriv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glTexParameteriv, "__pyx_t_5renpy_3gl2_6uguugl_glTexParameteriv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glTexSubImage2D, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glTexSubImage2D, "__pyx_t_5renpy_3gl2_6uguugl_glTexSubImage2D_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform1f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform1f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform1fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform1fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform1i, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform1i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1i_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform1iv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform1iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform1iv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform2f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform2f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform2fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform2fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform2i, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform2i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2i_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform2iv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform2iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform2iv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform3f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform3f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform3fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform3fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform3i, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform3i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3i_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform3iv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform3iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform3iv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform4f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform4f, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform4fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform4fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform4i, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform4i, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4i_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniform4iv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniform4iv, "__pyx_t_5renpy_3gl2_6uguugl_glUniform4iv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniformMatrix2fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix2fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix2fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniformMatrix3fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix3fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix3fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUniformMatrix4fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUniformMatrix4fv, "__pyx_t_5renpy_3gl2_6uguugl_glUniformMatrix4fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glUseProgram, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glUseProgram, "__pyx_t_5renpy_3gl2_6uguugl_glUseProgram_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glValidateProgram, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glValidateProgram, "__pyx_t_5renpy_3gl2_6uguugl_glValidateProgram_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib1f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib1fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib1fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib1fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib2f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib2fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib2fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib2fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib3f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib3fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib3fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib3fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib4f, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4f, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4f_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttrib4fv, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttrib4fv, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttrib4fv_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glVertexAttribPointer, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glVertexAttribPointer, "__pyx_t_5renpy_3gl2_6uguugl_glVertexAttribPointer_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportVoidPtr(__pyx_n_s_glViewport, (void *)&__pyx_v_5renpy_3gl2_6uguugl_glViewport, "__pyx_t_5renpy_3gl2_6uguugl_glViewport_type") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "renpy/gl2/uguugl.pyx":428
 * 
 * 
 * def load():             # <<<<<<<<<<<<<<
 * 
 *     global glActiveTexture
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_6uguugl_1load, NULL, __pyx_n_s_renpy_gl2_uguugl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_load, __pyx_t_1) < 0) __PYX_ERR(0, 428, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/uguugl.pyx":1
 * from sdl2 cimport SDL_GL_GetProcAddress             # <<<<<<<<<<<<<<
 * 
 * cdef void *find_gl_command(names):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.gl2.uguugl", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.gl2.uguugl");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrExport */
static int __Pyx_ExportVoidPtr(PyObject *name, void *p, const char *sig) {
    PyObject *d;
    PyObject *cobj = 0;
    d = PyDict_GetItem(__pyx_d, __pyx_n_s_pyx_capi);
    Py_XINCREF(d);
    if (!d) {
        d = PyDict_New();
        if (!d)
            goto bad;
        if (__Pyx_PyObject_SetAttrStr(__pyx_m, __pyx_n_s_pyx_capi, d) < 0)
            goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItem(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
