/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__angle__glenviron_shader
#define __PYX_HAVE_API__renpy__angle__glenviron_shader
#include "glcompat.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "glenviron_shader.pyx",
  "stringsource",
  "matrix.pxd",
  "render.pxd",
  "gldraw.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5renpy_7display_6matrix_Matrix;
struct __pyx_obj_5renpy_7display_6matrix_Matrix2D;
struct __pyx_obj_5renpy_7display_6render_Render;
struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw;
struct __pyx_obj_5renpy_5angle_6gldraw_Environ;
struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program;
struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron;
struct __pyx_opt_args_5renpy_7display_6render_6Render_blit;
struct __pyx_opt_args_5renpy_7display_6render_6Render_subpixel_blit;
struct __pyx_opt_args_5renpy_7display_6render_6Render_absolute_blit;

/* "renpy/display/render.pxd":107
 *     # operations ###############################################################
 * 
 *     cpdef int blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)             # <<<<<<<<<<<<<<
 *     cpdef int subpixel_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int absolute_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 */
struct __pyx_opt_args_5renpy_7display_6render_6Render_blit {
  int __pyx_n;
  PyObject *focus;
  PyObject *main;
  PyObject *index;
};

/* "renpy/display/render.pxd":108
 * 
 *     cpdef int blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int subpixel_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)             # <<<<<<<<<<<<<<
 *     cpdef int absolute_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 * 
 */
struct __pyx_opt_args_5renpy_7display_6render_6Render_subpixel_blit {
  int __pyx_n;
  PyObject *focus;
  PyObject *main;
  PyObject *index;
};

/* "renpy/display/render.pxd":109
 *     cpdef int blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int subpixel_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int absolute_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_5renpy_7display_6render_6Render_absolute_blit {
  int __pyx_n;
  PyObject *focus;
  PyObject *main;
  PyObject *index;
};

/* "renpy/display/matrix.pxd":1
 * cdef class Matrix:             # <<<<<<<<<<<<<<
 * 
 *     cdef float m[0]
 */
struct __pyx_obj_5renpy_7display_6matrix_Matrix {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix *__pyx_vtab;
  float m[0];
  float xdx;
  float xdy;
  float xdz;
  float xdw;
  float ydx;
  float ydy;
  float ydz;
  float ydw;
  float zdx;
  float zdy;
  float zdz;
  float zdw;
  float wdx;
  float wdy;
  float wdz;
  float wdw;
};


/* "renpy/display/matrix.pxd":43
 *     cpdef bint is_unit_aligned(Matrix self)
 * 
 * cdef class Matrix2D(Matrix):             # <<<<<<<<<<<<<<
 *     pass
 */
struct __pyx_obj_5renpy_7display_6matrix_Matrix2D {
  struct __pyx_obj_5renpy_7display_6matrix_Matrix __pyx_base;
};


/* "renpy/display/render.pxd":25
 * from renpy.display.matrix cimport Matrix, Matrix2D
 * 
 * cdef class Render:             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint mark, cache_killed
 */
struct __pyx_obj_5renpy_7display_6render_Render {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_7display_6render_Render *__pyx_vtab;
  int mark;
  int cache_killed;
  float width;
  float height;
  PyObject *layer_name;
  PyObject *children;
  PyObject *parents;
  PyObject *depends_on_list;
  int operation;
  double operation_complete;
  int operation_alpha;
  PyObject *operation_parameter;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *forward;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *reverse;
  double alpha;
  double over;
  PyObject *nearest;
  PyObject *focuses;
  PyObject *pass_focuses;
  PyObject *focus_screen;
  PyObject *draw_func;
  PyObject *render_of;
  int opaque;
  PyObject *visible_children;
  int xclipping;
  int yclipping;
  PyObject *surface;
  PyObject *alpha_surface;
  PyObject *half_cache;
  int modal;
  int text_input;
  PyObject *mesh;
  PyObject *shaders;
  PyObject *uniforms;
  PyObject *properties;
  PyObject *cached_texture;
  PyObject *cached_model;
  int loaded;
};


/* "gldraw.pxd":30
 * cdef class Environ
 * 
 * cdef class GLDraw:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint did_init
 */
struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_5angle_6gldraw_GLDraw *__pyx_vtab;
  int did_init;
  int did_texture_test;
  struct __pyx_obj_5renpy_5angle_6gldraw_Environ *environ;
  PyObject *rtt;
  PyObject *window;
  PyObject *virtual_size;
  PyObject *physical_size;
  PyObject *drawable_size;
  PyObject *virtual_box;
  PyObject *physical_box;
  PyObject *mouse_old_visible;
  PyObject *mouse_info;
  PyObject *texture_cache;
  double last_redraw_time;
  double redraw_period;
  PyObject *info;
  PyObject *old_fullscreen;
  PyObject *fullscreen_surface;
  PyObject *display_info;
  PyObject *clip_cache;
  int fast_dissolve;
  int allow_fixed;
  PyObject *default_clip;
  int did_render_to_texture;
  float dpi_scale;
  PyObject *ready_texture_queue;
  PyObject *clip_rtt_box;
  PyObject *draw_per_virt;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *virt_to_draw;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *draw_to_virt;
  int fast_redraw_frames;
};


/* "gldraw.pxd":28
 * cimport renpy.display.render as render
 * 
 * cdef class Environ             # <<<<<<<<<<<<<<
 * 
 * cdef class GLDraw:
 */
struct __pyx_obj_5renpy_5angle_6gldraw_Environ {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_5angle_6gldraw_Environ *__pyx_vtab;
};


/* "renpy/angle/glenviron_shader.pyx":336
 *     return program
 * 
 * cdef class Program(object):             # <<<<<<<<<<<<<<
 *     """
 *     Encapsulates a program.
 */
struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program {
  PyObject_HEAD
  GLuint program;
  GLint Vertex;
  GLint VertexTexCoord0;
  GLint VertexTexCoord1;
  GLint VertexTexCoord2;
  GLint Projection;
  GLint Color;
  GLint tex0;
  GLint tex1;
  GLint tex2;
  GLint clip0;
  GLint clip1;
  GLint offset;
  GLint multiplier;
  GLint done;
};


/* "renpy/angle/glenviron_shader.pyx":402
 * 
 * 
 * cdef class ShaderEnviron(Environ):             # <<<<<<<<<<<<<<
 *     """
 *     This is an environment that uses shaders.
 */
struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron {
  struct __pyx_obj_5renpy_5angle_6gldraw_Environ __pyx_base;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *program;
  float projection[16];
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *blit_program;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *blend_program;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *imageblend_program;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *blit_clip_program;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *blend_clip_program;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *imageblend_clip_program;
  int clipping;
  double clip_x0;
  double clip_y0;
  double clip_x1;
  double clip_y1;
  int viewport_x;
  int viewport_y;
  int viewport_w;
  int viewport_h;
};



/* "renpy/display/matrix.pxd":1
 * cdef class Matrix:             # <<<<<<<<<<<<<<
 * 
 *     cdef float m[0]
 */

struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix {
  void (*transform4)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float *, float, float, float, float);
  void (*transform3)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float, float, float, float);
  void (*transform2)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float, float, float, float);
  int (*is_unit_aligned)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix *__pyx_vtabptr_5renpy_7display_6matrix_Matrix;
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float *, float, float, float, float);
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float, float, float, float);
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float, float, float, float);


/* "renpy/display/matrix.pxd":43
 *     cpdef bint is_unit_aligned(Matrix self)
 * 
 * cdef class Matrix2D(Matrix):             # <<<<<<<<<<<<<<
 *     pass
 */

struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D {
  struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D *__pyx_vtabptr_5renpy_7display_6matrix_Matrix2D;


/* "renpy/display/render.pxd":25
 * from renpy.display.matrix cimport Matrix, Matrix2D
 * 
 * cdef class Render:             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint mark, cache_killed
 */

struct __pyx_vtabstruct_5renpy_7display_6render_Render {
  int (*blit)(struct __pyx_obj_5renpy_7display_6render_Render *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5renpy_7display_6render_6Render_blit *__pyx_optional_args);
  int (*subpixel_blit)(struct __pyx_obj_5renpy_7display_6render_Render *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5renpy_7display_6render_6Render_subpixel_blit *__pyx_optional_args);
  int (*absolute_blit)(struct __pyx_obj_5renpy_7display_6render_Render *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5renpy_7display_6render_6Render_absolute_blit *__pyx_optional_args);
};
static struct __pyx_vtabstruct_5renpy_7display_6render_Render *__pyx_vtabptr_5renpy_7display_6render_Render;


/* "gldraw.pxd":28
 * cimport renpy.display.render as render
 * 
 * cdef class Environ             # <<<<<<<<<<<<<<
 * 
 * cdef class GLDraw:
 */

struct __pyx_vtabstruct_5renpy_5angle_6gldraw_Environ {
  void (*blit)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *);
  void (*blend)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double);
  void (*imageblend)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double, int);
  void (*set_vertex)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, float *);
  void (*set_texture)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, int, float *);
  void (*set_color)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double, double, double, double);
  void (*set_clip)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, PyObject *, struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *);
  void (*unset_clip)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *);
  void (*ortho)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double, double, double, double, double, double);
  void (*viewport)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, int, int, int, int);
};
static struct __pyx_vtabstruct_5renpy_5angle_6gldraw_Environ *__pyx_vtabptr_5renpy_5angle_6gldraw_Environ;


/* "gldraw.pxd":30
 * cdef class Environ
 * 
 * cdef class GLDraw:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint did_init
 */

struct __pyx_vtabstruct_5renpy_5angle_6gldraw_GLDraw {
  PyObject *(*set_clip)(struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *, PyObject *, int __pyx_skip_dispatch);
  int (*draw_render_textures)(struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *, PyObject *, int, int __pyx_skip_dispatch);
  int (*draw_transformed)(struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *, PyObject *, PyObject *, double, double, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_5angle_6gldraw_GLDraw *__pyx_vtabptr_5renpy_5angle_6gldraw_GLDraw;


/* "renpy/angle/glenviron_shader.pyx":402
 * 
 * 
 * cdef class ShaderEnviron(Environ):             # <<<<<<<<<<<<<<
 *     """
 *     This is an environment that uses shaders.
 */

struct __pyx_vtabstruct_5renpy_5angle_16glenviron_shader_ShaderEnviron {
  struct __pyx_vtabstruct_5renpy_5angle_6gldraw_Environ __pyx_base;
  void (*project)(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *, PyObject *, PyObject *, PyObject *, double *, double *, double *);
};
static struct __pyx_vtabstruct_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_vtabptr_5renpy_5angle_16glenviron_shader_ShaderEnviron;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* IncludeStringH.proto */
#include <string.h>

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLint(GLint value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLuint(GLuint value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLhandleARB(GLhandleARB value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE GLint __Pyx_PyInt_As_GLint(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLuint __Pyx_PyInt_As_GLuint(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLenum __Pyx_PyInt_As_GLenum(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLhandleARB __Pyx_PyInt_As_GLhandleARB(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_blit(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_blend(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_fraction); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_imageblend(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_fraction, int __pyx_v_ramp); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_vertex(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, float *__pyx_v_vertices); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_texture(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, int __pyx_v_unit, float *__pyx_v_coords); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_color(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_r, double __pyx_v_g, double __pyx_v_b, double __pyx_v_a); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_ortho(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_left, double __pyx_v_right, double __pyx_v_bottom, double __pyx_v_top, double __pyx_v_near, double __pyx_v_far); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_project(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_z, double *__pyx_v_rv_x, double *__pyx_v_rv_y, double *__pyx_v_rv_z); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_clip(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, PyObject *__pyx_v_clip_box, struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *__pyx_v_draw); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_unset_clip(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *__pyx_v_draw); /* proto*/
static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_viewport(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, int __pyx_v_x, int __pyx_v_y, int __pyx_v_width, int __pyx_v_height); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float *__pyx_v_ow, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/

/* Module declarations from 'renpy.angle.gl' */
static int (*__pyx_f_5renpy_5angle_2gl_glAttachObjectARB)(GLhandleARB, GLhandleARB); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glCompileShaderARB)(GLhandleARB); /*proto*/
static GLhandleARB (*__pyx_f_5renpy_5angle_2gl_glCreateProgramObjectARB)(void); /*proto*/
static GLhandleARB (*__pyx_f_5renpy_5angle_2gl_glCreateShaderObjectARB)(GLenum); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glDeleteProgram)(GLuint); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glDeleteShader)(GLuint); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glDisable)(GLenum); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB)(GLuint); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glEnable)(GLenum); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glEnableVertexAttribArrayARB)(GLuint); /*proto*/
static GLint (*__pyx_f_5renpy_5angle_2gl_glGetAttribLocationARB)(GLhandleARB, GLchar *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glGetProgramInfoLog)(GLhandleARB, GLsizei, GLsizei *, GLchar *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glGetProgramiv)(GLuint, GLenum, GLint *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glGetShaderInfoLog)(GLhandleARB, GLsizei, GLsizei *, GLchar *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glGetShaderiv)(GLuint, GLenum, GLint *); /*proto*/
static GLint (*__pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB)(GLhandleARB, GLchar *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glLinkProgramARB)(GLhandleARB); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glScissor)(GLint, GLint, GLsizei, GLsizei); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glShaderSourceARB)(GLhandleARB, GLsizei, GLchar **, GLint *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glUniform1fARB)(GLint, GLfloat); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glUniform1iARB)(GLint, GLint); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glUniform2fARB)(GLint, GLfloat, GLfloat); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glUniform4fARB)(GLint, GLfloat, GLfloat, GLfloat, GLfloat); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glUniformMatrix4fvARB)(GLint, GLsizei, GLboolean, GLfloat *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glUseProgramObjectARB)(GLhandleARB); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glVertexAttribPointerARB)(GLuint, GLint, GLenum, GLboolean, GLsizei, GLubyte *); /*proto*/
static int (*__pyx_f_5renpy_5angle_2gl_glViewport)(GLint, GLint, GLsizei, GLsizei); /*proto*/

/* Module declarations from 'renpy.display.matrix' */
static PyTypeObject *__pyx_ptype_5renpy_7display_6matrix_Matrix = 0;
static PyTypeObject *__pyx_ptype_5renpy_7display_6matrix_Matrix2D = 0;

/* Module declarations from 'renpy.display.render' */
static PyTypeObject *__pyx_ptype_5renpy_7display_6render_Render = 0;

/* Module declarations from 'renpy.angle.gldraw' */
static PyTypeObject *__pyx_ptype_5renpy_5angle_6gldraw_Environ = 0;
static PyTypeObject *__pyx_ptype_5renpy_5angle_6gldraw_GLDraw = 0;

/* Module declarations from 'renpy.angle.glenviron_shader' */
static PyTypeObject *__pyx_ptype_5renpy_5angle_16glenviron_shader_Program = 0;
static PyTypeObject *__pyx_ptype_5renpy_5angle_16glenviron_shader_ShaderEnviron = 0;
static int __pyx_f_5renpy_5angle_16glenviron_shader_round(double); /*proto*/
static PyObject *__pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_Program__set_state(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *, PyObject *); /*proto*/
static PyObject *__pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_ShaderEnviron__set_state(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_carray_to_py_float(float *, Py_ssize_t); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_carray_to_tuple_float(float *, Py_ssize_t); /*proto*/
static int __Pyx_carray_from_py_float(PyObject *, float *, Py_ssize_t); /*proto*/
#define __Pyx_MODULE_NAME "renpy.angle.glenviron_shader"
extern int __pyx_module_is_main_renpy__angle__glenviron_shader;
int __pyx_module_is_main_renpy__angle__glenviron_shader = 0;

/* Implementation of 'renpy.angle.glenviron_shader' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_OverflowError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_IndexError;
static const char __pyx_k_[] = " ";
static const char __pyx_k_log[] = "log";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_kind[] = "kind";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_delete[] = "delete";
static const char __pyx_k_handle[] = "handle";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_shader[] = "shader";
static const char __pyx_k_source[] = "source";
static const char __pyx_k_status[] = "status";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_vertex[] = "vertex";
static const char __pyx_k_program[] = "program";
static const char __pyx_k_activate[] = "activate";
static const char __pyx_k_fragment[] = "fragment";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_lensource[] = "lensource";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_sourceptr[] = "sourceptr";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_log_length[] = "log_length";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_BLIT_SHADER[] = "BLIT_SHADER";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_BLEND_SHADER[] = "BLEND_SHADER";
static const char __pyx_k_check_status[] = "check_status";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_OverflowError[] = "OverflowError";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_vertex_shader[] = "vertex_shader";
static const char __pyx_k_Shader_error_s[] = "Shader error: %s";
static const char __pyx_k_VERTEX_SHADER1[] = "VERTEX_SHADER1";
static const char __pyx_k_VERTEX_SHADER2[] = "VERTEX_SHADER2";
static const char __pyx_k_VERTEX_SHADER3[] = "VERTEX_SHADER3";
static const char __pyx_k_compile_shader[] = "compile_shader";
static const char __pyx_k_compile_program[] = "compile_program";
static const char __pyx_k_disable_attribs[] = "disable_attribs";
static const char __pyx_k_fragment_shader[] = "fragment_shader";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_BLIT_CLIP_SHADER[] = "BLIT_CLIP_SHADER";
static const char __pyx_k_GLES_PORTABILITY[] = "GLES_PORTABILITY";
static const char __pyx_k_BLEND_CLIP_SHADER[] = "BLEND_CLIP_SHADER";
static const char __pyx_k_IMAGEBLEND_SHADER[] = "IMAGEBLEND_SHADER";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_pyx_unpickle_Program[] = "__pyx_unpickle_Program";
static const char __pyx_k_IMAGEBLEND_CLIP_SHADER[] = "IMAGEBLEND_CLIP_SHADER";
static const char __pyx_k_pyx_unpickle_ShaderEnviron[] = "__pyx_unpickle_ShaderEnviron";
static const char __pyx_k_renpy_angle_glenviron_shader[] = "renpy.angle.glenviron_shader";
static const char __pyx_k_home_tom_ab_renpy_renpy_angle_g[] = "/home/tom/ab/renpy/renpy/angle/glenviron_shader.pyx";
static const char __pyx_k_ifdef_GL_ES_ifdef_GL_FRAGMENT_P[] = "#ifdef GL_ES\n  #ifdef GL_FRAGMENT_PRECISION_HIGH\n    precision highp float;\n  # else\n    precision mediump float;\n  # endif\n#endif\n";
static const char __pyx_k_Incompatible_checksums_s_vs_0xb6[] = "Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xc2[] = "Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))";
static const char __pyx_k_uniform_mat4_Projection_attribut[] = "uniform mat4 Projection;\n\nattribute vec4 Vertex;\nattribute vec2 VertexTexCoord0;\n\nvarying vec2 TexCoord0;\n\nvarying vec2 pos;\n\nvoid main() {\n    TexCoord0 = VertexTexCoord0;\n\n    pos = Vertex.xy;\n    gl_Position = Projection * Vertex;\n}\n";
static const char __pyx_k_uniform_vec4_Color_uniform_sampl[] = "uniform vec4 Color;\nuniform sampler2D tex0;\n\nvarying vec2 TexCoord0;\n\nvarying vec2 pos;\nuniform vec2 clip0;\nuniform vec2 clip1;\n\nvoid main()\n{\n    vec4 color0 = texture2D(tex0, TexCoord0.st);\n    gl_FragColor = color0 * Color;\n}\n";
static const char __pyx_k_uniform_mat4_Projection_attribut_2[] = "uniform mat4 Projection;\n\nattribute vec4 Vertex;\nattribute vec2 VertexTexCoord0;\nattribute vec2 VertexTexCoord1;\n\nvarying vec2 TexCoord0;\nvarying vec2 TexCoord1;\n\nvarying vec2 pos;\n\nvoid main() {\n    TexCoord0 = VertexTexCoord0;\n    TexCoord1 = VertexTexCoord1;\n\n    pos = Vertex.xy;\n    gl_Position = Projection * Vertex;\n}\n";
static const char __pyx_k_uniform_mat4_Projection_attribut_3[] = "uniform mat4 Projection;\n\nattribute vec4 Vertex;\nattribute vec2 VertexTexCoord0;\nattribute vec2 VertexTexCoord1;\nattribute vec2 VertexTexCoord2;\n\nvarying vec2 TexCoord0;\nvarying vec2 TexCoord1;\nvarying vec2 TexCoord2;\n\nvarying vec2 pos;\n\nvoid main() {\n    TexCoord0 = VertexTexCoord0;\n    TexCoord1 = VertexTexCoord1;\n    TexCoord2 = VertexTexCoord2;\n\n    pos = Vertex.xy;\n    gl_Position = Projection * Vertex;\n}\n";
static const char __pyx_k_uniform_vec4_Color_uniform_sampl_2[] = "uniform vec4 Color;\nuniform sampler2D tex0;\n\nvarying vec2 TexCoord0;\n\nvarying vec2 pos;\nuniform vec2 clip0;\nuniform vec2 clip1;\n\nvoid main()\n{\n    if (pos.x < clip0.x || pos.y < clip0.y || pos.x >= clip1.x || pos.y >= clip1.y) {\n        discard;\n    }\n\n    vec4 color0 = texture2D(tex0, TexCoord0.st);\n    gl_FragColor = color0 * Color;\n}\n";
static const char __pyx_k_uniform_vec4_Color_uniform_sampl_3[] = "uniform vec4 Color;\nuniform sampler2D tex0;\nuniform sampler2D tex1;\nuniform float done;\n\nvarying vec2 TexCoord0;\nvarying vec2 TexCoord1;\n\nvarying vec2 pos;\nuniform vec2 clip0;\nuniform vec2 clip1;\n\nvoid main()\n{\n    vec4 color0 = texture2D(tex0, TexCoord0.st);\n    vec4 color1 = texture2D(tex1, TexCoord1.st);\n\n    gl_FragColor = mix(color0, color1, done) * Color;\n}\n";
static const char __pyx_k_uniform_vec4_Color_uniform_sampl_4[] = "uniform vec4 Color;\nuniform sampler2D tex0;\nuniform sampler2D tex1;\nuniform float done;\n\nvarying vec2 TexCoord0;\nvarying vec2 TexCoord1;\n\nvarying vec2 pos;\nuniform vec2 clip0;\nuniform vec2 clip1;\n\nvoid main()\n{\n    if (pos.x < clip0.x || pos.y < clip0.y || pos.x >= clip1.x || pos.y >= clip1.y) {\n        discard;\n    }\n\n    vec4 color0 = texture2D(tex0, TexCoord0.st);\n    vec4 color1 = texture2D(tex1, TexCoord1.st);\n\n    gl_FragColor = mix(color0, color1, done) * Color;\n}\n";
static const char __pyx_k_uniform_vec4_Color_uniform_sampl_5[] = "uniform vec4 Color;\nuniform sampler2D tex0;\nuniform sampler2D tex1;\nuniform sampler2D tex2;\nuniform float offset;\nuniform float multiplier;\n\nvarying vec2 TexCoord0;\nvarying vec2 TexCoord1;\nvarying vec2 TexCoord2;\n\nvarying vec2 pos;\nuniform vec2 clip0;\nuniform vec2 clip1;\n\nvoid main()\n{\n    vec4 color0 = texture2D(tex0, TexCoord0.st);\n    vec4 color1 = texture2D(tex1, TexCoord1.st);\n    vec4 color2 = texture2D(tex2, TexCoord2.st);\n\n    float a = clamp((color0.a + offset) * multiplier, 0.0, 1.0);\n\n    gl_FragColor = mix(color1, color2, a) * Color;\n}\n";
static const char __pyx_k_uniform_vec4_Color_uniform_sampl_6[] = "uniform vec4 Color;\nuniform sampler2D tex0;\nuniform sampler2D tex1;\nuniform sampler2D tex2;\nuniform float offset;\nuniform float multiplier;\n\nvarying vec2 TexCoord0;\nvarying vec2 TexCoord1;\nvarying vec2 TexCoord2;\n\nvarying vec2 pos;\nuniform vec2 clip0;\nuniform vec2 clip1;\n\nvoid main()\n{\n    if (pos.x < clip0.x || pos.y < clip0.y || pos.x >= clip1.x || pos.y >= clip1.y) {\n        discard;\n    }\n\n    vec4 color0 = texture2D(tex0, TexCoord0.st);\n    vec4 color1 = texture2D(tex1, TexCoord1.st);\n    vec4 color2 = texture2D(tex2, TexCoord2.st);\n\n    float a = clamp((color0.a + offset) * multiplier, 0.0, 1.0);\n\n    gl_FragColor = mix(color1, color2, a) * Color;\n}\n";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_n_s_BLEND_CLIP_SHADER;
static PyObject *__pyx_n_s_BLEND_SHADER;
static PyObject *__pyx_n_s_BLIT_CLIP_SHADER;
static PyObject *__pyx_n_s_BLIT_SHADER;
static PyObject *__pyx_n_s_GLES_PORTABILITY;
static PyObject *__pyx_n_s_IMAGEBLEND_CLIP_SHADER;
static PyObject *__pyx_n_s_IMAGEBLEND_SHADER;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xb6;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xc2;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_OverflowError;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_s_Shader_error_s;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_VERTEX_SHADER1;
static PyObject *__pyx_n_s_VERTEX_SHADER2;
static PyObject *__pyx_n_s_VERTEX_SHADER3;
static PyObject *__pyx_n_s_activate;
static PyObject *__pyx_n_s_check_status;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_compile_program;
static PyObject *__pyx_n_s_compile_shader;
static PyObject *__pyx_n_s_delete;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_disable_attribs;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_fragment;
static PyObject *__pyx_n_s_fragment_shader;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_handle;
static PyObject *__pyx_kp_s_home_tom_ab_renpy_renpy_angle_g;
static PyObject *__pyx_kp_s_ifdef_GL_ES_ifdef_GL_FRAGMENT_P;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_kind;
static PyObject *__pyx_n_s_lensource;
static PyObject *__pyx_n_s_log;
static PyObject *__pyx_n_s_log_length;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_program;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Program;
static PyObject *__pyx_n_s_pyx_unpickle_ShaderEnviron;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_renpy_angle_glenviron_shader;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shader;
static PyObject *__pyx_n_s_source;
static PyObject *__pyx_n_s_sourceptr;
static PyObject *__pyx_n_s_status;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_kp_s_uniform_mat4_Projection_attribut;
static PyObject *__pyx_kp_s_uniform_mat4_Projection_attribut_2;
static PyObject *__pyx_kp_s_uniform_mat4_Projection_attribut_3;
static PyObject *__pyx_kp_s_uniform_vec4_Color_uniform_sampl;
static PyObject *__pyx_kp_s_uniform_vec4_Color_uniform_sampl_2;
static PyObject *__pyx_kp_s_uniform_vec4_Color_uniform_sampl_3;
static PyObject *__pyx_kp_s_uniform_vec4_Color_uniform_sampl_4;
static PyObject *__pyx_kp_s_uniform_vec4_Color_uniform_sampl_5;
static PyObject *__pyx_kp_s_uniform_vec4_Color_uniform_sampl_6;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_vertex;
static PyObject *__pyx_n_s_vertex_shader;
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_check_status(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_handle, PyObject *__pyx_v_type); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_2compile_shader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_kind, PyObject *__pyx_v_source); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_4compile_program(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_vertex, PyObject *__pyx_v_fragment); /* proto */
static int __pyx_pf_5renpy_5angle_16glenviron_shader_7Program___init__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self, PyObject *__pyx_v_vertex, PyObject *__pyx_v_fragment); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_2disable_attribs(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_4delete(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_6__reduce_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_8__setstate_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_init(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_2deinit(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_4activate(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_program); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_6__reduce_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_8__setstate_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_6__pyx_unpickle_Program(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_8__pyx_unpickle_ShaderEnviron(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_5renpy_5angle_16glenviron_shader_Program(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_5angle_16glenviron_shader_ShaderEnviron(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_191470564;
static PyObject *__pyx_int_203683115;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_codeobj__3;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__9;
static PyObject *__pyx_codeobj__11;

/* "renpy/angle/glenviron_shader.pyx":33
 * from gldraw cimport *
 * 
 * cdef int round(double d):             # <<<<<<<<<<<<<<
 *     return <int> (d + .5)
 * 
 */

static int __pyx_f_5renpy_5angle_16glenviron_shader_round(double __pyx_v_d) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("round", 0);

  /* "renpy/angle/glenviron_shader.pyx":34
 * 
 * cdef int round(double d):
 *     return <int> (d + .5)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = ((int)(__pyx_v_d + .5));
  goto __pyx_L0;

  /* "renpy/angle/glenviron_shader.pyx":33
 * from gldraw cimport *
 * 
 * cdef int round(double d):             # <<<<<<<<<<<<<<
 *     return <int> (d + .5)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":263
 * 
 * 
 * def check_status(shader, handle, type):             # <<<<<<<<<<<<<<
 *     """
 *     Checks the status of a shader or program. If it fails, then an
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_1check_status(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_5angle_16glenviron_shader_check_status[] = "\n    Checks the status of a shader or program. If it fails, then an\n    exception is raised.\n    ";
static PyMethodDef __pyx_mdef_5renpy_5angle_16glenviron_shader_1check_status = {"check_status", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_1check_status, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_5angle_16glenviron_shader_check_status};
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_1check_status(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shader = 0;
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_type = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("check_status (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shader,&__pyx_n_s_handle,&__pyx_n_s_type,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shader)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("check_status", 1, 3, 3, 1); __PYX_ERR(0, 263, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("check_status", 1, 3, 3, 2); __PYX_ERR(0, 263, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "check_status") < 0)) __PYX_ERR(0, 263, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_shader = values[0];
    __pyx_v_handle = values[1];
    __pyx_v_type = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("check_status", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 263, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.check_status", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_check_status(__pyx_self, __pyx_v_shader, __pyx_v_handle, __pyx_v_type);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_check_status(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_shader, PyObject *__pyx_v_handle, PyObject *__pyx_v_type) {
  GLint __pyx_v_status;
  GLint __pyx_v_log_length;
  PyObject *__pyx_v_log = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  GLuint __pyx_t_2;
  GLenum __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  GLhandleARB __pyx_t_7;
  char *__pyx_t_8;
  __Pyx_RefNannySetupContext("check_status", 0);

  /* "renpy/angle/glenviron_shader.pyx":269
 *     """
 * 
 *     cdef GLint status = 0             # <<<<<<<<<<<<<<
 *     cdef GLint log_length = 0
 * 
 */
  __pyx_v_status = 0;

  /* "renpy/angle/glenviron_shader.pyx":270
 * 
 *     cdef GLint status = 0
 *     cdef GLint log_length = 0             # <<<<<<<<<<<<<<
 * 
 *     if shader:
 */
  __pyx_v_log_length = 0;

  /* "renpy/angle/glenviron_shader.pyx":272
 *     cdef GLint log_length = 0
 * 
 *     if shader:             # <<<<<<<<<<<<<<
 *         glGetShaderiv(handle, type, &status)
 *     else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_shader); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 272, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":273
 * 
 *     if shader:
 *         glGetShaderiv(handle, type, &status)             # <<<<<<<<<<<<<<
 *     else:
 *         glGetProgramiv(handle, type, &status)
 */
    __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_handle); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 273, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(0, 273, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glGetShaderiv(__pyx_t_2, __pyx_t_3, (&__pyx_v_status)); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 273, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":272
 *     cdef GLint log_length = 0
 * 
 *     if shader:             # <<<<<<<<<<<<<<
 *         glGetShaderiv(handle, type, &status)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":275
 *         glGetShaderiv(handle, type, &status)
 *     else:
 *         glGetProgramiv(handle, type, &status)             # <<<<<<<<<<<<<<
 * 
 *     if status == 1: # 0 for problems.
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_handle); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 275, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_type); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(0, 275, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glGetProgramiv(__pyx_t_2, __pyx_t_3, (&__pyx_v_status)); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 275, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "renpy/angle/glenviron_shader.pyx":277
 *         glGetProgramiv(handle, type, &status)
 * 
 *     if status == 1: # 0 for problems.             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_1 = ((__pyx_v_status == 1) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":278
 * 
 *     if status == 1: # 0 for problems.
 *         return             # <<<<<<<<<<<<<<
 * 
 *     if shader:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "renpy/angle/glenviron_shader.pyx":277
 *         glGetProgramiv(handle, type, &status)
 * 
 *     if status == 1: # 0 for problems.             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":280
 *         return
 * 
 *     if shader:             # <<<<<<<<<<<<<<
 *         glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &log_length)
 *     else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_shader); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 280, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":281
 * 
 *     if shader:
 *         glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &log_length)             # <<<<<<<<<<<<<<
 *     else:
 *         glGetProgramiv(handle, GL_INFO_LOG_LENGTH, &log_length)
 */
    __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_handle); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 281, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glGetShaderiv(__pyx_t_2, GL_INFO_LOG_LENGTH, (&__pyx_v_log_length)); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 281, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":280
 *         return
 * 
 *     if shader:             # <<<<<<<<<<<<<<
 *         glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &log_length)
 *     else:
 */
    goto __pyx_L5;
  }

  /* "renpy/angle/glenviron_shader.pyx":283
 *         glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &log_length)
 *     else:
 *         glGetProgramiv(handle, GL_INFO_LOG_LENGTH, &log_length)             # <<<<<<<<<<<<<<
 * 
 *     log = ' ' * log_length
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyInt_As_GLuint(__pyx_v_handle); if (unlikely((__pyx_t_2 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 283, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glGetProgramiv(__pyx_t_2, GL_INFO_LOG_LENGTH, (&__pyx_v_log_length)); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 283, __pyx_L1_error)
  }
  __pyx_L5:;

  /* "renpy/angle/glenviron_shader.pyx":285
 *         glGetProgramiv(handle, GL_INFO_LOG_LENGTH, &log_length)
 * 
 *     log = ' ' * log_length             # <<<<<<<<<<<<<<
 * 
 *     if shader:
 */
  __pyx_t_5 = __Pyx_PyInt_From_GLint(__pyx_v_log_length); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyNumber_Multiply(__pyx_kp_s_, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_log = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "renpy/angle/glenviron_shader.pyx":287
 *     log = ' ' * log_length
 * 
 *     if shader:             # <<<<<<<<<<<<<<
 *         glGetShaderInfoLog(handle, log_length, &log_length, <char *> log)
 *     else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_shader); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":288
 * 
 *     if shader:
 *         glGetShaderInfoLog(handle, log_length, &log_length, <char *> log)             # <<<<<<<<<<<<<<
 *     else:
 *         glGetProgramInfoLog(handle, log_length, &log_length, <char *> log)
 */
    __pyx_t_7 = __Pyx_PyInt_As_GLhandleARB(__pyx_v_handle); if (unlikely((__pyx_t_7 == ((GLhandleARB)-1)) && PyErr_Occurred())) __PYX_ERR(0, 288, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyObject_AsWritableString(__pyx_v_log); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 288, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glGetShaderInfoLog(__pyx_t_7, __pyx_v_log_length, (&__pyx_v_log_length), ((char *)__pyx_t_8)); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 288, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":287
 *     log = ' ' * log_length
 * 
 *     if shader:             # <<<<<<<<<<<<<<
 *         glGetShaderInfoLog(handle, log_length, &log_length, <char *> log)
 *     else:
 */
    goto __pyx_L6;
  }

  /* "renpy/angle/glenviron_shader.pyx":290
 *         glGetShaderInfoLog(handle, log_length, &log_length, <char *> log)
 *     else:
 *         glGetProgramInfoLog(handle, log_length, &log_length, <char *> log)             # <<<<<<<<<<<<<<
 * 
 *     raise Exception("Shader error: %s" % log)
 */
  /*else*/ {
    __pyx_t_7 = __Pyx_PyInt_As_GLhandleARB(__pyx_v_handle); if (unlikely((__pyx_t_7 == ((GLhandleARB)-1)) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L1_error)
    __pyx_t_8 = __Pyx_PyObject_AsWritableString(__pyx_v_log); if (unlikely((!__pyx_t_8) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glGetProgramInfoLog(__pyx_t_7, __pyx_v_log_length, (&__pyx_v_log_length), ((char *)__pyx_t_8)); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 290, __pyx_L1_error)
  }
  __pyx_L6:;

  /* "renpy/angle/glenviron_shader.pyx":292
 *         glGetProgramInfoLog(handle, log_length, &log_length, <char *> log)
 * 
 *     raise Exception("Shader error: %s" % log)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyString_Format(__pyx_kp_s_Shader_error_s, __pyx_v_log); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(0, 292, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":263
 * 
 * 
 * def check_status(shader, handle, type):             # <<<<<<<<<<<<<<
 *     """
 *     Checks the status of a shader or program. If it fails, then an
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.check_status", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_log);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":295
 * 
 * 
 * def compile_shader(kind, source):             # <<<<<<<<<<<<<<
 *     """
 *     Allocates and compiles a shader.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_3compile_shader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_5angle_16glenviron_shader_2compile_shader[] = "\n    Allocates and compiles a shader.\n    ";
static PyMethodDef __pyx_mdef_5renpy_5angle_16glenviron_shader_3compile_shader = {"compile_shader", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_3compile_shader, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_5angle_16glenviron_shader_2compile_shader};
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_3compile_shader(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_kind = 0;
  PyObject *__pyx_v_source = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compile_shader (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_kind,&__pyx_n_s_source,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kind)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_source)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile_shader", 1, 2, 2, 1); __PYX_ERR(0, 295, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compile_shader") < 0)) __PYX_ERR(0, 295, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_kind = values[0];
    __pyx_v_source = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compile_shader", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 295, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.compile_shader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_2compile_shader(__pyx_self, __pyx_v_kind, __pyx_v_source);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_2compile_shader(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_kind, PyObject *__pyx_v_source) {
  char *__pyx_v_sourceptr;
  int __pyx_v_lensource;
  GLhandleARB __pyx_v_handle;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  GLenum __pyx_t_3;
  GLhandleARB __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("compile_shader", 0);

  /* "renpy/angle/glenviron_shader.pyx":300
 *     """
 * 
 *     cdef char *sourceptr = <char *> source             # <<<<<<<<<<<<<<
 *     cdef int lensource = len(source)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_AsWritableString(__pyx_v_source); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 300, __pyx_L1_error)
  __pyx_v_sourceptr = ((char *)__pyx_t_1);

  /* "renpy/angle/glenviron_shader.pyx":301
 * 
 *     cdef char *sourceptr = <char *> source
 *     cdef int lensource = len(source)             # <<<<<<<<<<<<<<
 * 
 *     handle = glCreateShaderObjectARB(kind)
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_source); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 301, __pyx_L1_error)
  __pyx_v_lensource = __pyx_t_2;

  /* "renpy/angle/glenviron_shader.pyx":303
 *     cdef int lensource = len(source)
 * 
 *     handle = glCreateShaderObjectARB(kind)             # <<<<<<<<<<<<<<
 *     glShaderSourceARB(handle, 1, <GLcharARB **> &sourceptr, &lensource)
 *     glCompileShaderARB(handle)
 */
  __pyx_t_3 = __Pyx_PyInt_As_GLenum(__pyx_v_kind); if (unlikely((__pyx_t_3 == ((GLenum)-1)) && PyErr_Occurred())) __PYX_ERR(0, 303, __pyx_L1_error)
  __pyx_t_4 = __pyx_f_5renpy_5angle_2gl_glCreateShaderObjectARB(__pyx_t_3); if (unlikely(__pyx_t_4 == ((GLhandleARB)0) && PyErr_Occurred())) __PYX_ERR(0, 303, __pyx_L1_error)
  __pyx_v_handle = __pyx_t_4;

  /* "renpy/angle/glenviron_shader.pyx":304
 * 
 *     handle = glCreateShaderObjectARB(kind)
 *     glShaderSourceARB(handle, 1, <GLcharARB **> &sourceptr, &lensource)             # <<<<<<<<<<<<<<
 *     glCompileShaderARB(handle)
 * 
 */
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glShaderSourceARB(__pyx_v_handle, 1, ((GLcharARB **)(&__pyx_v_sourceptr)), (&__pyx_v_lensource)); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 304, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":305
 *     handle = glCreateShaderObjectARB(kind)
 *     glShaderSourceARB(handle, 1, <GLcharARB **> &sourceptr, &lensource)
 *     glCompileShaderARB(handle)             # <<<<<<<<<<<<<<
 * 
 *     check_status(True, handle, GL_OBJECT_COMPILE_STATUS_ARB)
 */
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glCompileShaderARB(__pyx_v_handle); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":307
 *     glCompileShaderARB(handle)
 * 
 *     check_status(True, handle, GL_OBJECT_COMPILE_STATUS_ARB)             # <<<<<<<<<<<<<<
 * 
 *     return handle
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_check_status); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_GLhandleARB(__pyx_v_handle); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(GL_OBJECT_COMPILE_STATUS_ARB); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_7)) {
    PyObject *__pyx_temp[4] = {__pyx_t_10, Py_True, __pyx_t_8, __pyx_t_9};
    __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
    PyObject *__pyx_temp[4] = {__pyx_t_10, Py_True, __pyx_t_8, __pyx_t_9};
    __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_11 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_INCREF(Py_True);
    __Pyx_GIVEREF(Py_True);
    PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_5, Py_True);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_5, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_5, __pyx_t_9);
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "renpy/angle/glenviron_shader.pyx":309
 *     check_status(True, handle, GL_OBJECT_COMPILE_STATUS_ARB)
 * 
 *     return handle             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = __Pyx_PyInt_From_GLhandleARB(__pyx_v_handle); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "renpy/angle/glenviron_shader.pyx":295
 * 
 * 
 * def compile_shader(kind, source):             # <<<<<<<<<<<<<<
 *     """
 *     Allocates and compiles a shader.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.compile_shader", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":312
 * 
 * 
 * def compile_program(vertex, fragment):             # <<<<<<<<<<<<<<
 *     """
 *     Compiles a pair of shaders into a program.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_5compile_program(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_5angle_16glenviron_shader_4compile_program[] = "\n    Compiles a pair of shaders into a program.\n    ";
static PyMethodDef __pyx_mdef_5renpy_5angle_16glenviron_shader_5compile_program = {"compile_program", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_5compile_program, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_5angle_16glenviron_shader_4compile_program};
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_5compile_program(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_vertex = 0;
  PyObject *__pyx_v_fragment = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compile_program (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_vertex,&__pyx_n_s_fragment,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_vertex)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fragment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compile_program", 1, 2, 2, 1); __PYX_ERR(0, 312, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compile_program") < 0)) __PYX_ERR(0, 312, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_vertex = values[0];
    __pyx_v_fragment = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compile_program", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 312, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.compile_program", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_4compile_program(__pyx_self, __pyx_v_vertex, __pyx_v_fragment);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_4compile_program(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_vertex, PyObject *__pyx_v_fragment) {
  PyObject *__pyx_v_vertex_shader = NULL;
  PyObject *__pyx_v_fragment_shader = NULL;
  GLhandleARB __pyx_v_program;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  GLhandleARB __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  GLuint __pyx_t_9;
  __Pyx_RefNannySetupContext("compile_program", 0);

  /* "renpy/angle/glenviron_shader.pyx":317
 *     """
 * 
 *     vertex_shader = compile_shader(GL_VERTEX_SHADER_ARB, vertex)             # <<<<<<<<<<<<<<
 *     fragment_shader = compile_shader(GL_FRAGMENT_SHADER_ARB, fragment)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_compile_shader); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(GL_VERTEX_SHADER_ARB); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, __pyx_v_vertex};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_3, __pyx_v_vertex};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_vertex);
    __Pyx_GIVEREF(__pyx_v_vertex);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_vertex);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_vertex_shader = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/angle/glenviron_shader.pyx":318
 * 
 *     vertex_shader = compile_shader(GL_VERTEX_SHADER_ARB, vertex)
 *     fragment_shader = compile_shader(GL_FRAGMENT_SHADER_ARB, fragment)             # <<<<<<<<<<<<<<
 * 
 *     program = glCreateProgramObjectARB()
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_compile_shader); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyInt_From_int(GL_FRAGMENT_SHADER_ARB); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_6, __pyx_v_fragment};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_6, __pyx_v_fragment};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_fragment);
    __Pyx_GIVEREF(__pyx_v_fragment);
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_v_fragment);
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_fragment_shader = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/angle/glenviron_shader.pyx":320
 *     fragment_shader = compile_shader(GL_FRAGMENT_SHADER_ARB, fragment)
 * 
 *     program = glCreateProgramObjectARB()             # <<<<<<<<<<<<<<
 * 
 *     glAttachObjectARB(program, vertex_shader)
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glCreateProgramObjectARB(); if (unlikely(__pyx_t_7 == ((GLhandleARB)0) && PyErr_Occurred())) __PYX_ERR(0, 320, __pyx_L1_error)
  __pyx_v_program = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":322
 *     program = glCreateProgramObjectARB()
 * 
 *     glAttachObjectARB(program, vertex_shader)             # <<<<<<<<<<<<<<
 *     glAttachObjectARB(program, fragment_shader)
 * 
 */
  __pyx_t_7 = __Pyx_PyInt_As_GLhandleARB(__pyx_v_vertex_shader); if (unlikely((__pyx_t_7 == ((GLhandleARB)-1)) && PyErr_Occurred())) __PYX_ERR(0, 322, __pyx_L1_error)
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glAttachObjectARB(__pyx_v_program, __pyx_t_7); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 322, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":323
 * 
 *     glAttachObjectARB(program, vertex_shader)
 *     glAttachObjectARB(program, fragment_shader)             # <<<<<<<<<<<<<<
 * 
 *     glLinkProgramARB(program)
 */
  __pyx_t_7 = __Pyx_PyInt_As_GLhandleARB(__pyx_v_fragment_shader); if (unlikely((__pyx_t_7 == ((GLhandleARB)-1)) && PyErr_Occurred())) __PYX_ERR(0, 323, __pyx_L1_error)
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glAttachObjectARB(__pyx_v_program, __pyx_t_7); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 323, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":325
 *     glAttachObjectARB(program, fragment_shader)
 * 
 *     glLinkProgramARB(program)             # <<<<<<<<<<<<<<
 * 
 *     check_status(False, program, GL_OBJECT_LINK_STATUS_ARB)
 */
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glLinkProgramARB(__pyx_v_program); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 325, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":327
 *     glLinkProgramARB(program)
 * 
 *     check_status(False, program, GL_OBJECT_LINK_STATUS_ARB)             # <<<<<<<<<<<<<<
 * 
 *     glUseProgramObjectARB(program)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_check_status); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyInt_From_GLhandleARB(__pyx_v_program); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_int(GL_OBJECT_LINK_STATUS_ARB); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, Py_False, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, Py_False, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(Py_False);
    __Pyx_GIVEREF(Py_False);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_5, Py_False);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_5, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_5, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/angle/glenviron_shader.pyx":329
 *     check_status(False, program, GL_OBJECT_LINK_STATUS_ARB)
 * 
 *     glUseProgramObjectARB(program)             # <<<<<<<<<<<<<<
 * 
 *     glDeleteShader(vertex_shader)
 */
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glUseProgramObjectARB(__pyx_v_program); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 329, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":331
 *     glUseProgramObjectARB(program)
 * 
 *     glDeleteShader(vertex_shader)             # <<<<<<<<<<<<<<
 *     glDeleteShader(fragment_shader)
 * 
 */
  __pyx_t_9 = __Pyx_PyInt_As_GLuint(__pyx_v_vertex_shader); if (unlikely((__pyx_t_9 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 331, __pyx_L1_error)
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glDeleteShader(__pyx_t_9); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 331, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":332
 * 
 *     glDeleteShader(vertex_shader)
 *     glDeleteShader(fragment_shader)             # <<<<<<<<<<<<<<
 * 
 *     return program
 */
  __pyx_t_9 = __Pyx_PyInt_As_GLuint(__pyx_v_fragment_shader); if (unlikely((__pyx_t_9 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 332, __pyx_L1_error)
  __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glDeleteShader(__pyx_t_9); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 332, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":334
 *     glDeleteShader(fragment_shader)
 * 
 *     return program             # <<<<<<<<<<<<<<
 * 
 * cdef class Program(object):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_GLhandleARB(__pyx_v_program); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/angle/glenviron_shader.pyx":312
 * 
 * 
 * def compile_program(vertex, fragment):             # <<<<<<<<<<<<<<
 *     """
 *     Compiles a pair of shaders into a program.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.compile_program", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_vertex_shader);
  __Pyx_XDECREF(__pyx_v_fragment_shader);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":364
 *     cdef GLint done
 * 
 *     def __init__(self, vertex, fragment):             # <<<<<<<<<<<<<<
 *         self.program = compile_program(vertex, fragment)
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_5angle_16glenviron_shader_7Program_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_5angle_16glenviron_shader_7Program_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_vertex = 0;
  PyObject *__pyx_v_fragment = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_vertex,&__pyx_n_s_fragment,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_vertex)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fragment)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 364, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 364, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_vertex = values[0];
    __pyx_v_fragment = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 364, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.Program.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_7Program___init__(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v_self), __pyx_v_vertex, __pyx_v_fragment);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_5angle_16glenviron_shader_7Program___init__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self, PyObject *__pyx_v_vertex, PyObject *__pyx_v_fragment) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  GLuint __pyx_t_6;
  GLint __pyx_t_7;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/angle/glenviron_shader.pyx":365
 * 
 *     def __init__(self, vertex, fragment):
 *         self.program = compile_program(vertex, fragment)             # <<<<<<<<<<<<<<
 * 
 *         self.Vertex = glGetAttribLocationARB(self.program, "Vertex")
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_compile_program); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_vertex, __pyx_v_fragment};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_vertex, __pyx_v_fragment};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_vertex);
    __Pyx_GIVEREF(__pyx_v_vertex);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_vertex);
    __Pyx_INCREF(__pyx_v_fragment);
    __Pyx_GIVEREF(__pyx_v_fragment);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_fragment);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_GLuint(__pyx_t_1); if (unlikely((__pyx_t_6 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->program = __pyx_t_6;

  /* "renpy/angle/glenviron_shader.pyx":367
 *         self.program = compile_program(vertex, fragment)
 * 
 *         self.Vertex = glGetAttribLocationARB(self.program, "Vertex")             # <<<<<<<<<<<<<<
 *         self.VertexTexCoord0 = glGetAttribLocationARB(self.program, "VertexTexCoord0")
 *         self.VertexTexCoord1 = glGetAttribLocationARB(self.program, "VertexTexCoord1")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetAttribLocationARB(__pyx_v_self->program, ((GLchar *)"Vertex")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L1_error)
  __pyx_v_self->Vertex = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":368
 * 
 *         self.Vertex = glGetAttribLocationARB(self.program, "Vertex")
 *         self.VertexTexCoord0 = glGetAttribLocationARB(self.program, "VertexTexCoord0")             # <<<<<<<<<<<<<<
 *         self.VertexTexCoord1 = glGetAttribLocationARB(self.program, "VertexTexCoord1")
 *         self.VertexTexCoord2 = glGetAttribLocationARB(self.program, "VertexTexCoord2")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetAttribLocationARB(__pyx_v_self->program, ((GLchar *)"VertexTexCoord0")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 368, __pyx_L1_error)
  __pyx_v_self->VertexTexCoord0 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":369
 *         self.Vertex = glGetAttribLocationARB(self.program, "Vertex")
 *         self.VertexTexCoord0 = glGetAttribLocationARB(self.program, "VertexTexCoord0")
 *         self.VertexTexCoord1 = glGetAttribLocationARB(self.program, "VertexTexCoord1")             # <<<<<<<<<<<<<<
 *         self.VertexTexCoord2 = glGetAttribLocationARB(self.program, "VertexTexCoord2")
 * 
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetAttribLocationARB(__pyx_v_self->program, ((GLchar *)"VertexTexCoord1")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_v_self->VertexTexCoord1 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":370
 *         self.VertexTexCoord0 = glGetAttribLocationARB(self.program, "VertexTexCoord0")
 *         self.VertexTexCoord1 = glGetAttribLocationARB(self.program, "VertexTexCoord1")
 *         self.VertexTexCoord2 = glGetAttribLocationARB(self.program, "VertexTexCoord2")             # <<<<<<<<<<<<<<
 * 
 *         self.Projection = glGetUniformLocationARB(self.program, "Projection")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetAttribLocationARB(__pyx_v_self->program, ((GLchar *)"VertexTexCoord2")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 370, __pyx_L1_error)
  __pyx_v_self->VertexTexCoord2 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":372
 *         self.VertexTexCoord2 = glGetAttribLocationARB(self.program, "VertexTexCoord2")
 * 
 *         self.Projection = glGetUniformLocationARB(self.program, "Projection")             # <<<<<<<<<<<<<<
 *         self.tex0 = glGetUniformLocationARB(self.program, "tex0")
 *         self.tex1 = glGetUniformLocationARB(self.program, "tex1")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"Projection")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 372, __pyx_L1_error)
  __pyx_v_self->Projection = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":373
 * 
 *         self.Projection = glGetUniformLocationARB(self.program, "Projection")
 *         self.tex0 = glGetUniformLocationARB(self.program, "tex0")             # <<<<<<<<<<<<<<
 *         self.tex1 = glGetUniformLocationARB(self.program, "tex1")
 *         self.tex2 = glGetUniformLocationARB(self.program, "tex2")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"tex0")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 373, __pyx_L1_error)
  __pyx_v_self->tex0 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":374
 *         self.Projection = glGetUniformLocationARB(self.program, "Projection")
 *         self.tex0 = glGetUniformLocationARB(self.program, "tex0")
 *         self.tex1 = glGetUniformLocationARB(self.program, "tex1")             # <<<<<<<<<<<<<<
 *         self.tex2 = glGetUniformLocationARB(self.program, "tex2")
 *         self.offset = glGetUniformLocationARB(self.program, "offset")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"tex1")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 374, __pyx_L1_error)
  __pyx_v_self->tex1 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":375
 *         self.tex0 = glGetUniformLocationARB(self.program, "tex0")
 *         self.tex1 = glGetUniformLocationARB(self.program, "tex1")
 *         self.tex2 = glGetUniformLocationARB(self.program, "tex2")             # <<<<<<<<<<<<<<
 *         self.offset = glGetUniformLocationARB(self.program, "offset")
 *         self.multiplier = glGetUniformLocationARB(self.program, "multiplier")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"tex2")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)
  __pyx_v_self->tex2 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":376
 *         self.tex1 = glGetUniformLocationARB(self.program, "tex1")
 *         self.tex2 = glGetUniformLocationARB(self.program, "tex2")
 *         self.offset = glGetUniformLocationARB(self.program, "offset")             # <<<<<<<<<<<<<<
 *         self.multiplier = glGetUniformLocationARB(self.program, "multiplier")
 *         self.done = glGetUniformLocationARB(self.program, "done")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"offset")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)
  __pyx_v_self->offset = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":377
 *         self.tex2 = glGetUniformLocationARB(self.program, "tex2")
 *         self.offset = glGetUniformLocationARB(self.program, "offset")
 *         self.multiplier = glGetUniformLocationARB(self.program, "multiplier")             # <<<<<<<<<<<<<<
 *         self.done = glGetUniformLocationARB(self.program, "done")
 *         self.Color = glGetUniformLocationARB(self.program, "Color")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"multiplier")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 377, __pyx_L1_error)
  __pyx_v_self->multiplier = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":378
 *         self.offset = glGetUniformLocationARB(self.program, "offset")
 *         self.multiplier = glGetUniformLocationARB(self.program, "multiplier")
 *         self.done = glGetUniformLocationARB(self.program, "done")             # <<<<<<<<<<<<<<
 *         self.Color = glGetUniformLocationARB(self.program, "Color")
 *         self.clip0 = glGetUniformLocationARB(self.program, "clip0")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"done")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 378, __pyx_L1_error)
  __pyx_v_self->done = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":379
 *         self.multiplier = glGetUniformLocationARB(self.program, "multiplier")
 *         self.done = glGetUniformLocationARB(self.program, "done")
 *         self.Color = glGetUniformLocationARB(self.program, "Color")             # <<<<<<<<<<<<<<
 *         self.clip0 = glGetUniformLocationARB(self.program, "clip0")
 *         self.clip1 = glGetUniformLocationARB(self.program, "clip1")
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"Color")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 379, __pyx_L1_error)
  __pyx_v_self->Color = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":380
 *         self.done = glGetUniformLocationARB(self.program, "done")
 *         self.Color = glGetUniformLocationARB(self.program, "Color")
 *         self.clip0 = glGetUniformLocationARB(self.program, "clip0")             # <<<<<<<<<<<<<<
 *         self.clip1 = glGetUniformLocationARB(self.program, "clip1")
 * 
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"clip0")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 380, __pyx_L1_error)
  __pyx_v_self->clip0 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":381
 *         self.Color = glGetUniformLocationARB(self.program, "Color")
 *         self.clip0 = glGetUniformLocationARB(self.program, "clip0")
 *         self.clip1 = glGetUniformLocationARB(self.program, "clip1")             # <<<<<<<<<<<<<<
 * 
 *     def disable_attribs(self):
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB(__pyx_v_self->program, ((GLchar *)"clip1")); if (unlikely(__pyx_t_7 == ((GLint)0) && PyErr_Occurred())) __PYX_ERR(0, 381, __pyx_L1_error)
  __pyx_v_self->clip1 = __pyx_t_7;

  /* "renpy/angle/glenviron_shader.pyx":364
 *     cdef GLint done
 * 
 *     def __init__(self, vertex, fragment):             # <<<<<<<<<<<<<<
 *         self.program = compile_program(vertex, fragment)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.Program.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":383
 *         self.clip1 = glGetUniformLocationARB(self.program, "clip1")
 * 
 *     def disable_attribs(self):             # <<<<<<<<<<<<<<
 *         # Disable the vertex attributes used by this program.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_3disable_attribs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_3disable_attribs(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disable_attribs (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_7Program_2disable_attribs(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_2disable_attribs(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("disable_attribs", 0);

  /* "renpy/angle/glenviron_shader.pyx":386
 *         # Disable the vertex attributes used by this program.
 * 
 *         if self.Vertex != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.Vertex)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->Vertex != -1L) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":387
 * 
 *         if self.Vertex != -1:
 *             glDisableVertexAttribArrayARB(self.Vertex)             # <<<<<<<<<<<<<<
 * 
 *         if self.VertexTexCoord0 != -1:
 */
    __pyx_t_2 = __pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB(__pyx_v_self->Vertex); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 387, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":386
 *         # Disable the vertex attributes used by this program.
 * 
 *         if self.Vertex != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.Vertex)
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":389
 *             glDisableVertexAttribArrayARB(self.Vertex)
 * 
 *         if self.VertexTexCoord0 != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord0)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->VertexTexCoord0 != -1L) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":390
 * 
 *         if self.VertexTexCoord0 != -1:
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord0)             # <<<<<<<<<<<<<<
 * 
 *         if self.VertexTexCoord1 != -1:
 */
    __pyx_t_2 = __pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB(__pyx_v_self->VertexTexCoord0); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 390, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":389
 *             glDisableVertexAttribArrayARB(self.Vertex)
 * 
 *         if self.VertexTexCoord0 != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord0)
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":392
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord0)
 * 
 *         if self.VertexTexCoord1 != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord1)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->VertexTexCoord1 != -1L) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":393
 * 
 *         if self.VertexTexCoord1 != -1:
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord1)             # <<<<<<<<<<<<<<
 * 
 *         if self.VertexTexCoord2 != -1:
 */
    __pyx_t_2 = __pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB(__pyx_v_self->VertexTexCoord1); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":392
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord0)
 * 
 *         if self.VertexTexCoord1 != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord1)
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":395
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord1)
 * 
 *         if self.VertexTexCoord2 != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord2)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->VertexTexCoord2 != -1L) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":396
 * 
 *         if self.VertexTexCoord2 != -1:
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord2)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self):
 */
    __pyx_t_2 = __pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB(__pyx_v_self->VertexTexCoord2); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 396, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":395
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord1)
 * 
 *         if self.VertexTexCoord2 != -1:             # <<<<<<<<<<<<<<
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord2)
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":383
 *         self.clip1 = glGetUniformLocationARB(self.program, "clip1")
 * 
 *     def disable_attribs(self):             # <<<<<<<<<<<<<<
 *         # Disable the vertex attributes used by this program.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.Program.disable_attribs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":398
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord2)
 * 
 *     def delete(self):             # <<<<<<<<<<<<<<
 *         glDeleteProgram(self.program)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_5delete(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_5delete(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_7Program_4delete(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_4delete(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "renpy/angle/glenviron_shader.pyx":399
 * 
 *     def delete(self):
 *         glDeleteProgram(self.program)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_f_5renpy_5angle_2gl_glDeleteProgram(__pyx_v_self->program); if (unlikely(__pyx_t_1 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 399, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":398
 *             glDisableVertexAttribArrayARB(self.VertexTexCoord2)
 * 
 *     def delete(self):             # <<<<<<<<<<<<<<
 *         glDeleteProgram(self.program)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.Program.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.Color, self.Projection, self.Vertex, self.VertexTexCoord0, self.VertexTexCoord1, self.VertexTexCoord2, self.clip0, self.clip1, self.done, self.multiplier, self.offset, self.program, self.tex0, self.tex1, self.tex2)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_7Program_6__reduce_cython__(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_6__reduce_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_t_18;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.Color, self.Projection, self.Vertex, self.VertexTexCoord0, self.VertexTexCoord1, self.VertexTexCoord2, self.clip0, self.clip1, self.done, self.multiplier, self.offset, self.program, self.tex0, self.tex1, self.tex2)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_GLint(__pyx_v_self->Color); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_GLint(__pyx_v_self->Projection); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_GLint(__pyx_v_self->Vertex); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_GLint(__pyx_v_self->VertexTexCoord0); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_GLint(__pyx_v_self->VertexTexCoord1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_GLint(__pyx_v_self->VertexTexCoord2); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_GLint(__pyx_v_self->clip0); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_GLint(__pyx_v_self->clip1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_GLint(__pyx_v_self->done); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyInt_From_GLint(__pyx_v_self->multiplier); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyInt_From_GLint(__pyx_v_self->offset); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __Pyx_PyInt_From_GLuint(__pyx_v_self->program); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyInt_From_GLint(__pyx_v_self->tex0); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = __Pyx_PyInt_From_GLint(__pyx_v_self->tex1); if (unlikely(!__pyx_t_14)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_15 = __Pyx_PyInt_From_GLint(__pyx_v_self->tex2); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_16 = PyTuple_New(15); if (unlikely(!__pyx_t_16)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_16, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_16, 3, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_16, 4, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_16, 5, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_16, 6, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_16, 7, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_16, 8, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_16, 9, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_16, 10, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_16, 11, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_16, 12, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_16, 13, __pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_15);
  PyTuple_SET_ITEM(__pyx_t_16, 14, __pyx_t_15);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_t_15 = 0;
  __pyx_v_state = __pyx_t_16;
  __pyx_t_16 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.Color, self.Projection, self.Vertex, self.VertexTexCoord0, self.VertexTexCoord1, self.VertexTexCoord2, self.clip0, self.clip1, self.done, self.multiplier, self.offset, self.program, self.tex0, self.tex1, self.tex2)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_16 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_16)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_v__dict = __pyx_t_16;
  __pyx_t_16 = 0;

  /* "(tree fragment)":5
 *     state = (self.Color, self.Projection, self.Vertex, self.VertexTexCoord0, self.VertexTexCoord1, self.VertexTexCoord2, self.clip0, self.clip1, self.done, self.multiplier, self.offset, self.program, self.tex0, self.tex1, self.tex2)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_17 = (__pyx_v__dict != Py_None);
  __pyx_t_18 = (__pyx_t_17 != 0);
  if (__pyx_t_18) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_16 = PyTuple_New(1); if (unlikely(!__pyx_t_16)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_v__dict);
    __pyx_t_15 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_16); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, __pyx_t_15);
    __pyx_t_15 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.Color, self.Projection, self.Vertex, self.VertexTexCoord0, self.VertexTexCoord1, self.VertexTexCoord2, self.clip0, self.clip1, self.done, self.multiplier, self.offset, self.program, self.tex0, self.tex1, self.tex2)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, None), state
 *     else:
 */
  __pyx_t_18 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_18) {

    /* "(tree fragment)":11
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_15 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_Program); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_16 = PyTuple_New(3); if (unlikely(!__pyx_t_16)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_16, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_191470564);
    __Pyx_GIVEREF(__pyx_int_191470564);
    PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_int_191470564);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_16, 2, Py_None);
    __pyx_t_14 = PyTuple_New(3); if (unlikely(!__pyx_t_14)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_15);
    PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_15);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_16);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_v_state);
    __pyx_t_15 = 0;
    __pyx_t_16 = 0;
    __pyx_r = __pyx_t_14;
    __pyx_t_14 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, None), state
 *     else:
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Program__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_14 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_Program); if (unlikely(!__pyx_t_14)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_16 = PyTuple_New(3); if (unlikely(!__pyx_t_16)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_16);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_16, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_191470564);
    __Pyx_GIVEREF(__pyx_int_191470564);
    PyTuple_SET_ITEM(__pyx_t_16, 1, __pyx_int_191470564);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_16, 2, __pyx_v_state);
    __pyx_t_15 = PyTuple_New(2); if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_16);
    PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_16);
    __pyx_t_14 = 0;
    __pyx_t_16 = 0;
    __pyx_r = __pyx_t_15;
    __pyx_t_15 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.Color, self.Projection, self.Vertex, self.VertexTexCoord0, self.VertexTexCoord1, self.VertexTexCoord2, self.clip0, self.clip1, self.done, self.multiplier, self.offset, self.program, self.tex0, self.tex1, self.tex2)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.Program.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Program__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_7Program_8__setstate_cython__(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_7Program_8__setstate_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Program__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_Program__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Program, (type(self), 0xb699be4, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Program__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.Program.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":430
 * 
 * 
 *     def init(self):             # <<<<<<<<<<<<<<
 * 
 *         self.blit_program = Program(VERTEX_SHADER1, BLIT_SHADER)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_1init(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_1init(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("init (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_init(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_init(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("init", 0);

  /* "renpy/angle/glenviron_shader.pyx":432
 *     def init(self):
 * 
 *         self.blit_program = Program(VERTEX_SHADER1, BLIT_SHADER)             # <<<<<<<<<<<<<<
 *         self.blit_clip_program = Program(VERTEX_SHADER1, BLIT_CLIP_SHADER)
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_VERTEX_SHADER1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_BLIT_SHADER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->blit_program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->blit_program));
  __pyx_v_self->blit_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/angle/glenviron_shader.pyx":433
 * 
 *         self.blit_program = Program(VERTEX_SHADER1, BLIT_SHADER)
 *         self.blit_clip_program = Program(VERTEX_SHADER1, BLIT_CLIP_SHADER)             # <<<<<<<<<<<<<<
 * 
 *         self.blend_program = Program(VERTEX_SHADER2, BLEND_SHADER)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_VERTEX_SHADER1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_BLIT_CLIP_SHADER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->blit_clip_program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->blit_clip_program));
  __pyx_v_self->blit_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":435
 *         self.blit_clip_program = Program(VERTEX_SHADER1, BLIT_CLIP_SHADER)
 * 
 *         self.blend_program = Program(VERTEX_SHADER2, BLEND_SHADER)             # <<<<<<<<<<<<<<
 *         self.blend_clip_program = Program(VERTEX_SHADER2, BLEND_CLIP_SHADER)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_VERTEX_SHADER2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_BLEND_SHADER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->blend_program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->blend_program));
  __pyx_v_self->blend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/angle/glenviron_shader.pyx":436
 * 
 *         self.blend_program = Program(VERTEX_SHADER2, BLEND_SHADER)
 *         self.blend_clip_program = Program(VERTEX_SHADER2, BLEND_CLIP_SHADER)             # <<<<<<<<<<<<<<
 * 
 *         self.imageblend_program = Program(VERTEX_SHADER3, IMAGEBLEND_SHADER)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_VERTEX_SHADER2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_BLEND_CLIP_SHADER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->blend_clip_program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->blend_clip_program));
  __pyx_v_self->blend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/angle/glenviron_shader.pyx":438
 *         self.blend_clip_program = Program(VERTEX_SHADER2, BLEND_CLIP_SHADER)
 * 
 *         self.imageblend_program = Program(VERTEX_SHADER3, IMAGEBLEND_SHADER)             # <<<<<<<<<<<<<<
 *         self.imageblend_clip_program = Program(VERTEX_SHADER3, IMAGEBLEND_CLIP_SHADER)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_VERTEX_SHADER3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IMAGEBLEND_SHADER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->imageblend_program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->imageblend_program));
  __pyx_v_self->imageblend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":439
 * 
 *         self.imageblend_program = Program(VERTEX_SHADER3, IMAGEBLEND_SHADER)
 *         self.imageblend_clip_program = Program(VERTEX_SHADER3, IMAGEBLEND_CLIP_SHADER)             # <<<<<<<<<<<<<<
 * 
 *         # The current program.
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_VERTEX_SHADER3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_IMAGEBLEND_CLIP_SHADER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->imageblend_clip_program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->imageblend_clip_program));
  __pyx_v_self->imageblend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/angle/glenviron_shader.pyx":442
 * 
 *         # The current program.
 *         self.program = None             # <<<<<<<<<<<<<<
 * 
 *     def deinit(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->program));
  __pyx_v_self->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None);

  /* "renpy/angle/glenviron_shader.pyx":430
 * 
 * 
 *     def init(self):             # <<<<<<<<<<<<<<
 * 
 *         self.blit_program = Program(VERTEX_SHADER1, BLIT_SHADER)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.ShaderEnviron.init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":444
 *         self.program = None
 * 
 *     def deinit(self):             # <<<<<<<<<<<<<<
 *         """
 *         Called before changing the GL context.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_3deinit(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_5angle_16glenviron_shader_13ShaderEnviron_2deinit[] = "\n        Called before changing the GL context.\n        ";
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_3deinit(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deinit (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_2deinit(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_2deinit(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("deinit", 0);

  /* "renpy/angle/glenviron_shader.pyx":449
 *         """
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 *             self.program = None
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_self->program) != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/angle/glenviron_shader.pyx":450
 * 
 *         if self.program is not None:
 *             self.program.disable_attribs()             # <<<<<<<<<<<<<<
 *             self.program = None
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->program), __pyx_n_s_disable_attribs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/angle/glenviron_shader.pyx":451
 *         if self.program is not None:
 *             self.program.disable_attribs()
 *             self.program = None             # <<<<<<<<<<<<<<
 * 
 *         self.blit_program.delete()
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->program);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->program));
    __pyx_v_self->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None);

    /* "renpy/angle/glenviron_shader.pyx":449
 *         """
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 *             self.program = None
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":453
 *             self.program = None
 * 
 *         self.blit_program.delete()             # <<<<<<<<<<<<<<
 *         self.blend_program.delete()
 *         self.imageblend_program.delete()
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->blit_program), __pyx_n_s_delete); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 453, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 453, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":454
 * 
 *         self.blit_program.delete()
 *         self.blend_program.delete()             # <<<<<<<<<<<<<<
 *         self.imageblend_program.delete()
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->blend_program), __pyx_n_s_delete); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 454, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":455
 *         self.blit_program.delete()
 *         self.blend_program.delete()
 *         self.imageblend_program.delete()             # <<<<<<<<<<<<<<
 * 
 *     def activate(self, Program program):
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->imageblend_program), __pyx_n_s_delete); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_5) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 455, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 455, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":444
 *         self.program = None
 * 
 *     def deinit(self):             # <<<<<<<<<<<<<<
 *         """
 *         Called before changing the GL context.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.ShaderEnviron.deinit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":457
 *         self.imageblend_program.delete()
 * 
 *     def activate(self, Program program):             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_5activate(PyObject *__pyx_v_self, PyObject *__pyx_v_program); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_5activate(PyObject *__pyx_v_self, PyObject *__pyx_v_program) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("activate (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_program), __pyx_ptype_5renpy_5angle_16glenviron_shader_Program, 1, "program", 0))) __PYX_ERR(0, 457, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_4activate(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self), ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v_program));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_4activate(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_program) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("activate", 0);

  /* "renpy/angle/glenviron_shader.pyx":459
 *     def activate(self, Program program):
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_self->program) != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/angle/glenviron_shader.pyx":460
 * 
 *         if self.program is not None:
 *             self.program.disable_attribs()             # <<<<<<<<<<<<<<
 * 
 *         self.program = program
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->program), __pyx_n_s_disable_attribs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 460, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 460, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/angle/glenviron_shader.pyx":459
 *     def activate(self, Program program):
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":462
 *             self.program.disable_attribs()
 * 
 *         self.program = program             # <<<<<<<<<<<<<<
 * 
 *         glUseProgramObjectARB(program.program)
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_program));
  __Pyx_GOTREF(__pyx_v_self->program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->program));
  __pyx_v_self->program = __pyx_v_program;

  /* "renpy/angle/glenviron_shader.pyx":464
 *         self.program = program
 * 
 *         glUseProgramObjectARB(program.program)             # <<<<<<<<<<<<<<
 *         glUniformMatrix4fvARB(program.Projection, 1, GL_FALSE, self.projection)
 * 
 */
  __pyx_t_6 = __pyx_f_5renpy_5angle_2gl_glUseProgramObjectARB(__pyx_v_program->program); if (unlikely(__pyx_t_6 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":465
 * 
 *         glUseProgramObjectARB(program.program)
 *         glUniformMatrix4fvARB(program.Projection, 1, GL_FALSE, self.projection)             # <<<<<<<<<<<<<<
 * 
 *         if self.clipping:
 */
  __pyx_t_6 = __pyx_f_5renpy_5angle_2gl_glUniformMatrix4fvARB(__pyx_v_program->Projection, 1, GL_FALSE, __pyx_v_self->projection); if (unlikely(__pyx_t_6 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 465, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":467
 *         glUniformMatrix4fvARB(program.Projection, 1, GL_FALSE, self.projection)
 * 
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             glUniform2fARB(program.clip0, self.clip_x0 - .01, self.clip_y0 - .01)
 *             glUniform2fARB(program.clip1, self.clip_x1 + .01, self.clip_y1 + .01)
 */
  __pyx_t_2 = (__pyx_v_self->clipping != 0);
  if (__pyx_t_2) {

    /* "renpy/angle/glenviron_shader.pyx":468
 * 
 *         if self.clipping:
 *             glUniform2fARB(program.clip0, self.clip_x0 - .01, self.clip_y0 - .01)             # <<<<<<<<<<<<<<
 *             glUniform2fARB(program.clip1, self.clip_x1 + .01, self.clip_y1 + .01)
 * 
 */
    __pyx_t_6 = __pyx_f_5renpy_5angle_2gl_glUniform2fARB(__pyx_v_program->clip0, (__pyx_v_self->clip_x0 - .01), (__pyx_v_self->clip_y0 - .01)); if (unlikely(__pyx_t_6 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 468, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":469
 *         if self.clipping:
 *             glUniform2fARB(program.clip0, self.clip_x0 - .01, self.clip_y0 - .01)
 *             glUniform2fARB(program.clip1, self.clip_x1 + .01, self.clip_y1 + .01)             # <<<<<<<<<<<<<<
 * 
 *     cdef void blit(self):
 */
    __pyx_t_6 = __pyx_f_5renpy_5angle_2gl_glUniform2fARB(__pyx_v_program->clip1, (__pyx_v_self->clip_x1 + .01), (__pyx_v_self->clip_y1 + .01)); if (unlikely(__pyx_t_6 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":467
 *         glUniformMatrix4fvARB(program.Projection, 1, GL_FALSE, self.projection)
 * 
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             glUniform2fARB(program.clip0, self.clip_x0 - .01, self.clip_y0 - .01)
 *             glUniform2fARB(program.clip1, self.clip_x1 + .01, self.clip_y1 + .01)
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":457
 *         self.imageblend_program.delete()
 * 
 *     def activate(self, Program program):             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.ShaderEnviron.activate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/angle/glenviron_shader.pyx":471
 *             glUniform2fARB(program.clip1, self.clip_x1 + .01, self.clip_y1 + .01)
 * 
 *     cdef void blit(self):             # <<<<<<<<<<<<<<
 * 
 *         if self.clipping:
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_blit(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self) {
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_program = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("blit", 0);

  /* "renpy/angle/glenviron_shader.pyx":473
 *     cdef void blit(self):
 * 
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             program = self.blit_clip_program
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->clipping != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":474
 * 
 *         if self.clipping:
 *             program = self.blit_clip_program             # <<<<<<<<<<<<<<
 *         else:
 *             program = self.blit_program
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->blit_clip_program);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":473
 *     cdef void blit(self):
 * 
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             program = self.blit_clip_program
 *         else:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":476
 *             program = self.blit_clip_program
 *         else:
 *             program = self.blit_program             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not program:
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)__pyx_v_self->blit_program);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "renpy/angle/glenviron_shader.pyx":478
 *             program = self.blit_program
 * 
 *         if self.program is not program:             # <<<<<<<<<<<<<<
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 */
  __pyx_t_1 = (__pyx_v_self->program != __pyx_v_program);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "renpy/angle/glenviron_shader.pyx":479
 * 
 *         if self.program is not program:
 *             self.activate(program)             # <<<<<<<<<<<<<<
 *             glUniform1iARB(program.tex0, 0)
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_activate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 479, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_program)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_program)};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_program)};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 479, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_program));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_program));
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, ((PyObject *)__pyx_v_program));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":480
 *         if self.program is not program:
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)             # <<<<<<<<<<<<<<
 * 
 *     cdef void blend(self, double fraction):
 */
    __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1iARB(__pyx_v_program->tex0, 0); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 480, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":478
 *             program = self.blit_program
 * 
 *         if self.program is not program:             # <<<<<<<<<<<<<<
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":471
 *             glUniform2fARB(program.clip1, self.clip_x1 + .01, self.clip_y1 + .01)
 * 
 *     cdef void blit(self):             # <<<<<<<<<<<<<<
 * 
 *         if self.clipping:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.blit", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_program);
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":482
 *             glUniform1iARB(program.tex0, 0)
 * 
 *     cdef void blend(self, double fraction):             # <<<<<<<<<<<<<<
 *         if self.clipping:
 *             program = self.blend_clip_program
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_blend(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_fraction) {
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_program = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("blend", 0);

  /* "renpy/angle/glenviron_shader.pyx":483
 * 
 *     cdef void blend(self, double fraction):
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             program = self.blend_clip_program
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->clipping != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":484
 *     cdef void blend(self, double fraction):
 *         if self.clipping:
 *             program = self.blend_clip_program             # <<<<<<<<<<<<<<
 *         else:
 *             program = self.blend_program
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->blend_clip_program);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":483
 * 
 *     cdef void blend(self, double fraction):
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             program = self.blend_clip_program
 *         else:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":486
 *             program = self.blend_clip_program
 *         else:
 *             program = self.blend_program             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not program:
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)__pyx_v_self->blend_program);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "renpy/angle/glenviron_shader.pyx":488
 *             program = self.blend_program
 * 
 *         if self.program is not program:             # <<<<<<<<<<<<<<
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 */
  __pyx_t_1 = (__pyx_v_self->program != __pyx_v_program);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "renpy/angle/glenviron_shader.pyx":489
 * 
 *         if self.program is not program:
 *             self.activate(program)             # <<<<<<<<<<<<<<
 *             glUniform1iARB(program.tex0, 0)
 *             glUniform1iARB(program.tex1, 1)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_activate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_program)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_program)};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_program)};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_program));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_program));
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, ((PyObject *)__pyx_v_program));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":490
 *         if self.program is not program:
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)             # <<<<<<<<<<<<<<
 *             glUniform1iARB(program.tex1, 1)
 * 
 */
    __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1iARB(__pyx_v_program->tex0, 0); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 490, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":491
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 *             glUniform1iARB(program.tex1, 1)             # <<<<<<<<<<<<<<
 * 
 *         glUniform1fARB(program.done, fraction)
 */
    __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1iARB(__pyx_v_program->tex1, 1); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 491, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":488
 *             program = self.blend_program
 * 
 *         if self.program is not program:             # <<<<<<<<<<<<<<
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":493
 *             glUniform1iARB(program.tex1, 1)
 * 
 *         glUniform1fARB(program.done, fraction)             # <<<<<<<<<<<<<<
 * 
 *     cdef void imageblend(self, double fraction, int ramp):
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1fARB(__pyx_v_program->done, __pyx_v_fraction); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 493, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":482
 *             glUniform1iARB(program.tex0, 0)
 * 
 *     cdef void blend(self, double fraction):             # <<<<<<<<<<<<<<
 *         if self.clipping:
 *             program = self.blend_clip_program
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.blend", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_program);
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":495
 *         glUniform1fARB(program.done, fraction)
 * 
 *     cdef void imageblend(self, double fraction, int ramp):             # <<<<<<<<<<<<<<
 * 
 *         if self.clipping:
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_imageblend(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_fraction, int __pyx_v_ramp) {
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v_program = NULL;
  double __pyx_v_start;
  double __pyx_v_end;
  double __pyx_v_offset;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("imageblend", 0);

  /* "renpy/angle/glenviron_shader.pyx":497
 *     cdef void imageblend(self, double fraction, int ramp):
 * 
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             program = self.imageblend_clip_program
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->clipping != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":498
 * 
 *         if self.clipping:
 *             program = self.imageblend_clip_program             # <<<<<<<<<<<<<<
 *         else:
 *             program = self.imageblend_program
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self->imageblend_clip_program);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":497
 *     cdef void imageblend(self, double fraction, int ramp):
 * 
 *         if self.clipping:             # <<<<<<<<<<<<<<
 *             program = self.imageblend_clip_program
 *         else:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":500
 *             program = self.imageblend_clip_program
 *         else:
 *             program = self.imageblend_program             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not program:
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)__pyx_v_self->imageblend_program);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "renpy/angle/glenviron_shader.pyx":502
 *             program = self.imageblend_program
 * 
 *         if self.program is not program:             # <<<<<<<<<<<<<<
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 */
  __pyx_t_1 = (__pyx_v_self->program != __pyx_v_program);
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {

    /* "renpy/angle/glenviron_shader.pyx":503
 * 
 *         if self.program is not program:
 *             self.activate(program)             # <<<<<<<<<<<<<<
 *             glUniform1iARB(program.tex0, 0)
 *             glUniform1iARB(program.tex1, 1)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_activate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_program)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_program)};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, ((PyObject *)__pyx_v_program)};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_program));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_program));
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, ((PyObject *)__pyx_v_program));
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":504
 *         if self.program is not program:
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)             # <<<<<<<<<<<<<<
 *             glUniform1iARB(program.tex1, 1)
 *             glUniform1iARB(program.tex2, 2)
 */
    __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1iARB(__pyx_v_program->tex0, 0); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 504, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":505
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 *             glUniform1iARB(program.tex1, 1)             # <<<<<<<<<<<<<<
 *             glUniform1iARB(program.tex2, 2)
 * 
 */
    __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1iARB(__pyx_v_program->tex1, 1); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 505, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":506
 *             glUniform1iARB(program.tex0, 0)
 *             glUniform1iARB(program.tex1, 1)
 *             glUniform1iARB(program.tex2, 2)             # <<<<<<<<<<<<<<
 * 
 *         # Prevent a DBZ if the user gives us a 0 ramp.
 */
    __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1iARB(__pyx_v_program->tex2, 2); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 506, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":502
 *             program = self.imageblend_program
 * 
 *         if self.program is not program:             # <<<<<<<<<<<<<<
 *             self.activate(program)
 *             glUniform1iARB(program.tex0, 0)
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":509
 * 
 *         # Prevent a DBZ if the user gives us a 0 ramp.
 *         if ramp < 1:             # <<<<<<<<<<<<<<
 *             ramp = 1
 * 
 */
  __pyx_t_3 = ((__pyx_v_ramp < 1) != 0);
  if (__pyx_t_3) {

    /* "renpy/angle/glenviron_shader.pyx":510
 *         # Prevent a DBZ if the user gives us a 0 ramp.
 *         if ramp < 1:
 *             ramp = 1             # <<<<<<<<<<<<<<
 * 
 *         # Compute the offset to apply to the alpha.
 */
    __pyx_v_ramp = 1;

    /* "renpy/angle/glenviron_shader.pyx":509
 * 
 *         # Prevent a DBZ if the user gives us a 0 ramp.
 *         if ramp < 1:             # <<<<<<<<<<<<<<
 *             ramp = 1
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":513
 * 
 *         # Compute the offset to apply to the alpha.
 *         start = -1.0             # <<<<<<<<<<<<<<
 *         end = ramp / 256.0
 *         offset = start + ( end - start) * fraction
 */
  __pyx_v_start = -1.0;

  /* "renpy/angle/glenviron_shader.pyx":514
 *         # Compute the offset to apply to the alpha.
 *         start = -1.0
 *         end = ramp / 256.0             # <<<<<<<<<<<<<<
 *         offset = start + ( end - start) * fraction
 * 
 */
  __pyx_v_end = (__pyx_v_ramp / 256.0);

  /* "renpy/angle/glenviron_shader.pyx":515
 *         start = -1.0
 *         end = ramp / 256.0
 *         offset = start + ( end - start) * fraction             # <<<<<<<<<<<<<<
 * 
 *         # Setup the multiplier and the offset.
 */
  __pyx_v_offset = (__pyx_v_start + ((__pyx_v_end - __pyx_v_start) * __pyx_v_fraction));

  /* "renpy/angle/glenviron_shader.pyx":518
 * 
 *         # Setup the multiplier and the offset.
 *         glUniform1fARB(program.multiplier, 256.0 / ramp)             # <<<<<<<<<<<<<<
 *         glUniform1fARB(program.offset, offset)
 * 
 */
  if (unlikely(__pyx_v_ramp == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 518, __pyx_L1_error)
  }
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1fARB(__pyx_v_program->multiplier, (256.0 / __pyx_v_ramp)); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 518, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":519
 *         # Setup the multiplier and the offset.
 *         glUniform1fARB(program.multiplier, 256.0 / ramp)
 *         glUniform1fARB(program.offset, offset)             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_vertex(self, float *vertices):
 */
  __pyx_t_7 = __pyx_f_5renpy_5angle_2gl_glUniform1fARB(__pyx_v_program->offset, __pyx_v_offset); if (unlikely(__pyx_t_7 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 519, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":495
 *         glUniform1fARB(program.done, fraction)
 * 
 *     cdef void imageblend(self, double fraction, int ramp):             # <<<<<<<<<<<<<<
 * 
 *         if self.clipping:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.imageblend", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_program);
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":521
 *         glUniform1fARB(program.offset, offset)
 * 
 *     cdef void set_vertex(self, float *vertices):             # <<<<<<<<<<<<<<
 *         glEnableVertexAttribArrayARB(self.program.Vertex)
 *         glVertexAttribPointerARB(self.program.Vertex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> vertices)
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_vertex(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, float *__pyx_v_vertices) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("set_vertex", 0);

  /* "renpy/angle/glenviron_shader.pyx":522
 * 
 *     cdef void set_vertex(self, float *vertices):
 *         glEnableVertexAttribArrayARB(self.program.Vertex)             # <<<<<<<<<<<<<<
 *         glVertexAttribPointerARB(self.program.Vertex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> vertices)
 * 
 */
  __pyx_t_1 = __pyx_f_5renpy_5angle_2gl_glEnableVertexAttribArrayARB(__pyx_v_self->program->Vertex); if (unlikely(__pyx_t_1 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 522, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":523
 *     cdef void set_vertex(self, float *vertices):
 *         glEnableVertexAttribArrayARB(self.program.Vertex)
 *         glVertexAttribPointerARB(self.program.Vertex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> vertices)             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_texture(self, int unit, float *coords):
 */
  __pyx_t_1 = __pyx_f_5renpy_5angle_2gl_glVertexAttribPointerARB(__pyx_v_self->program->Vertex, 2, GL_FLOAT, GL_FALSE, 0, ((GLubyte *)__pyx_v_vertices)); if (unlikely(__pyx_t_1 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 523, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":521
 *         glUniform1fARB(program.offset, offset)
 * 
 *     cdef void set_vertex(self, float *vertices):             # <<<<<<<<<<<<<<
 *         glEnableVertexAttribArrayARB(self.program.Vertex)
 *         glVertexAttribPointerARB(self.program.Vertex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> vertices)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.set_vertex", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":525
 *         glVertexAttribPointerARB(self.program.Vertex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> vertices)
 * 
 *     cdef void set_texture(self, int unit, float *coords):             # <<<<<<<<<<<<<<
 *         cdef tex
 * 
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_texture(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, int __pyx_v_unit, float *__pyx_v_coords) {
  PyObject *__pyx_v_tex = 0;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  GLuint __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("set_texture", 0);

  /* "renpy/angle/glenviron_shader.pyx":528
 *         cdef tex
 * 
 *         if unit == 0:             # <<<<<<<<<<<<<<
 *             tex = self.program.VertexTexCoord0
 *         elif unit == 1:
 */
  __pyx_t_1 = ((__pyx_v_unit == 0) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":529
 * 
 *         if unit == 0:
 *             tex = self.program.VertexTexCoord0             # <<<<<<<<<<<<<<
 *         elif unit == 1:
 *             tex = self.program.VertexTexCoord1
 */
    __pyx_t_2 = __Pyx_PyInt_From_GLint(__pyx_v_self->program->VertexTexCoord0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_tex = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":528
 *         cdef tex
 * 
 *         if unit == 0:             # <<<<<<<<<<<<<<
 *             tex = self.program.VertexTexCoord0
 *         elif unit == 1:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":530
 *         if unit == 0:
 *             tex = self.program.VertexTexCoord0
 *         elif unit == 1:             # <<<<<<<<<<<<<<
 *             tex = self.program.VertexTexCoord1
 *         elif unit == 2 and RENPY_THIRD_TEXTURE:
 */
  __pyx_t_1 = ((__pyx_v_unit == 1) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":531
 *             tex = self.program.VertexTexCoord0
 *         elif unit == 1:
 *             tex = self.program.VertexTexCoord1             # <<<<<<<<<<<<<<
 *         elif unit == 2 and RENPY_THIRD_TEXTURE:
 *             tex = self.program.VertexTexCoord2
 */
    __pyx_t_2 = __Pyx_PyInt_From_GLint(__pyx_v_self->program->VertexTexCoord1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 531, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_tex = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":530
 *         if unit == 0:
 *             tex = self.program.VertexTexCoord0
 *         elif unit == 1:             # <<<<<<<<<<<<<<
 *             tex = self.program.VertexTexCoord1
 *         elif unit == 2 and RENPY_THIRD_TEXTURE:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":532
 *         elif unit == 1:
 *             tex = self.program.VertexTexCoord1
 *         elif unit == 2 and RENPY_THIRD_TEXTURE:             # <<<<<<<<<<<<<<
 *             tex = self.program.VertexTexCoord2
 *         else:
 */
  __pyx_t_3 = ((__pyx_v_unit == 2) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = RENPY_THIRD_TEXTURE;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":533
 *             tex = self.program.VertexTexCoord1
 *         elif unit == 2 and RENPY_THIRD_TEXTURE:
 *             tex = self.program.VertexTexCoord2             # <<<<<<<<<<<<<<
 *         else:
 *             return
 */
    __pyx_t_2 = __Pyx_PyInt_From_GLint(__pyx_v_self->program->VertexTexCoord2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_tex = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "renpy/angle/glenviron_shader.pyx":532
 *         elif unit == 1:
 *             tex = self.program.VertexTexCoord1
 *         elif unit == 2 and RENPY_THIRD_TEXTURE:             # <<<<<<<<<<<<<<
 *             tex = self.program.VertexTexCoord2
 *         else:
 */
    goto __pyx_L3;
  }

  /* "renpy/angle/glenviron_shader.pyx":535
 *             tex = self.program.VertexTexCoord2
 *         else:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         if tex < 0:
 */
  /*else*/ {
    goto __pyx_L0;
  }
  __pyx_L3:;

  /* "renpy/angle/glenviron_shader.pyx":537
 *             return
 * 
 *         if tex < 0:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_tex, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":538
 * 
 *         if tex < 0:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         if coords != NULL:
 */
    goto __pyx_L0;

    /* "renpy/angle/glenviron_shader.pyx":537
 *             return
 * 
 *         if tex < 0:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":540
 *             return
 * 
 *         if coords != NULL:             # <<<<<<<<<<<<<<
 *             glVertexAttribPointerARB(tex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> coords)
 *             glEnableVertexAttribArrayARB(tex)
 */
  __pyx_t_1 = ((__pyx_v_coords != NULL) != 0);
  if (__pyx_t_1) {

    /* "renpy/angle/glenviron_shader.pyx":541
 * 
 *         if coords != NULL:
 *             glVertexAttribPointerARB(tex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> coords)             # <<<<<<<<<<<<<<
 *             glEnableVertexAttribArrayARB(tex)
 *         else:
 */
    __pyx_t_4 = __Pyx_PyInt_As_GLuint(__pyx_v_tex); if (unlikely((__pyx_t_4 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 541, __pyx_L1_error)
    __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glVertexAttribPointerARB(__pyx_t_4, 2, GL_FLOAT, GL_FALSE, 0, ((GLubyte *)__pyx_v_coords)); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 541, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":542
 *         if coords != NULL:
 *             glVertexAttribPointerARB(tex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> coords)
 *             glEnableVertexAttribArrayARB(tex)             # <<<<<<<<<<<<<<
 *         else:
 *             glDisableVertexAttribArrayARB(tex)
 */
    __pyx_t_4 = __Pyx_PyInt_As_GLuint(__pyx_v_tex); if (unlikely((__pyx_t_4 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 542, __pyx_L1_error)
    __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glEnableVertexAttribArrayARB(__pyx_t_4); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 542, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":540
 *             return
 * 
 *         if coords != NULL:             # <<<<<<<<<<<<<<
 *             glVertexAttribPointerARB(tex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> coords)
 *             glEnableVertexAttribArrayARB(tex)
 */
    goto __pyx_L7;
  }

  /* "renpy/angle/glenviron_shader.pyx":544
 *             glEnableVertexAttribArrayARB(tex)
 *         else:
 *             glDisableVertexAttribArrayARB(tex)             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_color(self, double r, double g, double b, double a):
 */
  /*else*/ {
    __pyx_t_4 = __Pyx_PyInt_As_GLuint(__pyx_v_tex); if (unlikely((__pyx_t_4 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 544, __pyx_L1_error)
    __pyx_t_5 = __pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB(__pyx_t_4); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 544, __pyx_L1_error)
  }
  __pyx_L7:;

  /* "renpy/angle/glenviron_shader.pyx":525
 *         glVertexAttribPointerARB(self.program.Vertex, 2, GL_FLOAT, GL_FALSE, 0, <GLubyte *> vertices)
 * 
 *     cdef void set_texture(self, int unit, float *coords):             # <<<<<<<<<<<<<<
 *         cdef tex
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.set_texture", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tex);
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":546
 *             glDisableVertexAttribArrayARB(tex)
 * 
 *     cdef void set_color(self, double r, double g, double b, double a):             # <<<<<<<<<<<<<<
 *         glUniform4fARB(self.program.Color, r, g, b, a)
 * 
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_color(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_r, double __pyx_v_g, double __pyx_v_b, double __pyx_v_a) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("set_color", 0);

  /* "renpy/angle/glenviron_shader.pyx":547
 * 
 *     cdef void set_color(self, double r, double g, double b, double a):
 *         glUniform4fARB(self.program.Color, r, g, b, a)             # <<<<<<<<<<<<<<
 * 
 *     cdef void ortho(self, double left, double right, double bottom, double top, double near, double far):
 */
  __pyx_t_1 = __pyx_f_5renpy_5angle_2gl_glUniform4fARB(__pyx_v_self->program->Color, __pyx_v_r, __pyx_v_g, __pyx_v_b, __pyx_v_a); if (unlikely(__pyx_t_1 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 547, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":546
 *             glDisableVertexAttribArrayARB(tex)
 * 
 *     cdef void set_color(self, double r, double g, double b, double a):             # <<<<<<<<<<<<<<
 *         glUniform4fARB(self.program.Color, r, g, b, a)
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.set_color", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":549
 *         glUniform4fARB(self.program.Color, r, g, b, a)
 * 
 *     cdef void ortho(self, double left, double right, double bottom, double top, double near, double far):             # <<<<<<<<<<<<<<
 * 
 *         self.projection[ 0] = 2 / (right - left)
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_ortho(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, double __pyx_v_left, double __pyx_v_right, double __pyx_v_bottom, double __pyx_v_top, double __pyx_v_near, double __pyx_v_far) {
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  double __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("ortho", 0);

  /* "renpy/angle/glenviron_shader.pyx":551
 *     cdef void ortho(self, double left, double right, double bottom, double top, double near, double far):
 * 
 *         self.projection[ 0] = 2 / (right - left)             # <<<<<<<<<<<<<<
 *         self.projection[ 4] = 0
 *         self.projection[ 8] = 0
 */
  __pyx_t_1 = (__pyx_v_right - __pyx_v_left);
  if (unlikely(__pyx_t_1 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 551, __pyx_L1_error)
  }
  (__pyx_v_self->projection[0]) = (2.0 / __pyx_t_1);

  /* "renpy/angle/glenviron_shader.pyx":552
 * 
 *         self.projection[ 0] = 2 / (right - left)
 *         self.projection[ 4] = 0             # <<<<<<<<<<<<<<
 *         self.projection[ 8] = 0
 *         self.projection[12] = -(right + left) / (right - left)
 */
  (__pyx_v_self->projection[4]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":553
 *         self.projection[ 0] = 2 / (right - left)
 *         self.projection[ 4] = 0
 *         self.projection[ 8] = 0             # <<<<<<<<<<<<<<
 *         self.projection[12] = -(right + left) / (right - left)
 * 
 */
  (__pyx_v_self->projection[8]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":554
 *         self.projection[ 4] = 0
 *         self.projection[ 8] = 0
 *         self.projection[12] = -(right + left) / (right - left)             # <<<<<<<<<<<<<<
 * 
 *         self.projection[ 1] = 0
 */
  __pyx_t_1 = (-(__pyx_v_right + __pyx_v_left));
  __pyx_t_2 = (__pyx_v_right - __pyx_v_left);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 554, __pyx_L1_error)
  }
  (__pyx_v_self->projection[12]) = (__pyx_t_1 / __pyx_t_2);

  /* "renpy/angle/glenviron_shader.pyx":556
 *         self.projection[12] = -(right + left) / (right - left)
 * 
 *         self.projection[ 1] = 0             # <<<<<<<<<<<<<<
 *         self.projection[ 5] = 2 / (top - bottom)
 *         self.projection[ 9] = 0
 */
  (__pyx_v_self->projection[1]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":557
 * 
 *         self.projection[ 1] = 0
 *         self.projection[ 5] = 2 / (top - bottom)             # <<<<<<<<<<<<<<
 *         self.projection[ 9] = 0
 *         self.projection[13] = -(top + bottom) / (top - bottom)
 */
  __pyx_t_2 = (__pyx_v_top - __pyx_v_bottom);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 557, __pyx_L1_error)
  }
  (__pyx_v_self->projection[5]) = (2.0 / __pyx_t_2);

  /* "renpy/angle/glenviron_shader.pyx":558
 *         self.projection[ 1] = 0
 *         self.projection[ 5] = 2 / (top - bottom)
 *         self.projection[ 9] = 0             # <<<<<<<<<<<<<<
 *         self.projection[13] = -(top + bottom) / (top - bottom)
 * 
 */
  (__pyx_v_self->projection[9]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":559
 *         self.projection[ 5] = 2 / (top - bottom)
 *         self.projection[ 9] = 0
 *         self.projection[13] = -(top + bottom) / (top - bottom)             # <<<<<<<<<<<<<<
 * 
 *         self.projection[ 2] = 0
 */
  __pyx_t_2 = (-(__pyx_v_top + __pyx_v_bottom));
  __pyx_t_1 = (__pyx_v_top - __pyx_v_bottom);
  if (unlikely(__pyx_t_1 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 559, __pyx_L1_error)
  }
  (__pyx_v_self->projection[13]) = (__pyx_t_2 / __pyx_t_1);

  /* "renpy/angle/glenviron_shader.pyx":561
 *         self.projection[13] = -(top + bottom) / (top - bottom)
 * 
 *         self.projection[ 2] = 0             # <<<<<<<<<<<<<<
 *         self.projection[ 6] = 0
 *         self.projection[10] = -2 / (far - near)
 */
  (__pyx_v_self->projection[2]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":562
 * 
 *         self.projection[ 2] = 0
 *         self.projection[ 6] = 0             # <<<<<<<<<<<<<<
 *         self.projection[10] = -2 / (far - near)
 *         self.projection[14] = -(far + near) / (far - near)
 */
  (__pyx_v_self->projection[6]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":563
 *         self.projection[ 2] = 0
 *         self.projection[ 6] = 0
 *         self.projection[10] = -2 / (far - near)             # <<<<<<<<<<<<<<
 *         self.projection[14] = -(far + near) / (far - near)
 * 
 */
  __pyx_t_1 = (__pyx_v_far - __pyx_v_near);
  if (unlikely(__pyx_t_1 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 563, __pyx_L1_error)
  }
  (__pyx_v_self->projection[10]) = (-2.0 / __pyx_t_1);

  /* "renpy/angle/glenviron_shader.pyx":564
 *         self.projection[ 6] = 0
 *         self.projection[10] = -2 / (far - near)
 *         self.projection[14] = -(far + near) / (far - near)             # <<<<<<<<<<<<<<
 * 
 *         self.projection[ 3] = 0
 */
  __pyx_t_1 = (-(__pyx_v_far + __pyx_v_near));
  __pyx_t_2 = (__pyx_v_far - __pyx_v_near);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 564, __pyx_L1_error)
  }
  (__pyx_v_self->projection[14]) = (__pyx_t_1 / __pyx_t_2);

  /* "renpy/angle/glenviron_shader.pyx":566
 *         self.projection[14] = -(far + near) / (far - near)
 * 
 *         self.projection[ 3] = 0             # <<<<<<<<<<<<<<
 *         self.projection[ 7] = 0
 *         self.projection[11] = 0
 */
  (__pyx_v_self->projection[3]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":567
 * 
 *         self.projection[ 3] = 0
 *         self.projection[ 7] = 0             # <<<<<<<<<<<<<<
 *         self.projection[11] = 0
 *         self.projection[15] = 1
 */
  (__pyx_v_self->projection[7]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":568
 *         self.projection[ 3] = 0
 *         self.projection[ 7] = 0
 *         self.projection[11] = 0             # <<<<<<<<<<<<<<
 *         self.projection[15] = 1
 * 
 */
  (__pyx_v_self->projection[11]) = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":569
 *         self.projection[ 7] = 0
 *         self.projection[11] = 0
 *         self.projection[15] = 1             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not None:
 */
  (__pyx_v_self->projection[15]) = 1.0;

  /* "renpy/angle/glenviron_shader.pyx":571
 *         self.projection[15] = 1
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  __pyx_t_3 = (((PyObject *)__pyx_v_self->program) != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "renpy/angle/glenviron_shader.pyx":572
 * 
 *         if self.program is not None:
 *             self.program.disable_attribs()             # <<<<<<<<<<<<<<
 * 
 *         self.program = None
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->program), __pyx_n_s_disable_attribs); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (__pyx_t_7) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 572, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 572, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "renpy/angle/glenviron_shader.pyx":571
 *         self.projection[15] = 1
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":574
 *             self.program.disable_attribs()
 * 
 *         self.program = None             # <<<<<<<<<<<<<<
 * 
 *     cdef void project(self, x, y, z, double *rv_x, double *rv_y, double *rv_z):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->program));
  __pyx_v_self->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None);

  /* "renpy/angle/glenviron_shader.pyx":549
 *         glUniform4fARB(self.program.Color, r, g, b, a)
 * 
 *     cdef void ortho(self, double left, double right, double bottom, double top, double near, double far):             # <<<<<<<<<<<<<<
 * 
 *         self.projection[ 0] = 2 / (right - left)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.ortho", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":576
 *         self.program = None
 * 
 *     cdef void project(self, x, y, z, double *rv_x, double *rv_y, double *rv_z):             # <<<<<<<<<<<<<<
 *         """
 *         Given a point, projects it using the projection.
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_project(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_z, double *__pyx_v_rv_x, double *__pyx_v_rv_y, double *__pyx_v_rv_z) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  double __pyx_t_4;
  __Pyx_RefNannySetupContext("project", 0);

  /* "renpy/angle/glenviron_shader.pyx":581
 *         """
 * 
 *         rv_x[0] = x * self.projection[0] + y * self.projection[4] + z * self.projection[8] + self.projection[12]             # <<<<<<<<<<<<<<
 *         rv_y[0] = x * self.projection[1] + y * self.projection[5] + z * self.projection[9] + self.projection[13]
 *         rv_z[0] = x * self.projection[2] + y * self.projection[6] + z * self.projection[10] + self.projection[14]
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_self->projection[0])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_x, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_self->projection[4])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_y, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble((__pyx_v_self->projection[8])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_z, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->projection[12])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 581, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  (__pyx_v_rv_x[0]) = __pyx_t_4;

  /* "renpy/angle/glenviron_shader.pyx":582
 * 
 *         rv_x[0] = x * self.projection[0] + y * self.projection[4] + z * self.projection[8] + self.projection[12]
 *         rv_y[0] = x * self.projection[1] + y * self.projection[5] + z * self.projection[9] + self.projection[13]             # <<<<<<<<<<<<<<
 *         rv_z[0] = x * self.projection[2] + y * self.projection[6] + z * self.projection[10] + self.projection[14]
 * 
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_self->projection[1])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_x, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_self->projection[5])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_y, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble((__pyx_v_self->projection[9])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_z, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->projection[13])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 582, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  (__pyx_v_rv_y[0]) = __pyx_t_4;

  /* "renpy/angle/glenviron_shader.pyx":583
 *         rv_x[0] = x * self.projection[0] + y * self.projection[4] + z * self.projection[8] + self.projection[12]
 *         rv_y[0] = x * self.projection[1] + y * self.projection[5] + z * self.projection[9] + self.projection[13]
 *         rv_z[0] = x * self.projection[2] + y * self.projection[6] + z * self.projection[10] + self.projection[14]             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_clip(self, tuple clip_box, GLDraw draw):
 */
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_self->projection[2])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_x, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_self->projection[6])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_y, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble((__pyx_v_self->projection[10])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_z, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyFloat_FromDouble((__pyx_v_self->projection[14])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  (__pyx_v_rv_z[0]) = __pyx_t_4;

  /* "renpy/angle/glenviron_shader.pyx":576
 *         self.program = None
 * 
 *     cdef void project(self, x, y, z, double *rv_x, double *rv_y, double *rv_z):             # <<<<<<<<<<<<<<
 *         """
 *         Given a point, projects it using the projection.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.project", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":585
 *         rv_z[0] = x * self.projection[2] + y * self.projection[6] + z * self.projection[10] + self.projection[14]
 * 
 *     cdef void set_clip(self, tuple clip_box, GLDraw draw):             # <<<<<<<<<<<<<<
 * 
 *         cdef double minx, miny, maxx, maxy, z
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_clip(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, PyObject *__pyx_v_clip_box, struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *__pyx_v_draw) {
  double __pyx_v_minx;
  double __pyx_v_miny;
  double __pyx_v_maxx;
  double __pyx_v_maxy;
  double __pyx_v_z;
  int __pyx_v_cx;
  int __pyx_v_cy;
  CYTHON_UNUSED int __pyx_v_cw;
  CYTHON_UNUSED int __pyx_v_ch;
  CYTHON_UNUSED int __pyx_v_psw;
  CYTHON_UNUSED int __pyx_v_psh;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  double __pyx_t_6;
  double __pyx_t_7;
  double __pyx_t_8;
  double __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("set_clip", 0);

  /* "renpy/angle/glenviron_shader.pyx":593
 *         cdef int psw, psh
 * 
 *         if clip_box == draw.default_clip:             # <<<<<<<<<<<<<<
 *             self.unset_clip(draw)
 *             return
 */
  __pyx_t_1 = PyObject_RichCompare(__pyx_v_clip_box, __pyx_v_draw->default_clip, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "renpy/angle/glenviron_shader.pyx":594
 * 
 *         if clip_box == draw.default_clip:
 *             self.unset_clip(draw)             # <<<<<<<<<<<<<<
 *             return
 * 
 */
    ((struct __pyx_vtabstruct_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.unset_clip(((struct __pyx_obj_5renpy_5angle_6gldraw_Environ *)__pyx_v_self), __pyx_v_draw);

    /* "renpy/angle/glenviron_shader.pyx":595
 *         if clip_box == draw.default_clip:
 *             self.unset_clip(draw)
 *             return             # <<<<<<<<<<<<<<
 * 
 *         minx, miny, maxx, maxy = clip_box
 */
    goto __pyx_L0;

    /* "renpy/angle/glenviron_shader.pyx":593
 *         cdef int psw, psh
 * 
 *         if clip_box == draw.default_clip:             # <<<<<<<<<<<<<<
 *             self.unset_clip(draw)
 *             return
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":597
 *             return
 * 
 *         minx, miny, maxx, maxy = clip_box             # <<<<<<<<<<<<<<
 *         psw, psh = draw.physical_size
 * 
 */
  if (likely(__pyx_v_clip_box != Py_None)) {
    PyObject* sequence = __pyx_v_clip_box;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 597, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 3); 
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_3,&__pyx_t_4,&__pyx_t_5};
      for (i=0; i < 4; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 597, __pyx_L1_error)
  }
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_7 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_8 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_8 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_9 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_minx = __pyx_t_6;
  __pyx_v_miny = __pyx_t_7;
  __pyx_v_maxx = __pyx_t_8;
  __pyx_v_maxy = __pyx_t_9;

  /* "renpy/angle/glenviron_shader.pyx":598
 * 
 *         minx, miny, maxx, maxy = clip_box
 *         psw, psh = draw.physical_size             # <<<<<<<<<<<<<<
 * 
 *         # The clipping box.
 */
  __pyx_t_5 = __pyx_v_draw->physical_size;
  __Pyx_INCREF(__pyx_t_5);
  if (likely(__pyx_t_5 != Py_None)) {
    PyObject* sequence = __pyx_t_5;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 598, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 598, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 598, __pyx_L1_error)
  }
  __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_psw = __pyx_t_10;
  __pyx_v_psh = __pyx_t_11;

  /* "renpy/angle/glenviron_shader.pyx":601
 * 
 *         # The clipping box.
 *         self.clipping = True             # <<<<<<<<<<<<<<
 *         self.clip_x0 = minx
 *         self.clip_y0 = miny
 */
  __pyx_v_self->clipping = 1;

  /* "renpy/angle/glenviron_shader.pyx":602
 *         # The clipping box.
 *         self.clipping = True
 *         self.clip_x0 = minx             # <<<<<<<<<<<<<<
 *         self.clip_y0 = miny
 *         self.clip_x1 = maxx
 */
  __pyx_v_self->clip_x0 = __pyx_v_minx;

  /* "renpy/angle/glenviron_shader.pyx":603
 *         self.clipping = True
 *         self.clip_x0 = minx
 *         self.clip_y0 = miny             # <<<<<<<<<<<<<<
 *         self.clip_x1 = maxx
 *         self.clip_y1 = maxy
 */
  __pyx_v_self->clip_y0 = __pyx_v_miny;

  /* "renpy/angle/glenviron_shader.pyx":604
 *         self.clip_x0 = minx
 *         self.clip_y0 = miny
 *         self.clip_x1 = maxx             # <<<<<<<<<<<<<<
 *         self.clip_y1 = maxy
 * 
 */
  __pyx_v_self->clip_x1 = __pyx_v_maxx;

  /* "renpy/angle/glenviron_shader.pyx":605
 *         self.clip_y0 = miny
 *         self.clip_x1 = maxx
 *         self.clip_y1 = maxy             # <<<<<<<<<<<<<<
 * 
 *         # Set the scissor rectangle to be slightly larger than the
 */
  __pyx_v_self->clip_y1 = __pyx_v_maxy;

  /* "renpy/angle/glenviron_shader.pyx":612
 *         # fragments.
 * 
 *         if draw.clip_rtt_box is None:             # <<<<<<<<<<<<<<
 * 
 *             z = 0
 */
  __pyx_t_2 = (__pyx_v_draw->clip_rtt_box == ((PyObject*)Py_None));
  __pyx_t_12 = (__pyx_t_2 != 0);
  if (__pyx_t_12) {

    /* "renpy/angle/glenviron_shader.pyx":614
 *         if draw.clip_rtt_box is None:
 * 
 *             z = 0             # <<<<<<<<<<<<<<
 * 
 *             # Project to normalized coordinates.
 */
    __pyx_v_z = 0.0;

    /* "renpy/angle/glenviron_shader.pyx":617
 * 
 *             # Project to normalized coordinates.
 *             self.project(minx, miny, z, &minx, &miny, &z)             # <<<<<<<<<<<<<<
 *             self.project(maxx, maxy, z, &maxx, &maxy, &z)
 * 
 */
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_minx); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 617, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_miny); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 617, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_z); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 617, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    ((struct __pyx_vtabstruct_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self->__pyx_base.__pyx_vtab)->project(__pyx_v_self, __pyx_t_5, __pyx_t_3, __pyx_t_4, (&__pyx_v_minx), (&__pyx_v_miny), (&__pyx_v_z));
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "renpy/angle/glenviron_shader.pyx":618
 *             # Project to normalized coordinates.
 *             self.project(minx, miny, z, &minx, &miny, &z)
 *             self.project(maxx, maxy, z, &maxx, &maxy, &z)             # <<<<<<<<<<<<<<
 * 
 *             # Convert to window coordinates.
 */
    __pyx_t_4 = PyFloat_FromDouble(__pyx_v_maxx); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_maxy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 618, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    ((struct __pyx_vtabstruct_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self->__pyx_base.__pyx_vtab)->project(__pyx_v_self, __pyx_t_4, __pyx_t_3, __pyx_t_5, (&__pyx_v_maxx), (&__pyx_v_maxy), (&__pyx_v_z));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "renpy/angle/glenviron_shader.pyx":621
 * 
 *             # Convert to window coordinates.
 *             minx = (minx + 1) * self.viewport_w / 2 + self.viewport_x             # <<<<<<<<<<<<<<
 *             maxx = (maxx + 1) * self.viewport_w / 2 + self.viewport_x
 *             miny = (miny + 1) * self.viewport_h / 2 + self.viewport_y
 */
    __pyx_v_minx = ((((__pyx_v_minx + 1.0) * __pyx_v_self->viewport_w) / 2.0) + __pyx_v_self->viewport_x);

    /* "renpy/angle/glenviron_shader.pyx":622
 *             # Convert to window coordinates.
 *             minx = (minx + 1) * self.viewport_w / 2 + self.viewport_x
 *             maxx = (maxx + 1) * self.viewport_w / 2 + self.viewport_x             # <<<<<<<<<<<<<<
 *             miny = (miny + 1) * self.viewport_h / 2 + self.viewport_y
 *             maxy = (maxy + 1) * self.viewport_h / 2 + self.viewport_y
 */
    __pyx_v_maxx = ((((__pyx_v_maxx + 1.0) * __pyx_v_self->viewport_w) / 2.0) + __pyx_v_self->viewport_x);

    /* "renpy/angle/glenviron_shader.pyx":623
 *             minx = (minx + 1) * self.viewport_w / 2 + self.viewport_x
 *             maxx = (maxx + 1) * self.viewport_w / 2 + self.viewport_x
 *             miny = (miny + 1) * self.viewport_h / 2 + self.viewport_y             # <<<<<<<<<<<<<<
 *             maxy = (maxy + 1) * self.viewport_h / 2 + self.viewport_y
 * 
 */
    __pyx_v_miny = ((((__pyx_v_miny + 1.0) * __pyx_v_self->viewport_h) / 2.0) + __pyx_v_self->viewport_y);

    /* "renpy/angle/glenviron_shader.pyx":624
 *             maxx = (maxx + 1) * self.viewport_w / 2 + self.viewport_x
 *             miny = (miny + 1) * self.viewport_h / 2 + self.viewport_y
 *             maxy = (maxy + 1) * self.viewport_h / 2 + self.viewport_y             # <<<<<<<<<<<<<<
 * 
 *             # Increase the bounding box, to ensure every relevant pixel is
 */
    __pyx_v_maxy = ((((__pyx_v_maxy + 1.0) * __pyx_v_self->viewport_h) / 2.0) + __pyx_v_self->viewport_y);

    /* "renpy/angle/glenviron_shader.pyx":628
 *             # Increase the bounding box, to ensure every relevant pixel is
 *             # in it. The shader will take care of enforcing the actual box.
 *             minx -= 1             # <<<<<<<<<<<<<<
 *             maxx += 1
 *             miny += 1
 */
    __pyx_v_minx = (__pyx_v_minx - 1.0);

    /* "renpy/angle/glenviron_shader.pyx":629
 *             # in it. The shader will take care of enforcing the actual box.
 *             minx -= 1
 *             maxx += 1             # <<<<<<<<<<<<<<
 *             miny += 1
 *             maxy -= 1
 */
    __pyx_v_maxx = (__pyx_v_maxx + 1.0);

    /* "renpy/angle/glenviron_shader.pyx":630
 *             minx -= 1
 *             maxx += 1
 *             miny += 1             # <<<<<<<<<<<<<<
 *             maxy -= 1
 * 
 */
    __pyx_v_miny = (__pyx_v_miny + 1.0);

    /* "renpy/angle/glenviron_shader.pyx":631
 *             maxx += 1
 *             miny += 1
 *             maxy -= 1             # <<<<<<<<<<<<<<
 * 
 *             if minx < 0:
 */
    __pyx_v_maxy = (__pyx_v_maxy - 1.0);

    /* "renpy/angle/glenviron_shader.pyx":633
 *             maxy -= 1
 * 
 *             if minx < 0:             # <<<<<<<<<<<<<<
 *                 minx = 0
 *             if miny < 0:
 */
    __pyx_t_12 = ((__pyx_v_minx < 0.0) != 0);
    if (__pyx_t_12) {

      /* "renpy/angle/glenviron_shader.pyx":634
 * 
 *             if minx < 0:
 *                 minx = 0             # <<<<<<<<<<<<<<
 *             if miny < 0:
 *                 miny = 0
 */
      __pyx_v_minx = 0.0;

      /* "renpy/angle/glenviron_shader.pyx":633
 *             maxy -= 1
 * 
 *             if minx < 0:             # <<<<<<<<<<<<<<
 *                 minx = 0
 *             if miny < 0:
 */
    }

    /* "renpy/angle/glenviron_shader.pyx":635
 *             if minx < 0:
 *                 minx = 0
 *             if miny < 0:             # <<<<<<<<<<<<<<
 *                 miny = 0
 * 
 */
    __pyx_t_12 = ((__pyx_v_miny < 0.0) != 0);
    if (__pyx_t_12) {

      /* "renpy/angle/glenviron_shader.pyx":636
 *                 minx = 0
 *             if miny < 0:
 *                 miny = 0             # <<<<<<<<<<<<<<
 * 
 *             glEnable(GL_SCISSOR_TEST)
 */
      __pyx_v_miny = 0.0;

      /* "renpy/angle/glenviron_shader.pyx":635
 *             if minx < 0:
 *                 minx = 0
 *             if miny < 0:             # <<<<<<<<<<<<<<
 *                 miny = 0
 * 
 */
    }

    /* "renpy/angle/glenviron_shader.pyx":638
 *                 miny = 0
 * 
 *             glEnable(GL_SCISSOR_TEST)             # <<<<<<<<<<<<<<
 *             glScissor(<GLint> round(minx), <GLint> round(maxy), <GLint> round(maxx - minx), <GLsizei> round(miny - maxy))
 * 
 */
    __pyx_t_11 = __pyx_f_5renpy_5angle_2gl_glEnable(GL_SCISSOR_TEST); if (unlikely(__pyx_t_11 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 638, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":639
 * 
 *             glEnable(GL_SCISSOR_TEST)
 *             glScissor(<GLint> round(minx), <GLint> round(maxy), <GLint> round(maxx - minx), <GLsizei> round(miny - maxy))             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
    __pyx_t_11 = __pyx_f_5renpy_5angle_2gl_glScissor(((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round(__pyx_v_minx)), ((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round(__pyx_v_maxy)), ((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round((__pyx_v_maxx - __pyx_v_minx))), ((GLsizei)__pyx_f_5renpy_5angle_16glenviron_shader_round((__pyx_v_miny - __pyx_v_maxy)))); if (unlikely(__pyx_t_11 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 639, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":612
 *         # fragments.
 * 
 *         if draw.clip_rtt_box is None:             # <<<<<<<<<<<<<<
 * 
 *             z = 0
 */
    goto __pyx_L4;
  }

  /* "renpy/angle/glenviron_shader.pyx":643
 *         else:
 * 
 *             cx, cy, cw, ch = draw.clip_rtt_box             # <<<<<<<<<<<<<<
 * 
 *             glEnable(GL_SCISSOR_TEST)
 */
  /*else*/ {
    __pyx_t_5 = __pyx_v_draw->clip_rtt_box;
    __Pyx_INCREF(__pyx_t_5);
    if (likely(__pyx_t_5 != Py_None)) {
      PyObject* sequence = __pyx_t_5;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 643, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_13 = PyTuple_GET_ITEM(sequence, 3); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_13);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_3,&__pyx_t_4,&__pyx_t_1,&__pyx_t_13};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 643, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 643, __pyx_L1_error)
    }
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_15 = __Pyx_PyInt_As_int(__pyx_t_13); if (unlikely((__pyx_t_15 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_v_cx = __pyx_t_11;
    __pyx_v_cy = __pyx_t_10;
    __pyx_v_cw = __pyx_t_14;
    __pyx_v_ch = __pyx_t_15;

    /* "renpy/angle/glenviron_shader.pyx":645
 *             cx, cy, cw, ch = draw.clip_rtt_box
 * 
 *             glEnable(GL_SCISSOR_TEST)             # <<<<<<<<<<<<<<
 *             glScissor(<GLint> round(minx - cx), <GLint> round(miny - cy), <GLint> round(maxx - minx), <GLint> round(maxy - miny))
 * 
 */
    __pyx_t_15 = __pyx_f_5renpy_5angle_2gl_glEnable(GL_SCISSOR_TEST); if (unlikely(__pyx_t_15 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 645, __pyx_L1_error)

    /* "renpy/angle/glenviron_shader.pyx":646
 * 
 *             glEnable(GL_SCISSOR_TEST)
 *             glScissor(<GLint> round(minx - cx), <GLint> round(miny - cy), <GLint> round(maxx - minx), <GLint> round(maxy - miny))             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not None:
 */
    __pyx_t_15 = __pyx_f_5renpy_5angle_2gl_glScissor(((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round((__pyx_v_minx - __pyx_v_cx))), ((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round((__pyx_v_miny - __pyx_v_cy))), ((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round((__pyx_v_maxx - __pyx_v_minx))), ((GLint)__pyx_f_5renpy_5angle_16glenviron_shader_round((__pyx_v_maxy - __pyx_v_miny)))); if (unlikely(__pyx_t_15 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 646, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "renpy/angle/glenviron_shader.pyx":648
 *             glScissor(<GLint> round(minx - cx), <GLint> round(miny - cy), <GLint> round(maxx - minx), <GLint> round(maxy - miny))
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  __pyx_t_12 = (((PyObject *)__pyx_v_self->program) != Py_None);
  __pyx_t_2 = (__pyx_t_12 != 0);
  if (__pyx_t_2) {

    /* "renpy/angle/glenviron_shader.pyx":649
 * 
 *         if self.program is not None:
 *             self.program.disable_attribs()             # <<<<<<<<<<<<<<
 * 
 *         self.program = None
 */
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->program), __pyx_n_s_disable_attribs); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 649, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_13);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_13, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_13); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 649, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "renpy/angle/glenviron_shader.pyx":648
 *             glScissor(<GLint> round(minx - cx), <GLint> round(miny - cy), <GLint> round(maxx - minx), <GLint> round(maxy - miny))
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":651
 *             self.program.disable_attribs()
 * 
 *         self.program = None             # <<<<<<<<<<<<<<
 * 
 *     cdef void unset_clip(self, GLDraw draw):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->program));
  __pyx_v_self->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None);

  /* "renpy/angle/glenviron_shader.pyx":585
 *         rv_z[0] = x * self.projection[2] + y * self.projection[6] + z * self.projection[10] + self.projection[14]
 * 
 *     cdef void set_clip(self, tuple clip_box, GLDraw draw):             # <<<<<<<<<<<<<<
 * 
 *         cdef double minx, miny, maxx, maxy, z
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.set_clip", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":653
 *         self.program = None
 * 
 *     cdef void unset_clip(self, GLDraw draw):             # <<<<<<<<<<<<<<
 * 
 *         glDisable(GL_SCISSOR_TEST)
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_unset_clip(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, CYTHON_UNUSED struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *__pyx_v_draw) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("unset_clip", 0);

  /* "renpy/angle/glenviron_shader.pyx":655
 *     cdef void unset_clip(self, GLDraw draw):
 * 
 *         glDisable(GL_SCISSOR_TEST)             # <<<<<<<<<<<<<<
 * 
 *         self.clipping = False
 */
  __pyx_t_1 = __pyx_f_5renpy_5angle_2gl_glDisable(GL_SCISSOR_TEST); if (unlikely(__pyx_t_1 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 655, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":657
 *         glDisable(GL_SCISSOR_TEST)
 * 
 *         self.clipping = False             # <<<<<<<<<<<<<<
 *         self.clip_x0 = 0
 *         self.clip_y0 = 0
 */
  __pyx_v_self->clipping = 0;

  /* "renpy/angle/glenviron_shader.pyx":658
 * 
 *         self.clipping = False
 *         self.clip_x0 = 0             # <<<<<<<<<<<<<<
 *         self.clip_y0 = 0
 *         self.clip_x1 = 65535
 */
  __pyx_v_self->clip_x0 = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":659
 *         self.clipping = False
 *         self.clip_x0 = 0
 *         self.clip_y0 = 0             # <<<<<<<<<<<<<<
 *         self.clip_x1 = 65535
 *         self.clip_y1 = 65535
 */
  __pyx_v_self->clip_y0 = 0.0;

  /* "renpy/angle/glenviron_shader.pyx":660
 *         self.clip_x0 = 0
 *         self.clip_y0 = 0
 *         self.clip_x1 = 65535             # <<<<<<<<<<<<<<
 *         self.clip_y1 = 65535
 * 
 */
  __pyx_v_self->clip_x1 = 65535.0;

  /* "renpy/angle/glenviron_shader.pyx":661
 *         self.clip_y0 = 0
 *         self.clip_x1 = 65535
 *         self.clip_y1 = 65535             # <<<<<<<<<<<<<<
 * 
 *         if self.program is not None:
 */
  __pyx_v_self->clip_y1 = 65535.0;

  /* "renpy/angle/glenviron_shader.pyx":663
 *         self.clip_y1 = 65535
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_self->program) != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "renpy/angle/glenviron_shader.pyx":664
 * 
 *         if self.program is not None:
 *             self.program.disable_attribs()             # <<<<<<<<<<<<<<
 * 
 *         self.program = None
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->program), __pyx_n_s_disable_attribs); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (__pyx_t_6) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 664, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 664, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "renpy/angle/glenviron_shader.pyx":663
 *         self.clip_y1 = 65535
 * 
 *         if self.program is not None:             # <<<<<<<<<<<<<<
 *             self.program.disable_attribs()
 * 
 */
  }

  /* "renpy/angle/glenviron_shader.pyx":666
 *             self.program.disable_attribs()
 * 
 *         self.program = None             # <<<<<<<<<<<<<<
 * 
 *     cdef void viewport(self, int x, int y, int width, int height):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->program);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->program));
  __pyx_v_self->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None);

  /* "renpy/angle/glenviron_shader.pyx":653
 *         self.program = None
 * 
 *     cdef void unset_clip(self, GLDraw draw):             # <<<<<<<<<<<<<<
 * 
 *         glDisable(GL_SCISSOR_TEST)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.unset_clip", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/angle/glenviron_shader.pyx":668
 *         self.program = None
 * 
 *     cdef void viewport(self, int x, int y, int width, int height):             # <<<<<<<<<<<<<<
 *         glViewport(x, y, width, height)
 *         self.viewport_x = x
 */

static void __pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_viewport(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, int __pyx_v_x, int __pyx_v_y, int __pyx_v_width, int __pyx_v_height) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("viewport", 0);

  /* "renpy/angle/glenviron_shader.pyx":669
 * 
 *     cdef void viewport(self, int x, int y, int width, int height):
 *         glViewport(x, y, width, height)             # <<<<<<<<<<<<<<
 *         self.viewport_x = x
 *         self.viewport_y = y
 */
  __pyx_t_1 = __pyx_f_5renpy_5angle_2gl_glViewport(__pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height); if (unlikely(__pyx_t_1 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 669, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":670
 *     cdef void viewport(self, int x, int y, int width, int height):
 *         glViewport(x, y, width, height)
 *         self.viewport_x = x             # <<<<<<<<<<<<<<
 *         self.viewport_y = y
 *         self.viewport_w = width
 */
  __pyx_v_self->viewport_x = __pyx_v_x;

  /* "renpy/angle/glenviron_shader.pyx":671
 *         glViewport(x, y, width, height)
 *         self.viewport_x = x
 *         self.viewport_y = y             # <<<<<<<<<<<<<<
 *         self.viewport_w = width
 *         self.viewport_h = height
 */
  __pyx_v_self->viewport_y = __pyx_v_y;

  /* "renpy/angle/glenviron_shader.pyx":672
 *         self.viewport_x = x
 *         self.viewport_y = y
 *         self.viewport_w = width             # <<<<<<<<<<<<<<
 *         self.viewport_h = height
 */
  __pyx_v_self->viewport_w = __pyx_v_width;

  /* "renpy/angle/glenviron_shader.pyx":673
 *         self.viewport_y = y
 *         self.viewport_w = width
 *         self.viewport_h = height             # <<<<<<<<<<<<<<
 */
  __pyx_v_self->viewport_h = __pyx_v_height;

  /* "renpy/angle/glenviron_shader.pyx":668
 *         self.program = None
 * 
 *     cdef void viewport(self, int x, int y, int width, int height):             # <<<<<<<<<<<<<<
 *         glViewport(x, y, width, height)
 *         self.viewport_x = x
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.angle.glenviron_shader.ShaderEnviron.viewport", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.blend_clip_program, self.blend_program, self.blit_clip_program, self.blit_program, self.clip_x0, self.clip_x1, self.clip_y0, self.clip_y1, self.clipping, self.imageblend_clip_program, self.imageblend_program, self.program, self.projection, self.viewport_h, self.viewport_w, self.viewport_x, self.viewport_y)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_6__reduce_cython__(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_6__reduce_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.blend_clip_program, self.blend_program, self.blit_clip_program, self.blit_program, self.clip_x0, self.clip_x1, self.clip_y0, self.clip_y1, self.clipping, self.imageblend_clip_program, self.imageblend_program, self.program, self.projection, self.viewport_h, self.viewport_w, self.viewport_x, self.viewport_y)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->clip_x0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->clip_x1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->clip_y0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->clip_y1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyBool_FromLong(__pyx_v_self->clipping); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_carray_to_py_float(__pyx_v_self->projection, 16); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->viewport_h); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_self->viewport_w); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_self->viewport_x); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_self->viewport_y); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyTuple_New(17); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->blend_clip_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->blend_clip_program));
  PyTuple_SET_ITEM(__pyx_t_11, 0, ((PyObject *)__pyx_v_self->blend_clip_program));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->blend_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->blend_program));
  PyTuple_SET_ITEM(__pyx_t_11, 1, ((PyObject *)__pyx_v_self->blend_program));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->blit_clip_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->blit_clip_program));
  PyTuple_SET_ITEM(__pyx_t_11, 2, ((PyObject *)__pyx_v_self->blit_clip_program));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->blit_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->blit_program));
  PyTuple_SET_ITEM(__pyx_t_11, 3, ((PyObject *)__pyx_v_self->blit_program));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_11, 4, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_11, 5, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_11, 6, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_11, 7, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_11, 8, __pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->imageblend_clip_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->imageblend_clip_program));
  PyTuple_SET_ITEM(__pyx_t_11, 9, ((PyObject *)__pyx_v_self->imageblend_clip_program));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->imageblend_program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->imageblend_program));
  PyTuple_SET_ITEM(__pyx_t_11, 10, ((PyObject *)__pyx_v_self->imageblend_program));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->program));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->program));
  PyTuple_SET_ITEM(__pyx_t_11, 11, ((PyObject *)__pyx_v_self->program));
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_11, 12, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_11, 13, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_11, 14, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_11, 15, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_11, 16, __pyx_t_10);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_11);
  __pyx_t_11 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.blend_clip_program, self.blend_program, self.blit_clip_program, self.blit_program, self.clip_x0, self.clip_x1, self.clip_y0, self.clip_y1, self.clipping, self.imageblend_clip_program, self.imageblend_program, self.program, self.projection, self.viewport_h, self.viewport_w, self.viewport_x, self.viewport_y)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_11 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v__dict = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "(tree fragment)":5
 *     state = (self.blend_clip_program, self.blend_program, self.blit_clip_program, self.blit_program, self.clip_x0, self.clip_x1, self.clip_y0, self.clip_y1, self.clipping, self.imageblend_clip_program, self.imageblend_program, self.program, self.projection, self.viewport_h, self.viewport_w, self.viewport_x, self.viewport_y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_12 = (__pyx_v__dict != Py_None);
  __pyx_t_13 = (__pyx_t_12 != 0);
  if (__pyx_t_13) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_v__dict);
    __pyx_t_10 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.blend_clip_program is not None or self.blend_program is not None or self.blit_clip_program is not None or self.blit_program is not None or self.imageblend_clip_program is not None or self.imageblend_program is not None or self.program is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.blend_clip_program, self.blend_program, self.blit_clip_program, self.blit_program, self.clip_x0, self.clip_x1, self.clip_y0, self.clip_y1, self.clipping, self.imageblend_clip_program, self.imageblend_program, self.program, self.projection, self.viewport_h, self.viewport_w, self.viewport_x, self.viewport_y)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.blend_clip_program is not None or self.blend_program is not None or self.blit_clip_program is not None or self.blit_program is not None or self.imageblend_clip_program is not None or self.imageblend_program is not None or self.program is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, None), state
 */
  /*else*/ {
    __pyx_t_12 = (((PyObject *)__pyx_v_self->blend_clip_program) != Py_None);
    __pyx_t_14 = (__pyx_t_12 != 0);
    if (!__pyx_t_14) {
    } else {
      __pyx_t_13 = __pyx_t_14;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_14 = (((PyObject *)__pyx_v_self->blend_program) != Py_None);
    __pyx_t_12 = (__pyx_t_14 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (((PyObject *)__pyx_v_self->blit_clip_program) != Py_None);
    __pyx_t_14 = (__pyx_t_12 != 0);
    if (!__pyx_t_14) {
    } else {
      __pyx_t_13 = __pyx_t_14;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_14 = (((PyObject *)__pyx_v_self->blit_program) != Py_None);
    __pyx_t_12 = (__pyx_t_14 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (((PyObject *)__pyx_v_self->imageblend_clip_program) != Py_None);
    __pyx_t_14 = (__pyx_t_12 != 0);
    if (!__pyx_t_14) {
    } else {
      __pyx_t_13 = __pyx_t_14;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_14 = (((PyObject *)__pyx_v_self->imageblend_program) != Py_None);
    __pyx_t_12 = (__pyx_t_14 != 0);
    if (!__pyx_t_12) {
    } else {
      __pyx_t_13 = __pyx_t_12;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_12 = (((PyObject *)__pyx_v_self->program) != Py_None);
    __pyx_t_14 = (__pyx_t_12 != 0);
    __pyx_t_13 = __pyx_t_14;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_13;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.blend_clip_program is not None or self.blend_program is not None or self.blit_clip_program is not None or self.blit_program is not None or self.imageblend_clip_program is not None or self.imageblend_program is not None or self.program is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, None), state
 *     else:
 */
  __pyx_t_13 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_13) {

    /* "(tree fragment)":11
 *         use_setstate = self.blend_clip_program is not None or self.blend_program is not None or self.blit_clip_program is not None or self.blit_program is not None or self.imageblend_clip_program is not None or self.imageblend_program is not None or self.program is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_ShaderEnviron); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_11, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_203683115);
    __Pyx_GIVEREF(__pyx_int_203683115);
    PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_int_203683115);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_11, 2, Py_None);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_11);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_v_state);
    __pyx_t_10 = 0;
    __pyx_t_11 = 0;
    __pyx_r = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.blend_clip_program is not None or self.blend_program is not None or self.blit_clip_program is not None or self.blit_program is not None or self.imageblend_clip_program is not None or self.imageblend_program is not None or self.program is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, None), state
 *     else:
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ShaderEnviron__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_9 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_ShaderEnviron); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = PyTuple_New(3); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_11, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_203683115);
    __Pyx_GIVEREF(__pyx_int_203683115);
    PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_int_203683115);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_v_state);
    __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_11);
    __pyx_t_9 = 0;
    __pyx_t_11 = 0;
    __pyx_r = __pyx_t_10;
    __pyx_t_10 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.blend_clip_program, self.blend_program, self.blit_clip_program, self.blit_program, self.clip_x0, self.clip_x1, self.clip_y0, self.clip_y1, self.clipping, self.imageblend_clip_program, self.imageblend_program, self.program, self.projection, self.viewport_h, self.viewport_w, self.viewport_x, self.viewport_y)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.ShaderEnviron.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ShaderEnviron__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_8__setstate_cython__(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_13ShaderEnviron_8__setstate_cython__(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ShaderEnviron__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_ShaderEnviron__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_ShaderEnviron, (type(self), 0xc23f52b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ShaderEnviron__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.ShaderEnviron.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xb699be4:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7__pyx_unpickle_Program(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_5angle_16glenviron_shader_7__pyx_unpickle_Program = {"__pyx_unpickle_Program", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_7__pyx_unpickle_Program, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_7__pyx_unpickle_Program(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Program (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Program", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Program", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Program") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Program", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.__pyx_unpickle_Program", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_6__pyx_unpickle_Program(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_6__pyx_unpickle_Program(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Program", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xb699be4:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xb699be4) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xb699be4:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)
 *     __pyx_result = Program.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0xb699be4:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Program.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xb6, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xb699be4:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)
 *     __pyx_result = Program.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_Program), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)
 *     __pyx_result = Program.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = Program.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_Program__set_state(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xb699be4 = (Color, Projection, Vertex, VertexTexCoord0, VertexTexCoord1, VertexTexCoord2, clip0, clip1, done, multiplier, offset, program, tex0, tex1, tex2))" % __pyx_checksum)
 *     __pyx_result = Program.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xb699be4:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.__pyx_unpickle_Program", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_Program__set_state(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  GLint __pyx_t_2;
  GLuint __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Program__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->Color = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->Projection = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->Vertex = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->VertexTexCoord0 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->VertexTexCoord1 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->VertexTexCoord2 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clip0 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clip1 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->done = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->multiplier = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->offset = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_GLuint(__pyx_t_1); if (unlikely((__pyx_t_3 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->program = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->tex0 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->tex1 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_GLint(__pyx_t_1); if (unlikely((__pyx_t_2 == ((GLint)-1)) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->tex2 = __pyx_t_2;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 15) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":12
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[15])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[15])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.__pyx_unpickle_Program__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ShaderEnviron(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xc23f52b:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_9__pyx_unpickle_ShaderEnviron(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_5angle_16glenviron_shader_9__pyx_unpickle_ShaderEnviron = {"__pyx_unpickle_ShaderEnviron", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_9__pyx_unpickle_ShaderEnviron, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_5angle_16glenviron_shader_9__pyx_unpickle_ShaderEnviron(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ShaderEnviron (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ShaderEnviron", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ShaderEnviron", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ShaderEnviron") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ShaderEnviron", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.__pyx_unpickle_ShaderEnviron", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_5angle_16glenviron_shader_8__pyx_unpickle_ShaderEnviron(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_5angle_16glenviron_shader_8__pyx_unpickle_ShaderEnviron(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ShaderEnviron", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_ShaderEnviron(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xc23f52b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xc23f52b) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_ShaderEnviron(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xc23f52b:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)
 *     __pyx_result = ShaderEnviron.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0xc23f52b:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ShaderEnviron.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xc2, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_ShaderEnviron(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0xc23f52b:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)
 *     __pyx_result = ShaderEnviron.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5renpy_5angle_16glenviron_shader_ShaderEnviron), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)
 *     __pyx_result = ShaderEnviron.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = ShaderEnviron.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_ShaderEnviron__set_state(((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xc23f52b = (blend_clip_program, blend_program, blit_clip_program, blit_program, clip_x0, clip_x1, clip_y0, clip_y1, clipping, imageblend_clip_program, imageblend_program, program, projection, viewport_h, viewport_w, viewport_x, viewport_y))" % __pyx_checksum)
 *     __pyx_result = ShaderEnviron.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ShaderEnviron(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xc23f52b:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.__pyx_unpickle_ShaderEnviron", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5renpy_5angle_16glenviron_shader___pyx_unpickle_ShaderEnviron__set_state(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  float __pyx_t_4[16];
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ShaderEnviron__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->blend_clip_program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->blend_clip_program));
  __pyx_v___pyx_result->blend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->blend_program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->blend_program));
  __pyx_v___pyx_result->blend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->blit_clip_program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->blit_clip_program));
  __pyx_v___pyx_result->blit_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->blit_program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->blit_program));
  __pyx_v___pyx_result->blit_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clip_x0 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clip_x1 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clip_y0 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clip_y1 = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->clipping = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->imageblend_clip_program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->imageblend_clip_program));
  __pyx_v___pyx_result->imageblend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->imageblend_program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->imageblend_program));
  __pyx_v___pyx_result->imageblend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_5angle_16glenviron_shader_Program))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->program);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->program));
  __pyx_v___pyx_result->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__Pyx_carray_from_py_float(__pyx_t_1, __pyx_t_4, 16) < 0)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  memcpy(&(__pyx_v___pyx_result->projection[0]), __pyx_t_4, sizeof(__pyx_v___pyx_result->projection[0]) * (16));
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->viewport_h = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->viewport_w = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->viewport_x = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->viewport_y = __pyx_t_5;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 17) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_3 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_3 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":12
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[17])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    if (!__pyx_t_11) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[17])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_ShaderEnviron__set_state(<ShaderEnviron> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ShaderEnviron__set_state(ShaderEnviron __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.blend_clip_program = __pyx_state[0]; __pyx_result.blend_program = __pyx_state[1]; __pyx_result.blit_clip_program = __pyx_state[2]; __pyx_result.blit_program = __pyx_state[3]; __pyx_result.clip_x0 = __pyx_state[4]; __pyx_result.clip_x1 = __pyx_state[5]; __pyx_result.clip_y0 = __pyx_state[6]; __pyx_result.clip_y1 = __pyx_state[7]; __pyx_result.clipping = __pyx_state[8]; __pyx_result.imageblend_clip_program = __pyx_state[9]; __pyx_result.imageblend_program = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.projection = __pyx_state[12]; __pyx_result.viewport_h = __pyx_state[13]; __pyx_result.viewport_w = __pyx_state[14]; __pyx_result.viewport_x = __pyx_state[15]; __pyx_result.viewport_y = __pyx_state[16]
 *     if len(__pyx_state) > 17 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("renpy.angle.glenviron_shader.__pyx_unpickle_ShaderEnviron__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/display/matrix.pxd":26
 * 
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float *__pyx_v_ow, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform4", 0);

  /* "renpy/display/matrix.pxd":27
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":28
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":29
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw             # <<<<<<<<<<<<<<
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 */
  (__pyx_v_oz[0]) = ((((__pyx_v_x * __pyx_v_self->zdx) + (__pyx_v_y * __pyx_v_self->zdy)) + (__pyx_v_z * __pyx_v_self->zdz)) + (__pyx_v_w * __pyx_v_self->zdw));

  /* "renpy/display/matrix.pxd":30
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 */
  (__pyx_v_ow[0]) = ((((__pyx_v_x * __pyx_v_self->wdx) + (__pyx_v_y * __pyx_v_self->wdy)) + (__pyx_v_z * __pyx_v_self->wdz)) + (__pyx_v_w * __pyx_v_self->wdw));

  /* "renpy/display/matrix.pxd":26
 * 
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/display/matrix.pxd":32
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform3", 0);

  /* "renpy/display/matrix.pxd":33
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":34
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":35
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 */
  (__pyx_v_oz[0]) = ((((__pyx_v_x * __pyx_v_self->zdx) + (__pyx_v_y * __pyx_v_self->zdy)) + (__pyx_v_z * __pyx_v_self->zdz)) + (__pyx_v_w * __pyx_v_self->zdw));

  /* "renpy/display/matrix.pxd":32
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform2", 0);

  /* "renpy/display/matrix.pxd":38
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 * 
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":39
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_unit_aligned(Matrix self)
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "carray.to_py":112
 * 
 * @cname("__Pyx_carray_to_py_float")
 * cdef inline list __Pyx_carray_to_py_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

static CYTHON_INLINE PyObject *__Pyx_carray_to_py_float(float *__pyx_v_v, Py_ssize_t __pyx_v_length) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("__Pyx_carray_to_py_float", 0);

  /* "carray.to_py":115
 *     cdef size_t i
 *     cdef object value
 *     l = PyList_New(length)             # <<<<<<<<<<<<<<
 *     for i in range(<size_t>length):
 *         value = v[i]
 */
  __pyx_t_1 = PyList_New(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_l = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "carray.to_py":116
 *     cdef object value
 *     l = PyList_New(length)
 *     for i in range(<size_t>length):             # <<<<<<<<<<<<<<
 *         value = v[i]
 *         Py_INCREF(value)
 */
  __pyx_t_2 = ((size_t)__pyx_v_length);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "carray.to_py":117
 *     l = PyList_New(length)
 *     for i in range(<size_t>length):
 *         value = v[i]             # <<<<<<<<<<<<<<
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)
 */
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v[__pyx_v_i])); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "carray.to_py":118
 *     for i in range(<size_t>length):
 *         value = v[i]
 *         Py_INCREF(value)             # <<<<<<<<<<<<<<
 *         PyList_SET_ITEM(l, i, value)
 *     return l
 */
    Py_INCREF(__pyx_v_value);

    /* "carray.to_py":119
 *         value = v[i]
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)             # <<<<<<<<<<<<<<
 *     return l
 * 
 */
    PyList_SET_ITEM(__pyx_v_l, __pyx_v_i, __pyx_v_value);
  }

  /* "carray.to_py":120
 *         Py_INCREF(value)
 *         PyList_SET_ITEM(l, i, value)
 *     return l             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_l);
  __pyx_r = __pyx_v_l;
  goto __pyx_L0;

  /* "carray.to_py":112
 * 
 * @cname("__Pyx_carray_to_py_float")
 * cdef inline list __Pyx_carray_to_py_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("carray.to_py.__Pyx_carray_to_py_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float")
 * cdef inline tuple __Pyx_carray_to_tuple_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

static CYTHON_INLINE PyObject *__Pyx_carray_to_tuple_float(float *__pyx_v_v, Py_ssize_t __pyx_v_length) {
  size_t __pyx_v_i;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("__Pyx_carray_to_tuple_float", 0);

  /* "carray.to_py":127
 *     cdef size_t i
 *     cdef object value
 *     t = PyTuple_New(length)             # <<<<<<<<<<<<<<
 *     for i in range(<size_t>length):
 *         value = v[i]
 */
  __pyx_t_1 = PyTuple_New(__pyx_v_length); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_t = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "carray.to_py":128
 *     cdef object value
 *     t = PyTuple_New(length)
 *     for i in range(<size_t>length):             # <<<<<<<<<<<<<<
 *         value = v[i]
 *         Py_INCREF(value)
 */
  __pyx_t_2 = ((size_t)__pyx_v_length);
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "carray.to_py":129
 *     t = PyTuple_New(length)
 *     for i in range(<size_t>length):
 *         value = v[i]             # <<<<<<<<<<<<<<
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)
 */
    __pyx_t_1 = PyFloat_FromDouble((__pyx_v_v[__pyx_v_i])); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "carray.to_py":130
 *     for i in range(<size_t>length):
 *         value = v[i]
 *         Py_INCREF(value)             # <<<<<<<<<<<<<<
 *         PyTuple_SET_ITEM(t, i, value)
 *     return t
 */
    Py_INCREF(__pyx_v_value);

    /* "carray.to_py":131
 *         value = v[i]
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)             # <<<<<<<<<<<<<<
 *     return t
 */
    PyTuple_SET_ITEM(__pyx_v_t, __pyx_v_i, __pyx_v_value);
  }

  /* "carray.to_py":132
 *         Py_INCREF(value)
 *         PyTuple_SET_ITEM(t, i, value)
 *     return t             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_t);
  __pyx_r = __pyx_v_t;
  goto __pyx_L0;

  /* "carray.to_py":124
 * 
 * @cname("__Pyx_carray_to_tuple_float")
 * cdef inline tuple __Pyx_carray_to_tuple_float(base_type *v, Py_ssize_t length):             # <<<<<<<<<<<<<<
 *     cdef size_t i
 *     cdef object value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("carray.to_py.__Pyx_carray_to_tuple_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "carray.from_py":77
 * 
 * @cname("__Pyx_carray_from_py_float")
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t i = length
 *     try:
 */

static int __Pyx_carray_from_py_float(PyObject *__pyx_v_o, float *__pyx_v_v, Py_ssize_t __pyx_v_length) {
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_item = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  float __pyx_t_11;
  char const *__pyx_t_12;
  __Pyx_RefNannySetupContext("__Pyx_carray_from_py_float", 0);

  /* "carray.from_py":78
 * @cname("__Pyx_carray_from_py_float")
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:
 *     cdef Py_ssize_t i = length             # <<<<<<<<<<<<<<
 *     try:
 *         i = len(o)
 */
  __pyx_v_i = __pyx_v_length;

  /* "carray.from_py":79
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:
 *     cdef Py_ssize_t i = length
 *     try:             # <<<<<<<<<<<<<<
 *         i = len(o)
 *     except (TypeError, OverflowError):
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    /*try:*/ {

      /* "carray.from_py":80
 *     cdef Py_ssize_t i = length
 *     try:
 *         i = len(o)             # <<<<<<<<<<<<<<
 *     except (TypeError, OverflowError):
 *         pass
 */
      __pyx_t_4 = PyObject_Length(__pyx_v_o); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 80, __pyx_L3_error)
      __pyx_v_i = __pyx_t_4;

      /* "carray.from_py":79
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:
 *     cdef Py_ssize_t i = length
 *     try:             # <<<<<<<<<<<<<<
 *         i = len(o)
 *     except (TypeError, OverflowError):
 */
    }
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;

    /* "carray.from_py":81
 *     try:
 *         i = len(o)
 *     except (TypeError, OverflowError):             # <<<<<<<<<<<<<<
 *         pass
 *     if i == length:
 */
    __pyx_t_5 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_OverflowError);
    if (__pyx_t_5) {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "carray.from_py":79
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:
 *     cdef Py_ssize_t i = length
 *     try:             # <<<<<<<<<<<<<<
 *         i = len(o)
 *     except (TypeError, OverflowError):
 */
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_1);
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_ExceptionReset(__pyx_t_1, __pyx_t_2, __pyx_t_3);
    __pyx_L8_try_end:;
  }

  /* "carray.from_py":83
 *     except (TypeError, OverflowError):
 *         pass
 *     if i == length:             # <<<<<<<<<<<<<<
 *         for i, item in enumerate(o):
 *             if i >= length:
 */
  __pyx_t_6 = ((__pyx_v_i == __pyx_v_length) != 0);
  if (__pyx_t_6) {

    /* "carray.from_py":84
 *         pass
 *     if i == length:
 *         for i, item in enumerate(o):             # <<<<<<<<<<<<<<
 *             if i >= length:
 *                 break
 */
    __pyx_t_4 = 0;
    if (likely(PyList_CheckExact(__pyx_v_o)) || PyTuple_CheckExact(__pyx_v_o)) {
      __pyx_t_7 = __pyx_v_o; __Pyx_INCREF(__pyx_t_7); __pyx_t_8 = 0;
      __pyx_t_9 = NULL;
    } else {
      __pyx_t_8 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_v_o); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 84, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 84, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_9)) {
        if (likely(PyList_CheckExact(__pyx_t_7))) {
          if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_8); __Pyx_INCREF(__pyx_t_10); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(1, 84, __pyx_L1_error)
          #else
          __pyx_t_10 = PySequence_ITEM(__pyx_t_7, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 84, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
        } else {
          if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_8); __Pyx_INCREF(__pyx_t_10); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(1, 84, __pyx_L1_error)
          #else
          __pyx_t_10 = PySequence_ITEM(__pyx_t_7, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 84, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
        }
      } else {
        __pyx_t_10 = __pyx_t_9(__pyx_t_7);
        if (unlikely(!__pyx_t_10)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(1, 84, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_10);
      }
      __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_10);
      __pyx_t_10 = 0;
      __pyx_v_i = __pyx_t_4;
      __pyx_t_4 = (__pyx_t_4 + 1);

      /* "carray.from_py":85
 *     if i == length:
 *         for i, item in enumerate(o):
 *             if i >= length:             # <<<<<<<<<<<<<<
 *                 break
 *             v[i] = item
 */
      __pyx_t_6 = ((__pyx_v_i >= __pyx_v_length) != 0);
      if (__pyx_t_6) {

        /* "carray.from_py":86
 *         for i, item in enumerate(o):
 *             if i >= length:
 *                 break             # <<<<<<<<<<<<<<
 *             v[i] = item
 *         else:
 */
        goto __pyx_L11_break;

        /* "carray.from_py":85
 *     if i == length:
 *         for i, item in enumerate(o):
 *             if i >= length:             # <<<<<<<<<<<<<<
 *                 break
 *             v[i] = item
 */
      }

      /* "carray.from_py":87
 *             if i >= length:
 *                 break
 *             v[i] = item             # <<<<<<<<<<<<<<
 *         else:
 *             i += 1  # convert index to length
 */
      __pyx_t_11 = __pyx_PyFloat_AsFloat(__pyx_v_item); if (unlikely((__pyx_t_11 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 87, __pyx_L1_error)
      (__pyx_v_v[__pyx_v_i]) = __pyx_t_11;

      /* "carray.from_py":84
 *         pass
 *     if i == length:
 *         for i, item in enumerate(o):             # <<<<<<<<<<<<<<
 *             if i >= length:
 *                 break
 */
    }
    /*else*/ {

      /* "carray.from_py":89
 *             v[i] = item
 *         else:
 *             i += 1  # convert index to length             # <<<<<<<<<<<<<<
 *             if i == length:
 *                 return 0
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "carray.from_py":90
 *         else:
 *             i += 1  # convert index to length
 *             if i == length:             # <<<<<<<<<<<<<<
 *                 return 0
 * 
 */
      __pyx_t_6 = ((__pyx_v_i == __pyx_v_length) != 0);
      if (__pyx_t_6) {

        /* "carray.from_py":91
 *             i += 1  # convert index to length
 *             if i == length:
 *                 return 0             # <<<<<<<<<<<<<<
 * 
 *     PyErr_Format(
 */
        __pyx_r = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L0;

        /* "carray.from_py":90
 *         else:
 *             i += 1  # convert index to length
 *             if i == length:             # <<<<<<<<<<<<<<
 *                 return 0
 * 
 */
      }
    }

    /* "carray.from_py":84
 *         pass
 *     if i == length:
 *         for i, item in enumerate(o):             # <<<<<<<<<<<<<<
 *             if i >= length:
 *                 break
 */
    __pyx_L11_break:;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

    /* "carray.from_py":83
 *     except (TypeError, OverflowError):
 *         pass
 *     if i == length:             # <<<<<<<<<<<<<<
 *         for i, item in enumerate(o):
 *             if i >= length:
 */
  }

  /* "carray.from_py":96
 *         IndexError,
 *         ("too many values found during array assignment, expected %zd"
 *          if i >= length else             # <<<<<<<<<<<<<<
 *          "not enough values found during array assignment, expected %zd, got %zd"),
 *         length, i)
 */
  if (((__pyx_v_i >= __pyx_v_length) != 0)) {
    __pyx_t_12 = ((char const *)"too many values found during array assignment, expected %zd");
  } else {
    __pyx_t_12 = ((char const *)"not enough values found during array assignment, expected %zd, got %zd");
  }

  /* "carray.from_py":93
 *                 return 0
 * 
 *     PyErr_Format(             # <<<<<<<<<<<<<<
 *         IndexError,
 *         ("too many values found during array assignment, expected %zd"
 */
  __pyx_t_7 = PyErr_Format(__pyx_builtin_IndexError, __pyx_t_12, __pyx_v_length, __pyx_v_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "carray.from_py":77
 * 
 * @cname("__Pyx_carray_from_py_float")
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t i = length
 *     try:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("carray.from_py.__Pyx_carray_from_py_float", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_5renpy_5angle_16glenviron_shader_Program(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_5renpy_5angle_16glenviron_shader_Program(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5renpy_5angle_16glenviron_shader_Program[] = {
  {"disable_attribs", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_3disable_attribs, METH_NOARGS, 0},
  {"delete", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_5delete, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_7Program_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_5angle_16glenviron_shader_Program = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.angle.glenviron_shader.Program", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_5angle_16glenviron_shader_Program, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "\n    Encapsulates a program.\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_5angle_16glenviron_shader_Program, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_5angle_16glenviron_shader_7Program_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_5angle_16glenviron_shader_Program, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_5angle_16glenviron_shader_ShaderEnviron __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron;

static PyObject *__pyx_tp_new_5renpy_5angle_16glenviron_shader_ShaderEnviron(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *p;
  PyObject *o = __pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5renpy_5angle_6gldraw_Environ*)__pyx_vtabptr_5renpy_5angle_16glenviron_shader_ShaderEnviron;
  p->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  p->blit_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  p->blend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  p->imageblend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  p->blit_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  p->blend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  p->imageblend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5renpy_5angle_16glenviron_shader_ShaderEnviron(PyObject *o) {
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *p = (struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->program);
  Py_CLEAR(p->blit_program);
  Py_CLEAR(p->blend_program);
  Py_CLEAR(p->imageblend_program);
  Py_CLEAR(p->blit_clip_program);
  Py_CLEAR(p->blend_clip_program);
  Py_CLEAR(p->imageblend_clip_program);
  #if CYTHON_USE_TYPE_SLOTS
  if (PyType_IS_GC(Py_TYPE(o)->tp_base))
  #endif
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_5renpy_5angle_6gldraw_Environ)) __pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_5renpy_5angle_16glenviron_shader_ShaderEnviron);
}

static int __pyx_tp_traverse_5renpy_5angle_16glenviron_shader_ShaderEnviron(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *p = (struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)o;
  e = ((likely(__pyx_ptype_5renpy_5angle_6gldraw_Environ)) ? ((__pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_traverse) ? __pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5renpy_5angle_16glenviron_shader_ShaderEnviron)); if (e) return e;
  if (p->program) {
    e = (*v)(((PyObject *)p->program), a); if (e) return e;
  }
  if (p->blit_program) {
    e = (*v)(((PyObject *)p->blit_program), a); if (e) return e;
  }
  if (p->blend_program) {
    e = (*v)(((PyObject *)p->blend_program), a); if (e) return e;
  }
  if (p->imageblend_program) {
    e = (*v)(((PyObject *)p->imageblend_program), a); if (e) return e;
  }
  if (p->blit_clip_program) {
    e = (*v)(((PyObject *)p->blit_clip_program), a); if (e) return e;
  }
  if (p->blend_clip_program) {
    e = (*v)(((PyObject *)p->blend_clip_program), a); if (e) return e;
  }
  if (p->imageblend_clip_program) {
    e = (*v)(((PyObject *)p->imageblend_clip_program), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_5angle_16glenviron_shader_ShaderEnviron(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *p = (struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *)o;
  if (likely(__pyx_ptype_5renpy_5angle_6gldraw_Environ)) { if (__pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_clear) __pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5renpy_5angle_16glenviron_shader_ShaderEnviron);
  tmp = ((PyObject*)p->program);
  p->program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->blit_program);
  p->blit_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->blend_program);
  p->blend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->imageblend_program);
  p->imageblend_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->blit_clip_program);
  p->blit_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->blend_clip_program);
  p->blend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->imageblend_clip_program);
  p->imageblend_clip_program = ((struct __pyx_obj_5renpy_5angle_16glenviron_shader_Program *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_5renpy_5angle_16glenviron_shader_ShaderEnviron[] = {
  {"init", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_1init, METH_NOARGS, 0},
  {"deinit", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_3deinit, METH_NOARGS, __pyx_doc_5renpy_5angle_16glenviron_shader_13ShaderEnviron_2deinit},
  {"activate", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_5activate, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_5angle_16glenviron_shader_13ShaderEnviron_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.angle.glenviron_shader.ShaderEnviron", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_5angle_16glenviron_shader_ShaderEnviron, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    This is an environment that uses shaders.\n    ", /*tp_doc*/
  __pyx_tp_traverse_5renpy_5angle_16glenviron_shader_ShaderEnviron, /*tp_traverse*/
  __pyx_tp_clear_5renpy_5angle_16glenviron_shader_ShaderEnviron, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_5angle_16glenviron_shader_ShaderEnviron, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_5angle_16glenviron_shader_ShaderEnviron, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_glenviron_shader(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_glenviron_shader},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "glenviron_shader",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_n_s_BLEND_CLIP_SHADER, __pyx_k_BLEND_CLIP_SHADER, sizeof(__pyx_k_BLEND_CLIP_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_BLEND_SHADER, __pyx_k_BLEND_SHADER, sizeof(__pyx_k_BLEND_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_BLIT_CLIP_SHADER, __pyx_k_BLIT_CLIP_SHADER, sizeof(__pyx_k_BLIT_CLIP_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_BLIT_SHADER, __pyx_k_BLIT_SHADER, sizeof(__pyx_k_BLIT_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_GLES_PORTABILITY, __pyx_k_GLES_PORTABILITY, sizeof(__pyx_k_GLES_PORTABILITY), 0, 0, 1, 1},
  {&__pyx_n_s_IMAGEBLEND_CLIP_SHADER, __pyx_k_IMAGEBLEND_CLIP_SHADER, sizeof(__pyx_k_IMAGEBLEND_CLIP_SHADER), 0, 0, 1, 1},
  {&__pyx_n_s_IMAGEBLEND_SHADER, __pyx_k_IMAGEBLEND_SHADER, sizeof(__pyx_k_IMAGEBLEND_SHADER), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xb6, __pyx_k_Incompatible_checksums_s_vs_0xb6, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xb6), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xc2, __pyx_k_Incompatible_checksums_s_vs_0xc2, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xc2), 0, 0, 1, 0},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_OverflowError, __pyx_k_OverflowError, sizeof(__pyx_k_OverflowError), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_s_Shader_error_s, __pyx_k_Shader_error_s, sizeof(__pyx_k_Shader_error_s), 0, 0, 1, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_VERTEX_SHADER1, __pyx_k_VERTEX_SHADER1, sizeof(__pyx_k_VERTEX_SHADER1), 0, 0, 1, 1},
  {&__pyx_n_s_VERTEX_SHADER2, __pyx_k_VERTEX_SHADER2, sizeof(__pyx_k_VERTEX_SHADER2), 0, 0, 1, 1},
  {&__pyx_n_s_VERTEX_SHADER3, __pyx_k_VERTEX_SHADER3, sizeof(__pyx_k_VERTEX_SHADER3), 0, 0, 1, 1},
  {&__pyx_n_s_activate, __pyx_k_activate, sizeof(__pyx_k_activate), 0, 0, 1, 1},
  {&__pyx_n_s_check_status, __pyx_k_check_status, sizeof(__pyx_k_check_status), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_compile_program, __pyx_k_compile_program, sizeof(__pyx_k_compile_program), 0, 0, 1, 1},
  {&__pyx_n_s_compile_shader, __pyx_k_compile_shader, sizeof(__pyx_k_compile_shader), 0, 0, 1, 1},
  {&__pyx_n_s_delete, __pyx_k_delete, sizeof(__pyx_k_delete), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_disable_attribs, __pyx_k_disable_attribs, sizeof(__pyx_k_disable_attribs), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_fragment, __pyx_k_fragment, sizeof(__pyx_k_fragment), 0, 0, 1, 1},
  {&__pyx_n_s_fragment_shader, __pyx_k_fragment_shader, sizeof(__pyx_k_fragment_shader), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 0, 1, 1},
  {&__pyx_kp_s_home_tom_ab_renpy_renpy_angle_g, __pyx_k_home_tom_ab_renpy_renpy_angle_g, sizeof(__pyx_k_home_tom_ab_renpy_renpy_angle_g), 0, 0, 1, 0},
  {&__pyx_kp_s_ifdef_GL_ES_ifdef_GL_FRAGMENT_P, __pyx_k_ifdef_GL_ES_ifdef_GL_FRAGMENT_P, sizeof(__pyx_k_ifdef_GL_ES_ifdef_GL_FRAGMENT_P), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_kind, __pyx_k_kind, sizeof(__pyx_k_kind), 0, 0, 1, 1},
  {&__pyx_n_s_lensource, __pyx_k_lensource, sizeof(__pyx_k_lensource), 0, 0, 1, 1},
  {&__pyx_n_s_log, __pyx_k_log, sizeof(__pyx_k_log), 0, 0, 1, 1},
  {&__pyx_n_s_log_length, __pyx_k_log_length, sizeof(__pyx_k_log_length), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_program, __pyx_k_program, sizeof(__pyx_k_program), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Program, __pyx_k_pyx_unpickle_Program, sizeof(__pyx_k_pyx_unpickle_Program), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ShaderEnviron, __pyx_k_pyx_unpickle_ShaderEnviron, sizeof(__pyx_k_pyx_unpickle_ShaderEnviron), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_angle_glenviron_shader, __pyx_k_renpy_angle_glenviron_shader, sizeof(__pyx_k_renpy_angle_glenviron_shader), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shader, __pyx_k_shader, sizeof(__pyx_k_shader), 0, 0, 1, 1},
  {&__pyx_n_s_source, __pyx_k_source, sizeof(__pyx_k_source), 0, 0, 1, 1},
  {&__pyx_n_s_sourceptr, __pyx_k_sourceptr, sizeof(__pyx_k_sourceptr), 0, 0, 1, 1},
  {&__pyx_n_s_status, __pyx_k_status, sizeof(__pyx_k_status), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_kp_s_uniform_mat4_Projection_attribut, __pyx_k_uniform_mat4_Projection_attribut, sizeof(__pyx_k_uniform_mat4_Projection_attribut), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_mat4_Projection_attribut_2, __pyx_k_uniform_mat4_Projection_attribut_2, sizeof(__pyx_k_uniform_mat4_Projection_attribut_2), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_mat4_Projection_attribut_3, __pyx_k_uniform_mat4_Projection_attribut_3, sizeof(__pyx_k_uniform_mat4_Projection_attribut_3), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_vec4_Color_uniform_sampl, __pyx_k_uniform_vec4_Color_uniform_sampl, sizeof(__pyx_k_uniform_vec4_Color_uniform_sampl), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_vec4_Color_uniform_sampl_2, __pyx_k_uniform_vec4_Color_uniform_sampl_2, sizeof(__pyx_k_uniform_vec4_Color_uniform_sampl_2), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_vec4_Color_uniform_sampl_3, __pyx_k_uniform_vec4_Color_uniform_sampl_3, sizeof(__pyx_k_uniform_vec4_Color_uniform_sampl_3), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_vec4_Color_uniform_sampl_4, __pyx_k_uniform_vec4_Color_uniform_sampl_4, sizeof(__pyx_k_uniform_vec4_Color_uniform_sampl_4), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_vec4_Color_uniform_sampl_5, __pyx_k_uniform_vec4_Color_uniform_sampl_5, sizeof(__pyx_k_uniform_vec4_Color_uniform_sampl_5), 0, 0, 1, 0},
  {&__pyx_kp_s_uniform_vec4_Color_uniform_sampl_6, __pyx_k_uniform_vec4_Color_uniform_sampl_6, sizeof(__pyx_k_uniform_vec4_Color_uniform_sampl_6), 0, 0, 1, 0},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_vertex, __pyx_k_vertex, sizeof(__pyx_k_vertex), 0, 0, 1, 1},
  {&__pyx_n_s_vertex_shader, __pyx_k_vertex_shader, sizeof(__pyx_k_vertex_shader), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(1, 116, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 81, __pyx_L1_error)
  __pyx_builtin_OverflowError = __Pyx_GetBuiltinName(__pyx_n_s_OverflowError); if (!__pyx_builtin_OverflowError) __PYX_ERR(1, 81, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(1, 84, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(1, 94, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "renpy/angle/glenviron_shader.pyx":263
 * 
 * 
 * def check_status(shader, handle, type):             # <<<<<<<<<<<<<<
 *     """
 *     Checks the status of a shader or program. If it fails, then an
 */
  __pyx_tuple__2 = PyTuple_Pack(6, __pyx_n_s_shader, __pyx_n_s_handle, __pyx_n_s_type, __pyx_n_s_status, __pyx_n_s_log_length, __pyx_n_s_log); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);
  __pyx_codeobj__3 = (PyObject*)__Pyx_PyCode_New(3, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__2, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_angle_g, __pyx_n_s_check_status, 263, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__3)) __PYX_ERR(0, 263, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":295
 * 
 * 
 * def compile_shader(kind, source):             # <<<<<<<<<<<<<<
 *     """
 *     Allocates and compiles a shader.
 */
  __pyx_tuple__4 = PyTuple_Pack(5, __pyx_n_s_kind, __pyx_n_s_source, __pyx_n_s_sourceptr, __pyx_n_s_lensource, __pyx_n_s_handle); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_angle_g, __pyx_n_s_compile_shader, 295, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 295, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":312
 * 
 * 
 * def compile_program(vertex, fragment):             # <<<<<<<<<<<<<<
 *     """
 *     Compiles a pair of shaders into a program.
 */
  __pyx_tuple__6 = PyTuple_Pack(5, __pyx_n_s_vertex, __pyx_n_s_fragment, __pyx_n_s_vertex_shader, __pyx_n_s_fragment_shader, __pyx_n_s_program); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_angle_g, __pyx_n_s_compile_program, 312, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 312, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xb699be4:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__8 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_codeobj__9 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__8, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Program, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__9)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__10 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);
  __pyx_codeobj__11 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__10, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ShaderEnviron, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__11)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_191470564 = PyInt_FromLong(191470564L); if (unlikely(!__pyx_int_191470564)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_203683115 = PyInt_FromLong(203683115L); if (unlikely(!__pyx_int_203683115)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initglenviron_shader(void); /*proto*/
PyMODINIT_FUNC initglenviron_shader(void)
#else
PyMODINIT_FUNC PyInit_glenviron_shader(void); /*proto*/
PyMODINIT_FUNC PyInit_glenviron_shader(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_glenviron_shader(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_glenviron_shader(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("glenviron_shader", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__angle__glenviron_shader) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.angle.glenviron_shader")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.angle.glenviron_shader", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_5renpy_5angle_16glenviron_shader_Program) < 0) __PYX_ERR(0, 336, __pyx_L1_error)
  __pyx_type_5renpy_5angle_16glenviron_shader_Program.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "Program", (PyObject *)&__pyx_type_5renpy_5angle_16glenviron_shader_Program) < 0) __PYX_ERR(0, 336, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_5angle_16glenviron_shader_Program) < 0) __PYX_ERR(0, 336, __pyx_L1_error)
  __pyx_ptype_5renpy_5angle_16glenviron_shader_Program = &__pyx_type_5renpy_5angle_16glenviron_shader_Program;
  __pyx_ptype_5renpy_5angle_6gldraw_Environ = __Pyx_ImportType("renpy.angle.gldraw", "Environ", sizeof(struct __pyx_obj_5renpy_5angle_6gldraw_Environ), 1); if (unlikely(!__pyx_ptype_5renpy_5angle_6gldraw_Environ)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_5angle_6gldraw_Environ = (struct __pyx_vtabstruct_5renpy_5angle_6gldraw_Environ*)__Pyx_GetVtable(__pyx_ptype_5renpy_5angle_6gldraw_Environ->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_5angle_6gldraw_Environ)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_5angle_16glenviron_shader_ShaderEnviron = &__pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base = *__pyx_vtabptr_5renpy_5angle_6gldraw_Environ;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.blit = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_blit;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.blend = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_blend;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.imageblend = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double, int))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_imageblend;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.set_vertex = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, float *))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_vertex;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.set_texture = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, int, float *))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_texture;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.set_color = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double, double, double, double))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_color;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.set_clip = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, PyObject *, struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_set_clip;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.unset_clip = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw *))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_unset_clip;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.ortho = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, double, double, double, double, double, double))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_ortho;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.__pyx_base.viewport = (void (*)(struct __pyx_obj_5renpy_5angle_6gldraw_Environ *, int, int, int, int))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_viewport;
  __pyx_vtable_5renpy_5angle_16glenviron_shader_ShaderEnviron.project = (void (*)(struct __pyx_obj_5renpy_5angle_16glenviron_shader_ShaderEnviron *, PyObject *, PyObject *, PyObject *, double *, double *, double *))__pyx_f_5renpy_5angle_16glenviron_shader_13ShaderEnviron_project;
  __pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron.tp_base = __pyx_ptype_5renpy_5angle_6gldraw_Environ;
  if (PyType_Ready(&__pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron) < 0) __PYX_ERR(0, 402, __pyx_L1_error)
  __pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron.tp_dict, __pyx_vtabptr_5renpy_5angle_16glenviron_shader_ShaderEnviron) < 0) __PYX_ERR(0, 402, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "ShaderEnviron", (PyObject *)&__pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron) < 0) __PYX_ERR(0, 402, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron) < 0) __PYX_ERR(0, 402, __pyx_L1_error)
  __pyx_ptype_5renpy_5angle_16glenviron_shader_ShaderEnviron = &__pyx_type_5renpy_5angle_16glenviron_shader_ShaderEnviron;
  /*--- Type import code ---*/
  __pyx_ptype_5renpy_7display_6matrix_Matrix = __Pyx_ImportType("renpy.display.matrix", "Matrix", sizeof(struct __pyx_obj_5renpy_7display_6matrix_Matrix), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6matrix_Matrix)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6matrix_Matrix = (struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6matrix_Matrix->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6matrix_Matrix)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_ptype_5renpy_7display_6matrix_Matrix2D = __Pyx_ImportType("renpy.display.matrix", "Matrix2D", sizeof(struct __pyx_obj_5renpy_7display_6matrix_Matrix2D), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6matrix_Matrix2D)) __PYX_ERR(2, 43, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6matrix_Matrix2D = (struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6matrix_Matrix2D->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6matrix_Matrix2D)) __PYX_ERR(2, 43, __pyx_L1_error)
  __pyx_ptype_5renpy_7display_6render_Render = __Pyx_ImportType("renpy.display.render", "Render", sizeof(struct __pyx_obj_5renpy_7display_6render_Render), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6render_Render)) __PYX_ERR(3, 25, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6render_Render = (struct __pyx_vtabstruct_5renpy_7display_6render_Render*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6render_Render->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6render_Render)) __PYX_ERR(3, 25, __pyx_L1_error)
  __pyx_ptype_5renpy_5angle_6gldraw_GLDraw = __Pyx_ImportType("renpy.angle.gldraw", "GLDraw", sizeof(struct __pyx_obj_5renpy_5angle_6gldraw_GLDraw), 1); if (unlikely(!__pyx_ptype_5renpy_5angle_6gldraw_GLDraw)) __PYX_ERR(4, 30, __pyx_L1_error)
  __pyx_vtabptr_5renpy_5angle_6gldraw_GLDraw = (struct __pyx_vtabstruct_5renpy_5angle_6gldraw_GLDraw*)__Pyx_GetVtable(__pyx_ptype_5renpy_5angle_6gldraw_GLDraw->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_5angle_6gldraw_GLDraw)) __PYX_ERR(4, 30, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("renpy.angle.gl"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glAttachObjectARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glAttachObjectARB, "int (GLhandleARB, GLhandleARB)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glCompileShaderARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glCompileShaderARB, "int (GLhandleARB)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glCreateProgramObjectARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glCreateProgramObjectARB, "GLhandleARB (void)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glCreateShaderObjectARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glCreateShaderObjectARB, "GLhandleARB (GLenum)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glDeleteProgram", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glDeleteProgram, "int (GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glDeleteShader", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glDeleteShader, "int (GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glDisable", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glDisable, "int (GLenum)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glDisableVertexAttribArrayARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glDisableVertexAttribArrayARB, "int (GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glEnable", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glEnable, "int (GLenum)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glEnableVertexAttribArrayARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glEnableVertexAttribArrayARB, "int (GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetAttribLocationARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glGetAttribLocationARB, "GLint (GLhandleARB, GLchar *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetProgramInfoLog", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glGetProgramInfoLog, "int (GLhandleARB, GLsizei, GLsizei *, GLchar *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetProgramiv", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glGetProgramiv, "int (GLuint, GLenum, GLint *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetShaderInfoLog", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glGetShaderInfoLog, "int (GLhandleARB, GLsizei, GLsizei *, GLchar *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetShaderiv", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glGetShaderiv, "int (GLuint, GLenum, GLint *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetUniformLocationARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glGetUniformLocationARB, "GLint (GLhandleARB, GLchar *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glLinkProgramARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glLinkProgramARB, "int (GLhandleARB)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glScissor", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glScissor, "int (GLint, GLint, GLsizei, GLsizei)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glShaderSourceARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glShaderSourceARB, "int (GLhandleARB, GLsizei, GLchar **, GLint *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glUniform1fARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glUniform1fARB, "int (GLint, GLfloat)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glUniform1iARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glUniform1iARB, "int (GLint, GLint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glUniform2fARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glUniform2fARB, "int (GLint, GLfloat, GLfloat)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glUniform4fARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glUniform4fARB, "int (GLint, GLfloat, GLfloat, GLfloat, GLfloat)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glUniformMatrix4fvARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glUniformMatrix4fvARB, "int (GLint, GLsizei, GLboolean, GLfloat *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glUseProgramObjectARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glUseProgramObjectARB, "int (GLhandleARB)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glVertexAttribPointerARB", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glVertexAttribPointerARB, "int (GLuint, GLint, GLenum, GLboolean, GLsizei, GLubyte *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glViewport", (void (**)(void))&__pyx_f_5renpy_5angle_2gl_glViewport, "int (GLint, GLint, GLsizei, GLsizei)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "renpy/angle/glenviron_shader.pyx":37
 * 
 * 
 * GLES_PORTABILITY = """\             # <<<<<<<<<<<<<<
 * #ifdef GL_ES
 *   #ifdef GL_FRAGMENT_PRECISION_HIGH
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GLES_PORTABILITY, __pyx_kp_s_ifdef_GL_ES_ifdef_GL_FRAGMENT_P) < 0) __PYX_ERR(0, 37, __pyx_L1_error)

  /* "renpy/angle/glenviron_shader.pyx":47
 * """
 * 
 * VERTEX_SHADER1 = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform mat4 Projection;
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_kp_s_uniform_mat4_Projection_attribut); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VERTEX_SHADER1, __pyx_t_3) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":65
 * """
 * 
 * VERTEX_SHADER2 = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform mat4 Projection;
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_kp_s_uniform_mat4_Projection_attribut_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VERTEX_SHADER2, __pyx_t_2) < 0) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/angle/glenviron_shader.pyx":86
 * """
 * 
 * VERTEX_SHADER3 = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform mat4 Projection;
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_kp_s_uniform_mat4_Projection_attribut_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_VERTEX_SHADER3, __pyx_t_3) < 0) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":113
 * 
 * 
 * BLIT_SHADER = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform vec4 Color;
 * uniform sampler2D tex0;
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_kp_s_uniform_vec4_Color_uniform_sampl); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLIT_SHADER, __pyx_t_2) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/angle/glenviron_shader.pyx":130
 * """
 * 
 * BLIT_CLIP_SHADER = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform vec4 Color;
 * uniform sampler2D tex0;
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_kp_s_uniform_vec4_Color_uniform_sampl_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLIT_CLIP_SHADER, __pyx_t_3) < 0) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":151
 * """
 * 
 * BLEND_SHADER = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform vec4 Color;
 * uniform sampler2D tex0;
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_kp_s_uniform_vec4_Color_uniform_sampl_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_SHADER, __pyx_t_2) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/angle/glenviron_shader.pyx":174
 * 
 * 
 * BLEND_CLIP_SHADER = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform vec4 Color;
 * uniform sampler2D tex0;
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_kp_s_uniform_vec4_Color_uniform_sampl_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BLEND_CLIP_SHADER, __pyx_t_3) < 0) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":200
 * """
 * 
 * IMAGEBLEND_SHADER = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform vec4 Color;
 * uniform sampler2D tex0;
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_kp_s_uniform_vec4_Color_uniform_sampl_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IMAGEBLEND_SHADER, __pyx_t_2) < 0) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/angle/glenviron_shader.pyx":228
 * """
 * 
 * IMAGEBLEND_CLIP_SHADER = GLES_PORTABILITY + """\             # <<<<<<<<<<<<<<
 * uniform vec4 Color;
 * uniform sampler2D tex0;
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_GLES_PORTABILITY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_kp_s_uniform_vec4_Color_uniform_sampl_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IMAGEBLEND_CLIP_SHADER, __pyx_t_3) < 0) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":263
 * 
 * 
 * def check_status(shader, handle, type):             # <<<<<<<<<<<<<<
 *     """
 *     Checks the status of a shader or program. If it fails, then an
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_5angle_16glenviron_shader_1check_status, NULL, __pyx_n_s_renpy_angle_glenviron_shader); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_check_status, __pyx_t_3) < 0) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":295
 * 
 * 
 * def compile_shader(kind, source):             # <<<<<<<<<<<<<<
 *     """
 *     Allocates and compiles a shader.
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_5angle_16glenviron_shader_3compile_shader, NULL, __pyx_n_s_renpy_angle_glenviron_shader); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compile_shader, __pyx_t_3) < 0) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":312
 * 
 * 
 * def compile_program(vertex, fragment):             # <<<<<<<<<<<<<<
 *     """
 *     Compiles a pair of shaders into a program.
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_5angle_16glenviron_shader_5compile_program, NULL, __pyx_n_s_renpy_angle_glenviron_shader); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compile_program, __pyx_t_3) < 0) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Program(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0xb699be4:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_5angle_16glenviron_shader_7__pyx_unpickle_Program, NULL, __pyx_n_s_renpy_angle_glenviron_shader); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Program, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "(tree fragment)":9
 *         __pyx_unpickle_Program__set_state(<Program> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Program__set_state(Program __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.Color = __pyx_state[0]; __pyx_result.Projection = __pyx_state[1]; __pyx_result.Vertex = __pyx_state[2]; __pyx_result.VertexTexCoord0 = __pyx_state[3]; __pyx_result.VertexTexCoord1 = __pyx_state[4]; __pyx_result.VertexTexCoord2 = __pyx_state[5]; __pyx_result.clip0 = __pyx_state[6]; __pyx_result.clip1 = __pyx_state[7]; __pyx_result.done = __pyx_state[8]; __pyx_result.multiplier = __pyx_state[9]; __pyx_result.offset = __pyx_state[10]; __pyx_result.program = __pyx_state[11]; __pyx_result.tex0 = __pyx_state[12]; __pyx_result.tex1 = __pyx_state[13]; __pyx_result.tex2 = __pyx_state[14]
 *     if len(__pyx_state) > 15 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_3 = PyCFunction_NewEx(&__pyx_mdef_5renpy_5angle_16glenviron_shader_9__pyx_unpickle_ShaderEnviron, NULL, __pyx_n_s_renpy_angle_glenviron_shader); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ShaderEnviron, __pyx_t_3) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/angle/glenviron_shader.pyx":1
 * # This file was automatically generated from renpy/gl/glenviron_shader.pyx             # <<<<<<<<<<<<<<
 * # Modifications will be automatically overwritten.
 * 
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_3) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "carray.from_py":77
 * 
 * @cname("__Pyx_carray_from_py_float")
 * cdef int __Pyx_carray_from_py_float(object o, base_type *v, Py_ssize_t length) except -1:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t i = length
 *     try:
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.angle.glenviron_shader", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.angle.glenviron_shader");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyErrExceptionMatches */
  #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
  static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
  static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ArgTypeTest */
    static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* WriteUnraisableException */
    static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseTooManyValuesToUnpack */
    static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
    static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
    static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* GetItemInt */
    static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* HasAttr */
    static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* ExtTypeTest */
    static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* SaveResetException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* CallNextTpDealloc */
    static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
    static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
    static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* SetupReduce */
    static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* GetVTable */
    static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* SetVTable */
    static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLint(GLint value) {
    const GLint neg_one = (GLint) -1, const_zero = (GLint) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLint) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLint) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLint) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLint) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLint) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLint),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLuint(GLuint value) {
    const GLuint neg_one = (GLuint) -1, const_zero = (GLuint) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLuint) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLuint) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLuint) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLuint) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLuint) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLuint),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLhandleARB(GLhandleARB value) {
    const GLhandleARB neg_one = (GLhandleARB) -1, const_zero = (GLhandleARB) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLhandleARB) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLhandleARB) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLhandleARB) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLhandleARB) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLhandleARB) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLhandleARB),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
    static CYTHON_INLINE GLint __Pyx_PyInt_As_GLint(PyObject *x) {
    const GLint neg_one = (GLint) -1, const_zero = (GLint) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLint) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLint, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLint) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLint) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLint, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) >= 2 * PyLong_SHIFT) {
                            return (GLint) (((((GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) >= 3 * PyLong_SHIFT) {
                            return (GLint) (((((((GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) >= 4 * PyLong_SHIFT) {
                            return (GLint) (((((((((GLint)digits[3]) << PyLong_SHIFT) | (GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLint) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLint) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLint) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLint) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLint, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLint,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLint) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLint) (((GLint)-1)*(((((GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLint) ((((((GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLint) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLint) (((GLint)-1)*(((((((GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLint) ((((((((GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLint) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLint) (((GLint)-1)*(((((((((GLint)digits[3]) << PyLong_SHIFT) | (GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLint) ((((((((((GLint)digits[3]) << PyLong_SHIFT) | (GLint)digits[2]) << PyLong_SHIFT) | (GLint)digits[1]) << PyLong_SHIFT) | (GLint)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLint) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLint) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLint, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLint val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLint) -1;
        }
    } else {
        GLint val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLint) -1;
        val = __Pyx_PyInt_As_GLint(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLint");
    return (GLint) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLint");
    return (GLint) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE GLuint __Pyx_PyInt_As_GLuint(PyObject *x) {
    const GLuint neg_one = (GLuint) -1, const_zero = (GLuint) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLuint) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLuint, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLuint) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLuint) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLuint, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLuint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 2 * PyLong_SHIFT) {
                            return (GLuint) (((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLuint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 3 * PyLong_SHIFT) {
                            return (GLuint) (((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLuint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 4 * PyLong_SHIFT) {
                            return (GLuint) (((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLuint) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLuint) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLuint) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLuint) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLuint, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLuint,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLuint) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLuint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLuint) ((((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLuint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLuint) ((((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLuint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLuint) ((((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLuint) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLuint) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLuint val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLuint) -1;
        }
    } else {
        GLuint val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLuint) -1;
        val = __Pyx_PyInt_As_GLuint(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLuint");
    return (GLuint) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLuint");
    return (GLuint) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE GLenum __Pyx_PyInt_As_GLenum(PyObject *x) {
    const GLenum neg_one = (GLenum) -1, const_zero = (GLenum) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLenum) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLenum, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLenum) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLenum) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLenum, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLenum) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) >= 2 * PyLong_SHIFT) {
                            return (GLenum) (((((GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLenum) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) >= 3 * PyLong_SHIFT) {
                            return (GLenum) (((((((GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLenum) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) >= 4 * PyLong_SHIFT) {
                            return (GLenum) (((((((((GLenum)digits[3]) << PyLong_SHIFT) | (GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLenum) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLenum) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLenum) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLenum) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLenum, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLenum,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLenum) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 2 * PyLong_SHIFT) {
                            return (GLenum) (((GLenum)-1)*(((((GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLenum) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 2 * PyLong_SHIFT) {
                            return (GLenum) ((((((GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLenum) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 3 * PyLong_SHIFT) {
                            return (GLenum) (((GLenum)-1)*(((((((GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLenum) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 3 * PyLong_SHIFT) {
                            return (GLenum) ((((((((GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLenum) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 4 * PyLong_SHIFT) {
                            return (GLenum) (((GLenum)-1)*(((((((((GLenum)digits[3]) << PyLong_SHIFT) | (GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLenum) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLenum, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLenum) - 1 > 4 * PyLong_SHIFT) {
                            return (GLenum) ((((((((((GLenum)digits[3]) << PyLong_SHIFT) | (GLenum)digits[2]) << PyLong_SHIFT) | (GLenum)digits[1]) << PyLong_SHIFT) | (GLenum)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLenum) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLenum) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLenum, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLenum val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLenum) -1;
        }
    } else {
        GLenum val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLenum) -1;
        val = __Pyx_PyInt_As_GLenum(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLenum");
    return (GLenum) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLenum");
    return (GLenum) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE GLhandleARB __Pyx_PyInt_As_GLhandleARB(PyObject *x) {
    const GLhandleARB neg_one = (GLhandleARB) -1, const_zero = (GLhandleARB) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLhandleARB) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLhandleARB, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLhandleARB) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLhandleARB) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLhandleARB, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLhandleARB) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) >= 2 * PyLong_SHIFT) {
                            return (GLhandleARB) (((((GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLhandleARB) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) >= 3 * PyLong_SHIFT) {
                            return (GLhandleARB) (((((((GLhandleARB)digits[2]) << PyLong_SHIFT) | (GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLhandleARB) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) >= 4 * PyLong_SHIFT) {
                            return (GLhandleARB) (((((((((GLhandleARB)digits[3]) << PyLong_SHIFT) | (GLhandleARB)digits[2]) << PyLong_SHIFT) | (GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLhandleARB) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLhandleARB) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLhandleARB, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLhandleARB) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLhandleARB, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLhandleARB) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLhandleARB, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLhandleARB,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLhandleARB) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) - 1 > 2 * PyLong_SHIFT) {
                            return (GLhandleARB) (((GLhandleARB)-1)*(((((GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLhandleARB) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) - 1 > 2 * PyLong_SHIFT) {
                            return (GLhandleARB) ((((((GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLhandleARB) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) - 1 > 3 * PyLong_SHIFT) {
                            return (GLhandleARB) (((GLhandleARB)-1)*(((((((GLhandleARB)digits[2]) << PyLong_SHIFT) | (GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLhandleARB) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) - 1 > 3 * PyLong_SHIFT) {
                            return (GLhandleARB) ((((((((GLhandleARB)digits[2]) << PyLong_SHIFT) | (GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLhandleARB) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) - 1 > 4 * PyLong_SHIFT) {
                            return (GLhandleARB) (((GLhandleARB)-1)*(((((((((GLhandleARB)digits[3]) << PyLong_SHIFT) | (GLhandleARB)digits[2]) << PyLong_SHIFT) | (GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLhandleARB) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLhandleARB, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLhandleARB) - 1 > 4 * PyLong_SHIFT) {
                            return (GLhandleARB) ((((((((((GLhandleARB)digits[3]) << PyLong_SHIFT) | (GLhandleARB)digits[2]) << PyLong_SHIFT) | (GLhandleARB)digits[1]) << PyLong_SHIFT) | (GLhandleARB)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLhandleARB) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLhandleARB, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLhandleARB) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLhandleARB, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLhandleARB val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLhandleARB) -1;
        }
    } else {
        GLhandleARB val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLhandleARB) -1;
        val = __Pyx_PyInt_As_GLhandleARB(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLhandleARB");
    return (GLhandleARB) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLhandleARB");
    return (GLhandleARB) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
    #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
    #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* FunctionImport */
    #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
