/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__gl2__gl2geometry
#define __PYX_HAVE_API__renpy__gl2__gl2geometry
#include <string.h>
#include <stdlib.h>
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "gl2geometry.pyx",
  "stringsource",
  "gl2geometry.pxd",
  "matrix.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5renpy_7display_6matrix_Matrix;
struct __pyx_obj_5renpy_7display_6matrix_Matrix2D;
struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon;
struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh;

/* "renpy/display/matrix.pxd":1
 * cdef class Matrix:             # <<<<<<<<<<<<<<
 * 
 *     cdef float m[0]
 */
struct __pyx_obj_5renpy_7display_6matrix_Matrix {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix *__pyx_vtab;
  float m[0];
  float xdx;
  float xdy;
  float xdz;
  float xdw;
  float ydx;
  float ydy;
  float ydz;
  float ydw;
  float zdx;
  float zdy;
  float zdz;
  float zdw;
  float wdx;
  float wdy;
  float wdz;
  float wdw;
};


/* "renpy/display/matrix.pxd":43
 *     cpdef bint is_unit_aligned(Matrix self)
 * 
 * cdef class Matrix2D(Matrix):             # <<<<<<<<<<<<<<
 *     pass
 */
struct __pyx_obj_5renpy_7display_6matrix_Matrix2D {
  struct __pyx_obj_5renpy_7display_6matrix_Matrix __pyx_base;
};


/* "renpy/gl2/gl2geometry.pxd":3
 * from renpy.display.matrix cimport Matrix
 * 
 * cdef class Polygon:             # <<<<<<<<<<<<<<
 * 
 *     # The number of points in the polygon.
 */
struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *__pyx_vtab;
  int points;
  int stride;
  float *data;
};


/* "renpy/gl2/gl2geometry.pxd":32
 * cpdef Polygon texture_rectangle(double pl, double pt, double pr, double pb, double tl, double tt, double tr, double tb)
 * 
 * cdef class Mesh:             # <<<<<<<<<<<<<<
 * 
 *     # The total number of points.
 */
struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *__pyx_vtab;
  int points;
  int stride;
  PyObject *attributes;
  PyObject *polygons;
  float *data;
  int polygon_count;
  int polygon_points;
};



/* "renpy/display/matrix.pxd":1
 * cdef class Matrix:             # <<<<<<<<<<<<<<
 * 
 *     cdef float m[0]
 */

struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix {
  void (*transform4)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float *, float, float, float, float);
  void (*transform3)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float, float, float, float);
  void (*transform2)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float, float, float, float);
  int (*is_unit_aligned)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix *__pyx_vtabptr_5renpy_7display_6matrix_Matrix;
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float *, float, float, float, float);
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float, float, float, float);
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float, float, float, float);


/* "renpy/display/matrix.pxd":43
 *     cpdef bint is_unit_aligned(Matrix self)
 * 
 * cdef class Matrix2D(Matrix):             # <<<<<<<<<<<<<<
 *     pass
 */

struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D {
  struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D *__pyx_vtabptr_5renpy_7display_6matrix_Matrix2D;


/* "renpy/gl2/gl2geometry.pyx":19
 * 
 * 
 * cdef class Polygon:             # <<<<<<<<<<<<<<
 * 
 *     def __init__(Polygon self, int stride, int points, data):
 */

struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*copy)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch);
  void (*offset_inplace)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, double, double, double, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*offset)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, double, double, double, int __pyx_skip_dispatch);
  void (*multiply_matrix_inplace)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*multiply_matrix)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
  void (*perspective_divide_inplace)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*perspective_divide)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*intersect)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *__pyx_vtabptr_5renpy_3gl2_11gl2geometry_Polygon;


/* "renpy/gl2/gl2geometry.pyx":543
 * 
 * 
 * cdef class Mesh:             # <<<<<<<<<<<<<<
 * 
 *     def __init__(Mesh self):
 */

struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh {
  float *(*get_data)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*copy)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch);
  void (*offset_inplace)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, double, double, double, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*offset)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, double, double, double, int __pyx_skip_dispatch);
  void (*multiply_matrix_inplace)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*multiply_matrix)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
  void (*perspective_divide_inplace)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*perspective_divide)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*intersect)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch);
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*crop)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *__pyx_vtabptr_5renpy_3gl2_11gl2geometry_Mesh;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* None.proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static float *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_get_data(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_v_offset); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_crop(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_op, int __pyx_skip_dispatch); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float *__pyx_v_ow, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/

/* Module declarations from 'renpy.display.matrix' */
static PyTypeObject *__pyx_ptype_5renpy_7display_6matrix_Matrix = 0;
static PyTypeObject *__pyx_ptype_5renpy_7display_6matrix_Matrix2D = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'libc.math' */

/* Module declarations from 'renpy.gl2.gl2geometry' */
static PyTypeObject *__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon = 0;
static PyTypeObject *__pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh = 0;
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_rectangle(double, double, double, double, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_texture_rectangle(double, double, double, double, double, double, double, double, int __pyx_skip_dispatch); /*proto*/
static CYTHON_INLINE float __pyx_f_5renpy_3gl2_11gl2geometry_get(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int, int); /*proto*/
static CYTHON_INLINE float *__pyx_f_5renpy_3gl2_11gl2geometry_ref(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int, int); /*proto*/
static CYTHON_INLINE float __pyx_f_5renpy_3gl2_11gl2geometry_set(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int, int, float); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_intersectLines(float, float, float, float, float, float, float, float, float *, float *); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_intersectOnce(float, float, float, float, float, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_restride_polygon(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int); /*proto*/
static PyObject *__pyx_f_5renpy_3gl2_11gl2geometry_intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5renpy_3gl2_11gl2geometry_barycentric(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int, int __pyx_skip_dispatch); /*proto*/
#define __Pyx_MODULE_NAME "renpy.gl2.gl2geometry"
extern int __pyx_module_is_main_renpy__gl2__gl2geometry;
int __pyx_module_is_main_renpy__gl2__gl2geometry = 0;

/* Implementation of 'renpy.gl2.gl2geometry' */
static PyObject *__pyx_builtin_NotImplemented;
static PyObject *__pyx_builtin_TypeError;
static const char __pyx_k_[] = "\n  [";
static const char __pyx_k_a[] = "a";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_g[] = "{:g}, ";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_p[] = "p";
static const char __pyx_k_r[] = "r";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_w[] = "w";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_z[] = "z";
static const char __pyx_k__2[] = "]";
static const char __pyx_k__3[] = ">";
static const char __pyx_k_pb[] = "pb";
static const char __pyx_k_pl[] = "pl";
static const char __pyx_k_pr[] = "pr";
static const char __pyx_k_pt[] = "pt";
static const char __pyx_k_tb[] = "tb";
static const char __pyx_k_tl[] = "tl";
static const char __pyx_k_tr[] = "tr";
static const char __pyx_k_tt[] = "tt";
static const char __pyx_k__10[] = ", ";
static const char __pyx_k_Mesh[] = "<Mesh ";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_crop[] = "crop";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_points[] = "points";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_stride[] = "stride";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_Polygon[] = "<Polygon";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_aPosition[] = "aPosition";
static const char __pyx_k_intersect[] = "intersect";
static const char __pyx_k_iteritems[] = "iteritems";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_new_stride[] = "new_stride";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_NotImplemented[] = "NotImplemented";
static const char __pyx_k_offset_inplace[] = "offset_inplace";
static const char __pyx_k_multiply_matrix[] = "multiply_matrix";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_barycentric_point[] = "barycentric_point";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_perspective_divide[] = "perspective_divide";
static const char __pyx_k_renpy_gl2_gl2geometry[] = "renpy.gl2.gl2geometry";
static const char __pyx_k_multiply_matrix_inplace[] = "multiply_matrix_inplace";
static const char __pyx_k_perspective_divide_inplace[] = "perspective_divide_inplace";
static const char __pyx_k_home_tom_ab_renpy_renpy_gl2_gl2[] = "/home/tom/ab/renpy/renpy/gl2/gl2geometry.pyx";
static const char __pyx_k_self_data_cannot_be_converted_to[] = "self.data cannot be converted to a Python object for pickling";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_Mesh;
static PyObject *__pyx_n_s_NotImplemented;
static PyObject *__pyx_kp_s_Polygon;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s__10;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_aPosition;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_barycentric_point;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_crop;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_kp_s_g;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_kp_s_home_tom_ab_renpy_renpy_gl2_gl2;
static PyObject *__pyx_n_s_intersect;
static PyObject *__pyx_n_s_iteritems;
static PyObject *__pyx_n_s_l;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_multiply_matrix;
static PyObject *__pyx_n_s_multiply_matrix_inplace;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new_stride;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_offset_inplace;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_pb;
static PyObject *__pyx_n_s_perspective_divide;
static PyObject *__pyx_n_s_perspective_divide_inplace;
static PyObject *__pyx_n_s_pl;
static PyObject *__pyx_n_s_points;
static PyObject *__pyx_n_s_pr;
static PyObject *__pyx_n_s_pt;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_renpy_gl2_gl2geometry;
static PyObject *__pyx_kp_s_self_data_cannot_be_converted_to;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_stride;
static PyObject *__pyx_n_s_t;
static PyObject *__pyx_n_s_tb;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_tl;
static PyObject *__pyx_n_s_tr;
static PyObject *__pyx_n_s_tt;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_z;
static int __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon___init__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_v_stride, int __pyx_v_points, PyObject *__pyx_v_data); /* proto */
static void __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_2__dealloc__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_4copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_6offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_8offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_10perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_12perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_14multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_16multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_18__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_20__rmul__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_22intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_24__repr__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_26__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_28__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_rectangle(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_l, double __pyx_v_t, double __pyx_v_r, double __pyx_v_b); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_2texture_rectangle(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_pl, double __pyx_v_pt, double __pyx_v_pr, double __pyx_v_pb, double __pyx_v_tl, double __pyx_v_tt, double __pyx_v_tr, double __pyx_v_tb); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4intersect(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b, int __pyx_v_new_stride, int __pyx_v_copy); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_6barycentric(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b, int __pyx_v_offset); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_8barycentric_point(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, float __pyx_v_x, float __pyx_v_y); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh___init__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static void __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_2__dealloc__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_4add_attribute(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6add_polygon(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8add_rectangle(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_w, double __pyx_v_h); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10add_texture_rectangle(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_pl, double __pyx_v_pt, double __pyx_v_pr, double __pyx_v_pb, double __pyx_v_tl, double __pyx_v_tt, double __pyx_v_tr, double __pyx_v_tb); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_12copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_16offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_18perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_20perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_22multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_24multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_26__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_28__rmul__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_30intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_32crop(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_op); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_34__repr__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6points___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6points_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6stride___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6stride_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_4__del__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_4__del__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_36__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_38__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_5renpy_3gl2_11gl2geometry_Polygon(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_3gl2_11gl2geometry_Mesh(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_00001;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_6;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_codeobj__14;

/* "renpy/gl2/gl2geometry.pyx":21
 * cdef class Polygon:
 * 
 *     def __init__(Polygon self, int stride, int points, data):             # <<<<<<<<<<<<<<
 *         """
 *         Allocates a new Polygon.
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon___init__[] = "\n        Allocates a new Polygon.\n\n        `stride`\n            The number of floats per vertex. This should be at least 3, for the\n            default aPosition vec3.\n\n        `points`\n            The number of vertices in the polygon that space is allocated for.\n            If `data` is given, this is also the number of points in the polygon.\n\n        `data`\n            If not None, an iterable of length stride * points, that gives the\n            vertex data for each of the points.\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_5renpy_3gl2_11gl2geometry_7Polygon___init__;
#endif
static int __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_stride;
  int __pyx_v_points;
  PyObject *__pyx_v_data = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_stride,&__pyx_n_s_points,&__pyx_n_s_data,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_points)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 21, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 21, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 21, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_stride = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_stride == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L3_error)
    __pyx_v_points = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_points == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 21, __pyx_L3_error)
    __pyx_v_data = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 21, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon___init__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), __pyx_v_stride, __pyx_v_points, __pyx_v_data);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon___init__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_v_stride, int __pyx_v_points, PyObject *__pyx_v_data) {
  int __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  float __pyx_t_5;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/gl2geometry.pyx":40
 *         cdef int i
 * 
 *         self.stride = stride             # <<<<<<<<<<<<<<
 * 
 *         self.data = <float *> malloc(sizeof(float) * points * stride)
 */
  __pyx_v_self->stride = __pyx_v_stride;

  /* "renpy/gl2/gl2geometry.pyx":42
 *         self.stride = stride
 * 
 *         self.data = <float *> malloc(sizeof(float) * points * stride)             # <<<<<<<<<<<<<<
 * 
 *         if data is None:
 */
  __pyx_v_self->data = ((float *)malloc((((sizeof(float)) * __pyx_v_points) * __pyx_v_stride)));

  /* "renpy/gl2/gl2geometry.pyx":44
 *         self.data = <float *> malloc(sizeof(float) * points * stride)
 * 
 *         if data is None:             # <<<<<<<<<<<<<<
 *             self.points = 0
 *         else:
 */
  __pyx_t_1 = (__pyx_v_data == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/gl2geometry.pyx":45
 * 
 *         if data is None:
 *             self.points = 0             # <<<<<<<<<<<<<<
 *         else:
 *             self.points = points
 */
    __pyx_v_self->points = 0;

    /* "renpy/gl2/gl2geometry.pyx":44
 *         self.data = <float *> malloc(sizeof(float) * points * stride)
 * 
 *         if data is None:             # <<<<<<<<<<<<<<
 *             self.points = 0
 *         else:
 */
    goto __pyx_L3;
  }

  /* "renpy/gl2/gl2geometry.pyx":47
 *             self.points = 0
 *         else:
 *             self.points = points             # <<<<<<<<<<<<<<
 * 
 *             for 0 <= i < stride * points:
 */
  /*else*/ {
    __pyx_v_self->points = __pyx_v_points;

    /* "renpy/gl2/gl2geometry.pyx":49
 *             self.points = points
 * 
 *             for 0 <= i < stride * points:             # <<<<<<<<<<<<<<
 *                 self.data[i] = data[i]
 * 
 */
    __pyx_t_3 = (__pyx_v_stride * __pyx_v_points);
    for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_3; __pyx_v_i++) {

      /* "renpy/gl2/gl2geometry.pyx":50
 * 
 *             for 0 <= i < stride * points:
 *                 self.data[i] = data[i]             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(Polygon self):
 */
      __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_data, __pyx_v_i, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __pyx_PyFloat_AsFloat(__pyx_t_4); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 50, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      (__pyx_v_self->data[__pyx_v_i]) = __pyx_t_5;
    }
  }
  __pyx_L3:;

  /* "renpy/gl2/gl2geometry.pyx":21
 * cdef class Polygon:
 * 
 *     def __init__(Polygon self, int stride, int points, data):             # <<<<<<<<<<<<<<
 *         """
 *         Allocates a new Polygon.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":52
 *                 self.data[i] = data[i]
 * 
 *     def __dealloc__(Polygon self):             # <<<<<<<<<<<<<<
 *         free(self.data)
 * 
 */

/* Python wrapper */
static void __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_2__dealloc__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_2__dealloc__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "renpy/gl2/gl2geometry.pyx":53
 * 
 *     def __dealloc__(Polygon self):
 *         free(self.data)             # <<<<<<<<<<<<<<
 * 
 *     cpdef Polygon copy(Polygon self):
 */
  free(__pyx_v_self->data);

  /* "renpy/gl2/gl2geometry.pyx":52
 *                 self.data[i] = data[i]
 * 
 *     def __dealloc__(Polygon self):             # <<<<<<<<<<<<<<
 *         free(self.data)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl2/gl2geometry.pyx":55
 *         free(self.data)
 * 
 *     cpdef Polygon copy(Polygon self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this polygon.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_5copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("copy", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_5copy)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 55, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":60
 *         """
 * 
 *         cdef Polygon rv = Polygon(self.stride, self.points, None)             # <<<<<<<<<<<<<<
 *         rv.points = self.points
 *         memcpy(rv.data, self.data, sizeof(float) * self.stride * self.points)
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->points); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/gl2/gl2geometry.pyx":61
 * 
 *         cdef Polygon rv = Polygon(self.stride, self.points, None)
 *         rv.points = self.points             # <<<<<<<<<<<<<<
 *         memcpy(rv.data, self.data, sizeof(float) * self.stride * self.points)
 *         return rv
 */
  __pyx_t_5 = __pyx_v_self->points;
  __pyx_v_rv->points = __pyx_t_5;

  /* "renpy/gl2/gl2geometry.pyx":62
 *         cdef Polygon rv = Polygon(self.stride, self.points, None)
 *         rv.points = self.points
 *         memcpy(rv.data, self.data, sizeof(float) * self.stride * self.points)             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  memcpy(__pyx_v_rv->data, __pyx_v_self->data, (((sizeof(float)) * __pyx_v_self->stride) * __pyx_v_self->points));

  /* "renpy/gl2/gl2geometry.pyx":63
 *         rv.points = self.points
 *         memcpy(rv.data, self.data, sizeof(float) * self.stride * self.points)
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     cpdef void offset_inplace(Polygon self, double x, double y, double z):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":55
 *         free(self.data)
 * 
 *     cpdef Polygon copy(Polygon self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this polygon.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_5copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_4copy[] = "\n        Returns a copy of this polygon.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_5copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_4copy(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_4copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("copy", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_copy(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":65
 *         return rv
 * 
 *     cpdef void offset_inplace(Polygon self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         cdef float *p = self.data
 *         cdef int i
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_7offset_inplace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch) {
  float *__pyx_v_p;
  CYTHON_UNUSED int __pyx_v_i;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  float *__pyx_t_10;
  long __pyx_t_11;
  __Pyx_RefNannySetupContext("offset_inplace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_offset_inplace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_7offset_inplace)) {
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 65, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 65, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyFloat_FromDouble(__pyx_v_z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 65, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = __pyx_t_1; __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 65, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":66
 * 
 *     cpdef void offset_inplace(Polygon self, double x, double y, double z):
 *         cdef float *p = self.data             # <<<<<<<<<<<<<<
 *         cdef int i
 * 
 */
  __pyx_t_10 = __pyx_v_self->data;
  __pyx_v_p = __pyx_t_10;

  /* "renpy/gl2/gl2geometry.pyx":69
 *         cdef int i
 * 
 *         for 0 <= i < self.points:             # <<<<<<<<<<<<<<
 *             p[X] += x
 *             p[Y] += y
 */
  __pyx_t_8 = __pyx_v_self->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_8; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":70
 * 
 *         for 0 <= i < self.points:
 *             p[X] += x             # <<<<<<<<<<<<<<
 *             p[Y] += y
 *             p[Z] += z
 */
    __pyx_t_11 = 0;
    (__pyx_v_p[__pyx_t_11]) = ((__pyx_v_p[__pyx_t_11]) + __pyx_v_x);

    /* "renpy/gl2/gl2geometry.pyx":71
 *         for 0 <= i < self.points:
 *             p[X] += x
 *             p[Y] += y             # <<<<<<<<<<<<<<
 *             p[Z] += z
 * 
 */
    __pyx_t_11 = 1;
    (__pyx_v_p[__pyx_t_11]) = ((__pyx_v_p[__pyx_t_11]) + __pyx_v_y);

    /* "renpy/gl2/gl2geometry.pyx":72
 *             p[X] += x
 *             p[Y] += y
 *             p[Z] += z             # <<<<<<<<<<<<<<
 * 
 *             p += self.stride
 */
    __pyx_t_11 = 2;
    (__pyx_v_p[__pyx_t_11]) = ((__pyx_v_p[__pyx_t_11]) + __pyx_v_z);

    /* "renpy/gl2/gl2geometry.pyx":74
 *             p[Z] += z
 * 
 *             p += self.stride             # <<<<<<<<<<<<<<
 * 
 *     cpdef Polygon offset(Polygon self, double x, double y, double z):
 */
    __pyx_v_p = (__pyx_v_p + __pyx_v_self->stride);
  }

  /* "renpy/gl2/gl2geometry.pyx":65
 *         return rv
 * 
 *     cpdef void offset_inplace(Polygon self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         cdef float *p = self.data
 *         cdef int i
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Polygon.offset_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_7offset_inplace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_7offset_inplace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_z;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("offset_inplace (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_z,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset_inplace", 1, 3, 3, 1); __PYX_ERR(0, 65, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_z)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset_inplace", 1, 3, 3, 2); __PYX_ERR(0, 65, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "offset_inplace") < 0)) __PYX_ERR(0, 65, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L3_error)
    __pyx_v_z = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_z == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 65, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("offset_inplace", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 65, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.offset_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_6offset_inplace(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_6offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("offset_inplace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset_inplace(__pyx_v_self, __pyx_v_x, __pyx_v_y, __pyx_v_z, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.offset_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":76
 *             p += self.stride
 * 
 *     cpdef Polygon offset(Polygon self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         cdef Polygon rv = self.copy()
 *         rv.offset_inplace(x, y, z)
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_9offset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("offset", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_9offset)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyFloat_FromDouble(__pyx_v_z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = __pyx_t_1; __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 76, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 76, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":77
 * 
 *     cpdef Polygon offset(Polygon self, double x, double y, double z):
 *         cdef Polygon rv = self.copy()             # <<<<<<<<<<<<<<
 *         rv.offset_inplace(x, y, z)
 *         return rv
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":78
 *     cpdef Polygon offset(Polygon self, double x, double y, double z):
 *         cdef Polygon rv = self.copy()
 *         rv.offset_inplace(x, y, z)             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_rv->__pyx_vtab)->offset_inplace(__pyx_v_rv, __pyx_v_x, __pyx_v_y, __pyx_v_z, 0);

  /* "renpy/gl2/gl2geometry.pyx":79
 *         cdef Polygon rv = self.copy()
 *         rv.offset_inplace(x, y, z)
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     cpdef void perspective_divide_inplace(Polygon self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":76
 *             p += self.stride
 * 
 *     cpdef Polygon offset(Polygon self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         cdef Polygon rv = self.copy()
 *         rv.offset_inplace(x, y, z)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_9offset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_9offset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_z;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("offset (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_z,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset", 1, 3, 3, 1); __PYX_ERR(0, 76, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_z)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset", 1, 3, 3, 2); __PYX_ERR(0, 76, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "offset") < 0)) __PYX_ERR(0, 76, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 76, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 76, __pyx_L3_error)
    __pyx_v_z = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_z == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 76, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("offset", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 76, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_8offset(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_8offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("offset", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset(__pyx_v_self, __pyx_v_x, __pyx_v_y, __pyx_v_z, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":81
 *         return rv
 * 
 *     cpdef void perspective_divide_inplace(Polygon self):             # <<<<<<<<<<<<<<
 *         """
 *         Performs a perspective divide on each point in this polygon, in place.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_11perspective_divide_inplace(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_skip_dispatch) {
  float *__pyx_v_p;
  CYTHON_UNUSED int __pyx_v_i;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  float *__pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  __Pyx_RefNannySetupContext("perspective_divide_inplace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_perspective_divide_inplace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_11perspective_divide_inplace)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":86
 *         """
 * 
 *         cdef float *p = self.data             # <<<<<<<<<<<<<<
 *         cdef int i
 * 
 */
  __pyx_t_5 = __pyx_v_self->data;
  __pyx_v_p = __pyx_t_5;

  /* "renpy/gl2/gl2geometry.pyx":89
 *         cdef int i
 * 
 *         for 0 <= i < self.points:             # <<<<<<<<<<<<<<
 *             if p[W]:
 *                 p[X] /= p[W]
 */
  __pyx_t_6 = __pyx_v_self->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_6; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":90
 * 
 *         for 0 <= i < self.points:
 *             if p[W]:             # <<<<<<<<<<<<<<
 *                 p[X] /= p[W]
 *                 p[Y] /= p[W]
 */
    __pyx_t_7 = ((__pyx_v_p[3]) != 0);
    if (__pyx_t_7) {

      /* "renpy/gl2/gl2geometry.pyx":91
 *         for 0 <= i < self.points:
 *             if p[W]:
 *                 p[X] /= p[W]             # <<<<<<<<<<<<<<
 *                 p[Y] /= p[W]
 *                 p[Z] /= p[W]
 */
      __pyx_t_8 = 0;
      if (unlikely((__pyx_v_p[3]) == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 91, __pyx_L1_error)
      }
      (__pyx_v_p[__pyx_t_8]) = ((__pyx_v_p[__pyx_t_8]) / (__pyx_v_p[3]));

      /* "renpy/gl2/gl2geometry.pyx":92
 *             if p[W]:
 *                 p[X] /= p[W]
 *                 p[Y] /= p[W]             # <<<<<<<<<<<<<<
 *                 p[Z] /= p[W]
 * 
 */
      __pyx_t_8 = 1;
      if (unlikely((__pyx_v_p[3]) == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 92, __pyx_L1_error)
      }
      (__pyx_v_p[__pyx_t_8]) = ((__pyx_v_p[__pyx_t_8]) / (__pyx_v_p[3]));

      /* "renpy/gl2/gl2geometry.pyx":93
 *                 p[X] /= p[W]
 *                 p[Y] /= p[W]
 *                 p[Z] /= p[W]             # <<<<<<<<<<<<<<
 * 
 *             p += self.stride
 */
      __pyx_t_8 = 2;
      if (unlikely((__pyx_v_p[3]) == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "float division");
        __PYX_ERR(0, 93, __pyx_L1_error)
      }
      (__pyx_v_p[__pyx_t_8]) = ((__pyx_v_p[__pyx_t_8]) / (__pyx_v_p[3]));

      /* "renpy/gl2/gl2geometry.pyx":90
 * 
 *         for 0 <= i < self.points:
 *             if p[W]:             # <<<<<<<<<<<<<<
 *                 p[X] /= p[W]
 *                 p[Y] /= p[W]
 */
    }

    /* "renpy/gl2/gl2geometry.pyx":95
 *                 p[Z] /= p[W]
 * 
 *             p += self.stride             # <<<<<<<<<<<<<<
 * 
 *     cpdef Polygon perspective_divide(Polygon self):
 */
    __pyx_v_p = (__pyx_v_p + __pyx_v_self->stride);
  }

  /* "renpy/gl2/gl2geometry.pyx":81
 *         return rv
 * 
 *     cpdef void perspective_divide_inplace(Polygon self):             # <<<<<<<<<<<<<<
 *         """
 *         Performs a perspective divide on each point in this polygon, in place.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Polygon.perspective_divide_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_11perspective_divide_inplace(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_10perspective_divide_inplace[] = "\n        Performs a perspective divide on each point in this polygon, in place.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_11perspective_divide_inplace(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("perspective_divide_inplace (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_10perspective_divide_inplace(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_10perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("perspective_divide_inplace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide_inplace(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.perspective_divide_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":97
 *             p += self.stride
 * 
 *     cpdef Polygon perspective_divide(Polygon self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a perspective divided copy of this polygon.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_13perspective_divide(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("perspective_divide", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_perspective_divide); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_13perspective_divide)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 97, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":102
 *         """
 * 
 *         cdef Polygon rv = self.copy()             # <<<<<<<<<<<<<<
 *         rv.perspective_divide_inplace()
 *         return rv
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":103
 * 
 *         cdef Polygon rv = self.copy()
 *         rv.perspective_divide_inplace()             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_rv->__pyx_vtab)->perspective_divide_inplace(__pyx_v_rv, 0);

  /* "renpy/gl2/gl2geometry.pyx":104
 *         cdef Polygon rv = self.copy()
 *         rv.perspective_divide_inplace()
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     cpdef void multiply_matrix_inplace(Polygon self, Matrix m):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":97
 *             p += self.stride
 * 
 *     cpdef Polygon perspective_divide(Polygon self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a perspective divided copy of this polygon.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.perspective_divide", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_13perspective_divide(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_12perspective_divide[] = "\n        Returns a perspective divided copy of this polygon.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_13perspective_divide(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("perspective_divide (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_12perspective_divide(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_12perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("perspective_divide", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.perspective_divide", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":106
 *         return rv
 * 
 *     cpdef void multiply_matrix_inplace(Polygon self, Matrix m):             # <<<<<<<<<<<<<<
 *         """
 *         Multiplies each point's position with the matrix `m`, in place.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_15multiply_matrix_inplace(PyObject *__pyx_v_self, PyObject *__pyx_v_m); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m, int __pyx_skip_dispatch) {
  CYTHON_UNUSED int __pyx_v_i;
  float *__pyx_v_p;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  float *__pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("multiply_matrix_inplace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_multiply_matrix_inplace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_15multiply_matrix_inplace)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_m)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_m)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_m)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_m));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_m));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_m));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":112
 * 
 *         cdef int i
 *         cdef float *p = self.data             # <<<<<<<<<<<<<<
 * 
 *         for 0 <= i < self.points:
 */
  __pyx_t_6 = __pyx_v_self->data;
  __pyx_v_p = __pyx_t_6;

  /* "renpy/gl2/gl2geometry.pyx":114
 *         cdef float *p = self.data
 * 
 *         for 0 <= i < self.points:             # <<<<<<<<<<<<<<
 *             m.transform4(p, p+1, p+2, p+3, p[0], p[1], p[2], p[3])
 *             p += self.stride
 */
  __pyx_t_7 = __pyx_v_self->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":115
 * 
 *         for 0 <= i < self.points:
 *             m.transform4(p, p+1, p+2, p+3, p[0], p[1], p[2], p[3])             # <<<<<<<<<<<<<<
 *             p += self.stride
 * 
 */
    __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(__pyx_v_m, __pyx_v_p, (__pyx_v_p + 1), (__pyx_v_p + 2), (__pyx_v_p + 3), (__pyx_v_p[0]), (__pyx_v_p[1]), (__pyx_v_p[2]), (__pyx_v_p[3]));

    /* "renpy/gl2/gl2geometry.pyx":116
 *         for 0 <= i < self.points:
 *             m.transform4(p, p+1, p+2, p+3, p[0], p[1], p[2], p[3])
 *             p += self.stride             # <<<<<<<<<<<<<<
 * 
 *     cpdef Polygon multiply_matrix(Polygon self, Matrix m):
 */
    __pyx_v_p = (__pyx_v_p + __pyx_v_self->stride);
  }

  /* "renpy/gl2/gl2geometry.pyx":106
 *         return rv
 * 
 *     cpdef void multiply_matrix_inplace(Polygon self, Matrix m):             # <<<<<<<<<<<<<<
 *         """
 *         Multiplies each point's position with the matrix `m`, in place.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Polygon.multiply_matrix_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_15multiply_matrix_inplace(PyObject *__pyx_v_self, PyObject *__pyx_v_m); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_14multiply_matrix_inplace[] = "\n        Multiplies each point's position with the matrix `m`, in place.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_15multiply_matrix_inplace(PyObject *__pyx_v_self, PyObject *__pyx_v_m) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("multiply_matrix_inplace (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_m), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "m", 0))) __PYX_ERR(0, 106, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_14multiply_matrix_inplace(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)__pyx_v_m));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_14multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("multiply_matrix_inplace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix_inplace(__pyx_v_self, __pyx_v_m, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.multiply_matrix_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":118
 *             p += self.stride
 * 
 *     cpdef Polygon multiply_matrix(Polygon self, Matrix m):             # <<<<<<<<<<<<<<
 *         """
 *         Multiplies each point's position with the matrix `m`, in place,
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_17multiply_matrix(PyObject *__pyx_v_self, PyObject *__pyx_v_m); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("multiply_matrix", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_multiply_matrix); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_17multiply_matrix)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_m)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_m)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_m)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 118, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_m));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_m));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_m));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 118, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 118, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":124
 *         """
 * 
 *         cdef Polygon rv = self.copy()             # <<<<<<<<<<<<<<
 *         rv.multiply_matrix_inplace(m)
 *         return rv
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":125
 * 
 *         cdef Polygon rv = self.copy()
 *         rv.multiply_matrix_inplace(m)             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_rv->__pyx_vtab)->multiply_matrix_inplace(__pyx_v_rv, __pyx_v_m, 0);

  /* "renpy/gl2/gl2geometry.pyx":126
 *         cdef Polygon rv = self.copy()
 *         rv.multiply_matrix_inplace(m)
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(self, other):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":118
 *             p += self.stride
 * 
 *     cpdef Polygon multiply_matrix(Polygon self, Matrix m):             # <<<<<<<<<<<<<<
 *         """
 *         Multiplies each point's position with the matrix `m`, in place,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.multiply_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_17multiply_matrix(PyObject *__pyx_v_self, PyObject *__pyx_v_m); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_16multiply_matrix[] = "\n        Multiplies each point's position with the matrix `m`, in place,\n        and returns a copy.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_17multiply_matrix(PyObject *__pyx_v_self, PyObject *__pyx_v_m) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("multiply_matrix (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_m), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "m", 0))) __PYX_ERR(0, 118, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_16multiply_matrix(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)__pyx_v_m));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_16multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_m) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("multiply_matrix", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix(__pyx_v_self, __pyx_v_m, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.multiply_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":128
 *         return rv
 * 
 *     def __mul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_19__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_19__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_18__mul__(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_18__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "renpy/gl2/gl2geometry.pyx":129
 * 
 *     def __mul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_5renpy_7display_6matrix_Matrix); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/gl2geometry.pyx":130
 *     def __mul__(self, other):
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)             # <<<<<<<<<<<<<<
 * 
 *         return NotImplemented
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_multiply_matrix); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 130, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_other); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_other};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_other};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_v_other);
        __Pyx_GIVEREF(__pyx_v_other);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_other);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":129
 * 
 *     def __mul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":132
 *             return self.multiply_matrix(other)
 * 
 *         return NotImplemented             # <<<<<<<<<<<<<<
 * 
 *     def __rmul__(self, other):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_builtin_NotImplemented);
  __pyx_r = __pyx_builtin_NotImplemented;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":128
 *         return rv
 * 
 *     def __mul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":134
 *         return NotImplemented
 * 
 *     def __rmul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_21__rmul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_21__rmul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__rmul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_20__rmul__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_20__rmul__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__rmul__", 0);

  /* "renpy/gl2/gl2geometry.pyx":135
 * 
 *     def __rmul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_5renpy_7display_6matrix_Matrix); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/gl2geometry.pyx":136
 *     def __rmul__(self, other):
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)             # <<<<<<<<<<<<<<
 * 
 *         return NotImplemented
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_5renpy_7display_6matrix_Matrix))))) __PYX_ERR(0, 136, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self->__pyx_vtab)->multiply_matrix(__pyx_v_self, ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":135
 * 
 *     def __rmul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":138
 *             return self.multiply_matrix(other)
 * 
 *         return NotImplemented             # <<<<<<<<<<<<<<
 * 
 *     cpdef Polygon intersect(Polygon self, Polygon other):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_builtin_NotImplemented);
  __pyx_r = __pyx_builtin_NotImplemented;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":134
 *         return NotImplemented
 * 
 *     def __rmul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__rmul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":140
 *         return NotImplemented
 * 
 *     cpdef Polygon intersect(Polygon self, Polygon other):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a new polygon that is the intersection of the current polygon
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_23intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_other, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_rv = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("intersect", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_intersect); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_23intersect)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_other)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_other)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 140, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_other));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_other));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_other));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 140, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":146
 *         """
 * 
 *         rv = intersect(other, self, self.stride + other.stride - 4, True)             # <<<<<<<<<<<<<<
 * 
 *         if rv is None:
 */
  __pyx_t_1 = __pyx_f_5renpy_3gl2_11gl2geometry_intersect(__pyx_v_other, __pyx_v_self, ((__pyx_v_self->stride + __pyx_v_other->stride) - 4), 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":148
 *         rv = intersect(other, self, self.stride + other.stride - 4, True)
 * 
 *         if rv is None:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  __pyx_t_6 = (__pyx_v_rv == Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "renpy/gl2/gl2geometry.pyx":149
 * 
 *         if rv is None:
 *             return None             # <<<<<<<<<<<<<<
 * 
 *         # We only need to interpolate OP if it has something other than
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(Py_None);
    __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)Py_None);
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":148
 *         rv = intersect(other, self, self.stride + other.stride - 4, True)
 * 
 *         if rv is None:             # <<<<<<<<<<<<<<
 *             return None
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":153
 *         # We only need to interpolate OP if it has something other than
 *         # position data.
 *         if other.stride > 4:             # <<<<<<<<<<<<<<
 *             barycentric(other, rv, self.stride - 4)
 * 
 */
  __pyx_t_7 = ((__pyx_v_other->stride > 4) != 0);
  if (__pyx_t_7) {

    /* "renpy/gl2/gl2geometry.pyx":154
 *         # position data.
 *         if other.stride > 4:
 *             barycentric(other, rv, self.stride - 4)             # <<<<<<<<<<<<<<
 * 
 *         barycentric(self, rv, 0)
 */
    if (!(likely(((__pyx_v_rv) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_rv, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 154, __pyx_L1_error)
    __pyx_t_1 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_other, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_rv), (__pyx_v_self->stride - 4), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl2/gl2geometry.pyx":153
 *         # We only need to interpolate OP if it has something other than
 *         # position data.
 *         if other.stride > 4:             # <<<<<<<<<<<<<<
 *             barycentric(other, rv, self.stride - 4)
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":156
 *             barycentric(other, rv, self.stride - 4)
 * 
 *         barycentric(self, rv, 0)             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
  if (!(likely(((__pyx_v_rv) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_rv, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 156, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_self, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_rv), 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":158
 *         barycentric(self, rv, 0)
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  if (!(likely(((__pyx_v_rv) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_rv, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_rv);
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":140
 *         return NotImplemented
 * 
 *     cpdef Polygon intersect(Polygon self, Polygon other):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a new polygon that is the intersection of the current polygon
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_23intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_22intersect[] = "\n        Returns a new polygon that is the intersection of the current polygon\n        and the other polygon.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_23intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("intersect (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "other", 0))) __PYX_ERR(0, 140, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_22intersect(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_22intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("intersect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_intersect(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":160
 *         return rv
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         rv = "<Polygon"
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_25__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_25__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_24__repr__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_24__repr__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self) {
  PyObject *__pyx_v_rv = NULL;
  PyObject *__pyx_v_p = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  long __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "renpy/gl2/gl2geometry.pyx":161
 * 
 *     def __repr__(self):
 *         rv = "<Polygon"             # <<<<<<<<<<<<<<
 * 
 *         for 0 <= p < self.points:
 */
  __Pyx_INCREF(__pyx_kp_s_Polygon);
  __pyx_v_rv = __pyx_kp_s_Polygon;

  /* "renpy/gl2/gl2geometry.pyx":163
 *         rv = "<Polygon"
 * 
 *         for 0 <= p < self.points:             # <<<<<<<<<<<<<<
 *             rv += "\n  ["
 *             for 0 <= i < self.stride:
 */
  __pyx_t_1 = __pyx_v_self->points;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2++) {
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 163, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_p, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "renpy/gl2/gl2geometry.pyx":164
 * 
 *         for 0 <= p < self.points:
 *             rv += "\n  ["             # <<<<<<<<<<<<<<
 *             for 0 <= i < self.stride:
 *                 rv += "{:g}, ".format(self.data[p * self.stride + i])
 */
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_kp_s_); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "renpy/gl2/gl2geometry.pyx":165
 *         for 0 <= p < self.points:
 *             rv += "\n  ["
 *             for 0 <= i < self.stride:             # <<<<<<<<<<<<<<
 *                 rv += "{:g}, ".format(self.data[p * self.stride + i])
 *             rv += "]"
 */
    __pyx_t_4 = __pyx_v_self->stride;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
      __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 165, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "renpy/gl2/gl2geometry.pyx":166
 *             rv += "\n  ["
 *             for 0 <= i < self.stride:
 *                 rv += "{:g}, ".format(self.data[p * self.stride + i])             # <<<<<<<<<<<<<<
 *             rv += "]"
 * 
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_g, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyNumber_Multiply(__pyx_v_p, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyNumber_Add(__pyx_t_8, __pyx_v_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_7); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = PyFloat_FromDouble((__pyx_v_self->data[__pyx_t_9])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      if (!__pyx_t_8) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 166, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 166, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 166, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_long(__pyx_v_i); if (unlikely((__pyx_t_5 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 165, __pyx_L1_error)
    }

    /* "renpy/gl2/gl2geometry.pyx":165
 *         for 0 <= p < self.points:
 *             rv += "\n  ["
 *             for 0 <= i < self.stride:             # <<<<<<<<<<<<<<
 *                 rv += "{:g}, ".format(self.data[p * self.stride + i])
 *             rv += "]"
 */
    __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "renpy/gl2/gl2geometry.pyx":167
 *             for 0 <= i < self.stride:
 *                 rv += "{:g}, ".format(self.data[p * self.stride + i])
 *             rv += "]"             # <<<<<<<<<<<<<<
 * 
 *         rv += ">"
 */
    __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_kp_s__2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyInt_As_long(__pyx_v_p); if (unlikely((__pyx_t_2 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 163, __pyx_L1_error)
  }

  /* "renpy/gl2/gl2geometry.pyx":163
 *         rv = "<Polygon"
 * 
 *         for 0 <= p < self.points:             # <<<<<<<<<<<<<<
 *             rv += "\n  ["
 *             for 0 <= i < self.stride:
 */
  __pyx_t_6 = __Pyx_PyInt_From_long(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_XDECREF_SET(__pyx_v_p, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "renpy/gl2/gl2geometry.pyx":169
 *             rv += "]"
 * 
 *         rv += ">"             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  __pyx_t_6 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_kp_s__3); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "renpy/gl2/gl2geometry.pyx":170
 * 
 *         rv += ">"
 *         return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":160
 *         return rv
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         rv = "<Polygon"
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_p);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_27__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_26__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_26__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_29__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_28__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_7Polygon_28__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Polygon.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":173
 * 
 * 
 * cpdef Polygon rectangle(double l, double t, double r, double b):             # <<<<<<<<<<<<<<
 *     """
 *     Generates a rectangle with one corner at (l, t, 0, 1) and the other at (r, b, 0, 1).
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_1rectangle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_rectangle(double __pyx_v_l, double __pyx_v_t, double __pyx_v_r, double __pyx_v_b, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  float *__pyx_v_p;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float *__pyx_t_2;
  __Pyx_RefNannySetupContext("rectangle", 0);

  /* "renpy/gl2/gl2geometry.pyx":178
 *     """
 * 
 *     cdef Polygon rv = Polygon(4, 4, None)             # <<<<<<<<<<<<<<
 * 
 *     rv.points = 4
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":180
 *     cdef Polygon rv = Polygon(4, 4, None)
 * 
 *     rv.points = 4             # <<<<<<<<<<<<<<
 * 
 *     cdef float *p = rv.data
 */
  __pyx_v_rv->points = 4;

  /* "renpy/gl2/gl2geometry.pyx":182
 *     rv.points = 4
 * 
 *     cdef float *p = rv.data             # <<<<<<<<<<<<<<
 * 
 *     p[X] = l
 */
  __pyx_t_2 = __pyx_v_rv->data;
  __pyx_v_p = __pyx_t_2;

  /* "renpy/gl2/gl2geometry.pyx":184
 *     cdef float *p = rv.data
 * 
 *     p[X] = l             # <<<<<<<<<<<<<<
 *     p[Y] = t
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_l;

  /* "renpy/gl2/gl2geometry.pyx":185
 * 
 *     p[X] = l
 *     p[Y] = t             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_t;

  /* "renpy/gl2/gl2geometry.pyx":186
 *     p[X] = l
 *     p[Y] = t
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 * 
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":187
 *     p[Y] = t
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 * 
 *     p += 4
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":189
 *     p[W] = 1
 * 
 *     p += 4             # <<<<<<<<<<<<<<
 * 
 *     p[X] = r
 */
  __pyx_v_p = (__pyx_v_p + 4);

  /* "renpy/gl2/gl2geometry.pyx":191
 *     p += 4
 * 
 *     p[X] = r             # <<<<<<<<<<<<<<
 *     p[Y] = t
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_r;

  /* "renpy/gl2/gl2geometry.pyx":192
 * 
 *     p[X] = r
 *     p[Y] = t             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_t;

  /* "renpy/gl2/gl2geometry.pyx":193
 *     p[X] = r
 *     p[Y] = t
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 * 
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":194
 *     p[Y] = t
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 * 
 *     p += 4
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":196
 *     p[W] = 1
 * 
 *     p += 4             # <<<<<<<<<<<<<<
 * 
 *     p[X] = r
 */
  __pyx_v_p = (__pyx_v_p + 4);

  /* "renpy/gl2/gl2geometry.pyx":198
 *     p += 4
 * 
 *     p[X] = r             # <<<<<<<<<<<<<<
 *     p[Y] = b
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_r;

  /* "renpy/gl2/gl2geometry.pyx":199
 * 
 *     p[X] = r
 *     p[Y] = b             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_b;

  /* "renpy/gl2/gl2geometry.pyx":200
 *     p[X] = r
 *     p[Y] = b
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 * 
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":201
 *     p[Y] = b
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 * 
 *     p += 4
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":203
 *     p[W] = 1
 * 
 *     p += 4             # <<<<<<<<<<<<<<
 * 
 *     p[X] = l
 */
  __pyx_v_p = (__pyx_v_p + 4);

  /* "renpy/gl2/gl2geometry.pyx":205
 *     p += 4
 * 
 *     p[X] = l             # <<<<<<<<<<<<<<
 *     p[Y] = b
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_l;

  /* "renpy/gl2/gl2geometry.pyx":206
 * 
 *     p[X] = l
 *     p[Y] = b             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_b;

  /* "renpy/gl2/gl2geometry.pyx":207
 *     p[X] = l
 *     p[Y] = b
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 * 
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":208
 *     p[Y] = b
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":210
 *     p[W] = 1
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":173
 * 
 * 
 * cpdef Polygon rectangle(double l, double t, double r, double b):             # <<<<<<<<<<<<<<
 *     """
 *     Generates a rectangle with one corner at (l, t, 0, 1) and the other at (r, b, 0, 1).
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_1rectangle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_rectangle[] = "\n    Generates a rectangle with one corner at (l, t, 0, 1) and the other at (r, b, 0, 1).\n    ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_1rectangle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_l;
  double __pyx_v_t;
  double __pyx_v_r;
  double __pyx_v_b;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("rectangle (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_l,&__pyx_n_s_t,&__pyx_n_s_r,&__pyx_n_s_b,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_l)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_t)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rectangle", 1, 4, 4, 1); __PYX_ERR(0, 173, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_r)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rectangle", 1, 4, 4, 2); __PYX_ERR(0, 173, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("rectangle", 1, 4, 4, 3); __PYX_ERR(0, 173, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "rectangle") < 0)) __PYX_ERR(0, 173, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_l = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_l == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
    __pyx_v_t = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_t == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
    __pyx_v_r = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_r == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
    __pyx_v_b = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_b == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("rectangle", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 173, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_rectangle(__pyx_self, __pyx_v_l, __pyx_v_t, __pyx_v_r, __pyx_v_b);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_rectangle(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_l, double __pyx_v_t, double __pyx_v_r, double __pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("rectangle", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_rectangle(__pyx_v_l, __pyx_v_t, __pyx_v_r, __pyx_v_b, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":213
 * 
 * 
 * cpdef Polygon texture_rectangle(double pl, double pt, double pr, double pb, double tl, double tt, double tr, double tb):             # <<<<<<<<<<<<<<
 *     """
 *     This generates a rectangle with teture coordinates. One corner is at (pl, pt, 0, 1) with
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_3texture_rectangle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_texture_rectangle(double __pyx_v_pl, double __pyx_v_pt, double __pyx_v_pr, double __pyx_v_pb, double __pyx_v_tl, double __pyx_v_tt, double __pyx_v_tr, double __pyx_v_tb, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  float *__pyx_v_p;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float *__pyx_t_2;
  __Pyx_RefNannySetupContext("texture_rectangle", 0);

  /* "renpy/gl2/gl2geometry.pyx":219
 *     """
 * 
 *     cdef Polygon rv = Polygon(6, 4, None)             # <<<<<<<<<<<<<<
 * 
 *     rv.points = 4
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":221
 *     cdef Polygon rv = Polygon(6, 4, None)
 * 
 *     rv.points = 4             # <<<<<<<<<<<<<<
 * 
 *     cdef float *p = rv.data
 */
  __pyx_v_rv->points = 4;

  /* "renpy/gl2/gl2geometry.pyx":223
 *     rv.points = 4
 * 
 *     cdef float *p = rv.data             # <<<<<<<<<<<<<<
 * 
 *     p[X] = pl
 */
  __pyx_t_2 = __pyx_v_rv->data;
  __pyx_v_p = __pyx_t_2;

  /* "renpy/gl2/gl2geometry.pyx":225
 *     cdef float *p = rv.data
 * 
 *     p[X] = pl             # <<<<<<<<<<<<<<
 *     p[Y] = pt
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_pl;

  /* "renpy/gl2/gl2geometry.pyx":226
 * 
 *     p[X] = pl
 *     p[Y] = pt             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_pt;

  /* "renpy/gl2/gl2geometry.pyx":227
 *     p[X] = pl
 *     p[Y] = pt
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 *     p[TX] = tl
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":228
 *     p[Y] = pt
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 *     p[TX] = tl
 *     p[TY] = tt
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":229
 *     p[Z] = 0
 *     p[W] = 1
 *     p[TX] = tl             # <<<<<<<<<<<<<<
 *     p[TY] = tt
 * 
 */
  (__pyx_v_p[4]) = __pyx_v_tl;

  /* "renpy/gl2/gl2geometry.pyx":230
 *     p[W] = 1
 *     p[TX] = tl
 *     p[TY] = tt             # <<<<<<<<<<<<<<
 * 
 *     p += 6
 */
  (__pyx_v_p[5]) = __pyx_v_tt;

  /* "renpy/gl2/gl2geometry.pyx":232
 *     p[TY] = tt
 * 
 *     p += 6             # <<<<<<<<<<<<<<
 * 
 *     p[X] = pr
 */
  __pyx_v_p = (__pyx_v_p + 6);

  /* "renpy/gl2/gl2geometry.pyx":234
 *     p += 6
 * 
 *     p[X] = pr             # <<<<<<<<<<<<<<
 *     p[Y] = pt
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_pr;

  /* "renpy/gl2/gl2geometry.pyx":235
 * 
 *     p[X] = pr
 *     p[Y] = pt             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_pt;

  /* "renpy/gl2/gl2geometry.pyx":236
 *     p[X] = pr
 *     p[Y] = pt
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 *     p[TX] = tr
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":237
 *     p[Y] = pt
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 *     p[TX] = tr
 *     p[TY] = tt
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":238
 *     p[Z] = 0
 *     p[W] = 1
 *     p[TX] = tr             # <<<<<<<<<<<<<<
 *     p[TY] = tt
 * 
 */
  (__pyx_v_p[4]) = __pyx_v_tr;

  /* "renpy/gl2/gl2geometry.pyx":239
 *     p[W] = 1
 *     p[TX] = tr
 *     p[TY] = tt             # <<<<<<<<<<<<<<
 * 
 *     p += 6
 */
  (__pyx_v_p[5]) = __pyx_v_tt;

  /* "renpy/gl2/gl2geometry.pyx":241
 *     p[TY] = tt
 * 
 *     p += 6             # <<<<<<<<<<<<<<
 * 
 *     p[X] = pr
 */
  __pyx_v_p = (__pyx_v_p + 6);

  /* "renpy/gl2/gl2geometry.pyx":243
 *     p += 6
 * 
 *     p[X] = pr             # <<<<<<<<<<<<<<
 *     p[Y] = pb
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_pr;

  /* "renpy/gl2/gl2geometry.pyx":244
 * 
 *     p[X] = pr
 *     p[Y] = pb             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_pb;

  /* "renpy/gl2/gl2geometry.pyx":245
 *     p[X] = pr
 *     p[Y] = pb
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 *     p[TX] = tr
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":246
 *     p[Y] = pb
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 *     p[TX] = tr
 *     p[TY] = tb
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":247
 *     p[Z] = 0
 *     p[W] = 1
 *     p[TX] = tr             # <<<<<<<<<<<<<<
 *     p[TY] = tb
 * 
 */
  (__pyx_v_p[4]) = __pyx_v_tr;

  /* "renpy/gl2/gl2geometry.pyx":248
 *     p[W] = 1
 *     p[TX] = tr
 *     p[TY] = tb             # <<<<<<<<<<<<<<
 * 
 *     p += 6
 */
  (__pyx_v_p[5]) = __pyx_v_tb;

  /* "renpy/gl2/gl2geometry.pyx":250
 *     p[TY] = tb
 * 
 *     p += 6             # <<<<<<<<<<<<<<
 * 
 *     p[X] = pl
 */
  __pyx_v_p = (__pyx_v_p + 6);

  /* "renpy/gl2/gl2geometry.pyx":252
 *     p += 6
 * 
 *     p[X] = pl             # <<<<<<<<<<<<<<
 *     p[Y] = pb
 *     p[Z] = 0
 */
  (__pyx_v_p[0]) = __pyx_v_pl;

  /* "renpy/gl2/gl2geometry.pyx":253
 * 
 *     p[X] = pl
 *     p[Y] = pb             # <<<<<<<<<<<<<<
 *     p[Z] = 0
 *     p[W] = 1
 */
  (__pyx_v_p[1]) = __pyx_v_pb;

  /* "renpy/gl2/gl2geometry.pyx":254
 *     p[X] = pl
 *     p[Y] = pb
 *     p[Z] = 0             # <<<<<<<<<<<<<<
 *     p[W] = 1
 *     p[TX] = tl
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":255
 *     p[Y] = pb
 *     p[Z] = 0
 *     p[W] = 1             # <<<<<<<<<<<<<<
 *     p[TX] = tl
 *     p[TY] = tb
 */
  (__pyx_v_p[3]) = 1.0;

  /* "renpy/gl2/gl2geometry.pyx":256
 *     p[Z] = 0
 *     p[W] = 1
 *     p[TX] = tl             # <<<<<<<<<<<<<<
 *     p[TY] = tb
 * 
 */
  (__pyx_v_p[4]) = __pyx_v_tl;

  /* "renpy/gl2/gl2geometry.pyx":257
 *     p[W] = 1
 *     p[TX] = tl
 *     p[TY] = tb             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
  (__pyx_v_p[5]) = __pyx_v_tb;

  /* "renpy/gl2/gl2geometry.pyx":259
 *     p[TY] = tb
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":213
 * 
 * 
 * cpdef Polygon texture_rectangle(double pl, double pt, double pr, double pb, double tl, double tt, double tr, double tb):             # <<<<<<<<<<<<<<
 *     """
 *     This generates a rectangle with teture coordinates. One corner is at (pl, pt, 0, 1) with
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.texture_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_3texture_rectangle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_2texture_rectangle[] = "\n    This generates a rectangle with teture coordinates. One corner is at (pl, pt, 0, 1) with\n    texture coordinates (tl, tt), while the other is at (pr, pb, 0, 1) / (tr, tb).\n    ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_3texture_rectangle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_pl;
  double __pyx_v_pt;
  double __pyx_v_pr;
  double __pyx_v_pb;
  double __pyx_v_tl;
  double __pyx_v_tt;
  double __pyx_v_tr;
  double __pyx_v_tb;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("texture_rectangle (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pl,&__pyx_n_s_pt,&__pyx_n_s_pr,&__pyx_n_s_pb,&__pyx_n_s_tl,&__pyx_n_s_tt,&__pyx_n_s_tr,&__pyx_n_s_tb,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pl)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 1); __PYX_ERR(0, 213, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 2); __PYX_ERR(0, 213, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 3); __PYX_ERR(0, 213, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tl)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 4); __PYX_ERR(0, 213, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 5); __PYX_ERR(0, 213, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 6); __PYX_ERR(0, 213, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, 7); __PYX_ERR(0, 213, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "texture_rectangle") < 0)) __PYX_ERR(0, 213, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_pl = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_pl == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_pt = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_pt == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_pr = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_pr == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_pb = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_pb == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_tl = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_tl == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_tt = __pyx_PyFloat_AsDouble(values[5]); if (unlikely((__pyx_v_tt == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_tr = __pyx_PyFloat_AsDouble(values[6]); if (unlikely((__pyx_v_tr == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
    __pyx_v_tb = __pyx_PyFloat_AsDouble(values[7]); if (unlikely((__pyx_v_tb == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("texture_rectangle", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 213, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.texture_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_2texture_rectangle(__pyx_self, __pyx_v_pl, __pyx_v_pt, __pyx_v_pr, __pyx_v_pb, __pyx_v_tl, __pyx_v_tt, __pyx_v_tr, __pyx_v_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_2texture_rectangle(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_pl, double __pyx_v_pt, double __pyx_v_pr, double __pyx_v_pb, double __pyx_v_tl, double __pyx_v_tt, double __pyx_v_tr, double __pyx_v_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("texture_rectangle", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_texture_rectangle(__pyx_v_pl, __pyx_v_pt, __pyx_v_pr, __pyx_v_pb, __pyx_v_tl, __pyx_v_tt, __pyx_v_tr, __pyx_v_tb, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.texture_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":263
 * 
 * # Accssor functions for the contents of polygons.
 * cdef inline float get(Polygon p, int index, int offset):             # <<<<<<<<<<<<<<
 *     return p.data[index * p.stride + offset]
 * 
 */

static CYTHON_INLINE float __pyx_f_5renpy_3gl2_11gl2geometry_get(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p, int __pyx_v_index, int __pyx_v_offset) {
  float __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get", 0);

  /* "renpy/gl2/gl2geometry.pyx":264
 * # Accssor functions for the contents of polygons.
 * cdef inline float get(Polygon p, int index, int offset):
 *     return p.data[index * p.stride + offset]             # <<<<<<<<<<<<<<
 * 
 * cdef inline float *ref(Polygon p, int index, int offset):
 */
  __pyx_r = (__pyx_v_p->data[((__pyx_v_index * __pyx_v_p->stride) + __pyx_v_offset)]);
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":263
 * 
 * # Accssor functions for the contents of polygons.
 * cdef inline float get(Polygon p, int index, int offset):             # <<<<<<<<<<<<<<
 *     return p.data[index * p.stride + offset]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":266
 *     return p.data[index * p.stride + offset]
 * 
 * cdef inline float *ref(Polygon p, int index, int offset):             # <<<<<<<<<<<<<<
 *     return &p.data[index * p.stride + offset]
 * 
 */

static CYTHON_INLINE float *__pyx_f_5renpy_3gl2_11gl2geometry_ref(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p, int __pyx_v_index, int __pyx_v_offset) {
  float *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("ref", 0);

  /* "renpy/gl2/gl2geometry.pyx":267
 * 
 * cdef inline float *ref(Polygon p, int index, int offset):
 *     return &p.data[index * p.stride + offset]             # <<<<<<<<<<<<<<
 * 
 * cdef inline float set(Polygon p, int index, int offset, float value):
 */
  __pyx_r = (&(__pyx_v_p->data[((__pyx_v_index * __pyx_v_p->stride) + __pyx_v_offset)]));
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":266
 *     return p.data[index * p.stride + offset]
 * 
 * cdef inline float *ref(Polygon p, int index, int offset):             # <<<<<<<<<<<<<<
 *     return &p.data[index * p.stride + offset]
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":269
 *     return &p.data[index * p.stride + offset]
 * 
 * cdef inline float set(Polygon p, int index, int offset, float value):             # <<<<<<<<<<<<<<
 *     p.data[index * p.stride + offset] = value
 *     return value
 */

static CYTHON_INLINE float __pyx_f_5renpy_3gl2_11gl2geometry_set(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p, int __pyx_v_index, int __pyx_v_offset, float __pyx_v_value) {
  float __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set", 0);

  /* "renpy/gl2/gl2geometry.pyx":270
 * 
 * cdef inline float set(Polygon p, int index, int offset, float value):
 *     p.data[index * p.stride + offset] = value             # <<<<<<<<<<<<<<
 *     return value
 * 
 */
  (__pyx_v_p->data[((__pyx_v_index * __pyx_v_p->stride) + __pyx_v_offset)]) = __pyx_v_value;

  /* "renpy/gl2/gl2geometry.pyx":271
 * cdef inline float set(Polygon p, int index, int offset, float value):
 *     p.data[index * p.stride + offset] = value
 *     return value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_value;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":269
 *     return &p.data[index * p.stride + offset]
 * 
 * cdef inline float set(Polygon p, int index, int offset, float value):             # <<<<<<<<<<<<<<
 *     p.data[index * p.stride + offset] = value
 *     return value
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":274
 * 
 * 
 * cdef void intersectLines(             # <<<<<<<<<<<<<<
 *     float x1, float y1,
 *     float x2, float y2,
 */

static void __pyx_f_5renpy_3gl2_11gl2geometry_intersectLines(float __pyx_v_x1, float __pyx_v_y1, float __pyx_v_x2, float __pyx_v_y2, float __pyx_v_x3, float __pyx_v_y3, float __pyx_v_x4, float __pyx_v_y4, float *__pyx_v_px, float *__pyx_v_py) {
  float __pyx_v_denom;
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  __Pyx_RefNannySetupContext("intersectLines", 0);

  /* "renpy/gl2/gl2geometry.pyx":290
 *     """
 * 
 *     cdef float denom = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4)             # <<<<<<<<<<<<<<
 *     px[0] = ((x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2) * (x3 * y4 - y3 * x4)) / denom
 *     py[0] = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) / denom
 */
  __pyx_v_denom = (((__pyx_v_x1 - __pyx_v_x2) * (__pyx_v_y3 - __pyx_v_y4)) - ((__pyx_v_y1 - __pyx_v_y2) * (__pyx_v_x3 - __pyx_v_x4)));

  /* "renpy/gl2/gl2geometry.pyx":291
 * 
 *     cdef float denom = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4)
 *     px[0] = ((x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2) * (x3 * y4 - y3 * x4)) / denom             # <<<<<<<<<<<<<<
 *     py[0] = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) / denom
 * 
 */
  __pyx_t_1 = ((((__pyx_v_x1 * __pyx_v_y2) - (__pyx_v_y1 * __pyx_v_x2)) * (__pyx_v_x3 - __pyx_v_x4)) - ((__pyx_v_x1 - __pyx_v_x2) * ((__pyx_v_x3 * __pyx_v_y4) - (__pyx_v_y3 * __pyx_v_x4))));
  if (unlikely(__pyx_v_denom == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 291, __pyx_L1_error)
  }
  (__pyx_v_px[0]) = (__pyx_t_1 / __pyx_v_denom);

  /* "renpy/gl2/gl2geometry.pyx":292
 *     cdef float denom = (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4)
 *     px[0] = ((x1 * y2 - y1 * x2) * (x3 - x4) - (x1 - x2) * (x3 * y4 - y3 * x4)) / denom
 *     py[0] = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) / denom             # <<<<<<<<<<<<<<
 * 
 * cdef Polygon intersectOnce(float winding, float a0x, float a0y, float a1x, float a1y, Polygon p, int rvstride):
 */
  __pyx_t_1 = ((((__pyx_v_x1 * __pyx_v_y2) - (__pyx_v_y1 * __pyx_v_x2)) * (__pyx_v_y3 - __pyx_v_y4)) - ((__pyx_v_y1 - __pyx_v_y2) * ((__pyx_v_x3 * __pyx_v_y4) - (__pyx_v_y3 * __pyx_v_x4))));
  if (unlikely(__pyx_v_denom == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 292, __pyx_L1_error)
  }
  (__pyx_v_py[0]) = (__pyx_t_1 / __pyx_v_denom);

  /* "renpy/gl2/gl2geometry.pyx":274
 * 
 * 
 * cdef void intersectLines(             # <<<<<<<<<<<<<<
 *     float x1, float y1,
 *     float x2, float y2,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.intersectLines", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl2/gl2geometry.pyx":294
 *     py[0] = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) / denom
 * 
 * cdef Polygon intersectOnce(float winding, float a0x, float a0y, float a1x, float a1y, Polygon p, int rvstride):             # <<<<<<<<<<<<<<
 * 
 *     # The vector from a0 to a1.
 */

static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_intersectOnce(float __pyx_v_winding, float __pyx_v_a0x, float __pyx_v_a0y, float __pyx_v_a1x, float __pyx_v_a1y, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p, int __pyx_v_rvstride) {
  float __pyx_v_vecax;
  float __pyx_v_vecay;
  float __pyx_v_vecpx;
  int __pyx_v_inside[0x80];
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_allin;
  PyObject *__pyx_v_vecpy = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("intersectOnce", 0);

  /* "renpy/gl2/gl2geometry.pyx":297
 * 
 *     # The vector from a0 to a1.
 *     cdef float vecax = a1x - a0x             # <<<<<<<<<<<<<<
 *     cdef float vecay = a1y - a0y
 * 
 */
  __pyx_v_vecax = (__pyx_v_a1x - __pyx_v_a0x);

  /* "renpy/gl2/gl2geometry.pyx":298
 *     # The vector from a0 to a1.
 *     cdef float vecax = a1x - a0x
 *     cdef float vecay = a1y - a0y             # <<<<<<<<<<<<<<
 * 
 *     # The vector from a0 to each point.
 */
  __pyx_v_vecay = (__pyx_v_a1y - __pyx_v_a0y);

  /* "renpy/gl2/gl2geometry.pyx":309
 *     cdef int i
 *     cdef int j
 *     cdef bint allin = True             # <<<<<<<<<<<<<<
 * 
 *     # Figure out which points are 'inside' the wound line.
 */
  __pyx_v_allin = 1;

  /* "renpy/gl2/gl2geometry.pyx":312
 * 
 *     # Figure out which points are 'inside' the wound line.
 *     for 0 <= i < p.points:             # <<<<<<<<<<<<<<
 *         vecpx = get(p, i, X) - a0x
 *         vecpy = get(p, i, Y) - a0y
 */
  __pyx_t_1 = __pyx_v_p->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":313
 *     # Figure out which points are 'inside' the wound line.
 *     for 0 <= i < p.points:
 *         vecpx = get(p, i, X) - a0x             # <<<<<<<<<<<<<<
 *         vecpy = get(p, i, Y) - a0y
 * 
 */
    __pyx_v_vecpx = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 0) - __pyx_v_a0x);

    /* "renpy/gl2/gl2geometry.pyx":314
 *     for 0 <= i < p.points:
 *         vecpx = get(p, i, X) - a0x
 *         vecpy = get(p, i, Y) - a0y             # <<<<<<<<<<<<<<
 * 
 *         inside[i] = winding * (vecax * vecpy - vecay * vecpx) <= 0.00001
 */
    __pyx_t_2 = PyFloat_FromDouble((__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 1) - __pyx_v_a0y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_vecpy, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":316
 *         vecpy = get(p, i, Y) - a0y
 * 
 *         inside[i] = winding * (vecax * vecpy - vecay * vecpx) <= 0.00001             # <<<<<<<<<<<<<<
 *         allin = allin and inside[i]
 * 
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_winding); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_vecax); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_v_vecpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyFloat_FromDouble((__pyx_v_vecay * __pyx_v_vecpx)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyNumber_Subtract(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Multiply(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_float_0_00001, Py_LE); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    (__pyx_v_inside[__pyx_v_i]) = __pyx_t_6;

    /* "renpy/gl2/gl2geometry.pyx":317
 * 
 *         inside[i] = winding * (vecax * vecpy - vecay * vecpx) <= 0.00001
 *         allin = allin and inside[i]             # <<<<<<<<<<<<<<
 * 
 *     # If all the points are inside, just return the polygon intact.
 */
    __pyx_t_7 = (__pyx_v_allin != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_7 = ((__pyx_v_inside[__pyx_v_i]) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L5_bool_binop_done:;
    __pyx_v_allin = __pyx_t_6;
  }

  /* "renpy/gl2/gl2geometry.pyx":320
 * 
 *     # If all the points are inside, just return the polygon intact.
 *     if allin:             # <<<<<<<<<<<<<<
 *         return p
 * 
 */
  __pyx_t_6 = (__pyx_v_allin != 0);
  if (__pyx_t_6) {

    /* "renpy/gl2/gl2geometry.pyx":321
 *     # If all the points are inside, just return the polygon intact.
 *     if allin:
 *         return p             # <<<<<<<<<<<<<<
 * 
 *     rv = Polygon(rvstride, p.points * 2, None)
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_p));
    __pyx_r = __pyx_v_p;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":320
 * 
 *     # If all the points are inside, just return the polygon intact.
 *     if allin:             # <<<<<<<<<<<<<<
 *         return p
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":323
 *         return p
 * 
 *     rv = Polygon(rvstride, p.points * 2, None)             # <<<<<<<<<<<<<<
 * 
 *     j = p.points - 1
 */
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_rvstride); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_long((__pyx_v_p->points * 2)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "renpy/gl2/gl2geometry.pyx":325
 *     rv = Polygon(rvstride, p.points * 2, None)
 * 
 *     j = p.points - 1             # <<<<<<<<<<<<<<
 * 
 *     for 0 <= i < p.points:
 */
  __pyx_v_j = (__pyx_v_p->points - 1);

  /* "renpy/gl2/gl2geometry.pyx":327
 *     j = p.points - 1
 * 
 *     for 0 <= i < p.points:             # <<<<<<<<<<<<<<
 *         if inside[i]:
 *             if not inside[j]:
 */
  __pyx_t_1 = __pyx_v_p->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":328
 * 
 *     for 0 <= i < p.points:
 *         if inside[i]:             # <<<<<<<<<<<<<<
 *             if not inside[j]:
 *                 intersectLines(
 */
    __pyx_t_6 = ((__pyx_v_inside[__pyx_v_i]) != 0);
    if (__pyx_t_6) {

      /* "renpy/gl2/gl2geometry.pyx":329
 *     for 0 <= i < p.points:
 *         if inside[i]:
 *             if not inside[j]:             # <<<<<<<<<<<<<<
 *                 intersectLines(
 *                     a0x, a0y, a1x, a1y,
 */
      __pyx_t_6 = ((!((__pyx_v_inside[__pyx_v_j]) != 0)) != 0);
      if (__pyx_t_6) {

        /* "renpy/gl2/gl2geometry.pyx":330
 *         if inside[i]:
 *             if not inside[j]:
 *                 intersectLines(             # <<<<<<<<<<<<<<
 *                     a0x, a0y, a1x, a1y,
 *                     get(p, j, X), get(p, j, Y), get(p, i, X), get(p, i, Y),
 */
        __pyx_f_5renpy_3gl2_11gl2geometry_intersectLines(__pyx_v_a0x, __pyx_v_a0y, __pyx_v_a1x, __pyx_v_a1y, __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_j, 0), __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_j, 1), __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 0), __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 1), __pyx_f_5renpy_3gl2_11gl2geometry_ref(__pyx_v_rv, __pyx_v_rv->points, 0), __pyx_f_5renpy_3gl2_11gl2geometry_ref(__pyx_v_rv, __pyx_v_rv->points, 1));

        /* "renpy/gl2/gl2geometry.pyx":335
 *                     ref(rv, rv.points, X), ref(rv, rv.points, Y))
 * 
 *                 rv.points += 1             # <<<<<<<<<<<<<<
 * 
 *             set(rv, rv.points, X, get(p, i, X))
 */
        __pyx_v_rv->points = (__pyx_v_rv->points + 1);

        /* "renpy/gl2/gl2geometry.pyx":329
 *     for 0 <= i < p.points:
 *         if inside[i]:
 *             if not inside[j]:             # <<<<<<<<<<<<<<
 *                 intersectLines(
 *                     a0x, a0y, a1x, a1y,
 */
      }

      /* "renpy/gl2/gl2geometry.pyx":337
 *                 rv.points += 1
 * 
 *             set(rv, rv.points, X, get(p, i, X))             # <<<<<<<<<<<<<<
 *             set(rv, rv.points, Y, get(p, i, Y))
 *             rv.points += 1
 */
      __pyx_f_5renpy_3gl2_11gl2geometry_set(__pyx_v_rv, __pyx_v_rv->points, 0, __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 0));

      /* "renpy/gl2/gl2geometry.pyx":338
 * 
 *             set(rv, rv.points, X, get(p, i, X))
 *             set(rv, rv.points, Y, get(p, i, Y))             # <<<<<<<<<<<<<<
 *             rv.points += 1
 * 
 */
      __pyx_f_5renpy_3gl2_11gl2geometry_set(__pyx_v_rv, __pyx_v_rv->points, 1, __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 1));

      /* "renpy/gl2/gl2geometry.pyx":339
 *             set(rv, rv.points, X, get(p, i, X))
 *             set(rv, rv.points, Y, get(p, i, Y))
 *             rv.points += 1             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
      __pyx_v_rv->points = (__pyx_v_rv->points + 1);

      /* "renpy/gl2/gl2geometry.pyx":328
 * 
 *     for 0 <= i < p.points:
 *         if inside[i]:             # <<<<<<<<<<<<<<
 *             if not inside[j]:
 *                 intersectLines(
 */
      goto __pyx_L10;
    }

    /* "renpy/gl2/gl2geometry.pyx":342
 * 
 *         else:
 *             if inside[j]:             # <<<<<<<<<<<<<<
 *                 intersectLines(
 *                     a0x, a0y, a1x, a1y,
 */
    /*else*/ {
      __pyx_t_6 = ((__pyx_v_inside[__pyx_v_j]) != 0);
      if (__pyx_t_6) {

        /* "renpy/gl2/gl2geometry.pyx":343
 *         else:
 *             if inside[j]:
 *                 intersectLines(             # <<<<<<<<<<<<<<
 *                     a0x, a0y, a1x, a1y,
 *                     get(p, j, X), get(p, j, Y), get(p, i, X), get(p, i, Y),
 */
        __pyx_f_5renpy_3gl2_11gl2geometry_intersectLines(__pyx_v_a0x, __pyx_v_a0y, __pyx_v_a1x, __pyx_v_a1y, __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_j, 0), __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_j, 1), __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 0), __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_p, __pyx_v_i, 1), __pyx_f_5renpy_3gl2_11gl2geometry_ref(__pyx_v_rv, __pyx_v_rv->points, 0), __pyx_f_5renpy_3gl2_11gl2geometry_ref(__pyx_v_rv, __pyx_v_rv->points, 1));

        /* "renpy/gl2/gl2geometry.pyx":348
 *                     ref(rv, rv.points, X), ref(rv, rv.points, Y))
 * 
 *                 rv.points += 1             # <<<<<<<<<<<<<<
 * 
 *         j = i
 */
        __pyx_v_rv->points = (__pyx_v_rv->points + 1);

        /* "renpy/gl2/gl2geometry.pyx":342
 * 
 *         else:
 *             if inside[j]:             # <<<<<<<<<<<<<<
 *                 intersectLines(
 *                     a0x, a0y, a1x, a1y,
 */
      }
    }
    __pyx_L10:;

    /* "renpy/gl2/gl2geometry.pyx":350
 *                 rv.points += 1
 * 
 *         j = i             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
    __pyx_v_j = __pyx_v_i;
  }

  /* "renpy/gl2/gl2geometry.pyx":352
 *         j = i
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":294
 *     py[0] = ((x1 * y2 - y1 * x2) * (y3 - y4) - (y1 - y2) * (x3 * y4 - y3 * x4)) / denom
 * 
 * cdef Polygon intersectOnce(float winding, float a0x, float a0y, float a1x, float a1y, Polygon p, int rvstride):             # <<<<<<<<<<<<<<
 * 
 *     # The vector from a0 to a1.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.intersectOnce", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_vecpy);
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":355
 * 
 * 
 * cdef Polygon restride_polygon(Polygon src, int new_stride):             # <<<<<<<<<<<<<<
 *     """
 *     Returns a copy of the polygon with the stride changed to
 */

static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_f_5renpy_3gl2_11gl2geometry_restride_polygon(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_src, int __pyx_v_new_stride) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  float *__pyx_v_ap;
  float *__pyx_v_bp;
  CYTHON_UNUSED int __pyx_v_i;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  float *__pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("restride_polygon", 0);

  /* "renpy/gl2/gl2geometry.pyx":361
 *     """
 * 
 *     cdef Polygon rv = Polygon(new_stride, src.points, None)             # <<<<<<<<<<<<<<
 * 
 *     cdef float *ap = src.data
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_new_stride); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_src->points); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/gl2/gl2geometry.pyx":363
 *     cdef Polygon rv = Polygon(new_stride, src.points, None)
 * 
 *     cdef float *ap = src.data             # <<<<<<<<<<<<<<
 *     cdef float *bp = rv.data
 * 
 */
  __pyx_t_4 = __pyx_v_src->data;
  __pyx_v_ap = __pyx_t_4;

  /* "renpy/gl2/gl2geometry.pyx":364
 * 
 *     cdef float *ap = src.data
 *     cdef float *bp = rv.data             # <<<<<<<<<<<<<<
 * 
 *     cdef int i
 */
  __pyx_t_4 = __pyx_v_rv->data;
  __pyx_v_bp = __pyx_t_4;

  /* "renpy/gl2/gl2geometry.pyx":368
 *     cdef int i
 * 
 *     for 0 <= i < src.points:             # <<<<<<<<<<<<<<
 *         bp[X] = ap[X]
 *         bp[Y] = ap[Y]
 */
  __pyx_t_5 = __pyx_v_src->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_5; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":369
 * 
 *     for 0 <= i < src.points:
 *         bp[X] = ap[X]             # <<<<<<<<<<<<<<
 *         bp[Y] = ap[Y]
 *         bp[Z] = ap[Z]
 */
    (__pyx_v_bp[0]) = (__pyx_v_ap[0]);

    /* "renpy/gl2/gl2geometry.pyx":370
 *     for 0 <= i < src.points:
 *         bp[X] = ap[X]
 *         bp[Y] = ap[Y]             # <<<<<<<<<<<<<<
 *         bp[Z] = ap[Z]
 * 
 */
    (__pyx_v_bp[1]) = (__pyx_v_ap[1]);

    /* "renpy/gl2/gl2geometry.pyx":371
 *         bp[X] = ap[X]
 *         bp[Y] = ap[Y]
 *         bp[Z] = ap[Z]             # <<<<<<<<<<<<<<
 * 
 *         ap += src.stride
 */
    (__pyx_v_bp[2]) = (__pyx_v_ap[2]);

    /* "renpy/gl2/gl2geometry.pyx":373
 *         bp[Z] = ap[Z]
 * 
 *         ap += src.stride             # <<<<<<<<<<<<<<
 *         bp += rv.stride
 * 
 */
    __pyx_v_ap = (__pyx_v_ap + __pyx_v_src->stride);

    /* "renpy/gl2/gl2geometry.pyx":374
 * 
 *         ap += src.stride
 *         bp += rv.stride             # <<<<<<<<<<<<<<
 * 
 *     rv.points = src.points
 */
    __pyx_v_bp = (__pyx_v_bp + __pyx_v_rv->stride);
  }

  /* "renpy/gl2/gl2geometry.pyx":376
 *         bp += rv.stride
 * 
 *     rv.points = src.points             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
  __pyx_t_5 = __pyx_v_src->points;
  __pyx_v_rv->points = __pyx_t_5;

  /* "renpy/gl2/gl2geometry.pyx":378
 *     rv.points = src.points
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":355
 * 
 * 
 * cdef Polygon restride_polygon(Polygon src, int new_stride):             # <<<<<<<<<<<<<<
 *     """
 *     Returns a copy of the polygon with the stride changed to
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.restride_polygon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":381
 * 
 * 
 * cpdef intersect(Polygon a, Polygon b, int new_stride, bint copy):             # <<<<<<<<<<<<<<
 *     """
 *     Given two Polygons, returns a new Polygon that is the intersecti of
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_5intersect(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5renpy_3gl2_11gl2geometry_intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b, int __pyx_v_new_stride, int __pyx_v_copy, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_i;
  float __pyx_v_a0x;
  float __pyx_v_a0y;
  float __pyx_v_a1x;
  float __pyx_v_a1y;
  float __pyx_v_winding;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_rv = 0;
  PyObject *__pyx_v_a2x = NULL;
  PyObject *__pyx_v_a2y = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  float __pyx_t_6;
  int __pyx_t_7;
  float __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("intersect", 0);

  /* "renpy/gl2/gl2geometry.pyx":394
 *     cdef float winding
 * 
 *     a0x = get(a, a.points-2, X)             # <<<<<<<<<<<<<<
 *     a0y = get(a, a.points-2, Y)
 * 
 */
  __pyx_v_a0x = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, (__pyx_v_a->points - 2), 0);

  /* "renpy/gl2/gl2geometry.pyx":395
 * 
 *     a0x = get(a, a.points-2, X)
 *     a0y = get(a, a.points-2, Y)             # <<<<<<<<<<<<<<
 * 
 *     a1x = get(a, a.points-1, X)
 */
  __pyx_v_a0y = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, (__pyx_v_a->points - 2), 1);

  /* "renpy/gl2/gl2geometry.pyx":397
 *     a0y = get(a, a.points-2, Y)
 * 
 *     a1x = get(a, a.points-1, X)             # <<<<<<<<<<<<<<
 *     a1y = get(a, a.points-1, Y)
 * 
 */
  __pyx_v_a1x = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, (__pyx_v_a->points - 1), 0);

  /* "renpy/gl2/gl2geometry.pyx":398
 * 
 *     a1x = get(a, a.points-1, X)
 *     a1y = get(a, a.points-1, Y)             # <<<<<<<<<<<<<<
 * 
 *     cdef Polygon rv = b
 */
  __pyx_v_a1y = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, (__pyx_v_a->points - 1), 1);

  /* "renpy/gl2/gl2geometry.pyx":400
 *     a1y = get(a, a.points-1, Y)
 * 
 *     cdef Polygon rv = b             # <<<<<<<<<<<<<<
 * 
 *     for 0 <= i < a.points:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_b));
  __pyx_v_rv = __pyx_v_b;

  /* "renpy/gl2/gl2geometry.pyx":402
 *     cdef Polygon rv = b
 * 
 *     for 0 <= i < a.points:             # <<<<<<<<<<<<<<
 *         a2x = get(a, i, X)
 *         a2y = get(a, i, Y)
 */
  __pyx_t_1 = __pyx_v_a->points;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":403
 * 
 *     for 0 <= i < a.points:
 *         a2x = get(a, i, X)             # <<<<<<<<<<<<<<
 *         a2y = get(a, i, Y)
 * 
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 403, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_a2x, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":404
 *     for 0 <= i < a.points:
 *         a2x = get(a, i, X)
 *         a2y = get(a, i, Y)             # <<<<<<<<<<<<<<
 * 
 *         winding = (a2x - a0x) * (a1y - a0y) - (a2y - a0y) * (a1x - a0x)
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, 1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_a2y, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":406
 *         a2y = get(a, i, Y)
 * 
 *         winding = (a2x - a0x) * (a1y - a0y) - (a2y - a0y) * (a1x - a0x)             # <<<<<<<<<<<<<<
 *         winding = copysign(1.0, winding)
 * 
 */
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_a0x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_a2x, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyFloat_FromDouble((__pyx_v_a1y - __pyx_v_a0y)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_a0y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_a2y, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyFloat_FromDouble((__pyx_v_a1x - __pyx_v_a0x)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyNumber_Multiply(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyNumber_Subtract(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 406, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_winding = __pyx_t_6;

    /* "renpy/gl2/gl2geometry.pyx":407
 * 
 *         winding = (a2x - a0x) * (a1y - a0y) - (a2y - a0y) * (a1x - a0x)
 *         winding = copysign(1.0, winding)             # <<<<<<<<<<<<<<
 * 
 *         if winding:
 */
    __pyx_v_winding = copysign(1.0, __pyx_v_winding);

    /* "renpy/gl2/gl2geometry.pyx":409
 *         winding = copysign(1.0, winding)
 * 
 *         if winding:             # <<<<<<<<<<<<<<
 *             rv = intersectOnce(winding, a1x, a1y, a2x, a2y, rv, new_stride)
 * 
 */
    __pyx_t_7 = (__pyx_v_winding != 0);
    if (__pyx_t_7) {

      /* "renpy/gl2/gl2geometry.pyx":410
 * 
 *         if winding:
 *             rv = intersectOnce(winding, a1x, a1y, a2x, a2y, rv, new_stride)             # <<<<<<<<<<<<<<
 * 
 *             if rv.points < 3:
 */
      __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_v_a2x); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L1_error)
      __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_v_a2y); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L1_error)
      __pyx_t_2 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_intersectOnce(__pyx_v_winding, __pyx_v_a1x, __pyx_v_a1y, __pyx_t_6, __pyx_t_8, __pyx_v_rv, __pyx_v_new_stride)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 410, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_rv, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "renpy/gl2/gl2geometry.pyx":412
 *             rv = intersectOnce(winding, a1x, a1y, a2x, a2y, rv, new_stride)
 * 
 *             if rv.points < 3:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
      __pyx_t_7 = ((__pyx_v_rv->points < 3) != 0);
      if (__pyx_t_7) {

        /* "renpy/gl2/gl2geometry.pyx":413
 * 
 *             if rv.points < 3:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         a0x = a1x
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(Py_None);
        __pyx_r = Py_None;
        goto __pyx_L0;

        /* "renpy/gl2/gl2geometry.pyx":412
 *             rv = intersectOnce(winding, a1x, a1y, a2x, a2y, rv, new_stride)
 * 
 *             if rv.points < 3:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
      }

      /* "renpy/gl2/gl2geometry.pyx":409
 *         winding = copysign(1.0, winding)
 * 
 *         if winding:             # <<<<<<<<<<<<<<
 *             rv = intersectOnce(winding, a1x, a1y, a2x, a2y, rv, new_stride)
 * 
 */
    }

    /* "renpy/gl2/gl2geometry.pyx":415
 *                 return None
 * 
 *         a0x = a1x             # <<<<<<<<<<<<<<
 *         a0y = a1y
 * 
 */
    __pyx_v_a0x = __pyx_v_a1x;

    /* "renpy/gl2/gl2geometry.pyx":416
 * 
 *         a0x = a1x
 *         a0y = a1y             # <<<<<<<<<<<<<<
 * 
 *         a1x = a2x
 */
    __pyx_v_a0y = __pyx_v_a1y;

    /* "renpy/gl2/gl2geometry.pyx":418
 *         a0y = a1y
 * 
 *         a1x = a2x             # <<<<<<<<<<<<<<
 *         a1y = a2y
 * 
 */
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_v_a2x); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 418, __pyx_L1_error)
    __pyx_v_a1x = __pyx_t_8;

    /* "renpy/gl2/gl2geometry.pyx":419
 * 
 *         a1x = a2x
 *         a1y = a2y             # <<<<<<<<<<<<<<
 * 
 *     if rv is b:
 */
    __pyx_t_8 = __pyx_PyFloat_AsFloat(__pyx_v_a2y); if (unlikely((__pyx_t_8 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 419, __pyx_L1_error)
    __pyx_v_a1y = __pyx_t_8;
  }

  /* "renpy/gl2/gl2geometry.pyx":421
 *         a1y = a2y
 * 
 *     if rv is b:             # <<<<<<<<<<<<<<
 *         if (not copy) and (new_stride == rv.stride):
 *             return rv
 */
  __pyx_t_7 = (__pyx_v_rv == __pyx_v_b);
  __pyx_t_9 = (__pyx_t_7 != 0);
  if (__pyx_t_9) {

    /* "renpy/gl2/gl2geometry.pyx":422
 * 
 *     if rv is b:
 *         if (not copy) and (new_stride == rv.stride):             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
    __pyx_t_7 = ((!(__pyx_v_copy != 0)) != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_9 = __pyx_t_7;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_7 = ((__pyx_v_new_stride == __pyx_v_rv->stride) != 0);
    __pyx_t_9 = __pyx_t_7;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_9) {

      /* "renpy/gl2/gl2geometry.pyx":423
 *     if rv is b:
 *         if (not copy) and (new_stride == rv.stride):
 *             return rv             # <<<<<<<<<<<<<<
 * 
 *         rv = restride_polygon(rv, new_stride)
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(((PyObject *)__pyx_v_rv));
      __pyx_r = ((PyObject *)__pyx_v_rv);
      goto __pyx_L0;

      /* "renpy/gl2/gl2geometry.pyx":422
 * 
 *     if rv is b:
 *         if (not copy) and (new_stride == rv.stride):             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
    }

    /* "renpy/gl2/gl2geometry.pyx":425
 *             return rv
 * 
 *         rv = restride_polygon(rv, new_stride)             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
    __pyx_t_2 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_restride_polygon(__pyx_v_rv, __pyx_v_new_stride)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_rv, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":421
 *         a1y = a2y
 * 
 *     if rv is b:             # <<<<<<<<<<<<<<
 *         if (not copy) and (new_stride == rv.stride):
 *             return rv
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":427
 *         rv = restride_polygon(rv, new_stride)
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = ((PyObject *)__pyx_v_rv);
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":381
 * 
 * 
 * cpdef intersect(Polygon a, Polygon b, int new_stride, bint copy):             # <<<<<<<<<<<<<<
 *     """
 *     Given two Polygons, returns a new Polygon that is the intersecti of
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_a2x);
  __Pyx_XDECREF(__pyx_v_a2y);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_5intersect(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4intersect[] = "\n    Given two Polygons, returns a new Polygon that is the intersecti of\n    the two. This assumes that both polygons are convex, and wound in the\n    same direction (clockwise or counterclockwise).\n\n    `new_stride` is the stride of the new Polygon.\n    ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_5intersect(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b = 0;
  int __pyx_v_new_stride;
  int __pyx_v_copy;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("intersect (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,&__pyx_n_s_new_stride,&__pyx_n_s_copy,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("intersect", 1, 4, 4, 1); __PYX_ERR(0, 381, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_new_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("intersect", 1, 4, 4, 2); __PYX_ERR(0, 381, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_copy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("intersect", 1, 4, 4, 3); __PYX_ERR(0, 381, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "intersect") < 0)) __PYX_ERR(0, 381, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_a = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)values[1]);
    __pyx_v_new_stride = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_new_stride == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 381, __pyx_L3_error)
    __pyx_v_copy = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_copy == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 381, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("intersect", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 381, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "a", 0))) __PYX_ERR(0, 381, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "b", 0))) __PYX_ERR(0, 381, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4intersect(__pyx_self, __pyx_v_a, __pyx_v_b, __pyx_v_new_stride, __pyx_v_copy);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4intersect(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b, int __pyx_v_new_stride, int __pyx_v_copy) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("intersect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5renpy_3gl2_11gl2geometry_intersect(__pyx_v_a, __pyx_v_b, __pyx_v_new_stride, __pyx_v_copy, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":430
 * 
 * 
 * cpdef barycentric(Polygon a, Polygon b, int offset):             # <<<<<<<<<<<<<<
 *     """
 *     This uses barycentric interpolation to transfer the attributes from
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7barycentric(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5renpy_3gl2_11gl2geometry_barycentric(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b, int __pyx_v_offset, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_k;
  float __pyx_v_ax0;
  float __pyx_v_ay0;
  float __pyx_v_az0;
  float __pyx_v_v0x;
  float __pyx_v_v0y;
  float __pyx_v_v0z;
  float __pyx_v_v1x;
  float __pyx_v_v1y;
  float __pyx_v_v1z;
  float __pyx_v_v2x;
  float __pyx_v_v2y;
  float __pyx_v_v2z;
  float __pyx_v_d00;
  float __pyx_v_d01;
  float __pyx_v_d11;
  float __pyx_v_d20;
  float __pyx_v_d21;
  float __pyx_v_d003;
  float __pyx_v_d013;
  float __pyx_v_d113;
  float __pyx_v_d203;
  float __pyx_v_d213;
  float __pyx_v_denom;
  float __pyx_v_u;
  float __pyx_v_v;
  float __pyx_v_w;
  PyObject *__pyx_v_denom3 = NULL;
  PyObject *__pyx_v_z = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  float __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("barycentric", 0);

  /* "renpy/gl2/gl2geometry.pyx":440
 *     cdef int k
 * 
 *     cdef float ax0 = get(a, 0, X)             # <<<<<<<<<<<<<<
 *     cdef float ay0 = get(a, 0, Y)
 *     cdef float az0 = get(a, 0, Z)
 */
  __pyx_v_ax0 = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 0, 0);

  /* "renpy/gl2/gl2geometry.pyx":441
 * 
 *     cdef float ax0 = get(a, 0, X)
 *     cdef float ay0 = get(a, 0, Y)             # <<<<<<<<<<<<<<
 *     cdef float az0 = get(a, 0, Z)
 * 
 */
  __pyx_v_ay0 = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 0, 1);

  /* "renpy/gl2/gl2geometry.pyx":442
 *     cdef float ax0 = get(a, 0, X)
 *     cdef float ay0 = get(a, 0, Y)
 *     cdef float az0 = get(a, 0, Z)             # <<<<<<<<<<<<<<
 * 
 *     cdef float v0x = get(a, 1, X) - ax0
 */
  __pyx_v_az0 = __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 0, 2);

  /* "renpy/gl2/gl2geometry.pyx":444
 *     cdef float az0 = get(a, 0, Z)
 * 
 *     cdef float v0x = get(a, 1, X) - ax0             # <<<<<<<<<<<<<<
 *     cdef float v0y = get(a, 1, Y) - ay0
 *     cdef float v0z = get(a, 1, Z) - az0
 */
  __pyx_v_v0x = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 1, 0) - __pyx_v_ax0);

  /* "renpy/gl2/gl2geometry.pyx":445
 * 
 *     cdef float v0x = get(a, 1, X) - ax0
 *     cdef float v0y = get(a, 1, Y) - ay0             # <<<<<<<<<<<<<<
 *     cdef float v0z = get(a, 1, Z) - az0
 * 
 */
  __pyx_v_v0y = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 1, 1) - __pyx_v_ay0);

  /* "renpy/gl2/gl2geometry.pyx":446
 *     cdef float v0x = get(a, 1, X) - ax0
 *     cdef float v0y = get(a, 1, Y) - ay0
 *     cdef float v0z = get(a, 1, Z) - az0             # <<<<<<<<<<<<<<
 * 
 *     cdef float v1x, v1y, v1z, v2x, v2y, v2z
 */
  __pyx_v_v0z = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 1, 2) - __pyx_v_az0);

  /* "renpy/gl2/gl2geometry.pyx":454
 *     cdef float u, v, w
 * 
 *     for 2 <= i < a.points:             # <<<<<<<<<<<<<<
 * 
 *         v1x = get(a, i, X) - ax0
 */
  __pyx_t_1 = __pyx_v_a->points;
  for (__pyx_v_i = 2; __pyx_v_i < __pyx_t_1; __pyx_v_i++) {

    /* "renpy/gl2/gl2geometry.pyx":456
 *     for 2 <= i < a.points:
 * 
 *         v1x = get(a, i, X) - ax0             # <<<<<<<<<<<<<<
 *         v1y = get(a, i, Y) - ay0
 *         v1z = get(a, i, Z) - az0
 */
    __pyx_v_v1x = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, 0) - __pyx_v_ax0);

    /* "renpy/gl2/gl2geometry.pyx":457
 * 
 *         v1x = get(a, i, X) - ax0
 *         v1y = get(a, i, Y) - ay0             # <<<<<<<<<<<<<<
 *         v1z = get(a, i, Z) - az0
 * 
 */
    __pyx_v_v1y = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, 1) - __pyx_v_ay0);

    /* "renpy/gl2/gl2geometry.pyx":458
 *         v1x = get(a, i, X) - ax0
 *         v1y = get(a, i, Y) - ay0
 *         v1z = get(a, i, Z) - az0             # <<<<<<<<<<<<<<
 * 
 *         d00 = v0x * v0x + v0y * v0y
 */
    __pyx_v_v1z = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, 2) - __pyx_v_az0);

    /* "renpy/gl2/gl2geometry.pyx":460
 *         v1z = get(a, i, Z) - az0
 * 
 *         d00 = v0x * v0x + v0y * v0y             # <<<<<<<<<<<<<<
 *         d01 = v0x * v1x + v0y * v1y
 *         d11 = v1x * v1x + v1y * v1y
 */
    __pyx_v_d00 = ((__pyx_v_v0x * __pyx_v_v0x) + (__pyx_v_v0y * __pyx_v_v0y));

    /* "renpy/gl2/gl2geometry.pyx":461
 * 
 *         d00 = v0x * v0x + v0y * v0y
 *         d01 = v0x * v1x + v0y * v1y             # <<<<<<<<<<<<<<
 *         d11 = v1x * v1x + v1y * v1y
 * 
 */
    __pyx_v_d01 = ((__pyx_v_v0x * __pyx_v_v1x) + (__pyx_v_v0y * __pyx_v_v1y));

    /* "renpy/gl2/gl2geometry.pyx":462
 *         d00 = v0x * v0x + v0y * v0y
 *         d01 = v0x * v1x + v0y * v1y
 *         d11 = v1x * v1x + v1y * v1y             # <<<<<<<<<<<<<<
 * 
 *         d003 = d00 + v0z * v0z
 */
    __pyx_v_d11 = ((__pyx_v_v1x * __pyx_v_v1x) + (__pyx_v_v1y * __pyx_v_v1y));

    /* "renpy/gl2/gl2geometry.pyx":464
 *         d11 = v1x * v1x + v1y * v1y
 * 
 *         d003 = d00 + v0z * v0z             # <<<<<<<<<<<<<<
 *         d013 = d01 + v0z * v1z
 *         d113 = d11 + v1z * v1z
 */
    __pyx_v_d003 = (__pyx_v_d00 + (__pyx_v_v0z * __pyx_v_v0z));

    /* "renpy/gl2/gl2geometry.pyx":465
 * 
 *         d003 = d00 + v0z * v0z
 *         d013 = d01 + v0z * v1z             # <<<<<<<<<<<<<<
 *         d113 = d11 + v1z * v1z
 * 
 */
    __pyx_v_d013 = (__pyx_v_d01 + (__pyx_v_v0z * __pyx_v_v1z));

    /* "renpy/gl2/gl2geometry.pyx":466
 *         d003 = d00 + v0z * v0z
 *         d013 = d01 + v0z * v1z
 *         d113 = d11 + v1z * v1z             # <<<<<<<<<<<<<<
 * 
 *         denom = d00 * d11 - d01 * d01
 */
    __pyx_v_d113 = (__pyx_v_d11 + (__pyx_v_v1z * __pyx_v_v1z));

    /* "renpy/gl2/gl2geometry.pyx":468
 *         d113 = d11 + v1z * v1z
 * 
 *         denom = d00 * d11 - d01 * d01             # <<<<<<<<<<<<<<
 *         denom3 = d003 * d113 - d013 * d013
 * 
 */
    __pyx_v_denom = ((__pyx_v_d00 * __pyx_v_d11) - (__pyx_v_d01 * __pyx_v_d01));

    /* "renpy/gl2/gl2geometry.pyx":469
 * 
 *         denom = d00 * d11 - d01 * d01
 *         denom3 = d003 * d113 - d013 * d013             # <<<<<<<<<<<<<<
 * 
 *         if denom and denom3:
 */
    __pyx_t_2 = PyFloat_FromDouble(((__pyx_v_d003 * __pyx_v_d113) - (__pyx_v_d013 * __pyx_v_d013))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_denom3, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":471
 *         denom3 = d003 * d113 - d013 * d013
 * 
 *         if denom and denom3:             # <<<<<<<<<<<<<<
 * 
 *             for 0 <= j < b.points:
 */
    __pyx_t_4 = (__pyx_v_denom != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_3 = __pyx_t_4;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_denom3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 471, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_4;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_3) {

      /* "renpy/gl2/gl2geometry.pyx":473
 *         if denom and denom3:
 * 
 *             for 0 <= j < b.points:             # <<<<<<<<<<<<<<
 * 
 *                 v2x = get(b, j, X) - ax0
 */
      __pyx_t_5 = __pyx_v_b->points;
      for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_5; __pyx_v_j++) {

        /* "renpy/gl2/gl2geometry.pyx":475
 *             for 0 <= j < b.points:
 * 
 *                 v2x = get(b, j, X) - ax0             # <<<<<<<<<<<<<<
 *                 v2y = get(b, j, Y) - ay0
 * 
 */
        __pyx_v_v2x = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_b, __pyx_v_j, 0) - __pyx_v_ax0);

        /* "renpy/gl2/gl2geometry.pyx":476
 * 
 *                 v2x = get(b, j, X) - ax0
 *                 v2y = get(b, j, Y) - ay0             # <<<<<<<<<<<<<<
 * 
 *                 d20 = v2x * v0x + v2y * v0y
 */
        __pyx_v_v2y = (__pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_b, __pyx_v_j, 1) - __pyx_v_ay0);

        /* "renpy/gl2/gl2geometry.pyx":478
 *                 v2y = get(b, j, Y) - ay0
 * 
 *                 d20 = v2x * v0x + v2y * v0y             # <<<<<<<<<<<<<<
 *                 d21 = v2x * v1x + v2y * v1y
 * 
 */
        __pyx_v_d20 = ((__pyx_v_v2x * __pyx_v_v0x) + (__pyx_v_v2y * __pyx_v_v0y));

        /* "renpy/gl2/gl2geometry.pyx":479
 * 
 *                 d20 = v2x * v0x + v2y * v0y
 *                 d21 = v2x * v1x + v2y * v1y             # <<<<<<<<<<<<<<
 * 
 *                 v = (d11 * d20 - d01 * d21) / denom
 */
        __pyx_v_d21 = ((__pyx_v_v2x * __pyx_v_v1x) + (__pyx_v_v2y * __pyx_v_v1y));

        /* "renpy/gl2/gl2geometry.pyx":481
 *                 d21 = v2x * v1x + v2y * v1y
 * 
 *                 v = (d11 * d20 - d01 * d21) / denom             # <<<<<<<<<<<<<<
 *                 w = (d00 * d21 - d01 * d20) / denom
 * 
 */
        __pyx_t_6 = ((__pyx_v_d11 * __pyx_v_d20) - (__pyx_v_d01 * __pyx_v_d21));
        if (unlikely(__pyx_v_denom == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 481, __pyx_L1_error)
        }
        __pyx_v_v = (__pyx_t_6 / __pyx_v_denom);

        /* "renpy/gl2/gl2geometry.pyx":482
 * 
 *                 v = (d11 * d20 - d01 * d21) / denom
 *                 w = (d00 * d21 - d01 * d20) / denom             # <<<<<<<<<<<<<<
 * 
 *                 if not ((-0.00001 <= v <= 1.00001) and (-0.00001 <= w <= 1.00001)):
 */
        __pyx_t_6 = ((__pyx_v_d00 * __pyx_v_d21) - (__pyx_v_d01 * __pyx_v_d20));
        if (unlikely(__pyx_v_denom == 0)) {
          PyErr_SetString(PyExc_ZeroDivisionError, "float division");
          __PYX_ERR(0, 482, __pyx_L1_error)
        }
        __pyx_v_w = (__pyx_t_6 / __pyx_v_denom);

        /* "renpy/gl2/gl2geometry.pyx":484
 *                 w = (d00 * d21 - d01 * d20) / denom
 * 
 *                 if not ((-0.00001 <= v <= 1.00001) and (-0.00001 <= w <= 1.00001)):             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
        __pyx_t_4 = (-0.00001 <= __pyx_v_v);
        if (__pyx_t_4) {
          __pyx_t_4 = (__pyx_v_v <= 1.00001);
        }
        __pyx_t_7 = (__pyx_t_4 != 0);
        if (__pyx_t_7) {
        } else {
          __pyx_t_3 = __pyx_t_7;
          goto __pyx_L11_bool_binop_done;
        }
        __pyx_t_7 = (-0.00001 <= __pyx_v_w);
        if (__pyx_t_7) {
          __pyx_t_7 = (__pyx_v_w <= 1.00001);
        }
        __pyx_t_4 = (__pyx_t_7 != 0);
        __pyx_t_3 = __pyx_t_4;
        __pyx_L11_bool_binop_done:;
        __pyx_t_4 = ((!__pyx_t_3) != 0);
        if (__pyx_t_4) {

          /* "renpy/gl2/gl2geometry.pyx":485
 * 
 *                 if not ((-0.00001 <= v <= 1.00001) and (-0.00001 <= w <= 1.00001)):
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 u = 1.0 - v - w
 */
          goto __pyx_L8_continue;

          /* "renpy/gl2/gl2geometry.pyx":484
 *                 w = (d00 * d21 - d01 * d20) / denom
 * 
 *                 if not ((-0.00001 <= v <= 1.00001) and (-0.00001 <= w <= 1.00001)):             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
        }

        /* "renpy/gl2/gl2geometry.pyx":487
 *                     continue
 * 
 *                 u = 1.0 - v - w             # <<<<<<<<<<<<<<
 * 
 *                 z = u * az0 + v * get(a, i-1, Z) + w * get(a, i, Z)
 */
        __pyx_v_u = ((1.0 - __pyx_v_v) - __pyx_v_w);

        /* "renpy/gl2/gl2geometry.pyx":489
 *                 u = 1.0 - v - w
 * 
 *                 z = u * az0 + v * get(a, i-1, Z) + w * get(a, i, Z)             # <<<<<<<<<<<<<<
 *                 set(b, j, Z, z)
 *                 set(b, j, W, 1.0)
 */
        __pyx_t_2 = PyFloat_FromDouble((((__pyx_v_u * __pyx_v_az0) + (__pyx_v_v * __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, (__pyx_v_i - 1), 2))) + (__pyx_v_w * __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, 2)))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_z, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "renpy/gl2/gl2geometry.pyx":490
 * 
 *                 z = u * az0 + v * get(a, i-1, Z) + w * get(a, i, Z)
 *                 set(b, j, Z, z)             # <<<<<<<<<<<<<<
 *                 set(b, j, W, 1.0)
 * 
 */
        __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_v_z); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 490, __pyx_L1_error)
        __pyx_f_5renpy_3gl2_11gl2geometry_set(__pyx_v_b, __pyx_v_j, 2, __pyx_t_6);

        /* "renpy/gl2/gl2geometry.pyx":491
 *                 z = u * az0 + v * get(a, i-1, Z) + w * get(a, i, Z)
 *                 set(b, j, Z, z)
 *                 set(b, j, W, 1.0)             # <<<<<<<<<<<<<<
 * 
 *                 v2z = z - az0
 */
        __pyx_f_5renpy_3gl2_11gl2geometry_set(__pyx_v_b, __pyx_v_j, 3, 1.0);

        /* "renpy/gl2/gl2geometry.pyx":493
 *                 set(b, j, W, 1.0)
 * 
 *                 v2z = z - az0             # <<<<<<<<<<<<<<
 * 
 *                 d203 = d20 + v2z * v0z
 */
        __pyx_t_2 = PyFloat_FromDouble(__pyx_v_az0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 493, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_8 = PyNumber_Subtract(__pyx_v_z, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 493, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_8); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 493, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_v2z = __pyx_t_6;

        /* "renpy/gl2/gl2geometry.pyx":495
 *                 v2z = z - az0
 * 
 *                 d203 = d20 + v2z * v0z             # <<<<<<<<<<<<<<
 *                 d213 = d21 + v2z * v1z
 * 
 */
        __pyx_v_d203 = (__pyx_v_d20 + (__pyx_v_v2z * __pyx_v_v0z));

        /* "renpy/gl2/gl2geometry.pyx":496
 * 
 *                 d203 = d20 + v2z * v0z
 *                 d213 = d21 + v2z * v1z             # <<<<<<<<<<<<<<
 * 
 *                 v = (d113 * d203 - d013 * d213) / denom3
 */
        __pyx_v_d213 = (__pyx_v_d21 + (__pyx_v_v2z * __pyx_v_v1z));

        /* "renpy/gl2/gl2geometry.pyx":498
 *                 d213 = d21 + v2z * v1z
 * 
 *                 v = (d113 * d203 - d013 * d213) / denom3             # <<<<<<<<<<<<<<
 *                 w = (d003 * d213 - d013 * d203) / denom3
 *                 u = 1.0 - v - w
 */
        __pyx_t_8 = PyFloat_FromDouble(((__pyx_v_d113 * __pyx_v_d203) - (__pyx_v_d013 * __pyx_v_d213))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 498, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_8, __pyx_v_denom3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 498, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_2); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 498, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_v_v = __pyx_t_6;

        /* "renpy/gl2/gl2geometry.pyx":499
 * 
 *                 v = (d113 * d203 - d013 * d213) / denom3
 *                 w = (d003 * d213 - d013 * d203) / denom3             # <<<<<<<<<<<<<<
 *                 u = 1.0 - v - w
 * 
 */
        __pyx_t_2 = PyFloat_FromDouble(((__pyx_v_d003 * __pyx_v_d213) - (__pyx_v_d013 * __pyx_v_d203))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_8 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_v_denom3); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_6 = __pyx_PyFloat_AsFloat(__pyx_t_8); if (unlikely((__pyx_t_6 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 499, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_w = __pyx_t_6;

        /* "renpy/gl2/gl2geometry.pyx":500
 *                 v = (d113 * d203 - d013 * d213) / denom3
 *                 w = (d003 * d213 - d013 * d203) / denom3
 *                 u = 1.0 - v - w             # <<<<<<<<<<<<<<
 * 
 *                 for 3 <= k < a.stride:
 */
        __pyx_v_u = ((1.0 - __pyx_v_v) - __pyx_v_w);

        /* "renpy/gl2/gl2geometry.pyx":502
 *                 u = 1.0 - v - w
 * 
 *                 for 3 <= k < a.stride:             # <<<<<<<<<<<<<<
 *                     set(b, j, k + offset,
 *                         u * get(a, 0, k) +
 */
        __pyx_t_9 = __pyx_v_a->stride;
        for (__pyx_v_k = 3; __pyx_v_k < __pyx_t_9; __pyx_v_k++) {

          /* "renpy/gl2/gl2geometry.pyx":503
 * 
 *                 for 3 <= k < a.stride:
 *                     set(b, j, k + offset,             # <<<<<<<<<<<<<<
 *                         u * get(a, 0, k) +
 *                         v * get(a, i-1, k) +
 */
          __pyx_f_5renpy_3gl2_11gl2geometry_set(__pyx_v_b, __pyx_v_j, (__pyx_v_k + __pyx_v_offset), (((__pyx_v_u * __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, 0, __pyx_v_k)) + (__pyx_v_v * __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, (__pyx_v_i - 1), __pyx_v_k))) + (__pyx_v_w * __pyx_f_5renpy_3gl2_11gl2geometry_get(__pyx_v_a, __pyx_v_i, __pyx_v_k))));
        }
        __pyx_L8_continue:;
      }

      /* "renpy/gl2/gl2geometry.pyx":471
 *         denom3 = d003 * d113 - d013 * d013
 * 
 *         if denom and denom3:             # <<<<<<<<<<<<<<
 * 
 *             for 0 <= j < b.points:
 */
    }

    /* "renpy/gl2/gl2geometry.pyx":508
 *                         w * get(a, i, k))
 * 
 *         v0x = v1x             # <<<<<<<<<<<<<<
 *         v0y = v1y
 * 
 */
    __pyx_v_v0x = __pyx_v_v1x;

    /* "renpy/gl2/gl2geometry.pyx":509
 * 
 *         v0x = v1x
 *         v0y = v1y             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_v0y = __pyx_v_v1y;
  }

  /* "renpy/gl2/gl2geometry.pyx":430
 * 
 * 
 * cpdef barycentric(Polygon a, Polygon b, int offset):             # <<<<<<<<<<<<<<
 *     """
 *     This uses barycentric interpolation to transfer the attributes from
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.barycentric", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_denom3);
  __Pyx_XDECREF(__pyx_v_z);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7barycentric(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_6barycentric[] = "\n    This uses barycentric interpolation to transfer the attributes from\n    Polygon `a` to Polygon `b`, starting at `offset`.\n    ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_7barycentric(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b = 0;
  int __pyx_v_offset;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("barycentric (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_b,&__pyx_n_s_offset,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("barycentric", 1, 3, 3, 1); __PYX_ERR(0, 430, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_offset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("barycentric", 1, 3, 3, 2); __PYX_ERR(0, 430, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "barycentric") < 0)) __PYX_ERR(0, 430, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_a = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)values[0]);
    __pyx_v_b = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)values[1]);
    __pyx_v_offset = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_offset == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 430, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("barycentric", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 430, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.barycentric", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "a", 0))) __PYX_ERR(0, 430, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "b", 0))) __PYX_ERR(0, 430, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_6barycentric(__pyx_self, __pyx_v_a, __pyx_v_b, __pyx_v_offset);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_6barycentric(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b, int __pyx_v_offset) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("barycentric", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_a, __pyx_v_b, __pyx_v_offset, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.barycentric", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":512
 * 
 * 
 * def barycentric_point(Polygon a, float x, float y):             # <<<<<<<<<<<<<<
 *     """
 *     This expects Polygon a to be a polygon with a stride of 6, with a
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_9barycentric_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_8barycentric_point[] = "\n    This expects Polygon a to be a polygon with a stride of 6, with a\n    position (x, y, z, 1) and a pair of attributes (tx, ty) at each\n    point.\n\n    Given `x` and `y`, this barycentricly interpolates the `x` and `y`\n    at the point, if it's inside the Polygon. It returns None, None\n    if the point is inside the polygon.\n\n    This is mostly intended to be used to interpolate screen-space\n    coordinates onto a Render's internal coordinate space.\n    ";
static PyMethodDef __pyx_mdef_5renpy_3gl2_11gl2geometry_9barycentric_point = {"barycentric_point", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_9barycentric_point, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_8barycentric_point};
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_9barycentric_point(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a = 0;
  float __pyx_v_x;
  float __pyx_v_y;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("barycentric_point (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_a,&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_a)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("barycentric_point", 1, 3, 3, 1); __PYX_ERR(0, 512, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("barycentric_point", 1, 3, 3, 2); __PYX_ERR(0, 512, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "barycentric_point") < 0)) __PYX_ERR(0, 512, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_a = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)values[0]);
    __pyx_v_x = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_x == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 512, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_y == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 512, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("barycentric_point", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 512, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.barycentric_point", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_a), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "a", 0))) __PYX_ERR(0, 512, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_8barycentric_point(__pyx_self, __pyx_v_a, __pyx_v_x, __pyx_v_y);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_8barycentric_point(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_a, float __pyx_v_x, float __pyx_v_y) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_b = 0;
  float *__pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  float *__pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("barycentric_point", 0);

  /* "renpy/gl2/gl2geometry.pyx":526
 *     """
 * 
 *     cdef Polygon b = Polygon(6, 1, None)             # <<<<<<<<<<<<<<
 *     b.points = 1
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_b = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":527
 * 
 *     cdef Polygon b = Polygon(6, 1, None)
 *     b.points = 1             # <<<<<<<<<<<<<<
 * 
 *     cdef float *p = b.data
 */
  __pyx_v_b->points = 1;

  /* "renpy/gl2/gl2geometry.pyx":529
 *     b.points = 1
 * 
 *     cdef float *p = b.data             # <<<<<<<<<<<<<<
 *     p[X] = x
 *     p[Y] = y
 */
  __pyx_t_2 = __pyx_v_b->data;
  __pyx_v_p = __pyx_t_2;

  /* "renpy/gl2/gl2geometry.pyx":530
 * 
 *     cdef float *p = b.data
 *     p[X] = x             # <<<<<<<<<<<<<<
 *     p[Y] = y
 *     p[Z] = 0.0
 */
  (__pyx_v_p[0]) = __pyx_v_x;

  /* "renpy/gl2/gl2geometry.pyx":531
 *     cdef float *p = b.data
 *     p[X] = x
 *     p[Y] = y             # <<<<<<<<<<<<<<
 *     p[Z] = 0.0
 *     p[W] = 0.0
 */
  (__pyx_v_p[1]) = __pyx_v_y;

  /* "renpy/gl2/gl2geometry.pyx":532
 *     p[X] = x
 *     p[Y] = y
 *     p[Z] = 0.0             # <<<<<<<<<<<<<<
 *     p[W] = 0.0
 * 
 */
  (__pyx_v_p[2]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":533
 *     p[Y] = y
 *     p[Z] = 0.0
 *     p[W] = 0.0             # <<<<<<<<<<<<<<
 * 
 *     barycentric(a, b, 0)
 */
  (__pyx_v_p[3]) = 0.0;

  /* "renpy/gl2/gl2geometry.pyx":535
 *     p[W] = 0.0
 * 
 *     barycentric(a, b, 0)             # <<<<<<<<<<<<<<
 * 
 *     if p[W] == 0:
 */
  __pyx_t_1 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_a, __pyx_v_b, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":537
 *     barycentric(a, b, 0)
 * 
 *     if p[W] == 0:             # <<<<<<<<<<<<<<
 *         return None, None
 * 
 */
  __pyx_t_3 = (((__pyx_v_p[3]) == 0.0) != 0);
  if (__pyx_t_3) {

    /* "renpy/gl2/gl2geometry.pyx":538
 * 
 *     if p[W] == 0:
 *         return None, None             # <<<<<<<<<<<<<<
 * 
 *     return (p[TX], p[TY])
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_tuple__9);
    __pyx_r = __pyx_tuple__9;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":537
 *     barycentric(a, b, 0)
 * 
 *     if p[W] == 0:             # <<<<<<<<<<<<<<
 *         return None, None
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":540
 *         return None, None
 * 
 *     return (p[TX], p[TY])             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble((__pyx_v_p[4])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyFloat_FromDouble((__pyx_v_p[5])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":512
 * 
 * 
 * def barycentric_point(Polygon a, float x, float y):             # <<<<<<<<<<<<<<
 *     """
 *     This expects Polygon a to be a polygon with a stride of 6, with a
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.barycentric_point", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":545
 * cdef class Mesh:
 * 
 *     def __init__(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Represents a mesh consisting of one or more polygons. A mesh needs
 */

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh___init__[] = "\n        Represents a mesh consisting of one or more polygons. A mesh needs\n        to be created either by:\n\n        * First, create the mesh object.\n        * Next, add any additional attributes to it.\n        * Finally, add polygons to the mesh.\n\n        Or by copying the mesh or calling a method that returns a new mesh.\n\n        After that, the inplace methods can be called to further change\n        the mesh.\n\n        Lastly, get_data can be called to access the mesh data. Once get_data\n        is called, the mesh becomes immutable. (This is usally called by\n        Shader.draw.)\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_5renpy_3gl2_11gl2geometry_4Mesh___init__;
#endif
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh___init__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh___init__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl2/gl2geometry.pyx":564
 *         """
 * 
 *         self.points = 0             # <<<<<<<<<<<<<<
 *         self.stride = 4
 *         self.polygons = [ ]
 */
  __pyx_v_self->points = 0;

  /* "renpy/gl2/gl2geometry.pyx":565
 * 
 *         self.points = 0
 *         self.stride = 4             # <<<<<<<<<<<<<<
 *         self.polygons = [ ]
 *         self.attributes = { "aPosition" : 0 }
 */
  __pyx_v_self->stride = 4;

  /* "renpy/gl2/gl2geometry.pyx":566
 *         self.points = 0
 *         self.stride = 4
 *         self.polygons = [ ]             # <<<<<<<<<<<<<<
 *         self.attributes = { "aPosition" : 0 }
 *         self.data = NULL
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 566, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->polygons);
  __Pyx_DECREF(__pyx_v_self->polygons);
  __pyx_v_self->polygons = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":567
 *         self.stride = 4
 *         self.polygons = [ ]
 *         self.attributes = { "aPosition" : 0 }             # <<<<<<<<<<<<<<
 *         self.data = NULL
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_aPosition, __pyx_int_0) < 0) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->attributes);
  __Pyx_DECREF(__pyx_v_self->attributes);
  __pyx_v_self->attributes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":568
 *         self.polygons = [ ]
 *         self.attributes = { "aPosition" : 0 }
 *         self.data = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(Mesh self):
 */
  __pyx_v_self->data = NULL;

  /* "renpy/gl2/gl2geometry.pyx":545
 * cdef class Mesh:
 * 
 *     def __init__(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Represents a mesh consisting of one or more polygons. A mesh needs
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":570
 *         self.data = NULL
 * 
 *     def __dealloc__(Mesh self):             # <<<<<<<<<<<<<<
 *         if self.data:
 *             free(self.data)
 */

/* Python wrapper */
static void __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_2__dealloc__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_2__dealloc__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "renpy/gl2/gl2geometry.pyx":571
 * 
 *     def __dealloc__(Mesh self):
 *         if self.data:             # <<<<<<<<<<<<<<
 *             free(self.data)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->data != 0);
  if (__pyx_t_1) {

    /* "renpy/gl2/gl2geometry.pyx":572
 *     def __dealloc__(Mesh self):
 *         if self.data:
 *             free(self.data)             # <<<<<<<<<<<<<<
 * 
 *     def add_attribute(Mesh self, name, size):
 */
    free(__pyx_v_self->data);

    /* "renpy/gl2/gl2geometry.pyx":571
 * 
 *     def __dealloc__(Mesh self):
 *         if self.data:             # <<<<<<<<<<<<<<
 *             free(self.data)
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":570
 *         self.data = NULL
 * 
 *     def __dealloc__(Mesh self):             # <<<<<<<<<<<<<<
 *         if self.data:
 *             free(self.data)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl2/gl2geometry.pyx":574
 *             free(self.data)
 * 
 *     def add_attribute(Mesh self, name, size):             # <<<<<<<<<<<<<<
 *         """
 *         Adds an attribute to this mesh.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_5add_attribute(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_4add_attribute[] = "\n        Adds an attribute to this mesh.\n\n        `name`\n            The name of the attribute.\n\n        `size`\n            The number of floats per vertex that make up the attribute.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_5add_attribute(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_attribute (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_attribute", 1, 2, 2, 1); __PYX_ERR(0, 574, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_attribute") < 0)) __PYX_ERR(0, 574, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_name = values[0];
    __pyx_v_size = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_attribute", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 574, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_attribute", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_4add_attribute(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), __pyx_v_name, __pyx_v_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_4add_attribute(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_name, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("add_attribute", 0);

  /* "renpy/gl2/gl2geometry.pyx":585
 *         """
 * 
 *         self.attributes[name] = self.stride             # <<<<<<<<<<<<<<
 *         self.stride += size
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_v_self->attributes == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 585, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_v_self->attributes, __pyx_v_name, __pyx_t_1) < 0)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":586
 * 
 *         self.attributes[name] = self.stride
 *         self.stride += size             # <<<<<<<<<<<<<<
 * 
 *     def add_polygon(Mesh self, data):
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_t_1, __pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->stride = __pyx_t_3;

  /* "renpy/gl2/gl2geometry.pyx":574
 *             free(self.data)
 * 
 *     def add_attribute(Mesh self, name, size):             # <<<<<<<<<<<<<<
 *         """
 *         Adds an attribute to this mesh.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_attribute", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":588
 *         self.stride += size
 * 
 *     def add_polygon(Mesh self, data):             # <<<<<<<<<<<<<<
 *         """
 *         Adds a polygon.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_7add_polygon(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_6add_polygon[] = "\n        Adds a polygon.\n\n        `data`\n            This is an iterable. It should have self.stride data for each\n            vertex, and the number of vertices is derived from the length\n            of the data.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_7add_polygon(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_polygon (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6add_polygon(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6add_polygon(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_data) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("add_polygon", 0);

  /* "renpy/gl2/gl2geometry.pyx":598
 *         """
 * 
 *         cdef Polygon p = Polygon(self.stride, len(data) // self.stride, data)             # <<<<<<<<<<<<<<
 *         self.points += p.points
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 598, __pyx_L1_error)
  if (unlikely(__pyx_v_self->stride == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 598, __pyx_L1_error)
  }
  else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((int)-1) > 0)) && unlikely(__pyx_v_self->stride == (int)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_t_2))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 598, __pyx_L1_error)
  }
  __pyx_t_3 = PyInt_FromSsize_t(__Pyx_div_Py_ssize_t(__pyx_t_2, __pyx_v_self->stride)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_data);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_p = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "renpy/gl2/gl2geometry.pyx":599
 * 
 *         cdef Polygon p = Polygon(self.stride, len(data) // self.stride, data)
 *         self.points += p.points             # <<<<<<<<<<<<<<
 * 
 *         self.polygons.append(p)
 */
  __pyx_v_self->points = (__pyx_v_self->points + __pyx_v_p->points);

  /* "renpy/gl2/gl2geometry.pyx":601
 *         self.points += p.points
 * 
 *         self.polygons.append(p)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 601, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->polygons, ((PyObject *)__pyx_v_p)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 601, __pyx_L1_error)

  /* "renpy/gl2/gl2geometry.pyx":588
 *         self.stride += size
 * 
 *     def add_polygon(Mesh self, data):             # <<<<<<<<<<<<<<
 *         """
 *         Adds a polygon.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_polygon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":604
 * 
 * 
 *     def add_rectangle(Mesh self, double x, double y, double w, double h):             # <<<<<<<<<<<<<<
 *         """
 *         Adds a polygon.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_9add_rectangle(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_8add_rectangle[] = "\n        Adds a polygon.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_9add_rectangle(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_w;
  double __pyx_v_h;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_rectangle (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_w,&__pyx_n_s_h,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_rectangle", 1, 4, 4, 1); __PYX_ERR(0, 604, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_rectangle", 1, 4, 4, 2); __PYX_ERR(0, 604, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_rectangle", 1, 4, 4, 3); __PYX_ERR(0, 604, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_rectangle") < 0)) __PYX_ERR(0, 604, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 604, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 604, __pyx_L3_error)
    __pyx_v_w = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_w == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 604, __pyx_L3_error)
    __pyx_v_h = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_h == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 604, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_rectangle", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 604, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8add_rectangle(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_w, __pyx_v_h);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8add_rectangle(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_w, double __pyx_v_h) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("add_rectangle", 0);

  /* "renpy/gl2/gl2geometry.pyx":609
 *         """
 * 
 *         self.points += 4             # <<<<<<<<<<<<<<
 *         self.polygons.append(rectangle(x, y, w, h))
 * 
 */
  __pyx_v_self->points = (__pyx_v_self->points + 4);

  /* "renpy/gl2/gl2geometry.pyx":610
 * 
 *         self.points += 4
 *         self.polygons.append(rectangle(x, y, w, h))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 610, __pyx_L1_error)
  }
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_rectangle(__pyx_v_x, __pyx_v_y, __pyx_v_w, __pyx_v_h, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->polygons, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":604
 * 
 * 
 *     def add_rectangle(Mesh self, double x, double y, double w, double h):             # <<<<<<<<<<<<<<
 *         """
 *         Adds a polygon.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":613
 * 
 * 
 *     def add_texture_rectangle(Mesh self, double pl, double pt, double pr, double pb, double tl, double tt, double tr, double tb):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a polygon corresponding to a texture rectangle.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_11add_texture_rectangle(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_10add_texture_rectangle[] = "\n        Returns a polygon corresponding to a texture rectangle.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_11add_texture_rectangle(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_pl;
  double __pyx_v_pt;
  double __pyx_v_pr;
  double __pyx_v_pb;
  double __pyx_v_tl;
  double __pyx_v_tt;
  double __pyx_v_tr;
  double __pyx_v_tb;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_texture_rectangle (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pl,&__pyx_n_s_pt,&__pyx_n_s_pr,&__pyx_n_s_pb,&__pyx_n_s_tl,&__pyx_n_s_tt,&__pyx_n_s_tr,&__pyx_n_s_tb,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pl)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 1); __PYX_ERR(0, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 2); __PYX_ERR(0, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 3); __PYX_ERR(0, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tl)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 4); __PYX_ERR(0, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 5); __PYX_ERR(0, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 6); __PYX_ERR(0, 613, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tb)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, 7); __PYX_ERR(0, 613, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_texture_rectangle") < 0)) __PYX_ERR(0, 613, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_pl = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_pl == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_pt = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_pt == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_pr = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_pr == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_pb = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_pb == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_tl = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_tl == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_tt = __pyx_PyFloat_AsDouble(values[5]); if (unlikely((__pyx_v_tt == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_tr = __pyx_PyFloat_AsDouble(values[6]); if (unlikely((__pyx_v_tr == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
    __pyx_v_tb = __pyx_PyFloat_AsDouble(values[7]); if (unlikely((__pyx_v_tb == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 613, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_texture_rectangle", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 613, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_texture_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10add_texture_rectangle(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), __pyx_v_pl, __pyx_v_pt, __pyx_v_pr, __pyx_v_pb, __pyx_v_tl, __pyx_v_tt, __pyx_v_tr, __pyx_v_tb);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10add_texture_rectangle(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_pl, double __pyx_v_pt, double __pyx_v_pr, double __pyx_v_pb, double __pyx_v_tl, double __pyx_v_tt, double __pyx_v_tr, double __pyx_v_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("add_texture_rectangle", 0);

  /* "renpy/gl2/gl2geometry.pyx":618
 *         """
 * 
 *         self.points += 4             # <<<<<<<<<<<<<<
 *         self.polygons.append(texture_rectangle(pl, pt, pr, pb, tl, tt, tr, tb))
 * 
 */
  __pyx_v_self->points = (__pyx_v_self->points + 4);

  /* "renpy/gl2/gl2geometry.pyx":619
 * 
 *         self.points += 4
 *         self.polygons.append(texture_rectangle(pl, pt, pr, pb, tl, tt, tr, tb))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 619, __pyx_L1_error)
  }
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_texture_rectangle(__pyx_v_pl, __pyx_v_pt, __pyx_v_pr, __pyx_v_pb, __pyx_v_tl, __pyx_v_tt, __pyx_v_tr, __pyx_v_tb, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->polygons, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":613
 * 
 * 
 *     def add_texture_rectangle(Mesh self, double pl, double pt, double pr, double pb, double tl, double tt, double tr, double tb):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a polygon corresponding to a texture rectangle.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.add_texture_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":622
 * 
 * 
 *     cdef float *get_data(Mesh self, int offset):             # <<<<<<<<<<<<<<
 *         """
 *         When called, this freezes the data of this mesh. It then returns
 */

static float *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_get_data(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_v_offset) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  int __pyx_v_i;
  int __pyx_v_count;
  int __pyx_v_points;
  float *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("get_data", 0);

  /* "renpy/gl2/gl2geometry.pyx":635
 *         cdef int points
 * 
 *         self.polygon_count = count = len(self.polygons)             # <<<<<<<<<<<<<<
 * 
 *         if not count:
 */
  __pyx_t_1 = __pyx_v_self->polygons;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 635, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_self->polygon_count = __pyx_t_2;
  __pyx_v_count = __pyx_t_2;

  /* "renpy/gl2/gl2geometry.pyx":637
 *         self.polygon_count = count = len(self.polygons)
 * 
 *         if not count:             # <<<<<<<<<<<<<<
 *             self.polygon_points = 0
 *             return NULL
 */
  __pyx_t_3 = ((!(__pyx_v_count != 0)) != 0);
  if (__pyx_t_3) {

    /* "renpy/gl2/gl2geometry.pyx":638
 * 
 *         if not count:
 *             self.polygon_points = 0             # <<<<<<<<<<<<<<
 *             return NULL
 * 
 */
    __pyx_v_self->polygon_points = 0;

    /* "renpy/gl2/gl2geometry.pyx":639
 *         if not count:
 *             self.polygon_points = 0
 *             return NULL             # <<<<<<<<<<<<<<
 * 
 *         p = self.polygons[0]
 */
    __pyx_r = NULL;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":637
 *         self.polygon_count = count = len(self.polygons)
 * 
 *         if not count:             # <<<<<<<<<<<<<<
 *             self.polygon_points = 0
 *             return NULL
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":641
 *             return NULL
 * 
 *         p = self.polygons[0]             # <<<<<<<<<<<<<<
 *         points = p.points
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 641, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_self->polygons, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 641, __pyx_L1_error)
  __pyx_v_p = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":642
 * 
 *         p = self.polygons[0]
 *         points = p.points             # <<<<<<<<<<<<<<
 * 
 *         if count == 1:
 */
  __pyx_t_4 = __pyx_v_p->points;
  __pyx_v_points = __pyx_t_4;

  /* "renpy/gl2/gl2geometry.pyx":644
 *         points = p.points
 * 
 *         if count == 1:             # <<<<<<<<<<<<<<
 *             self.polygon_points = points
 *             return p.data + offset
 */
  __pyx_t_3 = ((__pyx_v_count == 1) != 0);
  if (__pyx_t_3) {

    /* "renpy/gl2/gl2geometry.pyx":645
 * 
 *         if count == 1:
 *             self.polygon_points = points             # <<<<<<<<<<<<<<
 *             return p.data + offset
 * 
 */
    __pyx_v_self->polygon_points = __pyx_v_points;

    /* "renpy/gl2/gl2geometry.pyx":646
 *         if count == 1:
 *             self.polygon_points = points
 *             return p.data + offset             # <<<<<<<<<<<<<<
 * 
 *         if not self.data:
 */
    __pyx_r = (__pyx_v_p->data + __pyx_v_offset);
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":644
 *         points = p.points
 * 
 *         if count == 1:             # <<<<<<<<<<<<<<
 *             self.polygon_points = points
 *             return p.data + offset
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":648
 *             return p.data + offset
 * 
 *         if not self.data:             # <<<<<<<<<<<<<<
 *             self.data = <float *> malloc(self.points * self.stride * sizeof(float))
 * 
 */
  __pyx_t_3 = ((!(__pyx_v_self->data != 0)) != 0);
  if (__pyx_t_3) {

    /* "renpy/gl2/gl2geometry.pyx":649
 * 
 *         if not self.data:
 *             self.data = <float *> malloc(self.points * self.stride * sizeof(float))             # <<<<<<<<<<<<<<
 * 
 *             i = 0
 */
    __pyx_v_self->data = ((float *)malloc(((__pyx_v_self->points * __pyx_v_self->stride) * (sizeof(float)))));

    /* "renpy/gl2/gl2geometry.pyx":651
 *             self.data = <float *> malloc(self.points * self.stride * sizeof(float))
 * 
 *             i = 0             # <<<<<<<<<<<<<<
 * 
 *             for p in self.polygons:
 */
    __pyx_v_i = 0;

    /* "renpy/gl2/gl2geometry.pyx":653
 *             i = 0
 * 
 *             for p in self.polygons:             # <<<<<<<<<<<<<<
 *                 memcpy(&self.data[i], p.data, p.points * self.stride * sizeof(float))
 *                 i += p.points * self.stride
 */
    if (unlikely(__pyx_v_self->polygons == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 653, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    for (;;) {
      if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_5); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 653, __pyx_L1_error)
      #else
      __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 653, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
      if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 653, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_p, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "renpy/gl2/gl2geometry.pyx":654
 * 
 *             for p in self.polygons:
 *                 memcpy(&self.data[i], p.data, p.points * self.stride * sizeof(float))             # <<<<<<<<<<<<<<
 *                 i += p.points * self.stride
 * 
 */
      memcpy((&(__pyx_v_self->data[__pyx_v_i])), __pyx_v_p->data, ((__pyx_v_p->points * __pyx_v_self->stride) * (sizeof(float))));

      /* "renpy/gl2/gl2geometry.pyx":655
 *             for p in self.polygons:
 *                 memcpy(&self.data[i], p.data, p.points * self.stride * sizeof(float))
 *                 i += p.points * self.stride             # <<<<<<<<<<<<<<
 * 
 *                 if p.points != points:
 */
      __pyx_v_i = (__pyx_v_i + (__pyx_v_p->points * __pyx_v_self->stride));

      /* "renpy/gl2/gl2geometry.pyx":657
 *                 i += p.points * self.stride
 * 
 *                 if p.points != points:             # <<<<<<<<<<<<<<
 *                     points = 0
 * 
 */
      __pyx_t_3 = ((__pyx_v_p->points != __pyx_v_points) != 0);
      if (__pyx_t_3) {

        /* "renpy/gl2/gl2geometry.pyx":658
 * 
 *                 if p.points != points:
 *                     points = 0             # <<<<<<<<<<<<<<
 * 
 *             self.polygon_points = points
 */
        __pyx_v_points = 0;

        /* "renpy/gl2/gl2geometry.pyx":657
 *                 i += p.points * self.stride
 * 
 *                 if p.points != points:             # <<<<<<<<<<<<<<
 *                     points = 0
 * 
 */
      }

      /* "renpy/gl2/gl2geometry.pyx":653
 *             i = 0
 * 
 *             for p in self.polygons:             # <<<<<<<<<<<<<<
 *                 memcpy(&self.data[i], p.data, p.points * self.stride * sizeof(float))
 *                 i += p.points * self.stride
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl2/gl2geometry.pyx":660
 *                     points = 0
 * 
 *             self.polygon_points = points             # <<<<<<<<<<<<<<
 * 
 *         return self.data + offset
 */
    __pyx_v_self->polygon_points = __pyx_v_points;

    /* "renpy/gl2/gl2geometry.pyx":648
 *             return p.data + offset
 * 
 *         if not self.data:             # <<<<<<<<<<<<<<
 *             self.data = <float *> malloc(self.points * self.stride * sizeof(float))
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":662
 *             self.polygon_points = points
 * 
 *         return self.data + offset             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh copy(Mesh self):
 */
  __pyx_r = (__pyx_v_self->data + __pyx_v_offset);
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":622
 * 
 * 
 *     cdef float *get_data(Mesh self, int offset):             # <<<<<<<<<<<<<<
 *         """
 *         When called, this freezes the data of this mesh. It then returns
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Mesh.get_data", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":664
 *         return self.data + offset
 * 
 *     cpdef Mesh copy(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this Mesh.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_rv = NULL;
  PyObject *__pyx_v_i = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("copy", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13copy)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 664, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh))))) __PYX_ERR(0, 664, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":669
 *         """
 * 
 *         rv = Mesh()             # <<<<<<<<<<<<<<
 *         rv.stride = self.stride
 *         rv.points = self.points
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":670
 * 
 *         rv = Mesh()
 *         rv.stride = self.stride             # <<<<<<<<<<<<<<
 *         rv.points = self.points
 *         rv.polygons = [ i.copy() for i in self.polygons ]
 */
  __pyx_t_5 = __pyx_v_self->stride;
  __pyx_v_rv->stride = __pyx_t_5;

  /* "renpy/gl2/gl2geometry.pyx":671
 *         rv = Mesh()
 *         rv.stride = self.stride
 *         rv.points = self.points             # <<<<<<<<<<<<<<
 *         rv.polygons = [ i.copy() for i in self.polygons ]
 *         rv.attributes = self.attributes
 */
  __pyx_t_5 = __pyx_v_self->points;
  __pyx_v_rv->points = __pyx_t_5;

  /* "renpy/gl2/gl2geometry.pyx":672
 *         rv.stride = self.stride
 *         rv.points = self.points
 *         rv.polygons = [ i.copy() for i in self.polygons ]             # <<<<<<<<<<<<<<
 *         rv.attributes = self.attributes
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 672, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 672, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
  for (;;) {
    if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_3); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 672, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 672, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_i, __pyx_n_s_copy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 672, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_7) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 672, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 672, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_3))) __PYX_ERR(0, 672, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_rv->polygons);
  __Pyx_DECREF(__pyx_v_rv->polygons);
  __pyx_v_rv->polygons = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":673
 *         rv.points = self.points
 *         rv.polygons = [ i.copy() for i in self.polygons ]
 *         rv.attributes = self.attributes             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
  __pyx_t_1 = __pyx_v_self->attributes;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_rv->attributes);
  __Pyx_DECREF(__pyx_v_rv->attributes);
  __pyx_v_rv->attributes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":675
 *         rv.attributes = self.attributes
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     cpdef void offset_inplace(Mesh self, double x, double y, double z):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":664
 *         return self.data + offset
 * 
 *     cpdef Mesh copy(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this Mesh.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_12copy[] = "\n        Returns a copy of this Mesh.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_12copy(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_12copy(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("copy", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_copy(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":677
 *         return rv
 * 
 *     cpdef void offset_inplace(Mesh self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         """
 *         Offsets each polygon in the mesh, in place.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_15offset_inplace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  __Pyx_RefNannySetupContext("offset_inplace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_offset_inplace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_15offset_inplace)) {
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 677, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 677, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyFloat_FromDouble(__pyx_v_z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 677, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = __pyx_t_1; __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":684
 *         cdef Polygon p
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             p.offset_inplace(x, y, z)
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 684, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_10 = 0;
  for (;;) {
    if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_10); __Pyx_INCREF(__pyx_t_2); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 684, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 684, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_p, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":685
 * 
 *         for p in self.polygons:
 *             p.offset_inplace(x, y, z)             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh offset(self, double x, double y, double z):
 */
    ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_p->__pyx_vtab)->offset_inplace(__pyx_v_p, __pyx_v_x, __pyx_v_y, __pyx_v_z, 0);

    /* "renpy/gl2/gl2geometry.pyx":684
 *         cdef Polygon p
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             p.offset_inplace(x, y, z)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":677
 *         return rv
 * 
 *     cpdef void offset_inplace(Mesh self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         """
 *         Offsets each polygon in the mesh, in place.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Mesh.offset_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_15offset_inplace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_14offset_inplace[] = "\n        Offsets each polygon in the mesh, in place.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_15offset_inplace(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_z;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("offset_inplace (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_z,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset_inplace", 1, 3, 3, 1); __PYX_ERR(0, 677, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_z)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset_inplace", 1, 3, 3, 2); __PYX_ERR(0, 677, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "offset_inplace") < 0)) __PYX_ERR(0, 677, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 677, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 677, __pyx_L3_error)
    __pyx_v_z = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_z == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 677, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("offset_inplace", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 677, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.offset_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14offset_inplace(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14offset_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("offset_inplace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset_inplace(__pyx_v_self, __pyx_v_x, __pyx_v_y, __pyx_v_z, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.offset_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":687
 *             p.offset_inplace(x, y, z)
 * 
 *     cpdef Mesh offset(self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of the mesh with each polygon offset.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_17offset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("offset", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_17offset)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyFloat_FromDouble(__pyx_v_z); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_6 = __pyx_t_1; __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_t_3, __pyx_t_4, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 687, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_5);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh))))) __PYX_ERR(0, 687, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":692
 *         """
 * 
 *         cdef Mesh rv = self.copy()             # <<<<<<<<<<<<<<
 *         rv.offset_inplace(x, y, z)
 *         return rv
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":693
 * 
 *         cdef Mesh rv = self.copy()
 *         rv.offset_inplace(x, y, z)             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_rv->__pyx_vtab)->offset_inplace(__pyx_v_rv, __pyx_v_x, __pyx_v_y, __pyx_v_z, 0);

  /* "renpy/gl2/gl2geometry.pyx":694
 *         cdef Mesh rv = self.copy()
 *         rv.offset_inplace(x, y, z)
 *         return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":687
 *             p.offset_inplace(x, y, z)
 * 
 *     cpdef Mesh offset(self, double x, double y, double z):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of the mesh with each polygon offset.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_17offset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_16offset[] = "\n        Returns a copy of the mesh with each polygon offset.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_17offset(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  double __pyx_v_z;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("offset (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_z,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset", 1, 3, 3, 1); __PYX_ERR(0, 687, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_z)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("offset", 1, 3, 3, 2); __PYX_ERR(0, 687, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "offset") < 0)) __PYX_ERR(0, 687, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 687, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 687, __pyx_L3_error)
    __pyx_v_z = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_z == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 687, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("offset", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 687, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_16offset(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_z);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_16offset(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, double __pyx_v_z) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("offset", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset(__pyx_v_self, __pyx_v_x, __pyx_v_y, __pyx_v_z, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.offset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":697
 * 
 * 
 *     cpdef void perspective_divide_inplace(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Performs a perspective divide on this mesh, inplace.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_19perspective_divide_inplace(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  __Pyx_RefNannySetupContext("perspective_divide_inplace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_perspective_divide_inplace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_19perspective_divide_inplace)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":704
 *         cdef Polygon p
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             p.perspective_divide_inplace()
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 704, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 704, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_p, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":705
 * 
 *         for p in self.polygons:
 *             p.perspective_divide_inplace()             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh perspective_divide(Mesh self):
 */
    ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_p->__pyx_vtab)->perspective_divide_inplace(__pyx_v_p, 0);

    /* "renpy/gl2/gl2geometry.pyx":704
 *         cdef Polygon p
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             p.perspective_divide_inplace()
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":697
 * 
 * 
 *     cpdef void perspective_divide_inplace(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Performs a perspective divide on this mesh, inplace.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Mesh.perspective_divide_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_19perspective_divide_inplace(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_18perspective_divide_inplace[] = "\n        Performs a perspective divide on this mesh, inplace.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_19perspective_divide_inplace(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("perspective_divide_inplace (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_18perspective_divide_inplace(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_18perspective_divide_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("perspective_divide_inplace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide_inplace(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.perspective_divide_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":707
 *             p.perspective_divide_inplace()
 * 
 *     cpdef Mesh perspective_divide(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this mesh that has been perspective divided.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_21perspective_divide(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("perspective_divide", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_perspective_divide); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 707, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_21perspective_divide)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 707, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 707, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh))))) __PYX_ERR(0, 707, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":712
 *         """
 * 
 *         cdef Mesh rv = self.copy()             # <<<<<<<<<<<<<<
 *         rv.perspective_divide_inplace()
 *         return rv
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":713
 * 
 *         cdef Mesh rv = self.copy()
 *         rv.perspective_divide_inplace()             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_rv->__pyx_vtab)->perspective_divide_inplace(__pyx_v_rv, 0);

  /* "renpy/gl2/gl2geometry.pyx":714
 *         cdef Mesh rv = self.copy()
 *         rv.perspective_divide_inplace()
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     cpdef void multiply_matrix_inplace(Mesh self, Matrix matrix):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":707
 *             p.perspective_divide_inplace()
 * 
 *     cpdef Mesh perspective_divide(Mesh self):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this mesh that has been perspective divided.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.perspective_divide", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_21perspective_divide(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_20perspective_divide[] = "\n        Returns a copy of this mesh that has been perspective divided.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_21perspective_divide(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("perspective_divide (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_20perspective_divide(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_20perspective_divide(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("perspective_divide", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 707, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.perspective_divide", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":716
 *         return rv
 * 
 *     cpdef void multiply_matrix_inplace(Mesh self, Matrix matrix):             # <<<<<<<<<<<<<<
 *         """
 *         Multiplies the position data in this Mesh by the matrix, in place.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_23multiply_matrix_inplace(PyObject *__pyx_v_self, PyObject *__pyx_v_matrix); /*proto*/
static void __pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  __Pyx_RefNannySetupContext("multiply_matrix_inplace", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_multiply_matrix_inplace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_23multiply_matrix_inplace)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_matrix)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_matrix)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_matrix)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 716, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_matrix));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_matrix));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_matrix));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":723
 *         cdef Polygon p
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             p.multiply_matrix_inplace(matrix)
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 723, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
  for (;;) {
    if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 723, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 723, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 723, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_p, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":724
 * 
 *         for p in self.polygons:
 *             p.multiply_matrix_inplace(matrix)             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh multiply_matrix(Mesh self, Matrix matrix):
 */
    ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_p->__pyx_vtab)->multiply_matrix_inplace(__pyx_v_p, __pyx_v_matrix, 0);

    /* "renpy/gl2/gl2geometry.pyx":723
 *         cdef Polygon p
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             p.multiply_matrix_inplace(matrix)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":716
 *         return rv
 * 
 *     cpdef void multiply_matrix_inplace(Mesh self, Matrix matrix):             # <<<<<<<<<<<<<<
 *         """
 *         Multiplies the position data in this Mesh by the matrix, in place.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("renpy.gl2.gl2geometry.Mesh.multiply_matrix_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_23multiply_matrix_inplace(PyObject *__pyx_v_self, PyObject *__pyx_v_matrix); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_22multiply_matrix_inplace[] = "\n        Multiplies the position data in this Mesh by the matrix, in place.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_23multiply_matrix_inplace(PyObject *__pyx_v_self, PyObject *__pyx_v_matrix) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("multiply_matrix_inplace (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_matrix), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "matrix", 0))) __PYX_ERR(0, 716, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_22multiply_matrix_inplace(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)__pyx_v_matrix));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_22multiply_matrix_inplace(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("multiply_matrix_inplace", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix_inplace(__pyx_v_self, __pyx_v_matrix, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.multiply_matrix_inplace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":726
 *             p.multiply_matrix_inplace(matrix)
 * 
 *     cpdef Mesh multiply_matrix(Mesh self, Matrix matrix):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this Mesh with the position data multipled by Matrix.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_25multiply_matrix(PyObject *__pyx_v_self, PyObject *__pyx_v_matrix); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_rv = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("multiply_matrix", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_multiply_matrix); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_25multiply_matrix)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_matrix)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_matrix)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_matrix)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 726, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_matrix));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_matrix));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_matrix));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 726, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh))))) __PYX_ERR(0, 726, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":731
 *         """
 * 
 *         cdef Mesh rv = self.copy()             # <<<<<<<<<<<<<<
 *         rv.multiply_matrix_inplace(matrix)
 *         return rv
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self->__pyx_vtab)->copy(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":732
 * 
 *         cdef Mesh rv = self.copy()
 *         rv.multiply_matrix_inplace(matrix)             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_rv->__pyx_vtab)->multiply_matrix_inplace(__pyx_v_rv, __pyx_v_matrix, 0);

  /* "renpy/gl2/gl2geometry.pyx":733
 *         cdef Mesh rv = self.copy()
 *         rv.multiply_matrix_inplace(matrix)
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     def __mul__(self, other):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":726
 *             p.multiply_matrix_inplace(matrix)
 * 
 *     cpdef Mesh multiply_matrix(Mesh self, Matrix matrix):             # <<<<<<<<<<<<<<
 *         """
 *         Returns a copy of this Mesh with the position data multipled by Matrix.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.multiply_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_25multiply_matrix(PyObject *__pyx_v_self, PyObject *__pyx_v_matrix); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_24multiply_matrix[] = "\n        Returns a copy of this Mesh with the position data multipled by Matrix.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_25multiply_matrix(PyObject *__pyx_v_self, PyObject *__pyx_v_matrix) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("multiply_matrix (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_matrix), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "matrix", 0))) __PYX_ERR(0, 726, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_24multiply_matrix(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)__pyx_v_matrix));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_24multiply_matrix(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_matrix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("multiply_matrix", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix(__pyx_v_self, __pyx_v_matrix, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 726, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.multiply_matrix", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":735
 *         return rv
 * 
 *     def __mul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_27__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_27__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__mul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_26__mul__(((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_26__mul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__mul__", 0);

  /* "renpy/gl2/gl2geometry.pyx":736
 * 
 *     def __mul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_5renpy_7display_6matrix_Matrix); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/gl2geometry.pyx":737
 *     def __mul__(self, other):
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)             # <<<<<<<<<<<<<<
 * 
 *         return NotImplemented
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_multiply_matrix); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 737, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_other); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 737, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_other};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 737, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_other};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 737, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 737, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_v_other);
        __Pyx_GIVEREF(__pyx_v_other);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_other);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 737, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":736
 * 
 *     def __mul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":739
 *             return self.multiply_matrix(other)
 * 
 *         return NotImplemented             # <<<<<<<<<<<<<<
 * 
 *     def __rmul__(self, other):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_builtin_NotImplemented);
  __pyx_r = __pyx_builtin_NotImplemented;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":735
 *         return rv
 * 
 *     def __mul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.__mul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":741
 *         return NotImplemented
 * 
 *     def __rmul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_29__rmul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_29__rmul__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__rmul__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_28__rmul__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_28__rmul__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__rmul__", 0);

  /* "renpy/gl2/gl2geometry.pyx":742
 * 
 *     def __rmul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_5renpy_7display_6matrix_Matrix); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl2/gl2geometry.pyx":743
 *     def __rmul__(self, other):
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)             # <<<<<<<<<<<<<<
 * 
 *         return NotImplemented
 */
    __Pyx_XDECREF(__pyx_r);
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_5renpy_7display_6matrix_Matrix))))) __PYX_ERR(0, 743, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self->__pyx_vtab)->multiply_matrix(__pyx_v_self, ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":742
 * 
 *     def __rmul__(self, other):
 *         if isinstance(other, Matrix):             # <<<<<<<<<<<<<<
 *             return self.multiply_matrix(other)
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":745
 *             return self.multiply_matrix(other)
 * 
 *         return NotImplemented             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh intersect(Mesh self, Mesh other):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_builtin_NotImplemented);
  __pyx_r = __pyx_builtin_NotImplemented;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":741
 *         return NotImplemented
 * 
 *     def __rmul__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, Matrix):
 *             return self.multiply_matrix(other)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.__rmul__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":747
 *         return NotImplemented
 * 
 *     cpdef Mesh intersect(Mesh self, Mesh other):             # <<<<<<<<<<<<<<
 *         """
 *         Intersects this mesh with another mesh, and returns a new mesh that
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_31intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_other, int __pyx_skip_dispatch) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_rv = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_op = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_sp = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  PyObject *__pyx_7genexpr__pyx_v_k = NULL;
  PyObject *__pyx_7genexpr__pyx_v_v = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("intersect", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_intersect); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 747, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_31intersect)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 747, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_other)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 747, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_other)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 747, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 747, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_other));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_other));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_other));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 747, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh))))) __PYX_ERR(0, 747, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":755
 *         """
 * 
 *         rv = Mesh()             # <<<<<<<<<<<<<<
 *         rv.stride = self.stride + other.stride - 4
 *         rv.attributes = { k : v + self.stride - 4 for k, v in other.attributes.iteritems() }
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 755, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":756
 * 
 *         rv = Mesh()
 *         rv.stride = self.stride + other.stride - 4             # <<<<<<<<<<<<<<
 *         rv.attributes = { k : v + self.stride - 4 for k, v in other.attributes.iteritems() }
 *         rv.attributes.update(self.attributes)
 */
  __pyx_v_rv->stride = ((__pyx_v_self->stride + __pyx_v_other->stride) - 4);

  /* "renpy/gl2/gl2geometry.pyx":757
 *         rv = Mesh()
 *         rv.stride = self.stride + other.stride - 4
 *         rv.attributes = { k : v + self.stride - 4 for k, v in other.attributes.iteritems() }             # <<<<<<<<<<<<<<
 *         rv.attributes.update(self.attributes)
 * 
 */
  { /* enter inner scope */
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 757, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = 0;
    if (unlikely(__pyx_v_other->attributes == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "iteritems");
      __PYX_ERR(0, 757, __pyx_L5_error)
    }
    __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_other->attributes, 1, __pyx_n_s_iteritems, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 757, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
    while (1) {
      __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_7, &__pyx_t_6, &__pyx_t_3, &__pyx_t_5, NULL, __pyx_t_8);
      if (unlikely(__pyx_t_9 == 0)) break;
      if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 757, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_k, __pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_7genexpr__pyx_v_v, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 757, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = PyNumber_Add(__pyx_7genexpr__pyx_v_v, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 757, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyInt_SubtractObjC(__pyx_t_3, __pyx_int_4, 4, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 757, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(PyDict_SetItem(__pyx_t_1, (PyObject*)__pyx_7genexpr__pyx_v_k, (PyObject*)__pyx_t_5))) __PYX_ERR(0, 757, __pyx_L5_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_k); __pyx_7genexpr__pyx_v_k = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_v); __pyx_7genexpr__pyx_v_v = 0;
    goto __pyx_L8_exit_scope;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_k); __pyx_7genexpr__pyx_v_k = 0;
    __Pyx_XDECREF(__pyx_7genexpr__pyx_v_v); __pyx_7genexpr__pyx_v_v = 0;
    goto __pyx_L1_error;
    __pyx_L8_exit_scope:;
  } /* exit inner scope */
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_rv->attributes);
  __Pyx_DECREF(__pyx_v_rv->attributes);
  __pyx_v_rv->attributes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":758
 *         rv.stride = self.stride + other.stride - 4
 *         rv.attributes = { k : v + self.stride - 4 for k, v in other.attributes.iteritems() }
 *         rv.attributes.update(self.attributes)             # <<<<<<<<<<<<<<
 * 
 *         cdef Polygon op
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_rv->attributes, __pyx_n_s_update); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self->attributes); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_self->attributes};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_self->attributes};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 758, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_INCREF(__pyx_v_self->attributes);
      __Pyx_GIVEREF(__pyx_v_self->attributes);
      PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_v_self->attributes);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 758, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":764
 *         cdef Polygon p
 * 
 *         for op in other.polygons:             # <<<<<<<<<<<<<<
 *             for sp in self.polygons:
 *                 p = intersect(op, sp, rv.stride, True)
 */
  if (unlikely(__pyx_v_other->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 764, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_other->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
  for (;;) {
    if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 764, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 764, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 764, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_op, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":765
 * 
 *         for op in other.polygons:
 *             for sp in self.polygons:             # <<<<<<<<<<<<<<
 *                 p = intersect(op, sp, rv.stride, True)
 * 
 */
    if (unlikely(__pyx_v_self->polygons == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 765, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_3); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 765, __pyx_L1_error)
      #else
      __pyx_t_3 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 765, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 765, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_sp, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "renpy/gl2/gl2geometry.pyx":766
 *         for op in other.polygons:
 *             for sp in self.polygons:
 *                 p = intersect(op, sp, rv.stride, True)             # <<<<<<<<<<<<<<
 * 
 *                 if p is None:
 */
      __pyx_t_3 = __pyx_f_5renpy_3gl2_11gl2geometry_intersect(__pyx_v_op, __pyx_v_sp, __pyx_v_rv->stride, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_p, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "renpy/gl2/gl2geometry.pyx":768
 *                 p = intersect(op, sp, rv.stride, True)
 * 
 *                 if p is None:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      __pyx_t_10 = (((PyObject *)__pyx_v_p) == Py_None);
      __pyx_t_11 = (__pyx_t_10 != 0);
      if (__pyx_t_11) {

        /* "renpy/gl2/gl2geometry.pyx":769
 * 
 *                 if p is None:
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 # We only need to interpolate OP if it has something other than
 */
        goto __pyx_L11_continue;

        /* "renpy/gl2/gl2geometry.pyx":768
 *                 p = intersect(op, sp, rv.stride, True)
 * 
 *                 if p is None:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      }

      /* "renpy/gl2/gl2geometry.pyx":773
 *                 # We only need to interpolate OP if it has something other than
 *                 # position data.
 *                 if op.stride > 4:             # <<<<<<<<<<<<<<
 *                     barycentric(op, p, self.stride - 4)
 * 
 */
      __pyx_t_11 = ((__pyx_v_op->stride > 4) != 0);
      if (__pyx_t_11) {

        /* "renpy/gl2/gl2geometry.pyx":774
 *                 # position data.
 *                 if op.stride > 4:
 *                     barycentric(op, p, self.stride - 4)             # <<<<<<<<<<<<<<
 * 
 *                 barycentric(sp, p, 0)
 */
        __pyx_t_3 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_op, __pyx_v_p, (__pyx_v_self->stride - 4), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 774, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "renpy/gl2/gl2geometry.pyx":773
 *                 # We only need to interpolate OP if it has something other than
 *                 # position data.
 *                 if op.stride > 4:             # <<<<<<<<<<<<<<
 *                     barycentric(op, p, self.stride - 4)
 * 
 */
      }

      /* "renpy/gl2/gl2geometry.pyx":776
 *                     barycentric(op, p, self.stride - 4)
 * 
 *                 barycentric(sp, p, 0)             # <<<<<<<<<<<<<<
 * 
 *                 rv.polygons.append(p)
 */
      __pyx_t_3 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_sp, __pyx_v_p, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 776, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "renpy/gl2/gl2geometry.pyx":778
 *                 barycentric(sp, p, 0)
 * 
 *                 rv.polygons.append(p)             # <<<<<<<<<<<<<<
 *                 rv.points += p.points
 * 
 */
      if (unlikely(__pyx_v_rv->polygons == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
        __PYX_ERR(0, 778, __pyx_L1_error)
      }
      __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_rv->polygons, ((PyObject *)__pyx_v_p)); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 778, __pyx_L1_error)

      /* "renpy/gl2/gl2geometry.pyx":779
 * 
 *                 rv.polygons.append(p)
 *                 rv.points += p.points             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
      __pyx_v_rv->points = (__pyx_v_rv->points + __pyx_v_p->points);

      /* "renpy/gl2/gl2geometry.pyx":765
 * 
 *         for op in other.polygons:
 *             for sp in self.polygons:             # <<<<<<<<<<<<<<
 *                 p = intersect(op, sp, rv.stride, True)
 * 
 */
      __pyx_L11_continue:;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":764
 *         cdef Polygon p
 * 
 *         for op in other.polygons:             # <<<<<<<<<<<<<<
 *             for sp in self.polygons:
 *                 p = intersect(op, sp, rv.stride, True)
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":781
 *                 rv.points += p.points
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh crop(Mesh self, Polygon op):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":747
 *         return NotImplemented
 * 
 *     cpdef Mesh intersect(Mesh self, Mesh other):             # <<<<<<<<<<<<<<
 *         """
 *         Intersects this mesh with another mesh, and returns a new mesh that
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XDECREF((PyObject *)__pyx_v_op);
  __Pyx_XDECREF((PyObject *)__pyx_v_sp);
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_k);
  __Pyx_XDECREF(__pyx_7genexpr__pyx_v_v);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_31intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_30intersect[] = "\n        Intersects this mesh with another mesh, and returns a new mesh that\n        is the result. The resulting mesh has z- and w-coordinates from this\n        mesh. Attributes that are present in this mesh are taken from\n        this mesh, otherwise the attributes from the other mesh are used.\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_31intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("intersect (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh, 1, "other", 0))) __PYX_ERR(0, 747, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_30intersect(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_30intersect(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("intersect", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_intersect(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":783
 *         return rv
 * 
 *     cpdef Mesh crop(Mesh self, Polygon op):             # <<<<<<<<<<<<<<
 *         """
 *         This uses the polygon `op` to crop this mesh.
 */

static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_33crop(PyObject *__pyx_v_self, PyObject *__pyx_v_op); /*proto*/
static struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_crop(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_op, int __pyx_skip_dispatch) {
  int __pyx_v_same;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_sp = 0;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_p = 0;
  PyObject *__pyx_v_polygons = 0;
  int __pyx_v_points;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_rv = NULL;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("crop", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_crop); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 783, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_33crop)) {
      __Pyx_XDECREF(((PyObject *)__pyx_r));
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_op)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 783, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_op)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 783, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_op)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 783, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 783, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_INCREF(((PyObject *)__pyx_v_op));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_op));
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)__pyx_v_op));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 783, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh))))) __PYX_ERR(0, 783, __pyx_L1_error)
      __pyx_r = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl2/gl2geometry.pyx":791
 *         """
 * 
 *         same = True             # <<<<<<<<<<<<<<
 * 
 *         cdef Polygon sp
 */
  __pyx_v_same = 1;

  /* "renpy/gl2/gl2geometry.pyx":796
 *         cdef Polygon p
 * 
 *         cdef list polygons = [ ]             # <<<<<<<<<<<<<<
 *         cdef int points = 0
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_polygons = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":797
 * 
 *         cdef list polygons = [ ]
 *         cdef int points = 0             # <<<<<<<<<<<<<<
 * 
 *         for sp in self.polygons:
 */
  __pyx_v_points = 0;

  /* "renpy/gl2/gl2geometry.pyx":799
 *         cdef int points = 0
 * 
 *         for sp in self.polygons:             # <<<<<<<<<<<<<<
 *             p = intersect(op, sp, self.stride, False)
 * 
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 799, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
  for (;;) {
    if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 799, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_sp, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":800
 * 
 *         for sp in self.polygons:
 *             p = intersect(op, sp, self.stride, False)             # <<<<<<<<<<<<<<
 * 
 *             if p is None:
 */
    __pyx_t_2 = __pyx_f_5renpy_3gl2_11gl2geometry_intersect(__pyx_v_op, __pyx_v_sp, __pyx_v_self->stride, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 800, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon))))) __PYX_ERR(0, 800, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_p, ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl2/gl2geometry.pyx":802
 *             p = intersect(op, sp, self.stride, False)
 * 
 *             if p is None:             # <<<<<<<<<<<<<<
 *                 same = False
 *                 continue
 */
    __pyx_t_7 = (((PyObject *)__pyx_v_p) == Py_None);
    __pyx_t_8 = (__pyx_t_7 != 0);
    if (__pyx_t_8) {

      /* "renpy/gl2/gl2geometry.pyx":803
 * 
 *             if p is None:
 *                 same = False             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
      __pyx_v_same = 0;

      /* "renpy/gl2/gl2geometry.pyx":804
 *             if p is None:
 *                 same = False
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             if p is not sp:
 */
      goto __pyx_L3_continue;

      /* "renpy/gl2/gl2geometry.pyx":802
 *             p = intersect(op, sp, self.stride, False)
 * 
 *             if p is None:             # <<<<<<<<<<<<<<
 *                 same = False
 *                 continue
 */
    }

    /* "renpy/gl2/gl2geometry.pyx":806
 *                 continue
 * 
 *             if p is not sp:             # <<<<<<<<<<<<<<
 *                 same = False
 *                 barycentric(sp, p, 0)
 */
    __pyx_t_8 = (__pyx_v_p != __pyx_v_sp);
    __pyx_t_7 = (__pyx_t_8 != 0);
    if (__pyx_t_7) {

      /* "renpy/gl2/gl2geometry.pyx":807
 * 
 *             if p is not sp:
 *                 same = False             # <<<<<<<<<<<<<<
 *                 barycentric(sp, p, 0)
 * 
 */
      __pyx_v_same = 0;

      /* "renpy/gl2/gl2geometry.pyx":808
 *             if p is not sp:
 *                 same = False
 *                 barycentric(sp, p, 0)             # <<<<<<<<<<<<<<
 * 
 *             polygons.append(p)
 */
      __pyx_t_2 = __pyx_f_5renpy_3gl2_11gl2geometry_barycentric(__pyx_v_sp, __pyx_v_p, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "renpy/gl2/gl2geometry.pyx":806
 *                 continue
 * 
 *             if p is not sp:             # <<<<<<<<<<<<<<
 *                 same = False
 *                 barycentric(sp, p, 0)
 */
    }

    /* "renpy/gl2/gl2geometry.pyx":810
 *                 barycentric(sp, p, 0)
 * 
 *             polygons.append(p)             # <<<<<<<<<<<<<<
 *             points += p.points
 * 
 */
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_polygons, ((PyObject *)__pyx_v_p)); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 810, __pyx_L1_error)

    /* "renpy/gl2/gl2geometry.pyx":811
 * 
 *             polygons.append(p)
 *             points += p.points             # <<<<<<<<<<<<<<
 * 
 *         if same:
 */
    __pyx_v_points = (__pyx_v_points + __pyx_v_p->points);

    /* "renpy/gl2/gl2geometry.pyx":799
 *         cdef int points = 0
 * 
 *         for sp in self.polygons:             # <<<<<<<<<<<<<<
 *             p = intersect(op, sp, self.stride, False)
 * 
 */
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":813
 *             points += p.points
 * 
 *         if same:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  __pyx_t_7 = (__pyx_v_same != 0);
  if (__pyx_t_7) {

    /* "renpy/gl2/gl2geometry.pyx":814
 * 
 *         if same:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         rv = Mesh()
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = __pyx_v_self;
    goto __pyx_L0;

    /* "renpy/gl2/gl2geometry.pyx":813
 *             points += p.points
 * 
 *         if same:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  }

  /* "renpy/gl2/gl2geometry.pyx":816
 *             return self
 * 
 *         rv = Mesh()             # <<<<<<<<<<<<<<
 *         rv.stride = self.stride
 *         rv.attributes = self.attributes
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 816, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":817
 * 
 *         rv = Mesh()
 *         rv.stride = self.stride             # <<<<<<<<<<<<<<
 *         rv.attributes = self.attributes
 *         rv.polygons = polygons
 */
  __pyx_t_10 = __pyx_v_self->stride;
  __pyx_v_rv->stride = __pyx_t_10;

  /* "renpy/gl2/gl2geometry.pyx":818
 *         rv = Mesh()
 *         rv.stride = self.stride
 *         rv.attributes = self.attributes             # <<<<<<<<<<<<<<
 *         rv.polygons = polygons
 *         rv.points = points
 */
  __pyx_t_1 = __pyx_v_self->attributes;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_rv->attributes);
  __Pyx_DECREF(__pyx_v_rv->attributes);
  __pyx_v_rv->attributes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":819
 *         rv.stride = self.stride
 *         rv.attributes = self.attributes
 *         rv.polygons = polygons             # <<<<<<<<<<<<<<
 *         rv.points = points
 * 
 */
  __Pyx_INCREF(__pyx_v_polygons);
  __Pyx_GIVEREF(__pyx_v_polygons);
  __Pyx_GOTREF(__pyx_v_rv->polygons);
  __Pyx_DECREF(__pyx_v_rv->polygons);
  __pyx_v_rv->polygons = __pyx_v_polygons;

  /* "renpy/gl2/gl2geometry.pyx":820
 *         rv.attributes = self.attributes
 *         rv.polygons = polygons
 *         rv.points = points             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
  __pyx_v_rv->points = __pyx_v_points;

  /* "renpy/gl2/gl2geometry.pyx":822
 *         rv.points = points
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":783
 *         return rv
 * 
 *     cpdef Mesh crop(Mesh self, Polygon op):             # <<<<<<<<<<<<<<
 *         """
 *         This uses the polygon `op` to crop this mesh.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.crop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_sp);
  __Pyx_XDECREF((PyObject *)__pyx_v_p);
  __Pyx_XDECREF(__pyx_v_polygons);
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_33crop(PyObject *__pyx_v_self, PyObject *__pyx_v_op); /*proto*/
static char __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_32crop[] = "\n        This uses the polygon `op` to crop this mesh.\n\n        This may or may not return a new Mesh. (If op contains all of self, it\n        will return self.)\n        ";
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_33crop(PyObject *__pyx_v_self, PyObject *__pyx_v_op) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("crop (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_op), __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon, 1, "op", 0))) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_32crop(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)__pyx_v_op));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_32crop(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *__pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("crop", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_crop(__pyx_v_self, __pyx_v_op, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.crop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pyx":824
 *         return rv
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         rv = "<Mesh "
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_35__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_35__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_34__repr__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_34__repr__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_v_rv = NULL;
  PyObject *__pyx_v_p = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "renpy/gl2/gl2geometry.pyx":825
 * 
 *     def __repr__(self):
 *         rv = "<Mesh "             # <<<<<<<<<<<<<<
 * 
 *         for p in self.polygons:
 */
  __Pyx_INCREF(__pyx_kp_s_Mesh);
  __pyx_v_rv = __pyx_kp_s_Mesh;

  /* "renpy/gl2/gl2geometry.pyx":827
 *         rv = "<Mesh "
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             rv += repr(p)
 *             rv += ", "
 */
  if (unlikely(__pyx_v_self->polygons == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 827, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->polygons; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 827, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 827, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_p, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "renpy/gl2/gl2geometry.pyx":828
 * 
 *         for p in self.polygons:
 *             rv += repr(p)             # <<<<<<<<<<<<<<
 *             rv += ", "
 * 
 */
    __pyx_t_3 = PyObject_Repr(__pyx_v_p); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 828, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 828, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "renpy/gl2/gl2geometry.pyx":829
 *         for p in self.polygons:
 *             rv += repr(p)
 *             rv += ", "             # <<<<<<<<<<<<<<
 * 
 *         rv += ">"
 */
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_kp_s__10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "renpy/gl2/gl2geometry.pyx":827
 *         rv = "<Mesh "
 * 
 *         for p in self.polygons:             # <<<<<<<<<<<<<<
 *             rv += repr(p)
 *             rv += ", "
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":831
 *             rv += ", "
 * 
 *         rv += ">"             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
  __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_rv, __pyx_kp_s__3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_rv, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":832
 * 
 *         rv += ">"
 *         return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl2/gl2geometry.pyx":824
 *         return rv
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         rv = "<Mesh "
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_p);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pxd":35
 * 
 *     # The total number of points.
 *     cdef public int points             # <<<<<<<<<<<<<<
 * 
 *     # The stride - the amount of data per point, in floats.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6points_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6points_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6points___get__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6points___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->points); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.points.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6points_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6points_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6points_2__set__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6points_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 35, __pyx_L1_error)
  __pyx_v_self->points = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.points.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pxd":38
 * 
 *     # The stride - the amount of data per point, in floats.
 *     cdef public int stride             # <<<<<<<<<<<<<<
 * 
 *     # A map from an attribute to the offset of that attribute.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6stride_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6stride_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6stride___get__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6stride___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->stride); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.stride.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6stride_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6stride_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6stride_2__set__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_6stride_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 38, __pyx_L1_error)
  __pyx_v_self->stride = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.stride.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pxd":41
 * 
 *     # A map from an attribute to the offset of that attribute.
 *     cdef public dict attributes             # <<<<<<<<<<<<<<
 * 
 *     # A list of polygons that comprise the mesh.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes___get__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->attributes);
  __pyx_r = __pyx_v_self->attributes;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_2__set__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyDict_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 41, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->attributes);
  __Pyx_DECREF(__pyx_v_self->attributes);
  __pyx_v_self->attributes = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.attributes.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_4__del__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_4__del__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->attributes);
  __Pyx_DECREF(__pyx_v_self->attributes);
  __pyx_v_self->attributes = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pxd":44
 * 
 *     # A list of polygons that comprise the mesh.
 *     cdef public list polygons             # <<<<<<<<<<<<<<
 * 
 *     # An array that contains all the data from the points. This is NULL
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons___get__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->polygons);
  __pyx_r = __pyx_v_self->polygons;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_2__set__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(2, 44, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->polygons);
  __Pyx_DECREF(__pyx_v_self->polygons);
  __pyx_v_self->polygons = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.polygons.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_4__del__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_4__del__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->polygons);
  __Pyx_DECREF(__pyx_v_self->polygons);
  __pyx_v_self->polygons = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pxd":54
 *     # The count of polygons that comprise the mesh. This is  only valid after
 *     # get_data is called.
 *     cdef public int polygon_count             # <<<<<<<<<<<<<<
 * 
 *     # If all the polygons in the mesh have the same number of points, this is
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count___get__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->polygon_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.polygon_count.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_2__set__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 54, __pyx_L1_error)
  __pyx_v_self->polygon_count = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.polygon_count.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl2/gl2geometry.pxd":58
 *     # If all the polygons in the mesh have the same number of points, this is
 *     # that number of points. Else, it's 0.
 *     cdef public int polygon_points             # <<<<<<<<<<<<<<
 * 
 *     cpdef Mesh copy(Mesh self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points___get__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points___get__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->polygon_points); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.polygon_points.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_2__set__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_2__set__(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 58, __pyx_L1_error)
  __pyx_v_self->polygon_points = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.polygon_points.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_37__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_37__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_36__reduce_cython__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_36__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_39__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_39__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_38__setstate_cython__(((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_3gl2_11gl2geometry_4Mesh_38__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl2.gl2geometry.Mesh.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/display/matrix.pxd":26
 * 
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float *__pyx_v_ow, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform4", 0);

  /* "renpy/display/matrix.pxd":27
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":28
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":29
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw             # <<<<<<<<<<<<<<
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 */
  (__pyx_v_oz[0]) = ((((__pyx_v_x * __pyx_v_self->zdx) + (__pyx_v_y * __pyx_v_self->zdy)) + (__pyx_v_z * __pyx_v_self->zdz)) + (__pyx_v_w * __pyx_v_self->zdw));

  /* "renpy/display/matrix.pxd":30
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 */
  (__pyx_v_ow[0]) = ((((__pyx_v_x * __pyx_v_self->wdx) + (__pyx_v_y * __pyx_v_self->wdy)) + (__pyx_v_z * __pyx_v_self->wdz)) + (__pyx_v_w * __pyx_v_self->wdw));

  /* "renpy/display/matrix.pxd":26
 * 
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/display/matrix.pxd":32
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform3", 0);

  /* "renpy/display/matrix.pxd":33
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":34
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":35
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 */
  (__pyx_v_oz[0]) = ((((__pyx_v_x * __pyx_v_self->zdx) + (__pyx_v_y * __pyx_v_self->zdy)) + (__pyx_v_z * __pyx_v_self->zdz)) + (__pyx_v_w * __pyx_v_self->zdw));

  /* "renpy/display/matrix.pxd":32
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform2", 0);

  /* "renpy/display/matrix.pxd":38
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 * 
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":39
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_unit_aligned(Matrix self)
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}
static struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Polygon __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon;

static PyObject *__pyx_tp_new_5renpy_3gl2_11gl2geometry_Polygon(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *)o);
  p->__pyx_vtab = __pyx_vtabptr_5renpy_3gl2_11gl2geometry_Polygon;
  return o;
}

static void __pyx_tp_dealloc_5renpy_3gl2_11gl2geometry_Polygon(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_5renpy_3gl2_11gl2geometry_Polygon[] = {
  {"copy", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_5copy, METH_NOARGS, __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_4copy},
  {"offset_inplace", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_7offset_inplace, METH_VARARGS|METH_KEYWORDS, 0},
  {"offset", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_9offset, METH_VARARGS|METH_KEYWORDS, 0},
  {"perspective_divide_inplace", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_11perspective_divide_inplace, METH_NOARGS, __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_10perspective_divide_inplace},
  {"perspective_divide", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_13perspective_divide, METH_NOARGS, __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_12perspective_divide},
  {"multiply_matrix_inplace", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_15multiply_matrix_inplace, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_14multiply_matrix_inplace},
  {"multiply_matrix", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_17multiply_matrix, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_16multiply_matrix},
  {"__rmul__", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_21__rmul__, METH_O, 0},
  {"intersect", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_23intersect, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon_22intersect},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_27__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_29__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_Polygon = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_19__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_5renpy_3gl2_11gl2geometry_Polygon = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.gl2geometry.Polygon", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_11gl2geometry_Polygon, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_25__repr__, /*tp_repr*/
  &__pyx_tp_as_number_Polygon, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_11gl2geometry_Polygon, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_11gl2geometry_7Polygon_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_11gl2geometry_Polygon, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_3gl2_11gl2geometry_Mesh __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh;

static PyObject *__pyx_tp_new_5renpy_3gl2_11gl2geometry_Mesh(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)o);
  p->__pyx_vtab = __pyx_vtabptr_5renpy_3gl2_11gl2geometry_Mesh;
  p->attributes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->polygons = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5renpy_3gl2_11gl2geometry_Mesh(PyObject *o) {
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *p = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->attributes);
  Py_CLEAR(p->polygons);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5renpy_3gl2_11gl2geometry_Mesh(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *p = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)o;
  if (p->attributes) {
    e = (*v)(p->attributes, a); if (e) return e;
  }
  if (p->polygons) {
    e = (*v)(p->polygons, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_3gl2_11gl2geometry_Mesh(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *p = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *)o;
  tmp = ((PyObject*)p->attributes);
  p->attributes = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->polygons);
  p->polygons = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_points(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6points_1__get__(o);
}

static int __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_points(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6points_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_stride(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6stride_1__get__(o);
}

static int __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_stride(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_6stride_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_attributes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_1__get__(o);
}

static int __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_attributes(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_10attributes_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_polygons(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_1__get__(o);
}

static int __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_polygons(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_8polygons_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_count(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_1__get__(o);
}

static int __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_count(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13polygon_count_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_points(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_1__get__(o);
}

static int __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_points(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_14polygon_points_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5renpy_3gl2_11gl2geometry_Mesh[] = {
  {"add_attribute", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_5add_attribute, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_4add_attribute},
  {"add_polygon", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_7add_polygon, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_6add_polygon},
  {"add_rectangle", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_9add_rectangle, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_8add_rectangle},
  {"add_texture_rectangle", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_11add_texture_rectangle, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_10add_texture_rectangle},
  {"copy", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_13copy, METH_NOARGS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_12copy},
  {"offset_inplace", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_15offset_inplace, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_14offset_inplace},
  {"offset", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_17offset, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_16offset},
  {"perspective_divide_inplace", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_19perspective_divide_inplace, METH_NOARGS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_18perspective_divide_inplace},
  {"perspective_divide", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_21perspective_divide, METH_NOARGS, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_20perspective_divide},
  {"multiply_matrix_inplace", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_23multiply_matrix_inplace, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_22multiply_matrix_inplace},
  {"multiply_matrix", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_25multiply_matrix, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_24multiply_matrix},
  {"__rmul__", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_29__rmul__, METH_O, 0},
  {"intersect", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_31intersect, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_30intersect},
  {"crop", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_33crop, METH_O, __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh_32crop},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_37__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_39__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5renpy_3gl2_11gl2geometry_Mesh[] = {
  {(char *)"points", __pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_points, __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_points, (char *)0, 0},
  {(char *)"stride", __pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_stride, __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_stride, (char *)0, 0},
  {(char *)"attributes", __pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_attributes, __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_attributes, (char *)0, 0},
  {(char *)"polygons", __pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_polygons, __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_polygons, (char *)0, 0},
  {(char *)"polygon_count", __pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_count, __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_count, (char *)0, 0},
  {(char *)"polygon_points", __pyx_getprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_points, __pyx_setprop_5renpy_3gl2_11gl2geometry_4Mesh_polygon_points, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyNumberMethods __pyx_tp_as_number_Mesh = {
  0, /*nb_add*/
  0, /*nb_subtract*/
  __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_27__mul__, /*nb_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_divide*/
  #endif
  0, /*nb_remainder*/
  0, /*nb_divmod*/
  0, /*nb_power*/
  0, /*nb_negative*/
  0, /*nb_positive*/
  0, /*nb_absolute*/
  0, /*nb_nonzero*/
  0, /*nb_invert*/
  0, /*nb_lshift*/
  0, /*nb_rshift*/
  0, /*nb_and*/
  0, /*nb_xor*/
  0, /*nb_or*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_coerce*/
  #endif
  0, /*nb_int*/
  #if PY_MAJOR_VERSION < 3
  0, /*nb_long*/
  #else
  0, /*reserved*/
  #endif
  0, /*nb_float*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_oct*/
  #endif
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_hex*/
  #endif
  0, /*nb_inplace_add*/
  0, /*nb_inplace_subtract*/
  0, /*nb_inplace_multiply*/
  #if PY_MAJOR_VERSION < 3 || (CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03050000)
  0, /*nb_inplace_divide*/
  #endif
  0, /*nb_inplace_remainder*/
  0, /*nb_inplace_power*/
  0, /*nb_inplace_lshift*/
  0, /*nb_inplace_rshift*/
  0, /*nb_inplace_and*/
  0, /*nb_inplace_xor*/
  0, /*nb_inplace_or*/
  0, /*nb_floor_divide*/
  0, /*nb_true_divide*/
  0, /*nb_inplace_floor_divide*/
  0, /*nb_inplace_true_divide*/
  0, /*nb_index*/
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_matrix_multiply*/
  #endif
  #if PY_VERSION_HEX >= 0x03050000
  0, /*nb_inplace_matrix_multiply*/
  #endif
};

static PyTypeObject __pyx_type_5renpy_3gl2_11gl2geometry_Mesh = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl2.gl2geometry.Mesh", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_3gl2_11gl2geometry_Mesh, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_35__repr__, /*tp_repr*/
  &__pyx_tp_as_number_Mesh, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5renpy_3gl2_11gl2geometry_Mesh, /*tp_traverse*/
  __pyx_tp_clear_5renpy_3gl2_11gl2geometry_Mesh, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_3gl2_11gl2geometry_Mesh, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5renpy_3gl2_11gl2geometry_Mesh, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_3gl2_11gl2geometry_4Mesh_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_3gl2_11gl2geometry_Mesh, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"rectangle", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_1rectangle, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_rectangle},
  {"texture_rectangle", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_3texture_rectangle, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_2texture_rectangle},
  {"intersect", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_5intersect, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_4intersect},
  {"barycentric", (PyCFunction)__pyx_pw_5renpy_3gl2_11gl2geometry_7barycentric, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_3gl2_11gl2geometry_6barycentric},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_gl2geometry(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_gl2geometry},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "gl2geometry",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_Mesh, __pyx_k_Mesh, sizeof(__pyx_k_Mesh), 0, 0, 1, 0},
  {&__pyx_n_s_NotImplemented, __pyx_k_NotImplemented, sizeof(__pyx_k_NotImplemented), 0, 0, 1, 1},
  {&__pyx_kp_s_Polygon, __pyx_k_Polygon, sizeof(__pyx_k_Polygon), 0, 0, 1, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 1, 0},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_aPosition, __pyx_k_aPosition, sizeof(__pyx_k_aPosition), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_barycentric_point, __pyx_k_barycentric_point, sizeof(__pyx_k_barycentric_point), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_crop, __pyx_k_crop, sizeof(__pyx_k_crop), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_kp_s_g, __pyx_k_g, sizeof(__pyx_k_g), 0, 0, 1, 0},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_kp_s_home_tom_ab_renpy_renpy_gl2_gl2, __pyx_k_home_tom_ab_renpy_renpy_gl2_gl2, sizeof(__pyx_k_home_tom_ab_renpy_renpy_gl2_gl2), 0, 0, 1, 0},
  {&__pyx_n_s_intersect, __pyx_k_intersect, sizeof(__pyx_k_intersect), 0, 0, 1, 1},
  {&__pyx_n_s_iteritems, __pyx_k_iteritems, sizeof(__pyx_k_iteritems), 0, 0, 1, 1},
  {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_multiply_matrix, __pyx_k_multiply_matrix, sizeof(__pyx_k_multiply_matrix), 0, 0, 1, 1},
  {&__pyx_n_s_multiply_matrix_inplace, __pyx_k_multiply_matrix_inplace, sizeof(__pyx_k_multiply_matrix_inplace), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new_stride, __pyx_k_new_stride, sizeof(__pyx_k_new_stride), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_offset_inplace, __pyx_k_offset_inplace, sizeof(__pyx_k_offset_inplace), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_pb, __pyx_k_pb, sizeof(__pyx_k_pb), 0, 0, 1, 1},
  {&__pyx_n_s_perspective_divide, __pyx_k_perspective_divide, sizeof(__pyx_k_perspective_divide), 0, 0, 1, 1},
  {&__pyx_n_s_perspective_divide_inplace, __pyx_k_perspective_divide_inplace, sizeof(__pyx_k_perspective_divide_inplace), 0, 0, 1, 1},
  {&__pyx_n_s_pl, __pyx_k_pl, sizeof(__pyx_k_pl), 0, 0, 1, 1},
  {&__pyx_n_s_points, __pyx_k_points, sizeof(__pyx_k_points), 0, 0, 1, 1},
  {&__pyx_n_s_pr, __pyx_k_pr, sizeof(__pyx_k_pr), 0, 0, 1, 1},
  {&__pyx_n_s_pt, __pyx_k_pt, sizeof(__pyx_k_pt), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_gl2_gl2geometry, __pyx_k_renpy_gl2_gl2geometry, sizeof(__pyx_k_renpy_gl2_gl2geometry), 0, 0, 1, 1},
  {&__pyx_kp_s_self_data_cannot_be_converted_to, __pyx_k_self_data_cannot_be_converted_to, sizeof(__pyx_k_self_data_cannot_be_converted_to), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_stride, __pyx_k_stride, sizeof(__pyx_k_stride), 0, 0, 1, 1},
  {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
  {&__pyx_n_s_tb, __pyx_k_tb, sizeof(__pyx_k_tb), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_tl, __pyx_k_tl, sizeof(__pyx_k_tl), 0, 0, 1, 1},
  {&__pyx_n_s_tr, __pyx_k_tr, sizeof(__pyx_k_tr), 0, 0, 1, 1},
  {&__pyx_n_s_tt, __pyx_k_tt, sizeof(__pyx_k_tt), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_z, __pyx_k_z, sizeof(__pyx_k_z), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_NotImplemented = __Pyx_GetBuiltinName(__pyx_n_s_NotImplemented); if (!__pyx_builtin_NotImplemented) __PYX_ERR(0, 132, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(1, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "renpy/gl2/gl2geometry.pyx":178
 *     """
 * 
 *     cdef Polygon rv = Polygon(4, 4, None)             # <<<<<<<<<<<<<<
 * 
 *     rv.points = 4
 */
  __pyx_tuple__6 = PyTuple_Pack(3, __pyx_int_4, __pyx_int_4, Py_None); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "renpy/gl2/gl2geometry.pyx":219
 *     """
 * 
 *     cdef Polygon rv = Polygon(6, 4, None)             # <<<<<<<<<<<<<<
 * 
 *     rv.points = 4
 */
  __pyx_tuple__7 = PyTuple_Pack(3, __pyx_int_6, __pyx_int_4, Py_None); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "renpy/gl2/gl2geometry.pyx":526
 *     """
 * 
 *     cdef Polygon b = Polygon(6, 1, None)             # <<<<<<<<<<<<<<
 *     b.points = 1
 * 
 */
  __pyx_tuple__8 = PyTuple_Pack(3, __pyx_int_6, __pyx_int_1, Py_None); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "renpy/gl2/gl2geometry.pyx":538
 * 
 *     if p[W] == 0:
 *         return None, None             # <<<<<<<<<<<<<<
 * 
 *     return (p[TX], p[TY])
 */
  __pyx_tuple__9 = PyTuple_Pack(2, Py_None, Py_None); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "(tree fragment)":4
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.data cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_self_data_cannot_be_converted_to); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "renpy/gl2/gl2geometry.pyx":512
 * 
 * 
 * def barycentric_point(Polygon a, float x, float y):             # <<<<<<<<<<<<<<
 *     """
 *     This expects Polygon a to be a polygon with a stride of 6, with a
 */
  __pyx_tuple__13 = PyTuple_Pack(5, __pyx_n_s_a, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_b, __pyx_n_s_p); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl2_gl2, __pyx_n_s_barycentric_point, 512, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_00001 = PyFloat_FromDouble(0.00001); if (unlikely(!__pyx_float_0_00001)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initgl2geometry(void); /*proto*/
PyMODINIT_FUNC initgl2geometry(void)
#else
PyMODINIT_FUNC PyInit_gl2geometry(void); /*proto*/
PyMODINIT_FUNC PyInit_gl2geometry(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_gl2geometry(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_gl2geometry(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("gl2geometry", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__gl2__gl2geometry) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.gl2.gl2geometry")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.gl2.gl2geometry", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("rectangle", (void (*)(void))__pyx_f_5renpy_3gl2_11gl2geometry_rectangle, "struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(double, double, double, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("texture_rectangle", (void (*)(void))__pyx_f_5renpy_3gl2_11gl2geometry_texture_rectangle, "struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(double, double, double, double, double, double, double, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Type init code ---*/
  __pyx_vtabptr_5renpy_3gl2_11gl2geometry_Polygon = &__pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.copy = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_copy;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.offset_inplace = (void (*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, double, double, double, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset_inplace;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.offset = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, double, double, double, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_offset;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.multiply_matrix_inplace = (void (*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix_inplace;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.multiply_matrix = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_multiply_matrix;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.perspective_divide_inplace = (void (*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide_inplace;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.perspective_divide = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_perspective_divide;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Polygon.intersect = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_7Polygon_intersect;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_11gl2geometry_Polygon) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_11gl2geometry_Polygon.tp_print = 0;
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_5renpy_3gl2_11gl2geometry_Polygon, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 19, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_5renpy_3gl2_11gl2geometry_7Polygon___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_5renpy_3gl2_11gl2geometry_7Polygon___init__.doc = __pyx_doc_5renpy_3gl2_11gl2geometry_7Polygon___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_5renpy_3gl2_11gl2geometry_7Polygon___init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_11gl2geometry_Polygon.tp_dict, __pyx_vtabptr_5renpy_3gl2_11gl2geometry_Polygon) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Polygon", (PyObject *)&__pyx_type_5renpy_3gl2_11gl2geometry_Polygon) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_11gl2geometry_Polygon) < 0) __PYX_ERR(0, 19, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_11gl2geometry_Polygon = &__pyx_type_5renpy_3gl2_11gl2geometry_Polygon;
  __pyx_vtabptr_5renpy_3gl2_11gl2geometry_Mesh = &__pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.get_data = (float *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_get_data;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.copy = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_copy;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.offset_inplace = (void (*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, double, double, double, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset_inplace;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.offset = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, double, double, double, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_offset;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.multiply_matrix_inplace = (void (*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix_inplace;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.multiply_matrix = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_multiply_matrix;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.perspective_divide_inplace = (void (*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide_inplace;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.perspective_divide = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_perspective_divide;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.intersect = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_intersect;
  __pyx_vtable_5renpy_3gl2_11gl2geometry_Mesh.crop = (struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *(*)(struct __pyx_obj_5renpy_3gl2_11gl2geometry_Mesh *, struct __pyx_obj_5renpy_3gl2_11gl2geometry_Polygon *, int __pyx_skip_dispatch))__pyx_f_5renpy_3gl2_11gl2geometry_4Mesh_crop;
  if (PyType_Ready(&__pyx_type_5renpy_3gl2_11gl2geometry_Mesh) < 0) __PYX_ERR(0, 543, __pyx_L1_error)
  __pyx_type_5renpy_3gl2_11gl2geometry_Mesh.tp_print = 0;
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_5renpy_3gl2_11gl2geometry_Mesh, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 543, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_5renpy_3gl2_11gl2geometry_4Mesh___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_5renpy_3gl2_11gl2geometry_4Mesh___init__.doc = __pyx_doc_5renpy_3gl2_11gl2geometry_4Mesh___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_5renpy_3gl2_11gl2geometry_4Mesh___init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_5renpy_3gl2_11gl2geometry_Mesh.tp_dict, __pyx_vtabptr_5renpy_3gl2_11gl2geometry_Mesh) < 0) __PYX_ERR(0, 543, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Mesh", (PyObject *)&__pyx_type_5renpy_3gl2_11gl2geometry_Mesh) < 0) __PYX_ERR(0, 543, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_3gl2_11gl2geometry_Mesh) < 0) __PYX_ERR(0, 543, __pyx_L1_error)
  __pyx_ptype_5renpy_3gl2_11gl2geometry_Mesh = &__pyx_type_5renpy_3gl2_11gl2geometry_Mesh;
  /*--- Type import code ---*/
  __pyx_ptype_5renpy_7display_6matrix_Matrix = __Pyx_ImportType("renpy.display.matrix", "Matrix", sizeof(struct __pyx_obj_5renpy_7display_6matrix_Matrix), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6matrix_Matrix)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6matrix_Matrix = (struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6matrix_Matrix->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6matrix_Matrix)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_ptype_5renpy_7display_6matrix_Matrix2D = __Pyx_ImportType("renpy.display.matrix", "Matrix2D", sizeof(struct __pyx_obj_5renpy_7display_6matrix_Matrix2D), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6matrix_Matrix2D)) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6matrix_Matrix2D = (struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6matrix_Matrix2D->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6matrix_Matrix2D)) __PYX_ERR(3, 43, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "renpy/gl2/gl2geometry.pyx":512
 * 
 * 
 * def barycentric_point(Polygon a, float x, float y):             # <<<<<<<<<<<<<<
 *     """
 *     This expects Polygon a to be a polygon with a stride of 6, with a
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5renpy_3gl2_11gl2geometry_9barycentric_point, NULL, __pyx_n_s_renpy_gl2_gl2geometry); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_barycentric_point, __pyx_t_1) < 0) __PYX_ERR(0, 512, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl2/gl2geometry.pyx":1
 * from __future__ import print_function             # <<<<<<<<<<<<<<
 * 
 * from renpy.display.matrix cimport Matrix
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.gl2.gl2geometry", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.gl2.gl2geometry");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
  static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* ArgTypeTest */
  static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* KeywordStringCheck */
  static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* None */
  static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* IterFinish */
  static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectCallMethod0 */
  static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto bad;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *function = PyMethod_GET_FUNCTION(method);
            result = __Pyx_PyObject_CallOneArg(function, self);
            Py_DECREF(method);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
  static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
  static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
  static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
  static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
  static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
  static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* PyIntBinop */
  #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* SetVTable */
  static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
  static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* GetVTable */
  static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* CLineInTraceback */
  #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
  static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* ModuleImport */
  #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
