/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__text__ftfont
#define __PYX_HAVE_API__renpy__text__ftfont
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include "SDL.h"
#include "pygame_sdl2/pygame_sdl2.h"
#include "stdlib.h"
#include "pyfreetype.h"
#include "stdint.h"
#include "ttgsubtable.h"
#include "ftsupport.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "stringsource",
  "ftfont.pyx",
  "textsupport.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5renpy_4text_11textsupport_Glyph;
struct __pyx_obj_5renpy_4text_11textsupport_Line;
struct __pyx_obj_5renpy_4text_6ftfont_FTFace;
struct __pyx_obj_5renpy_4text_6ftfont_FTFont;

/* "textsupport.pxd":22
 * # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 * cdef enum split_t:             # <<<<<<<<<<<<<<
 *     SPLIT_NONE
 *     SPLIT_BEFORE
 */
enum __pyx_t_5renpy_4text_11textsupport_split_t {
  __pyx_e_5renpy_4text_11textsupport_SPLIT_NONE,
  __pyx_e_5renpy_4text_11textsupport_SPLIT_BEFORE,
  __pyx_e_5renpy_4text_11textsupport_SPLIT_INSTEAD
};

/* "textsupport.pxd":27
 *     SPLIT_INSTEAD
 * 
 * cdef enum ruby_t:             # <<<<<<<<<<<<<<
 *     RUBY_NONE
 *     RUBY_BOTTOM
 */
enum __pyx_t_5renpy_4text_11textsupport_ruby_t {
  __pyx_e_5renpy_4text_11textsupport_RUBY_NONE,
  __pyx_e_5renpy_4text_11textsupport_RUBY_BOTTOM,
  __pyx_e_5renpy_4text_11textsupport_RUBY_TOP,
  __pyx_e_5renpy_4text_11textsupport_RUBY_ALT
};
struct __pyx_t_5renpy_4text_6ftfont_glyph_cache;

/* "renpy/text/ftfont.pyx":43
 * 
 * # Represents a cached glyph.
 * cdef struct glyph_cache:             # <<<<<<<<<<<<<<
 * 
 *     # The character we're caching. -1 if we're not representing a valid
 */
struct __pyx_t_5renpy_4text_6ftfont_glyph_cache {
  int index;
  int width;
  float advance;
  FT_Bitmap bitmap;
  int bitmap_left;
  int bitmap_top;
};

/* "textsupport.pxd":33
 *     RUBY_ALT
 * 
 * cdef class Glyph:             # <<<<<<<<<<<<<<
 * 
 *     # The character this glyph represents.
 */
struct __pyx_obj_5renpy_4text_11textsupport_Glyph {
  PyObject_HEAD
  short x;
  short y;
  short delta_x_offset;
  unsigned int character;
  enum __pyx_t_5renpy_4text_11textsupport_split_t split;
  enum __pyx_t_5renpy_4text_11textsupport_ruby_t ruby;
  short ascent;
  short line_spacing;
  float width;
  float advance;
  float time;
  short hyperlink;
  int draw;
};


/* "textsupport.pxd":73
 * 
 * 
 * cdef class Line:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_5renpy_4text_11textsupport_Line {
  PyObject_HEAD
  short y;
  short height;
  PyObject *glyphs;
  float max_time;
  int eop;
};


/* "renpy/text/ftfont.pyx":134
 * 
 * 
 * cdef class FTFace:             # <<<<<<<<<<<<<<
 *     """
 *     Represents a freetype face.
 */
struct __pyx_obj_5renpy_4text_6ftfont_FTFace {
  PyObject_HEAD
  FT_StreamRec stream;
  FT_Open_Args open_args;
  FT_Face face;
  float size;
  PyObject *f;
  unsigned long offset;
  PyObject *fn;
};


/* "renpy/text/ftfont.pyx":192
 *         self.size = -1
 * 
 * cdef class FTFont:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_5renpy_4text_6ftfont_FTFont {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *__pyx_vtab;
  struct __pyx_obj_5renpy_4text_6ftfont_FTFace *face_object;
  FT_Face face;
  TTGSUBTable gsubtable;
  float size;
  float bold;
  int italic;
  int outline;
  int antialias;
  int vertical;
  int underline_offset;
  int underline_height;
  FT_Stroker stroker;
  int expand;
  int ascent;
  int descent;
  int height;
  int lineskip;
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache cache[0x100];
  int has_setup;
  int hinting;
};



struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont {
  PyObject *(*setup)(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *);
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *(*get_glyph)(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *, int);
};
static struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *__pyx_vtabptr_5renpy_4text_6ftfont_FTFont;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* None.proto */
static CYTHON_INLINE FT_Pos __Pyx_div_FT_Pos(FT_Pos, FT_Pos);

/* None.proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* GetItemIntUnicode.proto */
#define __Pyx_GetItemInt_Unicode(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Unicode_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "string index out of range"), (Py_UCS4)-1))
static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck);

/* tp_new.proto */
#define __Pyx_tp_new(type_obj, args) __Pyx_tp_new_kwargs(type_obj, args, NULL)
static CYTHON_INLINE PyObject* __Pyx_tp_new_kwargs(PyObject* type_obj, PyObject* args, PyObject* kwargs) {
    return (PyObject*) (((PyTypeObject*)type_obj)->tp_new((PyTypeObject*)type_obj, args, kwargs));
}

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_long(unsigned long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT_Pos(FT_Pos value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned long __Pyx_PyInt_As_unsigned_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE FT_Long __Pyx_PyInt_As_FT_Long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_5renpy_4text_6ftfont_6FTFont_setup(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto*/
static struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_f_5renpy_4text_6ftfont_6FTFont_get_glyph(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, int __pyx_v_index); /* proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'sdl2' */

/* Module declarations from 'pygame_sdl2' */

/* Module declarations from 'freetype' */

/* Module declarations from 'ttgsubtable' */

/* Module declarations from 'renpy.text.textsupport' */
static PyTypeObject *__pyx_ptype_5renpy_4text_11textsupport_Glyph = 0;
static PyTypeObject *__pyx_ptype_5renpy_4text_11textsupport_Line = 0;

/* Module declarations from 'renpy.text.ftfont' */
static PyTypeObject *__pyx_ptype_5renpy_4text_6ftfont_FTFace = 0;
static PyTypeObject *__pyx_ptype_5renpy_4text_6ftfont_FTFont = 0;
static FT_Library __pyx_v_5renpy_4text_6ftfont_library;
static int __pyx_f_5renpy_4text_6ftfont_is_zerowidth(unsigned int); /*proto*/
static unsigned long __pyx_f_5renpy_4text_6ftfont_io_func(FT_Stream, unsigned long, unsigned char *, unsigned long); /*proto*/
static void __pyx_f_5renpy_4text_6ftfont_close_func(FT_Stream); /*proto*/
#define __Pyx_MODULE_NAME "renpy.text.ftfont"
extern int __pyx_module_is_main_renpy__text__ftfont;
int __pyx_module_is_main_renpy__text__ftfont = 0;

/* Implementation of 'renpy.text.ftfont' */
static PyObject *__pyx_builtin_TypeError;
static const char __pyx_k_f[] = "f";
static const char __pyx_k_fn[] = "fn";
static const char __pyx_k_xo[] = "xo";
static const char __pyx_k_yo[] = "yo";
static const char __pyx_k_d_s[] = "%d: %s";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_bold[] = "bold";
static const char __pyx_k_code[] = "code";
static const char __pyx_k_face[] = "face";
static const char __pyx_k_game[] = "game";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_none[] = "none";
static const char __pyx_k_read[] = "read";
static const char __pyx_k_seek[] = "seek";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_tell[] = "tell";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_color[] = "color";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_renpy[] = "renpy";
static const char __pyx_k_bounds[] = "bounds";
static const char __pyx_k_config[] = "config";
static const char __pyx_k_glyphs[] = "glyphs";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_init_2[] = "init";
static const char __pyx_k_italic[] = "italic";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_pysurf[] = "pysurf";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_hinting[] = "hinting";
static const char __pyx_k_outline[] = "outline";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_bytecode[] = "bytecode";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_vertical[] = "vertical";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_antialias[] = "antialias";
static const char __pyx_k_font_size[] = "font_size";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_print_exc[] = "print_exc";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_traceback[] = "traceback";
static const char __pyx_k_underline[] = "underline";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_black_color[] = "black_color";
static const char __pyx_k_preferences[] = "preferences";
static const char __pyx_k_ftfont_scale[] = "ftfont_scale";
static const char __pyx_k_renpy_config[] = "renpy.config";
static const char __pyx_k_FreetypeError[] = "FreetypeError";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_strikethrough[] = "strikethrough";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_font_line_spacing[] = "font_line_spacing";
static const char __pyx_k_renpy_text_ftfont[] = "renpy.text.ftfont";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_FreetypeError___init[] = "FreetypeError.__init__";
static const char __pyx_k_ftfont_vertical_extent_scale[] = "ftfont_vertical_extent_scale";
static const char __pyx_k_home_tom_ab_renpy_renpy_text_ft[] = "/home/tom/ab/renpy/renpy/text/ftfont.pyx";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_self_face_self_open_args_self_st[] = "self.face,self.open_args,self.stream cannot be converted to a Python object for pickling";
static PyObject *__pyx_n_s_FreetypeError;
static PyObject *__pyx_n_s_FreetypeError___init;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_antialias;
static PyObject *__pyx_n_s_black_color;
static PyObject *__pyx_n_s_bold;
static PyObject *__pyx_n_s_bounds;
static PyObject *__pyx_n_s_bytecode;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_code;
static PyObject *__pyx_n_s_color;
static PyObject *__pyx_n_s_config;
static PyObject *__pyx_kp_s_d_s;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_f;
static PyObject *__pyx_n_s_face;
static PyObject *__pyx_n_s_fn;
static PyObject *__pyx_n_s_font_line_spacing;
static PyObject *__pyx_n_s_font_size;
static PyObject *__pyx_n_s_ftfont_scale;
static PyObject *__pyx_n_s_ftfont_vertical_extent_scale;
static PyObject *__pyx_n_s_game;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_glyphs;
static PyObject *__pyx_n_s_hinting;
static PyObject *__pyx_kp_s_home_tom_ab_renpy_renpy_text_ft;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_init_2;
static PyObject *__pyx_n_s_italic;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_none;
static PyObject *__pyx_n_s_outline;
static PyObject *__pyx_n_s_preferences;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_print_exc;
static PyObject *__pyx_n_s_pysurf;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_read;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_renpy;
static PyObject *__pyx_n_s_renpy_config;
static PyObject *__pyx_n_s_renpy_text_ftfont;
static PyObject *__pyx_n_s_seek;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self_face_self_open_args_self_st;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_strikethrough;
static PyObject *__pyx_n_s_tell;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_traceback;
static PyObject *__pyx_n_s_underline;
static PyObject *__pyx_n_s_vertical;
static PyObject *__pyx_n_s_xo;
static PyObject *__pyx_n_s_yo;
static PyObject *__pyx_pf_5renpy_4text_6ftfont_13FreetypeError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_code); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_init(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFace___init__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self, PyObject *__pyx_v_f, PyObject *__pyx_v_index, PyObject *__pyx_v_fn); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn_4__del__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFace_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFace_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont___cinit__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static void __pyx_pf_5renpy_4text_6ftfont_6FTFont_2__dealloc__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_4__init__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_face, float __pyx_v_size, float __pyx_v_bold, int __pyx_v_italic, int __pyx_v_outline, int __pyx_v_antialias, int __pyx_v_vertical, PyObject *__pyx_v_hinting); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6glyphs(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_s); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_8bounds(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_glyphs, PyObject *__pyx_v_bounds); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_10draw(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_pysurf, float __pyx_v_xo, int __pyx_v_yo, PyObject *__pyx_v_color, PyObject *__pyx_v_glyphs, int __pyx_v_underline, int __pyx_v_strikethrough, CYTHON_UNUSED PyObject *__pyx_v_black_color); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_offset___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_offset_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_height___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_height_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6expand___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_6expand_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6ascent___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_6ascent_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_7descent___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_7descent_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6height___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_6height_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_8lineskip___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_8lineskip_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_5renpy_4text_6ftfont_FTFace(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_4text_6ftfont_FTFont(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_1_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_2;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_codeobj__8;
static PyObject *__pyx_codeobj__10;

/* "renpy/text/ftfont.pyx":58
 * 
 * class FreetypeError(Exception):
 *     def __init__(self, code):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_13FreetypeError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_4text_6ftfont_13FreetypeError_1__init__ = {"__init__", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_13FreetypeError_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_4text_6ftfont_13FreetypeError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_code = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_code,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_code)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(1, 58, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 58, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_code = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 58, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FreetypeError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_13FreetypeError___init__(__pyx_self, __pyx_v_self, __pyx_v_code);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_13FreetypeError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_code) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/text/ftfont.pyx":59
 * class FreetypeError(Exception):
 *     def __init__(self, code):
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))             # <<<<<<<<<<<<<<
 * 
 * def init():
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_v_code); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 59, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyBytes_FromString(freetype_error_to_string(__pyx_t_3)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_code);
  __Pyx_GIVEREF(__pyx_v_code);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_code);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_d_s, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_3 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_3 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_3, 2+__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_3, 2+__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 59, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self);
    __Pyx_GIVEREF(__pyx_v_self);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_3, __pyx_v_self);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_3, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 59, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":58
 * 
 * class FreetypeError(Exception):
 *     def __init__(self, code):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.text.ftfont.FreetypeError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":61
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 * def init():             # <<<<<<<<<<<<<<
 * 
 *     cdef int error
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_1init(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_4text_6ftfont_1init = {"init", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_1init, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_4text_6ftfont_1init(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("init (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_init(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_init(CYTHON_UNUSED PyObject *__pyx_self) {
  int __pyx_v_error;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("init", 0);

  /* "renpy/text/ftfont.pyx":65
 *     cdef int error
 * 
 *     error = FT_Init_FreeType(&library)             # <<<<<<<<<<<<<<
 *     if error:
 *         raise FreetypeError(error)
 */
  __pyx_v_error = FT_Init_FreeType((&__pyx_v_5renpy_4text_6ftfont_library));

  /* "renpy/text/ftfont.pyx":66
 * 
 *     error = FT_Init_FreeType(&library)
 *     if error:             # <<<<<<<<<<<<<<
 *         raise FreetypeError(error)
 * 
 */
  __pyx_t_1 = (__pyx_v_error != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":67
 *     error = FT_Init_FreeType(&library)
 *     if error:
 *         raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 * cdef bint is_zerowidth(unsigned int char):
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 67, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 67, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 67, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 67, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 67, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(1, 67, __pyx_L1_error)

    /* "renpy/text/ftfont.pyx":66
 * 
 *     error = FT_Init_FreeType(&library)
 *     if error:             # <<<<<<<<<<<<<<
 *         raise FreetypeError(error)
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":61
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 * def init():             # <<<<<<<<<<<<<<
 * 
 *     cdef int error
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.text.ftfont.init", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":69
 *         raise FreetypeError(error)
 * 
 * cdef bint is_zerowidth(unsigned int char):             # <<<<<<<<<<<<<<
 *     if char == 0x200b: # Zero-width space.
 *         return True
 */

static int __pyx_f_5renpy_4text_6ftfont_is_zerowidth(unsigned int __pyx_v_char) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("is_zerowidth", 0);

  /* "renpy/text/ftfont.pyx":70
 * 
 * cdef bint is_zerowidth(unsigned int char):
 *     if char == 0x200b: # Zero-width space.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_t_1 = ((__pyx_v_char == 0x200b) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":71
 * cdef bint is_zerowidth(unsigned int char):
 *     if char == 0x200b: # Zero-width space.
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     if char == 0x200c: # Zero-width non-joiner.
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":70
 * 
 * cdef bint is_zerowidth(unsigned int char):
 *     if char == 0x200b: # Zero-width space.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":73
 *         return True
 * 
 *     if char == 0x200c: # Zero-width non-joiner.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_t_1 = ((__pyx_v_char == 0x200c) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":74
 * 
 *     if char == 0x200c: # Zero-width non-joiner.
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     if char == 0x200d: # Zero-width joiner.
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":73
 *         return True
 * 
 *     if char == 0x200c: # Zero-width non-joiner.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":76
 *         return True
 * 
 *     if char == 0x200d: # Zero-width joiner.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_t_1 = ((__pyx_v_char == 0x200d) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":77
 * 
 *     if char == 0x200d: # Zero-width joiner.
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     if char == 0x2060: # Word joiner.
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":76
 *         return True
 * 
 *     if char == 0x200d: # Zero-width joiner.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":79
 *         return True
 * 
 *     if char == 0x2060: # Word joiner.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_t_1 = ((__pyx_v_char == 0x2060) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":80
 * 
 *     if char == 0x2060: # Word joiner.
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     if char == 0xfeff: # Zero width non-breaking space.
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":79
 *         return True
 * 
 *     if char == 0x2060: # Word joiner.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":82
 *         return True
 * 
 *     if char == 0xfeff: # Zero width non-breaking space.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  __pyx_t_1 = ((__pyx_v_char == 0xfeff) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":83
 * 
 *     if char == 0xfeff: # Zero width non-breaking space.
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     return False
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":82
 *         return True
 * 
 *     if char == 0xfeff: # Zero width non-breaking space.             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":85
 *         return True
 * 
 *     return False             # <<<<<<<<<<<<<<
 * 
 * cdef unsigned long io_func(FT_Stream stream, unsigned long offset, unsigned char *buffer, unsigned long count):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":69
 *         raise FreetypeError(error)
 * 
 * cdef bint is_zerowidth(unsigned int char):             # <<<<<<<<<<<<<<
 *     if char == 0x200b: # Zero-width space.
 *         return True
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":87
 *     return False
 * 
 * cdef unsigned long io_func(FT_Stream stream, unsigned long offset, unsigned char *buffer, unsigned long count):             # <<<<<<<<<<<<<<
 *     """
 *     Seeks to offset, and then reads count bytes from the stream into buffer.
 */

static unsigned long __pyx_f_5renpy_4text_6ftfont_io_func(FT_Stream __pyx_v_stream, unsigned long __pyx_v_offset, unsigned char *__pyx_v_buffer, unsigned long __pyx_v_count) {
  struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_face = 0;
  char *__pyx_v_cbuf;
  PyObject *__pyx_v_f = NULL;
  PyObject *__pyx_v_buf = NULL;
  long __pyx_v_i;
  unsigned long __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  char *__pyx_t_11;
  Py_ssize_t __pyx_t_12;
  unsigned long __pyx_t_13;
  __Pyx_RefNannySetupContext("io_func", 0);

  /* "renpy/text/ftfont.pyx":95
 *     cdef char *cbuf
 * 
 *     face = <FTFace> stream.descriptor.pointer             # <<<<<<<<<<<<<<
 *     f = face.f
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_stream->descriptor.pointer);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_face = ((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":96
 * 
 *     face = <FTFace> stream.descriptor.pointer
 *     f = face.f             # <<<<<<<<<<<<<<
 * 
 *     if face.offset != offset:
 */
  __pyx_t_1 = __pyx_v_face->f;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_f = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":98
 *     f = face.f
 * 
 *     if face.offset != offset:             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  __pyx_t_2 = ((__pyx_v_face->offset != __pyx_v_offset) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":100
 *     if face.offset != offset:
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             f.seek(offset)
 *             face.offset = offset
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "renpy/text/ftfont.pyx":101
 * 
 *         try:
 *             f.seek(offset)             # <<<<<<<<<<<<<<
 *             face.offset = offset
 *         except:
 */
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_seek); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 101, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = __Pyx_PyInt_From_unsigned_long(__pyx_v_offset); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 101, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        if (!__pyx_t_8) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L4_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_7};
            __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L4_error)
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_7};
            __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L4_error)
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          } else
          #endif
          {
            __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 101, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
            __Pyx_GIVEREF(__pyx_t_7);
            PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_7);
            __pyx_t_7 = 0;
            __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L4_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "renpy/text/ftfont.pyx":102
 *         try:
 *             f.seek(offset)
 *             face.offset = offset             # <<<<<<<<<<<<<<
 *         except:
 *             traceback.print_exc()
 */
        __pyx_v_face->offset = __pyx_v_offset;

        /* "renpy/text/ftfont.pyx":100
 *     if face.offset != offset:
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             f.seek(offset)
 *             face.offset = offset
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L9_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/text/ftfont.pyx":103
 *             f.seek(offset)
 *             face.offset = offset
 *         except:             # <<<<<<<<<<<<<<
 *             traceback.print_exc()
 *             return -1
 */
      /*except:*/ {
        __Pyx_AddTraceback("renpy.text.ftfont.io_func", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_6, &__pyx_t_9) < 0) __PYX_ERR(1, 103, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_t_9);

        /* "renpy/text/ftfont.pyx":104
 *             face.offset = offset
 *         except:
 *             traceback.print_exc()             # <<<<<<<<<<<<<<
 *             return -1
 * 
 */
        __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_traceback); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 104, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_print_exc); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 104, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        if (__pyx_t_8) {
          __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 104, __pyx_L6_except_error)
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else {
          __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 104, __pyx_L6_except_error)
        }
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "renpy/text/ftfont.pyx":105
 *         except:
 *             traceback.print_exc()
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *     if count != 0:
 */
        __pyx_r = -1L;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_except_return;
      }
      __pyx_L6_except_error:;

      /* "renpy/text/ftfont.pyx":100
 *     if face.offset != offset:
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             f.seek(offset)
 *             face.offset = offset
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L7_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L9_try_end:;
    }

    /* "renpy/text/ftfont.pyx":98
 *     f = face.f
 * 
 *     if face.offset != offset:             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  }

  /* "renpy/text/ftfont.pyx":107
 *             return -1
 * 
 *     if count != 0:             # <<<<<<<<<<<<<<
 *         try:
 *             buf = f.read(count)
 */
  __pyx_t_2 = ((__pyx_v_count != 0) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":108
 * 
 *     if count != 0:
 *         try:             # <<<<<<<<<<<<<<
 *             buf = f.read(count)
 *             cbuf = buf
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_4, &__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_3);
      /*try:*/ {

        /* "renpy/text/ftfont.pyx":109
 *     if count != 0:
 *         try:
 *             buf = f.read(count)             # <<<<<<<<<<<<<<
 *             cbuf = buf
 *             count = len(buf)
 */
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_read); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 109, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_1 = __Pyx_PyInt_From_unsigned_long(__pyx_v_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 109, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_6, function);
          }
        }
        if (!__pyx_t_7) {
          __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 109, __pyx_L13_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_GOTREF(__pyx_t_9);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_1};
            __pyx_t_9 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 109, __pyx_L13_error)
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
            PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_1};
            __pyx_t_9 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 109, __pyx_L13_error)
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          } else
          #endif
          {
            __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 109, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_10);
            __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_7); __pyx_t_7 = NULL;
            __Pyx_GIVEREF(__pyx_t_1);
            PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_1);
            __pyx_t_1 = 0;
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 109, __pyx_L13_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_v_buf = __pyx_t_9;
        __pyx_t_9 = 0;

        /* "renpy/text/ftfont.pyx":110
 *         try:
 *             buf = f.read(count)
 *             cbuf = buf             # <<<<<<<<<<<<<<
 *             count = len(buf)
 * 
 */
        __pyx_t_11 = __Pyx_PyObject_AsWritableString(__pyx_v_buf); if (unlikely((!__pyx_t_11) && PyErr_Occurred())) __PYX_ERR(1, 110, __pyx_L13_error)
        __pyx_v_cbuf = __pyx_t_11;

        /* "renpy/text/ftfont.pyx":111
 *             buf = f.read(count)
 *             cbuf = buf
 *             count = len(buf)             # <<<<<<<<<<<<<<
 * 
 *             for i from 0 <= i < count:
 */
        __pyx_t_12 = PyObject_Length(__pyx_v_buf); if (unlikely(__pyx_t_12 == ((Py_ssize_t)-1))) __PYX_ERR(1, 111, __pyx_L13_error)
        __pyx_v_count = __pyx_t_12;

        /* "renpy/text/ftfont.pyx":113
 *             count = len(buf)
 * 
 *             for i from 0 <= i < count:             # <<<<<<<<<<<<<<
 *                 buffer[i] = cbuf[i]
 *         except:
 */
        __pyx_t_13 = __pyx_v_count;
        for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_13; __pyx_v_i++) {

          /* "renpy/text/ftfont.pyx":114
 * 
 *             for i from 0 <= i < count:
 *                 buffer[i] = cbuf[i]             # <<<<<<<<<<<<<<
 *         except:
 *             traceback.print_exc()
 */
          (__pyx_v_buffer[__pyx_v_i]) = (__pyx_v_cbuf[__pyx_v_i]);
        }

        /* "renpy/text/ftfont.pyx":108
 * 
 *     if count != 0:
 *         try:             # <<<<<<<<<<<<<<
 *             buf = f.read(count)
 *             cbuf = buf
 */
      }
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L18_try_end;
      __pyx_L13_error:;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "renpy/text/ftfont.pyx":115
 *             for i from 0 <= i < count:
 *                 buffer[i] = cbuf[i]
 *         except:             # <<<<<<<<<<<<<<
 *             traceback.print_exc()
 *             return -1
 */
      /*except:*/ {
        __Pyx_AddTraceback("renpy.text.ftfont.io_func", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_9, &__pyx_t_6, &__pyx_t_10) < 0) __PYX_ERR(1, 115, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_t_10);

        /* "renpy/text/ftfont.pyx":116
 *                 buffer[i] = cbuf[i]
 *         except:
 *             traceback.print_exc()             # <<<<<<<<<<<<<<
 *             return -1
 * 
 */
        __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_traceback); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 116, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_print_exc); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 116, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_7 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_7)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_7);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
          }
        }
        if (__pyx_t_7) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 116, __pyx_L15_except_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else {
          __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 116, __pyx_L15_except_error)
        }
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "renpy/text/ftfont.pyx":117
 *         except:
 *             traceback.print_exc()
 *             return -1             # <<<<<<<<<<<<<<
 * 
 *     face.offset += count
 */
        __pyx_r = -1L;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        goto __pyx_L16_except_return;
      }
      __pyx_L15_except_error:;

      /* "renpy/text/ftfont.pyx":108
 * 
 *     if count != 0:
 *         try:             # <<<<<<<<<<<<<<
 *             buf = f.read(count)
 *             cbuf = buf
 */
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_4, __pyx_t_3);
      goto __pyx_L1_error;
      __pyx_L16_except_return:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_4, __pyx_t_3);
      goto __pyx_L0;
      __pyx_L18_try_end:;
    }

    /* "renpy/text/ftfont.pyx":107
 *             return -1
 * 
 *     if count != 0:             # <<<<<<<<<<<<<<
 *         try:
 *             buf = f.read(count)
 */
  }

  /* "renpy/text/ftfont.pyx":119
 *             return -1
 * 
 *     face.offset += count             # <<<<<<<<<<<<<<
 * 
 *     return count
 */
  __pyx_v_face->offset = (__pyx_v_face->offset + __pyx_v_count);

  /* "renpy/text/ftfont.pyx":121
 *     face.offset += count
 * 
 *     return count             # <<<<<<<<<<<<<<
 * 
 * cdef void close_func(FT_Stream stream):
 */
  __pyx_r = __pyx_v_count;
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":87
 *     return False
 * 
 * cdef unsigned long io_func(FT_Stream stream, unsigned long offset, unsigned char *buffer, unsigned long count):             # <<<<<<<<<<<<<<
 *     """
 *     Seeks to offset, and then reads count bytes from the stream into buffer.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_WriteUnraisable("renpy.text.ftfont.io_func", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_face);
  __Pyx_XDECREF(__pyx_v_f);
  __Pyx_XDECREF(__pyx_v_buf);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":123
 *     return count
 * 
 * cdef void close_func(FT_Stream stream):             # <<<<<<<<<<<<<<
 *     """
 *     Close the stream.
 */

static void __pyx_f_5renpy_4text_6ftfont_close_func(CYTHON_UNUSED FT_Stream __pyx_v_stream) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close_func", 0);

  /* "renpy/text/ftfont.pyx":131
 *     """
 * 
 *     return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":123
 *     return count
 * 
 * cdef void close_func(FT_Stream stream):             # <<<<<<<<<<<<<<
 *     """
 *     Close the stream.
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/text/ftfont.pyx":154
 *         public object fn
 * 
 *     def __init__(self, f, index, fn):             # <<<<<<<<<<<<<<
 * 
 *         cdef int error
 */

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFace_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFace_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_f = 0;
  PyObject *__pyx_v_index = 0;
  PyObject *__pyx_v_fn = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_f,&__pyx_n_s_index,&__pyx_n_s_fn,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_f)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(1, 154, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fn)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(1, 154, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 154, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_f = values[0];
    __pyx_v_index = values[1];
    __pyx_v_fn = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 154, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFace.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFace___init__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_v_self), __pyx_v_f, __pyx_v_index, __pyx_v_fn);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFace___init__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self, PyObject *__pyx_v_f, PyObject *__pyx_v_index, PyObject *__pyx_v_fn) {
  int __pyx_v_error;
  unsigned long __pyx_v_size;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  unsigned long __pyx_t_4;
  FT_Long __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/text/ftfont.pyx":160
 * 
 *         # The filename.
 *         self.fn = fn             # <<<<<<<<<<<<<<
 * 
 *         # The file that the font is opened from.
 */
  __Pyx_INCREF(__pyx_v_fn);
  __Pyx_GIVEREF(__pyx_v_fn);
  __Pyx_GOTREF(__pyx_v_self->fn);
  __Pyx_DECREF(__pyx_v_self->fn);
  __pyx_v_self->fn = __pyx_v_fn;

  /* "renpy/text/ftfont.pyx":163
 * 
 *         # The file that the font is opened from.
 *         self.f = f             # <<<<<<<<<<<<<<
 * 
 *         f.seek(0, 2)
 */
  __Pyx_INCREF(__pyx_v_f);
  __Pyx_GIVEREF(__pyx_v_f);
  __Pyx_GOTREF(__pyx_v_self->f);
  __Pyx_DECREF(__pyx_v_self->f);
  __pyx_v_self->f = __pyx_v_f;

  /* "renpy/text/ftfont.pyx":165
 *         self.f = f
 * 
 *         f.seek(0, 2)             # <<<<<<<<<<<<<<
 *         size = f.tell()
 *         f.seek(0, 0)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_seek); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/text/ftfont.pyx":166
 * 
 *         f.seek(0, 2)
 *         size = f.tell()             # <<<<<<<<<<<<<<
 *         f.seek(0, 0)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_tell); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 166, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 166, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyInt_As_unsigned_long(__pyx_t_2); if (unlikely((__pyx_t_4 == (unsigned long)-1) && PyErr_Occurred())) __PYX_ERR(1, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_size = __pyx_t_4;

  /* "renpy/text/ftfont.pyx":167
 *         f.seek(0, 2)
 *         size = f.tell()
 *         f.seek(0, 0)             # <<<<<<<<<<<<<<
 * 
 *         # The offset within the stream we're currently at.
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_f, __pyx_n_s_seek); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":170
 * 
 *         # The offset within the stream we're currently at.
 *         self.offset = 0             # <<<<<<<<<<<<<<
 * 
 *         self.open_args.flags = FT_OPEN_STREAM
 */
  __pyx_v_self->offset = 0;

  /* "renpy/text/ftfont.pyx":172
 *         self.offset = 0
 * 
 *         self.open_args.flags = FT_OPEN_STREAM             # <<<<<<<<<<<<<<
 *         self.open_args.stream = &self.stream
 * 
 */
  __pyx_v_self->open_args.flags = FT_OPEN_STREAM;

  /* "renpy/text/ftfont.pyx":173
 * 
 *         self.open_args.flags = FT_OPEN_STREAM
 *         self.open_args.stream = &self.stream             # <<<<<<<<<<<<<<
 * 
 *         self.stream.size = size
 */
  __pyx_v_self->open_args.stream = (&__pyx_v_self->stream);

  /* "renpy/text/ftfont.pyx":175
 *         self.open_args.stream = &self.stream
 * 
 *         self.stream.size = size             # <<<<<<<<<<<<<<
 *         self.stream.pos = 0
 *         self.stream.descriptor.pointer = <void *> self
 */
  __pyx_v_self->stream.size = __pyx_v_size;

  /* "renpy/text/ftfont.pyx":176
 * 
 *         self.stream.size = size
 *         self.stream.pos = 0             # <<<<<<<<<<<<<<
 *         self.stream.descriptor.pointer = <void *> self
 *         self.stream.read = io_func
 */
  __pyx_v_self->stream.pos = 0;

  /* "renpy/text/ftfont.pyx":177
 *         self.stream.size = size
 *         self.stream.pos = 0
 *         self.stream.descriptor.pointer = <void *> self             # <<<<<<<<<<<<<<
 *         self.stream.read = io_func
 *         self.stream.close = close_func
 */
  __pyx_v_self->stream.descriptor.pointer = ((void *)__pyx_v_self);

  /* "renpy/text/ftfont.pyx":178
 *         self.stream.pos = 0
 *         self.stream.descriptor.pointer = <void *> self
 *         self.stream.read = io_func             # <<<<<<<<<<<<<<
 *         self.stream.close = close_func
 * 
 */
  __pyx_v_self->stream.read = __pyx_f_5renpy_4text_6ftfont_io_func;

  /* "renpy/text/ftfont.pyx":179
 *         self.stream.descriptor.pointer = <void *> self
 *         self.stream.read = io_func
 *         self.stream.close = close_func             # <<<<<<<<<<<<<<
 * 
 *         error = FT_Open_Face(library, &self.open_args, index, &self.face)
 */
  __pyx_v_self->stream.close = __pyx_f_5renpy_4text_6ftfont_close_func;

  /* "renpy/text/ftfont.pyx":181
 *         self.stream.close = close_func
 * 
 *         error = FT_Open_Face(library, &self.open_args, index, &self.face)             # <<<<<<<<<<<<<<
 *         if error:
 *             raise FreetypeError(error)
 */
  __pyx_t_5 = __Pyx_PyInt_As_FT_Long(__pyx_v_index); if (unlikely((__pyx_t_5 == ((FT_Long)-1)) && PyErr_Occurred())) __PYX_ERR(1, 181, __pyx_L1_error)
  __pyx_v_error = FT_Open_Face(__pyx_v_5renpy_4text_6ftfont_library, (&__pyx_v_self->open_args), __pyx_t_5, (&__pyx_v_self->face));

  /* "renpy/text/ftfont.pyx":182
 * 
 *         error = FT_Open_Face(library, &self.open_args, index, &self.face)
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  __pyx_t_6 = (__pyx_v_error != 0);
  if (__pyx_t_6) {

    /* "renpy/text/ftfont.pyx":183
 *         error = FT_Open_Face(library, &self.open_args, index, &self.face)
 *         if error:
 *             raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 *         error = FT_Select_Charmap(self.face, FT_ENCODING_UNICODE)
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_7) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 183, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 183, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 183, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 183, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 183, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 183, __pyx_L1_error)

    /* "renpy/text/ftfont.pyx":182
 * 
 *         error = FT_Open_Face(library, &self.open_args, index, &self.face)
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":185
 *             raise FreetypeError(error)
 * 
 *         error = FT_Select_Charmap(self.face, FT_ENCODING_UNICODE)             # <<<<<<<<<<<<<<
 *         if error:
 *             raise FreetypeError(error)
 */
  __pyx_v_error = FT_Select_Charmap(__pyx_v_self->face, FT_ENCODING_UNICODE);

  /* "renpy/text/ftfont.pyx":186
 * 
 *         error = FT_Select_Charmap(self.face, FT_ENCODING_UNICODE)
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  __pyx_t_6 = (__pyx_v_error != 0);
  if (__pyx_t_6) {

    /* "renpy/text/ftfont.pyx":187
 *         error = FT_Select_Charmap(self.face, FT_ENCODING_UNICODE)
 *         if error:
 *             raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 *         # The size the face is at.
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 187, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 187, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 187, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 187, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 187, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 187, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(1, 187, __pyx_L1_error)

    /* "renpy/text/ftfont.pyx":186
 * 
 *         error = FT_Select_Charmap(self.face, FT_ENCODING_UNICODE)
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":190
 * 
 *         # The size the face is at.
 *         self.size = -1             # <<<<<<<<<<<<<<
 * 
 * cdef class FTFont:
 */
  __pyx_v_self->size = -1.0;

  /* "renpy/text/ftfont.pyx":154
 *         public object fn
 * 
 *     def __init__(self, f, index, fn):             # <<<<<<<<<<<<<<
 * 
 *         cdef int error
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFace.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":152
 *         unsigned long offset
 * 
 *         public object fn             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, f, index, fn):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->fn);
  __pyx_r = __pyx_v_self->fn;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->fn);
  __Pyx_DECREF(__pyx_v_self->fn);
  __pyx_v_self->fn = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn_4__del__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFace_2fn_4__del__(struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->fn);
  __Pyx_DECREF(__pyx_v_self->fn);
  __pyx_v_self->fn = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFace_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFace_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFace_2__reduce_cython__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFace_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFace.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFace_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFace_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFace_4__setstate_cython__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFace_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFace *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFace.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":234
 *         int hinting
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < 256:
 *             self.cache[i].index = -1
 */

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont___cinit__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont___cinit__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  long __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "renpy/text/ftfont.pyx":235
 * 
 *     def __cinit__(self):
 *         for i from 0 <= i < 256:             # <<<<<<<<<<<<<<
 *             self.cache[i].index = -1
 *             FT_Bitmap_New(&(self.cache[i].bitmap))
 */
  for (__pyx_v_i = 0; __pyx_v_i < 0x100; __pyx_v_i++) {

    /* "renpy/text/ftfont.pyx":236
 *     def __cinit__(self):
 *         for i from 0 <= i < 256:
 *             self.cache[i].index = -1             # <<<<<<<<<<<<<<
 *             FT_Bitmap_New(&(self.cache[i].bitmap))
 * 
 */
    (__pyx_v_self->cache[__pyx_v_i]).index = -1;

    /* "renpy/text/ftfont.pyx":237
 *         for i from 0 <= i < 256:
 *             self.cache[i].index = -1
 *             FT_Bitmap_New(&(self.cache[i].bitmap))             # <<<<<<<<<<<<<<
 * 
 *         init_gsubtable(&self.gsubtable)
 */
    FT_Bitmap_New((&(__pyx_v_self->cache[__pyx_v_i]).bitmap));
  }

  /* "renpy/text/ftfont.pyx":239
 *             FT_Bitmap_New(&(self.cache[i].bitmap))
 * 
 *         init_gsubtable(&self.gsubtable)             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  init_gsubtable((&__pyx_v_self->gsubtable));

  /* "renpy/text/ftfont.pyx":234
 *         int hinting
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < 256:
 *             self.cache[i].index = -1
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":241
 *         init_gsubtable(&self.gsubtable)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < 256:
 *             FT_Bitmap_Done(library, &(self.cache[i].bitmap))
 */

/* Python wrapper */
static void __pyx_pw_5renpy_4text_6ftfont_6FTFont_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_5renpy_4text_6ftfont_6FTFont_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_5renpy_4text_6ftfont_6FTFont_2__dealloc__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5renpy_4text_6ftfont_6FTFont_2__dealloc__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  long __pyx_v_i;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "renpy/text/ftfont.pyx":242
 * 
 *     def __dealloc__(self):
 *         for i from 0 <= i < 256:             # <<<<<<<<<<<<<<
 *             FT_Bitmap_Done(library, &(self.cache[i].bitmap))
 * 
 */
  for (__pyx_v_i = 0; __pyx_v_i < 0x100; __pyx_v_i++) {

    /* "renpy/text/ftfont.pyx":243
 *     def __dealloc__(self):
 *         for i from 0 <= i < 256:
 *             FT_Bitmap_Done(library, &(self.cache[i].bitmap))             # <<<<<<<<<<<<<<
 * 
 *         if self.stroker != NULL:
 */
    FT_Bitmap_Done(__pyx_v_5renpy_4text_6ftfont_library, (&(__pyx_v_self->cache[__pyx_v_i]).bitmap));
  }

  /* "renpy/text/ftfont.pyx":245
 *             FT_Bitmap_Done(library, &(self.cache[i].bitmap))
 * 
 *         if self.stroker != NULL:             # <<<<<<<<<<<<<<
 *             FT_Stroker_Done(self.stroker)
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->stroker != NULL) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":246
 * 
 *         if self.stroker != NULL:
 *             FT_Stroker_Done(self.stroker)             # <<<<<<<<<<<<<<
 * 
 *         free_gsubtable(&self.gsubtable)
 */
    FT_Stroker_Done(__pyx_v_self->stroker);

    /* "renpy/text/ftfont.pyx":245
 *             FT_Bitmap_Done(library, &(self.cache[i].bitmap))
 * 
 *         if self.stroker != NULL:             # <<<<<<<<<<<<<<
 *             FT_Stroker_Done(self.stroker)
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":248
 *             FT_Stroker_Done(self.stroker)
 * 
 *         free_gsubtable(&self.gsubtable)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  free_gsubtable((&__pyx_v_self->gsubtable));

  /* "renpy/text/ftfont.pyx":241
 *         init_gsubtable(&self.gsubtable)
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < 256:
 *             FT_Bitmap_Done(library, &(self.cache[i].bitmap))
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/text/ftfont.pyx":251
 * 
 * 
 *     def __init__(self, face, float size, float bold, bint italic, int outline, bint antialias, bint vertical, hinting):             # <<<<<<<<<<<<<<
 * 
 *         if size < 1:
 */

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_5__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_5__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_face = 0;
  float __pyx_v_size;
  float __pyx_v_bold;
  int __pyx_v_italic;
  int __pyx_v_outline;
  int __pyx_v_antialias;
  int __pyx_v_vertical;
  PyObject *__pyx_v_hinting = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_face,&__pyx_n_s_size,&__pyx_n_s_bold,&__pyx_n_s_italic,&__pyx_n_s_outline,&__pyx_n_s_antialias,&__pyx_n_s_vertical,&__pyx_n_s_hinting,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_face)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 1); __PYX_ERR(1, 251, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bold)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 2); __PYX_ERR(1, 251, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_italic)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 3); __PYX_ERR(1, 251, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_outline)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 4); __PYX_ERR(1, 251, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_antialias)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 5); __PYX_ERR(1, 251, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_vertical)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 6); __PYX_ERR(1, 251, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_hinting)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, 7); __PYX_ERR(1, 251, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(1, 251, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_face = values[0];
    __pyx_v_size = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_size == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L3_error)
    __pyx_v_bold = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_bold == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L3_error)
    __pyx_v_italic = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_italic == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L3_error)
    __pyx_v_outline = __Pyx_PyInt_As_int(values[4]); if (unlikely((__pyx_v_outline == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L3_error)
    __pyx_v_antialias = __Pyx_PyObject_IsTrue(values[5]); if (unlikely((__pyx_v_antialias == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L3_error)
    __pyx_v_vertical = __Pyx_PyObject_IsTrue(values[6]); if (unlikely((__pyx_v_vertical == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 251, __pyx_L3_error)
    __pyx_v_hinting = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 251, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_4__init__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), __pyx_v_face, __pyx_v_size, __pyx_v_bold, __pyx_v_italic, __pyx_v_outline, __pyx_v_antialias, __pyx_v_vertical, __pyx_v_hinting);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_4__init__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_face, float __pyx_v_size, float __pyx_v_bold, int __pyx_v_italic, int __pyx_v_outline, int __pyx_v_antialias, int __pyx_v_vertical, PyObject *__pyx_v_hinting) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  float __pyx_t_9;
  FT_Face __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/text/ftfont.pyx":253
 *     def __init__(self, face, float size, float bold, bint italic, int outline, bint antialias, bint vertical, hinting):
 * 
 *         if size < 1:             # <<<<<<<<<<<<<<
 *             size = 1
 * 
 */
  __pyx_t_1 = ((__pyx_v_size < 1.0) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":254
 * 
 *         if size < 1:
 *             size = 1             # <<<<<<<<<<<<<<
 * 
 *         if bold:
 */
    __pyx_v_size = 1.0;

    /* "renpy/text/ftfont.pyx":253
 *     def __init__(self, face, float size, float bold, bint italic, int outline, bint antialias, bint vertical, hinting):
 * 
 *         if size < 1:             # <<<<<<<<<<<<<<
 *             size = 1
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":256
 *             size = 1
 * 
 *         if bold:             # <<<<<<<<<<<<<<
 *             antialias = True
 * 
 */
  __pyx_t_1 = (__pyx_v_bold != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":257
 * 
 *         if bold:
 *             antialias = True             # <<<<<<<<<<<<<<
 * 
 *         size = size * renpy.config.ftfont_scale.get(face.fn, 1.0) * renpy.game.preferences.font_size
 */
    __pyx_v_antialias = 1;

    /* "renpy/text/ftfont.pyx":256
 *             size = 1
 * 
 *         if bold:             # <<<<<<<<<<<<<<
 *             antialias = True
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":259
 *             antialias = True
 * 
 *         size = size * renpy.config.ftfont_scale.get(face.fn, 1.0) * renpy.game.preferences.font_size             # <<<<<<<<<<<<<<
 * 
 *         self.face_object = face
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_config); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ftfont_scale); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_get); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_face, __pyx_n_s_fn); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_float_1_0};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_float_1_0};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_6) {
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_INCREF(__pyx_float_1_0);
    __Pyx_GIVEREF(__pyx_float_1_0);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_float_1_0);
    __pyx_t_4 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Multiply(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_game); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_preferences); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_font_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __pyx_PyFloat_AsFloat(__pyx_t_3); if (unlikely((__pyx_t_9 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 259, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_size = __pyx_t_9;

  /* "renpy/text/ftfont.pyx":261
 *         size = size * renpy.config.ftfont_scale.get(face.fn, 1.0) * renpy.game.preferences.font_size
 * 
 *         self.face_object = face             # <<<<<<<<<<<<<<
 *         self.face = self.face_object.face
 * 
 */
  if (!(likely(((__pyx_v_face) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_face, __pyx_ptype_5renpy_4text_6ftfont_FTFace))))) __PYX_ERR(1, 261, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_face;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->face_object);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->face_object));
  __pyx_v_self->face_object = ((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "renpy/text/ftfont.pyx":262
 * 
 *         self.face_object = face
 *         self.face = self.face_object.face             # <<<<<<<<<<<<<<
 * 
 *         self.size = size
 */
  __pyx_t_10 = __pyx_v_self->face_object->face;
  __pyx_v_self->face = __pyx_t_10;

  /* "renpy/text/ftfont.pyx":264
 *         self.face = self.face_object.face
 * 
 *         self.size = size             # <<<<<<<<<<<<<<
 *         self.bold = bold
 *         self.italic = italic
 */
  __pyx_v_self->size = __pyx_v_size;

  /* "renpy/text/ftfont.pyx":265
 * 
 *         self.size = size
 *         self.bold = bold             # <<<<<<<<<<<<<<
 *         self.italic = italic
 *         self.outline = outline
 */
  __pyx_v_self->bold = __pyx_v_bold;

  /* "renpy/text/ftfont.pyx":266
 *         self.size = size
 *         self.bold = bold
 *         self.italic = italic             # <<<<<<<<<<<<<<
 *         self.outline = outline
 *         self.antialias = antialias
 */
  __pyx_v_self->italic = __pyx_v_italic;

  /* "renpy/text/ftfont.pyx":267
 *         self.bold = bold
 *         self.italic = italic
 *         self.outline = outline             # <<<<<<<<<<<<<<
 *         self.antialias = antialias
 *         self.vertical = vertical
 */
  __pyx_v_self->outline = __pyx_v_outline;

  /* "renpy/text/ftfont.pyx":268
 *         self.italic = italic
 *         self.outline = outline
 *         self.antialias = antialias             # <<<<<<<<<<<<<<
 *         self.vertical = vertical
 * 
 */
  __pyx_v_self->antialias = __pyx_v_antialias;

  /* "renpy/text/ftfont.pyx":269
 *         self.outline = outline
 *         self.antialias = antialias
 *         self.vertical = vertical             # <<<<<<<<<<<<<<
 * 
 *         LoadGSUBTable(&self.gsubtable, self.face)
 */
  __pyx_v_self->vertical = __pyx_v_vertical;

  /* "renpy/text/ftfont.pyx":271
 *         self.vertical = vertical
 * 
 *         LoadGSUBTable(&self.gsubtable, self.face)             # <<<<<<<<<<<<<<
 * 
 *         if outline == 0:
 */
  LoadGSUBTable((&__pyx_v_self->gsubtable), __pyx_v_self->face);

  /* "renpy/text/ftfont.pyx":273
 *         LoadGSUBTable(&self.gsubtable, self.face)
 * 
 *         if outline == 0:             # <<<<<<<<<<<<<<
 *             self.stroker = NULL;
 *             self.expand = 0
 */
  __pyx_t_1 = ((__pyx_v_outline == 0) != 0);
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":274
 * 
 *         if outline == 0:
 *             self.stroker = NULL;             # <<<<<<<<<<<<<<
 *             self.expand = 0
 * 
 */
    __pyx_v_self->stroker = NULL;

    /* "renpy/text/ftfont.pyx":275
 *         if outline == 0:
 *             self.stroker = NULL;
 *             self.expand = 0             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
    __pyx_v_self->expand = 0;

    /* "renpy/text/ftfont.pyx":273
 *         LoadGSUBTable(&self.gsubtable, self.face)
 * 
 *         if outline == 0:             # <<<<<<<<<<<<<<
 *             self.stroker = NULL;
 *             self.expand = 0
 */
    goto __pyx_L5;
  }

  /* "renpy/text/ftfont.pyx":278
 * 
 *         else:
 *             FT_Stroker_New(library, &self.stroker)             # <<<<<<<<<<<<<<
 *             FT_Stroker_Set(self.stroker, outline * 64, FT_STROKER_LINECAP_ROUND, FT_STROKER_LINEJOIN_ROUND, 0)
 *             self.expand = outline * 2
 */
  /*else*/ {
    FT_Stroker_New(__pyx_v_5renpy_4text_6ftfont_library, (&__pyx_v_self->stroker));

    /* "renpy/text/ftfont.pyx":279
 *         else:
 *             FT_Stroker_New(library, &self.stroker)
 *             FT_Stroker_Set(self.stroker, outline * 64, FT_STROKER_LINECAP_ROUND, FT_STROKER_LINEJOIN_ROUND, 0)             # <<<<<<<<<<<<<<
 *             self.expand = outline * 2
 * 
 */
    FT_Stroker_Set(__pyx_v_self->stroker, (__pyx_v_outline * 64), FT_STROKER_LINECAP_ROUND, FT_STROKER_LINEJOIN_ROUND, 0);

    /* "renpy/text/ftfont.pyx":280
 *             FT_Stroker_New(library, &self.stroker)
 *             FT_Stroker_Set(self.stroker, outline * 64, FT_STROKER_LINECAP_ROUND, FT_STROKER_LINEJOIN_ROUND, 0)
 *             self.expand = outline * 2             # <<<<<<<<<<<<<<
 * 
 *         self.has_setup = False
 */
    __pyx_v_self->expand = (__pyx_v_outline * 2);
  }
  __pyx_L5:;

  /* "renpy/text/ftfont.pyx":282
 *             self.expand = outline * 2
 * 
 *         self.has_setup = False             # <<<<<<<<<<<<<<
 * 
 *         if hinting == "bytecode":
 */
  __pyx_v_self->has_setup = 0;

  /* "renpy/text/ftfont.pyx":284
 *         self.has_setup = False
 * 
 *         if hinting == "bytecode":             # <<<<<<<<<<<<<<
 *             self.hinting = FT_LOAD_NO_AUTOHINT
 *         elif hinting == "none" or hinting is None:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_hinting, __pyx_n_s_bytecode, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(1, 284, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":285
 * 
 *         if hinting == "bytecode":
 *             self.hinting = FT_LOAD_NO_AUTOHINT             # <<<<<<<<<<<<<<
 *         elif hinting == "none" or hinting is None:
 *             self.hinting = FT_LOAD_NO_HINTING
 */
    __pyx_v_self->hinting = FT_LOAD_NO_AUTOHINT;

    /* "renpy/text/ftfont.pyx":284
 *         self.has_setup = False
 * 
 *         if hinting == "bytecode":             # <<<<<<<<<<<<<<
 *             self.hinting = FT_LOAD_NO_AUTOHINT
 *         elif hinting == "none" or hinting is None:
 */
    goto __pyx_L6;
  }

  /* "renpy/text/ftfont.pyx":286
 *         if hinting == "bytecode":
 *             self.hinting = FT_LOAD_NO_AUTOHINT
 *         elif hinting == "none" or hinting is None:             # <<<<<<<<<<<<<<
 *             self.hinting = FT_LOAD_NO_HINTING
 *         else:
 */
  __pyx_t_11 = (__Pyx_PyString_Equals(__pyx_v_hinting, __pyx_n_s_none, Py_EQ)); if (unlikely(__pyx_t_11 < 0)) __PYX_ERR(1, 286, __pyx_L1_error)
  if (!__pyx_t_11) {
  } else {
    __pyx_t_1 = __pyx_t_11;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_11 = (__pyx_v_hinting == Py_None);
  __pyx_t_12 = (__pyx_t_11 != 0);
  __pyx_t_1 = __pyx_t_12;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "renpy/text/ftfont.pyx":287
 *             self.hinting = FT_LOAD_NO_AUTOHINT
 *         elif hinting == "none" or hinting is None:
 *             self.hinting = FT_LOAD_NO_HINTING             # <<<<<<<<<<<<<<
 *         else:
 *             self.hinting = FT_LOAD_FORCE_AUTOHINT
 */
    __pyx_v_self->hinting = FT_LOAD_NO_HINTING;

    /* "renpy/text/ftfont.pyx":286
 *         if hinting == "bytecode":
 *             self.hinting = FT_LOAD_NO_AUTOHINT
 *         elif hinting == "none" or hinting is None:             # <<<<<<<<<<<<<<
 *             self.hinting = FT_LOAD_NO_HINTING
 *         else:
 */
    goto __pyx_L6;
  }

  /* "renpy/text/ftfont.pyx":289
 *             self.hinting = FT_LOAD_NO_HINTING
 *         else:
 *             self.hinting = FT_LOAD_FORCE_AUTOHINT             # <<<<<<<<<<<<<<
 * 
 *     cdef setup(self):
 */
  /*else*/ {
    __pyx_v_self->hinting = FT_LOAD_FORCE_AUTOHINT;
  }
  __pyx_L6:;

  /* "renpy/text/ftfont.pyx":251
 * 
 * 
 *     def __init__(self, face, float size, float bold, bint italic, int outline, bint antialias, bint vertical, hinting):             # <<<<<<<<<<<<<<
 * 
 *         if size < 1:
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":291
 *             self.hinting = FT_LOAD_FORCE_AUTOHINT
 * 
 *     cdef setup(self):             # <<<<<<<<<<<<<<
 *         """
 *         Changes the parameters of the face to match this font.
 */

static PyObject *__pyx_f_5renpy_4text_6ftfont_6FTFont_setup(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  int __pyx_v_error;
  FT_Face __pyx_v_face;
  FT_Fixed __pyx_v_scale;
  PyObject *__pyx_v_vextent_scale = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  FT_Face __pyx_t_1;
  int __pyx_t_2;
  float __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  FT_Fixed __pyx_t_9;
  int __pyx_t_10;
  FT_Long __pyx_t_11;
  __Pyx_RefNannySetupContext("setup", 0);

  /* "renpy/text/ftfont.pyx":301
 *         cdef float ascent_scale
 * 
 *         face = self.face             # <<<<<<<<<<<<<<
 * 
 *         if self.face_object.size != self.size:
 */
  __pyx_t_1 = __pyx_v_self->face;
  __pyx_v_face = __pyx_t_1;

  /* "renpy/text/ftfont.pyx":303
 *         face = self.face
 * 
 *         if self.face_object.size != self.size:             # <<<<<<<<<<<<<<
 *             self.face_object.size = self.size
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->face_object->size != __pyx_v_self->size) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":304
 * 
 *         if self.face_object.size != self.size:
 *             self.face_object.size = self.size             # <<<<<<<<<<<<<<
 * 
 *             error = FT_Set_Char_Size(face, 0, <int> (self.size * 64), 0, 0)
 */
    __pyx_t_3 = __pyx_v_self->size;
    __pyx_v_self->face_object->size = __pyx_t_3;

    /* "renpy/text/ftfont.pyx":306
 *             self.face_object.size = self.size
 * 
 *             error = FT_Set_Char_Size(face, 0, <int> (self.size * 64), 0, 0)             # <<<<<<<<<<<<<<
 *             if error:
 *                 raise FreetypeError(error)
 */
    __pyx_v_error = FT_Set_Char_Size(__pyx_v_face, 0, ((int)(__pyx_v_self->size * 64.0)), 0, 0);

    /* "renpy/text/ftfont.pyx":307
 * 
 *             error = FT_Set_Char_Size(face, 0, <int> (self.size * 64), 0, 0)
 *             if error:             # <<<<<<<<<<<<<<
 *                 raise FreetypeError(error)
 * 
 */
    __pyx_t_2 = (__pyx_v_error != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":308
 *             error = FT_Set_Char_Size(face, 0, <int> (self.size * 64), 0, 0)
 *             if error:
 *                 raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 *         if not self.has_setup:
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 308, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 308, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
          __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 308, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 308, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 308, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(1, 308, __pyx_L1_error)

      /* "renpy/text/ftfont.pyx":307
 * 
 *             error = FT_Set_Char_Size(face, 0, <int> (self.size * 64), 0, 0)
 *             if error:             # <<<<<<<<<<<<<<
 *                 raise FreetypeError(error)
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":303
 *         face = self.face
 * 
 *         if self.face_object.size != self.size:             # <<<<<<<<<<<<<<
 *             self.face_object.size = self.size
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":310
 *                 raise FreetypeError(error)
 * 
 *         if not self.has_setup:             # <<<<<<<<<<<<<<
 * 
 *             self.has_setup = True
 */
  __pyx_t_2 = ((!(__pyx_v_self->has_setup != 0)) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":312
 *         if not self.has_setup:
 * 
 *             self.has_setup = True             # <<<<<<<<<<<<<<
 * 
 *             scale = face.size.metrics.y_scale
 */
    __pyx_v_self->has_setup = 1;

    /* "renpy/text/ftfont.pyx":314
 *             self.has_setup = True
 * 
 *             scale = face.size.metrics.y_scale             # <<<<<<<<<<<<<<
 * 
 *             vextent_scale = renpy.config.ftfont_vertical_extent_scale.get(self.face_object.fn, 1.0)
 */
    __pyx_t_9 = __pyx_v_face->size->metrics.y_scale;
    __pyx_v_scale = __pyx_t_9;

    /* "renpy/text/ftfont.pyx":316
 *             scale = face.size.metrics.y_scale
 * 
 *             vextent_scale = renpy.config.ftfont_vertical_extent_scale.get(self.face_object.fn, 1.0)             # <<<<<<<<<<<<<<
 * 
 *             self.ascent = FT_CEIL(int(face.size.metrics.ascender * vextent_scale))
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_config); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_ftfont_vertical_extent_scale); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_get); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    __pyx_t_10 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
        __pyx_t_10 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_8)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->face_object->fn, __pyx_float_1_0};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 316, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->face_object->fn, __pyx_float_1_0};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 316, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 316, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->face_object->fn);
      __Pyx_GIVEREF(__pyx_v_self->face_object->fn);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_10, __pyx_v_self->face_object->fn);
      __Pyx_INCREF(__pyx_float_1_0);
      __Pyx_GIVEREF(__pyx_float_1_0);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_10, __pyx_float_1_0);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 316, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_vextent_scale = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "renpy/text/ftfont.pyx":318
 *             vextent_scale = renpy.config.ftfont_vertical_extent_scale.get(self.face_object.fn, 1.0)
 * 
 *             self.ascent = FT_CEIL(int(face.size.metrics.ascender * vextent_scale))             # <<<<<<<<<<<<<<
 *             self.descent = FT_FLOOR(int(face.size.metrics.descender * vextent_scale))
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_FT_Pos(__pyx_v_face->size->metrics.ascender); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = PyNumber_Multiply(__pyx_t_4, __pyx_v_vextent_scale); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyNumber_Int(__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 318, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_11 = __Pyx_PyInt_As_FT_Long(__pyx_t_4); if (unlikely((__pyx_t_11 == ((FT_Long)-1)) && PyErr_Occurred())) __PYX_ERR(1, 318, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_self->ascent = FT_CEIL(__pyx_t_11);

    /* "renpy/text/ftfont.pyx":319
 * 
 *             self.ascent = FT_CEIL(int(face.size.metrics.ascender * vextent_scale))
 *             self.descent = FT_FLOOR(int(face.size.metrics.descender * vextent_scale))             # <<<<<<<<<<<<<<
 * 
 *             if self.descent > 0:
 */
    __pyx_t_4 = __Pyx_PyInt_From_FT_Pos(__pyx_v_face->size->metrics.descender); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = PyNumber_Multiply(__pyx_t_4, __pyx_v_vextent_scale); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyNumber_Int(__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_11 = __Pyx_PyInt_As_FT_Long(__pyx_t_4); if (unlikely((__pyx_t_11 == ((FT_Long)-1)) && PyErr_Occurred())) __PYX_ERR(1, 319, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_self->descent = FT_FLOOR(__pyx_t_11);

    /* "renpy/text/ftfont.pyx":321
 *             self.descent = FT_FLOOR(int(face.size.metrics.descender * vextent_scale))
 * 
 *             if self.descent > 0:             # <<<<<<<<<<<<<<
 *                 self.descent = -self.descent
 * 
 */
    __pyx_t_2 = ((__pyx_v_self->descent > 0) != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":322
 * 
 *             if self.descent > 0:
 *                 self.descent = -self.descent             # <<<<<<<<<<<<<<
 * 
 *             self.ascent += self.expand
 */
      __pyx_v_self->descent = (-__pyx_v_self->descent);

      /* "renpy/text/ftfont.pyx":321
 *             self.descent = FT_FLOOR(int(face.size.metrics.descender * vextent_scale))
 * 
 *             if self.descent > 0:             # <<<<<<<<<<<<<<
 *                 self.descent = -self.descent
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":324
 *                 self.descent = -self.descent
 * 
 *             self.ascent += self.expand             # <<<<<<<<<<<<<<
 *             self.descent -= self.expand
 * 
 */
    __pyx_v_self->ascent = (__pyx_v_self->ascent + __pyx_v_self->expand);

    /* "renpy/text/ftfont.pyx":325
 * 
 *             self.ascent += self.expand
 *             self.descent -= self.expand             # <<<<<<<<<<<<<<
 * 
 *             self.height = self.ascent - self.descent
 */
    __pyx_v_self->descent = (__pyx_v_self->descent - __pyx_v_self->expand);

    /* "renpy/text/ftfont.pyx":327
 *             self.descent -= self.expand
 * 
 *             self.height = self.ascent - self.descent             # <<<<<<<<<<<<<<
 * 
 *             # This is probably more correct, but isn't what 6.12.1 did.
 */
    __pyx_v_self->height = (__pyx_v_self->ascent - __pyx_v_self->descent);

    /* "renpy/text/ftfont.pyx":335
 *             #     self.lineskip = self.height
 * 
 *             self.lineskip = <int> self.height * renpy.game.preferences.font_line_spacing             # <<<<<<<<<<<<<<
 * 
 *             if self.vertical:
 */
    __pyx_t_4 = __Pyx_PyInt_From_int(((int)__pyx_v_self->height)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_game); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_preferences); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_font_line_spacing); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyNumber_Multiply(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_8); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 335, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_self->lineskip = __pyx_t_10;

    /* "renpy/text/ftfont.pyx":337
 *             self.lineskip = <int> self.height * renpy.game.preferences.font_line_spacing
 * 
 *             if self.vertical:             # <<<<<<<<<<<<<<
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.ascender + face.descender - face.underline_position, scale))
 *             else:
 */
    __pyx_t_2 = (__pyx_v_self->vertical != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":338
 * 
 *             if self.vertical:
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.ascender + face.descender - face.underline_position, scale))             # <<<<<<<<<<<<<<
 *             else:
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.underline_position, scale))
 */
      __pyx_v_self->underline_offset = FT_FLOOR(FT_MulFix(((__pyx_v_face->ascender + __pyx_v_face->descender) - __pyx_v_face->underline_position), __pyx_v_scale));

      /* "renpy/text/ftfont.pyx":337
 *             self.lineskip = <int> self.height * renpy.game.preferences.font_line_spacing
 * 
 *             if self.vertical:             # <<<<<<<<<<<<<<
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.ascender + face.descender - face.underline_position, scale))
 *             else:
 */
      goto __pyx_L7;
    }

    /* "renpy/text/ftfont.pyx":340
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.ascender + face.descender - face.underline_position, scale))
 *             else:
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.underline_position, scale))             # <<<<<<<<<<<<<<
 *             self.underline_height = FT_FLOOR(FT_MulFix(face.underline_thickness, scale))
 * 
 */
    /*else*/ {
      __pyx_v_self->underline_offset = FT_FLOOR(FT_MulFix(__pyx_v_face->underline_position, __pyx_v_scale));
    }
    __pyx_L7:;

    /* "renpy/text/ftfont.pyx":341
 *             else:
 *                 self.underline_offset = FT_FLOOR(FT_MulFix(face.underline_position, scale))
 *             self.underline_height = FT_FLOOR(FT_MulFix(face.underline_thickness, scale))             # <<<<<<<<<<<<<<
 * 
 *             if self.underline_height < 1:
 */
    __pyx_v_self->underline_height = FT_FLOOR(FT_MulFix(__pyx_v_face->underline_thickness, __pyx_v_scale));

    /* "renpy/text/ftfont.pyx":343
 *             self.underline_height = FT_FLOOR(FT_MulFix(face.underline_thickness, scale))
 * 
 *             if self.underline_height < 1:             # <<<<<<<<<<<<<<
 *                 self.underline_height = 1
 * 
 */
    __pyx_t_2 = ((__pyx_v_self->underline_height < 1) != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":344
 * 
 *             if self.underline_height < 1:
 *                 self.underline_height = 1             # <<<<<<<<<<<<<<
 * 
 *             self.underline_height += self.expand
 */
      __pyx_v_self->underline_height = 1;

      /* "renpy/text/ftfont.pyx":343
 *             self.underline_height = FT_FLOOR(FT_MulFix(face.underline_thickness, scale))
 * 
 *             if self.underline_height < 1:             # <<<<<<<<<<<<<<
 *                 self.underline_height = 1
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":346
 *                 self.underline_height = 1
 * 
 *             self.underline_height += self.expand             # <<<<<<<<<<<<<<
 * 
 *         return
 */
    __pyx_v_self->underline_height = (__pyx_v_self->underline_height + __pyx_v_self->expand);

    /* "renpy/text/ftfont.pyx":310
 *                 raise FreetypeError(error)
 * 
 *         if not self.has_setup:             # <<<<<<<<<<<<<<
 * 
 *             self.has_setup = True
 */
  }

  /* "renpy/text/ftfont.pyx":348
 *             self.underline_height += self.expand
 * 
 *         return             # <<<<<<<<<<<<<<
 * 
 *     cdef glyph_cache *get_glyph(self, int index):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":291
 *             self.hinting = FT_LOAD_FORCE_AUTOHINT
 * 
 *     cdef setup(self):             # <<<<<<<<<<<<<<
 *         """
 *         Changes the parameters of the face to match this font.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.setup", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_vextent_scale);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":350
 *         return
 * 
 *     cdef glyph_cache *get_glyph(self, int index):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the glyph_cache object for a given glyph.
 */

static struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_f_5renpy_4text_6ftfont_6FTFont_get_glyph(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, int __pyx_v_index) {
  FT_Face __pyx_v_face;
  FT_Glyph __pyx_v_g;
  FT_BitmapGlyph __pyx_v_bg;
  FT_Matrix __pyx_v_shear;
  int __pyx_v_error;
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_v_rv;
  uint32_t __pyx_v_vindex;
  int __pyx_v_overhang;
  FT_Glyph_Metrics __pyx_v_metrics;
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_v_glyph_rotate;
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_r;
  __Pyx_RefNannyDeclarations
  FT_Face __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  FT_Glyph_Metrics __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("get_glyph", 0);

  /* "renpy/text/ftfont.pyx":369
 *         cdef int x, y, glyph_rotate
 * 
 *         face = self.face             # <<<<<<<<<<<<<<
 * 
 *         if self.vertical:
 */
  __pyx_t_1 = __pyx_v_self->face;
  __pyx_v_face = __pyx_t_1;

  /* "renpy/text/ftfont.pyx":371
 *         face = self.face
 * 
 *         if self.vertical:             # <<<<<<<<<<<<<<
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:
 *                 index = vindex
 */
  __pyx_t_2 = (__pyx_v_self->vertical != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":372
 * 
 *         if self.vertical:
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:             # <<<<<<<<<<<<<<
 *                 index = vindex
 *             if self.face.face_flags & 32 == 32: # FT_HAS_VERTICAL(face)
 */
    __pyx_t_2 = ((GetVerticalGlyph((&__pyx_v_self->gsubtable), __pyx_v_index, (&__pyx_v_vindex)) == 0) != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":373
 *         if self.vertical:
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:
 *                 index = vindex             # <<<<<<<<<<<<<<
 *             if self.face.face_flags & 32 == 32: # FT_HAS_VERTICAL(face)
 *                 glyph_rotate = 1
 */
      __pyx_v_index = __pyx_v_vindex;

      /* "renpy/text/ftfont.pyx":372
 * 
 *         if self.vertical:
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:             # <<<<<<<<<<<<<<
 *                 index = vindex
 *             if self.face.face_flags & 32 == 32: # FT_HAS_VERTICAL(face)
 */
    }

    /* "renpy/text/ftfont.pyx":374
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:
 *                 index = vindex
 *             if self.face.face_flags & 32 == 32: # FT_HAS_VERTICAL(face)             # <<<<<<<<<<<<<<
 *                 glyph_rotate = 1
 *             else:
 */
    __pyx_t_2 = (((__pyx_v_self->face->face_flags & 32) == 32) != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":375
 *                 index = vindex
 *             if self.face.face_flags & 32 == 32: # FT_HAS_VERTICAL(face)
 *                 glyph_rotate = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 # font doesn't have vertical metrics (simulate it)
 */
      __pyx_v_glyph_rotate = 1;

      /* "renpy/text/ftfont.pyx":374
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:
 *                 index = vindex
 *             if self.face.face_flags & 32 == 32: # FT_HAS_VERTICAL(face)             # <<<<<<<<<<<<<<
 *                 glyph_rotate = 1
 *             else:
 */
      goto __pyx_L5;
    }

    /* "renpy/text/ftfont.pyx":378
 *             else:
 *                 # font doesn't have vertical metrics (simulate it)
 *                 glyph_rotate = 2             # <<<<<<<<<<<<<<
 *         else:
 *             glyph_rotate = 0
 */
    /*else*/ {
      __pyx_v_glyph_rotate = 2;
    }
    __pyx_L5:;

    /* "renpy/text/ftfont.pyx":371
 *         face = self.face
 * 
 *         if self.vertical:             # <<<<<<<<<<<<<<
 *             if GetVerticalGlyph(&self.gsubtable, index, &vindex) == 0:
 *                 index = vindex
 */
    goto __pyx_L3;
  }

  /* "renpy/text/ftfont.pyx":380
 *                 glyph_rotate = 2
 *         else:
 *             glyph_rotate = 0             # <<<<<<<<<<<<<<
 * 
 *         rv = &(self.cache[index & 255])
 */
  /*else*/ {
    __pyx_v_glyph_rotate = 0;
  }
  __pyx_L3:;

  /* "renpy/text/ftfont.pyx":382
 *             glyph_rotate = 0
 * 
 *         rv = &(self.cache[index & 255])             # <<<<<<<<<<<<<<
 *         if rv.index == index:
 *             return rv
 */
  __pyx_v_rv = (&(__pyx_v_self->cache[(__pyx_v_index & 0xFF)]));

  /* "renpy/text/ftfont.pyx":383
 * 
 *         rv = &(self.cache[index & 255])
 *         if rv.index == index:             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
  __pyx_t_2 = ((__pyx_v_rv->index == __pyx_v_index) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":384
 *         rv = &(self.cache[index & 255])
 *         if rv.index == index:
 *             return rv             # <<<<<<<<<<<<<<
 * 
 *         rv.index = index
 */
    __pyx_r = __pyx_v_rv;
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":383
 * 
 *         rv = &(self.cache[index & 255])
 *         if rv.index == index:             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":386
 *             return rv
 * 
 *         rv.index = index             # <<<<<<<<<<<<<<
 * 
 *         error = FT_Load_Glyph(face, index, self.hinting)
 */
  __pyx_v_rv->index = __pyx_v_index;

  /* "renpy/text/ftfont.pyx":388
 *         rv.index = index
 * 
 *         error = FT_Load_Glyph(face, index, self.hinting)             # <<<<<<<<<<<<<<
 *         if error:
 *             raise FreetypeError(error)
 */
  __pyx_v_error = FT_Load_Glyph(__pyx_v_face, __pyx_v_index, __pyx_v_self->hinting);

  /* "renpy/text/ftfont.pyx":389
 * 
 *         error = FT_Load_Glyph(face, index, self.hinting)
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  __pyx_t_2 = (__pyx_v_error != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":390
 *         error = FT_Load_Glyph(face, index, self.hinting)
 *         if error:
 *             raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 *         error = FT_Get_Glyph(face.glyph, &g)
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 390, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 390, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 390, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 390, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 390, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 390, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 390, __pyx_L1_error)

    /* "renpy/text/ftfont.pyx":389
 * 
 *         error = FT_Load_Glyph(face, index, self.hinting)
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":392
 *             raise FreetypeError(error)
 * 
 *         error = FT_Get_Glyph(face.glyph, &g)             # <<<<<<<<<<<<<<
 * 
 *         if error:
 */
  __pyx_v_error = FT_Get_Glyph(__pyx_v_face->glyph, (&__pyx_v_g));

  /* "renpy/text/ftfont.pyx":394
 *         error = FT_Get_Glyph(face.glyph, &g)
 * 
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  __pyx_t_2 = (__pyx_v_error != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":395
 * 
 *         if error:
 *             raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 *         if g.format != FT_GLYPH_FORMAT_BITMAP:
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 395, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_7};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 395, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_7};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 395, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 395, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_7);
        __pyx_t_7 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 395, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 395, __pyx_L1_error)

    /* "renpy/text/ftfont.pyx":394
 *         error = FT_Get_Glyph(face.glyph, &g)
 * 
 *         if error:             # <<<<<<<<<<<<<<
 *             raise FreetypeError(error)
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":397
 *             raise FreetypeError(error)
 * 
 *         if g.format != FT_GLYPH_FORMAT_BITMAP:             # <<<<<<<<<<<<<<
 * 
 *             if self.italic:
 */
  __pyx_t_2 = ((__pyx_v_g->format != FT_GLYPH_FORMAT_BITMAP) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":399
 *         if g.format != FT_GLYPH_FORMAT_BITMAP:
 * 
 *             if self.italic:             # <<<<<<<<<<<<<<
 *                 shear.xx = 1 << 16
 *                 shear.xy = (207 << 16) / 1000 # taken from SDL_ttf.
 */
    __pyx_t_2 = (__pyx_v_self->italic != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":400
 * 
 *             if self.italic:
 *                 shear.xx = 1 << 16             # <<<<<<<<<<<<<<
 *                 shear.xy = (207 << 16) / 1000 # taken from SDL_ttf.
 *                 shear.yx = 0
 */
      __pyx_v_shear.xx = 0x10000;

      /* "renpy/text/ftfont.pyx":401
 *             if self.italic:
 *                 shear.xx = 1 << 16
 *                 shear.xy = (207 << 16) / 1000 # taken from SDL_ttf.             # <<<<<<<<<<<<<<
 *                 shear.yx = 0
 *                 shear.yy = 1 << 16
 */
      __pyx_v_shear.xy = 0x34FD;

      /* "renpy/text/ftfont.pyx":402
 *                 shear.xx = 1 << 16
 *                 shear.xy = (207 << 16) / 1000 # taken from SDL_ttf.
 *                 shear.yx = 0             # <<<<<<<<<<<<<<
 *                 shear.yy = 1 << 16
 * 
 */
      __pyx_v_shear.yx = 0;

      /* "renpy/text/ftfont.pyx":403
 *                 shear.xy = (207 << 16) / 1000 # taken from SDL_ttf.
 *                 shear.yx = 0
 *                 shear.yy = 1 << 16             # <<<<<<<<<<<<<<
 * 
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)
 */
      __pyx_v_shear.yy = 0x10000;

      /* "renpy/text/ftfont.pyx":405
 *                 shear.yy = 1 << 16
 * 
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)             # <<<<<<<<<<<<<<
 * 
 *             if glyph_rotate != 0:
 */
      FT_Outline_Transform((&((FT_OutlineGlyph)__pyx_v_g)->outline), (&__pyx_v_shear));

      /* "renpy/text/ftfont.pyx":399
 *         if g.format != FT_GLYPH_FORMAT_BITMAP:
 * 
 *             if self.italic:             # <<<<<<<<<<<<<<
 *                 shear.xx = 1 << 16
 *                 shear.xy = (207 << 16) / 1000 # taken from SDL_ttf.
 */
    }

    /* "renpy/text/ftfont.pyx":407
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)
 * 
 *             if glyph_rotate != 0:             # <<<<<<<<<<<<<<
 *                 metrics = face.glyph.metrics
 *                 # move the origin for vertical layout
 */
    __pyx_t_2 = ((__pyx_v_glyph_rotate != 0) != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":408
 * 
 *             if glyph_rotate != 0:
 *                 metrics = face.glyph.metrics             # <<<<<<<<<<<<<<
 *                 # move the origin for vertical layout
 *                 if glyph_rotate == 1:
 */
      __pyx_t_8 = __pyx_v_face->glyph->metrics;
      __pyx_v_metrics = __pyx_t_8;

      /* "renpy/text/ftfont.pyx":410
 *                 metrics = face.glyph.metrics
 *                 # move the origin for vertical layout
 *                 if glyph_rotate == 1:             # <<<<<<<<<<<<<<
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, metrics.vertBearingX - metrics.horiBearingX, -metrics.vertBearingY - metrics.horiBearingY)
 *                 else:
 */
      __pyx_t_2 = ((__pyx_v_glyph_rotate == 1) != 0);
      if (__pyx_t_2) {

        /* "renpy/text/ftfont.pyx":411
 *                 # move the origin for vertical layout
 *                 if glyph_rotate == 1:
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, metrics.vertBearingX - metrics.horiBearingX, -metrics.vertBearingY - metrics.horiBearingY)             # <<<<<<<<<<<<<<
 *                 else:
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, -metrics.horiAdvance / 2, -face.bbox.yMax)
 */
        FT_Outline_Translate((&((FT_OutlineGlyph)__pyx_v_g)->outline), (__pyx_v_metrics.vertBearingX - __pyx_v_metrics.horiBearingX), ((-__pyx_v_metrics.vertBearingY) - __pyx_v_metrics.horiBearingY));

        /* "renpy/text/ftfont.pyx":410
 *                 metrics = face.glyph.metrics
 *                 # move the origin for vertical layout
 *                 if glyph_rotate == 1:             # <<<<<<<<<<<<<<
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, metrics.vertBearingX - metrics.horiBearingX, -metrics.vertBearingY - metrics.horiBearingY)
 *                 else:
 */
        goto __pyx_L12;
      }

      /* "renpy/text/ftfont.pyx":413
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, metrics.vertBearingX - metrics.horiBearingX, -metrics.vertBearingY - metrics.horiBearingY)
 *                 else:
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, -metrics.horiAdvance / 2, -face.bbox.yMax)             # <<<<<<<<<<<<<<
 *                 shear.xx = 0
 *                 shear.xy = -(1 << 16)
 */
      /*else*/ {
        FT_Outline_Translate((&((FT_OutlineGlyph)__pyx_v_g)->outline), __Pyx_div_FT_Pos((-__pyx_v_metrics.horiAdvance), 2), (-__pyx_v_face->bbox.yMax));
      }
      __pyx_L12:;

      /* "renpy/text/ftfont.pyx":414
 *                 else:
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, -metrics.horiAdvance / 2, -face.bbox.yMax)
 *                 shear.xx = 0             # <<<<<<<<<<<<<<
 *                 shear.xy = -(1 << 16)
 *                 shear.yx = 1 << 16
 */
      __pyx_v_shear.xx = 0;

      /* "renpy/text/ftfont.pyx":415
 *                     FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, -metrics.horiAdvance / 2, -face.bbox.yMax)
 *                 shear.xx = 0
 *                 shear.xy = -(1 << 16)             # <<<<<<<<<<<<<<
 *                 shear.yx = 1 << 16
 *                 shear.yy = 0
 */
      __pyx_v_shear.xy = -65536L;

      /* "renpy/text/ftfont.pyx":416
 *                 shear.xx = 0
 *                 shear.xy = -(1 << 16)
 *                 shear.yx = 1 << 16             # <<<<<<<<<<<<<<
 *                 shear.yy = 0
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)
 */
      __pyx_v_shear.yx = 0x10000;

      /* "renpy/text/ftfont.pyx":417
 *                 shear.xy = -(1 << 16)
 *                 shear.yx = 1 << 16
 *                 shear.yy = 0             # <<<<<<<<<<<<<<
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)
 *                 # set vertical baseline to a half of the height
 */
      __pyx_v_shear.yy = 0;

      /* "renpy/text/ftfont.pyx":418
 *                 shear.yx = 1 << 16
 *                 shear.yy = 0
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)             # <<<<<<<<<<<<<<
 *                 # set vertical baseline to a half of the height
 *                 FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, 0, (face.bbox.yMax + face.bbox.yMin) / 2)
 */
      FT_Outline_Transform((&((FT_OutlineGlyph)__pyx_v_g)->outline), (&__pyx_v_shear));

      /* "renpy/text/ftfont.pyx":420
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)
 *                 # set vertical baseline to a half of the height
 *                 FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, 0, (face.bbox.yMax + face.bbox.yMin) / 2)             # <<<<<<<<<<<<<<
 * 
 *             if self.stroker != NULL:
 */
      FT_Outline_Translate((&((FT_OutlineGlyph)__pyx_v_g)->outline), 0, __Pyx_div_FT_Pos((__pyx_v_face->bbox.yMax + __pyx_v_face->bbox.yMin), 2));

      /* "renpy/text/ftfont.pyx":407
 *                 FT_Outline_Transform(&(<FT_OutlineGlyph> g).outline, &shear)
 * 
 *             if glyph_rotate != 0:             # <<<<<<<<<<<<<<
 *                 metrics = face.glyph.metrics
 *                 # move the origin for vertical layout
 */
    }

    /* "renpy/text/ftfont.pyx":422
 *                 FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, 0, (face.bbox.yMax + face.bbox.yMin) / 2)
 * 
 *             if self.stroker != NULL:             # <<<<<<<<<<<<<<
 *                 # FT_Glyph_StrokeBorder(&g, self.stroker, 0, 1)
 *                 FT_Glyph_Stroke(&g, self.stroker, 1)
 */
    __pyx_t_2 = ((__pyx_v_self->stroker != NULL) != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":424
 *             if self.stroker != NULL:
 *                 # FT_Glyph_StrokeBorder(&g, self.stroker, 0, 1)
 *                 FT_Glyph_Stroke(&g, self.stroker, 1)             # <<<<<<<<<<<<<<
 * 
 *             if self.antialias:
 */
      FT_Glyph_Stroke((&__pyx_v_g), __pyx_v_self->stroker, 1);

      /* "renpy/text/ftfont.pyx":422
 *                 FT_Outline_Translate(&(<FT_OutlineGlyph> g).outline, 0, (face.bbox.yMax + face.bbox.yMin) / 2)
 * 
 *             if self.stroker != NULL:             # <<<<<<<<<<<<<<
 *                 # FT_Glyph_StrokeBorder(&g, self.stroker, 0, 1)
 *                 FT_Glyph_Stroke(&g, self.stroker, 1)
 */
    }

    /* "renpy/text/ftfont.pyx":426
 *                 FT_Glyph_Stroke(&g, self.stroker, 1)
 * 
 *             if self.antialias:             # <<<<<<<<<<<<<<
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_NORMAL, NULL, 1)
 *             else:
 */
    __pyx_t_2 = (__pyx_v_self->antialias != 0);
    if (__pyx_t_2) {

      /* "renpy/text/ftfont.pyx":427
 * 
 *             if self.antialias:
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_NORMAL, NULL, 1)             # <<<<<<<<<<<<<<
 *             else:
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_MONO, NULL, 1)
 */
      FT_Glyph_To_Bitmap((&__pyx_v_g), FT_RENDER_MODE_NORMAL, NULL, 1);

      /* "renpy/text/ftfont.pyx":426
 *                 FT_Glyph_Stroke(&g, self.stroker, 1)
 * 
 *             if self.antialias:             # <<<<<<<<<<<<<<
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_NORMAL, NULL, 1)
 *             else:
 */
      goto __pyx_L14;
    }

    /* "renpy/text/ftfont.pyx":429
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_NORMAL, NULL, 1)
 *             else:
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_MONO, NULL, 1)             # <<<<<<<<<<<<<<
 * 
 *         bg = <FT_BitmapGlyph> g
 */
    /*else*/ {
      FT_Glyph_To_Bitmap((&__pyx_v_g), FT_RENDER_MODE_MONO, NULL, 1);
    }
    __pyx_L14:;

    /* "renpy/text/ftfont.pyx":397
 *             raise FreetypeError(error)
 * 
 *         if g.format != FT_GLYPH_FORMAT_BITMAP:             # <<<<<<<<<<<<<<
 * 
 *             if self.italic:
 */
  }

  /* "renpy/text/ftfont.pyx":431
 *                 FT_Glyph_To_Bitmap(&g, FT_RENDER_MODE_MONO, NULL, 1)
 * 
 *         bg = <FT_BitmapGlyph> g             # <<<<<<<<<<<<<<
 * 
 *         if bg.bitmap.pixel_mode != FT_PIXEL_MODE_GRAY:
 */
  __pyx_v_bg = ((FT_BitmapGlyph)__pyx_v_g);

  /* "renpy/text/ftfont.pyx":433
 *         bg = <FT_BitmapGlyph> g
 * 
 *         if bg.bitmap.pixel_mode != FT_PIXEL_MODE_GRAY:             # <<<<<<<<<<<<<<
 *             FT_Bitmap_Convert(library, &(bg.bitmap), &(rv.bitmap), 4)
 * 
 */
  __pyx_t_2 = ((__pyx_v_bg->bitmap.pixel_mode != FT_PIXEL_MODE_GRAY) != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":434
 * 
 *         if bg.bitmap.pixel_mode != FT_PIXEL_MODE_GRAY:
 *             FT_Bitmap_Convert(library, &(bg.bitmap), &(rv.bitmap), 4)             # <<<<<<<<<<<<<<
 * 
 *             # Freetype gives us a bitmap where values range from 0 to 1.
 */
    FT_Bitmap_Convert(__pyx_v_5renpy_4text_6ftfont_library, (&__pyx_v_bg->bitmap), (&__pyx_v_rv->bitmap), 4);

    /* "renpy/text/ftfont.pyx":437
 * 
 *             # Freetype gives us a bitmap where values range from 0 to 1.
 *             for y from 0 <= y < rv.bitmap.rows:             # <<<<<<<<<<<<<<
 *                 for x from 0 <= x < rv.bitmap.width:
 *                     if rv.bitmap.buffer[ y * rv.bitmap.pitch + x ]:
 */
    __pyx_t_9 = __pyx_v_rv->bitmap.rows;
    for (__pyx_v_y = 0; __pyx_v_y < __pyx_t_9; __pyx_v_y++) {

      /* "renpy/text/ftfont.pyx":438
 *             # Freetype gives us a bitmap where values range from 0 to 1.
 *             for y from 0 <= y < rv.bitmap.rows:
 *                 for x from 0 <= x < rv.bitmap.width:             # <<<<<<<<<<<<<<
 *                     if rv.bitmap.buffer[ y * rv.bitmap.pitch + x ]:
 *                         rv.bitmap.buffer[ y * rv.bitmap.pitch + x ] = 255
 */
      __pyx_t_10 = __pyx_v_rv->bitmap.width;
      for (__pyx_v_x = 0; __pyx_v_x < __pyx_t_10; __pyx_v_x++) {

        /* "renpy/text/ftfont.pyx":439
 *             for y from 0 <= y < rv.bitmap.rows:
 *                 for x from 0 <= x < rv.bitmap.width:
 *                     if rv.bitmap.buffer[ y * rv.bitmap.pitch + x ]:             # <<<<<<<<<<<<<<
 *                         rv.bitmap.buffer[ y * rv.bitmap.pitch + x ] = 255
 * 
 */
        __pyx_t_2 = ((__pyx_v_rv->bitmap.buffer[((__pyx_v_y * __pyx_v_rv->bitmap.pitch) + __pyx_v_x)]) != 0);
        if (__pyx_t_2) {

          /* "renpy/text/ftfont.pyx":440
 *                 for x from 0 <= x < rv.bitmap.width:
 *                     if rv.bitmap.buffer[ y * rv.bitmap.pitch + x ]:
 *                         rv.bitmap.buffer[ y * rv.bitmap.pitch + x ] = 255             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
          (__pyx_v_rv->bitmap.buffer[((__pyx_v_y * __pyx_v_rv->bitmap.pitch) + __pyx_v_x)]) = 0xFF;

          /* "renpy/text/ftfont.pyx":439
 *             for y from 0 <= y < rv.bitmap.rows:
 *                 for x from 0 <= x < rv.bitmap.width:
 *                     if rv.bitmap.buffer[ y * rv.bitmap.pitch + x ]:             # <<<<<<<<<<<<<<
 *                         rv.bitmap.buffer[ y * rv.bitmap.pitch + x ] = 255
 * 
 */
        }
      }
    }

    /* "renpy/text/ftfont.pyx":433
 *         bg = <FT_BitmapGlyph> g
 * 
 *         if bg.bitmap.pixel_mode != FT_PIXEL_MODE_GRAY:             # <<<<<<<<<<<<<<
 *             FT_Bitmap_Convert(library, &(bg.bitmap), &(rv.bitmap), 4)
 * 
 */
    goto __pyx_L15;
  }

  /* "renpy/text/ftfont.pyx":443
 * 
 *         else:
 *             FT_Bitmap_Copy(library, &(bg.bitmap), &(rv.bitmap))             # <<<<<<<<<<<<<<
 * 
 *         if self.bold:
 */
  /*else*/ {
    FT_Bitmap_Copy(__pyx_v_5renpy_4text_6ftfont_library, (&__pyx_v_bg->bitmap), (&__pyx_v_rv->bitmap));
  }
  __pyx_L15:;

  /* "renpy/text/ftfont.pyx":445
 *             FT_Bitmap_Copy(library, &(bg.bitmap), &(rv.bitmap))
 * 
 *         if self.bold:             # <<<<<<<<<<<<<<
 *             overhang = face.size.metrics.y_ppem / 10
 * 
 */
  __pyx_t_2 = (__pyx_v_self->bold != 0);
  if (__pyx_t_2) {

    /* "renpy/text/ftfont.pyx":446
 * 
 *         if self.bold:
 *             overhang = face.size.metrics.y_ppem / 10             # <<<<<<<<<<<<<<
 * 
 *             FT_Bitmap_Embolden(
 */
    __pyx_v_overhang = __Pyx_div_long(__pyx_v_face->size->metrics.y_ppem, 10);

    /* "renpy/text/ftfont.pyx":448
 *             overhang = face.size.metrics.y_ppem / 10
 * 
 *             FT_Bitmap_Embolden(             # <<<<<<<<<<<<<<
 *                 library,
 *                 &(rv.bitmap),
 */
    FT_Bitmap_Embolden(__pyx_v_5renpy_4text_6ftfont_library, (&__pyx_v_rv->bitmap), (__pyx_v_overhang << 6), 0);

    /* "renpy/text/ftfont.pyx":445
 *             FT_Bitmap_Copy(library, &(bg.bitmap), &(rv.bitmap))
 * 
 *         if self.bold:             # <<<<<<<<<<<<<<
 *             overhang = face.size.metrics.y_ppem / 10
 * 
 */
    goto __pyx_L21;
  }

  /* "renpy/text/ftfont.pyx":455
 * 
 *         else:
 *             overhang = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_v_overhang = 0;
  }
  __pyx_L21:;

  /* "renpy/text/ftfont.pyx":459
 * 
 *         # rv.width = FT_CEIL(face.glyph.metrics.width) + self.expand
 *         if glyph_rotate == 1:             # <<<<<<<<<<<<<<
 *             rv.advance = face.glyph.metrics.vertAdvance / 64.0 + self.expand + overhang
 *         elif glyph_rotate == 2:
 */
  switch (__pyx_v_glyph_rotate) {
    case 1:

    /* "renpy/text/ftfont.pyx":460
 *         # rv.width = FT_CEIL(face.glyph.metrics.width) + self.expand
 *         if glyph_rotate == 1:
 *             rv.advance = face.glyph.metrics.vertAdvance / 64.0 + self.expand + overhang             # <<<<<<<<<<<<<<
 *         elif glyph_rotate == 2:
 *             # rv.advance = (face.ascender - face.descender) / 64.0 + self.expand + overhang
 */
    __pyx_v_rv->advance = (((__pyx_v_face->glyph->metrics.vertAdvance / 64.0) + __pyx_v_self->expand) + __pyx_v_overhang);

    /* "renpy/text/ftfont.pyx":459
 * 
 *         # rv.width = FT_CEIL(face.glyph.metrics.width) + self.expand
 *         if glyph_rotate == 1:             # <<<<<<<<<<<<<<
 *             rv.advance = face.glyph.metrics.vertAdvance / 64.0 + self.expand + overhang
 *         elif glyph_rotate == 2:
 */
    break;

    /* "renpy/text/ftfont.pyx":461
 *         if glyph_rotate == 1:
 *             rv.advance = face.glyph.metrics.vertAdvance / 64.0 + self.expand + overhang
 *         elif glyph_rotate == 2:             # <<<<<<<<<<<<<<
 *             # rv.advance = (face.ascender - face.descender) / 64.0 + self.expand + overhang
 *             rv.advance = self.lineskip + overhang
 */
    case 2:

    /* "renpy/text/ftfont.pyx":463
 *         elif glyph_rotate == 2:
 *             # rv.advance = (face.ascender - face.descender) / 64.0 + self.expand + overhang
 *             rv.advance = self.lineskip + overhang             # <<<<<<<<<<<<<<
 *         else:
 *             rv.advance = face.glyph.metrics.horiAdvance / 64.0 + self.expand + overhang
 */
    __pyx_v_rv->advance = (__pyx_v_self->lineskip + __pyx_v_overhang);

    /* "renpy/text/ftfont.pyx":461
 *         if glyph_rotate == 1:
 *             rv.advance = face.glyph.metrics.vertAdvance / 64.0 + self.expand + overhang
 *         elif glyph_rotate == 2:             # <<<<<<<<<<<<<<
 *             # rv.advance = (face.ascender - face.descender) / 64.0 + self.expand + overhang
 *             rv.advance = self.lineskip + overhang
 */
    break;
    default:

    /* "renpy/text/ftfont.pyx":465
 *             rv.advance = self.lineskip + overhang
 *         else:
 *             rv.advance = face.glyph.metrics.horiAdvance / 64.0 + self.expand + overhang             # <<<<<<<<<<<<<<
 * 
 *         rv.bitmap_left = bg.left + self.expand / 2
 */
    __pyx_v_rv->advance = (((__pyx_v_face->glyph->metrics.horiAdvance / 64.0) + __pyx_v_self->expand) + __pyx_v_overhang);
    break;
  }

  /* "renpy/text/ftfont.pyx":467
 *             rv.advance = face.glyph.metrics.horiAdvance / 64.0 + self.expand + overhang
 * 
 *         rv.bitmap_left = bg.left + self.expand / 2             # <<<<<<<<<<<<<<
 *         rv.bitmap_top = bg.top - self.expand / 2
 * 
 */
  __pyx_v_rv->bitmap_left = (__pyx_v_bg->left + __Pyx_div_long(__pyx_v_self->expand, 2));

  /* "renpy/text/ftfont.pyx":468
 * 
 *         rv.bitmap_left = bg.left + self.expand / 2
 *         rv.bitmap_top = bg.top - self.expand / 2             # <<<<<<<<<<<<<<
 * 
 *         rv.width = rv.bitmap.width + rv.bitmap_left
 */
  __pyx_v_rv->bitmap_top = (__pyx_v_bg->top - __Pyx_div_long(__pyx_v_self->expand, 2));

  /* "renpy/text/ftfont.pyx":470
 *         rv.bitmap_top = bg.top - self.expand / 2
 * 
 *         rv.width = rv.bitmap.width + rv.bitmap_left             # <<<<<<<<<<<<<<
 * 
 *         FT_Done_Glyph(g)
 */
  __pyx_v_rv->width = (__pyx_v_rv->bitmap.width + __pyx_v_rv->bitmap_left);

  /* "renpy/text/ftfont.pyx":472
 *         rv.width = rv.bitmap.width + rv.bitmap_left
 * 
 *         FT_Done_Glyph(g)             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
  FT_Done_Glyph(__pyx_v_g);

  /* "renpy/text/ftfont.pyx":474
 *         FT_Done_Glyph(g)
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":350
 *         return
 * 
 *     cdef glyph_cache *get_glyph(self, int index):             # <<<<<<<<<<<<<<
 *         """
 *         Returns the glyph_cache object for a given glyph.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_WriteUnraisable("renpy.text.ftfont.FTFont.get_glyph", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":477
 * 
 * 
 *     def glyphs(self, unicode s):             # <<<<<<<<<<<<<<
 *         """
 *         Sizes s, returning a list of Glyph objects.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_7glyphs(PyObject *__pyx_v_self, PyObject *__pyx_v_s); /*proto*/
static char __pyx_doc_5renpy_4text_6ftfont_6FTFont_6glyphs[] = "\n        Sizes s, returning a list of Glyph objects.\n        ";
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_7glyphs(PyObject *__pyx_v_self, PyObject *__pyx_v_s) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glyphs (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_s), (&PyUnicode_Type), 1, "s", 1))) __PYX_ERR(1, 477, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6glyphs(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject*)__pyx_v_s));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6glyphs(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_s) {
  FT_Face __pyx_v_face;
  PyObject *__pyx_v_rv = 0;
  int __pyx_v_len_s;
  Py_UNICODE __pyx_v_c;
  Py_UNICODE __pyx_v_next_c;
  FT_UInt __pyx_v_index;
  FT_UInt __pyx_v_next_index;
  int __pyx_v_error;
  struct __pyx_obj_5renpy_4text_11textsupport_Glyph *__pyx_v_gl = 0;
  FT_Vector __pyx_v_kerning;
  int __pyx_v_kern;
  int __pyx_v_i;
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_v_cache;
  float __pyx_v_min_advance;
  float __pyx_v_next_min_advance;
  CYTHON_UNUSED FT_GlyphSlot __pyx_v_g;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  FT_Face __pyx_t_3;
  FT_GlyphSlot __pyx_t_4;
  int __pyx_t_5;
  Py_UCS4 __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  long __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  float __pyx_t_14;
  int __pyx_t_15;
  __Pyx_RefNannySetupContext("glyphs", 0);

  /* "renpy/text/ftfont.pyx":497
 *         cdef float min_advance, next_min_advance
 * 
 *         self.setup()             # <<<<<<<<<<<<<<
 * 
 *         len_s = len(s)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *)__pyx_v_self->__pyx_vtab)->setup(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":499
 *         self.setup()
 * 
 *         len_s = len(s)             # <<<<<<<<<<<<<<
 * 
 *         face = self.face
 */
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 499, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_s); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(1, 499, __pyx_L1_error)
  __pyx_v_len_s = __pyx_t_2;

  /* "renpy/text/ftfont.pyx":501
 *         len_s = len(s)
 * 
 *         face = self.face             # <<<<<<<<<<<<<<
 *         g = face.glyph
 * 
 */
  __pyx_t_3 = __pyx_v_self->face;
  __pyx_v_face = __pyx_t_3;

  /* "renpy/text/ftfont.pyx":502
 * 
 *         face = self.face
 *         g = face.glyph             # <<<<<<<<<<<<<<
 * 
 *         rv = [ ]
 */
  __pyx_t_4 = __pyx_v_face->glyph;
  __pyx_v_g = __pyx_t_4;

  /* "renpy/text/ftfont.pyx":504
 *         g = face.glyph
 * 
 *         rv = [ ]             # <<<<<<<<<<<<<<
 * 
 *         if len_s:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rv = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":506
 *         rv = [ ]
 * 
 *         if len_s:             # <<<<<<<<<<<<<<
 * 
 *             next_min_advance = 0
 */
  __pyx_t_5 = (__pyx_v_len_s != 0);
  if (__pyx_t_5) {

    /* "renpy/text/ftfont.pyx":508
 *         if len_s:
 * 
 *             next_min_advance = 0             # <<<<<<<<<<<<<<
 *             next_c = s[0]
 *             next_index = FT_Get_Char_Index(face, next_c)
 */
    __pyx_v_next_min_advance = 0.0;

    /* "renpy/text/ftfont.pyx":509
 * 
 *             next_min_advance = 0
 *             next_c = s[0]             # <<<<<<<<<<<<<<
 *             next_index = FT_Get_Char_Index(face, next_c)
 * 
 */
    __pyx_t_6 = __Pyx_GetItemInt_Unicode(__pyx_v_s, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(__pyx_t_6 == (Py_UCS4)-1)) __PYX_ERR(1, 509, __pyx_L1_error)
    __pyx_v_next_c = __pyx_t_6;

    /* "renpy/text/ftfont.pyx":510
 *             next_min_advance = 0
 *             next_c = s[0]
 *             next_index = FT_Get_Char_Index(face, next_c)             # <<<<<<<<<<<<<<
 * 
 *         for i from 0 <= i < len_s:
 */
    __pyx_v_next_index = FT_Get_Char_Index(__pyx_v_face, __pyx_v_next_c);

    /* "renpy/text/ftfont.pyx":506
 *         rv = [ ]
 * 
 *         if len_s:             # <<<<<<<<<<<<<<
 * 
 *             next_min_advance = 0
 */
  }

  /* "renpy/text/ftfont.pyx":512
 *             next_index = FT_Get_Char_Index(face, next_c)
 * 
 *         for i from 0 <= i < len_s:             # <<<<<<<<<<<<<<
 * 
 *             c = next_c
 */
  __pyx_t_7 = __pyx_v_len_s;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_7; __pyx_v_i++) {

    /* "renpy/text/ftfont.pyx":514
 *         for i from 0 <= i < len_s:
 * 
 *             c = next_c             # <<<<<<<<<<<<<<
 *             index = next_index
 *             min_advance = next_min_advance
 */
    __pyx_v_c = __pyx_v_next_c;

    /* "renpy/text/ftfont.pyx":515
 * 
 *             c = next_c
 *             index = next_index             # <<<<<<<<<<<<<<
 *             min_advance = next_min_advance
 * 
 */
    __pyx_v_index = __pyx_v_next_index;

    /* "renpy/text/ftfont.pyx":516
 *             c = next_c
 *             index = next_index
 *             min_advance = next_min_advance             # <<<<<<<<<<<<<<
 * 
 *             cache = self.get_glyph(index)
 */
    __pyx_v_min_advance = __pyx_v_next_min_advance;

    /* "renpy/text/ftfont.pyx":518
 *             min_advance = next_min_advance
 * 
 *             cache = self.get_glyph(index)             # <<<<<<<<<<<<<<
 * 
 *             gl = Glyph.__new__(Glyph)
 */
    __pyx_v_cache = ((struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *)__pyx_v_self->__pyx_vtab)->get_glyph(__pyx_v_self, __pyx_v_index);

    /* "renpy/text/ftfont.pyx":520
 *             cache = self.get_glyph(index)
 * 
 *             gl = Glyph.__new__(Glyph)             # <<<<<<<<<<<<<<
 * 
 *             gl.character = c
 */
    __pyx_t_1 = __Pyx_tp_new(((PyObject *)__pyx_ptype_5renpy_4text_11textsupport_Glyph), __pyx_empty_tuple); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_4text_11textsupport_Glyph)))) __PYX_ERR(1, 520, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_gl, ((struct __pyx_obj_5renpy_4text_11textsupport_Glyph *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "renpy/text/ftfont.pyx":522
 *             gl = Glyph.__new__(Glyph)
 * 
 *             gl.character = c             # <<<<<<<<<<<<<<
 *             gl.ascent = self.ascent
 *             gl.width = cache.width
 */
    __pyx_v_gl->character = __pyx_v_c;

    /* "renpy/text/ftfont.pyx":523
 * 
 *             gl.character = c
 *             gl.ascent = self.ascent             # <<<<<<<<<<<<<<
 *             gl.width = cache.width
 *             gl.line_spacing = self.lineskip
 */
    __pyx_t_8 = __pyx_v_self->ascent;
    __pyx_v_gl->ascent = __pyx_t_8;

    /* "renpy/text/ftfont.pyx":524
 *             gl.character = c
 *             gl.ascent = self.ascent
 *             gl.width = cache.width             # <<<<<<<<<<<<<<
 *             gl.line_spacing = self.lineskip
 *             gl.draw = True
 */
    __pyx_t_8 = __pyx_v_cache->width;
    __pyx_v_gl->width = __pyx_t_8;

    /* "renpy/text/ftfont.pyx":525
 *             gl.ascent = self.ascent
 *             gl.width = cache.width
 *             gl.line_spacing = self.lineskip             # <<<<<<<<<<<<<<
 *             gl.draw = True
 * 
 */
    __pyx_t_8 = __pyx_v_self->lineskip;
    __pyx_v_gl->line_spacing = __pyx_t_8;

    /* "renpy/text/ftfont.pyx":526
 *             gl.width = cache.width
 *             gl.line_spacing = self.lineskip
 *             gl.draw = True             # <<<<<<<<<<<<<<
 * 
 *             if i < len_s - 1:
 */
    __pyx_v_gl->draw = 1;

    /* "renpy/text/ftfont.pyx":528
 *             gl.draw = True
 * 
 *             if i < len_s - 1:             # <<<<<<<<<<<<<<
 *                 next_c = s[i + 1]
 *                 next_index = FT_Get_Char_Index(face, next_c)
 */
    __pyx_t_5 = ((__pyx_v_i < (__pyx_v_len_s - 1)) != 0);
    if (__pyx_t_5) {

      /* "renpy/text/ftfont.pyx":529
 * 
 *             if i < len_s - 1:
 *                 next_c = s[i + 1]             # <<<<<<<<<<<<<<
 *                 next_index = FT_Get_Char_Index(face, next_c)
 * 
 */
      __pyx_t_9 = (__pyx_v_i + 1);
      __pyx_t_6 = __Pyx_GetItemInt_Unicode(__pyx_v_s, __pyx_t_9, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(__pyx_t_6 == (Py_UCS4)-1)) __PYX_ERR(1, 529, __pyx_L1_error)
      __pyx_v_next_c = __pyx_t_6;

      /* "renpy/text/ftfont.pyx":530
 *             if i < len_s - 1:
 *                 next_c = s[i + 1]
 *                 next_index = FT_Get_Char_Index(face, next_c)             # <<<<<<<<<<<<<<
 * 
 *                 error = FT_Get_Kerning(face, index, next_index, FT_KERNING_DEFAULT, &kerning)
 */
      __pyx_v_next_index = FT_Get_Char_Index(__pyx_v_face, __pyx_v_next_c);

      /* "renpy/text/ftfont.pyx":532
 *                 next_index = FT_Get_Char_Index(face, next_c)
 * 
 *                 error = FT_Get_Kerning(face, index, next_index, FT_KERNING_DEFAULT, &kerning)             # <<<<<<<<<<<<<<
 *                 if error:
 *                     raise FreetypeError(error)
 */
      __pyx_v_error = FT_Get_Kerning(__pyx_v_face, __pyx_v_index, __pyx_v_next_index, FT_KERNING_DEFAULT, (&__pyx_v_kerning));

      /* "renpy/text/ftfont.pyx":533
 * 
 *                 error = FT_Get_Kerning(face, index, next_index, FT_KERNING_DEFAULT, &kerning)
 *                 if error:             # <<<<<<<<<<<<<<
 *                     raise FreetypeError(error)
 * 
 */
      __pyx_t_5 = (__pyx_v_error != 0);
      if (__pyx_t_5) {

        /* "renpy/text/ftfont.pyx":534
 *                 error = FT_Get_Kerning(face, index, next_index, FT_KERNING_DEFAULT, &kerning)
 *                 if error:
 *                     raise FreetypeError(error)             # <<<<<<<<<<<<<<
 * 
 *                 kern = FT_ROUND(kerning.x)
 */
        __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_FreetypeError); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 534, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = __Pyx_PyInt_From_int(__pyx_v_error); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 534, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_12)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
          }
        }
        if (!__pyx_t_12) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 534, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_10)) {
            PyObject *__pyx_temp[2] = {__pyx_t_12, __pyx_t_11};
            __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 534, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
            PyObject *__pyx_temp[2] = {__pyx_t_12, __pyx_t_11};
            __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 534, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          } else
          #endif
          {
            __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 534, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_12); __pyx_t_12 = NULL;
            __Pyx_GIVEREF(__pyx_t_11);
            PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_11);
            __pyx_t_11 = 0;
            __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 534, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_Raise(__pyx_t_1, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __PYX_ERR(1, 534, __pyx_L1_error)

        /* "renpy/text/ftfont.pyx":533
 * 
 *                 error = FT_Get_Kerning(face, index, next_index, FT_KERNING_DEFAULT, &kerning)
 *                 if error:             # <<<<<<<<<<<<<<
 *                     raise FreetypeError(error)
 * 
 */
      }

      /* "renpy/text/ftfont.pyx":536
 *                     raise FreetypeError(error)
 * 
 *                 kern = FT_ROUND(kerning.x)             # <<<<<<<<<<<<<<
 * 
 *                 if cache.advance + kern > min_advance:
 */
      __pyx_v_kern = FT_ROUND(__pyx_v_kerning.x);

      /* "renpy/text/ftfont.pyx":538
 *                 kern = FT_ROUND(kerning.x)
 * 
 *                 if cache.advance + kern > min_advance:             # <<<<<<<<<<<<<<
 *                     gl.advance = cache.advance + kern
 *                 else:
 */
      __pyx_t_5 = (((__pyx_v_cache->advance + __pyx_v_kern) > __pyx_v_min_advance) != 0);
      if (__pyx_t_5) {

        /* "renpy/text/ftfont.pyx":539
 * 
 *                 if cache.advance + kern > min_advance:
 *                     gl.advance = cache.advance + kern             # <<<<<<<<<<<<<<
 *                 else:
 *                     gl.advance = min_advance
 */
        __pyx_v_gl->advance = (__pyx_v_cache->advance + __pyx_v_kern);

        /* "renpy/text/ftfont.pyx":538
 *                 kern = FT_ROUND(kerning.x)
 * 
 *                 if cache.advance + kern > min_advance:             # <<<<<<<<<<<<<<
 *                     gl.advance = cache.advance + kern
 *                 else:
 */
        goto __pyx_L8;
      }

      /* "renpy/text/ftfont.pyx":541
 *                     gl.advance = cache.advance + kern
 *                 else:
 *                     gl.advance = min_advance             # <<<<<<<<<<<<<<
 * 
 *                 next_min_advance = cache.advance - gl.advance
 */
      /*else*/ {
        __pyx_v_gl->advance = __pyx_v_min_advance;
      }
      __pyx_L8:;

      /* "renpy/text/ftfont.pyx":543
 *                     gl.advance = min_advance
 * 
 *                 next_min_advance = cache.advance - gl.advance             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
      __pyx_v_next_min_advance = (__pyx_v_cache->advance - __pyx_v_gl->advance);

      /* "renpy/text/ftfont.pyx":528
 *             gl.draw = True
 * 
 *             if i < len_s - 1:             # <<<<<<<<<<<<<<
 *                 next_c = s[i + 1]
 *                 next_index = FT_Get_Char_Index(face, next_c)
 */
      goto __pyx_L6;
    }

    /* "renpy/text/ftfont.pyx":546
 * 
 *             else:
 *                 gl.advance = cache.advance             # <<<<<<<<<<<<<<
 * 
 *             if is_zerowidth(gl.character):
 */
    /*else*/ {
      __pyx_t_14 = __pyx_v_cache->advance;
      __pyx_v_gl->advance = __pyx_t_14;
    }
    __pyx_L6:;

    /* "renpy/text/ftfont.pyx":548
 *                 gl.advance = cache.advance
 * 
 *             if is_zerowidth(gl.character):             # <<<<<<<<<<<<<<
 *                 gl.width = 0
 *                 gl.advance = 0
 */
    __pyx_t_5 = (__pyx_f_5renpy_4text_6ftfont_is_zerowidth(__pyx_v_gl->character) != 0);
    if (__pyx_t_5) {

      /* "renpy/text/ftfont.pyx":549
 * 
 *             if is_zerowidth(gl.character):
 *                 gl.width = 0             # <<<<<<<<<<<<<<
 *                 gl.advance = 0
 *                 gl.draw = False
 */
      __pyx_v_gl->width = 0.0;

      /* "renpy/text/ftfont.pyx":550
 *             if is_zerowidth(gl.character):
 *                 gl.width = 0
 *                 gl.advance = 0             # <<<<<<<<<<<<<<
 *                 gl.draw = False
 * 
 */
      __pyx_v_gl->advance = 0.0;

      /* "renpy/text/ftfont.pyx":551
 *                 gl.width = 0
 *                 gl.advance = 0
 *                 gl.draw = False             # <<<<<<<<<<<<<<
 * 
 *             rv.append(gl)
 */
      __pyx_v_gl->draw = 0;

      /* "renpy/text/ftfont.pyx":548
 *                 gl.advance = cache.advance
 * 
 *             if is_zerowidth(gl.character):             # <<<<<<<<<<<<<<
 *                 gl.width = 0
 *                 gl.advance = 0
 */
    }

    /* "renpy/text/ftfont.pyx":553
 *                 gl.draw = False
 * 
 *             rv.append(gl)             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
    __pyx_t_15 = __Pyx_PyList_Append(__pyx_v_rv, ((PyObject *)__pyx_v_gl)); if (unlikely(__pyx_t_15 == ((int)-1))) __PYX_ERR(1, 553, __pyx_L1_error)
  }

  /* "renpy/text/ftfont.pyx":555
 *             rv.append(gl)
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     def bounds(self, glyphs, bounds):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":477
 * 
 * 
 *     def glyphs(self, unicode s):             # <<<<<<<<<<<<<<
 *         """
 *         Sizes s, returning a list of Glyph objects.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.glyphs", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XDECREF((PyObject *)__pyx_v_gl);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":557
 *         return rv
 * 
 *     def bounds(self, glyphs, bounds):             # <<<<<<<<<<<<<<
 *         """
 *         Given a list of glyphs, get the intersection of bounds and the area where the glyphs
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_9bounds(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_4text_6ftfont_6FTFont_8bounds[] = "\n        Given a list of glyphs, get the intersection of bounds and the area where the glyphs\n        will be drawn to. (Not including any offsets or expansions.)\n\n        Returns an x, y, w, h tuple.\n        ";
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_9bounds(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_glyphs = 0;
  PyObject *__pyx_v_bounds = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bounds (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_glyphs,&__pyx_n_s_bounds,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_glyphs)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bounds)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("bounds", 1, 2, 2, 1); __PYX_ERR(1, 557, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "bounds") < 0)) __PYX_ERR(1, 557, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_glyphs = values[0];
    __pyx_v_bounds = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("bounds", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 557, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_8bounds(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), __pyx_v_glyphs, __pyx_v_bounds);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_8bounds(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_glyphs, PyObject *__pyx_v_bounds) {
  struct __pyx_obj_5renpy_4text_11textsupport_Glyph *__pyx_v_glyph = 0;
  FT_Face __pyx_v_face;
  FT_UInt __pyx_v_index;
  int __pyx_v_bmx;
  int __pyx_v_bmy;
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_v_w;
  int __pyx_v_h;
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_v_cache;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  FT_Face __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  PyObject *(*__pyx_t_13)(PyObject *);
  int __pyx_t_14;
  __Pyx_RefNannySetupContext("bounds", 0);

  /* "renpy/text/ftfont.pyx":576
 *         cdef int x, y, w, h
 * 
 *         x, y, w, h = bounds             # <<<<<<<<<<<<<<
 * 
 *         face = self.face
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_bounds))) || (PyList_CheckExact(__pyx_v_bounds))) {
    PyObject* sequence = __pyx_v_bounds;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(1, 576, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 3); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 3); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
      for (i=0; i < 4; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(1, 576, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
    __pyx_t_5 = PyObject_GetIter(__pyx_v_bounds); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = Py_TYPE(__pyx_t_5)->tp_iternext;
    for (index=0; index < 4; index++) {
      PyObject* item = __pyx_t_6(__pyx_t_5); if (unlikely(!item)) goto __pyx_L3_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_5), 4) < 0) __PYX_ERR(1, 576, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(1, 576, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 576, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_x = __pyx_t_7;
  __pyx_v_y = __pyx_t_8;
  __pyx_v_w = __pyx_t_9;
  __pyx_v_h = __pyx_t_10;

  /* "renpy/text/ftfont.pyx":578
 *         x, y, w, h = bounds
 * 
 *         face = self.face             # <<<<<<<<<<<<<<
 * 
 *         self.setup()
 */
  __pyx_t_11 = __pyx_v_self->face;
  __pyx_v_face = __pyx_t_11;

  /* "renpy/text/ftfont.pyx":580
 *         face = self.face
 * 
 *         self.setup()             # <<<<<<<<<<<<<<
 * 
 *         for glyph in glyphs:
 */
  __pyx_t_4 = ((struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *)__pyx_v_self->__pyx_vtab)->setup(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/text/ftfont.pyx":582
 *         self.setup()
 * 
 *         for glyph in glyphs:             # <<<<<<<<<<<<<<
 * 
 *             if glyph.split == SPLIT_INSTEAD:
 */
  if (likely(PyList_CheckExact(__pyx_v_glyphs)) || PyTuple_CheckExact(__pyx_v_glyphs)) {
    __pyx_t_4 = __pyx_v_glyphs; __Pyx_INCREF(__pyx_t_4); __pyx_t_12 = 0;
    __pyx_t_13 = NULL;
  } else {
    __pyx_t_12 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_glyphs); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_13 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 582, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_13)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_12 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_12); __Pyx_INCREF(__pyx_t_3); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(1, 582, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 582, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_12 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_12); __Pyx_INCREF(__pyx_t_3); __pyx_t_12++; if (unlikely(0 < 0)) __PYX_ERR(1, 582, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_12); __pyx_t_12++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 582, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_13(__pyx_t_4);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(1, 582, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5renpy_4text_11textsupport_Glyph))))) __PYX_ERR(1, 582, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_glyph, ((struct __pyx_obj_5renpy_4text_11textsupport_Glyph *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "renpy/text/ftfont.pyx":584
 *         for glyph in glyphs:
 * 
 *             if glyph.split == SPLIT_INSTEAD:             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
    __pyx_t_14 = ((__pyx_v_glyph->split == __pyx_e_5renpy_4text_11textsupport_SPLIT_INSTEAD) != 0);
    if (__pyx_t_14) {

      /* "renpy/text/ftfont.pyx":585
 * 
 *             if glyph.split == SPLIT_INSTEAD:
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             if glyph.character == 0x200b:
 */
      goto __pyx_L5_continue;

      /* "renpy/text/ftfont.pyx":584
 *         for glyph in glyphs:
 * 
 *             if glyph.split == SPLIT_INSTEAD:             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":587
 *                 continue
 * 
 *             if glyph.character == 0x200b:             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
    __pyx_t_14 = ((__pyx_v_glyph->character == 0x200b) != 0);
    if (__pyx_t_14) {

      /* "renpy/text/ftfont.pyx":588
 * 
 *             if glyph.character == 0x200b:
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             index = FT_Get_Char_Index(face, <Py_UNICODE> glyph.character)
 */
      goto __pyx_L5_continue;

      /* "renpy/text/ftfont.pyx":587
 *                 continue
 * 
 *             if glyph.character == 0x200b:             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":590
 *                 continue
 * 
 *             index = FT_Get_Char_Index(face, <Py_UNICODE> glyph.character)             # <<<<<<<<<<<<<<
 *             cache = self.get_glyph(index)
 * 
 */
    __pyx_v_index = FT_Get_Char_Index(__pyx_v_face, ((Py_UNICODE)__pyx_v_glyph->character));

    /* "renpy/text/ftfont.pyx":591
 * 
 *             index = FT_Get_Char_Index(face, <Py_UNICODE> glyph.character)
 *             cache = self.get_glyph(index)             # <<<<<<<<<<<<<<
 * 
 *             bmx = <int> (glyph.x + .5) + cache.bitmap_left
 */
    __pyx_v_cache = ((struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *)__pyx_v_self->__pyx_vtab)->get_glyph(__pyx_v_self, __pyx_v_index);

    /* "renpy/text/ftfont.pyx":593
 *             cache = self.get_glyph(index)
 * 
 *             bmx = <int> (glyph.x + .5) + cache.bitmap_left             # <<<<<<<<<<<<<<
 *             bmy = glyph.y - cache.bitmap_top
 * 
 */
    __pyx_v_bmx = (((int)(__pyx_v_glyph->x + .5)) + __pyx_v_cache->bitmap_left);

    /* "renpy/text/ftfont.pyx":594
 * 
 *             bmx = <int> (glyph.x + .5) + cache.bitmap_left
 *             bmy = glyph.y - cache.bitmap_top             # <<<<<<<<<<<<<<
 * 
 *             if bmx < x:
 */
    __pyx_v_bmy = (__pyx_v_glyph->y - __pyx_v_cache->bitmap_top);

    /* "renpy/text/ftfont.pyx":596
 *             bmy = glyph.y - cache.bitmap_top
 * 
 *             if bmx < x:             # <<<<<<<<<<<<<<
 *                 x = bmx
 * 
 */
    __pyx_t_14 = ((__pyx_v_bmx < __pyx_v_x) != 0);
    if (__pyx_t_14) {

      /* "renpy/text/ftfont.pyx":597
 * 
 *             if bmx < x:
 *                 x = bmx             # <<<<<<<<<<<<<<
 * 
 *             if bmy < y:
 */
      __pyx_v_x = __pyx_v_bmx;

      /* "renpy/text/ftfont.pyx":596
 *             bmy = glyph.y - cache.bitmap_top
 * 
 *             if bmx < x:             # <<<<<<<<<<<<<<
 *                 x = bmx
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":599
 *                 x = bmx
 * 
 *             if bmy < y:             # <<<<<<<<<<<<<<
 *                 y = bmy
 * 
 */
    __pyx_t_14 = ((__pyx_v_bmy < __pyx_v_y) != 0);
    if (__pyx_t_14) {

      /* "renpy/text/ftfont.pyx":600
 * 
 *             if bmy < y:
 *                 y = bmy             # <<<<<<<<<<<<<<
 * 
 *             if bmx + cache.bitmap.width > w:
 */
      __pyx_v_y = __pyx_v_bmy;

      /* "renpy/text/ftfont.pyx":599
 *                 x = bmx
 * 
 *             if bmy < y:             # <<<<<<<<<<<<<<
 *                 y = bmy
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":602
 *                 y = bmy
 * 
 *             if bmx + cache.bitmap.width > w:             # <<<<<<<<<<<<<<
 *                 w = bmx + cache.bitmap.width
 * 
 */
    __pyx_t_14 = (((__pyx_v_bmx + __pyx_v_cache->bitmap.width) > __pyx_v_w) != 0);
    if (__pyx_t_14) {

      /* "renpy/text/ftfont.pyx":603
 * 
 *             if bmx + cache.bitmap.width > w:
 *                 w = bmx + cache.bitmap.width             # <<<<<<<<<<<<<<
 * 
 *             if bmy + cache.bitmap.rows > h:
 */
      __pyx_v_w = (__pyx_v_bmx + __pyx_v_cache->bitmap.width);

      /* "renpy/text/ftfont.pyx":602
 *                 y = bmy
 * 
 *             if bmx + cache.bitmap.width > w:             # <<<<<<<<<<<<<<
 *                 w = bmx + cache.bitmap.width
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":605
 *                 w = bmx + cache.bitmap.width
 * 
 *             if bmy + cache.bitmap.rows > h:             # <<<<<<<<<<<<<<
 *                 h = bmy + cache.bitmap.rows
 * 
 */
    __pyx_t_14 = (((__pyx_v_bmy + __pyx_v_cache->bitmap.rows) > __pyx_v_h) != 0);
    if (__pyx_t_14) {

      /* "renpy/text/ftfont.pyx":606
 * 
 *             if bmy + cache.bitmap.rows > h:
 *                 h = bmy + cache.bitmap.rows             # <<<<<<<<<<<<<<
 * 
 *         return x, y, w, h
 */
      __pyx_v_h = (__pyx_v_bmy + __pyx_v_cache->bitmap.rows);

      /* "renpy/text/ftfont.pyx":605
 *                 w = bmx + cache.bitmap.width
 * 
 *             if bmy + cache.bitmap.rows > h:             # <<<<<<<<<<<<<<
 *                 h = bmy + cache.bitmap.rows
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":582
 *         self.setup()
 * 
 *         for glyph in glyphs:             # <<<<<<<<<<<<<<
 * 
 *             if glyph.split == SPLIT_INSTEAD:
 */
    __pyx_L5_continue:;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/text/ftfont.pyx":608
 *                 h = bmy + cache.bitmap.rows
 * 
 *         return x, y, w, h             # <<<<<<<<<<<<<<
 * 
 *     def draw(self, pysurf, float xo, int yo, color, list glyphs, int underline, bint strikethrough, black_color):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_x); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_y); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_w); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_h); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_1);
  __pyx_t_4 = 0;
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "renpy/text/ftfont.pyx":557
 *         return rv
 * 
 *     def bounds(self, glyphs, bounds):             # <<<<<<<<<<<<<<
 *         """
 *         Given a list of glyphs, get the intersection of bounds and the area where the glyphs
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.bounds", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_glyph);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":610
 *         return x, y, w, h
 * 
 *     def draw(self, pysurf, float xo, int yo, color, list glyphs, int underline, bint strikethrough, black_color):             # <<<<<<<<<<<<<<
 *         """
 *         Draws a list of glyphs to surf, with the baseline starting at x, y.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_11draw(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_4text_6ftfont_6FTFont_10draw[] = "\n        Draws a list of glyphs to surf, with the baseline starting at x, y.\n        ";
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_11draw(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pysurf = 0;
  float __pyx_v_xo;
  int __pyx_v_yo;
  PyObject *__pyx_v_color = 0;
  PyObject *__pyx_v_glyphs = 0;
  int __pyx_v_underline;
  int __pyx_v_strikethrough;
  CYTHON_UNUSED PyObject *__pyx_v_black_color = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("draw (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pysurf,&__pyx_n_s_xo,&__pyx_n_s_yo,&__pyx_n_s_color,&__pyx_n_s_glyphs,&__pyx_n_s_underline,&__pyx_n_s_strikethrough,&__pyx_n_s_black_color,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pysurf)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_xo)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 1); __PYX_ERR(1, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_yo)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 2); __PYX_ERR(1, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_color)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 3); __PYX_ERR(1, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_glyphs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 4); __PYX_ERR(1, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_underline)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 5); __PYX_ERR(1, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_strikethrough)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 6); __PYX_ERR(1, 610, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_black_color)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, 7); __PYX_ERR(1, 610, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "draw") < 0)) __PYX_ERR(1, 610, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_pysurf = values[0];
    __pyx_v_xo = __pyx_PyFloat_AsFloat(values[1]); if (unlikely((__pyx_v_xo == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 610, __pyx_L3_error)
    __pyx_v_yo = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_yo == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 610, __pyx_L3_error)
    __pyx_v_color = values[3];
    __pyx_v_glyphs = ((PyObject*)values[4]);
    __pyx_v_underline = __Pyx_PyInt_As_int(values[5]); if (unlikely((__pyx_v_underline == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 610, __pyx_L3_error)
    __pyx_v_strikethrough = __Pyx_PyObject_IsTrue(values[6]); if (unlikely((__pyx_v_strikethrough == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 610, __pyx_L3_error)
    __pyx_v_black_color = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("draw", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 610, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.draw", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_glyphs), (&PyList_Type), 1, "glyphs", 1))) __PYX_ERR(1, 610, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_10draw(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), __pyx_v_pysurf, __pyx_v_xo, __pyx_v_yo, __pyx_v_color, __pyx_v_glyphs, __pyx_v_underline, __pyx_v_strikethrough, __pyx_v_black_color);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_10draw(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_pysurf, float __pyx_v_xo, int __pyx_v_yo, PyObject *__pyx_v_color, PyObject *__pyx_v_glyphs, int __pyx_v_underline, int __pyx_v_strikethrough, CYTHON_UNUSED PyObject *__pyx_v_black_color) {
  SDL_Surface *__pyx_v_surf;
  unsigned int __pyx_v_Sr;
  unsigned int __pyx_v_Sb;
  unsigned int __pyx_v_Sg;
  unsigned int __pyx_v_Sa;
  unsigned int __pyx_v_alpha;
  struct __pyx_obj_5renpy_4text_11textsupport_Glyph *__pyx_v_glyph = 0;
  FT_Face __pyx_v_face;
  CYTHON_UNUSED FT_GlyphSlot __pyx_v_g;
  FT_UInt __pyx_v_index;
  int __pyx_v_bmx;
  int __pyx_v_bmy;
  int __pyx_v_px;
  int __pyx_v_py;
  int __pyx_v_pxstart;
  int __pyx_v_ly;
  int __pyx_v_lh;
  int __pyx_v_rows;
  int __pyx_v_width;
  int __pyx_v_underline_x;
  int __pyx_v_underline_end;
  int __pyx_v_expand;
  int __pyx_v_x;
  int __pyx_v_y;
  unsigned char *__pyx_v_pixels;
  unsigned char *__pyx_v_line;
  unsigned char *__pyx_v_gline;
  int __pyx_v_pitch;
  struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *__pyx_v_cache;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  unsigned int __pyx_t_7;
  unsigned int __pyx_t_8;
  unsigned int __pyx_t_9;
  unsigned int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  FT_Face __pyx_t_13;
  FT_GlyphSlot __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  long __pyx_t_18;
  long __pyx_t_19;
  __Pyx_RefNannySetupContext("draw", 0);

  /* "renpy/text/ftfont.pyx":638
 *         cdef glyph_cache *cache
 * 
 *         Sr, Sg, Sb, Sa = color             # <<<<<<<<<<<<<<
 * 
 *         if Sa == 0:
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_color))) || (PyList_CheckExact(__pyx_v_color))) {
    PyObject* sequence = __pyx_v_color;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(1, 638, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 3); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 3); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
      for (i=0; i < 4; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(1, 638, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
    __pyx_t_5 = PyObject_GetIter(__pyx_v_color); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 638, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = Py_TYPE(__pyx_t_5)->tp_iternext;
    for (index=0; index < 4; index++) {
      PyObject* item = __pyx_t_6(__pyx_t_5); if (unlikely(!item)) goto __pyx_L3_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_5), 4) < 0) __PYX_ERR(1, 638, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(1, 638, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_7 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_7 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyInt_As_unsigned_int(__pyx_t_2); if (unlikely((__pyx_t_8 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_unsigned_int(__pyx_t_3); if (unlikely((__pyx_t_9 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyInt_As_unsigned_int(__pyx_t_4); if (unlikely((__pyx_t_10 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_Sr = __pyx_t_7;
  __pyx_v_Sg = __pyx_t_8;
  __pyx_v_Sb = __pyx_t_9;
  __pyx_v_Sa = __pyx_t_10;

  /* "renpy/text/ftfont.pyx":640
 *         Sr, Sg, Sb, Sa = color
 * 
 *         if Sa == 0:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  __pyx_t_11 = ((__pyx_v_Sa == 0) != 0);
  if (__pyx_t_11) {

    /* "renpy/text/ftfont.pyx":641
 * 
 *         if Sa == 0:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self.setup()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "renpy/text/ftfont.pyx":640
 *         Sr, Sg, Sb, Sa = color
 * 
 *         if Sa == 0:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  }

  /* "renpy/text/ftfont.pyx":643
 *             return
 * 
 *         self.setup()             # <<<<<<<<<<<<<<
 * 
 *         surf = PySurface_AsSurface(pysurf)
 */
  __pyx_t_4 = ((struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *)__pyx_v_self->__pyx_vtab)->setup(__pyx_v_self); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/text/ftfont.pyx":645
 *         self.setup()
 * 
 *         surf = PySurface_AsSurface(pysurf)             # <<<<<<<<<<<<<<
 *         pixels = <unsigned char *> surf.pixels
 *         pitch = surf.pitch
 */
  __pyx_v_surf = PySurface_AsSurface(__pyx_v_pysurf);

  /* "renpy/text/ftfont.pyx":646
 * 
 *         surf = PySurface_AsSurface(pysurf)
 *         pixels = <unsigned char *> surf.pixels             # <<<<<<<<<<<<<<
 *         pitch = surf.pitch
 * 
 */
  __pyx_v_pixels = ((unsigned char *)__pyx_v_surf->pixels);

  /* "renpy/text/ftfont.pyx":647
 *         surf = PySurface_AsSurface(pysurf)
 *         pixels = <unsigned char *> surf.pixels
 *         pitch = surf.pitch             # <<<<<<<<<<<<<<
 * 
 *         face = self.face
 */
  __pyx_t_12 = __pyx_v_surf->pitch;
  __pyx_v_pitch = __pyx_t_12;

  /* "renpy/text/ftfont.pyx":649
 *         pitch = surf.pitch
 * 
 *         face = self.face             # <<<<<<<<<<<<<<
 *         g = face.glyph
 * 
 */
  __pyx_t_13 = __pyx_v_self->face;
  __pyx_v_face = __pyx_t_13;

  /* "renpy/text/ftfont.pyx":650
 * 
 *         face = self.face
 *         g = face.glyph             # <<<<<<<<<<<<<<
 * 
 *         expand = self.expand
 */
  __pyx_t_14 = __pyx_v_face->glyph;
  __pyx_v_g = __pyx_t_14;

  /* "renpy/text/ftfont.pyx":652
 *         g = face.glyph
 * 
 *         expand = self.expand             # <<<<<<<<<<<<<<
 * 
 *         for glyph in glyphs:
 */
  __pyx_t_12 = __pyx_v_self->expand;
  __pyx_v_expand = __pyx_t_12;

  /* "renpy/text/ftfont.pyx":654
 *         expand = self.expand
 * 
 *         for glyph in glyphs:             # <<<<<<<<<<<<<<
 * 
 *             if glyph.split == SPLIT_INSTEAD:
 */
  if (unlikely(__pyx_v_glyphs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(1, 654, __pyx_L1_error)
  }
  __pyx_t_4 = __pyx_v_glyphs; __Pyx_INCREF(__pyx_t_4); __pyx_t_15 = 0;
  for (;;) {
    if (__pyx_t_15 >= PyList_GET_SIZE(__pyx_t_4)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_15); __Pyx_INCREF(__pyx_t_3); __pyx_t_15++; if (unlikely(0 < 0)) __PYX_ERR(1, 654, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_4, __pyx_t_15); __pyx_t_15++; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 654, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5renpy_4text_11textsupport_Glyph))))) __PYX_ERR(1, 654, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_glyph, ((struct __pyx_obj_5renpy_4text_11textsupport_Glyph *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "renpy/text/ftfont.pyx":656
 *         for glyph in glyphs:
 * 
 *             if glyph.split == SPLIT_INSTEAD:             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
    __pyx_t_11 = ((__pyx_v_glyph->split == __pyx_e_5renpy_4text_11textsupport_SPLIT_INSTEAD) != 0);
    if (__pyx_t_11) {

      /* "renpy/text/ftfont.pyx":657
 * 
 *             if glyph.split == SPLIT_INSTEAD:
 *                 continue             # <<<<<<<<<<<<<<
 * 
 *             x = <int> (glyph.x + xo)
 */
      goto __pyx_L6_continue;

      /* "renpy/text/ftfont.pyx":656
 *         for glyph in glyphs:
 * 
 *             if glyph.split == SPLIT_INSTEAD:             # <<<<<<<<<<<<<<
 *                 continue
 * 
 */
    }

    /* "renpy/text/ftfont.pyx":659
 *                 continue
 * 
 *             x = <int> (glyph.x + xo)             # <<<<<<<<<<<<<<
 *             y = <int> (glyph.y + yo)
 * 
 */
    __pyx_v_x = ((int)(__pyx_v_glyph->x + __pyx_v_xo));

    /* "renpy/text/ftfont.pyx":660
 * 
 *             x = <int> (glyph.x + xo)
 *             y = <int> (glyph.y + yo)             # <<<<<<<<<<<<<<
 * 
 *             underline_x = x - glyph.delta_x_offset
 */
    __pyx_v_y = ((int)(__pyx_v_glyph->y + __pyx_v_yo));

    /* "renpy/text/ftfont.pyx":662
 *             y = <int> (glyph.y + yo)
 * 
 *             underline_x = x - glyph.delta_x_offset             # <<<<<<<<<<<<<<
 *             underline_end = x + <int> glyph.advance + expand
 * 
 */
    __pyx_v_underline_x = (__pyx_v_x - __pyx_v_glyph->delta_x_offset);

    /* "renpy/text/ftfont.pyx":663
 * 
 *             underline_x = x - glyph.delta_x_offset
 *             underline_end = x + <int> glyph.advance + expand             # <<<<<<<<<<<<<<
 * 
 *             index = FT_Get_Char_Index(face, <Py_UNICODE> glyph.character)
 */
    __pyx_v_underline_end = ((__pyx_v_x + ((int)__pyx_v_glyph->advance)) + __pyx_v_expand);

    /* "renpy/text/ftfont.pyx":665
 *             underline_end = x + <int> glyph.advance + expand
 * 
 *             index = FT_Get_Char_Index(face, <Py_UNICODE> glyph.character)             # <<<<<<<<<<<<<<
 *             cache = self.get_glyph(index)
 * 
 */
    __pyx_v_index = FT_Get_Char_Index(__pyx_v_face, ((Py_UNICODE)__pyx_v_glyph->character));

    /* "renpy/text/ftfont.pyx":666
 * 
 *             index = FT_Get_Char_Index(face, <Py_UNICODE> glyph.character)
 *             cache = self.get_glyph(index)             # <<<<<<<<<<<<<<
 * 
 *             # with nogil used to be here, but it slowed things down.
 */
    __pyx_v_cache = ((struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont *)__pyx_v_self->__pyx_vtab)->get_glyph(__pyx_v_self, __pyx_v_index);

    /* "renpy/text/ftfont.pyx":670
 *             # with nogil used to be here, but it slowed things down.
 * 
 *             bmx = <int> (x + .5) + cache.bitmap_left             # <<<<<<<<<<<<<<
 *             bmy = y - cache.bitmap_top
 * 
 */
    __pyx_v_bmx = (((int)(__pyx_v_x + .5)) + __pyx_v_cache->bitmap_left);

    /* "renpy/text/ftfont.pyx":671
 * 
 *             bmx = <int> (x + .5) + cache.bitmap_left
 *             bmy = y - cache.bitmap_top             # <<<<<<<<<<<<<<
 * 
 *             if bmx < 0:
 */
    __pyx_v_bmy = (__pyx_v_y - __pyx_v_cache->bitmap_top);

    /* "renpy/text/ftfont.pyx":673
 *             bmy = y - cache.bitmap_top
 * 
 *             if bmx < 0:             # <<<<<<<<<<<<<<
 *                 pxstart = -bmx
 *                 bmx = 0
 */
    __pyx_t_11 = ((__pyx_v_bmx < 0) != 0);
    if (__pyx_t_11) {

      /* "renpy/text/ftfont.pyx":674
 * 
 *             if bmx < 0:
 *                 pxstart = -bmx             # <<<<<<<<<<<<<<
 *                 bmx = 0
 *             else:
 */
      __pyx_v_pxstart = (-__pyx_v_bmx);

      /* "renpy/text/ftfont.pyx":675
 *             if bmx < 0:
 *                 pxstart = -bmx
 *                 bmx = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 pxstart = 0
 */
      __pyx_v_bmx = 0;

      /* "renpy/text/ftfont.pyx":673
 *             bmy = y - cache.bitmap_top
 * 
 *             if bmx < 0:             # <<<<<<<<<<<<<<
 *                 pxstart = -bmx
 *                 bmx = 0
 */
      goto __pyx_L9;
    }

    /* "renpy/text/ftfont.pyx":677
 *                 bmx = 0
 *             else:
 *                 pxstart = 0             # <<<<<<<<<<<<<<
 * 
 *             rows = min(cache.bitmap.rows, surf.h - bmy)
 */
    /*else*/ {
      __pyx_v_pxstart = 0;
    }
    __pyx_L9:;

    /* "renpy/text/ftfont.pyx":679
 *                 pxstart = 0
 * 
 *             rows = min(cache.bitmap.rows, surf.h - bmy)             # <<<<<<<<<<<<<<
 *             width = min(cache.bitmap.width, surf.w - bmx)
 * 
 */
    __pyx_t_12 = (__pyx_v_surf->h - __pyx_v_bmy);
    __pyx_t_16 = __pyx_v_cache->bitmap.rows;
    if (((__pyx_t_12 < __pyx_t_16) != 0)) {
      __pyx_t_17 = __pyx_t_12;
    } else {
      __pyx_t_17 = __pyx_t_16;
    }
    __pyx_v_rows = __pyx_t_17;

    /* "renpy/text/ftfont.pyx":680
 * 
 *             rows = min(cache.bitmap.rows, surf.h - bmy)
 *             width = min(cache.bitmap.width, surf.w - bmx)             # <<<<<<<<<<<<<<
 * 
 *             underline_end = min(underline_end, surf.w - 1)
 */
    __pyx_t_17 = (__pyx_v_surf->w - __pyx_v_bmx);
    __pyx_t_12 = __pyx_v_cache->bitmap.width;
    if (((__pyx_t_17 < __pyx_t_12) != 0)) {
      __pyx_t_16 = __pyx_t_17;
    } else {
      __pyx_t_16 = __pyx_t_12;
    }
    __pyx_v_width = __pyx_t_16;

    /* "renpy/text/ftfont.pyx":682
 *             width = min(cache.bitmap.width, surf.w - bmx)
 * 
 *             underline_end = min(underline_end, surf.w - 1)             # <<<<<<<<<<<<<<
 * 
 *             if glyph.draw:
 */
    __pyx_t_18 = (__pyx_v_surf->w - 1);
    __pyx_t_16 = __pyx_v_underline_end;
    if (((__pyx_t_18 < __pyx_t_16) != 0)) {
      __pyx_t_19 = __pyx_t_18;
    } else {
      __pyx_t_19 = __pyx_t_16;
    }
    __pyx_v_underline_end = __pyx_t_19;

    /* "renpy/text/ftfont.pyx":684
 *             underline_end = min(underline_end, surf.w - 1)
 * 
 *             if glyph.draw:             # <<<<<<<<<<<<<<
 * 
 *                 for py from 0 <= py < rows:
 */
    __pyx_t_11 = (__pyx_v_glyph->draw != 0);
    if (__pyx_t_11) {

      /* "renpy/text/ftfont.pyx":686
 *             if glyph.draw:
 * 
 *                 for py from 0 <= py < rows:             # <<<<<<<<<<<<<<
 * 
 *                     if bmy < 0:
 */
      __pyx_t_16 = __pyx_v_rows;
      for (__pyx_v_py = 0; __pyx_v_py < __pyx_t_16; __pyx_v_py++) {

        /* "renpy/text/ftfont.pyx":688
 *                 for py from 0 <= py < rows:
 * 
 *                     if bmy < 0:             # <<<<<<<<<<<<<<
 *                         bmy += 1
 *                         continue
 */
        __pyx_t_11 = ((__pyx_v_bmy < 0) != 0);
        if (__pyx_t_11) {

          /* "renpy/text/ftfont.pyx":689
 * 
 *                     if bmy < 0:
 *                         bmy += 1             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
          __pyx_v_bmy = (__pyx_v_bmy + 1);

          /* "renpy/text/ftfont.pyx":690
 *                     if bmy < 0:
 *                         bmy += 1
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                     line = pixels + bmy * pitch + bmx * 4
 */
          goto __pyx_L11_continue;

          /* "renpy/text/ftfont.pyx":688
 *                 for py from 0 <= py < rows:
 * 
 *                     if bmy < 0:             # <<<<<<<<<<<<<<
 *                         bmy += 1
 *                         continue
 */
        }

        /* "renpy/text/ftfont.pyx":692
 *                         continue
 * 
 *                     line = pixels + bmy * pitch + bmx * 4             # <<<<<<<<<<<<<<
 *                     gline = cache.bitmap.buffer + py * cache.bitmap.pitch + pxstart
 * 
 */
        __pyx_v_line = ((__pyx_v_pixels + (__pyx_v_bmy * __pyx_v_pitch)) + (__pyx_v_bmx * 4));

        /* "renpy/text/ftfont.pyx":693
 * 
 *                     line = pixels + bmy * pitch + bmx * 4
 *                     gline = cache.bitmap.buffer + py * cache.bitmap.pitch + pxstart             # <<<<<<<<<<<<<<
 * 
 *                     for px from 0 <= px < width:
 */
        __pyx_v_gline = ((__pyx_v_cache->bitmap.buffer + (__pyx_v_py * __pyx_v_cache->bitmap.pitch)) + __pyx_v_pxstart);

        /* "renpy/text/ftfont.pyx":695
 *                     gline = cache.bitmap.buffer + py * cache.bitmap.pitch + pxstart
 * 
 *                     for px from 0 <= px < width:             # <<<<<<<<<<<<<<
 * 
 *                         alpha = gline[0]
 */
        __pyx_t_17 = __pyx_v_width;
        for (__pyx_v_px = 0; __pyx_v_px < __pyx_t_17; __pyx_v_px++) {

          /* "renpy/text/ftfont.pyx":697
 *                     for px from 0 <= px < width:
 * 
 *                         alpha = gline[0]             # <<<<<<<<<<<<<<
 * 
 *                         # Modulate Sa by the glyph's alpha.
 */
          __pyx_v_alpha = (__pyx_v_gline[0]);

          /* "renpy/text/ftfont.pyx":701
 *                         # Modulate Sa by the glyph's alpha.
 * 
 *                         alpha = (alpha * Sa + Sa) >> 8             # <<<<<<<<<<<<<<
 * 
 *                         # Only draw if we increase the alpha - a cheap way to
 */
          __pyx_v_alpha = (((__pyx_v_alpha * __pyx_v_Sa) + __pyx_v_Sa) >> 8);

          /* "renpy/text/ftfont.pyx":705
 *                         # Only draw if we increase the alpha - a cheap way to
 *                         # allow overlapping characters.
 *                         if line[3] < alpha:             # <<<<<<<<<<<<<<
 * 
 *                             line[0] = Sr
 */
          __pyx_t_11 = (((__pyx_v_line[3]) < __pyx_v_alpha) != 0);
          if (__pyx_t_11) {

            /* "renpy/text/ftfont.pyx":707
 *                         if line[3] < alpha:
 * 
 *                             line[0] = Sr             # <<<<<<<<<<<<<<
 *                             line[1] = Sg
 *                             line[2] = Sb
 */
            (__pyx_v_line[0]) = __pyx_v_Sr;

            /* "renpy/text/ftfont.pyx":708
 * 
 *                             line[0] = Sr
 *                             line[1] = Sg             # <<<<<<<<<<<<<<
 *                             line[2] = Sb
 *                             line[3] = alpha
 */
            (__pyx_v_line[1]) = __pyx_v_Sg;

            /* "renpy/text/ftfont.pyx":709
 *                             line[0] = Sr
 *                             line[1] = Sg
 *                             line[2] = Sb             # <<<<<<<<<<<<<<
 *                             line[3] = alpha
 * 
 */
            (__pyx_v_line[2]) = __pyx_v_Sb;

            /* "renpy/text/ftfont.pyx":710
 *                             line[1] = Sg
 *                             line[2] = Sb
 *                             line[3] = alpha             # <<<<<<<<<<<<<<
 * 
 *                         gline += 1
 */
            (__pyx_v_line[3]) = __pyx_v_alpha;

            /* "renpy/text/ftfont.pyx":705
 *                         # Only draw if we increase the alpha - a cheap way to
 *                         # allow overlapping characters.
 *                         if line[3] < alpha:             # <<<<<<<<<<<<<<
 * 
 *                             line[0] = Sr
 */
          }

          /* "renpy/text/ftfont.pyx":712
 *                             line[3] = alpha
 * 
 *                         gline += 1             # <<<<<<<<<<<<<<
 *                         line += 4
 * 
 */
          __pyx_v_gline = (__pyx_v_gline + 1);

          /* "renpy/text/ftfont.pyx":713
 * 
 *                         gline += 1
 *                         line += 4             # <<<<<<<<<<<<<<
 * 
 *                     bmy += 1
 */
          __pyx_v_line = (__pyx_v_line + 4);
        }

        /* "renpy/text/ftfont.pyx":715
 *                         line += 4
 * 
 *                     bmy += 1             # <<<<<<<<<<<<<<
 * 
 *             # Underlining.
 */
        __pyx_v_bmy = (__pyx_v_bmy + 1);
        __pyx_L11_continue:;
      }

      /* "renpy/text/ftfont.pyx":684
 *             underline_end = min(underline_end, surf.w - 1)
 * 
 *             if glyph.draw:             # <<<<<<<<<<<<<<
 * 
 *                 for py from 0 <= py < rows:
 */
    }

    /* "renpy/text/ftfont.pyx":718
 * 
 *             # Underlining.
 *             if underline:             # <<<<<<<<<<<<<<
 * 
 *                 ly = y - self.underline_offset - 1
 */
    __pyx_t_11 = (__pyx_v_underline != 0);
    if (__pyx_t_11) {

      /* "renpy/text/ftfont.pyx":720
 *             if underline:
 * 
 *                 ly = y - self.underline_offset - 1             # <<<<<<<<<<<<<<
 *                 lh = self.underline_height * underline
 * 
 */
      __pyx_v_ly = ((__pyx_v_y - __pyx_v_self->underline_offset) - 1);

      /* "renpy/text/ftfont.pyx":721
 * 
 *                 ly = y - self.underline_offset - 1
 *                 lh = self.underline_height * underline             # <<<<<<<<<<<<<<
 * 
 *                 for py from ly <= py < min(ly + lh, surf.h):
 */
      __pyx_v_lh = (__pyx_v_self->underline_height * __pyx_v_underline);

      /* "renpy/text/ftfont.pyx":723
 *                 lh = self.underline_height * underline
 * 
 *                 for py from ly <= py < min(ly + lh, surf.h):             # <<<<<<<<<<<<<<
 *                     for px from underline_x <= px < underline_end:
 *                         line = pixels + py * pitch + px * 4
 */
      __pyx_t_16 = __pyx_v_surf->h;
      __pyx_t_17 = (__pyx_v_ly + __pyx_v_lh);
      if (((__pyx_t_16 < __pyx_t_17) != 0)) {
        __pyx_t_12 = __pyx_t_16;
      } else {
        __pyx_t_12 = __pyx_t_17;
      }
      __pyx_t_16 = __pyx_t_12;
      for (__pyx_v_py = __pyx_v_ly; __pyx_v_py < __pyx_t_16; __pyx_v_py++) {

        /* "renpy/text/ftfont.pyx":724
 * 
 *                 for py from ly <= py < min(ly + lh, surf.h):
 *                     for px from underline_x <= px < underline_end:             # <<<<<<<<<<<<<<
 *                         line = pixels + py * pitch + px * 4
 * 
 */
        __pyx_t_12 = __pyx_v_underline_end;
        for (__pyx_v_px = __pyx_v_underline_x; __pyx_v_px < __pyx_t_12; __pyx_v_px++) {

          /* "renpy/text/ftfont.pyx":725
 *                 for py from ly <= py < min(ly + lh, surf.h):
 *                     for px from underline_x <= px < underline_end:
 *                         line = pixels + py * pitch + px * 4             # <<<<<<<<<<<<<<
 * 
 *                         line[0] = Sr
 */
          __pyx_v_line = ((__pyx_v_pixels + (__pyx_v_py * __pyx_v_pitch)) + (__pyx_v_px * 4));

          /* "renpy/text/ftfont.pyx":727
 *                         line = pixels + py * pitch + px * 4
 * 
 *                         line[0] = Sr             # <<<<<<<<<<<<<<
 *                         line[1] = Sg
 *                         line[2] = Sb
 */
          (__pyx_v_line[0]) = __pyx_v_Sr;

          /* "renpy/text/ftfont.pyx":728
 * 
 *                         line[0] = Sr
 *                         line[1] = Sg             # <<<<<<<<<<<<<<
 *                         line[2] = Sb
 *                         line[3] = Sa
 */
          (__pyx_v_line[1]) = __pyx_v_Sg;

          /* "renpy/text/ftfont.pyx":729
 *                         line[0] = Sr
 *                         line[1] = Sg
 *                         line[2] = Sb             # <<<<<<<<<<<<<<
 *                         line[3] = Sa
 * 
 */
          (__pyx_v_line[2]) = __pyx_v_Sb;

          /* "renpy/text/ftfont.pyx":730
 *                         line[1] = Sg
 *                         line[2] = Sb
 *                         line[3] = Sa             # <<<<<<<<<<<<<<
 * 
 *             # Strikethrough.
 */
          (__pyx_v_line[3]) = __pyx_v_Sa;
        }
      }

      /* "renpy/text/ftfont.pyx":718
 * 
 *             # Underlining.
 *             if underline:             # <<<<<<<<<<<<<<
 * 
 *                 ly = y - self.underline_offset - 1
 */
    }

    /* "renpy/text/ftfont.pyx":733
 * 
 *             # Strikethrough.
 *             if strikethrough:             # <<<<<<<<<<<<<<
 *                 ly = y - self.ascent + self.height / 2
 *                 lh = self.height / 10
 */
    __pyx_t_11 = (__pyx_v_strikethrough != 0);
    if (__pyx_t_11) {

      /* "renpy/text/ftfont.pyx":734
 *             # Strikethrough.
 *             if strikethrough:
 *                 ly = y - self.ascent + self.height / 2             # <<<<<<<<<<<<<<
 *                 lh = self.height / 10
 *                 if lh < 1:
 */
      __pyx_v_ly = ((__pyx_v_y - __pyx_v_self->ascent) + __Pyx_div_long(__pyx_v_self->height, 2));

      /* "renpy/text/ftfont.pyx":735
 *             if strikethrough:
 *                 ly = y - self.ascent + self.height / 2
 *                 lh = self.height / 10             # <<<<<<<<<<<<<<
 *                 if lh < 1:
 *                     lh = 1
 */
      __pyx_v_lh = __Pyx_div_long(__pyx_v_self->height, 10);

      /* "renpy/text/ftfont.pyx":736
 *                 ly = y - self.ascent + self.height / 2
 *                 lh = self.height / 10
 *                 if lh < 1:             # <<<<<<<<<<<<<<
 *                     lh = 1
 * 
 */
      __pyx_t_11 = ((__pyx_v_lh < 1) != 0);
      if (__pyx_t_11) {

        /* "renpy/text/ftfont.pyx":737
 *                 lh = self.height / 10
 *                 if lh < 1:
 *                     lh = 1             # <<<<<<<<<<<<<<
 * 
 *                 for py from ly <= py < (ly + lh):
 */
        __pyx_v_lh = 1;

        /* "renpy/text/ftfont.pyx":736
 *                 ly = y - self.ascent + self.height / 2
 *                 lh = self.height / 10
 *                 if lh < 1:             # <<<<<<<<<<<<<<
 *                     lh = 1
 * 
 */
      }

      /* "renpy/text/ftfont.pyx":739
 *                     lh = 1
 * 
 *                 for py from ly <= py < (ly + lh):             # <<<<<<<<<<<<<<
 *                     for px from underline_x <= px < underline_end:
 *                         line = pixels + py * pitch + px * 4
 */
      __pyx_t_16 = (__pyx_v_ly + __pyx_v_lh);
      for (__pyx_v_py = __pyx_v_ly; __pyx_v_py < __pyx_t_16; __pyx_v_py++) {

        /* "renpy/text/ftfont.pyx":740
 * 
 *                 for py from ly <= py < (ly + lh):
 *                     for px from underline_x <= px < underline_end:             # <<<<<<<<<<<<<<
 *                         line = pixels + py * pitch + px * 4
 * 
 */
        __pyx_t_12 = __pyx_v_underline_end;
        for (__pyx_v_px = __pyx_v_underline_x; __pyx_v_px < __pyx_t_12; __pyx_v_px++) {

          /* "renpy/text/ftfont.pyx":741
 *                 for py from ly <= py < (ly + lh):
 *                     for px from underline_x <= px < underline_end:
 *                         line = pixels + py * pitch + px * 4             # <<<<<<<<<<<<<<
 * 
 *                         line[0] = Sr
 */
          __pyx_v_line = ((__pyx_v_pixels + (__pyx_v_py * __pyx_v_pitch)) + (__pyx_v_px * 4));

          /* "renpy/text/ftfont.pyx":743
 *                         line = pixels + py * pitch + px * 4
 * 
 *                         line[0] = Sr             # <<<<<<<<<<<<<<
 *                         line[1] = Sg
 *                         line[2] = Sb
 */
          (__pyx_v_line[0]) = __pyx_v_Sr;

          /* "renpy/text/ftfont.pyx":744
 * 
 *                         line[0] = Sr
 *                         line[1] = Sg             # <<<<<<<<<<<<<<
 *                         line[2] = Sb
 *                         line[3] = Sa
 */
          (__pyx_v_line[1]) = __pyx_v_Sg;

          /* "renpy/text/ftfont.pyx":745
 *                         line[0] = Sr
 *                         line[1] = Sg
 *                         line[2] = Sb             # <<<<<<<<<<<<<<
 *                         line[3] = Sa
 * 
 */
          (__pyx_v_line[2]) = __pyx_v_Sb;

          /* "renpy/text/ftfont.pyx":746
 *                         line[1] = Sg
 *                         line[2] = Sb
 *                         line[3] = Sa             # <<<<<<<<<<<<<<
 * 
 */
          (__pyx_v_line[3]) = __pyx_v_Sa;
        }
      }

      /* "renpy/text/ftfont.pyx":733
 * 
 *             # Strikethrough.
 *             if strikethrough:             # <<<<<<<<<<<<<<
 *                 ly = y - self.ascent + self.height / 2
 *                 lh = self.height / 10
 */
    }

    /* "renpy/text/ftfont.pyx":654
 *         expand = self.expand
 * 
 *         for glyph in glyphs:             # <<<<<<<<<<<<<<
 * 
 *             if glyph.split == SPLIT_INSTEAD:
 */
    __pyx_L6_continue:;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/text/ftfont.pyx":610
 *         return x, y, w, h
 * 
 *     def draw(self, pysurf, float xo, int yo, color, list glyphs, int underline, bint strikethrough, black_color):             # <<<<<<<<<<<<<<
 *         """
 *         Draws a list of glyphs to surf, with the baseline starting at x, y.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.draw", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_glyph);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":211
 * 
 *         # The offset and height of the underline.
 *         public int underline_offset             # <<<<<<<<<<<<<<
 *         public int underline_height
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_offset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_offset_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_offset___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_offset___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->underline_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.underline_offset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_offset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_offset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_offset_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_offset_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 211, __pyx_L1_error)
  __pyx_v_self->underline_offset = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.underline_offset.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":212
 *         # The offset and height of the underline.
 *         public int underline_offset
 *         public int underline_height             # <<<<<<<<<<<<<<
 * 
 *         # The stroker object.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_height_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_height_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_height___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_height___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->underline_height); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.underline_height.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_height_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_16underline_height_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 212, __pyx_L1_error)
  __pyx_v_self->underline_height = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.underline_height.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":218
 * 
 *         # The number of pixels the outlines are expanded by.
 *         public int expand             # <<<<<<<<<<<<<<
 * 
 *         # Basic Y-direction metrics for this font.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_6expand_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_6expand_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6expand___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6expand___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->expand); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.expand.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_6expand_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_6expand_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6expand_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_6expand_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 218, __pyx_L1_error)
  __pyx_v_self->expand = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.expand.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":221
 * 
 *         # Basic Y-direction metrics for this font.
 *         public int ascent             # <<<<<<<<<<<<<<
 *         public int descent
 *         public int height
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_6ascent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_6ascent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6ascent___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6ascent___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->ascent); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.ascent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_6ascent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_6ascent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6ascent_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_6ascent_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 221, __pyx_L1_error)
  __pyx_v_self->ascent = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.ascent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":222
 *         # Basic Y-direction metrics for this font.
 *         public int ascent
 *         public int descent             # <<<<<<<<<<<<<<
 *         public int height
 *         public int lineskip
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_7descent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_7descent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_7descent___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_7descent___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->descent); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.descent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_7descent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_7descent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_7descent_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_7descent_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 222, __pyx_L1_error)
  __pyx_v_self->descent = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.descent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":223
 *         public int ascent
 *         public int descent
 *         public int height             # <<<<<<<<<<<<<<
 *         public int lineskip
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_6height_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_6height_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6height___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_6height___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.height.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_6height_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_6height_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 223, __pyx_L1_error)
  __pyx_v_self->height = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.height.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/text/ftfont.pyx":224
 *         public int descent
 *         public int height
 *         public int lineskip             # <<<<<<<<<<<<<<
 * 
 *         glyph_cache cache[256]
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_8lineskip_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_8lineskip_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_8lineskip___get__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_8lineskip___get__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->lineskip); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.lineskip.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_8lineskip_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_4text_6ftfont_6FTFont_8lineskip_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_8lineskip_2__set__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_4text_6ftfont_6FTFont_8lineskip_2__set__(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 224, __pyx_L1_error)
  __pyx_v_self->lineskip = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.lineskip.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_12__reduce_cython__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_12__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_4text_6ftfont_6FTFont_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_4text_6ftfont_6FTFont_14__setstate_cython__(((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_4text_6ftfont_6FTFont_14__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_4text_6ftfont_FTFont *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.text.ftfont.FTFont.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_5renpy_4text_6ftfont_FTFace(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_4text_6ftfont_FTFace *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)o);
  p->f = Py_None; Py_INCREF(Py_None);
  p->fn = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5renpy_4text_6ftfont_FTFace(PyObject *o) {
  struct __pyx_obj_5renpy_4text_6ftfont_FTFace *p = (struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->f);
  Py_CLEAR(p->fn);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5renpy_4text_6ftfont_FTFace(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_4text_6ftfont_FTFace *p = (struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)o;
  if (p->f) {
    e = (*v)(p->f, a); if (e) return e;
  }
  if (p->fn) {
    e = (*v)(p->fn, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_4text_6ftfont_FTFace(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_4text_6ftfont_FTFace *p = (struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)o;
  tmp = ((PyObject*)p->f);
  p->f = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->fn);
  p->fn = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFace_fn(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFace_fn(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFace_2fn_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_5renpy_4text_6ftfont_FTFace[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFace_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFace_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5renpy_4text_6ftfont_FTFace[] = {
  {(char *)"fn", __pyx_getprop_5renpy_4text_6ftfont_6FTFace_fn, __pyx_setprop_5renpy_4text_6ftfont_6FTFace_fn, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_4text_6ftfont_FTFace = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.text.ftfont.FTFace", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_4text_6ftfont_FTFace), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_4text_6ftfont_FTFace, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    Represents a freetype face.\n    ", /*tp_doc*/
  __pyx_tp_traverse_5renpy_4text_6ftfont_FTFace, /*tp_traverse*/
  __pyx_tp_clear_5renpy_4text_6ftfont_FTFace, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_4text_6ftfont_FTFace, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5renpy_4text_6ftfont_FTFace, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_4text_6ftfont_6FTFace_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_4text_6ftfont_FTFace, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_4text_6ftfont_FTFont __pyx_vtable_5renpy_4text_6ftfont_FTFont;

static PyObject *__pyx_tp_new_5renpy_4text_6ftfont_FTFont(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_4text_6ftfont_FTFont *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)o);
  p->__pyx_vtab = __pyx_vtabptr_5renpy_4text_6ftfont_FTFont;
  p->face_object = ((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_5renpy_4text_6ftfont_6FTFont_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_5renpy_4text_6ftfont_FTFont(PyObject *o) {
  struct __pyx_obj_5renpy_4text_6ftfont_FTFont *p = (struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_5renpy_4text_6ftfont_6FTFont_3__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->face_object);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5renpy_4text_6ftfont_FTFont(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_4text_6ftfont_FTFont *p = (struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)o;
  if (p->face_object) {
    e = (*v)(((PyObject *)p->face_object), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_4text_6ftfont_FTFont(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_4text_6ftfont_FTFont *p = (struct __pyx_obj_5renpy_4text_6ftfont_FTFont *)o;
  tmp = ((PyObject*)p->face_object);
  p->face_object = ((struct __pyx_obj_5renpy_4text_6ftfont_FTFace *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_underline_offset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_offset_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_underline_offset(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_offset_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_underline_height(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_height_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_underline_height(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_16underline_height_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_expand(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_6expand_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_expand(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_6expand_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_ascent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_6ascent_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_ascent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_6ascent_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_descent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_7descent_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_descent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_7descent_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_height(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_6height_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_height(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_6height_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_4text_6ftfont_6FTFont_lineskip(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_4text_6ftfont_6FTFont_8lineskip_1__get__(o);
}

static int __pyx_setprop_5renpy_4text_6ftfont_6FTFont_lineskip(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_4text_6ftfont_6FTFont_8lineskip_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5renpy_4text_6ftfont_FTFont[] = {
  {"glyphs", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFont_7glyphs, METH_O, __pyx_doc_5renpy_4text_6ftfont_6FTFont_6glyphs},
  {"bounds", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFont_9bounds, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_4text_6ftfont_6FTFont_8bounds},
  {"draw", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFont_11draw, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_4text_6ftfont_6FTFont_10draw},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFont_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_4text_6ftfont_6FTFont_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5renpy_4text_6ftfont_FTFont[] = {
  {(char *)"underline_offset", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_underline_offset, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_underline_offset, (char *)0, 0},
  {(char *)"underline_height", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_underline_height, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_underline_height, (char *)0, 0},
  {(char *)"expand", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_expand, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_expand, (char *)0, 0},
  {(char *)"ascent", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_ascent, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_ascent, (char *)0, 0},
  {(char *)"descent", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_descent, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_descent, (char *)0, 0},
  {(char *)"height", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_height, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_height, (char *)0, 0},
  {(char *)"lineskip", __pyx_getprop_5renpy_4text_6ftfont_6FTFont_lineskip, __pyx_setprop_5renpy_4text_6ftfont_6FTFont_lineskip, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_4text_6ftfont_FTFont = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.text.ftfont.FTFont", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_4text_6ftfont_FTFont), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_4text_6ftfont_FTFont, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5renpy_4text_6ftfont_FTFont, /*tp_traverse*/
  __pyx_tp_clear_5renpy_4text_6ftfont_FTFont, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_4text_6ftfont_FTFont, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5renpy_4text_6ftfont_FTFont, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_4text_6ftfont_6FTFont_5__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_4text_6ftfont_FTFont, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_ftfont(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_ftfont},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "ftfont",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_FreetypeError, __pyx_k_FreetypeError, sizeof(__pyx_k_FreetypeError), 0, 0, 1, 1},
  {&__pyx_n_s_FreetypeError___init, __pyx_k_FreetypeError___init, sizeof(__pyx_k_FreetypeError___init), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_antialias, __pyx_k_antialias, sizeof(__pyx_k_antialias), 0, 0, 1, 1},
  {&__pyx_n_s_black_color, __pyx_k_black_color, sizeof(__pyx_k_black_color), 0, 0, 1, 1},
  {&__pyx_n_s_bold, __pyx_k_bold, sizeof(__pyx_k_bold), 0, 0, 1, 1},
  {&__pyx_n_s_bounds, __pyx_k_bounds, sizeof(__pyx_k_bounds), 0, 0, 1, 1},
  {&__pyx_n_s_bytecode, __pyx_k_bytecode, sizeof(__pyx_k_bytecode), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_code, __pyx_k_code, sizeof(__pyx_k_code), 0, 0, 1, 1},
  {&__pyx_n_s_color, __pyx_k_color, sizeof(__pyx_k_color), 0, 0, 1, 1},
  {&__pyx_n_s_config, __pyx_k_config, sizeof(__pyx_k_config), 0, 0, 1, 1},
  {&__pyx_kp_s_d_s, __pyx_k_d_s, sizeof(__pyx_k_d_s), 0, 0, 1, 0},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 0, 1, 1},
  {&__pyx_n_s_face, __pyx_k_face, sizeof(__pyx_k_face), 0, 0, 1, 1},
  {&__pyx_n_s_fn, __pyx_k_fn, sizeof(__pyx_k_fn), 0, 0, 1, 1},
  {&__pyx_n_s_font_line_spacing, __pyx_k_font_line_spacing, sizeof(__pyx_k_font_line_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_font_size, __pyx_k_font_size, sizeof(__pyx_k_font_size), 0, 0, 1, 1},
  {&__pyx_n_s_ftfont_scale, __pyx_k_ftfont_scale, sizeof(__pyx_k_ftfont_scale), 0, 0, 1, 1},
  {&__pyx_n_s_ftfont_vertical_extent_scale, __pyx_k_ftfont_vertical_extent_scale, sizeof(__pyx_k_ftfont_vertical_extent_scale), 0, 0, 1, 1},
  {&__pyx_n_s_game, __pyx_k_game, sizeof(__pyx_k_game), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_glyphs, __pyx_k_glyphs, sizeof(__pyx_k_glyphs), 0, 0, 1, 1},
  {&__pyx_n_s_hinting, __pyx_k_hinting, sizeof(__pyx_k_hinting), 0, 0, 1, 1},
  {&__pyx_kp_s_home_tom_ab_renpy_renpy_text_ft, __pyx_k_home_tom_ab_renpy_renpy_text_ft, sizeof(__pyx_k_home_tom_ab_renpy_renpy_text_ft), 0, 0, 1, 0},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_init_2, __pyx_k_init_2, sizeof(__pyx_k_init_2), 0, 0, 1, 1},
  {&__pyx_n_s_italic, __pyx_k_italic, sizeof(__pyx_k_italic), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_none, __pyx_k_none, sizeof(__pyx_k_none), 0, 0, 1, 1},
  {&__pyx_n_s_outline, __pyx_k_outline, sizeof(__pyx_k_outline), 0, 0, 1, 1},
  {&__pyx_n_s_preferences, __pyx_k_preferences, sizeof(__pyx_k_preferences), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_print_exc, __pyx_k_print_exc, sizeof(__pyx_k_print_exc), 0, 0, 1, 1},
  {&__pyx_n_s_pysurf, __pyx_k_pysurf, sizeof(__pyx_k_pysurf), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_read, __pyx_k_read, sizeof(__pyx_k_read), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_renpy, __pyx_k_renpy, sizeof(__pyx_k_renpy), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_config, __pyx_k_renpy_config, sizeof(__pyx_k_renpy_config), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_text_ftfont, __pyx_k_renpy_text_ftfont, sizeof(__pyx_k_renpy_text_ftfont), 0, 0, 1, 1},
  {&__pyx_n_s_seek, __pyx_k_seek, sizeof(__pyx_k_seek), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self_face_self_open_args_self_st, __pyx_k_self_face_self_open_args_self_st, sizeof(__pyx_k_self_face_self_open_args_self_st), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_strikethrough, __pyx_k_strikethrough, sizeof(__pyx_k_strikethrough), 0, 0, 1, 1},
  {&__pyx_n_s_tell, __pyx_k_tell, sizeof(__pyx_k_tell), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_traceback, __pyx_k_traceback, sizeof(__pyx_k_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_underline, __pyx_k_underline, sizeof(__pyx_k_underline), 0, 0, 1, 1},
  {&__pyx_n_s_vertical, __pyx_k_vertical, sizeof(__pyx_k_vertical), 0, 0, 1, 1},
  {&__pyx_n_s_xo, __pyx_k_xo, sizeof(__pyx_k_xo), 0, 0, 1, 1},
  {&__pyx_n_s_yo, __pyx_k_yo, sizeof(__pyx_k_yo), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "renpy/text/ftfont.pyx":165
 *         self.f = f
 * 
 *         f.seek(0, 2)             # <<<<<<<<<<<<<<
 *         size = f.tell()
 *         f.seek(0, 0)
 */
  __pyx_tuple_ = PyTuple_Pack(2, __pyx_int_0, __pyx_int_2); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "renpy/text/ftfont.pyx":167
 *         f.seek(0, 2)
 *         size = f.tell()
 *         f.seek(0, 0)             # <<<<<<<<<<<<<<
 * 
 *         # The offset within the stream we're currently at.
 */
  __pyx_tuple__2 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_0); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_self_face_self_open_args_self_st); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "(tree fragment)":4
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.face,self.open_args,self.stream cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_self_face_self_open_args_self_st); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "renpy/text/ftfont.pyx":58
 * 
 * class FreetypeError(Exception):
 *     def __init__(self, code):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 */
  __pyx_tuple__7 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_code); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__7, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_text_ft, __pyx_n_s_init, 58, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) __PYX_ERR(1, 58, __pyx_L1_error)

  /* "renpy/text/ftfont.pyx":61
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 * def init():             # <<<<<<<<<<<<<<
 * 
 *     cdef int error
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_error); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_text_ft, __pyx_n_s_init_2, 61, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initftfont(void); /*proto*/
PyMODINIT_FUNC initftfont(void)
#else
PyMODINIT_FUNC PyInit_ftfont(void); /*proto*/
PyMODINIT_FUNC PyInit_ftfont(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_ftfont(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_ftfont(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(1, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("ftfont", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(1, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(1, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(1, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__text__ftfont) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(1, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.text.ftfont")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.text.ftfont", __pyx_m) < 0)) __PYX_ERR(1, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_5renpy_4text_6ftfont_FTFace) < 0) __PYX_ERR(1, 134, __pyx_L1_error)
  __pyx_type_5renpy_4text_6ftfont_FTFace.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "FTFace", (PyObject *)&__pyx_type_5renpy_4text_6ftfont_FTFace) < 0) __PYX_ERR(1, 134, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_4text_6ftfont_FTFace) < 0) __PYX_ERR(1, 134, __pyx_L1_error)
  __pyx_ptype_5renpy_4text_6ftfont_FTFace = &__pyx_type_5renpy_4text_6ftfont_FTFace;
  __pyx_vtabptr_5renpy_4text_6ftfont_FTFont = &__pyx_vtable_5renpy_4text_6ftfont_FTFont;
  __pyx_vtable_5renpy_4text_6ftfont_FTFont.setup = (PyObject *(*)(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *))__pyx_f_5renpy_4text_6ftfont_6FTFont_setup;
  __pyx_vtable_5renpy_4text_6ftfont_FTFont.get_glyph = (struct __pyx_t_5renpy_4text_6ftfont_glyph_cache *(*)(struct __pyx_obj_5renpy_4text_6ftfont_FTFont *, int))__pyx_f_5renpy_4text_6ftfont_6FTFont_get_glyph;
  if (PyType_Ready(&__pyx_type_5renpy_4text_6ftfont_FTFont) < 0) __PYX_ERR(1, 192, __pyx_L1_error)
  __pyx_type_5renpy_4text_6ftfont_FTFont.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_4text_6ftfont_FTFont.tp_dict, __pyx_vtabptr_5renpy_4text_6ftfont_FTFont) < 0) __PYX_ERR(1, 192, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "FTFont", (PyObject *)&__pyx_type_5renpy_4text_6ftfont_FTFont) < 0) __PYX_ERR(1, 192, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_4text_6ftfont_FTFont) < 0) __PYX_ERR(1, 192, __pyx_L1_error)
  __pyx_ptype_5renpy_4text_6ftfont_FTFont = &__pyx_type_5renpy_4text_6ftfont_FTFont;
  /*--- Type import code ---*/
  __pyx_ptype_5renpy_4text_11textsupport_Glyph = __Pyx_ImportType("renpy.text.textsupport", "Glyph", sizeof(struct __pyx_obj_5renpy_4text_11textsupport_Glyph), 1); if (unlikely(!__pyx_ptype_5renpy_4text_11textsupport_Glyph)) __PYX_ERR(2, 33, __pyx_L1_error)
  __pyx_ptype_5renpy_4text_11textsupport_Line = __Pyx_ImportType("renpy.text.textsupport", "Line", sizeof(struct __pyx_obj_5renpy_4text_11textsupport_Line), 1); if (unlikely(!__pyx_ptype_5renpy_4text_11textsupport_Line)) __PYX_ERR(2, 73, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  #endif

  /* "renpy/text/ftfont.pyx":27
 * from sdl2 cimport *
 * from pygame_sdl2 cimport *
 * import_pygame_sdl2()             # <<<<<<<<<<<<<<
 * 
 * from freetype cimport *
 */
  import_pygame_sdl2();

  /* "renpy/text/ftfont.pyx":32
 * from ttgsubtable cimport *
 * from textsupport cimport Glyph, SPLIT_INSTEAD
 * import traceback             # <<<<<<<<<<<<<<
 * 
 * import renpy.config
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_traceback, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_traceback, __pyx_t_1) < 0) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":34
 * import traceback
 * 
 * import renpy.config             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "ftsupport.h":
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_renpy_config, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_renpy, __pyx_t_1) < 0) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":57
 * 
 * 
 * class FreetypeError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, code):
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_1, __pyx_n_s_FreetypeError, __pyx_n_s_FreetypeError, (PyObject *) NULL, __pyx_n_s_renpy_text_ftfont, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "renpy/text/ftfont.pyx":58
 * 
 * class FreetypeError(Exception):
 *     def __init__(self, code):             # <<<<<<<<<<<<<<
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5renpy_4text_6ftfont_13FreetypeError_1__init__, 0, __pyx_n_s_FreetypeError___init, NULL, __pyx_n_s_renpy_text_ftfont, __pyx_d, ((PyObject *)__pyx_codeobj__8)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyObject_SetItem(__pyx_t_3, __pyx_n_s_init, __pyx_t_4) < 0) __PYX_ERR(1, 58, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/text/ftfont.pyx":57
 * 
 * 
 * class FreetypeError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, code):
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 */
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_FreetypeError, __pyx_t_1, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FreetypeError, __pyx_t_4) < 0) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":61
 *         Exception.__init__(self, "%d: %s" % (code, freetype_error_to_string(code)))
 * 
 * def init():             # <<<<<<<<<<<<<<
 * 
 *     cdef int error
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_5renpy_4text_6ftfont_1init, NULL, __pyx_n_s_renpy_text_ftfont); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_init_2, __pyx_t_1) < 0) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/text/ftfont.pyx":1
 * #@PydevCodeAnalysisIgnore             # <<<<<<<<<<<<<<
 * # Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 * #
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.text.ftfont", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.text.ftfont");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* SaveResetException */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
  #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCallNoArg */
    #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* WriteUnraisableException */
      static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* KeywordStringCheck */
      static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* ExtTypeTest */
      static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* BytesEquals */
      static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
      static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* None */
      static CYTHON_INLINE FT_Pos __Pyx_div_FT_Pos(FT_Pos a, FT_Pos b) {
    FT_Pos q = a / b;
    FT_Pos r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* None */
      static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* ArgTypeTest */
      static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* GetItemIntUnicode */
      static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(ustring) < 0)) return (Py_UCS4)-1;
    if (wraparound | boundscheck) {
        length = __Pyx_PyUnicode_GET_LENGTH(ustring);
        if (wraparound & unlikely(i < 0)) i += length;
        if ((!boundscheck) || likely((0 <= i) & (i < length))) {
            return __Pyx_PyUnicode_READ_CHAR(ustring, i);
        } else {
            PyErr_SetString(PyExc_IndexError, "string index out of range");
            return (Py_UCS4)-1;
        }
    } else {
        return __Pyx_PyUnicode_READ_CHAR(ustring, i);
    }
}

/* RaiseTooManyValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* SetupReduce */
      static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* SetVTable */
      static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* Import */
      static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CalculateMetaclass */
      static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
      static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
      static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Py3ClassCreate */
          static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
          #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
          static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
          #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_long(unsigned long value) {
    const unsigned long neg_one = (unsigned long) -1, const_zero = (unsigned long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FT_Pos(FT_Pos value) {
    const FT_Pos neg_one = (FT_Pos) -1, const_zero = (FT_Pos) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(FT_Pos) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(FT_Pos) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT_Pos) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(FT_Pos) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FT_Pos) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(FT_Pos),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
          static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE unsigned long __Pyx_PyInt_As_unsigned_long(PyObject *x) {
    const unsigned long neg_one = (unsigned long) -1, const_zero = (unsigned long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned long, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) >= 2 * PyLong_SHIFT) {
                            return (unsigned long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) >= 3 * PyLong_SHIFT) {
                            return (unsigned long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) >= 4 * PyLong_SHIFT) {
                            return (unsigned long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned long) (((unsigned long)-1)*(((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned long) ((((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned long) (((unsigned long)-1)*(((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned long) ((((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned long) (((unsigned long)-1)*(((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned long) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned long) ((((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned long) -1;
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned long) -1;
        val = __Pyx_PyInt_As_unsigned_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned long");
    return (unsigned long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned long");
    return (unsigned long) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE FT_Long __Pyx_PyInt_As_FT_Long(PyObject *x) {
    const FT_Long neg_one = (FT_Long) -1, const_zero = (FT_Long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(FT_Long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(FT_Long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (FT_Long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT_Long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(FT_Long, digit, digits[0])
                case 2:
                    if (8 * sizeof(FT_Long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) >= 2 * PyLong_SHIFT) {
                            return (FT_Long) (((((FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT_Long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) >= 3 * PyLong_SHIFT) {
                            return (FT_Long) (((((((FT_Long)digits[2]) << PyLong_SHIFT) | (FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT_Long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) >= 4 * PyLong_SHIFT) {
                            return (FT_Long) (((((((((FT_Long)digits[3]) << PyLong_SHIFT) | (FT_Long)digits[2]) << PyLong_SHIFT) | (FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (FT_Long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(FT_Long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT_Long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT_Long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT_Long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (FT_Long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(FT_Long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(FT_Long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(FT_Long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) - 1 > 2 * PyLong_SHIFT) {
                            return (FT_Long) (((FT_Long)-1)*(((((FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(FT_Long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) - 1 > 2 * PyLong_SHIFT) {
                            return (FT_Long) ((((((FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(FT_Long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) - 1 > 3 * PyLong_SHIFT) {
                            return (FT_Long) (((FT_Long)-1)*(((((((FT_Long)digits[2]) << PyLong_SHIFT) | (FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(FT_Long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) - 1 > 3 * PyLong_SHIFT) {
                            return (FT_Long) ((((((((FT_Long)digits[2]) << PyLong_SHIFT) | (FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(FT_Long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) - 1 > 4 * PyLong_SHIFT) {
                            return (FT_Long) (((FT_Long)-1)*(((((((((FT_Long)digits[3]) << PyLong_SHIFT) | (FT_Long)digits[2]) << PyLong_SHIFT) | (FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(FT_Long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(FT_Long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(FT_Long) - 1 > 4 * PyLong_SHIFT) {
                            return (FT_Long) ((((((((((FT_Long)digits[3]) << PyLong_SHIFT) | (FT_Long)digits[2]) << PyLong_SHIFT) | (FT_Long)digits[1]) << PyLong_SHIFT) | (FT_Long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(FT_Long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT_Long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(FT_Long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(FT_Long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            FT_Long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (FT_Long) -1;
        }
    } else {
        FT_Long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (FT_Long) -1;
        val = __Pyx_PyInt_As_FT_Long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to FT_Long");
    return (FT_Long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to FT_Long");
    return (FT_Long) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FastTypeChecks */
          #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
          static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
          #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
          #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
          static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
