/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__gl__gltexture
#define __PYX_HAVE_API__renpy__gl__gltexture
#include "glcompat.h"
#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include "SDL.h"
#include "pygame_sdl2/pygame_sdl2.h"
#include <stdlib.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "gltexture.pyx",
  "gltexture.pxd",
  "stringsource",
  "matrix.pxd",
  "render.pxd",
  "gldraw.pxd",
  "type.pxd",
};
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif


/*--- Type declarations ---*/
struct __pyx_obj_5renpy_7display_6matrix_Matrix;
struct __pyx_obj_5renpy_7display_6matrix_Matrix2D;
struct __pyx_obj_5renpy_7display_6render_Render;
struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw;
struct __pyx_obj_5renpy_2gl_6gldraw_Environ;
struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore;
struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid;
struct __pyx_opt_args_5renpy_7display_6render_6Render_blit;
struct __pyx_opt_args_5renpy_7display_6render_6Render_subpixel_blit;
struct __pyx_opt_args_5renpy_7display_6render_6Render_absolute_blit;

/* "renpy/display/render.pxd":107
 *     # operations ###############################################################
 * 
 *     cpdef int blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)             # <<<<<<<<<<<<<<
 *     cpdef int subpixel_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int absolute_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 */
struct __pyx_opt_args_5renpy_7display_6render_6Render_blit {
  int __pyx_n;
  PyObject *focus;
  PyObject *main;
  PyObject *index;
};

/* "renpy/display/render.pxd":108
 * 
 *     cpdef int blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int subpixel_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)             # <<<<<<<<<<<<<<
 *     cpdef int absolute_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 * 
 */
struct __pyx_opt_args_5renpy_7display_6render_6Render_subpixel_blit {
  int __pyx_n;
  PyObject *focus;
  PyObject *main;
  PyObject *index;
};

/* "renpy/display/render.pxd":109
 *     cpdef int blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int subpixel_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)
 *     cpdef int absolute_blit(Render self, source, tuple pos, object focus=*, object main=*, object index=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_5renpy_7display_6render_6Render_absolute_blit {
  int __pyx_n;
  PyObject *focus;
  PyObject *main;
  PyObject *index;
};

/* "renpy/display/matrix.pxd":1
 * cdef class Matrix:             # <<<<<<<<<<<<<<
 * 
 *     cdef float m[0]
 */
struct __pyx_obj_5renpy_7display_6matrix_Matrix {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix *__pyx_vtab;
  float m[0];
  float xdx;
  float xdy;
  float xdz;
  float xdw;
  float ydx;
  float ydy;
  float ydz;
  float ydw;
  float zdx;
  float zdy;
  float zdz;
  float zdw;
  float wdx;
  float wdy;
  float wdz;
  float wdw;
};


/* "renpy/display/matrix.pxd":43
 *     cpdef bint is_unit_aligned(Matrix self)
 * 
 * cdef class Matrix2D(Matrix):             # <<<<<<<<<<<<<<
 *     pass
 */
struct __pyx_obj_5renpy_7display_6matrix_Matrix2D {
  struct __pyx_obj_5renpy_7display_6matrix_Matrix __pyx_base;
};


/* "renpy/display/render.pxd":25
 * from renpy.display.matrix cimport Matrix, Matrix2D
 * 
 * cdef class Render:             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint mark, cache_killed
 */
struct __pyx_obj_5renpy_7display_6render_Render {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_7display_6render_Render *__pyx_vtab;
  int mark;
  int cache_killed;
  float width;
  float height;
  PyObject *layer_name;
  PyObject *children;
  PyObject *parents;
  PyObject *depends_on_list;
  int operation;
  double operation_complete;
  int operation_alpha;
  PyObject *operation_parameter;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *forward;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *reverse;
  double alpha;
  double over;
  PyObject *nearest;
  PyObject *focuses;
  PyObject *pass_focuses;
  PyObject *focus_screen;
  PyObject *draw_func;
  PyObject *render_of;
  int opaque;
  PyObject *visible_children;
  int xclipping;
  int yclipping;
  PyObject *surface;
  PyObject *alpha_surface;
  PyObject *half_cache;
  int modal;
  int text_input;
  PyObject *mesh;
  PyObject *shaders;
  PyObject *uniforms;
  PyObject *properties;
  PyObject *cached_texture;
  PyObject *cached_model;
  int loaded;
};


/* "gldraw.pxd":27
 * cdef class Environ
 * 
 * cdef class GLDraw:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint did_init
 */
struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_2gl_6gldraw_GLDraw *__pyx_vtab;
  int did_init;
  int did_texture_test;
  struct __pyx_obj_5renpy_2gl_6gldraw_Environ *environ;
  PyObject *rtt;
  PyObject *window;
  PyObject *virtual_size;
  PyObject *physical_size;
  PyObject *drawable_size;
  PyObject *virtual_box;
  PyObject *physical_box;
  PyObject *mouse_old_visible;
  PyObject *mouse_info;
  PyObject *texture_cache;
  double last_redraw_time;
  double redraw_period;
  PyObject *info;
  PyObject *old_fullscreen;
  PyObject *fullscreen_surface;
  PyObject *display_info;
  PyObject *clip_cache;
  int fast_dissolve;
  int allow_fixed;
  PyObject *default_clip;
  int did_render_to_texture;
  float dpi_scale;
  PyObject *ready_texture_queue;
  PyObject *clip_rtt_box;
  PyObject *draw_per_virt;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *virt_to_draw;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *draw_to_virt;
  int fast_redraw_frames;
};


/* "gldraw.pxd":25
 * cimport renpy.display.render as render
 * 
 * cdef class Environ             # <<<<<<<<<<<<<<
 * 
 * cdef class GLDraw:
 */
struct __pyx_obj_5renpy_2gl_6gldraw_Environ {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *__pyx_vtab;
};


/* "renpy/gl/gltexture.pxd":26
 * from gldraw cimport Environ
 * 
 * cdef class TextureCore:             # <<<<<<<<<<<<<<
 *     cdef public int width
 *     cdef public int height
 */
struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *__pyx_vtab;
  int width;
  int height;
  int generation;
  unsigned int number;
  int format;
  double xmul;
  double xadd;
  double ymul;
  double yadd;
  PyObject *premult;
  PyObject *premult_size;
  int premult_left;
  int premult_right;
  int premult_top;
  int premult_bottom;
  int nearest;
  PyObject *free_list;
  PyObject *debug;
};


/* "renpy/gl/gltexture.pxd":52
 *     cdef public object debug
 * 
 * cdef class TextureGrid:             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *__pyx_vtab;
  PyObject *__weakref__;
  int width;
  int height;
  PyObject *rows;
  PyObject *columns;
  PyObject *tiles;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *half_cache;
  PyObject *debug;
  int ready;
};



/* "renpy/display/matrix.pxd":1
 * cdef class Matrix:             # <<<<<<<<<<<<<<
 * 
 *     cdef float m[0]
 */

struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix {
  void (*transform4)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float *, float, float, float, float);
  void (*transform3)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float, float, float, float);
  void (*transform2)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float, float, float, float);
  int (*is_unit_aligned)(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix *__pyx_vtabptr_5renpy_7display_6matrix_Matrix;
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float *, float, float, float, float);
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float *, float, float, float, float);
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *, float *, float *, float, float, float, float);


/* "renpy/display/matrix.pxd":43
 *     cpdef bint is_unit_aligned(Matrix self)
 * 
 * cdef class Matrix2D(Matrix):             # <<<<<<<<<<<<<<
 *     pass
 */

struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D {
  struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D *__pyx_vtabptr_5renpy_7display_6matrix_Matrix2D;


/* "renpy/display/render.pxd":25
 * from renpy.display.matrix cimport Matrix, Matrix2D
 * 
 * cdef class Render:             # <<<<<<<<<<<<<<
 * 
 *     cdef public bint mark, cache_killed
 */

struct __pyx_vtabstruct_5renpy_7display_6render_Render {
  int (*blit)(struct __pyx_obj_5renpy_7display_6render_Render *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5renpy_7display_6render_6Render_blit *__pyx_optional_args);
  int (*subpixel_blit)(struct __pyx_obj_5renpy_7display_6render_Render *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5renpy_7display_6render_6Render_subpixel_blit *__pyx_optional_args);
  int (*absolute_blit)(struct __pyx_obj_5renpy_7display_6render_Render *, PyObject *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5renpy_7display_6render_6Render_absolute_blit *__pyx_optional_args);
};
static struct __pyx_vtabstruct_5renpy_7display_6render_Render *__pyx_vtabptr_5renpy_7display_6render_Render;


/* "gldraw.pxd":25
 * cimport renpy.display.render as render
 * 
 * cdef class Environ             # <<<<<<<<<<<<<<
 * 
 * cdef class GLDraw:
 */

struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ {
  void (*blit)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *);
  void (*blend)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, double);
  void (*imageblend)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, double, int);
  void (*set_vertex)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, float *);
  void (*set_texture)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, float *);
  void (*set_color)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, double, double, double, double);
  void (*set_clip)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, PyObject *, struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw *);
  void (*unset_clip)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw *);
  void (*ortho)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, double, double, double, double, double, double);
  void (*viewport)(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int, int, int);
};
static struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *__pyx_vtabptr_5renpy_2gl_6gldraw_Environ;


/* "gldraw.pxd":27
 * cdef class Environ
 * 
 * cdef class GLDraw:             # <<<<<<<<<<<<<<
 * 
 *     cdef bint did_init
 */

struct __pyx_vtabstruct_5renpy_2gl_6gldraw_GLDraw {
  PyObject *(*set_clip)(struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw *, PyObject *, int __pyx_skip_dispatch);
  int (*draw_render_textures)(struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw *, PyObject *, int, int __pyx_skip_dispatch);
  int (*draw_transformed)(struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw *, PyObject *, PyObject *, double, double, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, int, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_2gl_6gldraw_GLDraw *__pyx_vtabptr_5renpy_2gl_6gldraw_GLDraw;


/* "renpy/gl/gltexture.pyx":256
 *     return True
 * 
 * cdef class TextureCore:             # <<<<<<<<<<<<<<
 *     """
 *     This object stores information about an OpenGL texture.
 */

struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore {
  void (*make_ready)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *);
  void (*make_nearest)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *);
  void (*make_linear)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *);
  int (*allocate)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *__pyx_vtabptr_5renpy_2gl_9gltexture_TextureCore;


/* "renpy/gl/gltexture.pyx":659
 * 
 * 
 * cdef class TextureGrid(object):             # <<<<<<<<<<<<<<
 *     """
 *     This represents one or more textures that cover a rectangular
 */

struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid {
  void (*make_ready)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *__pyx_vtabptr_5renpy_2gl_9gltexture_TextureGrid;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_SubtractObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o,n,NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* pop.proto */
static CYTHON_INLINE PyObject* __Pyx__PyObject_Pop(PyObject* L);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE PyObject* __Pyx_PyList_Pop(PyObject* L);
#define __Pyx_PyObject_Pop(L) (likely(PyList_CheckExact(L)) ?\
    __Pyx_PyList_Pop(L) : __Pyx__PyObject_Pop(L))
#else
#define __Pyx_PyList_Pop(L)  __Pyx__PyObject_Pop(L)
#define __Pyx_PyObject_Pop(L)  __Pyx__PyObject_Pop(L)
#endif

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    ((likely((cfunc)->func)) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
             ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
              (PY_VERSION_HEX >= 0x030600B1 && (cfunc)->flag == METH_FASTCALL ?\
                (PY_VERSION_HEX >= 0x030700A0 ?\
                    (*(__Pyx_PyCFunctionFast)(cfunc)->func)(self, &PyTuple_GET_ITEM(__pyx_empty_tuple, 0), 0) :\
                    (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &PyTuple_GET_ITEM(__pyx_empty_tuple, 0), 0, NULL)) :\
              (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
                    (*(__Pyx_PyCFunctionFastWithKeywords)(cfunc)->func)(self, &PyTuple_GET_ITEM(__pyx_empty_tuple, 0), 0, NULL) :\
                    __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLenum(GLenum value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLuint __Pyx_PyInt_As_GLuint(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE GLsizei __Pyx_PyInt_As_GLsizei(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_nearest(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto*/
static void __pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_linear(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto*/
static void __pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_ready(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto*/
static int __pyx_f_5renpy_2gl_9gltexture_11TextureCore_allocate(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_5renpy_2gl_9gltexture_11TextureGrid_make_ready(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, int __pyx_v_nearest, int __pyx_skip_dispatch); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float *__pyx_v_ow, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/
static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w); /* proto*/

/* Module declarations from 'renpy.display.matrix' */
static PyTypeObject *__pyx_ptype_5renpy_7display_6matrix_Matrix = 0;
static PyTypeObject *__pyx_ptype_5renpy_7display_6matrix_Matrix2D = 0;

/* Module declarations from 'renpy.display.render' */
static PyTypeObject *__pyx_ptype_5renpy_7display_6render_Render = 0;

/* Module declarations from 'renpy.gl.gldraw' */
static PyTypeObject *__pyx_ptype_5renpy_2gl_6gldraw_Environ = 0;
static PyTypeObject *__pyx_ptype_5renpy_2gl_6gldraw_GLDraw = 0;

/* Module declarations from 'renpy.gl.gl' */
static int (*__pyx_f_5renpy_2gl_2gl_glActiveTextureARB)(GLenum); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glBindTexture)(GLenum, GLuint); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glDeleteTextures)(GLsizei, GLuint *); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glDrawArrays)(GLenum, GLint, GLsizei); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glGenTextures)(GLsizei, GLuint *); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glGetIntegerv)(GLenum, GLint *); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glReadPixels)(GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, GLubyte *); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glTexImage2D)(GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, GLubyte *); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glTexParameteri)(GLenum, GLenum, GLint); /*proto*/
static int (*__pyx_f_5renpy_2gl_2gl_glTexSubImage2D)(GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, GLubyte *); /*proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'libc.stddef' */

/* Module declarations from 'sdl2' */

/* Module declarations from 'pygame_sdl2' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.string' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'renpy.gl.gltexture' */
static PyTypeObject *__pyx_ptype_5renpy_2gl_9gltexture_TextureCore = 0;
static PyTypeObject *__pyx_ptype_5renpy_2gl_9gltexture_TextureGrid = 0;
static GLenum __pyx_v_5renpy_2gl_9gltexture_tex_format;
static GLenum __pyx_v_5renpy_2gl_9gltexture_tex_internalformat;
static GLenum __pyx_v_5renpy_2gl_9gltexture_tex_type;
static GLenum __pyx_v_5renpy_2gl_9gltexture_rtt_format;
static GLenum __pyx_v_5renpy_2gl_9gltexture_rtt_internalformat;
static GLenum __pyx_v_5renpy_2gl_9gltexture_rtt_type;
static PyObject *__pyx_f_5renpy_2gl_9gltexture_blit(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, double, double, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture_blend(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, double, double, double, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture_imageblend(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, double, double, double, int, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int __pyx_skip_dispatch); /*proto*/
static void __pyx_f_5renpy_2gl_9gltexture_draw_rectangle(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, double, double, double, double, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *, float, float, struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *, float, float, struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *, float, float); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureCore__set_state(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *, PyObject *); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureGrid__set_state(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "renpy.gl.gltexture"
extern int __pyx_module_is_main_renpy__gl__gltexture;
int __pyx_module_is_main_renpy__gl__gltexture = 0;

/* Implementation of 'renpy.gl.gltexture' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_StopIteration;
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_min;
static PyObject *__pyx_builtin_xrange;
static const char __pyx_k_a[] = "a";
static const char __pyx_k_h[] = "h";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_l[] = "l";
static const char __pyx_k_p[] = "p";
static const char __pyx_k_t[] = "t";
static const char __pyx_k_w[] = "w";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_ix[] = "ix";
static const char __pyx_k_iy[] = "iy";
static const char __pyx_k_op[] = "op";
static const char __pyx_k_pp[] = "pp";
static const char __pyx_k_rv[] = "rv";
static const char __pyx_k_sx[] = "sx";
static const char __pyx_k_sy[] = "sy";
static const char __pyx_k_tg[] = "tg";
static const char __pyx_k__27[] = "_";
static const char __pyx_k_add[] = "add";
static const char __pyx_k_cur[] = "cur";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_log[] = "log";
static const char __pyx_k_min[] = "min";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_out[] = "out";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_row[] = "row";
static const char __pyx_k_rtt[] = "rtt";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_tex[] = "tex";
static const char __pyx_k_tg0[] = "tg0";
static const char __pyx_k_tg1[] = "tg1";
static const char __pyx_k_tg2[] = "tg2";
static const char __pyx_k_zip[] = "zip";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_draw[] = "draw";
static const char __pyx_k_game[] = "game";
static const char __pyx_k_ioff[] = "ioff";
static const char __pyx_k_loop[] = "loop";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_next[] = "next";
static const char __pyx_k_over[] = "over";
static const char __pyx_k_pend[] = "pend";
static const char __pyx_k_ramp[] = "ramp";
static const char __pyx_k_rowi[] = "rowi";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_surf[] = "surf";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_time[] = "time";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_SIZES[] = "SIZES";
static const char __pyx_k_alpha[] = "alpha";
static const char __pyx_k_clear[] = "clear";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_itile[] = "itile";
static const char __pyx_k_nargs[] = "nargs";
static const char __pyx_k_pixel[] = "pixel";
static const char __pyx_k_ppend[] = "ppend";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_renpy[] = "renpy";
static const char __pyx_k_state[] = "state";
static const char __pyx_k_tiles[] = "tiles";
static const char __pyx_k_width[] = "width";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_bitmap[] = "bitmap";
static const char __pyx_k_colnum[] = "colnum";
static const char __pyx_k_config[] = "config";
static const char __pyx_k_coords[] = "coords";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_gldraw[] = "gldraw";
static const char __pyx_k_height[] = "height";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_iwidth[] = "iwidth";
static const char __pyx_k_little[] = "little";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_outrow[] = "outrow";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_pixels[] = "pixels";
static const char __pyx_k_pwidth[] = "pwidth";
static const char __pyx_k_pysurf[] = "pysurf";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_render[] = "render";
static const char __pyx_k_rv_col[] = "rv_col";
static const char __pyx_k_rv_row[] = "rv_row";
static const char __pyx_k_sizeof[] = "__sizeof__";
static const char __pyx_k_texcol[] = "texcol";
static const char __pyx_k_texrow[] = "texrow";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_use_gl[] = "use_gl";
static const char __pyx_k_values[] = "values";
static const char __pyx_k_xrange[] = "xrange";
static const char __pyx_k_Texture[] = "Texture";
static const char __pyx_k_cleanup[] = "cleanup";
static const char __pyx_k_discard[] = "discard";
static const char __pyx_k_display[] = "display";
static const char __pyx_k_environ[] = "environ";
static const char __pyx_k_gl_npot[] = "gl_npot";
static const char __pyx_k_minsize[] = "minsize";
static const char __pyx_k_nearest[] = "nearest";
static const char __pyx_k_outtile[] = "outtile";
static const char __pyx_k_pheight[] = "pheight";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_reverse[] = "reverse";
static const char __pyx_k_texnums[] = "texnums";
static const char __pyx_k_texrows[] = "texrows";
static const char __pyx_k_MAX_SIZE[] = "MAX_SIZE";
static const char __pyx_k_allocate[] = "allocate";
static const char __pyx_k_fraction[] = "fraction";
static const char __pyx_k_get_size[] = "get_size";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_max_size[] = "max_size";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_row_size[] = "row_size";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_texwidth[] = "texwidth";
static const char __pyx_k_use_gles[] = "use_gles";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_byteorder[] = "byteorder";
static const char __pyx_k_developer[] = "developer";
static const char __pyx_k_draw_func[] = "draw_func";
static const char __pyx_k_free_list[] = "free_list";
static const char __pyx_k_get_masks[] = "get_masks";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_render_to[] = "render_to";
static const char __pyx_k_row_index[] = "row_index";
static const char __pyx_k_texcoords[] = "texcoords";
static const char __pyx_k_texheight[] = "texheight";
static const char __pyx_k_transform[] = "transform";
static const char __pyx_k_transient[] = "transient";
static const char __pyx_k_align_axes[] = "align_axes";
static const char __pyx_k_border_top[] = "border_top";
static const char __pyx_k_deallocate[] = "deallocate";
static const char __pyx_k_generation[] = "generation";
static const char __pyx_k_make_ready[] = "make_ready";
static const char __pyx_k_orig_width[] = "orig_width";
static const char __pyx_k_pixels_end[] = "pixels_end";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_texcolumns[] = "texcolumns";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_border_left[] = "border_left";
static const char __pyx_k_collections[] = "collections";
static const char __pyx_k_defaultdict[] = "defaultdict";
static const char __pyx_k_fill_factor[] = "fill_factor";
static const char __pyx_k_hw_max_size[] = "hw_max_size";
static const char __pyx_k_left_border[] = "left_border";
static const char __pyx_k_old_gl_npot[] = "old_gl_npot";
static const char __pyx_k_preferences[] = "preferences";
static const char __pyx_k_premultiply[] = "premultiply";
static const char __pyx_k_border_right[] = "border_right";
static const char __pyx_k_load_surface[] = "load_surface";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_right_border[] = "right_border";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_tiling_cache[] = "tiling_cache";
static const char __pyx_k_StopIteration[] = "StopIteration";
static const char __pyx_k_alloc_texture[] = "alloc_texture";
static const char __pyx_k_border_bottom[] = "border_bottom";
static const char __pyx_k_free_textures[] = "free_textures";
static const char __pyx_k_physical_size[] = "physical_size";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_texture_count[] = "texture_count";
static const char __pyx_k_compute_subrow[] = "compute_subrow";
static const char __pyx_k_compute_tiling[] = "compute_tiling";
static const char __pyx_k_get_size_limit[] = "get_size_limit";
static const char __pyx_k_internalformat[] = "internalformat";
static const char __pyx_k_Texture_testing[] = "Texture testing:";
static const char __pyx_k_min_fill_factor[] = "min_fill_factor";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_texture_numbers[] = "texture_numbers";
static const char __pyx_k_Texture___sizeof[] = "Texture.__sizeof__";
static const char __pyx_k_dealloc_textures[] = "dealloc_textures";
static const char __pyx_k_0_px_textures_work[] = "- {0}px textures work.";
static const char __pyx_k_Texture___getstate[] = "Texture.__getstate__";
static const char __pyx_k_Texture___setstate[] = "Texture.__setstate__";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_load_premultiplied[] = "load_premultiplied";
static const char __pyx_k_npot_free_textures[] = "npot_free_textures";
static const char __pyx_k_renpy_gl_gltexture[] = "renpy.gl.gltexture";
static const char __pyx_k_test_texture_sizes[] = "test_texture_sizes";
static const char __pyx_k_texture_generation[] = "texture_generation";
static const char __pyx_k_total_texture_size[] = "total_texture_size";
static const char __pyx_k_Can_t_pickle_a_texture[] = "Can't pickle a texture.";
static const char __pyx_k_pyx_unpickle_TextureCore[] = "__pyx_unpickle_TextureCore";
static const char __pyx_k_pyx_unpickle_TextureGrid[] = "__pyx_unpickle_TextureGrid";
static const char __pyx_k_texture_grid_from_drawing[] = "texture_grid_from_drawing";
static const char __pyx_k_texture_grid_from_surface[] = "texture_grid_from_surface";
static const char __pyx_k_Hardware_max_texture_size_d[] = "- Hardware max texture size: %d";
static const char __pyx_k_Error_loading_0_px_bitmap_1_x[] = "- Error loading {0}px bitmap: {1:x}";
static const char __pyx_k_Could_not_allocate_0_px_bitmap[] = "- Could not allocate {0}px bitmap.";
static const char __pyx_k_Error_drawing_0_px_texture_1_x[] = "- Error drawing {0}px texture: {1:x}";
static const char __pyx_k_Error_reading_0_px_texture_1_x[] = "- Error reading {0}px texture: {1:x}";
static const char __pyx_k_Ignored_error_at_start_of_testi[] = "- Ignored error at start of testing: {0:x}";
static const char __pyx_k_Incorrect_pixel_color_in_0_px_t[] = "- Incorrect pixel color in {0}px texture: ({1:x}, {1:x}, {1:x})";
static const char __pyx_k_We_need_to_be_a_real_python_cla[] = "\n    We need to be a real python class, not a C extension, to ensure that\n    the __del__ method is called.\n    ";
static const char __pyx_k_home_tom_ab_renpy_renpy_gl_glte[] = "/home/tom/ab/renpy/renpy/gl/gltexture.pyx";
static const char __pyx_k_Incompatible_checksums_s_vs_0x44[] = "Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x88[] = "Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))";
static const char __pyx_k_Textures_are_not_rendering_prope[] = "Textures are not rendering properly.";
static PyObject *__pyx_kp_s_0_px_textures_work;
static PyObject *__pyx_kp_s_Can_t_pickle_a_texture;
static PyObject *__pyx_kp_s_Could_not_allocate_0_px_bitmap;
static PyObject *__pyx_kp_s_Error_drawing_0_px_texture_1_x;
static PyObject *__pyx_kp_s_Error_loading_0_px_bitmap_1_x;
static PyObject *__pyx_kp_s_Error_reading_0_px_texture_1_x;
static PyObject *__pyx_kp_s_Hardware_max_texture_size_d;
static PyObject *__pyx_kp_s_Ignored_error_at_start_of_testi;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x44;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x88;
static PyObject *__pyx_kp_s_Incorrect_pixel_color_in_0_px_t;
static PyObject *__pyx_n_s_MAX_SIZE;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_SIZES;
static PyObject *__pyx_n_s_StopIteration;
static PyObject *__pyx_n_s_Texture;
static PyObject *__pyx_n_s_Texture___getstate;
static PyObject *__pyx_n_s_Texture___setstate;
static PyObject *__pyx_n_s_Texture___sizeof;
static PyObject *__pyx_kp_s_Texture_testing;
static PyObject *__pyx_kp_s_Textures_are_not_rendering_prope;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_We_need_to_be_a_real_python_cla;
static PyObject *__pyx_n_s__27;
static PyObject *__pyx_n_s_a;
static PyObject *__pyx_n_s_add;
static PyObject *__pyx_n_s_align_axes;
static PyObject *__pyx_n_s_alloc_texture;
static PyObject *__pyx_n_s_allocate;
static PyObject *__pyx_n_s_alpha;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_bitmap;
static PyObject *__pyx_n_s_border_bottom;
static PyObject *__pyx_n_s_border_left;
static PyObject *__pyx_n_s_border_right;
static PyObject *__pyx_n_s_border_top;
static PyObject *__pyx_n_s_byteorder;
static PyObject *__pyx_n_s_cleanup;
static PyObject *__pyx_n_s_clear;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_collections;
static PyObject *__pyx_n_s_colnum;
static PyObject *__pyx_n_s_compute_subrow;
static PyObject *__pyx_n_s_compute_tiling;
static PyObject *__pyx_n_s_config;
static PyObject *__pyx_n_s_coords;
static PyObject *__pyx_n_s_cur;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_dealloc_textures;
static PyObject *__pyx_n_s_deallocate;
static PyObject *__pyx_n_s_defaultdict;
static PyObject *__pyx_n_s_developer;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_discard;
static PyObject *__pyx_n_s_display;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_draw;
static PyObject *__pyx_n_s_draw_func;
static PyObject *__pyx_n_s_environ;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_fill_factor;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_fraction;
static PyObject *__pyx_n_s_free_list;
static PyObject *__pyx_n_s_free_textures;
static PyObject *__pyx_n_s_game;
static PyObject *__pyx_n_s_generation;
static PyObject *__pyx_n_s_get_masks;
static PyObject *__pyx_n_s_get_size;
static PyObject *__pyx_n_s_get_size_limit;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_gl_npot;
static PyObject *__pyx_n_s_gldraw;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_n_s_height;
static PyObject *__pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte;
static PyObject *__pyx_n_s_hw_max_size;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_internalformat;
static PyObject *__pyx_n_s_ioff;
static PyObject *__pyx_n_s_itile;
static PyObject *__pyx_n_s_iwidth;
static PyObject *__pyx_n_s_ix;
static PyObject *__pyx_n_s_iy;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_l;
static PyObject *__pyx_n_s_left_border;
static PyObject *__pyx_n_s_little;
static PyObject *__pyx_n_s_load_premultiplied;
static PyObject *__pyx_n_s_load_surface;
static PyObject *__pyx_n_s_log;
static PyObject *__pyx_n_s_loop;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_make_ready;
static PyObject *__pyx_n_s_max_size;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_min;
static PyObject *__pyx_n_s_min_fill_factor;
static PyObject *__pyx_n_s_minsize;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nargs;
static PyObject *__pyx_n_s_nearest;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_next;
static PyObject *__pyx_n_s_npot_free_textures;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_old_gl_npot;
static PyObject *__pyx_n_s_op;
static PyObject *__pyx_n_s_orig_width;
static PyObject *__pyx_n_s_out;
static PyObject *__pyx_n_s_outrow;
static PyObject *__pyx_n_s_outtile;
static PyObject *__pyx_n_s_over;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_pend;
static PyObject *__pyx_n_s_pheight;
static PyObject *__pyx_n_s_physical_size;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pixel;
static PyObject *__pyx_n_s_pixels;
static PyObject *__pyx_n_s_pixels_end;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_pp;
static PyObject *__pyx_n_s_ppend;
static PyObject *__pyx_n_s_preferences;
static PyObject *__pyx_n_s_premultiply;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_pwidth;
static PyObject *__pyx_n_s_pysurf;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_TextureCore;
static PyObject *__pyx_n_s_pyx_unpickle_TextureGrid;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_ramp;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_render;
static PyObject *__pyx_n_s_render_to;
static PyObject *__pyx_n_s_renpy;
static PyObject *__pyx_n_s_renpy_gl_gltexture;
static PyObject *__pyx_n_s_reverse;
static PyObject *__pyx_n_s_right_border;
static PyObject *__pyx_n_s_row;
static PyObject *__pyx_n_s_row_index;
static PyObject *__pyx_n_s_row_size;
static PyObject *__pyx_n_s_rowi;
static PyObject *__pyx_n_s_rtt;
static PyObject *__pyx_n_s_rv;
static PyObject *__pyx_n_s_rv_col;
static PyObject *__pyx_n_s_rv_row;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_sizeof;
static PyObject *__pyx_n_s_state;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_surf;
static PyObject *__pyx_n_s_sx;
static PyObject *__pyx_n_s_sy;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_t;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_test_texture_sizes;
static PyObject *__pyx_n_s_tex;
static PyObject *__pyx_n_s_texcol;
static PyObject *__pyx_n_s_texcolumns;
static PyObject *__pyx_n_s_texcoords;
static PyObject *__pyx_n_s_texheight;
static PyObject *__pyx_n_s_texnums;
static PyObject *__pyx_n_s_texrow;
static PyObject *__pyx_n_s_texrows;
static PyObject *__pyx_n_s_texture_count;
static PyObject *__pyx_n_s_texture_generation;
static PyObject *__pyx_n_s_texture_grid_from_drawing;
static PyObject *__pyx_n_s_texture_grid_from_surface;
static PyObject *__pyx_n_s_texture_numbers;
static PyObject *__pyx_n_s_texwidth;
static PyObject *__pyx_n_s_tg;
static PyObject *__pyx_n_s_tg0;
static PyObject *__pyx_n_s_tg1;
static PyObject *__pyx_n_s_tg2;
static PyObject *__pyx_n_s_tiles;
static PyObject *__pyx_n_s_tiling_cache;
static PyObject *__pyx_n_s_time;
static PyObject *__pyx_n_s_total_texture_size;
static PyObject *__pyx_n_s_transform;
static PyObject *__pyx_n_s_transient;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_gl;
static PyObject *__pyx_n_s_use_gles;
static PyObject *__pyx_n_s_values;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_width;
static PyObject *__pyx_n_s_write;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_xrange;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_use_gles(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_2use_gl(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_4test_texture_sizes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, PyObject *__pyx_v_draw); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore___init__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, int __pyx_v_width, int __pyx_v_height); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_2__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_4load_surface(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_surf, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_w, PyObject *__pyx_v_h, PyObject *__pyx_v_border_left, PyObject *__pyx_v_border_top, PyObject *__pyx_v_border_right, PyObject *__pyx_v_border_bottom); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6render_to(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_draw_func, PyObject *__pyx_v_rtt, PyObject *__pyx_v_environ); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_8allocate(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10deallocate(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5width___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5width_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6height___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6height_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10generation___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10generation_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6number___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6number_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_12__reduce_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_14__setstate_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_7Texture___sizeof__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_7Texture_2__getstate__(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_7Texture_4__setstate__(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_state); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_6alloc_texture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width, PyObject *__pyx_v_height); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_8dealloc_textures(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_10cleanup(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_12compute_subrow(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_row, PyObject *__pyx_v_offset, PyObject *__pyx_v_width); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid___init__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_width, PyObject *__pyx_v_height); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_2__getstate__(CYTHON_UNUSED struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_4__setstate__(CYTHON_UNUSED struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_state); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6get_size(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_8subsurface(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_rect); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10make_ready(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, int __pyx_v_nearest); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5width___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5width_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6height___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6height_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5ready___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5ready_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_12__reduce_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_14__setstate_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_14compute_tiling(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width, PyObject *__pyx_v_max_size, PyObject *__pyx_v_min_fill_factor); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_16texture_grid_from_surface(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_surf, PyObject *__pyx_v_transient); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_18texture_grid_from_drawing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_draw_func, PyObject *__pyx_v_rtt, PyObject *__pyx_v_environ); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_20align_axes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_args); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_22blit(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_24blend(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, double __pyx_v_fraction, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_26imageblend(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg2, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, double __pyx_v_fraction, int __pyx_v_ramp, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_28premultiply(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pysurf, int __pyx_v_x, int __pyx_v_y, int __pyx_v_w, int __pyx_v_h, int __pyx_v_border_left, int __pyx_v_border_top, int __pyx_v_border_right, int __pyx_v_border_bottom); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_30load_premultiplied(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_update, PyObject *__pyx_v_rtt); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_32__pyx_unpickle_TextureCore(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_5renpy_2gl_9gltexture_34__pyx_unpickle_TextureGrid(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_5renpy_2gl_9gltexture_TextureCore(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5renpy_2gl_9gltexture_TextureGrid(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyList_Type_pop = {0, &__pyx_n_s_pop, 0, 0, 0};
static PyObject *__pyx_float_0_5;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_4;
static PyObject *__pyx_int_64;
static PyObject *__pyx_int_2048;
static PyObject *__pyx_int_71597589;
static PyObject *__pyx_int_143547127;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_slice__4;
static PyObject *__pyx_slice__5;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_codeobj__7;
static PyObject *__pyx_codeobj__8;
static PyObject *__pyx_codeobj__10;
static PyObject *__pyx_codeobj__12;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__24;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__29;
static PyObject *__pyx_codeobj__31;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__41;

/* "renpy/gl/gltexture.pyx":66
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gles():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_1use_gles(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_1use_gles = {"use_gles", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_1use_gles, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_1use_gles(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("use_gles (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_use_gles(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_use_gles(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("use_gles", 0);

  /* "renpy/gl/gltexture.pyx":74
 *     global rtt_type
 * 
 *     tex_format = GL_RGBA             # <<<<<<<<<<<<<<
 *     tex_internalformat = GL_RGBA
 *     tex_type = GL_UNSIGNED_BYTE
 */
  __pyx_v_5renpy_2gl_9gltexture_tex_format = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":75
 * 
 *     tex_format = GL_RGBA
 *     tex_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 *     tex_type = GL_UNSIGNED_BYTE
 * 
 */
  __pyx_v_5renpy_2gl_9gltexture_tex_internalformat = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":76
 *     tex_format = GL_RGBA
 *     tex_internalformat = GL_RGBA
 *     tex_type = GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * 
 *     rtt_format = GL_RGBA
 */
  __pyx_v_5renpy_2gl_9gltexture_tex_type = GL_UNSIGNED_BYTE;

  /* "renpy/gl/gltexture.pyx":78
 *     tex_type = GL_UNSIGNED_BYTE
 * 
 *     rtt_format = GL_RGBA             # <<<<<<<<<<<<<<
 *     rtt_internalformat = GL_RGBA
 *     rtt_type = GL_UNSIGNED_BYTE
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_format = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":79
 * 
 *     rtt_format = GL_RGBA
 *     rtt_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_internalformat = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":80
 *     rtt_format = GL_RGBA
 *     rtt_internalformat = GL_RGBA
 *     rtt_type = GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * 
 * def use_gl():
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_type = GL_UNSIGNED_BYTE;

  /* "renpy/gl/gltexture.pyx":66
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gles():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":82
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gl():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_3use_gl(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_3use_gl = {"use_gl", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_3use_gl, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_3use_gl(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("use_gl (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_2use_gl(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_2use_gl(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("use_gl", 0);

  /* "renpy/gl/gltexture.pyx":91
 * 
 *     # Optimize for the case of little-endian systems that use ARGB.
 *     if sys.byteorder == 'little':             # <<<<<<<<<<<<<<
 *         tex_format = GL_BGRA
 *         tex_internalformat = GL_RGBA
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_byteorder); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__Pyx_PyString_Equals(__pyx_t_2, __pyx_n_s_little, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":92
 *     # Optimize for the case of little-endian systems that use ARGB.
 *     if sys.byteorder == 'little':
 *         tex_format = GL_BGRA             # <<<<<<<<<<<<<<
 *         tex_internalformat = GL_RGBA
 *         tex_type = GL_UNSIGNED_INT_8_8_8_8_REV
 */
    __pyx_v_5renpy_2gl_9gltexture_tex_format = GL_BGRA;

    /* "renpy/gl/gltexture.pyx":93
 *     if sys.byteorder == 'little':
 *         tex_format = GL_BGRA
 *         tex_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 *         tex_type = GL_UNSIGNED_INT_8_8_8_8_REV
 *     else:
 */
    __pyx_v_5renpy_2gl_9gltexture_tex_internalformat = GL_RGBA;

    /* "renpy/gl/gltexture.pyx":94
 *         tex_format = GL_BGRA
 *         tex_internalformat = GL_RGBA
 *         tex_type = GL_UNSIGNED_INT_8_8_8_8_REV             # <<<<<<<<<<<<<<
 *     else:
 *         tex_format = GL_RGBA
 */
    __pyx_v_5renpy_2gl_9gltexture_tex_type = GL_UNSIGNED_INT_8_8_8_8_REV;

    /* "renpy/gl/gltexture.pyx":91
 * 
 *     # Optimize for the case of little-endian systems that use ARGB.
 *     if sys.byteorder == 'little':             # <<<<<<<<<<<<<<
 *         tex_format = GL_BGRA
 *         tex_internalformat = GL_RGBA
 */
    goto __pyx_L3;
  }

  /* "renpy/gl/gltexture.pyx":96
 *         tex_type = GL_UNSIGNED_INT_8_8_8_8_REV
 *     else:
 *         tex_format = GL_RGBA             # <<<<<<<<<<<<<<
 *         tex_internalformat = GL_RGBA
 *         tex_type = GL_UNSIGNED_BYTE
 */
  /*else*/ {
    __pyx_v_5renpy_2gl_9gltexture_tex_format = GL_RGBA;

    /* "renpy/gl/gltexture.pyx":97
 *     else:
 *         tex_format = GL_RGBA
 *         tex_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 *         tex_type = GL_UNSIGNED_BYTE
 * 
 */
    __pyx_v_5renpy_2gl_9gltexture_tex_internalformat = GL_RGBA;

    /* "renpy/gl/gltexture.pyx":98
 *         tex_format = GL_RGBA
 *         tex_internalformat = GL_RGBA
 *         tex_type = GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * 
 *     rtt_format = GL_RGBA
 */
    __pyx_v_5renpy_2gl_9gltexture_tex_type = GL_UNSIGNED_BYTE;
  }
  __pyx_L3:;

  /* "renpy/gl/gltexture.pyx":100
 *         tex_type = GL_UNSIGNED_BYTE
 * 
 *     rtt_format = GL_RGBA             # <<<<<<<<<<<<<<
 *     rtt_internalformat = GL_RGBA
 *     rtt_type = GL_UNSIGNED_BYTE
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_format = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":101
 * 
 *     rtt_format = GL_RGBA
 *     rtt_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_internalformat = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":102
 *     rtt_format = GL_RGBA
 *     rtt_internalformat = GL_RGBA
 *     rtt_type = GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * 
 * def test_texture_sizes(Environ environ, draw):
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_type = GL_UNSIGNED_BYTE;

  /* "renpy/gl/gltexture.pyx":82
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gl():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl.gltexture.use_gl", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":104
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def test_texture_sizes(Environ environ, draw):             # <<<<<<<<<<<<<<
 *     """
 *     Tests each possible texture size to see if it can be used. We test the
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_5test_texture_sizes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_4test_texture_sizes[] = "\n    Tests each possible texture size to see if it can be used. We test the\n    texture by creating a texture of the appropriate size, drawing it to the\n    screen, and then checking to see if the texture was drawn properly.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_5test_texture_sizes = {"test_texture_sizes", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_5test_texture_sizes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_4test_texture_sizes};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_5test_texture_sizes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ = 0;
  PyObject *__pyx_v_draw = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("test_texture_sizes (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_environ,&__pyx_n_s_draw,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_environ)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_draw)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("test_texture_sizes", 1, 2, 2, 1); __PYX_ERR(0, 104, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "test_texture_sizes") < 0)) __PYX_ERR(0, 104, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_environ = ((struct __pyx_obj_5renpy_2gl_6gldraw_Environ *)values[0]);
    __pyx_v_draw = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("test_texture_sizes", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 104, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.test_texture_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_environ), __pyx_ptype_5renpy_2gl_6gldraw_Environ, 1, "environ", 0))) __PYX_ERR(0, 104, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_4test_texture_sizes(__pyx_self, __pyx_v_environ, __pyx_v_draw);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_4test_texture_sizes(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, PyObject *__pyx_v_draw) {
  int __pyx_v_i;
  int __pyx_v_size;
  GLuint __pyx_v_tex;
  unsigned char *__pyx_v_bitmap;
  GLfloat __pyx_v_coords[6];
  GLfloat __pyx_v_texcoords[6];
  unsigned char __pyx_v_pixel[4];
  int __pyx_v_hw_max_size;
  GLenum __pyx_v_error;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  long __pyx_t_10;
  long __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  __Pyx_RefNannySetupContext("test_texture_sizes", 0);

  /* "renpy/gl/gltexture.pyx":123
 *     global SIZES
 * 
 *     renpy.display.log.write("Texture testing:")             # <<<<<<<<<<<<<<
 * 
 *     # There could be an error queued up from an ANGLE reset. Purge it before we do the
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_display); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_log); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_write); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":127
 *     # There could be an error queued up from an ANGLE reset. Purge it before we do the
 *     # texture testing.
 *     error = realGlGetError()             # <<<<<<<<<<<<<<
 *     if error != GL_NO_ERROR:
 *         renpy.display.log.write("- Ignored error at start of testing: {0:x}".format(error))
 */
  __pyx_v_error = glGetError();

  /* "renpy/gl/gltexture.pyx":128
 *     # texture testing.
 *     error = realGlGetError()
 *     if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *         renpy.display.log.write("- Ignored error at start of testing: {0:x}".format(error))
 * 
 */
  __pyx_t_3 = ((__pyx_v_error != GL_NO_ERROR) != 0);
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":129
 *     error = realGlGetError()
 *     if error != GL_NO_ERROR:
 *         renpy.display.log.write("- Ignored error at start of testing: {0:x}".format(error))             # <<<<<<<<<<<<<<
 * 
 *     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &hw_max_size)
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_display); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_log); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_write); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Ignored_error_at_start_of_testi, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyInt_From_GLenum(__pyx_v_error); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 129, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_7) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_6};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_6);
        __pyx_t_6 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_2};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_2};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":128
 *     # texture testing.
 *     error = realGlGetError()
 *     if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *         renpy.display.log.write("- Ignored error at start of testing: {0:x}".format(error))
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":131
 *         renpy.display.log.write("- Ignored error at start of testing: {0:x}".format(error))
 * 
 *     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &hw_max_size)             # <<<<<<<<<<<<<<
 * 
 *     renpy.display.log.write("- Hardware max texture size: %d", hw_max_size)
 */
  __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glGetIntegerv(GL_MAX_TEXTURE_SIZE, (&__pyx_v_hw_max_size)); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 131, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":133
 *     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &hw_max_size)
 * 
 *     renpy.display.log.write("- Hardware max texture size: %d", hw_max_size)             # <<<<<<<<<<<<<<
 *     hw_max_size = min(2048, hw_max_size)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_display); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_log); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_write); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_hw_max_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_9 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_kp_s_Hardware_max_texture_size_d, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_kp_s_Hardware_max_texture_size_d, __pyx_t_4};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_kp_s_Hardware_max_texture_size_d);
    __Pyx_GIVEREF(__pyx_kp_s_Hardware_max_texture_size_d);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_9, __pyx_kp_s_Hardware_max_texture_size_d);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_9, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":134
 * 
 *     renpy.display.log.write("- Hardware max texture size: %d", hw_max_size)
 *     hw_max_size = min(2048, hw_max_size)             # <<<<<<<<<<<<<<
 * 
 *     SIZES = [ ]
 */
  __pyx_t_9 = __pyx_v_hw_max_size;
  __pyx_t_10 = 0x800;
  if (((__pyx_t_9 < __pyx_t_10) != 0)) {
    __pyx_t_11 = __pyx_t_9;
  } else {
    __pyx_t_11 = __pyx_t_10;
  }
  __pyx_v_hw_max_size = __pyx_t_11;

  /* "renpy/gl/gltexture.pyx":136
 *     hw_max_size = min(2048, hw_max_size)
 * 
 *     SIZES = [ ]             # <<<<<<<<<<<<<<
 * 
 *     size = 64
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SIZES, __pyx_t_1) < 0) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":138
 *     SIZES = [ ]
 * 
 *     size = 64             # <<<<<<<<<<<<<<
 *     while size <= hw_max_size:
 * 
 */
  __pyx_v_size = 64;

  /* "renpy/gl/gltexture.pyx":139
 * 
 *     size = 64
 *     while size <= hw_max_size:             # <<<<<<<<<<<<<<
 * 
 *         # Create an all-red bitmap of the given size.
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_size <= __pyx_v_hw_max_size) != 0);
    if (!__pyx_t_3) break;

    /* "renpy/gl/gltexture.pyx":142
 * 
 *         # Create an all-red bitmap of the given size.
 *         bitmap = <unsigned char *> calloc(size * size, 4)             # <<<<<<<<<<<<<<
 * 
 *         if not bitmap:
 */
    __pyx_v_bitmap = ((unsigned char *)calloc((__pyx_v_size * __pyx_v_size), 4));

    /* "renpy/gl/gltexture.pyx":144
 *         bitmap = <unsigned char *> calloc(size * size, 4)
 * 
 *         if not bitmap:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Could not allocate {0}px bitmap.".format(size))
 *             break
 */
    __pyx_t_3 = ((!(__pyx_v_bitmap != 0)) != 0);
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":145
 * 
 *         if not bitmap:
 *             renpy.display.log.write("- Could not allocate {0}px bitmap.".format(size))             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_display); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_log); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_write); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Could_not_allocate_0_px_bitmap, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 145, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_6) {
        __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 145, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_8);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_2};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_2};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_2);
          __pyx_t_2 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_8};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_8};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_GIVEREF(__pyx_t_8);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_8);
          __pyx_t_8 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":146
 *         if not bitmap:
 *             renpy.display.log.write("- Could not allocate {0}px bitmap.".format(size))
 *             break             # <<<<<<<<<<<<<<
 * 
 * 
 */
      goto __pyx_L5_break;

      /* "renpy/gl/gltexture.pyx":144
 *         bitmap = <unsigned char *> calloc(size * size, 4)
 * 
 *         if not bitmap:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Could not allocate {0}px bitmap.".format(size))
 *             break
 */
    }

    /* "renpy/gl/gltexture.pyx":149
 * 
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             if tex_format == GL_RGBA:
 * 
 */
    {
        #ifdef WITH_THREAD
        PyThreadState *_save;
        Py_UNBLOCK_THREADS
        __Pyx_FastGIL_Remember();
        #endif
        /*try:*/ {

          /* "renpy/gl/gltexture.pyx":150
 * 
 *         with nogil:
 *             if tex_format == GL_RGBA:             # <<<<<<<<<<<<<<
 * 
 *                 for i from 0 <= i < size * size:
 */
          __pyx_t_3 = ((__pyx_v_5renpy_2gl_9gltexture_tex_format == GL_RGBA) != 0);
          if (__pyx_t_3) {

            /* "renpy/gl/gltexture.pyx":152
 *             if tex_format == GL_RGBA:
 * 
 *                 for i from 0 <= i < size * size:             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 0] = 0xff # r
 *                     bitmap[i * 4 + 1] = 0x00 # g
 */
            __pyx_t_9 = (__pyx_v_size * __pyx_v_size);
            for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_9; __pyx_v_i++) {

              /* "renpy/gl/gltexture.pyx":153
 * 
 *                 for i from 0 <= i < size * size:
 *                     bitmap[i * 4 + 0] = 0xff # r             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 1] = 0x00 # g
 *                     bitmap[i * 4 + 2] = 0x00 # b
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 0)]) = 0xff;

              /* "renpy/gl/gltexture.pyx":154
 *                 for i from 0 <= i < size * size:
 *                     bitmap[i * 4 + 0] = 0xff # r
 *                     bitmap[i * 4 + 1] = 0x00 # g             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 2] = 0x00 # b
 *                     bitmap[i * 4 + 3] = 0xff # a
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 1)]) = 0x00;

              /* "renpy/gl/gltexture.pyx":155
 *                     bitmap[i * 4 + 0] = 0xff # r
 *                     bitmap[i * 4 + 1] = 0x00 # g
 *                     bitmap[i * 4 + 2] = 0x00 # b             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 3] = 0xff # a
 * 
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 2)]) = 0x00;

              /* "renpy/gl/gltexture.pyx":156
 *                     bitmap[i * 4 + 1] = 0x00 # g
 *                     bitmap[i * 4 + 2] = 0x00 # b
 *                     bitmap[i * 4 + 3] = 0xff # a             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 3)]) = 0xff;
            }

            /* "renpy/gl/gltexture.pyx":150
 * 
 *         with nogil:
 *             if tex_format == GL_RGBA:             # <<<<<<<<<<<<<<
 * 
 *                 for i from 0 <= i < size * size:
 */
            goto __pyx_L12;
          }

          /* "renpy/gl/gltexture.pyx":160
 *             else:
 * 
 *                 for i from 0 <= i < size * size:             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 0] = 0x00 # b
 *                     bitmap[i * 4 + 1] = 0x00 # g
 */
          /*else*/ {
            __pyx_t_9 = (__pyx_v_size * __pyx_v_size);
            for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_9; __pyx_v_i++) {

              /* "renpy/gl/gltexture.pyx":161
 * 
 *                 for i from 0 <= i < size * size:
 *                     bitmap[i * 4 + 0] = 0x00 # b             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 1] = 0x00 # g
 *                     bitmap[i * 4 + 2] = 0xff # r
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 0)]) = 0x00;

              /* "renpy/gl/gltexture.pyx":162
 *                 for i from 0 <= i < size * size:
 *                     bitmap[i * 4 + 0] = 0x00 # b
 *                     bitmap[i * 4 + 1] = 0x00 # g             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 2] = 0xff # r
 *                     bitmap[i * 4 + 3] = 0xff # a
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 1)]) = 0x00;

              /* "renpy/gl/gltexture.pyx":163
 *                     bitmap[i * 4 + 0] = 0x00 # b
 *                     bitmap[i * 4 + 1] = 0x00 # g
 *                     bitmap[i * 4 + 2] = 0xff # r             # <<<<<<<<<<<<<<
 *                     bitmap[i * 4 + 3] = 0xff # a
 * 
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 2)]) = 0xff;

              /* "renpy/gl/gltexture.pyx":164
 *                     bitmap[i * 4 + 1] = 0x00 # g
 *                     bitmap[i * 4 + 2] = 0xff # r
 *                     bitmap[i * 4 + 3] = 0xff # a             # <<<<<<<<<<<<<<
 * 
 *         # Create a texture of the given size.
 */
              (__pyx_v_bitmap[((__pyx_v_i * 4) + 3)]) = 0xff;
            }
          }
          __pyx_L12:;
        }

        /* "renpy/gl/gltexture.pyx":149
 * 
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             if tex_format == GL_RGBA:
 * 
 */
        /*finally:*/ {
          /*normal exit:*/{
            #ifdef WITH_THREAD
            __Pyx_FastGIL_Forget();
            Py_BLOCK_THREADS
            #endif
            goto __pyx_L11;
          }
          __pyx_L11:;
        }
    }

    /* "renpy/gl/gltexture.pyx":167
 * 
 *         # Create a texture of the given size.
 *         glActiveTextureARB(GL_TEXTURE0)             # <<<<<<<<<<<<<<
 *         glGenTextures(1, &tex)
 *         glBindTexture(GL_TEXTURE_2D, tex)
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glActiveTextureARB(GL_TEXTURE0); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 167, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":168
 *         # Create a texture of the given size.
 *         glActiveTextureARB(GL_TEXTURE0)
 *         glGenTextures(1, &tex)             # <<<<<<<<<<<<<<
 *         glBindTexture(GL_TEXTURE_2D, tex)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glGenTextures(1, (&__pyx_v_tex)); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 168, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":169
 *         glActiveTextureARB(GL_TEXTURE0)
 *         glGenTextures(1, &tex)
 *         glBindTexture(GL_TEXTURE_2D, tex)             # <<<<<<<<<<<<<<
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_tex); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 169, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":170
 *         glGenTextures(1, &tex)
 *         glBindTexture(GL_TEXTURE_2D, tex)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);             # <<<<<<<<<<<<<<
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 *         glTexImage2D(GL_TEXTURE_2D, 0, tex_internalformat, size, size, 0, tex_format, tex_type, bitmap)
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 170, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":171
 *         glBindTexture(GL_TEXTURE_2D, tex)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);             # <<<<<<<<<<<<<<
 *         glTexImage2D(GL_TEXTURE_2D, 0, tex_internalformat, size, size, 0, tex_format, tex_type, bitmap)
 * 
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 171, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":172
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 *         glTexImage2D(GL_TEXTURE_2D, 0, tex_internalformat, size, size, 0, tex_format, tex_type, bitmap)             # <<<<<<<<<<<<<<
 * 
 *         # Free the bitmap.
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glTexImage2D(GL_TEXTURE_2D, 0, __pyx_v_5renpy_2gl_9gltexture_tex_internalformat, __pyx_v_size, __pyx_v_size, 0, __pyx_v_5renpy_2gl_9gltexture_tex_format, __pyx_v_5renpy_2gl_9gltexture_tex_type, __pyx_v_bitmap); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 172, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":175
 * 
 *         # Free the bitmap.
 *         free(bitmap)             # <<<<<<<<<<<<<<
 * 
 *         error = realGlGetError()
 */
    free(__pyx_v_bitmap);

    /* "renpy/gl/gltexture.pyx":177
 *         free(bitmap)
 * 
 *         error = realGlGetError()             # <<<<<<<<<<<<<<
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error loading {0}px bitmap: {1:x}".format(size, error))
 */
    __pyx_v_error = glGetError();

    /* "renpy/gl/gltexture.pyx":178
 * 
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Error loading {0}px bitmap: {1:x}".format(size, error))
 *             glDeleteTextures(1, &tex)
 */
    __pyx_t_3 = ((__pyx_v_error != GL_NO_ERROR) != 0);
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":179
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error loading {0}px bitmap: {1:x}".format(size, error))             # <<<<<<<<<<<<<<
 *             glDeleteTextures(1, &tex)
 *             break
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_display); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_log); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_write); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Error_loading_0_px_bitmap_1_x, __pyx_n_s_format); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyInt_From_GLenum(__pyx_v_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 179, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_2};
        __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_4, __pyx_t_2};
        __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_12 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 179, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_9, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_9, __pyx_t_2);
        __pyx_t_4 = 0;
        __pyx_t_2 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_12, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 179, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_8) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_5};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_5};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 179, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_8); __pyx_t_8 = NULL;
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":180
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error loading {0}px bitmap: {1:x}".format(size, error))
 *             glDeleteTextures(1, &tex)             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glDeleteTextures(1, (&__pyx_v_tex)); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 180, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":181
 *             renpy.display.log.write("- Error loading {0}px bitmap: {1:x}".format(size, error))
 *             glDeleteTextures(1, &tex)
 *             break             # <<<<<<<<<<<<<<
 * 
 *         # Vertex coordinates.
 */
      goto __pyx_L5_break;

      /* "renpy/gl/gltexture.pyx":178
 * 
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Error loading {0}px bitmap: {1:x}".format(size, error))
 *             glDeleteTextures(1, &tex)
 */
    }

    /* "renpy/gl/gltexture.pyx":184
 * 
 *         # Vertex coordinates.
 *         coords[0] = 0             # <<<<<<<<<<<<<<
 *         coords[1] = 0
 * 
 */
    (__pyx_v_coords[0]) = 0.0;

    /* "renpy/gl/gltexture.pyx":185
 *         # Vertex coordinates.
 *         coords[0] = 0
 *         coords[1] = 0             # <<<<<<<<<<<<<<
 * 
 *         coords[2] = 0
 */
    (__pyx_v_coords[1]) = 0.0;

    /* "renpy/gl/gltexture.pyx":187
 *         coords[1] = 0
 * 
 *         coords[2] = 0             # <<<<<<<<<<<<<<
 *         coords[3] = size
 * 
 */
    (__pyx_v_coords[2]) = 0.0;

    /* "renpy/gl/gltexture.pyx":188
 * 
 *         coords[2] = 0
 *         coords[3] = size             # <<<<<<<<<<<<<<
 * 
 *         coords[4] = size
 */
    (__pyx_v_coords[3]) = __pyx_v_size;

    /* "renpy/gl/gltexture.pyx":190
 *         coords[3] = size
 * 
 *         coords[4] = size             # <<<<<<<<<<<<<<
 *         coords[5] = 0
 * 
 */
    (__pyx_v_coords[4]) = __pyx_v_size;

    /* "renpy/gl/gltexture.pyx":191
 * 
 *         coords[4] = size
 *         coords[5] = 0             # <<<<<<<<<<<<<<
 * 
 *         # Texture coordinates.
 */
    (__pyx_v_coords[5]) = 0.0;

    /* "renpy/gl/gltexture.pyx":194
 * 
 *         # Texture coordinates.
 *         texcoords[0] = 0             # <<<<<<<<<<<<<<
 *         texcoords[1] = 0
 * 
 */
    (__pyx_v_texcoords[0]) = 0.0;

    /* "renpy/gl/gltexture.pyx":195
 *         # Texture coordinates.
 *         texcoords[0] = 0
 *         texcoords[1] = 0             # <<<<<<<<<<<<<<
 * 
 *         texcoords[2] = 0
 */
    (__pyx_v_texcoords[1]) = 0.0;

    /* "renpy/gl/gltexture.pyx":197
 *         texcoords[1] = 0
 * 
 *         texcoords[2] = 0             # <<<<<<<<<<<<<<
 *         texcoords[3] = 1.0
 * 
 */
    (__pyx_v_texcoords[2]) = 0.0;

    /* "renpy/gl/gltexture.pyx":198
 * 
 *         texcoords[2] = 0
 *         texcoords[3] = 1.0             # <<<<<<<<<<<<<<
 * 
 *         texcoords[4] = 1.0
 */
    (__pyx_v_texcoords[3]) = 1.0;

    /* "renpy/gl/gltexture.pyx":200
 *         texcoords[3] = 1.0
 * 
 *         texcoords[4] = 1.0             # <<<<<<<<<<<<<<
 *         texcoords[5] = 0
 * 
 */
    (__pyx_v_texcoords[4]) = 1.0;

    /* "renpy/gl/gltexture.pyx":201
 * 
 *         texcoords[4] = 1.0
 *         texcoords[5] = 0             # <<<<<<<<<<<<<<
 * 
 *         # Draw the triangles.
 */
    (__pyx_v_texcoords[5]) = 0.0;

    /* "renpy/gl/gltexture.pyx":204
 * 
 *         # Draw the triangles.
 *         environ.viewport(0, 0, 800, 600)             # <<<<<<<<<<<<<<
 *         environ.ortho(0, 800, 0, 600, -1.0, 1.0)
 * 
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->viewport(__pyx_v_environ, 0, 0, 0x320, 0x258);

    /* "renpy/gl/gltexture.pyx":205
 *         # Draw the triangles.
 *         environ.viewport(0, 0, 800, 600)
 *         environ.ortho(0, 800, 0, 600, -1.0, 1.0)             # <<<<<<<<<<<<<<
 * 
 *         environ.unset_clip(draw)
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->ortho(__pyx_v_environ, 0.0, 800.0, 0.0, 600.0, -1.0, 1.0);

    /* "renpy/gl/gltexture.pyx":207
 *         environ.ortho(0, 800, 0, 600, -1.0, 1.0)
 * 
 *         environ.unset_clip(draw)             # <<<<<<<<<<<<<<
 *         environ.blit()
 *         environ.set_color(1.0, 1.0, 1.0, 1.0)
 */
    if (!(likely(((__pyx_v_draw) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_draw, __pyx_ptype_5renpy_2gl_6gldraw_GLDraw))))) __PYX_ERR(0, 207, __pyx_L1_error)
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->unset_clip(__pyx_v_environ, ((struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw *)__pyx_v_draw));

    /* "renpy/gl/gltexture.pyx":208
 * 
 *         environ.unset_clip(draw)
 *         environ.blit()             # <<<<<<<<<<<<<<
 *         environ.set_color(1.0, 1.0, 1.0, 1.0)
 *         environ.set_texture(0, texcoords)
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->blit(__pyx_v_environ);

    /* "renpy/gl/gltexture.pyx":209
 *         environ.unset_clip(draw)
 *         environ.blit()
 *         environ.set_color(1.0, 1.0, 1.0, 1.0)             # <<<<<<<<<<<<<<
 *         environ.set_texture(0, texcoords)
 *         environ.set_vertex(coords)
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_color(__pyx_v_environ, 1.0, 1.0, 1.0, 1.0);

    /* "renpy/gl/gltexture.pyx":210
 *         environ.blit()
 *         environ.set_color(1.0, 1.0, 1.0, 1.0)
 *         environ.set_texture(0, texcoords)             # <<<<<<<<<<<<<<
 *         environ.set_vertex(coords)
 *         glDrawArrays(GL_TRIANGLES, 0, 3)
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 0, __pyx_v_texcoords);

    /* "renpy/gl/gltexture.pyx":211
 *         environ.set_color(1.0, 1.0, 1.0, 1.0)
 *         environ.set_texture(0, texcoords)
 *         environ.set_vertex(coords)             # <<<<<<<<<<<<<<
 *         glDrawArrays(GL_TRIANGLES, 0, 3)
 * 
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_vertex(__pyx_v_environ, __pyx_v_coords);

    /* "renpy/gl/gltexture.pyx":212
 *         environ.set_texture(0, texcoords)
 *         environ.set_vertex(coords)
 *         glDrawArrays(GL_TRIANGLES, 0, 3)             # <<<<<<<<<<<<<<
 * 
 *         # Delete the texture.
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glDrawArrays(GL_TRIANGLES, 0, 3); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 212, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":215
 * 
 *         # Delete the texture.
 *         glDeleteTextures(1, &tex)             # <<<<<<<<<<<<<<
 * 
 *         error = realGlGetError()
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glDeleteTextures(1, (&__pyx_v_tex)); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 215, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":217
 *         glDeleteTextures(1, &tex)
 * 
 *         error = realGlGetError()             # <<<<<<<<<<<<<<
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error drawing {0}px texture: {1:x}".format(size, error))
 */
    __pyx_v_error = glGetError();

    /* "renpy/gl/gltexture.pyx":218
 * 
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Error drawing {0}px texture: {1:x}".format(size, error))
 *             break
 */
    __pyx_t_3 = ((__pyx_v_error != GL_NO_ERROR) != 0);
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":219
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error drawing {0}px texture: {1:x}".format(size, error))             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_display); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_log); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_write); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Error_drawing_0_px_texture_1_x, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_2 = __Pyx_PyInt_From_GLenum(__pyx_v_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_8, __pyx_t_2};
        __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_8, __pyx_t_2};
        __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_9, __pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_9, __pyx_t_2);
        __pyx_t_8 = 0;
        __pyx_t_2 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_12);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_12, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_12)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_7};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_7};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 219, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 219, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":220
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error drawing {0}px texture: {1:x}".format(size, error))
 *             break             # <<<<<<<<<<<<<<
 * 
 *         # Check the pixel color.
 */
      goto __pyx_L5_break;

      /* "renpy/gl/gltexture.pyx":218
 * 
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Error drawing {0}px texture: {1:x}".format(size, error))
 *             break
 */
    }

    /* "renpy/gl/gltexture.pyx":223
 * 
 *         # Check the pixel color.
 *         glReadPixels(0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel)             # <<<<<<<<<<<<<<
 * 
 *         error = realGlGetError()
 */
    __pyx_t_9 = __pyx_f_5renpy_2gl_2gl_glReadPixels(0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, __pyx_v_pixel); if (unlikely(__pyx_t_9 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 223, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":225
 *         glReadPixels(0, 0, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel)
 * 
 *         error = realGlGetError()             # <<<<<<<<<<<<<<
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error reading {0}px texture: {1:x}".format(size, error))
 */
    __pyx_v_error = glGetError();

    /* "renpy/gl/gltexture.pyx":226
 * 
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Error reading {0}px texture: {1:x}".format(size, error))
 *             break
 */
    __pyx_t_3 = ((__pyx_v_error != GL_NO_ERROR) != 0);
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":227
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error reading {0}px texture: {1:x}".format(size, error))             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_display); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_log); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_write); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Error_reading_0_px_texture_1_x, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = __Pyx_PyInt_From_GLenum(__pyx_v_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_2};
        __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_5, __pyx_t_2};
        __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 227, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_9, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_9, __pyx_t_2);
        __pyx_t_5 = 0;
        __pyx_t_2 = 0;
        __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_4, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      if (!__pyx_t_7) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_12};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
          PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_12};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        } else
        #endif
        {
          __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 227, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7); __pyx_t_7 = NULL;
          __Pyx_GIVEREF(__pyx_t_12);
          PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_12);
          __pyx_t_12 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 227, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":228
 *         if error != GL_NO_ERROR:
 *             renpy.display.log.write("- Error reading {0}px texture: {1:x}".format(size, error))
 *             break             # <<<<<<<<<<<<<<
 * 
 *         if pixel[0] != 0xff or pixel[1] != 0x00 or pixel[2] != 0x00:
 */
      goto __pyx_L5_break;

      /* "renpy/gl/gltexture.pyx":226
 * 
 *         error = realGlGetError()
 *         if error != GL_NO_ERROR:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Error reading {0}px texture: {1:x}".format(size, error))
 *             break
 */
    }

    /* "renpy/gl/gltexture.pyx":230
 *             break
 * 
 *         if pixel[0] != 0xff or pixel[1] != 0x00 or pixel[2] != 0x00:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Incorrect pixel color in {0}px texture: ({1:x}, {1:x}, {1:x})".format(size, pixel[0], pixel[1], pixel[2]))
 *             break
 */
    __pyx_t_13 = (((__pyx_v_pixel[0]) != 0xff) != 0);
    if (!__pyx_t_13) {
    } else {
      __pyx_t_3 = __pyx_t_13;
      goto __pyx_L21_bool_binop_done;
    }
    __pyx_t_13 = (((__pyx_v_pixel[1]) != 0x00) != 0);
    if (!__pyx_t_13) {
    } else {
      __pyx_t_3 = __pyx_t_13;
      goto __pyx_L21_bool_binop_done;
    }
    __pyx_t_13 = (((__pyx_v_pixel[2]) != 0x00) != 0);
    __pyx_t_3 = __pyx_t_13;
    __pyx_L21_bool_binop_done:;
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":231
 * 
 *         if pixel[0] != 0xff or pixel[1] != 0x00 or pixel[2] != 0x00:
 *             renpy.display.log.write("- Incorrect pixel color in {0}px texture: ({1:x}, {1:x}, {1:x})".format(size, pixel[0], pixel[1], pixel[2]))             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_display); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_log); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_write); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Incorrect_pixel_color_in_0_px_t, __pyx_n_s_format); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_2 = __Pyx_PyInt_From_unsigned_char((__pyx_v_pixel[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = __Pyx_PyInt_From_unsigned_char((__pyx_v_pixel[1])); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PyInt_From_unsigned_char((__pyx_v_pixel[2])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_14 = NULL;
      __pyx_t_9 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_12);
        if (likely(__pyx_t_14)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_12, function);
          __pyx_t_9 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_12)) {
        PyObject *__pyx_temp[5] = {__pyx_t_14, __pyx_t_7, __pyx_t_2, __pyx_t_5, __pyx_t_8};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_9, 4+__pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 231, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
        PyObject *__pyx_temp[5] = {__pyx_t_14, __pyx_t_7, __pyx_t_2, __pyx_t_5, __pyx_t_8};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_9, 4+__pyx_t_9); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 231, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_15 = PyTuple_New(4+__pyx_t_9); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 231, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        if (__pyx_t_14) {
          __Pyx_GIVEREF(__pyx_t_14); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_14); __pyx_t_14 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_15, 0+__pyx_t_9, __pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_15, 1+__pyx_t_9, __pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_15, 2+__pyx_t_9, __pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_15, 3+__pyx_t_9, __pyx_t_8);
        __pyx_t_7 = 0;
        __pyx_t_2 = 0;
        __pyx_t_5 = 0;
        __pyx_t_8 = 0;
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_15, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 231, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      }
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_12)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_12) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_12, __pyx_t_6};
          __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_12, __pyx_t_6};
          __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_15 = PyTuple_New(1+1); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 231, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12); __pyx_t_12 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_15, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":232
 *         if pixel[0] != 0xff or pixel[1] != 0x00 or pixel[2] != 0x00:
 *             renpy.display.log.write("- Incorrect pixel color in {0}px texture: ({1:x}, {1:x}, {1:x})".format(size, pixel[0], pixel[1], pixel[2]))
 *             break             # <<<<<<<<<<<<<<
 * 
 *         # Record success.
 */
      goto __pyx_L5_break;

      /* "renpy/gl/gltexture.pyx":230
 *             break
 * 
 *         if pixel[0] != 0xff or pixel[1] != 0x00 or pixel[2] != 0x00:             # <<<<<<<<<<<<<<
 *             renpy.display.log.write("- Incorrect pixel color in {0}px texture: ({1:x}, {1:x}, {1:x})".format(size, pixel[0], pixel[1], pixel[2]))
 *             break
 */
    }

    /* "renpy/gl/gltexture.pyx":235
 * 
 *         # Record success.
 *         renpy.display.log.write("- {0}px textures work.".format(size))             # <<<<<<<<<<<<<<
 * 
 *         SIZES.append(size)
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_display); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_log); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_write); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0_px_textures_work, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_12 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 235, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
      }
    }
    if (!__pyx_t_8) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_12); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_12};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
        PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_12};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8); __pyx_t_8 = NULL;
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_12);
        __pyx_t_12 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_15))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_15);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_15);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_15, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_15, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_15)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_15)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_15, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 235, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":237
 *         renpy.display.log.write("- {0}px textures work.".format(size))
 * 
 *         SIZES.append(size)             # <<<<<<<<<<<<<<
 *         MAX_SIZE = size
 * 
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __pyx_t_16 = __Pyx_PyObject_Append(__pyx_t_1, __pyx_t_15); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

    /* "renpy/gl/gltexture.pyx":238
 * 
 *         SIZES.append(size)
 *         MAX_SIZE = size             # <<<<<<<<<<<<<<
 * 
 *         # Double the size and try again.
 */
    __pyx_t_15 = __Pyx_PyInt_From_int(__pyx_v_size); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_MAX_SIZE, __pyx_t_15) < 0) __PYX_ERR(0, 238, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;

    /* "renpy/gl/gltexture.pyx":241
 * 
 *         # Double the size and try again.
 *         size *= 2             # <<<<<<<<<<<<<<
 * 
 *     # Clean up.
 */
    __pyx_v_size = (__pyx_v_size * 2);
  }
  __pyx_L5_break:;

  /* "renpy/gl/gltexture.pyx":244
 * 
 *     # Clean up.
 *     environ.set_texture(0, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if MAX_SIZE > 2048:
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 0, NULL);

  /* "renpy/gl/gltexture.pyx":246
 *     environ.set_texture(0, NULL)
 * 
 *     if MAX_SIZE > 2048:             # <<<<<<<<<<<<<<
 *         MAX_SIZE = 2048
 * 
 */
  __pyx_t_15 = __Pyx_GetModuleGlobalName(__pyx_n_s_MAX_SIZE); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_15, __pyx_int_2048, Py_GT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":247
 * 
 *     if MAX_SIZE > 2048:
 *         MAX_SIZE = 2048             # <<<<<<<<<<<<<<
 * 
 *     if not SIZES:
 */
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_MAX_SIZE, __pyx_int_2048) < 0) __PYX_ERR(0, 247, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":246
 *     environ.set_texture(0, NULL)
 * 
 *     if MAX_SIZE > 2048:             # <<<<<<<<<<<<<<
 *         MAX_SIZE = 2048
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":249
 *         MAX_SIZE = 2048
 * 
 *     if not SIZES:             # <<<<<<<<<<<<<<
 *         renpy.display.log.write("Textures are not rendering properly.")
 *         return False
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = ((!__pyx_t_3) != 0);
  if (__pyx_t_13) {

    /* "renpy/gl/gltexture.pyx":250
 * 
 *     if not SIZES:
 *         renpy.display.log.write("Textures are not rendering properly.")             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_display); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_log); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __pyx_t_15 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_write); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_15, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":251
 *     if not SIZES:
 *         renpy.display.log.write("Textures are not rendering properly.")
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     SIZES.reverse()
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":249
 *         MAX_SIZE = 2048
 * 
 *     if not SIZES:             # <<<<<<<<<<<<<<
 *         renpy.display.log.write("Textures are not rendering properly.")
 *         return False
 */
  }

  /* "renpy/gl/gltexture.pyx":253
 *         return False
 * 
 *     SIZES.reverse()             # <<<<<<<<<<<<<<
 *     return True
 * 
 */
  __pyx_t_15 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_15, __pyx_n_s_reverse); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  __pyx_t_15 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_15)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_15);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (__pyx_t_15) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_15); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":254
 * 
 *     SIZES.reverse()
 *     return True             # <<<<<<<<<<<<<<
 * 
 * cdef class TextureCore:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_True);
  __pyx_r = Py_True;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":104
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def test_texture_sizes(Environ environ, draw):             # <<<<<<<<<<<<<<
 *     """
 *     Tests each possible texture size to see if it can be used. We test the
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("renpy.gl.gltexture.test_texture_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":261
 *     """
 * 
 *     def __init__(TextureCore self, int width, int height):             # <<<<<<<<<<<<<<
 * 
 *         # The width and height of this texture.
 */

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_width;
  int __pyx_v_height;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 261, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 261, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_width = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_width == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 261, __pyx_L3_error)
    __pyx_v_height = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_height == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 261, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 261, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore___init__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore___init__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, int __pyx_v_width, int __pyx_v_height) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl/gltexture.pyx":264
 * 
 *         # The width and height of this texture.
 *         self.width = width             # <<<<<<<<<<<<<<
 *         self.height = height
 * 
 */
  __pyx_v_self->width = __pyx_v_width;

  /* "renpy/gl/gltexture.pyx":265
 *         # The width and height of this texture.
 *         self.width = width
 *         self.height = height             # <<<<<<<<<<<<<<
 * 
 *         # The number of the OpenGL texture this texture object
 */
  __pyx_v_self->height = __pyx_v_height;

  /* "renpy/gl/gltexture.pyx":269
 *         # The number of the OpenGL texture this texture object
 *         # represents.
 *         self.generation = 0             # <<<<<<<<<<<<<<
 *         self.number = 0
 * 
 */
  __pyx_v_self->generation = 0;

  /* "renpy/gl/gltexture.pyx":270
 *         # represents.
 *         self.generation = 0
 *         self.number = 0             # <<<<<<<<<<<<<<
 * 
 *         # These contained the premultiplied (but not GPU-loaded)
 */
  __pyx_v_self->number = 0;

  /* "renpy/gl/gltexture.pyx":276
 *         # needed.
 * 
 *         self.premult = None             # <<<<<<<<<<<<<<
 *         self.premult_size = None
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->premult);
  __Pyx_DECREF(__pyx_v_self->premult);
  __pyx_v_self->premult = Py_None;

  /* "renpy/gl/gltexture.pyx":277
 * 
 *         self.premult = None
 *         self.premult_size = None             # <<<<<<<<<<<<<<
 * 
 *         # True if we're in NEAREST mode. False if we're in LINEAR mode.
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->premult_size);
  __Pyx_DECREF(__pyx_v_self->premult_size);
  __pyx_v_self->premult_size = ((PyObject*)Py_None);

  /* "renpy/gl/gltexture.pyx":280
 * 
 *         # True if we're in NEAREST mode. False if we're in LINEAR mode.
 *         self.nearest = False             # <<<<<<<<<<<<<<
 * 
 *         # The free list we should be put on, or None if we're already on
 */
  __pyx_v_self->nearest = 0;

  /* "renpy/gl/gltexture.pyx":284
 *         # The free list we should be put on, or None if we're already on
 *         # a free list.
 *         self.free_list = None             # <<<<<<<<<<<<<<
 * 
 *     def __del__(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->free_list);
  __Pyx_DECREF(__pyx_v_self->free_list);
  __pyx_v_self->free_list = ((PyObject*)Py_None);

  /* "renpy/gl/gltexture.pyx":261
 *     """
 * 
 *     def __init__(TextureCore self, int width, int height):             # <<<<<<<<<<<<<<
 * 
 *         # The width and height of this texture.
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":286
 *         self.free_list = None
 * 
 *     def __del__(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef unsigned int num
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_3__del__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_3__del__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_2__del__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_2__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "renpy/gl/gltexture.pyx":291
 * 
 *         # Release the surface.
 *         self.premult = None             # <<<<<<<<<<<<<<
 *         self.premult_size = None
 *         self.premult_left = 0
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->premult);
  __Pyx_DECREF(__pyx_v_self->premult);
  __pyx_v_self->premult = Py_None;

  /* "renpy/gl/gltexture.pyx":292
 *         # Release the surface.
 *         self.premult = None
 *         self.premult_size = None             # <<<<<<<<<<<<<<
 *         self.premult_left = 0
 *         self.premult_right = 0
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->premult_size);
  __Pyx_DECREF(__pyx_v_self->premult_size);
  __pyx_v_self->premult_size = ((PyObject*)Py_None);

  /* "renpy/gl/gltexture.pyx":293
 *         self.premult = None
 *         self.premult_size = None
 *         self.premult_left = 0             # <<<<<<<<<<<<<<
 *         self.premult_right = 0
 *         self.premult_top = 0
 */
  __pyx_v_self->premult_left = 0;

  /* "renpy/gl/gltexture.pyx":294
 *         self.premult_size = None
 *         self.premult_left = 0
 *         self.premult_right = 0             # <<<<<<<<<<<<<<
 *         self.premult_top = 0
 *         self.premult_bottom = 0
 */
  __pyx_v_self->premult_right = 0;

  /* "renpy/gl/gltexture.pyx":295
 *         self.premult_left = 0
 *         self.premult_right = 0
 *         self.premult_top = 0             # <<<<<<<<<<<<<<
 *         self.premult_bottom = 0
 * 
 */
  __pyx_v_self->premult_top = 0;

  /* "renpy/gl/gltexture.pyx":296
 *         self.premult_right = 0
 *         self.premult_top = 0
 *         self.premult_bottom = 0             # <<<<<<<<<<<<<<
 * 
 *         # The test needs to be here so we don't try to append during
 */
  __pyx_v_self->premult_bottom = 0;

  /* "renpy/gl/gltexture.pyx":300
 *         # The test needs to be here so we don't try to append during
 *         # interpreter shutdown.
 *         if self.free_list is not None:             # <<<<<<<<<<<<<<
 *             try:
 *                 self.free_list.append(self)
 */
  __pyx_t_1 = (__pyx_v_self->free_list != ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl/gltexture.pyx":301
 *         # interpreter shutdown.
 *         if self.free_list is not None:
 *             try:             # <<<<<<<<<<<<<<
 *                 self.free_list.append(self)
 *                 self.free_list = None
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "renpy/gl/gltexture.pyx":302
 *         if self.free_list is not None:
 *             try:
 *                 self.free_list.append(self)             # <<<<<<<<<<<<<<
 *                 self.free_list = None
 *             except TypeError:
 */
        if (unlikely(__pyx_v_self->free_list == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
          __PYX_ERR(0, 302, __pyx_L4_error)
        }
        __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_self->free_list, ((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 302, __pyx_L4_error)

        /* "renpy/gl/gltexture.pyx":303
 *             try:
 *                 self.free_list.append(self)
 *                 self.free_list = None             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 pass # Let's not error on shutdown.
 */
        __Pyx_INCREF(Py_None);
        __Pyx_GIVEREF(Py_None);
        __Pyx_GOTREF(__pyx_v_self->free_list);
        __Pyx_DECREF(__pyx_v_self->free_list);
        __pyx_v_self->free_list = ((PyObject*)Py_None);

        /* "renpy/gl/gltexture.pyx":301
 *         # interpreter shutdown.
 *         if self.free_list is not None:
 *             try:             # <<<<<<<<<<<<<<
 *                 self.free_list.append(self)
 *                 self.free_list = None
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L9_try_end;
      __pyx_L4_error:;

      /* "renpy/gl/gltexture.pyx":304
 *                 self.free_list.append(self)
 *                 self.free_list = None
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 pass # Let's not error on shutdown.
 * 
 */
      __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_TypeError);
      if (__pyx_t_7) {
        __Pyx_ErrRestore(0,0,0);
        goto __pyx_L5_exception_handled;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "renpy/gl/gltexture.pyx":301
 *         # interpreter shutdown.
 *         if self.free_list is not None:
 *             try:             # <<<<<<<<<<<<<<
 *                 self.free_list.append(self)
 *                 self.free_list = None
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L5_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      __pyx_L9_try_end:;
    }

    /* "renpy/gl/gltexture.pyx":300
 *         # The test needs to be here so we don't try to append during
 *         # interpreter shutdown.
 *         if self.free_list is not None:             # <<<<<<<<<<<<<<
 *             try:
 *                 self.free_list.append(self)
 */
  }

  /* "renpy/gl/gltexture.pyx":286
 *         self.free_list = None
 * 
 *     def __del__(self):             # <<<<<<<<<<<<<<
 * 
 *         cdef unsigned int num
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":308
 * 
 * 
 *     def load_surface(self, surf, x, y, w, h,             # <<<<<<<<<<<<<<
 *                      border_left, border_top, border_right, border_bottom):
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5load_surface(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_11TextureCore_4load_surface[] = "\n        Loads a pygame surface into this texture rectangle.\n        ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5load_surface(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_surf = 0;
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_w = 0;
  PyObject *__pyx_v_h = 0;
  PyObject *__pyx_v_border_left = 0;
  PyObject *__pyx_v_border_top = 0;
  PyObject *__pyx_v_border_right = 0;
  PyObject *__pyx_v_border_bottom = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("load_surface (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_surf,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_w,&__pyx_n_s_h,&__pyx_n_s_border_left,&__pyx_n_s_border_top,&__pyx_n_s_border_right,&__pyx_n_s_border_bottom,0};
    PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_surf)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 1); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 2); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 3); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 4); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_left)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 5); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_top)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 6); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_right)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 7); __PYX_ERR(0, 308, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_bottom)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, 8); __PYX_ERR(0, 308, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "load_surface") < 0)) __PYX_ERR(0, 308, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 9) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
    }
    __pyx_v_surf = values[0];
    __pyx_v_x = values[1];
    __pyx_v_y = values[2];
    __pyx_v_w = values[3];
    __pyx_v_h = values[4];
    __pyx_v_border_left = values[5];
    __pyx_v_border_top = values[6];
    __pyx_v_border_right = values[7];
    __pyx_v_border_bottom = values[8];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("load_surface", 1, 9, 9, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 308, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.load_surface", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_4load_surface(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), __pyx_v_surf, __pyx_v_x, __pyx_v_y, __pyx_v_w, __pyx_v_h, __pyx_v_border_left, __pyx_v_border_top, __pyx_v_border_right, __pyx_v_border_bottom);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_4load_surface(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_surf, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_w, PyObject *__pyx_v_h, PyObject *__pyx_v_border_left, PyObject *__pyx_v_border_top, PyObject *__pyx_v_border_right, PyObject *__pyx_v_border_bottom) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("load_surface", 0);

  /* "renpy/gl/gltexture.pyx":319
 *         # texture loading only occurs in the GL thread.
 * 
 *         self.premult = premultiply(             # <<<<<<<<<<<<<<
 *             surf, x, y, w, h,
 *             border_left, border_top, border_right, border_bottom)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_premultiply); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "renpy/gl/gltexture.pyx":321
 *         self.premult = premultiply(
 *             surf, x, y, w, h,
 *             border_left, border_top, border_right, border_bottom)             # <<<<<<<<<<<<<<
 * 
 *         self.premult_size = (w, h)
 */
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[10] = {__pyx_t_3, __pyx_v_surf, __pyx_v_x, __pyx_v_y, __pyx_v_w, __pyx_v_h, __pyx_v_border_left, __pyx_v_border_top, __pyx_v_border_right, __pyx_v_border_bottom};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 9+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[10] = {__pyx_t_3, __pyx_v_surf, __pyx_v_x, __pyx_v_y, __pyx_v_w, __pyx_v_h, __pyx_v_border_left, __pyx_v_border_top, __pyx_v_border_right, __pyx_v_border_bottom};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 9+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(9+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_surf);
    __Pyx_GIVEREF(__pyx_v_surf);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_surf);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_x);
    __Pyx_INCREF(__pyx_v_y);
    __Pyx_GIVEREF(__pyx_v_y);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_y);
    __Pyx_INCREF(__pyx_v_w);
    __Pyx_GIVEREF(__pyx_v_w);
    PyTuple_SET_ITEM(__pyx_t_5, 3+__pyx_t_4, __pyx_v_w);
    __Pyx_INCREF(__pyx_v_h);
    __Pyx_GIVEREF(__pyx_v_h);
    PyTuple_SET_ITEM(__pyx_t_5, 4+__pyx_t_4, __pyx_v_h);
    __Pyx_INCREF(__pyx_v_border_left);
    __Pyx_GIVEREF(__pyx_v_border_left);
    PyTuple_SET_ITEM(__pyx_t_5, 5+__pyx_t_4, __pyx_v_border_left);
    __Pyx_INCREF(__pyx_v_border_top);
    __Pyx_GIVEREF(__pyx_v_border_top);
    PyTuple_SET_ITEM(__pyx_t_5, 6+__pyx_t_4, __pyx_v_border_top);
    __Pyx_INCREF(__pyx_v_border_right);
    __Pyx_GIVEREF(__pyx_v_border_right);
    PyTuple_SET_ITEM(__pyx_t_5, 7+__pyx_t_4, __pyx_v_border_right);
    __Pyx_INCREF(__pyx_v_border_bottom);
    __Pyx_GIVEREF(__pyx_v_border_bottom);
    PyTuple_SET_ITEM(__pyx_t_5, 8+__pyx_t_4, __pyx_v_border_bottom);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":319
 *         # texture loading only occurs in the GL thread.
 * 
 *         self.premult = premultiply(             # <<<<<<<<<<<<<<
 *             surf, x, y, w, h,
 *             border_left, border_top, border_right, border_bottom)
 */
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->premult);
  __Pyx_DECREF(__pyx_v_self->premult);
  __pyx_v_self->premult = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":323
 *             border_left, border_top, border_right, border_bottom)
 * 
 *         self.premult_size = (w, h)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_w);
  __Pyx_GIVEREF(__pyx_v_w);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_w);
  __Pyx_INCREF(__pyx_v_h);
  __Pyx_GIVEREF(__pyx_v_h);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_h);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->premult_size);
  __Pyx_DECREF(__pyx_v_self->premult_size);
  __pyx_v_self->premult_size = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":308
 * 
 * 
 *     def load_surface(self, surf, x, y, w, h,             # <<<<<<<<<<<<<<
 *                      border_left, border_top, border_right, border_bottom):
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.load_surface", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":326
 * 
 * 
 *     cdef void make_nearest(TextureCore self):             # <<<<<<<<<<<<<<
 *         """
 *         Causes this texture to be rendered in nearest-neighbor mode.
 */

static void __pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_nearest(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("make_nearest", 0);

  /* "renpy/gl/gltexture.pyx":331
 *         """
 * 
 *         if self.nearest:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  __pyx_t_1 = (__pyx_v_self->nearest != 0);
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":332
 * 
 *         if self.nearest:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         glBindTexture(GL_TEXTURE_2D, self.number)
 */
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":331
 *         """
 * 
 *         if self.nearest:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":334
 *             return
 * 
 *         glBindTexture(GL_TEXTURE_2D, self.number)             # <<<<<<<<<<<<<<
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
 */
  __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_self->number); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 334, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":335
 * 
 *         glBindTexture(GL_TEXTURE_2D, self.number)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)             # <<<<<<<<<<<<<<
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
 * 
 */
  __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 335, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":336
 *         glBindTexture(GL_TEXTURE_2D, self.number)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)             # <<<<<<<<<<<<<<
 * 
 *         self.nearest = True
 */
  __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 336, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":338
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
 * 
 *         self.nearest = True             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->nearest = 1;

  /* "renpy/gl/gltexture.pyx":326
 * 
 * 
 *     cdef void make_nearest(TextureCore self):             # <<<<<<<<<<<<<<
 *         """
 *         Causes this texture to be rendered in nearest-neighbor mode.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.gl.gltexture.TextureCore.make_nearest", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl/gltexture.pyx":341
 * 
 * 
 *     cdef void make_linear(TextureCore self):             # <<<<<<<<<<<<<<
 *         """
 *         Causes this texture to be rendered in linear interpolation mode.
 */

static void __pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_linear(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("make_linear", 0);

  /* "renpy/gl/gltexture.pyx":346
 *         """
 * 
 *         if not self.nearest:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  __pyx_t_1 = ((!(__pyx_v_self->nearest != 0)) != 0);
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":347
 * 
 *         if not self.nearest:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         glBindTexture(GL_TEXTURE_2D, self.number)
 */
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":346
 *         """
 * 
 *         if not self.nearest:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":349
 *             return
 * 
 *         glBindTexture(GL_TEXTURE_2D, self.number)             # <<<<<<<<<<<<<<
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 */
  __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_self->number); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 349, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":350
 * 
 *         glBindTexture(GL_TEXTURE_2D, self.number)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)             # <<<<<<<<<<<<<<
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 * 
 */
  __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 350, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":351
 *         glBindTexture(GL_TEXTURE_2D, self.number)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)             # <<<<<<<<<<<<<<
 * 
 *         self.nearest = False
 */
  __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 351, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":353
 *         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 * 
 *         self.nearest = False             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_self->nearest = 0;

  /* "renpy/gl/gltexture.pyx":341
 * 
 * 
 *     cdef void make_linear(TextureCore self):             # <<<<<<<<<<<<<<
 *         """
 *         Causes this texture to be rendered in linear interpolation mode.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.gl.gltexture.TextureCore.make_linear", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl/gltexture.pyx":356
 * 
 * 
 *     cdef void make_ready(TextureCore self):             # <<<<<<<<<<<<<<
 *         """
 *         Makes the texture ready for use.
 */

static void __pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_ready(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_v_w = NULL;
  PyObject *__pyx_v_h = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("make_ready", 0);

  /* "renpy/gl/gltexture.pyx":361
 *         """
 * 
 *         self.allocate()             # <<<<<<<<<<<<<<
 * 
 *         if self.premult:
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self->__pyx_vtab)->allocate(__pyx_v_self, 0);

  /* "renpy/gl/gltexture.pyx":363
 *         self.allocate()
 * 
 *         if self.premult:             # <<<<<<<<<<<<<<
 * 
 *             w, h = self.premult_size
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->premult); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 363, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":365
 *         if self.premult:
 * 
 *             w, h = self.premult_size             # <<<<<<<<<<<<<<
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 */
    __pyx_t_2 = __pyx_v_self->premult_size;
    __Pyx_INCREF(__pyx_t_2);
    if (likely(__pyx_t_2 != Py_None)) {
      PyObject* sequence = __pyx_t_2;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 365, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 365, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 365, __pyx_L1_error)
    }
    __pyx_v_w = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v_h = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "renpy/gl/gltexture.pyx":367
 *             w, h = self.premult_size
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_self->number); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 367, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":368
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 368, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":369
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 369, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":370
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
 * 
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 370, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":371
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)             # <<<<<<<<<<<<<<
 * 
 *             self.nearest = False
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 371, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":373
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
 * 
 *             self.nearest = False             # <<<<<<<<<<<<<<
 * 
 *             # If we haven't initialized the texture yet, and we're
 */
    __pyx_v_self->nearest = 0;

    /* "renpy/gl/gltexture.pyx":377
 *             # If we haven't initialized the texture yet, and we're
 *             # smaller than it, load in the empty texture.
 *             if w < self.width or h < self.height:             # <<<<<<<<<<<<<<
 * 
 *                 if self.format != tex_internalformat:
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_w, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!__pyx_t_6) {
    } else {
      __pyx_t_1 = __pyx_t_6;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = PyObject_RichCompare(__pyx_v_h, __pyx_t_4, Py_LT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 377, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_1 = __pyx_t_6;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_1) {

      /* "renpy/gl/gltexture.pyx":379
 *             if w < self.width or h < self.height:
 * 
 *                 if self.format != tex_internalformat:             # <<<<<<<<<<<<<<
 *                     load_premultiplied(
 *                         None,
 */
      __pyx_t_1 = ((__pyx_v_self->format != __pyx_v_5renpy_2gl_9gltexture_tex_internalformat) != 0);
      if (__pyx_t_1) {

        /* "renpy/gl/gltexture.pyx":380
 * 
 *                 if self.format != tex_internalformat:
 *                     load_premultiplied(             # <<<<<<<<<<<<<<
 *                         None,
 *                         self.width,
 */
        __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_load_premultiplied); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);

        /* "renpy/gl/gltexture.pyx":382
 *                     load_premultiplied(
 *                         None,
 *                         self.width,             # <<<<<<<<<<<<<<
 *                         self.height,
 *                         0,
 */
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 382, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);

        /* "renpy/gl/gltexture.pyx":383
 *                         None,
 *                         self.width,
 *                         self.height,             # <<<<<<<<<<<<<<
 *                         0,
 *                         False,
 */
        __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 383, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);

        /* "renpy/gl/gltexture.pyx":385
 *                         self.height,
 *                         0,
 *                         False,             # <<<<<<<<<<<<<<
 *                         )
 * 
 */
        __pyx_t_8 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[6] = {__pyx_t_8, Py_None, __pyx_t_3, __pyx_t_7, __pyx_int_0, Py_False};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 5+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[6] = {__pyx_t_8, Py_None, __pyx_t_3, __pyx_t_7, __pyx_int_0, Py_False};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 5+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_9 = PyTuple_New(5+__pyx_t_5); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          if (__pyx_t_8) {
            __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
          }
          __Pyx_INCREF(Py_None);
          __Pyx_GIVEREF(Py_None);
          PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_5, Py_None);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_5, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_5, __pyx_t_7);
          __Pyx_INCREF(__pyx_int_0);
          __Pyx_GIVEREF(__pyx_int_0);
          PyTuple_SET_ITEM(__pyx_t_9, 3+__pyx_t_5, __pyx_int_0);
          __Pyx_INCREF(Py_False);
          __Pyx_GIVEREF(Py_False);
          PyTuple_SET_ITEM(__pyx_t_9, 4+__pyx_t_5, Py_False);
          __pyx_t_3 = 0;
          __pyx_t_7 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "renpy/gl/gltexture.pyx":388
 *                         )
 * 
 *                     self.format = tex_internalformat             # <<<<<<<<<<<<<<
 * 
 *             # Otherwise, either load or replace the texture.
 */
        __pyx_v_self->format = __pyx_v_5renpy_2gl_9gltexture_tex_internalformat;

        /* "renpy/gl/gltexture.pyx":379
 *             if w < self.width or h < self.height:
 * 
 *                 if self.format != tex_internalformat:             # <<<<<<<<<<<<<<
 *                     load_premultiplied(
 *                         None,
 */
      }

      /* "renpy/gl/gltexture.pyx":377
 *             # If we haven't initialized the texture yet, and we're
 *             # smaller than it, load in the empty texture.
 *             if w < self.width or h < self.height:             # <<<<<<<<<<<<<<
 * 
 *                 if self.format != tex_internalformat:
 */
    }

    /* "renpy/gl/gltexture.pyx":391
 * 
 *             # Otherwise, either load or replace the texture.
 *             load_premultiplied(             # <<<<<<<<<<<<<<
 *                 self.premult,
 *                 w,
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_load_premultiplied); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "renpy/gl/gltexture.pyx":395
 *                 w,
 *                 h,
 *                 (self.format == tex_internalformat),             # <<<<<<<<<<<<<<
 *                 False,
 *                 )
 */
    __pyx_t_9 = __Pyx_PyBool_FromLong((__pyx_v_self->format == __pyx_v_5renpy_2gl_9gltexture_tex_internalformat)); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);

    /* "renpy/gl/gltexture.pyx":396
 *                 h,
 *                 (self.format == tex_internalformat),
 *                 False,             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
    __pyx_t_7 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[6] = {__pyx_t_7, __pyx_v_self->premult, __pyx_v_w, __pyx_v_h, __pyx_t_9, Py_False};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 5+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[6] = {__pyx_t_7, __pyx_v_self->premult, __pyx_v_w, __pyx_v_h, __pyx_t_9, Py_False};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_5, 5+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(5+__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_self->premult);
      __Pyx_GIVEREF(__pyx_v_self->premult);
      PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_v_self->premult);
      __Pyx_INCREF(__pyx_v_w);
      __Pyx_GIVEREF(__pyx_v_w);
      PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_v_w);
      __Pyx_INCREF(__pyx_v_h);
      __Pyx_GIVEREF(__pyx_v_h);
      PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_5, __pyx_v_h);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_3, 3+__pyx_t_5, __pyx_t_9);
      __Pyx_INCREF(Py_False);
      __Pyx_GIVEREF(Py_False);
      PyTuple_SET_ITEM(__pyx_t_3, 4+__pyx_t_5, Py_False);
      __pyx_t_9 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":401
 *             # Needs to be here twice, since we may not go through the w < SIDE
 *             # h < SIDE thing all the time.
 *             self.format = tex_internalformat             # <<<<<<<<<<<<<<
 * 
 *             # Finally, load in the default math.
 */
    __pyx_v_self->format = __pyx_v_5renpy_2gl_9gltexture_tex_internalformat;

    /* "renpy/gl/gltexture.pyx":404
 * 
 *             # Finally, load in the default math.
 *             self.xadd = self.yadd = 0             # <<<<<<<<<<<<<<
 *             self.xmul = 1.0 / self.width
 *             self.ymul = 1.0 / self.height
 */
    __pyx_v_self->xadd = 0.0;
    __pyx_v_self->yadd = 0.0;

    /* "renpy/gl/gltexture.pyx":405
 *             # Finally, load in the default math.
 *             self.xadd = self.yadd = 0
 *             self.xmul = 1.0 / self.width             # <<<<<<<<<<<<<<
 *             self.ymul = 1.0 / self.height
 * 
 */
    if (unlikely(__pyx_v_self->width == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 405, __pyx_L1_error)
    }
    __pyx_v_self->xmul = (1.0 / __pyx_v_self->width);

    /* "renpy/gl/gltexture.pyx":406
 *             self.xadd = self.yadd = 0
 *             self.xmul = 1.0 / self.width
 *             self.ymul = 1.0 / self.height             # <<<<<<<<<<<<<<
 * 
 *             # We don't need to be loaded anymore.
 */
    if (unlikely(__pyx_v_self->height == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 406, __pyx_L1_error)
    }
    __pyx_v_self->ymul = (1.0 / __pyx_v_self->height);

    /* "renpy/gl/gltexture.pyx":409
 * 
 *             # We don't need to be loaded anymore.
 *             self.premult = None             # <<<<<<<<<<<<<<
 *             self.premult_size = None
 * 
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->premult);
    __Pyx_DECREF(__pyx_v_self->premult);
    __pyx_v_self->premult = Py_None;

    /* "renpy/gl/gltexture.pyx":410
 *             # We don't need to be loaded anymore.
 *             self.premult = None
 *             self.premult_size = None             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->premult_size);
    __Pyx_DECREF(__pyx_v_self->premult_size);
    __pyx_v_self->premult_size = ((PyObject*)Py_None);

    /* "renpy/gl/gltexture.pyx":363
 *         self.allocate()
 * 
 *         if self.premult:             # <<<<<<<<<<<<<<
 * 
 *             w, h = self.premult_size
 */
  }

  /* "renpy/gl/gltexture.pyx":356
 * 
 * 
 *     cdef void make_ready(TextureCore self):             # <<<<<<<<<<<<<<
 *         """
 *         Makes the texture ready for use.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_WriteUnraisable("renpy.gl.gltexture.TextureCore.make_ready", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_w);
  __Pyx_XDECREF(__pyx_v_h);
  __Pyx_RefNannyFinishContext();
}

/* "renpy/gl/gltexture.pyx":413
 * 
 * 
 *     def render_to(self, x, y, draw_func, rtt, environ):             # <<<<<<<<<<<<<<
 * 
 *         self.allocate()
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_7render_to(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_7render_to(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_x = 0;
  PyObject *__pyx_v_y = 0;
  PyObject *__pyx_v_draw_func = 0;
  PyObject *__pyx_v_rtt = 0;
  PyObject *__pyx_v_environ = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("render_to (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_draw_func,&__pyx_n_s_rtt,&__pyx_n_s_environ,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("render_to", 1, 5, 5, 1); __PYX_ERR(0, 413, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_draw_func)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("render_to", 1, 5, 5, 2); __PYX_ERR(0, 413, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_rtt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("render_to", 1, 5, 5, 3); __PYX_ERR(0, 413, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_environ)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("render_to", 1, 5, 5, 4); __PYX_ERR(0, 413, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "render_to") < 0)) __PYX_ERR(0, 413, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_x = values[0];
    __pyx_v_y = values[1];
    __pyx_v_draw_func = values[2];
    __pyx_v_rtt = values[3];
    __pyx_v_environ = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("render_to", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 413, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.render_to", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6render_to(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), __pyx_v_x, __pyx_v_y, __pyx_v_draw_func, __pyx_v_rtt, __pyx_v_environ);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6render_to(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_x, PyObject *__pyx_v_y, PyObject *__pyx_v_draw_func, PyObject *__pyx_v_rtt, PyObject *__pyx_v_environ) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("render_to", 0);

  /* "renpy/gl/gltexture.pyx":415
 *     def render_to(self, x, y, draw_func, rtt, environ):
 * 
 *         self.allocate()             # <<<<<<<<<<<<<<
 * 
 *         if self.format != rtt_format:
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self->__pyx_vtab)->allocate(__pyx_v_self, 0);

  /* "renpy/gl/gltexture.pyx":417
 *         self.allocate()
 * 
 *         if self.format != rtt_format:             # <<<<<<<<<<<<<<
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 */
  __pyx_t_1 = ((__pyx_v_self->format != __pyx_v_5renpy_2gl_9gltexture_rtt_format) != 0);
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":419
 *         if self.format != rtt_format:
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 */
    __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_self->number); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 419, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":420
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
 */
    __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 420, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":421
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
 */
    __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 421, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":422
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)             # <<<<<<<<<<<<<<
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
 * 
 */
    __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 422, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":423
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE)
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)             # <<<<<<<<<<<<<<
 * 
 *             self.nearest = False
 */
    __pyx_t_2 = __pyx_f_5renpy_2gl_2gl_glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE); if (unlikely(__pyx_t_2 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 423, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":425
 *             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE)
 * 
 *             self.nearest = False             # <<<<<<<<<<<<<<
 * 
 *             load_premultiplied(
 */
    __pyx_v_self->nearest = 0;

    /* "renpy/gl/gltexture.pyx":427
 *             self.nearest = False
 * 
 *             load_premultiplied(             # <<<<<<<<<<<<<<
 *                 None,
 *                 self.width,
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_load_premultiplied); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "renpy/gl/gltexture.pyx":429
 *             load_premultiplied(
 *                 None,
 *                 self.width,             # <<<<<<<<<<<<<<
 *                 self.height,
 *                 0,
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "renpy/gl/gltexture.pyx":430
 *                 None,
 *                 self.width,
 *                 self.height,             # <<<<<<<<<<<<<<
 *                 0,
 *                 True)
 */
    __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "renpy/gl/gltexture.pyx":432
 *                 self.height,
 *                 0,
 *                 True)             # <<<<<<<<<<<<<<
 * 
 *             self.format = rtt_format
 */
    __pyx_t_7 = NULL;
    __pyx_t_2 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_2 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[6] = {__pyx_t_7, Py_None, __pyx_t_5, __pyx_t_6, __pyx_int_0, Py_True};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_2, 5+__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[6] = {__pyx_t_7, Py_None, __pyx_t_5, __pyx_t_6, __pyx_int_0, Py_True};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_2, 5+__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(5+__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(Py_None);
      __Pyx_GIVEREF(Py_None);
      PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_2, Py_None);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_2, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_2, __pyx_t_6);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyTuple_SET_ITEM(__pyx_t_8, 3+__pyx_t_2, __pyx_int_0);
      __Pyx_INCREF(Py_True);
      __Pyx_GIVEREF(Py_True);
      PyTuple_SET_ITEM(__pyx_t_8, 4+__pyx_t_2, Py_True);
      __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":434
 *                 True)
 * 
 *             self.format = rtt_format             # <<<<<<<<<<<<<<
 * 
 *         rtt.render(environ, self.number, x, y, self.width, self.height, draw_func)
 */
    __pyx_v_self->format = __pyx_v_5renpy_2gl_9gltexture_rtt_format;

    /* "renpy/gl/gltexture.pyx":417
 *         self.allocate()
 * 
 *         if self.format != rtt_format:             # <<<<<<<<<<<<<<
 * 
 *             glBindTexture(GL_TEXTURE_2D, self.number)
 */
  }

  /* "renpy/gl/gltexture.pyx":436
 *             self.format = rtt_format
 * 
 *         rtt.render(environ, self.number, x, y, self.width, self.height, draw_func)             # <<<<<<<<<<<<<<
 * 
 *         self.xadd = 0
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_rtt, __pyx_n_s_render); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->number); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_2 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[8] = {__pyx_t_7, __pyx_v_environ, __pyx_t_8, __pyx_v_x, __pyx_v_y, __pyx_t_6, __pyx_t_5, __pyx_v_draw_func};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_2, 7+__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[8] = {__pyx_t_7, __pyx_v_environ, __pyx_t_8, __pyx_v_x, __pyx_v_y, __pyx_t_6, __pyx_t_5, __pyx_v_draw_func};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_2, 7+__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(7+__pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_environ);
    __Pyx_GIVEREF(__pyx_v_environ);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_2, __pyx_v_environ);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_2, __pyx_t_8);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_2, __pyx_v_x);
    __Pyx_INCREF(__pyx_v_y);
    __Pyx_GIVEREF(__pyx_v_y);
    PyTuple_SET_ITEM(__pyx_t_9, 3+__pyx_t_2, __pyx_v_y);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 4+__pyx_t_2, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_9, 5+__pyx_t_2, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_draw_func);
    __Pyx_GIVEREF(__pyx_v_draw_func);
    PyTuple_SET_ITEM(__pyx_t_9, 6+__pyx_t_2, __pyx_v_draw_func);
    __pyx_t_8 = 0;
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 436, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":438
 *         rtt.render(environ, self.number, x, y, self.width, self.height, draw_func)
 * 
 *         self.xadd = 0             # <<<<<<<<<<<<<<
 *         self.yadd = 0
 *         self.xmul = 1.0 / self.width
 */
  __pyx_v_self->xadd = 0.0;

  /* "renpy/gl/gltexture.pyx":439
 * 
 *         self.xadd = 0
 *         self.yadd = 0             # <<<<<<<<<<<<<<
 *         self.xmul = 1.0 / self.width
 *         self.ymul = 1.0 / self.height
 */
  __pyx_v_self->yadd = 0.0;

  /* "renpy/gl/gltexture.pyx":440
 *         self.xadd = 0
 *         self.yadd = 0
 *         self.xmul = 1.0 / self.width             # <<<<<<<<<<<<<<
 *         self.ymul = 1.0 / self.height
 * 
 */
  if (unlikely(__pyx_v_self->width == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 440, __pyx_L1_error)
  }
  __pyx_v_self->xmul = (1.0 / __pyx_v_self->width);

  /* "renpy/gl/gltexture.pyx":441
 *         self.yadd = 0
 *         self.xmul = 1.0 / self.width
 *         self.ymul = 1.0 / self.height             # <<<<<<<<<<<<<<
 * 
 *     cpdef int allocate(self):
 */
  if (unlikely(__pyx_v_self->height == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(0, 441, __pyx_L1_error)
  }
  __pyx_v_self->ymul = (1.0 / __pyx_v_self->height);

  /* "renpy/gl/gltexture.pyx":413
 * 
 * 
 *     def render_to(self, x, y, draw_func, rtt, environ):             # <<<<<<<<<<<<<<
 * 
 *         self.allocate()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.render_to", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":443
 *         self.ymul = 1.0 / self.height
 * 
 *     cpdef int allocate(self):             # <<<<<<<<<<<<<<
 *         """
 *         This allocates a texture number, if necessary.
 */

static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9allocate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_5renpy_2gl_9gltexture_11TextureCore_allocate(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, int __pyx_skip_dispatch) {
  unsigned int __pyx_v_texnums[1];
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("allocate", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_allocate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9allocate)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 443, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl/gltexture.pyx":453
 *         cdef unsigned int texnums[1]
 * 
 *         if self.number != 0:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  __pyx_t_6 = ((__pyx_v_self->number != 0) != 0);
  if (__pyx_t_6) {

    /* "renpy/gl/gltexture.pyx":454
 * 
 *         if self.number != 0:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *         glGenTextures(1, texnums)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":453
 *         cdef unsigned int texnums[1]
 * 
 *         if self.number != 0:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":456
 *             return 0
 * 
 *         glGenTextures(1, texnums)             # <<<<<<<<<<<<<<
 * 
 *         self.number = texnums[0]
 */
  __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glGenTextures(1, __pyx_v_texnums); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 456, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":458
 *         glGenTextures(1, texnums)
 * 
 *         self.number = texnums[0]             # <<<<<<<<<<<<<<
 *         self.format = 0
 * 
 */
  __pyx_v_self->number = (__pyx_v_texnums[0]);

  /* "renpy/gl/gltexture.pyx":459
 * 
 *         self.number = texnums[0]
 *         self.format = 0             # <<<<<<<<<<<<<<
 * 
 *         texture_generation[self.number] = generation
 */
  __pyx_v_self->format = 0;

  /* "renpy/gl/gltexture.pyx":461
 *         self.format = 0
 * 
 *         texture_generation[self.number] = generation             # <<<<<<<<<<<<<<
 * 
 *         texture_numbers.add(texnums[0])
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_generation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_generation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(__Pyx_SetItemInt(__pyx_t_2, __pyx_v_self->number, __pyx_t_1, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1) < 0)) __PYX_ERR(0, 461, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":463
 *         texture_generation[self.number] = generation
 * 
 *         texture_numbers.add(texnums[0])             # <<<<<<<<<<<<<<
 * 
 *         total_texture_size += self.width * self.height * 4
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_numbers); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_add); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int((__pyx_v_texnums[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 463, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":465
 *         texture_numbers.add(texnums[0])
 * 
 *         total_texture_size += self.width * self.height * 4             # <<<<<<<<<<<<<<
 *         texture_count += 1
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_total_texture_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_long(((__pyx_v_self->width * __pyx_v_self->height) * 4)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyNumber_InPlaceAdd(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_total_texture_size, __pyx_t_7) < 0) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "renpy/gl/gltexture.pyx":466
 * 
 *         total_texture_size += self.width * self.height * 4
 *         texture_count += 1             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_count); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_PyInt_AddObjC(__pyx_t_7, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_count, __pyx_t_3) < 0) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":468
 *         texture_count += 1
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def deallocate(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":443
 *         self.ymul = 1.0 / self.height
 * 
 *     cpdef int allocate(self):             # <<<<<<<<<<<<<<
 *         """
 *         This allocates a texture number, if necessary.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_WriteUnraisable("renpy.gl.gltexture.TextureCore.allocate", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9allocate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_11TextureCore_8allocate[] = "\n        This allocates a texture number, if necessary.\n        ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9allocate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("allocate (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_8allocate(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_8allocate(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("allocate", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_f_5renpy_2gl_9gltexture_11TextureCore_allocate(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.allocate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":470
 *         return 0
 * 
 *     def deallocate(self):             # <<<<<<<<<<<<<<
 *         """
 *         Deallocates this texture. The texture must have been removed from a
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_11deallocate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_11TextureCore_10deallocate[] = "\n        Deallocates this texture. The texture must have been removed from a\n        free list before this is called.\n        ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_11deallocate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("deallocate (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10deallocate(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10deallocate(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  GLuint __pyx_v_texnums[1];
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  unsigned int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("deallocate", 0);

  /* "renpy/gl/gltexture.pyx":479
 *         global texture_count
 * 
 *         if self.number == 0:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->number == 0) != 0);
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":480
 * 
 *         if self.number == 0:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         cdef GLuint texnums[1]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":479
 *         global texture_count
 * 
 *         if self.number == 0:             # <<<<<<<<<<<<<<
 *             return
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":484
 *         cdef GLuint texnums[1]
 * 
 *         texnums[0] = self.number             # <<<<<<<<<<<<<<
 * 
 *         if texture_generation[self.number] == self.generation:
 */
  __pyx_t_2 = __pyx_v_self->number;
  (__pyx_v_texnums[0]) = __pyx_t_2;

  /* "renpy/gl/gltexture.pyx":486
 *         texnums[0] = self.number
 * 
 *         if texture_generation[self.number] == self.generation:             # <<<<<<<<<<<<<<
 *             glDeleteTextures(1, texnums)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_generation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, __pyx_v_self->number, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->generation); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":487
 * 
 *         if texture_generation[self.number] == self.generation:
 *             glDeleteTextures(1, texnums)             # <<<<<<<<<<<<<<
 * 
 *         self.number = 0
 */
    __pyx_t_6 = __pyx_f_5renpy_2gl_2gl_glDeleteTextures(1, __pyx_v_texnums); if (unlikely(__pyx_t_6 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 487, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":486
 *         texnums[0] = self.number
 * 
 *         if texture_generation[self.number] == self.generation:             # <<<<<<<<<<<<<<
 *             glDeleteTextures(1, texnums)
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":489
 *             glDeleteTextures(1, texnums)
 * 
 *         self.number = 0             # <<<<<<<<<<<<<<
 * 
 *         texture_numbers.discard(self.number)
 */
  __pyx_v_self->number = 0;

  /* "renpy/gl/gltexture.pyx":491
 *         self.number = 0
 * 
 *         texture_numbers.discard(self.number)             # <<<<<<<<<<<<<<
 *         total_texture_size -= self.width * self.height * 4
 *         texture_count -= 1
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_numbers); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_discard); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->number); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 491, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_5);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_3};
      __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 491, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_7, __pyx_t_3};
      __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 491, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 491, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 491, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":492
 * 
 *         texture_numbers.discard(self.number)
 *         total_texture_size -= self.width * self.height * 4             # <<<<<<<<<<<<<<
 *         texture_count -= 1
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_total_texture_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyInt_From_long(((__pyx_v_self->width * __pyx_v_self->height) * 4)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = PyNumber_InPlaceSubtract(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_total_texture_size, __pyx_t_8) < 0) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "renpy/gl/gltexture.pyx":493
 *         texture_numbers.discard(self.number)
 *         total_texture_size -= self.width * self.height * 4
 *         texture_count -= 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_count); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyInt_SubtractObjC(__pyx_t_8, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_count, __pyx_t_4) < 0) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":470
 *         return 0
 * 
 *     def deallocate(self):             # <<<<<<<<<<<<<<
 *         """
 *         Deallocates this texture. The texture must have been removed from a
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.deallocate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":27
 * 
 * cdef class TextureCore:
 *     cdef public int width             # <<<<<<<<<<<<<<
 *     cdef public int height
 *     cdef public int generation
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5width___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5width___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5width_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5width_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 27, __pyx_L1_error)
  __pyx_v_self->width = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":28
 * cdef class TextureCore:
 *     cdef public int width
 *     cdef public int height             # <<<<<<<<<<<<<<
 *     cdef public int generation
 *     cdef public unsigned int number
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6height_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6height_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6height___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6height___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.height.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6height_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6height_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 28, __pyx_L1_error)
  __pyx_v_self->height = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.height.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":29
 *     cdef public int width
 *     cdef public int height
 *     cdef public int generation             # <<<<<<<<<<<<<<
 *     cdef public unsigned int number
 *     cdef int format
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_10generation_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_10generation_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10generation___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10generation___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->generation); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.generation.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_10generation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_10generation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10generation_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_10generation_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 29, __pyx_L1_error)
  __pyx_v_self->generation = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.generation.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":30
 *     cdef public int height
 *     cdef public int generation
 *     cdef public unsigned int number             # <<<<<<<<<<<<<<
 *     cdef int format
 *     cdef double xmul
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6number_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6number_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6number___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6number___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->number); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.number.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6number_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6number_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6number_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_6number_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  unsigned int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_unsigned_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_v_self->number = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.number.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":43
 *     cdef int premult_bottom
 *     cdef bint nearest
 *     cdef public list free_list             # <<<<<<<<<<<<<<
 * 
 *     cdef void make_ready(TextureCore)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->free_list);
  __pyx_r = __pyx_v_self->free_list;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 43, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->free_list);
  __Pyx_DECREF(__pyx_v_self->free_list);
  __pyx_v_self->free_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.free_list.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list_4__del__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_9free_list_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->free_list);
  __Pyx_DECREF(__pyx_v_self->free_list);
  __pyx_v_self->free_list = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":50
 *     cpdef int allocate(TextureCore)
 * 
 *     cdef public object debug             # <<<<<<<<<<<<<<
 * 
 * cdef class TextureGrid:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->debug);
  __pyx_r = __pyx_v_self->debug;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug_4__del__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_5debug_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.debug, self.format, self.free_list, self.generation, self.height, self.nearest, self.number, self.premult, self.premult_bottom, self.premult_left, self.premult_right, self.premult_size, self.premult_top, self.width, self.xadd, self.xmul, self.yadd, self.ymul)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_12__reduce_cython__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_12__reduce_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_t_17;
  int __pyx_t_18;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.debug, self.format, self.free_list, self.generation, self.height, self.nearest, self.number, self.premult, self.premult_bottom, self.premult_left, self.premult_right, self.premult_size, self.premult_top, self.width, self.xadd, self.xmul, self.yadd, self.ymul)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->format); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->generation); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->nearest); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->number); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_int(__pyx_v_self->premult_bottom); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_self->premult_left); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_v_self->premult_right); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyInt_From_int(__pyx_v_self->premult_top); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyFloat_FromDouble(__pyx_v_self->xadd); if (unlikely(!__pyx_t_11)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = PyFloat_FromDouble(__pyx_v_self->xmul); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = PyFloat_FromDouble(__pyx_v_self->yadd); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = PyFloat_FromDouble(__pyx_v_self->ymul); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_15 = PyTuple_New(18); if (unlikely(!__pyx_t_15)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __Pyx_INCREF(__pyx_v_self->debug);
  __Pyx_GIVEREF(__pyx_v_self->debug);
  PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_v_self->debug);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->free_list);
  __Pyx_GIVEREF(__pyx_v_self->free_list);
  PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_v_self->free_list);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_15, 3, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_15, 4, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_15, 5, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_15, 6, __pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->premult);
  __Pyx_GIVEREF(__pyx_v_self->premult);
  PyTuple_SET_ITEM(__pyx_t_15, 7, __pyx_v_self->premult);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_15, 8, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_15, 9, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_15, 10, __pyx_t_8);
  __Pyx_INCREF(__pyx_v_self->premult_size);
  __Pyx_GIVEREF(__pyx_v_self->premult_size);
  PyTuple_SET_ITEM(__pyx_t_15, 11, __pyx_v_self->premult_size);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_15, 12, __pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_15, 13, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_15, 14, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_15, 15, __pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_13);
  PyTuple_SET_ITEM(__pyx_t_15, 16, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_15, 17, __pyx_t_14);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __pyx_t_13 = 0;
  __pyx_t_14 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_15);
  __pyx_t_15 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.debug, self.format, self.free_list, self.generation, self.height, self.nearest, self.number, self.premult, self.premult_bottom, self.premult_left, self.premult_right, self.premult_size, self.premult_top, self.width, self.xadd, self.xmul, self.yadd, self.ymul)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_15 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_15)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_v__dict = __pyx_t_15;
  __pyx_t_15 = 0;

  /* "(tree fragment)":5
 *     state = (self.debug, self.format, self.free_list, self.generation, self.height, self.nearest, self.number, self.premult, self.premult_bottom, self.premult_left, self.premult_right, self.premult_size, self.premult_top, self.width, self.xadd, self.xmul, self.yadd, self.ymul)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_16 = (__pyx_v__dict != Py_None);
  __pyx_t_17 = (__pyx_t_16 != 0);
  if (__pyx_t_17) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_15 = PyTuple_New(1); if (unlikely(!__pyx_t_15)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_v__dict);
    __pyx_t_14 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_15); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_14));
    __pyx_t_14 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.debug is not None or self.free_list is not None or self.premult is not None or self.premult_size is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.debug, self.format, self.free_list, self.generation, self.height, self.nearest, self.number, self.premult, self.premult_bottom, self.premult_left, self.premult_right, self.premult_size, self.premult_top, self.width, self.xadd, self.xmul, self.yadd, self.ymul)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.debug is not None or self.free_list is not None or self.premult is not None or self.premult_size is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, None), state
 */
  /*else*/ {
    __pyx_t_16 = (__pyx_v_self->debug != Py_None);
    __pyx_t_18 = (__pyx_t_16 != 0);
    if (!__pyx_t_18) {
    } else {
      __pyx_t_17 = __pyx_t_18;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_18 = (__pyx_v_self->free_list != ((PyObject*)Py_None));
    __pyx_t_16 = (__pyx_t_18 != 0);
    if (!__pyx_t_16) {
    } else {
      __pyx_t_17 = __pyx_t_16;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_16 = (__pyx_v_self->premult != Py_None);
    __pyx_t_18 = (__pyx_t_16 != 0);
    if (!__pyx_t_18) {
    } else {
      __pyx_t_17 = __pyx_t_18;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_18 = (__pyx_v_self->premult_size != ((PyObject*)Py_None));
    __pyx_t_16 = (__pyx_t_18 != 0);
    __pyx_t_17 = __pyx_t_16;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_17;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.debug is not None or self.free_list is not None or self.premult is not None or self.premult_size is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, None), state
 *     else:
 */
  __pyx_t_17 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_17) {

    /* "(tree fragment)":11
 *         use_setstate = self.debug is not None or self.free_list is not None or self.premult is not None or self.premult_size is not None
 *     if use_setstate:
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_14 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_TextureCore); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_15 = PyTuple_New(3); if (unlikely(!__pyx_t_15)) __PYX_ERR(2, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_15, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_71597589);
    __Pyx_GIVEREF(__pyx_int_71597589);
    PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_int_71597589);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_15, 2, Py_None);
    __pyx_t_13 = PyTuple_New(3); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_15);
    PyTuple_SET_ITEM(__pyx_t_13, 1, __pyx_t_15);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_13, 2, __pyx_v_state);
    __pyx_t_14 = 0;
    __pyx_t_15 = 0;
    __pyx_r = __pyx_t_13;
    __pyx_t_13 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.debug is not None or self.free_list is not None or self.premult is not None or self.premult_size is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, None), state
 *     else:
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_TextureCore__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_13 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_TextureCore); if (unlikely(!__pyx_t_13)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_15 = PyTuple_New(3); if (unlikely(!__pyx_t_15)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_15, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_71597589);
    __Pyx_GIVEREF(__pyx_int_71597589);
    PyTuple_SET_ITEM(__pyx_t_15, 1, __pyx_int_71597589);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_15, 2, __pyx_v_state);
    __pyx_t_14 = PyTuple_New(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_13);
    PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_15);
    PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_15);
    __pyx_t_13 = 0;
    __pyx_t_15 = 0;
    __pyx_r = __pyx_t_14;
    __pyx_t_14 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.debug, self.format, self.free_list, self.generation, self.height, self.nearest, self.number, self.premult, self.premult_bottom, self.premult_left, self.premult_right, self.premult_size, self.premult_top, self.width, self.xadd, self.xmul, self.yadd, self.ymul)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_TextureCore__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureCore_14__setstate_cython__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureCore_14__setstate_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_TextureCore__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureCore__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_TextureCore, (type(self), 0x4447e15, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_TextureCore__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureCore.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":502
 *     """
 * 
 *     def __sizeof__(self):             # <<<<<<<<<<<<<<
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7Texture_1__sizeof__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_7Texture_1__sizeof__ = {"__sizeof__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_7Texture_1__sizeof__, METH_O, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7Texture_1__sizeof__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__sizeof__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_7Texture___sizeof__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_7Texture___sizeof__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("__sizeof__", 0);

  /* "renpy/gl/gltexture.pyx":503
 * 
 *     def __sizeof__(self):
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4             # <<<<<<<<<<<<<<
 * 
 *     def __getstate__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureCore), __pyx_n_s_sizeof); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_v_self};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 503, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_self);
      __Pyx_GIVEREF(__pyx_v_self);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_self);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 503, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_width); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_height); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyNumber_Multiply(__pyx_t_3, __pyx_int_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":502
 *     """
 * 
 *     def __sizeof__(self):             # <<<<<<<<<<<<<<
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl.gltexture.Texture.__sizeof__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":505
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7Texture_3__getstate__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_7Texture_3__getstate__ = {"__getstate__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_7Texture_3__getstate__, METH_O, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7Texture_3__getstate__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_7Texture_2__getstate__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_7Texture_2__getstate__(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("__getstate__", 0);

  /* "renpy/gl/gltexture.pyx":506
 * 
 *     def __getstate__(self):
 *         if renpy.config.developer:             # <<<<<<<<<<<<<<
 *             raise Exception("Can't pickle a texture.")
 *         else:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_developer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":507
 *     def __getstate__(self):
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")             # <<<<<<<<<<<<<<
 *         else:
 *             return { }
 */
    __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 507, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":506
 * 
 *     def __getstate__(self):
 *         if renpy.config.developer:             # <<<<<<<<<<<<<<
 *             raise Exception("Can't pickle a texture.")
 *         else:
 */
  }

  /* "renpy/gl/gltexture.pyx":509
 *             raise Exception("Can't pickle a texture.")
 *         else:
 *             return { }             # <<<<<<<<<<<<<<
 * 
 *     def __setstate__(self, state):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 509, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "renpy/gl/gltexture.pyx":505
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl.gltexture.Texture.__getstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":511
 *             return { }
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7Texture_5__setstate__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_7Texture_5__setstate__ = {"__setstate__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_7Texture_5__setstate__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7Texture_5__setstate__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_self = 0;
  CYTHON_UNUSED PyObject *__pyx_v_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_state,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__setstate__", 1, 2, 2, 1); __PYX_ERR(0, 511, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__setstate__") < 0)) __PYX_ERR(0, 511, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_state = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 511, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.Texture.__setstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_7Texture_4__setstate__(__pyx_self, __pyx_v_self, __pyx_v_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_7Texture_4__setstate__(CYTHON_UNUSED PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "renpy/gl/gltexture.pyx":512
 * 
 *     def __setstate__(self, state):
 *         return             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":511
 *             return { }
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         return
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":531
 * # This allocates a texture, either from the free list, or by asking
 * # gl.
 * def alloc_texture(width, height):             # <<<<<<<<<<<<<<
 *     """
 *     Allocate a texture, either from the freelist or by asking GL. The
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7alloc_texture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_6alloc_texture[] = "\n    Allocate a texture, either from the freelist or by asking GL. The\n    returned texture has a reference count of 1.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_7alloc_texture = {"alloc_texture", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_7alloc_texture, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_6alloc_texture};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_7alloc_texture(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("alloc_texture (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("alloc_texture", 1, 2, 2, 1); __PYX_ERR(0, 531, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "alloc_texture") < 0)) __PYX_ERR(0, 531, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_width = values[0];
    __pyx_v_height = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("alloc_texture", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 531, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.alloc_texture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_6alloc_texture(__pyx_self, __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_6alloc_texture(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width, PyObject *__pyx_v_height) {
  PyObject *__pyx_v_rv = NULL;
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("alloc_texture", 0);

  /* "renpy/gl/gltexture.pyx":537
 *     """
 * 
 *     if renpy.game.preferences.gl_npot:             # <<<<<<<<<<<<<<
 *         rv = Texture(width, height)
 *         rv.free_list = npot_free_textures
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_game); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_preferences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_gl_npot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":538
 * 
 *     if renpy.game.preferences.gl_npot:
 *         rv = Texture(width, height)             # <<<<<<<<<<<<<<
 *         rv.free_list = npot_free_textures
 *         rv.generation = generation
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_Texture); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 538, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_width, __pyx_v_height};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 538, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_width, __pyx_v_height};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 538, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 538, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_INCREF(__pyx_v_width);
      __Pyx_GIVEREF(__pyx_v_width);
      PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_width);
      __Pyx_INCREF(__pyx_v_height);
      __Pyx_GIVEREF(__pyx_v_height);
      PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_height);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 538, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_rv = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":539
 *     if renpy.game.preferences.gl_npot:
 *         rv = Texture(width, height)
 *         rv.free_list = npot_free_textures             # <<<<<<<<<<<<<<
 *         rv.generation = generation
 *         return rv
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_npot_free_textures); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_rv, __pyx_n_s_free_list, __pyx_t_2) < 0) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":540
 *         rv = Texture(width, height)
 *         rv.free_list = npot_free_textures
 *         rv.generation = generation             # <<<<<<<<<<<<<<
 *         return rv
 * 
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_generation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 540, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_rv, __pyx_n_s_generation, __pyx_t_2) < 0) __PYX_ERR(0, 540, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":541
 *         rv.free_list = npot_free_textures
 *         rv.generation = generation
 *         return rv             # <<<<<<<<<<<<<<
 * 
 *     l = free_textures[width, height]
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_rv);
    __pyx_r = __pyx_v_rv;
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":537
 *     """
 * 
 *     if renpy.game.preferences.gl_npot:             # <<<<<<<<<<<<<<
 *         rv = Texture(width, height)
 *         rv.free_list = npot_free_textures
 */
  }

  /* "renpy/gl/gltexture.pyx":543
 *         return rv
 * 
 *     l = free_textures[width, height]             # <<<<<<<<<<<<<<
 * 
 *     if l:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_free_textures); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_width);
  __Pyx_GIVEREF(__pyx_v_width);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_width);
  __Pyx_INCREF(__pyx_v_height);
  __Pyx_GIVEREF(__pyx_v_height);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_height);
  __pyx_t_6 = PyObject_GetItem(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_l = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "renpy/gl/gltexture.pyx":545
 *     l = free_textures[width, height]
 * 
 *     if l:             # <<<<<<<<<<<<<<
 *         rv = l.pop()
 *     else:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_l); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 545, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":546
 * 
 *     if l:
 *         rv = l.pop()             # <<<<<<<<<<<<<<
 *     else:
 *         rv = Texture(width, height)
 */
    __pyx_t_6 = __Pyx_PyObject_Pop(__pyx_v_l); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 546, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_rv = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "renpy/gl/gltexture.pyx":545
 *     l = free_textures[width, height]
 * 
 *     if l:             # <<<<<<<<<<<<<<
 *         rv = l.pop()
 *     else:
 */
    goto __pyx_L4;
  }

  /* "renpy/gl/gltexture.pyx":548
 *         rv = l.pop()
 *     else:
 *         rv = Texture(width, height)             # <<<<<<<<<<<<<<
 * 
 *     rv.free_list = l
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_Texture); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 548, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = NULL;
    __pyx_t_5 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_5 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_width, __pyx_v_height};
      __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_width, __pyx_v_height};
      __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_GOTREF(__pyx_t_6);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_v_width);
      __Pyx_GIVEREF(__pyx_v_width);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_v_width);
      __Pyx_INCREF(__pyx_v_height);
      __Pyx_GIVEREF(__pyx_v_height);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_v_height);
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_rv = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __pyx_L4:;

  /* "renpy/gl/gltexture.pyx":550
 *         rv = Texture(width, height)
 * 
 *     rv.free_list = l             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_rv, __pyx_n_s_free_list, __pyx_v_l) < 0) __PYX_ERR(0, 550, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":552
 *     rv.free_list = l
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":531
 * # This allocates a texture, either from the free list, or by asking
 * # gl.
 * def alloc_texture(width, height):             # <<<<<<<<<<<<<<
 *     """
 *     Allocate a texture, either from the freelist or by asking GL. The
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.gl.gltexture.alloc_texture", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":555
 * 
 * 
 * def dealloc_textures():             # <<<<<<<<<<<<<<
 *     cdef GLuint texnums[1]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_9dealloc_textures(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_9dealloc_textures = {"dealloc_textures", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_9dealloc_textures, METH_NOARGS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_9dealloc_textures(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dealloc_textures (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_8dealloc_textures(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_8dealloc_textures(CYTHON_UNUSED PyObject *__pyx_self) {
  GLuint __pyx_v_texnums[1];
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  Py_ssize_t __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  GLuint __pyx_t_12;
  int __pyx_t_13;
  __Pyx_RefNannySetupContext("dealloc_textures", 0);

  /* "renpy/gl/gltexture.pyx":558
 *     cdef GLuint texnums[1]
 * 
 *     for l in free_textures.values():             # <<<<<<<<<<<<<<
 *         for t in l:
 *             t.deallocate()
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_free_textures); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_values); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 558, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 558, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 558, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 558, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 558, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 558, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_l, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":559
 * 
 *     for l in free_textures.values():
 *         for t in l:             # <<<<<<<<<<<<<<
 *             t.deallocate()
 * 
 */
    if (likely(PyList_CheckExact(__pyx_v_l)) || PyTuple_CheckExact(__pyx_v_l)) {
      __pyx_t_1 = __pyx_v_l; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
      __pyx_t_7 = NULL;
    } else {
      __pyx_t_6 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_l); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 559, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_7 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 559, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_7)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 559, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 559, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_2); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 559, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 559, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_7(__pyx_t_1);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 559, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":560
 *     for l in free_textures.values():
 *         for t in l:
 *             t.deallocate()             # <<<<<<<<<<<<<<
 * 
 *     for i in npot_free_textures:
 */
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_t, __pyx_n_s_deallocate); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 560, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (__pyx_t_9) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":559
 * 
 *     for l in free_textures.values():
 *         for t in l:             # <<<<<<<<<<<<<<
 *             t.deallocate()
 * 
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":558
 *     cdef GLuint texnums[1]
 * 
 *     for l in free_textures.values():             # <<<<<<<<<<<<<<
 *         for t in l:
 *             t.deallocate()
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":562
 *             t.deallocate()
 * 
 *     for i in npot_free_textures:             # <<<<<<<<<<<<<<
 *         i.deallocate()
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_npot_free_textures); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 562, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 562, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 562, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 562, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 562, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_5(__pyx_t_1);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 562, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":563
 * 
 *     for i in npot_free_textures:
 *         i.deallocate()             # <<<<<<<<<<<<<<
 * 
 *     npot_free_textures[:] = [ ]
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_i, __pyx_n_s_deallocate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 563, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_8) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 563, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 563, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":562
 *             t.deallocate()
 * 
 *     for i in npot_free_textures:             # <<<<<<<<<<<<<<
 *         i.deallocate()
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":565
 *         i.deallocate()
 * 
 *     npot_free_textures[:] = [ ]             # <<<<<<<<<<<<<<
 * 
 *     if not renpy.game.preferences.gl_npot:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_npot_free_textures); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_PyObject_SetSlice(__pyx_t_3, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__4, 0, 0, 1) < 0) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":567
 *     npot_free_textures[:] = [ ]
 * 
 *     if not renpy.game.preferences.gl_npot:             # <<<<<<<<<<<<<<
 * 
 *         for t in texture_numbers:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_game); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_preferences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_gl_npot); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_11 = ((!__pyx_t_10) != 0);
  if (__pyx_t_11) {

    /* "renpy/gl/gltexture.pyx":569
 *     if not renpy.game.preferences.gl_npot:
 * 
 *         for t in texture_numbers:             # <<<<<<<<<<<<<<
 *             texnums[0] = t
 *             glDeleteTextures(1, texnums)
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_texture_numbers); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
      __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_4 = 0;
      __pyx_t_5 = NULL;
    } else {
      __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_5 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 569, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    for (;;) {
      if (likely(!__pyx_t_5)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 569, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        } else {
          if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 569, __pyx_L1_error)
          #else
          __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          #endif
        }
      } else {
        __pyx_t_3 = __pyx_t_5(__pyx_t_1);
        if (unlikely(!__pyx_t_3)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 569, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "renpy/gl/gltexture.pyx":570
 * 
 *         for t in texture_numbers:
 *             texnums[0] = t             # <<<<<<<<<<<<<<
 *             glDeleteTextures(1, texnums)
 * 
 */
      __pyx_t_12 = __Pyx_PyInt_As_GLuint(__pyx_v_t); if (unlikely((__pyx_t_12 == ((GLuint)-1)) && PyErr_Occurred())) __PYX_ERR(0, 570, __pyx_L1_error)
      (__pyx_v_texnums[0]) = __pyx_t_12;

      /* "renpy/gl/gltexture.pyx":571
 *         for t in texture_numbers:
 *             texnums[0] = t
 *             glDeleteTextures(1, texnums)             # <<<<<<<<<<<<<<
 * 
 *     free_textures.clear()
 */
      __pyx_t_13 = __pyx_f_5renpy_2gl_2gl_glDeleteTextures(1, __pyx_v_texnums); if (unlikely(__pyx_t_13 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 571, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":569
 *     if not renpy.game.preferences.gl_npot:
 * 
 *         for t in texture_numbers:             # <<<<<<<<<<<<<<
 *             texnums[0] = t
 *             glDeleteTextures(1, texnums)
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":567
 *     npot_free_textures[:] = [ ]
 * 
 *     if not renpy.game.preferences.gl_npot:             # <<<<<<<<<<<<<<
 * 
 *         for t in texture_numbers:
 */
  }

  /* "renpy/gl/gltexture.pyx":573
 *             glDeleteTextures(1, texnums)
 * 
 *     free_textures.clear()             # <<<<<<<<<<<<<<
 * 
 *     # Do not reset texture numbers - we don't want to reuse a number that's
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_free_textures); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_clear); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 573, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":579
 * 
 *     global generation
 *     generation += 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_generation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_AddObjC(__pyx_t_1, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_generation, __pyx_t_2) < 0) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":555
 * 
 * 
 * def dealloc_textures():             # <<<<<<<<<<<<<<
 *     cdef GLuint texnums[1]
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("renpy.gl.gltexture.dealloc_textures", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":583
 * 
 * 
 * def cleanup():             # <<<<<<<<<<<<<<
 *     """
 *     This is called once per frame.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11cleanup(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_10cleanup[] = "\n    This is called once per frame.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_11cleanup = {"cleanup", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11cleanup, METH_NOARGS, __pyx_doc_5renpy_2gl_9gltexture_10cleanup};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11cleanup(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cleanup (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_10cleanup(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_10cleanup(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_v_l = NULL;
  PyObject *__pyx_v_t = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  PyObject *(*__pyx_t_5)(PyObject *);
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("cleanup", 0);

  /* "renpy/gl/gltexture.pyx":592
 *     # making it unlikely we'll constantly allocate/deallocate textures.
 * 
 *     for l in free_textures.values():             # <<<<<<<<<<<<<<
 *         if len(l) > 1:
 *             t = l.pop()
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_free_textures); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_values); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 592, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 592, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_4); __Pyx_INCREF(__pyx_t_1); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 592, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_5(__pyx_t_3);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 592, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_XDECREF_SET(__pyx_v_l, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":593
 * 
 *     for l in free_textures.values():
 *         if len(l) > 1:             # <<<<<<<<<<<<<<
 *             t = l.pop()
 *             t.deallocate()
 */
    __pyx_t_6 = PyObject_Length(__pyx_v_l); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 593, __pyx_L1_error)
    __pyx_t_7 = ((__pyx_t_6 > 1) != 0);
    if (__pyx_t_7) {

      /* "renpy/gl/gltexture.pyx":594
 *     for l in free_textures.values():
 *         if len(l) > 1:
 *             t = l.pop()             # <<<<<<<<<<<<<<
 *             t.deallocate()
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_Pop(__pyx_v_l); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":595
 *         if len(l) > 1:
 *             t = l.pop()
 *             t.deallocate()             # <<<<<<<<<<<<<<
 * 
 *     for i in npot_free_textures:
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_t, __pyx_n_s_deallocate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 595, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (__pyx_t_8) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 595, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 595, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":593
 * 
 *     for l in free_textures.values():
 *         if len(l) > 1:             # <<<<<<<<<<<<<<
 *             t = l.pop()
 *             t.deallocate()
 */
    }

    /* "renpy/gl/gltexture.pyx":592
 *     # making it unlikely we'll constantly allocate/deallocate textures.
 * 
 *     for l in free_textures.values():             # <<<<<<<<<<<<<<
 *         if len(l) > 1:
 *             t = l.pop()
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":597
 *             t.deallocate()
 * 
 *     for i in npot_free_textures:             # <<<<<<<<<<<<<<
 *         i.deallocate()
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_npot_free_textures); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
  } else {
    __pyx_t_4 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 597, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_5)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_4 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 597, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_4 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_4); __Pyx_INCREF(__pyx_t_3); __pyx_t_4++; if (unlikely(0 < 0)) __PYX_ERR(0, 597, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_4); __pyx_t_4++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 597, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_5(__pyx_t_1);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 597, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":598
 * 
 *     for i in npot_free_textures:
 *         i.deallocate()             # <<<<<<<<<<<<<<
 * 
 *     npot_free_textures[:] = [ ]
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_i, __pyx_n_s_deallocate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 598, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_8) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    } else {
      __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":597
 *             t.deallocate()
 * 
 *     for i in npot_free_textures:             # <<<<<<<<<<<<<<
 *         i.deallocate()
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":600
 *         i.deallocate()
 * 
 *     npot_free_textures[:] = [ ]             # <<<<<<<<<<<<<<
 * 
 * def compute_subrow(row, offset, width):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_npot_free_textures); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_PyObject_SetSlice(__pyx_t_3, __pyx_t_1, 0, 0, NULL, NULL, &__pyx_slice__5, 0, 0, 1) < 0) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":583
 * 
 * 
 * def cleanup():             # <<<<<<<<<<<<<<
 *     """
 *     This is called once per frame.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.gl.gltexture.cleanup", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_l);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":602
 *     npot_free_textures[:] = [ ]
 * 
 * def compute_subrow(row, offset, width):             # <<<<<<<<<<<<<<
 *     """
 *     Given a row (or column), this computes a subrow starting at the given
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_13compute_subrow(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_12compute_subrow[] = "\n    Given a row (or column), this computes a subrow starting at the given\n    offset and having the given width.\n\n    It returns two list. The first is a list of (offset, width,\n    output-tile) tuples. The second is a list of integers, giving the\n    input tile corresponding to each output tile.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_13compute_subrow = {"compute_subrow", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_13compute_subrow, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_12compute_subrow};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_13compute_subrow(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_row = 0;
  PyObject *__pyx_v_offset = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compute_subrow (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_row,&__pyx_n_s_offset,&__pyx_n_s_width,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_row)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_offset)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compute_subrow", 1, 3, 3, 1); __PYX_ERR(0, 602, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compute_subrow", 1, 3, 3, 2); __PYX_ERR(0, 602, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compute_subrow") < 0)) __PYX_ERR(0, 602, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_row = values[0];
    __pyx_v_offset = values[1];
    __pyx_v_width = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compute_subrow", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 602, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.compute_subrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_12compute_subrow(__pyx_self, __pyx_v_row, __pyx_v_offset, __pyx_v_width);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_12compute_subrow(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_row, PyObject *__pyx_v_offset, PyObject *__pyx_v_width) {
  PyObject *__pyx_v_rowi = NULL;
  PyObject *__pyx_v_outrow = NULL;
  PyObject *__pyx_v_tiles = NULL;
  PyObject *__pyx_v_outtile = NULL;
  PyObject *__pyx_v_ioff = NULL;
  PyObject *__pyx_v_iwidth = NULL;
  PyObject *__pyx_v_itile = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("compute_subrow", 0);
  __Pyx_INCREF(__pyx_v_offset);
  __Pyx_INCREF(__pyx_v_width);

  /* "renpy/gl/gltexture.pyx":613
 * 
 *     # An iterator over row.
 *     rowi = iter(row)             # <<<<<<<<<<<<<<
 * 
 *     # The output row and the output tile list.
 */
  __pyx_t_1 = PyObject_GetIter(__pyx_v_row); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rowi = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":616
 * 
 *     # The output row and the output tile list.
 *     outrow = [ ]             # <<<<<<<<<<<<<<
 *     tiles = [ ]
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_outrow = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":617
 *     # The output row and the output tile list.
 *     outrow = [ ]
 *     tiles = [ ]             # <<<<<<<<<<<<<<
 * 
 *     # The output tile index.
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_tiles = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":620
 * 
 *     # The output tile index.
 *     outtile = 0             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_outtile = __pyx_int_0;

  /* "renpy/gl/gltexture.pyx":622
 *     outtile = 0
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         ioff, iwidth, itile = rowi.next()
 * 
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "renpy/gl/gltexture.pyx":623
 * 
 *     try:
 *         ioff, iwidth, itile = rowi.next()             # <<<<<<<<<<<<<<
 * 
 *         # Consume the offset.
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_rowi, __pyx_n_s_next); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 623, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
        }
      }
      if (__pyx_t_6) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 623, __pyx_L3_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 623, __pyx_L3_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_7);
        #else
        __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 623, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 623, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_6);
        __pyx_t_7 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 623, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 623, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
        index = 0; __pyx_t_5 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_5)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        index = 1; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        index = 2; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 3) < 0) __PYX_ERR(0, 623, __pyx_L3_error)
        __pyx_t_9 = NULL;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_9 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 623, __pyx_L3_error)
        __pyx_L10_unpacking_done:;
      }
      __pyx_v_ioff = __pyx_t_5;
      __pyx_t_5 = 0;
      __pyx_v_iwidth = __pyx_t_6;
      __pyx_t_6 = 0;
      __pyx_v_itile = __pyx_t_7;
      __pyx_t_7 = 0;

      /* "renpy/gl/gltexture.pyx":626
 * 
 *         # Consume the offset.
 *         while True:             # <<<<<<<<<<<<<<
 *             if offset < iwidth:
 *                 ioff += offset
 */
      while (1) {

        /* "renpy/gl/gltexture.pyx":627
 *         # Consume the offset.
 *         while True:
 *             if offset < iwidth:             # <<<<<<<<<<<<<<
 *                 ioff += offset
 *                 iwidth -= offset
 */
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_offset, __pyx_v_iwidth, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 627, __pyx_L3_error)
        __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 627, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_10) {

          /* "renpy/gl/gltexture.pyx":628
 *         while True:
 *             if offset < iwidth:
 *                 ioff += offset             # <<<<<<<<<<<<<<
 *                 iwidth -= offset
 *                 break
 */
          __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_ioff, __pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_ioff, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "renpy/gl/gltexture.pyx":629
 *             if offset < iwidth:
 *                 ioff += offset
 *                 iwidth -= offset             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
          __pyx_t_1 = PyNumber_InPlaceSubtract(__pyx_v_iwidth, __pyx_v_offset); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 629, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_iwidth, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "renpy/gl/gltexture.pyx":630
 *                 ioff += offset
 *                 iwidth -= offset
 *                 break             # <<<<<<<<<<<<<<
 * 
 * 
 */
          goto __pyx_L12_break;

          /* "renpy/gl/gltexture.pyx":627
 *         # Consume the offset.
 *         while True:
 *             if offset < iwidth:             # <<<<<<<<<<<<<<
 *                 ioff += offset
 *                 iwidth -= offset
 */
        }

        /* "renpy/gl/gltexture.pyx":633
 * 
 * 
 *             offset -= iwidth             # <<<<<<<<<<<<<<
 *             ioff, iwidth, itile = rowi.next()
 * 
 */
        __pyx_t_1 = PyNumber_InPlaceSubtract(__pyx_v_offset, __pyx_v_iwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 633, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_offset, __pyx_t_1);
        __pyx_t_1 = 0;

        /* "renpy/gl/gltexture.pyx":634
 * 
 *             offset -= iwidth
 *             ioff, iwidth, itile = rowi.next()             # <<<<<<<<<<<<<<
 * 
 *         # Consume the width.
 */
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_rowi, __pyx_n_s_next); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 634, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_7, function);
          }
        }
        if (__pyx_t_6) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else {
          __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 634, __pyx_L3_error)
        }
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
          PyObject* sequence = __pyx_t_1;
          #if !CYTHON_COMPILING_IN_PYPY
          Py_ssize_t size = Py_SIZE(sequence);
          #else
          Py_ssize_t size = PySequence_Size(sequence);
          #endif
          if (unlikely(size != 3)) {
            if (size > 3) __Pyx_RaiseTooManyValuesError(3);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 634, __pyx_L3_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
            __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
          } else {
            __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
            __pyx_t_5 = PyList_GET_ITEM(sequence, 2); 
          }
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_5);
          #else
          __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 634, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_7);
          __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 634, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_5 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 634, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
          #endif
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 634, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
          index = 0; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L14_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_7);
          index = 1; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L14_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_6);
          index = 2; __pyx_t_5 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_5)) goto __pyx_L14_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_5);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 3) < 0) __PYX_ERR(0, 634, __pyx_L3_error)
          __pyx_t_9 = NULL;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          goto __pyx_L15_unpacking_done;
          __pyx_L14_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_9 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 634, __pyx_L3_error)
          __pyx_L15_unpacking_done:;
        }
        __Pyx_DECREF_SET(__pyx_v_ioff, __pyx_t_7);
        __pyx_t_7 = 0;
        __Pyx_DECREF_SET(__pyx_v_iwidth, __pyx_t_6);
        __pyx_t_6 = 0;
        __Pyx_DECREF_SET(__pyx_v_itile, __pyx_t_5);
        __pyx_t_5 = 0;
      }
      __pyx_L12_break:;

      /* "renpy/gl/gltexture.pyx":637
 * 
 *         # Consume the width.
 *         while True:             # <<<<<<<<<<<<<<
 * 
 *             if width < iwidth:
 */
      while (1) {

        /* "renpy/gl/gltexture.pyx":639
 *         while True:
 * 
 *             if width < iwidth:             # <<<<<<<<<<<<<<
 *                 outrow.append((ioff, width, outtile))
 *                 tiles.append(itile)
 */
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_width, __pyx_v_iwidth, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 639, __pyx_L3_error)
        __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 639, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_10) {

          /* "renpy/gl/gltexture.pyx":640
 * 
 *             if width < iwidth:
 *                 outrow.append((ioff, width, outtile))             # <<<<<<<<<<<<<<
 *                 tiles.append(itile)
 *                 outtile += 1
 */
          __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(__pyx_v_ioff);
          __Pyx_GIVEREF(__pyx_v_ioff);
          PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_ioff);
          __Pyx_INCREF(__pyx_v_width);
          __Pyx_GIVEREF(__pyx_v_width);
          PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_width);
          __Pyx_INCREF(__pyx_v_outtile);
          __Pyx_GIVEREF(__pyx_v_outtile);
          PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_outtile);
          __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_outrow, __pyx_t_1); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 640, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

          /* "renpy/gl/gltexture.pyx":641
 *             if width < iwidth:
 *                 outrow.append((ioff, width, outtile))
 *                 tiles.append(itile)             # <<<<<<<<<<<<<<
 *                 outtile += 1
 *                 break
 */
          __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_tiles, __pyx_v_itile); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 641, __pyx_L3_error)

          /* "renpy/gl/gltexture.pyx":642
 *                 outrow.append((ioff, width, outtile))
 *                 tiles.append(itile)
 *                 outtile += 1             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
          __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_outtile, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 642, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF_SET(__pyx_v_outtile, __pyx_t_1);
          __pyx_t_1 = 0;

          /* "renpy/gl/gltexture.pyx":643
 *                 tiles.append(itile)
 *                 outtile += 1
 *                 break             # <<<<<<<<<<<<<<
 * 
 *             outrow.append((ioff, iwidth, outtile))
 */
          goto __pyx_L17_break;

          /* "renpy/gl/gltexture.pyx":639
 *         while True:
 * 
 *             if width < iwidth:             # <<<<<<<<<<<<<<
 *                 outrow.append((ioff, width, outtile))
 *                 tiles.append(itile)
 */
        }

        /* "renpy/gl/gltexture.pyx":645
 *                 break
 * 
 *             outrow.append((ioff, iwidth, outtile))             # <<<<<<<<<<<<<<
 *             tiles.append(itile)
 *             outtile += 1
 */
        __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 645, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_v_ioff);
        __Pyx_GIVEREF(__pyx_v_ioff);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_ioff);
        __Pyx_INCREF(__pyx_v_iwidth);
        __Pyx_GIVEREF(__pyx_v_iwidth);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_iwidth);
        __Pyx_INCREF(__pyx_v_outtile);
        __Pyx_GIVEREF(__pyx_v_outtile);
        PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_outtile);
        __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_outrow, __pyx_t_1); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 645, __pyx_L3_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "renpy/gl/gltexture.pyx":646
 * 
 *             outrow.append((ioff, iwidth, outtile))
 *             tiles.append(itile)             # <<<<<<<<<<<<<<
 *             outtile += 1
 * 
 */
        __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_tiles, __pyx_v_itile); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 646, __pyx_L3_error)

        /* "renpy/gl/gltexture.pyx":647
 *             outrow.append((ioff, iwidth, outtile))
 *             tiles.append(itile)
 *             outtile += 1             # <<<<<<<<<<<<<<
 * 
 *             width -= iwidth
 */
        __pyx_t_1 = __Pyx_PyInt_AddObjC(__pyx_v_outtile, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 647, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_outtile, __pyx_t_1);
        __pyx_t_1 = 0;

        /* "renpy/gl/gltexture.pyx":649
 *             outtile += 1
 * 
 *             width -= iwidth             # <<<<<<<<<<<<<<
 * 
 *             ioff, iwidth, itile = rowi.next()
 */
        __pyx_t_1 = PyNumber_InPlaceSubtract(__pyx_v_width, __pyx_v_iwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_width, __pyx_t_1);
        __pyx_t_1 = 0;

        /* "renpy/gl/gltexture.pyx":651
 *             width -= iwidth
 * 
 *             ioff, iwidth, itile = rowi.next()             # <<<<<<<<<<<<<<
 * 
 *     except StopIteration:
 */
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_rowi, __pyx_n_s_next); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 651, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        if (__pyx_t_6) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L3_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else {
          __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L3_error)
        }
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
          PyObject* sequence = __pyx_t_1;
          #if !CYTHON_COMPILING_IN_PYPY
          Py_ssize_t size = Py_SIZE(sequence);
          #else
          Py_ssize_t size = PySequence_Size(sequence);
          #endif
          if (unlikely(size != 3)) {
            if (size > 3) __Pyx_RaiseTooManyValuesError(3);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 651, __pyx_L3_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
            __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
          } else {
            __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
            __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
          }
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          #else
          __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 651, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 651, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 651, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 651, __pyx_L3_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
          index = 0; __pyx_t_5 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_5)) goto __pyx_L19_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_5);
          index = 1; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L19_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_6);
          index = 2; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L19_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_7);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 3) < 0) __PYX_ERR(0, 651, __pyx_L3_error)
          __pyx_t_9 = NULL;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          goto __pyx_L20_unpacking_done;
          __pyx_L19_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_9 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 651, __pyx_L3_error)
          __pyx_L20_unpacking_done:;
        }
        __Pyx_DECREF_SET(__pyx_v_ioff, __pyx_t_5);
        __pyx_t_5 = 0;
        __Pyx_DECREF_SET(__pyx_v_iwidth, __pyx_t_6);
        __pyx_t_6 = 0;
        __Pyx_DECREF_SET(__pyx_v_itile, __pyx_t_7);
        __pyx_t_7 = 0;
      }
      __pyx_L17_break:;

      /* "renpy/gl/gltexture.pyx":622
 *     outtile = 0
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         ioff, iwidth, itile = rowi.next()
 * 
 */
    }
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":653
 *             ioff, iwidth, itile = rowi.next()
 * 
 *     except StopIteration:             # <<<<<<<<<<<<<<
 *         pass
 * 
 */
    __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_StopIteration);
    if (__pyx_t_12) {
      __Pyx_ErrRestore(0,0,0);
      goto __pyx_L4_exception_handled;
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "renpy/gl/gltexture.pyx":622
 *     outtile = 0
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         ioff, iwidth, itile = rowi.next()
 * 
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L4_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    __pyx_L8_try_end:;
  }

  /* "renpy/gl/gltexture.pyx":656
 *         pass
 * 
 *     return outrow, tiles             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_outrow);
  __Pyx_GIVEREF(__pyx_v_outrow);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_outrow);
  __Pyx_INCREF(__pyx_v_tiles);
  __Pyx_GIVEREF(__pyx_v_tiles);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_tiles);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":602
 *     npot_free_textures[:] = [ ]
 * 
 * def compute_subrow(row, offset, width):             # <<<<<<<<<<<<<<
 *     """
 *     Given a row (or column), this computes a subrow starting at the given
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("renpy.gl.gltexture.compute_subrow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rowi);
  __Pyx_XDECREF(__pyx_v_outrow);
  __Pyx_XDECREF(__pyx_v_tiles);
  __Pyx_XDECREF(__pyx_v_outtile);
  __Pyx_XDECREF(__pyx_v_ioff);
  __Pyx_XDECREF(__pyx_v_iwidth);
  __Pyx_XDECREF(__pyx_v_itile);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_width);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":665
 *     """
 * 
 *     def __init__(self, width, height): #@DuplicatedSignature             # <<<<<<<<<<<<<<
 * 
 *         # The width and height of this TextureGrid
 */

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 665, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 665, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_width = values[0];
    __pyx_v_height = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 665, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid___init__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid___init__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_width, PyObject *__pyx_v_height) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "renpy/gl/gltexture.pyx":668
 * 
 *         # The width and height of this TextureGrid
 *         self.width = width             # <<<<<<<<<<<<<<
 *         self.height = height
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_width); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 668, __pyx_L1_error)
  __pyx_v_self->width = __pyx_t_1;

  /* "renpy/gl/gltexture.pyx":669
 *         # The width and height of this TextureGrid
 *         self.width = width
 *         self.height = height             # <<<<<<<<<<<<<<
 * 
 *         # For each row of tiles, a tuple giving:
 */
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_height); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 669, __pyx_L1_error)
  __pyx_v_self->height = __pyx_t_1;

  /* "renpy/gl/gltexture.pyx":675
 *         # - The height of the row.
 *         # - The rowindex in tiles.
 *         self.rows = [ ]             # <<<<<<<<<<<<<<
 * 
 *         # For each column of tiles, a tuple giving.
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->rows);
  __Pyx_DECREF(__pyx_v_self->rows);
  __pyx_v_self->rows = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":681
 *         # - The width of the column.
 *         # - The colindex in tiles.
 *         self.columns = [ ]             # <<<<<<<<<<<<<<
 * 
 *         # The actual grid of texture titles, a list of lists of
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->columns);
  __Pyx_DECREF(__pyx_v_self->columns);
  __pyx_v_self->columns = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":686
 *         # textures. This is looked up by looking up rowindex and
 *         # colindex.
 *         self.tiles = [ ]             # <<<<<<<<<<<<<<
 * 
 *         # If it exists, a TextureGrid that is half the size of this
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->tiles);
  __Pyx_DECREF(__pyx_v_self->tiles);
  __pyx_v_self->tiles = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":690
 *         # If it exists, a TextureGrid that is half the size of this
 *         # one.
 *         self.half_cache = None             # <<<<<<<<<<<<<<
 * 
 *         # Has the texture been made ready once?
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->half_cache);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->half_cache));
  __pyx_v_self->half_cache = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)Py_None);

  /* "renpy/gl/gltexture.pyx":693
 * 
 *         # Has the texture been made ready once?
 *         self.ready = False             # <<<<<<<<<<<<<<
 * 
 *     def __getstate__(self): #@DuplicatedSignature
 */
  __pyx_v_self->ready = 0;

  /* "renpy/gl/gltexture.pyx":665
 *     """
 * 
 *     def __init__(self, width, height): #@DuplicatedSignature             # <<<<<<<<<<<<<<
 * 
 *         # The width and height of this TextureGrid
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":695
 *         self.ready = False
 * 
 *     def __getstate__(self): #@DuplicatedSignature             # <<<<<<<<<<<<<<
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_3__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_3__getstate__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_2__getstate__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_2__getstate__(CYTHON_UNUSED struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("__getstate__", 0);

  /* "renpy/gl/gltexture.pyx":696
 * 
 *     def __getstate__(self): #@DuplicatedSignature
 *         if renpy.config.developer:             # <<<<<<<<<<<<<<
 *             raise Exception("Can't pickle a texture.")
 *         else:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 696, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_config); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 696, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_developer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 696, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 696, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":697
 *     def __getstate__(self): #@DuplicatedSignature
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")             # <<<<<<<<<<<<<<
 *         else:
 *             return { }
 */
    __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 697, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":696
 * 
 *     def __getstate__(self): #@DuplicatedSignature
 *         if renpy.config.developer:             # <<<<<<<<<<<<<<
 *             raise Exception("Can't pickle a texture.")
 *         else:
 */
  }

  /* "renpy/gl/gltexture.pyx":699
 *             raise Exception("Can't pickle a texture.")
 *         else:
 *             return { }             # <<<<<<<<<<<<<<
 * 
 *     def __setstate__(self, state): #@DuplicatedSignature
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "renpy/gl/gltexture.pyx":695
 *         self.ready = False
 * 
 *     def __getstate__(self): #@DuplicatedSignature             # <<<<<<<<<<<<<<
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.__getstate__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":701
 *             return { }
 * 
 *     def __setstate__(self, state): #@DuplicatedSignature             # <<<<<<<<<<<<<<
 *         return
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_state); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5__setstate__(PyObject *__pyx_v_self, PyObject *__pyx_v_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_4__setstate__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_4__setstate__(CYTHON_UNUSED struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate__", 0);

  /* "renpy/gl/gltexture.pyx":702
 * 
 *     def __setstate__(self, state): #@DuplicatedSignature
 *         return             # <<<<<<<<<<<<<<
 * 
 *     def get_size(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":701
 *             return { }
 * 
 *     def __setstate__(self, state): #@DuplicatedSignature             # <<<<<<<<<<<<<<
 *         return
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":704
 *         return
 * 
 *     def get_size(self):             # <<<<<<<<<<<<<<
 *         return self.width, self.height
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_7get_size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_7get_size(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_size (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6get_size(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6get_size(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("get_size", 0);

  /* "renpy/gl/gltexture.pyx":705
 * 
 *     def get_size(self):
 *         return self.width, self.height             # <<<<<<<<<<<<<<
 * 
 *     def subsurface(self, rect):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":704
 *         return
 * 
 *     def get_size(self):             # <<<<<<<<<<<<<<
 *         return self.width, self.height
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.get_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":707
 *         return self.width, self.height
 * 
 *     def subsurface(self, rect):             # <<<<<<<<<<<<<<
 *         """
 *         This produces a texture grid containing a rectangle "cut out"
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_9subsurface(PyObject *__pyx_v_self, PyObject *__pyx_v_rect); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_11TextureGrid_8subsurface[] = "\n        This produces a texture grid containing a rectangle \"cut out\"\n        of this texture grid.\n        ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_9subsurface(PyObject *__pyx_v_self, PyObject *__pyx_v_rect) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("subsurface (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_8subsurface(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_rect));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_8subsurface(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_rect) {
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_y = NULL;
  PyObject *__pyx_v_w = NULL;
  PyObject *__pyx_v_h = NULL;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_rv = NULL;
  PyObject *__pyx_v_rowtiles = NULL;
  PyObject *__pyx_v_coltiles = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_row = NULL;
  PyObject *__pyx_v_j = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  Py_ssize_t __pyx_t_10;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("subsurface", 0);

  /* "renpy/gl/gltexture.pyx":713
 *         """
 * 
 *         (x, y, w, h) = rect             # <<<<<<<<<<<<<<
 * 
 *         rv = TextureGrid(w, h)
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_rect))) || (PyList_CheckExact(__pyx_v_rect))) {
    PyObject* sequence = __pyx_v_rect;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 713, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 3); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 3); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
      for (i=0; i < 4; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 713, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_2,&__pyx_t_3,&__pyx_t_4};
    __pyx_t_5 = PyObject_GetIter(__pyx_v_rect); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 713, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = Py_TYPE(__pyx_t_5)->tp_iternext;
    for (index=0; index < 4; index++) {
      PyObject* item = __pyx_t_6(__pyx_t_5); if (unlikely(!item)) goto __pyx_L3_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_5), 4) < 0) __PYX_ERR(0, 713, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 713, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_x = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_y = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_w = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_h = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":715
 *         (x, y, w, h) = rect
 * 
 *         rv = TextureGrid(w, h)             # <<<<<<<<<<<<<<
 * 
 *         rv.rows, rowtiles = compute_subrow(self.rows, y, h)
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_w);
  __Pyx_GIVEREF(__pyx_v_w);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_w);
  __Pyx_INCREF(__pyx_v_h);
  __Pyx_GIVEREF(__pyx_v_h);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_h);
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureGrid), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_rv = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":717
 *         rv = TextureGrid(w, h)
 * 
 *         rv.rows, rowtiles = compute_subrow(self.rows, y, h)             # <<<<<<<<<<<<<<
 *         rv.columns, coltiles = compute_subrow(self.columns, x, w)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_compute_subrow); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_self->rows, __pyx_v_y, __pyx_v_h};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_self->rows, __pyx_v_y, __pyx_v_h};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->rows);
    __Pyx_GIVEREF(__pyx_v_self->rows);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_7, __pyx_v_self->rows);
    __Pyx_INCREF(__pyx_v_y);
    __Pyx_GIVEREF(__pyx_v_y);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_7, __pyx_v_y);
    __Pyx_INCREF(__pyx_v_h);
    __Pyx_GIVEREF(__pyx_v_h);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_7, __pyx_v_h);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
    PyObject* sequence = __pyx_t_3;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 717, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_1);
    #else
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 717, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext;
    index = 0; __pyx_t_4 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    index = 1; __pyx_t_1 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_1)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_2), 2) < 0) __PYX_ERR(0, 717, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 717, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  if (!(likely(PyList_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_4)->tp_name), 0))) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_rv->rows);
  __Pyx_DECREF(__pyx_v_rv->rows);
  __pyx_v_rv->rows = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_rowtiles = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":718
 * 
 *         rv.rows, rowtiles = compute_subrow(self.rows, y, h)
 *         rv.columns, coltiles = compute_subrow(self.columns, x, w)             # <<<<<<<<<<<<<<
 * 
 *         for i in rowtiles:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_compute_subrow); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_self->columns, __pyx_v_x, __pyx_v_w};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_self->columns, __pyx_v_x, __pyx_v_w};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else
  #endif
  {
    __pyx_t_2 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self->columns);
    __Pyx_GIVEREF(__pyx_v_self->columns);
    PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_7, __pyx_v_self->columns);
    __Pyx_INCREF(__pyx_v_x);
    __Pyx_GIVEREF(__pyx_v_x);
    PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_7, __pyx_v_x);
    __Pyx_INCREF(__pyx_v_w);
    __Pyx_GIVEREF(__pyx_v_w);
    PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_7, __pyx_v_w);
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
    PyObject* sequence = __pyx_t_3;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 718, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 718, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_6(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L7_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_6(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L7_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_4), 2) < 0) __PYX_ERR(0, 718, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L8_unpacking_done;
    __pyx_L7_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 718, __pyx_L1_error)
    __pyx_L8_unpacking_done:;
  }
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 718, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_rv->columns);
  __Pyx_DECREF(__pyx_v_rv->columns);
  __pyx_v_rv->columns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_v_coltiles = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":720
 *         rv.columns, coltiles = compute_subrow(self.columns, x, w)
 * 
 *         for i in rowtiles:             # <<<<<<<<<<<<<<
 *             row = [ ]
 *             for j in coltiles:
 */
  if (likely(PyList_CheckExact(__pyx_v_rowtiles)) || PyTuple_CheckExact(__pyx_v_rowtiles)) {
    __pyx_t_3 = __pyx_v_rowtiles; __Pyx_INCREF(__pyx_t_3); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_rowtiles); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 720, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 720, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 720, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_8); __Pyx_INCREF(__pyx_t_2); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 720, __pyx_L1_error)
        #else
        __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      }
    } else {
      __pyx_t_2 = __pyx_t_9(__pyx_t_3);
      if (unlikely(!__pyx_t_2)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 720, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":721
 * 
 *         for i in rowtiles:
 *             row = [ ]             # <<<<<<<<<<<<<<
 *             for j in coltiles:
 *                 row.append(self.tiles[i][j])
 */
    __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 721, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_row, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":722
 *         for i in rowtiles:
 *             row = [ ]
 *             for j in coltiles:             # <<<<<<<<<<<<<<
 *                 row.append(self.tiles[i][j])
 * 
 */
    if (likely(PyList_CheckExact(__pyx_v_coltiles)) || PyTuple_CheckExact(__pyx_v_coltiles)) {
      __pyx_t_2 = __pyx_v_coltiles; __Pyx_INCREF(__pyx_t_2); __pyx_t_10 = 0;
      __pyx_t_11 = NULL;
    } else {
      __pyx_t_10 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_coltiles); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 722, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_11 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 722, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_11)) {
        if (likely(PyList_CheckExact(__pyx_t_2))) {
          if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_10); __Pyx_INCREF(__pyx_t_1); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 722, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_10); __Pyx_INCREF(__pyx_t_1); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 722, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 722, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_11(__pyx_t_2);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 722, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_XDECREF_SET(__pyx_v_j, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":723
 *             row = [ ]
 *             for j in coltiles:
 *                 row.append(self.tiles[i][j])             # <<<<<<<<<<<<<<
 * 
 *             rv.tiles.append(row)
 */
      if (unlikely(__pyx_v_self->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 723, __pyx_L1_error)
      }
      __pyx_t_1 = PyObject_GetItem(__pyx_v_self->tiles, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 723, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_4 = PyObject_GetItem(__pyx_t_1, __pyx_v_j); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 723, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_row, __pyx_t_4); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 723, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "renpy/gl/gltexture.pyx":722
 *         for i in rowtiles:
 *             row = [ ]
 *             for j in coltiles:             # <<<<<<<<<<<<<<
 *                 row.append(self.tiles[i][j])
 * 
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":725
 *                 row.append(self.tiles[i][j])
 * 
 *             rv.tiles.append(row)             # <<<<<<<<<<<<<<
 * 
 *         return rv
 */
    if (unlikely(__pyx_v_rv->tiles == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 725, __pyx_L1_error)
    }
    __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_rv->tiles, __pyx_v_row); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 725, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":720
 *         rv.columns, coltiles = compute_subrow(self.columns, x, w)
 * 
 *         for i in rowtiles:             # <<<<<<<<<<<<<<
 *             row = [ ]
 *             for j in coltiles:
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":727
 *             rv.tiles.append(row)
 * 
 *         return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = ((PyObject *)__pyx_v_rv);
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":707
 *         return self.width, self.height
 * 
 *     def subsurface(self, rect):             # <<<<<<<<<<<<<<
 *         """
 *         This produces a texture grid containing a rectangle "cut out"
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.subsurface", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XDECREF(__pyx_v_w);
  __Pyx_XDECREF(__pyx_v_h);
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_rowtiles);
  __Pyx_XDECREF(__pyx_v_coltiles);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_row);
  __Pyx_XDECREF(__pyx_v_j);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":730
 * 
 * 
 *     cpdef void make_ready(self, bint nearest):             # <<<<<<<<<<<<<<
 *         """
 *         Makes ready all the tile-textures in this texture grid.
 */

static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_11make_ready(PyObject *__pyx_v_self, PyObject *__pyx_arg_nearest); /*proto*/
static void __pyx_f_5renpy_2gl_9gltexture_11TextureGrid_make_ready(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, int __pyx_v_nearest, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_row = 0;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_t = 0;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("make_ready", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_make_ready); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_11make_ready)) {
      __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_nearest); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 730, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 730, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl/gltexture.pyx":738
 *         cdef TextureCore t
 * 
 *         self.ready = True             # <<<<<<<<<<<<<<
 * 
 *         for row in self.tiles:
 */
  __pyx_v_self->ready = 1;

  /* "renpy/gl/gltexture.pyx":740
 *         self.ready = True
 * 
 *         for row in self.tiles:             # <<<<<<<<<<<<<<
 *             for t in row:
 *                 t.make_ready()
 */
  if (unlikely(__pyx_v_self->tiles == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 740, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_self->tiles; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
  for (;;) {
    if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_2); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 740, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 740, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 740, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_row, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":741
 * 
 *         for row in self.tiles:
 *             for t in row:             # <<<<<<<<<<<<<<
 *                 t.make_ready()
 * 
 */
    if (unlikely(__pyx_v_row == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 741, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_v_row; __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
    for (;;) {
      if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_4); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 741, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      if (!(likely(((__pyx_t_4) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_4, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_XDECREF_SET(__pyx_v_t, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "renpy/gl/gltexture.pyx":742
 *         for row in self.tiles:
 *             for t in row:
 *                 t.make_ready()             # <<<<<<<<<<<<<<
 * 
 *                 if nearest:
 */
      ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t->__pyx_vtab)->make_ready(__pyx_v_t);

      /* "renpy/gl/gltexture.pyx":744
 *                 t.make_ready()
 * 
 *                 if nearest:             # <<<<<<<<<<<<<<
 *                     t.make_nearest()
 *                 else:
 */
      __pyx_t_9 = (__pyx_v_nearest != 0);
      if (__pyx_t_9) {

        /* "renpy/gl/gltexture.pyx":745
 * 
 *                 if nearest:
 *                     t.make_nearest()             # <<<<<<<<<<<<<<
 *                 else:
 *                     t.make_linear()
 */
        ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t->__pyx_vtab)->make_nearest(__pyx_v_t);

        /* "renpy/gl/gltexture.pyx":744
 *                 t.make_ready()
 * 
 *                 if nearest:             # <<<<<<<<<<<<<<
 *                     t.make_nearest()
 *                 else:
 */
        goto __pyx_L7;
      }

      /* "renpy/gl/gltexture.pyx":747
 *                     t.make_nearest()
 *                 else:
 *                     t.make_linear()             # <<<<<<<<<<<<<<
 * 
 * 
 */
      /*else*/ {
        ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t->__pyx_vtab)->make_linear(__pyx_v_t);
      }
      __pyx_L7:;

      /* "renpy/gl/gltexture.pyx":741
 * 
 *         for row in self.tiles:
 *             for t in row:             # <<<<<<<<<<<<<<
 *                 t.make_ready()
 * 
 */
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":740
 *         self.ready = True
 * 
 *         for row in self.tiles:             # <<<<<<<<<<<<<<
 *             for t in row:
 *                 t.make_ready()
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":730
 * 
 * 
 *     cpdef void make_ready(self, bint nearest):             # <<<<<<<<<<<<<<
 *         """
 *         Makes ready all the tile-textures in this texture grid.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("renpy.gl.gltexture.TextureGrid.make_ready", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_row);
  __Pyx_XDECREF((PyObject *)__pyx_v_t);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_11make_ready(PyObject *__pyx_v_self, PyObject *__pyx_arg_nearest); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_11TextureGrid_10make_ready[] = "\n        Makes ready all the tile-textures in this texture grid.\n        ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_11make_ready(PyObject *__pyx_v_self, PyObject *__pyx_arg_nearest) {
  int __pyx_v_nearest;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("make_ready (wrapper)", 0);
  assert(__pyx_arg_nearest); {
    __pyx_v_nearest = __Pyx_PyObject_IsTrue(__pyx_arg_nearest); if (unlikely((__pyx_v_nearest == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 730, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.make_ready", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10make_ready(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((int)__pyx_v_nearest));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10make_ready(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, int __pyx_v_nearest) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("make_ready", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_void_to_None(__pyx_f_5renpy_2gl_9gltexture_11TextureGrid_make_ready(__pyx_v_self, __pyx_v_nearest, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.make_ready", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":57
 *     cdef object __weakref__
 * 
 *     cdef public int width             # <<<<<<<<<<<<<<
 *     cdef public int height
 *     cdef list rows
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5width___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5width___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5width_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5width_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 57, __pyx_L1_error)
  __pyx_v_self->width = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":58
 * 
 *     cdef public int width
 *     cdef public int height             # <<<<<<<<<<<<<<
 *     cdef list rows
 *     cdef list columns
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_6height_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_6height_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6height___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6height___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.height.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_6height_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6height_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_6height_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 58, __pyx_L1_error)
  __pyx_v_self->height = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.height.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":62
 *     cdef list columns
 *     cdef list tiles # list of lists.
 *     cdef public TextureGrid half_cache             # <<<<<<<<<<<<<<
 * 
 *     cpdef void make_ready(self, bint nearest)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->half_cache));
  __pyx_r = ((PyObject *)__pyx_v_self->half_cache);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid))))) __PYX_ERR(1, 62, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->half_cache);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->half_cache));
  __pyx_v_self->half_cache = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.half_cache.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_4__del__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->half_cache);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->half_cache));
  __pyx_v_self->half_cache = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":66
 *     cpdef void make_ready(self, bint nearest)
 * 
 *     cdef public object debug             # <<<<<<<<<<<<<<
 *     cdef public bint ready
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->debug);
  __pyx_r = __pyx_v_self->debug;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug_4__del__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5debug_4__del__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->debug);
  __Pyx_DECREF(__pyx_v_self->debug);
  __pyx_v_self->debug = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pxd":67
 * 
 *     cdef public object debug
 *     cdef public bint ready             # <<<<<<<<<<<<<<
 * 
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5ready_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5ready_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5ready___get__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5ready___get__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->ready); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.ready.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5ready_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5ready_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5ready_2__set__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_5ready_2__set__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 67, __pyx_L1_error)
  __pyx_v_self->ready = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.ready.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.columns, self.debug, self.half_cache, self.height, self.ready, self.rows, self.tiles, self.width)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_12__reduce_cython__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_12__reduce_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.columns, self.debug, self.half_cache, self.height, self.ready, self.rows, self.tiles, self.width)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->height); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->ready); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->width); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(8); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->columns);
  __Pyx_GIVEREF(__pyx_v_self->columns);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->columns);
  __Pyx_INCREF(__pyx_v_self->debug);
  __Pyx_GIVEREF(__pyx_v_self->debug);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_self->debug);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->half_cache));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->half_cache));
  PyTuple_SET_ITEM(__pyx_t_4, 2, ((PyObject *)__pyx_v_self->half_cache));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->rows);
  __Pyx_GIVEREF(__pyx_v_self->rows);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_v_self->rows);
  __Pyx_INCREF(__pyx_v_self->tiles);
  __Pyx_GIVEREF(__pyx_v_self->tiles);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_v_self->tiles);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 7, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.columns, self.debug, self.half_cache, self.height, self.ready, self.rows, self.tiles, self.width)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":5
 *     state = (self.columns, self.debug, self.half_cache, self.height, self.ready, self.rows, self.tiles, self.width)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.columns is not None or self.debug is not None or self.half_cache is not None or self.rows is not None or self.tiles is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.columns, self.debug, self.half_cache, self.height, self.ready, self.rows, self.tiles, self.width)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self.columns is not None or self.debug is not None or self.half_cache is not None or self.rows is not None or self.tiles is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->columns != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->debug != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->half_cache) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->rows != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->tiles != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.columns is not None or self.debug is not None or self.half_cache is not None or self.rows is not None or self.tiles is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":11
 *         use_setstate = self.columns is not None or self.debug is not None or self.half_cache is not None or self.rows is not None or self.tiles is not None
 *     if use_setstate:
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_TextureGrid); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_143547127);
    __Pyx_GIVEREF(__pyx_int_143547127);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_143547127);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self.columns is not None or self.debug is not None or self.half_cache is not None or self.rows is not None or self.tiles is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, None), state
 *     else:
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_TextureGrid__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_TextureGrid); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_143547127);
    __Pyx_GIVEREF(__pyx_int_143547127);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_143547127);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.columns, self.debug, self.half_cache, self.height, self.ready, self.rows, self.tiles, self.width)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_TextureGrid__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_14__setstate_cython__(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_11TextureGrid_14__setstate_cython__(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_TextureGrid__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureGrid__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_TextureGrid, (type(self), 0x88e5af7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_TextureGrid__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.TextureGrid.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":757
 * tiling_cache = { }
 * 
 * def compute_tiling(width, max_size, min_fill_factor):             # <<<<<<<<<<<<<<
 *     """
 *     This computes a tiling for an image with the given width (or
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_15compute_tiling(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_14compute_tiling[] = "\n    This computes a tiling for an image with the given width (or\n    height). It takes a width as an argument, and returns two lists.\n\n    The first is a list of (offset, width, index) tuples, as are used\n    in TextureGrid to determine how to blit an image to the screen.\n\n    The second is a list of (offset, copy-width, total-width) tuples\n    that are used to create the tiles.\n\n    While we're thinking about this as if it's working horizontally\n    (x, width, etc), it\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_15compute_tiling = {"compute_tiling", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_15compute_tiling, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_14compute_tiling};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_15compute_tiling(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_max_size = 0;
  PyObject *__pyx_v_min_fill_factor = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compute_tiling (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_width,&__pyx_n_s_max_size,&__pyx_n_s_min_fill_factor,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compute_tiling", 1, 3, 3, 1); __PYX_ERR(0, 757, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_min_fill_factor)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compute_tiling", 1, 3, 3, 2); __PYX_ERR(0, 757, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compute_tiling") < 0)) __PYX_ERR(0, 757, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_width = values[0];
    __pyx_v_max_size = values[1];
    __pyx_v_min_fill_factor = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compute_tiling", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 757, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.compute_tiling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_14compute_tiling(__pyx_self, __pyx_v_width, __pyx_v_max_size, __pyx_v_min_fill_factor);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_14compute_tiling(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width, PyObject *__pyx_v_max_size, PyObject *__pyx_v_min_fill_factor) {
  PyObject *__pyx_v_gl_npot = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_orig_width = NULL;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_row = NULL;
  PyObject *__pyx_v_tiles = NULL;
  PyObject *__pyx_v_row_index = NULL;
  PyObject *__pyx_v_left_border = NULL;
  PyObject *__pyx_v_right_border = NULL;
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_v_row_size = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("compute_tiling", 0);
  __Pyx_INCREF(__pyx_v_width);
  __Pyx_INCREF(__pyx_v_max_size);

  /* "renpy/gl/gltexture.pyx":775
 * 
 * 
 *     gl_npot = renpy.game.preferences.gl_npot             # <<<<<<<<<<<<<<
 * 
 *     if old_gl_npot != gl_npot:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_game); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_preferences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_gl_npot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 775, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_gl_npot = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":777
 *     gl_npot = renpy.game.preferences.gl_npot
 * 
 *     if old_gl_npot != gl_npot:             # <<<<<<<<<<<<<<
 *         old_gl_npot = gl_npot
 *         tiling_cache.clear()
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_old_gl_npot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_v_gl_npot, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":778
 * 
 *     if old_gl_npot != gl_npot:
 *         old_gl_npot = gl_npot             # <<<<<<<<<<<<<<
 *         tiling_cache.clear()
 * 
 */
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_old_gl_npot, __pyx_v_gl_npot) < 0) __PYX_ERR(0, 778, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":779
 *     if old_gl_npot != gl_npot:
 *         old_gl_npot = gl_npot
 *         tiling_cache.clear()             # <<<<<<<<<<<<<<
 * 
 *     if gl_npot:
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_tiling_cache); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 779, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_clear); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 779, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_2) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 779, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 779, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":777
 *     gl_npot = renpy.game.preferences.gl_npot
 * 
 *     if old_gl_npot != gl_npot:             # <<<<<<<<<<<<<<
 *         old_gl_npot = gl_npot
 *         tiling_cache.clear()
 */
  }

  /* "renpy/gl/gltexture.pyx":781
 *         tiling_cache.clear()
 * 
 *     if gl_npot:             # <<<<<<<<<<<<<<
 *         max_size = min(SIZES[0], max_size)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_gl_npot); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 781, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":782
 * 
 *     if gl_npot:
 *         max_size = min(SIZES[0], max_size)             # <<<<<<<<<<<<<<
 * 
 *     orig_width = width
 */
    __Pyx_INCREF(__pyx_v_max_size);
    __pyx_t_1 = __pyx_v_max_size;
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 782, __pyx_L1_error)
    __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 782, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (__pyx_t_3) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_4 = __pyx_t_1;
    } else {
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_4 = __pyx_t_2;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_t_4;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_max_size, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":781
 *         tiling_cache.clear()
 * 
 *     if gl_npot:             # <<<<<<<<<<<<<<
 *         max_size = min(SIZES[0], max_size)
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":784
 *         max_size = min(SIZES[0], max_size)
 * 
 *     orig_width = width             # <<<<<<<<<<<<<<
 * 
 *     # Check the cache.
 */
  __Pyx_INCREF(__pyx_v_width);
  __pyx_v_orig_width = __pyx_v_width;

  /* "renpy/gl/gltexture.pyx":787
 * 
 *     # Check the cache.
 *     key = (width, max_size)             # <<<<<<<<<<<<<<
 *     if key in tiling_cache:
 *         return tiling_cache[key]
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 787, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_width);
  __Pyx_GIVEREF(__pyx_v_width);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_width);
  __Pyx_INCREF(__pyx_v_max_size);
  __Pyx_GIVEREF(__pyx_v_max_size);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_max_size);
  __pyx_v_key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":788
 *     # Check the cache.
 *     key = (width, max_size)
 *     if key in tiling_cache:             # <<<<<<<<<<<<<<
 *         return tiling_cache[key]
 * 
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_tiling_cache); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__Pyx_PySequence_ContainsTF(__pyx_v_key, __pyx_t_1, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 788, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_3 != 0);
  if (__pyx_t_6) {

    /* "renpy/gl/gltexture.pyx":789
 *     key = (width, max_size)
 *     if key in tiling_cache:
 *         return tiling_cache[key]             # <<<<<<<<<<<<<<
 * 
 *     # width is the remaining width, not including any borders
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_tiling_cache); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyObject_GetItem(__pyx_t_1, __pyx_v_key); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 789, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;

    /* "renpy/gl/gltexture.pyx":788
 *     # Check the cache.
 *     key = (width, max_size)
 *     if key in tiling_cache:             # <<<<<<<<<<<<<<
 *         return tiling_cache[key]
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":795
 * 
 *     # The x-offset, relative to the left side of the surface.
 *     x = 0             # <<<<<<<<<<<<<<
 * 
 *     # The list of row tuples.
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_x = __pyx_int_0;

  /* "renpy/gl/gltexture.pyx":798
 * 
 *     # The list of row tuples.
 *     row = [ ]             # <<<<<<<<<<<<<<
 * 
 *     # The list of tile tuples.
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 798, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_row = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":801
 * 
 *     # The list of tile tuples.
 *     tiles = [ ]             # <<<<<<<<<<<<<<
 * 
 *     # The index into the row.
 */
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 801, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_tiles = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":804
 * 
 *     # The index into the row.
 *     row_index = 0             # <<<<<<<<<<<<<<
 * 
 *     if gl_npot and width <= max_size:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_row_index = __pyx_int_0;

  /* "renpy/gl/gltexture.pyx":806
 *     row_index = 0
 * 
 *     if gl_npot and width <= max_size:             # <<<<<<<<<<<<<<
 *         left_border = 0
 *         right_border = 0
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_gl_npot); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 806, __pyx_L1_error)
  if (__pyx_t_3) {
  } else {
    __pyx_t_6 = __pyx_t_3;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_4 = PyObject_RichCompare(__pyx_v_width, __pyx_v_max_size, Py_LE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 806, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __pyx_t_3;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_6) {

    /* "renpy/gl/gltexture.pyx":807
 * 
 *     if gl_npot and width <= max_size:
 *         left_border = 0             # <<<<<<<<<<<<<<
 *         right_border = 0
 *     else:
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_v_left_border = __pyx_int_0;

    /* "renpy/gl/gltexture.pyx":808
 *     if gl_npot and width <= max_size:
 *         left_border = 0
 *         right_border = 0             # <<<<<<<<<<<<<<
 *     else:
 *         left_border = 1
 */
    __Pyx_INCREF(__pyx_int_0);
    __pyx_v_right_border = __pyx_int_0;

    /* "renpy/gl/gltexture.pyx":806
 *     row_index = 0
 * 
 *     if gl_npot and width <= max_size:             # <<<<<<<<<<<<<<
 *         left_border = 0
 *         right_border = 0
 */
    goto __pyx_L6;
  }

  /* "renpy/gl/gltexture.pyx":810
 *         right_border = 0
 *     else:
 *         left_border = 1             # <<<<<<<<<<<<<<
 *         right_border = 1
 * 
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_int_1);
    __pyx_v_left_border = __pyx_int_1;

    /* "renpy/gl/gltexture.pyx":811
 *     else:
 *         left_border = 1
 *         right_border = 1             # <<<<<<<<<<<<<<
 * 
 *     while width:
 */
    __Pyx_INCREF(__pyx_int_1);
    __pyx_v_right_border = __pyx_int_1;
  }
  __pyx_L6:;

  /* "renpy/gl/gltexture.pyx":813
 *         right_border = 1
 * 
 *     while width:             # <<<<<<<<<<<<<<
 * 
 *         if gl_npot:
 */
  while (1) {
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_width); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 813, __pyx_L1_error)
    if (!__pyx_t_6) break;

    /* "renpy/gl/gltexture.pyx":815
 *     while width:
 * 
 *         if gl_npot:             # <<<<<<<<<<<<<<
 * 
 *             size = min(width + left_border + right_border, max_size)
 */
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_gl_npot); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 815, __pyx_L1_error)
    if (__pyx_t_6) {

      /* "renpy/gl/gltexture.pyx":817
 *         if gl_npot:
 * 
 *             size = min(width + left_border + right_border, max_size)             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
      __Pyx_INCREF(__pyx_v_max_size);
      __pyx_t_4 = __pyx_v_max_size;
      __pyx_t_1 = PyNumber_Add(__pyx_v_width, __pyx_v_left_border); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 817, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_v_right_border); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 817, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 817, __pyx_L1_error)
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 817, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (__pyx_t_6) {
        __Pyx_INCREF(__pyx_t_4);
        __pyx_t_1 = __pyx_t_4;
      } else {
        __Pyx_INCREF(__pyx_t_2);
        __pyx_t_1 = __pyx_t_2;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __pyx_t_1;
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_size, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "renpy/gl/gltexture.pyx":815
 *     while width:
 * 
 *         if gl_npot:             # <<<<<<<<<<<<<<
 * 
 *             size = min(width + left_border + right_border, max_size)
 */
      goto __pyx_L11;
    }

    /* "renpy/gl/gltexture.pyx":822
 * 
 *             # Figure out the texture size to use.
 *             for size in SIZES:             # <<<<<<<<<<<<<<
 *                 if size > max_size:
 *                     continue
 */
    /*else*/ {
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 822, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (likely(PyList_CheckExact(__pyx_t_4)) || PyTuple_CheckExact(__pyx_t_4)) {
        __pyx_t_1 = __pyx_t_4; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
        __pyx_t_8 = NULL;
      } else {
        __pyx_t_7 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 822, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 822, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      for (;;) {
        if (likely(!__pyx_t_8)) {
          if (likely(PyList_CheckExact(__pyx_t_1))) {
            if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 822, __pyx_L1_error)
            #else
            __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 822, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            #endif
          } else {
            if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_4); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 822, __pyx_L1_error)
            #else
            __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 822, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            #endif
          }
        } else {
          __pyx_t_4 = __pyx_t_8(__pyx_t_1);
          if (unlikely(!__pyx_t_4)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 822, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_XDECREF_SET(__pyx_v_size, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "renpy/gl/gltexture.pyx":823
 *             # Figure out the texture size to use.
 *             for size in SIZES:
 *                 if size > max_size:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
        __pyx_t_4 = PyObject_RichCompare(__pyx_v_size, __pyx_v_max_size, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 823, __pyx_L1_error)
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 823, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (__pyx_t_6) {

          /* "renpy/gl/gltexture.pyx":824
 *             for size in SIZES:
 *                 if size > max_size:
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 # Ensure each texture is full enough.
 */
          goto __pyx_L12_continue;

          /* "renpy/gl/gltexture.pyx":823
 *             # Figure out the texture size to use.
 *             for size in SIZES:
 *                 if size > max_size:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
        }

        /* "renpy/gl/gltexture.pyx":827
 * 
 *                 # Ensure each texture is full enough.
 *                 if size * min_fill_factor <= width + left_border + right_border:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        __pyx_t_4 = PyNumber_Multiply(__pyx_v_size, __pyx_v_min_fill_factor); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 827, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = PyNumber_Add(__pyx_v_width, __pyx_v_left_border); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 827, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_5 = PyNumber_Add(__pyx_t_2, __pyx_v_right_border); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 827, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = PyObject_RichCompare(__pyx_t_4, __pyx_t_5, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 827, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 827, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (__pyx_t_6) {

          /* "renpy/gl/gltexture.pyx":828
 *                 # Ensure each texture is full enough.
 *                 if size * min_fill_factor <= width + left_border + right_border:
 *                     break             # <<<<<<<<<<<<<<
 * 
 *         # The number of pixels to display to the user from this tile.
 */
          goto __pyx_L13_break;

          /* "renpy/gl/gltexture.pyx":827
 * 
 *                 # Ensure each texture is full enough.
 *                 if size * min_fill_factor <= width + left_border + right_border:             # <<<<<<<<<<<<<<
 *                     break
 * 
 */
        }

        /* "renpy/gl/gltexture.pyx":822
 * 
 *             # Figure out the texture size to use.
 *             for size in SIZES:             # <<<<<<<<<<<<<<
 *                 if size > max_size:
 *                     continue
 */
        __pyx_L12_continue:;
      }
      __pyx_L13_break:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L11:;

    /* "renpy/gl/gltexture.pyx":831
 * 
 *         # The number of pixels to display to the user from this tile.
 *         row_size = min(width, size - left_border - right_border)             # <<<<<<<<<<<<<<
 * 
 *         #Add to the results.
 */
    if (unlikely(!__pyx_v_size)) { __Pyx_RaiseUnboundLocalError("size"); __PYX_ERR(0, 831, __pyx_L1_error) }
    __pyx_t_1 = PyNumber_Subtract(__pyx_v_size, __pyx_v_left_border); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Subtract(__pyx_t_1, __pyx_v_right_border); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v_width);
    __pyx_t_1 = __pyx_v_width;
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 831, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 831, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_6) {
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_5 = __pyx_t_2;
    } else {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = __pyx_t_1;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __pyx_t_5;
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v_row_size, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":834
 * 
 *         #Add to the results.
 *         row.append((left_border, row_size, row_index))             # <<<<<<<<<<<<<<
 *         tiles.append((x - left_border, row_size + left_border + right_border, size))
 * 
 */
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_left_border);
    __Pyx_GIVEREF(__pyx_v_left_border);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_left_border);
    __Pyx_INCREF(__pyx_v_row_size);
    __Pyx_GIVEREF(__pyx_v_row_size);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_row_size);
    __Pyx_INCREF(__pyx_v_row_index);
    __Pyx_GIVEREF(__pyx_v_row_index);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_row_index);
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_row, __pyx_t_2); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 834, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":835
 *         #Add to the results.
 *         row.append((left_border, row_size, row_index))
 *         tiles.append((x - left_border, row_size + left_border + right_border, size))             # <<<<<<<<<<<<<<
 * 
 *         # Update the counters.
 */
    __pyx_t_2 = PyNumber_Subtract(__pyx_v_x, __pyx_v_left_border); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyNumber_Add(__pyx_v_row_size, __pyx_v_left_border); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyNumber_Add(__pyx_t_5, __pyx_v_right_border); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_v_size)) { __Pyx_RaiseUnboundLocalError("size"); __PYX_ERR(0, 835, __pyx_L1_error) }
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_size);
    __pyx_t_2 = 0;
    __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_tiles, __pyx_t_5); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 835, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":838
 * 
 *         # Update the counters.
 *         row_index += 1             # <<<<<<<<<<<<<<
 *         x += row_size
 *         width -= row_size
 */
    __pyx_t_5 = __Pyx_PyInt_AddObjC(__pyx_v_row_index, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 838, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_row_index, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":839
 *         # Update the counters.
 *         row_index += 1
 *         x += row_size             # <<<<<<<<<<<<<<
 *         width -= row_size
 * 
 */
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_x, __pyx_v_row_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 839, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":840
 *         row_index += 1
 *         x += row_size
 *         width -= row_size             # <<<<<<<<<<<<<<
 * 
 *     tiling_cache[key] = (row, tiles)
 */
    __pyx_t_5 = PyNumber_InPlaceSubtract(__pyx_v_width, __pyx_v_row_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 840, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_width, __pyx_t_5);
    __pyx_t_5 = 0;
  }

  /* "renpy/gl/gltexture.pyx":842
 *         width -= row_size
 * 
 *     tiling_cache[key] = (row, tiles)             # <<<<<<<<<<<<<<
 * 
 *     return row, tiles
 */
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_row);
  __Pyx_GIVEREF(__pyx_v_row);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_row);
  __Pyx_INCREF(__pyx_v_tiles);
  __Pyx_GIVEREF(__pyx_v_tiles);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_tiles);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_tiling_cache); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_key, __pyx_t_5) < 0)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":844
 *     tiling_cache[key] = (row, tiles)
 * 
 *     return row, tiles             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_row);
  __Pyx_GIVEREF(__pyx_v_row);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_row);
  __Pyx_INCREF(__pyx_v_tiles);
  __Pyx_GIVEREF(__pyx_v_tiles);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_tiles);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":757
 * tiling_cache = { }
 * 
 * def compute_tiling(width, max_size, min_fill_factor):             # <<<<<<<<<<<<<<
 *     """
 *     This computes a tiling for an image with the given width (or
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("renpy.gl.gltexture.compute_tiling", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_gl_npot);
  __Pyx_XDECREF(__pyx_v_orig_width);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_row);
  __Pyx_XDECREF(__pyx_v_tiles);
  __Pyx_XDECREF(__pyx_v_row_index);
  __Pyx_XDECREF(__pyx_v_left_border);
  __Pyx_XDECREF(__pyx_v_right_border);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XDECREF(__pyx_v_row_size);
  __Pyx_XDECREF(__pyx_v_width);
  __Pyx_XDECREF(__pyx_v_max_size);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":847
 * 
 * 
 * def texture_grid_from_surface(surf, transient):             # <<<<<<<<<<<<<<
 *     """
 *     This takes a Surface and turns it into a TextureGrid.
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_17texture_grid_from_surface(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_16texture_grid_from_surface[] = "\n    This takes a Surface and turns it into a TextureGrid.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_17texture_grid_from_surface = {"texture_grid_from_surface", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_17texture_grid_from_surface, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_16texture_grid_from_surface};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_17texture_grid_from_surface(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_surf = 0;
  PyObject *__pyx_v_transient = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("texture_grid_from_surface (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_surf,&__pyx_n_s_transient,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_surf)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transient)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_grid_from_surface", 1, 2, 2, 1); __PYX_ERR(0, 847, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "texture_grid_from_surface") < 0)) __PYX_ERR(0, 847, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_surf = values[0];
    __pyx_v_transient = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("texture_grid_from_surface", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 847, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.texture_grid_from_surface", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_16texture_grid_from_surface(__pyx_self, __pyx_v_surf, __pyx_v_transient);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_16texture_grid_from_surface(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_surf, PyObject *__pyx_v_transient) {
  PyObject *__pyx_v_max_size = NULL;
  double __pyx_v_fill_factor;
  PyObject *__pyx_v_width = NULL;
  PyObject *__pyx_v_height = NULL;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_rv = NULL;
  PyObject *__pyx_v_texcolumns = NULL;
  PyObject *__pyx_v_texrows = NULL;
  PyObject *__pyx_v_rv_row = NULL;
  PyObject *__pyx_v_texrow = NULL;
  PyObject *__pyx_v_border_top = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_border_bottom = NULL;
  PyObject *__pyx_v_y = NULL;
  PyObject *__pyx_v_texheight = NULL;
  PyObject *__pyx_v_row = NULL;
  CYTHON_UNUSED long __pyx_v_colnum;
  PyObject *__pyx_v_rv_col = NULL;
  PyObject *__pyx_v_texcol = NULL;
  PyObject *__pyx_v_border_left = NULL;
  PyObject *__pyx_v_border_right = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_texwidth = NULL;
  PyObject *__pyx_v_tex = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  __Pyx_RefNannySetupContext("texture_grid_from_surface", 0);

  /* "renpy/gl/gltexture.pyx":852
 *     """
 * 
 *     if renpy.game.preferences.gl_npot:             # <<<<<<<<<<<<<<
 *         max_size = SIZES[0]
 *         fill_factor = 0.5
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_game); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_preferences); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_gl_npot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 852, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":853
 * 
 *     if renpy.game.preferences.gl_npot:
 *         max_size = SIZES[0]             # <<<<<<<<<<<<<<
 *         fill_factor = 0.5
 * 
 */
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_max_size = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":854
 *     if renpy.game.preferences.gl_npot:
 *         max_size = SIZES[0]
 *         fill_factor = 0.5             # <<<<<<<<<<<<<<
 * 
 *     elif transient:
 */
    __pyx_v_fill_factor = 0.5;

    /* "renpy/gl/gltexture.pyx":852
 *     """
 * 
 *     if renpy.game.preferences.gl_npot:             # <<<<<<<<<<<<<<
 *         max_size = SIZES[0]
 *         fill_factor = 0.5
 */
    goto __pyx_L3;
  }

  /* "renpy/gl/gltexture.pyx":856
 *         fill_factor = 0.5
 * 
 *     elif transient:             # <<<<<<<<<<<<<<
 *         max_size = SIZES[0]
 *         fill_factor = 0.5
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_transient); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 856, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":857
 * 
 *     elif transient:
 *         max_size = SIZES[0]             # <<<<<<<<<<<<<<
 *         fill_factor = 0.5
 *     else:
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_SIZES); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 857, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_max_size = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":858
 *     elif transient:
 *         max_size = SIZES[0]
 *         fill_factor = 0.5             # <<<<<<<<<<<<<<
 *     else:
 *         max_size = MAX_SIZE
 */
    __pyx_v_fill_factor = 0.5;

    /* "renpy/gl/gltexture.pyx":856
 *         fill_factor = 0.5
 * 
 *     elif transient:             # <<<<<<<<<<<<<<
 *         max_size = SIZES[0]
 *         fill_factor = 0.5
 */
    goto __pyx_L3;
  }

  /* "renpy/gl/gltexture.pyx":860
 *         fill_factor = 0.5
 *     else:
 *         max_size = MAX_SIZE             # <<<<<<<<<<<<<<
 *         fill_factor = .66
 * 
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_MAX_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 860, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_max_size = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":861
 *     else:
 *         max_size = MAX_SIZE
 *         fill_factor = .66             # <<<<<<<<<<<<<<
 * 
 *     width, height = surf.get_size()
 */
    __pyx_v_fill_factor = .66;
  }
  __pyx_L3:;

  /* "renpy/gl/gltexture.pyx":863
 *         fill_factor = .66
 * 
 *     width, height = surf.get_size()             # <<<<<<<<<<<<<<
 * 
 *     rv = TextureGrid(width, height)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_surf, __pyx_n_s_get_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 863, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 863, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 863, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_4);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_5 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_5)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_6(__pyx_t_5); if (unlikely(!__pyx_t_1)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_4 = __pyx_t_6(__pyx_t_5); if (unlikely(!__pyx_t_4)) goto __pyx_L4_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_5), 2) < 0) __PYX_ERR(0, 863, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L5_unpacking_done;
    __pyx_L4_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 863, __pyx_L1_error)
    __pyx_L5_unpacking_done:;
  }
  __pyx_v_width = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_height = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":865
 *     width, height = surf.get_size()
 * 
 *     rv = TextureGrid(width, height)             # <<<<<<<<<<<<<<
 * 
 *     rv.columns, texcolumns = compute_tiling(width, max_size, fill_factor)
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_width);
  __Pyx_GIVEREF(__pyx_v_width);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_width);
  __Pyx_INCREF(__pyx_v_height);
  __Pyx_GIVEREF(__pyx_v_height);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_height);
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureGrid), __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 865, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_rv = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":867
 *     rv = TextureGrid(width, height)
 * 
 *     rv.columns, texcolumns = compute_tiling(width, max_size, fill_factor)             # <<<<<<<<<<<<<<
 *     rv.rows, texrows = compute_tiling(height, max_size, fill_factor)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_compute_tiling); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_fill_factor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_width, __pyx_v_max_size, __pyx_t_1};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_width, __pyx_v_max_size, __pyx_t_1};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_width);
    __Pyx_GIVEREF(__pyx_v_width);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_width);
    __Pyx_INCREF(__pyx_v_max_size);
    __Pyx_GIVEREF(__pyx_v_max_size);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_max_size);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
    PyObject* sequence = __pyx_t_4;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 867, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_8 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_8);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    #endif
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_1 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_2)) goto __pyx_L6_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_8 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_8)) goto __pyx_L6_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_8);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_1), 2) < 0) __PYX_ERR(0, 867, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L7_unpacking_done;
    __pyx_L6_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 867, __pyx_L1_error)
    __pyx_L7_unpacking_done:;
  }
  if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 867, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_rv->columns);
  __Pyx_DECREF(__pyx_v_rv->columns);
  __pyx_v_rv->columns = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_texcolumns = __pyx_t_8;
  __pyx_t_8 = 0;

  /* "renpy/gl/gltexture.pyx":868
 * 
 *     rv.columns, texcolumns = compute_tiling(width, max_size, fill_factor)
 *     rv.rows, texrows = compute_tiling(height, max_size, fill_factor)             # <<<<<<<<<<<<<<
 * 
 *     for rv_row, texrow in zip(rv.rows, texrows):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_compute_tiling); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_fill_factor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_height, __pyx_v_max_size, __pyx_t_2};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_height, __pyx_v_max_size, __pyx_t_2};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_height);
    __Pyx_GIVEREF(__pyx_v_height);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_7, __pyx_v_height);
    __Pyx_INCREF(__pyx_v_max_size);
    __Pyx_GIVEREF(__pyx_v_max_size);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_7, __pyx_v_max_size);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_7, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
    PyObject* sequence = __pyx_t_4;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 868, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 868, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext;
    index = 0; __pyx_t_8 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_8)) goto __pyx_L8_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_8);
    index = 1; __pyx_t_5 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_5)) goto __pyx_L8_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_2), 2) < 0) __PYX_ERR(0, 868, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L9_unpacking_done;
    __pyx_L8_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 868, __pyx_L1_error)
    __pyx_L9_unpacking_done:;
  }
  if (!(likely(PyList_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_8)->tp_name), 0))) __PYX_ERR(0, 868, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_8);
  __Pyx_GOTREF(__pyx_v_rv->rows);
  __Pyx_DECREF(__pyx_v_rv->rows);
  __pyx_v_rv->rows = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_v_texrows = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":870
 *     rv.rows, texrows = compute_tiling(height, max_size, fill_factor)
 * 
 *     for rv_row, texrow in zip(rv.rows, texrows):             # <<<<<<<<<<<<<<
 *         border_top, _, border_bottom = rv_row
 *         y, height, texheight = texrow
 */
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_rv->rows);
  __Pyx_GIVEREF(__pyx_v_rv->rows);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_rv->rows);
  __Pyx_INCREF(__pyx_v_texrows);
  __Pyx_GIVEREF(__pyx_v_texrows);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_texrows);
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 870, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
    __pyx_t_4 = __pyx_t_5; __Pyx_INCREF(__pyx_t_4); __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 870, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 870, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_9); __Pyx_INCREF(__pyx_t_5); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 870, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_9); __Pyx_INCREF(__pyx_t_5); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 870, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_4, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_10(__pyx_t_4);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 870, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 870, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      #else
      __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_1 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 870, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext;
      index = 0; __pyx_t_8 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_8)) goto __pyx_L12_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      index = 1; __pyx_t_2 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_2)) goto __pyx_L12_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_1), 2) < 0) __PYX_ERR(0, 870, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L13_unpacking_done;
      __pyx_L12_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 870, __pyx_L1_error)
      __pyx_L13_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_rv_row, __pyx_t_8);
    __pyx_t_8 = 0;
    __Pyx_XDECREF_SET(__pyx_v_texrow, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":871
 * 
 *     for rv_row, texrow in zip(rv.rows, texrows):
 *         border_top, _, border_bottom = rv_row             # <<<<<<<<<<<<<<
 *         y, height, texheight = texrow
 * 
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_rv_row))) || (PyList_CheckExact(__pyx_v_rv_row))) {
      PyObject* sequence = __pyx_v_rv_row;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 871, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_8 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_8);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 871, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 871, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 871, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_1 = PyObject_GetIter(__pyx_v_rv_row); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 871, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_5)) goto __pyx_L14_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_2 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_2)) goto __pyx_L14_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 2; __pyx_t_8 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_8)) goto __pyx_L14_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_1), 3) < 0) __PYX_ERR(0, 871, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L15_unpacking_done;
      __pyx_L14_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 871, __pyx_L1_error)
      __pyx_L15_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_border_top, __pyx_t_5);
    __pyx_t_5 = 0;
    __Pyx_XDECREF_SET(__pyx_v__, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_border_bottom, __pyx_t_8);
    __pyx_t_8 = 0;

    /* "renpy/gl/gltexture.pyx":872
 *     for rv_row, texrow in zip(rv.rows, texrows):
 *         border_top, _, border_bottom = rv_row
 *         y, height, texheight = texrow             # <<<<<<<<<<<<<<
 * 
 *         row = [ ]
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_texrow))) || (PyList_CheckExact(__pyx_v_texrow))) {
      PyObject* sequence = __pyx_v_texrow;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 872, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_5 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      #else
      __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_1 = PyObject_GetIter(__pyx_v_texrow); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 872, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_6 = Py_TYPE(__pyx_t_1)->tp_iternext;
      index = 0; __pyx_t_8 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_8)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      index = 1; __pyx_t_2 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_2)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 2; __pyx_t_5 = __pyx_t_6(__pyx_t_1); if (unlikely(!__pyx_t_5)) goto __pyx_L16_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_1), 3) < 0) __PYX_ERR(0, 872, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L17_unpacking_done;
      __pyx_L16_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 872, __pyx_L1_error)
      __pyx_L17_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_y, __pyx_t_8);
    __pyx_t_8 = 0;
    __Pyx_DECREF_SET(__pyx_v_height, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_texheight, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":874
 *         y, height, texheight = texrow
 * 
 *         row = [ ]             # <<<<<<<<<<<<<<
 * 
 *         colnum = 0
 */
    __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 874, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF_SET(__pyx_v_row, ((PyObject*)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":876
 *         row = [ ]
 * 
 *         colnum = 0             # <<<<<<<<<<<<<<
 * 
 *         for rv_col, texcol in zip(rv.columns, texcolumns):
 */
    __pyx_v_colnum = 0;

    /* "renpy/gl/gltexture.pyx":878
 *         colnum = 0
 * 
 *         for rv_col, texcol in zip(rv.columns, texcolumns):             # <<<<<<<<<<<<<<
 *             border_left, _, border_right = rv_col
 *             x, width, texwidth = texcol
 */
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 878, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_rv->columns);
    __Pyx_GIVEREF(__pyx_v_rv->columns);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_rv->columns);
    __Pyx_INCREF(__pyx_v_texcolumns);
    __Pyx_GIVEREF(__pyx_v_texcolumns);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_texcolumns);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 878, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_5 = __pyx_t_2; __Pyx_INCREF(__pyx_t_5); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_11 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 878, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_12 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 878, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_11); __Pyx_INCREF(__pyx_t_2); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 878, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_5, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 878, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_11); __Pyx_INCREF(__pyx_t_2); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 878, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_5, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 878, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_12(__pyx_t_5);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 878, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 878, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_1);
        #else
        __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_13 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 878, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_6 = Py_TYPE(__pyx_t_13)->tp_iternext;
        index = 0; __pyx_t_8 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_8)) goto __pyx_L20_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        index = 1; __pyx_t_1 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_1)) goto __pyx_L20_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_13), 2) < 0) __PYX_ERR(0, 878, __pyx_L1_error)
        __pyx_t_6 = NULL;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L21_unpacking_done;
        __pyx_L20_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_6 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 878, __pyx_L1_error)
        __pyx_L21_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_rv_col, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_XDECREF_SET(__pyx_v_texcol, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":879
 * 
 *         for rv_col, texcol in zip(rv.columns, texcolumns):
 *             border_left, _, border_right = rv_col             # <<<<<<<<<<<<<<
 *             x, width, texwidth = texcol
 * 
 */
      if ((likely(PyTuple_CheckExact(__pyx_v_rv_col))) || (PyList_CheckExact(__pyx_v_rv_col))) {
        PyObject* sequence = __pyx_v_rv_col;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 879, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_8 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_8);
        #else
        __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 879, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 879, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_8 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 879, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      } else {
        Py_ssize_t index = -1;
        __pyx_t_13 = PyObject_GetIter(__pyx_v_rv_col); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 879, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_6 = Py_TYPE(__pyx_t_13)->tp_iternext;
        index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_2)) goto __pyx_L22_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_2);
        index = 1; __pyx_t_1 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_1)) goto __pyx_L22_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 2; __pyx_t_8 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_8)) goto __pyx_L22_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_13), 3) < 0) __PYX_ERR(0, 879, __pyx_L1_error)
        __pyx_t_6 = NULL;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L23_unpacking_done;
        __pyx_L22_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_6 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 879, __pyx_L1_error)
        __pyx_L23_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_border_left, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v__, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_border_right, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "renpy/gl/gltexture.pyx":880
 *         for rv_col, texcol in zip(rv.columns, texcolumns):
 *             border_left, _, border_right = rv_col
 *             x, width, texwidth = texcol             # <<<<<<<<<<<<<<
 * 
 *             tex = alloc_texture(texwidth, texheight)
 */
      if ((likely(PyTuple_CheckExact(__pyx_v_texcol))) || (PyList_CheckExact(__pyx_v_texcol))) {
        PyObject* sequence = __pyx_v_texcol;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 880, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_1 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_1 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_2 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        #else
        __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_1 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        Py_ssize_t index = -1;
        __pyx_t_13 = PyObject_GetIter(__pyx_v_texcol); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 880, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_6 = Py_TYPE(__pyx_t_13)->tp_iternext;
        index = 0; __pyx_t_8 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_8)) goto __pyx_L24_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_8);
        index = 1; __pyx_t_1 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_1)) goto __pyx_L24_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_1);
        index = 2; __pyx_t_2 = __pyx_t_6(__pyx_t_13); if (unlikely(!__pyx_t_2)) goto __pyx_L24_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_2);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_13), 3) < 0) __PYX_ERR(0, 880, __pyx_L1_error)
        __pyx_t_6 = NULL;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L25_unpacking_done;
        __pyx_L24_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_6 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 880, __pyx_L1_error)
        __pyx_L25_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_x, __pyx_t_8);
      __pyx_t_8 = 0;
      __Pyx_DECREF_SET(__pyx_v_width, __pyx_t_1);
      __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_texwidth, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":882
 *             x, width, texwidth = texcol
 * 
 *             tex = alloc_texture(texwidth, texheight)             # <<<<<<<<<<<<<<
 *             tex.load_surface(surf, x, y, width, height,
 *                              border_left, border_top, border_right, border_bottom)
 */
      __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_alloc_texture); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 882, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_v_texwidth, __pyx_v_texheight};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_v_texwidth, __pyx_v_texheight};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_13 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        if (__pyx_t_8) {
          __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_8); __pyx_t_8 = NULL;
        }
        __Pyx_INCREF(__pyx_v_texwidth);
        __Pyx_GIVEREF(__pyx_v_texwidth);
        PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_7, __pyx_v_texwidth);
        __Pyx_INCREF(__pyx_v_texheight);
        __Pyx_GIVEREF(__pyx_v_texheight);
        PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_7, __pyx_v_texheight);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_13, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 882, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF_SET(__pyx_v_tex, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":883
 * 
 *             tex = alloc_texture(texwidth, texheight)
 *             tex.load_surface(surf, x, y, width, height,             # <<<<<<<<<<<<<<
 *                              border_left, border_top, border_right, border_bottom)
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_tex, __pyx_n_s_load_surface); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 883, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "renpy/gl/gltexture.pyx":884
 *             tex = alloc_texture(texwidth, texheight)
 *             tex.load_surface(surf, x, y, width, height,
 *                              border_left, border_top, border_right, border_bottom)             # <<<<<<<<<<<<<<
 * 
 *             row.append(tex)
 */
      __pyx_t_13 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_13)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_13);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[10] = {__pyx_t_13, __pyx_v_surf, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height, __pyx_v_border_left, __pyx_v_border_top, __pyx_v_border_right, __pyx_v_border_bottom};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_7, 9+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[10] = {__pyx_t_13, __pyx_v_surf, __pyx_v_x, __pyx_v_y, __pyx_v_width, __pyx_v_height, __pyx_v_border_left, __pyx_v_border_top, __pyx_v_border_right, __pyx_v_border_bottom};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_7, 9+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(9+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 883, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__pyx_t_13) {
          __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_13); __pyx_t_13 = NULL;
        }
        __Pyx_INCREF(__pyx_v_surf);
        __Pyx_GIVEREF(__pyx_v_surf);
        PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_surf);
        __Pyx_INCREF(__pyx_v_x);
        __Pyx_GIVEREF(__pyx_v_x);
        PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_x);
        __Pyx_INCREF(__pyx_v_y);
        __Pyx_GIVEREF(__pyx_v_y);
        PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_v_y);
        __Pyx_INCREF(__pyx_v_width);
        __Pyx_GIVEREF(__pyx_v_width);
        PyTuple_SET_ITEM(__pyx_t_8, 3+__pyx_t_7, __pyx_v_width);
        __Pyx_INCREF(__pyx_v_height);
        __Pyx_GIVEREF(__pyx_v_height);
        PyTuple_SET_ITEM(__pyx_t_8, 4+__pyx_t_7, __pyx_v_height);
        __Pyx_INCREF(__pyx_v_border_left);
        __Pyx_GIVEREF(__pyx_v_border_left);
        PyTuple_SET_ITEM(__pyx_t_8, 5+__pyx_t_7, __pyx_v_border_left);
        __Pyx_INCREF(__pyx_v_border_top);
        __Pyx_GIVEREF(__pyx_v_border_top);
        PyTuple_SET_ITEM(__pyx_t_8, 6+__pyx_t_7, __pyx_v_border_top);
        __Pyx_INCREF(__pyx_v_border_right);
        __Pyx_GIVEREF(__pyx_v_border_right);
        PyTuple_SET_ITEM(__pyx_t_8, 7+__pyx_t_7, __pyx_v_border_right);
        __Pyx_INCREF(__pyx_v_border_bottom);
        __Pyx_GIVEREF(__pyx_v_border_bottom);
        PyTuple_SET_ITEM(__pyx_t_8, 8+__pyx_t_7, __pyx_v_border_bottom);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 883, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":886
 *                              border_left, border_top, border_right, border_bottom)
 * 
 *             row.append(tex)             # <<<<<<<<<<<<<<
 * 
 *         rv.tiles.append(row)
 */
      __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_row, __pyx_v_tex); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 886, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":878
 *         colnum = 0
 * 
 *         for rv_col, texcol in zip(rv.columns, texcolumns):             # <<<<<<<<<<<<<<
 *             border_left, _, border_right = rv_col
 *             x, width, texwidth = texcol
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":888
 *             row.append(tex)
 * 
 *         rv.tiles.append(row)             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
    if (unlikely(__pyx_v_rv->tiles == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 888, __pyx_L1_error)
    }
    __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_rv->tiles, __pyx_v_row); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 888, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":870
 *     rv.rows, texrows = compute_tiling(height, max_size, fill_factor)
 * 
 *     for rv_row, texrow in zip(rv.rows, texrows):             # <<<<<<<<<<<<<<
 *         border_top, _, border_bottom = rv_row
 *         y, height, texheight = texrow
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "renpy/gl/gltexture.pyx":890
 *         rv.tiles.append(row)
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = ((PyObject *)__pyx_v_rv);
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":847
 * 
 * 
 * def texture_grid_from_surface(surf, transient):             # <<<<<<<<<<<<<<
 *     """
 *     This takes a Surface and turns it into a TextureGrid.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("renpy.gl.gltexture.texture_grid_from_surface", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_max_size);
  __Pyx_XDECREF(__pyx_v_width);
  __Pyx_XDECREF(__pyx_v_height);
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_texcolumns);
  __Pyx_XDECREF(__pyx_v_texrows);
  __Pyx_XDECREF(__pyx_v_rv_row);
  __Pyx_XDECREF(__pyx_v_texrow);
  __Pyx_XDECREF(__pyx_v_border_top);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_border_bottom);
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XDECREF(__pyx_v_texheight);
  __Pyx_XDECREF(__pyx_v_row);
  __Pyx_XDECREF(__pyx_v_rv_col);
  __Pyx_XDECREF(__pyx_v_texcol);
  __Pyx_XDECREF(__pyx_v_border_left);
  __Pyx_XDECREF(__pyx_v_border_right);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_texwidth);
  __Pyx_XDECREF(__pyx_v_tex);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":893
 * 
 * 
 * def texture_grid_from_drawing(width, height, draw_func, rtt, environ):             # <<<<<<<<<<<<<<
 *     """
 *     This creates a texture grid of `width` by `height` by using
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_19texture_grid_from_drawing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_18texture_grid_from_drawing[] = "\n    This creates a texture grid of `width` by `height` by using\n    draw_func to draw to the screen.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_19texture_grid_from_drawing = {"texture_grid_from_drawing", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_19texture_grid_from_drawing, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_18texture_grid_from_drawing};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_19texture_grid_from_drawing(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_draw_func = 0;
  PyObject *__pyx_v_rtt = 0;
  PyObject *__pyx_v_environ = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("texture_grid_from_drawing (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_draw_func,&__pyx_n_s_rtt,&__pyx_n_s_environ,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_grid_from_drawing", 1, 5, 5, 1); __PYX_ERR(0, 893, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_draw_func)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_grid_from_drawing", 1, 5, 5, 2); __PYX_ERR(0, 893, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_rtt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_grid_from_drawing", 1, 5, 5, 3); __PYX_ERR(0, 893, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_environ)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("texture_grid_from_drawing", 1, 5, 5, 4); __PYX_ERR(0, 893, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "texture_grid_from_drawing") < 0)) __PYX_ERR(0, 893, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_width = values[0];
    __pyx_v_height = values[1];
    __pyx_v_draw_func = values[2];
    __pyx_v_rtt = values[3];
    __pyx_v_environ = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("texture_grid_from_drawing", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 893, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.texture_grid_from_drawing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_18texture_grid_from_drawing(__pyx_self, __pyx_v_width, __pyx_v_height, __pyx_v_draw_func, __pyx_v_rtt, __pyx_v_environ);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_18texture_grid_from_drawing(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_draw_func, PyObject *__pyx_v_rtt, PyObject *__pyx_v_environ) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_rv = NULL;
  PyObject *__pyx_v_gldraw = NULL;
  PyObject *__pyx_v_pwidth = NULL;
  PyObject *__pyx_v_pheight = NULL;
  PyObject *__pyx_v_texcolumns = NULL;
  PyObject *__pyx_v_texrows = NULL;
  PyObject *__pyx_v_y = NULL;
  PyObject *__pyx_v_texheight = NULL;
  PyObject *__pyx_v_row = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_texwidth = NULL;
  PyObject *__pyx_v_tex = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  Py_ssize_t __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  __Pyx_RefNannySetupContext("texture_grid_from_drawing", 0);
  __Pyx_INCREF(__pyx_v_width);
  __Pyx_INCREF(__pyx_v_height);

  /* "renpy/gl/gltexture.pyx":899
 *     """
 * 
 *     rv = TextureGrid(width, height)             # <<<<<<<<<<<<<<
 * 
 *     gldraw = renpy.display.draw
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_width);
  __Pyx_GIVEREF(__pyx_v_width);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_width);
  __Pyx_INCREF(__pyx_v_height);
  __Pyx_GIVEREF(__pyx_v_height);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_height);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureGrid), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_rv = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":901
 *     rv = TextureGrid(width, height)
 * 
 *     gldraw = renpy.display.draw             # <<<<<<<<<<<<<<
 *     pwidth, pheight = gldraw.physical_size
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_renpy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_display); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_draw); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_gldraw = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":902
 * 
 *     gldraw = renpy.display.draw
 *     pwidth, pheight = gldraw.physical_size             # <<<<<<<<<<<<<<
 * 
 *     rv.columns, texcolumns = compute_tiling(width, rtt.get_size_limit(pwidth), 0.5)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_gldraw, __pyx_n_s_physical_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 902, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 902, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 902, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 902, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < 0) __PYX_ERR(0, 902, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 902, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_pwidth = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_pheight = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":904
 *     pwidth, pheight = gldraw.physical_size
 * 
 *     rv.columns, texcolumns = compute_tiling(width, rtt.get_size_limit(pwidth), 0.5)             # <<<<<<<<<<<<<<
 *     rv.rows, texrows = compute_tiling(height, rtt.get_size_limit(pheight), 0.5)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_compute_tiling); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_rtt, __pyx_n_s_get_size_limit); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_pwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_pwidth};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v_pwidth};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 904, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v_pwidth);
      __Pyx_GIVEREF(__pyx_v_pwidth);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_pwidth);
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_width, __pyx_t_1, __pyx_float_0_5};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_v_width, __pyx_t_1, __pyx_float_0_5};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_width);
    __Pyx_GIVEREF(__pyx_v_width);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_8, __pyx_v_width);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_8, __pyx_t_1);
    __Pyx_INCREF(__pyx_float_0_5);
    __Pyx_GIVEREF(__pyx_float_0_5);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_8, __pyx_float_0_5);
    __pyx_t_1 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 904, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_7);
    #else
    __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 904, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_1)->tp_iternext;
    index = 0; __pyx_t_3 = __pyx_t_5(__pyx_t_1); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 1; __pyx_t_7 = __pyx_t_5(__pyx_t_1); if (unlikely(!__pyx_t_7)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_1), 2) < 0) __PYX_ERR(0, 904, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 904, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 904, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_rv->columns);
  __Pyx_DECREF(__pyx_v_rv->columns);
  __pyx_v_rv->columns = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_v_texcolumns = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "renpy/gl/gltexture.pyx":905
 * 
 *     rv.columns, texcolumns = compute_tiling(width, rtt.get_size_limit(pwidth), 0.5)
 *     rv.rows, texrows = compute_tiling(height, rtt.get_size_limit(pheight), 0.5)             # <<<<<<<<<<<<<<
 * 
 *     for y, height, texheight in texrows:
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_compute_tiling); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_rtt, __pyx_n_s_get_size_limit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 905, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_pheight); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_pheight};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 905, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_pheight};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 905, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 905, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_pheight);
      __Pyx_GIVEREF(__pyx_v_pheight);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_v_pheight);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 905, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_7)) {
    PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_height, __pyx_t_3, __pyx_float_0_5};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
    PyObject *__pyx_temp[4] = {__pyx_t_1, __pyx_v_height, __pyx_t_3, __pyx_float_0_5};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_1) {
      __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1); __pyx_t_1 = NULL;
    }
    __Pyx_INCREF(__pyx_v_height);
    __Pyx_GIVEREF(__pyx_v_height);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_8, __pyx_v_height);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_8, __pyx_t_3);
    __Pyx_INCREF(__pyx_float_0_5);
    __Pyx_GIVEREF(__pyx_float_0_5);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_8, __pyx_float_0_5);
    __pyx_t_3 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
    PyObject* sequence = __pyx_t_2;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 905, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_6);
    #else
    __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    #endif
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 905, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_7 = __pyx_t_5(__pyx_t_3); if (unlikely(!__pyx_t_7)) goto __pyx_L7_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_7);
    index = 1; __pyx_t_6 = __pyx_t_5(__pyx_t_3); if (unlikely(!__pyx_t_6)) goto __pyx_L7_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_3), 2) < 0) __PYX_ERR(0, 905, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L8_unpacking_done;
    __pyx_L7_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 905, __pyx_L1_error)
    __pyx_L8_unpacking_done:;
  }
  if (!(likely(PyList_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(0, 905, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_7);
  __Pyx_GOTREF(__pyx_v_rv->rows);
  __Pyx_DECREF(__pyx_v_rv->rows);
  __pyx_v_rv->rows = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;
  __pyx_v_texrows = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "renpy/gl/gltexture.pyx":907
 *     rv.rows, texrows = compute_tiling(height, rtt.get_size_limit(pheight), 0.5)
 * 
 *     for y, height, texheight in texrows:             # <<<<<<<<<<<<<<
 *         row = [ ]
 * 
 */
  if (likely(PyList_CheckExact(__pyx_v_texrows)) || PyTuple_CheckExact(__pyx_v_texrows)) {
    __pyx_t_2 = __pyx_v_texrows; __Pyx_INCREF(__pyx_t_2); __pyx_t_9 = 0;
    __pyx_t_10 = NULL;
  } else {
    __pyx_t_9 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_texrows); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 907, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 907, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_10)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_6); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 907, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 907, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      } else {
        if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_9); __Pyx_INCREF(__pyx_t_6); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 907, __pyx_L1_error)
        #else
        __pyx_t_6 = PySequence_ITEM(__pyx_t_2, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 907, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
      }
    } else {
      __pyx_t_6 = __pyx_t_10(__pyx_t_2);
      if (unlikely(!__pyx_t_6)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 907, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_6);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
      PyObject* sequence = __pyx_t_6;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 907, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_7 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_1 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      #else
      __pyx_t_7 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 907, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 907, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_4 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 907, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
      index = 0; __pyx_t_7 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_7)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_7);
      index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 2; __pyx_t_1 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_1)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_1);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 3) < 0) __PYX_ERR(0, 907, __pyx_L1_error)
      __pyx_t_5 = NULL;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L12_unpacking_done;
      __pyx_L11_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_5 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 907, __pyx_L1_error)
      __pyx_L12_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_y, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_DECREF_SET(__pyx_v_height, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_texheight, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "renpy/gl/gltexture.pyx":908
 * 
 *     for y, height, texheight in texrows:
 *         row = [ ]             # <<<<<<<<<<<<<<
 * 
 *         for x, width, texwidth in texcolumns:
 */
    __pyx_t_6 = PyList_New(0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_XDECREF_SET(__pyx_v_row, ((PyObject*)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "renpy/gl/gltexture.pyx":910
 *         row = [ ]
 * 
 *         for x, width, texwidth in texcolumns:             # <<<<<<<<<<<<<<
 * 
 *             tex = alloc_texture(texwidth, texheight)
 */
    if (likely(PyList_CheckExact(__pyx_v_texcolumns)) || PyTuple_CheckExact(__pyx_v_texcolumns)) {
      __pyx_t_6 = __pyx_v_texcolumns; __Pyx_INCREF(__pyx_t_6); __pyx_t_11 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_11 = -1; __pyx_t_6 = PyObject_GetIter(__pyx_v_texcolumns); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 910, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_12 = Py_TYPE(__pyx_t_6)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 910, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_6))) {
          if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_6)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_11); __Pyx_INCREF(__pyx_t_1); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 910, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_6, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_11 >= PyTuple_GET_SIZE(__pyx_t_6)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_6, __pyx_t_11); __Pyx_INCREF(__pyx_t_1); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 910, __pyx_L1_error)
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_6, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_12(__pyx_t_6);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 910, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 910, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_4 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_4);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 910, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 910, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_4 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 910, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_13 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 910, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_5 = Py_TYPE(__pyx_t_13)->tp_iternext;
        index = 0; __pyx_t_3 = __pyx_t_5(__pyx_t_13); if (unlikely(!__pyx_t_3)) goto __pyx_L15_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_7 = __pyx_t_5(__pyx_t_13); if (unlikely(!__pyx_t_7)) goto __pyx_L15_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        index = 2; __pyx_t_4 = __pyx_t_5(__pyx_t_13); if (unlikely(!__pyx_t_4)) goto __pyx_L15_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_13), 3) < 0) __PYX_ERR(0, 910, __pyx_L1_error)
        __pyx_t_5 = NULL;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L16_unpacking_done;
        __pyx_L15_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_5 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 910, __pyx_L1_error)
        __pyx_L16_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_x, __pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_width, __pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XDECREF_SET(__pyx_v_texwidth, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "renpy/gl/gltexture.pyx":912
 *         for x, width, texwidth in texcolumns:
 * 
 *             tex = alloc_texture(texwidth, texheight)             # <<<<<<<<<<<<<<
 *             tex.render_to(x, y, draw_func, rtt, environ)
 * 
 */
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_alloc_texture); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 912, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_texwidth, __pyx_v_texheight};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_texwidth, __pyx_v_texheight};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_3 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 912, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_INCREF(__pyx_v_texwidth);
        __Pyx_GIVEREF(__pyx_v_texwidth);
        PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_8, __pyx_v_texwidth);
        __Pyx_INCREF(__pyx_v_texheight);
        __Pyx_GIVEREF(__pyx_v_texheight);
        PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_8, __pyx_v_texheight);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 912, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF_SET(__pyx_v_tex, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":913
 * 
 *             tex = alloc_texture(texwidth, texheight)
 *             tex.render_to(x, y, draw_func, rtt, environ)             # <<<<<<<<<<<<<<
 * 
 *             row.append(tex)
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_tex, __pyx_n_s_render_to); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 913, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_8 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[6] = {__pyx_t_3, __pyx_v_x, __pyx_v_y, __pyx_v_draw_func, __pyx_v_rtt, __pyx_v_environ};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 5+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 913, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[6] = {__pyx_t_3, __pyx_v_x, __pyx_v_y, __pyx_v_draw_func, __pyx_v_rtt, __pyx_v_environ};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_8, 5+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 913, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(5+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 913, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (__pyx_t_3) {
          __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
        }
        __Pyx_INCREF(__pyx_v_x);
        __Pyx_GIVEREF(__pyx_v_x);
        PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_8, __pyx_v_x);
        __Pyx_INCREF(__pyx_v_y);
        __Pyx_GIVEREF(__pyx_v_y);
        PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_8, __pyx_v_y);
        __Pyx_INCREF(__pyx_v_draw_func);
        __Pyx_GIVEREF(__pyx_v_draw_func);
        PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_8, __pyx_v_draw_func);
        __Pyx_INCREF(__pyx_v_rtt);
        __Pyx_GIVEREF(__pyx_v_rtt);
        PyTuple_SET_ITEM(__pyx_t_7, 3+__pyx_t_8, __pyx_v_rtt);
        __Pyx_INCREF(__pyx_v_environ);
        __Pyx_GIVEREF(__pyx_v_environ);
        PyTuple_SET_ITEM(__pyx_t_7, 4+__pyx_t_8, __pyx_v_environ);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 913, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "renpy/gl/gltexture.pyx":915
 *             tex.render_to(x, y, draw_func, rtt, environ)
 * 
 *             row.append(tex)             # <<<<<<<<<<<<<<
 * 
 *         rv.tiles.append(row)
 */
      __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_row, __pyx_v_tex); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 915, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":910
 *         row = [ ]
 * 
 *         for x, width, texwidth in texcolumns:             # <<<<<<<<<<<<<<
 * 
 *             tex = alloc_texture(texwidth, texheight)
 */
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "renpy/gl/gltexture.pyx":917
 *             row.append(tex)
 * 
 *         rv.tiles.append(row)             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
    if (unlikely(__pyx_v_rv->tiles == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
      __PYX_ERR(0, 917, __pyx_L1_error)
    }
    __pyx_t_14 = __Pyx_PyList_Append(__pyx_v_rv->tiles, __pyx_v_row); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 917, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":907
 *     rv.rows, texrows = compute_tiling(height, rtt.get_size_limit(pheight), 0.5)
 * 
 *     for y, height, texheight in texrows:             # <<<<<<<<<<<<<<
 *         row = [ ]
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":919
 *         rv.tiles.append(row)
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_rv));
  __pyx_r = ((PyObject *)__pyx_v_rv);
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":893
 * 
 * 
 * def texture_grid_from_drawing(width, height, draw_func, rtt, environ):             # <<<<<<<<<<<<<<
 *     """
 *     This creates a texture grid of `width` by `height` by using
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("renpy.gl.gltexture.texture_grid_from_drawing", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_gldraw);
  __Pyx_XDECREF(__pyx_v_pwidth);
  __Pyx_XDECREF(__pyx_v_pheight);
  __Pyx_XDECREF(__pyx_v_texcolumns);
  __Pyx_XDECREF(__pyx_v_texrows);
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XDECREF(__pyx_v_texheight);
  __Pyx_XDECREF(__pyx_v_row);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_texwidth);
  __Pyx_XDECREF(__pyx_v_tex);
  __Pyx_XDECREF(__pyx_v_width);
  __Pyx_XDECREF(__pyx_v_height);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":922
 * 
 * 
 * def align_axes(*args):             # <<<<<<<<<<<<<<
 *     """
 *     This takes n axes, each a list consisting of (offset, size, index)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_21align_axes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_20align_axes[] = "\n    This takes n axes, each a list consisting of (offset, size, index)\n    tuples. It returns a list of n lists. For each of the n lists, the\n    ith element will be the same size. The indexes will appear in the\n    same order they did in the original list.\n\n    This is used to combine the grids of two or more texgrids into a\n    single grid.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_21align_axes = {"align_axes", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_21align_axes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_20align_axes};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_21align_axes(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("align_axes (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "align_axes", 0))) return NULL;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_20align_axes(__pyx_self, __pyx_v_args);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_20align_axes(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_args) {
  PyObject *__pyx_v_rv = NULL;
  PyObject *__pyx_v_i = NULL;
  PyObject *__pyx_v_cur = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_v_index = NULL;
  Py_ssize_t __pyx_v_nargs;
  int __pyx_v_loop;
  PyObject *__pyx_v_minsize = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  __Pyx_RefNannySetupContext("align_axes", 0);

  /* "renpy/gl/gltexture.pyx":934
 * 
 *     # The lists we're building.
 *     rv = [ [] for i in args ]             # <<<<<<<<<<<<<<
 * 
 *     # If we have an empty list, do nothing.
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 934, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_args; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 934, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 934, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_rv = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":937
 * 
 *     # If we have an empty list, do nothing.
 *     for i in args:             # <<<<<<<<<<<<<<
 *         if not i:
 *             return rv
 */
  __pyx_t_1 = __pyx_v_args; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 937, __pyx_L1_error)
    #else
    __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 937, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":938
 *     # If we have an empty list, do nothing.
 *     for i in args:
 *         if not i:             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_i); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 938, __pyx_L1_error)
    __pyx_t_6 = ((!__pyx_t_5) != 0);
    if (__pyx_t_6) {

      /* "renpy/gl/gltexture.pyx":939
 *     for i in args:
 *         if not i:
 *             return rv             # <<<<<<<<<<<<<<
 * 
 *     # The index of the current element, for each member of args.
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_rv);
      __pyx_r = __pyx_v_rv;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;

      /* "renpy/gl/gltexture.pyx":938
 *     # If we have an empty list, do nothing.
 *     for i in args:
 *         if not i:             # <<<<<<<<<<<<<<
 *             return rv
 * 
 */
    }

    /* "renpy/gl/gltexture.pyx":937
 * 
 *     # If we have an empty list, do nothing.
 *     for i in args:             # <<<<<<<<<<<<<<
 *         if not i:
 *             return rv
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":942
 * 
 *     # The index of the current element, for each member of args.
 *     cur = [ 0 for i in args ]             # <<<<<<<<<<<<<<
 * 
 *     # The parts of the current element.
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 942, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_args; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 942, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 942, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_int_0))) __PYX_ERR(0, 942, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_cur = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":945
 * 
 *     # The parts of the current element.
 *     offset = [ i[0][0] for i in args ]             # <<<<<<<<<<<<<<
 *     size = [ i[0][1] for i in args ]
 *     index = [ i[0][2] for i in args ]
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 945, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_args; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 945, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_i, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 945, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_offset = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":946
 *     # The parts of the current element.
 *     offset = [ i[0][0] for i in args ]
 *     size = [ i[0][1] for i in args ]             # <<<<<<<<<<<<<<
 *     index = [ i[0][2] for i in args ]
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 946, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_args; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_7); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 946, __pyx_L1_error)
    #else
    __pyx_t_7 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 946, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_v_i, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 946, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_7, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 946, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 946, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_size = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":947
 *     offset = [ i[0][0] for i in args ]
 *     size = [ i[0][1] for i in args ]
 *     index = [ i[0][2] for i in args ]             # <<<<<<<<<<<<<<
 * 
 *     # Cache it here to save function calls.
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 947, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_args; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_4); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 947, __pyx_L1_error)
    #else
    __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_i, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = __Pyx_GetItemInt(__pyx_t_4, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 947, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 947, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_index = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":950
 * 
 *     # Cache it here to save function calls.
 *     nargs = len(args)             # <<<<<<<<<<<<<<
 * 
 *     loop = True
 */
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v_args); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 950, __pyx_L1_error)
  __pyx_v_nargs = __pyx_t_3;

  /* "renpy/gl/gltexture.pyx":952
 *     nargs = len(args)
 * 
 *     loop = True             # <<<<<<<<<<<<<<
 * 
 *     while loop:
 */
  __pyx_v_loop = 1;

  /* "renpy/gl/gltexture.pyx":954
 *     loop = True
 * 
 *     while loop:             # <<<<<<<<<<<<<<
 * 
 *         # Figure out the minimum size.
 */
  while (1) {
    __pyx_t_6 = (__pyx_v_loop != 0);
    if (!__pyx_t_6) break;

    /* "renpy/gl/gltexture.pyx":957
 * 
 *         # Figure out the minimum size.
 *         minsize = min(size)             # <<<<<<<<<<<<<<
 * 
 *         for i in xrange(nargs):
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 957, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_size);
    __Pyx_GIVEREF(__pyx_v_size);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_size);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_min, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 957, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_minsize, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "renpy/gl/gltexture.pyx":959
 *         minsize = min(size)
 * 
 *         for i in xrange(nargs):             # <<<<<<<<<<<<<<
 * 
 *             # Create an entry of minsize.
 */
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_nargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_xrange, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 959, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_1 = __pyx_t_2; __Pyx_INCREF(__pyx_t_1); __pyx_t_3 = 0;
      __pyx_t_8 = NULL;
    } else {
      __pyx_t_3 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 959, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 959, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_8)) {
        if (likely(PyList_CheckExact(__pyx_t_1))) {
          if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 959, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 959, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 959, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_1, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 959, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_8(__pyx_t_1);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 959, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":962
 * 
 *             # Create an entry of minsize.
 *             rv[i].append((offset[i], minsize, index[i]))             # <<<<<<<<<<<<<<
 * 
 *             # Adjust the offset and size.
 */
      __pyx_t_2 = PyObject_GetItem(__pyx_v_rv, __pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = PyObject_GetItem(__pyx_v_offset, __pyx_v_i); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 962, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_4 = PyObject_GetItem(__pyx_v_index, __pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 962, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 962, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7);
      __Pyx_INCREF(__pyx_v_minsize);
      __Pyx_GIVEREF(__pyx_v_minsize);
      PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_minsize);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_4);
      __pyx_t_7 = 0;
      __pyx_t_4 = 0;
      __pyx_t_10 = __Pyx_PyObject_Append(__pyx_t_2, __pyx_t_9); if (unlikely(__pyx_t_10 == ((int)-1))) __PYX_ERR(0, 962, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "renpy/gl/gltexture.pyx":965
 * 
 *             # Adjust the offset and size.
 *             offset[i] += minsize             # <<<<<<<<<<<<<<
 *             size[i] -= minsize
 * 
 */
      __Pyx_INCREF(__pyx_v_i);
      __pyx_t_9 = __pyx_v_i;
      __pyx_t_2 = PyObject_GetItem(__pyx_v_offset, __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 965, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_t_2, __pyx_v_minsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 965, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(PyObject_SetItem(__pyx_v_offset, __pyx_t_9, __pyx_t_4) < 0)) __PYX_ERR(0, 965, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "renpy/gl/gltexture.pyx":966
 *             # Adjust the offset and size.
 *             offset[i] += minsize
 *             size[i] -= minsize             # <<<<<<<<<<<<<<
 * 
 *             # If the size fell to 0, then load the next element from
 */
      __Pyx_INCREF(__pyx_v_i);
      __pyx_t_9 = __pyx_v_i;
      __pyx_t_4 = PyObject_GetItem(__pyx_v_size, __pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 966, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = PyNumber_InPlaceSubtract(__pyx_t_4, __pyx_v_minsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 966, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(PyObject_SetItem(__pyx_v_size, __pyx_t_9, __pyx_t_2) < 0)) __PYX_ERR(0, 966, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "renpy/gl/gltexture.pyx":971
 *             # the arguments into offset, size, index. If we can't
 *             # we're done.
 *             if size[i] == 0:             # <<<<<<<<<<<<<<
 *                 cur[i] += 1
 * 
 */
      __pyx_t_9 = PyObject_GetItem(__pyx_v_size, __pyx_v_i); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 971, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_2 = __Pyx_PyInt_EqObjC(__pyx_t_9, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 971, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 971, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_6) {

        /* "renpy/gl/gltexture.pyx":972
 *             # we're done.
 *             if size[i] == 0:
 *                 cur[i] += 1             # <<<<<<<<<<<<<<
 * 
 *                 if cur[i] >= len(args[i]):
 */
        __Pyx_INCREF(__pyx_v_i);
        __pyx_t_2 = __pyx_v_i;
        __pyx_t_9 = PyObject_GetItem(__pyx_v_cur, __pyx_t_2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 972, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_4 = __Pyx_PyInt_AddObjC(__pyx_t_9, __pyx_int_1, 1, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 972, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (unlikely(PyObject_SetItem(__pyx_v_cur, __pyx_t_2, __pyx_t_4) < 0)) __PYX_ERR(0, 972, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "renpy/gl/gltexture.pyx":974
 *                 cur[i] += 1
 * 
 *                 if cur[i] >= len(args[i]):             # <<<<<<<<<<<<<<
 *                     loop = False
 *                 else:
 */
        __pyx_t_2 = PyObject_GetItem(__pyx_v_cur, __pyx_v_i); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 974, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = PyObject_GetItem(__pyx_v_args, __pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 974, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_11 = PyObject_Length(__pyx_t_4); if (unlikely(__pyx_t_11 == ((Py_ssize_t)-1))) __PYX_ERR(0, 974, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyInt_FromSsize_t(__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 974, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_GE); __Pyx_XGOTREF(__pyx_t_9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 974, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 974, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        if (__pyx_t_6) {

          /* "renpy/gl/gltexture.pyx":975
 * 
 *                 if cur[i] >= len(args[i]):
 *                     loop = False             # <<<<<<<<<<<<<<
 *                 else:
 *                     offset[i], size[i], index[i] = args[i][cur[i]]
 */
          __pyx_v_loop = 0;

          /* "renpy/gl/gltexture.pyx":974
 *                 cur[i] += 1
 * 
 *                 if cur[i] >= len(args[i]):             # <<<<<<<<<<<<<<
 *                     loop = False
 *                 else:
 */
          goto __pyx_L21;
        }

        /* "renpy/gl/gltexture.pyx":977
 *                     loop = False
 *                 else:
 *                     offset[i], size[i], index[i] = args[i][cur[i]]             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
        /*else*/ {
          __pyx_t_9 = PyObject_GetItem(__pyx_v_args, __pyx_v_i); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 977, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_4 = PyObject_GetItem(__pyx_v_cur, __pyx_v_i); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 977, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_2 = PyObject_GetItem(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 977, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
            PyObject* sequence = __pyx_t_2;
            #if !CYTHON_COMPILING_IN_PYPY
            Py_ssize_t size = Py_SIZE(sequence);
            #else
            Py_ssize_t size = PySequence_Size(sequence);
            #endif
            if (unlikely(size != 3)) {
              if (size > 3) __Pyx_RaiseTooManyValuesError(3);
              else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
              __PYX_ERR(0, 977, __pyx_L1_error)
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            if (likely(PyTuple_CheckExact(sequence))) {
              __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
              __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
              __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
            } else {
              __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
              __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
              __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
            }
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(__pyx_t_7);
            #else
            __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 977, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 977, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __pyx_t_7 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 977, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);
            #endif
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          } else {
            Py_ssize_t index = -1;
            __pyx_t_12 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 977, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
            index = 0; __pyx_t_4 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_4)) goto __pyx_L22_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_4);
            index = 1; __pyx_t_9 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L22_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_9);
            index = 2; __pyx_t_7 = __pyx_t_13(__pyx_t_12); if (unlikely(!__pyx_t_7)) goto __pyx_L22_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_7);
            if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 3) < 0) __PYX_ERR(0, 977, __pyx_L1_error)
            __pyx_t_13 = NULL;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            goto __pyx_L23_unpacking_done;
            __pyx_L22_unpacking_failed:;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_t_13 = NULL;
            if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
            __PYX_ERR(0, 977, __pyx_L1_error)
            __pyx_L23_unpacking_done:;
          }
          if (unlikely(PyObject_SetItem(__pyx_v_offset, __pyx_v_i, __pyx_t_4) < 0)) __PYX_ERR(0, 977, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(PyObject_SetItem(__pyx_v_size, __pyx_v_i, __pyx_t_9) < 0)) __PYX_ERR(0, 977, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(PyObject_SetItem(__pyx_v_index, __pyx_v_i, __pyx_t_7) < 0)) __PYX_ERR(0, 977, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_L21:;

        /* "renpy/gl/gltexture.pyx":971
 *             # the arguments into offset, size, index. If we can't
 *             # we're done.
 *             if size[i] == 0:             # <<<<<<<<<<<<<<
 *                 cur[i] += 1
 * 
 */
      }

      /* "renpy/gl/gltexture.pyx":959
 *         minsize = min(size)
 * 
 *         for i in xrange(nargs):             # <<<<<<<<<<<<<<
 * 
 *             # Create an entry of minsize.
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "renpy/gl/gltexture.pyx":979
 *                     offset[i], size[i], index[i] = args[i][cur[i]]
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":922
 * 
 * 
 * def align_axes(*args):             # <<<<<<<<<<<<<<
 *     """
 *     This takes n axes, each a list consisting of (offset, size, index)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("renpy.gl.gltexture.align_axes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF(__pyx_v_cur);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XDECREF(__pyx_v_minsize);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":982
 * 
 * 
 * cpdef blit(TextureGrid tg, double sx, double sy, Matrix transform, double alpha, double over, Environ environ, bint nearest):             # <<<<<<<<<<<<<<
 *     """
 *     This draws texgrid `tg` to the screen. `sx` and `sy` are offsets from
 */

static PyObject *__pyx_pw_5renpy_2gl_9gltexture_23blit(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture_blit(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_v_texx;
  int __pyx_v_texy;
  int __pyx_v_texw;
  int __pyx_v_texh;
  PyObject *__pyx_v_rowindex = NULL;
  PyObject *__pyx_v_colindex = NULL;
  PyObject *__pyx_v_tex = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *(*__pyx_t_8)(PyObject *);
  int __pyx_t_9;
  int __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("blit", 0);

  /* "renpy/gl/gltexture.pyx":998
 *     cdef int texx, texy, texw, texh
 * 
 *     tg.make_ready(nearest)             # <<<<<<<<<<<<<<
 * 
 *     environ.blit()
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_tg->__pyx_vtab)->make_ready(__pyx_v_tg, __pyx_v_nearest, 0);

  /* "renpy/gl/gltexture.pyx":1000
 *     tg.make_ready(nearest)
 * 
 *     environ.blit()             # <<<<<<<<<<<<<<
 *     environ.set_color(alpha, alpha, alpha, over * alpha)
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->blit(__pyx_v_environ);

  /* "renpy/gl/gltexture.pyx":1001
 * 
 *     environ.blit()
 *     environ.set_color(alpha, alpha, alpha, over * alpha)             # <<<<<<<<<<<<<<
 * 
 *     y = 0
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_color(__pyx_v_environ, __pyx_v_alpha, __pyx_v_alpha, __pyx_v_alpha, (__pyx_v_over * __pyx_v_alpha));

  /* "renpy/gl/gltexture.pyx":1003
 *     environ.set_color(alpha, alpha, alpha, over * alpha)
 * 
 *     y = 0             # <<<<<<<<<<<<<<
 * 
 *     for texy, texh, rowindex in tg.rows:
 */
  __pyx_v_y = 0;

  /* "renpy/gl/gltexture.pyx":1005
 *     y = 0
 * 
 *     for texy, texh, rowindex in tg.rows:             # <<<<<<<<<<<<<<
 *         x = 0
 * 
 */
  if (unlikely(__pyx_v_tg->rows == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 1005, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_v_tg->rows; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 1005, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1005, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_4 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1005, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1005, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1005, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_7 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1005, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = Py_TYPE(__pyx_t_7)->tp_iternext;
      index = 0; __pyx_t_4 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      index = 1; __pyx_t_5 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 2; __pyx_t_6 = __pyx_t_8(__pyx_t_7); if (unlikely(!__pyx_t_6)) goto __pyx_L5_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_7), 3) < 0) __PYX_ERR(0, 1005, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L6_unpacking_done;
      __pyx_L5_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1005, __pyx_L1_error)
      __pyx_L6_unpacking_done:;
    }
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1005, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_texy = __pyx_t_9;
    __pyx_v_texh = __pyx_t_10;
    __Pyx_XDECREF_SET(__pyx_v_rowindex, __pyx_t_6);
    __pyx_t_6 = 0;

    /* "renpy/gl/gltexture.pyx":1006
 * 
 *     for texy, texh, rowindex in tg.rows:
 *         x = 0             # <<<<<<<<<<<<<<
 * 
 *         for texx, texw, colindex in tg.columns:
 */
    __pyx_v_x = 0;

    /* "renpy/gl/gltexture.pyx":1008
 *         x = 0
 * 
 *         for texx, texw, colindex in tg.columns:             # <<<<<<<<<<<<<<
 * 
 *             tex = tg.tiles[rowindex][colindex]
 */
    if (unlikely(__pyx_v_tg->columns == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 1008, __pyx_L1_error)
    }
    __pyx_t_3 = __pyx_v_tg->columns; __Pyx_INCREF(__pyx_t_3); __pyx_t_11 = 0;
    for (;;) {
      if (__pyx_t_11 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_11); __Pyx_INCREF(__pyx_t_6); __pyx_t_11++; if (unlikely(0 < 0)) __PYX_ERR(0, 1008, __pyx_L1_error)
      #else
      __pyx_t_6 = PySequence_ITEM(__pyx_t_3, __pyx_t_11); __pyx_t_11++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
        PyObject* sequence = __pyx_t_6;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1008, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_4 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_7 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        #else
        __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_7 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_12 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1008, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_8 = Py_TYPE(__pyx_t_12)->tp_iternext;
        index = 0; __pyx_t_5 = __pyx_t_8(__pyx_t_12); if (unlikely(!__pyx_t_5)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        index = 1; __pyx_t_4 = __pyx_t_8(__pyx_t_12); if (unlikely(!__pyx_t_4)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_4);
        index = 2; __pyx_t_7 = __pyx_t_8(__pyx_t_12); if (unlikely(!__pyx_t_7)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_12), 3) < 0) __PYX_ERR(0, 1008, __pyx_L1_error)
        __pyx_t_8 = NULL;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_8 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1008, __pyx_L1_error)
        __pyx_L10_unpacking_done:;
      }
      __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_texx = __pyx_t_10;
      __pyx_v_texw = __pyx_t_9;
      __Pyx_XDECREF_SET(__pyx_v_colindex, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "renpy/gl/gltexture.pyx":1010
 *         for texx, texw, colindex in tg.columns:
 * 
 *             tex = tg.tiles[rowindex][colindex]             # <<<<<<<<<<<<<<
 * 
 *             draw_rectangle(
 */
      if (unlikely(__pyx_v_tg->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1010, __pyx_L1_error)
      }
      __pyx_t_6 = PyObject_GetItem(__pyx_v_tg->tiles, __pyx_v_rowindex); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1010, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = PyObject_GetItem(__pyx_t_6, __pyx_v_colindex); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1010, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF_SET(__pyx_v_tex, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "renpy/gl/gltexture.pyx":1018
 *                 texw, texh,
 *                 transform,
 *                 tex, texx, texy,             # <<<<<<<<<<<<<<
 *                 None, 0, 0,
 *                 None, 0, 0,
 */
      if (!(likely(((__pyx_v_tex) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_tex, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 1018, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1012
 *             tex = tg.tiles[rowindex][colindex]
 * 
 *             draw_rectangle(             # <<<<<<<<<<<<<<
 *                 environ,
 *                 sx, sy,
 */
      __pyx_f_5renpy_2gl_9gltexture_draw_rectangle(__pyx_v_environ, __pyx_v_sx, __pyx_v_sy, __pyx_v_x, __pyx_v_y, __pyx_v_texw, __pyx_v_texh, __pyx_v_transform, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_tex), __pyx_v_texx, __pyx_v_texy, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)Py_None), 0.0, 0.0, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)Py_None), 0.0, 0.0);

      /* "renpy/gl/gltexture.pyx":1023
 *                 )
 * 
 *             x += texw             # <<<<<<<<<<<<<<
 * 
 *         y += texh
 */
      __pyx_v_x = (__pyx_v_x + __pyx_v_texw);

      /* "renpy/gl/gltexture.pyx":1008
 *         x = 0
 * 
 *         for texx, texw, colindex in tg.columns:             # <<<<<<<<<<<<<<
 * 
 *             tex = tg.tiles[rowindex][colindex]
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":1025
 *             x += texw
 * 
 *         y += texh             # <<<<<<<<<<<<<<
 * 
 * cpdef blend(TextureGrid tg0, TextureGrid tg1, double sx, double sy, Matrix transform, double alpha, double over, double fraction, Environ environ, bint nearest):
 */
    __pyx_v_y = (__pyx_v_y + __pyx_v_texh);

    /* "renpy/gl/gltexture.pyx":1005
 *     y = 0
 * 
 *     for texy, texh, rowindex in tg.rows:             # <<<<<<<<<<<<<<
 *         x = 0
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":982
 * 
 * 
 * cpdef blit(TextureGrid tg, double sx, double sy, Matrix transform, double alpha, double over, Environ environ, bint nearest):             # <<<<<<<<<<<<<<
 *     """
 *     This draws texgrid `tg` to the screen. `sx` and `sy` are offsets from
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("renpy.gl.gltexture.blit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rowindex);
  __Pyx_XDECREF(__pyx_v_colindex);
  __Pyx_XDECREF(__pyx_v_tex);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_23blit(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_22blit[] = "\n    This draws texgrid `tg` to the screen. `sx` and `sy` are offsets from\n    the upper-left corner of the screen.\n\n    `transform` is the transform to apply to the texgrid, when going from\n    texgrid coordinates to screen coordinates.\n\n    `alpha` is the alpha multiplier applied, from 0.0 to 1.0.\n\n    `over` is the over blending factor.\n    ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_23blit(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg = 0;
  double __pyx_v_sx;
  double __pyx_v_sy;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform = 0;
  double __pyx_v_alpha;
  double __pyx_v_over;
  struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ = 0;
  int __pyx_v_nearest;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("blit (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_tg,&__pyx_n_s_sx,&__pyx_n_s_sy,&__pyx_n_s_transform,&__pyx_n_s_alpha,&__pyx_n_s_over,&__pyx_n_s_environ,&__pyx_n_s_nearest,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tg)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 1); __PYX_ERR(0, 982, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 2); __PYX_ERR(0, 982, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transform)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 3); __PYX_ERR(0, 982, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 4); __PYX_ERR(0, 982, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_over)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 5); __PYX_ERR(0, 982, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_environ)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 6); __PYX_ERR(0, 982, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nearest)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, 7); __PYX_ERR(0, 982, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "blit") < 0)) __PYX_ERR(0, 982, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_tg = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)values[0]);
    __pyx_v_sx = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_sx == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 982, __pyx_L3_error)
    __pyx_v_sy = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_sy == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 982, __pyx_L3_error)
    __pyx_v_transform = ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)values[3]);
    __pyx_v_alpha = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_alpha == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 982, __pyx_L3_error)
    __pyx_v_over = __pyx_PyFloat_AsDouble(values[5]); if (unlikely((__pyx_v_over == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 982, __pyx_L3_error)
    __pyx_v_environ = ((struct __pyx_obj_5renpy_2gl_6gldraw_Environ *)values[6]);
    __pyx_v_nearest = __Pyx_PyObject_IsTrue(values[7]); if (unlikely((__pyx_v_nearest == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 982, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("blit", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 982, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.blit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tg), __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid, 1, "tg", 0))) __PYX_ERR(0, 982, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transform), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "transform", 0))) __PYX_ERR(0, 982, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_environ), __pyx_ptype_5renpy_2gl_6gldraw_Environ, 1, "environ", 0))) __PYX_ERR(0, 982, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_22blit(__pyx_self, __pyx_v_tg, __pyx_v_sx, __pyx_v_sy, __pyx_v_transform, __pyx_v_alpha, __pyx_v_over, __pyx_v_environ, __pyx_v_nearest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_22blit(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("blit", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5renpy_2gl_9gltexture_blit(__pyx_v_tg, __pyx_v_sx, __pyx_v_sy, __pyx_v_transform, __pyx_v_alpha, __pyx_v_over, __pyx_v_environ, __pyx_v_nearest, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 982, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.blit", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":1027
 *         y += texh
 * 
 * cpdef blend(TextureGrid tg0, TextureGrid tg1, double sx, double sy, Matrix transform, double alpha, double over, double fraction, Environ environ, bint nearest):             # <<<<<<<<<<<<<<
 *     """
 *     Blends two textures to the screen.
 */

static PyObject *__pyx_pw_5renpy_2gl_9gltexture_25blend(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture_blend(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, double __pyx_v_fraction, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_y = NULL;
  PyObject *__pyx_v_rows0 = NULL;
  PyObject *__pyx_v_rows1 = NULL;
  PyObject *__pyx_v_cols0 = NULL;
  PyObject *__pyx_v_cols1 = NULL;
  PyObject *__pyx_v_t0y = NULL;
  PyObject *__pyx_v_t0h = NULL;
  PyObject *__pyx_v_t0ri = NULL;
  PyObject *__pyx_v_t1y = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t1h = NULL;
  PyObject *__pyx_v_t1ri = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_t0x = NULL;
  PyObject *__pyx_v_t0w = NULL;
  PyObject *__pyx_v_t0ci = NULL;
  PyObject *__pyx_v_t1x = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t1w = NULL;
  PyObject *__pyx_v_t1ci = NULL;
  PyObject *__pyx_v_t0 = NULL;
  PyObject *__pyx_v_t1 = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *(*__pyx_t_6)(PyObject *);
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  Py_ssize_t __pyx_t_13;
  PyObject *(*__pyx_t_14)(PyObject *);
  PyObject *__pyx_t_15 = NULL;
  double __pyx_t_16;
  double __pyx_t_17;
  double __pyx_t_18;
  double __pyx_t_19;
  float __pyx_t_20;
  float __pyx_t_21;
  float __pyx_t_22;
  float __pyx_t_23;
  __Pyx_RefNannySetupContext("blend", 0);

  /* "renpy/gl/gltexture.pyx":1045
 *     """
 * 
 *     tg0.make_ready(nearest)             # <<<<<<<<<<<<<<
 *     tg1.make_ready(nearest)
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_tg0->__pyx_vtab)->make_ready(__pyx_v_tg0, __pyx_v_nearest, 0);

  /* "renpy/gl/gltexture.pyx":1046
 * 
 *     tg0.make_ready(nearest)
 *     tg1.make_ready(nearest)             # <<<<<<<<<<<<<<
 * 
 *     environ.blend(fraction)
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_tg1->__pyx_vtab)->make_ready(__pyx_v_tg1, __pyx_v_nearest, 0);

  /* "renpy/gl/gltexture.pyx":1048
 *     tg1.make_ready(nearest)
 * 
 *     environ.blend(fraction)             # <<<<<<<<<<<<<<
 *     environ.set_color(alpha, alpha, alpha, over * alpha)
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->blend(__pyx_v_environ, __pyx_v_fraction);

  /* "renpy/gl/gltexture.pyx":1049
 * 
 *     environ.blend(fraction)
 *     environ.set_color(alpha, alpha, alpha, over * alpha)             # <<<<<<<<<<<<<<
 * 
 *     y = 0
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_color(__pyx_v_environ, __pyx_v_alpha, __pyx_v_alpha, __pyx_v_alpha, (__pyx_v_over * __pyx_v_alpha));

  /* "renpy/gl/gltexture.pyx":1051
 *     environ.set_color(alpha, alpha, alpha, over * alpha)
 * 
 *     y = 0             # <<<<<<<<<<<<<<
 * 
 *     rows0, rows1 = align_axes(tg0.rows, tg1.rows)
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_y = __pyx_int_0;

  /* "renpy/gl/gltexture.pyx":1053
 *     y = 0
 * 
 *     rows0, rows1 = align_axes(tg0.rows, tg1.rows)             # <<<<<<<<<<<<<<
 *     cols0, cols1 = align_axes(tg0.columns, tg1.columns)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_align_axes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_tg0->rows, __pyx_v_tg1->rows};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_v_tg0->rows, __pyx_v_tg1->rows};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_tg0->rows);
    __Pyx_GIVEREF(__pyx_v_tg0->rows);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_tg0->rows);
    __Pyx_INCREF(__pyx_v_tg1->rows);
    __Pyx_GIVEREF(__pyx_v_tg1->rows);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_tg1->rows);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1053, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1053, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_6(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_5 = __pyx_t_6(__pyx_t_3); if (unlikely(!__pyx_t_5)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_3), 2) < 0) __PYX_ERR(0, 1053, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1053, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_rows0 = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_rows1 = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":1054
 * 
 *     rows0, rows1 = align_axes(tg0.rows, tg1.rows)
 *     cols0, cols1 = align_axes(tg0.columns, tg1.columns)             # <<<<<<<<<<<<<<
 * 
 *     # t0 = texture 0, t1 = texture 1.
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_align_axes); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1054, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_tg0->columns, __pyx_v_tg1->columns};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_2, __pyx_v_tg0->columns, __pyx_v_tg1->columns};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_tg0->columns);
    __Pyx_GIVEREF(__pyx_v_tg0->columns);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_4, __pyx_v_tg0->columns);
    __Pyx_INCREF(__pyx_v_tg1->columns);
    __Pyx_GIVEREF(__pyx_v_tg1->columns);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_4, __pyx_v_tg1->columns);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1054, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1054, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext;
    index = 0; __pyx_t_5 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    index = 1; __pyx_t_3 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_2), 2) < 0) __PYX_ERR(0, 1054, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1054, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  __pyx_v_cols0 = __pyx_t_5;
  __pyx_t_5 = 0;
  __pyx_v_cols1 = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":1060
 *     # w, h - width and height to draw.
 *     # ri, ci - row index, column index in tiles.
 *     for (t0y, t0h, t0ri), (t1y, t1h, t1ri) in zip(rows0, rows1):             # <<<<<<<<<<<<<<
 *         x = 0
 * 
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1060, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_rows0);
  __Pyx_GIVEREF(__pyx_v_rows0);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_rows0);
  __Pyx_INCREF(__pyx_v_rows1);
  __Pyx_GIVEREF(__pyx_v_rows1);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_rows1);
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1060, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_3)) || PyTuple_CheckExact(__pyx_t_3)) {
    __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1060, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_8 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1060, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 1060, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1060, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_3); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 1060, __pyx_L1_error)
        #else
        __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1060, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        #endif
      }
    } else {
      __pyx_t_3 = __pyx_t_8(__pyx_t_1);
      if (unlikely(!__pyx_t_3)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1060, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_3);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1060, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      #else
      __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_9 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = Py_TYPE(__pyx_t_9)->tp_iternext;
      index = 0; __pyx_t_5 = __pyx_t_6(__pyx_t_9); if (unlikely(!__pyx_t_5)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_2 = __pyx_t_6(__pyx_t_9); if (unlikely(!__pyx_t_2)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_9), 2) < 0) __PYX_ERR(0, 1060, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L10_unpacking_done;
      __pyx_L9_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1060, __pyx_L1_error)
      __pyx_L10_unpacking_done:;
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1060, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      #else
      __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_6 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_9 = __pyx_t_6(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_10 = __pyx_t_6(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      index = 2; __pyx_t_11 = __pyx_t_6(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_12), 3) < 0) __PYX_ERR(0, 1060, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L12_unpacking_done;
      __pyx_L11_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1060, __pyx_L1_error)
      __pyx_L12_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_t0y, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t0h, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t0ri, __pyx_t_11);
    __pyx_t_11 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1060, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_9);
      #else
      __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_9 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1060, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_6 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_11 = __pyx_t_6(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 1; __pyx_t_10 = __pyx_t_6(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      index = 2; __pyx_t_9 = __pyx_t_6(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_12), 3) < 0) __PYX_ERR(0, 1060, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L14_unpacking_done;
      __pyx_L13_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1060, __pyx_L1_error)
      __pyx_L14_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_t1y, __pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t1h, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t1ri, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "renpy/gl/gltexture.pyx":1061
 *     # ri, ci - row index, column index in tiles.
 *     for (t0y, t0h, t0ri), (t1y, t1h, t1ri) in zip(rows0, rows1):
 *         x = 0             # <<<<<<<<<<<<<<
 * 
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci) in zip(cols0, cols1):
 */
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_XDECREF_SET(__pyx_v_x, __pyx_int_0);

    /* "renpy/gl/gltexture.pyx":1063
 *         x = 0
 * 
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci) in zip(cols0, cols1):             # <<<<<<<<<<<<<<
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]
 */
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1063, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_cols0);
    __Pyx_GIVEREF(__pyx_v_cols0);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_cols0);
    __Pyx_INCREF(__pyx_v_cols1);
    __Pyx_GIVEREF(__pyx_v_cols1);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_cols1);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1063, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_13 = 0;
      __pyx_t_14 = NULL;
    } else {
      __pyx_t_13 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1063, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1063, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_14)) {
        if (likely(PyList_CheckExact(__pyx_t_3))) {
          if (__pyx_t_13 >= PyList_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_2); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1063, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1063, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_13 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_13); __Pyx_INCREF(__pyx_t_2); __pyx_t_13++; if (unlikely(0 < 0)) __PYX_ERR(0, 1063, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_13); __pyx_t_13++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1063, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_14(__pyx_t_3);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1063, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1063, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_9);
        #else
        __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_10 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_6 = Py_TYPE(__pyx_t_10)->tp_iternext;
        index = 0; __pyx_t_5 = __pyx_t_6(__pyx_t_10); if (unlikely(!__pyx_t_5)) goto __pyx_L17_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        index = 1; __pyx_t_9 = __pyx_t_6(__pyx_t_10); if (unlikely(!__pyx_t_9)) goto __pyx_L17_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_9);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_10), 2) < 0) __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_t_6 = NULL;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        goto __pyx_L18_unpacking_done;
        __pyx_L17_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_6 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_L18_unpacking_done:;
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
        PyObject* sequence = __pyx_t_5;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1063, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_12 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_12 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_12);
        #else
        __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_12 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        #endif
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_15 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_6 = Py_TYPE(__pyx_t_15)->tp_iternext;
        index = 0; __pyx_t_10 = __pyx_t_6(__pyx_t_15); if (unlikely(!__pyx_t_10)) goto __pyx_L19_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 1; __pyx_t_11 = __pyx_t_6(__pyx_t_15); if (unlikely(!__pyx_t_11)) goto __pyx_L19_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        index = 2; __pyx_t_12 = __pyx_t_6(__pyx_t_15); if (unlikely(!__pyx_t_12)) goto __pyx_L19_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_12);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_15), 3) < 0) __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_t_6 = NULL;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        goto __pyx_L20_unpacking_done;
        __pyx_L19_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_6 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_L20_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_t0x, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t0w, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t0ci, __pyx_t_12);
      __pyx_t_12 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_9))) || (PyList_CheckExact(__pyx_t_9))) {
        PyObject* sequence = __pyx_t_9;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1063, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_10 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        #else
        __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_10 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_15 = PyObject_GetIter(__pyx_t_9); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1063, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_t_6 = Py_TYPE(__pyx_t_15)->tp_iternext;
        index = 0; __pyx_t_12 = __pyx_t_6(__pyx_t_15); if (unlikely(!__pyx_t_12)) goto __pyx_L21_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_12);
        index = 1; __pyx_t_11 = __pyx_t_6(__pyx_t_15); if (unlikely(!__pyx_t_11)) goto __pyx_L21_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        index = 2; __pyx_t_10 = __pyx_t_6(__pyx_t_15); if (unlikely(!__pyx_t_10)) goto __pyx_L21_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_15), 3) < 0) __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_t_6 = NULL;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        goto __pyx_L22_unpacking_done;
        __pyx_L21_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __pyx_t_6 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1063, __pyx_L1_error)
        __pyx_L22_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_t1x, __pyx_t_12);
      __pyx_t_12 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t1w, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t1ci, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "renpy/gl/gltexture.pyx":1065
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci) in zip(cols0, cols1):
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]             # <<<<<<<<<<<<<<
 *             t1 = tg1.tiles[t1ri][t1ci]
 * 
 */
      if (unlikely(__pyx_v_tg0->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1065, __pyx_L1_error)
      }
      __pyx_t_2 = PyObject_GetItem(__pyx_v_tg0->tiles, __pyx_v_t0ri); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_9 = PyObject_GetItem(__pyx_t_2, __pyx_v_t0ci); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t0, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "renpy/gl/gltexture.pyx":1066
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]
 *             t1 = tg1.tiles[t1ri][t1ci]             # <<<<<<<<<<<<<<
 * 
 *             draw_rectangle(
 */
      if (unlikely(__pyx_v_tg1->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1066, __pyx_L1_error)
      }
      __pyx_t_9 = PyObject_GetItem(__pyx_v_tg1->tiles, __pyx_v_t1ri); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1066, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_2 = PyObject_GetItem(__pyx_t_9, __pyx_v_t1ci); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1066, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t1, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":1071
 *                 environ,
 *                 sx, sy,
 *                 x, y,             # <<<<<<<<<<<<<<
 *                 t0w, t0h,
 *                 transform,
 */
      __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_v_x); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1071, __pyx_L1_error)
      __pyx_t_17 = __pyx_PyFloat_AsDouble(__pyx_v_y); if (unlikely((__pyx_t_17 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1071, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1072
 *                 sx, sy,
 *                 x, y,
 *                 t0w, t0h,             # <<<<<<<<<<<<<<
 *                 transform,
 *                 t0, t0x, t0y,
 */
      __pyx_t_18 = __pyx_PyFloat_AsDouble(__pyx_v_t0w); if (unlikely((__pyx_t_18 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1072, __pyx_L1_error)
      __pyx_t_19 = __pyx_PyFloat_AsDouble(__pyx_v_t0h); if (unlikely((__pyx_t_19 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1072, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1074
 *                 t0w, t0h,
 *                 transform,
 *                 t0, t0x, t0y,             # <<<<<<<<<<<<<<
 *                 t1, t1x, t1y,
 *                 None, 0, 0,
 */
      if (!(likely(((__pyx_v_t0) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_t0, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 1074, __pyx_L1_error)
      __pyx_t_20 = __pyx_PyFloat_AsFloat(__pyx_v_t0x); if (unlikely((__pyx_t_20 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1074, __pyx_L1_error)
      __pyx_t_21 = __pyx_PyFloat_AsFloat(__pyx_v_t0y); if (unlikely((__pyx_t_21 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1074, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1075
 *                 transform,
 *                 t0, t0x, t0y,
 *                 t1, t1x, t1y,             # <<<<<<<<<<<<<<
 *                 None, 0, 0,
 *                 )
 */
      if (!(likely(((__pyx_v_t1) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_t1, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 1075, __pyx_L1_error)
      __pyx_t_22 = __pyx_PyFloat_AsFloat(__pyx_v_t1x); if (unlikely((__pyx_t_22 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1075, __pyx_L1_error)
      __pyx_t_23 = __pyx_PyFloat_AsFloat(__pyx_v_t1y); if (unlikely((__pyx_t_23 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1075, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1068
 *             t1 = tg1.tiles[t1ri][t1ci]
 * 
 *             draw_rectangle(             # <<<<<<<<<<<<<<
 *                 environ,
 *                 sx, sy,
 */
      __pyx_f_5renpy_2gl_9gltexture_draw_rectangle(__pyx_v_environ, __pyx_v_sx, __pyx_v_sy, __pyx_t_16, __pyx_t_17, __pyx_t_18, __pyx_t_19, __pyx_v_transform, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t0), __pyx_t_20, __pyx_t_21, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t1), __pyx_t_22, __pyx_t_23, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)Py_None), 0.0, 0.0);

      /* "renpy/gl/gltexture.pyx":1079
 *                 )
 * 
 *             x += t0w             # <<<<<<<<<<<<<<
 * 
 *         y += t0h
 */
      __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_x, __pyx_v_t0w); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1079, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "renpy/gl/gltexture.pyx":1063
 *         x = 0
 * 
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci) in zip(cols0, cols1):             # <<<<<<<<<<<<<<
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":1081
 *             x += t0w
 * 
 *         y += t0h             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_y, __pyx_v_t0h); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1081, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_y, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "renpy/gl/gltexture.pyx":1060
 *     # w, h - width and height to draw.
 *     # ri, ci - row index, column index in tiles.
 *     for (t0y, t0h, t0ri), (t1y, t1h, t1ri) in zip(rows0, rows1):             # <<<<<<<<<<<<<<
 *         x = 0
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":1027
 *         y += texh
 * 
 * cpdef blend(TextureGrid tg0, TextureGrid tg1, double sx, double sy, Matrix transform, double alpha, double over, double fraction, Environ environ, bint nearest):             # <<<<<<<<<<<<<<
 *     """
 *     Blends two textures to the screen.
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("renpy.gl.gltexture.blend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XDECREF(__pyx_v_rows0);
  __Pyx_XDECREF(__pyx_v_rows1);
  __Pyx_XDECREF(__pyx_v_cols0);
  __Pyx_XDECREF(__pyx_v_cols1);
  __Pyx_XDECREF(__pyx_v_t0y);
  __Pyx_XDECREF(__pyx_v_t0h);
  __Pyx_XDECREF(__pyx_v_t0ri);
  __Pyx_XDECREF(__pyx_v_t1y);
  __Pyx_XDECREF(__pyx_v_t1h);
  __Pyx_XDECREF(__pyx_v_t1ri);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_t0x);
  __Pyx_XDECREF(__pyx_v_t0w);
  __Pyx_XDECREF(__pyx_v_t0ci);
  __Pyx_XDECREF(__pyx_v_t1x);
  __Pyx_XDECREF(__pyx_v_t1w);
  __Pyx_XDECREF(__pyx_v_t1ci);
  __Pyx_XDECREF(__pyx_v_t0);
  __Pyx_XDECREF(__pyx_v_t1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_25blend(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_24blend[] = "\n    Blends two textures to the screen.\n\n    `tg0` and `tg1` are the texture.\n\n    `sx` and `sy` are the offsets from the upper-left corner of the screen.\n\n    `transform` is the transform to apply to the texgrid, when going from\n    texgrid coordinates to screen coordinates.\n\n    `alpha` is the alpha multiplier applied, from 0.0 to 1.0.\n\n    `over` is the over blending factor.\n\n    `fraction` is the fraction of the second texture to show.\n    ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_25blend(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0 = 0;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1 = 0;
  double __pyx_v_sx;
  double __pyx_v_sy;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform = 0;
  double __pyx_v_alpha;
  double __pyx_v_over;
  double __pyx_v_fraction;
  struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ = 0;
  int __pyx_v_nearest;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("blend (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_tg0,&__pyx_n_s_tg1,&__pyx_n_s_sx,&__pyx_n_s_sy,&__pyx_n_s_transform,&__pyx_n_s_alpha,&__pyx_n_s_over,&__pyx_n_s_fraction,&__pyx_n_s_environ,&__pyx_n_s_nearest,0};
    PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tg0)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tg1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 1); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 2); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 3); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transform)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 4); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 5); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_over)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 6); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fraction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 7); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_environ)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 8); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nearest)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, 9); __PYX_ERR(0, 1027, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "blend") < 0)) __PYX_ERR(0, 1027, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 10) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
    }
    __pyx_v_tg0 = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)values[0]);
    __pyx_v_tg1 = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)values[1]);
    __pyx_v_sx = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_sx == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1027, __pyx_L3_error)
    __pyx_v_sy = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_sy == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1027, __pyx_L3_error)
    __pyx_v_transform = ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)values[4]);
    __pyx_v_alpha = __pyx_PyFloat_AsDouble(values[5]); if (unlikely((__pyx_v_alpha == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1027, __pyx_L3_error)
    __pyx_v_over = __pyx_PyFloat_AsDouble(values[6]); if (unlikely((__pyx_v_over == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1027, __pyx_L3_error)
    __pyx_v_fraction = __pyx_PyFloat_AsDouble(values[7]); if (unlikely((__pyx_v_fraction == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1027, __pyx_L3_error)
    __pyx_v_environ = ((struct __pyx_obj_5renpy_2gl_6gldraw_Environ *)values[8]);
    __pyx_v_nearest = __Pyx_PyObject_IsTrue(values[9]); if (unlikely((__pyx_v_nearest == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1027, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("blend", 1, 10, 10, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1027, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.blend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tg0), __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid, 1, "tg0", 0))) __PYX_ERR(0, 1027, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tg1), __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid, 1, "tg1", 0))) __PYX_ERR(0, 1027, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transform), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "transform", 0))) __PYX_ERR(0, 1027, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_environ), __pyx_ptype_5renpy_2gl_6gldraw_Environ, 1, "environ", 0))) __PYX_ERR(0, 1027, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_24blend(__pyx_self, __pyx_v_tg0, __pyx_v_tg1, __pyx_v_sx, __pyx_v_sy, __pyx_v_transform, __pyx_v_alpha, __pyx_v_over, __pyx_v_fraction, __pyx_v_environ, __pyx_v_nearest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_24blend(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, double __pyx_v_fraction, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("blend", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5renpy_2gl_9gltexture_blend(__pyx_v_tg0, __pyx_v_tg1, __pyx_v_sx, __pyx_v_sy, __pyx_v_transform, __pyx_v_alpha, __pyx_v_over, __pyx_v_fraction, __pyx_v_environ, __pyx_v_nearest, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1027, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.blend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":1084
 * 
 * 
 * cpdef imageblend(TextureGrid tg0, TextureGrid tg1, TextureGrid tg2, double sx, double sy, Matrix transform, double alpha, double over, double fraction, int ramp, Environ environ, bint nearest):             # <<<<<<<<<<<<<<
 *     """
 *     This uses texture 0 to control the blending of tetures 1 and 2 to
 */

static PyObject *__pyx_pw_5renpy_2gl_9gltexture_27imageblend(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_5renpy_2gl_9gltexture_imageblend(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg2, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, double __pyx_v_fraction, int __pyx_v_ramp, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_y = NULL;
  PyObject *__pyx_v_rows0 = NULL;
  PyObject *__pyx_v_rows1 = NULL;
  PyObject *__pyx_v_rows2 = NULL;
  PyObject *__pyx_v_cols0 = NULL;
  PyObject *__pyx_v_cols1 = NULL;
  PyObject *__pyx_v_cols2 = NULL;
  PyObject *__pyx_v_t0y = NULL;
  PyObject *__pyx_v_t0h = NULL;
  PyObject *__pyx_v_t0ri = NULL;
  PyObject *__pyx_v_t1y = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t1h = NULL;
  PyObject *__pyx_v_t1ri = NULL;
  PyObject *__pyx_v_t2y = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t2h = NULL;
  PyObject *__pyx_v_t2ri = NULL;
  PyObject *__pyx_v_x = NULL;
  PyObject *__pyx_v_t0x = NULL;
  PyObject *__pyx_v_t0w = NULL;
  PyObject *__pyx_v_t0ci = NULL;
  PyObject *__pyx_v_t1x = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t1w = NULL;
  PyObject *__pyx_v_t1ci = NULL;
  PyObject *__pyx_v_t2x = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t2w = NULL;
  PyObject *__pyx_v_t2ci = NULL;
  PyObject *__pyx_v_t0 = NULL;
  PyObject *__pyx_v_t1 = NULL;
  PyObject *__pyx_v_t2 = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  double __pyx_t_17;
  double __pyx_t_18;
  double __pyx_t_19;
  double __pyx_t_20;
  float __pyx_t_21;
  float __pyx_t_22;
  float __pyx_t_23;
  float __pyx_t_24;
  float __pyx_t_25;
  float __pyx_t_26;
  __Pyx_RefNannySetupContext("imageblend", 0);

  /* "renpy/gl/gltexture.pyx":1107
 *     """
 * 
 *     tg0.make_ready(nearest)             # <<<<<<<<<<<<<<
 *     tg1.make_ready(nearest)
 *     tg2.make_ready(nearest)
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_tg0->__pyx_vtab)->make_ready(__pyx_v_tg0, __pyx_v_nearest, 0);

  /* "renpy/gl/gltexture.pyx":1108
 * 
 *     tg0.make_ready(nearest)
 *     tg1.make_ready(nearest)             # <<<<<<<<<<<<<<
 *     tg2.make_ready(nearest)
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_tg1->__pyx_vtab)->make_ready(__pyx_v_tg1, __pyx_v_nearest, 0);

  /* "renpy/gl/gltexture.pyx":1109
 *     tg0.make_ready(nearest)
 *     tg1.make_ready(nearest)
 *     tg2.make_ready(nearest)             # <<<<<<<<<<<<<<
 * 
 *     environ.imageblend(fraction, ramp)
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v_tg2->__pyx_vtab)->make_ready(__pyx_v_tg2, __pyx_v_nearest, 0);

  /* "renpy/gl/gltexture.pyx":1111
 *     tg2.make_ready(nearest)
 * 
 *     environ.imageblend(fraction, ramp)             # <<<<<<<<<<<<<<
 *     environ.set_color(alpha, alpha, alpha, over * alpha)
 * 
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->imageblend(__pyx_v_environ, __pyx_v_fraction, __pyx_v_ramp);

  /* "renpy/gl/gltexture.pyx":1112
 * 
 *     environ.imageblend(fraction, ramp)
 *     environ.set_color(alpha, alpha, alpha, over * alpha)             # <<<<<<<<<<<<<<
 * 
 *     y = 0
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_color(__pyx_v_environ, __pyx_v_alpha, __pyx_v_alpha, __pyx_v_alpha, (__pyx_v_over * __pyx_v_alpha));

  /* "renpy/gl/gltexture.pyx":1114
 *     environ.set_color(alpha, alpha, alpha, over * alpha)
 * 
 *     y = 0             # <<<<<<<<<<<<<<
 * 
 *     rows0, rows1, rows2 = align_axes(tg0.rows, tg1.rows, tg2.rows)
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_v_y = __pyx_int_0;

  /* "renpy/gl/gltexture.pyx":1116
 *     y = 0
 * 
 *     rows0, rows1, rows2 = align_axes(tg0.rows, tg1.rows, tg2.rows)             # <<<<<<<<<<<<<<
 *     cols0, cols1, cols2 = align_axes(tg0.columns, tg1.columns, tg2.columns)
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_align_axes); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_tg0->rows, __pyx_v_tg1->rows, __pyx_v_tg2->rows};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_tg0->rows, __pyx_v_tg1->rows, __pyx_v_tg2->rows};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_tg0->rows);
    __Pyx_GIVEREF(__pyx_v_tg0->rows);
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, __pyx_v_tg0->rows);
    __Pyx_INCREF(__pyx_v_tg1->rows);
    __Pyx_GIVEREF(__pyx_v_tg1->rows);
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, __pyx_v_tg1->rows);
    __Pyx_INCREF(__pyx_v_tg2->rows);
    __Pyx_GIVEREF(__pyx_v_tg2->rows);
    PyTuple_SET_ITEM(__pyx_t_5, 2+__pyx_t_4, __pyx_v_tg2->rows);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1116, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_6 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    index = 2; __pyx_t_3 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 3) < 0) __PYX_ERR(0, 1116, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1116, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_v_rows0 = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_rows1 = __pyx_t_5;
  __pyx_t_5 = 0;
  __pyx_v_rows2 = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "renpy/gl/gltexture.pyx":1117
 * 
 *     rows0, rows1, rows2 = align_axes(tg0.rows, tg1.rows, tg2.rows)
 *     cols0, cols1, cols2 = align_axes(tg0.columns, tg1.columns, tg2.columns)             # <<<<<<<<<<<<<<
 * 
 *     # t0 = texture 0, t1 = texture 1.
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_align_axes); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_tg0->columns, __pyx_v_tg1->columns, __pyx_v_tg2->columns};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_5, __pyx_v_tg0->columns, __pyx_v_tg1->columns, __pyx_v_tg2->columns};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_4, 3+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_2 = PyTuple_New(3+__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(__pyx_v_tg0->columns);
    __Pyx_GIVEREF(__pyx_v_tg0->columns);
    PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_4, __pyx_v_tg0->columns);
    __Pyx_INCREF(__pyx_v_tg1->columns);
    __Pyx_GIVEREF(__pyx_v_tg1->columns);
    PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_4, __pyx_v_tg1->columns);
    __Pyx_INCREF(__pyx_v_tg2->columns);
    __Pyx_GIVEREF(__pyx_v_tg2->columns);
    PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_4, __pyx_v_tg2->columns);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 3)) {
      if (size > 3) __Pyx_RaiseTooManyValuesError(3);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 1117, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_5 = PyTuple_GET_ITEM(sequence, 2); 
    } else {
      __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_5 = PyList_GET_ITEM(sequence, 2); 
    }
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_6 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1117, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
    index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    index = 1; __pyx_t_2 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_2)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 2; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 3) < 0) __PYX_ERR(0, 1117, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 1117, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }
  __pyx_v_cols0 = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_cols1 = __pyx_t_2;
  __pyx_t_2 = 0;
  __pyx_v_cols2 = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":1123
 *     # w, h - width and height to draw.
 *     # ri, ci - row index, column index in tiles.
 *     for (t0y, t0h, t0ri), (t1y, t1h, t1ri), (t2y, t2h, t2ri) in zip(rows0, rows1, rows2):             # <<<<<<<<<<<<<<
 *         x = 0
 * 
 */
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_rows0);
  __Pyx_GIVEREF(__pyx_v_rows0);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_rows0);
  __Pyx_INCREF(__pyx_v_rows1);
  __Pyx_GIVEREF(__pyx_v_rows1);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_rows1);
  __Pyx_INCREF(__pyx_v_rows2);
  __Pyx_GIVEREF(__pyx_v_rows2);
  PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_rows2);
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_t_5)) || PyTuple_CheckExact(__pyx_t_5)) {
    __pyx_t_1 = __pyx_t_5; __Pyx_INCREF(__pyx_t_1); __pyx_t_8 = 0;
    __pyx_t_9 = NULL;
  } else {
    __pyx_t_8 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1123, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  for (;;) {
    if (likely(!__pyx_t_9)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_5); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 1123, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_8 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_8); __Pyx_INCREF(__pyx_t_5); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 1123, __pyx_L1_error)
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_1, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1123, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_9(__pyx_t_1);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 1123, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_5))) || (PyList_CheckExact(__pyx_t_5))) {
      PyObject* sequence = __pyx_t_5;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1123, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_6 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      #else
      __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_10 = PyObject_GetIter(__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_7 = Py_TYPE(__pyx_t_10)->tp_iternext;
      index = 0; __pyx_t_2 = __pyx_t_7(__pyx_t_10); if (unlikely(!__pyx_t_2)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_3 = __pyx_t_7(__pyx_t_10); if (unlikely(!__pyx_t_3)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 2; __pyx_t_6 = __pyx_t_7(__pyx_t_10); if (unlikely(!__pyx_t_6)) goto __pyx_L9_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_10), 3) < 0) __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L10_unpacking_done;
      __pyx_L9_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_L10_unpacking_done:;
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1123, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_12 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_12 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_12);
      #else
      __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_13 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_7 = Py_TYPE(__pyx_t_13)->tp_iternext;
      index = 0; __pyx_t_10 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_10)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      index = 1; __pyx_t_11 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_11)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 2; __pyx_t_12 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_12)) goto __pyx_L11_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_12);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_13), 3) < 0) __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      goto __pyx_L12_unpacking_done;
      __pyx_L11_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_L12_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_t0y, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t0h, __pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t0ri, __pyx_t_12);
    __pyx_t_12 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1123, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_12 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_12 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_10);
      #else
      __pyx_t_12 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_10 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_13 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_7 = Py_TYPE(__pyx_t_13)->tp_iternext;
      index = 0; __pyx_t_12 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_12)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_12);
      index = 1; __pyx_t_11 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_11)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 2; __pyx_t_10 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_10)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_13), 3) < 0) __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      goto __pyx_L14_unpacking_done;
      __pyx_L13_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_L14_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_t1y, __pyx_t_12);
    __pyx_t_12 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t1h, __pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t1ri, __pyx_t_10);
    __pyx_t_10 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
      PyObject* sequence = __pyx_t_6;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1123, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_12 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_12 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_12);
      #else
      __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      #endif
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_13 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_7 = Py_TYPE(__pyx_t_13)->tp_iternext;
      index = 0; __pyx_t_10 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_10)) goto __pyx_L15_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      index = 1; __pyx_t_11 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_11)) goto __pyx_L15_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 2; __pyx_t_12 = __pyx_t_7(__pyx_t_13); if (unlikely(!__pyx_t_12)) goto __pyx_L15_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_12);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_13), 3) < 0) __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_t_7 = NULL;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      goto __pyx_L16_unpacking_done;
      __pyx_L15_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_7 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1123, __pyx_L1_error)
      __pyx_L16_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_t2y, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t2h, __pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t2ri, __pyx_t_12);
    __pyx_t_12 = 0;

    /* "renpy/gl/gltexture.pyx":1124
 *     # ri, ci - row index, column index in tiles.
 *     for (t0y, t0h, t0ri), (t1y, t1h, t1ri), (t2y, t2h, t2ri) in zip(rows0, rows1, rows2):
 *         x = 0             # <<<<<<<<<<<<<<
 * 
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci), (t2x, t2w, t2ci) in zip(cols0, cols1, cols2):
 */
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_XDECREF_SET(__pyx_v_x, __pyx_int_0);

    /* "renpy/gl/gltexture.pyx":1126
 *         x = 0
 * 
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci), (t2x, t2w, t2ci) in zip(cols0, cols1, cols2):             # <<<<<<<<<<<<<<
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]
 */
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_cols0);
    __Pyx_GIVEREF(__pyx_v_cols0);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_cols0);
    __Pyx_INCREF(__pyx_v_cols1);
    __Pyx_GIVEREF(__pyx_v_cols1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_cols1);
    __Pyx_INCREF(__pyx_v_cols2);
    __Pyx_GIVEREF(__pyx_v_cols2);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_cols2);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_5, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1126, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (likely(PyList_CheckExact(__pyx_t_6)) || PyTuple_CheckExact(__pyx_t_6)) {
      __pyx_t_5 = __pyx_t_6; __Pyx_INCREF(__pyx_t_5); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_15 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1126, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_5))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_14); __Pyx_INCREF(__pyx_t_6); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1126, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_5, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1126, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_6 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_14); __Pyx_INCREF(__pyx_t_6); __pyx_t_14++; if (unlikely(0 < 0)) __PYX_ERR(0, 1126, __pyx_L1_error)
          #else
          __pyx_t_6 = PySequence_ITEM(__pyx_t_5, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1126, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          #endif
        }
      } else {
        __pyx_t_6 = __pyx_t_15(__pyx_t_5);
        if (unlikely(!__pyx_t_6)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 1126, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_6);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_6))) || (PyList_CheckExact(__pyx_t_6))) {
        PyObject* sequence = __pyx_t_6;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1126, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_12 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_12 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_12);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_12 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        #endif
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_11 = PyObject_GetIter(__pyx_t_6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_7 = Py_TYPE(__pyx_t_11)->tp_iternext;
        index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_11); if (unlikely(!__pyx_t_3)) goto __pyx_L19_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_2 = __pyx_t_7(__pyx_t_11); if (unlikely(!__pyx_t_2)) goto __pyx_L19_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_2);
        index = 2; __pyx_t_12 = __pyx_t_7(__pyx_t_11); if (unlikely(!__pyx_t_12)) goto __pyx_L19_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_12);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_11), 3) < 0) __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_t_7 = NULL;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L20_unpacking_done;
        __pyx_L19_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_7 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_L20_unpacking_done:;
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
        PyObject* sequence = __pyx_t_3;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1126, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_13 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_13 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_13);
        #else
        __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_13 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        #endif
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_16 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_7 = Py_TYPE(__pyx_t_16)->tp_iternext;
        index = 0; __pyx_t_11 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_11)) goto __pyx_L21_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        index = 1; __pyx_t_10 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_10)) goto __pyx_L21_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 2; __pyx_t_13 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_13)) goto __pyx_L21_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_13);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_16), 3) < 0) __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_t_7 = NULL;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        goto __pyx_L22_unpacking_done;
        __pyx_L21_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_7 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_L22_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_t0x, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t0w, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t0ci, __pyx_t_13);
      __pyx_t_13 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1126, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_13 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_13 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_11 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        #else
        __pyx_t_13 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_16 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_7 = Py_TYPE(__pyx_t_16)->tp_iternext;
        index = 0; __pyx_t_13 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_13)) goto __pyx_L23_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_13);
        index = 1; __pyx_t_10 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_10)) goto __pyx_L23_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 2; __pyx_t_11 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_11)) goto __pyx_L23_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_16), 3) < 0) __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_t_7 = NULL;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        goto __pyx_L24_unpacking_done;
        __pyx_L23_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_7 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_L24_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_t1x, __pyx_t_13);
      __pyx_t_13 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t1w, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t1ci, __pyx_t_11);
      __pyx_t_11 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_12))) || (PyList_CheckExact(__pyx_t_12))) {
        PyObject* sequence = __pyx_t_12;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 3)) {
          if (size > 3) __Pyx_RaiseTooManyValuesError(3);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1126, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
          __pyx_t_13 = PyTuple_GET_ITEM(sequence, 2); 
        } else {
          __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
          __pyx_t_13 = PyList_GET_ITEM(sequence, 2); 
        }
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_13);
        #else
        __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_13 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        #endif
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_16 = PyObject_GetIter(__pyx_t_12); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1126, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_7 = Py_TYPE(__pyx_t_16)->tp_iternext;
        index = 0; __pyx_t_11 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_11)) goto __pyx_L25_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_11);
        index = 1; __pyx_t_10 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_10)) goto __pyx_L25_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 2; __pyx_t_13 = __pyx_t_7(__pyx_t_16); if (unlikely(!__pyx_t_13)) goto __pyx_L25_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_13);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_16), 3) < 0) __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_t_7 = NULL;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        goto __pyx_L26_unpacking_done;
        __pyx_L25_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_7 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 1126, __pyx_L1_error)
        __pyx_L26_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_t2x, __pyx_t_11);
      __pyx_t_11 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t2w, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t2ci, __pyx_t_13);
      __pyx_t_13 = 0;

      /* "renpy/gl/gltexture.pyx":1128
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci), (t2x, t2w, t2ci) in zip(cols0, cols1, cols2):
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]             # <<<<<<<<<<<<<<
 *             t1 = tg1.tiles[t1ri][t1ci]
 *             t2 = tg2.tiles[t2ri][t2ci]
 */
      if (unlikely(__pyx_v_tg0->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1128, __pyx_L1_error)
      }
      __pyx_t_6 = PyObject_GetItem(__pyx_v_tg0->tiles, __pyx_v_t0ri); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_12 = PyObject_GetItem(__pyx_t_6, __pyx_v_t0ci); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1128, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t0, __pyx_t_12);
      __pyx_t_12 = 0;

      /* "renpy/gl/gltexture.pyx":1129
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]
 *             t1 = tg1.tiles[t1ri][t1ci]             # <<<<<<<<<<<<<<
 *             t2 = tg2.tiles[t2ri][t2ci]
 * 
 */
      if (unlikely(__pyx_v_tg1->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1129, __pyx_L1_error)
      }
      __pyx_t_12 = PyObject_GetItem(__pyx_v_tg1->tiles, __pyx_v_t1ri); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1129, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_6 = PyObject_GetItem(__pyx_t_12, __pyx_v_t1ci); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1129, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t1, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "renpy/gl/gltexture.pyx":1130
 *             t0 = tg0.tiles[t0ri][t0ci]
 *             t1 = tg1.tiles[t1ri][t1ci]
 *             t2 = tg2.tiles[t2ri][t2ci]             # <<<<<<<<<<<<<<
 * 
 *             draw_rectangle(
 */
      if (unlikely(__pyx_v_tg2->tiles == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1130, __pyx_L1_error)
      }
      __pyx_t_6 = PyObject_GetItem(__pyx_v_tg2->tiles, __pyx_v_t2ri); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_12 = PyObject_GetItem(__pyx_t_6, __pyx_v_t2ci); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF_SET(__pyx_v_t2, __pyx_t_12);
      __pyx_t_12 = 0;

      /* "renpy/gl/gltexture.pyx":1135
 *                 environ,
 *                 sx, sy,
 *                 x, y,             # <<<<<<<<<<<<<<
 *                 t0w, t0h,
 *                 transform,
 */
      __pyx_t_17 = __pyx_PyFloat_AsDouble(__pyx_v_x); if (unlikely((__pyx_t_17 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1135, __pyx_L1_error)
      __pyx_t_18 = __pyx_PyFloat_AsDouble(__pyx_v_y); if (unlikely((__pyx_t_18 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1135, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1136
 *                 sx, sy,
 *                 x, y,
 *                 t0w, t0h,             # <<<<<<<<<<<<<<
 *                 transform,
 *                 t0, t0x, t0y,
 */
      __pyx_t_19 = __pyx_PyFloat_AsDouble(__pyx_v_t0w); if (unlikely((__pyx_t_19 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1136, __pyx_L1_error)
      __pyx_t_20 = __pyx_PyFloat_AsDouble(__pyx_v_t0h); if (unlikely((__pyx_t_20 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1136, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1138
 *                 t0w, t0h,
 *                 transform,
 *                 t0, t0x, t0y,             # <<<<<<<<<<<<<<
 *                 t1, t1x, t1y,
 *                 t2, t2x, t2y,
 */
      if (!(likely(((__pyx_v_t0) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_t0, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 1138, __pyx_L1_error)
      __pyx_t_21 = __pyx_PyFloat_AsFloat(__pyx_v_t0x); if (unlikely((__pyx_t_21 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1138, __pyx_L1_error)
      __pyx_t_22 = __pyx_PyFloat_AsFloat(__pyx_v_t0y); if (unlikely((__pyx_t_22 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1138, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1139
 *                 transform,
 *                 t0, t0x, t0y,
 *                 t1, t1x, t1y,             # <<<<<<<<<<<<<<
 *                 t2, t2x, t2y,
 *                 )
 */
      if (!(likely(((__pyx_v_t1) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_t1, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 1139, __pyx_L1_error)
      __pyx_t_23 = __pyx_PyFloat_AsFloat(__pyx_v_t1x); if (unlikely((__pyx_t_23 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1139, __pyx_L1_error)
      __pyx_t_24 = __pyx_PyFloat_AsFloat(__pyx_v_t1y); if (unlikely((__pyx_t_24 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1139, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1140
 *                 t0, t0x, t0y,
 *                 t1, t1x, t1y,
 *                 t2, t2x, t2y,             # <<<<<<<<<<<<<<
 *                 )
 * 
 */
      if (!(likely(((__pyx_v_t2) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_t2, __pyx_ptype_5renpy_2gl_9gltexture_TextureCore))))) __PYX_ERR(0, 1140, __pyx_L1_error)
      __pyx_t_25 = __pyx_PyFloat_AsFloat(__pyx_v_t2x); if (unlikely((__pyx_t_25 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1140, __pyx_L1_error)
      __pyx_t_26 = __pyx_PyFloat_AsFloat(__pyx_v_t2y); if (unlikely((__pyx_t_26 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 1140, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1132
 *             t2 = tg2.tiles[t2ri][t2ci]
 * 
 *             draw_rectangle(             # <<<<<<<<<<<<<<
 *                 environ,
 *                 sx, sy,
 */
      __pyx_f_5renpy_2gl_9gltexture_draw_rectangle(__pyx_v_environ, __pyx_v_sx, __pyx_v_sy, __pyx_t_17, __pyx_t_18, __pyx_t_19, __pyx_t_20, __pyx_v_transform, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t0), __pyx_t_21, __pyx_t_22, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t1), __pyx_t_23, __pyx_t_24, ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v_t2), __pyx_t_25, __pyx_t_26);

      /* "renpy/gl/gltexture.pyx":1143
 *                 )
 * 
 *             x += t0w             # <<<<<<<<<<<<<<
 * 
 *         y += t0h
 */
      __pyx_t_12 = PyNumber_InPlaceAdd(__pyx_v_x, __pyx_v_t0w); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1143, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF_SET(__pyx_v_x, __pyx_t_12);
      __pyx_t_12 = 0;

      /* "renpy/gl/gltexture.pyx":1126
 *         x = 0
 * 
 *         for (t0x, t0w, t0ci), (t1x, t1w, t1ci), (t2x, t2w, t2ci) in zip(cols0, cols1, cols2):             # <<<<<<<<<<<<<<
 * 
 *             t0 = tg0.tiles[t0ri][t0ci]
 */
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":1145
 *             x += t0w
 * 
 *         y += t0h             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_5 = PyNumber_InPlaceAdd(__pyx_v_y, __pyx_v_t0h); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1145, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_y, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "renpy/gl/gltexture.pyx":1123
 *     # w, h - width and height to draw.
 *     # ri, ci - row index, column index in tiles.
 *     for (t0y, t0h, t0ri), (t1y, t1h, t1ri), (t2y, t2h, t2ri) in zip(rows0, rows1, rows2):             # <<<<<<<<<<<<<<
 *         x = 0
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/gl/gltexture.pyx":1084
 * 
 * 
 * cpdef imageblend(TextureGrid tg0, TextureGrid tg1, TextureGrid tg2, double sx, double sy, Matrix transform, double alpha, double over, double fraction, int ramp, Environ environ, bint nearest):             # <<<<<<<<<<<<<<
 *     """
 *     This uses texture 0 to control the blending of tetures 1 and 2 to
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("renpy.gl.gltexture.imageblend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_y);
  __Pyx_XDECREF(__pyx_v_rows0);
  __Pyx_XDECREF(__pyx_v_rows1);
  __Pyx_XDECREF(__pyx_v_rows2);
  __Pyx_XDECREF(__pyx_v_cols0);
  __Pyx_XDECREF(__pyx_v_cols1);
  __Pyx_XDECREF(__pyx_v_cols2);
  __Pyx_XDECREF(__pyx_v_t0y);
  __Pyx_XDECREF(__pyx_v_t0h);
  __Pyx_XDECREF(__pyx_v_t0ri);
  __Pyx_XDECREF(__pyx_v_t1y);
  __Pyx_XDECREF(__pyx_v_t1h);
  __Pyx_XDECREF(__pyx_v_t1ri);
  __Pyx_XDECREF(__pyx_v_t2y);
  __Pyx_XDECREF(__pyx_v_t2h);
  __Pyx_XDECREF(__pyx_v_t2ri);
  __Pyx_XDECREF(__pyx_v_x);
  __Pyx_XDECREF(__pyx_v_t0x);
  __Pyx_XDECREF(__pyx_v_t0w);
  __Pyx_XDECREF(__pyx_v_t0ci);
  __Pyx_XDECREF(__pyx_v_t1x);
  __Pyx_XDECREF(__pyx_v_t1w);
  __Pyx_XDECREF(__pyx_v_t1ci);
  __Pyx_XDECREF(__pyx_v_t2x);
  __Pyx_XDECREF(__pyx_v_t2w);
  __Pyx_XDECREF(__pyx_v_t2ci);
  __Pyx_XDECREF(__pyx_v_t0);
  __Pyx_XDECREF(__pyx_v_t1);
  __Pyx_XDECREF(__pyx_v_t2);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_27imageblend(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_26imageblend[] = "\n    This uses texture 0 to control the blending of tetures 1 and 2 to\n    the screen.\n\n    `tg0`, `tg1`, and `tg2` are the textures.\n\n    `sx` and `sy` are the offsets from the upper-left corner of the screen.\n\n    `transform` is the transform to apply to the texgrid, when going from\n    texgrid coordinates to screen coordinates.\n\n    `over` is the over blending factor.\n\n    `additive` is the additive blending factor, which is 1.0 for fully additive,\n    and 0.0 for fully over blending.\n\n    `fraction` is the fraction of the second texture to show.\n\n    `ramp` is the length of the blending ramp to use.\n\n    ";
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_27imageblend(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0 = 0;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1 = 0;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg2 = 0;
  double __pyx_v_sx;
  double __pyx_v_sy;
  struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform = 0;
  double __pyx_v_alpha;
  double __pyx_v_over;
  double __pyx_v_fraction;
  int __pyx_v_ramp;
  struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ = 0;
  int __pyx_v_nearest;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("imageblend (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_tg0,&__pyx_n_s_tg1,&__pyx_n_s_tg2,&__pyx_n_s_sx,&__pyx_n_s_sy,&__pyx_n_s_transform,&__pyx_n_s_alpha,&__pyx_n_s_over,&__pyx_n_s_fraction,&__pyx_n_s_ramp,&__pyx_n_s_environ,&__pyx_n_s_nearest,0};
    PyObject* values[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 12: values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
        CYTHON_FALLTHROUGH;
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tg0)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tg1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 1); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tg2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 2); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sx)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 3); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 4); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_transform)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 5); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 6); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_over)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 7); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_fraction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 8); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ramp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 9); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_environ)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 10); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 11:
        if (likely((values[11] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nearest)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, 11); __PYX_ERR(0, 1084, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "imageblend") < 0)) __PYX_ERR(0, 1084, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 12) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
      values[11] = PyTuple_GET_ITEM(__pyx_args, 11);
    }
    __pyx_v_tg0 = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)values[0]);
    __pyx_v_tg1 = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)values[1]);
    __pyx_v_tg2 = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)values[2]);
    __pyx_v_sx = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_sx == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
    __pyx_v_sy = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_sy == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
    __pyx_v_transform = ((struct __pyx_obj_5renpy_7display_6matrix_Matrix *)values[5]);
    __pyx_v_alpha = __pyx_PyFloat_AsDouble(values[6]); if (unlikely((__pyx_v_alpha == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
    __pyx_v_over = __pyx_PyFloat_AsDouble(values[7]); if (unlikely((__pyx_v_over == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
    __pyx_v_fraction = __pyx_PyFloat_AsDouble(values[8]); if (unlikely((__pyx_v_fraction == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
    __pyx_v_ramp = __Pyx_PyInt_As_int(values[9]); if (unlikely((__pyx_v_ramp == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
    __pyx_v_environ = ((struct __pyx_obj_5renpy_2gl_6gldraw_Environ *)values[10]);
    __pyx_v_nearest = __Pyx_PyObject_IsTrue(values[11]); if (unlikely((__pyx_v_nearest == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1084, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("imageblend", 1, 12, 12, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1084, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.imageblend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tg0), __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid, 1, "tg0", 0))) __PYX_ERR(0, 1084, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tg1), __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid, 1, "tg1", 0))) __PYX_ERR(0, 1084, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tg2), __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid, 1, "tg2", 0))) __PYX_ERR(0, 1084, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transform), __pyx_ptype_5renpy_7display_6matrix_Matrix, 1, "transform", 0))) __PYX_ERR(0, 1084, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_environ), __pyx_ptype_5renpy_2gl_6gldraw_Environ, 1, "environ", 0))) __PYX_ERR(0, 1084, __pyx_L1_error)
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_26imageblend(__pyx_self, __pyx_v_tg0, __pyx_v_tg1, __pyx_v_tg2, __pyx_v_sx, __pyx_v_sy, __pyx_v_transform, __pyx_v_alpha, __pyx_v_over, __pyx_v_fraction, __pyx_v_ramp, __pyx_v_environ, __pyx_v_nearest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_26imageblend(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg0, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg1, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v_tg2, double __pyx_v_sx, double __pyx_v_sy, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, double __pyx_v_alpha, double __pyx_v_over, double __pyx_v_fraction, int __pyx_v_ramp, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, int __pyx_v_nearest) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("imageblend", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5renpy_2gl_9gltexture_imageblend(__pyx_v_tg0, __pyx_v_tg1, __pyx_v_tg2, __pyx_v_sx, __pyx_v_sy, __pyx_v_transform, __pyx_v_alpha, __pyx_v_over, __pyx_v_fraction, __pyx_v_ramp, __pyx_v_environ, __pyx_v_nearest, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1084, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.gl.gltexture.imageblend", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":1148
 * 
 * 
 * def premultiply(             # <<<<<<<<<<<<<<
 *     object pysurf,
 *     int x,
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_29premultiply(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5renpy_2gl_9gltexture_28premultiply[] = "\n    Creates a string containing the premultiplied image data for\n    for the (x, y, w, h) box inside pysurf. The various border_\n    parameters control the addition of a border on the sides.\n    ";
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_29premultiply = {"premultiply", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_29premultiply, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_28premultiply};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_29premultiply(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pysurf = 0;
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_v_w;
  int __pyx_v_h;
  int __pyx_v_border_left;
  int __pyx_v_border_top;
  int __pyx_v_border_right;
  int __pyx_v_border_bottom;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("premultiply (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pysurf,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_w,&__pyx_n_s_h,&__pyx_n_s_border_left,&__pyx_n_s_border_top,&__pyx_n_s_border_right,&__pyx_n_s_border_bottom,0};
    PyObject* values[9] = {0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pysurf)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 1); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 2); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 3); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_h)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 4); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_left)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 5); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_top)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 6); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_right)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 7); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_border_bottom)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, 8); __PYX_ERR(0, 1148, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "premultiply") < 0)) __PYX_ERR(0, 1148, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 9) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
    }
    __pyx_v_pysurf = values[0];
    __pyx_v_x = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_x == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1150, __pyx_L3_error)
    __pyx_v_y = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_y == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1151, __pyx_L3_error)
    __pyx_v_w = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_w == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1152, __pyx_L3_error)
    __pyx_v_h = __Pyx_PyInt_As_int(values[4]); if (unlikely((__pyx_v_h == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1153, __pyx_L3_error)
    __pyx_v_border_left = __Pyx_PyObject_IsTrue(values[5]); if (unlikely((__pyx_v_border_left == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1154, __pyx_L3_error)
    __pyx_v_border_top = __Pyx_PyObject_IsTrue(values[6]); if (unlikely((__pyx_v_border_top == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1154, __pyx_L3_error)
    __pyx_v_border_right = __Pyx_PyObject_IsTrue(values[7]); if (unlikely((__pyx_v_border_right == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1154, __pyx_L3_error)
    __pyx_v_border_bottom = __Pyx_PyObject_IsTrue(values[8]); if (unlikely((__pyx_v_border_bottom == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1154, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("premultiply", 1, 9, 9, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1148, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.premultiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_28premultiply(__pyx_self, __pyx_v_pysurf, __pyx_v_x, __pyx_v_y, __pyx_v_w, __pyx_v_h, __pyx_v_border_left, __pyx_v_border_top, __pyx_v_border_right, __pyx_v_border_bottom);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_28premultiply(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pysurf, int __pyx_v_x, int __pyx_v_y, int __pyx_v_w, int __pyx_v_h, int __pyx_v_border_left, int __pyx_v_border_top, int __pyx_v_border_right, int __pyx_v_border_bottom) {
  unsigned int __pyx_v_alpha;
  PyObject *__pyx_v_rv = NULL;
  unsigned char *__pyx_v_out;
  unsigned char *__pyx_v_pixels;
  unsigned char *__pyx_v_pixels_end;
  SDL_Surface *__pyx_v_surf;
  unsigned char *__pyx_v_p;
  unsigned char *__pyx_v_op;
  unsigned char *__pyx_v_pend;
  unsigned int __pyx_v_a;
  unsigned int *__pyx_v_pp;
  unsigned int *__pyx_v_ppend;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  unsigned char *__pyx_t_5;
  __Pyx_RefNannySetupContext("premultiply", 0);

  /* "renpy/gl/gltexture.pyx":1168
 *     cdef unsigned int alpha
 * 
 *     if pysurf.get_masks()[3]:             # <<<<<<<<<<<<<<
 *         alpha = True
 *     else:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_pysurf, __pyx_n_s_get_masks); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1168, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_1, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 1168, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_4) {

    /* "renpy/gl/gltexture.pyx":1169
 * 
 *     if pysurf.get_masks()[3]:
 *         alpha = True             # <<<<<<<<<<<<<<
 *     else:
 *         alpha = False
 */
    __pyx_v_alpha = 1;

    /* "renpy/gl/gltexture.pyx":1168
 *     cdef unsigned int alpha
 * 
 *     if pysurf.get_masks()[3]:             # <<<<<<<<<<<<<<
 *         alpha = True
 *     else:
 */
    goto __pyx_L3;
  }

  /* "renpy/gl/gltexture.pyx":1171
 *         alpha = True
 *     else:
 *         alpha = False             # <<<<<<<<<<<<<<
 * 
 *     # Allocate an uninitialized string.
 */
  /*else*/ {
    __pyx_v_alpha = 0;
  }
  __pyx_L3:;

  /* "renpy/gl/gltexture.pyx":1174
 * 
 *     # Allocate an uninitialized string.
 *     rv = PyString_FromStringAndSize(<char *>NULL, w * h * 4)             # <<<<<<<<<<<<<<
 * 
 *     # Out is where we put the output.
 */
  __pyx_t_2 = PyString_FromStringAndSize(((char *)NULL), ((__pyx_v_w * __pyx_v_h) * 4)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_rv = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":1177
 * 
 *     # Out is where we put the output.
 *     cdef unsigned char *out = rv             # <<<<<<<<<<<<<<
 * 
 *     # The pixels in the source image.
 */
  __pyx_t_5 = __Pyx_PyObject_AsWritableUString(__pyx_v_rv); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1177, __pyx_L1_error)
  __pyx_v_out = __pyx_t_5;

  /* "renpy/gl/gltexture.pyx":1201
 *     cdef unsigned int *ppend
 * 
 *     surf = PySurface_AsSurface(pysurf)             # <<<<<<<<<<<<<<
 *     pixels = <unsigned char *> surf.pixels
 * 
 */
  __pyx_v_surf = PySurface_AsSurface(__pyx_v_pysurf);

  /* "renpy/gl/gltexture.pyx":1202
 * 
 *     surf = PySurface_AsSurface(pysurf)
 *     pixels = <unsigned char *> surf.pixels             # <<<<<<<<<<<<<<
 * 
 *     # The start of the pixel data to read out.
 */
  __pyx_v_pixels = ((unsigned char *)__pyx_v_surf->pixels);

  /* "renpy/gl/gltexture.pyx":1205
 * 
 *     # The start of the pixel data to read out.
 *     pixels += y * surf.pitch             # <<<<<<<<<<<<<<
 *     pixels += x * 4
 * 
 */
  __pyx_v_pixels = (__pyx_v_pixels + (__pyx_v_y * __pyx_v_surf->pitch));

  /* "renpy/gl/gltexture.pyx":1206
 *     # The start of the pixel data to read out.
 *     pixels += y * surf.pitch
 *     pixels += x * 4             # <<<<<<<<<<<<<<
 * 
 *     # A pointer to the row past the last pixel data we read out.
 */
  __pyx_v_pixels = (__pyx_v_pixels + (__pyx_v_x * 4));

  /* "renpy/gl/gltexture.pyx":1209
 * 
 *     # A pointer to the row past the last pixel data we read out.
 *     pixels_end = pixels + h * surf.pitch             # <<<<<<<<<<<<<<
 * 
 *     # A pointer to the output byte to write.
 */
  __pyx_v_pixels_end = (__pyx_v_pixels + (__pyx_v_h * __pyx_v_surf->pitch));

  /* "renpy/gl/gltexture.pyx":1212
 * 
 *     # A pointer to the output byte to write.
 *     op = out             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_v_op = __pyx_v_out;

  /* "renpy/gl/gltexture.pyx":1214
 *     op = out
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         while pixels < pixels_end:
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      __Pyx_FastGIL_Remember();
      #endif
      /*try:*/ {

        /* "renpy/gl/gltexture.pyx":1216
 *     with nogil:
 * 
 *         while pixels < pixels_end:             # <<<<<<<<<<<<<<
 * 
 *             # The start and end of the current row.
 */
        while (1) {
          __pyx_t_4 = ((__pyx_v_pixels < __pyx_v_pixels_end) != 0);
          if (!__pyx_t_4) break;

          /* "renpy/gl/gltexture.pyx":1219
 * 
 *             # The start and end of the current row.
 *             p = pixels             # <<<<<<<<<<<<<<
 *             pend = p + w * 4
 * 
 */
          __pyx_v_p = __pyx_v_pixels;

          /* "renpy/gl/gltexture.pyx":1220
 *             # The start and end of the current row.
 *             p = pixels
 *             pend = p + w * 4             # <<<<<<<<<<<<<<
 * 
 *             # Advance to the next row.
 */
          __pyx_v_pend = (__pyx_v_p + (__pyx_v_w * 4));

          /* "renpy/gl/gltexture.pyx":1223
 * 
 *             # Advance to the next row.
 *             pixels += surf.pitch             # <<<<<<<<<<<<<<
 * 
 *             if tex_format == GL_RGBA:
 */
          __pyx_v_pixels = (__pyx_v_pixels + __pyx_v_surf->pitch);

          /* "renpy/gl/gltexture.pyx":1225
 *             pixels += surf.pitch
 * 
 *             if tex_format == GL_RGBA:             # <<<<<<<<<<<<<<
 * 
 *                 # RGBA path.
 */
          __pyx_t_4 = ((__pyx_v_5renpy_2gl_9gltexture_tex_format == GL_RGBA) != 0);
          if (__pyx_t_4) {

            /* "renpy/gl/gltexture.pyx":1229
 *                 # RGBA path.
 * 
 *                 if alpha:             # <<<<<<<<<<<<<<
 * 
 *                     while p < pend:
 */
            __pyx_t_4 = (__pyx_v_alpha != 0);
            if (__pyx_t_4) {

              /* "renpy/gl/gltexture.pyx":1231
 *                 if alpha:
 * 
 *                     while p < pend:             # <<<<<<<<<<<<<<
 * 
 *                         a = p[3]
 */
              while (1) {
                __pyx_t_4 = ((__pyx_v_p < __pyx_v_pend) != 0);
                if (!__pyx_t_4) break;

                /* "renpy/gl/gltexture.pyx":1233
 *                     while p < pend:
 * 
 *                         a = p[3]             # <<<<<<<<<<<<<<
 * 
 *                         op[0] = (p[0] * a + a) >> 8
 */
                __pyx_v_a = (__pyx_v_p[3]);

                /* "renpy/gl/gltexture.pyx":1235
 *                         a = p[3]
 * 
 *                         op[0] = (p[0] * a + a) >> 8             # <<<<<<<<<<<<<<
 *                         op[1] = (p[1] * a + a) >> 8
 *                         op[2] = (p[2] * a + a) >> 8
 */
                (__pyx_v_op[0]) = ((((__pyx_v_p[0]) * __pyx_v_a) + __pyx_v_a) >> 8);

                /* "renpy/gl/gltexture.pyx":1236
 * 
 *                         op[0] = (p[0] * a + a) >> 8
 *                         op[1] = (p[1] * a + a) >> 8             # <<<<<<<<<<<<<<
 *                         op[2] = (p[2] * a + a) >> 8
 *                         op[3] = a
 */
                (__pyx_v_op[1]) = ((((__pyx_v_p[1]) * __pyx_v_a) + __pyx_v_a) >> 8);

                /* "renpy/gl/gltexture.pyx":1237
 *                         op[0] = (p[0] * a + a) >> 8
 *                         op[1] = (p[1] * a + a) >> 8
 *                         op[2] = (p[2] * a + a) >> 8             # <<<<<<<<<<<<<<
 *                         op[3] = a
 * 
 */
                (__pyx_v_op[2]) = ((((__pyx_v_p[2]) * __pyx_v_a) + __pyx_v_a) >> 8);

                /* "renpy/gl/gltexture.pyx":1238
 *                         op[1] = (p[1] * a + a) >> 8
 *                         op[2] = (p[2] * a + a) >> 8
 *                         op[3] = a             # <<<<<<<<<<<<<<
 * 
 *                         p += 4
 */
                (__pyx_v_op[3]) = __pyx_v_a;

                /* "renpy/gl/gltexture.pyx":1240
 *                         op[3] = a
 * 
 *                         p += 4             # <<<<<<<<<<<<<<
 *                         op += 4
 * 
 */
                __pyx_v_p = (__pyx_v_p + 4);

                /* "renpy/gl/gltexture.pyx":1241
 * 
 *                         p += 4
 *                         op += 4             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
                __pyx_v_op = (__pyx_v_op + 4);
              }

              /* "renpy/gl/gltexture.pyx":1229
 *                 # RGBA path.
 * 
 *                 if alpha:             # <<<<<<<<<<<<<<
 * 
 *                     while p < pend:
 */
              goto __pyx_L10;
            }

            /* "renpy/gl/gltexture.pyx":1245
 *                 else:
 * 
 *                     while p < pend:             # <<<<<<<<<<<<<<
 * 
 *                         (<unsigned int *> op)[0] = (<unsigned int *> p)[0]
 */
            /*else*/ {
              while (1) {
                __pyx_t_4 = ((__pyx_v_p < __pyx_v_pend) != 0);
                if (!__pyx_t_4) break;

                /* "renpy/gl/gltexture.pyx":1247
 *                     while p < pend:
 * 
 *                         (<unsigned int *> op)[0] = (<unsigned int *> p)[0]             # <<<<<<<<<<<<<<
 *                         op[3] = 255
 * 
 */
                (((unsigned int *)__pyx_v_op)[0]) = (((unsigned int *)__pyx_v_p)[0]);

                /* "renpy/gl/gltexture.pyx":1248
 * 
 *                         (<unsigned int *> op)[0] = (<unsigned int *> p)[0]
 *                         op[3] = 255             # <<<<<<<<<<<<<<
 * 
 *                         p += 4
 */
                (__pyx_v_op[3]) = 0xFF;

                /* "renpy/gl/gltexture.pyx":1250
 *                         op[3] = 255
 * 
 *                         p += 4             # <<<<<<<<<<<<<<
 *                         op += 4
 * 
 */
                __pyx_v_p = (__pyx_v_p + 4);

                /* "renpy/gl/gltexture.pyx":1251
 * 
 *                         p += 4
 *                         op += 4             # <<<<<<<<<<<<<<
 * 
 *             else:
 */
                __pyx_v_op = (__pyx_v_op + 4);
              }
            }
            __pyx_L10:;

            /* "renpy/gl/gltexture.pyx":1225
 *             pixels += surf.pitch
 * 
 *             if tex_format == GL_RGBA:             # <<<<<<<<<<<<<<
 * 
 *                 # RGBA path.
 */
            goto __pyx_L9;
          }

          /* "renpy/gl/gltexture.pyx":1257
 *                 # BGRA Path.
 * 
 *                 if alpha:             # <<<<<<<<<<<<<<
 * 
 *                     while p < pend:
 */
          /*else*/ {
            __pyx_t_4 = (__pyx_v_alpha != 0);
            if (__pyx_t_4) {

              /* "renpy/gl/gltexture.pyx":1259
 *                 if alpha:
 * 
 *                     while p < pend:             # <<<<<<<<<<<<<<
 * 
 *                         a = p[3]
 */
              while (1) {
                __pyx_t_4 = ((__pyx_v_p < __pyx_v_pend) != 0);
                if (!__pyx_t_4) break;

                /* "renpy/gl/gltexture.pyx":1261
 *                     while p < pend:
 * 
 *                         a = p[3]             # <<<<<<<<<<<<<<
 * 
 *                         op[0] = (p[2] * a + a) >> 8 # b
 */
                __pyx_v_a = (__pyx_v_p[3]);

                /* "renpy/gl/gltexture.pyx":1263
 *                         a = p[3]
 * 
 *                         op[0] = (p[2] * a + a) >> 8 # b             # <<<<<<<<<<<<<<
 *                         op[1] = (p[1] * a + a) >> 8 # g
 *                         op[2] = (p[0] * a + a) >> 8 # r
 */
                (__pyx_v_op[0]) = ((((__pyx_v_p[2]) * __pyx_v_a) + __pyx_v_a) >> 8);

                /* "renpy/gl/gltexture.pyx":1264
 * 
 *                         op[0] = (p[2] * a + a) >> 8 # b
 *                         op[1] = (p[1] * a + a) >> 8 # g             # <<<<<<<<<<<<<<
 *                         op[2] = (p[0] * a + a) >> 8 # r
 *                         op[3] = a
 */
                (__pyx_v_op[1]) = ((((__pyx_v_p[1]) * __pyx_v_a) + __pyx_v_a) >> 8);

                /* "renpy/gl/gltexture.pyx":1265
 *                         op[0] = (p[2] * a + a) >> 8 # b
 *                         op[1] = (p[1] * a + a) >> 8 # g
 *                         op[2] = (p[0] * a + a) >> 8 # r             # <<<<<<<<<<<<<<
 *                         op[3] = a
 * 
 */
                (__pyx_v_op[2]) = ((((__pyx_v_p[0]) * __pyx_v_a) + __pyx_v_a) >> 8);

                /* "renpy/gl/gltexture.pyx":1266
 *                         op[1] = (p[1] * a + a) >> 8 # g
 *                         op[2] = (p[0] * a + a) >> 8 # r
 *                         op[3] = a             # <<<<<<<<<<<<<<
 * 
 *                         p += 4
 */
                (__pyx_v_op[3]) = __pyx_v_a;

                /* "renpy/gl/gltexture.pyx":1268
 *                         op[3] = a
 * 
 *                         p += 4             # <<<<<<<<<<<<<<
 *                         op += 4
 * 
 */
                __pyx_v_p = (__pyx_v_p + 4);

                /* "renpy/gl/gltexture.pyx":1269
 * 
 *                         p += 4
 *                         op += 4             # <<<<<<<<<<<<<<
 * 
 *                 else:
 */
                __pyx_v_op = (__pyx_v_op + 4);
              }

              /* "renpy/gl/gltexture.pyx":1257
 *                 # BGRA Path.
 * 
 *                 if alpha:             # <<<<<<<<<<<<<<
 * 
 *                     while p < pend:
 */
              goto __pyx_L15;
            }

            /* "renpy/gl/gltexture.pyx":1273
 *                 else:
 * 
 *                     while p < pend:             # <<<<<<<<<<<<<<
 * 
 *                         op[0] = p[2] # b
 */
            /*else*/ {
              while (1) {
                __pyx_t_4 = ((__pyx_v_p < __pyx_v_pend) != 0);
                if (!__pyx_t_4) break;

                /* "renpy/gl/gltexture.pyx":1275
 *                     while p < pend:
 * 
 *                         op[0] = p[2] # b             # <<<<<<<<<<<<<<
 *                         op[1] = p[1] # g
 *                         op[2] = p[0] # r
 */
                (__pyx_v_op[0]) = (__pyx_v_p[2]);

                /* "renpy/gl/gltexture.pyx":1276
 * 
 *                         op[0] = p[2] # b
 *                         op[1] = p[1] # g             # <<<<<<<<<<<<<<
 *                         op[2] = p[0] # r
 *                         op[3] = 0xff # a
 */
                (__pyx_v_op[1]) = (__pyx_v_p[1]);

                /* "renpy/gl/gltexture.pyx":1277
 *                         op[0] = p[2] # b
 *                         op[1] = p[1] # g
 *                         op[2] = p[0] # r             # <<<<<<<<<<<<<<
 *                         op[3] = 0xff # a
 * 
 */
                (__pyx_v_op[2]) = (__pyx_v_p[0]);

                /* "renpy/gl/gltexture.pyx":1278
 *                         op[1] = p[1] # g
 *                         op[2] = p[0] # r
 *                         op[3] = 0xff # a             # <<<<<<<<<<<<<<
 * 
 *                         p += 4
 */
                (__pyx_v_op[3]) = 0xff;

                /* "renpy/gl/gltexture.pyx":1280
 *                         op[3] = 0xff # a
 * 
 *                         p += 4             # <<<<<<<<<<<<<<
 *                         op += 4
 * 
 */
                __pyx_v_p = (__pyx_v_p + 4);

                /* "renpy/gl/gltexture.pyx":1281
 * 
 *                         p += 4
 *                         op += 4             # <<<<<<<<<<<<<<
 * 
 *         if border_left:
 */
                __pyx_v_op = (__pyx_v_op + 4);
              }
            }
            __pyx_L15:;
          }
          __pyx_L9:;
        }

        /* "renpy/gl/gltexture.pyx":1283
 *                         op += 4
 * 
 *         if border_left:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             ppend = pp + w * h
 */
        __pyx_t_4 = (__pyx_v_border_left != 0);
        if (__pyx_t_4) {

          /* "renpy/gl/gltexture.pyx":1284
 * 
 *         if border_left:
 *             pp = <unsigned int *> (out)             # <<<<<<<<<<<<<<
 *             ppend = pp + w * h
 * 
 */
          __pyx_v_pp = ((unsigned int *)__pyx_v_out);

          /* "renpy/gl/gltexture.pyx":1285
 *         if border_left:
 *             pp = <unsigned int *> (out)
 *             ppend = pp + w * h             # <<<<<<<<<<<<<<
 * 
 *             while pp < ppend:
 */
          __pyx_v_ppend = (__pyx_v_pp + (__pyx_v_w * __pyx_v_h));

          /* "renpy/gl/gltexture.pyx":1287
 *             ppend = pp + w * h
 * 
 *             while pp < ppend:             # <<<<<<<<<<<<<<
 *                 pp[0] = pp[1]
 *                 pp += w
 */
          while (1) {
            __pyx_t_4 = ((__pyx_v_pp < __pyx_v_ppend) != 0);
            if (!__pyx_t_4) break;

            /* "renpy/gl/gltexture.pyx":1288
 * 
 *             while pp < ppend:
 *                 pp[0] = pp[1]             # <<<<<<<<<<<<<<
 *                 pp += w
 * 
 */
            (__pyx_v_pp[0]) = (__pyx_v_pp[1]);

            /* "renpy/gl/gltexture.pyx":1289
 *             while pp < ppend:
 *                 pp[0] = pp[1]
 *                 pp += w             # <<<<<<<<<<<<<<
 * 
 *         if border_right:
 */
            __pyx_v_pp = (__pyx_v_pp + __pyx_v_w);
          }

          /* "renpy/gl/gltexture.pyx":1283
 *                         op += 4
 * 
 *         if border_left:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             ppend = pp + w * h
 */
        }

        /* "renpy/gl/gltexture.pyx":1291
 *                 pp += w
 * 
 *         if border_right:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             pp += w - 2
 */
        __pyx_t_4 = (__pyx_v_border_right != 0);
        if (__pyx_t_4) {

          /* "renpy/gl/gltexture.pyx":1292
 * 
 *         if border_right:
 *             pp = <unsigned int *> (out)             # <<<<<<<<<<<<<<
 *             pp += w - 2
 *             ppend = pp + w * h
 */
          __pyx_v_pp = ((unsigned int *)__pyx_v_out);

          /* "renpy/gl/gltexture.pyx":1293
 *         if border_right:
 *             pp = <unsigned int *> (out)
 *             pp += w - 2             # <<<<<<<<<<<<<<
 *             ppend = pp + w * h
 * 
 */
          __pyx_v_pp = (__pyx_v_pp + (__pyx_v_w - 2));

          /* "renpy/gl/gltexture.pyx":1294
 *             pp = <unsigned int *> (out)
 *             pp += w - 2
 *             ppend = pp + w * h             # <<<<<<<<<<<<<<
 * 
 *             while pp < ppend:
 */
          __pyx_v_ppend = (__pyx_v_pp + (__pyx_v_w * __pyx_v_h));

          /* "renpy/gl/gltexture.pyx":1296
 *             ppend = pp + w * h
 * 
 *             while pp < ppend:             # <<<<<<<<<<<<<<
 *                 pp[1] = pp[0]
 *                 pp += w
 */
          while (1) {
            __pyx_t_4 = ((__pyx_v_pp < __pyx_v_ppend) != 0);
            if (!__pyx_t_4) break;

            /* "renpy/gl/gltexture.pyx":1297
 * 
 *             while pp < ppend:
 *                 pp[1] = pp[0]             # <<<<<<<<<<<<<<
 *                 pp += w
 * 
 */
            (__pyx_v_pp[1]) = (__pyx_v_pp[0]);

            /* "renpy/gl/gltexture.pyx":1298
 *             while pp < ppend:
 *                 pp[1] = pp[0]
 *                 pp += w             # <<<<<<<<<<<<<<
 * 
 *         if border_top:
 */
            __pyx_v_pp = (__pyx_v_pp + __pyx_v_w);
          }

          /* "renpy/gl/gltexture.pyx":1291
 *                 pp += w
 * 
 *         if border_right:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             pp += w - 2
 */
        }

        /* "renpy/gl/gltexture.pyx":1300
 *                 pp += w
 * 
 *         if border_top:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             ppend = pp + w
 */
        __pyx_t_4 = (__pyx_v_border_top != 0);
        if (__pyx_t_4) {

          /* "renpy/gl/gltexture.pyx":1301
 * 
 *         if border_top:
 *             pp = <unsigned int *> (out)             # <<<<<<<<<<<<<<
 *             ppend = pp + w
 * 
 */
          __pyx_v_pp = ((unsigned int *)__pyx_v_out);

          /* "renpy/gl/gltexture.pyx":1302
 *         if border_top:
 *             pp = <unsigned int *> (out)
 *             ppend = pp + w             # <<<<<<<<<<<<<<
 * 
 *             while pp < ppend:
 */
          __pyx_v_ppend = (__pyx_v_pp + __pyx_v_w);

          /* "renpy/gl/gltexture.pyx":1304
 *             ppend = pp + w
 * 
 *             while pp < ppend:             # <<<<<<<<<<<<<<
 *                 pp[0] = pp[w]
 *                 pp += 1
 */
          while (1) {
            __pyx_t_4 = ((__pyx_v_pp < __pyx_v_ppend) != 0);
            if (!__pyx_t_4) break;

            /* "renpy/gl/gltexture.pyx":1305
 * 
 *             while pp < ppend:
 *                 pp[0] = pp[w]             # <<<<<<<<<<<<<<
 *                 pp += 1
 * 
 */
            (__pyx_v_pp[0]) = (__pyx_v_pp[__pyx_v_w]);

            /* "renpy/gl/gltexture.pyx":1306
 *             while pp < ppend:
 *                 pp[0] = pp[w]
 *                 pp += 1             # <<<<<<<<<<<<<<
 * 
 *         if border_bottom:
 */
            __pyx_v_pp = (__pyx_v_pp + 1);
          }

          /* "renpy/gl/gltexture.pyx":1300
 *                 pp += w
 * 
 *         if border_top:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             ppend = pp + w
 */
        }

        /* "renpy/gl/gltexture.pyx":1308
 *                 pp += 1
 * 
 *         if border_bottom:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             pp += (h - 2) * w
 */
        __pyx_t_4 = (__pyx_v_border_bottom != 0);
        if (__pyx_t_4) {

          /* "renpy/gl/gltexture.pyx":1309
 * 
 *         if border_bottom:
 *             pp = <unsigned int *> (out)             # <<<<<<<<<<<<<<
 *             pp += (h - 2) * w
 *             ppend = pp + w
 */
          __pyx_v_pp = ((unsigned int *)__pyx_v_out);

          /* "renpy/gl/gltexture.pyx":1310
 *         if border_bottom:
 *             pp = <unsigned int *> (out)
 *             pp += (h - 2) * w             # <<<<<<<<<<<<<<
 *             ppend = pp + w
 * 
 */
          __pyx_v_pp = (__pyx_v_pp + ((__pyx_v_h - 2) * __pyx_v_w));

          /* "renpy/gl/gltexture.pyx":1311
 *             pp = <unsigned int *> (out)
 *             pp += (h - 2) * w
 *             ppend = pp + w             # <<<<<<<<<<<<<<
 * 
 *             while pp < ppend:
 */
          __pyx_v_ppend = (__pyx_v_pp + __pyx_v_w);

          /* "renpy/gl/gltexture.pyx":1313
 *             ppend = pp + w
 * 
 *             while pp < ppend:             # <<<<<<<<<<<<<<
 *                 pp[w] = pp[0]
 *                 pp += 1
 */
          while (1) {
            __pyx_t_4 = ((__pyx_v_pp < __pyx_v_ppend) != 0);
            if (!__pyx_t_4) break;

            /* "renpy/gl/gltexture.pyx":1314
 * 
 *             while pp < ppend:
 *                 pp[w] = pp[0]             # <<<<<<<<<<<<<<
 *                 pp += 1
 * 
 */
            (__pyx_v_pp[__pyx_v_w]) = (__pyx_v_pp[0]);

            /* "renpy/gl/gltexture.pyx":1315
 *             while pp < ppend:
 *                 pp[w] = pp[0]
 *                 pp += 1             # <<<<<<<<<<<<<<
 * 
 *     return rv
 */
            __pyx_v_pp = (__pyx_v_pp + 1);
          }

          /* "renpy/gl/gltexture.pyx":1308
 *                 pp += 1
 * 
 *         if border_bottom:             # <<<<<<<<<<<<<<
 *             pp = <unsigned int *> (out)
 *             pp += (h - 2) * w
 */
        }
      }

      /* "renpy/gl/gltexture.pyx":1214
 *     op = out
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         while pixels < pixels_end:
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          __Pyx_FastGIL_Forget();
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "renpy/gl/gltexture.pyx":1317
 *                 pp += 1
 * 
 *     return rv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rv);
  __pyx_r = __pyx_v_rv;
  goto __pyx_L0;

  /* "renpy/gl/gltexture.pyx":1148
 * 
 * 
 * def premultiply(             # <<<<<<<<<<<<<<
 *     object pysurf,
 *     int x,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("renpy.gl.gltexture.premultiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_rv);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":1320
 * 
 * 
 * def load_premultiplied(data, width, height, update, rtt):             # <<<<<<<<<<<<<<
 * 
 *     cdef char *pixels
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_31load_premultiplied(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_31load_premultiplied = {"load_premultiplied", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_31load_premultiplied, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_31load_premultiplied(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_data = 0;
  PyObject *__pyx_v_width = 0;
  PyObject *__pyx_v_height = 0;
  PyObject *__pyx_v_update = 0;
  PyObject *__pyx_v_rtt = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("load_premultiplied (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_data,&__pyx_n_s_width,&__pyx_n_s_height,&__pyx_n_s_update,&__pyx_n_s_rtt,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_data)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_premultiplied", 1, 5, 5, 1); __PYX_ERR(0, 1320, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_premultiplied", 1, 5, 5, 2); __PYX_ERR(0, 1320, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_update)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_premultiplied", 1, 5, 5, 3); __PYX_ERR(0, 1320, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_rtt)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("load_premultiplied", 1, 5, 5, 4); __PYX_ERR(0, 1320, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "load_premultiplied") < 0)) __PYX_ERR(0, 1320, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_data = values[0];
    __pyx_v_width = values[1];
    __pyx_v_height = values[2];
    __pyx_v_update = values[3];
    __pyx_v_rtt = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("load_premultiplied", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 1320, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.load_premultiplied", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_30load_premultiplied(__pyx_self, __pyx_v_data, __pyx_v_width, __pyx_v_height, __pyx_v_update, __pyx_v_rtt);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_30load_premultiplied(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data, PyObject *__pyx_v_width, PyObject *__pyx_v_height, PyObject *__pyx_v_update, PyObject *__pyx_v_rtt) {
  char *__pyx_v_pixels;
  GLenum __pyx_v_internalformat;
  GLenum __pyx_v_format;
  GLenum __pyx_v_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  char *__pyx_t_2;
  GLsizei __pyx_t_3;
  GLsizei __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("load_premultiplied", 0);

  /* "renpy/gl/gltexture.pyx":1328
 *     cdef GLenum type
 * 
 *     if rtt:             # <<<<<<<<<<<<<<
 *         internalformat = rtt_internalformat
 *         format = rtt_format
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_rtt); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1328, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":1329
 * 
 *     if rtt:
 *         internalformat = rtt_internalformat             # <<<<<<<<<<<<<<
 *         format = rtt_format
 *         type = rtt_type
 */
    __pyx_v_internalformat = __pyx_v_5renpy_2gl_9gltexture_rtt_internalformat;

    /* "renpy/gl/gltexture.pyx":1330
 *     if rtt:
 *         internalformat = rtt_internalformat
 *         format = rtt_format             # <<<<<<<<<<<<<<
 *         type = rtt_type
 *     else:
 */
    __pyx_v_format = __pyx_v_5renpy_2gl_9gltexture_rtt_format;

    /* "renpy/gl/gltexture.pyx":1331
 *         internalformat = rtt_internalformat
 *         format = rtt_format
 *         type = rtt_type             # <<<<<<<<<<<<<<
 *     else:
 *         internalformat = tex_internalformat
 */
    __pyx_v_type = __pyx_v_5renpy_2gl_9gltexture_rtt_type;

    /* "renpy/gl/gltexture.pyx":1328
 *     cdef GLenum type
 * 
 *     if rtt:             # <<<<<<<<<<<<<<
 *         internalformat = rtt_internalformat
 *         format = rtt_format
 */
    goto __pyx_L3;
  }

  /* "renpy/gl/gltexture.pyx":1333
 *         type = rtt_type
 *     else:
 *         internalformat = tex_internalformat             # <<<<<<<<<<<<<<
 *         format = tex_format
 *         type = tex_type
 */
  /*else*/ {
    __pyx_v_internalformat = __pyx_v_5renpy_2gl_9gltexture_tex_internalformat;

    /* "renpy/gl/gltexture.pyx":1334
 *     else:
 *         internalformat = tex_internalformat
 *         format = tex_format             # <<<<<<<<<<<<<<
 *         type = tex_type
 * 
 */
    __pyx_v_format = __pyx_v_5renpy_2gl_9gltexture_tex_format;

    /* "renpy/gl/gltexture.pyx":1335
 *         internalformat = tex_internalformat
 *         format = tex_format
 *         type = tex_type             # <<<<<<<<<<<<<<
 * 
 *     if data:
 */
    __pyx_v_type = __pyx_v_5renpy_2gl_9gltexture_tex_type;
  }
  __pyx_L3:;

  /* "renpy/gl/gltexture.pyx":1337
 *         type = tex_type
 * 
 *     if data:             # <<<<<<<<<<<<<<
 *         pixels = data
 *     else:
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_data); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1337, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":1338
 * 
 *     if data:
 *         pixels = data             # <<<<<<<<<<<<<<
 *     else:
 *         pixels = NULL
 */
    __pyx_t_2 = __Pyx_PyObject_AsWritableString(__pyx_v_data); if (unlikely((!__pyx_t_2) && PyErr_Occurred())) __PYX_ERR(0, 1338, __pyx_L1_error)
    __pyx_v_pixels = __pyx_t_2;

    /* "renpy/gl/gltexture.pyx":1337
 *         type = tex_type
 * 
 *     if data:             # <<<<<<<<<<<<<<
 *         pixels = data
 *     else:
 */
    goto __pyx_L4;
  }

  /* "renpy/gl/gltexture.pyx":1340
 *         pixels = data
 *     else:
 *         pixels = NULL             # <<<<<<<<<<<<<<
 * 
 *     if update:
 */
  /*else*/ {
    __pyx_v_pixels = NULL;
  }
  __pyx_L4:;

  /* "renpy/gl/gltexture.pyx":1342
 *         pixels = NULL
 * 
 *     if update:             # <<<<<<<<<<<<<<
 *         glTexSubImage2D(
 *             GL_TEXTURE_2D,
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_update); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 1342, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "renpy/gl/gltexture.pyx":1348
 *             0,
 *             0,
 *             width,             # <<<<<<<<<<<<<<
 *             height,
 *             format,
 */
    __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1348, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1349
 *             0,
 *             width,
 *             height,             # <<<<<<<<<<<<<<
 *             format,
 *             type,
 */
    __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1349, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1343
 * 
 *     if update:
 *         glTexSubImage2D(             # <<<<<<<<<<<<<<
 *             GL_TEXTURE_2D,
 *             0,
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, __pyx_t_3, __pyx_t_4, __pyx_v_format, __pyx_v_type, ((GLubyte *)__pyx_v_pixels)); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1343, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1342
 *         pixels = NULL
 * 
 *     if update:             # <<<<<<<<<<<<<<
 *         glTexSubImage2D(
 *             GL_TEXTURE_2D,
 */
    goto __pyx_L5;
  }

  /* "renpy/gl/gltexture.pyx":1355
 * 
 *     else:
 *         glTexImage2D(             # <<<<<<<<<<<<<<
 *             GL_TEXTURE_2D,
 *             0,
 */
  /*else*/ {

    /* "renpy/gl/gltexture.pyx":1359
 *             0,
 *             internalformat,
 *             width,             # <<<<<<<<<<<<<<
 *             height,
 *             0,
 */
    __pyx_t_4 = __Pyx_PyInt_As_GLsizei(__pyx_v_width); if (unlikely((__pyx_t_4 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1359, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1360
 *             internalformat,
 *             width,
 *             height,             # <<<<<<<<<<<<<<
 *             0,
 *             format,
 */
    __pyx_t_3 = __Pyx_PyInt_As_GLsizei(__pyx_v_height); if (unlikely((__pyx_t_3 == ((GLsizei)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1360, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1355
 * 
 *     else:
 *         glTexImage2D(             # <<<<<<<<<<<<<<
 *             GL_TEXTURE_2D,
 *             0,
 */
    __pyx_t_5 = __pyx_f_5renpy_2gl_2gl_glTexImage2D(GL_TEXTURE_2D, 0, __pyx_v_internalformat, __pyx_t_4, __pyx_t_3, 0, __pyx_v_format, __pyx_v_type, ((GLubyte *)__pyx_v_pixels)); if (unlikely(__pyx_t_5 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1355, __pyx_L1_error)
  }
  __pyx_L5:;

  /* "renpy/gl/gltexture.pyx":1320
 * 
 * 
 * def load_premultiplied(data, width, height, update, rtt):             # <<<<<<<<<<<<<<
 * 
 *     cdef char *pixels
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.load_premultiplied", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/gl/gltexture.pyx":1366
 *             <GLubyte *> pixels)
 * 
 * cdef void draw_rectangle(             # <<<<<<<<<<<<<<
 *     Environ environ,
 *     double sx,
 */

static void __pyx_f_5renpy_2gl_9gltexture_draw_rectangle(struct __pyx_obj_5renpy_2gl_6gldraw_Environ *__pyx_v_environ, double __pyx_v_sx, double __pyx_v_sy, double __pyx_v_x, double __pyx_v_y, double __pyx_v_w, double __pyx_v_h, struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_transform, struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_tex0, float __pyx_v_tex0x, float __pyx_v_tex0y, struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_tex1, float __pyx_v_tex1x, float __pyx_v_tex1y, struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v_tex2, float __pyx_v_tex2x, float __pyx_v_tex2y) {
  int __pyx_v_has_tex0;
  int __pyx_v_has_tex1;
  int __pyx_v_has_tex2;
  double __pyx_v_t0u0;
  double __pyx_v_t0v0;
  double __pyx_v_t0u1;
  double __pyx_v_t0v1;
  double __pyx_v_t1u0;
  double __pyx_v_t1v0;
  double __pyx_v_t1u1;
  double __pyx_v_t1v1;
  double __pyx_v_t2u0;
  double __pyx_v_t2v0;
  double __pyx_v_t2u1;
  double __pyx_v_t2v1;
  double __pyx_v_xdx;
  double __pyx_v_xdy;
  double __pyx_v_ydx;
  double __pyx_v_ydy;
  double __pyx_v_x0;
  double __pyx_v_y0;
  double __pyx_v_x1;
  double __pyx_v_y1;
  double __pyx_v_x2;
  double __pyx_v_y2;
  double __pyx_v_x3;
  double __pyx_v_y3;
  double __pyx_v_xadd;
  double __pyx_v_yadd;
  double __pyx_v_xmul;
  double __pyx_v_ymul;
  GLfloat __pyx_v_tex0coords[8];
  GLfloat __pyx_v_tex1coords[8];
  GLfloat __pyx_v_tex2coords[8];
  GLfloat __pyx_v_vcoords[8];
  __Pyx_RefNannyDeclarations
  float __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  double __pyx_t_5;
  __Pyx_RefNannySetupContext("draw_rectangle", 0);

  /* "renpy/gl/gltexture.pyx":1407
 * 
 *     # Texture coordinates.
 *     cdef double t0u0 = 0, t0v0 = 0, t0u1 = 0, t0v1 = 0             # <<<<<<<<<<<<<<
 *     cdef double t1u0 = 0, t1v0 = 0, t1u1 = 0, t1v1 = 0
 *     cdef double t2u0 = 0, t2v0 = 0, t2u1 = 0, t2v1 = 0
 */
  __pyx_v_t0u0 = 0.0;
  __pyx_v_t0v0 = 0.0;
  __pyx_v_t0u1 = 0.0;
  __pyx_v_t0v1 = 0.0;

  /* "renpy/gl/gltexture.pyx":1408
 *     # Texture coordinates.
 *     cdef double t0u0 = 0, t0v0 = 0, t0u1 = 0, t0v1 = 0
 *     cdef double t1u0 = 0, t1v0 = 0, t1u1 = 0, t1v1 = 0             # <<<<<<<<<<<<<<
 *     cdef double t2u0 = 0, t2v0 = 0, t2u1 = 0, t2v1 = 0
 * 
 */
  __pyx_v_t1u0 = 0.0;
  __pyx_v_t1v0 = 0.0;
  __pyx_v_t1u1 = 0.0;
  __pyx_v_t1v1 = 0.0;

  /* "renpy/gl/gltexture.pyx":1409
 *     cdef double t0u0 = 0, t0v0 = 0, t0u1 = 0, t0v1 = 0
 *     cdef double t1u0 = 0, t1v0 = 0, t1u1 = 0, t1v1 = 0
 *     cdef double t2u0 = 0, t2v0 = 0, t2u1 = 0, t2v1 = 0             # <<<<<<<<<<<<<<
 * 
 *     # Pull apart the transform.
 */
  __pyx_v_t2u0 = 0.0;
  __pyx_v_t2v0 = 0.0;
  __pyx_v_t2u1 = 0.0;
  __pyx_v_t2v1 = 0.0;

  /* "renpy/gl/gltexture.pyx":1412
 * 
 *     # Pull apart the transform.
 *     cdef double xdx = transform.xdx             # <<<<<<<<<<<<<<
 *     cdef double xdy = transform.xdy
 *     cdef double ydx = transform.ydx
 */
  __pyx_t_1 = __pyx_v_transform->xdx;
  __pyx_v_xdx = __pyx_t_1;

  /* "renpy/gl/gltexture.pyx":1413
 *     # Pull apart the transform.
 *     cdef double xdx = transform.xdx
 *     cdef double xdy = transform.xdy             # <<<<<<<<<<<<<<
 *     cdef double ydx = transform.ydx
 *     cdef double ydy = transform.ydy
 */
  __pyx_t_1 = __pyx_v_transform->xdy;
  __pyx_v_xdy = __pyx_t_1;

  /* "renpy/gl/gltexture.pyx":1414
 *     cdef double xdx = transform.xdx
 *     cdef double xdy = transform.xdy
 *     cdef double ydx = transform.ydx             # <<<<<<<<<<<<<<
 *     cdef double ydy = transform.ydy
 * 
 */
  __pyx_t_1 = __pyx_v_transform->ydx;
  __pyx_v_ydx = __pyx_t_1;

  /* "renpy/gl/gltexture.pyx":1415
 *     cdef double xdy = transform.xdy
 *     cdef double ydx = transform.ydx
 *     cdef double ydy = transform.ydy             # <<<<<<<<<<<<<<
 * 
 *     # Transform the vertex coordinates to screen-space.
 */
  __pyx_t_1 = __pyx_v_transform->ydy;
  __pyx_v_ydy = __pyx_t_1;

  /* "renpy/gl/gltexture.pyx":1418
 * 
 *     # Transform the vertex coordinates to screen-space.
 *     cdef double x0 = (x + 0) * xdx + (y + 0) * xdy + sx             # <<<<<<<<<<<<<<
 *     cdef double y0 = (x + 0) * ydx + (y + 0) * ydy + sy
 * 
 */
  __pyx_v_x0 = ((((__pyx_v_x + 0.0) * __pyx_v_xdx) + ((__pyx_v_y + 0.0) * __pyx_v_xdy)) + __pyx_v_sx);

  /* "renpy/gl/gltexture.pyx":1419
 *     # Transform the vertex coordinates to screen-space.
 *     cdef double x0 = (x + 0) * xdx + (y + 0) * xdy + sx
 *     cdef double y0 = (x + 0) * ydx + (y + 0) * ydy + sy             # <<<<<<<<<<<<<<
 * 
 *     cdef double x1 = (x + w) * xdx + (y + 0) * xdy + sx
 */
  __pyx_v_y0 = ((((__pyx_v_x + 0.0) * __pyx_v_ydx) + ((__pyx_v_y + 0.0) * __pyx_v_ydy)) + __pyx_v_sy);

  /* "renpy/gl/gltexture.pyx":1421
 *     cdef double y0 = (x + 0) * ydx + (y + 0) * ydy + sy
 * 
 *     cdef double x1 = (x + w) * xdx + (y + 0) * xdy + sx             # <<<<<<<<<<<<<<
 *     cdef double y1 = (x + w) * ydx + (y + 0) * ydy + sy
 * 
 */
  __pyx_v_x1 = ((((__pyx_v_x + __pyx_v_w) * __pyx_v_xdx) + ((__pyx_v_y + 0.0) * __pyx_v_xdy)) + __pyx_v_sx);

  /* "renpy/gl/gltexture.pyx":1422
 * 
 *     cdef double x1 = (x + w) * xdx + (y + 0) * xdy + sx
 *     cdef double y1 = (x + w) * ydx + (y + 0) * ydy + sy             # <<<<<<<<<<<<<<
 * 
 *     cdef double x2 = (x + 0) * xdx + (y + h) * xdy + sx
 */
  __pyx_v_y1 = ((((__pyx_v_x + __pyx_v_w) * __pyx_v_ydx) + ((__pyx_v_y + 0.0) * __pyx_v_ydy)) + __pyx_v_sy);

  /* "renpy/gl/gltexture.pyx":1424
 *     cdef double y1 = (x + w) * ydx + (y + 0) * ydy + sy
 * 
 *     cdef double x2 = (x + 0) * xdx + (y + h) * xdy + sx             # <<<<<<<<<<<<<<
 *     cdef double y2 = (x + 0) * ydx + (y + h) * ydy + sy
 * 
 */
  __pyx_v_x2 = ((((__pyx_v_x + 0.0) * __pyx_v_xdx) + ((__pyx_v_y + __pyx_v_h) * __pyx_v_xdy)) + __pyx_v_sx);

  /* "renpy/gl/gltexture.pyx":1425
 * 
 *     cdef double x2 = (x + 0) * xdx + (y + h) * xdy + sx
 *     cdef double y2 = (x + 0) * ydx + (y + h) * ydy + sy             # <<<<<<<<<<<<<<
 * 
 *     cdef double x3 = (x + w) * xdx + (y + h) * xdy + sx
 */
  __pyx_v_y2 = ((((__pyx_v_x + 0.0) * __pyx_v_ydx) + ((__pyx_v_y + __pyx_v_h) * __pyx_v_ydy)) + __pyx_v_sy);

  /* "renpy/gl/gltexture.pyx":1427
 *     cdef double y2 = (x + 0) * ydx + (y + h) * ydy + sy
 * 
 *     cdef double x3 = (x + w) * xdx + (y + h) * xdy + sx             # <<<<<<<<<<<<<<
 *     cdef double y3 = (x + w) * ydx + (y + h) * ydy + sy
 * 
 */
  __pyx_v_x3 = ((((__pyx_v_x + __pyx_v_w) * __pyx_v_xdx) + ((__pyx_v_y + __pyx_v_h) * __pyx_v_xdy)) + __pyx_v_sx);

  /* "renpy/gl/gltexture.pyx":1428
 * 
 *     cdef double x3 = (x + w) * xdx + (y + h) * xdy + sx
 *     cdef double y3 = (x + w) * ydx + (y + h) * ydy + sy             # <<<<<<<<<<<<<<
 * 
 *     # Compute the texture coordinates, and set up the textures.
 */
  __pyx_v_y3 = ((((__pyx_v_x + __pyx_v_w) * __pyx_v_ydx) + ((__pyx_v_y + __pyx_v_h) * __pyx_v_ydy)) + __pyx_v_sy);

  /* "renpy/gl/gltexture.pyx":1433
 *     cdef double xadd, yadd, xmul, ymul
 * 
 *     if tex0 is not None:             # <<<<<<<<<<<<<<
 * 
 *         has_tex0 = 1
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_tex0) != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":1435
 *     if tex0 is not None:
 * 
 *         has_tex0 = 1             # <<<<<<<<<<<<<<
 * 
 *         glActiveTextureARB(GL_TEXTURE0)
 */
    __pyx_v_has_tex0 = 1;

    /* "renpy/gl/gltexture.pyx":1437
 *         has_tex0 = 1
 * 
 *         glActiveTextureARB(GL_TEXTURE0)             # <<<<<<<<<<<<<<
 *         glBindTexture(GL_TEXTURE_2D, tex0.number)
 * 
 */
    __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glActiveTextureARB(GL_TEXTURE0); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1437, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1438
 * 
 *         glActiveTextureARB(GL_TEXTURE0)
 *         glBindTexture(GL_TEXTURE_2D, tex0.number)             # <<<<<<<<<<<<<<
 * 
 *         xadd = tex0.xadd
 */
    __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_tex0->number); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1438, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1440
 *         glBindTexture(GL_TEXTURE_2D, tex0.number)
 * 
 *         xadd = tex0.xadd             # <<<<<<<<<<<<<<
 *         yadd = tex0.yadd
 *         xmul = tex0.xmul
 */
    __pyx_t_5 = __pyx_v_tex0->xadd;
    __pyx_v_xadd = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1441
 * 
 *         xadd = tex0.xadd
 *         yadd = tex0.yadd             # <<<<<<<<<<<<<<
 *         xmul = tex0.xmul
 *         ymul = tex0.ymul
 */
    __pyx_t_5 = __pyx_v_tex0->yadd;
    __pyx_v_yadd = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1442
 *         xadd = tex0.xadd
 *         yadd = tex0.yadd
 *         xmul = tex0.xmul             # <<<<<<<<<<<<<<
 *         ymul = tex0.ymul
 * 
 */
    __pyx_t_5 = __pyx_v_tex0->xmul;
    __pyx_v_xmul = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1443
 *         yadd = tex0.yadd
 *         xmul = tex0.xmul
 *         ymul = tex0.ymul             # <<<<<<<<<<<<<<
 * 
 *         t0u0 = xadd + xmul * (tex0x + 0)
 */
    __pyx_t_5 = __pyx_v_tex0->ymul;
    __pyx_v_ymul = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1445
 *         ymul = tex0.ymul
 * 
 *         t0u0 = xadd + xmul * (tex0x + 0)             # <<<<<<<<<<<<<<
 *         t0u1 = xadd + xmul * (tex0x + w)
 *         t0v0 = yadd + ymul * (tex0y + 0)
 */
    __pyx_v_t0u0 = (__pyx_v_xadd + (__pyx_v_xmul * (__pyx_v_tex0x + 0.0)));

    /* "renpy/gl/gltexture.pyx":1446
 * 
 *         t0u0 = xadd + xmul * (tex0x + 0)
 *         t0u1 = xadd + xmul * (tex0x + w)             # <<<<<<<<<<<<<<
 *         t0v0 = yadd + ymul * (tex0y + 0)
 *         t0v1 = yadd + ymul * (tex0y + h)
 */
    __pyx_v_t0u1 = (__pyx_v_xadd + (__pyx_v_xmul * (__pyx_v_tex0x + __pyx_v_w)));

    /* "renpy/gl/gltexture.pyx":1447
 *         t0u0 = xadd + xmul * (tex0x + 0)
 *         t0u1 = xadd + xmul * (tex0x + w)
 *         t0v0 = yadd + ymul * (tex0y + 0)             # <<<<<<<<<<<<<<
 *         t0v1 = yadd + ymul * (tex0y + h)
 * 
 */
    __pyx_v_t0v0 = (__pyx_v_yadd + (__pyx_v_ymul * (__pyx_v_tex0y + 0.0)));

    /* "renpy/gl/gltexture.pyx":1448
 *         t0u1 = xadd + xmul * (tex0x + w)
 *         t0v0 = yadd + ymul * (tex0y + 0)
 *         t0v1 = yadd + ymul * (tex0y + h)             # <<<<<<<<<<<<<<
 * 
 *     else:
 */
    __pyx_v_t0v1 = (__pyx_v_yadd + (__pyx_v_ymul * (__pyx_v_tex0y + __pyx_v_h)));

    /* "renpy/gl/gltexture.pyx":1433
 *     cdef double xadd, yadd, xmul, ymul
 * 
 *     if tex0 is not None:             # <<<<<<<<<<<<<<
 * 
 *         has_tex0 = 1
 */
    goto __pyx_L3;
  }

  /* "renpy/gl/gltexture.pyx":1451
 * 
 *     else:
 *         has_tex0 = 0             # <<<<<<<<<<<<<<
 * 
 *     if tex1 is not None:
 */
  /*else*/ {
    __pyx_v_has_tex0 = 0;
  }
  __pyx_L3:;

  /* "renpy/gl/gltexture.pyx":1453
 *         has_tex0 = 0
 * 
 *     if tex1 is not None:             # <<<<<<<<<<<<<<
 * 
 *         has_tex1 = 1
 */
  __pyx_t_3 = (((PyObject *)__pyx_v_tex1) != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "renpy/gl/gltexture.pyx":1455
 *     if tex1 is not None:
 * 
 *         has_tex1 = 1             # <<<<<<<<<<<<<<
 * 
 *         glActiveTextureARB(GL_TEXTURE1)
 */
    __pyx_v_has_tex1 = 1;

    /* "renpy/gl/gltexture.pyx":1457
 *         has_tex1 = 1
 * 
 *         glActiveTextureARB(GL_TEXTURE1)             # <<<<<<<<<<<<<<
 *         glBindTexture(GL_TEXTURE_2D, tex1.number)
 * 
 */
    __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glActiveTextureARB(GL_TEXTURE1); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1457, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1458
 * 
 *         glActiveTextureARB(GL_TEXTURE1)
 *         glBindTexture(GL_TEXTURE_2D, tex1.number)             # <<<<<<<<<<<<<<
 * 
 *         xadd = tex1.xadd
 */
    __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_tex1->number); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1458, __pyx_L1_error)

    /* "renpy/gl/gltexture.pyx":1460
 *         glBindTexture(GL_TEXTURE_2D, tex1.number)
 * 
 *         xadd = tex1.xadd             # <<<<<<<<<<<<<<
 *         yadd = tex1.yadd
 *         xmul = tex1.xmul
 */
    __pyx_t_5 = __pyx_v_tex1->xadd;
    __pyx_v_xadd = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1461
 * 
 *         xadd = tex1.xadd
 *         yadd = tex1.yadd             # <<<<<<<<<<<<<<
 *         xmul = tex1.xmul
 *         ymul = tex1.ymul
 */
    __pyx_t_5 = __pyx_v_tex1->yadd;
    __pyx_v_yadd = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1462
 *         xadd = tex1.xadd
 *         yadd = tex1.yadd
 *         xmul = tex1.xmul             # <<<<<<<<<<<<<<
 *         ymul = tex1.ymul
 * 
 */
    __pyx_t_5 = __pyx_v_tex1->xmul;
    __pyx_v_xmul = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1463
 *         yadd = tex1.yadd
 *         xmul = tex1.xmul
 *         ymul = tex1.ymul             # <<<<<<<<<<<<<<
 * 
 *         t1u0 = xadd + xmul * (tex1x + 0)
 */
    __pyx_t_5 = __pyx_v_tex1->ymul;
    __pyx_v_ymul = __pyx_t_5;

    /* "renpy/gl/gltexture.pyx":1465
 *         ymul = tex1.ymul
 * 
 *         t1u0 = xadd + xmul * (tex1x + 0)             # <<<<<<<<<<<<<<
 *         t1u1 = xadd + xmul * (tex1x + w)
 *         t1v0 = yadd + ymul * (tex1y + 0)
 */
    __pyx_v_t1u0 = (__pyx_v_xadd + (__pyx_v_xmul * (__pyx_v_tex1x + 0.0)));

    /* "renpy/gl/gltexture.pyx":1466
 * 
 *         t1u0 = xadd + xmul * (tex1x + 0)
 *         t1u1 = xadd + xmul * (tex1x + w)             # <<<<<<<<<<<<<<
 *         t1v0 = yadd + ymul * (tex1y + 0)
 *         t1v1 = yadd + ymul * (tex1y + h)
 */
    __pyx_v_t1u1 = (__pyx_v_xadd + (__pyx_v_xmul * (__pyx_v_tex1x + __pyx_v_w)));

    /* "renpy/gl/gltexture.pyx":1467
 *         t1u0 = xadd + xmul * (tex1x + 0)
 *         t1u1 = xadd + xmul * (tex1x + w)
 *         t1v0 = yadd + ymul * (tex1y + 0)             # <<<<<<<<<<<<<<
 *         t1v1 = yadd + ymul * (tex1y + h)
 * 
 */
    __pyx_v_t1v0 = (__pyx_v_yadd + (__pyx_v_ymul * (__pyx_v_tex1y + 0.0)));

    /* "renpy/gl/gltexture.pyx":1468
 *         t1u1 = xadd + xmul * (tex1x + w)
 *         t1v0 = yadd + ymul * (tex1y + 0)
 *         t1v1 = yadd + ymul * (tex1y + h)             # <<<<<<<<<<<<<<
 * 
 *     else:
 */
    __pyx_v_t1v1 = (__pyx_v_yadd + (__pyx_v_ymul * (__pyx_v_tex1y + __pyx_v_h)));

    /* "renpy/gl/gltexture.pyx":1453
 *         has_tex0 = 0
 * 
 *     if tex1 is not None:             # <<<<<<<<<<<<<<
 * 
 *         has_tex1 = 1
 */
    goto __pyx_L4;
  }

  /* "renpy/gl/gltexture.pyx":1471
 * 
 *     else:
 *         has_tex1 = 0             # <<<<<<<<<<<<<<
 * 
 *     if RENPY_THIRD_TEXTURE:
 */
  /*else*/ {
    __pyx_v_has_tex1 = 0;
  }
  __pyx_L4:;

  /* "renpy/gl/gltexture.pyx":1473
 *         has_tex1 = 0
 * 
 *     if RENPY_THIRD_TEXTURE:             # <<<<<<<<<<<<<<
 *         if tex2 is not None:
 * 
 */
  if (RENPY_THIRD_TEXTURE) {

    /* "renpy/gl/gltexture.pyx":1474
 * 
 *     if RENPY_THIRD_TEXTURE:
 *         if tex2 is not None:             # <<<<<<<<<<<<<<
 * 
 *             has_tex2 = 1
 */
    __pyx_t_2 = (((PyObject *)__pyx_v_tex2) != Py_None);
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":1476
 *         if tex2 is not None:
 * 
 *             has_tex2 = 1             # <<<<<<<<<<<<<<
 * 
 *             glActiveTextureARB(GL_TEXTURE2)
 */
      __pyx_v_has_tex2 = 1;

      /* "renpy/gl/gltexture.pyx":1478
 *             has_tex2 = 1
 * 
 *             glActiveTextureARB(GL_TEXTURE2)             # <<<<<<<<<<<<<<
 *             glBindTexture(GL_TEXTURE_2D, tex2.number)
 * 
 */
      __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glActiveTextureARB(GL_TEXTURE2); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1478, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1479
 * 
 *             glActiveTextureARB(GL_TEXTURE2)
 *             glBindTexture(GL_TEXTURE_2D, tex2.number)             # <<<<<<<<<<<<<<
 * 
 *             xadd = tex2.xadd
 */
      __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glBindTexture(GL_TEXTURE_2D, __pyx_v_tex2->number); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1479, __pyx_L1_error)

      /* "renpy/gl/gltexture.pyx":1481
 *             glBindTexture(GL_TEXTURE_2D, tex2.number)
 * 
 *             xadd = tex2.xadd             # <<<<<<<<<<<<<<
 *             yadd = tex2.yadd
 *             xmul = tex2.xmul
 */
      __pyx_t_5 = __pyx_v_tex2->xadd;
      __pyx_v_xadd = __pyx_t_5;

      /* "renpy/gl/gltexture.pyx":1482
 * 
 *             xadd = tex2.xadd
 *             yadd = tex2.yadd             # <<<<<<<<<<<<<<
 *             xmul = tex2.xmul
 *             ymul = tex2.ymul
 */
      __pyx_t_5 = __pyx_v_tex2->yadd;
      __pyx_v_yadd = __pyx_t_5;

      /* "renpy/gl/gltexture.pyx":1483
 *             xadd = tex2.xadd
 *             yadd = tex2.yadd
 *             xmul = tex2.xmul             # <<<<<<<<<<<<<<
 *             ymul = tex2.ymul
 * 
 */
      __pyx_t_5 = __pyx_v_tex2->xmul;
      __pyx_v_xmul = __pyx_t_5;

      /* "renpy/gl/gltexture.pyx":1484
 *             yadd = tex2.yadd
 *             xmul = tex2.xmul
 *             ymul = tex2.ymul             # <<<<<<<<<<<<<<
 * 
 *             t2u0 = xadd + xmul * (tex2x + 0)
 */
      __pyx_t_5 = __pyx_v_tex2->ymul;
      __pyx_v_ymul = __pyx_t_5;

      /* "renpy/gl/gltexture.pyx":1486
 *             ymul = tex2.ymul
 * 
 *             t2u0 = xadd + xmul * (tex2x + 0)             # <<<<<<<<<<<<<<
 *             t2u1 = xadd + xmul * (tex2x + w)
 *             t2v0 = yadd + ymul * (tex2y + 0)
 */
      __pyx_v_t2u0 = (__pyx_v_xadd + (__pyx_v_xmul * (__pyx_v_tex2x + 0.0)));

      /* "renpy/gl/gltexture.pyx":1487
 * 
 *             t2u0 = xadd + xmul * (tex2x + 0)
 *             t2u1 = xadd + xmul * (tex2x + w)             # <<<<<<<<<<<<<<
 *             t2v0 = yadd + ymul * (tex2y + 0)
 *             t2v1 = yadd + ymul * (tex2y + h)
 */
      __pyx_v_t2u1 = (__pyx_v_xadd + (__pyx_v_xmul * (__pyx_v_tex2x + __pyx_v_w)));

      /* "renpy/gl/gltexture.pyx":1488
 *             t2u0 = xadd + xmul * (tex2x + 0)
 *             t2u1 = xadd + xmul * (tex2x + w)
 *             t2v0 = yadd + ymul * (tex2y + 0)             # <<<<<<<<<<<<<<
 *             t2v1 = yadd + ymul * (tex2y + h)
 * 
 */
      __pyx_v_t2v0 = (__pyx_v_yadd + (__pyx_v_ymul * (__pyx_v_tex2y + 0.0)));

      /* "renpy/gl/gltexture.pyx":1489
 *             t2u1 = xadd + xmul * (tex2x + w)
 *             t2v0 = yadd + ymul * (tex2y + 0)
 *             t2v1 = yadd + ymul * (tex2y + h)             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
      __pyx_v_t2v1 = (__pyx_v_yadd + (__pyx_v_ymul * (__pyx_v_tex2y + __pyx_v_h)));

      /* "renpy/gl/gltexture.pyx":1474
 * 
 *     if RENPY_THIRD_TEXTURE:
 *         if tex2 is not None:             # <<<<<<<<<<<<<<
 * 
 *             has_tex2 = 1
 */
      goto __pyx_L6;
    }

    /* "renpy/gl/gltexture.pyx":1492
 * 
 *         else:
 *             has_tex2 = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_v_has_tex2 = 0;
    }
    __pyx_L6:;

    /* "renpy/gl/gltexture.pyx":1473
 *         has_tex1 = 0
 * 
 *     if RENPY_THIRD_TEXTURE:             # <<<<<<<<<<<<<<
 *         if tex2 is not None:
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":1502
 *     cdef GLfloat vcoords[8]
 * 
 *     if has_tex0:             # <<<<<<<<<<<<<<
 *         tex0coords[0] = t0u0
 *         tex0coords[1] = t0v0
 */
  __pyx_t_3 = (__pyx_v_has_tex0 != 0);
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":1503
 * 
 *     if has_tex0:
 *         tex0coords[0] = t0u0             # <<<<<<<<<<<<<<
 *         tex0coords[1] = t0v0
 *         tex0coords[2] = t0u1
 */
    (__pyx_v_tex0coords[0]) = __pyx_v_t0u0;

    /* "renpy/gl/gltexture.pyx":1504
 *     if has_tex0:
 *         tex0coords[0] = t0u0
 *         tex0coords[1] = t0v0             # <<<<<<<<<<<<<<
 *         tex0coords[2] = t0u1
 *         tex0coords[3] = t0v0
 */
    (__pyx_v_tex0coords[1]) = __pyx_v_t0v0;

    /* "renpy/gl/gltexture.pyx":1505
 *         tex0coords[0] = t0u0
 *         tex0coords[1] = t0v0
 *         tex0coords[2] = t0u1             # <<<<<<<<<<<<<<
 *         tex0coords[3] = t0v0
 *         tex0coords[4] = t0u0
 */
    (__pyx_v_tex0coords[2]) = __pyx_v_t0u1;

    /* "renpy/gl/gltexture.pyx":1506
 *         tex0coords[1] = t0v0
 *         tex0coords[2] = t0u1
 *         tex0coords[3] = t0v0             # <<<<<<<<<<<<<<
 *         tex0coords[4] = t0u0
 *         tex0coords[5] = t0v1
 */
    (__pyx_v_tex0coords[3]) = __pyx_v_t0v0;

    /* "renpy/gl/gltexture.pyx":1507
 *         tex0coords[2] = t0u1
 *         tex0coords[3] = t0v0
 *         tex0coords[4] = t0u0             # <<<<<<<<<<<<<<
 *         tex0coords[5] = t0v1
 *         tex0coords[6] = t0u1
 */
    (__pyx_v_tex0coords[4]) = __pyx_v_t0u0;

    /* "renpy/gl/gltexture.pyx":1508
 *         tex0coords[3] = t0v0
 *         tex0coords[4] = t0u0
 *         tex0coords[5] = t0v1             # <<<<<<<<<<<<<<
 *         tex0coords[6] = t0u1
 *         tex0coords[7] = t0v1
 */
    (__pyx_v_tex0coords[5]) = __pyx_v_t0v1;

    /* "renpy/gl/gltexture.pyx":1509
 *         tex0coords[4] = t0u0
 *         tex0coords[5] = t0v1
 *         tex0coords[6] = t0u1             # <<<<<<<<<<<<<<
 *         tex0coords[7] = t0v1
 * 
 */
    (__pyx_v_tex0coords[6]) = __pyx_v_t0u1;

    /* "renpy/gl/gltexture.pyx":1510
 *         tex0coords[5] = t0v1
 *         tex0coords[6] = t0u1
 *         tex0coords[7] = t0v1             # <<<<<<<<<<<<<<
 * 
 *         environ.set_texture(0, tex0coords)
 */
    (__pyx_v_tex0coords[7]) = __pyx_v_t0v1;

    /* "renpy/gl/gltexture.pyx":1512
 *         tex0coords[7] = t0v1
 * 
 *         environ.set_texture(0, tex0coords)             # <<<<<<<<<<<<<<
 *     else:
 *         environ.set_texture(0, NULL)
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 0, __pyx_v_tex0coords);

    /* "renpy/gl/gltexture.pyx":1502
 *     cdef GLfloat vcoords[8]
 * 
 *     if has_tex0:             # <<<<<<<<<<<<<<
 *         tex0coords[0] = t0u0
 *         tex0coords[1] = t0v0
 */
    goto __pyx_L7;
  }

  /* "renpy/gl/gltexture.pyx":1514
 *         environ.set_texture(0, tex0coords)
 *     else:
 *         environ.set_texture(0, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 0, NULL);
  }
  __pyx_L7:;

  /* "renpy/gl/gltexture.pyx":1517
 * 
 * 
 *     if has_tex1:             # <<<<<<<<<<<<<<
 *         tex1coords[0] = t1u0
 *         tex1coords[1] = t1v0
 */
  __pyx_t_3 = (__pyx_v_has_tex1 != 0);
  if (__pyx_t_3) {

    /* "renpy/gl/gltexture.pyx":1518
 * 
 *     if has_tex1:
 *         tex1coords[0] = t1u0             # <<<<<<<<<<<<<<
 *         tex1coords[1] = t1v0
 *         tex1coords[2] = t1u1
 */
    (__pyx_v_tex1coords[0]) = __pyx_v_t1u0;

    /* "renpy/gl/gltexture.pyx":1519
 *     if has_tex1:
 *         tex1coords[0] = t1u0
 *         tex1coords[1] = t1v0             # <<<<<<<<<<<<<<
 *         tex1coords[2] = t1u1
 *         tex1coords[3] = t1v0
 */
    (__pyx_v_tex1coords[1]) = __pyx_v_t1v0;

    /* "renpy/gl/gltexture.pyx":1520
 *         tex1coords[0] = t1u0
 *         tex1coords[1] = t1v0
 *         tex1coords[2] = t1u1             # <<<<<<<<<<<<<<
 *         tex1coords[3] = t1v0
 *         tex1coords[4] = t1u0
 */
    (__pyx_v_tex1coords[2]) = __pyx_v_t1u1;

    /* "renpy/gl/gltexture.pyx":1521
 *         tex1coords[1] = t1v0
 *         tex1coords[2] = t1u1
 *         tex1coords[3] = t1v0             # <<<<<<<<<<<<<<
 *         tex1coords[4] = t1u0
 *         tex1coords[5] = t1v1
 */
    (__pyx_v_tex1coords[3]) = __pyx_v_t1v0;

    /* "renpy/gl/gltexture.pyx":1522
 *         tex1coords[2] = t1u1
 *         tex1coords[3] = t1v0
 *         tex1coords[4] = t1u0             # <<<<<<<<<<<<<<
 *         tex1coords[5] = t1v1
 *         tex1coords[6] = t1u1
 */
    (__pyx_v_tex1coords[4]) = __pyx_v_t1u0;

    /* "renpy/gl/gltexture.pyx":1523
 *         tex1coords[3] = t1v0
 *         tex1coords[4] = t1u0
 *         tex1coords[5] = t1v1             # <<<<<<<<<<<<<<
 *         tex1coords[6] = t1u1
 *         tex1coords[7] = t1v1
 */
    (__pyx_v_tex1coords[5]) = __pyx_v_t1v1;

    /* "renpy/gl/gltexture.pyx":1524
 *         tex1coords[4] = t1u0
 *         tex1coords[5] = t1v1
 *         tex1coords[6] = t1u1             # <<<<<<<<<<<<<<
 *         tex1coords[7] = t1v1
 * 
 */
    (__pyx_v_tex1coords[6]) = __pyx_v_t1u1;

    /* "renpy/gl/gltexture.pyx":1525
 *         tex1coords[5] = t1v1
 *         tex1coords[6] = t1u1
 *         tex1coords[7] = t1v1             # <<<<<<<<<<<<<<
 * 
 *         environ.set_texture(1, tex1coords)
 */
    (__pyx_v_tex1coords[7]) = __pyx_v_t1v1;

    /* "renpy/gl/gltexture.pyx":1527
 *         tex1coords[7] = t1v1
 * 
 *         environ.set_texture(1, tex1coords)             # <<<<<<<<<<<<<<
 *     else:
 *         environ.set_texture(1, NULL)
 */
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 1, __pyx_v_tex1coords);

    /* "renpy/gl/gltexture.pyx":1517
 * 
 * 
 *     if has_tex1:             # <<<<<<<<<<<<<<
 *         tex1coords[0] = t1u0
 *         tex1coords[1] = t1v0
 */
    goto __pyx_L8;
  }

  /* "renpy/gl/gltexture.pyx":1529
 *         environ.set_texture(1, tex1coords)
 *     else:
 *         environ.set_texture(1, NULL)             # <<<<<<<<<<<<<<
 * 
 *     if RENPY_THIRD_TEXTURE:
 */
  /*else*/ {
    ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 1, NULL);
  }
  __pyx_L8:;

  /* "renpy/gl/gltexture.pyx":1531
 *         environ.set_texture(1, NULL)
 * 
 *     if RENPY_THIRD_TEXTURE:             # <<<<<<<<<<<<<<
 *         if has_tex2:
 * 
 */
  if (RENPY_THIRD_TEXTURE) {

    /* "renpy/gl/gltexture.pyx":1532
 * 
 *     if RENPY_THIRD_TEXTURE:
 *         if has_tex2:             # <<<<<<<<<<<<<<
 * 
 *             tex2coords[0] = t2u0
 */
    __pyx_t_3 = (__pyx_v_has_tex2 != 0);
    if (__pyx_t_3) {

      /* "renpy/gl/gltexture.pyx":1534
 *         if has_tex2:
 * 
 *             tex2coords[0] = t2u0             # <<<<<<<<<<<<<<
 *             tex2coords[1] = t2v0
 *             tex2coords[2] = t2u1
 */
      (__pyx_v_tex2coords[0]) = __pyx_v_t2u0;

      /* "renpy/gl/gltexture.pyx":1535
 * 
 *             tex2coords[0] = t2u0
 *             tex2coords[1] = t2v0             # <<<<<<<<<<<<<<
 *             tex2coords[2] = t2u1
 *             tex2coords[3] = t2v0
 */
      (__pyx_v_tex2coords[1]) = __pyx_v_t2v0;

      /* "renpy/gl/gltexture.pyx":1536
 *             tex2coords[0] = t2u0
 *             tex2coords[1] = t2v0
 *             tex2coords[2] = t2u1             # <<<<<<<<<<<<<<
 *             tex2coords[3] = t2v0
 *             tex2coords[4] = t2u0
 */
      (__pyx_v_tex2coords[2]) = __pyx_v_t2u1;

      /* "renpy/gl/gltexture.pyx":1537
 *             tex2coords[1] = t2v0
 *             tex2coords[2] = t2u1
 *             tex2coords[3] = t2v0             # <<<<<<<<<<<<<<
 *             tex2coords[4] = t2u0
 *             tex2coords[5] = t2v1
 */
      (__pyx_v_tex2coords[3]) = __pyx_v_t2v0;

      /* "renpy/gl/gltexture.pyx":1538
 *             tex2coords[2] = t2u1
 *             tex2coords[3] = t2v0
 *             tex2coords[4] = t2u0             # <<<<<<<<<<<<<<
 *             tex2coords[5] = t2v1
 *             tex2coords[6] = t2u1
 */
      (__pyx_v_tex2coords[4]) = __pyx_v_t2u0;

      /* "renpy/gl/gltexture.pyx":1539
 *             tex2coords[3] = t2v0
 *             tex2coords[4] = t2u0
 *             tex2coords[5] = t2v1             # <<<<<<<<<<<<<<
 *             tex2coords[6] = t2u1
 *             tex2coords[7] = t2v1
 */
      (__pyx_v_tex2coords[5]) = __pyx_v_t2v1;

      /* "renpy/gl/gltexture.pyx":1540
 *             tex2coords[4] = t2u0
 *             tex2coords[5] = t2v1
 *             tex2coords[6] = t2u1             # <<<<<<<<<<<<<<
 *             tex2coords[7] = t2v1
 * 
 */
      (__pyx_v_tex2coords[6]) = __pyx_v_t2u1;

      /* "renpy/gl/gltexture.pyx":1541
 *             tex2coords[5] = t2v1
 *             tex2coords[6] = t2u1
 *             tex2coords[7] = t2v1             # <<<<<<<<<<<<<<
 * 
 *             environ.set_texture(2, tex2coords)
 */
      (__pyx_v_tex2coords[7]) = __pyx_v_t2v1;

      /* "renpy/gl/gltexture.pyx":1543
 *             tex2coords[7] = t2v1
 * 
 *             environ.set_texture(2, tex2coords)             # <<<<<<<<<<<<<<
 *         else:
 *             environ.set_texture(2, NULL)
 */
      ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 2, __pyx_v_tex2coords);

      /* "renpy/gl/gltexture.pyx":1532
 * 
 *     if RENPY_THIRD_TEXTURE:
 *         if has_tex2:             # <<<<<<<<<<<<<<
 * 
 *             tex2coords[0] = t2u0
 */
      goto __pyx_L10;
    }

    /* "renpy/gl/gltexture.pyx":1545
 *             environ.set_texture(2, tex2coords)
 *         else:
 *             environ.set_texture(2, NULL)             # <<<<<<<<<<<<<<
 * 
 *     vcoords[0] = x0
 */
    /*else*/ {
      ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_texture(__pyx_v_environ, 2, NULL);
    }
    __pyx_L10:;

    /* "renpy/gl/gltexture.pyx":1531
 *         environ.set_texture(1, NULL)
 * 
 *     if RENPY_THIRD_TEXTURE:             # <<<<<<<<<<<<<<
 *         if has_tex2:
 * 
 */
  }

  /* "renpy/gl/gltexture.pyx":1547
 *             environ.set_texture(2, NULL)
 * 
 *     vcoords[0] = x0             # <<<<<<<<<<<<<<
 *     vcoords[1] = y0
 *     vcoords[2] = x1
 */
  (__pyx_v_vcoords[0]) = __pyx_v_x0;

  /* "renpy/gl/gltexture.pyx":1548
 * 
 *     vcoords[0] = x0
 *     vcoords[1] = y0             # <<<<<<<<<<<<<<
 *     vcoords[2] = x1
 *     vcoords[3] = y1
 */
  (__pyx_v_vcoords[1]) = __pyx_v_y0;

  /* "renpy/gl/gltexture.pyx":1549
 *     vcoords[0] = x0
 *     vcoords[1] = y0
 *     vcoords[2] = x1             # <<<<<<<<<<<<<<
 *     vcoords[3] = y1
 *     vcoords[4] = x2
 */
  (__pyx_v_vcoords[2]) = __pyx_v_x1;

  /* "renpy/gl/gltexture.pyx":1550
 *     vcoords[1] = y0
 *     vcoords[2] = x1
 *     vcoords[3] = y1             # <<<<<<<<<<<<<<
 *     vcoords[4] = x2
 *     vcoords[5] = y2
 */
  (__pyx_v_vcoords[3]) = __pyx_v_y1;

  /* "renpy/gl/gltexture.pyx":1551
 *     vcoords[2] = x1
 *     vcoords[3] = y1
 *     vcoords[4] = x2             # <<<<<<<<<<<<<<
 *     vcoords[5] = y2
 *     vcoords[6] = x3
 */
  (__pyx_v_vcoords[4]) = __pyx_v_x2;

  /* "renpy/gl/gltexture.pyx":1552
 *     vcoords[3] = y1
 *     vcoords[4] = x2
 *     vcoords[5] = y2             # <<<<<<<<<<<<<<
 *     vcoords[6] = x3
 *     vcoords[7] = y3
 */
  (__pyx_v_vcoords[5]) = __pyx_v_y2;

  /* "renpy/gl/gltexture.pyx":1553
 *     vcoords[4] = x2
 *     vcoords[5] = y2
 *     vcoords[6] = x3             # <<<<<<<<<<<<<<
 *     vcoords[7] = y3
 * 
 */
  (__pyx_v_vcoords[6]) = __pyx_v_x3;

  /* "renpy/gl/gltexture.pyx":1554
 *     vcoords[5] = y2
 *     vcoords[6] = x3
 *     vcoords[7] = y3             # <<<<<<<<<<<<<<
 * 
 *     environ.set_vertex(vcoords)
 */
  (__pyx_v_vcoords[7]) = __pyx_v_y3;

  /* "renpy/gl/gltexture.pyx":1556
 *     vcoords[7] = y3
 * 
 *     environ.set_vertex(vcoords)             # <<<<<<<<<<<<<<
 * 
 *     glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)
 */
  ((struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ *)__pyx_v_environ->__pyx_vtab)->set_vertex(__pyx_v_environ, __pyx_v_vcoords);

  /* "renpy/gl/gltexture.pyx":1558
 *     environ.set_vertex(vcoords)
 * 
 *     glDrawArrays(GL_TRIANGLE_STRIP, 0, 4)             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_t_4 = __pyx_f_5renpy_2gl_2gl_glDrawArrays(GL_TRIANGLE_STRIP, 0, 4); if (unlikely(__pyx_t_4 == ((int)0) && PyErr_Occurred())) __PYX_ERR(0, 1558, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":1366
 *             <GLubyte *> pixels)
 * 
 * cdef void draw_rectangle(             # <<<<<<<<<<<<<<
 *     Environ environ,
 *     double sx,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("renpy.gl.gltexture.draw_rectangle", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "(tree fragment)":1
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4447e15:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_33__pyx_unpickle_TextureCore(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_33__pyx_unpickle_TextureCore = {"__pyx_unpickle_TextureCore", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_33__pyx_unpickle_TextureCore, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_33__pyx_unpickle_TextureCore(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_TextureCore (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TextureCore", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TextureCore", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_TextureCore") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TextureCore", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.__pyx_unpickle_TextureCore", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_32__pyx_unpickle_TextureCore(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_32__pyx_unpickle_TextureCore(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_TextureCore", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x4447e15:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x4447e15) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x4447e15:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)
 *     __pyx_result = TextureCore.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x4447e15:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = TextureCore.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x44, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x4447e15:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)
 *     __pyx_result = TextureCore.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureCore), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)
 *     __pyx_result = TextureCore.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = TextureCore.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureCore__set_state(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4447e15 = (debug, format, free_list, generation, height, nearest, number, premult, premult_bottom, premult_left, premult_right, premult_size, premult_top, width, xadd, xmul, yadd, ymul))" % __pyx_checksum)
 *     __pyx_result = TextureCore.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4447e15:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.gl.gltexture.__pyx_unpickle_TextureCore", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureCore__set_state(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  unsigned int __pyx_t_4;
  double __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_TextureCore__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->debug);
  __Pyx_DECREF(__pyx_v___pyx_result->debug);
  __pyx_v___pyx_result->debug = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->format = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->free_list);
  __Pyx_DECREF(__pyx_v___pyx_result->free_list);
  __pyx_v___pyx_result->free_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->generation = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->height = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->nearest = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_4 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->number = __pyx_t_4;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->premult);
  __Pyx_DECREF(__pyx_v___pyx_result->premult);
  __pyx_v___pyx_result->premult = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->premult_bottom = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->premult_left = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->premult_right = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->premult_size);
  __Pyx_DECREF(__pyx_v___pyx_result->premult_size);
  __pyx_v___pyx_result->premult_size = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->premult_top = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->width = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->xadd = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->xmul = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->yadd = __pyx_t_5;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->ymul = __pyx_t_5;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 11, __pyx_L1_error)
  }
  __pyx_t_6 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(2, 11, __pyx_L1_error)
  __pyx_t_7 = ((__pyx_t_6 > 18) != 0);
  if (__pyx_t_7) {
  } else {
    __pyx_t_3 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(2, 11, __pyx_L1_error)
  __pyx_t_8 = (__pyx_t_7 != 0);
  __pyx_t_3 = __pyx_t_8;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":12
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[18])             # <<<<<<<<<<<<<<
 */
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_update); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 12, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    if (!__pyx_t_11) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
        PyObject *__pyx_temp[2] = {__pyx_t_11, __pyx_t_9};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else
      #endif
      {
        __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[18])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("renpy.gl.gltexture.__pyx_unpickle_TextureCore__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_TextureGrid(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x88e5af7:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_35__pyx_unpickle_TextureGrid(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_5renpy_2gl_9gltexture_35__pyx_unpickle_TextureGrid = {"__pyx_unpickle_TextureGrid", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_35__pyx_unpickle_TextureGrid, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_5renpy_2gl_9gltexture_35__pyx_unpickle_TextureGrid(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_TextureGrid (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TextureGrid", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TextureGrid", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_TextureGrid") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_TextureGrid", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("renpy.gl.gltexture.__pyx_unpickle_TextureGrid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5renpy_2gl_9gltexture_34__pyx_unpickle_TextureGrid(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_2gl_9gltexture_34__pyx_unpickle_TextureGrid(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_TextureGrid", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_TextureGrid(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x88e5af7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x88e5af7) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_TextureGrid(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x88e5af7:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)
 *     __pyx_result = TextureGrid.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x88e5af7:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = TextureGrid.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x88, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_TextureGrid(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x88e5af7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)
 *     __pyx_result = TextureGrid.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureGrid), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)
 *     __pyx_result = TextureGrid.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = TextureGrid.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureGrid__set_state(((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x88e5af7 = (columns, debug, half_cache, height, ready, rows, tiles, width))" % __pyx_checksum)
 *     __pyx_result = TextureGrid.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_TextureGrid(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x88e5af7:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("renpy.gl.gltexture.__pyx_unpickle_TextureGrid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_5renpy_2gl_9gltexture___pyx_unpickle_TextureGrid__set_state(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_TextureGrid__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->columns);
  __Pyx_DECREF(__pyx_v___pyx_result->columns);
  __pyx_v___pyx_result->columns = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->debug);
  __Pyx_DECREF(__pyx_v___pyx_result->debug);
  __pyx_v___pyx_result->debug = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid))))) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->half_cache);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->half_cache));
  __pyx_v___pyx_result->half_cache = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->height = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->ready = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->rows);
  __Pyx_DECREF(__pyx_v___pyx_result->rows);
  __pyx_v___pyx_result->rows = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->tiles);
  __Pyx_DECREF(__pyx_v___pyx_result->tiles);
  __pyx_v___pyx_result->tiles = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->width = __pyx_t_2;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 11, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 11, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 8) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 11, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":12
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[8])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 12, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    if (!__pyx_t_9) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_7};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
        PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_7};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else
      #endif
      {
        __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
        __pyx_t_7 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[8])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_TextureGrid__set_state(<TextureGrid> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureGrid__set_state(TextureGrid __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.columns = __pyx_state[0]; __pyx_result.debug = __pyx_state[1]; __pyx_result.half_cache = __pyx_state[2]; __pyx_result.height = __pyx_state[3]; __pyx_result.ready = __pyx_state[4]; __pyx_result.rows = __pyx_state[5]; __pyx_result.tiles = __pyx_state[6]; __pyx_result.width = __pyx_state[7]
 *     if len(__pyx_state) > 8 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("renpy.gl.gltexture.__pyx_unpickle_TextureGrid__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/display/matrix.pxd":26
 * 
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform4(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float *__pyx_v_ow, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform4", 0);

  /* "renpy/display/matrix.pxd":27
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":28
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":29
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw             # <<<<<<<<<<<<<<
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 */
  (__pyx_v_oz[0]) = ((((__pyx_v_x * __pyx_v_self->zdx) + (__pyx_v_y * __pyx_v_self->zdy)) + (__pyx_v_z * __pyx_v_self->zdz)) + (__pyx_v_w * __pyx_v_self->zdw));

  /* "renpy/display/matrix.pxd":30
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 */
  (__pyx_v_ow[0]) = ((((__pyx_v_x * __pyx_v_self->wdx) + (__pyx_v_y * __pyx_v_self->wdy)) + (__pyx_v_z * __pyx_v_self->wdz)) + (__pyx_v_w * __pyx_v_self->wdw));

  /* "renpy/display/matrix.pxd":26
 * 
 * 
 *     cdef inline void transform4(Matrix self, float *ox, float *oy, float *oz, float *ow, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/display/matrix.pxd":32
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform3(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float *__pyx_v_oz, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform3", 0);

  /* "renpy/display/matrix.pxd":33
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":34
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":35
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw             # <<<<<<<<<<<<<<
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 */
  (__pyx_v_oz[0]) = ((((__pyx_v_x * __pyx_v_self->zdx) + (__pyx_v_y * __pyx_v_self->zdy)) + (__pyx_v_z * __pyx_v_self->zdz)) + (__pyx_v_w * __pyx_v_self->zdw));

  /* "renpy/display/matrix.pxd":32
 *         ow[0] = x * self.wdx + y * self.wdy + z * self.wdz + w * self.wdw
 * 
 *     cdef inline void transform3(Matrix self, float *ox, float *oy, float *oz, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

static CYTHON_INLINE void __pyx_f_5renpy_7display_6matrix_6Matrix_transform2(struct __pyx_obj_5renpy_7display_6matrix_Matrix *__pyx_v_self, float *__pyx_v_ox, float *__pyx_v_oy, float __pyx_v_x, float __pyx_v_y, float __pyx_v_z, float __pyx_v_w) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform2", 0);

  /* "renpy/display/matrix.pxd":38
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw             # <<<<<<<<<<<<<<
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 * 
 */
  (__pyx_v_ox[0]) = ((((__pyx_v_x * __pyx_v_self->xdx) + (__pyx_v_y * __pyx_v_self->xdy)) + (__pyx_v_z * __pyx_v_self->xdz)) + (__pyx_v_w * __pyx_v_self->xdw));

  /* "renpy/display/matrix.pxd":39
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_unit_aligned(Matrix self)
 */
  (__pyx_v_oy[0]) = ((((__pyx_v_x * __pyx_v_self->ydx) + (__pyx_v_y * __pyx_v_self->ydy)) + (__pyx_v_z * __pyx_v_self->ydz)) + (__pyx_v_w * __pyx_v_self->ydw));

  /* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}
static struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureCore __pyx_vtable_5renpy_2gl_9gltexture_TextureCore;

static PyObject *__pyx_tp_new_5renpy_2gl_9gltexture_TextureCore(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)o);
  p->__pyx_vtab = __pyx_vtabptr_5renpy_2gl_9gltexture_TextureCore;
  p->premult = Py_None; Py_INCREF(Py_None);
  p->premult_size = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->free_list = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->debug = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5renpy_2gl_9gltexture_TextureCore(PyObject *o) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *p = (struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->premult);
  Py_CLEAR(p->premult_size);
  Py_CLEAR(p->free_list);
  Py_CLEAR(p->debug);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5renpy_2gl_9gltexture_TextureCore(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *p = (struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)o;
  if (p->premult) {
    e = (*v)(p->premult, a); if (e) return e;
  }
  if (p->premult_size) {
    e = (*v)(p->premult_size, a); if (e) return e;
  }
  if (p->free_list) {
    e = (*v)(p->free_list, a); if (e) return e;
  }
  if (p->debug) {
    e = (*v)(p->debug, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_2gl_9gltexture_TextureCore(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *p = (struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *)o;
  tmp = ((PyObject*)p->premult);
  p->premult = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->premult_size);
  p->premult_size = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->free_list);
  p->free_list = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->debug);
  p->debug = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5width_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5width_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_height(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6height_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_height(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6height_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_generation(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_10generation_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_generation(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_10generation_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_number(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6number_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_number(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_6number_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_free_list(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_free_list(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9free_list_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_debug(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_debug(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5debug_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_5renpy_2gl_9gltexture_TextureCore[] = {
  {"__del__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_3__del__, METH_NOARGS, 0},
  {"load_surface", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_5load_surface, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_11TextureCore_4load_surface},
  {"render_to", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_7render_to, METH_VARARGS|METH_KEYWORDS, 0},
  {"allocate", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_9allocate, METH_NOARGS, __pyx_doc_5renpy_2gl_9gltexture_11TextureCore_8allocate},
  {"deallocate", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_11deallocate, METH_NOARGS, __pyx_doc_5renpy_2gl_9gltexture_11TextureCore_10deallocate},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureCore_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5renpy_2gl_9gltexture_TextureCore[] = {
  {(char *)"width", __pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_width, __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_width, (char *)0, 0},
  {(char *)"height", __pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_height, __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_height, (char *)0, 0},
  {(char *)"generation", __pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_generation, __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_generation, (char *)0, 0},
  {(char *)"number", __pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_number, __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_number, (char *)0, 0},
  {(char *)"free_list", __pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_free_list, __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_free_list, (char *)0, 0},
  {(char *)"debug", __pyx_getprop_5renpy_2gl_9gltexture_11TextureCore_debug, __pyx_setprop_5renpy_2gl_9gltexture_11TextureCore_debug, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_2gl_9gltexture_TextureCore = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl.gltexture.TextureCore", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_2gl_9gltexture_TextureCore, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    This object stores information about an OpenGL texture.\n    ", /*tp_doc*/
  __pyx_tp_traverse_5renpy_2gl_9gltexture_TextureCore, /*tp_traverse*/
  __pyx_tp_clear_5renpy_2gl_9gltexture_TextureCore, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_2gl_9gltexture_TextureCore, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5renpy_2gl_9gltexture_TextureCore, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_2gl_9gltexture_11TextureCore_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_2gl_9gltexture_TextureCore, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5renpy_2gl_9gltexture_TextureGrid __pyx_vtable_5renpy_2gl_9gltexture_TextureGrid;

static PyObject *__pyx_tp_new_5renpy_2gl_9gltexture_TextureGrid(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)o);
  p->__pyx_vtab = __pyx_vtabptr_5renpy_2gl_9gltexture_TextureGrid;
  p->rows = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->columns = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->tiles = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->half_cache = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)Py_None); Py_INCREF(Py_None);
  p->debug = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5renpy_2gl_9gltexture_TextureGrid(PyObject *o) {
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *p = (struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  Py_CLEAR(p->rows);
  Py_CLEAR(p->columns);
  Py_CLEAR(p->tiles);
  Py_CLEAR(p->half_cache);
  Py_CLEAR(p->debug);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5renpy_2gl_9gltexture_TextureGrid(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *p = (struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)o;
  if (p->rows) {
    e = (*v)(p->rows, a); if (e) return e;
  }
  if (p->columns) {
    e = (*v)(p->columns, a); if (e) return e;
  }
  if (p->tiles) {
    e = (*v)(p->tiles, a); if (e) return e;
  }
  if (p->half_cache) {
    e = (*v)(((PyObject *)p->half_cache), a); if (e) return e;
  }
  if (p->debug) {
    e = (*v)(p->debug, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5renpy_2gl_9gltexture_TextureGrid(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *p = (struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)o;
  tmp = ((PyObject*)p->rows);
  p->rows = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->columns);
  p->columns = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->tiles);
  p->tiles = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->half_cache);
  p->half_cache = ((struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->debug);
  p->debug = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5width_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5width_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_height(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_6height_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_height(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_6height_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_half_cache(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_half_cache(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_10half_cache_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_debug(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_debug(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5debug_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_ready(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5ready_1__get__(o);
}

static int __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_ready(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5ready_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5renpy_2gl_9gltexture_TextureGrid[] = {
  {"__getstate__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_3__getstate__, METH_NOARGS, 0},
  {"__setstate__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_5__setstate__, METH_O, 0},
  {"get_size", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_7get_size, METH_NOARGS, 0},
  {"subsurface", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_9subsurface, METH_O, __pyx_doc_5renpy_2gl_9gltexture_11TextureGrid_8subsurface},
  {"make_ready", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_11make_ready, METH_O, __pyx_doc_5renpy_2gl_9gltexture_11TextureGrid_10make_ready},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5renpy_2gl_9gltexture_TextureGrid[] = {
  {(char *)"width", __pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_width, __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_width, (char *)0, 0},
  {(char *)"height", __pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_height, __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_height, (char *)0, 0},
  {(char *)"half_cache", __pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_half_cache, __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_half_cache, (char *)0, 0},
  {(char *)"debug", __pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_debug, __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_debug, (char *)0, 0},
  {(char *)"ready", __pyx_getprop_5renpy_2gl_9gltexture_11TextureGrid_ready, __pyx_setprop_5renpy_2gl_9gltexture_11TextureGrid_ready, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_2gl_9gltexture_TextureGrid = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.gl.gltexture.TextureGrid", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_2gl_9gltexture_TextureGrid, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    This represents one or more textures that cover a rectangular\n    area.\n    ", /*tp_doc*/
  __pyx_tp_traverse_5renpy_2gl_9gltexture_TextureGrid, /*tp_traverse*/
  __pyx_tp_clear_5renpy_2gl_9gltexture_TextureGrid, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_2gl_9gltexture_TextureGrid, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5renpy_2gl_9gltexture_TextureGrid, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5renpy_2gl_9gltexture_11TextureGrid_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_2gl_9gltexture_TextureGrid, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"blit", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_23blit, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_22blit},
  {"blend", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_25blend, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_24blend},
  {"imageblend", (PyCFunction)__pyx_pw_5renpy_2gl_9gltexture_27imageblend, METH_VARARGS|METH_KEYWORDS, __pyx_doc_5renpy_2gl_9gltexture_26imageblend},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_gltexture(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_gltexture},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "gltexture",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_0_px_textures_work, __pyx_k_0_px_textures_work, sizeof(__pyx_k_0_px_textures_work), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_t_pickle_a_texture, __pyx_k_Can_t_pickle_a_texture, sizeof(__pyx_k_Can_t_pickle_a_texture), 0, 0, 1, 0},
  {&__pyx_kp_s_Could_not_allocate_0_px_bitmap, __pyx_k_Could_not_allocate_0_px_bitmap, sizeof(__pyx_k_Could_not_allocate_0_px_bitmap), 0, 0, 1, 0},
  {&__pyx_kp_s_Error_drawing_0_px_texture_1_x, __pyx_k_Error_drawing_0_px_texture_1_x, sizeof(__pyx_k_Error_drawing_0_px_texture_1_x), 0, 0, 1, 0},
  {&__pyx_kp_s_Error_loading_0_px_bitmap_1_x, __pyx_k_Error_loading_0_px_bitmap_1_x, sizeof(__pyx_k_Error_loading_0_px_bitmap_1_x), 0, 0, 1, 0},
  {&__pyx_kp_s_Error_reading_0_px_texture_1_x, __pyx_k_Error_reading_0_px_texture_1_x, sizeof(__pyx_k_Error_reading_0_px_texture_1_x), 0, 0, 1, 0},
  {&__pyx_kp_s_Hardware_max_texture_size_d, __pyx_k_Hardware_max_texture_size_d, sizeof(__pyx_k_Hardware_max_texture_size_d), 0, 0, 1, 0},
  {&__pyx_kp_s_Ignored_error_at_start_of_testi, __pyx_k_Ignored_error_at_start_of_testi, sizeof(__pyx_k_Ignored_error_at_start_of_testi), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x44, __pyx_k_Incompatible_checksums_s_vs_0x44, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x44), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x88, __pyx_k_Incompatible_checksums_s_vs_0x88, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x88), 0, 0, 1, 0},
  {&__pyx_kp_s_Incorrect_pixel_color_in_0_px_t, __pyx_k_Incorrect_pixel_color_in_0_px_t, sizeof(__pyx_k_Incorrect_pixel_color_in_0_px_t), 0, 0, 1, 0},
  {&__pyx_n_s_MAX_SIZE, __pyx_k_MAX_SIZE, sizeof(__pyx_k_MAX_SIZE), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_SIZES, __pyx_k_SIZES, sizeof(__pyx_k_SIZES), 0, 0, 1, 1},
  {&__pyx_n_s_StopIteration, __pyx_k_StopIteration, sizeof(__pyx_k_StopIteration), 0, 0, 1, 1},
  {&__pyx_n_s_Texture, __pyx_k_Texture, sizeof(__pyx_k_Texture), 0, 0, 1, 1},
  {&__pyx_n_s_Texture___getstate, __pyx_k_Texture___getstate, sizeof(__pyx_k_Texture___getstate), 0, 0, 1, 1},
  {&__pyx_n_s_Texture___setstate, __pyx_k_Texture___setstate, sizeof(__pyx_k_Texture___setstate), 0, 0, 1, 1},
  {&__pyx_n_s_Texture___sizeof, __pyx_k_Texture___sizeof, sizeof(__pyx_k_Texture___sizeof), 0, 0, 1, 1},
  {&__pyx_kp_s_Texture_testing, __pyx_k_Texture_testing, sizeof(__pyx_k_Texture_testing), 0, 0, 1, 0},
  {&__pyx_kp_s_Textures_are_not_rendering_prope, __pyx_k_Textures_are_not_rendering_prope, sizeof(__pyx_k_Textures_are_not_rendering_prope), 0, 0, 1, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_We_need_to_be_a_real_python_cla, __pyx_k_We_need_to_be_a_real_python_cla, sizeof(__pyx_k_We_need_to_be_a_real_python_cla), 0, 0, 1, 0},
  {&__pyx_n_s__27, __pyx_k__27, sizeof(__pyx_k__27), 0, 0, 1, 1},
  {&__pyx_n_s_a, __pyx_k_a, sizeof(__pyx_k_a), 0, 0, 1, 1},
  {&__pyx_n_s_add, __pyx_k_add, sizeof(__pyx_k_add), 0, 0, 1, 1},
  {&__pyx_n_s_align_axes, __pyx_k_align_axes, sizeof(__pyx_k_align_axes), 0, 0, 1, 1},
  {&__pyx_n_s_alloc_texture, __pyx_k_alloc_texture, sizeof(__pyx_k_alloc_texture), 0, 0, 1, 1},
  {&__pyx_n_s_allocate, __pyx_k_allocate, sizeof(__pyx_k_allocate), 0, 0, 1, 1},
  {&__pyx_n_s_alpha, __pyx_k_alpha, sizeof(__pyx_k_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_bitmap, __pyx_k_bitmap, sizeof(__pyx_k_bitmap), 0, 0, 1, 1},
  {&__pyx_n_s_border_bottom, __pyx_k_border_bottom, sizeof(__pyx_k_border_bottom), 0, 0, 1, 1},
  {&__pyx_n_s_border_left, __pyx_k_border_left, sizeof(__pyx_k_border_left), 0, 0, 1, 1},
  {&__pyx_n_s_border_right, __pyx_k_border_right, sizeof(__pyx_k_border_right), 0, 0, 1, 1},
  {&__pyx_n_s_border_top, __pyx_k_border_top, sizeof(__pyx_k_border_top), 0, 0, 1, 1},
  {&__pyx_n_s_byteorder, __pyx_k_byteorder, sizeof(__pyx_k_byteorder), 0, 0, 1, 1},
  {&__pyx_n_s_cleanup, __pyx_k_cleanup, sizeof(__pyx_k_cleanup), 0, 0, 1, 1},
  {&__pyx_n_s_clear, __pyx_k_clear, sizeof(__pyx_k_clear), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_collections, __pyx_k_collections, sizeof(__pyx_k_collections), 0, 0, 1, 1},
  {&__pyx_n_s_colnum, __pyx_k_colnum, sizeof(__pyx_k_colnum), 0, 0, 1, 1},
  {&__pyx_n_s_compute_subrow, __pyx_k_compute_subrow, sizeof(__pyx_k_compute_subrow), 0, 0, 1, 1},
  {&__pyx_n_s_compute_tiling, __pyx_k_compute_tiling, sizeof(__pyx_k_compute_tiling), 0, 0, 1, 1},
  {&__pyx_n_s_config, __pyx_k_config, sizeof(__pyx_k_config), 0, 0, 1, 1},
  {&__pyx_n_s_coords, __pyx_k_coords, sizeof(__pyx_k_coords), 0, 0, 1, 1},
  {&__pyx_n_s_cur, __pyx_k_cur, sizeof(__pyx_k_cur), 0, 0, 1, 1},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_dealloc_textures, __pyx_k_dealloc_textures, sizeof(__pyx_k_dealloc_textures), 0, 0, 1, 1},
  {&__pyx_n_s_deallocate, __pyx_k_deallocate, sizeof(__pyx_k_deallocate), 0, 0, 1, 1},
  {&__pyx_n_s_defaultdict, __pyx_k_defaultdict, sizeof(__pyx_k_defaultdict), 0, 0, 1, 1},
  {&__pyx_n_s_developer, __pyx_k_developer, sizeof(__pyx_k_developer), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_discard, __pyx_k_discard, sizeof(__pyx_k_discard), 0, 0, 1, 1},
  {&__pyx_n_s_display, __pyx_k_display, sizeof(__pyx_k_display), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_draw, __pyx_k_draw, sizeof(__pyx_k_draw), 0, 0, 1, 1},
  {&__pyx_n_s_draw_func, __pyx_k_draw_func, sizeof(__pyx_k_draw_func), 0, 0, 1, 1},
  {&__pyx_n_s_environ, __pyx_k_environ, sizeof(__pyx_k_environ), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_fill_factor, __pyx_k_fill_factor, sizeof(__pyx_k_fill_factor), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_fraction, __pyx_k_fraction, sizeof(__pyx_k_fraction), 0, 0, 1, 1},
  {&__pyx_n_s_free_list, __pyx_k_free_list, sizeof(__pyx_k_free_list), 0, 0, 1, 1},
  {&__pyx_n_s_free_textures, __pyx_k_free_textures, sizeof(__pyx_k_free_textures), 0, 0, 1, 1},
  {&__pyx_n_s_game, __pyx_k_game, sizeof(__pyx_k_game), 0, 0, 1, 1},
  {&__pyx_n_s_generation, __pyx_k_generation, sizeof(__pyx_k_generation), 0, 0, 1, 1},
  {&__pyx_n_s_get_masks, __pyx_k_get_masks, sizeof(__pyx_k_get_masks), 0, 0, 1, 1},
  {&__pyx_n_s_get_size, __pyx_k_get_size, sizeof(__pyx_k_get_size), 0, 0, 1, 1},
  {&__pyx_n_s_get_size_limit, __pyx_k_get_size_limit, sizeof(__pyx_k_get_size_limit), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_gl_npot, __pyx_k_gl_npot, sizeof(__pyx_k_gl_npot), 0, 0, 1, 1},
  {&__pyx_n_s_gldraw, __pyx_k_gldraw, sizeof(__pyx_k_gldraw), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_n_s_height, __pyx_k_height, sizeof(__pyx_k_height), 0, 0, 1, 1},
  {&__pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_k_home_tom_ab_renpy_renpy_gl_glte, sizeof(__pyx_k_home_tom_ab_renpy_renpy_gl_glte), 0, 0, 1, 0},
  {&__pyx_n_s_hw_max_size, __pyx_k_hw_max_size, sizeof(__pyx_k_hw_max_size), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_internalformat, __pyx_k_internalformat, sizeof(__pyx_k_internalformat), 0, 0, 1, 1},
  {&__pyx_n_s_ioff, __pyx_k_ioff, sizeof(__pyx_k_ioff), 0, 0, 1, 1},
  {&__pyx_n_s_itile, __pyx_k_itile, sizeof(__pyx_k_itile), 0, 0, 1, 1},
  {&__pyx_n_s_iwidth, __pyx_k_iwidth, sizeof(__pyx_k_iwidth), 0, 0, 1, 1},
  {&__pyx_n_s_ix, __pyx_k_ix, sizeof(__pyx_k_ix), 0, 0, 1, 1},
  {&__pyx_n_s_iy, __pyx_k_iy, sizeof(__pyx_k_iy), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_l, __pyx_k_l, sizeof(__pyx_k_l), 0, 0, 1, 1},
  {&__pyx_n_s_left_border, __pyx_k_left_border, sizeof(__pyx_k_left_border), 0, 0, 1, 1},
  {&__pyx_n_s_little, __pyx_k_little, sizeof(__pyx_k_little), 0, 0, 1, 1},
  {&__pyx_n_s_load_premultiplied, __pyx_k_load_premultiplied, sizeof(__pyx_k_load_premultiplied), 0, 0, 1, 1},
  {&__pyx_n_s_load_surface, __pyx_k_load_surface, sizeof(__pyx_k_load_surface), 0, 0, 1, 1},
  {&__pyx_n_s_log, __pyx_k_log, sizeof(__pyx_k_log), 0, 0, 1, 1},
  {&__pyx_n_s_loop, __pyx_k_loop, sizeof(__pyx_k_loop), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_make_ready, __pyx_k_make_ready, sizeof(__pyx_k_make_ready), 0, 0, 1, 1},
  {&__pyx_n_s_max_size, __pyx_k_max_size, sizeof(__pyx_k_max_size), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_min, __pyx_k_min, sizeof(__pyx_k_min), 0, 0, 1, 1},
  {&__pyx_n_s_min_fill_factor, __pyx_k_min_fill_factor, sizeof(__pyx_k_min_fill_factor), 0, 0, 1, 1},
  {&__pyx_n_s_minsize, __pyx_k_minsize, sizeof(__pyx_k_minsize), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nargs, __pyx_k_nargs, sizeof(__pyx_k_nargs), 0, 0, 1, 1},
  {&__pyx_n_s_nearest, __pyx_k_nearest, sizeof(__pyx_k_nearest), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_next, __pyx_k_next, sizeof(__pyx_k_next), 0, 0, 1, 1},
  {&__pyx_n_s_npot_free_textures, __pyx_k_npot_free_textures, sizeof(__pyx_k_npot_free_textures), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_old_gl_npot, __pyx_k_old_gl_npot, sizeof(__pyx_k_old_gl_npot), 0, 0, 1, 1},
  {&__pyx_n_s_op, __pyx_k_op, sizeof(__pyx_k_op), 0, 0, 1, 1},
  {&__pyx_n_s_orig_width, __pyx_k_orig_width, sizeof(__pyx_k_orig_width), 0, 0, 1, 1},
  {&__pyx_n_s_out, __pyx_k_out, sizeof(__pyx_k_out), 0, 0, 1, 1},
  {&__pyx_n_s_outrow, __pyx_k_outrow, sizeof(__pyx_k_outrow), 0, 0, 1, 1},
  {&__pyx_n_s_outtile, __pyx_k_outtile, sizeof(__pyx_k_outtile), 0, 0, 1, 1},
  {&__pyx_n_s_over, __pyx_k_over, sizeof(__pyx_k_over), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_pend, __pyx_k_pend, sizeof(__pyx_k_pend), 0, 0, 1, 1},
  {&__pyx_n_s_pheight, __pyx_k_pheight, sizeof(__pyx_k_pheight), 0, 0, 1, 1},
  {&__pyx_n_s_physical_size, __pyx_k_physical_size, sizeof(__pyx_k_physical_size), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pixel, __pyx_k_pixel, sizeof(__pyx_k_pixel), 0, 0, 1, 1},
  {&__pyx_n_s_pixels, __pyx_k_pixels, sizeof(__pyx_k_pixels), 0, 0, 1, 1},
  {&__pyx_n_s_pixels_end, __pyx_k_pixels_end, sizeof(__pyx_k_pixels_end), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_pp, __pyx_k_pp, sizeof(__pyx_k_pp), 0, 0, 1, 1},
  {&__pyx_n_s_ppend, __pyx_k_ppend, sizeof(__pyx_k_ppend), 0, 0, 1, 1},
  {&__pyx_n_s_preferences, __pyx_k_preferences, sizeof(__pyx_k_preferences), 0, 0, 1, 1},
  {&__pyx_n_s_premultiply, __pyx_k_premultiply, sizeof(__pyx_k_premultiply), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_pwidth, __pyx_k_pwidth, sizeof(__pyx_k_pwidth), 0, 0, 1, 1},
  {&__pyx_n_s_pysurf, __pyx_k_pysurf, sizeof(__pyx_k_pysurf), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_TextureCore, __pyx_k_pyx_unpickle_TextureCore, sizeof(__pyx_k_pyx_unpickle_TextureCore), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_TextureGrid, __pyx_k_pyx_unpickle_TextureGrid, sizeof(__pyx_k_pyx_unpickle_TextureGrid), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_ramp, __pyx_k_ramp, sizeof(__pyx_k_ramp), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_render, __pyx_k_render, sizeof(__pyx_k_render), 0, 0, 1, 1},
  {&__pyx_n_s_render_to, __pyx_k_render_to, sizeof(__pyx_k_render_to), 0, 0, 1, 1},
  {&__pyx_n_s_renpy, __pyx_k_renpy, sizeof(__pyx_k_renpy), 0, 0, 1, 1},
  {&__pyx_n_s_renpy_gl_gltexture, __pyx_k_renpy_gl_gltexture, sizeof(__pyx_k_renpy_gl_gltexture), 0, 0, 1, 1},
  {&__pyx_n_s_reverse, __pyx_k_reverse, sizeof(__pyx_k_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_right_border, __pyx_k_right_border, sizeof(__pyx_k_right_border), 0, 0, 1, 1},
  {&__pyx_n_s_row, __pyx_k_row, sizeof(__pyx_k_row), 0, 0, 1, 1},
  {&__pyx_n_s_row_index, __pyx_k_row_index, sizeof(__pyx_k_row_index), 0, 0, 1, 1},
  {&__pyx_n_s_row_size, __pyx_k_row_size, sizeof(__pyx_k_row_size), 0, 0, 1, 1},
  {&__pyx_n_s_rowi, __pyx_k_rowi, sizeof(__pyx_k_rowi), 0, 0, 1, 1},
  {&__pyx_n_s_rtt, __pyx_k_rtt, sizeof(__pyx_k_rtt), 0, 0, 1, 1},
  {&__pyx_n_s_rv, __pyx_k_rv, sizeof(__pyx_k_rv), 0, 0, 1, 1},
  {&__pyx_n_s_rv_col, __pyx_k_rv_col, sizeof(__pyx_k_rv_col), 0, 0, 1, 1},
  {&__pyx_n_s_rv_row, __pyx_k_rv_row, sizeof(__pyx_k_rv_row), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_sizeof, __pyx_k_sizeof, sizeof(__pyx_k_sizeof), 0, 0, 1, 1},
  {&__pyx_n_s_state, __pyx_k_state, sizeof(__pyx_k_state), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_surf, __pyx_k_surf, sizeof(__pyx_k_surf), 0, 0, 1, 1},
  {&__pyx_n_s_sx, __pyx_k_sx, sizeof(__pyx_k_sx), 0, 0, 1, 1},
  {&__pyx_n_s_sy, __pyx_k_sy, sizeof(__pyx_k_sy), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_t, __pyx_k_t, sizeof(__pyx_k_t), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_test_texture_sizes, __pyx_k_test_texture_sizes, sizeof(__pyx_k_test_texture_sizes), 0, 0, 1, 1},
  {&__pyx_n_s_tex, __pyx_k_tex, sizeof(__pyx_k_tex), 0, 0, 1, 1},
  {&__pyx_n_s_texcol, __pyx_k_texcol, sizeof(__pyx_k_texcol), 0, 0, 1, 1},
  {&__pyx_n_s_texcolumns, __pyx_k_texcolumns, sizeof(__pyx_k_texcolumns), 0, 0, 1, 1},
  {&__pyx_n_s_texcoords, __pyx_k_texcoords, sizeof(__pyx_k_texcoords), 0, 0, 1, 1},
  {&__pyx_n_s_texheight, __pyx_k_texheight, sizeof(__pyx_k_texheight), 0, 0, 1, 1},
  {&__pyx_n_s_texnums, __pyx_k_texnums, sizeof(__pyx_k_texnums), 0, 0, 1, 1},
  {&__pyx_n_s_texrow, __pyx_k_texrow, sizeof(__pyx_k_texrow), 0, 0, 1, 1},
  {&__pyx_n_s_texrows, __pyx_k_texrows, sizeof(__pyx_k_texrows), 0, 0, 1, 1},
  {&__pyx_n_s_texture_count, __pyx_k_texture_count, sizeof(__pyx_k_texture_count), 0, 0, 1, 1},
  {&__pyx_n_s_texture_generation, __pyx_k_texture_generation, sizeof(__pyx_k_texture_generation), 0, 0, 1, 1},
  {&__pyx_n_s_texture_grid_from_drawing, __pyx_k_texture_grid_from_drawing, sizeof(__pyx_k_texture_grid_from_drawing), 0, 0, 1, 1},
  {&__pyx_n_s_texture_grid_from_surface, __pyx_k_texture_grid_from_surface, sizeof(__pyx_k_texture_grid_from_surface), 0, 0, 1, 1},
  {&__pyx_n_s_texture_numbers, __pyx_k_texture_numbers, sizeof(__pyx_k_texture_numbers), 0, 0, 1, 1},
  {&__pyx_n_s_texwidth, __pyx_k_texwidth, sizeof(__pyx_k_texwidth), 0, 0, 1, 1},
  {&__pyx_n_s_tg, __pyx_k_tg, sizeof(__pyx_k_tg), 0, 0, 1, 1},
  {&__pyx_n_s_tg0, __pyx_k_tg0, sizeof(__pyx_k_tg0), 0, 0, 1, 1},
  {&__pyx_n_s_tg1, __pyx_k_tg1, sizeof(__pyx_k_tg1), 0, 0, 1, 1},
  {&__pyx_n_s_tg2, __pyx_k_tg2, sizeof(__pyx_k_tg2), 0, 0, 1, 1},
  {&__pyx_n_s_tiles, __pyx_k_tiles, sizeof(__pyx_k_tiles), 0, 0, 1, 1},
  {&__pyx_n_s_tiling_cache, __pyx_k_tiling_cache, sizeof(__pyx_k_tiling_cache), 0, 0, 1, 1},
  {&__pyx_n_s_time, __pyx_k_time, sizeof(__pyx_k_time), 0, 0, 1, 1},
  {&__pyx_n_s_total_texture_size, __pyx_k_total_texture_size, sizeof(__pyx_k_total_texture_size), 0, 0, 1, 1},
  {&__pyx_n_s_transform, __pyx_k_transform, sizeof(__pyx_k_transform), 0, 0, 1, 1},
  {&__pyx_n_s_transient, __pyx_k_transient, sizeof(__pyx_k_transient), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_gl, __pyx_k_use_gl, sizeof(__pyx_k_use_gl), 0, 0, 1, 1},
  {&__pyx_n_s_use_gles, __pyx_k_use_gles, sizeof(__pyx_k_use_gles), 0, 0, 1, 1},
  {&__pyx_n_s_values, __pyx_k_values, sizeof(__pyx_k_values), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_width, __pyx_k_width, sizeof(__pyx_k_width), 0, 0, 1, 1},
  {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_xrange, __pyx_k_xrange, sizeof(__pyx_k_xrange), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 304, __pyx_L1_error)
  __pyx_builtin_StopIteration = __Pyx_GetBuiltinName(__pyx_n_s_StopIteration); if (!__pyx_builtin_StopIteration) __PYX_ERR(0, 653, __pyx_L1_error)
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) __PYX_ERR(0, 870, __pyx_L1_error)
  __pyx_builtin_min = __Pyx_GetBuiltinName(__pyx_n_s_min); if (!__pyx_builtin_min) __PYX_ERR(0, 957, __pyx_L1_error)
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_xrange) __PYX_ERR(0, 959, __pyx_L1_error)
  #else
  __pyx_builtin_xrange = __Pyx_GetBuiltinName(__pyx_n_s_xrange); if (!__pyx_builtin_xrange) __PYX_ERR(0, 959, __pyx_L1_error)
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "renpy/gl/gltexture.pyx":123
 *     global SIZES
 * 
 *     renpy.display.log.write("Texture testing:")             # <<<<<<<<<<<<<<
 * 
 *     # There could be an error queued up from an ANGLE reset. Purge it before we do the
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_Texture_testing); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "renpy/gl/gltexture.pyx":250
 * 
 *     if not SIZES:
 *         renpy.display.log.write("Textures are not rendering properly.")             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_Textures_are_not_rendering_prope); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "renpy/gl/gltexture.pyx":507
 *     def __getstate__(self):
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")             # <<<<<<<<<<<<<<
 *         else:
 *             return { }
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_Can_t_pickle_a_texture); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "renpy/gl/gltexture.pyx":565
 *         i.deallocate()
 * 
 *     npot_free_textures[:] = [ ]             # <<<<<<<<<<<<<<
 * 
 *     if not renpy.game.preferences.gl_npot:
 */
  __pyx_slice__4 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__4)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__4);
  __Pyx_GIVEREF(__pyx_slice__4);

  /* "renpy/gl/gltexture.pyx":600
 *         i.deallocate()
 * 
 *     npot_free_textures[:] = [ ]             # <<<<<<<<<<<<<<
 * 
 * def compute_subrow(row, offset, width):
 */
  __pyx_slice__5 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__5)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__5);
  __Pyx_GIVEREF(__pyx_slice__5);

  /* "renpy/gl/gltexture.pyx":697
 *     def __getstate__(self): #@DuplicatedSignature
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")             # <<<<<<<<<<<<<<
 *         else:
 *             return { }
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_Can_t_pickle_a_texture); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "renpy/gl/gltexture.pyx":66
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gles():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_use_gles, 66, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(0, 66, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":82
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gl():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */
  __pyx_codeobj__8 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_use_gl, 82, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__8)) __PYX_ERR(0, 82, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":104
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def test_texture_sizes(Environ environ, draw):             # <<<<<<<<<<<<<<
 *     """
 *     Tests each possible texture size to see if it can be used. We test the
 */
  __pyx_tuple__9 = PyTuple_Pack(11, __pyx_n_s_environ, __pyx_n_s_draw, __pyx_n_s_i, __pyx_n_s_size, __pyx_n_s_tex, __pyx_n_s_bitmap, __pyx_n_s_coords, __pyx_n_s_texcoords, __pyx_n_s_pixel, __pyx_n_s_hw_max_size, __pyx_n_s_error); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_codeobj__10 = (PyObject*)__Pyx_PyCode_New(2, 0, 11, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__9, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_test_texture_sizes, 104, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__10)) __PYX_ERR(0, 104, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":502
 *     """
 * 
 *     def __sizeof__(self):             # <<<<<<<<<<<<<<
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_codeobj__12 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__11, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_sizeof, 502, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__12)) __PYX_ERR(0, 502, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":505
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_getstate, 505, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(0, 505, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":511
 *             return { }
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_state); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_setstate, 511, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(0, 511, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":531
 * # This allocates a texture, either from the free list, or by asking
 * # gl.
 * def alloc_texture(width, height):             # <<<<<<<<<<<<<<
 *     """
 *     Allocate a texture, either from the freelist or by asking GL. The
 */
  __pyx_tuple__17 = PyTuple_Pack(4, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_rv, __pyx_n_s_l); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_alloc_texture, 531, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 531, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":555
 * 
 * 
 * def dealloc_textures():             # <<<<<<<<<<<<<<
 *     cdef GLuint texnums[1]
 * 
 */
  __pyx_tuple__19 = PyTuple_Pack(4, __pyx_n_s_texnums, __pyx_n_s_l, __pyx_n_s_t, __pyx_n_s_i); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(0, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_dealloc_textures, 555, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 555, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":583
 * 
 * 
 * def cleanup():             # <<<<<<<<<<<<<<
 *     """
 *     This is called once per frame.
 */
  __pyx_tuple__21 = PyTuple_Pack(3, __pyx_n_s_l, __pyx_n_s_t, __pyx_n_s_i); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(0, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_cleanup, 583, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 583, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":602
 *     npot_free_textures[:] = [ ]
 * 
 * def compute_subrow(row, offset, width):             # <<<<<<<<<<<<<<
 *     """
 *     Given a row (or column), this computes a subrow starting at the given
 */
  __pyx_tuple__23 = PyTuple_Pack(10, __pyx_n_s_row, __pyx_n_s_offset, __pyx_n_s_width, __pyx_n_s_rowi, __pyx_n_s_outrow, __pyx_n_s_tiles, __pyx_n_s_outtile, __pyx_n_s_ioff, __pyx_n_s_iwidth, __pyx_n_s_itile); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_codeobj__24 = (PyObject*)__Pyx_PyCode_New(3, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__23, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_compute_subrow, 602, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__24)) __PYX_ERR(0, 602, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":757
 * tiling_cache = { }
 * 
 * def compute_tiling(width, max_size, min_fill_factor):             # <<<<<<<<<<<<<<
 *     """
 *     This computes a tiling for an image with the given width (or
 */
  __pyx_tuple__25 = PyTuple_Pack(14, __pyx_n_s_width, __pyx_n_s_max_size, __pyx_n_s_min_fill_factor, __pyx_n_s_gl_npot, __pyx_n_s_orig_width, __pyx_n_s_key, __pyx_n_s_x, __pyx_n_s_row, __pyx_n_s_tiles, __pyx_n_s_row_index, __pyx_n_s_left_border, __pyx_n_s_right_border, __pyx_n_s_size, __pyx_n_s_row_size); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(3, 0, 14, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_compute_tiling, 757, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 757, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":847
 * 
 * 
 * def texture_grid_from_surface(surf, transient):             # <<<<<<<<<<<<<<
 *     """
 *     This takes a Surface and turns it into a TextureGrid.
 */
  __pyx_tuple__28 = PyTuple_Pack(25, __pyx_n_s_surf, __pyx_n_s_transient, __pyx_n_s_max_size, __pyx_n_s_fill_factor, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_rv, __pyx_n_s_texcolumns, __pyx_n_s_texrows, __pyx_n_s_rv_row, __pyx_n_s_texrow, __pyx_n_s_border_top, __pyx_n_s__27, __pyx_n_s_border_bottom, __pyx_n_s_y, __pyx_n_s_texheight, __pyx_n_s_row, __pyx_n_s_colnum, __pyx_n_s_rv_col, __pyx_n_s_texcol, __pyx_n_s_border_left, __pyx_n_s_border_right, __pyx_n_s_x, __pyx_n_s_texwidth, __pyx_n_s_tex); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(2, 0, 25, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_texture_grid_from_surface, 847, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 847, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":893
 * 
 * 
 * def texture_grid_from_drawing(width, height, draw_func, rtt, environ):             # <<<<<<<<<<<<<<
 *     """
 *     This creates a texture grid of `width` by `height` by using
 */
  __pyx_tuple__30 = PyTuple_Pack(17, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_draw_func, __pyx_n_s_rtt, __pyx_n_s_environ, __pyx_n_s_rv, __pyx_n_s_gldraw, __pyx_n_s_pwidth, __pyx_n_s_pheight, __pyx_n_s_texcolumns, __pyx_n_s_texrows, __pyx_n_s_y, __pyx_n_s_texheight, __pyx_n_s_row, __pyx_n_s_x, __pyx_n_s_texwidth, __pyx_n_s_tex); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(5, 0, 17, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_texture_grid_from_drawing, 893, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(0, 893, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":922
 * 
 * 
 * def align_axes(*args):             # <<<<<<<<<<<<<<
 *     """
 *     This takes n axes, each a list consisting of (offset, size, index)
 */
  __pyx_tuple__32 = PyTuple_Pack(10, __pyx_n_s_args, __pyx_n_s_rv, __pyx_n_s_i, __pyx_n_s_cur, __pyx_n_s_offset, __pyx_n_s_size, __pyx_n_s_index, __pyx_n_s_nargs, __pyx_n_s_loop, __pyx_n_s_minsize); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(0, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_align_axes, 922, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 922, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":1148
 * 
 * 
 * def premultiply(             # <<<<<<<<<<<<<<
 *     object pysurf,
 *     int x,
 */
  __pyx_tuple__34 = PyTuple_Pack(23, __pyx_n_s_pysurf, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_w, __pyx_n_s_h, __pyx_n_s_border_left, __pyx_n_s_border_top, __pyx_n_s_border_right, __pyx_n_s_border_bottom, __pyx_n_s_ix, __pyx_n_s_iy, __pyx_n_s_alpha, __pyx_n_s_rv, __pyx_n_s_out, __pyx_n_s_pixels, __pyx_n_s_pixels_end, __pyx_n_s_surf, __pyx_n_s_p, __pyx_n_s_op, __pyx_n_s_pend, __pyx_n_s_a, __pyx_n_s_pp, __pyx_n_s_ppend); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(9, 0, 23, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_premultiply, 1148, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(0, 1148, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":1320
 * 
 * 
 * def load_premultiplied(data, width, height, update, rtt):             # <<<<<<<<<<<<<<
 * 
 *     cdef char *pixels
 */
  __pyx_tuple__36 = PyTuple_Pack(9, __pyx_n_s_data, __pyx_n_s_width, __pyx_n_s_height, __pyx_n_s_update, __pyx_n_s_rtt, __pyx_n_s_pixels, __pyx_n_s_internalformat, __pyx_n_s_format, __pyx_n_s_type); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(5, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_tom_ab_renpy_renpy_gl_glte, __pyx_n_s_load_premultiplied, 1320, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(0, 1320, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4447e15:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__38 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_TextureCore, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__40 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_TextureGrid, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyList_Type_pop.type = (PyObject*)&PyList_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_5 = PyFloat_FromDouble(0.5); if (unlikely(!__pyx_float_0_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_4 = PyInt_FromLong(4); if (unlikely(!__pyx_int_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_64 = PyInt_FromLong(64); if (unlikely(!__pyx_int_64)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2048 = PyInt_FromLong(2048); if (unlikely(!__pyx_int_2048)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_71597589 = PyInt_FromLong(71597589L); if (unlikely(!__pyx_int_71597589)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_143547127 = PyInt_FromLong(143547127L); if (unlikely(!__pyx_int_143547127)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initgltexture(void); /*proto*/
PyMODINIT_FUNC initgltexture(void)
#else
PyMODINIT_FUNC PyInit_gltexture(void); /*proto*/
PyMODINIT_FUNC PyInit_gltexture(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_gltexture(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_gltexture(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("gltexture", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__gl__gltexture) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.gl.gltexture")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.gl.gltexture", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("blit", (void (*)(void))__pyx_f_5renpy_2gl_9gltexture_blit, "PyObject *(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, double, double, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("blend", (void (*)(void))__pyx_f_5renpy_2gl_9gltexture_blend, "PyObject *(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, double, double, double, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("imageblend", (void (*)(void))__pyx_f_5renpy_2gl_9gltexture_imageblend, "PyObject *(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, double, double, struct __pyx_obj_5renpy_7display_6matrix_Matrix *, double, double, double, int, struct __pyx_obj_5renpy_2gl_6gldraw_Environ *, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Type init code ---*/
  __pyx_vtabptr_5renpy_2gl_9gltexture_TextureCore = &__pyx_vtable_5renpy_2gl_9gltexture_TextureCore;
  __pyx_vtable_5renpy_2gl_9gltexture_TextureCore.make_ready = (void (*)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *))__pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_ready;
  __pyx_vtable_5renpy_2gl_9gltexture_TextureCore.make_nearest = (void (*)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *))__pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_nearest;
  __pyx_vtable_5renpy_2gl_9gltexture_TextureCore.make_linear = (void (*)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *))__pyx_f_5renpy_2gl_9gltexture_11TextureCore_make_linear;
  __pyx_vtable_5renpy_2gl_9gltexture_TextureCore.allocate = (int (*)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureCore *, int __pyx_skip_dispatch))__pyx_f_5renpy_2gl_9gltexture_11TextureCore_allocate;
  if (PyType_Ready(&__pyx_type_5renpy_2gl_9gltexture_TextureCore) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_type_5renpy_2gl_9gltexture_TextureCore.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_2gl_9gltexture_TextureCore.tp_dict, __pyx_vtabptr_5renpy_2gl_9gltexture_TextureCore) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "TextureCore", (PyObject *)&__pyx_type_5renpy_2gl_9gltexture_TextureCore) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_2gl_9gltexture_TextureCore) < 0) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_ptype_5renpy_2gl_9gltexture_TextureCore = &__pyx_type_5renpy_2gl_9gltexture_TextureCore;
  __pyx_vtabptr_5renpy_2gl_9gltexture_TextureGrid = &__pyx_vtable_5renpy_2gl_9gltexture_TextureGrid;
  __pyx_vtable_5renpy_2gl_9gltexture_TextureGrid.make_ready = (void (*)(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid *, int, int __pyx_skip_dispatch))__pyx_f_5renpy_2gl_9gltexture_11TextureGrid_make_ready;
  if (PyType_Ready(&__pyx_type_5renpy_2gl_9gltexture_TextureGrid) < 0) __PYX_ERR(0, 659, __pyx_L1_error)
  __pyx_type_5renpy_2gl_9gltexture_TextureGrid.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_2gl_9gltexture_TextureGrid.tp_dict, __pyx_vtabptr_5renpy_2gl_9gltexture_TextureGrid) < 0) __PYX_ERR(0, 659, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "TextureGrid", (PyObject *)&__pyx_type_5renpy_2gl_9gltexture_TextureGrid) < 0) __PYX_ERR(0, 659, __pyx_L1_error)
  if (__pyx_type_5renpy_2gl_9gltexture_TextureGrid.tp_weaklistoffset == 0) __pyx_type_5renpy_2gl_9gltexture_TextureGrid.tp_weaklistoffset = offsetof(struct __pyx_obj_5renpy_2gl_9gltexture_TextureGrid, __weakref__);
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_2gl_9gltexture_TextureGrid) < 0) __PYX_ERR(0, 659, __pyx_L1_error)
  __pyx_ptype_5renpy_2gl_9gltexture_TextureGrid = &__pyx_type_5renpy_2gl_9gltexture_TextureGrid;
  /*--- Type import code ---*/
  __pyx_ptype_5renpy_7display_6matrix_Matrix = __Pyx_ImportType("renpy.display.matrix", "Matrix", sizeof(struct __pyx_obj_5renpy_7display_6matrix_Matrix), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6matrix_Matrix)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6matrix_Matrix = (struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6matrix_Matrix->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6matrix_Matrix)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_ptype_5renpy_7display_6matrix_Matrix2D = __Pyx_ImportType("renpy.display.matrix", "Matrix2D", sizeof(struct __pyx_obj_5renpy_7display_6matrix_Matrix2D), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6matrix_Matrix2D)) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6matrix_Matrix2D = (struct __pyx_vtabstruct_5renpy_7display_6matrix_Matrix2D*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6matrix_Matrix2D->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6matrix_Matrix2D)) __PYX_ERR(3, 43, __pyx_L1_error)
  __pyx_ptype_5renpy_7display_6render_Render = __Pyx_ImportType("renpy.display.render", "Render", sizeof(struct __pyx_obj_5renpy_7display_6render_Render), 1); if (unlikely(!__pyx_ptype_5renpy_7display_6render_Render)) __PYX_ERR(4, 25, __pyx_L1_error)
  __pyx_vtabptr_5renpy_7display_6render_Render = (struct __pyx_vtabstruct_5renpy_7display_6render_Render*)__Pyx_GetVtable(__pyx_ptype_5renpy_7display_6render_Render->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_7display_6render_Render)) __PYX_ERR(4, 25, __pyx_L1_error)
  __pyx_ptype_5renpy_2gl_6gldraw_Environ = __Pyx_ImportType("renpy.gl.gldraw", "Environ", sizeof(struct __pyx_obj_5renpy_2gl_6gldraw_Environ), 1); if (unlikely(!__pyx_ptype_5renpy_2gl_6gldraw_Environ)) __PYX_ERR(5, 25, __pyx_L1_error)
  __pyx_vtabptr_5renpy_2gl_6gldraw_Environ = (struct __pyx_vtabstruct_5renpy_2gl_6gldraw_Environ*)__Pyx_GetVtable(__pyx_ptype_5renpy_2gl_6gldraw_Environ->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_2gl_6gldraw_Environ)) __PYX_ERR(5, 25, __pyx_L1_error)
  __pyx_ptype_5renpy_2gl_6gldraw_GLDraw = __Pyx_ImportType("renpy.gl.gldraw", "GLDraw", sizeof(struct __pyx_obj_5renpy_2gl_6gldraw_GLDraw), 1); if (unlikely(!__pyx_ptype_5renpy_2gl_6gldraw_GLDraw)) __PYX_ERR(5, 27, __pyx_L1_error)
  __pyx_vtabptr_5renpy_2gl_6gldraw_GLDraw = (struct __pyx_vtabstruct_5renpy_2gl_6gldraw_GLDraw*)__Pyx_GetVtable(__pyx_ptype_5renpy_2gl_6gldraw_GLDraw->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_2gl_6gldraw_GLDraw)) __PYX_ERR(5, 27, __pyx_L1_error)
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(6, 9, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("renpy.gl.gl"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glActiveTextureARB", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glActiveTextureARB, "int (GLenum)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glBindTexture", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glBindTexture, "int (GLenum, GLuint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glDeleteTextures", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glDeleteTextures, "int (GLsizei, GLuint *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glDrawArrays", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glDrawArrays, "int (GLenum, GLint, GLsizei)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGenTextures", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glGenTextures, "int (GLsizei, GLuint *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glGetIntegerv", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glGetIntegerv, "int (GLenum, GLint *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glReadPixels", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glReadPixels, "int (GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, GLubyte *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glTexImage2D", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glTexImage2D, "int (GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, GLubyte *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glTexParameteri", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glTexParameteri, "int (GLenum, GLenum, GLint)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "glTexSubImage2D", (void (**)(void))&__pyx_f_5renpy_2gl_2gl_glTexSubImage2D, "int (GLenum, GLint, GLint, GLint, GLsizei, GLsizei, GLenum, GLenum, GLubyte *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "renpy/gl/gltexture.pyx":33
 * from sdl2 cimport *
 * from pygame_sdl2 cimport *
 * import_pygame_sdl2()             # <<<<<<<<<<<<<<
 * 
 * from cpython.string cimport PyString_FromStringAndSize
 */
  import_pygame_sdl2();

  /* "renpy/gl/gltexture.pyx":38
 * from libc.stdlib cimport calloc, free
 * 
 * import sys             # <<<<<<<<<<<<<<
 * import time
 * import collections
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_2) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":39
 * 
 * import sys
 * import time             # <<<<<<<<<<<<<<
 * import collections
 * import renpy
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_time, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_time, __pyx_t_2) < 0) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":40
 * import sys
 * import time
 * import collections             # <<<<<<<<<<<<<<
 * import renpy
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_collections, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_collections, __pyx_t_2) < 0) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":41
 * import time
 * import collections
 * import renpy             # <<<<<<<<<<<<<<
 * 
 * # The maximum size of a texture.
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_renpy, 0, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_renpy, __pyx_t_2) < 0) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":44
 * 
 * # The maximum size of a texture.
 * MAX_SIZE = 2048             # <<<<<<<<<<<<<<
 * 
 * # Possible sizes for a texture, ordered from largest to smallest.
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_MAX_SIZE, __pyx_int_2048) < 0) __PYX_ERR(0, 44, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":48
 * # Possible sizes for a texture, ordered from largest to smallest.
 * # (Now set in test_texture_sizes.)
 * SIZES = [ 64 ]             # <<<<<<<<<<<<<<
 * 
 * # A list of texture number allocated.
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_int_64);
  __Pyx_GIVEREF(__pyx_int_64);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_int_64);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SIZES, __pyx_t_2) < 0) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":51
 * 
 * # A list of texture number allocated.
 * texture_numbers = set()             # <<<<<<<<<<<<<<
 * 
 * # The texture generation.
 */
  __pyx_t_2 = PySet_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_numbers, __pyx_t_2) < 0) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":54
 * 
 * # The texture generation.
 * generation = 1             # <<<<<<<<<<<<<<
 * 
 * # A map from texture number to generation
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_generation, __pyx_int_1) < 0) __PYX_ERR(0, 54, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":57
 * 
 * # A map from texture number to generation
 * texture_generation = { }             # <<<<<<<<<<<<<<
 * 
 * cdef GLenum tex_format = GL_RGBA
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_generation, __pyx_t_2) < 0) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":59
 * texture_generation = { }
 * 
 * cdef GLenum tex_format = GL_RGBA             # <<<<<<<<<<<<<<
 * cdef GLenum tex_internalformat = GL_RGBA
 * cdef GLenum tex_type = GL_UNSIGNED_BYTE
 */
  __pyx_v_5renpy_2gl_9gltexture_tex_format = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":60
 * 
 * cdef GLenum tex_format = GL_RGBA
 * cdef GLenum tex_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 * cdef GLenum tex_type = GL_UNSIGNED_BYTE
 * cdef GLenum rtt_format = GL_RGBA
 */
  __pyx_v_5renpy_2gl_9gltexture_tex_internalformat = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":61
 * cdef GLenum tex_format = GL_RGBA
 * cdef GLenum tex_internalformat = GL_RGBA
 * cdef GLenum tex_type = GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * cdef GLenum rtt_format = GL_RGBA
 * cdef GLenum rtt_internalformat = GL_RGBA
 */
  __pyx_v_5renpy_2gl_9gltexture_tex_type = GL_UNSIGNED_BYTE;

  /* "renpy/gl/gltexture.pyx":62
 * cdef GLenum tex_internalformat = GL_RGBA
 * cdef GLenum tex_type = GL_UNSIGNED_BYTE
 * cdef GLenum rtt_format = GL_RGBA             # <<<<<<<<<<<<<<
 * cdef GLenum rtt_internalformat = GL_RGBA
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_format = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":63
 * cdef GLenum tex_type = GL_UNSIGNED_BYTE
 * cdef GLenum rtt_format = GL_RGBA
 * cdef GLenum rtt_internalformat = GL_RGBA             # <<<<<<<<<<<<<<
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE
 * 
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_internalformat = GL_RGBA;

  /* "renpy/gl/gltexture.pyx":64
 * cdef GLenum rtt_format = GL_RGBA
 * cdef GLenum rtt_internalformat = GL_RGBA
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE             # <<<<<<<<<<<<<<
 * 
 * def use_gles():
 */
  __pyx_v_5renpy_2gl_9gltexture_rtt_type = GL_UNSIGNED_BYTE;

  /* "renpy/gl/gltexture.pyx":66
 * cdef GLenum rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gles():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_1use_gles, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_use_gles, __pyx_t_2) < 0) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":82
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def use_gl():             # <<<<<<<<<<<<<<
 *     global tex_format
 *     global tex_internalformat
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_3use_gl, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_use_gl, __pyx_t_2) < 0) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":104
 *     rtt_type = GL_UNSIGNED_BYTE
 * 
 * def test_texture_sizes(Environ environ, draw):             # <<<<<<<<<<<<<<
 *     """
 *     Tests each possible texture size to see if it can be used. We test the
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_5test_texture_sizes, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test_texture_sizes, __pyx_t_2) < 0) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":496
 * 
 * 
 * class Texture(TextureCore):             # <<<<<<<<<<<<<<
 *     """
 *     We need to be a real python class, not a C extension, to ensure that
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureCore));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureCore));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_5renpy_2gl_9gltexture_TextureCore));
  __pyx_t_3 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_3, __pyx_t_2, __pyx_n_s_Texture, __pyx_n_s_Texture, (PyObject *) NULL, __pyx_n_s_renpy_gl_gltexture, __pyx_kp_s_We_need_to_be_a_real_python_cla); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "renpy/gl/gltexture.pyx":502
 *     """
 * 
 *     def __sizeof__(self):             # <<<<<<<<<<<<<<
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_7Texture_1__sizeof__, 0, __pyx_n_s_Texture___sizeof, NULL, __pyx_n_s_renpy_gl_gltexture, __pyx_d, ((PyObject *)__pyx_codeobj__12)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetItem(__pyx_t_4, __pyx_n_s_sizeof, __pyx_t_5) < 0) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":505
 *         return TextureCore.__sizeof__(self) + self.width * self.height * 4
 * 
 *     def __getstate__(self):             # <<<<<<<<<<<<<<
 *         if renpy.config.developer:
 *             raise Exception("Can't pickle a texture.")
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_7Texture_3__getstate__, 0, __pyx_n_s_Texture___getstate, NULL, __pyx_n_s_renpy_gl_gltexture, __pyx_d, ((PyObject *)__pyx_codeobj__14)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetItem(__pyx_t_4, __pyx_n_s_getstate, __pyx_t_5) < 0) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":511
 *             return { }
 * 
 *     def __setstate__(self, state):             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_5 = __Pyx_CyFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_7Texture_5__setstate__, 0, __pyx_n_s_Texture___setstate, NULL, __pyx_n_s_renpy_gl_gltexture, __pyx_d, ((PyObject *)__pyx_codeobj__16)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyObject_SetItem(__pyx_t_4, __pyx_n_s_setstate, __pyx_t_5) < 0) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "renpy/gl/gltexture.pyx":496
 * 
 * 
 * class Texture(TextureCore):             # <<<<<<<<<<<<<<
 *     """
 *     We need to be a real python class, not a C extension, to ensure that
 */
  __pyx_t_5 = __Pyx_Py3ClassCreate(__pyx_t_3, __pyx_n_s_Texture, __pyx_t_2, __pyx_t_4, NULL, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Texture, __pyx_t_5) < 0) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":517
 * # This is a map from texture sizes to a list of free textures of that
 * # size.
 * free_textures = collections.defaultdict(list)             # <<<<<<<<<<<<<<
 * 
 * # A list of NPOT textures that need to be freed.
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_collections); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_defaultdict); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)(&PyList_Type))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)(&PyList_Type))};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)(&PyList_Type))};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 517, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)(&PyList_Type)));
      __Pyx_GIVEREF(((PyObject *)(&PyList_Type)));
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)(&PyList_Type)));
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_free_textures, __pyx_t_2) < 0) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":520
 * 
 * # A list of NPOT textures that need to be freed.
 * npot_free_textures = [ ]             # <<<<<<<<<<<<<<
 * 
 * # The number of textues that have been allocated but not deallocated.
 */
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_npot_free_textures, __pyx_t_2) < 0) __PYX_ERR(0, 520, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":523
 * 
 * # The number of textues that have been allocated but not deallocated.
 * texture_count = 0             # <<<<<<<<<<<<<<
 * 
 * # The total size (in bytes) of all the textures that have been allocated
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_count, __pyx_int_0) < 0) __PYX_ERR(0, 523, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":527
 * # The total size (in bytes) of all the textures that have been allocated
 * # but not deallocated.
 * total_texture_size = 0             # <<<<<<<<<<<<<<
 * 
 * # This allocates a texture, either from the free list, or by asking
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_total_texture_size, __pyx_int_0) < 0) __PYX_ERR(0, 527, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":531
 * # This allocates a texture, either from the free list, or by asking
 * # gl.
 * def alloc_texture(width, height):             # <<<<<<<<<<<<<<
 *     """
 *     Allocate a texture, either from the freelist or by asking GL. The
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_7alloc_texture, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_alloc_texture, __pyx_t_2) < 0) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":555
 * 
 * 
 * def dealloc_textures():             # <<<<<<<<<<<<<<
 *     cdef GLuint texnums[1]
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_9dealloc_textures, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_dealloc_textures, __pyx_t_2) < 0) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":583
 * 
 * 
 * def cleanup():             # <<<<<<<<<<<<<<
 *     """
 *     This is called once per frame.
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_11cleanup, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cleanup, __pyx_t_2) < 0) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":602
 *     npot_free_textures[:] = [ ]
 * 
 * def compute_subrow(row, offset, width):             # <<<<<<<<<<<<<<
 *     """
 *     Given a row (or column), this computes a subrow starting at the given
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_13compute_subrow, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compute_subrow, __pyx_t_2) < 0) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":752
 * 
 * # The old value of the gl_npot preference.
 * old_gl_npot = None             # <<<<<<<<<<<<<<
 * 
 * # This is a cache from (width, size) to the results of compute_tiling.
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_old_gl_npot, Py_None) < 0) __PYX_ERR(0, 752, __pyx_L1_error)

  /* "renpy/gl/gltexture.pyx":755
 * 
 * # This is a cache from (width, size) to the results of compute_tiling.
 * tiling_cache = { }             # <<<<<<<<<<<<<<
 * 
 * def compute_tiling(width, max_size, min_fill_factor):
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 755, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_tiling_cache, __pyx_t_2) < 0) __PYX_ERR(0, 755, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":757
 * tiling_cache = { }
 * 
 * def compute_tiling(width, max_size, min_fill_factor):             # <<<<<<<<<<<<<<
 *     """
 *     This computes a tiling for an image with the given width (or
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_15compute_tiling, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compute_tiling, __pyx_t_2) < 0) __PYX_ERR(0, 757, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":847
 * 
 * 
 * def texture_grid_from_surface(surf, transient):             # <<<<<<<<<<<<<<
 *     """
 *     This takes a Surface and turns it into a TextureGrid.
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_17texture_grid_from_surface, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_grid_from_surface, __pyx_t_2) < 0) __PYX_ERR(0, 847, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":893
 * 
 * 
 * def texture_grid_from_drawing(width, height, draw_func, rtt, environ):             # <<<<<<<<<<<<<<
 *     """
 *     This creates a texture grid of `width` by `height` by using
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_19texture_grid_from_drawing, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_texture_grid_from_drawing, __pyx_t_2) < 0) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":922
 * 
 * 
 * def align_axes(*args):             # <<<<<<<<<<<<<<
 *     """
 *     This takes n axes, each a list consisting of (offset, size, index)
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_21align_axes, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 922, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_align_axes, __pyx_t_2) < 0) __PYX_ERR(0, 922, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":1148
 * 
 * 
 * def premultiply(             # <<<<<<<<<<<<<<
 *     object pysurf,
 *     int x,
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_29premultiply, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_premultiply, __pyx_t_2) < 0) __PYX_ERR(0, 1148, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":1320
 * 
 * 
 * def load_premultiplied(data, width, height, update, rtt):             # <<<<<<<<<<<<<<
 * 
 *     cdef char *pixels
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_31load_premultiplied, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_load_premultiplied, __pyx_t_2) < 0) __PYX_ERR(0, 1320, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_TextureCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4447e15:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_33__pyx_unpickle_TextureCore, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_TextureCore, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":9
 *         __pyx_unpickle_TextureCore__set_state(<TextureCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_TextureCore__set_state(TextureCore __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.debug = __pyx_state[0]; __pyx_result.format = __pyx_state[1]; __pyx_result.free_list = __pyx_state[2]; __pyx_result.generation = __pyx_state[3]; __pyx_result.height = __pyx_state[4]; __pyx_result.nearest = __pyx_state[5]; __pyx_result.number = __pyx_state[6]; __pyx_result.premult = __pyx_state[7]; __pyx_result.premult_bottom = __pyx_state[8]; __pyx_result.premult_left = __pyx_state[9]; __pyx_result.premult_right = __pyx_state[10]; __pyx_result.premult_size = __pyx_state[11]; __pyx_result.premult_top = __pyx_state[12]; __pyx_result.width = __pyx_state[13]; __pyx_result.xadd = __pyx_state[14]; __pyx_result.xmul = __pyx_state[15]; __pyx_result.yadd = __pyx_state[16]; __pyx_result.ymul = __pyx_state[17]
 *     if len(__pyx_state) > 18 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_5renpy_2gl_9gltexture_35__pyx_unpickle_TextureGrid, NULL, __pyx_n_s_renpy_gl_gltexture); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_TextureGrid, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/gl/gltexture.pyx":1
 * #@PydevCodeAnalysisIgnore             # <<<<<<<<<<<<<<
 * #cython: profile=False
 * # Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "renpy/display/matrix.pxd":37
 *         oz[0] = x * self.zdx + y * self.zdy + z * self.zdz + w * self.zdw
 * 
 *     cdef inline void transform2(Matrix self, float *ox, float *oy, float x, float y, float z, float w):             # <<<<<<<<<<<<<<
 *         ox[0] = x * self.xdx + y * self.xdy + z * self.xdz + w * self.xdw
 *         oy[0] = x * self.ydx + y * self.ydy + z * self.ydz + w * self.ydw
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.gl.gltexture", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.gl.gltexture");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* BytesEquals */
  static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
  static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* RaiseArgTupleInvalid */
  static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
  static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
  static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
  static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyObjectCallMethod1 */
  static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = NULL;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
    goto done;
done:
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
    result = __Pyx__PyObject_CallMethod1(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* append */
  static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* SaveResetException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
    static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseTooManyValuesToUnpack */
    static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
    static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
    static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* SetItemInt */
    static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o))) {
#else
    if (is_list || PySequence_Check(o)) {
#endif
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* PyIntBinop */
      #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

/* GetItemInt */
      static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyIntBinop */
      #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_SubtractObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a - b);
            if (likely((x^a) >= 0 || (x^~b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
#ifdef HAVE_LONG_LONG
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
#endif
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
#ifdef HAVE_LONG_LONG
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
#endif
                    }
                default: return PyLong_Type.tp_as_number->nb_subtract(op1, op2);
            }
        }
                x = a - b;
            return PyLong_FromLong(x);
#ifdef HAVE_LONG_LONG
        long_long:
                llx = lla - llb;
            return PyLong_FromLongLong(llx);
#endif
        
        
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("subtract", return NULL)
            result = ((double)a) - (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
}
#endif

/* GetAttr */
      static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
      static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* RaiseException */
      #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectCallMethod0 */
      static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto bad;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *function = PyMethod_GET_FUNCTION(method);
            result = __Pyx_PyObject_CallOneArg(function, self);
            Py_DECREF(method);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* UnpackUnboundCMethod */
      static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
      static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* pop */
      static CYTHON_INLINE PyObject* __Pyx__PyObject_Pop(PyObject* L) {
    if (Py_TYPE(L) == &PySet_Type) {
        return PySet_Pop(L);
    }
    return __Pyx_PyObject_CallMethod0(L, __pyx_n_s_pop);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE PyObject* __Pyx_PyList_Pop(PyObject* L) {
    if (likely(PyList_GET_SIZE(L) > (((PyListObject*)L)->allocated >> 1))) {
        Py_SIZE(L) -= 1;
        return PyList_GET_ITEM(L, PyList_GET_SIZE(L));
    }
    return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyList_Type_pop, L);
}
#endif

/* SliceObject */
      static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* None */
      static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* KeywordStringCheck */
      static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* PyIntBinop */
      #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* Import */
      static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
      static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
      static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* SetVTable */
      static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
      static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* GetVTable */
      static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* CalculateMetaclass */
      static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
      static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
      static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Py3ClassCreate */
          static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
          #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
          static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
          #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_GLenum(GLenum value) {
    const GLenum neg_one = (GLenum) -1, const_zero = (GLenum) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(GLenum) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(GLenum) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLenum) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(GLenum) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(GLenum) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(GLenum),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value) {
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned char),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
          static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE GLuint __Pyx_PyInt_As_GLuint(PyObject *x) {
    const GLuint neg_one = (GLuint) -1, const_zero = (GLuint) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLuint) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLuint, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLuint) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLuint) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLuint, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLuint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 2 * PyLong_SHIFT) {
                            return (GLuint) (((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLuint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 3 * PyLong_SHIFT) {
                            return (GLuint) (((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLuint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) >= 4 * PyLong_SHIFT) {
                            return (GLuint) (((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLuint) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLuint) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLuint) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLuint) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLuint, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLuint,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLuint) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLuint) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                            return (GLuint) ((((((GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLuint) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLuint) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                            return (GLuint) ((((((((GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLuint) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLuint) (((GLuint)-1)*(((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLuint) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLuint, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLuint) - 1 > 4 * PyLong_SHIFT) {
                            return (GLuint) ((((((((((GLuint)digits[3]) << PyLong_SHIFT) | (GLuint)digits[2]) << PyLong_SHIFT) | (GLuint)digits[1]) << PyLong_SHIFT) | (GLuint)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLuint) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLuint) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLuint, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLuint val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLuint) -1;
        }
    } else {
        GLuint val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLuint) -1;
        val = __Pyx_PyInt_As_GLuint(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLuint");
    return (GLuint) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLuint");
    return (GLuint) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE GLsizei __Pyx_PyInt_As_GLsizei(PyObject *x) {
    const GLsizei neg_one = (GLsizei) -1, const_zero = (GLsizei) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(GLsizei) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(GLsizei, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (GLsizei) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLsizei) 0;
                case  1: __PYX_VERIFY_RETURN_INT(GLsizei, digit, digits[0])
                case 2:
                    if (8 * sizeof(GLsizei) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) >= 2 * PyLong_SHIFT) {
                            return (GLsizei) (((((GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLsizei) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) >= 3 * PyLong_SHIFT) {
                            return (GLsizei) (((((((GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLsizei) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) >= 4 * PyLong_SHIFT) {
                            return (GLsizei) (((((((((GLsizei)digits[3]) << PyLong_SHIFT) | (GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (GLsizei) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(GLsizei) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLsizei) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (GLsizei) 0;
                case -1: __PYX_VERIFY_RETURN_INT(GLsizei, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(GLsizei,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(GLsizei) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 2 * PyLong_SHIFT) {
                            return (GLsizei) (((GLsizei)-1)*(((((GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(GLsizei) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 2 * PyLong_SHIFT) {
                            return (GLsizei) ((((((GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(GLsizei) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 3 * PyLong_SHIFT) {
                            return (GLsizei) (((GLsizei)-1)*(((((((GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(GLsizei) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 3 * PyLong_SHIFT) {
                            return (GLsizei) ((((((((GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(GLsizei) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 4 * PyLong_SHIFT) {
                            return (GLsizei) (((GLsizei)-1)*(((((((((GLsizei)digits[3]) << PyLong_SHIFT) | (GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(GLsizei) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(GLsizei, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(GLsizei) - 1 > 4 * PyLong_SHIFT) {
                            return (GLsizei) ((((((((((GLsizei)digits[3]) << PyLong_SHIFT) | (GLsizei)digits[2]) << PyLong_SHIFT) | (GLsizei)digits[1]) << PyLong_SHIFT) | (GLsizei)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(GLsizei) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(GLsizei) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(GLsizei, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            GLsizei val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (GLsizei) -1;
        }
    } else {
        GLsizei val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (GLsizei) -1;
        val = __Pyx_PyInt_As_GLsizei(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to GLsizei");
    return (GLsizei) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to GLsizei");
    return (GLsizei) -1;
}

/* FastTypeChecks */
          #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
          static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
          static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* ModuleImport */
          #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
          #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* FunctionImport */
          #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
          static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
