/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__renpy__styledata__styleclass
#define __PYX_HAVE_API__renpy__styledata__styleclass
#include <string.h>
#include <stdio.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "stringsource",
  "gen/styleclass.pxi",
  "style.pxd",
  "styleclass.pyx",
  "type.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5renpy_5style_StyleCore;
struct __pyx_obj_5renpy_9styledata_10styleclass_Style;

/* "renpy/style.pxd":39
 * # value
 * #     The value of the style property.
 * ctypedef int (*property_function)(PyObject **cache, int *cache_priorities, int priority, object value) except -1             # <<<<<<<<<<<<<<
 * 
 * cdef inline void assign(int index, PyObject **cache, int *cache_priorities, int priority, PyObject *value):
 */
typedef int (*__pyx_t_5renpy_5style_property_function)(PyObject **, int *, int, PyObject *);

/* "renpy/style.pxd":84
 * cdef void register_property_function(name, property_function function)
 * 
 * cdef class StyleCore:             # <<<<<<<<<<<<<<
 * 
 *     ############################################################## Public Fields
 */
struct __pyx_obj_5renpy_5style_StyleCore {
  PyObject_HEAD
  struct __pyx_vtabstruct_5renpy_5style_StyleCore *__pyx_vtab;
  PyObject *name;
  PyObject *parent;
  PyObject *properties;
  PyObject *prefix;
  PyObject *help;
  int built;
  int building;
  struct __pyx_obj_5renpy_5style_StyleCore *down_parent;
  struct __pyx_obj_5renpy_5style_StyleCore *left_parent;
  PyObject **cache;
  int prefix_offset;
};


/* "gen/styleclass.pxi":1
 * cdef class Style(StyleCore):             # <<<<<<<<<<<<<<
 * 
 *     property black_color:
 */
struct __pyx_obj_5renpy_9styledata_10styleclass_Style {
  struct __pyx_obj_5renpy_5style_StyleCore __pyx_base;
};



/* "renpy/style.pxd":84
 * cdef void register_property_function(name, property_function function)
 * 
 * cdef class StyleCore:             # <<<<<<<<<<<<<<
 * 
 *     ############################################################## Public Fields
 */

struct __pyx_vtabstruct_5renpy_5style_StyleCore {
  PyObject *(*_get)(struct __pyx_obj_5renpy_5style_StyleCore *, int, int __pyx_skip_dispatch);
  PyObject *(*_get_unoffset)(struct __pyx_obj_5renpy_5style_StyleCore *, int, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5renpy_5style_StyleCore *__pyx_vtabptr_5renpy_5style_StyleCore;


/* "gen/styleclass.pxi":1
 * cdef class Style(StyleCore):             # <<<<<<<<<<<<<<
 * 
 *     property black_color:
 */

struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style {
  struct __pyx_vtabstruct_5renpy_5style_StyleCore __pyx_base;
};
static struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *__pyx_vtabptr_5renpy_9styledata_10styleclass_Style;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'renpy.style' */
static PyTypeObject *__pyx_ptype_5renpy_5style_StyleCore = 0;

/* Module declarations from 'renpy.styledata.styleclass' */
static PyTypeObject *__pyx_ptype_5renpy_9styledata_10styleclass_Style = 0;
#define __Pyx_MODULE_NAME "renpy.styledata.styleclass"
extern int __pyx_module_is_main_renpy__styledata__styleclass;
int __pyx_module_is_main_renpy__styledata__styleclass = 0;

/* Implementation of 'renpy.styledata.styleclass' */
static PyObject *__pyx_builtin_TypeError;
static const char __pyx_k_alt[] = "alt";
static const char __pyx_k_args[] = "_args";
static const char __pyx_k_bold[] = "bold";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_font[] = "font";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_xfit[] = "xfit";
static const char __pyx_k_xpos[] = "xpos";
static const char __pyx_k_yfit[] = "yfit";
static const char __pyx_k_ypos[] = "ypos";
static const char __pyx_k_caret[] = "caret";
static const char __pyx_k_child[] = "child";
static const char __pyx_k_color[] = "color";
static const char __pyx_k_debug[] = "debug";
static const char __pyx_k_mouse[] = "mouse";
static const char __pyx_k_thumb[] = "thumb";
static const char __pyx_k_xfill[] = "xfill";
static const char __pyx_k_yfill[] = "yfill";
static const char __pyx_k_italic[] = "italic";
static const char __pyx_k_layout[] = "layout";
static const char __pyx_k_prefix[] = "prefix";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_unique[] = "_unique";
static const char __pyx_k_aft_bar[] = "aft_bar";
static const char __pyx_k_delattr[] = "delattr";
static const char __pyx_k_hinting[] = "hinting";
static const char __pyx_k_justify[] = "justify";
static const char __pyx_k_kerning[] = "kerning";
static const char __pyx_k_spacing[] = "spacing";
static const char __pyx_k_xanchor[] = "xanchor";
static const char __pyx_k_xoffset[] = "xoffset";
static const char __pyx_k_yanchor[] = "yanchor";
static const char __pyx_k_yoffset[] = "yoffset";
static const char __pyx_k_box_wrap[] = "box_wrap";
static const char __pyx_k_clipping[] = "clipping";
static const char __pyx_k_fore_bar[] = "fore_bar";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_language[] = "language";
static const char __pyx_k_outlines[] = "outlines";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_slow_cps[] = "slow_cps";
static const char __pyx_k_subpixel[] = "subpixel";
static const char __pyx_k_vertical[] = "vertical";
static const char __pyx_k_xmaximum[] = "xmaximum";
static const char __pyx_k_xminimum[] = "xminimum";
static const char __pyx_k_xspacing[] = "xspacing";
static const char __pyx_k_ymaximum[] = "ymaximum";
static const char __pyx_k_yminimum[] = "yminimum";
static const char __pyx_k_yspacing[] = "yspacing";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_antialias[] = "antialias";
static const char __pyx_k_duplicate[] = "_duplicate";
static const char __pyx_k_fit_first[] = "fit_first";
static const char __pyx_k_min_width[] = "min_width";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_underline[] = "underline";
static const char __pyx_k_aft_gutter[] = "aft_gutter";
static const char __pyx_k_background[] = "background";
static const char __pyx_k_bar_invert[] = "bar_invert";
static const char __pyx_k_box_layout[] = "box_layout";
static const char __pyx_k_focus_mask[] = "focus_mask";
static const char __pyx_k_focus_rect[] = "focus_rect";
static const char __pyx_k_foreground[] = "foreground";
static const char __pyx_k_key_events[] = "key_events";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_ruby_style[] = "ruby_style";
static const char __pyx_k_size_group[] = "size_group";
static const char __pyx_k_text_align[] = "text_align";
static const char __pyx_k_top_margin[] = "top_margin";
static const char __pyx_k_black_color[] = "black_color";
static const char __pyx_k_box_reverse[] = "box_reverse";
static const char __pyx_k_drop_shadow[] = "drop_shadow";
static const char __pyx_k_fore_gutter[] = "fore_gutter";
static const char __pyx_k_hover_sound[] = "hover_sound";
static const char __pyx_k_left_margin[] = "left_margin";
static const char __pyx_k_rest_indent[] = "rest_indent";
static const char __pyx_k_time_policy[] = "time_policy";
static const char __pyx_k_top_padding[] = "top_padding";
static const char __pyx_k_bar_resizing[] = "bar_resizing";
static const char __pyx_k_bar_vertical[] = "bar_vertical";
static const char __pyx_k_duplicatable[] = "_duplicatable";
static const char __pyx_k_first_indent[] = "first_indent";
static const char __pyx_k_left_padding[] = "left_padding";
static const char __pyx_k_line_leading[] = "line_leading";
static const char __pyx_k_line_spacing[] = "line_spacing";
static const char __pyx_k_right_margin[] = "right_margin";
static const char __pyx_k_text_y_fudge[] = "text_y_fudge";
static const char __pyx_k_thumb_offset[] = "thumb_offset";
static const char __pyx_k_thumb_shadow[] = "thumb_shadow";
static const char __pyx_k_unscrollable[] = "unscrollable";
static const char __pyx_k_altruby_style[] = "altruby_style";
static const char __pyx_k_bottom_margin[] = "bottom_margin";
static const char __pyx_k_first_spacing[] = "first_spacing";
static const char __pyx_k_order_reverse[] = "order_reverse";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_right_padding[] = "right_padding";
static const char __pyx_k_strikethrough[] = "strikethrough";
static const char __pyx_k_activate_sound[] = "activate_sound";
static const char __pyx_k_adjust_spacing[] = "adjust_spacing";
static const char __pyx_k_bottom_padding[] = "bottom_padding";
static const char __pyx_k_keyboard_focus[] = "keyboard_focus";
static const char __pyx_k_newline_indent[] = "newline_indent";
static const char __pyx_k_slow_abortable[] = "slow_abortable";
static const char __pyx_k_subtitle_width[] = "subtitle_width";
static const char __pyx_k_outline_scaling[] = "outline_scaling";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_box_wrap_spacing[] = "box_wrap_spacing";
static const char __pyx_k_drop_shadow_color[] = "drop_shadow_color";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_line_overlap_split[] = "line_overlap_split";
static const char __pyx_k_hyperlink_functions[] = "hyperlink_functions";
static const char __pyx_k_slow_cps_multiplier[] = "slow_cps_multiplier";
static const char __pyx_k_self_cache_cannot_be_converted_t[] = "self.cache cannot be converted to a Python object for pickling";
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_activate_sound;
static PyObject *__pyx_n_s_adjust_spacing;
static PyObject *__pyx_n_s_aft_bar;
static PyObject *__pyx_n_s_aft_gutter;
static PyObject *__pyx_n_s_alt;
static PyObject *__pyx_n_s_altruby_style;
static PyObject *__pyx_n_s_antialias;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_background;
static PyObject *__pyx_n_s_bar_invert;
static PyObject *__pyx_n_s_bar_resizing;
static PyObject *__pyx_n_s_bar_vertical;
static PyObject *__pyx_n_s_black_color;
static PyObject *__pyx_n_s_bold;
static PyObject *__pyx_n_s_bottom_margin;
static PyObject *__pyx_n_s_bottom_padding;
static PyObject *__pyx_n_s_box_layout;
static PyObject *__pyx_n_s_box_reverse;
static PyObject *__pyx_n_s_box_wrap;
static PyObject *__pyx_n_s_box_wrap_spacing;
static PyObject *__pyx_n_s_caret;
static PyObject *__pyx_n_s_child;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_clipping;
static PyObject *__pyx_n_s_color;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_debug;
static PyObject *__pyx_n_s_delattr;
static PyObject *__pyx_n_s_drop_shadow;
static PyObject *__pyx_n_s_drop_shadow_color;
static PyObject *__pyx_n_s_duplicatable;
static PyObject *__pyx_n_s_duplicate;
static PyObject *__pyx_n_s_first_indent;
static PyObject *__pyx_n_s_first_spacing;
static PyObject *__pyx_n_s_fit_first;
static PyObject *__pyx_n_s_focus_mask;
static PyObject *__pyx_n_s_focus_rect;
static PyObject *__pyx_n_s_font;
static PyObject *__pyx_n_s_fore_bar;
static PyObject *__pyx_n_s_fore_gutter;
static PyObject *__pyx_n_s_foreground;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_hinting;
static PyObject *__pyx_n_s_hover_sound;
static PyObject *__pyx_n_s_hyperlink_functions;
static PyObject *__pyx_n_s_italic;
static PyObject *__pyx_n_s_justify;
static PyObject *__pyx_n_s_kerning;
static PyObject *__pyx_n_s_key_events;
static PyObject *__pyx_n_s_keyboard_focus;
static PyObject *__pyx_n_s_language;
static PyObject *__pyx_n_s_layout;
static PyObject *__pyx_n_s_left_margin;
static PyObject *__pyx_n_s_left_padding;
static PyObject *__pyx_n_s_line_leading;
static PyObject *__pyx_n_s_line_overlap_split;
static PyObject *__pyx_n_s_line_spacing;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_min_width;
static PyObject *__pyx_n_s_mouse;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_newline_indent;
static PyObject *__pyx_n_s_order_reverse;
static PyObject *__pyx_n_s_outline_scaling;
static PyObject *__pyx_n_s_outlines;
static PyObject *__pyx_n_s_prefix;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_rest_indent;
static PyObject *__pyx_n_s_right_margin;
static PyObject *__pyx_n_s_right_padding;
static PyObject *__pyx_n_s_ruby_style;
static PyObject *__pyx_kp_s_self_cache_cannot_be_converted_t;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_size_group;
static PyObject *__pyx_n_s_slow_abortable;
static PyObject *__pyx_n_s_slow_cps;
static PyObject *__pyx_n_s_slow_cps_multiplier;
static PyObject *__pyx_n_s_spacing;
static PyObject *__pyx_n_s_strikethrough;
static PyObject *__pyx_n_s_subpixel;
static PyObject *__pyx_n_s_subtitle_width;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_text_align;
static PyObject *__pyx_n_s_text_y_fudge;
static PyObject *__pyx_n_s_thumb;
static PyObject *__pyx_n_s_thumb_offset;
static PyObject *__pyx_n_s_thumb_shadow;
static PyObject *__pyx_n_s_time_policy;
static PyObject *__pyx_n_s_top_margin;
static PyObject *__pyx_n_s_top_padding;
static PyObject *__pyx_n_s_underline;
static PyObject *__pyx_n_s_unique;
static PyObject *__pyx_n_s_unscrollable;
static PyObject *__pyx_n_s_vertical;
static PyObject *__pyx_n_s_xanchor;
static PyObject *__pyx_n_s_xfill;
static PyObject *__pyx_n_s_xfit;
static PyObject *__pyx_n_s_xmaximum;
static PyObject *__pyx_n_s_xminimum;
static PyObject *__pyx_n_s_xoffset;
static PyObject *__pyx_n_s_xpos;
static PyObject *__pyx_n_s_xspacing;
static PyObject *__pyx_n_s_yanchor;
static PyObject *__pyx_n_s_yfill;
static PyObject *__pyx_n_s_yfit;
static PyObject *__pyx_n_s_ymaximum;
static PyObject *__pyx_n_s_yminimum;
static PyObject *__pyx_n_s_yoffset;
static PyObject *__pyx_n_s_ypos;
static PyObject *__pyx_n_s_yspacing;
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4font___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10background___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8language___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5color___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4size___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5child___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_5renpy_9styledata_10styleclass_Style(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__91;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__93;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__95;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__97;
static PyObject *__pyx_tuple__98;

/* "gen/styleclass.pxi":4
 * 
 *     property black_color:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(0)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":5
 *     property black_color:
 *         def __get__(self):
 *             return self._get(0)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'black_color' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":4
 * 
 *     property black_color:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(0)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.black_color.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":6
 *         def __get__(self):
 *             return self._get(0)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'black_color' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":7
 *             return self._get(0)
 *         def __set__(self, value):
 *             self.properties.append({ 'black_color' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('black_color')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 7, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_black_color, __pyx_v_value) < 0) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":6
 *         def __get__(self):
 *             return self._get(0)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'black_color' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.black_color.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":8
 *         def __set__(self, value):
 *             self.properties.append({ 'black_color' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('black_color')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11black_color_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":9
 *             self.properties.append({ 'black_color' : value })
 *         def __del__(self):
 *             self.delattr('black_color')             # <<<<<<<<<<<<<<
 * 
 *     property bottom_margin:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":8
 *         def __set__(self, value):
 *             self.properties.append({ 'black_color' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('black_color')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.black_color.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":12
 * 
 *     property bottom_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(1)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":13
 *     property bottom_margin:
 *         def __get__(self):
 *             return self._get(1)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_margin' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":12
 * 
 *     property bottom_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(1)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bottom_margin.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":14
 *         def __get__(self):
 *             return self._get(1)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bottom_margin' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":15
 *             return self._get(1)
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_margin' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('bottom_margin')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 15, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bottom_margin, __pyx_v_value) < 0) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":14
 *         def __get__(self):
 *             return self._get(1)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bottom_margin' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bottom_margin.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":16
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bottom_margin')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13bottom_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":17
 *             self.properties.append({ 'bottom_margin' : value })
 *         def __del__(self):
 *             self.delattr('bottom_margin')             # <<<<<<<<<<<<<<
 * 
 *     property line_spacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":16
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bottom_margin')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bottom_margin.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":20
 * 
 *     property line_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(2)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":21
 *     property line_spacing:
 *         def __get__(self):
 *             return self._get(2)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'line_spacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":20
 * 
 *     property line_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(2)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_spacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":22
 *         def __get__(self):
 *             return self._get(2)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'line_spacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":23
 *             return self._get(2)
 *         def __set__(self, value):
 *             self.properties.append({ 'line_spacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('line_spacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 23, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_line_spacing, __pyx_v_value) < 0) __PYX_ERR(1, 23, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":22
 *         def __get__(self):
 *             return self._get(2)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'line_spacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_spacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":24
 *         def __set__(self, value):
 *             self.properties.append({ 'line_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('line_spacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":25
 *             self.properties.append({ 'line_spacing' : value })
 *         def __del__(self):
 *             self.delattr('line_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property bar_resizing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":24
 *         def __set__(self, value):
 *             self.properties.append({ 'line_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('line_spacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_spacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":28
 * 
 *     property bar_resizing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(3)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":29
 *     property bar_resizing:
 *         def __get__(self):
 *             return self._get(3)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_resizing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 3, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":28
 * 
 *     property bar_resizing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(3)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_resizing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":30
 *         def __get__(self):
 *             return self._get(3)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bar_resizing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":31
 *             return self._get(3)
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_resizing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('bar_resizing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 31, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bar_resizing, __pyx_v_value) < 0) __PYX_ERR(1, 31, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":30
 *         def __get__(self):
 *             return self._get(3)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bar_resizing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_resizing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":32
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_resizing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bar_resizing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_resizing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":33
 *             self.properties.append({ 'bar_resizing' : value })
 *         def __del__(self):
 *             self.delattr('bar_resizing')             # <<<<<<<<<<<<<<
 * 
 *     property left_margin:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":32
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_resizing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bar_resizing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_resizing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":36
 * 
 *     property left_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(4)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":37
 *     property left_margin:
 *         def __get__(self):
 *             return self._get(4)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'left_margin' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 4, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":36
 * 
 *     property left_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(4)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.left_margin.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":38
 *         def __get__(self):
 *             return self._get(4)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'left_margin' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":39
 *             return self._get(4)
 *         def __set__(self, value):
 *             self.properties.append({ 'left_margin' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('left_margin')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 39, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_left_margin, __pyx_v_value) < 0) __PYX_ERR(1, 39, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":38
 *         def __get__(self):
 *             return self._get(4)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'left_margin' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.left_margin.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":40
 *         def __set__(self, value):
 *             self.properties.append({ 'left_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('left_margin')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11left_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":41
 *             self.properties.append({ 'left_margin' : value })
 *         def __del__(self):
 *             self.delattr('left_margin')             # <<<<<<<<<<<<<<
 * 
 *     property yminimum:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":40
 *         def __set__(self, value):
 *             self.properties.append({ 'left_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('left_margin')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.left_margin.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":44
 * 
 *     property yminimum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(5)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":45
 *     property yminimum:
 *         def __get__(self):
 *             return self._get(5)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'yminimum' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 5, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":44
 * 
 *     property yminimum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(5)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yminimum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":46
 *         def __get__(self):
 *             return self._get(5)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yminimum' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":47
 *             return self._get(5)
 *         def __set__(self, value):
 *             self.properties.append({ 'yminimum' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('yminimum')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 47, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_yminimum, __pyx_v_value) < 0) __PYX_ERR(1, 47, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":46
 *         def __get__(self):
 *             return self._get(5)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yminimum' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yminimum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":48
 *         def __set__(self, value):
 *             self.properties.append({ 'yminimum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yminimum')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yminimum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":49
 *             self.properties.append({ 'yminimum' : value })
 *         def __del__(self):
 *             self.delattr('yminimum')             # <<<<<<<<<<<<<<
 * 
 *     property alt:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":48
 *         def __set__(self, value):
 *             self.properties.append({ 'yminimum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yminimum')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yminimum.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":52
 * 
 *     property alt:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(6)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":53
 *     property alt:
 *         def __get__(self):
 *             return self._get(6)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'alt' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 6, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":52
 * 
 *     property alt:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(6)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.alt.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":54
 *         def __get__(self):
 *             return self._get(6)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'alt' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":55
 *             return self._get(6)
 *         def __set__(self, value):
 *             self.properties.append({ 'alt' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('alt')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 55, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_alt, __pyx_v_value) < 0) __PYX_ERR(1, 55, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 55, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":54
 *         def __get__(self):
 *             return self._get(6)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'alt' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.alt.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":56
 *         def __set__(self, value):
 *             self.properties.append({ 'alt' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('alt')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_3alt_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":57
 *             self.properties.append({ 'alt' : value })
 *         def __del__(self):
 *             self.delattr('alt')             # <<<<<<<<<<<<<<
 * 
 *     property font:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":56
 *         def __set__(self, value):
 *             self.properties.append({ 'alt' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('alt')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.alt.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":60
 * 
 *     property font:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(7)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4font___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":61
 *     property font:
 *         def __get__(self):
 *             return self._get(7)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'font' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 7, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":60
 * 
 *     property font:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(7)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.font.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":62
 *         def __get__(self):
 *             return self._get(7)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'font' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":63
 *             return self._get(7)
 *         def __set__(self, value):
 *             self.properties.append({ 'font' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('font')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 63, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_font, __pyx_v_value) < 0) __PYX_ERR(1, 63, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":62
 *         def __get__(self):
 *             return self._get(7)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'font' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.font.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":64
 *         def __set__(self, value):
 *             self.properties.append({ 'font' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('font')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4font_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":65
 *             self.properties.append({ 'font' : value })
 *         def __del__(self):
 *             self.delattr('font')             # <<<<<<<<<<<<<<
 * 
 *     property underline:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":64
 *         def __set__(self, value):
 *             self.properties.append({ 'font' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('font')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.font.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":68
 * 
 *     property underline:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(8)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":69
 *     property underline:
 *         def __get__(self):
 *             return self._get(8)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'underline' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 8, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 69, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":68
 * 
 *     property underline:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(8)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.underline.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":70
 *         def __get__(self):
 *             return self._get(8)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'underline' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":71
 *             return self._get(8)
 *         def __set__(self, value):
 *             self.properties.append({ 'underline' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('underline')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 71, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_underline, __pyx_v_value) < 0) __PYX_ERR(1, 71, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":70
 *         def __get__(self):
 *             return self._get(8)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'underline' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.underline.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":72
 *         def __set__(self, value):
 *             self.properties.append({ 'underline' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('underline')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9underline_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":73
 *             self.properties.append({ 'underline' : value })
 *         def __del__(self):
 *             self.delattr('underline')             # <<<<<<<<<<<<<<
 * 
 *     property layout:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":72
 *         def __set__(self, value):
 *             self.properties.append({ 'underline' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('underline')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.underline.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":76
 * 
 *     property layout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(9)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":77
 *     property layout:
 *         def __get__(self):
 *             return self._get(9)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'layout' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 9, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":76
 * 
 *     property layout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(9)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.layout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":78
 *         def __get__(self):
 *             return self._get(9)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'layout' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":79
 *             return self._get(9)
 *         def __set__(self, value):
 *             self.properties.append({ 'layout' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('layout')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 79, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_layout, __pyx_v_value) < 0) __PYX_ERR(1, 79, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":78
 *         def __get__(self):
 *             return self._get(9)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'layout' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.layout.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":80
 *         def __set__(self, value):
 *             self.properties.append({ 'layout' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('layout')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6layout_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":81
 *             self.properties.append({ 'layout' : value })
 *         def __del__(self):
 *             self.delattr('layout')             # <<<<<<<<<<<<<<
 * 
 *     property thumb:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":80
 *         def __set__(self, value):
 *             self.properties.append({ 'layout' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('layout')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.layout.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":84
 * 
 *     property thumb:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(10)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":85
 *     property thumb:
 *         def __get__(self):
 *             return self._get(10)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 10, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":84
 * 
 *     property thumb:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(10)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":86
 *         def __get__(self):
 *             return self._get(10)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'thumb' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":87
 *             return self._get(10)
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('thumb')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 87, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_thumb, __pyx_v_value) < 0) __PYX_ERR(1, 87, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":86
 *         def __get__(self):
 *             return self._get(10)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'thumb' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":88
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('thumb')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5thumb_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":89
 *             self.properties.append({ 'thumb' : value })
 *         def __del__(self):
 *             self.delattr('thumb')             # <<<<<<<<<<<<<<
 * 
 *     property box_reverse:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":88
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('thumb')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":92
 * 
 *     property box_reverse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(11)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":93
 *     property box_reverse:
 *         def __get__(self):
 *             return self._get(11)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'box_reverse' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 11, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":92
 * 
 *     property box_reverse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(11)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_reverse.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":94
 *         def __get__(self):
 *             return self._get(11)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_reverse' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":95
 *             return self._get(11)
 *         def __set__(self, value):
 *             self.properties.append({ 'box_reverse' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('box_reverse')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 95, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_box_reverse, __pyx_v_value) < 0) __PYX_ERR(1, 95, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 95, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":94
 *         def __get__(self):
 *             return self._get(11)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_reverse' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_reverse.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":96
 *         def __set__(self, value):
 *             self.properties.append({ 'box_reverse' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_reverse')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11box_reverse_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":97
 *             self.properties.append({ 'box_reverse' : value })
 *         def __del__(self):
 *             self.delattr('box_reverse')             # <<<<<<<<<<<<<<
 * 
 *     property outline_scaling:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":96
 *         def __set__(self, value):
 *             self.properties.append({ 'box_reverse' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_reverse')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_reverse.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":100
 * 
 *     property outline_scaling:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(12)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":101
 *     property outline_scaling:
 *         def __get__(self):
 *             return self._get(12)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'outline_scaling' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 12, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":100
 * 
 *     property outline_scaling:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(12)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.outline_scaling.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":102
 *         def __get__(self):
 *             return self._get(12)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'outline_scaling' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":103
 *             return self._get(12)
 *         def __set__(self, value):
 *             self.properties.append({ 'outline_scaling' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('outline_scaling')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 103, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_outline_scaling, __pyx_v_value) < 0) __PYX_ERR(1, 103, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 103, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":102
 *         def __get__(self):
 *             return self._get(12)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'outline_scaling' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.outline_scaling.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":104
 *         def __set__(self, value):
 *             self.properties.append({ 'outline_scaling' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('outline_scaling')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_15outline_scaling_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":105
 *             self.properties.append({ 'outline_scaling' : value })
 *         def __del__(self):
 *             self.delattr('outline_scaling')             # <<<<<<<<<<<<<<
 * 
 *     property text_y_fudge:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":104
 *         def __set__(self, value):
 *             self.properties.append({ 'outline_scaling' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('outline_scaling')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.outline_scaling.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":108
 * 
 *     property text_y_fudge:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(13)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":109
 *     property text_y_fudge:
 *         def __get__(self):
 *             return self._get(13)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'text_y_fudge' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 13, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":108
 * 
 *     property text_y_fudge:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(13)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.text_y_fudge.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":110
 *         def __get__(self):
 *             return self._get(13)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'text_y_fudge' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":111
 *             return self._get(13)
 *         def __set__(self, value):
 *             self.properties.append({ 'text_y_fudge' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('text_y_fudge')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 111, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_text_y_fudge, __pyx_v_value) < 0) __PYX_ERR(1, 111, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 111, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":110
 *         def __get__(self):
 *             return self._get(13)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'text_y_fudge' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.text_y_fudge.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":112
 *         def __set__(self, value):
 *             self.properties.append({ 'text_y_fudge' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('text_y_fudge')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":113
 *             self.properties.append({ 'text_y_fudge' : value })
 *         def __del__(self):
 *             self.delattr('text_y_fudge')             # <<<<<<<<<<<<<<
 * 
 *     property aft_gutter:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":112
 *         def __set__(self, value):
 *             self.properties.append({ 'text_y_fudge' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('text_y_fudge')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.text_y_fudge.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":116
 * 
 *     property aft_gutter:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(14)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":117
 *     property aft_gutter:
 *         def __get__(self):
 *             return self._get(14)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_gutter' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 14, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":116
 * 
 *     property aft_gutter:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(14)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.aft_gutter.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":118
 *         def __get__(self):
 *             return self._get(14)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'aft_gutter' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":119
 *             return self._get(14)
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_gutter' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('aft_gutter')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 119, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_aft_gutter, __pyx_v_value) < 0) __PYX_ERR(1, 119, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 119, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":118
 *         def __get__(self):
 *             return self._get(14)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'aft_gutter' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.aft_gutter.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":120
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_gutter' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('aft_gutter')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10aft_gutter_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":121
 *             self.properties.append({ 'aft_gutter' : value })
 *         def __del__(self):
 *             self.delattr('aft_gutter')             # <<<<<<<<<<<<<<
 * 
 *     property right_margin:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":120
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_gutter' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('aft_gutter')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.aft_gutter.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":124
 * 
 *     property right_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(15)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":125
 *     property right_margin:
 *         def __get__(self):
 *             return self._get(15)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'right_margin' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 15, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":124
 * 
 *     property right_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(15)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.right_margin.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":126
 *         def __get__(self):
 *             return self._get(15)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'right_margin' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":127
 *             return self._get(15)
 *         def __set__(self, value):
 *             self.properties.append({ 'right_margin' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('right_margin')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 127, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_right_margin, __pyx_v_value) < 0) __PYX_ERR(1, 127, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 127, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":126
 *         def __get__(self):
 *             return self._get(15)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'right_margin' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.right_margin.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":128
 *         def __set__(self, value):
 *             self.properties.append({ 'right_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('right_margin')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12right_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":129
 *             self.properties.append({ 'right_margin' : value })
 *         def __del__(self):
 *             self.delattr('right_margin')             # <<<<<<<<<<<<<<
 * 
 *     property rest_indent:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":128
 *         def __set__(self, value):
 *             self.properties.append({ 'right_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('right_margin')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.right_margin.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":132
 * 
 *     property rest_indent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(16)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":133
 *     property rest_indent:
 *         def __get__(self):
 *             return self._get(16)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'rest_indent' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 16, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":132
 * 
 *     property rest_indent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(16)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.rest_indent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":134
 *         def __get__(self):
 *             return self._get(16)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'rest_indent' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":135
 *             return self._get(16)
 *         def __set__(self, value):
 *             self.properties.append({ 'rest_indent' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('rest_indent')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 135, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_rest_indent, __pyx_v_value) < 0) __PYX_ERR(1, 135, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":134
 *         def __get__(self):
 *             return self._get(16)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'rest_indent' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.rest_indent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":136
 *         def __set__(self, value):
 *             self.properties.append({ 'rest_indent' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('rest_indent')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11rest_indent_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":137
 *             self.properties.append({ 'rest_indent' : value })
 *         def __del__(self):
 *             self.delattr('rest_indent')             # <<<<<<<<<<<<<<
 * 
 *     property background:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":136
 *         def __set__(self, value):
 *             self.properties.append({ 'rest_indent' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('rest_indent')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.rest_indent.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":140
 * 
 *     property background:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(17)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10background___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":141
 *     property background:
 *         def __get__(self):
 *             return self._get(17)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'background' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 17, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":140
 * 
 *     property background:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(17)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.background.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":142
 *         def __get__(self):
 *             return self._get(17)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'background' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":143
 *             return self._get(17)
 *         def __set__(self, value):
 *             self.properties.append({ 'background' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('background')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 143, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_background, __pyx_v_value) < 0) __PYX_ERR(1, 143, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 143, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":142
 *         def __get__(self):
 *             return self._get(17)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'background' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.background.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":144
 *         def __set__(self, value):
 *             self.properties.append({ 'background' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('background')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10background_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":145
 *             self.properties.append({ 'background' : value })
 *         def __del__(self):
 *             self.delattr('background')             # <<<<<<<<<<<<<<
 * 
 *     property outlines:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":144
 *         def __set__(self, value):
 *             self.properties.append({ 'background' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('background')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.background.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":148
 * 
 *     property outlines:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(18)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":149
 *     property outlines:
 *         def __get__(self):
 *             return self._get(18)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'outlines' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 18, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":148
 * 
 *     property outlines:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(18)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.outlines.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":150
 *         def __get__(self):
 *             return self._get(18)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'outlines' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":151
 *             return self._get(18)
 *         def __set__(self, value):
 *             self.properties.append({ 'outlines' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('outlines')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 151, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_outlines, __pyx_v_value) < 0) __PYX_ERR(1, 151, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":150
 *         def __get__(self):
 *             return self._get(18)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'outlines' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.outlines.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":152
 *         def __set__(self, value):
 *             self.properties.append({ 'outlines' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('outlines')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8outlines_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":153
 *             self.properties.append({ 'outlines' : value })
 *         def __del__(self):
 *             self.delattr('outlines')             # <<<<<<<<<<<<<<
 * 
 *     property ymaximum:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":152
 *         def __set__(self, value):
 *             self.properties.append({ 'outlines' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('outlines')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.outlines.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":156
 * 
 *     property ymaximum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(19)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":157
 *     property ymaximum:
 *         def __get__(self):
 *             return self._get(19)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'ymaximum' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 19, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":156
 * 
 *     property ymaximum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(19)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ymaximum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":158
 *         def __get__(self):
 *             return self._get(19)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'ymaximum' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":159
 *             return self._get(19)
 *         def __set__(self, value):
 *             self.properties.append({ 'ymaximum' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('ymaximum')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 159, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_ymaximum, __pyx_v_value) < 0) __PYX_ERR(1, 159, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 159, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":158
 *         def __get__(self):
 *             return self._get(19)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'ymaximum' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ymaximum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":160
 *         def __set__(self, value):
 *             self.properties.append({ 'ymaximum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('ymaximum')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8ymaximum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":161
 *             self.properties.append({ 'ymaximum' : value })
 *         def __del__(self):
 *             self.delattr('ymaximum')             # <<<<<<<<<<<<<<
 * 
 *     property yspacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":160
 *         def __set__(self, value):
 *             self.properties.append({ 'ymaximum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('ymaximum')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ymaximum.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":164
 * 
 *     property yspacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(20)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":165
 *     property yspacing:
 *         def __get__(self):
 *             return self._get(20)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'yspacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 20, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":164
 * 
 *     property yspacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(20)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yspacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":166
 *         def __get__(self):
 *             return self._get(20)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yspacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":167
 *             return self._get(20)
 *         def __set__(self, value):
 *             self.properties.append({ 'yspacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('yspacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 167, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_yspacing, __pyx_v_value) < 0) __PYX_ERR(1, 167, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":166
 *         def __get__(self):
 *             return self._get(20)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yspacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yspacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":168
 *         def __set__(self, value):
 *             self.properties.append({ 'yspacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yspacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8yspacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":169
 *             self.properties.append({ 'yspacing' : value })
 *         def __del__(self):
 *             self.delattr('yspacing')             # <<<<<<<<<<<<<<
 * 
 *     property size_group:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":168
 *         def __set__(self, value):
 *             self.properties.append({ 'yspacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yspacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yspacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":172
 * 
 *     property size_group:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(21)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":173
 *     property size_group:
 *         def __get__(self):
 *             return self._get(21)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'size_group' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 21, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":172
 * 
 *     property size_group:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(21)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.size_group.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":174
 *         def __get__(self):
 *             return self._get(21)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'size_group' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":175
 *             return self._get(21)
 *         def __set__(self, value):
 *             self.properties.append({ 'size_group' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('size_group')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 175, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_size_group, __pyx_v_value) < 0) __PYX_ERR(1, 175, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":174
 *         def __get__(self):
 *             return self._get(21)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'size_group' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.size_group.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":176
 *         def __set__(self, value):
 *             self.properties.append({ 'size_group' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('size_group')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10size_group_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":177
 *             self.properties.append({ 'size_group' : value })
 *         def __del__(self):
 *             self.delattr('size_group')             # <<<<<<<<<<<<<<
 * 
 *     property debug:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":176
 *         def __set__(self, value):
 *             self.properties.append({ 'size_group' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('size_group')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.size_group.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":180
 * 
 *     property debug:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(22)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":181
 *     property debug:
 *         def __get__(self):
 *             return self._get(22)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'debug' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 22, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":180
 * 
 *     property debug:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(22)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.debug.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":182
 *         def __get__(self):
 *             return self._get(22)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'debug' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":183
 *             return self._get(22)
 *         def __set__(self, value):
 *             self.properties.append({ 'debug' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('debug')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 183, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_debug, __pyx_v_value) < 0) __PYX_ERR(1, 183, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 183, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":182
 *         def __get__(self):
 *             return self._get(22)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'debug' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.debug.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":184
 *         def __set__(self, value):
 *             self.properties.append({ 'debug' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('debug')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5debug_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":185
 *             self.properties.append({ 'debug' : value })
 *         def __del__(self):
 *             self.delattr('debug')             # <<<<<<<<<<<<<<
 * 
 *     property clipping:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":184
 *         def __set__(self, value):
 *             self.properties.append({ 'debug' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('debug')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.debug.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":188
 * 
 *     property clipping:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(23)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":189
 *     property clipping:
 *         def __get__(self):
 *             return self._get(23)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'clipping' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 23, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":188
 * 
 *     property clipping:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(23)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.clipping.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":190
 *         def __get__(self):
 *             return self._get(23)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'clipping' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":191
 *             return self._get(23)
 *         def __set__(self, value):
 *             self.properties.append({ 'clipping' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('clipping')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 191, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_clipping, __pyx_v_value) < 0) __PYX_ERR(1, 191, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 191, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":190
 *         def __get__(self):
 *             return self._get(23)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'clipping' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.clipping.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":192
 *         def __set__(self, value):
 *             self.properties.append({ 'clipping' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('clipping')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8clipping_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":193
 *             self.properties.append({ 'clipping' : value })
 *         def __del__(self):
 *             self.delattr('clipping')             # <<<<<<<<<<<<<<
 * 
 *     property first_spacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":192
 *         def __set__(self, value):
 *             self.properties.append({ 'clipping' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('clipping')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.clipping.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":196
 * 
 *     property first_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(24)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":197
 *     property first_spacing:
 *         def __get__(self):
 *             return self._get(24)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'first_spacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 24, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":196
 * 
 *     property first_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(24)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.first_spacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":198
 *         def __get__(self):
 *             return self._get(24)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'first_spacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":199
 *             return self._get(24)
 *         def __set__(self, value):
 *             self.properties.append({ 'first_spacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('first_spacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 199, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_first_spacing, __pyx_v_value) < 0) __PYX_ERR(1, 199, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 199, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":198
 *         def __get__(self):
 *             return self._get(24)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'first_spacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.first_spacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":200
 *         def __set__(self, value):
 *             self.properties.append({ 'first_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('first_spacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13first_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":201
 *             self.properties.append({ 'first_spacing' : value })
 *         def __del__(self):
 *             self.delattr('first_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property subtitle_width:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":200
 *         def __set__(self, value):
 *             self.properties.append({ 'first_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('first_spacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.first_spacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":204
 * 
 *     property subtitle_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(25)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":205
 *     property subtitle_width:
 *         def __get__(self):
 *             return self._get(25)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'subtitle_width' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 25, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":204
 * 
 *     property subtitle_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(25)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.subtitle_width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":206
 *         def __get__(self):
 *             return self._get(25)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'subtitle_width' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":207
 *             return self._get(25)
 *         def __set__(self, value):
 *             self.properties.append({ 'subtitle_width' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('subtitle_width')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 207, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_subtitle_width, __pyx_v_value) < 0) __PYX_ERR(1, 207, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 207, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":206
 *         def __get__(self):
 *             return self._get(25)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'subtitle_width' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.subtitle_width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":208
 *         def __set__(self, value):
 *             self.properties.append({ 'subtitle_width' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('subtitle_width')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14subtitle_width_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":209
 *             self.properties.append({ 'subtitle_width' : value })
 *         def __del__(self):
 *             self.delattr('subtitle_width')             # <<<<<<<<<<<<<<
 * 
 *     property subpixel:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":208
 *         def __set__(self, value):
 *             self.properties.append({ 'subtitle_width' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('subtitle_width')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.subtitle_width.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":212
 * 
 *     property subpixel:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(26)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":213
 *     property subpixel:
 *         def __get__(self):
 *             return self._get(26)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'subpixel' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 26, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":212
 * 
 *     property subpixel:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(26)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.subpixel.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":214
 *         def __get__(self):
 *             return self._get(26)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'subpixel' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":215
 *             return self._get(26)
 *         def __set__(self, value):
 *             self.properties.append({ 'subpixel' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('subpixel')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 215, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 215, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_subpixel, __pyx_v_value) < 0) __PYX_ERR(1, 215, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":214
 *         def __get__(self):
 *             return self._get(26)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'subpixel' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.subpixel.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":216
 *         def __set__(self, value):
 *             self.properties.append({ 'subpixel' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('subpixel')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8subpixel_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":217
 *             self.properties.append({ 'subpixel' : value })
 *         def __del__(self):
 *             self.delattr('subpixel')             # <<<<<<<<<<<<<<
 * 
 *     property strikethrough:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":216
 *         def __set__(self, value):
 *             self.properties.append({ 'subpixel' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('subpixel')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.subpixel.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":220
 * 
 *     property strikethrough:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(27)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":221
 *     property strikethrough:
 *         def __get__(self):
 *             return self._get(27)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'strikethrough' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 27, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 221, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":220
 * 
 *     property strikethrough:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(27)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.strikethrough.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":222
 *         def __get__(self):
 *             return self._get(27)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'strikethrough' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":223
 *             return self._get(27)
 *         def __set__(self, value):
 *             self.properties.append({ 'strikethrough' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('strikethrough')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 223, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_strikethrough, __pyx_v_value) < 0) __PYX_ERR(1, 223, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":222
 *         def __get__(self):
 *             return self._get(27)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'strikethrough' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.strikethrough.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":224
 *         def __set__(self, value):
 *             self.properties.append({ 'strikethrough' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('strikethrough')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13strikethrough_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":225
 *             self.properties.append({ 'strikethrough' : value })
 *         def __del__(self):
 *             self.delattr('strikethrough')             # <<<<<<<<<<<<<<
 * 
 *     property hyperlink_functions:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":224
 *         def __set__(self, value):
 *             self.properties.append({ 'strikethrough' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('strikethrough')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.strikethrough.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":228
 * 
 *     property hyperlink_functions:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(28)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":229
 *     property hyperlink_functions:
 *         def __get__(self):
 *             return self._get(28)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'hyperlink_functions' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 28, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":228
 * 
 *     property hyperlink_functions:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(28)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hyperlink_functions.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":230
 *         def __get__(self):
 *             return self._get(28)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'hyperlink_functions' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":231
 *             return self._get(28)
 *         def __set__(self, value):
 *             self.properties.append({ 'hyperlink_functions' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('hyperlink_functions')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 231, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_hyperlink_functions, __pyx_v_value) < 0) __PYX_ERR(1, 231, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 231, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":230
 *         def __get__(self):
 *             return self._get(28)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'hyperlink_functions' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hyperlink_functions.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":232
 *         def __set__(self, value):
 *             self.properties.append({ 'hyperlink_functions' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('hyperlink_functions')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":233
 *             self.properties.append({ 'hyperlink_functions' : value })
 *         def __del__(self):
 *             self.delattr('hyperlink_functions')             # <<<<<<<<<<<<<<
 * 
 *     property xminimum:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":232
 *         def __set__(self, value):
 *             self.properties.append({ 'hyperlink_functions' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('hyperlink_functions')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hyperlink_functions.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":236
 * 
 *     property xminimum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(29)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":237
 *     property xminimum:
 *         def __get__(self):
 *             return self._get(29)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xminimum' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 29, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":236
 * 
 *     property xminimum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(29)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xminimum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":238
 *         def __get__(self):
 *             return self._get(29)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xminimum' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":239
 *             return self._get(29)
 *         def __set__(self, value):
 *             self.properties.append({ 'xminimum' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xminimum')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 239, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xminimum, __pyx_v_value) < 0) __PYX_ERR(1, 239, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":238
 *         def __get__(self):
 *             return self._get(29)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xminimum' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xminimum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":240
 *         def __set__(self, value):
 *             self.properties.append({ 'xminimum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xminimum')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xminimum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":241
 *             self.properties.append({ 'xminimum' : value })
 *         def __del__(self):
 *             self.delattr('xminimum')             # <<<<<<<<<<<<<<
 * 
 *     property min_width:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":240
 *         def __set__(self, value):
 *             self.properties.append({ 'xminimum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xminimum')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xminimum.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":244
 * 
 *     property min_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(30)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":245
 *     property min_width:
 *         def __get__(self):
 *             return self._get(30)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'min_width' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 30, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":244
 * 
 *     property min_width:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(30)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.min_width.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":246
 *         def __get__(self):
 *             return self._get(30)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'min_width' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":247
 *             return self._get(30)
 *         def __set__(self, value):
 *             self.properties.append({ 'min_width' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('min_width')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 247, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_min_width, __pyx_v_value) < 0) __PYX_ERR(1, 247, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 247, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":246
 *         def __get__(self):
 *             return self._get(30)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'min_width' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.min_width.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":248
 *         def __set__(self, value):
 *             self.properties.append({ 'min_width' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('min_width')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9min_width_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":249
 *             self.properties.append({ 'min_width' : value })
 *         def __del__(self):
 *             self.delattr('min_width')             # <<<<<<<<<<<<<<
 * 
 *     property focus_mask:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__31, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":248
 *         def __set__(self, value):
 *             self.properties.append({ 'min_width' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('min_width')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.min_width.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":252
 * 
 *     property focus_mask:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(31)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":253
 *     property focus_mask:
 *         def __get__(self):
 *             return self._get(31)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_mask' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 31, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":252
 * 
 *     property focus_mask:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(31)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.focus_mask.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":254
 *         def __get__(self):
 *             return self._get(31)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'focus_mask' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":255
 *             return self._get(31)
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_mask' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('focus_mask')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 255, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_focus_mask, __pyx_v_value) < 0) __PYX_ERR(1, 255, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 255, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":254
 *         def __get__(self):
 *             return self._get(31)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'focus_mask' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.focus_mask.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":256
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_mask' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('focus_mask')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_mask_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":257
 *             self.properties.append({ 'focus_mask' : value })
 *         def __del__(self):
 *             self.delattr('focus_mask')             # <<<<<<<<<<<<<<
 * 
 *     property yfit:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":256
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_mask' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('focus_mask')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.focus_mask.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":260
 * 
 *     property yfit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(32)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":261
 *     property yfit:
 *         def __get__(self):
 *             return self._get(32)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'yfit' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 32, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":260
 * 
 *     property yfit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(32)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yfit.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":262
 *         def __get__(self):
 *             return self._get(32)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yfit' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":263
 *             return self._get(32)
 *         def __set__(self, value):
 *             self.properties.append({ 'yfit' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('yfit')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 263, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_yfit, __pyx_v_value) < 0) __PYX_ERR(1, 263, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":262
 *         def __get__(self):
 *             return self._get(32)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yfit' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yfit.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":264
 *         def __set__(self, value):
 *             self.properties.append({ 'yfit' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yfit')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4yfit_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":265
 *             self.properties.append({ 'yfit' : value })
 *         def __del__(self):
 *             self.delattr('yfit')             # <<<<<<<<<<<<<<
 * 
 *     property drop_shadow_color:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":264
 *         def __set__(self, value):
 *             self.properties.append({ 'yfit' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yfit')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yfit.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":268
 * 
 *     property drop_shadow_color:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(33)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":269
 *     property drop_shadow_color:
 *         def __get__(self):
 *             return self._get(33)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow_color' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 33, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":268
 * 
 *     property drop_shadow_color:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(33)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.drop_shadow_color.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":270
 *         def __get__(self):
 *             return self._get(33)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'drop_shadow_color' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":271
 *             return self._get(33)
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow_color' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('drop_shadow_color')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 271, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_drop_shadow_color, __pyx_v_value) < 0) __PYX_ERR(1, 271, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 271, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":270
 *         def __get__(self):
 *             return self._get(33)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'drop_shadow_color' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.drop_shadow_color.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":272
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow_color' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('drop_shadow_color')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":273
 *             self.properties.append({ 'drop_shadow_color' : value })
 *         def __del__(self):
 *             self.delattr('drop_shadow_color')             # <<<<<<<<<<<<<<
 * 
 *     property fit_first:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__34, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":272
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow_color' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('drop_shadow_color')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.drop_shadow_color.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":276
 * 
 *     property fit_first:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(34)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":277
 *     property fit_first:
 *         def __get__(self):
 *             return self._get(34)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'fit_first' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 34, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":276
 * 
 *     property fit_first:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(34)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fit_first.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":278
 *         def __get__(self):
 *             return self._get(34)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'fit_first' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":279
 *             return self._get(34)
 *         def __set__(self, value):
 *             self.properties.append({ 'fit_first' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('fit_first')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 279, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fit_first, __pyx_v_value) < 0) __PYX_ERR(1, 279, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 279, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":278
 *         def __get__(self):
 *             return self._get(34)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'fit_first' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fit_first.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":280
 *         def __set__(self, value):
 *             self.properties.append({ 'fit_first' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('fit_first')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9fit_first_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":281
 *             self.properties.append({ 'fit_first' : value })
 *         def __del__(self):
 *             self.delattr('fit_first')             # <<<<<<<<<<<<<<
 * 
 *     property top_padding:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":280
 *         def __set__(self, value):
 *             self.properties.append({ 'fit_first' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('fit_first')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fit_first.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":284
 * 
 *     property top_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(35)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":285
 *     property top_padding:
 *         def __get__(self):
 *             return self._get(35)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'top_padding' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 35, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":284
 * 
 *     property top_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(35)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.top_padding.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":286
 *         def __get__(self):
 *             return self._get(35)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'top_padding' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":287
 *             return self._get(35)
 *         def __set__(self, value):
 *             self.properties.append({ 'top_padding' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('top_padding')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 287, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_top_padding, __pyx_v_value) < 0) __PYX_ERR(1, 287, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 287, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":286
 *         def __get__(self):
 *             return self._get(35)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'top_padding' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.top_padding.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":288
 *         def __set__(self, value):
 *             self.properties.append({ 'top_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('top_padding')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11top_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":289
 *             self.properties.append({ 'top_padding' : value })
 *         def __del__(self):
 *             self.delattr('top_padding')             # <<<<<<<<<<<<<<
 * 
 *     property newline_indent:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":288
 *         def __set__(self, value):
 *             self.properties.append({ 'top_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('top_padding')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.top_padding.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":292
 * 
 *     property newline_indent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(36)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":293
 *     property newline_indent:
 *         def __get__(self):
 *             return self._get(36)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'newline_indent' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 36, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":292
 * 
 *     property newline_indent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(36)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.newline_indent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":294
 *         def __get__(self):
 *             return self._get(36)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'newline_indent' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":295
 *             return self._get(36)
 *         def __set__(self, value):
 *             self.properties.append({ 'newline_indent' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('newline_indent')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 295, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_newline_indent, __pyx_v_value) < 0) __PYX_ERR(1, 295, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 295, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":294
 *         def __get__(self):
 *             return self._get(36)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'newline_indent' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.newline_indent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":296
 *         def __set__(self, value):
 *             self.properties.append({ 'newline_indent' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('newline_indent')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14newline_indent_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":297
 *             self.properties.append({ 'newline_indent' : value })
 *         def __del__(self):
 *             self.delattr('newline_indent')             # <<<<<<<<<<<<<<
 * 
 *     property text_align:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":296
 *         def __set__(self, value):
 *             self.properties.append({ 'newline_indent' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('newline_indent')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.newline_indent.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":300
 * 
 *     property text_align:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(37)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":301
 *     property text_align:
 *         def __get__(self):
 *             return self._get(37)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'text_align' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 37, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":300
 * 
 *     property text_align:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(37)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.text_align.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":302
 *         def __get__(self):
 *             return self._get(37)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'text_align' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":303
 *             return self._get(37)
 *         def __set__(self, value):
 *             self.properties.append({ 'text_align' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('text_align')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 303, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_text_align, __pyx_v_value) < 0) __PYX_ERR(1, 303, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 303, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":302
 *         def __get__(self):
 *             return self._get(37)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'text_align' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.text_align.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":304
 *         def __set__(self, value):
 *             self.properties.append({ 'text_align' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('text_align')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10text_align_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":305
 *             self.properties.append({ 'text_align' : value })
 *         def __del__(self):
 *             self.delattr('text_align')             # <<<<<<<<<<<<<<
 * 
 *     property fore_bar:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":304
 *         def __set__(self, value):
 *             self.properties.append({ 'text_align' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('text_align')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.text_align.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":308
 * 
 *     property fore_bar:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(38)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":309
 *     property fore_bar:
 *         def __get__(self):
 *             return self._get(38)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_bar' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 38, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":308
 * 
 *     property fore_bar:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(38)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fore_bar.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":310
 *         def __get__(self):
 *             return self._get(38)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'fore_bar' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":311
 *             return self._get(38)
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_bar' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('fore_bar')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 311, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fore_bar, __pyx_v_value) < 0) __PYX_ERR(1, 311, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 311, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":310
 *         def __get__(self):
 *             return self._get(38)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'fore_bar' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fore_bar.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":312
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_bar' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('fore_bar')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8fore_bar_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":313
 *             self.properties.append({ 'fore_bar' : value })
 *         def __del__(self):
 *             self.delattr('fore_bar')             # <<<<<<<<<<<<<<
 * 
 *     property aft_bar:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":312
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_bar' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('fore_bar')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fore_bar.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":316
 * 
 *     property aft_bar:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(39)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":317
 *     property aft_bar:
 *         def __get__(self):
 *             return self._get(39)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_bar' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 39, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":316
 * 
 *     property aft_bar:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(39)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.aft_bar.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":318
 *         def __get__(self):
 *             return self._get(39)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'aft_bar' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":319
 *             return self._get(39)
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_bar' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('aft_bar')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 319, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_aft_bar, __pyx_v_value) < 0) __PYX_ERR(1, 319, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 319, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":318
 *         def __get__(self):
 *             return self._get(39)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'aft_bar' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.aft_bar.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":320
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_bar' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('aft_bar')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7aft_bar_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":321
 *             self.properties.append({ 'aft_bar' : value })
 *         def __del__(self):
 *             self.delattr('aft_bar')             # <<<<<<<<<<<<<<
 * 
 *     property foreground:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":320
 *         def __set__(self, value):
 *             self.properties.append({ 'aft_bar' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('aft_bar')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.aft_bar.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":324
 * 
 *     property foreground:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(40)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":325
 *     property foreground:
 *         def __get__(self):
 *             return self._get(40)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'foreground' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 40, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":324
 * 
 *     property foreground:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(40)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.foreground.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":326
 *         def __get__(self):
 *             return self._get(40)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'foreground' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":327
 *             return self._get(40)
 *         def __set__(self, value):
 *             self.properties.append({ 'foreground' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('foreground')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 327, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 327, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_foreground, __pyx_v_value) < 0) __PYX_ERR(1, 327, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 327, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":326
 *         def __get__(self):
 *             return self._get(40)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'foreground' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.foreground.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":328
 *         def __set__(self, value):
 *             self.properties.append({ 'foreground' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('foreground')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10foreground_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":329
 *             self.properties.append({ 'foreground' : value })
 *         def __del__(self):
 *             self.delattr('foreground')             # <<<<<<<<<<<<<<
 * 
 *     property bold:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__41, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":328
 *         def __set__(self, value):
 *             self.properties.append({ 'foreground' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('foreground')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.foreground.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":332
 * 
 *     property bold:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(41)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":333
 *     property bold:
 *         def __get__(self):
 *             return self._get(41)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'bold' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 41, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":332
 * 
 *     property bold:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(41)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bold.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":334
 *         def __get__(self):
 *             return self._get(41)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bold' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":335
 *             return self._get(41)
 *         def __set__(self, value):
 *             self.properties.append({ 'bold' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('bold')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 335, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bold, __pyx_v_value) < 0) __PYX_ERR(1, 335, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 335, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":334
 *         def __get__(self):
 *             return self._get(41)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bold' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bold.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":336
 *         def __set__(self, value):
 *             self.properties.append({ 'bold' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bold')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4bold_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":337
 *             self.properties.append({ 'bold' : value })
 *         def __del__(self):
 *             self.delattr('bold')             # <<<<<<<<<<<<<<
 * 
 *     property vertical:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__42, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":336
 *         def __set__(self, value):
 *             self.properties.append({ 'bold' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bold')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bold.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":340
 * 
 *     property vertical:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(42)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":341
 *     property vertical:
 *         def __get__(self):
 *             return self._get(42)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'vertical' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 42, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 341, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":340
 * 
 *     property vertical:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(42)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.vertical.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":342
 *         def __get__(self):
 *             return self._get(42)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'vertical' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":343
 *             return self._get(42)
 *         def __set__(self, value):
 *             self.properties.append({ 'vertical' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('vertical')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 343, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_vertical, __pyx_v_value) < 0) __PYX_ERR(1, 343, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 343, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":342
 *         def __get__(self):
 *             return self._get(42)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'vertical' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.vertical.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":344
 *         def __set__(self, value):
 *             self.properties.append({ 'vertical' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('vertical')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8vertical_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":345
 *             self.properties.append({ 'vertical' : value })
 *         def __del__(self):
 *             self.delattr('vertical')             # <<<<<<<<<<<<<<
 * 
 *     property drop_shadow:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__43, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":344
 *         def __set__(self, value):
 *             self.properties.append({ 'vertical' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('vertical')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.vertical.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":348
 * 
 *     property drop_shadow:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(43)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":349
 *     property drop_shadow:
 *         def __get__(self):
 *             return self._get(43)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 43, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":348
 * 
 *     property drop_shadow:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(43)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.drop_shadow.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":350
 *         def __get__(self):
 *             return self._get(43)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'drop_shadow' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":351
 *             return self._get(43)
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('drop_shadow')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 351, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_drop_shadow, __pyx_v_value) < 0) __PYX_ERR(1, 351, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 351, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":350
 *         def __get__(self):
 *             return self._get(43)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'drop_shadow' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.drop_shadow.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":352
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('drop_shadow')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11drop_shadow_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":353
 *             self.properties.append({ 'drop_shadow' : value })
 *         def __del__(self):
 *             self.delattr('drop_shadow')             # <<<<<<<<<<<<<<
 * 
 *     property spacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__44, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":352
 *         def __set__(self, value):
 *             self.properties.append({ 'drop_shadow' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('drop_shadow')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.drop_shadow.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":356
 * 
 *     property spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(44)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":357
 *     property spacing:
 *         def __get__(self):
 *             return self._get(44)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'spacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 44, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":356
 * 
 *     property spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(44)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.spacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":358
 *         def __get__(self):
 *             return self._get(44)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'spacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":359
 *             return self._get(44)
 *         def __set__(self, value):
 *             self.properties.append({ 'spacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('spacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 359, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 359, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_spacing, __pyx_v_value) < 0) __PYX_ERR(1, 359, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 359, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":358
 *         def __get__(self):
 *             return self._get(44)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'spacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.spacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":360
 *         def __set__(self, value):
 *             self.properties.append({ 'spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('spacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":361
 *             self.properties.append({ 'spacing' : value })
 *         def __del__(self):
 *             self.delattr('spacing')             # <<<<<<<<<<<<<<
 * 
 *     property ypos:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__45, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":360
 *         def __set__(self, value):
 *             self.properties.append({ 'spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('spacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.spacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":364
 * 
 *     property ypos:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(45)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":365
 *     property ypos:
 *         def __get__(self):
 *             return self._get(45)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'ypos' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 45, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":364
 * 
 *     property ypos:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(45)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ypos.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":366
 *         def __get__(self):
 *             return self._get(45)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'ypos' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":367
 *             return self._get(45)
 *         def __set__(self, value):
 *             self.properties.append({ 'ypos' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('ypos')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 367, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_ypos, __pyx_v_value) < 0) __PYX_ERR(1, 367, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 367, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":366
 *         def __get__(self):
 *             return self._get(45)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'ypos' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ypos.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":368
 *         def __set__(self, value):
 *             self.properties.append({ 'ypos' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('ypos')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4ypos_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":369
 *             self.properties.append({ 'ypos' : value })
 *         def __del__(self):
 *             self.delattr('ypos')             # <<<<<<<<<<<<<<
 * 
 *     property activate_sound:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__46, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":368
 *         def __set__(self, value):
 *             self.properties.append({ 'ypos' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('ypos')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ypos.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":372
 * 
 *     property activate_sound:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(46)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":373
 *     property activate_sound:
 *         def __get__(self):
 *             return self._get(46)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'activate_sound' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 46, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":372
 * 
 *     property activate_sound:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(46)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.activate_sound.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":374
 *         def __get__(self):
 *             return self._get(46)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'activate_sound' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":375
 *             return self._get(46)
 *         def __set__(self, value):
 *             self.properties.append({ 'activate_sound' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('activate_sound')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 375, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_activate_sound, __pyx_v_value) < 0) __PYX_ERR(1, 375, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 375, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":374
 *         def __get__(self):
 *             return self._get(46)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'activate_sound' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.activate_sound.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":376
 *         def __set__(self, value):
 *             self.properties.append({ 'activate_sound' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('activate_sound')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14activate_sound_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":377
 *             self.properties.append({ 'activate_sound' : value })
 *         def __del__(self):
 *             self.delattr('activate_sound')             # <<<<<<<<<<<<<<
 * 
 *     property left_padding:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__47, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":376
 *         def __set__(self, value):
 *             self.properties.append({ 'activate_sound' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('activate_sound')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.activate_sound.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":380
 * 
 *     property left_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(47)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":381
 *     property left_padding:
 *         def __get__(self):
 *             return self._get(47)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'left_padding' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 47, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":380
 * 
 *     property left_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(47)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.left_padding.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":382
 *         def __get__(self):
 *             return self._get(47)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'left_padding' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":383
 *             return self._get(47)
 *         def __set__(self, value):
 *             self.properties.append({ 'left_padding' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('left_padding')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 383, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_left_padding, __pyx_v_value) < 0) __PYX_ERR(1, 383, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 383, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":382
 *         def __get__(self):
 *             return self._get(47)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'left_padding' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.left_padding.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":384
 *         def __set__(self, value):
 *             self.properties.append({ 'left_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('left_padding')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12left_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":385
 *             self.properties.append({ 'left_padding' : value })
 *         def __del__(self):
 *             self.delattr('left_padding')             # <<<<<<<<<<<<<<
 * 
 *     property xoffset:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__48, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":384
 *         def __set__(self, value):
 *             self.properties.append({ 'left_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('left_padding')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.left_padding.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":388
 * 
 *     property xoffset:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(48)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":389
 *     property xoffset:
 *         def __get__(self):
 *             return self._get(48)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xoffset' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 48, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":388
 * 
 *     property xoffset:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(48)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xoffset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":390
 *         def __get__(self):
 *             return self._get(48)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xoffset' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":391
 *             return self._get(48)
 *         def __set__(self, value):
 *             self.properties.append({ 'xoffset' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xoffset')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 391, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xoffset, __pyx_v_value) < 0) __PYX_ERR(1, 391, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 391, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":390
 *         def __get__(self):
 *             return self._get(48)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xoffset' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xoffset.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":392
 *         def __set__(self, value):
 *             self.properties.append({ 'xoffset' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xoffset')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xoffset_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":393
 *             self.properties.append({ 'xoffset' : value })
 *         def __del__(self):
 *             self.delattr('xoffset')             # <<<<<<<<<<<<<<
 * 
 *     property language:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":392
 *         def __set__(self, value):
 *             self.properties.append({ 'xoffset' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xoffset')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xoffset.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":396
 * 
 *     property language:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(49)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8language___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":397
 *     property language:
 *         def __get__(self):
 *             return self._get(49)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'language' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 49, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":396
 * 
 *     property language:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(49)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.language.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":398
 *         def __get__(self):
 *             return self._get(49)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'language' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":399
 *             return self._get(49)
 *         def __set__(self, value):
 *             self.properties.append({ 'language' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('language')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 399, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_language, __pyx_v_value) < 0) __PYX_ERR(1, 399, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 399, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":398
 *         def __get__(self):
 *             return self._get(49)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'language' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.language.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":400
 *         def __set__(self, value):
 *             self.properties.append({ 'language' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('language')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8language_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":401
 *             self.properties.append({ 'language' : value })
 *         def __del__(self):
 *             self.delattr('language')             # <<<<<<<<<<<<<<
 * 
 *     property time_policy:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__50, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":400
 *         def __set__(self, value):
 *             self.properties.append({ 'language' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('language')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.language.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":404
 * 
 *     property time_policy:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(50)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":405
 *     property time_policy:
 *         def __get__(self):
 *             return self._get(50)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'time_policy' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 50, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 405, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":404
 * 
 *     property time_policy:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(50)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.time_policy.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":406
 *         def __get__(self):
 *             return self._get(50)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'time_policy' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":407
 *             return self._get(50)
 *         def __set__(self, value):
 *             self.properties.append({ 'time_policy' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('time_policy')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 407, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 407, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_time_policy, __pyx_v_value) < 0) __PYX_ERR(1, 407, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 407, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":406
 *         def __get__(self):
 *             return self._get(50)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'time_policy' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.time_policy.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":408
 *         def __set__(self, value):
 *             self.properties.append({ 'time_policy' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('time_policy')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11time_policy_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":409
 *             self.properties.append({ 'time_policy' : value })
 *         def __del__(self):
 *             self.delattr('time_policy')             # <<<<<<<<<<<<<<
 * 
 *     property xmaximum:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__51, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":408
 *         def __set__(self, value):
 *             self.properties.append({ 'time_policy' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('time_policy')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.time_policy.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":412
 * 
 *     property xmaximum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(51)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":413
 *     property xmaximum:
 *         def __get__(self):
 *             return self._get(51)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xmaximum' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 51, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 413, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":412
 * 
 *     property xmaximum:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(51)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xmaximum.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":414
 *         def __get__(self):
 *             return self._get(51)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xmaximum' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":415
 *             return self._get(51)
 *         def __set__(self, value):
 *             self.properties.append({ 'xmaximum' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xmaximum')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 415, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xmaximum, __pyx_v_value) < 0) __PYX_ERR(1, 415, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 415, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":414
 *         def __get__(self):
 *             return self._get(51)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xmaximum' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xmaximum.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":416
 *         def __set__(self, value):
 *             self.properties.append({ 'xmaximum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xmaximum')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xmaximum_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":417
 *             self.properties.append({ 'xmaximum' : value })
 *         def __del__(self):
 *             self.delattr('xmaximum')             # <<<<<<<<<<<<<<
 * 
 *     property hover_sound:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__52, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":416
 *         def __set__(self, value):
 *             self.properties.append({ 'xmaximum' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xmaximum')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xmaximum.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":420
 * 
 *     property hover_sound:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(52)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":421
 *     property hover_sound:
 *         def __get__(self):
 *             return self._get(52)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'hover_sound' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 52, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 421, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":420
 * 
 *     property hover_sound:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(52)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hover_sound.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":422
 *         def __get__(self):
 *             return self._get(52)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'hover_sound' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":423
 *             return self._get(52)
 *         def __set__(self, value):
 *             self.properties.append({ 'hover_sound' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('hover_sound')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 423, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 423, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_hover_sound, __pyx_v_value) < 0) __PYX_ERR(1, 423, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 423, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":422
 *         def __get__(self):
 *             return self._get(52)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'hover_sound' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hover_sound.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":424
 *         def __set__(self, value):
 *             self.properties.append({ 'hover_sound' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('hover_sound')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11hover_sound_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":425
 *             self.properties.append({ 'hover_sound' : value })
 *         def __del__(self):
 *             self.delattr('hover_sound')             # <<<<<<<<<<<<<<
 * 
 *     property yanchor:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__53, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":424
 *         def __set__(self, value):
 *             self.properties.append({ 'hover_sound' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('hover_sound')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hover_sound.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":428
 * 
 *     property yanchor:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(53)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":429
 *     property yanchor:
 *         def __get__(self):
 *             return self._get(53)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'yanchor' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 53, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 429, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":428
 * 
 *     property yanchor:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(53)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yanchor.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":430
 *         def __get__(self):
 *             return self._get(53)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yanchor' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":431
 *             return self._get(53)
 *         def __set__(self, value):
 *             self.properties.append({ 'yanchor' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('yanchor')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 431, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 431, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_yanchor, __pyx_v_value) < 0) __PYX_ERR(1, 431, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 431, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":430
 *         def __get__(self):
 *             return self._get(53)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yanchor' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yanchor.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":432
 *         def __set__(self, value):
 *             self.properties.append({ 'yanchor' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yanchor')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yanchor_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":433
 *             self.properties.append({ 'yanchor' : value })
 *         def __del__(self):
 *             self.delattr('yanchor')             # <<<<<<<<<<<<<<
 * 
 *     property fore_gutter:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__54, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":432
 *         def __set__(self, value):
 *             self.properties.append({ 'yanchor' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yanchor')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yanchor.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":436
 * 
 *     property fore_gutter:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(54)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":437
 *     property fore_gutter:
 *         def __get__(self):
 *             return self._get(54)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_gutter' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 54, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":436
 * 
 *     property fore_gutter:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(54)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fore_gutter.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":438
 *         def __get__(self):
 *             return self._get(54)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'fore_gutter' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":439
 *             return self._get(54)
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_gutter' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('fore_gutter')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 439, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_fore_gutter, __pyx_v_value) < 0) __PYX_ERR(1, 439, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 439, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":438
 *         def __get__(self):
 *             return self._get(54)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'fore_gutter' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fore_gutter.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":440
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_gutter' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('fore_gutter')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_11fore_gutter_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":441
 *             self.properties.append({ 'fore_gutter' : value })
 *         def __del__(self):
 *             self.delattr('fore_gutter')             # <<<<<<<<<<<<<<
 * 
 *     property adjust_spacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__55, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":440
 *         def __set__(self, value):
 *             self.properties.append({ 'fore_gutter' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('fore_gutter')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.fore_gutter.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":444
 * 
 *     property adjust_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(55)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":445
 *     property adjust_spacing:
 *         def __get__(self):
 *             return self._get(55)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'adjust_spacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 55, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":444
 * 
 *     property adjust_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(55)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.adjust_spacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":446
 *         def __get__(self):
 *             return self._get(55)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'adjust_spacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":447
 *             return self._get(55)
 *         def __set__(self, value):
 *             self.properties.append({ 'adjust_spacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('adjust_spacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 447, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_adjust_spacing, __pyx_v_value) < 0) __PYX_ERR(1, 447, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":446
 *         def __get__(self):
 *             return self._get(55)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'adjust_spacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.adjust_spacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":448
 *         def __set__(self, value):
 *             self.properties.append({ 'adjust_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('adjust_spacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":449
 *             self.properties.append({ 'adjust_spacing' : value })
 *         def __del__(self):
 *             self.delattr('adjust_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property xfill:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__56, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":448
 *         def __set__(self, value):
 *             self.properties.append({ 'adjust_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('adjust_spacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.adjust_spacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":452
 * 
 *     property xfill:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(56)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":453
 *     property xfill:
 *         def __get__(self):
 *             return self._get(56)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xfill' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 56, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":452
 * 
 *     property xfill:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(56)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xfill.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":454
 *         def __get__(self):
 *             return self._get(56)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xfill' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":455
 *             return self._get(56)
 *         def __set__(self, value):
 *             self.properties.append({ 'xfill' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xfill')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 455, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xfill, __pyx_v_value) < 0) __PYX_ERR(1, 455, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 455, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":454
 *         def __get__(self):
 *             return self._get(56)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xfill' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xfill.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":456
 *         def __set__(self, value):
 *             self.properties.append({ 'xfill' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xfill')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5xfill_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":457
 *             self.properties.append({ 'xfill' : value })
 *         def __del__(self):
 *             self.delattr('xfill')             # <<<<<<<<<<<<<<
 * 
 *     property color:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__57, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":456
 *         def __set__(self, value):
 *             self.properties.append({ 'xfill' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xfill')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xfill.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":460
 * 
 *     property color:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(57)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5color___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":461
 *     property color:
 *         def __get__(self):
 *             return self._get(57)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'color' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 57, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 461, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":460
 * 
 *     property color:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(57)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.color.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":462
 *         def __get__(self):
 *             return self._get(57)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'color' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":463
 *             return self._get(57)
 *         def __set__(self, value):
 *             self.properties.append({ 'color' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('color')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 463, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_color, __pyx_v_value) < 0) __PYX_ERR(1, 463, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 463, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":462
 *         def __get__(self):
 *             return self._get(57)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'color' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.color.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":464
 *         def __set__(self, value):
 *             self.properties.append({ 'color' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('color')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5color_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":465
 *             self.properties.append({ 'color' : value })
 *         def __del__(self):
 *             self.delattr('color')             # <<<<<<<<<<<<<<
 * 
 *     property key_events:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__58, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":464
 *         def __set__(self, value):
 *             self.properties.append({ 'color' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('color')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.color.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":468
 * 
 *     property key_events:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(58)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":469
 *     property key_events:
 *         def __get__(self):
 *             return self._get(58)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'key_events' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 58, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":468
 * 
 *     property key_events:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(58)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.key_events.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":470
 *         def __get__(self):
 *             return self._get(58)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'key_events' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":471
 *             return self._get(58)
 *         def __set__(self, value):
 *             self.properties.append({ 'key_events' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('key_events')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 471, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_key_events, __pyx_v_value) < 0) __PYX_ERR(1, 471, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 471, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":470
 *         def __get__(self):
 *             return self._get(58)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'key_events' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.key_events.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":472
 *         def __set__(self, value):
 *             self.properties.append({ 'key_events' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('key_events')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10key_events_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":473
 *             self.properties.append({ 'key_events' : value })
 *         def __del__(self):
 *             self.delattr('key_events')             # <<<<<<<<<<<<<<
 * 
 *     property xspacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__59, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":472
 *         def __set__(self, value):
 *             self.properties.append({ 'key_events' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('key_events')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.key_events.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":476
 * 
 *     property xspacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(59)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":477
 *     property xspacing:
 *         def __get__(self):
 *             return self._get(59)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xspacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 59, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":476
 * 
 *     property xspacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(59)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xspacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":478
 *         def __get__(self):
 *             return self._get(59)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xspacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":479
 *             return self._get(59)
 *         def __set__(self, value):
 *             self.properties.append({ 'xspacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xspacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 479, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xspacing, __pyx_v_value) < 0) __PYX_ERR(1, 479, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 479, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":478
 *         def __get__(self):
 *             return self._get(59)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xspacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xspacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":480
 *         def __set__(self, value):
 *             self.properties.append({ 'xspacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xspacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8xspacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":481
 *             self.properties.append({ 'xspacing' : value })
 *         def __del__(self):
 *             self.delattr('xspacing')             # <<<<<<<<<<<<<<
 * 
 *     property size:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__60, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":480
 *         def __set__(self, value):
 *             self.properties.append({ 'xspacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xspacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xspacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":484
 * 
 *     property size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(60)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4size___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":485
 *     property size:
 *         def __get__(self):
 *             return self._get(60)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'size' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 60, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":484
 * 
 *     property size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(60)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":486
 *         def __get__(self):
 *             return self._get(60)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'size' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":487
 *             return self._get(60)
 *         def __set__(self, value):
 *             self.properties.append({ 'size' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('size')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 487, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_size, __pyx_v_value) < 0) __PYX_ERR(1, 487, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 487, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":486
 *         def __get__(self):
 *             return self._get(60)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'size' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":488
 *         def __set__(self, value):
 *             self.properties.append({ 'size' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('size')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4size_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":489
 *             self.properties.append({ 'size' : value })
 *         def __del__(self):
 *             self.delattr('size')             # <<<<<<<<<<<<<<
 * 
 *     property caret:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__61, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":488
 *         def __set__(self, value):
 *             self.properties.append({ 'size' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('size')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.size.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":492
 * 
 *     property caret:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(61)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":493
 *     property caret:
 *         def __get__(self):
 *             return self._get(61)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'caret' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 61, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":492
 * 
 *     property caret:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(61)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.caret.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":494
 *         def __get__(self):
 *             return self._get(61)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'caret' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":495
 *             return self._get(61)
 *         def __set__(self, value):
 *             self.properties.append({ 'caret' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('caret')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 495, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_caret, __pyx_v_value) < 0) __PYX_ERR(1, 495, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 495, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":494
 *         def __get__(self):
 *             return self._get(61)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'caret' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.caret.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":496
 *         def __set__(self, value):
 *             self.properties.append({ 'caret' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('caret')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5caret_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":497
 *             self.properties.append({ 'caret' : value })
 *         def __del__(self):
 *             self.delattr('caret')             # <<<<<<<<<<<<<<
 * 
 *     property bottom_padding:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__62, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":496
 *         def __set__(self, value):
 *             self.properties.append({ 'caret' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('caret')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.caret.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":500
 * 
 *     property bottom_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(62)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":501
 *     property bottom_padding:
 *         def __get__(self):
 *             return self._get(62)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_padding' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 62, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":500
 * 
 *     property bottom_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(62)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bottom_padding.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":502
 *         def __get__(self):
 *             return self._get(62)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bottom_padding' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":503
 *             return self._get(62)
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_padding' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('bottom_padding')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 503, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bottom_padding, __pyx_v_value) < 0) __PYX_ERR(1, 503, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 503, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":502
 *         def __get__(self):
 *             return self._get(62)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bottom_padding' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bottom_padding.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":504
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bottom_padding')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14bottom_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":505
 *             self.properties.append({ 'bottom_padding' : value })
 *         def __del__(self):
 *             self.delattr('bottom_padding')             # <<<<<<<<<<<<<<
 * 
 *     property box_wrap:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__63, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":504
 *         def __set__(self, value):
 *             self.properties.append({ 'bottom_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bottom_padding')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bottom_padding.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":508
 * 
 *     property box_wrap:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(63)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":509
 *     property box_wrap:
 *         def __get__(self):
 *             return self._get(63)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 63, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 509, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":508
 * 
 *     property box_wrap:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(63)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_wrap.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":510
 *         def __get__(self):
 *             return self._get(63)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_wrap' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":511
 *             return self._get(63)
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('box_wrap')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 511, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_box_wrap, __pyx_v_value) < 0) __PYX_ERR(1, 511, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 511, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":510
 *         def __get__(self):
 *             return self._get(63)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_wrap' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_wrap.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":512
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_wrap')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8box_wrap_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":513
 *             self.properties.append({ 'box_wrap' : value })
 *         def __del__(self):
 *             self.delattr('box_wrap')             # <<<<<<<<<<<<<<
 * 
 *     property line_overlap_split:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__64, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":512
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_wrap')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_wrap.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":516
 * 
 *     property line_overlap_split:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(64)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":517
 *     property line_overlap_split:
 *         def __get__(self):
 *             return self._get(64)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'line_overlap_split' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 64, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":516
 * 
 *     property line_overlap_split:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(64)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_overlap_split.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":518
 *         def __get__(self):
 *             return self._get(64)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'line_overlap_split' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":519
 *             return self._get(64)
 *         def __set__(self, value):
 *             self.properties.append({ 'line_overlap_split' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('line_overlap_split')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 519, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_line_overlap_split, __pyx_v_value) < 0) __PYX_ERR(1, 519, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 519, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":518
 *         def __get__(self):
 *             return self._get(64)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'line_overlap_split' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_overlap_split.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":520
 *         def __set__(self, value):
 *             self.properties.append({ 'line_overlap_split' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('line_overlap_split')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":521
 *             self.properties.append({ 'line_overlap_split' : value })
 *         def __del__(self):
 *             self.delattr('line_overlap_split')             # <<<<<<<<<<<<<<
 * 
 *     property box_wrap_spacing:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__65, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":520
 *         def __set__(self, value):
 *             self.properties.append({ 'line_overlap_split' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('line_overlap_split')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_overlap_split.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":524
 * 
 *     property box_wrap_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(65)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":525
 *     property box_wrap_spacing:
 *         def __get__(self):
 *             return self._get(65)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap_spacing' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 65, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 525, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":524
 * 
 *     property box_wrap_spacing:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(65)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_wrap_spacing.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":526
 *         def __get__(self):
 *             return self._get(65)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_wrap_spacing' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":527
 *             return self._get(65)
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap_spacing' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('box_wrap_spacing')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 527, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_box_wrap_spacing, __pyx_v_value) < 0) __PYX_ERR(1, 527, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 527, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":526
 *         def __get__(self):
 *             return self._get(65)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_wrap_spacing' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_wrap_spacing.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":528
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_wrap_spacing')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":529
 *             self.properties.append({ 'box_wrap_spacing' : value })
 *         def __del__(self):
 *             self.delattr('box_wrap_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property kerning:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__66, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":528
 *         def __set__(self, value):
 *             self.properties.append({ 'box_wrap_spacing' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_wrap_spacing')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_wrap_spacing.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":532
 * 
 *     property kerning:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(66)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":533
 *     property kerning:
 *         def __get__(self):
 *             return self._get(66)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'kerning' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 66, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":532
 * 
 *     property kerning:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(66)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.kerning.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":534
 *         def __get__(self):
 *             return self._get(66)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'kerning' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":535
 *             return self._get(66)
 *         def __set__(self, value):
 *             self.properties.append({ 'kerning' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('kerning')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 535, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_kerning, __pyx_v_value) < 0) __PYX_ERR(1, 535, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":534
 *         def __get__(self):
 *             return self._get(66)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'kerning' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.kerning.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":536
 *         def __set__(self, value):
 *             self.properties.append({ 'kerning' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('kerning')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7kerning_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":537
 *             self.properties.append({ 'kerning' : value })
 *         def __del__(self):
 *             self.delattr('kerning')             # <<<<<<<<<<<<<<
 * 
 *     property italic:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__67, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":536
 *         def __set__(self, value):
 *             self.properties.append({ 'kerning' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('kerning')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.kerning.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":540
 * 
 *     property italic:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(67)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":541
 *     property italic:
 *         def __get__(self):
 *             return self._get(67)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'italic' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 67, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 541, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":540
 * 
 *     property italic:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(67)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.italic.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":542
 *         def __get__(self):
 *             return self._get(67)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'italic' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":543
 *             return self._get(67)
 *         def __set__(self, value):
 *             self.properties.append({ 'italic' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('italic')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 543, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_italic, __pyx_v_value) < 0) __PYX_ERR(1, 543, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 543, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":542
 *         def __get__(self):
 *             return self._get(67)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'italic' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.italic.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":544
 *         def __set__(self, value):
 *             self.properties.append({ 'italic' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('italic')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_6italic_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":545
 *             self.properties.append({ 'italic' : value })
 *         def __del__(self):
 *             self.delattr('italic')             # <<<<<<<<<<<<<<
 * 
 *     property ruby_style:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__68, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":544
 *         def __set__(self, value):
 *             self.properties.append({ 'italic' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('italic')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.italic.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":548
 * 
 *     property ruby_style:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(68)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":549
 *     property ruby_style:
 *         def __get__(self):
 *             return self._get(68)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'ruby_style' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 68, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":548
 * 
 *     property ruby_style:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(68)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ruby_style.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":550
 *         def __get__(self):
 *             return self._get(68)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'ruby_style' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":551
 *             return self._get(68)
 *         def __set__(self, value):
 *             self.properties.append({ 'ruby_style' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('ruby_style')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 551, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_ruby_style, __pyx_v_value) < 0) __PYX_ERR(1, 551, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 551, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":550
 *         def __get__(self):
 *             return self._get(68)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'ruby_style' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ruby_style.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":552
 *         def __set__(self, value):
 *             self.properties.append({ 'ruby_style' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('ruby_style')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10ruby_style_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":553
 *             self.properties.append({ 'ruby_style' : value })
 *         def __del__(self):
 *             self.delattr('ruby_style')             # <<<<<<<<<<<<<<
 * 
 *     property box_layout:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__69, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":552
 *         def __set__(self, value):
 *             self.properties.append({ 'ruby_style' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('ruby_style')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.ruby_style.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":556
 * 
 *     property box_layout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(69)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":557
 *     property box_layout:
 *         def __get__(self):
 *             return self._get(69)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'box_layout' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 69, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":556
 * 
 *     property box_layout:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(69)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_layout.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":558
 *         def __get__(self):
 *             return self._get(69)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_layout' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":559
 *             return self._get(69)
 *         def __set__(self, value):
 *             self.properties.append({ 'box_layout' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('box_layout')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 559, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_box_layout, __pyx_v_value) < 0) __PYX_ERR(1, 559, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 559, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":558
 *         def __get__(self):
 *             return self._get(69)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'box_layout' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_layout.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":560
 *         def __set__(self, value):
 *             self.properties.append({ 'box_layout' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_layout')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10box_layout_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":561
 *             self.properties.append({ 'box_layout' : value })
 *         def __del__(self):
 *             self.delattr('box_layout')             # <<<<<<<<<<<<<<
 * 
 *     property xpos:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__70, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":560
 *         def __set__(self, value):
 *             self.properties.append({ 'box_layout' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('box_layout')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.box_layout.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":564
 * 
 *     property xpos:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(70)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":565
 *     property xpos:
 *         def __get__(self):
 *             return self._get(70)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xpos' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 70, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":564
 * 
 *     property xpos:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(70)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xpos.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":566
 *         def __get__(self):
 *             return self._get(70)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xpos' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":567
 *             return self._get(70)
 *         def __set__(self, value):
 *             self.properties.append({ 'xpos' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xpos')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 567, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xpos, __pyx_v_value) < 0) __PYX_ERR(1, 567, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 567, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":566
 *         def __get__(self):
 *             return self._get(70)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xpos' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xpos.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":568
 *         def __set__(self, value):
 *             self.properties.append({ 'xpos' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xpos')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xpos_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":569
 *             self.properties.append({ 'xpos' : value })
 *         def __del__(self):
 *             self.delattr('xpos')             # <<<<<<<<<<<<<<
 * 
 *     property xanchor:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__71, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":568
 *         def __set__(self, value):
 *             self.properties.append({ 'xpos' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xpos')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xpos.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":572
 * 
 *     property xanchor:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(71)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":573
 *     property xanchor:
 *         def __get__(self):
 *             return self._get(71)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xanchor' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 71, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 573, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":572
 * 
 *     property xanchor:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(71)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xanchor.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":574
 *         def __get__(self):
 *             return self._get(71)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xanchor' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":575
 *             return self._get(71)
 *         def __set__(self, value):
 *             self.properties.append({ 'xanchor' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xanchor')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 575, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xanchor, __pyx_v_value) < 0) __PYX_ERR(1, 575, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 575, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":574
 *         def __get__(self):
 *             return self._get(71)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xanchor' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xanchor.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":576
 *         def __set__(self, value):
 *             self.properties.append({ 'xanchor' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xanchor')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7xanchor_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":577
 *             self.properties.append({ 'xanchor' : value })
 *         def __del__(self):
 *             self.delattr('xanchor')             # <<<<<<<<<<<<<<
 * 
 *     property top_margin:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__72, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":576
 *         def __set__(self, value):
 *             self.properties.append({ 'xanchor' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xanchor')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xanchor.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":580
 * 
 *     property top_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(72)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":581
 *     property top_margin:
 *         def __get__(self):
 *             return self._get(72)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'top_margin' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 72, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 581, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":580
 * 
 *     property top_margin:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(72)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.top_margin.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":582
 *         def __get__(self):
 *             return self._get(72)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'top_margin' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":583
 *             return self._get(72)
 *         def __set__(self, value):
 *             self.properties.append({ 'top_margin' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('top_margin')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 583, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_top_margin, __pyx_v_value) < 0) __PYX_ERR(1, 583, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 583, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":582
 *         def __get__(self):
 *             return self._get(72)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'top_margin' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.top_margin.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":584
 *         def __set__(self, value):
 *             self.properties.append({ 'top_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('top_margin')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10top_margin_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":585
 *             self.properties.append({ 'top_margin' : value })
 *         def __del__(self):
 *             self.delattr('top_margin')             # <<<<<<<<<<<<<<
 * 
 *     property yoffset:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__73, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":584
 *         def __set__(self, value):
 *             self.properties.append({ 'top_margin' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('top_margin')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.top_margin.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":588
 * 
 *     property yoffset:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(73)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":589
 *     property yoffset:
 *         def __get__(self):
 *             return self._get(73)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'yoffset' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 73, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":588
 * 
 *     property yoffset:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(73)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yoffset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":590
 *         def __get__(self):
 *             return self._get(73)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yoffset' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":591
 *             return self._get(73)
 *         def __set__(self, value):
 *             self.properties.append({ 'yoffset' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('yoffset')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 591, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 591, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_yoffset, __pyx_v_value) < 0) __PYX_ERR(1, 591, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 591, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":590
 *         def __get__(self):
 *             return self._get(73)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yoffset' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yoffset.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":592
 *         def __set__(self, value):
 *             self.properties.append({ 'yoffset' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yoffset')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7yoffset_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":593
 *             self.properties.append({ 'yoffset' : value })
 *         def __del__(self):
 *             self.delattr('yoffset')             # <<<<<<<<<<<<<<
 * 
 *     property child:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__74, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":592
 *         def __set__(self, value):
 *             self.properties.append({ 'yoffset' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yoffset')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yoffset.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":596
 * 
 *     property child:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(74)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5child___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":597
 *     property child:
 *         def __get__(self):
 *             return self._get(74)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'child' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 74, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":596
 * 
 *     property child:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(74)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.child.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":598
 *         def __get__(self):
 *             return self._get(74)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'child' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":599
 *             return self._get(74)
 *         def __set__(self, value):
 *             self.properties.append({ 'child' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('child')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 599, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_child, __pyx_v_value) < 0) __PYX_ERR(1, 599, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":598
 *         def __get__(self):
 *             return self._get(74)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'child' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.child.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":600
 *         def __set__(self, value):
 *             self.properties.append({ 'child' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('child')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5child_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":601
 *             self.properties.append({ 'child' : value })
 *         def __del__(self):
 *             self.delattr('child')             # <<<<<<<<<<<<<<
 * 
 *     property thumb_offset:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__75, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":600
 *         def __set__(self, value):
 *             self.properties.append({ 'child' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('child')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.child.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":604
 * 
 *     property thumb_offset:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(75)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":605
 *     property thumb_offset:
 *         def __get__(self):
 *             return self._get(75)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_offset' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 75, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":604
 * 
 *     property thumb_offset:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(75)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb_offset.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":606
 *         def __get__(self):
 *             return self._get(75)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'thumb_offset' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":607
 *             return self._get(75)
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_offset' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('thumb_offset')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 607, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_thumb_offset, __pyx_v_value) < 0) __PYX_ERR(1, 607, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":606
 *         def __get__(self):
 *             return self._get(75)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'thumb_offset' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb_offset.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":608
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_offset' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('thumb_offset')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_offset_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":609
 *             self.properties.append({ 'thumb_offset' : value })
 *         def __del__(self):
 *             self.delattr('thumb_offset')             # <<<<<<<<<<<<<<
 * 
 *     property xfit:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__76, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":608
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_offset' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('thumb_offset')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb_offset.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":612
 * 
 *     property xfit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(76)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":613
 *     property xfit:
 *         def __get__(self):
 *             return self._get(76)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'xfit' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 76, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":612
 * 
 *     property xfit:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(76)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xfit.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":614
 *         def __get__(self):
 *             return self._get(76)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xfit' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":615
 *             return self._get(76)
 *         def __set__(self, value):
 *             self.properties.append({ 'xfit' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('xfit')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 615, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 615, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_xfit, __pyx_v_value) < 0) __PYX_ERR(1, 615, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 615, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":614
 *         def __get__(self):
 *             return self._get(76)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'xfit' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xfit.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":616
 *         def __set__(self, value):
 *             self.properties.append({ 'xfit' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xfit')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_4xfit_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":617
 *             self.properties.append({ 'xfit' : value })
 *         def __del__(self):
 *             self.delattr('xfit')             # <<<<<<<<<<<<<<
 * 
 *     property hinting:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__77, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":616
 *         def __set__(self, value):
 *             self.properties.append({ 'xfit' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('xfit')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.xfit.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":620
 * 
 *     property hinting:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(77)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":621
 *     property hinting:
 *         def __get__(self):
 *             return self._get(77)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'hinting' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 77, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":620
 * 
 *     property hinting:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(77)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hinting.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":622
 *         def __get__(self):
 *             return self._get(77)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'hinting' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":623
 *             return self._get(77)
 *         def __set__(self, value):
 *             self.properties.append({ 'hinting' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('hinting')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 623, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_hinting, __pyx_v_value) < 0) __PYX_ERR(1, 623, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 623, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":622
 *         def __get__(self):
 *             return self._get(77)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'hinting' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hinting.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":624
 *         def __set__(self, value):
 *             self.properties.append({ 'hinting' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('hinting')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7hinting_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":625
 *             self.properties.append({ 'hinting' : value })
 *         def __del__(self):
 *             self.delattr('hinting')             # <<<<<<<<<<<<<<
 * 
 *     property altruby_style:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__78, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":624
 *         def __set__(self, value):
 *             self.properties.append({ 'hinting' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('hinting')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.hinting.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":628
 * 
 *     property altruby_style:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(78)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":629
 *     property altruby_style:
 *         def __get__(self):
 *             return self._get(78)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'altruby_style' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 78, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":628
 * 
 *     property altruby_style:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(78)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.altruby_style.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":630
 *         def __get__(self):
 *             return self._get(78)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'altruby_style' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":631
 *             return self._get(78)
 *         def __set__(self, value):
 *             self.properties.append({ 'altruby_style' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('altruby_style')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 631, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_altruby_style, __pyx_v_value) < 0) __PYX_ERR(1, 631, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 631, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":630
 *         def __get__(self):
 *             return self._get(78)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'altruby_style' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.altruby_style.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":632
 *         def __set__(self, value):
 *             self.properties.append({ 'altruby_style' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('altruby_style')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13altruby_style_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":633
 *             self.properties.append({ 'altruby_style' : value })
 *         def __del__(self):
 *             self.delattr('altruby_style')             # <<<<<<<<<<<<<<
 * 
 *     property unscrollable:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__79, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":632
 *         def __set__(self, value):
 *             self.properties.append({ 'altruby_style' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('altruby_style')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.altruby_style.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":636
 * 
 *     property unscrollable:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(79)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":637
 *     property unscrollable:
 *         def __get__(self):
 *             return self._get(79)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'unscrollable' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 79, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":636
 * 
 *     property unscrollable:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(79)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.unscrollable.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":638
 *         def __get__(self):
 *             return self._get(79)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'unscrollable' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":639
 *             return self._get(79)
 *         def __set__(self, value):
 *             self.properties.append({ 'unscrollable' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('unscrollable')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 639, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_unscrollable, __pyx_v_value) < 0) __PYX_ERR(1, 639, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 639, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":638
 *         def __get__(self):
 *             return self._get(79)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'unscrollable' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.unscrollable.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":640
 *         def __set__(self, value):
 *             self.properties.append({ 'unscrollable' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('unscrollable')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12unscrollable_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":641
 *             self.properties.append({ 'unscrollable' : value })
 *         def __del__(self):
 *             self.delattr('unscrollable')             # <<<<<<<<<<<<<<
 * 
 *     property order_reverse:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__80, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":640
 *         def __set__(self, value):
 *             self.properties.append({ 'unscrollable' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('unscrollable')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.unscrollable.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":644
 * 
 *     property order_reverse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(80)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":645
 *     property order_reverse:
 *         def __get__(self):
 *             return self._get(80)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'order_reverse' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 80, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":644
 * 
 *     property order_reverse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(80)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.order_reverse.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":646
 *         def __get__(self):
 *             return self._get(80)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'order_reverse' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":647
 *             return self._get(80)
 *         def __set__(self, value):
 *             self.properties.append({ 'order_reverse' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('order_reverse')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 647, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_order_reverse, __pyx_v_value) < 0) __PYX_ERR(1, 647, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 647, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":646
 *         def __get__(self):
 *             return self._get(80)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'order_reverse' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.order_reverse.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":648
 *         def __set__(self, value):
 *             self.properties.append({ 'order_reverse' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('order_reverse')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13order_reverse_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":649
 *             self.properties.append({ 'order_reverse' : value })
 *         def __del__(self):
 *             self.delattr('order_reverse')             # <<<<<<<<<<<<<<
 * 
 *     property mouse:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__81, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":648
 *         def __set__(self, value):
 *             self.properties.append({ 'order_reverse' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('order_reverse')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.order_reverse.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":652
 * 
 *     property mouse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(81)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":653
 *     property mouse:
 *         def __get__(self):
 *             return self._get(81)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'mouse' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 81, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":652
 * 
 *     property mouse:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(81)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.mouse.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":654
 *         def __get__(self):
 *             return self._get(81)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'mouse' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":655
 *             return self._get(81)
 *         def __set__(self, value):
 *             self.properties.append({ 'mouse' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('mouse')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 655, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_mouse, __pyx_v_value) < 0) __PYX_ERR(1, 655, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 655, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":654
 *         def __get__(self):
 *             return self._get(81)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'mouse' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.mouse.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":656
 *         def __set__(self, value):
 *             self.properties.append({ 'mouse' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('mouse')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5mouse_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":657
 *             self.properties.append({ 'mouse' : value })
 *         def __del__(self):
 *             self.delattr('mouse')             # <<<<<<<<<<<<<<
 * 
 *     property keyboard_focus:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 657, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__82, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 657, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":656
 *         def __set__(self, value):
 *             self.properties.append({ 'mouse' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('mouse')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.mouse.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":660
 * 
 *     property keyboard_focus:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(82)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":661
 *     property keyboard_focus:
 *         def __get__(self):
 *             return self._get(82)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'keyboard_focus' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 82, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":660
 * 
 *     property keyboard_focus:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(82)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.keyboard_focus.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":662
 *         def __get__(self):
 *             return self._get(82)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'keyboard_focus' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":663
 *             return self._get(82)
 *         def __set__(self, value):
 *             self.properties.append({ 'keyboard_focus' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('keyboard_focus')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 663, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_keyboard_focus, __pyx_v_value) < 0) __PYX_ERR(1, 663, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 663, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":662
 *         def __get__(self):
 *             return self._get(82)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'keyboard_focus' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.keyboard_focus.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":664
 *         def __set__(self, value):
 *             self.properties.append({ 'keyboard_focus' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('keyboard_focus')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":665
 *             self.properties.append({ 'keyboard_focus' : value })
 *         def __del__(self):
 *             self.delattr('keyboard_focus')             # <<<<<<<<<<<<<<
 * 
 *     property first_indent:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__83, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":664
 *         def __set__(self, value):
 *             self.properties.append({ 'keyboard_focus' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('keyboard_focus')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.keyboard_focus.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":668
 * 
 *     property first_indent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(83)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":669
 *     property first_indent:
 *         def __get__(self):
 *             return self._get(83)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'first_indent' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 83, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":668
 * 
 *     property first_indent:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(83)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.first_indent.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":670
 *         def __get__(self):
 *             return self._get(83)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'first_indent' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":671
 *             return self._get(83)
 *         def __set__(self, value):
 *             self.properties.append({ 'first_indent' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('first_indent')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 671, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_first_indent, __pyx_v_value) < 0) __PYX_ERR(1, 671, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 671, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":670
 *         def __get__(self):
 *             return self._get(83)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'first_indent' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.first_indent.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":672
 *         def __set__(self, value):
 *             self.properties.append({ 'first_indent' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('first_indent')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12first_indent_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":673
 *             self.properties.append({ 'first_indent' : value })
 *         def __del__(self):
 *             self.delattr('first_indent')             # <<<<<<<<<<<<<<
 * 
 *     property bar_vertical:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__84, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":672
 *         def __set__(self, value):
 *             self.properties.append({ 'first_indent' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('first_indent')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.first_indent.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":676
 * 
 *     property bar_vertical:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(84)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":677
 *     property bar_vertical:
 *         def __get__(self):
 *             return self._get(84)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_vertical' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 84, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 677, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":676
 * 
 *     property bar_vertical:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(84)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_vertical.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":678
 *         def __get__(self):
 *             return self._get(84)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bar_vertical' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":679
 *             return self._get(84)
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_vertical' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('bar_vertical')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 679, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bar_vertical, __pyx_v_value) < 0) __PYX_ERR(1, 679, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 679, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":678
 *         def __get__(self):
 *             return self._get(84)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bar_vertical' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_vertical.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":680
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_vertical' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bar_vertical')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12bar_vertical_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":681
 *             self.properties.append({ 'bar_vertical' : value })
 *         def __del__(self):
 *             self.delattr('bar_vertical')             # <<<<<<<<<<<<<<
 * 
 *     property right_padding:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__85, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":680
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_vertical' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bar_vertical')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_vertical.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":684
 * 
 *     property right_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(85)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":685
 *     property right_padding:
 *         def __get__(self):
 *             return self._get(85)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'right_padding' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 85, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 685, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":684
 * 
 *     property right_padding:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(85)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.right_padding.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":686
 *         def __get__(self):
 *             return self._get(85)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'right_padding' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":687
 *             return self._get(85)
 *         def __set__(self, value):
 *             self.properties.append({ 'right_padding' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('right_padding')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 687, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_right_padding, __pyx_v_value) < 0) __PYX_ERR(1, 687, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 687, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":686
 *         def __get__(self):
 *             return self._get(85)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'right_padding' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.right_padding.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":688
 *         def __set__(self, value):
 *             self.properties.append({ 'right_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('right_padding')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_13right_padding_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":689
 *             self.properties.append({ 'right_padding' : value })
 *         def __del__(self):
 *             self.delattr('right_padding')             # <<<<<<<<<<<<<<
 * 
 *     property focus_rect:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__86, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":688
 *         def __set__(self, value):
 *             self.properties.append({ 'right_padding' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('right_padding')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.right_padding.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":692
 * 
 *     property focus_rect:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(86)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":693
 *     property focus_rect:
 *         def __get__(self):
 *             return self._get(86)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_rect' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 86, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":692
 * 
 *     property focus_rect:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(86)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.focus_rect.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":694
 *         def __get__(self):
 *             return self._get(86)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'focus_rect' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":695
 *             return self._get(86)
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_rect' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('focus_rect')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 695, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_focus_rect, __pyx_v_value) < 0) __PYX_ERR(1, 695, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 695, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":694
 *         def __get__(self):
 *             return self._get(86)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'focus_rect' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.focus_rect.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":696
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_rect' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('focus_rect')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10focus_rect_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":697
 *             self.properties.append({ 'focus_rect' : value })
 *         def __del__(self):
 *             self.delattr('focus_rect')             # <<<<<<<<<<<<<<
 * 
 *     property antialias:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__87, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":696
 *         def __set__(self, value):
 *             self.properties.append({ 'focus_rect' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('focus_rect')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.focus_rect.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":700
 * 
 *     property antialias:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(87)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":701
 *     property antialias:
 *         def __get__(self):
 *             return self._get(87)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'antialias' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 87, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":700
 * 
 *     property antialias:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(87)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.antialias.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":702
 *         def __get__(self):
 *             return self._get(87)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'antialias' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":703
 *             return self._get(87)
 *         def __set__(self, value):
 *             self.properties.append({ 'antialias' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('antialias')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 703, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_antialias, __pyx_v_value) < 0) __PYX_ERR(1, 703, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 703, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":702
 *         def __get__(self):
 *             return self._get(87)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'antialias' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.antialias.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":704
 *         def __set__(self, value):
 *             self.properties.append({ 'antialias' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('antialias')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_9antialias_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":705
 *             self.properties.append({ 'antialias' : value })
 *         def __del__(self):
 *             self.delattr('antialias')             # <<<<<<<<<<<<<<
 * 
 *     property slow_cps:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__88, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":704
 *         def __set__(self, value):
 *             self.properties.append({ 'antialias' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('antialias')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.antialias.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":708
 * 
 *     property slow_cps:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(88)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":709
 *     property slow_cps:
 *         def __get__(self):
 *             return self._get(88)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 88, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":708
 * 
 *     property slow_cps:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(88)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_cps.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":710
 *         def __get__(self):
 *             return self._get(88)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'slow_cps' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":711
 *             return self._get(88)
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('slow_cps')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 711, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_slow_cps, __pyx_v_value) < 0) __PYX_ERR(1, 711, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 711, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":710
 *         def __get__(self):
 *             return self._get(88)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'slow_cps' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_cps.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":712
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('slow_cps')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_8slow_cps_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":713
 *             self.properties.append({ 'slow_cps' : value })
 *         def __del__(self):
 *             self.delattr('slow_cps')             # <<<<<<<<<<<<<<
 * 
 *     property bar_invert:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__89, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":712
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('slow_cps')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_cps.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":716
 * 
 *     property bar_invert:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(89)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":717
 *     property bar_invert:
 *         def __get__(self):
 *             return self._get(89)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_invert' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 89, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":716
 * 
 *     property bar_invert:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(89)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_invert.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":718
 *         def __get__(self):
 *             return self._get(89)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bar_invert' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":719
 *             return self._get(89)
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_invert' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('bar_invert')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 719, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_bar_invert, __pyx_v_value) < 0) __PYX_ERR(1, 719, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 719, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":718
 *         def __get__(self):
 *             return self._get(89)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'bar_invert' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_invert.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":720
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_invert' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bar_invert')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_10bar_invert_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":721
 *             self.properties.append({ 'bar_invert' : value })
 *         def __del__(self):
 *             self.delattr('bar_invert')             # <<<<<<<<<<<<<<
 * 
 *     property line_leading:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__90, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":720
 *         def __set__(self, value):
 *             self.properties.append({ 'bar_invert' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('bar_invert')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.bar_invert.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":724
 * 
 *     property line_leading:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(90)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":725
 *     property line_leading:
 *         def __get__(self):
 *             return self._get(90)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'line_leading' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 90, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":724
 * 
 *     property line_leading:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(90)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_leading.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":726
 *         def __get__(self):
 *             return self._get(90)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'line_leading' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":727
 *             return self._get(90)
 *         def __set__(self, value):
 *             self.properties.append({ 'line_leading' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('line_leading')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 727, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 727, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_line_leading, __pyx_v_value) < 0) __PYX_ERR(1, 727, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 727, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":726
 *         def __get__(self):
 *             return self._get(90)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'line_leading' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_leading.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":728
 *         def __set__(self, value):
 *             self.properties.append({ 'line_leading' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('line_leading')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12line_leading_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":729
 *             self.properties.append({ 'line_leading' : value })
 *         def __del__(self):
 *             self.delattr('line_leading')             # <<<<<<<<<<<<<<
 * 
 *     property slow_cps_multiplier:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__91, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":728
 *         def __set__(self, value):
 *             self.properties.append({ 'line_leading' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('line_leading')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.line_leading.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":732
 * 
 *     property slow_cps_multiplier:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(91)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":733
 *     property slow_cps_multiplier:
 *         def __get__(self):
 *             return self._get(91)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 91, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":732
 * 
 *     property slow_cps_multiplier:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(91)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_cps_multiplier.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":734
 *         def __get__(self):
 *             return self._get(91)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":735
 *             return self._get(91)
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps_multiplier' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('slow_cps_multiplier')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 735, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_slow_cps_multiplier, __pyx_v_value) < 0) __PYX_ERR(1, 735, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 735, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":734
 *         def __get__(self):
 *             return self._get(91)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_cps_multiplier.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":736
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('slow_cps_multiplier')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":737
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 *         def __del__(self):
 *             self.delattr('slow_cps_multiplier')             # <<<<<<<<<<<<<<
 * 
 *     property thumb_shadow:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__92, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":736
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('slow_cps_multiplier')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_cps_multiplier.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":740
 * 
 *     property thumb_shadow:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(92)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":741
 *     property thumb_shadow:
 *         def __get__(self):
 *             return self._get(92)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_shadow' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 92, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":740
 * 
 *     property thumb_shadow:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(92)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb_shadow.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":742
 *         def __get__(self):
 *             return self._get(92)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'thumb_shadow' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":743
 *             return self._get(92)
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_shadow' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('thumb_shadow')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 743, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_thumb_shadow, __pyx_v_value) < 0) __PYX_ERR(1, 743, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 743, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":742
 *         def __get__(self):
 *             return self._get(92)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'thumb_shadow' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb_shadow.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":744
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_shadow' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('thumb_shadow')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":745
 *             self.properties.append({ 'thumb_shadow' : value })
 *         def __del__(self):
 *             self.delattr('thumb_shadow')             # <<<<<<<<<<<<<<
 * 
 *     property yfill:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__93, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":744
 *         def __set__(self, value):
 *             self.properties.append({ 'thumb_shadow' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('thumb_shadow')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.thumb_shadow.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":748
 * 
 *     property yfill:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(93)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":749
 *     property yfill:
 *         def __get__(self):
 *             return self._get(93)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'yfill' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 93, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 749, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":748
 * 
 *     property yfill:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(93)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yfill.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":750
 *         def __get__(self):
 *             return self._get(93)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yfill' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":751
 *             return self._get(93)
 *         def __set__(self, value):
 *             self.properties.append({ 'yfill' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('yfill')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 751, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 751, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_yfill, __pyx_v_value) < 0) __PYX_ERR(1, 751, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 751, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":750
 *         def __get__(self):
 *             return self._get(93)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'yfill' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yfill.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":752
 *         def __set__(self, value):
 *             self.properties.append({ 'yfill' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yfill')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_5yfill_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":753
 *             self.properties.append({ 'yfill' : value })
 *         def __del__(self):
 *             self.delattr('yfill')             # <<<<<<<<<<<<<<
 * 
 *     property slow_abortable:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__94, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":752
 *         def __set__(self, value):
 *             self.properties.append({ 'yfill' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('yfill')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.yfill.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":756
 * 
 *     property slow_abortable:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(94)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":757
 *     property slow_abortable:
 *         def __get__(self):
 *             return self._get(94)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_abortable' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 94, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 757, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":756
 * 
 *     property slow_abortable:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(94)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_abortable.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":758
 *         def __get__(self):
 *             return self._get(94)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'slow_abortable' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":759
 *             return self._get(94)
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_abortable' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('slow_abortable')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 759, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 759, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_slow_abortable, __pyx_v_value) < 0) __PYX_ERR(1, 759, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 759, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":758
 *         def __get__(self):
 *             return self._get(94)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'slow_abortable' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_abortable.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":760
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_abortable' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('slow_abortable')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_14slow_abortable_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":761
 *             self.properties.append({ 'slow_abortable' : value })
 *         def __del__(self):
 *             self.delattr('slow_abortable')             # <<<<<<<<<<<<<<
 * 
 *     property justify:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__95, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":760
 *         def __set__(self, value):
 *             self.properties.append({ 'slow_abortable' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('slow_abortable')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.slow_abortable.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":764
 * 
 *     property justify:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(95)
 *         def __set__(self, value):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify___get__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify___get__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gen/styleclass.pxi":765
 *     property justify:
 *         def __get__(self):
 *             return self._get(95)             # <<<<<<<<<<<<<<
 *         def __set__(self, value):
 *             self.properties.append({ 'justify' : value })
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._get(((struct __pyx_obj_5renpy_5style_StyleCore *)__pyx_v_self), 95, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 765, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gen/styleclass.pxi":764
 * 
 *     property justify:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._get(95)
 *         def __set__(self, value):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.justify.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":766
 *         def __get__(self):
 *             return self._get(95)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'justify' : value })
 *         def __del__(self):
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify_2__set__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify_2__set__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "gen/styleclass.pxi":767
 *             return self._get(95)
 *         def __set__(self, value):
 *             self.properties.append({ 'justify' : value })             # <<<<<<<<<<<<<<
 *         def __del__(self):
 *             self.delattr('justify')
 */
  if (unlikely(__pyx_v_self->__pyx_base.properties == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(1, 767, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 767, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_justify, __pyx_v_value) < 0) __PYX_ERR(1, 767, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyList_Append(__pyx_v_self->__pyx_base.properties, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 767, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gen/styleclass.pxi":766
 *         def __get__(self):
 *             return self._get(95)
 *         def __set__(self, value):             # <<<<<<<<<<<<<<
 *             self.properties.append({ 'justify' : value })
 *         def __del__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.justify.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gen/styleclass.pxi":768
 *         def __set__(self, value):
 *             self.properties.append({ 'justify' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('justify')
 * 
 */

/* Python wrapper */
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify_4__del__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5renpy_9styledata_10styleclass_5Style_7justify_4__del__(struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "gen/styleclass.pxi":769
 *             self.properties.append({ 'justify' : value })
 *         def __del__(self):
 *             self.delattr('justify')             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_delattr); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__96, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "gen/styleclass.pxi":768
 *         def __set__(self, value):
 *             self.properties.append({ 'justify' : value })
 *         def __del__(self):             # <<<<<<<<<<<<<<
 *             self.delattr('justify')
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.justify.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style___reduce_cython__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style___reduce_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__97, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 */

/* Python wrapper */
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_5renpy_9styledata_10styleclass_5Style_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5renpy_9styledata_10styleclass_5Style_2__setstate_cython__(((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5renpy_9styledata_10styleclass_5Style_2__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_5renpy_9styledata_10styleclass_Style *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__98, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("renpy.styledata.styleclass.Style.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "renpy/style.pxd":41
 * ctypedef int (*property_function)(PyObject **cache, int *cache_priorities, int priority, object value) except -1
 * 
 * cdef inline void assign(int index, PyObject **cache, int *cache_priorities, int priority, PyObject *value):             # <<<<<<<<<<<<<<
 *     """
 *     Assigns `value` to `index` in `cache`, if it has a higher (or equal) priority than the value
 */

static CYTHON_INLINE void __pyx_f_5renpy_5style_assign(int __pyx_v_index, PyObject **__pyx_v_cache, int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_value) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("assign", 0);

  /* "renpy/style.pxd":47
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_1 = ((__pyx_v_priority < (__pyx_v_cache_priorities[__pyx_v_index])) != 0);
  if (__pyx_t_1) {

    /* "renpy/style.pxd":48
 * 
 *     if priority < cache_priorities[index]:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     Py_XDECREF(cache[index])
 */
    goto __pyx_L0;

    /* "renpy/style.pxd":47
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "renpy/style.pxd":50
 *         return
 * 
 *     Py_XDECREF(cache[index])             # <<<<<<<<<<<<<<
 *     Py_XINCREF(value)
 * 
 */
  Py_XDECREF((__pyx_v_cache[__pyx_v_index]));

  /* "renpy/style.pxd":51
 * 
 *     Py_XDECREF(cache[index])
 *     Py_XINCREF(value)             # <<<<<<<<<<<<<<
 * 
 *     cache[index] = value
 */
  Py_XINCREF(__pyx_v_value);

  /* "renpy/style.pxd":53
 *     Py_XINCREF(value)
 * 
 *     cache[index] = value             # <<<<<<<<<<<<<<
 *     cache_priorities[index] = priority
 * 
 */
  (__pyx_v_cache[__pyx_v_index]) = __pyx_v_value;

  /* "renpy/style.pxd":54
 * 
 *     cache[index] = value
 *     cache_priorities[index] = priority             # <<<<<<<<<<<<<<
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):
 */
  (__pyx_v_cache_priorities[__pyx_v_index]) = __pyx_v_priority;

  /* "renpy/style.pxd":41
 * ctypedef int (*property_function)(PyObject **cache, int *cache_priorities, int priority, object value) except -1
 * 
 * cdef inline void assign(int index, PyObject **cache, int *cache_priorities, int priority, PyObject *value):             # <<<<<<<<<<<<<<
 *     """
 *     Assigns `value` to `index` in `cache`, if it has a higher (or equal) priority than the value
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "renpy/style.pxd":56
 *     cache_priorities[index] = priority
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):             # <<<<<<<<<<<<<<
 *     """
 *     Like assign, but if the value can be duplicated, duplicates it and assigns
 */

static CYTHON_INLINE void __pyx_f_5renpy_5style_assign_prefixed(int __pyx_v_index, PyObject **__pyx_v_cache, int *__pyx_v_cache_priorities, int __pyx_v_priority, PyObject *__pyx_v_d, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_v_args = NULL;
  PyObject *__pyx_v_dd = NULL;
  PyObject *__pyx_v_value;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("assign_prefixed", 0);

  /* "renpy/style.pxd":62
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_1 = ((__pyx_v_priority < (__pyx_v_cache_priorities[__pyx_v_index])) != 0);
  if (__pyx_t_1) {

    /* "renpy/style.pxd":63
 * 
 *     if priority < cache_priorities[index]:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     if (d is not None) and d._duplicatable:
 */
    goto __pyx_L0;

    /* "renpy/style.pxd":62
 *     """
 * 
 *     if priority < cache_priorities[index]:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "renpy/style.pxd":65
 *         return
 * 
 *     if (d is not None) and d._duplicatable:             # <<<<<<<<<<<<<<
 * 
 *         args = d._args.copy(prefix=prefix)
 */
  __pyx_t_2 = (__pyx_v_d != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_duplicatable); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(2, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __pyx_t_3;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "renpy/style.pxd":67
 *     if (d is not None) and d._duplicatable:
 * 
 *         args = d._args.copy(prefix=prefix)             # <<<<<<<<<<<<<<
 *         dd = d._duplicate(args)
 *         dd._unique()
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_args); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_copy); if (unlikely(!__pyx_t_5)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_prefix, __pyx_v_prefix) < 0) __PYX_ERR(2, 67, __pyx_L1_error)
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_args = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "renpy/style.pxd":68
 * 
 *         args = d._args.copy(prefix=prefix)
 *         dd = d._duplicate(args)             # <<<<<<<<<<<<<<
 *         dd._unique()
 *     else:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_d, __pyx_n_s_duplicate); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_v_args); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_args};
        __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_6);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_v_args};
        __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_6);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_v_args);
        __Pyx_GIVEREF(__pyx_v_args);
        PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_v_args);
        __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_dd = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "renpy/style.pxd":69
 *         args = d._args.copy(prefix=prefix)
 *         dd = d._duplicate(args)
 *         dd._unique()             # <<<<<<<<<<<<<<
 *     else:
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_dd, __pyx_n_s_unique); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (__pyx_t_7) {
      __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 69, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __pyx_t_6 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 69, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "renpy/style.pxd":65
 *         return
 * 
 *     if (d is not None) and d._duplicatable:             # <<<<<<<<<<<<<<
 * 
 *         args = d._args.copy(prefix=prefix)
 */
    goto __pyx_L4;
  }

  /* "renpy/style.pxd":72
 *     else:
 * 
 *         dd = d             # <<<<<<<<<<<<<<
 * 
 *     cdef PyObject *value = <PyObject *> dd
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_d);
    __pyx_v_dd = __pyx_v_d;
  }
  __pyx_L4:;

  /* "renpy/style.pxd":74
 *         dd = d
 * 
 *     cdef PyObject *value = <PyObject *> dd             # <<<<<<<<<<<<<<
 * 
 *     Py_XDECREF(cache[index])
 */
  __pyx_v_value = ((PyObject *)__pyx_v_dd);

  /* "renpy/style.pxd":76
 *     cdef PyObject *value = <PyObject *> dd
 * 
 *     Py_XDECREF(cache[index])             # <<<<<<<<<<<<<<
 *     Py_XINCREF(value)
 * 
 */
  Py_XDECREF((__pyx_v_cache[__pyx_v_index]));

  /* "renpy/style.pxd":77
 * 
 *     Py_XDECREF(cache[index])
 *     Py_XINCREF(value)             # <<<<<<<<<<<<<<
 * 
 *     cache[index] = value
 */
  Py_XINCREF(__pyx_v_value);

  /* "renpy/style.pxd":79
 *     Py_XINCREF(value)
 * 
 *     cache[index] = value             # <<<<<<<<<<<<<<
 *     cache_priorities[index] = priority
 * 
 */
  (__pyx_v_cache[__pyx_v_index]) = __pyx_v_value;

  /* "renpy/style.pxd":80
 * 
 *     cache[index] = value
 *     cache_priorities[index] = priority             # <<<<<<<<<<<<<<
 * 
 * cdef void register_property_function(name, property_function function)
 */
  (__pyx_v_cache_priorities[__pyx_v_index]) = __pyx_v_priority;

  /* "renpy/style.pxd":56
 *     cache_priorities[index] = priority
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):             # <<<<<<<<<<<<<<
 *     """
 *     Like assign, but if the value can be duplicated, duplicates it and assigns
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_WriteUnraisable("renpy.style.assign_prefixed", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_dd);
  __Pyx_RefNannyFinishContext();
}
static struct __pyx_vtabstruct_5renpy_9styledata_10styleclass_Style __pyx_vtable_5renpy_9styledata_10styleclass_Style;

static PyObject *__pyx_tp_new_5renpy_9styledata_10styleclass_Style(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5renpy_9styledata_10styleclass_Style *p;
  PyObject *o = __pyx_ptype_5renpy_5style_StyleCore->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5renpy_9styledata_10styleclass_Style *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_5renpy_5style_StyleCore*)__pyx_vtabptr_5renpy_9styledata_10styleclass_Style;
  return o;
}

static void __pyx_tp_dealloc_5renpy_9styledata_10styleclass_Style(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_5renpy_5style_StyleCore)) __pyx_ptype_5renpy_5style_StyleCore->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_5renpy_9styledata_10styleclass_Style);
}

static int __pyx_tp_traverse_5renpy_9styledata_10styleclass_Style(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_5renpy_5style_StyleCore)) ? ((__pyx_ptype_5renpy_5style_StyleCore->tp_traverse) ? __pyx_ptype_5renpy_5style_StyleCore->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_5renpy_9styledata_10styleclass_Style)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_5renpy_9styledata_10styleclass_Style(PyObject *o) {
  if (likely(__pyx_ptype_5renpy_5style_StyleCore)) { if (__pyx_ptype_5renpy_5style_StyleCore->tp_clear) __pyx_ptype_5renpy_5style_StyleCore->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_5renpy_9styledata_10styleclass_Style);
  return 0;
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_black_color(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_black_color(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11black_color_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_bottom_margin(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bottom_margin(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13bottom_margin_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_line_spacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_line_spacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_spacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_bar_resizing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bar_resizing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_resizing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_left_margin(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_left_margin(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11left_margin_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_yminimum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yminimum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yminimum_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_alt(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_alt(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_3alt_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_font(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_font(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4font_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_underline(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_underline(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9underline_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_layout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_layout(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_6layout_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_thumb(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_thumb(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5thumb_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_reverse(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_reverse(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11box_reverse_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_outline_scaling(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_outline_scaling(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_15outline_scaling_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_text_y_fudge(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_text_y_fudge(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12text_y_fudge_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_aft_gutter(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_aft_gutter(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10aft_gutter_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_right_margin(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_right_margin(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12right_margin_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_rest_indent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_rest_indent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11rest_indent_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_background(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_background(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10background_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_outlines(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_outlines(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8outlines_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_ymaximum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_ymaximum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8ymaximum_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_yspacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yspacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8yspacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_size_group(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_size_group(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10size_group_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_debug(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_debug(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5debug_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_clipping(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_clipping(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8clipping_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_first_spacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_first_spacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13first_spacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_subtitle_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_subtitle_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14subtitle_width_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_subpixel(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_subpixel(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8subpixel_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_strikethrough(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_strikethrough(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13strikethrough_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_hyperlink_functions(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_hyperlink_functions(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_19hyperlink_functions_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xminimum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xminimum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xminimum_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_min_width(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_min_width(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9min_width_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_focus_mask(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_focus_mask(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_mask_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_yfit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yfit(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4yfit_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_drop_shadow_color(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_drop_shadow_color(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_17drop_shadow_color_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_fit_first(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_fit_first(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9fit_first_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_top_padding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_top_padding(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11top_padding_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_newline_indent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_newline_indent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14newline_indent_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_text_align(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_text_align(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10text_align_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_fore_bar(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_fore_bar(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8fore_bar_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_aft_bar(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_aft_bar(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7aft_bar_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_foreground(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_foreground(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10foreground_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_bold(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bold(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4bold_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_vertical(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_vertical(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8vertical_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_drop_shadow(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_drop_shadow(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11drop_shadow_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_spacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_spacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7spacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_ypos(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_ypos(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4ypos_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_activate_sound(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_activate_sound(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14activate_sound_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_left_padding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_left_padding(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12left_padding_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xoffset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xoffset(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xoffset_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_language(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_language(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8language_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_time_policy(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_time_policy(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11time_policy_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xmaximum(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xmaximum(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xmaximum_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_hover_sound(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_hover_sound(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11hover_sound_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_yanchor(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yanchor(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yanchor_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_fore_gutter(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_fore_gutter(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_11fore_gutter_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_adjust_spacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_adjust_spacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14adjust_spacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xfill(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xfill(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5xfill_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_color(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_color(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5color_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_key_events(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_key_events(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10key_events_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xspacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xspacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8xspacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4size_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_caret(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_caret(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5caret_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_bottom_padding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bottom_padding(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14bottom_padding_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_wrap(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_wrap(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8box_wrap_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_line_overlap_split(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_line_overlap_split(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_18line_overlap_split_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_wrap_spacing(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_wrap_spacing(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_16box_wrap_spacing_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_kerning(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_kerning(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7kerning_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_italic(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_italic(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_6italic_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_ruby_style(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_ruby_style(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10ruby_style_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_layout(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_layout(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10box_layout_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xpos(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xpos(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xpos_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xanchor(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xanchor(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7xanchor_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_top_margin(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_top_margin(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10top_margin_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_yoffset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yoffset(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7yoffset_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_child(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_child(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5child_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_thumb_offset(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_thumb_offset(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_offset_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_xfit(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xfit(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_4xfit_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_hinting(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_hinting(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7hinting_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_altruby_style(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_altruby_style(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13altruby_style_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_unscrollable(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_unscrollable(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12unscrollable_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_order_reverse(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_order_reverse(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13order_reverse_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_mouse(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_mouse(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5mouse_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_keyboard_focus(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_keyboard_focus(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14keyboard_focus_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_first_indent(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_first_indent(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12first_indent_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_bar_vertical(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bar_vertical(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12bar_vertical_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_right_padding(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_right_padding(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_13right_padding_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_focus_rect(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_focus_rect(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10focus_rect_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_antialias(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_antialias(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_9antialias_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_slow_cps(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_slow_cps(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_8slow_cps_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_bar_invert(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bar_invert(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_10bar_invert_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_line_leading(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_line_leading(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12line_leading_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_slow_cps_multiplier(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_slow_cps_multiplier(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_19slow_cps_multiplier_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_thumb_shadow(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_thumb_shadow(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_12thumb_shadow_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_yfill(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yfill(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_5yfill_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_slow_abortable(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_slow_abortable(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_14slow_abortable_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5renpy_9styledata_10styleclass_5Style_justify(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_1__get__(o);
}

static int __pyx_setprop_5renpy_9styledata_10styleclass_5Style_justify(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_3__set__(o, v);
  }
  else {
    return __pyx_pw_5renpy_9styledata_10styleclass_5Style_7justify_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_5renpy_9styledata_10styleclass_Style[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_5renpy_9styledata_10styleclass_5Style_1__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_5renpy_9styledata_10styleclass_5Style_3__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5renpy_9styledata_10styleclass_Style[] = {
  {(char *)"black_color", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_black_color, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_black_color, (char *)0, 0},
  {(char *)"bottom_margin", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_bottom_margin, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bottom_margin, (char *)0, 0},
  {(char *)"line_spacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_line_spacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_line_spacing, (char *)0, 0},
  {(char *)"bar_resizing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_bar_resizing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bar_resizing, (char *)0, 0},
  {(char *)"left_margin", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_left_margin, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_left_margin, (char *)0, 0},
  {(char *)"yminimum", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_yminimum, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yminimum, (char *)0, 0},
  {(char *)"alt", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_alt, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_alt, (char *)0, 0},
  {(char *)"font", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_font, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_font, (char *)0, 0},
  {(char *)"underline", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_underline, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_underline, (char *)0, 0},
  {(char *)"layout", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_layout, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_layout, (char *)0, 0},
  {(char *)"thumb", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_thumb, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_thumb, (char *)0, 0},
  {(char *)"box_reverse", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_reverse, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_reverse, (char *)0, 0},
  {(char *)"outline_scaling", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_outline_scaling, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_outline_scaling, (char *)0, 0},
  {(char *)"text_y_fudge", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_text_y_fudge, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_text_y_fudge, (char *)0, 0},
  {(char *)"aft_gutter", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_aft_gutter, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_aft_gutter, (char *)0, 0},
  {(char *)"right_margin", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_right_margin, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_right_margin, (char *)0, 0},
  {(char *)"rest_indent", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_rest_indent, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_rest_indent, (char *)0, 0},
  {(char *)"background", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_background, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_background, (char *)0, 0},
  {(char *)"outlines", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_outlines, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_outlines, (char *)0, 0},
  {(char *)"ymaximum", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_ymaximum, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_ymaximum, (char *)0, 0},
  {(char *)"yspacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_yspacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yspacing, (char *)0, 0},
  {(char *)"size_group", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_size_group, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_size_group, (char *)0, 0},
  {(char *)"debug", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_debug, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_debug, (char *)0, 0},
  {(char *)"clipping", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_clipping, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_clipping, (char *)0, 0},
  {(char *)"first_spacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_first_spacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_first_spacing, (char *)0, 0},
  {(char *)"subtitle_width", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_subtitle_width, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_subtitle_width, (char *)0, 0},
  {(char *)"subpixel", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_subpixel, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_subpixel, (char *)0, 0},
  {(char *)"strikethrough", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_strikethrough, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_strikethrough, (char *)0, 0},
  {(char *)"hyperlink_functions", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_hyperlink_functions, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_hyperlink_functions, (char *)0, 0},
  {(char *)"xminimum", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xminimum, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xminimum, (char *)0, 0},
  {(char *)"min_width", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_min_width, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_min_width, (char *)0, 0},
  {(char *)"focus_mask", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_focus_mask, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_focus_mask, (char *)0, 0},
  {(char *)"yfit", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_yfit, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yfit, (char *)0, 0},
  {(char *)"drop_shadow_color", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_drop_shadow_color, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_drop_shadow_color, (char *)0, 0},
  {(char *)"fit_first", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_fit_first, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_fit_first, (char *)0, 0},
  {(char *)"top_padding", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_top_padding, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_top_padding, (char *)0, 0},
  {(char *)"newline_indent", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_newline_indent, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_newline_indent, (char *)0, 0},
  {(char *)"text_align", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_text_align, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_text_align, (char *)0, 0},
  {(char *)"fore_bar", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_fore_bar, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_fore_bar, (char *)0, 0},
  {(char *)"aft_bar", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_aft_bar, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_aft_bar, (char *)0, 0},
  {(char *)"foreground", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_foreground, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_foreground, (char *)0, 0},
  {(char *)"bold", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_bold, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bold, (char *)0, 0},
  {(char *)"vertical", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_vertical, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_vertical, (char *)0, 0},
  {(char *)"drop_shadow", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_drop_shadow, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_drop_shadow, (char *)0, 0},
  {(char *)"spacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_spacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_spacing, (char *)0, 0},
  {(char *)"ypos", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_ypos, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_ypos, (char *)0, 0},
  {(char *)"activate_sound", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_activate_sound, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_activate_sound, (char *)0, 0},
  {(char *)"left_padding", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_left_padding, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_left_padding, (char *)0, 0},
  {(char *)"xoffset", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xoffset, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xoffset, (char *)0, 0},
  {(char *)"language", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_language, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_language, (char *)0, 0},
  {(char *)"time_policy", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_time_policy, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_time_policy, (char *)0, 0},
  {(char *)"xmaximum", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xmaximum, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xmaximum, (char *)0, 0},
  {(char *)"hover_sound", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_hover_sound, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_hover_sound, (char *)0, 0},
  {(char *)"yanchor", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_yanchor, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yanchor, (char *)0, 0},
  {(char *)"fore_gutter", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_fore_gutter, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_fore_gutter, (char *)0, 0},
  {(char *)"adjust_spacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_adjust_spacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_adjust_spacing, (char *)0, 0},
  {(char *)"xfill", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xfill, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xfill, (char *)0, 0},
  {(char *)"color", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_color, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_color, (char *)0, 0},
  {(char *)"key_events", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_key_events, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_key_events, (char *)0, 0},
  {(char *)"xspacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xspacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xspacing, (char *)0, 0},
  {(char *)"size", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_size, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_size, (char *)0, 0},
  {(char *)"caret", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_caret, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_caret, (char *)0, 0},
  {(char *)"bottom_padding", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_bottom_padding, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bottom_padding, (char *)0, 0},
  {(char *)"box_wrap", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_wrap, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_wrap, (char *)0, 0},
  {(char *)"line_overlap_split", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_line_overlap_split, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_line_overlap_split, (char *)0, 0},
  {(char *)"box_wrap_spacing", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_wrap_spacing, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_wrap_spacing, (char *)0, 0},
  {(char *)"kerning", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_kerning, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_kerning, (char *)0, 0},
  {(char *)"italic", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_italic, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_italic, (char *)0, 0},
  {(char *)"ruby_style", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_ruby_style, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_ruby_style, (char *)0, 0},
  {(char *)"box_layout", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_box_layout, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_box_layout, (char *)0, 0},
  {(char *)"xpos", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xpos, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xpos, (char *)0, 0},
  {(char *)"xanchor", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xanchor, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xanchor, (char *)0, 0},
  {(char *)"top_margin", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_top_margin, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_top_margin, (char *)0, 0},
  {(char *)"yoffset", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_yoffset, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yoffset, (char *)0, 0},
  {(char *)"child", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_child, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_child, (char *)0, 0},
  {(char *)"thumb_offset", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_thumb_offset, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_thumb_offset, (char *)0, 0},
  {(char *)"xfit", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_xfit, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_xfit, (char *)0, 0},
  {(char *)"hinting", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_hinting, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_hinting, (char *)0, 0},
  {(char *)"altruby_style", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_altruby_style, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_altruby_style, (char *)0, 0},
  {(char *)"unscrollable", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_unscrollable, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_unscrollable, (char *)0, 0},
  {(char *)"order_reverse", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_order_reverse, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_order_reverse, (char *)0, 0},
  {(char *)"mouse", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_mouse, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_mouse, (char *)0, 0},
  {(char *)"keyboard_focus", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_keyboard_focus, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_keyboard_focus, (char *)0, 0},
  {(char *)"first_indent", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_first_indent, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_first_indent, (char *)0, 0},
  {(char *)"bar_vertical", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_bar_vertical, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bar_vertical, (char *)0, 0},
  {(char *)"right_padding", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_right_padding, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_right_padding, (char *)0, 0},
  {(char *)"focus_rect", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_focus_rect, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_focus_rect, (char *)0, 0},
  {(char *)"antialias", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_antialias, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_antialias, (char *)0, 0},
  {(char *)"slow_cps", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_slow_cps, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_slow_cps, (char *)0, 0},
  {(char *)"bar_invert", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_bar_invert, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_bar_invert, (char *)0, 0},
  {(char *)"line_leading", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_line_leading, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_line_leading, (char *)0, 0},
  {(char *)"slow_cps_multiplier", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_slow_cps_multiplier, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_slow_cps_multiplier, (char *)0, 0},
  {(char *)"thumb_shadow", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_thumb_shadow, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_thumb_shadow, (char *)0, 0},
  {(char *)"yfill", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_yfill, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_yfill, (char *)0, 0},
  {(char *)"slow_abortable", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_slow_abortable, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_slow_abortable, (char *)0, 0},
  {(char *)"justify", __pyx_getprop_5renpy_9styledata_10styleclass_5Style_justify, __pyx_setprop_5renpy_9styledata_10styleclass_5Style_justify, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5renpy_9styledata_10styleclass_Style = {
  PyVarObject_HEAD_INIT(0, 0)
  "renpy.styledata.styleclass.Style", /*tp_name*/
  sizeof(struct __pyx_obj_5renpy_9styledata_10styleclass_Style), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5renpy_9styledata_10styleclass_Style, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5renpy_9styledata_10styleclass_Style, /*tp_traverse*/
  __pyx_tp_clear_5renpy_9styledata_10styleclass_Style, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5renpy_9styledata_10styleclass_Style, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5renpy_9styledata_10styleclass_Style, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5renpy_9styledata_10styleclass_Style, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_styleclass(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_styleclass},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "styleclass",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_activate_sound, __pyx_k_activate_sound, sizeof(__pyx_k_activate_sound), 0, 0, 1, 1},
  {&__pyx_n_s_adjust_spacing, __pyx_k_adjust_spacing, sizeof(__pyx_k_adjust_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_aft_bar, __pyx_k_aft_bar, sizeof(__pyx_k_aft_bar), 0, 0, 1, 1},
  {&__pyx_n_s_aft_gutter, __pyx_k_aft_gutter, sizeof(__pyx_k_aft_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_alt, __pyx_k_alt, sizeof(__pyx_k_alt), 0, 0, 1, 1},
  {&__pyx_n_s_altruby_style, __pyx_k_altruby_style, sizeof(__pyx_k_altruby_style), 0, 0, 1, 1},
  {&__pyx_n_s_antialias, __pyx_k_antialias, sizeof(__pyx_k_antialias), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_background, __pyx_k_background, sizeof(__pyx_k_background), 0, 0, 1, 1},
  {&__pyx_n_s_bar_invert, __pyx_k_bar_invert, sizeof(__pyx_k_bar_invert), 0, 0, 1, 1},
  {&__pyx_n_s_bar_resizing, __pyx_k_bar_resizing, sizeof(__pyx_k_bar_resizing), 0, 0, 1, 1},
  {&__pyx_n_s_bar_vertical, __pyx_k_bar_vertical, sizeof(__pyx_k_bar_vertical), 0, 0, 1, 1},
  {&__pyx_n_s_black_color, __pyx_k_black_color, sizeof(__pyx_k_black_color), 0, 0, 1, 1},
  {&__pyx_n_s_bold, __pyx_k_bold, sizeof(__pyx_k_bold), 0, 0, 1, 1},
  {&__pyx_n_s_bottom_margin, __pyx_k_bottom_margin, sizeof(__pyx_k_bottom_margin), 0, 0, 1, 1},
  {&__pyx_n_s_bottom_padding, __pyx_k_bottom_padding, sizeof(__pyx_k_bottom_padding), 0, 0, 1, 1},
  {&__pyx_n_s_box_layout, __pyx_k_box_layout, sizeof(__pyx_k_box_layout), 0, 0, 1, 1},
  {&__pyx_n_s_box_reverse, __pyx_k_box_reverse, sizeof(__pyx_k_box_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_box_wrap, __pyx_k_box_wrap, sizeof(__pyx_k_box_wrap), 0, 0, 1, 1},
  {&__pyx_n_s_box_wrap_spacing, __pyx_k_box_wrap_spacing, sizeof(__pyx_k_box_wrap_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_caret, __pyx_k_caret, sizeof(__pyx_k_caret), 0, 0, 1, 1},
  {&__pyx_n_s_child, __pyx_k_child, sizeof(__pyx_k_child), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_clipping, __pyx_k_clipping, sizeof(__pyx_k_clipping), 0, 0, 1, 1},
  {&__pyx_n_s_color, __pyx_k_color, sizeof(__pyx_k_color), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_debug, __pyx_k_debug, sizeof(__pyx_k_debug), 0, 0, 1, 1},
  {&__pyx_n_s_delattr, __pyx_k_delattr, sizeof(__pyx_k_delattr), 0, 0, 1, 1},
  {&__pyx_n_s_drop_shadow, __pyx_k_drop_shadow, sizeof(__pyx_k_drop_shadow), 0, 0, 1, 1},
  {&__pyx_n_s_drop_shadow_color, __pyx_k_drop_shadow_color, sizeof(__pyx_k_drop_shadow_color), 0, 0, 1, 1},
  {&__pyx_n_s_duplicatable, __pyx_k_duplicatable, sizeof(__pyx_k_duplicatable), 0, 0, 1, 1},
  {&__pyx_n_s_duplicate, __pyx_k_duplicate, sizeof(__pyx_k_duplicate), 0, 0, 1, 1},
  {&__pyx_n_s_first_indent, __pyx_k_first_indent, sizeof(__pyx_k_first_indent), 0, 0, 1, 1},
  {&__pyx_n_s_first_spacing, __pyx_k_first_spacing, sizeof(__pyx_k_first_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_fit_first, __pyx_k_fit_first, sizeof(__pyx_k_fit_first), 0, 0, 1, 1},
  {&__pyx_n_s_focus_mask, __pyx_k_focus_mask, sizeof(__pyx_k_focus_mask), 0, 0, 1, 1},
  {&__pyx_n_s_focus_rect, __pyx_k_focus_rect, sizeof(__pyx_k_focus_rect), 0, 0, 1, 1},
  {&__pyx_n_s_font, __pyx_k_font, sizeof(__pyx_k_font), 0, 0, 1, 1},
  {&__pyx_n_s_fore_bar, __pyx_k_fore_bar, sizeof(__pyx_k_fore_bar), 0, 0, 1, 1},
  {&__pyx_n_s_fore_gutter, __pyx_k_fore_gutter, sizeof(__pyx_k_fore_gutter), 0, 0, 1, 1},
  {&__pyx_n_s_foreground, __pyx_k_foreground, sizeof(__pyx_k_foreground), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_hinting, __pyx_k_hinting, sizeof(__pyx_k_hinting), 0, 0, 1, 1},
  {&__pyx_n_s_hover_sound, __pyx_k_hover_sound, sizeof(__pyx_k_hover_sound), 0, 0, 1, 1},
  {&__pyx_n_s_hyperlink_functions, __pyx_k_hyperlink_functions, sizeof(__pyx_k_hyperlink_functions), 0, 0, 1, 1},
  {&__pyx_n_s_italic, __pyx_k_italic, sizeof(__pyx_k_italic), 0, 0, 1, 1},
  {&__pyx_n_s_justify, __pyx_k_justify, sizeof(__pyx_k_justify), 0, 0, 1, 1},
  {&__pyx_n_s_kerning, __pyx_k_kerning, sizeof(__pyx_k_kerning), 0, 0, 1, 1},
  {&__pyx_n_s_key_events, __pyx_k_key_events, sizeof(__pyx_k_key_events), 0, 0, 1, 1},
  {&__pyx_n_s_keyboard_focus, __pyx_k_keyboard_focus, sizeof(__pyx_k_keyboard_focus), 0, 0, 1, 1},
  {&__pyx_n_s_language, __pyx_k_language, sizeof(__pyx_k_language), 0, 0, 1, 1},
  {&__pyx_n_s_layout, __pyx_k_layout, sizeof(__pyx_k_layout), 0, 0, 1, 1},
  {&__pyx_n_s_left_margin, __pyx_k_left_margin, sizeof(__pyx_k_left_margin), 0, 0, 1, 1},
  {&__pyx_n_s_left_padding, __pyx_k_left_padding, sizeof(__pyx_k_left_padding), 0, 0, 1, 1},
  {&__pyx_n_s_line_leading, __pyx_k_line_leading, sizeof(__pyx_k_line_leading), 0, 0, 1, 1},
  {&__pyx_n_s_line_overlap_split, __pyx_k_line_overlap_split, sizeof(__pyx_k_line_overlap_split), 0, 0, 1, 1},
  {&__pyx_n_s_line_spacing, __pyx_k_line_spacing, sizeof(__pyx_k_line_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_min_width, __pyx_k_min_width, sizeof(__pyx_k_min_width), 0, 0, 1, 1},
  {&__pyx_n_s_mouse, __pyx_k_mouse, sizeof(__pyx_k_mouse), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_newline_indent, __pyx_k_newline_indent, sizeof(__pyx_k_newline_indent), 0, 0, 1, 1},
  {&__pyx_n_s_order_reverse, __pyx_k_order_reverse, sizeof(__pyx_k_order_reverse), 0, 0, 1, 1},
  {&__pyx_n_s_outline_scaling, __pyx_k_outline_scaling, sizeof(__pyx_k_outline_scaling), 0, 0, 1, 1},
  {&__pyx_n_s_outlines, __pyx_k_outlines, sizeof(__pyx_k_outlines), 0, 0, 1, 1},
  {&__pyx_n_s_prefix, __pyx_k_prefix, sizeof(__pyx_k_prefix), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_rest_indent, __pyx_k_rest_indent, sizeof(__pyx_k_rest_indent), 0, 0, 1, 1},
  {&__pyx_n_s_right_margin, __pyx_k_right_margin, sizeof(__pyx_k_right_margin), 0, 0, 1, 1},
  {&__pyx_n_s_right_padding, __pyx_k_right_padding, sizeof(__pyx_k_right_padding), 0, 0, 1, 1},
  {&__pyx_n_s_ruby_style, __pyx_k_ruby_style, sizeof(__pyx_k_ruby_style), 0, 0, 1, 1},
  {&__pyx_kp_s_self_cache_cannot_be_converted_t, __pyx_k_self_cache_cannot_be_converted_t, sizeof(__pyx_k_self_cache_cannot_be_converted_t), 0, 0, 1, 0},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_size_group, __pyx_k_size_group, sizeof(__pyx_k_size_group), 0, 0, 1, 1},
  {&__pyx_n_s_slow_abortable, __pyx_k_slow_abortable, sizeof(__pyx_k_slow_abortable), 0, 0, 1, 1},
  {&__pyx_n_s_slow_cps, __pyx_k_slow_cps, sizeof(__pyx_k_slow_cps), 0, 0, 1, 1},
  {&__pyx_n_s_slow_cps_multiplier, __pyx_k_slow_cps_multiplier, sizeof(__pyx_k_slow_cps_multiplier), 0, 0, 1, 1},
  {&__pyx_n_s_spacing, __pyx_k_spacing, sizeof(__pyx_k_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_strikethrough, __pyx_k_strikethrough, sizeof(__pyx_k_strikethrough), 0, 0, 1, 1},
  {&__pyx_n_s_subpixel, __pyx_k_subpixel, sizeof(__pyx_k_subpixel), 0, 0, 1, 1},
  {&__pyx_n_s_subtitle_width, __pyx_k_subtitle_width, sizeof(__pyx_k_subtitle_width), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_text_align, __pyx_k_text_align, sizeof(__pyx_k_text_align), 0, 0, 1, 1},
  {&__pyx_n_s_text_y_fudge, __pyx_k_text_y_fudge, sizeof(__pyx_k_text_y_fudge), 0, 0, 1, 1},
  {&__pyx_n_s_thumb, __pyx_k_thumb, sizeof(__pyx_k_thumb), 0, 0, 1, 1},
  {&__pyx_n_s_thumb_offset, __pyx_k_thumb_offset, sizeof(__pyx_k_thumb_offset), 0, 0, 1, 1},
  {&__pyx_n_s_thumb_shadow, __pyx_k_thumb_shadow, sizeof(__pyx_k_thumb_shadow), 0, 0, 1, 1},
  {&__pyx_n_s_time_policy, __pyx_k_time_policy, sizeof(__pyx_k_time_policy), 0, 0, 1, 1},
  {&__pyx_n_s_top_margin, __pyx_k_top_margin, sizeof(__pyx_k_top_margin), 0, 0, 1, 1},
  {&__pyx_n_s_top_padding, __pyx_k_top_padding, sizeof(__pyx_k_top_padding), 0, 0, 1, 1},
  {&__pyx_n_s_underline, __pyx_k_underline, sizeof(__pyx_k_underline), 0, 0, 1, 1},
  {&__pyx_n_s_unique, __pyx_k_unique, sizeof(__pyx_k_unique), 0, 0, 1, 1},
  {&__pyx_n_s_unscrollable, __pyx_k_unscrollable, sizeof(__pyx_k_unscrollable), 0, 0, 1, 1},
  {&__pyx_n_s_vertical, __pyx_k_vertical, sizeof(__pyx_k_vertical), 0, 0, 1, 1},
  {&__pyx_n_s_xanchor, __pyx_k_xanchor, sizeof(__pyx_k_xanchor), 0, 0, 1, 1},
  {&__pyx_n_s_xfill, __pyx_k_xfill, sizeof(__pyx_k_xfill), 0, 0, 1, 1},
  {&__pyx_n_s_xfit, __pyx_k_xfit, sizeof(__pyx_k_xfit), 0, 0, 1, 1},
  {&__pyx_n_s_xmaximum, __pyx_k_xmaximum, sizeof(__pyx_k_xmaximum), 0, 0, 1, 1},
  {&__pyx_n_s_xminimum, __pyx_k_xminimum, sizeof(__pyx_k_xminimum), 0, 0, 1, 1},
  {&__pyx_n_s_xoffset, __pyx_k_xoffset, sizeof(__pyx_k_xoffset), 0, 0, 1, 1},
  {&__pyx_n_s_xpos, __pyx_k_xpos, sizeof(__pyx_k_xpos), 0, 0, 1, 1},
  {&__pyx_n_s_xspacing, __pyx_k_xspacing, sizeof(__pyx_k_xspacing), 0, 0, 1, 1},
  {&__pyx_n_s_yanchor, __pyx_k_yanchor, sizeof(__pyx_k_yanchor), 0, 0, 1, 1},
  {&__pyx_n_s_yfill, __pyx_k_yfill, sizeof(__pyx_k_yfill), 0, 0, 1, 1},
  {&__pyx_n_s_yfit, __pyx_k_yfit, sizeof(__pyx_k_yfit), 0, 0, 1, 1},
  {&__pyx_n_s_ymaximum, __pyx_k_ymaximum, sizeof(__pyx_k_ymaximum), 0, 0, 1, 1},
  {&__pyx_n_s_yminimum, __pyx_k_yminimum, sizeof(__pyx_k_yminimum), 0, 0, 1, 1},
  {&__pyx_n_s_yoffset, __pyx_k_yoffset, sizeof(__pyx_k_yoffset), 0, 0, 1, 1},
  {&__pyx_n_s_ypos, __pyx_k_ypos, sizeof(__pyx_k_ypos), 0, 0, 1, 1},
  {&__pyx_n_s_yspacing, __pyx_k_yspacing, sizeof(__pyx_k_yspacing), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 2, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "gen/styleclass.pxi":9
 *             self.properties.append({ 'black_color' : value })
 *         def __del__(self):
 *             self.delattr('black_color')             # <<<<<<<<<<<<<<
 * 
 *     property bottom_margin:
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_n_s_black_color); if (unlikely(!__pyx_tuple_)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "gen/styleclass.pxi":17
 *             self.properties.append({ 'bottom_margin' : value })
 *         def __del__(self):
 *             self.delattr('bottom_margin')             # <<<<<<<<<<<<<<
 * 
 *     property line_spacing:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_n_s_bottom_margin); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "gen/styleclass.pxi":25
 *             self.properties.append({ 'line_spacing' : value })
 *         def __del__(self):
 *             self.delattr('line_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property bar_resizing:
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_n_s_line_spacing); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "gen/styleclass.pxi":33
 *             self.properties.append({ 'bar_resizing' : value })
 *         def __del__(self):
 *             self.delattr('bar_resizing')             # <<<<<<<<<<<<<<
 * 
 *     property left_margin:
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_s_bar_resizing); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "gen/styleclass.pxi":41
 *             self.properties.append({ 'left_margin' : value })
 *         def __del__(self):
 *             self.delattr('left_margin')             # <<<<<<<<<<<<<<
 * 
 *     property yminimum:
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_n_s_left_margin); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(1, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "gen/styleclass.pxi":49
 *             self.properties.append({ 'yminimum' : value })
 *         def __del__(self):
 *             self.delattr('yminimum')             # <<<<<<<<<<<<<<
 * 
 *     property alt:
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_n_s_yminimum); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "gen/styleclass.pxi":57
 *             self.properties.append({ 'alt' : value })
 *         def __del__(self):
 *             self.delattr('alt')             # <<<<<<<<<<<<<<
 * 
 *     property font:
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_s_alt); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(1, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "gen/styleclass.pxi":65
 *             self.properties.append({ 'font' : value })
 *         def __del__(self):
 *             self.delattr('font')             # <<<<<<<<<<<<<<
 * 
 *     property underline:
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_n_s_font); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "gen/styleclass.pxi":73
 *             self.properties.append({ 'underline' : value })
 *         def __del__(self):
 *             self.delattr('underline')             # <<<<<<<<<<<<<<
 * 
 *     property layout:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_underline); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "gen/styleclass.pxi":81
 *             self.properties.append({ 'layout' : value })
 *         def __del__(self):
 *             self.delattr('layout')             # <<<<<<<<<<<<<<
 * 
 *     property thumb:
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_n_s_layout); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "gen/styleclass.pxi":89
 *             self.properties.append({ 'thumb' : value })
 *         def __del__(self):
 *             self.delattr('thumb')             # <<<<<<<<<<<<<<
 * 
 *     property box_reverse:
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_thumb); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "gen/styleclass.pxi":97
 *             self.properties.append({ 'box_reverse' : value })
 *         def __del__(self):
 *             self.delattr('box_reverse')             # <<<<<<<<<<<<<<
 * 
 *     property outline_scaling:
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_n_s_box_reverse); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "gen/styleclass.pxi":105
 *             self.properties.append({ 'outline_scaling' : value })
 *         def __del__(self):
 *             self.delattr('outline_scaling')             # <<<<<<<<<<<<<<
 * 
 *     property text_y_fudge:
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_n_s_outline_scaling); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "gen/styleclass.pxi":113
 *             self.properties.append({ 'text_y_fudge' : value })
 *         def __del__(self):
 *             self.delattr('text_y_fudge')             # <<<<<<<<<<<<<<
 * 
 *     property aft_gutter:
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_n_s_text_y_fudge); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(1, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "gen/styleclass.pxi":121
 *             self.properties.append({ 'aft_gutter' : value })
 *         def __del__(self):
 *             self.delattr('aft_gutter')             # <<<<<<<<<<<<<<
 * 
 *     property right_margin:
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_n_s_aft_gutter); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "gen/styleclass.pxi":129
 *             self.properties.append({ 'right_margin' : value })
 *         def __del__(self):
 *             self.delattr('right_margin')             # <<<<<<<<<<<<<<
 * 
 *     property rest_indent:
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_n_s_right_margin); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(1, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "gen/styleclass.pxi":137
 *             self.properties.append({ 'rest_indent' : value })
 *         def __del__(self):
 *             self.delattr('rest_indent')             # <<<<<<<<<<<<<<
 * 
 *     property background:
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_n_s_rest_indent); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "gen/styleclass.pxi":145
 *             self.properties.append({ 'background' : value })
 *         def __del__(self):
 *             self.delattr('background')             # <<<<<<<<<<<<<<
 * 
 *     property outlines:
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_n_s_background); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(1, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "gen/styleclass.pxi":153
 *             self.properties.append({ 'outlines' : value })
 *         def __del__(self):
 *             self.delattr('outlines')             # <<<<<<<<<<<<<<
 * 
 *     property ymaximum:
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_n_s_outlines); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "gen/styleclass.pxi":161
 *             self.properties.append({ 'ymaximum' : value })
 *         def __del__(self):
 *             self.delattr('ymaximum')             # <<<<<<<<<<<<<<
 * 
 *     property yspacing:
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_n_s_ymaximum); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(1, 161, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "gen/styleclass.pxi":169
 *             self.properties.append({ 'yspacing' : value })
 *         def __del__(self):
 *             self.delattr('yspacing')             # <<<<<<<<<<<<<<
 * 
 *     property size_group:
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_n_s_yspacing); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "gen/styleclass.pxi":177
 *             self.properties.append({ 'size_group' : value })
 *         def __del__(self):
 *             self.delattr('size_group')             # <<<<<<<<<<<<<<
 * 
 *     property debug:
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_n_s_size_group); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(1, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "gen/styleclass.pxi":185
 *             self.properties.append({ 'debug' : value })
 *         def __del__(self):
 *             self.delattr('debug')             # <<<<<<<<<<<<<<
 * 
 *     property clipping:
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_n_s_debug); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "gen/styleclass.pxi":193
 *             self.properties.append({ 'clipping' : value })
 *         def __del__(self):
 *             self.delattr('clipping')             # <<<<<<<<<<<<<<
 * 
 *     property first_spacing:
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_n_s_clipping); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(1, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "gen/styleclass.pxi":201
 *             self.properties.append({ 'first_spacing' : value })
 *         def __del__(self):
 *             self.delattr('first_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property subtitle_width:
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_n_s_first_spacing); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "gen/styleclass.pxi":209
 *             self.properties.append({ 'subtitle_width' : value })
 *         def __del__(self):
 *             self.delattr('subtitle_width')             # <<<<<<<<<<<<<<
 * 
 *     property subpixel:
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_n_s_subtitle_width); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(1, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "gen/styleclass.pxi":217
 *             self.properties.append({ 'subpixel' : value })
 *         def __del__(self):
 *             self.delattr('subpixel')             # <<<<<<<<<<<<<<
 * 
 *     property strikethrough:
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_n_s_subpixel); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 217, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "gen/styleclass.pxi":225
 *             self.properties.append({ 'strikethrough' : value })
 *         def __del__(self):
 *             self.delattr('strikethrough')             # <<<<<<<<<<<<<<
 * 
 *     property hyperlink_functions:
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_n_s_strikethrough); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(1, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "gen/styleclass.pxi":233
 *             self.properties.append({ 'hyperlink_functions' : value })
 *         def __del__(self):
 *             self.delattr('hyperlink_functions')             # <<<<<<<<<<<<<<
 * 
 *     property xminimum:
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_n_s_hyperlink_functions); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(1, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "gen/styleclass.pxi":241
 *             self.properties.append({ 'xminimum' : value })
 *         def __del__(self):
 *             self.delattr('xminimum')             # <<<<<<<<<<<<<<
 * 
 *     property min_width:
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_n_s_xminimum); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "gen/styleclass.pxi":249
 *             self.properties.append({ 'min_width' : value })
 *         def __del__(self):
 *             self.delattr('min_width')             # <<<<<<<<<<<<<<
 * 
 *     property focus_mask:
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_n_s_min_width); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "gen/styleclass.pxi":257
 *             self.properties.append({ 'focus_mask' : value })
 *         def __del__(self):
 *             self.delattr('focus_mask')             # <<<<<<<<<<<<<<
 * 
 *     property yfit:
 */
  __pyx_tuple__32 = PyTuple_Pack(1, __pyx_n_s_focus_mask); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(1, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "gen/styleclass.pxi":265
 *             self.properties.append({ 'yfit' : value })
 *         def __del__(self):
 *             self.delattr('yfit')             # <<<<<<<<<<<<<<
 * 
 *     property drop_shadow_color:
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_n_s_yfit); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(1, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "gen/styleclass.pxi":273
 *             self.properties.append({ 'drop_shadow_color' : value })
 *         def __del__(self):
 *             self.delattr('drop_shadow_color')             # <<<<<<<<<<<<<<
 * 
 *     property fit_first:
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_n_s_drop_shadow_color); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(1, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "gen/styleclass.pxi":281
 *             self.properties.append({ 'fit_first' : value })
 *         def __del__(self):
 *             self.delattr('fit_first')             # <<<<<<<<<<<<<<
 * 
 *     property top_padding:
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_n_s_fit_first); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(1, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "gen/styleclass.pxi":289
 *             self.properties.append({ 'top_padding' : value })
 *         def __del__(self):
 *             self.delattr('top_padding')             # <<<<<<<<<<<<<<
 * 
 *     property newline_indent:
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_n_s_top_padding); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(1, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "gen/styleclass.pxi":297
 *             self.properties.append({ 'newline_indent' : value })
 *         def __del__(self):
 *             self.delattr('newline_indent')             # <<<<<<<<<<<<<<
 * 
 *     property text_align:
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_n_s_newline_indent); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(1, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);

  /* "gen/styleclass.pxi":305
 *             self.properties.append({ 'text_align' : value })
 *         def __del__(self):
 *             self.delattr('text_align')             # <<<<<<<<<<<<<<
 * 
 *     property fore_bar:
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_n_s_text_align); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(1, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);

  /* "gen/styleclass.pxi":313
 *             self.properties.append({ 'fore_bar' : value })
 *         def __del__(self):
 *             self.delattr('fore_bar')             # <<<<<<<<<<<<<<
 * 
 *     property aft_bar:
 */
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_n_s_fore_bar); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(1, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);

  /* "gen/styleclass.pxi":321
 *             self.properties.append({ 'aft_bar' : value })
 *         def __del__(self):
 *             self.delattr('aft_bar')             # <<<<<<<<<<<<<<
 * 
 *     property foreground:
 */
  __pyx_tuple__40 = PyTuple_Pack(1, __pyx_n_s_aft_bar); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(1, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "gen/styleclass.pxi":329
 *             self.properties.append({ 'foreground' : value })
 *         def __del__(self):
 *             self.delattr('foreground')             # <<<<<<<<<<<<<<
 * 
 *     property bold:
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_n_s_foreground); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(1, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "gen/styleclass.pxi":337
 *             self.properties.append({ 'bold' : value })
 *         def __del__(self):
 *             self.delattr('bold')             # <<<<<<<<<<<<<<
 * 
 *     property vertical:
 */
  __pyx_tuple__42 = PyTuple_Pack(1, __pyx_n_s_bold); if (unlikely(!__pyx_tuple__42)) __PYX_ERR(1, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "gen/styleclass.pxi":345
 *             self.properties.append({ 'vertical' : value })
 *         def __del__(self):
 *             self.delattr('vertical')             # <<<<<<<<<<<<<<
 * 
 *     property drop_shadow:
 */
  __pyx_tuple__43 = PyTuple_Pack(1, __pyx_n_s_vertical); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(1, 345, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "gen/styleclass.pxi":353
 *             self.properties.append({ 'drop_shadow' : value })
 *         def __del__(self):
 *             self.delattr('drop_shadow')             # <<<<<<<<<<<<<<
 * 
 *     property spacing:
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_n_s_drop_shadow); if (unlikely(!__pyx_tuple__44)) __PYX_ERR(1, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "gen/styleclass.pxi":361
 *             self.properties.append({ 'spacing' : value })
 *         def __del__(self):
 *             self.delattr('spacing')             # <<<<<<<<<<<<<<
 * 
 *     property ypos:
 */
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_n_s_spacing); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(1, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "gen/styleclass.pxi":369
 *             self.properties.append({ 'ypos' : value })
 *         def __del__(self):
 *             self.delattr('ypos')             # <<<<<<<<<<<<<<
 * 
 *     property activate_sound:
 */
  __pyx_tuple__46 = PyTuple_Pack(1, __pyx_n_s_ypos); if (unlikely(!__pyx_tuple__46)) __PYX_ERR(1, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "gen/styleclass.pxi":377
 *             self.properties.append({ 'activate_sound' : value })
 *         def __del__(self):
 *             self.delattr('activate_sound')             # <<<<<<<<<<<<<<
 * 
 *     property left_padding:
 */
  __pyx_tuple__47 = PyTuple_Pack(1, __pyx_n_s_activate_sound); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(1, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);

  /* "gen/styleclass.pxi":385
 *             self.properties.append({ 'left_padding' : value })
 *         def __del__(self):
 *             self.delattr('left_padding')             # <<<<<<<<<<<<<<
 * 
 *     property xoffset:
 */
  __pyx_tuple__48 = PyTuple_Pack(1, __pyx_n_s_left_padding); if (unlikely(!__pyx_tuple__48)) __PYX_ERR(1, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);

  /* "gen/styleclass.pxi":393
 *             self.properties.append({ 'xoffset' : value })
 *         def __del__(self):
 *             self.delattr('xoffset')             # <<<<<<<<<<<<<<
 * 
 *     property language:
 */
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_n_s_xoffset); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(1, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);

  /* "gen/styleclass.pxi":401
 *             self.properties.append({ 'language' : value })
 *         def __del__(self):
 *             self.delattr('language')             # <<<<<<<<<<<<<<
 * 
 *     property time_policy:
 */
  __pyx_tuple__50 = PyTuple_Pack(1, __pyx_n_s_language); if (unlikely(!__pyx_tuple__50)) __PYX_ERR(1, 401, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "gen/styleclass.pxi":409
 *             self.properties.append({ 'time_policy' : value })
 *         def __del__(self):
 *             self.delattr('time_policy')             # <<<<<<<<<<<<<<
 * 
 *     property xmaximum:
 */
  __pyx_tuple__51 = PyTuple_Pack(1, __pyx_n_s_time_policy); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(1, 409, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "gen/styleclass.pxi":417
 *             self.properties.append({ 'xmaximum' : value })
 *         def __del__(self):
 *             self.delattr('xmaximum')             # <<<<<<<<<<<<<<
 * 
 *     property hover_sound:
 */
  __pyx_tuple__52 = PyTuple_Pack(1, __pyx_n_s_xmaximum); if (unlikely(!__pyx_tuple__52)) __PYX_ERR(1, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "gen/styleclass.pxi":425
 *             self.properties.append({ 'hover_sound' : value })
 *         def __del__(self):
 *             self.delattr('hover_sound')             # <<<<<<<<<<<<<<
 * 
 *     property yanchor:
 */
  __pyx_tuple__53 = PyTuple_Pack(1, __pyx_n_s_hover_sound); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(1, 425, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);

  /* "gen/styleclass.pxi":433
 *             self.properties.append({ 'yanchor' : value })
 *         def __del__(self):
 *             self.delattr('yanchor')             # <<<<<<<<<<<<<<
 * 
 *     property fore_gutter:
 */
  __pyx_tuple__54 = PyTuple_Pack(1, __pyx_n_s_yanchor); if (unlikely(!__pyx_tuple__54)) __PYX_ERR(1, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "gen/styleclass.pxi":441
 *             self.properties.append({ 'fore_gutter' : value })
 *         def __del__(self):
 *             self.delattr('fore_gutter')             # <<<<<<<<<<<<<<
 * 
 *     property adjust_spacing:
 */
  __pyx_tuple__55 = PyTuple_Pack(1, __pyx_n_s_fore_gutter); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(1, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "gen/styleclass.pxi":449
 *             self.properties.append({ 'adjust_spacing' : value })
 *         def __del__(self):
 *             self.delattr('adjust_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property xfill:
 */
  __pyx_tuple__56 = PyTuple_Pack(1, __pyx_n_s_adjust_spacing); if (unlikely(!__pyx_tuple__56)) __PYX_ERR(1, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "gen/styleclass.pxi":457
 *             self.properties.append({ 'xfill' : value })
 *         def __del__(self):
 *             self.delattr('xfill')             # <<<<<<<<<<<<<<
 * 
 *     property color:
 */
  __pyx_tuple__57 = PyTuple_Pack(1, __pyx_n_s_xfill); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(1, 457, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);

  /* "gen/styleclass.pxi":465
 *             self.properties.append({ 'color' : value })
 *         def __del__(self):
 *             self.delattr('color')             # <<<<<<<<<<<<<<
 * 
 *     property key_events:
 */
  __pyx_tuple__58 = PyTuple_Pack(1, __pyx_n_s_color); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(1, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);

  /* "gen/styleclass.pxi":473
 *             self.properties.append({ 'key_events' : value })
 *         def __del__(self):
 *             self.delattr('key_events')             # <<<<<<<<<<<<<<
 * 
 *     property xspacing:
 */
  __pyx_tuple__59 = PyTuple_Pack(1, __pyx_n_s_key_events); if (unlikely(!__pyx_tuple__59)) __PYX_ERR(1, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);

  /* "gen/styleclass.pxi":481
 *             self.properties.append({ 'xspacing' : value })
 *         def __del__(self):
 *             self.delattr('xspacing')             # <<<<<<<<<<<<<<
 * 
 *     property size:
 */
  __pyx_tuple__60 = PyTuple_Pack(1, __pyx_n_s_xspacing); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(1, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);

  /* "gen/styleclass.pxi":489
 *             self.properties.append({ 'size' : value })
 *         def __del__(self):
 *             self.delattr('size')             # <<<<<<<<<<<<<<
 * 
 *     property caret:
 */
  __pyx_tuple__61 = PyTuple_Pack(1, __pyx_n_s_size); if (unlikely(!__pyx_tuple__61)) __PYX_ERR(1, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "gen/styleclass.pxi":497
 *             self.properties.append({ 'caret' : value })
 *         def __del__(self):
 *             self.delattr('caret')             # <<<<<<<<<<<<<<
 * 
 *     property bottom_padding:
 */
  __pyx_tuple__62 = PyTuple_Pack(1, __pyx_n_s_caret); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(1, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "gen/styleclass.pxi":505
 *             self.properties.append({ 'bottom_padding' : value })
 *         def __del__(self):
 *             self.delattr('bottom_padding')             # <<<<<<<<<<<<<<
 * 
 *     property box_wrap:
 */
  __pyx_tuple__63 = PyTuple_Pack(1, __pyx_n_s_bottom_padding); if (unlikely(!__pyx_tuple__63)) __PYX_ERR(1, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);

  /* "gen/styleclass.pxi":513
 *             self.properties.append({ 'box_wrap' : value })
 *         def __del__(self):
 *             self.delattr('box_wrap')             # <<<<<<<<<<<<<<
 * 
 *     property line_overlap_split:
 */
  __pyx_tuple__64 = PyTuple_Pack(1, __pyx_n_s_box_wrap); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(1, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "gen/styleclass.pxi":521
 *             self.properties.append({ 'line_overlap_split' : value })
 *         def __del__(self):
 *             self.delattr('line_overlap_split')             # <<<<<<<<<<<<<<
 * 
 *     property box_wrap_spacing:
 */
  __pyx_tuple__65 = PyTuple_Pack(1, __pyx_n_s_line_overlap_split); if (unlikely(!__pyx_tuple__65)) __PYX_ERR(1, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "gen/styleclass.pxi":529
 *             self.properties.append({ 'box_wrap_spacing' : value })
 *         def __del__(self):
 *             self.delattr('box_wrap_spacing')             # <<<<<<<<<<<<<<
 * 
 *     property kerning:
 */
  __pyx_tuple__66 = PyTuple_Pack(1, __pyx_n_s_box_wrap_spacing); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(1, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "gen/styleclass.pxi":537
 *             self.properties.append({ 'kerning' : value })
 *         def __del__(self):
 *             self.delattr('kerning')             # <<<<<<<<<<<<<<
 * 
 *     property italic:
 */
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_n_s_kerning); if (unlikely(!__pyx_tuple__67)) __PYX_ERR(1, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);

  /* "gen/styleclass.pxi":545
 *             self.properties.append({ 'italic' : value })
 *         def __del__(self):
 *             self.delattr('italic')             # <<<<<<<<<<<<<<
 * 
 *     property ruby_style:
 */
  __pyx_tuple__68 = PyTuple_Pack(1, __pyx_n_s_italic); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(1, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);

  /* "gen/styleclass.pxi":553
 *             self.properties.append({ 'ruby_style' : value })
 *         def __del__(self):
 *             self.delattr('ruby_style')             # <<<<<<<<<<<<<<
 * 
 *     property box_layout:
 */
  __pyx_tuple__69 = PyTuple_Pack(1, __pyx_n_s_ruby_style); if (unlikely(!__pyx_tuple__69)) __PYX_ERR(1, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);

  /* "gen/styleclass.pxi":561
 *             self.properties.append({ 'box_layout' : value })
 *         def __del__(self):
 *             self.delattr('box_layout')             # <<<<<<<<<<<<<<
 * 
 *     property xpos:
 */
  __pyx_tuple__70 = PyTuple_Pack(1, __pyx_n_s_box_layout); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(1, 561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);

  /* "gen/styleclass.pxi":569
 *             self.properties.append({ 'xpos' : value })
 *         def __del__(self):
 *             self.delattr('xpos')             # <<<<<<<<<<<<<<
 * 
 *     property xanchor:
 */
  __pyx_tuple__71 = PyTuple_Pack(1, __pyx_n_s_xpos); if (unlikely(!__pyx_tuple__71)) __PYX_ERR(1, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);

  /* "gen/styleclass.pxi":577
 *             self.properties.append({ 'xanchor' : value })
 *         def __del__(self):
 *             self.delattr('xanchor')             # <<<<<<<<<<<<<<
 * 
 *     property top_margin:
 */
  __pyx_tuple__72 = PyTuple_Pack(1, __pyx_n_s_xanchor); if (unlikely(!__pyx_tuple__72)) __PYX_ERR(1, 577, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);

  /* "gen/styleclass.pxi":585
 *             self.properties.append({ 'top_margin' : value })
 *         def __del__(self):
 *             self.delattr('top_margin')             # <<<<<<<<<<<<<<
 * 
 *     property yoffset:
 */
  __pyx_tuple__73 = PyTuple_Pack(1, __pyx_n_s_top_margin); if (unlikely(!__pyx_tuple__73)) __PYX_ERR(1, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);

  /* "gen/styleclass.pxi":593
 *             self.properties.append({ 'yoffset' : value })
 *         def __del__(self):
 *             self.delattr('yoffset')             # <<<<<<<<<<<<<<
 * 
 *     property child:
 */
  __pyx_tuple__74 = PyTuple_Pack(1, __pyx_n_s_yoffset); if (unlikely(!__pyx_tuple__74)) __PYX_ERR(1, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);

  /* "gen/styleclass.pxi":601
 *             self.properties.append({ 'child' : value })
 *         def __del__(self):
 *             self.delattr('child')             # <<<<<<<<<<<<<<
 * 
 *     property thumb_offset:
 */
  __pyx_tuple__75 = PyTuple_Pack(1, __pyx_n_s_child); if (unlikely(!__pyx_tuple__75)) __PYX_ERR(1, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);

  /* "gen/styleclass.pxi":609
 *             self.properties.append({ 'thumb_offset' : value })
 *         def __del__(self):
 *             self.delattr('thumb_offset')             # <<<<<<<<<<<<<<
 * 
 *     property xfit:
 */
  __pyx_tuple__76 = PyTuple_Pack(1, __pyx_n_s_thumb_offset); if (unlikely(!__pyx_tuple__76)) __PYX_ERR(1, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);

  /* "gen/styleclass.pxi":617
 *             self.properties.append({ 'xfit' : value })
 *         def __del__(self):
 *             self.delattr('xfit')             # <<<<<<<<<<<<<<
 * 
 *     property hinting:
 */
  __pyx_tuple__77 = PyTuple_Pack(1, __pyx_n_s_xfit); if (unlikely(!__pyx_tuple__77)) __PYX_ERR(1, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "gen/styleclass.pxi":625
 *             self.properties.append({ 'hinting' : value })
 *         def __del__(self):
 *             self.delattr('hinting')             # <<<<<<<<<<<<<<
 * 
 *     property altruby_style:
 */
  __pyx_tuple__78 = PyTuple_Pack(1, __pyx_n_s_hinting); if (unlikely(!__pyx_tuple__78)) __PYX_ERR(1, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);

  /* "gen/styleclass.pxi":633
 *             self.properties.append({ 'altruby_style' : value })
 *         def __del__(self):
 *             self.delattr('altruby_style')             # <<<<<<<<<<<<<<
 * 
 *     property unscrollable:
 */
  __pyx_tuple__79 = PyTuple_Pack(1, __pyx_n_s_altruby_style); if (unlikely(!__pyx_tuple__79)) __PYX_ERR(1, 633, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);

  /* "gen/styleclass.pxi":641
 *             self.properties.append({ 'unscrollable' : value })
 *         def __del__(self):
 *             self.delattr('unscrollable')             # <<<<<<<<<<<<<<
 * 
 *     property order_reverse:
 */
  __pyx_tuple__80 = PyTuple_Pack(1, __pyx_n_s_unscrollable); if (unlikely(!__pyx_tuple__80)) __PYX_ERR(1, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "gen/styleclass.pxi":649
 *             self.properties.append({ 'order_reverse' : value })
 *         def __del__(self):
 *             self.delattr('order_reverse')             # <<<<<<<<<<<<<<
 * 
 *     property mouse:
 */
  __pyx_tuple__81 = PyTuple_Pack(1, __pyx_n_s_order_reverse); if (unlikely(!__pyx_tuple__81)) __PYX_ERR(1, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);

  /* "gen/styleclass.pxi":657
 *             self.properties.append({ 'mouse' : value })
 *         def __del__(self):
 *             self.delattr('mouse')             # <<<<<<<<<<<<<<
 * 
 *     property keyboard_focus:
 */
  __pyx_tuple__82 = PyTuple_Pack(1, __pyx_n_s_mouse); if (unlikely(!__pyx_tuple__82)) __PYX_ERR(1, 657, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);

  /* "gen/styleclass.pxi":665
 *             self.properties.append({ 'keyboard_focus' : value })
 *         def __del__(self):
 *             self.delattr('keyboard_focus')             # <<<<<<<<<<<<<<
 * 
 *     property first_indent:
 */
  __pyx_tuple__83 = PyTuple_Pack(1, __pyx_n_s_keyboard_focus); if (unlikely(!__pyx_tuple__83)) __PYX_ERR(1, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);

  /* "gen/styleclass.pxi":673
 *             self.properties.append({ 'first_indent' : value })
 *         def __del__(self):
 *             self.delattr('first_indent')             # <<<<<<<<<<<<<<
 * 
 *     property bar_vertical:
 */
  __pyx_tuple__84 = PyTuple_Pack(1, __pyx_n_s_first_indent); if (unlikely(!__pyx_tuple__84)) __PYX_ERR(1, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);

  /* "gen/styleclass.pxi":681
 *             self.properties.append({ 'bar_vertical' : value })
 *         def __del__(self):
 *             self.delattr('bar_vertical')             # <<<<<<<<<<<<<<
 * 
 *     property right_padding:
 */
  __pyx_tuple__85 = PyTuple_Pack(1, __pyx_n_s_bar_vertical); if (unlikely(!__pyx_tuple__85)) __PYX_ERR(1, 681, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);

  /* "gen/styleclass.pxi":689
 *             self.properties.append({ 'right_padding' : value })
 *         def __del__(self):
 *             self.delattr('right_padding')             # <<<<<<<<<<<<<<
 * 
 *     property focus_rect:
 */
  __pyx_tuple__86 = PyTuple_Pack(1, __pyx_n_s_right_padding); if (unlikely(!__pyx_tuple__86)) __PYX_ERR(1, 689, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "gen/styleclass.pxi":697
 *             self.properties.append({ 'focus_rect' : value })
 *         def __del__(self):
 *             self.delattr('focus_rect')             # <<<<<<<<<<<<<<
 * 
 *     property antialias:
 */
  __pyx_tuple__87 = PyTuple_Pack(1, __pyx_n_s_focus_rect); if (unlikely(!__pyx_tuple__87)) __PYX_ERR(1, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "gen/styleclass.pxi":705
 *             self.properties.append({ 'antialias' : value })
 *         def __del__(self):
 *             self.delattr('antialias')             # <<<<<<<<<<<<<<
 * 
 *     property slow_cps:
 */
  __pyx_tuple__88 = PyTuple_Pack(1, __pyx_n_s_antialias); if (unlikely(!__pyx_tuple__88)) __PYX_ERR(1, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "gen/styleclass.pxi":713
 *             self.properties.append({ 'slow_cps' : value })
 *         def __del__(self):
 *             self.delattr('slow_cps')             # <<<<<<<<<<<<<<
 * 
 *     property bar_invert:
 */
  __pyx_tuple__89 = PyTuple_Pack(1, __pyx_n_s_slow_cps); if (unlikely(!__pyx_tuple__89)) __PYX_ERR(1, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "gen/styleclass.pxi":721
 *             self.properties.append({ 'bar_invert' : value })
 *         def __del__(self):
 *             self.delattr('bar_invert')             # <<<<<<<<<<<<<<
 * 
 *     property line_leading:
 */
  __pyx_tuple__90 = PyTuple_Pack(1, __pyx_n_s_bar_invert); if (unlikely(!__pyx_tuple__90)) __PYX_ERR(1, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);

  /* "gen/styleclass.pxi":729
 *             self.properties.append({ 'line_leading' : value })
 *         def __del__(self):
 *             self.delattr('line_leading')             # <<<<<<<<<<<<<<
 * 
 *     property slow_cps_multiplier:
 */
  __pyx_tuple__91 = PyTuple_Pack(1, __pyx_n_s_line_leading); if (unlikely(!__pyx_tuple__91)) __PYX_ERR(1, 729, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);

  /* "gen/styleclass.pxi":737
 *             self.properties.append({ 'slow_cps_multiplier' : value })
 *         def __del__(self):
 *             self.delattr('slow_cps_multiplier')             # <<<<<<<<<<<<<<
 * 
 *     property thumb_shadow:
 */
  __pyx_tuple__92 = PyTuple_Pack(1, __pyx_n_s_slow_cps_multiplier); if (unlikely(!__pyx_tuple__92)) __PYX_ERR(1, 737, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);

  /* "gen/styleclass.pxi":745
 *             self.properties.append({ 'thumb_shadow' : value })
 *         def __del__(self):
 *             self.delattr('thumb_shadow')             # <<<<<<<<<<<<<<
 * 
 *     property yfill:
 */
  __pyx_tuple__93 = PyTuple_Pack(1, __pyx_n_s_thumb_shadow); if (unlikely(!__pyx_tuple__93)) __PYX_ERR(1, 745, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__93);
  __Pyx_GIVEREF(__pyx_tuple__93);

  /* "gen/styleclass.pxi":753
 *             self.properties.append({ 'yfill' : value })
 *         def __del__(self):
 *             self.delattr('yfill')             # <<<<<<<<<<<<<<
 * 
 *     property slow_abortable:
 */
  __pyx_tuple__94 = PyTuple_Pack(1, __pyx_n_s_yfill); if (unlikely(!__pyx_tuple__94)) __PYX_ERR(1, 753, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);

  /* "gen/styleclass.pxi":761
 *             self.properties.append({ 'slow_abortable' : value })
 *         def __del__(self):
 *             self.delattr('slow_abortable')             # <<<<<<<<<<<<<<
 * 
 *     property justify:
 */
  __pyx_tuple__95 = PyTuple_Pack(1, __pyx_n_s_slow_abortable); if (unlikely(!__pyx_tuple__95)) __PYX_ERR(1, 761, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__95);
  __Pyx_GIVEREF(__pyx_tuple__95);

  /* "gen/styleclass.pxi":769
 *             self.properties.append({ 'justify' : value })
 *         def __del__(self):
 *             self.delattr('justify')             # <<<<<<<<<<<<<<
 * 
 */
  __pyx_tuple__96 = PyTuple_Pack(1, __pyx_n_s_justify); if (unlikely(!__pyx_tuple__96)) __PYX_ERR(1, 769, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 */
  __pyx_tuple__97 = PyTuple_Pack(1, __pyx_kp_s_self_cache_cannot_be_converted_t); if (unlikely(!__pyx_tuple__97)) __PYX_ERR(0, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__97);
  __Pyx_GIVEREF(__pyx_tuple__97);

  /* "(tree fragment)":4
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("self.cache cannot be converted to a Python object for pickling")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__98 = PyTuple_Pack(1, __pyx_kp_s_self_cache_cannot_be_converted_t); if (unlikely(!__pyx_tuple__98)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__98);
  __Pyx_GIVEREF(__pyx_tuple__98);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(3, 1, __pyx_L1_error);
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initstyleclass(void); /*proto*/
PyMODINIT_FUNC initstyleclass(void)
#else
PyMODINIT_FUNC PyInit_styleclass(void); /*proto*/
PyMODINIT_FUNC PyInit_styleclass(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_styleclass(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_styleclass(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(3, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("styleclass", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(3, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(3, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(3, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_renpy__styledata__styleclass) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(3, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "renpy.styledata.styleclass")) {
      if (unlikely(PyDict_SetItemString(modules, "renpy.styledata.styleclass", __pyx_m) < 0)) __PYX_ERR(3, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_ptype_5renpy_5style_StyleCore = __Pyx_ImportType("renpy.style", "StyleCore", sizeof(struct __pyx_obj_5renpy_5style_StyleCore), 1); if (unlikely(!__pyx_ptype_5renpy_5style_StyleCore)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_5style_StyleCore = (struct __pyx_vtabstruct_5renpy_5style_StyleCore*)__Pyx_GetVtable(__pyx_ptype_5renpy_5style_StyleCore->tp_dict); if (unlikely(!__pyx_vtabptr_5renpy_5style_StyleCore)) __PYX_ERR(3, 1, __pyx_L1_error)
  __pyx_vtabptr_5renpy_9styledata_10styleclass_Style = &__pyx_vtable_5renpy_9styledata_10styleclass_Style;
  __pyx_vtable_5renpy_9styledata_10styleclass_Style.__pyx_base = *__pyx_vtabptr_5renpy_5style_StyleCore;
  __pyx_type_5renpy_9styledata_10styleclass_Style.tp_base = __pyx_ptype_5renpy_5style_StyleCore;
  if (PyType_Ready(&__pyx_type_5renpy_9styledata_10styleclass_Style) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_type_5renpy_9styledata_10styleclass_Style.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5renpy_9styledata_10styleclass_Style.tp_dict, __pyx_vtabptr_5renpy_9styledata_10styleclass_Style) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Style", (PyObject *)&__pyx_type_5renpy_9styledata_10styleclass_Style) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_5renpy_9styledata_10styleclass_Style) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_ptype_5renpy_9styledata_10styleclass_Style = &__pyx_type_5renpy_9styledata_10styleclass_Style;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(4, 9, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  #endif

  /* "renpy/styledata/styleclass.pyx":1
 * # Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>             # <<<<<<<<<<<<<<
 * #
 * # Permission is hereby granted, free of charge, to any person
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(3, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "renpy/style.pxd":56
 *     cache_priorities[index] = priority
 * 
 * cdef inline void assign_prefixed(int index, PyObject **cache, int *cache_priorities, int priority, d, prefix):             # <<<<<<<<<<<<<<
 *     """
 *     Like assign, but if the value can be duplicated, duplicates it and assigns
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init renpy.styledata.styleclass", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init renpy.styledata.styleclass");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* WriteUnraisableException */
  static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* CallNextTpDealloc */
  static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
  static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
  static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* GetVTable */
  static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* SetVTable */
  static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
  static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CLineInTraceback */
  #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
  #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
  #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
  static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
